MONARCH: Multiscale Online Nonhydrostatic AtmospheRe CHemistry model
====================================================================

The Multiscale Online Nonhydrostatic AtmospheRe CHemistry model (MONARCH) is a chemical weather prediction system, formerly known as NMMB/BSC-CTM, that can be run either globally or regionally. MONARCH, developed at the [Barcelona Supercomputing Center](https://www.bsc.es/), is based on the online coupling of the meteorological Nonhydrostatic Multiscale Model on the B-grid (NMMB) with a full aerosol-chemistry module. The NMMB model was developed by the National Oceanic and Atmospheric Administration - Environmental Modeling Center (NOAA/EMC) and distributed on an AS IS basis, without warranties or conditions of any kind, to the Barcelona Supercomputing Center through a Memorandum of Understanding.

The main characteristics of MONARCH are as follows:

- Nonhydrostatic meteorological core allowing both global and regional simulations with embedded telescoping nests.
- Gas-aerosol chemistry packages to solve the tropospheric chemistry of the atmosphere.
- Different meteorology-driven emissions computed on-line: mineral dust, sea salt and biogenic gas species.
- Multiple mineral dust and sea salt source functions available.
- Aerosol-radiation-interaction available.
- Interface with the emission model HERMESv3 to allow rapid deployment of new emission inventories.
- Data assimilation capability coupling MONARCH with a local ensemble transform Kalman filter (LETKF) scheme.

Version 2.0.0

Released 2021-08-05

**Source:** <https://earth.bsc.es/gitlab/es/monarch>

Copyright (C) 2008-2021 Earth Sciences Department, Barcelona Supercomputing Center
Licensed under the [GNU General Public License version 3](https://www.gnu.org/licenses/quick-guide-gplv3.html). For detailis see the file LICENSE.

## Avaliability

MONARCH is distributed on an AS IS basis, without warranties or conditions of any kind, either express or implied. See the License for the specific language governing permissions and limitations under the License.

## Citing

* Klose, M., Jorba, O., Gonçalves Ageitos, M., Escribano, J., Dawson, M. L., Obiso, V., Di Tomaso, E., Basart, S., Montané Pinto, G., Macchia, F., Ginoux, P., Guerschman, J., Prigent, C., Huang, Y., Kok, J. F., Miller, R. L., and Pérez García-Pando, C.: Mineral dust cycle in the Multiscale Online Nonhydrostatic AtmospheRe CHemistry model (MONARCH) Version 2.0, Geosci. Model Dev. Discuss. [preprint], https://doi.org/10.5194/gmd-2021-32, in review, 2021.
* Di Tomaso, E., Schutgens, N. A. J., Jorba, O., and Pérez García-Pando, C.: Assimilation of MODIS Dark Target and Deep Blue observations in the dust aerosol component of NMMB-MONARCH version 1.0, Geosci. Model Dev., 10, 1107-1129, https://doi.org/10.5194/gmd-10-1107-2017, 2017.
* Badia, A., Jorba, O., Voulgarakis, A., Dabdub, D., Pérez García-Pando, C., Hilboll, A., Gonçalves, M., and Janjic, Z.: Description and evaluation of the Multiscale Online Nonhydrostatic AtmospheRe CHemistry model (NMMB-MONARCH) version 1.0: gas-phase chemistry at global scale, Geosci. Model Dev., 10, 609-638, https://doi.org/10.5194/gmd-10-609-2017, 2017.
* Spada, M.: Development and evaluation of an atmospheric aerosol module implemented within the NMMB/BSC-CTM, Ph.D. thesis, Universitat Politècnica de Catalunya, http://hdl.handle.net/2117/95991, 2015.
* Badia, A. and Jorba, O.: Gas-phase evaluation of the online NMMB/BSC-CTM model over Europe for 2010 in the framework of the AQMEII-Phase2 project, Atmospheric Environment, http://dx.doi.org/10.1016/j.atmosenv.2014.05.055, 2014.
* Spada, M., Jorba, O., Pérez García-Pando, C., Janjic, Z., and Baldasano, J. M.: Modeling and evaluation of the global sea-salt aerosol distribution: sensitivity to size-resolved and sea-surface temperature dependent emission schemes, Atmospheric Chemistry and Physics,13, 11 735–11 755, http://dx.doi.org/10.5194/acp-13-11735-2013, 2013.
* Jorba, O., Dabdub, D., Blaszczak-Boxe, C., Pérez, C., Janjic, Z., Baldasano, J. M., Spada, M., Badia, A., and Gonçalves, M.: Potential significance of photoexcited NO2 on global air quality with the NMMB/BSC chemical transport model, Journal of Geophysical Research: Atmospheres, 117, http://dx.doi.org/10.1029/2012JD017730, 2012.
* Haustein, K., Pérez, C., Baldasano, J. M., Jorba, O., S., B., Miller, R., Janjic, Z., Black, T., Nickovic, S., Todd, M., and Washington, R.: Atmospheric dust modeling from meso to global scales with the online NMMB/BSC-Dust model - Part 2: Experimental campaigns in Northern Africa, Atmos. Chem. Phys., 12, 2933–2958, http://dx.doi.org/10.5194/acp-12-2933-2012, 2012.
* Pérez, C., Haustein, K., Janjic, Z., Jorba, O., Huneeus, N., Baldasano, J. M., Black, T., Basart, S., Nickovic, S., Miller, R. L., Perlwitz, J. P., Schulz, M., and Thomson, M.: Atmospheric dust modeling from meso to global scales with the online NMMB/BSC-Dust model Part 1: Model description, annual simulations and evaluation, Atmospheric Chemistry and Physics, 11, 13 001–13 027, http://dx.doi.org/10.5194/acp-11-13001-2011, 2011.

# Dependencies

* Fortran 2008 compiler - <https://gcc.gnu.org/fortran/> or similar
* CMake version 2.6.4 or higher - <http://www.cmake.org/>
* MPI parallel support - <http://www.open-mpi.org/> or similar
* NetCDF version 4 or higher - <http://www.unidata.ucar.edu/software/netcdf/>
* ESMF version 6.3.0r - <https://earthsystemmodeling.org>
* NCEPLIBS - <https://noaa-emc.github.io/NCEPLIBS/>
* wgrib version 1.8 - <https://www.cpc.ncep.noaa.gov/products/wesley/wgrib.html>
* cnvgrib - <https://www.nco.ncep.noaa.gov/pmb/codes/GRIB2/>

Optional dependencies:

* CAMP version 1.0.0 - <https://github.com/open-atmos/camp>
* PartMC version 2.4.0 - <https://github.com/compdyn/partmc/tree/chem_mod>
* json-fortran for JSON input file support - <https://github.com/jacobwilliams/json-fortran>
* SuiteSparse - <http://www.suitesparse.com>
* CVODE version 3.4 - <https://computing.llnl.gov/projects/sundials/cvode>

# Directory Structure
```
 clean.sh
 compile.sh
 DATA -> $(PATH_DATA)/DATA_monarchv2.0/
 DOC
 JOB
 MODEL
 OUTPUT
 PREPROC
```

# Configure and Build

To compile and prepare running scripts:

1. Edit compile.sh
2. Set PATH, compile options and compilers
3. Run compile.sh

It will create the model binaries and required scripts to run the system.

# Execution

To execute an experiment:

1. Go to `JOB/TEMPLATE` folder
2. Edit `NMMBrrtm_RUN_TEMPLATE.sh`
3. Adapt `bsub_template.{platform}.cmd` to your HPC environment
4. Submit the job

# Build Documentation

To compile the documentation:

1. Check that Doxygen is installed:

        doxygen --version

2. Go to the DOC folder
3. Run doxygen:

        doxygen Doxyfile

4. Navigate to ``DOC/build/html/index.html`` in a web browser.
