!!!!!  ==============================================================  !!!!!
!!!!!              sw-rrtm3 radiation package description              !!!!!
!!!!!  ==============================================================  !!!!!
!                                                                          !
!   this package includes ncep's modifications of the rrtm-sw radiation    !
!   code from aer inc.                                                     !
!                                                                          !
!    the sw-rrtm3 package includes these parts:                            !
!                                                                          !
!       'radsw_rrtm3_param.f'                                              !
!       'radsw_rrtm3_datatb.f'                                             !
!       'radsw_rrtm3_main.f'                                               !
!                                                                          !
!    the 'radsw_rrtm3_param.f' contains:                                   !
!                                                                          !
!       'module_radsw_parameters'  -- band parameters set up               !
!                                                                          !
!    the 'radsw_rrtm3_datatb.f' contains:                                  !
!                                                                          !
!       'module_radsw_ref'         -- reference temperature and pressure   !
!       'module_radsw_cldprtb'     -- cloud property coefficients table    !
!       'module_radsw_sflux'       -- spectral solar flux distribution     !
!       'module_radsw_kgbnn'       -- absorption coeffients for 14         !
!                                     bands, where nn = 16-29              !
!                                                                          !
!    the 'radsw_rrtm3_main.f' contains:                                    !
!                                                                          !
!       'module_radsw_main'        -- main sw radiation transfer           !
!                                                                          !
!    in the main module 'module_radsw_main' there are only two             !
!    externally callable subroutines:                                      !
!                                                                          !
!       'swrad'      -- main rrtm3 sw radiation routine                    !
!       'rswinit'    -- initialization routine                             !
!                                                                          !
!    all the sw radiation subprograms become contained subprograms         !
!    in module 'module_radsw_main' and many of them are not directly       !
!    accessable from places outside the module.                            !
!                                                                          !
!    compilation sequence is:                                              !
!                                                                          !
!       'radsw_rrtm3_param.f'                                              !
!       'radsw_rrtm3_datatb.f'                                             !
!       'radsw_rrtm3_main.f'                                               !
!                                                                          !
!    and all should be put in front of routines that use sw modules        !
!                                                                          !
!   ncep modifications history log:                                        !
!                                                                          !
!       see list in program "radsw_rrtm3_main.f"                           !
!                                                                          !
!!!!!  ==============================================================  !!!!!
!!!!!                         end descriptions                         !!!!!
!!!!!  ==============================================================  !!!!!


!========================================!
      module module_radsw_ref            !
!........................................!
!
      use physparam,               only : kind_phys
!
      implicit none
!
      public

!  ---  reference pressure and temperature
!     real (kind=kind_phys), dimension(59) :: pref, preflog, tref
      real (kind=kind_phys), dimension(59) :: preflog, tref

!  ...  these pressures are chosen such that the ln of the first pressure
!       has only a few non-zero digits (i.e. ln(pref(1)) = 6.96000) and
!       each subsequent ln(pressure) differs from the previous one by 0.2.

!     data  pref(:)  /                                                  &
!    &     1.05363e+03,8.62642e+02,7.06272e+02,5.78246e+02,4.73428e+02, &
!    &     3.87610e+02,3.17348e+02,2.59823e+02,2.12725e+02,1.74164e+02, &
!    &     1.42594e+02,1.16746e+02,9.55835e+01,7.82571e+01,6.40715e+01, &
!    &     5.24573e+01,4.29484e+01,3.51632e+01,2.87892e+01,2.35706e+01, &
!    &     1.92980e+01,1.57998e+01,1.29358e+01,1.05910e+01,8.67114e+00, &
!    &     7.09933e+00,5.81244e+00,4.75882e+00,3.89619e+00,3.18993e+00, &
!    &     2.61170e+00,2.13828e+00,1.75067e+00,1.43333e+00,1.17351e+00, &
!    &     9.60789e-01,7.86628e-01,6.44036e-01,5.27292e-01,4.31710e-01, &
!    &     3.53455e-01,2.89384e-01,2.36928e-01,1.93980e-01,1.58817e-01, &
!    &     1.30029e-01,1.06458e-01,8.71608e-02,7.13612e-02,5.84256e-02, &
!    &     4.78349e-02,3.91639e-02,3.20647e-02,2.62523e-02,2.14936e-02, &
!    &     1.75975e-02,1.44076e-02,1.17959e-02,9.65769e-03 /

      data  preflog(:)  /                                               &
     &      6.9600e+00, 6.7600e+00, 6.5600e+00, 6.3600e+00, 6.1600e+00, &
     &      5.9600e+00, 5.7600e+00, 5.5600e+00, 5.3600e+00, 5.1600e+00, &
     &      4.9600e+00, 4.7600e+00, 4.5600e+00, 4.3600e+00, 4.1600e+00, &
     &      3.9600e+00, 3.7600e+00, 3.5600e+00, 3.3600e+00, 3.1600e+00, &
     &      2.9600e+00, 2.7600e+00, 2.5600e+00, 2.3600e+00, 2.1600e+00, &
     &      1.9600e+00, 1.7600e+00, 1.5600e+00, 1.3600e+00, 1.1600e+00, &
     &      9.6000e-01, 7.6000e-01, 5.6000e-01, 3.6000e-01, 1.6000e-01, &
     &     -4.0000e-02,-2.4000e-01,-4.4000e-01,-6.4000e-01,-8.4000e-01, &
     &     -1.0400e+00,-1.2400e+00,-1.4400e+00,-1.6400e+00,-1.8400e+00, &
     &     -2.0400e+00,-2.2400e+00,-2.4400e+00,-2.6400e+00,-2.8400e+00, &
     &     -3.0400e+00,-3.2400e+00,-3.4400e+00,-3.6400e+00,-3.8400e+00, &
     &     -4.0400e+00,-4.2400e+00,-4.4400e+00,-4.6400e+00 /

!  ...  these are the temperatures associated with the respective
!       pressures for the MLS standard atmosphere.
      data  tref(:)  /                                                  &
     &      2.9420e+02, 2.8799e+02, 2.7894e+02, 2.6925e+02, 2.5983e+02, &
     &      2.5017e+02, 2.4077e+02, 2.3179e+02, 2.2306e+02, 2.1578e+02, &
     &      2.1570e+02, 2.1570e+02, 2.1570e+02, 2.1706e+02, 2.1858e+02, &
     &      2.2018e+02, 2.2174e+02, 2.2328e+02, 2.2479e+02, 2.2655e+02, &
     &      2.2834e+02, 2.3113e+02, 2.3401e+02, 2.3703e+02, 2.4022e+02, &
     &      2.4371e+02, 2.4726e+02, 2.5085e+02, 2.5457e+02, 2.5832e+02, &
     &      2.6216e+02, 2.6606e+02, 2.6999e+02, 2.7340e+02, 2.7536e+02, &
     &      2.7568e+02, 2.7372e+02, 2.7163e+02, 2.6955e+02, 2.6593e+02, &
     &      2.6211e+02, 2.5828e+02, 2.5360e+02, 2.4854e+02, 2.4348e+02, &
     &      2.3809e+02, 2.3206e+02, 2.2603e+02, 2.2000e+02, 2.1435e+02, &
     &      2.0887e+02, 2.0340e+02, 1.9792e+02, 1.9290e+02, 1.8809e+02, &
     &      1.8329e+02, 1.7849e+02, 1.7394e+02, 1.7212e+02 /

!........................................!
      end module module_radsw_ref        !
!========================================!


!========================================!
      module module_radsw_cldprtb        !
!........................................!
!
! ***********************  module descriptions  ********************** !
!                                                                      !
!     this module contains coefficients of cloud-optical properties    !
!     for each of the spectral bands.  modified from aer/ecmwf rrtm    !
!     sw radiation package subroutine "susrtop".                       !
!                                                                      !
!     modify history:                                                  !
!       jan 2004   --  yu-tai hou    convert subroutine to data        !
!                      module for ncep models.                         !
!       jun 2008   --  yu-tai hou    modified to use aer's newer       !
!                      release v3.5 data table for cloud ice particles !
!                                                                      !
! *********         the original program descriptions        ********* !
!                                                                      !
!     adapted from j. delamere,  atmospheric & environmental research. !
!     by jjmorcrette,  ecmwf    02/10/29                               !
!     revision:  j.d. 2.6 2002/04/04 18:29:47                          !
!     reformatted for f90 by jjmorcrette, ecmwf                        !
!                                                                      !
!  name     type     purpose
!  ----   : ----   : --------------------------------------------------!
! xxxliq1 : real   : optical properties (extinction coefficient, single!
!                    scattering albedo, assymetry factor) from         !
!                    hu & stamnes, 1993, j. clim., 6, 728-742          !
! xbari   : real   : optical properties (extinction coefficient, single!
!                    scattering albedo, assymetry factor) calculated   !
!                    based on ebert and curry method, jgr, 1992        !
! xxxice2 : real   : optical properties (extinction coefficient, single!
!                    scattering albedo, assymetry factor) from         !
!                    streamer v3.0, key.j.streamer user's guide,       !
!                    cooperative institute for meteorological satellite!
!                    studies, 2001, 96 pp.                             !
! xxxice3 : real   : optical properties (extinction coefficient, single!
!                    scattering albedo, assymetry factor) from         !
!                    fu, 1996, j. clim., 9,                            !
!                                                                      !
! *************************  end description  ************************ !

      use physparam,               only : kind_phys
      use module_radsw_parameters, only : nblow, nbhgh
!
      implicit none
!
      private

!
! === for iflagliq = 0,
!     cloud optical properties are external inputs to the main program

! === everything below is for iflagliq >= 1.

      real (kind=kind_phys), dimension(58,nblow:nbhgh), public ::       &
     &       extliq1, ssaliq1, asyliq1
      real (kind=kind_phys), dimension(43,nblow:nbhgh), public ::       &
     &       extice2, ssaice2, asyice2
      real (kind=kind_phys), dimension(46,nblow:nbhgh), public ::       &
     &       extice3, ssaice3, asyice3, fdlice3
      real (kind=kind_phys), dimension(5), public ::                    &
     &       abari, bbari, cbari, dbari, ebari, fbari

!  --- ...  coefficients from ebert and curry method
      data abari(:)/ 3.448e-03,3.448e-03,3.448e-03,3.448e-03,3.448e-03 /
      data bbari(:)/ 2.431e+00,2.431e+00,2.431e+00,2.431e+00,2.431e+00 /
      data cbari(:)/ 1.000e-05,1.100e-04,1.240e-02,3.779e-02,4.666e-01 /
      data dbari(:)/ 0.000e+00,1.405e-05,6.867e-04,1.284e-03,2.050e-05 /
      data ebari(:)/ 7.661e-01,7.730e-01,7.865e-01,8.172e-01,9.595e-01 /
      data fbari(:)/ 5.851e-04,5.665e-04,7.204e-04,7.463e-04,1.076e-04 /

!  --- ...  extinction coefficient from hu and stamnes
      data  extliq1(:, 16)   /                                          &
     & 8.981463e-01,6.317895e-01,4.557508e-01,3.481624e-01,2.797950e-01,&
     & 2.342753e-01,2.026934e-01,1.800102e-01,1.632408e-01,1.505384e-01,&
     & 1.354524e-01,1.246520e-01,1.154342e-01,1.074756e-01,1.005353e-01,&
     & 9.442987e-02,8.901760e-02,8.418693e-02,7.984904e-02,7.593229e-02,&
     & 7.237827e-02,6.913887e-02,6.617415e-02,6.345061e-02,6.094001e-02,&
     & 5.861834e-02,5.646506e-02,5.446250e-02,5.249596e-02,5.081114e-02,&
     & 4.922243e-02,4.772189e-02,4.630243e-02,4.495766e-02,4.368189e-02,&
     & 4.246995e-02,4.131720e-02,4.021941e-02,3.917276e-02,3.817376e-02,&
     & 3.721926e-02,3.630635e-02,3.543237e-02,3.459491e-02,3.379171e-02,&
     & 3.302073e-02,3.228007e-02,3.156798e-02,3.088284e-02,3.022315e-02,&
     & 2.958753e-02,2.897468e-02,2.838340e-02,2.781258e-02,2.726117e-02,&
     & 2.672821e-02,2.621278e-02,2.5714e-02 /
      data  extliq1(:, 17)   /                                          &
     & 8.293797e-01,6.048371e-01,4.465706e-01,3.460387e-01,2.800064e-01,&
     & 2.346584e-01,2.022399e-01,1.782626e-01,1.600153e-01,1.457903e-01,&
     & 1.334061e-01,1.228548e-01,1.138396e-01,1.060486e-01,9.924856e-02,&
     & 9.326208e-02,8.795158e-02,8.320883e-02,7.894750e-02,7.509792e-02,&
     & 7.160323e-02,6.841653e-02,6.549889e-02,6.281763e-02,6.034516e-02,&
     & 5.805802e-02,5.593615e-02,5.396226e-02,5.202302e-02,5.036246e-02,&
     & 4.879606e-02,4.731610e-02,4.591565e-02,4.458852e-02,4.332912e-02,&
     & 4.213243e-02,4.099390e-02,3.990941e-02,3.887522e-02,3.788792e-02,&
     & 3.694440e-02,3.604183e-02,3.517760e-02,3.434934e-02,3.355485e-02,&
     & 3.279211e-02,3.205925e-02,3.135458e-02,3.067648e-02,3.002349e-02,&
     & 2.939425e-02,2.878748e-02,2.820200e-02,2.763673e-02,2.709062e-02,&
     & 2.656272e-02,2.605214e-02,2.5558e-02 /
      data  extliq1(:, 18)   /                                          &
     & 9.193685e-01,6.128292e-01,4.344150e-01,3.303048e-01,2.659500e-01,&
     & 2.239727e-01,1.953457e-01,1.751012e-01,1.603515e-01,1.493360e-01,&
     & 1.323791e-01,1.219335e-01,1.130076e-01,1.052926e-01,9.855839e-02,&
     & 9.262925e-02,8.736918e-02,8.267112e-02,7.844965e-02,7.463585e-02,&
     & 7.117343e-02,6.801601e-02,6.512503e-02,6.246815e-02,6.001806e-02,&
     & 5.775154e-02,5.564872e-02,5.369250e-02,5.176284e-02,5.011536e-02,&
     & 4.856099e-02,4.709211e-02,4.570193e-02,4.438430e-02,4.313375e-02,&
     & 4.194529e-02,4.081443e-02,3.973712e-02,3.870966e-02,3.772866e-02,&
     & 3.679108e-02,3.589409e-02,3.503514e-02,3.421185e-02,3.342206e-02,&
     & 3.266377e-02,3.193513e-02,3.123447e-02,3.056018e-02,2.991081e-02,&
     & 2.928502e-02,2.868154e-02,2.809920e-02,2.753692e-02,2.699367e-02,&
     & 2.646852e-02,2.596057e-02,2.5469e-02 /
      data  extliq1(:, 19)   /                                          &
     & 9.136931e-01,5.743244e-01,4.080708e-01,3.150572e-01,2.577261e-01,&
     & 2.197900e-01,1.933037e-01,1.740212e-01,1.595056e-01,1.482756e-01,&
     & 1.312164e-01,1.209246e-01,1.121227e-01,1.045095e-01,9.785967e-02,&
     & 9.200149e-02,8.680170e-02,8.215531e-02,7.797850e-02,7.420361e-02,&
     & 7.077530e-02,6.764798e-02,6.478369e-02,6.215063e-02,5.972189e-02,&
     & 5.747458e-02,5.538913e-02,5.344866e-02,5.153216e-02,4.989745e-02,&
     & 4.835476e-02,4.689661e-02,4.551629e-02,4.420777e-02,4.296563e-02,&
     & 4.178497e-02,4.066137e-02,3.959081e-02,3.856963e-02,3.759452e-02,&
     & 3.666244e-02,3.577061e-02,3.491650e-02,3.409777e-02,3.331227e-02,&
     & 3.255803e-02,3.183322e-02,3.113617e-02,3.046530e-02,2.981918e-02,&
     & 2.919646e-02,2.859591e-02,2.801635e-02,2.745671e-02,2.691599e-02,&
     & 2.639324e-02,2.588759e-02,2.5398e-02 /
      data  extliq1(:, 20)   /                                          &
     & 8.447548e-01,5.326840e-01,3.921523e-01,3.119082e-01,2.597055e-01,&
     & 2.228737e-01,1.954157e-01,1.741155e-01,1.570881e-01,1.431520e-01,&
     & 1.302034e-01,1.200491e-01,1.113571e-01,1.038330e-01,9.725657e-02,&
     & 9.145949e-02,8.631112e-02,8.170840e-02,7.756901e-02,7.382641e-02,&
     & 7.042616e-02,6.732338e-02,6.448069e-02,6.186672e-02,5.945494e-02,&
     & 5.722277e-02,5.515089e-02,5.322262e-02,5.132153e-02,4.969799e-02,&
     & 4.816556e-02,4.671686e-02,4.534525e-02,4.404480e-02,4.281014e-02,&
     & 4.163643e-02,4.051930e-02,3.945479e-02,3.843927e-02,3.746945e-02,&
     & 3.654234e-02,3.565518e-02,3.480547e-02,3.399088e-02,3.320930e-02,&
     & 3.245876e-02,3.173745e-02,3.104371e-02,3.037600e-02,2.973287e-02,&
     & 2.911300e-02,2.851516e-02,2.793818e-02,2.738101e-02,2.684264e-02,&
     & 2.632214e-02,2.581863e-02,2.5331e-02 /
      data  extliq1(:, 21)   /                                          &
     & 7.727642e-01,5.034865e-01,3.808673e-01,3.080333e-01,2.586453e-01,&
     & 2.224989e-01,1.947060e-01,1.725821e-01,1.545096e-01,1.394456e-01,&
     & 1.288683e-01,1.188852e-01,1.103317e-01,1.029214e-01,9.643967e-02,&
     & 9.072239e-02,8.564194e-02,8.109758e-02,7.700875e-02,7.331026e-02,&
     & 6.994879e-02,6.688028e-02,6.406807e-02,6.148133e-02,5.909400e-02,&
     & 5.688388e-02,5.483197e-02,5.292185e-02,5.103763e-02,4.942905e-02,&
     & 4.791039e-02,4.647438e-02,4.511453e-02,4.382497e-02,4.260043e-02,&
     & 4.143616e-02,4.032784e-02,3.927155e-02,3.826375e-02,3.730117e-02,&
     & 3.638087e-02,3.550013e-02,3.465646e-02,3.384759e-02,3.307141e-02,&
     & 3.232598e-02,3.160953e-02,3.092040e-02,3.025706e-02,2.961810e-02,&
     & 2.900220e-02,2.840814e-02,2.783478e-02,2.728106e-02,2.674599e-02,&
     & 2.622864e-02,2.572816e-02,2.5244e-02 /
      data  extliq1(:, 22)   /                                          &
     & 7.416833e-01,4.959591e-01,3.775057e-01,3.056353e-01,2.565943e-01,&
     & 2.206935e-01,1.931479e-01,1.712860e-01,1.534837e-01,1.386906e-01,&
     & 1.281198e-01,1.182344e-01,1.097595e-01,1.024137e-01,9.598552e-02,&
     & 9.031320e-02,8.527093e-02,8.075927e-02,7.669869e-02,7.302481e-02,&
     & 6.968491e-02,6.663542e-02,6.384008e-02,6.126838e-02,5.889452e-02,&
     & 5.669654e-02,5.465558e-02,5.275540e-02,5.087937e-02,4.927904e-02,&
     & 4.776796e-02,4.633895e-02,4.498557e-02,4.370202e-02,4.248306e-02,&
     & 4.132399e-02,4.022052e-02,3.916878e-02,3.816523e-02,3.720665e-02,&
     & 3.629011e-02,3.541290e-02,3.457257e-02,3.376685e-02,3.299365e-02,&
     & 3.225105e-02,3.153728e-02,3.085069e-02,3.018977e-02,2.955310e-02,&
     & 2.893940e-02,2.834742e-02,2.777606e-02,2.722424e-02,2.669099e-02,&
     & 2.617539e-02,2.567658e-02,2.5194e-02 /
      data  extliq1(:, 23)   /                                          &
     & 7.058580e-01,4.866573e-01,3.712238e-01,2.998638e-01,2.513441e-01,&
     & 2.161972e-01,1.895576e-01,1.686669e-01,1.518437e-01,1.380046e-01,&
     & 1.267564e-01,1.170399e-01,1.087026e-01,1.014704e-01,9.513729e-02,&
     & 8.954555e-02,8.457221e-02,8.012009e-02,7.611136e-02,7.248294e-02,&
     & 6.918317e-02,6.616934e-02,6.340584e-02,6.086273e-02,5.851465e-02,&
     & 5.634001e-02,5.432027e-02,5.243946e-02,5.058070e-02,4.899628e-02,&
     & 4.749975e-02,4.608411e-02,4.474303e-02,4.347082e-02,4.226237e-02,&
     & 4.111303e-02,4.001861e-02,3.897528e-02,3.797959e-02,3.702835e-02,&
     & 3.611867e-02,3.524791e-02,3.441364e-02,3.361360e-02,3.284577e-02,&
     & 3.210823e-02,3.139923e-02,3.071716e-02,3.006052e-02,2.942791e-02,&
     & 2.881806e-02,2.822974e-02,2.766185e-02,2.711335e-02,2.658326e-02,&
     & 2.607066e-02,2.557473e-02,2.5095e-02 /
      data  extliq1(:, 24)   /                                          &
     & 6.822779e-01,4.750373e-01,3.634834e-01,2.940726e-01,2.468060e-01,&
     & 2.125768e-01,1.866586e-01,1.663588e-01,1.500326e-01,1.366192e-01,&
     & 1.253472e-01,1.158052e-01,1.076101e-01,1.004954e-01,9.426089e-02,&
     & 8.875268e-02,8.385090e-02,7.946063e-02,7.550578e-02,7.192466e-02,&
     & 6.866669e-02,6.569001e-02,6.295971e-02,6.044642e-02,5.812526e-02,&
     & 5.597500e-02,5.397746e-02,5.211690e-02,5.027505e-02,4.870703e-02,&
     & 4.722555e-02,4.582373e-02,4.449540e-02,4.323497e-02,4.203742e-02,&
     & 4.089821e-02,3.981321e-02,3.877867e-02,3.779118e-02,3.684762e-02,&
     & 3.594514e-02,3.508114e-02,3.425322e-02,3.345917e-02,3.269698e-02,&
     & 3.196477e-02,3.126082e-02,3.058352e-02,2.993141e-02,2.930310e-02,&
     & 2.869732e-02,2.811289e-02,2.754869e-02,2.700371e-02,2.647698e-02,&
     & 2.596760e-02,2.547473e-02,2.4998e-02 /
      data  extliq1(:, 25)   /                                          &
     & 6.666233e-01,4.662044e-01,3.579517e-01,2.902984e-01,2.440475e-01,&
     & 2.104431e-01,1.849277e-01,1.648970e-01,1.487555e-01,1.354714e-01,&
     & 1.244173e-01,1.149913e-01,1.068903e-01,9.985323e-02,9.368351e-02,&
     & 8.823009e-02,8.337507e-02,7.902511e-02,7.510529e-02,7.155482e-02,&
     & 6.832386e-02,6.537113e-02,6.266218e-02,6.016802e-02,5.786408e-02,&
     & 5.572939e-02,5.374598e-02,5.189830e-02,5.006825e-02,4.851081e-02,&
     & 4.703906e-02,4.564623e-02,4.432621e-02,4.307349e-02,4.188312e-02,&
     & 4.075060e-02,3.967183e-02,3.864313e-02,3.766111e-02,3.672269e-02,&
     & 3.582505e-02,3.496559e-02,3.414196e-02,3.335198e-02,3.259362e-02,&
     & 3.186505e-02,3.116454e-02,3.049052e-02,2.984152e-02,2.921617e-02,&
     & 2.861322e-02,2.803148e-02,2.746986e-02,2.692733e-02,2.640295e-02,&
     & 2.589582e-02,2.540510e-02,2.4930e-02 /
      data  extliq1(:, 26)   /                                          &
     & 6.535669e-01,4.585865e-01,3.529226e-01,2.867245e-01,2.413848e-01,&
     & 2.083956e-01,1.833191e-01,1.636150e-01,1.477247e-01,1.346392e-01,&
     & 1.236449e-01,1.143095e-01,1.062828e-01,9.930773e-02,9.319029e-02,&
     & 8.778150e-02,8.296497e-02,7.864847e-02,7.475799e-02,7.123343e-02,&
     & 6.802549e-02,6.509332e-02,6.240285e-02,5.992538e-02,5.763657e-02,&
     & 5.551566e-02,5.354483e-02,5.170870e-02,4.988866e-02,4.834061e-02,&
     & 4.687751e-02,4.549264e-02,4.417999e-02,4.293410e-02,4.175006e-02,&
     & 4.062344e-02,3.955019e-02,3.852663e-02,3.754943e-02,3.661553e-02,&
     & 3.572214e-02,3.486669e-02,3.404683e-02,3.326040e-02,3.250542e-02,&
     & 3.178003e-02,3.108254e-02,3.041139e-02,2.976511e-02,2.914235e-02,&
     & 2.854187e-02,2.796247e-02,2.740309e-02,2.686271e-02,2.634038e-02,&
     & 2.583520e-02,2.534636e-02,2.4873e-02 /
      data  extliq1(:, 27)   /                                          &
     & 6.448790e-01,4.541425e-01,3.503348e-01,2.850494e-01,2.401966e-01,&
     & 2.074811e-01,1.825631e-01,1.629515e-01,1.471142e-01,1.340574e-01,&
     & 1.231462e-01,1.138628e-01,1.058802e-01,9.894286e-02,9.285818e-02,&
     & 8.747802e-02,8.268676e-02,7.839271e-02,7.452230e-02,7.101580e-02,&
     & 6.782418e-02,6.490685e-02,6.222991e-02,5.976484e-02,5.748742e-02,&
     & 5.537703e-02,5.341593e-02,5.158883e-02,4.977355e-02,4.823172e-02,&
     & 4.677430e-02,4.539465e-02,4.408680e-02,4.284533e-02,4.166539e-02,&
     & 4.054257e-02,3.947283e-02,3.845256e-02,3.747842e-02,3.654737e-02,&
     & 3.565665e-02,3.480370e-02,3.398620e-02,3.320198e-02,3.244908e-02,&
     & 3.172566e-02,3.103002e-02,3.036062e-02,2.971600e-02,2.909482e-02,&
     & 2.849582e-02,2.791785e-02,2.735982e-02,2.682072e-02,2.629960e-02,&
     & 2.579559e-02,2.530786e-02,2.4836e-02 /
      data  extliq1(:, 28)   /                                          &
     & 6.422688e-01,4.528453e-01,3.497232e-01,2.847724e-01,2.400815e-01,&
     & 2.074403e-01,1.825502e-01,1.629415e-01,1.470934e-01,1.340183e-01,&
     & 1.230935e-01,1.138049e-01,1.058201e-01,9.888245e-02,9.279878e-02,&
     & 8.742053e-02,8.263175e-02,7.834058e-02,7.447327e-02,7.097000e-02,&
     & 6.778167e-02,6.486765e-02,6.219400e-02,5.973215e-02,5.745790e-02,&
     & 5.535059e-02,5.339250e-02,5.156831e-02,4.975308e-02,4.821235e-02,&
     & 4.675596e-02,4.537727e-02,4.407030e-02,4.282968e-02,4.165053e-02,&
     & 4.052845e-02,3.945941e-02,3.843980e-02,3.746628e-02,3.653583e-02,&
     & 3.564567e-02,3.479326e-02,3.397626e-02,3.319253e-02,3.244008e-02,&
     & 3.171711e-02,3.102189e-02,3.035289e-02,2.970866e-02,2.908784e-02,&
     & 2.848920e-02,2.791156e-02,2.735385e-02,2.681507e-02,2.629425e-02,&
     & 2.579053e-02,2.530308e-02,2.4831e-02 /
      data  extliq1(:, 29)   /                                          &
     & 4.614710e-01,4.556116e-01,4.056568e-01,3.529833e-01,3.060334e-01,&
     & 2.658127e-01,2.316095e-01,2.024325e-01,1.773749e-01,1.556867e-01,&
     & 1.455558e-01,1.332882e-01,1.229052e-01,1.140067e-01,1.062981e-01,&
     & 9.955703e-02,9.361333e-02,8.833420e-02,8.361467e-02,7.937071e-02,&
     & 7.553420e-02,7.204942e-02,6.887031e-02,6.595851e-02,6.328178e-02,&
     & 6.081286e-02,5.852854e-02,5.640892e-02,5.431269e-02,5.252561e-02,&
     & 5.084345e-02,4.925727e-02,4.775910e-02,4.634182e-02,4.499907e-02,&
     & 4.372512e-02,4.251484e-02,4.136357e-02,4.026710e-02,3.922162e-02,&
     & 3.822365e-02,3.727004e-02,3.635790e-02,3.548457e-02,3.464764e-02,&
     & 3.384488e-02,3.307424e-02,3.233384e-02,3.162192e-02,3.093688e-02,&
     & 3.027723e-02,2.964158e-02,2.902864e-02,2.843722e-02,2.786621e-02,&
     & 2.731457e-02,2.678133e-02,2.6266e-02 /

!  --- ...  single scattering albedo from hu and stamnes
      data  ssaliq1(:, 16)   /                                          &
     & 8.143821e-01,7.836739e-01,7.550722e-01,7.306269e-01,7.105612e-01,&
     & 6.946649e-01,6.825556e-01,6.737762e-01,6.678448e-01,6.642830e-01,&
     & 6.679741e-01,6.584607e-01,6.505598e-01,6.440951e-01,6.388901e-01,&
     & 6.347689e-01,6.315549e-01,6.290718e-01,6.271432e-01,6.255928e-01,&
     & 6.242441e-01,6.229207e-01,6.214464e-01,6.196445e-01,6.173388e-01,&
     & 6.143527e-01,6.105099e-01,6.056339e-01,6.108290e-01,6.073939e-01,&
     & 6.043073e-01,6.015473e-01,5.990913e-01,5.969173e-01,5.950028e-01,&
     & 5.933257e-01,5.918636e-01,5.905944e-01,5.894957e-01,5.885453e-01,&
     & 5.877209e-01,5.870003e-01,5.863611e-01,5.857811e-01,5.852381e-01,&
     & 5.847098e-01,5.841738e-01,5.836081e-01,5.829901e-01,5.822979e-01,&
     & 5.815089e-01,5.806011e-01,5.795521e-01,5.783396e-01,5.769413e-01,&
     & 5.753351e-01,5.734986e-01,5.7141e-01 /
      data  ssaliq1(:, 17)   /                                          &
     & 8.165821e-01,8.002015e-01,7.816921e-01,7.634131e-01,7.463721e-01,&
     & 7.312469e-01,7.185883e-01,7.088975e-01,7.026671e-01,7.004020e-01,&
     & 7.042138e-01,6.960930e-01,6.894243e-01,6.840459e-01,6.797957e-01,&
     & 6.765119e-01,6.740325e-01,6.721955e-01,6.708391e-01,6.698013e-01,&
     & 6.689201e-01,6.680339e-01,6.669805e-01,6.655982e-01,6.637250e-01,&
     & 6.611992e-01,6.578588e-01,6.535420e-01,6.584449e-01,6.553992e-01,&
     & 6.526547e-01,6.501917e-01,6.479905e-01,6.460313e-01,6.442945e-01,&
     & 6.427605e-01,6.414094e-01,6.402217e-01,6.391775e-01,6.382573e-01,&
     & 6.374413e-01,6.367099e-01,6.360433e-01,6.354218e-01,6.348257e-01,&
     & 6.342355e-01,6.336313e-01,6.329935e-01,6.323023e-01,6.315383e-01,&
     & 6.306814e-01,6.297122e-01,6.286110e-01,6.273579e-01,6.259333e-01,&
     & 6.243176e-01,6.224910e-01,6.2043e-01 /
      data  ssaliq1(:, 18)   /                                          &
     & 9.900163e-01,9.854307e-01,9.797730e-01,9.733113e-01,9.664245e-01,&
     & 9.594976e-01,9.529055e-01,9.470112e-01,9.421695e-01,9.387304e-01,&
     & 9.344918e-01,9.305302e-01,9.267048e-01,9.230072e-01,9.194289e-01,&
     & 9.159616e-01,9.125968e-01,9.093260e-01,9.061409e-01,9.030330e-01,&
     & 8.999940e-01,8.970154e-01,8.940888e-01,8.912058e-01,8.883579e-01,&
     & 8.855368e-01,8.827341e-01,8.799413e-01,8.777423e-01,8.749566e-01,&
     & 8.722298e-01,8.695605e-01,8.669469e-01,8.643875e-01,8.618806e-01,&
     & 8.594246e-01,8.570179e-01,8.546589e-01,8.523459e-01,8.500773e-01,&
     & 8.478516e-01,8.456670e-01,8.435219e-01,8.414148e-01,8.393439e-01,&
     & 8.373078e-01,8.353047e-01,8.333330e-01,8.313911e-01,8.294774e-01,&
     & 8.275904e-01,8.257282e-01,8.238893e-01,8.220721e-01,8.202751e-01,&
     & 8.184965e-01,8.167346e-01,8.1499e-01 /
      data  ssaliq1(:, 19)   /                                          &
     & 9.999916e-01,9.987396e-01,9.966900e-01,9.950738e-01,9.937531e-01,&
     & 9.925912e-01,9.914525e-01,9.902018e-01,9.887046e-01,9.868263e-01,&
     & 9.849039e-01,9.832372e-01,9.815265e-01,9.797770e-01,9.779940e-01,&
     & 9.761827e-01,9.743481e-01,9.724955e-01,9.706303e-01,9.687575e-01,&
     & 9.668823e-01,9.650100e-01,9.631457e-01,9.612947e-01,9.594622e-01,&
     & 9.576534e-01,9.558734e-01,9.541275e-01,9.522059e-01,9.504258e-01,&
     & 9.486459e-01,9.468676e-01,9.450921e-01,9.433208e-01,9.415548e-01,&
     & 9.397955e-01,9.380441e-01,9.363022e-01,9.345706e-01,9.328510e-01,&
     & 9.311445e-01,9.294524e-01,9.277761e-01,9.261167e-01,9.244755e-01,&
     & 9.228540e-01,9.212534e-01,9.196748e-01,9.181197e-01,9.165894e-01,&
     & 9.150851e-01,9.136080e-01,9.121596e-01,9.107410e-01,9.093536e-01,&
     & 9.079987e-01,9.066775e-01,9.0539e-01 /
      data  ssaliq1(:, 20)   /                                          &
     & 9.979493e-01,9.964113e-01,9.950014e-01,9.937045e-01,9.924964e-01,&
     & 9.913546e-01,9.902575e-01,9.891843e-01,9.881136e-01,9.870238e-01,&
     & 9.859934e-01,9.849372e-01,9.838873e-01,9.828434e-01,9.818052e-01,&
     & 9.807725e-01,9.797450e-01,9.787225e-01,9.777047e-01,9.766914e-01,&
     & 9.756823e-01,9.746771e-01,9.736756e-01,9.726775e-01,9.716827e-01,&
     & 9.706907e-01,9.697014e-01,9.687145e-01,9.678060e-01,9.668108e-01,&
     & 9.658218e-01,9.648391e-01,9.638629e-01,9.628936e-01,9.619313e-01,&
     & 9.609763e-01,9.600287e-01,9.590888e-01,9.581569e-01,9.572330e-01,&
     & 9.563176e-01,9.554108e-01,9.545128e-01,9.536239e-01,9.527443e-01,&
     & 9.518741e-01,9.510137e-01,9.501633e-01,9.493230e-01,9.484931e-01,&
     & 9.476740e-01,9.468656e-01,9.460683e-01,9.452824e-01,9.445080e-01,&
     & 9.437454e-01,9.429948e-01,9.4226e-01 /
      data  ssaliq1(:, 21)   /                                          &
     & 9.988742e-01,9.982668e-01,9.976935e-01,9.971497e-01,9.966314e-01,&
     & 9.961344e-01,9.956545e-01,9.951873e-01,9.947286e-01,9.942741e-01,&
     & 9.938457e-01,9.933947e-01,9.929473e-01,9.925032e-01,9.920621e-01,&
     & 9.916237e-01,9.911875e-01,9.907534e-01,9.903209e-01,9.898898e-01,&
     & 9.894597e-01,9.890304e-01,9.886015e-01,9.881726e-01,9.877435e-01,&
     & 9.873138e-01,9.868833e-01,9.864516e-01,9.860698e-01,9.856317e-01,&
     & 9.851957e-01,9.847618e-01,9.843302e-01,9.839008e-01,9.834739e-01,&
     & 9.830494e-01,9.826275e-01,9.822083e-01,9.817918e-01,9.813782e-01,&
     & 9.809675e-01,9.805598e-01,9.801552e-01,9.797538e-01,9.793556e-01,&
     & 9.789608e-01,9.785695e-01,9.781817e-01,9.777975e-01,9.774171e-01,&
     & 9.770404e-01,9.766676e-01,9.762988e-01,9.759340e-01,9.755733e-01,&
     & 9.752169e-01,9.748649e-01,9.7452e-01 /
      data  ssaliq1(:, 22)   /                                          &
     & 9.994441e-01,9.991608e-01,9.988949e-01,9.986439e-01,9.984054e-01,&
     & 9.981768e-01,9.979557e-01,9.977396e-01,9.975258e-01,9.973120e-01,&
     & 9.971011e-01,9.968852e-01,9.966708e-01,9.964578e-01,9.962462e-01,&
     & 9.960357e-01,9.958264e-01,9.956181e-01,9.954108e-01,9.952043e-01,&
     & 9.949987e-01,9.947937e-01,9.945892e-01,9.943853e-01,9.941818e-01,&
     & 9.939786e-01,9.937757e-01,9.935728e-01,9.933922e-01,9.931825e-01,&
     & 9.929739e-01,9.927661e-01,9.925592e-01,9.923534e-01,9.921485e-01,&
     & 9.919447e-01,9.917421e-01,9.915406e-01,9.913403e-01,9.911412e-01,&
     & 9.909435e-01,9.907470e-01,9.905519e-01,9.903581e-01,9.901659e-01,&
     & 9.899751e-01,9.897858e-01,9.895981e-01,9.894120e-01,9.892276e-01,&
     & 9.890447e-01,9.888637e-01,9.886845e-01,9.885070e-01,9.883314e-01,&
     & 9.881576e-01,9.879859e-01,9.8782e-01 /
      data  ssaliq1(:, 23)   /                                          &
     & 9.999138e-01,9.998730e-01,9.998338e-01,9.997965e-01,9.997609e-01,&
     & 9.997270e-01,9.996944e-01,9.996629e-01,9.996321e-01,9.996016e-01,&
     & 9.995690e-01,9.995372e-01,9.995057e-01,9.994744e-01,9.994433e-01,&
     & 9.994124e-01,9.993817e-01,9.993510e-01,9.993206e-01,9.992903e-01,&
     & 9.992600e-01,9.992299e-01,9.991998e-01,9.991698e-01,9.991398e-01,&
     & 9.991098e-01,9.990799e-01,9.990499e-01,9.990231e-01,9.989920e-01,&
     & 9.989611e-01,9.989302e-01,9.988996e-01,9.988690e-01,9.988386e-01,&
     & 9.988084e-01,9.987783e-01,9.987485e-01,9.987187e-01,9.986891e-01,&
     & 9.986598e-01,9.986306e-01,9.986017e-01,9.985729e-01,9.985443e-01,&
     & 9.985160e-01,9.984879e-01,9.984600e-01,9.984324e-01,9.984050e-01,&
     & 9.983778e-01,9.983509e-01,9.983243e-01,9.982980e-01,9.982719e-01,&
     & 9.982461e-01,9.982206e-01,9.9820e-01 /
      data  ssaliq1(:, 24)   /                                          &
     & 9.999985e-01,9.999979e-01,9.999972e-01,9.999966e-01,9.999961e-01,&
     & 9.999955e-01,9.999950e-01,9.999944e-01,9.999938e-01,9.999933e-01,&
     & 9.999927e-01,9.999921e-01,9.999915e-01,9.999910e-01,9.999904e-01,&
     & 9.999899e-01,9.999893e-01,9.999888e-01,9.999882e-01,9.999877e-01,&
     & 9.999871e-01,9.999866e-01,9.999861e-01,9.999855e-01,9.999850e-01,&
     & 9.999844e-01,9.999839e-01,9.999833e-01,9.999828e-01,9.999823e-01,&
     & 9.999817e-01,9.999812e-01,9.999807e-01,9.999801e-01,9.999796e-01,&
     & 9.999791e-01,9.999786e-01,9.999781e-01,9.999776e-01,9.999770e-01,&
     & 9.999765e-01,9.999761e-01,9.999756e-01,9.999751e-01,9.999746e-01,&
     & 9.999741e-01,9.999736e-01,9.999732e-01,9.999727e-01,9.999722e-01,&
     & 9.999718e-01,9.999713e-01,9.999709e-01,9.999705e-01,9.999701e-01,&
     & 9.999697e-01,9.999692e-01,9.9997e-01 /
      data  ssaliq1(:, 25)   /                                          &
     & 9.999999e-01,9.999998e-01,9.999997e-01,9.999997e-01,9.999997e-01,&
     & 9.999996e-01,9.999996e-01,9.999995e-01,9.999995e-01,9.999994e-01,&
     & 9.999994e-01,9.999993e-01,9.999993e-01,9.999992e-01,9.999992e-01,&
     & 9.999991e-01,9.999991e-01,9.999991e-01,9.999990e-01,9.999989e-01,&
     & 9.999989e-01,9.999989e-01,9.999988e-01,9.999988e-01,9.999987e-01,&
     & 9.999987e-01,9.999986e-01,9.999986e-01,9.999985e-01,9.999985e-01,&
     & 9.999984e-01,9.999984e-01,9.999984e-01,9.999983e-01,9.999983e-01,&
     & 9.999982e-01,9.999982e-01,9.999982e-01,9.999981e-01,9.999980e-01,&
     & 9.999980e-01,9.999980e-01,9.999979e-01,9.999979e-01,9.999978e-01,&
     & 9.999978e-01,9.999977e-01,9.999977e-01,9.999977e-01,9.999976e-01,&
     & 9.999976e-01,9.999975e-01,9.999975e-01,9.999974e-01,9.999974e-01,&
     & 9.999974e-01,9.999973e-01,1.0000e+00 /
      data  ssaliq1(:, 26)   /                                          &
     & 9.999997e-01,9.999995e-01,9.999993e-01,9.999992e-01,9.999990e-01,&
     & 9.999989e-01,9.999988e-01,9.999987e-01,9.999986e-01,9.999985e-01,&
     & 9.999984e-01,9.999983e-01,9.999982e-01,9.999981e-01,9.999980e-01,&
     & 9.999978e-01,9.999977e-01,9.999976e-01,9.999975e-01,9.999974e-01,&
     & 9.999973e-01,9.999972e-01,9.999970e-01,9.999969e-01,9.999968e-01,&
     & 9.999967e-01,9.999966e-01,9.999965e-01,9.999964e-01,9.999963e-01,&
     & 9.999962e-01,9.999961e-01,9.999959e-01,9.999958e-01,9.999957e-01,&
     & 9.999956e-01,9.999955e-01,9.999954e-01,9.999953e-01,9.999952e-01,&
     & 9.999951e-01,9.999949e-01,9.999949e-01,9.999947e-01,9.999946e-01,&
     & 9.999945e-01,9.999944e-01,9.999943e-01,9.999942e-01,9.999941e-01,&
     & 9.999940e-01,9.999939e-01,9.999938e-01,9.999937e-01,9.999936e-01,&
     & 9.999935e-01,9.999934e-01,9.9999e-01 /
      data  ssaliq1(:, 27)   /                                          &
     & 9.999984e-01,9.999976e-01,9.999969e-01,9.999962e-01,9.999956e-01,&
     & 9.999950e-01,9.999945e-01,9.999940e-01,9.999935e-01,9.999931e-01,&
     & 9.999926e-01,9.999920e-01,9.999914e-01,9.999908e-01,9.999903e-01,&
     & 9.999897e-01,9.999891e-01,9.999886e-01,9.999880e-01,9.999874e-01,&
     & 9.999868e-01,9.999863e-01,9.999857e-01,9.999851e-01,9.999846e-01,&
     & 9.999840e-01,9.999835e-01,9.999829e-01,9.999824e-01,9.999818e-01,&
     & 9.999812e-01,9.999806e-01,9.999800e-01,9.999795e-01,9.999789e-01,&
     & 9.999783e-01,9.999778e-01,9.999773e-01,9.999767e-01,9.999761e-01,&
     & 9.999756e-01,9.999750e-01,9.999745e-01,9.999739e-01,9.999734e-01,&
     & 9.999729e-01,9.999723e-01,9.999718e-01,9.999713e-01,9.999708e-01,&
     & 9.999703e-01,9.999697e-01,9.999692e-01,9.999687e-01,9.999683e-01,&
     & 9.999678e-01,9.999673e-01,9.9997e-01 /
      data  ssaliq1(:, 28)   /                                          &
     & 9.999981e-01,9.999973e-01,9.999965e-01,9.999958e-01,9.999951e-01,&
     & 9.999943e-01,9.999937e-01,9.999930e-01,9.999924e-01,9.999918e-01,&
     & 9.999912e-01,9.999905e-01,9.999897e-01,9.999890e-01,9.999883e-01,&
     & 9.999876e-01,9.999869e-01,9.999862e-01,9.999855e-01,9.999847e-01,&
     & 9.999840e-01,9.999834e-01,9.999827e-01,9.999819e-01,9.999812e-01,&
     & 9.999805e-01,9.999799e-01,9.999791e-01,9.999785e-01,9.999778e-01,&
     & 9.999771e-01,9.999764e-01,9.999757e-01,9.999750e-01,9.999743e-01,&
     & 9.999736e-01,9.999729e-01,9.999722e-01,9.999715e-01,9.999709e-01,&
     & 9.999701e-01,9.999695e-01,9.999688e-01,9.999682e-01,9.999675e-01,&
     & 9.999669e-01,9.999662e-01,9.999655e-01,9.999649e-01,9.999642e-01,&
     & 9.999636e-01,9.999630e-01,9.999624e-01,9.999618e-01,9.999612e-01,&
     & 9.999606e-01,9.999600e-01,9.9996e-01 /
      data  ssaliq1(:, 29)   /                                          &
     & 8.505737e-01,8.465102e-01,8.394829e-01,8.279508e-01,8.110806e-01,&
     & 7.900397e-01,7.669615e-01,7.444422e-01,7.253055e-01,7.124831e-01,&
     & 7.016434e-01,6.885485e-01,6.767340e-01,6.661029e-01,6.565577e-01,&
     & 6.480013e-01,6.403373e-01,6.334697e-01,6.273034e-01,6.217440e-01,&
     & 6.166983e-01,6.120740e-01,6.077796e-01,6.037249e-01,5.998207e-01,&
     & 5.959788e-01,5.921123e-01,5.881354e-01,5.891285e-01,5.851143e-01,&
     & 5.814653e-01,5.781606e-01,5.751792e-01,5.724998e-01,5.701016e-01,&
     & 5.679634e-01,5.660642e-01,5.643829e-01,5.628984e-01,5.615898e-01,&
     & 5.604359e-01,5.594158e-01,5.585083e-01,5.576924e-01,5.569470e-01,&
     & 5.562512e-01,5.555838e-01,5.549239e-01,5.542503e-01,5.535420e-01,&
     & 5.527781e-01,5.519374e-01,5.509989e-01,5.499417e-01,5.487445e-01,&
     & 5.473865e-01,5.458466e-01,5.4410e-01 /

!  --- ...  asymmetry parameter from hu and stamnes
      data  asyliq1(:, 16)   /                                          &
     & 8.133297e-01,8.133528e-01,8.173865e-01,8.243205e-01,8.333063e-01,&
     & 8.436317e-01,8.546611e-01,8.657934e-01,8.764345e-01,8.859837e-01,&
     & 8.627394e-01,8.824569e-01,8.976887e-01,9.089541e-01,9.167699e-01,&
     & 9.216517e-01,9.241147e-01,9.246743e-01,9.238469e-01,9.221504e-01,&
     & 9.201045e-01,9.182299e-01,9.170491e-01,9.170862e-01,9.188653e-01,&
     & 9.229111e-01,9.297468e-01,9.398950e-01,9.203269e-01,9.260693e-01,&
     & 9.309373e-01,9.349918e-01,9.382935e-01,9.409030e-01,9.428809e-01,&
     & 9.442881e-01,9.451851e-01,9.456331e-01,9.456926e-01,9.454247e-01,&
     & 9.448902e-01,9.441503e-01,9.432661e-01,9.422987e-01,9.413094e-01,&
     & 9.403594e-01,9.395102e-01,9.388230e-01,9.383594e-01,9.381810e-01,&
     & 9.383489e-01,9.389251e-01,9.399707e-01,9.415475e-01,9.437167e-01,&
     & 9.465399e-01,9.500786e-01,9.5439e-01 /
      data  asyliq1(:, 17)   /                                          &
     & 8.794448e-01,8.819306e-01,8.837667e-01,8.853832e-01,8.871010e-01,&
     & 8.892675e-01,8.922584e-01,8.964666e-01,9.022940e-01,9.101456e-01,&
     & 8.839999e-01,9.035610e-01,9.184568e-01,9.292315e-01,9.364282e-01,&
     & 9.405887e-01,9.422554e-01,9.419703e-01,9.402759e-01,9.377159e-01,&
     & 9.348345e-01,9.321769e-01,9.302888e-01,9.297166e-01,9.310075e-01,&
     & 9.347080e-01,9.413643e-01,9.515216e-01,9.306286e-01,9.361781e-01,&
     & 9.408374e-01,9.446692e-01,9.477363e-01,9.501013e-01,9.518268e-01,&
     & 9.529756e-01,9.536105e-01,9.537938e-01,9.535886e-01,9.530574e-01,&
     & 9.522633e-01,9.512688e-01,9.501370e-01,9.489306e-01,9.477126e-01,&
     & 9.465459e-01,9.454934e-01,9.446183e-01,9.439833e-01,9.436519e-01,&
     & 9.436866e-01,9.441508e-01,9.451073e-01,9.466195e-01,9.487501e-01,&
     & 9.515621e-01,9.551185e-01,9.5948e-01 /
      data  asyliq1(:, 18)   /                                          &
     & 8.478817e-01,8.269312e-01,8.161352e-01,8.135960e-01,8.173586e-01,&
     & 8.254167e-01,8.357072e-01,8.461167e-01,8.544952e-01,8.586776e-01,&
     & 8.335562e-01,8.524273e-01,8.669052e-01,8.775014e-01,8.847277e-01,&
     & 8.890958e-01,8.911173e-01,8.913038e-01,8.901669e-01,8.882182e-01,&
     & 8.859692e-01,8.839315e-01,8.826164e-01,8.825356e-01,8.842004e-01,&
     & 8.881223e-01,8.948131e-01,9.047837e-01,8.855951e-01,8.911796e-01,&
     & 8.959229e-01,8.998837e-01,9.031209e-01,9.056939e-01,9.076609e-01,&
     & 9.090812e-01,9.100134e-01,9.105167e-01,9.106496e-01,9.104712e-01,&
     & 9.100404e-01,9.094159e-01,9.086568e-01,9.078218e-01,9.069697e-01,&
     & 9.061595e-01,9.054499e-01,9.048999e-01,9.045683e-01,9.045142e-01,&
     & 9.047962e-01,9.054730e-01,9.066037e-01,9.082472e-01,9.104623e-01,&
     & 9.133079e-01,9.168427e-01,9.2113e-01 /
      data  asyliq1(:, 19)   /                                          &
     & 8.216697e-01,7.982871e-01,7.891147e-01,7.909083e-01,8.003833e-01,&
     & 8.142516e-01,8.292290e-01,8.420356e-01,8.493945e-01,8.480316e-01,&
     & 8.212381e-01,8.394984e-01,8.534095e-01,8.634813e-01,8.702242e-01,&
     & 8.741483e-01,8.757638e-01,8.755808e-01,8.741095e-01,8.718604e-01,&
     & 8.693433e-01,8.670686e-01,8.655464e-01,8.652872e-01,8.668006e-01,&
     & 8.705973e-01,8.771874e-01,8.870809e-01,8.678284e-01,8.732315e-01,&
     & 8.778084e-01,8.816166e-01,8.847146e-01,8.871603e-01,8.890116e-01,&
     & 8.903266e-01,8.911632e-01,8.915796e-01,8.916337e-01,8.913834e-01,&
     & 8.908869e-01,8.902022e-01,8.893873e-01,8.885001e-01,8.875986e-01,&
     & 8.867411e-01,8.859852e-01,8.853891e-01,8.850111e-01,8.849089e-01,&
     & 8.851405e-01,8.857639e-01,8.868372e-01,8.884185e-01,8.905656e-01,&
     & 8.933368e-01,8.967899e-01,9.0098e-01 /
      data  asyliq1(:, 20)   /                                          &
     & 8.063610e-01,7.938147e-01,7.921304e-01,7.985092e-01,8.101339e-01,&
     & 8.242175e-01,8.379913e-01,8.486920e-01,8.535547e-01,8.498083e-01,&
     & 8.224849e-01,8.405509e-01,8.542436e-01,8.640770e-01,8.705653e-01,&
     & 8.742227e-01,8.755630e-01,8.751004e-01,8.733491e-01,8.708231e-01,&
     & 8.680365e-01,8.655035e-01,8.637381e-01,8.632544e-01,8.645665e-01,&
     & 8.681885e-01,8.746346e-01,8.844188e-01,8.648180e-01,8.700563e-01,&
     & 8.744672e-01,8.781087e-01,8.810393e-01,8.833174e-01,8.850011e-01,&
     & 8.861485e-01,8.868183e-01,8.870687e-01,8.869579e-01,8.865441e-01,&
     & 8.858857e-01,8.850412e-01,8.840686e-01,8.830263e-01,8.819726e-01,&
     & 8.809658e-01,8.800642e-01,8.793260e-01,8.788099e-01,8.785737e-01,&
     & 8.786758e-01,8.791746e-01,8.801283e-01,8.815955e-01,8.836340e-01,&
     & 8.863024e-01,8.896592e-01,8.9376e-01 /
      data  asyliq1(:, 21)   /                                          &
     & 7.885899e-01,7.937172e-01,8.020658e-01,8.123971e-01,8.235502e-01,&
     & 8.343776e-01,8.437336e-01,8.504711e-01,8.534421e-01,8.514978e-01,&
     & 8.238888e-01,8.417463e-01,8.552057e-01,8.647853e-01,8.710038e-01,&
     & 8.743798e-01,8.754319e-01,8.746786e-01,8.726386e-01,8.698303e-01,&
     & 8.667724e-01,8.639836e-01,8.619823e-01,8.612870e-01,8.624165e-01,&
     & 8.658893e-01,8.722241e-01,8.819394e-01,8.620216e-01,8.671239e-01,&
     & 8.713983e-01,8.749032e-01,8.776970e-01,8.798385e-01,8.813860e-01,&
     & 8.823980e-01,8.829332e-01,8.830500e-01,8.828068e-01,8.822623e-01,&
     & 8.814750e-01,8.805031e-01,8.794056e-01,8.782407e-01,8.770672e-01,&
     & 8.759432e-01,8.749275e-01,8.740784e-01,8.734547e-01,8.731146e-01,&
     & 8.731170e-01,8.735199e-01,8.743823e-01,8.757625e-01,8.777191e-01,&
     & 8.803105e-01,8.835953e-01,8.8763e-01 /
      data  asyliq1(:, 22)   /                                          &
     & 7.811516e-01,7.962229e-01,8.096199e-01,8.212996e-01,8.312212e-01,&
     & 8.393430e-01,8.456236e-01,8.500214e-01,8.524950e-01,8.530031e-01,&
     & 8.251485e-01,8.429043e-01,8.562461e-01,8.656954e-01,8.717737e-01,&
     & 8.750020e-01,8.759022e-01,8.749953e-01,8.728027e-01,8.698461e-01,&
     & 8.666466e-01,8.637257e-01,8.616047e-01,8.608051e-01,8.618483e-01,&
     & 8.652557e-01,8.715487e-01,8.812485e-01,8.611645e-01,8.662052e-01,&
     & 8.704173e-01,8.738594e-01,8.765901e-01,8.786678e-01,8.801517e-01,&
     & 8.810999e-01,8.815713e-01,8.816246e-01,8.813185e-01,8.807114e-01,&
     & 8.798621e-01,8.788290e-01,8.776713e-01,8.764470e-01,8.752152e-01,&
     & 8.740343e-01,8.729631e-01,8.720602e-01,8.713842e-01,8.709936e-01,&
     & 8.709475e-01,8.713041e-01,8.721221e-01,8.734602e-01,8.753774e-01,&
     & 8.779319e-01,8.811825e-01,8.8519e-01 /
      data  asyliq1(:, 23)   /                                          &
     & 7.865744e-01,8.093340e-01,8.257596e-01,8.369940e-01,8.441574e-01,&
     & 8.483602e-01,8.507096e-01,8.523139e-01,8.542834e-01,8.577321e-01,&
     & 8.288960e-01,8.465308e-01,8.597175e-01,8.689830e-01,8.748542e-01,&
     & 8.778584e-01,8.785222e-01,8.773728e-01,8.749370e-01,8.717419e-01,&
     & 8.683145e-01,8.651816e-01,8.628704e-01,8.619077e-01,8.628205e-01,&
     & 8.661356e-01,8.723803e-01,8.820815e-01,8.616715e-01,8.666389e-01,&
     & 8.707753e-01,8.741398e-01,8.767912e-01,8.787885e-01,8.801908e-01,&
     & 8.810570e-01,8.814460e-01,8.814167e-01,8.810283e-01,8.803395e-01,&
     & 8.794095e-01,8.782971e-01,8.770613e-01,8.757610e-01,8.744553e-01,&
     & 8.732031e-01,8.720634e-01,8.710951e-01,8.703572e-01,8.699086e-01,&
     & 8.698084e-01,8.701155e-01,8.708887e-01,8.721872e-01,8.740698e-01,&
     & 8.765957e-01,8.798235e-01,8.8381e-01 /
      data  asyliq1(:, 24)   /                                          &
     & 8.069513e-01,8.262939e-01,8.398241e-01,8.486352e-01,8.538213e-01,&
     & 8.564743e-01,8.576854e-01,8.585455e-01,8.601452e-01,8.635755e-01,&
     & 8.337383e-01,8.512655e-01,8.643049e-01,8.733896e-01,8.790535e-01,&
     & 8.818295e-01,8.822518e-01,8.808533e-01,8.781676e-01,8.747284e-01,&
     & 8.710690e-01,8.677229e-01,8.652236e-01,8.641047e-01,8.648993e-01,&
     & 8.681413e-01,8.743640e-01,8.841007e-01,8.633558e-01,8.682719e-01,&
     & 8.723543e-01,8.756621e-01,8.782547e-01,8.801915e-01,8.815318e-01,&
     & 8.823347e-01,8.826598e-01,8.825663e-01,8.821135e-01,8.813608e-01,&
     & 8.803674e-01,8.791928e-01,8.778960e-01,8.765366e-01,8.751738e-01,&
     & 8.738670e-01,8.726755e-01,8.716585e-01,8.708755e-01,8.703856e-01,&
     & 8.702483e-01,8.705229e-01,8.712687e-01,8.725448e-01,8.744109e-01,&
     & 8.769260e-01,8.801496e-01,8.8414e-01 /
      data  asyliq1(:, 25)   /                                          &
     & 8.252182e-01,8.379244e-01,8.471709e-01,8.535760e-01,8.577540e-01,&
     & 8.603183e-01,8.618820e-01,8.630578e-01,8.644587e-01,8.666970e-01,&
     & 8.362159e-01,8.536817e-01,8.666387e-01,8.756240e-01,8.811746e-01,&
     & 8.838273e-01,8.841191e-01,8.825871e-01,8.797681e-01,8.761992e-01,&
     & 8.724174e-01,8.689593e-01,8.663623e-01,8.651632e-01,8.658988e-01,&
     & 8.691064e-01,8.753226e-01,8.850847e-01,8.641620e-01,8.690500e-01,&
     & 8.731026e-01,8.763795e-01,8.789400e-01,8.808438e-01,8.821503e-01,&
     & 8.829191e-01,8.832095e-01,8.830813e-01,8.825938e-01,8.818064e-01,&
     & 8.807787e-01,8.795704e-01,8.782408e-01,8.768493e-01,8.754557e-01,&
     & 8.741193e-01,8.728995e-01,8.718561e-01,8.710484e-01,8.705360e-01,&
     & 8.703782e-01,8.706347e-01,8.713650e-01,8.726285e-01,8.744849e-01,&
     & 8.769933e-01,8.802136e-01,8.8421e-01 /
      data  asyliq1(:, 26)   /                                          &
     & 8.370583e-01,8.467920e-01,8.537769e-01,8.585136e-01,8.615034e-01,&
     & 8.632474e-01,8.642468e-01,8.650026e-01,8.660161e-01,8.677882e-01,&
     & 8.369760e-01,8.543821e-01,8.672699e-01,8.761782e-01,8.816454e-01,&
     & 8.842103e-01,8.844114e-01,8.827872e-01,8.798766e-01,8.762179e-01,&
     & 8.723500e-01,8.688112e-01,8.661403e-01,8.648758e-01,8.655563e-01,&
     & 8.687206e-01,8.749072e-01,8.846546e-01,8.636289e-01,8.684849e-01,&
     & 8.725054e-01,8.757501e-01,8.782785e-01,8.801503e-01,8.814249e-01,&
     & 8.821620e-01,8.824211e-01,8.822620e-01,8.817440e-01,8.809268e-01,&
     & 8.798699e-01,8.786330e-01,8.772756e-01,8.758572e-01,8.744374e-01,&
     & 8.730760e-01,8.718323e-01,8.707660e-01,8.699366e-01,8.694039e-01,&
     & 8.692271e-01,8.694661e-01,8.701803e-01,8.714293e-01,8.732727e-01,&
     & 8.757702e-01,8.789811e-01,8.8297e-01 /
      data  asyliq1(:, 27)   /                                          &
     & 8.430819e-01,8.510060e-01,8.567270e-01,8.606533e-01,8.631934e-01,&
     & 8.647554e-01,8.657471e-01,8.665760e-01,8.676496e-01,8.693754e-01,&
     & 8.384298e-01,8.557913e-01,8.686214e-01,8.774605e-01,8.828495e-01,&
     & 8.853287e-01,8.854393e-01,8.837215e-01,8.807161e-01,8.769639e-01,&
     & 8.730053e-01,8.693812e-01,8.666321e-01,8.652988e-01,8.659219e-01,&
     & 8.690419e-01,8.751999e-01,8.849360e-01,8.638013e-01,8.686371e-01,&
     & 8.726369e-01,8.758605e-01,8.783674e-01,8.802176e-01,8.814705e-01,&
     & 8.821859e-01,8.824234e-01,8.822429e-01,8.817038e-01,8.808658e-01,&
     & 8.797887e-01,8.785323e-01,8.771560e-01,8.757196e-01,8.742828e-01,&
     & 8.729052e-01,8.716467e-01,8.705666e-01,8.697250e-01,8.691812e-01,&
     & 8.689950e-01,8.692264e-01,8.699346e-01,8.711795e-01,8.730209e-01,&
     & 8.755181e-01,8.787312e-01,8.8272e-01 /
      data  asyliq1(:, 28)   /                                          &
     & 8.452284e-01,8.522700e-01,8.572973e-01,8.607031e-01,8.628802e-01,&
     & 8.642215e-01,8.651198e-01,8.659679e-01,8.671588e-01,8.690853e-01,&
     & 8.383803e-01,8.557485e-01,8.685851e-01,8.774303e-01,8.828245e-01,&
     & 8.853077e-01,8.854207e-01,8.837034e-01,8.806962e-01,8.769398e-01,&
     & 8.729740e-01,8.693393e-01,8.665761e-01,8.652247e-01,8.658253e-01,&
     & 8.689182e-01,8.750438e-01,8.847424e-01,8.636140e-01,8.684449e-01,&
     & 8.724400e-01,8.756589e-01,8.781613e-01,8.800072e-01,8.812559e-01,&
     & 8.819671e-01,8.822007e-01,8.820165e-01,8.814737e-01,8.806322e-01,&
     & 8.795518e-01,8.782923e-01,8.769129e-01,8.754737e-01,8.740342e-01,&
     & 8.726542e-01,8.713934e-01,8.703111e-01,8.694677e-01,8.689222e-01,&
     & 8.687344e-01,8.689646e-01,8.696715e-01,8.709156e-01,8.727563e-01,&
     & 8.752531e-01,8.784659e-01,8.8245e-01 /
      data  asyliq1(:, 29)   /                                          &
     & 7.800869e-01,8.091120e-01,8.325369e-01,8.466266e-01,8.515495e-01,&
     & 8.499371e-01,8.456203e-01,8.430521e-01,8.470286e-01,8.625431e-01,&
     & 8.402261e-01,8.610822e-01,8.776608e-01,8.904485e-01,8.999294e-01,&
     & 9.065860e-01,9.108995e-01,9.133503e-01,9.144187e-01,9.145855e-01,&
     & 9.143320e-01,9.141402e-01,9.144933e-01,9.158754e-01,9.187716e-01,&
     & 9.236677e-01,9.310503e-01,9.414058e-01,9.239108e-01,9.300719e-01,&
     & 9.353612e-01,9.398378e-01,9.435609e-01,9.465895e-01,9.489829e-01,&
     & 9.508000e-01,9.521002e-01,9.529424e-01,9.533860e-01,9.534902e-01,&
     & 9.533143e-01,9.529177e-01,9.523596e-01,9.516997e-01,9.509973e-01,&
     & 9.503121e-01,9.497037e-01,9.492317e-01,9.489558e-01,9.489356e-01,&
     & 9.492311e-01,9.499019e-01,9.510077e-01,9.526084e-01,9.547636e-01,&
     & 9.575331e-01,9.609766e-01,9.6515e-01 /

!  --- ...  spherical ice particle parameterization from streamer v3
!           extinction units (ext coef/iwc): [(m^-1)/(g m^-3)]
      data  extice2(:, 16)   /                                          &
     & 4.101824e-01,2.435514e-01,1.713697e-01,1.314865e-01,1.063406e-01,&
     & 8.910701e-02,7.659480e-02,6.711784e-02,5.970353e-02,5.375249e-02,&
     & 4.887577e-02,4.481025e-02,4.137171e-02,3.842744e-02,3.587948e-02,&
     & 3.365396e-02,3.169419e-02,2.995593e-02,2.840419e-02,2.701091e-02,&
     & 2.575336e-02,2.461293e-02,2.357423e-02,2.262443e-02,2.175276e-02,&
     & 2.095012e-02,2.020875e-02,1.952199e-02,1.888412e-02,1.829018e-02,&
     & 1.773586e-02,1.721738e-02,1.673144e-02,1.627510e-02,1.584579e-02,&
     & 1.544122e-02,1.505934e-02,1.469833e-02,1.435654e-02,1.403251e-02,&
     & 1.372492e-02,1.343255e-02,1.315433e-02 /
      data  extice2(:, 17)   /                                          &
     & 3.836650e-01,2.304055e-01,1.637265e-01,1.266681e-01,1.031602e-01,&
     & 8.695191e-02,7.511544e-02,6.610009e-02,5.900909e-02,5.328833e-02,&
     & 4.857728e-02,4.463133e-02,4.127880e-02,3.839567e-02,3.589013e-02,&
     & 3.369280e-02,3.175027e-02,3.002079e-02,2.847121e-02,2.707493e-02,&
     & 2.581031e-02,2.465962e-02,2.360815e-02,2.264363e-02,2.175571e-02,&
     & 2.093563e-02,2.017592e-02,1.947015e-02,1.881278e-02,1.819901e-02,&
     & 1.762463e-02,1.708598e-02,1.657982e-02,1.610330e-02,1.565390e-02,&
     & 1.522937e-02,1.482768e-02,1.444706e-02,1.408588e-02,1.374270e-02,&
     & 1.341619e-02,1.310517e-02,1.280857e-02 /
      data  extice2(:, 18)   /                                          &
     & 4.152673e-01,2.436816e-01,1.702243e-01,1.299704e-01,1.047528e-01,&
     & 8.756039e-02,7.513327e-02,6.575690e-02,5.844616e-02,5.259609e-02,&
     & 4.781531e-02,4.383980e-02,4.048517e-02,3.761891e-02,3.514342e-02,&
     & 3.298525e-02,3.108814e-02,2.940825e-02,2.791096e-02,2.656858e-02,&
     & 2.535869e-02,2.426297e-02,2.326627e-02,2.235602e-02,2.152164e-02,&
     & 2.075420e-02,2.004613e-02,1.939091e-02,1.878296e-02,1.821744e-02,&
     & 1.769015e-02,1.719741e-02,1.673600e-02,1.630308e-02,1.589615e-02,&
     & 1.551298e-02,1.515159e-02,1.481021e-02,1.448726e-02,1.418131e-02,&
     & 1.389109e-02,1.361544e-02,1.335330e-02 /
      data  extice2(:, 19)   /                                          &
     & 3.873250e-01,2.331609e-01,1.655002e-01,1.277753e-01,1.038247e-01,&
     & 8.731780e-02,7.527638e-02,6.611873e-02,5.892850e-02,5.313885e-02,&
     & 4.838068e-02,4.440356e-02,4.103167e-02,3.813804e-02,3.562870e-02,&
     & 3.343269e-02,3.149539e-02,2.977414e-02,2.823510e-02,2.685112e-02,&
     & 2.560015e-02,2.446411e-02,2.342805e-02,2.247948e-02,2.160789e-02,&
     & 2.080438e-02,2.006139e-02,1.937238e-02,1.873177e-02,1.813469e-02,&
     & 1.757689e-02,1.705468e-02,1.656479e-02,1.610435e-02,1.567081e-02,&
     & 1.526192e-02,1.487565e-02,1.451020e-02,1.416396e-02,1.383546e-02,&
     & 1.352339e-02,1.322657e-02,1.294392e-02 /
      data  extice2(:, 20)   /                                          &
     & 3.784280e-01,2.291396e-01,1.632551e-01,1.263775e-01,1.028944e-01,&
     & 8.666975e-02,7.480952e-02,6.577335e-02,5.866714e-02,5.293694e-02,&
     & 4.822153e-02,4.427547e-02,4.092626e-02,3.804918e-02,3.555184e-02,&
     & 3.336440e-02,3.143307e-02,2.971577e-02,2.817912e-02,2.679632e-02,&
     & 2.554558e-02,2.440903e-02,2.337187e-02,2.242173e-02,2.154821e-02,&
     & 2.074249e-02,1.999706e-02,1.930546e-02,1.866212e-02,1.806221e-02,&
     & 1.750152e-02,1.697637e-02,1.648352e-02,1.602010e-02,1.558358e-02,&
     & 1.517172e-02,1.478250e-02,1.441413e-02,1.406498e-02,1.373362e-02,&
     & 1.341872e-02,1.311911e-02,1.283371e-02 /
      data  extice2(:, 21)   /                                          &
     & 3.719909e-01,2.259490e-01,1.613144e-01,1.250648e-01,1.019462e-01,&
     & 8.595358e-02,7.425064e-02,6.532618e-02,5.830218e-02,5.263421e-02,&
     & 4.796697e-02,4.405891e-02,4.074013e-02,3.788776e-02,3.541071e-02,&
     & 3.324008e-02,3.132280e-02,2.961733e-02,2.809071e-02,2.671645e-02,&
     & 2.547302e-02,2.434276e-02,2.331102e-02,2.236558e-02,2.149614e-02,&
     & 2.069397e-02,1.995163e-02,1.926272e-02,1.862174e-02,1.802389e-02,&
     & 1.746500e-02,1.694142e-02,1.644994e-02,1.598772e-02,1.555225e-02,&
     & 1.514129e-02,1.475286e-02,1.438515e-02,1.403659e-02,1.370572e-02,&
     & 1.339124e-02,1.309197e-02,1.280685e-02 /
      data  extice2(:, 22)   /                                          &
     & 3.713158e-01,2.253816e-01,1.608461e-01,1.246718e-01,1.016109e-01,&
     & 8.566332e-02,7.399666e-02,6.510199e-02,5.810290e-02,5.245608e-02,&
     & 4.780702e-02,4.391478e-02,4.060989e-02,3.776982e-02,3.530374e-02,&
     & 3.314296e-02,3.123458e-02,2.953719e-02,2.801794e-02,2.665043e-02,&
     & 2.541321e-02,2.428868e-02,2.326224e-02,2.232173e-02,2.145688e-02,&
     & 2.065899e-02,1.992067e-02,1.923552e-02,1.859808e-02,1.800356e-02,&
     & 1.744782e-02,1.692721e-02,1.643855e-02,1.597900e-02,1.554606e-02,&
     & 1.513751e-02,1.475137e-02,1.438586e-02,1.403938e-02,1.371050e-02,&
     & 1.339793e-02,1.310050e-02,1.281713e-02 /
      data  extice2(:, 23)   /                                          &
     & 3.605883e-01,2.204388e-01,1.580431e-01,1.229033e-01,1.004203e-01,&
     & 8.482616e-02,7.338941e-02,6.465105e-02,5.776176e-02,5.219398e-02,&
     & 4.760288e-02,4.375369e-02,4.048111e-02,3.766539e-02,3.521771e-02,&
     & 3.307079e-02,3.117277e-02,2.948303e-02,2.796929e-02,2.660560e-02,&
     & 2.537086e-02,2.424772e-02,2.322182e-02,2.228114e-02,2.141556e-02,&
     & 2.061649e-02,1.987661e-02,1.918962e-02,1.855009e-02,1.795330e-02,&
     & 1.739514e-02,1.687199e-02,1.638069e-02,1.591845e-02,1.548276e-02,&
     & 1.507143e-02,1.468249e-02,1.431416e-02,1.396486e-02,1.363318e-02,&
     & 1.331781e-02,1.301759e-02,1.273147e-02 /
      data  extice2(:, 24)   /                                          &
     & 3.527890e-01,2.168469e-01,1.560090e-01,1.216216e-01,9.955787e-02,&
     & 8.421942e-02,7.294827e-02,6.432192e-02,5.751081e-02,5.199888e-02,&
     & 4.744835e-02,4.362899e-02,4.037847e-02,3.757910e-02,3.514351e-02,&
     & 3.300546e-02,3.111382e-02,2.942853e-02,2.791775e-02,2.655584e-02,&
     & 2.532195e-02,2.419892e-02,2.317255e-02,2.223092e-02,2.136402e-02,&
     & 2.056334e-02,1.982160e-02,1.913258e-02,1.849087e-02,1.789178e-02,&
     & 1.733124e-02,1.680565e-02,1.631187e-02,1.584711e-02,1.540889e-02,&
     & 1.499502e-02,1.460354e-02,1.423269e-02,1.388088e-02,1.354670e-02,&
     & 1.322887e-02,1.292620e-02,1.263767e-02 /
      data  extice2(:, 25)   /                                          &
     & 3.477874e-01,2.143515e-01,1.544887e-01,1.205942e-01,9.881779e-02,&
     & 8.366261e-02,7.251586e-02,6.397790e-02,5.723183e-02,5.176908e-02,&
     & 4.725658e-02,4.346715e-02,4.024055e-02,3.746055e-02,3.504080e-02,&
     & 3.291583e-02,3.103507e-02,2.935891e-02,2.785582e-02,2.650042e-02,&
     & 2.527206e-02,2.415376e-02,2.313142e-02,2.219326e-02,2.132934e-02,&
     & 2.053122e-02,1.979169e-02,1.910456e-02,1.846448e-02,1.786680e-02,&
     & 1.730745e-02,1.678289e-02,1.628998e-02,1.582595e-02,1.538835e-02,&
     & 1.497499e-02,1.458393e-02,1.421341e-02,1.386187e-02,1.352788e-02,&
     & 1.321019e-02,1.290762e-02,1.261913e-02 /
      data  extice2(:, 26)   /                                          &
     & 3.453721e-01,2.130744e-01,1.536698e-01,1.200140e-01,9.838078e-02,&
     & 8.331940e-02,7.223803e-02,6.374775e-02,5.703770e-02,5.160290e-02,&
     & 4.711259e-02,4.334110e-02,4.012923e-02,3.736150e-02,3.495208e-02,&
     & 3.283589e-02,3.096267e-02,2.929302e-02,2.779560e-02,2.644517e-02,&
     & 2.522119e-02,2.410677e-02,2.308788e-02,2.215281e-02,2.129165e-02,&
     & 2.049602e-02,1.975874e-02,1.907365e-02,1.843542e-02,1.783943e-02,&
     & 1.728162e-02,1.675847e-02,1.626685e-02,1.580401e-02,1.536750e-02,&
     & 1.495515e-02,1.456502e-02,1.419537e-02,1.384463e-02,1.351139e-02,&
     & 1.319438e-02,1.289246e-02,1.260456e-02 /
      data  extice2(:, 27)   /                                          &
     & 3.417883e-01,2.113379e-01,1.526395e-01,1.193347e-01,9.790253e-02,&
     & 8.296715e-02,7.196979e-02,6.353806e-02,5.687024e-02,5.146670e-02,&
     & 4.700001e-02,4.324667e-02,4.004894e-02,3.729233e-02,3.489172e-02,&
     & 3.278257e-02,3.091499e-02,2.924987e-02,2.775609e-02,2.640859e-02,&
     & 2.518695e-02,2.407439e-02,2.305697e-02,2.212303e-02,2.126273e-02,&
     & 2.046774e-02,1.973090e-02,1.904610e-02,1.840801e-02,1.781204e-02,&
     & 1.725417e-02,1.673086e-02,1.623902e-02,1.577590e-02,1.533906e-02,&
     & 1.492634e-02,1.453580e-02,1.416571e-02,1.381450e-02,1.348078e-02,&
     & 1.316327e-02,1.286082e-02,1.257240e-02 /
      data  extice2(:, 28)   /                                          &
     & 3.416111e-01,2.114124e-01,1.527734e-01,1.194809e-01,9.804612e-02,&
     & 8.310287e-02,7.209595e-02,6.365442e-02,5.697710e-02,5.156460e-02,&
     & 4.708957e-02,4.332850e-02,4.012361e-02,3.736037e-02,3.495364e-02,&
     & 3.283879e-02,3.096593e-02,2.929589e-02,2.779751e-02,2.644571e-02,&
     & 2.522004e-02,2.410369e-02,2.308271e-02,2.214542e-02,2.128195e-02,&
     & 2.048396e-02,1.974429e-02,1.905679e-02,1.841614e-02,1.781774e-02,&
     & 1.725754e-02,1.673203e-02,1.623807e-02,1.577293e-02,1.533416e-02,&
     & 1.491958e-02,1.452727e-02,1.415547e-02,1.380262e-02,1.346732e-02,&
     & 1.314830e-02,1.284439e-02,1.255456e-02 /
      data  extice2(:, 29)   /                                          &
     & 4.196611e-01,2.493642e-01,1.761261e-01,1.357197e-01,1.102161e-01,&
     & 9.269376e-02,7.992985e-02,7.022538e-02,6.260168e-02,5.645603e-02,&
     & 5.139732e-02,4.716088e-02,4.356133e-02,4.046498e-02,3.777303e-02,&
     & 3.541094e-02,3.332137e-02,3.145954e-02,2.978998e-02,2.828419e-02,&
     & 2.691905e-02,2.567559e-02,2.453811e-02,2.349350e-02,2.253072e-02,&
     & 2.164042e-02,2.081464e-02,2.004652e-02,1.933015e-02,1.866041e-02,&
     & 1.803283e-02,1.744348e-02,1.688894e-02,1.636616e-02,1.587244e-02,&
     & 1.540539e-02,1.496287e-02,1.454295e-02,1.414392e-02,1.376423e-02,&
     & 1.340247e-02,1.305739e-02,1.272784e-02 /

!  --- ...  single-scattering albedo from streamer v3, unitless
      data  ssaice2(:, 16)   /                                          &
     & 6.630615e-01,6.451169e-01,6.333696e-01,6.246927e-01,6.178420e-01,&
     & 6.121976e-01,6.074069e-01,6.032505e-01,5.995830e-01,5.963030e-01,&
     & 5.933372e-01,5.906311e-01,5.881427e-01,5.858395e-01,5.836955e-01,&
     & 5.816896e-01,5.798046e-01,5.780264e-01,5.763429e-01,5.747441e-01,&
     & 5.732213e-01,5.717672e-01,5.703754e-01,5.690403e-01,5.677571e-01,&
     & 5.665215e-01,5.653297e-01,5.641782e-01,5.630643e-01,5.619850e-01,&
     & 5.609381e-01,5.599214e-01,5.589328e-01,5.579707e-01,5.570333e-01,&
     & 5.561193e-01,5.552272e-01,5.543558e-01,5.535041e-01,5.526708e-01,&
     & 5.518551e-01,5.510561e-01,5.502729e-01 /
      data  ssaice2(:, 17)   /                                          &
     & 7.689749e-01,7.398171e-01,7.205819e-01,7.065690e-01,6.956928e-01,&
     & 6.868989e-01,6.795813e-01,6.733606e-01,6.679838e-01,6.632742e-01,&
     & 6.591036e-01,6.553766e-01,6.520197e-01,6.489757e-01,6.461991e-01,&
     & 6.436531e-01,6.413075e-01,6.391375e-01,6.371221e-01,6.352438e-01,&
     & 6.334876e-01,6.318406e-01,6.302918e-01,6.288315e-01,6.274512e-01,&
     & 6.261436e-01,6.249022e-01,6.237211e-01,6.225953e-01,6.215201e-01,&
     & 6.204914e-01,6.195055e-01,6.185592e-01,6.176492e-01,6.167730e-01,&
     & 6.159280e-01,6.151120e-01,6.143228e-01,6.135587e-01,6.128177e-01,&
     & 6.120984e-01,6.113993e-01,6.107189e-01 /
      data  ssaice2(:, 18)   /                                          &
     & 9.956167e-01,9.814770e-01,9.716104e-01,9.639746e-01,9.577179e-01,&
     & 9.524010e-01,9.477672e-01,9.436527e-01,9.399467e-01,9.365708e-01,&
     & 9.334672e-01,9.305921e-01,9.279118e-01,9.253993e-01,9.230330e-01,&
     & 9.207954e-01,9.186719e-01,9.166501e-01,9.147199e-01,9.128722e-01,&
     & 9.110997e-01,9.093956e-01,9.077544e-01,9.061708e-01,9.046406e-01,&
     & 9.031598e-01,9.017248e-01,9.003326e-01,8.989804e-01,8.976655e-01,&
     & 8.963857e-01,8.951389e-01,8.939233e-01,8.927370e-01,8.915785e-01,&
     & 8.904464e-01,8.893392e-01,8.882559e-01,8.871951e-01,8.861559e-01,&
     & 8.851373e-01,8.841383e-01,8.831581e-01 /
      data  ssaice2(:, 19)   /                                          &
     & 9.723177e-01,9.452119e-01,9.267592e-01,9.127393e-01,9.014238e-01,&
     & 8.919334e-01,8.837584e-01,8.765773e-01,8.701736e-01,8.643950e-01,&
     & 8.591299e-01,8.542942e-01,8.498230e-01,8.456651e-01,8.417794e-01,&
     & 8.381324e-01,8.346964e-01,8.314484e-01,8.283687e-01,8.254408e-01,&
     & 8.226505e-01,8.199854e-01,8.174348e-01,8.149891e-01,8.126403e-01,&
     & 8.103808e-01,8.082041e-01,8.061044e-01,8.040765e-01,8.021156e-01,&
     & 8.002174e-01,7.983781e-01,7.965941e-01,7.948622e-01,7.931795e-01,&
     & 7.915432e-01,7.899508e-01,7.884002e-01,7.868891e-01,7.854156e-01,&
     & 7.839779e-01,7.825742e-01,7.812031e-01 /
      data  ssaice2(:, 20)   /                                          &
     & 9.933294e-01,9.860917e-01,9.811564e-01,9.774008e-01,9.743652e-01,&
     & 9.718155e-01,9.696159e-01,9.676810e-01,9.659531e-01,9.643915e-01,&
     & 9.629667e-01,9.616561e-01,9.604426e-01,9.593125e-01,9.582548e-01,&
     & 9.572607e-01,9.563227e-01,9.554347e-01,9.545915e-01,9.537888e-01,&
     & 9.530226e-01,9.522898e-01,9.515874e-01,9.509130e-01,9.502643e-01,&
     & 9.496394e-01,9.490366e-01,9.484542e-01,9.478910e-01,9.473456e-01,&
     & 9.468169e-01,9.463039e-01,9.458056e-01,9.453212e-01,9.448499e-01,&
     & 9.443910e-01,9.439438e-01,9.435077e-01,9.430821e-01,9.426666e-01,&
     & 9.422607e-01,9.418638e-01,9.414756e-01 /
      data  ssaice2(:, 21)   /                                          &
     & 9.900787e-01,9.828880e-01,9.779258e-01,9.741173e-01,9.710184e-01,&
     & 9.684012e-01,9.661332e-01,9.641301e-01,9.623352e-01,9.607083e-01,&
     & 9.592198e-01,9.578474e-01,9.565739e-01,9.553856e-01,9.542715e-01,&
     & 9.532226e-01,9.522314e-01,9.512919e-01,9.503986e-01,9.495472e-01,&
     & 9.487337e-01,9.479549e-01,9.472077e-01,9.464897e-01,9.457985e-01,&
     & 9.451322e-01,9.444890e-01,9.438673e-01,9.432656e-01,9.426826e-01,&
     & 9.421173e-01,9.415684e-01,9.410351e-01,9.405164e-01,9.400115e-01,&
     & 9.395198e-01,9.390404e-01,9.385728e-01,9.381164e-01,9.376707e-01,&
     & 9.372350e-01,9.368091e-01,9.363923e-01 /
      data  ssaice2(:, 22)   /                                          &
     & 9.986793e-01,9.985239e-01,9.983911e-01,9.982715e-01,9.981606e-01,&
     & 9.980562e-01,9.979567e-01,9.978613e-01,9.977691e-01,9.976798e-01,&
     & 9.975929e-01,9.975081e-01,9.974251e-01,9.973438e-01,9.972640e-01,&
     & 9.971855e-01,9.971083e-01,9.970322e-01,9.969571e-01,9.968830e-01,&
     & 9.968099e-01,9.967375e-01,9.966660e-01,9.965951e-01,9.965250e-01,&
     & 9.964555e-01,9.963867e-01,9.963185e-01,9.962508e-01,9.961836e-01,&
     & 9.961170e-01,9.960508e-01,9.959851e-01,9.959198e-01,9.958550e-01,&
     & 9.957906e-01,9.957266e-01,9.956629e-01,9.955997e-01,9.955367e-01,&
     & 9.954742e-01,9.954119e-01,9.953500e-01 /
      data  ssaice2(:, 23)   /                                          &
     & 9.997944e-01,9.997791e-01,9.997664e-01,9.997547e-01,9.997436e-01,&
     & 9.997327e-01,9.997219e-01,9.997110e-01,9.996999e-01,9.996886e-01,&
     & 9.996771e-01,9.996653e-01,9.996533e-01,9.996409e-01,9.996282e-01,&
     & 9.996152e-01,9.996019e-01,9.995883e-01,9.995743e-01,9.995599e-01,&
     & 9.995453e-01,9.995302e-01,9.995149e-01,9.994992e-01,9.994831e-01,&
     & 9.994667e-01,9.994500e-01,9.994329e-01,9.994154e-01,9.993976e-01,&
     & 9.993795e-01,9.993610e-01,9.993422e-01,9.993230e-01,9.993035e-01,&
     & 9.992837e-01,9.992635e-01,9.992429e-01,9.992221e-01,9.992008e-01,&
     & 9.991793e-01,9.991574e-01,9.991352e-01 /
      data  ssaice2(:, 24)   /                                          &
     & 9.999949e-01,9.999947e-01,9.999943e-01,9.999939e-01,9.999934e-01,&
     & 9.999927e-01,9.999920e-01,9.999913e-01,9.999904e-01,9.999895e-01,&
     & 9.999885e-01,9.999874e-01,9.999863e-01,9.999851e-01,9.999838e-01,&
     & 9.999824e-01,9.999810e-01,9.999795e-01,9.999780e-01,9.999764e-01,&
     & 9.999747e-01,9.999729e-01,9.999711e-01,9.999692e-01,9.999673e-01,&
     & 9.999653e-01,9.999632e-01,9.999611e-01,9.999589e-01,9.999566e-01,&
     & 9.999543e-01,9.999519e-01,9.999495e-01,9.999470e-01,9.999444e-01,&
     & 9.999418e-01,9.999392e-01,9.999364e-01,9.999336e-01,9.999308e-01,&
     & 9.999279e-01,9.999249e-01,9.999219e-01 /
      data  ssaice2(:, 25)   /                                          &
     & 9.999997e-01,9.999997e-01,9.999997e-01,9.999996e-01,9.999996e-01,&
     & 9.999995e-01,9.999994e-01,9.999993e-01,9.999993e-01,9.999992e-01,&
     & 9.999991e-01,9.999989e-01,9.999988e-01,9.999987e-01,9.999986e-01,&
     & 9.999984e-01,9.999983e-01,9.999981e-01,9.999980e-01,9.999978e-01,&
     & 9.999976e-01,9.999974e-01,9.999972e-01,9.999971e-01,9.999969e-01,&
     & 9.999966e-01,9.999964e-01,9.999962e-01,9.999960e-01,9.999957e-01,&
     & 9.999955e-01,9.999953e-01,9.999950e-01,9.999947e-01,9.999945e-01,&
     & 9.999942e-01,9.999939e-01,9.999936e-01,9.999934e-01,9.999931e-01,&
     & 9.999928e-01,9.999925e-01,9.999921e-01 /
      data  ssaice2(:, 26)   /                                          &
     & 9.999997e-01,9.999996e-01,9.999996e-01,9.999995e-01,9.999994e-01,&
     & 9.999993e-01,9.999992e-01,9.999991e-01,9.999990e-01,9.999989e-01,&
     & 9.999987e-01,9.999986e-01,9.999984e-01,9.999982e-01,9.999980e-01,&
     & 9.999978e-01,9.999976e-01,9.999974e-01,9.999972e-01,9.999970e-01,&
     & 9.999967e-01,9.999965e-01,9.999962e-01,9.999959e-01,9.999956e-01,&
     & 9.999954e-01,9.999951e-01,9.999947e-01,9.999944e-01,9.999941e-01,&
     & 9.999938e-01,9.999934e-01,9.999931e-01,9.999927e-01,9.999923e-01,&
     & 9.999920e-01,9.999916e-01,9.999912e-01,9.999908e-01,9.999904e-01,&
     & 9.999899e-01,9.999895e-01,9.999891e-01 /
      data  ssaice2(:, 27)   /                                          &
     & 9.999987e-01,9.999987e-01,9.999985e-01,9.999984e-01,9.999982e-01,&
     & 9.999980e-01,9.999978e-01,9.999976e-01,9.999973e-01,9.999970e-01,&
     & 9.999967e-01,9.999964e-01,9.999960e-01,9.999956e-01,9.999952e-01,&
     & 9.999948e-01,9.999944e-01,9.999939e-01,9.999934e-01,9.999929e-01,&
     & 9.999924e-01,9.999918e-01,9.999913e-01,9.999907e-01,9.999901e-01,&
     & 9.999894e-01,9.999888e-01,9.999881e-01,9.999874e-01,9.999867e-01,&
     & 9.999860e-01,9.999853e-01,9.999845e-01,9.999837e-01,9.999829e-01,&
     & 9.999821e-01,9.999813e-01,9.999804e-01,9.999796e-01,9.999787e-01,&
     & 9.999778e-01,9.999768e-01,9.999759e-01 /
      data  ssaice2(:, 28)   /                                          &
     & 9.999989e-01,9.999989e-01,9.999987e-01,9.999986e-01,9.999984e-01,&
     & 9.999982e-01,9.999980e-01,9.999978e-01,9.999975e-01,9.999972e-01,&
     & 9.999969e-01,9.999966e-01,9.999962e-01,9.999958e-01,9.999954e-01,&
     & 9.999950e-01,9.999945e-01,9.999941e-01,9.999936e-01,9.999931e-01,&
     & 9.999925e-01,9.999920e-01,9.999914e-01,9.999908e-01,9.999902e-01,&
     & 9.999896e-01,9.999889e-01,9.999883e-01,9.999876e-01,9.999869e-01,&
     & 9.999861e-01,9.999854e-01,9.999846e-01,9.999838e-01,9.999830e-01,&
     & 9.999822e-01,9.999814e-01,9.999805e-01,9.999796e-01,9.999787e-01,&
     & 9.999778e-01,9.999769e-01,9.999759e-01 /
      data  ssaice2(:, 29)   /                                          &
     & 7.042143e-01,6.691161e-01,6.463240e-01,6.296590e-01,6.166381e-01,&
     & 6.060183e-01,5.970908e-01,5.894144e-01,5.826968e-01,5.767343e-01,&
     & 5.713804e-01,5.665256e-01,5.620867e-01,5.579987e-01,5.542101e-01,&
     & 5.506794e-01,5.473727e-01,5.442620e-01,5.413239e-01,5.385389e-01,&
     & 5.358901e-01,5.333633e-01,5.309460e-01,5.286277e-01,5.263988e-01,&
     & 5.242512e-01,5.221777e-01,5.201719e-01,5.182280e-01,5.163410e-01,&
     & 5.145062e-01,5.127197e-01,5.109776e-01,5.092766e-01,5.076137e-01,&
     & 5.059860e-01,5.043911e-01,5.028266e-01,5.012904e-01,4.997805e-01,&
     & 4.982951e-01,4.968326e-01,4.953913e-01 /

!  --- ...  asymmetry factor from streamer v3, unitless
      data  asyice2(:, 16)   /                                          &
     & 7.946655e-01,8.547685e-01,8.806016e-01,8.949880e-01,9.041676e-01,&
     & 9.105399e-01,9.152249e-01,9.188160e-01,9.216573e-01,9.239620e-01,&
     & 9.258695e-01,9.274745e-01,9.288441e-01,9.300267e-01,9.310584e-01,&
     & 9.319665e-01,9.327721e-01,9.334918e-01,9.341387e-01,9.347236e-01,&
     & 9.352551e-01,9.357402e-01,9.361850e-01,9.365942e-01,9.369722e-01,&
     & 9.373225e-01,9.376481e-01,9.379516e-01,9.382352e-01,9.385010e-01,&
     & 9.387505e-01,9.389854e-01,9.392070e-01,9.394163e-01,9.396145e-01,&
     & 9.398024e-01,9.399809e-01,9.401508e-01,9.403126e-01,9.404670e-01,&
     & 9.406144e-01,9.407555e-01,9.408906e-01 /
      data  asyice2(:, 17)   /                                          &
     & 9.078091e-01,9.195850e-01,9.267250e-01,9.317083e-01,9.354632e-01,&
     & 9.384323e-01,9.408597e-01,9.428935e-01,9.446301e-01,9.461351e-01,&
     & 9.474555e-01,9.486259e-01,9.496722e-01,9.506146e-01,9.514688e-01,&
     & 9.522476e-01,9.529612e-01,9.536181e-01,9.542251e-01,9.547883e-01,&
     & 9.553124e-01,9.558019e-01,9.562601e-01,9.566904e-01,9.570953e-01,&
     & 9.574773e-01,9.578385e-01,9.581806e-01,9.585054e-01,9.588142e-01,&
     & 9.591083e-01,9.593888e-01,9.596569e-01,9.599135e-01,9.601593e-01,&
     & 9.603952e-01,9.606219e-01,9.608399e-01,9.610499e-01,9.612523e-01,&
     & 9.614477e-01,9.616365e-01,9.618192e-01 /
      data  asyice2(:, 18)   /                                          &
     & 8.322045e-01,8.528693e-01,8.648167e-01,8.729163e-01,8.789054e-01,&
     & 8.835845e-01,8.873819e-01,8.905511e-01,8.932532e-01,8.955965e-01,&
     & 8.976567e-01,8.994887e-01,9.011334e-01,9.026221e-01,9.039791e-01,&
     & 9.052237e-01,9.063715e-01,9.074349e-01,9.084245e-01,9.093489e-01,&
     & 9.102154e-01,9.110303e-01,9.117987e-01,9.125253e-01,9.132140e-01,&
     & 9.138682e-01,9.144910e-01,9.150850e-01,9.156524e-01,9.161955e-01,&
     & 9.167160e-01,9.172157e-01,9.176959e-01,9.181581e-01,9.186034e-01,&
     & 9.190330e-01,9.194478e-01,9.198488e-01,9.202368e-01,9.206126e-01,&
     & 9.209768e-01,9.213301e-01,9.216731e-01 /
      data  asyice2(:, 19)   /                                          &
     & 8.116560e-01,8.488278e-01,8.674331e-01,8.788148e-01,8.865810e-01,&
     & 8.922595e-01,8.966149e-01,9.000747e-01,9.028980e-01,9.052513e-01,&
     & 9.072468e-01,9.089632e-01,9.104574e-01,9.117713e-01,9.129371e-01,&
     & 9.139793e-01,9.149174e-01,9.157668e-01,9.165400e-01,9.172473e-01,&
     & 9.178970e-01,9.184962e-01,9.190508e-01,9.195658e-01,9.200455e-01,&
     & 9.204935e-01,9.209130e-01,9.213067e-01,9.216771e-01,9.220262e-01,&
     & 9.223560e-01,9.226680e-01,9.229636e-01,9.232443e-01,9.235112e-01,&
     & 9.237652e-01,9.240074e-01,9.242385e-01,9.244594e-01,9.246708e-01,&
     & 9.248733e-01,9.250674e-01,9.252536e-01 /
      data  asyice2(:, 20)   /                                          &
     & 8.047113e-01,8.402864e-01,8.570332e-01,8.668455e-01,8.733206e-01,&
     & 8.779272e-01,8.813796e-01,8.840676e-01,8.862225e-01,8.879904e-01,&
     & 8.894682e-01,8.907228e-01,8.918019e-01,8.927404e-01,8.935645e-01,&
     & 8.942943e-01,8.949452e-01,8.955296e-01,8.960574e-01,8.965366e-01,&
     & 8.969736e-01,8.973740e-01,8.977422e-01,8.980820e-01,8.983966e-01,&
     & 8.986889e-01,8.989611e-01,8.992153e-01,8.994533e-01,8.996766e-01,&
     & 8.998865e-01,9.000843e-01,9.002709e-01,9.004474e-01,9.006146e-01,&
     & 9.007731e-01,9.009237e-01,9.010670e-01,9.012034e-01,9.013336e-01,&
     & 9.014579e-01,9.015767e-01,9.016904e-01 /
      data  asyice2(:, 21)   /                                          &
     & 8.179122e-01,8.480726e-01,8.621945e-01,8.704354e-01,8.758555e-01,&
     & 8.797007e-01,8.825750e-01,8.848078e-01,8.865939e-01,8.880564e-01,&
     & 8.892765e-01,8.903105e-01,8.911982e-01,8.919689e-01,8.926446e-01,&
     & 8.932419e-01,8.937738e-01,8.942506e-01,8.946806e-01,8.950702e-01,&
     & 8.954251e-01,8.957497e-01,8.960477e-01,8.963223e-01,8.965762e-01,&
     & 8.968116e-01,8.970306e-01,8.972347e-01,8.974255e-01,8.976042e-01,&
     & 8.977720e-01,8.979298e-01,8.980784e-01,8.982188e-01,8.983515e-01,&
     & 8.984771e-01,8.985963e-01,8.987095e-01,8.988171e-01,8.989195e-01,&
     & 8.990172e-01,8.991104e-01,8.991994e-01 /
      data  asyice2(:, 22)   /                                          &
     & 8.169789e-01,8.455024e-01,8.586925e-01,8.663283e-01,8.713217e-01,&
     & 8.748488e-01,8.774765e-01,8.795122e-01,8.811370e-01,8.824649e-01,&
     & 8.835711e-01,8.845073e-01,8.853103e-01,8.860068e-01,8.866170e-01,&
     & 8.871560e-01,8.876358e-01,8.880658e-01,8.884533e-01,8.888044e-01,&
     & 8.891242e-01,8.894166e-01,8.896851e-01,8.899324e-01,8.901612e-01,&
     & 8.903733e-01,8.905706e-01,8.907545e-01,8.909265e-01,8.910876e-01,&
     & 8.912388e-01,8.913812e-01,8.915153e-01,8.916419e-01,8.917617e-01,&
     & 8.918752e-01,8.919829e-01,8.920851e-01,8.921824e-01,8.922751e-01,&
     & 8.923635e-01,8.924478e-01,8.925284e-01 /
      data  asyice2(:, 23)   /                                          &
     & 8.387642e-01,8.569979e-01,8.658630e-01,8.711825e-01,8.747605e-01,&
     & 8.773472e-01,8.793129e-01,8.808621e-01,8.821179e-01,8.831583e-01,&
     & 8.840361e-01,8.847875e-01,8.854388e-01,8.860094e-01,8.865138e-01,&
     & 8.869634e-01,8.873668e-01,8.877310e-01,8.880617e-01,8.883635e-01,&
     & 8.886401e-01,8.888947e-01,8.891298e-01,8.893477e-01,8.895504e-01,&
     & 8.897393e-01,8.899159e-01,8.900815e-01,8.902370e-01,8.903833e-01,&
     & 8.905214e-01,8.906518e-01,8.907753e-01,8.908924e-01,8.910036e-01,&
     & 8.911094e-01,8.912101e-01,8.913062e-01,8.913979e-01,8.914856e-01,&
     & 8.915695e-01,8.916498e-01,8.917269e-01 /
      data  asyice2(:, 24)   /                                          &
     & 8.522208e-01,8.648132e-01,8.711224e-01,8.749901e-01,8.776354e-01,&
     & 8.795743e-01,8.810649e-01,8.822518e-01,8.832225e-01,8.840333e-01,&
     & 8.847224e-01,8.853162e-01,8.858342e-01,8.862906e-01,8.866962e-01,&
     & 8.870595e-01,8.873871e-01,8.876842e-01,8.879551e-01,8.882032e-01,&
     & 8.884316e-01,8.886425e-01,8.888380e-01,8.890199e-01,8.891895e-01,&
     & 8.893481e-01,8.894968e-01,8.896366e-01,8.897683e-01,8.898926e-01,&
     & 8.900102e-01,8.901215e-01,8.902272e-01,8.903276e-01,8.904232e-01,&
     & 8.905144e-01,8.906014e-01,8.906845e-01,8.907640e-01,8.908402e-01,&
     & 8.909132e-01,8.909834e-01,8.910507e-01 /
      data  asyice2(:, 25)   /                                          &
     & 8.578202e-01,8.683033e-01,8.735431e-01,8.767488e-01,8.789378e-01,&
     & 8.805399e-01,8.817701e-01,8.827485e-01,8.835480e-01,8.842152e-01,&
     & 8.847817e-01,8.852696e-01,8.856949e-01,8.860694e-01,8.864020e-01,&
     & 8.866997e-01,8.869681e-01,8.872113e-01,8.874330e-01,8.876360e-01,&
     & 8.878227e-01,8.879951e-01,8.881548e-01,8.883033e-01,8.884418e-01,&
     & 8.885712e-01,8.886926e-01,8.888066e-01,8.889139e-01,8.890152e-01,&
     & 8.891110e-01,8.892017e-01,8.892877e-01,8.893695e-01,8.894473e-01,&
     & 8.895214e-01,8.895921e-01,8.896597e-01,8.897243e-01,8.897862e-01,&
     & 8.898456e-01,8.899025e-01,8.899572e-01 /
      data  asyice2(:, 26)   /                                          &
     & 8.625615e-01,8.713831e-01,8.755799e-01,8.780560e-01,8.796983e-01,&
     & 8.808714e-01,8.817534e-01,8.824420e-01,8.829953e-01,8.834501e-01,&
     & 8.838310e-01,8.841549e-01,8.844338e-01,8.846767e-01,8.848902e-01,&
     & 8.850795e-01,8.852484e-01,8.854002e-01,8.855374e-01,8.856620e-01,&
     & 8.857758e-01,8.858800e-01,8.859759e-01,8.860644e-01,8.861464e-01,&
     & 8.862225e-01,8.862935e-01,8.863598e-01,8.864218e-01,8.864800e-01,&
     & 8.865347e-01,8.865863e-01,8.866349e-01,8.866809e-01,8.867245e-01,&
     & 8.867658e-01,8.868050e-01,8.868423e-01,8.868778e-01,8.869117e-01,&
     & 8.869440e-01,8.869749e-01,8.870044e-01 /
      data  asyice2(:, 27)   /                                          &
     & 8.587495e-01,8.684764e-01,8.728189e-01,8.752872e-01,8.768846e-01,&
     & 8.780060e-01,8.788386e-01,8.794824e-01,8.799960e-01,8.804159e-01,&
     & 8.807660e-01,8.810626e-01,8.813175e-01,8.815390e-01,8.817335e-01,&
     & 8.819057e-01,8.820593e-01,8.821973e-01,8.823220e-01,8.824353e-01,&
     & 8.825387e-01,8.826336e-01,8.827209e-01,8.828016e-01,8.828764e-01,&
     & 8.829459e-01,8.830108e-01,8.830715e-01,8.831283e-01,8.831817e-01,&
     & 8.832320e-01,8.832795e-01,8.833244e-01,8.833668e-01,8.834071e-01,&
     & 8.834454e-01,8.834817e-01,8.835164e-01,8.835495e-01,8.835811e-01,&
     & 8.836113e-01,8.836402e-01,8.836679e-01 /
      data  asyice2(:, 28)   /                                          &
     & 8.561110e-01,8.678583e-01,8.727554e-01,8.753892e-01,8.770154e-01,&
     & 8.781109e-01,8.788949e-01,8.794812e-01,8.799348e-01,8.802952e-01,&
     & 8.805880e-01,8.808300e-01,8.810331e-01,8.812058e-01,8.813543e-01,&
     & 8.814832e-01,8.815960e-01,8.816956e-01,8.817839e-01,8.818629e-01,&
     & 8.819339e-01,8.819979e-01,8.820560e-01,8.821089e-01,8.821573e-01,&
     & 8.822016e-01,8.822425e-01,8.822801e-01,8.823150e-01,8.823474e-01,&
     & 8.823775e-01,8.824056e-01,8.824318e-01,8.824564e-01,8.824795e-01,&
     & 8.825011e-01,8.825215e-01,8.825408e-01,8.825589e-01,8.825761e-01,&
     & 8.825924e-01,8.826078e-01,8.826224e-01 /
      data  asyice2(:, 29)   /                                          &
     & 8.311124e-01,8.688197e-01,8.900274e-01,9.040696e-01,9.142334e-01,&
     & 9.220181e-01,9.282195e-01,9.333048e-01,9.375689e-01,9.412085e-01,&
     & 9.443604e-01,9.471230e-01,9.495694e-01,9.517549e-01,9.537224e-01,&
     & 9.555057e-01,9.571316e-01,9.586222e-01,9.599952e-01,9.612656e-01,&
     & 9.624458e-01,9.635461e-01,9.645756e-01,9.655418e-01,9.664513e-01,&
     & 9.673098e-01,9.681222e-01,9.688928e-01,9.696256e-01,9.703237e-01,&
     & 9.709903e-01,9.716280e-01,9.722391e-01,9.728258e-01,9.733901e-01,&
     & 9.739336e-01,9.744579e-01,9.749645e-01,9.754546e-01,9.759294e-01,&
     & 9.763901e-01,9.768376e-01,9.772727e-01 /

!  --- ...  hexagonal ice particle parameterization from fu
!           extinction units (ext coef/iwc): [(m^-1)/(g m^-3)]
      data  extice3(:, 16)   /                                          &
     & 5.194013e-01,3.215089e-01,2.327917e-01,1.824424e-01,1.499977e-01,&
     & 1.273492e-01,1.106421e-01,9.780982e-02,8.764435e-02,7.939266e-02,&
     & 7.256081e-02,6.681137e-02,6.190600e-02,5.767154e-02,5.397915e-02,&
     & 5.073102e-02,4.785151e-02,4.528125e-02,4.297296e-02,4.088853e-02,&
     & 3.899690e-02,3.727251e-02,3.569411e-02,3.424393e-02,3.290694e-02,&
     & 3.167040e-02,3.052340e-02,2.945654e-02,2.846172e-02,2.753188e-02,&
     & 2.666085e-02,2.584322e-02,2.507423e-02,2.434967e-02,2.366579e-02,&
     & 2.301926e-02,2.240711e-02,2.182666e-02,2.127551e-02,2.075150e-02,&
     & 2.025267e-02,1.977725e-02,1.932364e-02,1.889035e-02,1.847607e-02,&
     & 1.807956e-02 /
      data  extice3(:, 17)   /                                          &
     & 4.901155e-01,3.065286e-01,2.230800e-01,1.753951e-01,1.445402e-01,&
     & 1.229417e-01,1.069777e-01,9.469760e-02,8.495824e-02,7.704501e-02,&
     & 7.048834e-02,6.496693e-02,6.025353e-02,5.618286e-02,5.263186e-02,&
     & 4.950698e-02,4.673585e-02,4.426164e-02,4.203904e-02,4.003153e-02,&
     & 3.820932e-02,3.654790e-02,3.502688e-02,3.362919e-02,3.234041e-02,&
     & 3.114829e-02,3.004234e-02,2.901356e-02,2.805413e-02,2.715727e-02,&
     & 2.631705e-02,2.552828e-02,2.478637e-02,2.408725e-02,2.342734e-02,&
     & 2.280343e-02,2.221264e-02,2.165242e-02,2.112043e-02,2.061461e-02,&
     & 2.013308e-02,1.967411e-02,1.923616e-02,1.881783e-02,1.841781e-02,&
     & 1.803494e-02 /
      data  extice3(:, 18)   /                                          &
     & 5.056264e-01,3.160261e-01,2.298442e-01,1.805973e-01,1.487318e-01,&
     & 1.264258e-01,1.099389e-01,9.725656e-02,8.719819e-02,7.902576e-02,&
     & 7.225433e-02,6.655206e-02,6.168427e-02,5.748028e-02,5.381296e-02,&
     & 5.058572e-02,4.772383e-02,4.516857e-02,4.287317e-02,4.079990e-02,&
     & 3.891801e-02,3.720217e-02,3.563133e-02,3.418786e-02,3.285686e-02,&
     & 3.162569e-02,3.048352e-02,2.942104e-02,2.843018e-02,2.750395e-02,&
     & 2.663621e-02,2.582160e-02,2.505539e-02,2.433337e-02,2.365185e-02,&
     & 2.300750e-02,2.239736e-02,2.181878e-02,2.126937e-02,2.074699e-02,&
     & 2.024968e-02,1.977567e-02,1.932338e-02,1.889134e-02,1.847823e-02,&
     & 1.808281e-02 /
      data  extice3(:, 19)   /                                          &
     & 4.881605e-01,3.055237e-01,2.225070e-01,1.750688e-01,1.443736e-01,&
     & 1.228869e-01,1.070054e-01,9.478893e-02,8.509997e-02,7.722769e-02,&
     & 7.070495e-02,6.521211e-02,6.052311e-02,5.647351e-02,5.294088e-02,&
     & 4.983217e-02,4.707539e-02,4.461398e-02,4.240288e-02,4.040575e-02,&
     & 3.859298e-02,3.694016e-02,3.542701e-02,3.403655e-02,3.275444e-02,&
     & 3.156849e-02,3.046827e-02,2.944481e-02,2.849034e-02,2.759812e-02,&
     & 2.676226e-02,2.597757e-02,2.523949e-02,2.454400e-02,2.388750e-02,&
     & 2.326682e-02,2.267909e-02,2.212176e-02,2.159253e-02,2.108933e-02,&
     & 2.061028e-02,2.015369e-02,1.971801e-02,1.930184e-02,1.890389e-02,&
     & 1.852300e-02 /
      data  extice3(:, 20)   /                                          &
     & 5.103703e-01,3.188144e-01,2.317435e-01,1.819887e-01,1.497944e-01,&
     & 1.272584e-01,1.106013e-01,9.778822e-02,8.762610e-02,7.936938e-02,&
     & 7.252809e-02,6.676701e-02,6.184901e-02,5.760165e-02,5.389651e-02,&
     & 5.063598e-02,4.774457e-02,4.516295e-02,4.284387e-02,4.074922e-02,&
     & 3.884792e-02,3.711438e-02,3.552734e-02,3.406898e-02,3.272425e-02,&
     & 3.148038e-02,3.032643e-02,2.925299e-02,2.825191e-02,2.731612e-02,&
     & 2.643943e-02,2.561642e-02,2.484230e-02,2.411284e-02,2.342429e-02,&
     & 2.277329e-02,2.215686e-02,2.157231e-02,2.101724e-02,2.048946e-02,&
     & 1.998702e-02,1.950813e-02,1.905118e-02,1.861468e-02,1.819730e-02,&
     & 1.779781e-02 /
      data  extice3(:, 21)   /                                          &
     & 5.031161e-01,3.144511e-01,2.286942e-01,1.796903e-01,1.479819e-01,&
     & 1.257860e-01,1.093803e-01,9.676059e-02,8.675183e-02,7.861971e-02,&
     & 7.188168e-02,6.620754e-02,6.136376e-02,5.718050e-02,5.353127e-02,&
     & 5.031995e-02,4.747218e-02,4.492952e-02,4.264544e-02,4.058240e-02,&
     & 3.870979e-02,3.700242e-02,3.543933e-02,3.400297e-02,3.267854e-02,&
     & 3.145345e-02,3.031691e-02,2.925967e-02,2.827370e-02,2.735203e-02,&
     & 2.648858e-02,2.567798e-02,2.491555e-02,2.419710e-02,2.351893e-02,&
     & 2.287776e-02,2.227063e-02,2.169491e-02,2.114821e-02,2.062840e-02,&
     & 2.013354e-02,1.966188e-02,1.921182e-02,1.878191e-02,1.837083e-02,&
     & 1.797737e-02 /
      data  extice3(:, 22)   /                                          &
     & 4.949453e-01,3.095918e-01,2.253402e-01,1.771964e-01,1.460446e-01,&
     & 1.242383e-01,1.081206e-01,9.572235e-02,8.588928e-02,7.789990e-02,&
     & 7.128013e-02,6.570559e-02,6.094684e-02,5.683701e-02,5.325183e-02,&
     & 5.009688e-02,4.729909e-02,4.480106e-02,4.255708e-02,4.053025e-02,&
     & 3.869051e-02,3.701310e-02,3.547745e-02,3.406631e-02,3.276512e-02,&
     & 3.156153e-02,3.044494e-02,2.940626e-02,2.843759e-02,2.753211e-02,&
     & 2.668381e-02,2.588744e-02,2.513839e-02,2.443255e-02,2.376629e-02,&
     & 2.313637e-02,2.253990e-02,2.197428e-02,2.143718e-02,2.092649e-02,&
     & 2.044032e-02,1.997694e-02,1.953478e-02,1.911241e-02,1.870855e-02,&
     & 1.832199e-02 /
      data  extice3(:, 23)   /                                          &
     & 5.052816e-01,3.157665e-01,2.296233e-01,1.803986e-01,1.485473e-01,&
     & 1.262514e-01,1.097718e-01,9.709524e-02,8.704139e-02,7.887264e-02,&
     & 7.210424e-02,6.640454e-02,6.153894e-02,5.733683e-02,5.367116e-02,&
     & 5.044537e-02,4.758477e-02,4.503066e-02,4.273629e-02,4.066395e-02,&
     & 3.878291e-02,3.706784e-02,3.549771e-02,3.405488e-02,3.272448e-02,&
     & 3.149387e-02,3.035221e-02,2.929020e-02,2.829979e-02,2.737397e-02,&
     & 2.650663e-02,2.569238e-02,2.492651e-02,2.420482e-02,2.352361e-02,&
     & 2.287954e-02,2.226968e-02,2.169136e-02,2.114220e-02,2.062005e-02,&
     & 2.012296e-02,1.964917e-02,1.919709e-02,1.876524e-02,1.835231e-02,&
     & 1.795707e-02 /
      data  extice3(:, 24)   /                                          &
     & 5.042067e-01,3.151195e-01,2.291708e-01,1.800573e-01,1.482779e-01,&
     & 1.260324e-01,1.095900e-01,9.694202e-02,8.691087e-02,7.876056e-02,&
     & 7.200745e-02,6.632062e-02,6.146600e-02,5.727338e-02,5.361599e-02,&
     & 5.039749e-02,4.754334e-02,4.499500e-02,4.270580e-02,4.063815e-02,&
     & 3.876135e-02,3.705016e-02,3.548357e-02,3.404400e-02,3.271661e-02,&
     & 3.148877e-02,3.034969e-02,2.929008e-02,2.830191e-02,2.737818e-02,&
     & 2.651279e-02,2.570039e-02,2.493624e-02,2.421618e-02,2.353650e-02,&
     & 2.289390e-02,2.228541e-02,2.170840e-02,2.116048e-02,2.063950e-02,&
     & 2.014354e-02,1.967082e-02,1.921975e-02,1.878888e-02,1.837688e-02,&
     & 1.798254e-02 /
      data  extice3(:, 25)   /                                          &
     & 5.022507e-01,3.139246e-01,2.283218e-01,1.794059e-01,1.477544e-01,&
     & 1.255984e-01,1.092222e-01,9.662516e-02,8.663439e-02,7.851688e-02,&
     & 7.179095e-02,6.612700e-02,6.129193e-02,5.711618e-02,5.347351e-02,&
     & 5.026796e-02,4.742530e-02,4.488721e-02,4.260724e-02,4.054790e-02,&
     & 3.867866e-02,3.697435e-02,3.541407e-02,3.398029e-02,3.265824e-02,&
     & 3.143535e-02,3.030085e-02,2.924551e-02,2.826131e-02,2.734130e-02,&
     & 2.647939e-02,2.567026e-02,2.490919e-02,2.419203e-02,2.351509e-02,&
     & 2.287507e-02,2.226903e-02,2.169434e-02,2.114862e-02,2.062975e-02,&
     & 2.013578e-02,1.966496e-02,1.921571e-02,1.878658e-02,1.837623e-02,&
     & 1.798348e-02 /
      data  extice3(:, 26)   /                                          &
     & 5.068316e-01,3.166869e-01,2.302576e-01,1.808693e-01,1.489122e-01,&
     & 1.265423e-01,1.100080e-01,9.728926e-02,8.720201e-02,7.900612e-02,&
     & 7.221524e-02,6.649660e-02,6.161484e-02,5.739877e-02,5.372093e-02,&
     & 5.048442e-02,4.761431e-02,4.505172e-02,4.274972e-02,4.067050e-02,&
     & 3.878321e-02,3.706244e-02,3.548710e-02,3.403948e-02,3.270466e-02,&
     & 3.146995e-02,3.032450e-02,2.925897e-02,2.826527e-02,2.733638e-02,&
     & 2.646615e-02,2.564920e-02,2.488078e-02,2.415670e-02,2.347322e-02,&
     & 2.282702e-02,2.221513e-02,2.163489e-02,2.108390e-02,2.056002e-02,&
     & 2.006128e-02,1.958591e-02,1.913232e-02,1.869904e-02,1.828474e-02,&
     & 1.788819e-02 /
      data  extice3(:, 27)   /                                          &
     & 5.077707e-01,3.172636e-01,2.306695e-01,1.811871e-01,1.491691e-01,&
     & 1.267565e-01,1.101907e-01,9.744773e-02,8.734125e-02,7.912973e-02,&
     & 7.232591e-02,6.659637e-02,6.170530e-02,5.748120e-02,5.379634e-02,&
     & 5.055367e-02,4.767809e-02,4.511061e-02,4.280423e-02,4.072104e-02,&
     & 3.883015e-02,3.710611e-02,3.552776e-02,3.407738e-02,3.274002e-02,&
     & 3.150296e-02,3.035532e-02,2.928776e-02,2.829216e-02,2.736150e-02,&
     & 2.648961e-02,2.567111e-02,2.490123e-02,2.417576e-02,2.349098e-02,&
     & 2.284354e-02,2.223049e-02,2.164914e-02,2.109711e-02,2.057222e-02,&
     & 2.007253e-02,1.959626e-02,1.914181e-02,1.870770e-02,1.829261e-02,&
     & 1.789531e-02 /
      data  extice3(:, 28)   /                                          &
     & 5.062281e-01,3.163402e-01,2.300275e-01,1.807060e-01,1.487921e-01,&
     & 1.264523e-01,1.099403e-01,9.723879e-02,8.716516e-02,7.898034e-02,&
     & 7.219863e-02,6.648771e-02,6.161254e-02,5.740217e-02,5.372929e-02,&
     & 5.049716e-02,4.763092e-02,4.507179e-02,4.277290e-02,4.069649e-02,&
     & 3.881175e-02,3.709331e-02,3.552008e-02,3.407442e-02,3.274141e-02,&
     & 3.150837e-02,3.036447e-02,2.930037e-02,2.830801e-02,2.738037e-02,&
     & 2.651132e-02,2.569547e-02,2.492810e-02,2.420499e-02,2.352243e-02,&
     & 2.287710e-02,2.226604e-02,2.168658e-02,2.113634e-02,2.061316e-02,&
     & 2.011510e-02,1.964038e-02,1.918740e-02,1.875471e-02,1.834096e-02,&
     & 1.794495e-02 /
      data  extice3(:, 29)   /                                          &
     & 1.338834e-01,1.924912e-01,1.755523e-01,1.534793e-01,1.343937e-01,&
     & 1.187883e-01,1.060654e-01,9.559106e-02,8.685880e-02,7.948698e-02,&
     & 7.319086e-02,6.775669e-02,6.302215e-02,5.886236e-02,5.517996e-02,&
     & 5.189810e-02,4.895539e-02,4.630225e-02,4.389823e-02,4.171002e-02,&
     & 3.970998e-02,3.787493e-02,3.618537e-02,3.462471e-02,3.317880e-02,&
     & 3.183547e-02,3.058421e-02,2.941590e-02,2.832256e-02,2.729724e-02,&
     & 2.633377e-02,2.542675e-02,2.457136e-02,2.376332e-02,2.299882e-02,&
     & 2.227443e-02,2.158707e-02,2.093400e-02,2.031270e-02,1.972091e-02,&
     & 1.915659e-02,1.861787e-02,1.810304e-02,1.761055e-02,1.713899e-02,&
     & 1.668704e-02 /

!  --- ...  single-scattering albedo from fu, unitless
      data  ssaice3(:, 16)   /                                          &
     & 6.749442e-01,6.649947e-01,6.565828e-01,6.489928e-01,6.420046e-01,&
     & 6.355231e-01,6.294964e-01,6.238901e-01,6.186783e-01,6.138395e-01,&
     & 6.093543e-01,6.052049e-01,6.013742e-01,5.978457e-01,5.946030e-01,&
     & 5.916302e-01,5.889115e-01,5.864310e-01,5.841731e-01,5.821221e-01,&
     & 5.802624e-01,5.785785e-01,5.770549e-01,5.756759e-01,5.744262e-01,&
     & 5.732901e-01,5.722524e-01,5.712974e-01,5.704097e-01,5.695739e-01,&
     & 5.687747e-01,5.679964e-01,5.672238e-01,5.664415e-01,5.656340e-01,&
     & 5.647860e-01,5.638821e-01,5.629070e-01,5.618452e-01,5.606815e-01,&
     & 5.594006e-01,5.579870e-01,5.564255e-01,5.547008e-01,5.527976e-01,&
     & 5.507005e-01 /
      data  ssaice3(:, 17)   /                                          &
     & 7.628550e-01,7.567297e-01,7.508463e-01,7.451972e-01,7.397745e-01,&
     & 7.345705e-01,7.295775e-01,7.247881e-01,7.201945e-01,7.157894e-01,&
     & 7.115652e-01,7.075145e-01,7.036300e-01,6.999044e-01,6.963304e-01,&
     & 6.929007e-01,6.896083e-01,6.864460e-01,6.834067e-01,6.804833e-01,&
     & 6.776690e-01,6.749567e-01,6.723397e-01,6.698109e-01,6.673637e-01,&
     & 6.649913e-01,6.626870e-01,6.604441e-01,6.582561e-01,6.561163e-01,&
     & 6.540182e-01,6.519554e-01,6.499215e-01,6.479099e-01,6.459145e-01,&
     & 6.439289e-01,6.419468e-01,6.399621e-01,6.379686e-01,6.359601e-01,&
     & 6.339306e-01,6.318740e-01,6.297845e-01,6.276559e-01,6.254825e-01,&
     & 6.232583e-01 /
      data  ssaice3(:, 18)   /                                          &
     & 9.924147e-01,9.882792e-01,9.842257e-01,9.802522e-01,9.763566e-01,&
     & 9.725367e-01,9.687905e-01,9.651157e-01,9.615104e-01,9.579725e-01,&
     & 9.544997e-01,9.510901e-01,9.477416e-01,9.444520e-01,9.412194e-01,&
     & 9.380415e-01,9.349165e-01,9.318421e-01,9.288164e-01,9.258373e-01,&
     & 9.229027e-01,9.200106e-01,9.171589e-01,9.143457e-01,9.115688e-01,&
     & 9.088263e-01,9.061161e-01,9.034362e-01,9.007846e-01,8.981592e-01,&
     & 8.955581e-01,8.929792e-01,8.904206e-01,8.878803e-01,8.853562e-01,&
     & 8.828464e-01,8.803488e-01,8.778616e-01,8.753827e-01,8.729102e-01,&
     & 8.704421e-01,8.679764e-01,8.655112e-01,8.630445e-01,8.605744e-01,&
     & 8.580989e-01 /
      data  ssaice3(:, 19)   /                                          &
     & 9.629413e-01,9.517182e-01,9.409209e-01,9.305366e-01,9.205529e-01,&
     & 9.109569e-01,9.017362e-01,8.928780e-01,8.843699e-01,8.761992e-01,&
     & 8.683536e-01,8.608204e-01,8.535873e-01,8.466417e-01,8.399712e-01,&
     & 8.335635e-01,8.274062e-01,8.214868e-01,8.157932e-01,8.103129e-01,&
     & 8.050336e-01,7.999432e-01,7.950294e-01,7.902798e-01,7.856825e-01,&
     & 7.812250e-01,7.768954e-01,7.726815e-01,7.685711e-01,7.645522e-01,&
     & 7.606126e-01,7.567404e-01,7.529234e-01,7.491498e-01,7.454074e-01,&
     & 7.416844e-01,7.379688e-01,7.342485e-01,7.305118e-01,7.267468e-01,&
     & 7.229415e-01,7.190841e-01,7.151628e-01,7.111657e-01,7.070811e-01,&
     & 7.028972e-01 /
      data  ssaice3(:, 20)   /                                          &
     & 9.942270e-01,9.909206e-01,9.876775e-01,9.844960e-01,9.813746e-01,&
     & 9.783114e-01,9.753049e-01,9.723535e-01,9.694553e-01,9.666088e-01,&
     & 9.638123e-01,9.610641e-01,9.583626e-01,9.557060e-01,9.530928e-01,&
     & 9.505211e-01,9.479895e-01,9.454961e-01,9.430393e-01,9.406174e-01,&
     & 9.382288e-01,9.358717e-01,9.335446e-01,9.312456e-01,9.289731e-01,&
     & 9.267255e-01,9.245010e-01,9.222980e-01,9.201147e-01,9.179496e-01,&
     & 9.158008e-01,9.136667e-01,9.115457e-01,9.094359e-01,9.073358e-01,&
     & 9.052436e-01,9.031577e-01,9.010763e-01,8.989977e-01,8.969203e-01,&
     & 8.948423e-01,8.927620e-01,8.906778e-01,8.885879e-01,8.864907e-01,&
     & 8.843843e-01 /
      data  ssaice3(:, 21)   /                                          &
     & 9.934014e-01,9.899331e-01,9.865537e-01,9.832610e-01,9.800523e-01,&
     & 9.769254e-01,9.738777e-01,9.709069e-01,9.680106e-01,9.651862e-01,&
     & 9.624315e-01,9.597439e-01,9.571212e-01,9.545608e-01,9.520605e-01,&
     & 9.496177e-01,9.472301e-01,9.448954e-01,9.426111e-01,9.403749e-01,&
     & 9.381843e-01,9.360370e-01,9.339307e-01,9.318629e-01,9.298313e-01,&
     & 9.278336e-01,9.258673e-01,9.239302e-01,9.220198e-01,9.201338e-01,&
     & 9.182700e-01,9.164258e-01,9.145991e-01,9.127874e-01,9.109884e-01,&
     & 9.091999e-01,9.074194e-01,9.056447e-01,9.038735e-01,9.021033e-01,&
     & 9.003320e-01,8.985572e-01,8.967766e-01,8.949879e-01,8.931888e-01,&
     & 8.913770e-01 /
      data  ssaice3(:, 22)   /                                          &
     & 9.994833e-01,9.992055e-01,9.989278e-01,9.986500e-01,9.983724e-01,&
     & 9.980947e-01,9.978172e-01,9.975397e-01,9.972623e-01,9.969849e-01,&
     & 9.967077e-01,9.964305e-01,9.961535e-01,9.958765e-01,9.955997e-01,&
     & 9.953230e-01,9.950464e-01,9.947699e-01,9.944936e-01,9.942174e-01,&
     & 9.939414e-01,9.936656e-01,9.933899e-01,9.931144e-01,9.928390e-01,&
     & 9.925639e-01,9.922889e-01,9.920141e-01,9.917396e-01,9.914652e-01,&
     & 9.911911e-01,9.909171e-01,9.906434e-01,9.903700e-01,9.900967e-01,&
     & 9.898237e-01,9.895510e-01,9.892784e-01,9.890062e-01,9.887342e-01,&
     & 9.884625e-01,9.881911e-01,9.879199e-01,9.876490e-01,9.873784e-01,&
     & 9.871081e-01 /
      data  ssaice3(:, 23)   /                                          &
     & 9.999343e-01,9.998917e-01,9.998492e-01,9.998067e-01,9.997642e-01,&
     & 9.997218e-01,9.996795e-01,9.996372e-01,9.995949e-01,9.995528e-01,&
     & 9.995106e-01,9.994686e-01,9.994265e-01,9.993845e-01,9.993426e-01,&
     & 9.993007e-01,9.992589e-01,9.992171e-01,9.991754e-01,9.991337e-01,&
     & 9.990921e-01,9.990505e-01,9.990089e-01,9.989674e-01,9.989260e-01,&
     & 9.988846e-01,9.988432e-01,9.988019e-01,9.987606e-01,9.987194e-01,&
     & 9.986782e-01,9.986370e-01,9.985959e-01,9.985549e-01,9.985139e-01,&
     & 9.984729e-01,9.984319e-01,9.983910e-01,9.983502e-01,9.983094e-01,&
     & 9.982686e-01,9.982279e-01,9.981872e-01,9.981465e-01,9.981059e-01,&
     & 9.980653e-01 /
      data  ssaice3(:, 24)   /                                          &
     & 9.999978e-01,9.999965e-01,9.999952e-01,9.999939e-01,9.999926e-01,&
     & 9.999913e-01,9.999900e-01,9.999887e-01,9.999873e-01,9.999860e-01,&
     & 9.999847e-01,9.999834e-01,9.999821e-01,9.999808e-01,9.999795e-01,&
     & 9.999782e-01,9.999769e-01,9.999756e-01,9.999743e-01,9.999730e-01,&
     & 9.999717e-01,9.999704e-01,9.999691e-01,9.999678e-01,9.999665e-01,&
     & 9.999652e-01,9.999639e-01,9.999626e-01,9.999613e-01,9.999600e-01,&
     & 9.999587e-01,9.999574e-01,9.999561e-01,9.999548e-01,9.999535e-01,&
     & 9.999522e-01,9.999509e-01,9.999496e-01,9.999483e-01,9.999470e-01,&
     & 9.999457e-01,9.999444e-01,9.999431e-01,9.999418e-01,9.999405e-01,&
     & 9.999392e-01 /
      data  ssaice3(:, 25)   /                                          &
     & 9.999994e-01,9.999993e-01,9.999991e-01,9.999990e-01,9.999989e-01,&
     & 9.999987e-01,9.999986e-01,9.999984e-01,9.999983e-01,9.999982e-01,&
     & 9.999980e-01,9.999979e-01,9.999977e-01,9.999976e-01,9.999975e-01,&
     & 9.999973e-01,9.999972e-01,9.999970e-01,9.999969e-01,9.999967e-01,&
     & 9.999966e-01,9.999965e-01,9.999963e-01,9.999962e-01,9.999960e-01,&
     & 9.999959e-01,9.999957e-01,9.999956e-01,9.999954e-01,9.999953e-01,&
     & 9.999952e-01,9.999950e-01,9.999949e-01,9.999947e-01,9.999946e-01,&
     & 9.999944e-01,9.999943e-01,9.999941e-01,9.999940e-01,9.999939e-01,&
     & 9.999937e-01,9.999936e-01,9.999934e-01,9.999933e-01,9.999931e-01,&
     & 9.999930e-01 /
      data  ssaice3(:, 26)   /                                          &
     & 9.999997e-01,9.999995e-01,9.999992e-01,9.999990e-01,9.999987e-01,&
     & 9.999985e-01,9.999983e-01,9.999980e-01,9.999978e-01,9.999976e-01,&
     & 9.999973e-01,9.999971e-01,9.999969e-01,9.999967e-01,9.999965e-01,&
     & 9.999963e-01,9.999960e-01,9.999958e-01,9.999956e-01,9.999954e-01,&
     & 9.999952e-01,9.999950e-01,9.999948e-01,9.999946e-01,9.999944e-01,&
     & 9.999942e-01,9.999939e-01,9.999937e-01,9.999935e-01,9.999933e-01,&
     & 9.999931e-01,9.999929e-01,9.999927e-01,9.999925e-01,9.999923e-01,&
     & 9.999920e-01,9.999918e-01,9.999916e-01,9.999914e-01,9.999911e-01,&
     & 9.999909e-01,9.999907e-01,9.999905e-01,9.999902e-01,9.999900e-01,&
     & 9.999897e-01 /
      data  ssaice3(:, 27)   /                                          &
     & 9.999991e-01,9.999985e-01,9.999980e-01,9.999974e-01,9.999968e-01,&
     & 9.999963e-01,9.999957e-01,9.999951e-01,9.999946e-01,9.999940e-01,&
     & 9.999934e-01,9.999929e-01,9.999923e-01,9.999918e-01,9.999912e-01,&
     & 9.999907e-01,9.999901e-01,9.999896e-01,9.999891e-01,9.999885e-01,&
     & 9.999880e-01,9.999874e-01,9.999869e-01,9.999863e-01,9.999858e-01,&
     & 9.999853e-01,9.999847e-01,9.999842e-01,9.999836e-01,9.999831e-01,&
     & 9.999826e-01,9.999820e-01,9.999815e-01,9.999809e-01,9.999804e-01,&
     & 9.999798e-01,9.999793e-01,9.999787e-01,9.999782e-01,9.999776e-01,&
     & 9.999770e-01,9.999765e-01,9.999759e-01,9.999754e-01,9.999748e-01,&
     & 9.999742e-01 /
      data  ssaice3(:, 28)   /                                          &
     & 9.999975e-01,9.999961e-01,9.999946e-01,9.999931e-01,9.999917e-01,&
     & 9.999903e-01,9.999888e-01,9.999874e-01,9.999859e-01,9.999845e-01,&
     & 9.999831e-01,9.999816e-01,9.999802e-01,9.999788e-01,9.999774e-01,&
     & 9.999759e-01,9.999745e-01,9.999731e-01,9.999717e-01,9.999702e-01,&
     & 9.999688e-01,9.999674e-01,9.999660e-01,9.999646e-01,9.999631e-01,&
     & 9.999617e-01,9.999603e-01,9.999589e-01,9.999574e-01,9.999560e-01,&
     & 9.999546e-01,9.999532e-01,9.999517e-01,9.999503e-01,9.999489e-01,&
     & 9.999474e-01,9.999460e-01,9.999446e-01,9.999431e-01,9.999417e-01,&
     & 9.999403e-01,9.999388e-01,9.999374e-01,9.999359e-01,9.999345e-01,&
     & 9.999330e-01 /
      data  ssaice3(:, 29)   /                                          &
     & 4.526500e-01,5.287890e-01,5.410487e-01,5.459865e-01,5.485149e-01,&
     & 5.498914e-01,5.505895e-01,5.508310e-01,5.507364e-01,5.503793e-01,&
     & 5.498090e-01,5.490612e-01,5.481637e-01,5.471395e-01,5.460083e-01,&
     & 5.447878e-01,5.434946e-01,5.421442e-01,5.407514e-01,5.393309e-01,&
     & 5.378970e-01,5.364641e-01,5.350464e-01,5.336582e-01,5.323140e-01,&
     & 5.310283e-01,5.298158e-01,5.286914e-01,5.276704e-01,5.267680e-01,&
     & 5.260000e-01,5.253823e-01,5.249311e-01,5.246629e-01,5.245946e-01,&
     & 5.247434e-01,5.251268e-01,5.257626e-01,5.266693e-01,5.278653e-01,&
     & 5.293698e-01,5.312022e-01,5.333823e-01,5.359305e-01,5.388676e-01,&
     & 5.422146e-01 /

!  --- ...  asymmetry factor from fu, unitless
      data  asyice3(:, 16)   /                                          &
     & 8.340752e-01,8.435170e-01,8.517487e-01,8.592064e-01,8.660387e-01,&
     & 8.723204e-01,8.780997e-01,8.834137e-01,8.882934e-01,8.927662e-01,&
     & 8.968577e-01,9.005914e-01,9.039899e-01,9.070745e-01,9.098659e-01,&
     & 9.123836e-01,9.146466e-01,9.166734e-01,9.184817e-01,9.200886e-01,&
     & 9.215109e-01,9.227648e-01,9.238661e-01,9.248304e-01,9.256727e-01,&
     & 9.264078e-01,9.270505e-01,9.276150e-01,9.281156e-01,9.285662e-01,&
     & 9.289806e-01,9.293726e-01,9.297557e-01,9.301435e-01,9.305491e-01,&
     & 9.309859e-01,9.314671e-01,9.320055e-01,9.326140e-01,9.333053e-01,&
     & 9.340919e-01,9.349861e-01,9.360000e-01,9.371451e-01,9.384329e-01,&
     & 9.398744e-01 /
      data  asyice3(:, 17)   /                                          &
     & 8.728160e-01,8.777333e-01,8.823754e-01,8.867535e-01,8.908785e-01,&
     & 8.947611e-01,8.984118e-01,9.018408e-01,9.050582e-01,9.080739e-01,&
     & 9.108976e-01,9.135388e-01,9.160068e-01,9.183106e-01,9.204595e-01,&
     & 9.224620e-01,9.243271e-01,9.260632e-01,9.276788e-01,9.291822e-01,&
     & 9.305817e-01,9.318853e-01,9.331012e-01,9.342372e-01,9.353013e-01,&
     & 9.363013e-01,9.372450e-01,9.381400e-01,9.389939e-01,9.398145e-01,&
     & 9.406092e-01,9.413856e-01,9.421511e-01,9.429131e-01,9.436790e-01,&
     & 9.444561e-01,9.452517e-01,9.460729e-01,9.469270e-01,9.478209e-01,&
     & 9.487617e-01,9.497562e-01,9.508112e-01,9.519335e-01,9.531294e-01,&
     & 9.544055e-01 /
      data  asyice3(:, 18)   /                                          &
     & 7.897566e-01,7.948704e-01,7.998041e-01,8.045623e-01,8.091495e-01,&
     & 8.135702e-01,8.178290e-01,8.219305e-01,8.258790e-01,8.296792e-01,&
     & 8.333355e-01,8.368524e-01,8.402343e-01,8.434856e-01,8.466108e-01,&
     & 8.496143e-01,8.525004e-01,8.552737e-01,8.579384e-01,8.604990e-01,&
     & 8.629597e-01,8.653250e-01,8.675992e-01,8.697867e-01,8.718916e-01,&
     & 8.739185e-01,8.758715e-01,8.777551e-01,8.795734e-01,8.813308e-01,&
     & 8.830315e-01,8.846799e-01,8.862802e-01,8.878366e-01,8.893534e-01,&
     & 8.908350e-01,8.922854e-01,8.937090e-01,8.951099e-01,8.964925e-01,&
     & 8.978609e-01,8.992192e-01,9.005718e-01,9.019229e-01,9.032765e-01,&
     & 9.046369e-01 /
      data  asyice3(:, 19)   /                                          &
     & 7.812615e-01,7.887764e-01,7.959664e-01,8.028413e-01,8.094109e-01,&
     & 8.156849e-01,8.216730e-01,8.273846e-01,8.328294e-01,8.380166e-01,&
     & 8.429556e-01,8.476556e-01,8.521258e-01,8.563753e-01,8.604131e-01,&
     & 8.642481e-01,8.678893e-01,8.713455e-01,8.746254e-01,8.777378e-01,&
     & 8.806914e-01,8.834948e-01,8.861566e-01,8.886854e-01,8.910897e-01,&
     & 8.933779e-01,8.955586e-01,8.976402e-01,8.996311e-01,9.015398e-01,&
     & 9.033745e-01,9.051436e-01,9.068555e-01,9.085185e-01,9.101410e-01,&
     & 9.117311e-01,9.132972e-01,9.148476e-01,9.163905e-01,9.179340e-01,&
     & 9.194864e-01,9.210559e-01,9.226505e-01,9.242784e-01,9.259476e-01,&
     & 9.276661e-01 /
      data  asyice3(:, 20)   /                                          &
     & 7.640720e-01,7.691119e-01,7.739941e-01,7.787222e-01,7.832998e-01,&
     & 7.877304e-01,7.920177e-01,7.961652e-01,8.001765e-01,8.040551e-01,&
     & 8.078044e-01,8.114280e-01,8.149294e-01,8.183119e-01,8.215791e-01,&
     & 8.247344e-01,8.277812e-01,8.307229e-01,8.335629e-01,8.363046e-01,&
     & 8.389514e-01,8.415067e-01,8.439738e-01,8.463560e-01,8.486568e-01,&
     & 8.508795e-01,8.530274e-01,8.551039e-01,8.571122e-01,8.590558e-01,&
     & 8.609378e-01,8.627618e-01,8.645309e-01,8.662485e-01,8.679178e-01,&
     & 8.695423e-01,8.711251e-01,8.726697e-01,8.741792e-01,8.756571e-01,&
     & 8.771065e-01,8.785307e-01,8.799331e-01,8.813169e-01,8.826854e-01,&
     & 8.840419e-01 /
      data  asyice3(:, 21)   /                                          &
     & 7.602598e-01,7.651572e-01,7.699014e-01,7.744962e-01,7.789452e-01,&
     & 7.832522e-01,7.874205e-01,7.914538e-01,7.953555e-01,7.991290e-01,&
     & 8.027777e-01,8.063049e-01,8.097140e-01,8.130081e-01,8.161906e-01,&
     & 8.192645e-01,8.222331e-01,8.250993e-01,8.278664e-01,8.305374e-01,&
     & 8.331153e-01,8.356030e-01,8.380037e-01,8.403201e-01,8.425553e-01,&
     & 8.447121e-01,8.467935e-01,8.488022e-01,8.507412e-01,8.526132e-01,&
     & 8.544210e-01,8.561675e-01,8.578554e-01,8.594875e-01,8.610665e-01,&
     & 8.625951e-01,8.640760e-01,8.655119e-01,8.669055e-01,8.682594e-01,&
     & 8.695763e-01,8.708587e-01,8.721094e-01,8.733308e-01,8.745255e-01,&
     & 8.756961e-01 /
      data  asyice3(:, 22)   /                                          &
     & 7.568957e-01,7.606995e-01,7.644072e-01,7.680204e-01,7.715402e-01,&
     & 7.749682e-01,7.783057e-01,7.815541e-01,7.847148e-01,7.877892e-01,&
     & 7.907786e-01,7.936846e-01,7.965084e-01,7.992515e-01,8.019153e-01,&
     & 8.045011e-01,8.070103e-01,8.094444e-01,8.118048e-01,8.140927e-01,&
     & 8.163097e-01,8.184571e-01,8.205364e-01,8.225488e-01,8.244958e-01,&
     & 8.263789e-01,8.281993e-01,8.299586e-01,8.316580e-01,8.332991e-01,&
     & 8.348831e-01,8.364115e-01,8.378857e-01,8.393071e-01,8.406770e-01,&
     & 8.419969e-01,8.432682e-01,8.444923e-01,8.456706e-01,8.468044e-01,&
     & 8.478952e-01,8.489444e-01,8.499533e-01,8.509234e-01,8.518561e-01,&
     & 8.527528e-01 /
      data  asyice3(:, 23)   /                                          &
     & 7.575066e-01,7.606912e-01,7.638236e-01,7.669035e-01,7.699306e-01,&
     & 7.729046e-01,7.758254e-01,7.786926e-01,7.815060e-01,7.842654e-01,&
     & 7.869705e-01,7.896211e-01,7.922168e-01,7.947574e-01,7.972428e-01,&
     & 7.996726e-01,8.020466e-01,8.043646e-01,8.066262e-01,8.088313e-01,&
     & 8.109796e-01,8.130709e-01,8.151049e-01,8.170814e-01,8.190001e-01,&
     & 8.208608e-01,8.226632e-01,8.244071e-01,8.260924e-01,8.277186e-01,&
     & 8.292856e-01,8.307932e-01,8.322411e-01,8.336291e-01,8.349570e-01,&
     & 8.362244e-01,8.374312e-01,8.385772e-01,8.396621e-01,8.406856e-01,&
     & 8.416476e-01,8.425479e-01,8.433861e-01,8.441620e-01,8.448755e-01,&
     & 8.455263e-01 /
      data  asyice3(:, 24)   /                                          &
     & 7.568829e-01,7.597947e-01,7.626745e-01,7.655212e-01,7.683337e-01,&
     & 7.711111e-01,7.738523e-01,7.765565e-01,7.792225e-01,7.818494e-01,&
     & 7.844362e-01,7.869819e-01,7.894854e-01,7.919459e-01,7.943623e-01,&
     & 7.967337e-01,7.990590e-01,8.013373e-01,8.035676e-01,8.057488e-01,&
     & 8.078802e-01,8.099605e-01,8.119890e-01,8.139645e-01,8.158862e-01,&
     & 8.177530e-01,8.195641e-01,8.213183e-01,8.230149e-01,8.246527e-01,&
     & 8.262308e-01,8.277483e-01,8.292042e-01,8.305976e-01,8.319275e-01,&
     & 8.331929e-01,8.343929e-01,8.355265e-01,8.365928e-01,8.375909e-01,&
     & 8.385197e-01,8.393784e-01,8.401659e-01,8.408815e-01,8.415240e-01,&
     & 8.420926e-01 /
      data  asyice3(:, 25)   /                                          &
     & 7.548616e-01,7.575454e-01,7.602153e-01,7.628696e-01,7.655067e-01,&
     & 7.681249e-01,7.707225e-01,7.732978e-01,7.758492e-01,7.783750e-01,&
     & 7.808735e-01,7.833430e-01,7.857819e-01,7.881886e-01,7.905612e-01,&
     & 7.928983e-01,7.951980e-01,7.974588e-01,7.996789e-01,8.018567e-01,&
     & 8.039905e-01,8.060787e-01,8.081196e-01,8.101115e-01,8.120527e-01,&
     & 8.139416e-01,8.157764e-01,8.175557e-01,8.192776e-01,8.209405e-01,&
     & 8.225427e-01,8.240826e-01,8.255585e-01,8.269688e-01,8.283117e-01,&
     & 8.295856e-01,8.307889e-01,8.319198e-01,8.329767e-01,8.339579e-01,&
     & 8.348619e-01,8.356868e-01,8.364311e-01,8.370930e-01,8.376710e-01,&
     & 8.381633e-01 /
      data  asyice3(:, 26)   /                                          &
     & 7.491854e-01,7.518523e-01,7.545089e-01,7.571534e-01,7.597839e-01,&
     & 7.623987e-01,7.649959e-01,7.675737e-01,7.701303e-01,7.726639e-01,&
     & 7.751727e-01,7.776548e-01,7.801084e-01,7.825318e-01,7.849230e-01,&
     & 7.872804e-01,7.896020e-01,7.918862e-01,7.941309e-01,7.963345e-01,&
     & 7.984951e-01,8.006109e-01,8.026802e-01,8.047009e-01,8.066715e-01,&
     & 8.085900e-01,8.104546e-01,8.122636e-01,8.140150e-01,8.157072e-01,&
     & 8.173382e-01,8.189063e-01,8.204096e-01,8.218464e-01,8.232148e-01,&
     & 8.245130e-01,8.257391e-01,8.268915e-01,8.279682e-01,8.289675e-01,&
     & 8.298875e-01,8.307264e-01,8.314824e-01,8.321537e-01,8.327385e-01,&
     & 8.332350e-01 /
      data  asyice3(:, 27)   /                                          &
     & 7.397086e-01,7.424069e-01,7.450955e-01,7.477725e-01,7.504362e-01,&
     & 7.530846e-01,7.557159e-01,7.583283e-01,7.609199e-01,7.634888e-01,&
     & 7.660332e-01,7.685512e-01,7.710411e-01,7.735009e-01,7.759288e-01,&
     & 7.783229e-01,7.806814e-01,7.830024e-01,7.852841e-01,7.875246e-01,&
     & 7.897221e-01,7.918748e-01,7.939807e-01,7.960380e-01,7.980449e-01,&
     & 7.999995e-01,8.019000e-01,8.037445e-01,8.055311e-01,8.072581e-01,&
     & 8.089235e-01,8.105255e-01,8.120623e-01,8.135319e-01,8.149326e-01,&
     & 8.162626e-01,8.175198e-01,8.187025e-01,8.198089e-01,8.208371e-01,&
     & 8.217852e-01,8.226514e-01,8.234338e-01,8.241306e-01,8.247399e-01,&
     & 8.252599e-01 /
      data  asyice3(:, 28)   /                                          &
     & 7.224533e-01,7.251681e-01,7.278728e-01,7.305654e-01,7.332444e-01,&
     & 7.359078e-01,7.385539e-01,7.411808e-01,7.437869e-01,7.463702e-01,&
     & 7.489291e-01,7.514616e-01,7.539661e-01,7.564408e-01,7.588837e-01,&
     & 7.612933e-01,7.636676e-01,7.660049e-01,7.683034e-01,7.705612e-01,&
     & 7.727767e-01,7.749480e-01,7.770733e-01,7.791509e-01,7.811789e-01,&
     & 7.831556e-01,7.850791e-01,7.869478e-01,7.887597e-01,7.905131e-01,&
     & 7.922062e-01,7.938372e-01,7.954044e-01,7.969059e-01,7.983399e-01,&
     & 7.997047e-01,8.009985e-01,8.022195e-01,8.033658e-01,8.044357e-01,&
     & 8.054275e-01,8.063392e-01,8.071692e-01,8.079157e-01,8.085768e-01,&
     & 8.091507e-01 /
      data  asyice3(:, 29)   /                                          &
     & 8.850026e-01,9.005489e-01,9.069242e-01,9.121799e-01,9.168987e-01,&
     & 9.212259e-01,9.252176e-01,9.289028e-01,9.323000e-01,9.354235e-01,&
     & 9.382858e-01,9.408985e-01,9.432734e-01,9.454218e-01,9.473557e-01,&
     & 9.490871e-01,9.506282e-01,9.519917e-01,9.531904e-01,9.542374e-01,&
     & 9.551461e-01,9.559298e-01,9.566023e-01,9.571775e-01,9.576692e-01,&
     & 9.580916e-01,9.584589e-01,9.587853e-01,9.590851e-01,9.593729e-01,&
     & 9.596632e-01,9.599705e-01,9.603096e-01,9.606954e-01,9.611427e-01,&
     & 9.616667e-01,9.622826e-01,9.630060e-01,9.638524e-01,9.648379e-01,&
     & 9.659788e-01,9.672916e-01,9.687933e-01,9.705014e-01,9.724337e-01,&
     & 9.746084e-01 /

!  --- ...  fdelta from fu, unitless
      data  fdlice3(:, 16)   /                                          &
     & 4.959277e-02,4.685292e-02,4.426104e-02,4.181231e-02,3.950191e-02,&
     & 3.732500e-02,3.527675e-02,3.335235e-02,3.154697e-02,2.985578e-02,&
     & 2.827395e-02,2.679666e-02,2.541909e-02,2.413640e-02,2.294378e-02,&
     & 2.183639e-02,2.080940e-02,1.985801e-02,1.897736e-02,1.816265e-02,&
     & 1.740905e-02,1.671172e-02,1.606585e-02,1.546661e-02,1.490917e-02,&
     & 1.438870e-02,1.390038e-02,1.343939e-02,1.300089e-02,1.258006e-02,&
     & 1.217208e-02,1.177212e-02,1.137536e-02,1.097696e-02,1.057210e-02,&
     & 1.015596e-02,9.723704e-03,9.270516e-03,8.791565e-03,8.282026e-03,&
     & 7.737072e-03,7.151879e-03,6.521619e-03,5.841467e-03,5.106597e-03,&
     & 4.312183e-03 /
      data  fdlice3(:, 17)   /                                          &
     & 5.071224e-02,5.000217e-02,4.933872e-02,4.871992e-02,4.814380e-02,&
     & 4.760839e-02,4.711170e-02,4.665177e-02,4.622662e-02,4.583426e-02,&
     & 4.547274e-02,4.514007e-02,4.483428e-02,4.455340e-02,4.429544e-02,&
     & 4.405844e-02,4.384041e-02,4.363939e-02,4.345340e-02,4.328047e-02,&
     & 4.311861e-02,4.296586e-02,4.282024e-02,4.267977e-02,4.254248e-02,&
     & 4.240640e-02,4.226955e-02,4.212995e-02,4.198564e-02,4.183462e-02,&
     & 4.167494e-02,4.150462e-02,4.132167e-02,4.112413e-02,4.091003e-02,&
     & 4.067737e-02,4.042420e-02,4.014854e-02,3.984840e-02,3.952183e-02,&
     & 3.916683e-02,3.878144e-02,3.836368e-02,3.791158e-02,3.742316e-02,&
     & 3.689645e-02 /
      data  fdlice3(:, 18)   /                                          &
     & 1.062938e-01,1.065234e-01,1.067822e-01,1.070682e-01,1.073793e-01,&
     & 1.077137e-01,1.080693e-01,1.084442e-01,1.088364e-01,1.092439e-01,&
     & 1.096647e-01,1.100970e-01,1.105387e-01,1.109878e-01,1.114423e-01,&
     & 1.119004e-01,1.123599e-01,1.128190e-01,1.132757e-01,1.137279e-01,&
     & 1.141738e-01,1.146113e-01,1.150385e-01,1.154534e-01,1.158540e-01,&
     & 1.162383e-01,1.166045e-01,1.169504e-01,1.172741e-01,1.175738e-01,&
     & 1.178472e-01,1.180926e-01,1.183080e-01,1.184913e-01,1.186405e-01,&
     & 1.187538e-01,1.188291e-01,1.188645e-01,1.188580e-01,1.188076e-01,&
     & 1.187113e-01,1.185672e-01,1.183733e-01,1.181277e-01,1.178282e-01,&
     & 1.174731e-01 /
      data  fdlice3(:, 19)   /                                          &
     & 1.076195e-01,1.065195e-01,1.054696e-01,1.044673e-01,1.035099e-01,&
     & 1.025951e-01,1.017203e-01,1.008831e-01,1.000808e-01,9.931116e-02,&
     & 9.857151e-02,9.785939e-02,9.717230e-02,9.650774e-02,9.586322e-02,&
     & 9.523623e-02,9.462427e-02,9.402484e-02,9.343544e-02,9.285358e-02,&
     & 9.227675e-02,9.170245e-02,9.112818e-02,9.055144e-02,8.996974e-02,&
     & 8.938056e-02,8.878142e-02,8.816981e-02,8.754323e-02,8.689919e-02,&
     & 8.623517e-02,8.554869e-02,8.483724e-02,8.409832e-02,8.332943e-02,&
     & 8.252807e-02,8.169175e-02,8.081795e-02,7.990419e-02,7.894796e-02,&
     & 7.794676e-02,7.689809e-02,7.579945e-02,7.464834e-02,7.344227e-02,&
     & 7.217872e-02 /
      data  fdlice3(:, 20)   /                                          &
     & 1.119014e-01,1.122706e-01,1.126690e-01,1.130947e-01,1.135456e-01,&
     & 1.140199e-01,1.145154e-01,1.150302e-01,1.155623e-01,1.161096e-01,&
     & 1.166703e-01,1.172422e-01,1.178233e-01,1.184118e-01,1.190055e-01,&
     & 1.196025e-01,1.202008e-01,1.207983e-01,1.213931e-01,1.219832e-01,&
     & 1.225665e-01,1.231411e-01,1.237050e-01,1.242561e-01,1.247926e-01,&
     & 1.253122e-01,1.258132e-01,1.262934e-01,1.267509e-01,1.271836e-01,&
     & 1.275896e-01,1.279669e-01,1.283134e-01,1.286272e-01,1.289063e-01,&
     & 1.291486e-01,1.293522e-01,1.295150e-01,1.296351e-01,1.297104e-01,&
     & 1.297390e-01,1.297189e-01,1.296480e-01,1.295244e-01,1.293460e-01,&
     & 1.291109e-01 /
      data  fdlice3(:, 21)   /                                          &
     & 1.133298e-01,1.136777e-01,1.140556e-01,1.144615e-01,1.148934e-01,&
     & 1.153492e-01,1.158269e-01,1.163243e-01,1.168396e-01,1.173706e-01,&
     & 1.179152e-01,1.184715e-01,1.190374e-01,1.196108e-01,1.201897e-01,&
     & 1.207720e-01,1.213558e-01,1.219389e-01,1.225194e-01,1.230951e-01,&
     & 1.236640e-01,1.242241e-01,1.247733e-01,1.253096e-01,1.258309e-01,&
     & 1.263352e-01,1.268205e-01,1.272847e-01,1.277257e-01,1.281415e-01,&
     & 1.285300e-01,1.288893e-01,1.292173e-01,1.295118e-01,1.297710e-01,&
     & 1.299927e-01,1.301748e-01,1.303154e-01,1.304124e-01,1.304637e-01,&
     & 1.304673e-01,1.304212e-01,1.303233e-01,1.301715e-01,1.299638e-01,&
     & 1.296983e-01 /
      data  fdlice3(:, 22)   /                                          &
     & 1.145360e-01,1.153256e-01,1.161453e-01,1.169929e-01,1.178666e-01,&
     & 1.187641e-01,1.196835e-01,1.206227e-01,1.215796e-01,1.225522e-01,&
     & 1.235383e-01,1.245361e-01,1.255433e-01,1.265579e-01,1.275779e-01,&
     & 1.286011e-01,1.296257e-01,1.306494e-01,1.316703e-01,1.326862e-01,&
     & 1.336951e-01,1.346950e-01,1.356838e-01,1.366594e-01,1.376198e-01,&
     & 1.385629e-01,1.394866e-01,1.403889e-01,1.412678e-01,1.421212e-01,&
     & 1.429469e-01,1.437430e-01,1.445074e-01,1.452381e-01,1.459329e-01,&
     & 1.465899e-01,1.472069e-01,1.477819e-01,1.483128e-01,1.487976e-01,&
     & 1.492343e-01,1.496207e-01,1.499548e-01,1.502346e-01,1.504579e-01,&
     & 1.506227e-01 /
      data  fdlice3(:, 23)   /                                          &
     & 1.153263e-01,1.161445e-01,1.169932e-01,1.178703e-01,1.187738e-01,&
     & 1.197016e-01,1.206516e-01,1.216217e-01,1.226099e-01,1.236141e-01,&
     & 1.246322e-01,1.256621e-01,1.267017e-01,1.277491e-01,1.288020e-01,&
     & 1.298584e-01,1.309163e-01,1.319736e-01,1.330281e-01,1.340778e-01,&
     & 1.351207e-01,1.361546e-01,1.371775e-01,1.381873e-01,1.391820e-01,&
     & 1.401593e-01,1.411174e-01,1.420540e-01,1.429671e-01,1.438547e-01,&
     & 1.447146e-01,1.455449e-01,1.463433e-01,1.471078e-01,1.478364e-01,&
     & 1.485270e-01,1.491774e-01,1.497857e-01,1.503497e-01,1.508674e-01,&
     & 1.513367e-01,1.517554e-01,1.521216e-01,1.524332e-01,1.526880e-01,&
     & 1.528840e-01 /
      data  fdlice3(:, 24)   /                                          &
     & 1.160842e-01,1.169118e-01,1.177697e-01,1.186556e-01,1.195676e-01,&
     & 1.205036e-01,1.214616e-01,1.224394e-01,1.234349e-01,1.244463e-01,&
     & 1.254712e-01,1.265078e-01,1.275539e-01,1.286075e-01,1.296664e-01,&
     & 1.307287e-01,1.317923e-01,1.328550e-01,1.339149e-01,1.349699e-01,&
     & 1.360179e-01,1.370567e-01,1.380845e-01,1.390991e-01,1.400984e-01,&
     & 1.410803e-01,1.420429e-01,1.429840e-01,1.439016e-01,1.447936e-01,&
     & 1.456579e-01,1.464925e-01,1.472953e-01,1.480642e-01,1.487972e-01,&
     & 1.494923e-01,1.501472e-01,1.507601e-01,1.513287e-01,1.518511e-01,&
     & 1.523252e-01,1.527489e-01,1.531201e-01,1.534368e-01,1.536969e-01,&
     & 1.538984e-01 /
      data  fdlice3(:, 25)   /                                          &
     & 1.168725e-01,1.177088e-01,1.185747e-01,1.194680e-01,1.203867e-01,&
     & 1.213288e-01,1.222923e-01,1.232750e-01,1.242750e-01,1.252903e-01,&
     & 1.263187e-01,1.273583e-01,1.284069e-01,1.294626e-01,1.305233e-01,&
     & 1.315870e-01,1.326517e-01,1.337152e-01,1.347756e-01,1.358308e-01,&
     & 1.368788e-01,1.379175e-01,1.389449e-01,1.399590e-01,1.409577e-01,&
     & 1.419389e-01,1.429007e-01,1.438410e-01,1.447577e-01,1.456488e-01,&
     & 1.465123e-01,1.473461e-01,1.481483e-01,1.489166e-01,1.496492e-01,&
     & 1.503439e-01,1.509988e-01,1.516118e-01,1.521808e-01,1.527038e-01,&
     & 1.531788e-01,1.536037e-01,1.539764e-01,1.542951e-01,1.545575e-01,&
     & 1.547617e-01 /
      data  fdlice3(:, 26)   /                                          &
     & 1.180509e-01,1.189025e-01,1.197820e-01,1.206875e-01,1.216171e-01,&
     & 1.225687e-01,1.235404e-01,1.245303e-01,1.255363e-01,1.265564e-01,&
     & 1.275888e-01,1.286313e-01,1.296821e-01,1.307392e-01,1.318006e-01,&
     & 1.328643e-01,1.339284e-01,1.349908e-01,1.360497e-01,1.371029e-01,&
     & 1.381486e-01,1.391848e-01,1.402095e-01,1.412208e-01,1.422165e-01,&
     & 1.431949e-01,1.441539e-01,1.450915e-01,1.460058e-01,1.468947e-01,&
     & 1.477564e-01,1.485888e-01,1.493900e-01,1.501580e-01,1.508907e-01,&
     & 1.515864e-01,1.522428e-01,1.528582e-01,1.534305e-01,1.539578e-01,&
     & 1.544380e-01,1.548692e-01,1.552494e-01,1.555767e-01,1.558490e-01,&
     & 1.560645e-01 /
      data  fdlice3(:, 27)   /                                          &
     & 1.200480e-01,1.209267e-01,1.218304e-01,1.227575e-01,1.237059e-01,&
     & 1.246739e-01,1.256595e-01,1.266610e-01,1.276765e-01,1.287041e-01,&
     & 1.297420e-01,1.307883e-01,1.318412e-01,1.328988e-01,1.339593e-01,&
     & 1.350207e-01,1.360813e-01,1.371393e-01,1.381926e-01,1.392396e-01,&
     & 1.402783e-01,1.413069e-01,1.423235e-01,1.433263e-01,1.443134e-01,&
     & 1.452830e-01,1.462332e-01,1.471622e-01,1.480681e-01,1.489490e-01,&
     & 1.498032e-01,1.506286e-01,1.514236e-01,1.521863e-01,1.529147e-01,&
     & 1.536070e-01,1.542614e-01,1.548761e-01,1.554491e-01,1.559787e-01,&
     & 1.564629e-01,1.568999e-01,1.572879e-01,1.576249e-01,1.579093e-01,&
     & 1.581390e-01 /
      data  fdlice3(:, 28)   /                                          &
     & 1.247813e-01,1.256496e-01,1.265417e-01,1.274560e-01,1.283905e-01,&
     & 1.293436e-01,1.303135e-01,1.312983e-01,1.322964e-01,1.333060e-01,&
     & 1.343252e-01,1.353523e-01,1.363855e-01,1.374231e-01,1.384632e-01,&
     & 1.395042e-01,1.405441e-01,1.415813e-01,1.426140e-01,1.436404e-01,&
     & 1.446587e-01,1.456672e-01,1.466640e-01,1.476475e-01,1.486157e-01,&
     & 1.495671e-01,1.504997e-01,1.514117e-01,1.523016e-01,1.531673e-01,&
     & 1.540073e-01,1.548197e-01,1.556026e-01,1.563545e-01,1.570734e-01,&
     & 1.577576e-01,1.584054e-01,1.590149e-01,1.595843e-01,1.601120e-01,&
     & 1.605962e-01,1.610349e-01,1.614266e-01,1.617693e-01,1.620614e-01,&
     & 1.623011e-01 /
      data  fdlice3(:, 29)   /                                          &
     & 1.006055e-01,9.549582e-02,9.063960e-02,8.602900e-02,8.165612e-02,&
     & 7.751308e-02,7.359199e-02,6.988496e-02,6.638412e-02,6.308156e-02,&
     & 5.996942e-02,5.703979e-02,5.428481e-02,5.169657e-02,4.926719e-02,&
     & 4.698880e-02,4.485349e-02,4.285339e-02,4.098061e-02,3.922727e-02,&
     & 3.758547e-02,3.604733e-02,3.460497e-02,3.325051e-02,3.197604e-02,&
     & 3.077369e-02,2.963558e-02,2.855381e-02,2.752050e-02,2.652776e-02,&
     & 2.556772e-02,2.463247e-02,2.371415e-02,2.280485e-02,2.189670e-02,&
     & 2.098180e-02,2.005228e-02,1.910024e-02,1.811781e-02,1.709709e-02,&
     & 1.603020e-02,1.490925e-02,1.372635e-02,1.247363e-02,1.114319e-02,&
     & 9.727157e-03 /

!  --- ...  coefficients to compute tau, ssa, asy for rain drop and
!           (chou 1999), and snowflake (fu 2001 private communication)
      real (kind=kind_phys), public  :: a0r, a1r, a0s, a1s
      data a0r,a1r / 3.07e-3, 0.0 /,    a0s,a1s / 0.0,     1.5 /  ! fu's coeff

      real (kind=kind_phys), dimension(nblow:nbhgh), public ::          &
     &       b0r, b0s, b1s, c0r, c0s
      data b0r  / 0.466, 0.437, 0.416, 0.391, 0.374, 0.352, 0.183,      &
     &            0.048, 0.012, 0.000, 0.000, 0.000, 0.000, 0.496 /
      data c0r  / 0.975, 0.965, 0.960, 0.955, 0.952, 0.950, 0.944,      &
     &            0.894, 0.884, 0.883, 0.883, 0.883, 0.883, 0.980 /
      data b0s  / 7*0.460, 2*0.000,   4*0.000, 0.460 /
      data b1s  / 7*0.000, 2*1.62e-5, 4*0.000, 0.000 /
      data c0s  / 7*0.970, 2*0.970,   4*0.700, 0.970 /


!........................................!
      end module module_radsw_cldprtb    !
!========================================!


!========================================!
      module module_radsw_sflux          !
!........................................!
!
! ***********************  module descriptions  ********************** !
!                                                                      !
!     this module contains spectral distribution of solar radiation    !
!     flux used to obtain the incoming solar flux at toa.              !
!                                                                      !
!     modify history:                                                  !
!      originally by j.delamere, atmospheric & environmental research  !
!                          in 14 bands kgb data table.                 !
!      feb. 18, 2004   --  yu-tai hou    move the reference data table !
!                          to a new data module.                       !
!                                                                      !
! *************************  end description  ************************ !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NGMAX, NG16, NG17, NG18, NG19,&
     &                                    NG20, NG21, NG22, NG23, NG24, &
     &                                    NG25, NG26, NG27, NG28, NG29, &
     &                                    nblow, nbhgh
!
      implicit none
!
      public
!
      integer, parameter :: MFS01 = 1      !
      integer, parameter :: MFS02 = 5      !
      integer, parameter :: MFS03 = 9      !
      integer, parameter :: MFB01 = 7      !
      integer, parameter :: MFB02 = 2      !
      integer, parameter :: MFB03 = 5      !
!
      real (kind=kind_phys), dimension(nblow:nbhgh), public ::          &
     &       strrat, specwt

!  ---  original strrat
!     data  strrat  / 2.52131e+2, 3.64641e-1, 3.89589e+1, 5.49281e+0,   &
!    &    0.00000e+0, 4.53210e-3, 2.27080e-2, 0.00000e+0, 1.24692e-1,   &
!    &    0.00000e+0, 0.00000e+0, 0.00000e+0, 6.67029e-7, 0.00000e+0 /
!  ---  strrat(22) has been multified by factor o2adj=1.6
      data  strrat  / 2.52131e+2, 3.64641e-1, 3.89589e+1, 5.49281e+0,   &
     &    0.00000e+0, 4.53210e-3, 3.63328e-2, 0.00000e+0, 1.24692e-1,   &
     &    0.00000e+0, 0.00000e+0, 0.00000e+0, 6.67029e-7, 0.00000e+0 /

      data  specwt  / 8.,4.,8.,8.,0.,8.,8.,0.,8.,0.,0.,0.,4.,0. /
!
      integer, dimension(nblow:nbhgh), public :: layreffr, ix1, ix2, ibx

      data  layreffr/ 18,30, 6, 3, 3, 8, 2, 6, 1, 2, 0,32,58,49 /
      data  ix1     /  1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 3, 0 /
      data  ix2     /  5, 2, 5, 2, 0, 2, 6, 0, 6, 0, 0, 0, 6, 0 /
      data  ibx     /  1, 1, 1, 2, 2, 3, 4, 3, 5, 4, 5, 6, 2, 7 /

      real (kind=kind_phys), parameter, public :: scalekur=50.15/48.37
!
      real (kind=kind_phys), target, public ::                          &
     &       sfluxref01(NGMAX,MFS01,MFB01),                             &
     &       sfluxref02(NGMAX,MFS02,MFB02),                             &
     &       sfluxref03(NGMAX,MFS03,MFB03)

!  ---  setup solar sfluxref01
!  ...  band 16, NG16=6
      data   sfluxref01(:,:,1)  /                          .36511300e+1,&
     & .32277700e+1,.26913900e+1,.18039990e+1,.68068900e+0,.54590270e-1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ...  band 20, NG20=10
      data   sfluxref01(:,:,2)  /                          .93408100e+1,&
     & .89372000e+1,.81934600e+1,.73919600e+1,.61212700e+1,.52395600e+1,&
     & .42494100e+1,.32001300e+1,.23949790e+1,.55783362e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ...  band 23, NG23=10
      data   sfluxref01(:,:,3)  /                          .10462440e+3,&
     & .94796000e+2,.40829400e+2,.35180100e+2,.28694700e+2,.21575100e+2,&
     & .14638800e+2,.15911100e+1,.23587800e+1,.14541238e+1,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ...  band 25, NG25=6
      data   sfluxref01(:,:,4)  /                          .42685800e+2,&
     & .45772000e+2,.90953400e+2,.88238400e+2,.75011180e+2,.45315347e+1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ...  band 26, NG26=6
      data   sfluxref01(:,:,5)  /                          .29007900e+2,&
     & .28408800e+2,.33338200e+2,.21820300e+2,.15982813e+2,.93700512e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!     the following (NG27) values were obtained using the "low resolution"
!     version of the kurucz solar source function.  for unknown reasons,
!     the total irradiance in this band differs from the corresponding
!     total in the "high-resolution" version of the Kurucz function.
!     therefore, below these values are scaled by the factor scalekur.

!  ...  band 27, NG27=8
      data   sfluxref01(:,:,6)  /                          .14052600e+2,&
     & .11479400e+2,.87259000e+1,.55696600e+1,.38092700e+1,.15769000e+1,&
     & .29680079e+1,.19043253e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ...  band 29, NG29=12
      data   sfluxref01(:,:,7)  /                          .13288000e+1,&
     & .21401800e+1,.19761200e+1,.17900000e+1,.27421900e+1,.18615160e+1,&
     & .83448990e+0,.12731000e+0,.44698800e-1,.30744100e-1,.11672800e-1,&
     & .16557300e-2,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ---  setup solar sfluxref02
!  ...  band 17, NG17=12
      data   sfluxref02(:,:,1)  /                                       &
     & .31561300e+1,.30344900e+1,.29206900e+1,.26387400e+1,.23458100e+1,&
     & .37790500e+1,.12908500e+1,.97039020e+0,.78781300e-1,.10810270e+0,&
     & .29412900e-1,.12635380e-1,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.28314700e+1,.29591900e+1,.29667400e+1,.27767700e+1,&
     & .24682600e+1,.38472400e+1,.13027900e+1,.97894900e+0,.80212200e-1,&
     & .11088140e+0,.29905100e-1,.12675180e-1,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.28230000e+1,.29484500e+1,.29588700e+1,&
     & .27759300e+1,.24709600e+1,.38644300e+1,.13079600e+1,.98108860e+0,&
     & .80199600e-1,.11130020e+0,.30051500e-1,.12844740e-1,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.28171500e+1,.29378900e+1,&
     & .29509100e+1,.27704600e+1,.24771600e+1,.38795600e+1,.13127700e+1,&
     & .98414460e+0,.80339100e-1,.11153510e+0,.30072000e-1,.13100450e-1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.28233500e+1,&
     & .29316800e+1,.29145500e+1,.27521300e+1,.24916800e+1,.39013400e+1,&
     & .13240100e+1,.99059630e+0,.80519700e-1,.11150350e+0,.30566700e-1,&
     & .13155950e-1,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ...  band 28, NG28=6
      data   sfluxref02(:,:,2)  /                                       &
     & .10615600e+1,.59991000e+0,.82253900e+0,.47011400e+0,.12054496e+0,&
     & .52716294e-2,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.10759800e+1,.58509900e+0,.82292900e+0,.47011400e+0,&
     & .12059320e+0,.52233221e-2,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.46164700e+0,.40611300e+0,.64001400e+0,&
     & .44662400e+0,.10729748e+1,.52565480e-1,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.13266900e+0,.17505800e+0,&
     & .74740500e+0,.82625100e+0,.11385304e+1,.60026497e-1,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.75480000e-1,&
     & .23224600e+0,.74740500e+0,.77667600e+0,.11821991e+1,.65932601e-1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0/

!  ---  setup solar sfluxref03
!  ...  band 18, NG18=8
      data   sfluxref03(:,:,1)  /                                       &
     & .36584000e+1,.35437500e+1,.33448100e+1,.31053400e+1,.52272000e+1,&
     & .34812500e+1,.12703318e+1,.98650010e-1,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.38637200e+1,.34852100e+1,.33079000e+1,.30810300e+1,&
     & .51827400e+1,.34533000e+1,.12591244e+1,.96720520e-1,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.39037000e+1,.35065700e+1,.33062900e+1,&
     & .30604600e+1,.51688900e+1,.34381600e+1,.12512870e+1,.94381020e-1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.39316500e+1,.35205800e+1,&
     & .33134600e+1,.30494400e+1,.51550700e+1,.34226800e+1,.12456526e+1,&
     & .91203220e-1,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.39408200e+1,&
     & .35522100e+1,.33186300e+1,.30473000e+1,.51324600e+1,.34110100e+1,&
     & .12380195e+1,.89290320e-1,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .39419800e+1,.35874300e+1,.33210600e+1,.30586600e+1,.51105400e+1,&
     & .33948500e+1,.12272675e+1,.87955720e-1,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.39359600e+1,.36336600e+1,.33314400e+1,.30625200e+1,&
     & .50954600e+1,.33706900e+1,.12121929e+1,.87814620e-1,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.39252000e+1,.36907800e+1,.33565600e+1,&
     & .30705500e+1,.50829200e+1,.33242900e+1,.11916105e+1,.87812520e-1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.38072100e+1,.37443700e+1,&
     & .35020500e+1,.31800900e+1,.50494500e+1,.32007600e+1,.11580073e+1,&
     & .87812920e-1,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0 /

!  ...  band 19, NG19=8
      data   sfluxref03(:,:,2)  /                                       &
     & .32579100e+1,.32969700e+1,.31603100e+1,.29611500e+1,.50305700e+1,&
     & .33767800e+1,.12463848e+1,.97623090e-1,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.32276900e+1,.32881700e+1,.31668700e+1,.29766200e+1,&
     & .50388700e+1,.33829100e+1,.12493943e+1,.97174500e-1,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.32229400e+1,.32778000e+1,.31742400e+1,&
     & .29714300e+1,.50477800e+1,.33835100e+1,.12527049e+1,.97276400e-1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.32244500e+1,.32611300e+1,&
     & .31843800e+1,.29692100e+1,.50516500e+1,.33840300e+1,.12553459e+1,&
     & .97505500e-1,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.32249700e+1,&
     & .32510900e+1,.31874100e+1,.29697000e+1,.50548000e+1,.33852500e+1,&
     & .12565675e+1,.97922400e-1,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .32263200e+1,.32417400e+1,.31852400e+1,.29740200e+1,.50554900e+1,&
     & .33899800e+1,.12562855e+1,.98616200e-1,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.32279300e+1,.32358900e+1,.31772000e+1,.29786900e+1,&
     & .50572900e+1,.33942500e+1,.12571685e+1,.99276100e-1,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.32296600e+1,.32408700e+1,.31567600e+1,&
     & .29817100e+1,.50586900e+1,.34020900e+1,.12586610e+1,.99259200e-1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.32724000e+1,.32466600e+1,&
     & .31388600e+1,.29523800e+1,.50465000e+1,.34105900e+1,.12610396e+1,&
     & .99262600e-1,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0 /

!  ...  band 21, NG21=10
      data   sfluxref03(:,:,3)  /                                       &
     & .16164300e+2,.15580600e+2,.14725400e+2,.13554100e+2,.11951900e+2,&
     & .10444100e+2,.83788400e+1,.62638400e+1,.47495780e+1,.11187392e+1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.15645100e+2,.15317000e+2,.14698700e+2,.13735000e+2,&
     & .12226700e+2,.10516460e+2,.84715000e+1,.63887300e+1,.48059700e+1,&
     & .11263635e+1,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.15609200e+2,.15329300e+2,.14688100e+2,&
     & .13669300e+2,.12234200e+2,.10520100e+2,.84944200e+1,.64213800e+1,&
     & .48319990e+1,.11333414e+1,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.15578600e+2,.15342200e+2,&
     & .14689400e+2,.13604000e+2,.12256700e+2,.10494000e+2,.85352100e+1,&
     & .64442700e+1,.48477890e+1,.11392313e+1,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.15538000e+2,&
     & .15382600e+2,.14657500e+2,.13572200e+2,.12264600e+2,.10476720e+2,&
     & .85715800e+1,.64634300e+1,.48602370e+1,.114454305e+1,.0000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .15512400e+2,.15398600e+2,.14624000e+2,.13553500e+2,.12246800e+2,&
     & .10488910e+2,.86043400e+1,.6479850e+1,.48727470e+1,.115032015e+1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.15491000e+2,.15402800e+2,.14577200e+2,.13550700e+2,&
     & .12212200e+2,.10527350e+2,.86265000e+1,.64964400e+1,.48903570e+1,&
     & .115687915e+1,.0000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.15456200e+2,.15392800e+2,.14551000e+2,&
     & .13512200e+2,.12189000e+2,.10582600e+2,.86584200e+1,.65155800e+1,&
     & .49081390e+1,.116541315e+1,.0000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.15006900e+2,.15147900e+2,&
     & .14780200e+2,.13608500e+2,.12279300e+2,.10692900e+2,.87272300e+1,&
     & .65711400e+1,.49500240e+1,.116747315e+1,.0000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0 /

!  ...  band 22, NG22=2
      data   sfluxref03(:,:,4)  /                                       &
     & .22870690e+2,.142292348e+1,.0000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.22872700e+2,.142092788e+1,.0000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.22877410e+2,.141623238e+1,.0000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.0000000e+0,.22879360e+2,.141425888e+1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.22880890e+2,&
     & .14127222e+1,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .22882260e+2,.141135631e+1,.0000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.22883070e+2,.141054121e+1,.0000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.22884580e+2,.140903474e+1,.0000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.0000000e+0,.22885890e+2,.140772281e+1,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0 /

!  ...  band 24, NG24=8
      data   sfluxref03(:,:,5)  /                                       &
     & .67485000e+2,.60119600e+2,.46277600e+2,.31190000e+2,.10594640e+2,&
     & .15530850e+1,.82783000e+0,.13921180e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.66669400e+2,.58294800e+2,.47994500e+2,.32163300e+2,&
     & .10551730e+2,.15462830e+1,.82782300e+0,.13922180e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.66436200e+2,.58556400e+2,.47978900e+2,&
     & .32183500e+2,.10528620e+2,.15397570e+1,.82447300e+0,.13921180e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.66368100e+2,.58663000e+2,&
     & .47964300e+2,.32188200e+2,.10508680e+2,.15344650e+1,.82105100e+0,&
     & .13921180e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.66325500e+2,&
     & .58727000e+2,.47967600e+2,.32196900e+2,.10482800e+2,.15295910e+1,&
     & .81841600e+0,.13922180e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .66303600e+2,.58785900e+2,.47960400e+2,.32194700e+2,.10462370e+2,&
     & .15250500e+1,.81580900e+0,.13921180e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.66332200e+2,.58793500e+2,.47946700e+2,.32196300e+2,&
     & .10445690e+2,.15221860e+1,.81413300e+0,.13631770e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.66389100e+2,.58828200e+2,.47920000e+2,&
     & .32155700e+2,.10430340e+2,.15208810e+1,.81012300e+0,.13260880e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.67319100e+2,.58382700e+2,&
     & .47610800e+2,.32032700e+2,.10390010e+2,.15156730e+1,.80297600e+0,&
     & .13316830e+0,.00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0,&
     & .00000000e+0,.00000000e+0,.00000000e+0,.00000000e+0 /

!........................................!
      end module module_radsw_sflux      !
!========================================!


!========================================!
      module module_radsw_kgb16          !
!........................................!
!
! ***************  the original program descriptions  *************** !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 16:  2600-3250 cm-1 (low - h2o,ch4; high - ch4)            !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep  Feb 2004                                      !
!                                                                     !
! ************************  end description  ************************ !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG16

!
      implicit none
!
      private
!
      integer,  public :: MSA16, MSB16, MSF16, MFR16
      parameter (MSA16=585, MSB16=235, MSF16=10, MFR16=3)

      real (kind=kind_phys), public ::           selfref(MSF16,NG16),   &
     &       absa(MSA16,NG16), absb(MSB16,NG16), forref(MFR16,NG16)

!  ---  rayleigh extinction coefficient at v = 2925 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 2.91e-10

!     the array absa(585,NG16) (ka(9,5,13,NG16)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first  index in the array, js, runs from 1 to 9, and corresponds
!     to different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 6, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .6164391E-04,.1393981E-03,.1485874E-03,.1470456E-03,.1378359E-03,&
     & .1251049E-03,.1080809E-03,.8380100E-04,.4375937E-04,.6303123E-04,&
     & .1387829E-03,.1476732E-03,.1465675E-03,.1376116E-03,.1247762E-03,&
     & .1082831E-03,.8452911E-04,.4545314E-04,.6421152E-04,.1383722E-03,&
     & .1469969E-03,.1464177E-03,.1376712E-03,.1247454E-03,.1085938E-03,&
     & .8527181E-04,.4714073E-04,.6487941E-04,.1376604E-03,.1463450E-03,&
     & .1456654E-03,.1375752E-03,.1247169E-03,.1088288E-03,.8599534E-04,&
     & .4882216E-04,.6543285E-04,.1371565E-03,.1459883E-03,.1450442E-03,&
     & .1376047E-03,.1246488E-03,.1084938E-03,.8677500E-04,.5041239E-04,&
     & .5313622E-04,.1212259E-03,.1294107E-03,.1289969E-03,.1215056E-03,&
     & .1099177E-03,.9560988E-04,.7413197E-04,.3741326E-04,.5429633E-04,&
     & .1208023E-03,.1288524E-03,.1287190E-03,.1215251E-03,.1095731E-03,&
     & .9593147E-04,.7493945E-04,.3890369E-04,.5532831E-04,.1205422E-03,&
     & .1285240E-03,.1281017E-03,.1216844E-03,.1097557E-03,.9586096E-04,&
     & .7569726E-04,.4033143E-04,.5592005E-04,.1201155E-03,.1280667E-03,&
     & .1275655E-03,.1215386E-03,.1099924E-03,.9556109E-04,.7641040E-04,&
     & .4181164E-04,.5617841E-04,.1196101E-03,.1275438E-03,.1270047E-03,&
     & .1210925E-03,.1101989E-03,.9565779E-04,.7709991E-04,.4321251E-04,&
     & .4531290E-04,.1048278E-03,.1125651E-03,.1126461E-03,.1064403E-03,&
     & .9599592E-04,.8416227E-04,.6509075E-04,.3260217E-04,.4640094E-04,&
     & .1045811E-03,.1122478E-03,.1120739E-03,.1065744E-03,.9607551E-04,&
     & .8395460E-04,.6589635E-04,.3382481E-04,.4717230E-04,.1043514E-03,&
     & .1118439E-03,.1116296E-03,.1065479E-03,.9636785E-04,.8372961E-04,&
     & .6667225E-04,.3501452E-04,.4770015E-04,.1042243E-03,.1115519E-03,&
     & .1111144E-03,.1064012E-03,.9685136E-04,.8410622E-04,.6743058E-04,&
     & .3619213E-04,.4786332E-04,.1037274E-03,.1110955E-03,.1107897E-03,&
     & .1063675E-03,.9719954E-04,.8449192E-04,.6808715E-04,.3740757E-04,&
     & .3843039E-04,.9043110E-04,.9770352E-04,.9799335E-04,.9304240E-04,&
     & .8389000E-04,.7321734E-04,.5677836E-04,.2994598E-04,.3931732E-04,&
     & .9024615E-04,.9732057E-04,.9745928E-04,.9306943E-04,.8414269E-04,&
     & .7304152E-04,.5752626E-04,.3089971E-04,.3999765E-04,.9015208E-04,&
     & .9690713E-04,.9702354E-04,.9292115E-04,.8443883E-04,.7337197E-04,&
     & .5829869E-04,.3187537E-04,.4043089E-04,.8996634E-04,.9668335E-04,&
     & .9670285E-04,.9286755E-04,.8481230E-04,.7380442E-04,.5906049E-04,&
     & .3283818E-04,.4073202E-04,.8972393E-04,.9645513E-04,.9660020E-04,&
     & .9303706E-04,.8536068E-04,.7423279E-04,.5981517E-04,.3379450E-04/

      data   absa(181:315, 1)   /                                       &
     & .3239990E-04,.7798454E-04,.8455791E-04,.8504346E-04,.8081384E-04,&
     & .7294914E-04,.6337302E-04,.4924460E-04,.2913833E-04,.3322912E-04,&
     & .7774352E-04,.8406820E-04,.8454204E-04,.8094382E-04,.7323153E-04,&
     & .6358291E-04,.4994292E-04,.2993189E-04,.3380830E-04,.7760287E-04,&
     & .8375655E-04,.8415223E-04,.8078676E-04,.7365225E-04,.6386151E-04,&
     & .5066949E-04,.3071453E-04,.3417420E-04,.7744463E-04,.8351059E-04,&
     & .8385781E-04,.8079307E-04,.7414802E-04,.6412908E-04,.5142302E-04,&
     & .3151682E-04,.3444762E-04,.7723103E-04,.8340026E-04,.8386295E-04,&
     & .8097163E-04,.7469899E-04,.6465444E-04,.5217398E-04,.3235645E-04,&
     & .2715767E-04,.6701770E-04,.7283633E-04,.7341902E-04,.6985462E-04,&
     & .6319692E-04,.5481578E-04,.4247223E-04,.2909771E-04,.2792146E-04,&
     & .6685295E-04,.7241685E-04,.7300079E-04,.6986342E-04,.6356893E-04,&
     & .5491074E-04,.4310004E-04,.2966308E-04,.2845821E-04,.6661135E-04,&
     & .7209651E-04,.7266374E-04,.6978860E-04,.6395350E-04,.5508302E-04,&
     & .4376855E-04,.3030854E-04,.2875713E-04,.6641988E-04,.7186205E-04,&
     & .7238777E-04,.6998777E-04,.6431932E-04,.5551791E-04,.4448282E-04,&
     & .3094504E-04,.2891644E-04,.6615229E-04,.7169260E-04,.7229235E-04,&
     & .7001252E-04,.6472305E-04,.5607907E-04,.4487260E-04,.3162719E-04,&
     & .2261551E-04,.5728902E-04,.6237413E-04,.6297079E-04,.5987738E-04,&
     & .5427552E-04,.4703020E-04,.3639661E-04,.2945807E-04,.2329813E-04,&
     & .5709378E-04,.6201885E-04,.6258230E-04,.5996529E-04,.5461263E-04,&
     & .4701194E-04,.3698989E-04,.3029693E-04,.2379683E-04,.5686371E-04,&
     & .6178860E-04,.6234127E-04,.5990707E-04,.5500772E-04,.4729286E-04,&
     & .3760923E-04,.3119772E-04,.2405414E-04,.5664356E-04,.6153544E-04,&
     & .6214186E-04,.6002313E-04,.5529452E-04,.4777815E-04,.3824621E-04,&
     & .3201199E-04,.2417921E-04,.5638188E-04,.6131618E-04,.6200411E-04,&
     & .5999630E-04,.5566905E-04,.4832075E-04,.3841731E-04,.3273083E-04/

      data   absa(316:450, 1)   /                                       &
     & .1878381E-04,.4894749E-04,.5331015E-04,.5389221E-04,.5125674E-04,&
     & .4647673E-04,.4004232E-04,.3106983E-04,.2910087E-04,.1940228E-04,&
     & .4877469E-04,.5302872E-04,.5356337E-04,.5126792E-04,.4678873E-04,&
     & .4012482E-04,.3162012E-04,.3075290E-04,.1985918E-04,.4855059E-04,&
     & .5279719E-04,.5333080E-04,.5126898E-04,.4707665E-04,.4046924E-04,&
     & .3217345E-04,.3150876E-04,.2014062E-04,.4829423E-04,.5255189E-04,&
     & .5319514E-04,.5130651E-04,.4737582E-04,.4091784E-04,.3262566E-04,&
     & .3216455E-04,.2025329E-04,.4801466E-04,.5234235E-04,.5308048E-04,&
     & .5128923E-04,.4759043E-04,.4140204E-04,.3278753E-04,.3333379E-04,&
     & .1546471E-04,.4162009E-04,.4540125E-04,.4587423E-04,.4356518E-04,&
     & .3940508E-04,.3397440E-04,.2638191E-04,.3148426E-04,.1606324E-04,&
     & .4144674E-04,.4510294E-04,.4559038E-04,.4360139E-04,.3960990E-04,&
     & .3411097E-04,.2689026E-04,.3297831E-04,.1647624E-04,.4124716E-04,&
     & .4486029E-04,.4540089E-04,.4359331E-04,.3994622E-04,.3443250E-04,&
     & .2738815E-04,.3466130E-04,.1674831E-04,.4101176E-04,.4463266E-04,&
     & .4523420E-04,.4362159E-04,.4026871E-04,.3484293E-04,.2762312E-04,&
     & .3608726E-04,.1688109E-04,.4072506E-04,.4442676E-04,.4508760E-04,&
     & .4359419E-04,.4044664E-04,.3522608E-04,.2789812E-04,.3743236E-04,&
     & .1262626E-04,.3515542E-04,.3839650E-04,.3883879E-04,.3680058E-04,&
     & .3322886E-04,.2866957E-04,.2229743E-04,.3542581E-04,.1323548E-04,&
     & .3501861E-04,.3816682E-04,.3862136E-04,.3691204E-04,.3341643E-04,&
     & .2882784E-04,.2274500E-04,.3810724E-04,.1360997E-04,.3480644E-04,&
     & .3790261E-04,.3840467E-04,.3688940E-04,.3369037E-04,.2909482E-04,&
     & .2319725E-04,.4051484E-04,.1386341E-04,.3459077E-04,.3769098E-04,&
     & .3821868E-04,.3684829E-04,.3399846E-04,.2945016E-04,.2334606E-04,&
     & .4250812E-04,.1401007E-04,.3434705E-04,.3748890E-04,.3806620E-04,&
     & .3681278E-04,.3416954E-04,.2981567E-04,.2357415E-04,.4469942E-04/

      data   absa(451:585, 1)   /                                       &
     & .1054345E-04,.2952630E-04,.3222392E-04,.3265638E-04,.3106790E-04,&
     & .2796314E-04,.2415008E-04,.1892216E-04,.3133027E-04,.1103113E-04,&
     & .2937313E-04,.3202245E-04,.3244622E-04,.3107680E-04,.2818612E-04,&
     & .2432663E-04,.1931576E-04,.3420418E-04,.1132648E-04,.2917577E-04,&
     & .3179835E-04,.3224295E-04,.3105314E-04,.2839919E-04,.2458295E-04,&
     & .1960995E-04,.3745408E-04,.1151773E-04,.2897659E-04,.3158949E-04,&
     & .3206591E-04,.3099578E-04,.2862811E-04,.2491522E-04,.1969533E-04,&
     & .4018024E-04,.1162646E-04,.2873280E-04,.3141795E-04,.3192928E-04,&
     & .3092122E-04,.2876193E-04,.2527302E-04,.1991005E-04,.4272251E-04,&
     & .8773859E-05,.2472331E-04,.2698123E-04,.2731286E-04,.2605313E-04,&
     & .2347524E-04,.2028179E-04,.1599354E-04,.2710338E-04,.9175082E-05,&
     & .2455642E-04,.2677932E-04,.2714788E-04,.2603439E-04,.2367042E-04,&
     & .2044068E-04,.1632502E-04,.2976471E-04,.9405766E-05,.2437939E-04,&
     & .2657517E-04,.2695641E-04,.2600791E-04,.2387449E-04,.2068968E-04,&
     & .1643613E-04,.3198691E-04,.9551613E-05,.2419222E-04,.2638939E-04,&
     & .2678440E-04,.2590383E-04,.2398600E-04,.2098502E-04,.1655507E-04,&
     & .3463054E-04,.9634553E-05,.2396313E-04,.2622876E-04,.2664925E-04,&
     & .2582504E-04,.2410681E-04,.2125060E-04,.1672669E-04,.3742745E-04,&
     & .7283549E-05,.2064472E-04,.2252697E-04,.2280007E-04,.2176915E-04,&
     & .1965230E-04,.1697791E-04,.1344921E-04,.2262985E-04,.7614582E-05,&
     & .2049273E-04,.2234335E-04,.2265284E-04,.2175650E-04,.1980433E-04,&
     & .1713132E-04,.1373534E-04,.2489034E-04,.7799854E-05,.2032918E-04,&
     & .2216475E-04,.2247253E-04,.2168451E-04,.1997844E-04,.1730534E-04,&
     & .1371476E-04,.2674112E-04,.7917832E-05,.2015404E-04,.2198881E-04,&
     & .2232428E-04,.2159377E-04,.2002799E-04,.1753711E-04,.1382698E-04,&
     & .2881746E-04,.7981504E-05,.1995296E-04,.2184003E-04,.2217179E-04,&
     & .2152204E-04,.2013182E-04,.1776050E-04,.1402084E-04,.3110145E-04/

      data   absa(  1:180, 2)   /                                       &
     & .2930165E-03,.4205176E-03,.4603190E-03,.4714149E-03,.4679470E-03,&
     & .4501949E-03,.4185459E-03,.3688035E-03,.2684515E-03,.2936114E-03,&
     & .4252087E-03,.4672427E-03,.4798112E-03,.4769734E-03,.4593456E-03,&
     & .4281470E-03,.3781642E-03,.2764789E-03,.2946377E-03,.4315576E-03,&
     & .4755024E-03,.4896418E-03,.4876102E-03,.4700407E-03,.4395838E-03,&
     & .3880213E-03,.2860406E-03,.2952749E-03,.4370625E-03,.4830699E-03,&
     & .4992608E-03,.4981682E-03,.4812406E-03,.4500990E-03,.3988892E-03,&
     & .2947867E-03,.2963643E-03,.4435447E-03,.4919681E-03,.5108647E-03,&
     & .5100757E-03,.4946687E-03,.4633694E-03,.4116274E-03,.3068946E-03,&
     & .2564192E-03,.3741028E-03,.4121845E-03,.4222514E-03,.4178533E-03,&
     & .4017900E-03,.3723948E-03,.3265092E-03,.2280897E-03,.2572159E-03,&
     & .3784588E-03,.4184094E-03,.4294373E-03,.4260348E-03,.4103047E-03,&
     & .3806553E-03,.3334951E-03,.2357658E-03,.2585085E-03,.3843346E-03,&
     & .4259985E-03,.4392394E-03,.4361957E-03,.4206279E-03,.3905943E-03,&
     & .3429498E-03,.2431183E-03,.2596736E-03,.3898247E-03,.4335148E-03,&
     & .4488328E-03,.4466516E-03,.4312067E-03,.4012724E-03,.3532432E-03,&
     & .2517667E-03,.2607431E-03,.3954864E-03,.4412810E-03,.4585918E-03,&
     & .4573106E-03,.4422671E-03,.4128750E-03,.3642066E-03,.2619245E-03,&
     & .2235048E-03,.3314541E-03,.3652461E-03,.3744704E-03,.3699739E-03,&
     & .3548398E-03,.3279534E-03,.2865252E-03,.1924155E-03,.2243217E-03,&
     & .3351441E-03,.3704468E-03,.3812002E-03,.3769484E-03,.3624388E-03,&
     & .3347617E-03,.2927657E-03,.1990504E-03,.2253955E-03,.3397624E-03,&
     & .3767056E-03,.3889755E-03,.3852321E-03,.3709575E-03,.3432829E-03,&
     & .3003296E-03,.2050004E-03,.2269437E-03,.3455225E-03,.3848163E-03,&
     & .3985839E-03,.3955107E-03,.3808205E-03,.3527054E-03,.3092164E-03,&
     & .2125103E-03,.2280141E-03,.3507535E-03,.3920562E-03,.4069340E-03,&
     & .4049634E-03,.3903833E-03,.3628870E-03,.3187283E-03,.2207668E-03,&
     & .1945146E-03,.2918639E-03,.3214502E-03,.3303679E-03,.3254231E-03,&
     & .3116270E-03,.2876070E-03,.2498710E-03,.1637253E-03,.1954096E-03,&
     & .2951961E-03,.3261050E-03,.3361412E-03,.3314562E-03,.3181672E-03,&
     & .2938352E-03,.2559070E-03,.1693706E-03,.1965264E-03,.2992768E-03,&
     & .3318268E-03,.3430041E-03,.3386437E-03,.3253725E-03,.3007759E-03,&
     & .2623706E-03,.1739993E-03,.1975452E-03,.3039553E-03,.3382589E-03,&
     & .3504664E-03,.3467920E-03,.3335199E-03,.3082884E-03,.2693916E-03,&
     & .1801702E-03,.1992020E-03,.3099779E-03,.3465283E-03,.3595055E-03,&
     & .3565400E-03,.3430057E-03,.3175611E-03,.2774855E-03,.1867273E-03/

      data   absa(181:315, 2)   /                                       &
     & .1692110E-03,.2560783E-03,.2816239E-03,.2892751E-03,.2854340E-03,&
     & .2729454E-03,.2512636E-03,.2176013E-03,.1406685E-03,.1700158E-03,&
     & .2591285E-03,.2857505E-03,.2942600E-03,.2906779E-03,.2782649E-03,&
     & .2565024E-03,.2227555E-03,.1451134E-03,.1711402E-03,.2627639E-03,&
     & .2908575E-03,.3002244E-03,.2969592E-03,.2845936E-03,.2626853E-03,&
     & .2280973E-03,.1489243E-03,.1723113E-03,.2667721E-03,.2966815E-03,&
     & .3070349E-03,.3041790E-03,.2912783E-03,.2694000E-03,.2338131E-03,&
     & .1538692E-03,.1740563E-03,.2722405E-03,.3040666E-03,.3151440E-03,&
     & .3128365E-03,.2994324E-03,.2773169E-03,.2405489E-03,.1587634E-03,&
     & .1467247E-03,.2239121E-03,.2459341E-03,.2521661E-03,.2491285E-03,&
     & .2381498E-03,.2185025E-03,.1886728E-03,.1245807E-03,.1473884E-03,&
     & .2265685E-03,.2495294E-03,.2564125E-03,.2535594E-03,.2425776E-03,&
     & .2232033E-03,.1930135E-03,.1274953E-03,.1482897E-03,.2296257E-03,&
     & .2540362E-03,.2615549E-03,.2589723E-03,.2479233E-03,.2285370E-03,&
     & .1976049E-03,.1306204E-03,.1494392E-03,.2332170E-03,.2591337E-03,&
     & .2675927E-03,.2648872E-03,.2537659E-03,.2341528E-03,.2022251E-03,&
     & .1344888E-03,.1508093E-03,.2374102E-03,.2648426E-03,.2740852E-03,&
     & .2718576E-03,.2604781E-03,.2406433E-03,.2081661E-03,.1378196E-03,&
     & .1263109E-03,.1951428E-03,.2144975E-03,.2195749E-03,.2170080E-03,&
     & .2069188E-03,.1894328E-03,.1633783E-03,.1200887E-03,.1271134E-03,&
     & .1973810E-03,.2175147E-03,.2232648E-03,.2204590E-03,.2109194E-03,&
     & .1937750E-03,.1670597E-03,.1213160E-03,.1278627E-03,.2000293E-03,&
     & .2212521E-03,.2276502E-03,.2250313E-03,.2152307E-03,.1982160E-03,&
     & .1708010E-03,.1229746E-03,.1288762E-03,.2031265E-03,.2255973E-03,&
     & .2327902E-03,.2301752E-03,.2203853E-03,.2029333E-03,.1746039E-03,&
     & .1249396E-03,.1300624E-03,.2067082E-03,.2305851E-03,.2383986E-03,&
     & .2363202E-03,.2260966E-03,.2085247E-03,.1797897E-03,.1276706E-03/

      data   absa(316:450, 2)   /                                       &
     & .1087530E-03,.1695227E-03,.1869845E-03,.1912855E-03,.1886016E-03,&
     & .1794107E-03,.1640395E-03,.1407348E-03,.1388109E-03,.1095657E-03,&
     & .1712971E-03,.1895043E-03,.1945158E-03,.1916236E-03,.1827598E-03,&
     & .1678635E-03,.1438343E-03,.1366925E-03,.1103222E-03,.1736665E-03,&
     & .1926247E-03,.1982051E-03,.1955376E-03,.1864091E-03,.1715841E-03,&
     & .1469470E-03,.1364063E-03,.1111854E-03,.1763283E-03,.1961855E-03,&
     & .2023728E-03,.2000777E-03,.1907608E-03,.1757564E-03,.1506354E-03,&
     & .1369112E-03,.1122634E-03,.1793268E-03,.2003957E-03,.2070334E-03,&
     & .2051943E-03,.1958970E-03,.1805692E-03,.1551262E-03,.1374744E-03,&
     & .9300705E-04,.1464182E-03,.1620111E-03,.1662152E-03,.1638421E-03,&
     & .1556181E-03,.1415254E-03,.1207125E-03,.2197460E-03,.9379197E-04,&
     & .1478848E-03,.1642219E-03,.1688554E-03,.1664207E-03,.1584541E-03,&
     & .1446859E-03,.1233342E-03,.2178811E-03,.9457710E-04,.1498107E-03,&
     & .1668492E-03,.1719850E-03,.1697382E-03,.1615171E-03,.1478254E-03,&
     & .1260245E-03,.2169464E-03,.9536836E-04,.1520740E-03,.1699345E-03,&
     & .1755822E-03,.1735872E-03,.1652759E-03,.1515057E-03,.1293800E-03,&
     & .2166357E-03,.9627944E-04,.1547433E-03,.1735275E-03,.1795990E-03,&
     & .1780373E-03,.1697434E-03,.1556408E-03,.1331356E-03,.2164129E-03,&
     & .7917379E-04,.1259529E-03,.1400562E-03,.1440600E-03,.1421290E-03,&
     & .1348884E-03,.1219352E-03,.1031837E-03,.3271209E-03,.7977861E-04,&
     & .1271040E-03,.1418834E-03,.1462764E-03,.1442107E-03,.1372758E-03,&
     & .1246562E-03,.1054164E-03,.3261360E-03,.8053422E-04,.1287862E-03,&
     & .1442112E-03,.1488988E-03,.1470409E-03,.1399784E-03,.1274996E-03,&
     & .1079546E-03,.3247606E-03,.8124880E-04,.1308583E-03,.1468362E-03,&
     & .1519274E-03,.1504180E-03,.1431923E-03,.1307815E-03,.1110513E-03,&
     & .3234572E-03,.8201452E-04,.1331325E-03,.1497495E-03,.1553833E-03,&
     & .1542308E-03,.1471022E-03,.1343535E-03,.1143694E-03,.3218489E-03/

      data   absa(451:585, 2)   /                                       &
     & .6728851E-04,.1080906E-03,.1208580E-03,.1247323E-03,.1232827E-03,&
     & .1175266E-03,.1060571E-03,.8883780E-04,.3501958E-03,.6791800E-04,&
     & .1091898E-03,.1225378E-03,.1268241E-03,.1253901E-03,.1196653E-03,&
     & .1084469E-03,.9082183E-04,.3442870E-03,.6851447E-04,.1107863E-03,&
     & .1246494E-03,.1293024E-03,.1281246E-03,.1222490E-03,.1111838E-03,&
     & .9324574E-04,.3420183E-03,.6908353E-04,.1126804E-03,.1269928E-03,&
     & .1321156E-03,.1312354E-03,.1252083E-03,.1140710E-03,.9617117E-04,&
     & .3390113E-03,.6957312E-04,.1148165E-03,.1296320E-03,.1352293E-03,&
     & .1346612E-03,.1287087E-03,.1172723E-03,.9910752E-04,.3385633E-03,&
     & .5686255E-04,.9216035E-04,.1034858E-03,.1071516E-03,.1060652E-03,&
     & .1014159E-03,.9163836E-04,.7623805E-04,.3310437E-03,.5734770E-04,&
     & .9329051E-04,.1049670E-03,.1090060E-03,.1081383E-03,.1034188E-03,&
     & .9390315E-04,.7812498E-04,.3244905E-03,.5781298E-04,.9473394E-04,&
     & .1068652E-03,.1112574E-03,.1106069E-03,.1057391E-03,.9631206E-04,&
     & .8054235E-04,.3210568E-03,.5822217E-04,.9640942E-04,.1089655E-03,&
     & .1137847E-03,.1134036E-03,.1085277E-03,.9896999E-04,.8309374E-04,&
     & .3173487E-03,.5853550E-04,.9825196E-04,.1113394E-03,.1165990E-03,&
     & .1165067E-03,.1117023E-03,.1018552E-03,.8573085E-04,.3168446E-03,&
     & .4783824E-04,.7831477E-04,.8819578E-04,.9148052E-04,.9081639E-04,&
     & .8696960E-04,.7879106E-04,.6529420E-04,.2796540E-03,.4821354E-04,&
     & .7934773E-04,.8956662E-04,.9314137E-04,.9269390E-04,.8882919E-04,&
     & .8080982E-04,.6704726E-04,.2741660E-03,.4859146E-04,.8061493E-04,&
     & .9116325E-04,.9515541E-04,.9486227E-04,.9088828E-04,.8295551E-04,&
     & .6935405E-04,.2709349E-03,.4883862E-04,.8204290E-04,.9304806E-04,&
     & .9741929E-04,.9740051E-04,.9346855E-04,.8534014E-04,.7158356E-04,&
     & .2681460E-03,.4903521E-04,.8353294E-04,.9510393E-04,.9997200E-04,&
     & .1001833E-03,.9622958E-04,.8790924E-04,.7395389E-04,.2673945E-03/

      data   absa(  1:180, 3)   /                                       &
     & .8184572E-03,.1288796E-02,.1495187E-02,.1632713E-02,.1719945E-02,&
     & .1761605E-02,.1759635E-02,.1698408E-02,.1367318E-02,.8243726E-03,&
     & .1301794E-02,.1514658E-02,.1658937E-02,.1753464E-02,.1802877E-02,&
     & .1805416E-02,.1745195E-02,.1405620E-02,.8348880E-03,.1320873E-02,&
     & .1542641E-02,.1692748E-02,.1795401E-02,.1852780E-02,.1858131E-02,&
     & .1800382E-02,.1449500E-02,.8418718E-03,.1340423E-02,.1571743E-02,&
     & .1728235E-02,.1837120E-02,.1903874E-02,.1915521E-02,.1859941E-02,&
     & .1503155E-02,.8531253E-03,.1366891E-02,.1607512E-02,.1771740E-02,&
     & .1886777E-02,.1961375E-02,.1981251E-02,.1927613E-02,.1561788E-02,&
     & .7302326E-03,.1151907E-02,.1334524E-02,.1456839E-02,.1533051E-02,&
     & .1567636E-02,.1559387E-02,.1499443E-02,.1187382E-02,.7372791E-03,&
     & .1164596E-02,.1352581E-02,.1480270E-02,.1561840E-02,.1601428E-02,&
     & .1596993E-02,.1539888E-02,.1219763E-02,.7483530E-03,.1183063E-02,&
     & .1378407E-02,.1510357E-02,.1596909E-02,.1642217E-02,.1642410E-02,&
     & .1586010E-02,.1257897E-02,.7577399E-03,.1203485E-02,.1406414E-02,&
     & .1543415E-02,.1634037E-02,.1686222E-02,.1692333E-02,.1636545E-02,&
     & .1301186E-02,.7660501E-03,.1226282E-02,.1437446E-02,.1580408E-02,&
     & .1675024E-02,.1733502E-02,.1745352E-02,.1693309E-02,.1350969E-02,&
     & .6502794E-03,.1026616E-02,.1186960E-02,.1295393E-02,.1364677E-02,&
     & .1394889E-02,.1381785E-02,.1317502E-02,.1017363E-02,.6567559E-03,&
     & .1037505E-02,.1202583E-02,.1315591E-02,.1388459E-02,.1421306E-02,&
     & .1413547E-02,.1351496E-02,.1044406E-02,.6651689E-03,.1050207E-02,&
     & .1223023E-02,.1339830E-02,.1416067E-02,.1452943E-02,.1449484E-02,&
     & .1390425E-02,.1076530E-02,.6772722E-03,.1071331E-02,.1249580E-02,&
     & .1370662E-02,.1449539E-02,.1490615E-02,.1491899E-02,.1434093E-02,&
     & .1111777E-02,.6855463E-03,.1091648E-02,.1276403E-02,.1402863E-02,&
     & .1483596E-02,.1529594E-02,.1535466E-02,.1481180E-02,.1152638E-02,&
     & .5768479E-03,.9115621E-03,.1053538E-02,.1150084E-02,.1211847E-02,&
     & .1236742E-02,.1220110E-02,.1153457E-02,.8691713E-03,.5830135E-03,&
     & .9209284E-03,.1066914E-02,.1166765E-02,.1232185E-02,.1259290E-02,&
     & .1246437E-02,.1180938E-02,.8912098E-03,.5910480E-03,.9345088E-03,&
     & .1084575E-02,.1187678E-02,.1255979E-02,.1286682E-02,.1276512E-02,&
     & .1213240E-02,.9180720E-03,.6008298E-03,.9515132E-03,.1106486E-02,&
     & .1212563E-02,.1283329E-02,.1317431E-02,.1311088E-02,.1249258E-02,&
     & .9471810E-03,.6138388E-03,.9709949E-03,.1132455E-02,.1242798E-02,&
     & .1315868E-02,.1353280E-02,.1350757E-02,.1291032E-02,.9816019E-03/

      data   absa(181:315, 3)   /                                       &
     & .5093571E-03,.8080237E-03,.9381643E-03,.1019789E-02,.1071192E-02,&
     & .1090471E-02,.1073488E-02,.1005758E-02,.7418709E-03,.5151626E-03,&
     & .8165236E-03,.9496789E-03,.1034266E-02,.1088284E-02,.1110067E-02,&
     & .1094940E-02,.1028625E-02,.7599047E-03,.5224485E-03,.8286429E-03,&
     & .9624412E-03,.1052107E-02,.1109000E-02,.1133457E-02,.1120200E-02,&
     & .1055094E-02,.7812729E-03,.5316862E-03,.8438037E-03,.9798289E-03,&
     & .1073448E-02,.1132718E-02,.1160426E-02,.1149517E-02,.1085478E-02,&
     & .8053947E-03,.5444831E-03,.8638786E-03,.1002962E-02,.1099774E-02,&
     & .1161623E-02,.1191798E-02,.1183243E-02,.1121002E-02,.8340163E-03,&
     & .4476728E-03,.7152034E-03,.8303044E-03,.9027280E-03,.9422368E-03,&
     & .9566089E-03,.9387208E-03,.8743692E-03,.6315594E-03,.4531060E-03,&
     & .7229985E-03,.8401962E-03,.9128749E-03,.9571607E-03,.9729227E-03,&
     & .9565139E-03,.8930810E-03,.6455309E-03,.4601132E-03,.7340161E-03,&
     & .8531068E-03,.9280096E-03,.9748476E-03,.9926457E-03,.9777088E-03,&
     & .9149552E-03,.6618730E-03,.4687606E-03,.7470819E-03,.8686951E-03,&
     & .9463629E-03,.9951022E-03,.1015485E-02,.1002244E-02,.9403643E-03,&
     & .6808008E-03,.4785595E-03,.7635702E-03,.8869579E-03,.9676686E-03,&
     & .1018859E-02,.1041246E-02,.1029537E-02,.9684597E-03,.7044125E-03,&
     & .3929615E-03,.6307616E-03,.7313469E-03,.7929589E-03,.8247908E-03,&
     & .8356659E-03,.8173907E-03,.7575170E-03,.5383971E-03,.3978536E-03,&
     & .6382438E-03,.7406018E-03,.8039178E-03,.8378859E-03,.8488038E-03,&
     & .8315811E-03,.7727276E-03,.5497760E-03,.4043732E-03,.6478463E-03,&
     & .7517647E-03,.8168558E-03,.8526219E-03,.8656075E-03,.8491574E-03,&
     & .7907891E-03,.5627384E-03,.4122128E-03,.6599763E-03,.7657659E-03,&
     & .8304335E-03,.8701303E-03,.8849440E-03,.8695717E-03,.8115473E-03,&
     & .5782796E-03,.4216792E-03,.6747622E-03,.7823998E-03,.8487735E-03,&
     & .8905701E-03,.9067120E-03,.8925209E-03,.8346838E-03,.5962302E-03/

      data   absa(316:450, 3)   /                                       &
     & .3452083E-03,.5538022E-03,.6406411E-03,.6932554E-03,.7213690E-03,&
     & .7269628E-03,.7089993E-03,.6545119E-03,.4847209E-03,.3494976E-03,&
     & .5608628E-03,.6492121E-03,.7026395E-03,.7319618E-03,.7381217E-03,&
     & .7204878E-03,.6670675E-03,.4924614E-03,.3555168E-03,.5695884E-03,&
     & .6593390E-03,.7138651E-03,.7429187E-03,.7522301E-03,.7349547E-03,&
     & .6817461E-03,.5010009E-03,.3629721E-03,.5805421E-03,.6721512E-03,&
     & .7278957E-03,.7573896E-03,.7683553E-03,.7514496E-03,.6984177E-03,&
     & .5116719E-03,.3715828E-03,.5938995E-03,.6869808E-03,.7439941E-03,&
     & .7750335E-03,.7864892E-03,.7704733E-03,.7174510E-03,.5246067E-03,&
     & .3053224E-03,.4856259E-03,.5588804E-03,.6031584E-03,.6264927E-03,&
     & .6296159E-03,.6127370E-03,.5636624E-03,.6772975E-03,.3090662E-03,&
     & .4924553E-03,.5671508E-03,.6120265E-03,.6363728E-03,.6394116E-03,&
     & .6225004E-03,.5739117E-03,.6640269E-03,.3140506E-03,.5003281E-03,&
     & .5762886E-03,.6218129E-03,.6467350E-03,.6509293E-03,.6346838E-03,&
     & .5862457E-03,.6537700E-03,.3203596E-03,.5098203E-03,.5876621E-03,&
     & .6339281E-03,.6587374E-03,.6641130E-03,.6482634E-03,.5998107E-03,&
     & .6477927E-03,.3280678E-03,.5213843E-03,.6008020E-03,.6482465E-03,&
     & .6724424E-03,.6792561E-03,.6641693E-03,.6153933E-03,.6458298E-03,&
     & .2733291E-03,.4289828E-03,.4894421E-03,.5250706E-03,.5435356E-03,&
     & .5455983E-03,.5286069E-03,.4846966E-03,.1525269E-02,.2767436E-03,&
     & .4347254E-03,.4967461E-03,.5327268E-03,.5517500E-03,.5540458E-03,&
     & .5369648E-03,.4938952E-03,.1462778E-02,.2810846E-03,.4416485E-03,&
     & .5047342E-03,.5418403E-03,.5610033E-03,.5634743E-03,.5472960E-03,&
     & .5038891E-03,.1415701E-02,.2866866E-03,.4502441E-03,.5146077E-03,&
     & .5523415E-03,.5718914E-03,.5740346E-03,.5587333E-03,.5149488E-03,&
     & .1375472E-02,.2934865E-03,.4607363E-03,.5263019E-03,.5647079E-03,&
     & .5847436E-03,.5869370E-03,.5719471E-03,.5277797E-03,.1341507E-02/

      data   absa(451:585, 3)   /                                       &
     & .2437828E-03,.3835942E-03,.4355111E-03,.4643060E-03,.4763641E-03,&
     & .4751451E-03,.4583198E-03,.4193574E-03,.1825342E-02,.2475557E-03,&
     & .3898469E-03,.4421366E-03,.4710873E-03,.4839473E-03,.4831741E-03,&
     & .4661299E-03,.4275995E-03,.1756933E-02,.2523063E-03,.3973500E-03,&
     & .4497777E-03,.4789579E-03,.4924096E-03,.4921150E-03,.4752010E-03,&
     & .4363639E-03,.1708388E-02,.2580622E-03,.4058625E-03,.4589941E-03,&
     & .4884064E-03,.5025671E-03,.5020326E-03,.4855219E-03,.4461901E-03,&
     & .1669299E-02,.2648941E-03,.4157912E-03,.4698491E-03,.4998329E-03,&
     & .5143630E-03,.5136245E-03,.4973797E-03,.4575998E-03,.1623002E-02,&
     & .2144367E-03,.3401170E-03,.3854223E-03,.4099266E-03,.4196610E-03,&
     & .4161366E-03,.3989091E-03,.3620378E-03,.1821655E-02,.2181060E-03,&
     & .3463069E-03,.3922259E-03,.4165637E-03,.4265488E-03,.4233797E-03,&
     & .4058075E-03,.3692157E-03,.1752071E-02,.2229514E-03,.3537530E-03,&
     & .4002355E-03,.4242150E-03,.4345415E-03,.4315545E-03,.4136194E-03,&
     & .3770147E-03,.1694072E-02,.2286334E-03,.3622049E-03,.4092201E-03,&
     & .4334592E-03,.4440346E-03,.4406362E-03,.4225253E-03,.3857130E-03,&
     & .1645638E-02,.2354630E-03,.3723794E-03,.4193491E-03,.4443949E-03,&
     & .4547391E-03,.4513798E-03,.4332059E-03,.3959499E-03,.1604607E-02,&
     & .1873934E-03,.2995165E-03,.3394711E-03,.3611157E-03,.3690992E-03,&
     & .3648908E-03,.3479275E-03,.3123745E-03,.1574790E-02,.1910454E-03,&
     & .3056861E-03,.3461932E-03,.3679395E-03,.3758939E-03,.3716238E-03,&
     & .3543646E-03,.3186636E-03,.1513357E-02,.1954531E-03,.3127911E-03,&
     & .3540282E-03,.3758084E-03,.3835301E-03,.3790064E-03,.3616108E-03,&
     & .3254549E-03,.1464333E-02,.2011139E-03,.3213571E-03,.3628469E-03,&
     & .3848293E-03,.3922803E-03,.3876951E-03,.3697429E-03,.3333521E-03,&
     & .1424128E-02,.2075183E-03,.3308876E-03,.3732484E-03,.3952328E-03,&
     & .4025443E-03,.3978826E-03,.3793694E-03,.3424789E-03,.1389880E-02/

      data   absa(  1:180, 4)   /                                       &
     & .3736324E-02,.6100560E-02,.7728410E-02,.8959576E-02,.9913897E-02,&
     & .1060534E-01,.1095043E-01,.1073709E-01,.9408838E-02,.3792238E-02,&
     & .6178500E-02,.7827205E-02,.9076284E-02,.1004324E-01,.1073477E-01,&
     & .1108085E-01,.1083940E-01,.9519649E-02,.3871754E-02,.6278718E-02,&
     & .7949468E-02,.9221394E-02,.1019996E-01,.1088941E-01,.1123148E-01,&
     & .1097176E-01,.9657396E-02,.3934975E-02,.6363492E-02,.8064193E-02,&
     & .9370476E-02,.1036793E-01,.1105745E-01,.1140542E-01,.1112862E-01,&
     & .9816845E-02,.4020724E-02,.6468817E-02,.8206395E-02,.9541906E-02,&
     & .1055918E-01,.1126073E-01,.1160795E-01,.1130758E-01,.9996192E-02,&
     & .3402273E-02,.5620650E-02,.7030748E-02,.8098078E-02,.8924472E-02,&
     & .9508944E-02,.9801255E-02,.9592074E-02,.8097867E-02,.3451518E-02,&
     & .5698898E-02,.7132591E-02,.8219355E-02,.9056752E-02,.9642702E-02,&
     & .9930623E-02,.9700146E-02,.8198722E-02,.3525436E-02,.5795932E-02,&
     & .7258246E-02,.8364978E-02,.9212267E-02,.9796289E-02,.1008263E-01,&
     & .9835560E-02,.8322568E-02,.3594242E-02,.5888324E-02,.7386430E-02,&
     & .8518910E-02,.9379100E-02,.9970774E-02,.1025951E-01,.9989379E-02,&
     & .8466107E-02,.3660490E-02,.5981362E-02,.7519542E-02,.8678334E-02,&
     & .9565719E-02,.1017025E-01,.1045322E-01,.1016255E-01,.8625539E-02,&
     & .3102796E-02,.5141091E-02,.6344760E-02,.7250524E-02,.7949175E-02,&
     & .8434987E-02,.8669628E-02,.8481084E-02,.6903275E-02,.3148146E-02,&
     & .5218469E-02,.6443866E-02,.7368450E-02,.8075641E-02,.8560889E-02,&
     & .8792122E-02,.8585149E-02,.6988914E-02,.3201454E-02,.5303035E-02,&
     & .6555873E-02,.7497631E-02,.8217284E-02,.8706091E-02,.8936316E-02,&
     & .8708232E-02,.7092276E-02,.3275760E-02,.5404005E-02,.6690045E-02,&
     & .7649565E-02,.8382414E-02,.8880227E-02,.9103836E-02,.8853150E-02,&
     & .7213900E-02,.3333969E-02,.5494628E-02,.6816793E-02,.7800860E-02,&
     & .8559397E-02,.9065543E-02,.9281723E-02,.9013731E-02,.7356112E-02,&
     & .2831427E-02,.4682414E-02,.5696173E-02,.6452550E-02,.7031440E-02,&
     & .7425488E-02,.7615078E-02,.7445178E-02,.5905547E-02,.2873398E-02,&
     & .4758548E-02,.5791245E-02,.6565599E-02,.7146886E-02,.7543948E-02,&
     & .7728470E-02,.7539994E-02,.5975231E-02,.2922042E-02,.4836804E-02,&
     & .5896068E-02,.6683182E-02,.7278970E-02,.7678598E-02,.7858498E-02,&
     & .7652488E-02,.6063670E-02,.2979003E-02,.4922733E-02,.6010505E-02,&
     & .6817378E-02,.7426977E-02,.7832832E-02,.8004073E-02,.7783099E-02,&
     & .6166540E-02,.3051509E-02,.5034845E-02,.6147088E-02,.6974054E-02,&
     & .7599704E-02,.8009148E-02,.8169023E-02,.7931032E-02,.6289201E-02/

      data   absa(181:315, 4)   /                                       &
     & .2581422E-02,.4252396E-02,.5095766E-02,.5725588E-02,.6193716E-02,&
     & .6511713E-02,.6657529E-02,.6499613E-02,.5066125E-02,.2622021E-02,&
     & .4323928E-02,.5186112E-02,.5827234E-02,.6301641E-02,.6620209E-02,&
     & .6758605E-02,.6586843E-02,.5124677E-02,.2667849E-02,.4398967E-02,&
     & .5285995E-02,.5935349E-02,.6423419E-02,.6740405E-02,.6875009E-02,&
     & .6687854E-02,.5199137E-02,.2719644E-02,.4480894E-02,.5391504E-02,&
     & .6059082E-02,.6557515E-02,.6877697E-02,.7004932E-02,.6806128E-02,&
     & .5284911E-02,.2786609E-02,.4583687E-02,.5519294E-02,.6203484E-02,&
     & .6711011E-02,.7035046E-02,.7152458E-02,.6938895E-02,.5385742E-02,&
     & .2339609E-02,.3846094E-02,.4554211E-02,.5067608E-02,.5443590E-02,&
     & .5691422E-02,.5796061E-02,.5651788E-02,.4350403E-02,.2380923E-02,&
     & .3911794E-02,.4637326E-02,.5161799E-02,.5541208E-02,.5788128E-02,&
     & .5888423E-02,.5729114E-02,.4399487E-02,.2424975E-02,.3982801E-02,&
     & .4723535E-02,.5261943E-02,.5649668E-02,.5897396E-02,.5992483E-02,&
     & .5819539E-02,.4460491E-02,.2474162E-02,.4061310E-02,.4818668E-02,&
     & .5372412E-02,.5769446E-02,.6018440E-02,.6107034E-02,.5923338E-02,&
     & .4530683E-02,.2528477E-02,.4146106E-02,.4927901E-02,.5493040E-02,&
     & .5902213E-02,.6153417E-02,.6234974E-02,.6038909E-02,.4613086E-02,&
     & .2108102E-02,.3463767E-02,.4067283E-02,.4486100E-02,.4783508E-02,&
     & .4968566E-02,.5039362E-02,.4903349E-02,.3736969E-02,.2149169E-02,&
     & .3526123E-02,.4141192E-02,.4568003E-02,.4870426E-02,.5056936E-02,&
     & .5123533E-02,.4973057E-02,.3776037E-02,.2192220E-02,.3593626E-02,&
     & .4218969E-02,.4658171E-02,.4967303E-02,.5154377E-02,.5214872E-02,&
     & .5052489E-02,.3823705E-02,.2239930E-02,.3665815E-02,.4305224E-02,&
     & .4760112E-02,.5074184E-02,.5261541E-02,.5315916E-02,.5141203E-02,&
     & .3879448E-02,.2291457E-02,.3745249E-02,.4403192E-02,.4867213E-02,&
     & .5191513E-02,.5380406E-02,.5427689E-02,.5240706E-02,.3946442E-02/

      data   absa(316:450, 4)   /                                       &
     & .1888206E-02,.3101989E-02,.3625968E-02,.3971182E-02,.4202567E-02,&
     & .4344619E-02,.4384538E-02,.4251354E-02,.3181821E-02,.1928325E-02,&
     & .3162679E-02,.3693527E-02,.4045345E-02,.4282687E-02,.4424622E-02,&
     & .4458656E-02,.4312656E-02,.3213796E-02,.1970231E-02,.3226941E-02,&
     & .3764384E-02,.4127675E-02,.4371933E-02,.4510091E-02,.4538985E-02,&
     & .4381451E-02,.3253041E-02,.2016368E-02,.3294141E-02,.3842606E-02,&
     & .4216009E-02,.4467372E-02,.4604053E-02,.4627489E-02,.4457971E-02,&
     & .3298935E-02,.2065463E-02,.3369431E-02,.3930153E-02,.4311858E-02,&
     & .4569900E-02,.4708801E-02,.4723164E-02,.4542339E-02,.3355283E-02,&
     & .1679110E-02,.2762441E-02,.3217170E-02,.3511424E-02,.3699441E-02,&
     & .3807111E-02,.3820305E-02,.3685883E-02,.2768596E-02,.1718111E-02,&
     & .2820629E-02,.3280981E-02,.3579835E-02,.3771324E-02,.3877601E-02,&
     & .3885055E-02,.3740150E-02,.2792302E-02,.1759096E-02,.2880777E-02,&
     & .3347449E-02,.3654997E-02,.3849489E-02,.3952528E-02,.3955472E-02,&
     & .3800453E-02,.2822088E-02,.1804159E-02,.2943869E-02,.3420297E-02,&
     & .3734906E-02,.3934456E-02,.4035234E-02,.4031563E-02,.3866511E-02,&
     & .2856445E-02,.1852018E-02,.3014249E-02,.3500208E-02,.3821348E-02,&
     & .4026208E-02,.4126369E-02,.4113254E-02,.3937685E-02,.2897579E-02,&
     & .1485209E-02,.2447369E-02,.2842719E-02,.3098235E-02,.3261684E-02,&
     & .3343620E-02,.3333197E-02,.3190317E-02,.4072685E-02,.1522667E-02,&
     & .2503774E-02,.2902412E-02,.3163342E-02,.3327673E-02,.3406943E-02,&
     & .3392715E-02,.3239642E-02,.4013646E-02,.1562338E-02,.2561154E-02,&
     & .2966329E-02,.3232897E-02,.3398043E-02,.3474218E-02,.3456041E-02,&
     & .3293328E-02,.3960211E-02,.1605822E-02,.2619463E-02,.3035388E-02,&
     & .3307311E-02,.3472770E-02,.3548127E-02,.3523210E-02,.3351454E-02,&
     & .3918895E-02,.1652396E-02,.2684419E-02,.3109623E-02,.3386474E-02,&
     & .3553371E-02,.3627950E-02,.3594923E-02,.3414383E-02,.3895295E-02/

      data   absa(451:585, 4)   /                                       &
     & .1327826E-02,.2178249E-02,.2523484E-02,.2748178E-02,.2891129E-02,&
     & .2953926E-02,.2925626E-02,.2774151E-02,.5032627E-02,.1363836E-02,&
     & .2231507E-02,.2582331E-02,.2811256E-02,.2953861E-02,.3013652E-02,&
     & .2982332E-02,.2820761E-02,.4894171E-02,.1402555E-02,.2284583E-02,&
     & .2644926E-02,.2877676E-02,.3020940E-02,.3077742E-02,.3041427E-02,&
     & .2870356E-02,.4763815E-02,.1444850E-02,.2341577E-02,.2711315E-02,&
     & .2948500E-02,.3091276E-02,.3147103E-02,.3103922E-02,.2923523E-02,&
     & .4657359E-02,.1491126E-02,.2405746E-02,.2783605E-02,.3023985E-02,&
     & .3167614E-02,.3221278E-02,.3170403E-02,.2981808E-02,.4593950E-02,&
     & .1198258E-02,.1938520E-02,.2240507E-02,.2433881E-02,.2549827E-02,&
     & .2596355E-02,.2561600E-02,.2410177E-02,.5165038E-02,.1231984E-02,&
     & .1988536E-02,.2296997E-02,.2493517E-02,.2610218E-02,.2654628E-02,&
     & .2615348E-02,.2452955E-02,.4998695E-02,.1267303E-02,.2040113E-02,&
     & .2355817E-02,.2557194E-02,.2673983E-02,.2716145E-02,.2670737E-02,&
     & .2497659E-02,.4849281E-02,.1307299E-02,.2095435E-02,.2418826E-02,&
     & .2623945E-02,.2740846E-02,.2781831E-02,.2728441E-02,.2545893E-02,&
     & .4735365E-02,.1350768E-02,.2157279E-02,.2487383E-02,.2694912E-02,&
     & .2813081E-02,.2851205E-02,.2790976E-02,.2599947E-02,.4638902E-02,&
     & .1094611E-02,.1729442E-02,.1987775E-02,.2148646E-02,.2243224E-02,&
     & .2274670E-02,.2235199E-02,.2090113E-02,.4614858E-02,.1125402E-02,&
     & .1775981E-02,.2041820E-02,.2205584E-02,.2300560E-02,.2330200E-02,&
     & .2284917E-02,.2128857E-02,.4463987E-02,.1159259E-02,.1824840E-02,&
     & .2098113E-02,.2265637E-02,.2360409E-02,.2388227E-02,.2336076E-02,&
     & .2169572E-02,.4338182E-02,.1196616E-02,.1877338E-02,.2157749E-02,&
     & .2328747E-02,.2423999E-02,.2448955E-02,.2390335E-02,.2214290E-02,&
     & .4232022E-02,.1237500E-02,.1937731E-02,.2222832E-02,.2395916E-02,&
     & .2492122E-02,.2513855E-02,.2450198E-02,.2264343E-02,.4147518E-02/

      data   absa(  1:180, 5)   /                                       &
     & .7569383E-01,.7127726E-01,.7072055E-01,.7167894E-01,.7293550E-01,&
     & .7244916E-01,.7056707E-01,.6926245E-01,.7313178E-01,.7613347E-01,&
     & .7151997E-01,.7077247E-01,.7153549E-01,.7263614E-01,.7201405E-01,&
     & .7009892E-01,.6889573E-01,.7285660E-01,.7647835E-01,.7168221E-01,&
     & .7076014E-01,.7134341E-01,.7231870E-01,.7159794E-01,.6969923E-01,&
     & .6859261E-01,.7264748E-01,.7670016E-01,.7175750E-01,.7068442E-01,&
     & .7110712E-01,.7197697E-01,.7120664E-01,.6933990E-01,.6836243E-01,&
     & .7255382E-01,.7691373E-01,.7183982E-01,.7062341E-01,.7088895E-01,&
     & .7164992E-01,.7087382E-01,.6904623E-01,.6822798E-01,.7257501E-01,&
     & .7421380E-01,.7089850E-01,.7145266E-01,.7330346E-01,.7466460E-01,&
     & .7395589E-01,.7188620E-01,.7007745E-01,.7342812E-01,.7470315E-01,&
     & .7115943E-01,.7149768E-01,.7312749E-01,.7437682E-01,.7354337E-01,&
     & .7146237E-01,.6973763E-01,.7320381E-01,.7511576E-01,.7137057E-01,&
     & .7150213E-01,.7293552E-01,.7408060E-01,.7316493E-01,.7110190E-01,&
     & .6944710E-01,.7305691E-01,.7543079E-01,.7151417E-01,.7146139E-01,&
     & .7272686E-01,.7376978E-01,.7280501E-01,.7076206E-01,.6923458E-01,&
     & .7300795E-01,.7568786E-01,.7161383E-01,.7139697E-01,.7250907E-01,&
     & .7343069E-01,.7248294E-01,.7048791E-01,.6909555E-01,.7306492E-01,&
     & .7230673E-01,.7021426E-01,.7189722E-01,.7451771E-01,.7579082E-01,&
     & .7503288E-01,.7282416E-01,.7049154E-01,.7299906E-01,.7286315E-01,&
     & .7051801E-01,.7197408E-01,.7438907E-01,.7552182E-01,.7466506E-01,&
     & .7245855E-01,.7016762E-01,.7281308E-01,.7332177E-01,.7075910E-01,&
     & .7199437E-01,.7424541E-01,.7524477E-01,.7430774E-01,.7210437E-01,&
     & .6988899E-01,.7270099E-01,.7375460E-01,.7098234E-01,.7200912E-01,&
     & .7408957E-01,.7497762E-01,.7399034E-01,.7179310E-01,.6966381E-01,&
     & .7266033E-01,.7410607E-01,.7113688E-01,.7196716E-01,.7387325E-01,&
     & .7467008E-01,.7369947E-01,.7152362E-01,.6950511E-01,.7269904E-01,&
     & .7006148E-01,.6923816E-01,.7203176E-01,.7518932E-01,.7630010E-01,&
     & .7558723E-01,.7324666E-01,.7030687E-01,.7173847E-01,.7070776E-01,&
     & .6960480E-01,.7215580E-01,.7510526E-01,.7608714E-01,.7528596E-01,&
     & .7291553E-01,.7001785E-01,.7160799E-01,.7124096E-01,.6989489E-01,&
     & .7221590E-01,.7501608E-01,.7584481E-01,.7498936E-01,.7259196E-01,&
     & .6976034E-01,.7152087E-01,.7170772E-01,.7013752E-01,.7222754E-01,&
     & .7487010E-01,.7559690E-01,.7471465E-01,.7231022E-01,.6954421E-01,&
     & .7149408E-01,.7217297E-01,.7036437E-01,.7224475E-01,.7470987E-01,&
     & .7536578E-01,.7448130E-01,.7207643E-01,.6939518E-01,.7153955E-01/

      data   absa(181:315, 5)   /                                       &
     & .6746543E-01,.6789511E-01,.7175811E-01,.7512073E-01,.7623306E-01,&
     & .7554875E-01,.7299624E-01,.6947460E-01,.6972139E-01,.6817808E-01,&
     & .6831863E-01,.7191880E-01,.7511721E-01,.7607693E-01,.7533587E-01,&
     & .7272951E-01,.6921640E-01,.6962219E-01,.6878127E-01,.6866239E-01,&
     & .7201451E-01,.7507768E-01,.7588347E-01,.7512346E-01,.7245380E-01,&
     & .6898486E-01,.6955882E-01,.6931769E-01,.6894903E-01,.7206127E-01,&
     & .7499506E-01,.7568911E-01,.7490453E-01,.7221264E-01,.6878897E-01,&
     & .6955676E-01,.6984062E-01,.6921967E-01,.7210380E-01,.7490218E-01,&
     & .7552190E-01,.7471412E-01,.7202114E-01,.6866058E-01,.6962597E-01,&
     & .6453797E-01,.6619435E-01,.7096321E-01,.7435120E-01,.7557506E-01,&
     & .7489358E-01,.7213608E-01,.6797549E-01,.6697223E-01,.6531448E-01,&
     & .6667806E-01,.7120136E-01,.7443855E-01,.7550608E-01,.7477578E-01,&
     & .7192188E-01,.6776652E-01,.6691692E-01,.6599601E-01,.6708647E-01,&
     & .7137864E-01,.7446953E-01,.7539664E-01,.7463387E-01,.7169803E-01,&
     & .6757635E-01,.6690442E-01,.6658455E-01,.6741621E-01,.7148953E-01,&
     & .7446547E-01,.7527439E-01,.7447360E-01,.7150646E-01,.6741051E-01,&
     & .6692725E-01,.6710838E-01,.6769670E-01,.7154133E-01,.7442810E-01,&
     & .7515746E-01,.7430715E-01,.7133898E-01,.6729366E-01,.6700931E-01,&
     & .6127817E-01,.6414812E-01,.6958749E-01,.7293419E-01,.7429145E-01,&
     & .7361146E-01,.7059124E-01,.6583890E-01,.6355716E-01,.6212572E-01,&
     & .6469369E-01,.6993766E-01,.7312422E-01,.7433484E-01,.7357556E-01,&
     & .7045275E-01,.6569828E-01,.6357688E-01,.6288706E-01,.6516584E-01,&
     & .7021271E-01,.7326619E-01,.7432982E-01,.7348213E-01,.7031560E-01,&
     & .6555824E-01,.6360213E-01,.6354801E-01,.6556305E-01,.7040342E-01,&
     & .7334105E-01,.7430526E-01,.7337063E-01,.7018757E-01,.6542809E-01,&
     & .6365427E-01,.6412664E-01,.6589381E-01,.7053384E-01,.7337186E-01,&
     & .7426386E-01,.7326847E-01,.7006536E-01,.6533352E-01,.6375247E-01/

      data   absa(316:450, 5)   /                                       &
     & .5769446E-01,.6178260E-01,.6765439E-01,.7096937E-01,.7236097E-01,&
     & .7162253E-01,.6841692E-01,.6322137E-01,.5970130E-01,.5863799E-01,&
     & .6240216E-01,.6811545E-01,.7126851E-01,.7252091E-01,.7168532E-01,&
     & .6838432E-01,.6313347E-01,.5975507E-01,.5948073E-01,.6294022E-01,&
     & .6847433E-01,.7150605E-01,.7262755E-01,.7166820E-01,.6832826E-01,&
     & .6301598E-01,.5980255E-01,.6019384E-01,.6338337E-01,.6874609E-01,&
     & .7165133E-01,.7270983E-01,.7163959E-01,.6825300E-01,.6292421E-01,&
     & .5988086E-01,.6082893E-01,.6376171E-01,.6894902E-01,.7174796E-01,&
     & .7275284E-01,.7161814E-01,.6819849E-01,.6284853E-01,.5997885E-01,&
     & .5384218E-01,.5913297E-01,.6512641E-01,.6847022E-01,.6981558E-01,&
     & .6898244E-01,.6574451E-01,.6014063E-01,.5493742E-01,.5486164E-01,&
     & .5981373E-01,.6568933E-01,.6889734E-01,.7011656E-01,.6915288E-01,&
     & .6580592E-01,.6008501E-01,.5501675E-01,.5576632E-01,.6040455E-01,&
     & .6615529E-01,.6921938E-01,.7034252E-01,.6924287E-01,.6580255E-01,&
     & .6001888E-01,.5509485E-01,.5653718E-01,.6090010E-01,.6649772E-01,&
     & .6944333E-01,.7051698E-01,.6931819E-01,.6580285E-01,.5995294E-01,&
     & .5518924E-01,.5724006E-01,.6133632E-01,.6675529E-01,.6962363E-01,&
     & .7062688E-01,.6937140E-01,.6580880E-01,.5991082E-01,.5530789E-01,&
     & .4986911E-01,.5625720E-01,.6219195E-01,.6550333E-01,.6673087E-01,&
     & .6585114E-01,.6260934E-01,.5664518E-01,.4665462E-01,.5094960E-01,&
     & .5701871E-01,.6284240E-01,.6603147E-01,.6716332E-01,.6612920E-01,&
     & .6275302E-01,.5663929E-01,.4676883E-01,.5188210E-01,.5764984E-01,&
     & .6336343E-01,.6646529E-01,.6749570E-01,.6632743E-01,.6282719E-01,&
     & .5662832E-01,.4689733E-01,.5269455E-01,.5818682E-01,.6376950E-01,&
     & .6678498E-01,.6774711E-01,.6647465E-01,.6289082E-01,.5661653E-01,&
     & .4705226E-01,.5340814E-01,.5863202E-01,.6410648E-01,.6704487E-01,&
     & .6795408E-01,.6661171E-01,.6295124E-01,.5663255E-01,.4724050E-01/

      data   absa(451:585, 5)   /                                       &
     & .4634040E-01,.5350954E-01,.5924519E-01,.6235764E-01,.6342928E-01,&
     & .6250075E-01,.5915701E-01,.5294180E-01,.3981576E-01,.4736892E-01,&
     & .5426407E-01,.5988551E-01,.6297853E-01,.6392000E-01,.6284030E-01,&
     & .5933938E-01,.5297399E-01,.3998733E-01,.4826061E-01,.5489369E-01,&
     & .6040498E-01,.6345651E-01,.6430714E-01,.6308766E-01,.5949295E-01,&
     & .5300748E-01,.4016986E-01,.4903376E-01,.5542677E-01,.6085024E-01,&
     & .6383044E-01,.6465029E-01,.6331517E-01,.5962492E-01,.5307512E-01,&
     & .4039950E-01,.4974119E-01,.5589358E-01,.6123797E-01,.6415006E-01,&
     & .6494753E-01,.6353101E-01,.5976611E-01,.5317271E-01,.4065617E-01,&
     & .4271897E-01,.5052183E-01,.5599851E-01,.5891678E-01,.5984422E-01,&
     & .5881804E-01,.5542417E-01,.4913608E-01,.3713988E-01,.4368596E-01,&
     & .5124809E-01,.5664483E-01,.5957881E-01,.6039071E-01,.5919218E-01,&
     & .5567250E-01,.4923797E-01,.3723585E-01,.4453270E-01,.5188770E-01,&
     & .5720401E-01,.6010232E-01,.6087078E-01,.5953237E-01,.5588506E-01,&
     & .4936407E-01,.3745797E-01,.4529021E-01,.5243444E-01,.5768876E-01,&
     & .6053783E-01,.6126963E-01,.5984905E-01,.5610497E-01,.4950780E-01,&
     & .3755909E-01,.4600687E-01,.5293281E-01,.5813232E-01,.6093595E-01,&
     & .6163456E-01,.6016722E-01,.5635091E-01,.4968371E-01,.3773583E-01,&
     & .3908364E-01,.4730786E-01,.5250316E-01,.5524959E-01,.5601420E-01,&
     & .5485012E-01,.5154058E-01,.4532019E-01,.3566372E-01,.3998663E-01,&
     & .4803344E-01,.5319099E-01,.5592987E-01,.5663652E-01,.5530572E-01,&
     & .5183777E-01,.4553398E-01,.3583037E-01,.4078498E-01,.4865307E-01,&
     & .5379331E-01,.5649229E-01,.5715869E-01,.5574035E-01,.5214488E-01,&
     & .4574777E-01,.3594123E-01,.4154329E-01,.4922239E-01,.5434666E-01,&
     & .5701428E-01,.5763600E-01,.5616885E-01,.5247198E-01,.4597428E-01,&
     & .3604552E-01,.4229775E-01,.4978726E-01,.5486411E-01,.5751694E-01,&
     & .5809005E-01,.5659205E-01,.5281871E-01,.4623884E-01,.3624776E-01/

      data   absa(  1:180, 6)   /                                       &
     & .4846620E+00,.4254449E+00,.3666979E+00,.3085003E+00,.2601346E+00,&
     & .2464956E+00,.2660087E+00,.3021063E+00,.3287759E+00,.4745246E+00,&
     & .4166752E+00,.3592695E+00,.3025580E+00,.2553943E+00,.2432268E+00,&
     & .2629249E+00,.2987363E+00,.3245474E+00,.4658709E+00,.4092568E+00,&
     & .3530642E+00,.2977163E+00,.2515626E+00,.2405793E+00,.2602783E+00,&
     & .2957536E+00,.3209745E+00,.4575145E+00,.4020922E+00,.3470681E+00,&
     & .2930793E+00,.2481591E+00,.2381801E+00,.2579191E+00,.2931017E+00,&
     & .3178371E+00,.4497966E+00,.3954548E+00,.3415131E+00,.2891031E+00,&
     & .2456319E+00,.2359968E+00,.2560087E+00,.2910169E+00,.3153507E+00,&
     & .5233277E+00,.4592284E+00,.3955317E+00,.3326601E+00,.2881387E+00,&
     & .2814594E+00,.3054605E+00,.3475919E+00,.3858173E+00,.5126637E+00,&
     & .4500477E+00,.3878258E+00,.3267077E+00,.2828332E+00,.2774387E+00,&
     & .3014139E+00,.3430855E+00,.3803776E+00,.5032811E+00,.4419694E+00,&
     & .3810629E+00,.3215530E+00,.2784520E+00,.2739908E+00,.2977829E+00,&
     & .3391699E+00,.3757057E+00,.4943912E+00,.4343294E+00,.3746578E+00,&
     & .3166441E+00,.2747404E+00,.2708931E+00,.2946970E+00,.3358529E+00,&
     & .3717479E+00,.4856925E+00,.4268266E+00,.3684341E+00,.3122038E+00,&
     & .2717769E+00,.2679905E+00,.2921861E+00,.3332832E+00,.3686250E+00,&
     & .5659721E+00,.4964838E+00,.4273442E+00,.3613333E+00,.3229778E+00,&
     & .3207717E+00,.3487909E+00,.3970188E+00,.4483696E+00,.5545946E+00,&
     & .4866491E+00,.4190737E+00,.3544937E+00,.3171184E+00,.3157821E+00,&
     & .3434204E+00,.3913987E+00,.4416557E+00,.5441901E+00,.4776455E+00,&
     & .4115043E+00,.3481950E+00,.3119787E+00,.3113749E+00,.3389006E+00,&
     & .3865528E+00,.4358447E+00,.5345476E+00,.4693294E+00,.4045767E+00,&
     & .3427926E+00,.3075766E+00,.3073312E+00,.3350898E+00,.3825859E+00,&
     & .4311541E+00,.5246854E+00,.4609334E+00,.3978727E+00,.3380061E+00,&
     & .3038003E+00,.3036818E+00,.3320521E+00,.3795333E+00,.4274614E+00,&
     & .6114156E+00,.5360856E+00,.4610841E+00,.3944939E+00,.3631225E+00,&
     & .3640797E+00,.3957193E+00,.4512458E+00,.5140119E+00,.5990211E+00,&
     & .5253614E+00,.4520735E+00,.3868300E+00,.3563118E+00,.3576976E+00,&
     & .3893012E+00,.4443146E+00,.5057705E+00,.5878031E+00,.5156587E+00,&
     & .4439389E+00,.3797822E+00,.3504051E+00,.3520245E+00,.3838216E+00,&
     & .4383993E+00,.4987384E+00,.5771932E+00,.5065469E+00,.4365968E+00,&
     & .3737334E+00,.3451671E+00,.3469143E+00,.3791776E+00,.4335158E+00,&
     & .4929659E+00,.5672493E+00,.4980854E+00,.4298759E+00,.3686864E+00,&
     & .3406738E+00,.3425037E+00,.3753823E+00,.4296076E+00,.4882770E+00/

      data   absa(181:315, 6)   /                                       &
     & .6599457E+00,.5784248E+00,.4971773E+00,.4337450E+00,.4071444E+00,&
     & .4110564E+00,.4472427E+00,.5099709E+00,.5830348E+00,.6468482E+00,&
     & .5670871E+00,.4876631E+00,.4250777E+00,.3993382E+00,.4031458E+00,&
     & .4395792E+00,.5018776E+00,.5735256E+00,.6348396E+00,.5566698E+00,&
     & .4790978E+00,.4172804E+00,.3925206E+00,.3961110E+00,.4330389E+00,&
     & .4948939E+00,.5653109E+00,.6234166E+00,.5469278E+00,.4713504E+00,&
     & .4102887E+00,.3864066E+00,.3900014E+00,.4273461E+00,.4889966E+00,&
     & .5583254E+00,.6128604E+00,.5379244E+00,.4642610E+00,.4042901E+00,&
     & .3809946E+00,.3847891E+00,.4225472E+00,.4840732E+00,.5525507E+00,&
     & .7118405E+00,.6237026E+00,.5373473E+00,.4788483E+00,.4552052E+00,&
     & .4622830E+00,.5030364E+00,.5735317E+00,.6569249E+00,.6980451E+00,&
     & .6117423E+00,.5269042E+00,.4690257E+00,.4461057E+00,.4528737E+00,&
     & .4940744E+00,.5641178E+00,.6458719E+00,.6851401E+00,.6005735E+00,&
     & .5173938E+00,.4602048E+00,.4380661E+00,.4445391E+00,.4863645E+00,&
     & .5558444E+00,.6361244E+00,.6731826E+00,.5903092E+00,.5088902E+00,&
     & .4521990E+00,.4308774E+00,.4373808E+00,.4795420E+00,.5488504E+00,&
     & .6279339E+00,.6618869E+00,.5806518E+00,.5011956E+00,.4451372E+00,&
     & .4243261E+00,.4313958E+00,.4738124E+00,.5429837E+00,.6210544E+00,&
     & .7668972E+00,.6717524E+00,.5822052E+00,.5289402E+00,.5074279E+00,&
     & .5174762E+00,.5633483E+00,.6412958E+00,.7351146E+00,.7523724E+00,&
     & .6591380E+00,.5705110E+00,.5178052E+00,.4967826E+00,.5066462E+00,&
     & .5528830E+00,.6302350E+00,.7222090E+00,.7385919E+00,.6472212E+00,&
     & .5598496E+00,.5075245E+00,.4872782E+00,.4974170E+00,.5435990E+00,&
     & .6206660E+00,.7110675E+00,.7257857E+00,.6361589E+00,.5503268E+00,&
     & .4984685E+00,.4786719E+00,.4893603E+00,.5354629E+00,.6125751E+00,&
     & .7016399E+00,.7138973E+00,.6259486E+00,.5417002E+00,.4905437E+00,&
     & .4710327E+00,.4822383E+00,.5286586E+00,.6057803E+00,.6935878E+00/

      data   absa(316:450, 6)   /                                       &
     & .8249134E+00,.7223579E+00,.6318367E+00,.5828461E+00,.5642052E+00,&
     & .5774889E+00,.6274321E+00,.7112179E+00,.8154752E+00,.8094743E+00,&
     & .7089425E+00,.6188253E+00,.5703294E+00,.5519242E+00,.5651410E+00,&
     & .6152177E+00,.6988740E+00,.8012054E+00,.7948100E+00,.6962635E+00,&
     & .6071335E+00,.5587642E+00,.5408447E+00,.5546817E+00,.6044782E+00,&
     & .6883629E+00,.7888678E+00,.7814861E+00,.6847483E+00,.5966332E+00,&
     & .5488036E+00,.5307506E+00,.5452968E+00,.5952041E+00,.6791251E+00,&
     & .7781169E+00,.7689261E+00,.6739251E+00,.5871415E+00,.5399440E+00,&
     & .5219107E+00,.5368668E+00,.5871449E+00,.6714540E+00,.7691532E+00,&
     & .8852145E+00,.7750456E+00,.6871055E+00,.6408984E+00,.6254709E+00,&
     & .6421647E+00,.6941504E+00,.7835789E+00,.8985979E+00,.8691028E+00,&
     & .7609767E+00,.6728330E+00,.6267061E+00,.6112664E+00,.6281600E+00,&
     & .6803225E+00,.7701312E+00,.8830032E+00,.8539650E+00,.7478668E+00,&
     & .6598673E+00,.6140672E+00,.5985796E+00,.6161528E+00,.6684674E+00,&
     & .7583442E+00,.8692469E+00,.8400620E+00,.7358241E+00,.6484499E+00,&
     & .6030312E+00,.5871483E+00,.6051785E+00,.6578534E+00,.7481252E+00,&
     & .8573340E+00,.8267308E+00,.7243453E+00,.6382877E+00,.5930271E+00,&
     & .5773599E+00,.5955257E+00,.6486068E+00,.7393848E+00,.8471526E+00,&
     & .9457456E+00,.8285939E+00,.7451511E+00,.7019917E+00,.6899693E+00,&
     & .7089637E+00,.7625561E+00,.8572460E+00,.9824613E+00,.9291323E+00,&
     & .8136971E+00,.7298541E+00,.6864425E+00,.6740084E+00,.6934007E+00,&
     & .7472875E+00,.8425945E+00,.9655705E+00,.9139136E+00,.8002614E+00,&
     & .7162229E+00,.6723573E+00,.6598915E+00,.6797955E+00,.7340436E+00,&
     & .8294611E+00,.9506098E+00,.8996396E+00,.7878323E+00,.7040173E+00,&
     & .6600638E+00,.6474290E+00,.6677087E+00,.7222527E+00,.8180613E+00,&
     & .9373243E+00,.8863343E+00,.7763696E+00,.6928464E+00,.6489803E+00,&
     & .6362699E+00,.6568098E+00,.7118980E+00,.8079921E+00,.9257131E+00/

      data   absa(451:585, 6)   /                                       &
     & .9984509E+00,.8767023E+00,.7981236E+00,.7590051E+00,.7500479E+00,&
     & .7704148E+00,.8258274E+00,.9236821E+00,.1057116E+01,.9825580E+00,&
     & .8620354E+00,.7830513E+00,.7424078E+00,.7334532E+00,.7542144E+00,&
     & .8101460E+00,.9086138E+00,.1040043E+01,.9678310E+00,.8486729E+00,&
     & .7694638E+00,.7279026E+00,.7187504E+00,.7400787E+00,.7961034E+00,&
     & .8951540E+00,.1024689E+01,.9541401E+00,.8363966E+00,.7569500E+00,&
     & .7151152E+00,.7053524E+00,.7272043E+00,.7836771E+00,.8829254E+00,&
     & .1010610E+01,.9409238E+00,.8247336E+00,.7452605E+00,.7034806E+00,&
     & .6932724E+00,.7155504E+00,.7725713E+00,.8720814E+00,.9982035E+00,&
     & .1051209E+01,.9265442E+00,.8532619E+00,.8179703E+00,.8119095E+00,&
     & .8341371E+00,.8903756E+00,.9888756E+00,.1095956E+01,.1036235E+01,&
     & .9123511E+00,.8382101E+00,.8010032E+00,.7947542E+00,.8176096E+00,&
     & .8740008E+00,.9731309E+00,.1080421E+01,.1022216E+01,.8989889E+00,&
     & .8242808E+00,.7860446E+00,.7790076E+00,.8024437E+00,.8593689E+00,&
     & .9586866E+00,.1065303E+01,.1008831E+01,.8865583E+00,.8113619E+00,&
     & .7726307E+00,.7650415E+00,.7885957E+00,.8460657E+00,.9457463E+00,&
     & .1053284E+01,.9956237E+00,.8746029E+00,.7991663E+00,.7602091E+00,&
     & .7523241E+00,.7758775E+00,.8338469E+00,.9341164E+00,.1042323E+01,&
     & .1103094E+01,.9779325E+00,.9101471E+00,.8783812E+00,.8752086E+00,&
     & .8997168E+00,.9549270E+00,.1052106E+01,.1130777E+01,.1089042E+01,&
     & .9638679E+00,.8946771E+00,.8612419E+00,.8571856E+00,.8822649E+00,&
     & .9380724E+00,.1035137E+01,.1114750E+01,.1075742E+01,.9508309E+00,&
     & .8802630E+00,.8459304E+00,.8411000E+00,.8660581E+00,.9224269E+00,&
     & .1019820E+01,.1101053E+01,.1062410E+01,.9382231E+00,.8666081E+00,&
     & .8315446E+00,.8263075E+00,.8509882E+00,.9079071E+00,.1005967E+01,&
     & .1089316E+01,.1048876E+01,.9255641E+00,.8536249E+00,.8179717E+00,&
     & .8126191E+00,.8371360E+00,.8945910E+00,.9933411E+00,.1078307E+01/

!     the array absb(235,6) (kb(5,13:59,6)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 6,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      data   absb(  1:120, 1)   /                                       &
     & .2455108E-02,.2494284E-02,.2520068E-02,.2540222E-02,.2547453E-02,&
     & .2053818E-02,.2083023E-02,.2101215E-02,.2116145E-02,.2123172E-02,&
     & .1734639E-02,.1758680E-02,.1772416E-02,.1781767E-02,.1788224E-02,&
     & .1482314E-02,.1503678E-02,.1512899E-02,.1519630E-02,.1524757E-02,&
     & .1284745E-02,.1304855E-02,.1312793E-02,.1316311E-02,.1318005E-02,&
     & .1129086E-02,.1146597E-02,.1156453E-02,.1156435E-02,.1156401E-02,&
     & .9921713E-03,.1008990E-02,.1018291E-02,.1020213E-02,.1018365E-02,&
     & .8471850E-03,.8618578E-03,.8693996E-03,.8719597E-03,.8699465E-03,&
     & .7178765E-03,.7312448E-03,.7368809E-03,.7383122E-03,.7377363E-03,&
     & .6069271E-03,.6192294E-03,.6230026E-03,.6249060E-03,.6240653E-03,&
     & .5123512E-03,.5240166E-03,.5274574E-03,.5289784E-03,.5283974E-03,&
     & .4327672E-03,.4433865E-03,.4469695E-03,.4481477E-03,.4479432E-03,&
     & .3651769E-03,.3741862E-03,.3784929E-03,.3797155E-03,.3799202E-03,&
     & .3081802E-03,.3145936E-03,.3193752E-03,.3209357E-03,.3216224E-03,&
     & .2600564E-03,.2641825E-03,.2692918E-03,.2713207E-03,.2719491E-03,&
     & .2189889E-03,.2226081E-03,.2271490E-03,.2294557E-03,.2302383E-03,&
     & .1846009E-03,.1885356E-03,.1916569E-03,.1944705E-03,.1953784E-03,&
     & .1569872E-03,.1601747E-03,.1622966E-03,.1649634E-03,.1666332E-03,&
     & .1341940E-03,.1363878E-03,.1381676E-03,.1401546E-03,.1421984E-03,&
     & .1150202E-03,.1162012E-03,.1182134E-03,.1193516E-03,.1212431E-03,&
     & .9900466E-04,.9989409E-04,.1008343E-03,.1022803E-03,.1033529E-03,&
     & .8523323E-04,.8592559E-04,.8646982E-04,.8752563E-04,.8834547E-04,&
     & .7355247E-04,.7410040E-04,.7455079E-04,.7505627E-04,.7561834E-04,&
     & .6386391E-04,.6399156E-04,.6452062E-04,.6507966E-04,.6511678E-04/

      data   absb(121:235, 1)   /                                       &
     & .5378377E-04,.5383807E-04,.5419748E-04,.5461584E-04,.5475942E-04,&
     & .4529476E-04,.4536607E-04,.4551849E-04,.4582121E-04,.4594236E-04,&
     & .3805002E-04,.3835500E-04,.3837164E-04,.3841587E-04,.3870224E-04,&
     & .3134721E-04,.3153158E-04,.3158800E-04,.3159241E-04,.3173210E-04,&
     & .2572678E-04,.2581660E-04,.2592636E-04,.2593449E-04,.2604293E-04,&
     & .2111340E-04,.2114297E-04,.2123906E-04,.2126974E-04,.2132702E-04,&
     & .1724042E-04,.1721678E-04,.1726492E-04,.1735856E-04,.1734777E-04,&
     & .1400853E-04,.1398539E-04,.1400256E-04,.1407486E-04,.1407744E-04,&
     & .1138599E-04,.1138052E-04,.1136754E-04,.1138325E-04,.1144462E-04,&
     & .9234875E-05,.9228129E-05,.9225511E-05,.9219129E-05,.9274721E-05,&
     & .7450018E-05,.7494669E-05,.7491514E-05,.7465424E-05,.7498018E-05,&
     & .5987641E-05,.6060252E-05,.6077441E-05,.6062453E-05,.6062401E-05,&
     & .4810640E-05,.4877970E-05,.4932441E-05,.4916522E-05,.4908248E-05,&
     & .3869095E-05,.3916912E-05,.3981052E-05,.3989636E-05,.3985085E-05,&
     & .3125584E-05,.3158316E-05,.3201951E-05,.3234304E-05,.3229423E-05,&
     & .2526771E-05,.2539982E-05,.2565764E-05,.2606671E-05,.2620099E-05,&
     & .2033741E-05,.2048230E-05,.2064945E-05,.2094217E-05,.2117279E-05,&
     & .1636949E-05,.1651544E-05,.1664769E-05,.1682082E-05,.1707938E-05,&
     & .1320644E-05,.1329014E-05,.1342957E-05,.1355072E-05,.1374367E-05,&
     & .1063144E-05,.1069247E-05,.1082985E-05,.1092178E-05,.1104704E-05,&
     & .8523022E-06,.8613254E-06,.8703544E-06,.8805758E-06,.8889748E-06,&
     & .6821306E-06,.6939226E-06,.7000980E-06,.7103676E-06,.7161581E-06,&
     & .5473000E-06,.5610035E-06,.5662359E-06,.5739016E-06,.5795604E-06/

      data   absb(  1:120, 2)   /                                       &
     & .1286743E-01,.1298016E-01,.1307688E-01,.1314196E-01,.1318687E-01,&
     & .1084483E-01,.1095286E-01,.1104372E-01,.1107916E-01,.1110333E-01,&
     & .9173853E-02,.9249031E-02,.9308829E-02,.9349446E-02,.9373813E-02,&
     & .7773824E-02,.7815445E-02,.7855507E-02,.7887027E-02,.7894494E-02,&
     & .6635671E-02,.6658973E-02,.6677297E-02,.6693544E-02,.6694290E-02,&
     & .5714053E-02,.5714835E-02,.5716276E-02,.5728622E-02,.5719424E-02,&
     & .4953027E-02,.4935260E-02,.4928449E-02,.4921707E-02,.4912914E-02,&
     & .4202051E-02,.4179936E-02,.4170445E-02,.4154196E-02,.4145879E-02,&
     & .3546756E-02,.3522656E-02,.3509107E-02,.3496226E-02,.3483466E-02,&
     & .2986324E-02,.2962134E-02,.2946586E-02,.2935316E-02,.2924589E-02,&
     & .2520037E-02,.2493976E-02,.2477958E-02,.2467618E-02,.2459527E-02,&
     & .2128530E-02,.2102617E-02,.2086920E-02,.2077359E-02,.2068955E-02,&
     & .1796044E-02,.1774099E-02,.1757802E-02,.1748933E-02,.1740649E-02,&
     & .1511042E-02,.1493208E-02,.1478896E-02,.1469321E-02,.1461744E-02,&
     & .1272579E-02,.1259504E-02,.1246012E-02,.1235685E-02,.1229631E-02,&
     & .1074471E-02,.1063160E-02,.1050792E-02,.1041724E-02,.1036092E-02,&
     & .9106300E-03,.8993222E-03,.8888252E-03,.8805708E-03,.8768352E-03,&
     & .7732921E-03,.7632043E-03,.7552638E-03,.7485823E-03,.7491950E-03,&
     & .6597355E-03,.6512362E-03,.6451463E-03,.6404871E-03,.6392845E-03,&
     & .5654918E-03,.5596233E-03,.5545424E-03,.5500160E-03,.5486305E-03,&
     & .4880680E-03,.4838686E-03,.4792352E-03,.4746633E-03,.4742358E-03,&
     & .4240329E-03,.4198366E-03,.4158480E-03,.4147475E-03,.4118226E-03,&
     & .3707047E-03,.3670166E-03,.3635404E-03,.3627127E-03,.3598610E-03,&
     & .3269270E-03,.3238737E-03,.3206578E-03,.3192890E-03,.3166364E-03/

      data   absb(121:235, 2)   /                                       &
     & .2789970E-03,.2765079E-03,.2739652E-03,.2724815E-03,.2694007E-03,&
     & .2383170E-03,.2362195E-03,.2338966E-03,.2321906E-03,.2293814E-03,&
     & .2043421E-03,.2022653E-03,.1998851E-03,.1974199E-03,.1958820E-03,&
     & .1703638E-03,.1685921E-03,.1664446E-03,.1644332E-03,.1632693E-03,&
     & .1414648E-03,.1398222E-03,.1381078E-03,.1364105E-03,.1354620E-03,&
     & .1171714E-03,.1160394E-03,.1146163E-03,.1132446E-03,.1124903E-03,&
     & .9653894E-04,.9566019E-04,.9459747E-04,.9340503E-04,.9220628E-04,&
     & .7932991E-04,.7856317E-04,.7774927E-04,.7682321E-04,.7581347E-04,&
     & .6515421E-04,.6441758E-04,.6392018E-04,.6313941E-04,.6221003E-04,&
     & .5347743E-04,.5297072E-04,.5245890E-04,.5184866E-04,.5107466E-04,&
     & .4396154E-04,.4347687E-04,.4303733E-04,.4259063E-04,.4196826E-04,&
     & .3615915E-04,.3568995E-04,.3529007E-04,.3492298E-04,.3444102E-04,&
     & .2966497E-04,.2925157E-04,.2889217E-04,.2855828E-04,.2821985E-04,&
     & .2430508E-04,.2403992E-04,.2370451E-04,.2340581E-04,.2311943E-04,&
     & .1995845E-04,.1971625E-04,.1943619E-04,.1919411E-04,.1893202E-04,&
     & .1630421E-04,.1613833E-04,.1595861E-04,.1573142E-04,.1549637E-04,&
     & .1329543E-04,.1321689E-04,.1306432E-04,.1287719E-04,.1268925E-04,&
     & .1082302E-04,.1082655E-04,.1071501E-04,.1058492E-04,.1042620E-04,&
     & .8829908E-05,.8878449E-05,.8804000E-05,.8713675E-05,.8582871E-05,&
     & .7229747E-05,.7274906E-05,.7243006E-05,.7161434E-05,.7060580E-05,&
     & .5920720E-05,.5930473E-05,.5943800E-05,.5878317E-05,.5803165E-05,&
     & .4850374E-05,.4842730E-05,.4872181E-05,.4827114E-05,.4773736E-05,&
     & .3998381E-05,.3995704E-05,.4020867E-05,.3990354E-05,.3948657E-05/

      data   absb(  1:120, 3)   /                                       &
     & .5002398E-01,.5100799E-01,.5223444E-01,.5366230E-01,.5534006E-01,&
     & .4416861E-01,.4512229E-01,.4624963E-01,.4763751E-01,.4926106E-01,&
     & .3889418E-01,.3989178E-01,.4102991E-01,.4233451E-01,.4377504E-01,&
     & .3413273E-01,.3507125E-01,.3618746E-01,.3742424E-01,.3870722E-01,&
     & .2995928E-01,.3086478E-01,.3189959E-01,.3307320E-01,.3425277E-01,&
     & .2638019E-01,.2725271E-01,.2821585E-01,.2927539E-01,.3033342E-01,&
     & .2335920E-01,.2414637E-01,.2503317E-01,.2599890E-01,.2699514E-01,&
     & .2029208E-01,.2105095E-01,.2183638E-01,.2270657E-01,.2359078E-01,&
     & .1752887E-01,.1822677E-01,.1893693E-01,.1971216E-01,.2049024E-01,&
     & .1514301E-01,.1576265E-01,.1644949E-01,.1710856E-01,.1777047E-01,&
     & .1307907E-01,.1369568E-01,.1429327E-01,.1484508E-01,.1540994E-01,&
     & .1135002E-01,.1190407E-01,.1240533E-01,.1289830E-01,.1336006E-01,&
     & .9861342E-02,.1032874E-01,.1078971E-01,.1119067E-01,.1159988E-01,&
     & .8543786E-02,.8972343E-02,.9361670E-02,.9707599E-02,.1007270E-01,&
     & .7399186E-02,.7765826E-02,.8086950E-02,.8410397E-02,.8722569E-02,&
     & .6411375E-02,.6715139E-02,.6991916E-02,.7280002E-02,.7586032E-02,&
     & .5526867E-02,.5785983E-02,.6041283E-02,.6303233E-02,.6579648E-02,&
     & .4774311E-02,.4997352E-02,.5227375E-02,.5480339E-02,.5877541E-02,&
     & .4116828E-02,.4325497E-02,.4545463E-02,.4770394E-02,.5153971E-02,&
     & .3578769E-02,.3764998E-02,.3969666E-02,.4197341E-02,.4545465E-02,&
     & .3124717E-02,.3305659E-02,.3500480E-02,.3709794E-02,.4038550E-02,&
     & .2748728E-02,.2923454E-02,.3112152E-02,.3399419E-02,.3626701E-02,&
     & .2422516E-02,.2586643E-02,.2766372E-02,.3049770E-02,.3276646E-02,&
     & .2131791E-02,.2289101E-02,.2457066E-02,.2736600E-02,.2953943E-02/

      data   absb(121:235, 3)   /                                       &
     & .1828665E-02,.1969373E-02,.2131950E-02,.2386197E-02,.2591184E-02,&
     & .1565325E-02,.1696357E-02,.1846360E-02,.2083995E-02,.2278216E-02,&
     & .1339532E-02,.1462299E-02,.1603891E-02,.1761979E-02,.2010098E-02,&
     & .1123112E-02,.1231996E-02,.1361344E-02,.1506360E-02,.1731174E-02,&
     & .9386379E-03,.1036085E-02,.1150679E-02,.1280160E-02,.1487488E-02,&
     & .7829286E-03,.8681007E-03,.9712611E-03,.1088601E-02,.1279327E-02,&
     & .6439906E-03,.7179533E-03,.8068447E-03,.9121069E-03,.1038346E-02,&
     & .5251100E-03,.5895043E-03,.6665253E-03,.7582909E-03,.8728263E-03,&
     & .4272101E-03,.4822133E-03,.5483728E-03,.6283465E-03,.7297410E-03,&
     & .3455385E-03,.3919227E-03,.4490025E-03,.5186610E-03,.6080615E-03,&
     & .2772875E-03,.3163585E-03,.3648993E-03,.4245665E-03,.5029312E-03,&
     & .2215243E-03,.2545308E-03,.2950909E-03,.3456400E-03,.4140863E-03,&
     & .1762697E-03,.2034714E-03,.2368499E-03,.2802197E-03,.3391266E-03,&
     & .1409510E-03,.1632263E-03,.1911441E-03,.2283913E-03,.2789978E-03,&
     & .1124638E-03,.1310401E-03,.1540205E-03,.1857656E-03,.2289863E-03,&
     & .8954236E-04,.1047453E-03,.1237097E-03,.1505540E-03,.1870030E-03,&
     & .7098317E-04,.8306387E-04,.9912454E-04,.1212505E-03,.1519558E-03,&
     & .5681762E-04,.6665919E-04,.7982178E-04,.9805318E-04,.1239971E-03,&
     & .4562520E-04,.5359104E-04,.6438194E-04,.7935790E-04,.1011203E-03,&
     & .3653132E-04,.4295380E-04,.5170112E-04,.6400146E-04,.8208662E-04,&
     & .2913090E-04,.3439039E-04,.4140146E-04,.5133409E-04,.6632514E-04,&
     & .2332055E-04,.2755437E-04,.3319705E-04,.4115994E-04,.5350282E-04,&
     & .1945717E-04,.2306888E-04,.2793905E-04,.3476686E-04,.4551954E-04/

      data   absb(  1:120, 4)   /                                       &
     & .2878234E+00,.2961252E+00,.3050441E+00,.3147630E+00,.3253959E+00,&
     & .2743822E+00,.2825368E+00,.2911238E+00,.3003062E+00,.3104320E+00,&
     & .2632621E+00,.2715744E+00,.2807089E+00,.2905758E+00,.3014280E+00,&
     & .2470711E+00,.2556028E+00,.2648564E+00,.2753382E+00,.2872254E+00,&
     & .2312754E+00,.2397378E+00,.2488918E+00,.2593077E+00,.2714135E+00,&
     & .2159198E+00,.2248574E+00,.2346644E+00,.2452257E+00,.2571951E+00,&
     & .2025752E+00,.2115209E+00,.2212761E+00,.2319679E+00,.2443755E+00,&
     & .1870918E+00,.1958476E+00,.2056213E+00,.2168964E+00,.2297077E+00,&
     & .1719062E+00,.1808017E+00,.1908654E+00,.2024728E+00,.2155574E+00,&
     & .1580850E+00,.1674409E+00,.1780281E+00,.1900585E+00,.2031710E+00,&
     & .1462403E+00,.1558822E+00,.1666091E+00,.1789235E+00,.1922377E+00,&
     & .1362161E+00,.1459406E+00,.1567969E+00,.1692597E+00,.1829005E+00,&
     & .1274087E+00,.1370931E+00,.1484132E+00,.1609035E+00,.1746367E+00,&
     & .1194990E+00,.1297509E+00,.1410742E+00,.1539492E+00,.1679681E+00,&
     & .1123619E+00,.1230023E+00,.1347556E+00,.1477261E+00,.1617005E+00,&
     & .1061193E+00,.1170526E+00,.1291360E+00,.1421656E+00,.1564595E+00,&
     & .1006760E+00,.1119479E+00,.1241710E+00,.1376384E+00,.1522244E+00,&
     & .9607904E-01,.1075948E+00,.1202361E+00,.1342448E+00,.1520624E+00,&
     & .9236167E-01,.1042029E+00,.1173986E+00,.1321571E+00,.1502113E+00,&
     & .8989190E-01,.1021932E+00,.1159955E+00,.1310628E+00,.1496169E+00,&
     & .8834869E-01,.1013779E+00,.1156618E+00,.1310736E+00,.1499564E+00,&
     & .8767587E-01,.1011140E+00,.1157497E+00,.1341003E+00,.1509449E+00,&
     & .8642676E-01,.1002241E+00,.1151696E+00,.1339998E+00,.1509668E+00,&
     & .8444357E-01,.9839768E-01,.1137899E+00,.1328639E+00,.1500538E+00/

      data   absb(121:235, 4)   /                                       &
     & .8008552E-01,.9397404E-01,.1092312E+00,.1284086E+00,.1456923E+00,&
     & .7606488E-01,.8974941E-01,.1049552E+00,.1241503E+00,.1414966E+00,&
     & .7241439E-01,.8585292E-01,.1009126E+00,.1172255E+00,.1376072E+00,&
     & .6737037E-01,.8040867E-01,.9512233E-01,.1111766E+00,.1313617E+00,&
     & .6249839E-01,.7518420E-01,.8941538E-01,.1051429E+00,.1250058E+00,&
     & .5796755E-01,.7023093E-01,.8399825E-01,.9941523E-01,.1191081E+00,&
     & .5290880E-01,.6463286E-01,.7799108E-01,.9283777E-01,.1092238E+00,&
     & .4784261E-01,.5904001E-01,.7178305E-01,.8613347E-01,.1019882E+00,&
     & .4314406E-01,.5383664E-01,.6605552E-01,.7975409E-01,.9510036E-01,&
     & .3853322E-01,.4870478E-01,.6029343E-01,.7337748E-01,.8815869E-01,&
     & .3389054E-01,.4343173E-01,.5438033E-01,.6682346E-01,.8091510E-01,&
     & .2963117E-01,.3852479E-01,.4883852E-01,.6062832E-01,.7400013E-01,&
     & .2572970E-01,.3391246E-01,.4359826E-01,.5471276E-01,.6736039E-01,&
     & .2233416E-01,.2991585E-01,.3901273E-01,.4954145E-01,.6156569E-01,&
     & .1939210E-01,.2638719E-01,.3485100E-01,.4478336E-01,.5620570E-01,&
     & .1668291E-01,.2307789E-01,.3098475E-01,.4031918E-01,.5116350E-01,&
     & .1420262E-01,.2005395E-01,.2735039E-01,.3608786E-01,.4625548E-01,&
     & .1217748E-01,.1754282E-01,.2430328E-01,.3256355E-01,.4217429E-01,&
     & .1044921E-01,.1536327E-01,.2163899E-01,.2944083E-01,.3855156E-01,&
     & .8884509E-02,.1335574E-01,.1918440E-01,.2646426E-01,.3516363E-01,&
     & .7483675E-02,.1152870E-01,.1691180E-01,.2368487E-01,.3190062E-01,&
     & .6305579E-02,.9961440E-02,.1489973E-01,.2122728E-01,.2897708E-01,&
     & .5828592E-02,.9332797E-02,.1413857E-01,.2030539E-01,.2791717E-01/

      data   absb(  1:120, 5)   /                                       &
     & .9803355E+01,.1002921E+02,.1022928E+02,.1041986E+02,.1060893E+02,&
     & .8901187E+01,.9109354E+01,.9307795E+01,.9507481E+01,.9704484E+01,&
     & .8037550E+01,.8238379E+01,.8440990E+01,.8639971E+01,.8841430E+01,&
     & .7256235E+01,.7456728E+01,.7656282E+01,.7858569E+01,.8063361E+01,&
     & .6549121E+01,.6747154E+01,.6948541E+01,.7154546E+01,.7363018E+01,&
     & .5920127E+01,.6112982E+01,.6313644E+01,.6520192E+01,.6735708E+01,&
     & .5391811E+01,.5564840E+01,.5751524E+01,.5963164E+01,.6187711E+01,&
     & .4979063E+01,.5141211E+01,.5319204E+01,.5518536E+01,.5724852E+01,&
     & .4685729E+01,.4829264E+01,.4999371E+01,.5176536E+01,.5377588E+01,&
     & .4509733E+01,.4627971E+01,.4769025E+01,.4939138E+01,.5138382E+01,&
     & .4441848E+01,.4547145E+01,.4654076E+01,.4777266E+01,.4966192E+01,&
     & .4392672E+01,.4508636E+01,.4631472E+01,.4750610E+01,.4893241E+01,&
     & .4379451E+01,.4509691E+01,.4639368E+01,.4771997E+01,.4917485E+01,&
     & .4383266E+01,.4532070E+01,.4679856E+01,.4811910E+01,.4982307E+01,&
     & .4373711E+01,.4538212E+01,.4701877E+01,.4859289E+01,.5032829E+01,&
     & .4371956E+01,.4552831E+01,.4722242E+01,.4907609E+01,.5087424E+01,&
     & .4332160E+01,.4518631E+01,.4697783E+01,.4891344E+01,.5092684E+01,&
     & .4305857E+01,.4488111E+01,.4683919E+01,.4875401E+01,.5098906E+01,&
     & .4247319E+01,.4442550E+01,.4629624E+01,.4824728E+01,.5055952E+01,&
     & .4214744E+01,.4395695E+01,.4595960E+01,.4805492E+01,.5017792E+01,&
     & .4194067E+01,.4374577E+01,.4586487E+01,.4793725E+01,.5028709E+01,&
     & .4199133E+01,.4388763E+01,.4601898E+01,.4846199E+01,.5088486E+01,&
     & .4205331E+01,.4415572E+01,.4636671E+01,.4889061E+01,.5185128E+01,&
     & .4239611E+01,.4448023E+01,.4670143E+01,.4942226E+01,.5271353E+01/

      data   absb(121:235, 5)   /                                       &
     & .4176844E+01,.4378265E+01,.4607587E+01,.4898813E+01,.5254033E+01,&
     & .4099489E+01,.4293386E+01,.4533269E+01,.4877379E+01,.5230256E+01,&
     & .4007521E+01,.4213195E+01,.4480775E+01,.4835666E+01,.5209135E+01,&
     & .3883977E+01,.4090925E+01,.4376746E+01,.4733176E+01,.5114008E+01,&
     & .3759955E+01,.3970704E+01,.4272560E+01,.4618469E+01,.4997992E+01,&
     & .3631179E+01,.3846064E+01,.4161525E+01,.4505341E+01,.4888289E+01,&
     & .3481107E+01,.3699160E+01,.4022359E+01,.4359341E+01,.4726661E+01,&
     & .3305633E+01,.3543449E+01,.3873804E+01,.4204009E+01,.4566058E+01,&
     & .3130039E+01,.3398800E+01,.3728888E+01,.4056038E+01,.4401957E+01,&
     & .2968975E+01,.3253192E+01,.3573124E+01,.3904086E+01,.4239970E+01,&
     & .2811010E+01,.3105040E+01,.3413054E+01,.3741891E+01,.4067138E+01,&
     & .2663058E+01,.2951303E+01,.3256042E+01,.3581236E+01,.3900989E+01,&
     & .2515451E+01,.2806289E+01,.3104377E+01,.3428028E+01,.3740708E+01,&
     & .2378861E+01,.2672260E+01,.2967412E+01,.3283792E+01,.3592902E+01,&
     & .2250954E+01,.2549314E+01,.2842896E+01,.3146414E+01,.3454348E+01,&
     & .2126384E+01,.2425968E+01,.2718676E+01,.3012722E+01,.3313565E+01,&
     & .2000566E+01,.2300064E+01,.2594150E+01,.2881717E+01,.3180487E+01,&
     & .1893973E+01,.2183036E+01,.2478401E+01,.2762266E+01,.3053944E+01,&
     & .1797191E+01,.2073967E+01,.2365713E+01,.2650269E+01,.2928594E+01,&
     & .1699063E+01,.1967053E+01,.2254958E+01,.2539431E+01,.2812870E+01,&
     & .1602483E+01,.1870209E+01,.2142381E+01,.2424027E+01,.2698355E+01,&
     & .1508607E+01,.1774262E+01,.2038629E+01,.2313372E+01,.2585830E+01,&
     & .1519868E+01,.1744553E+01,.1991354E+01,.2260467E+01,.2531035E+01/

      data   absb(  1:120, 6)   /                                       &
     & .2781168E+03,.2745845E+03,.2712239E+03,.2678704E+03,.2644459E+03,&
     & .2904946E+03,.2871655E+03,.2838470E+03,.2803941E+03,.2768948E+03,&
     & .3022413E+03,.2989983E+03,.2955817E+03,.2921152E+03,.2885458E+03,&
     & .3130565E+03,.3098089E+03,.3064324E+03,.3028900E+03,.2992469E+03,&
     & .3228674E+03,.3196371E+03,.3162446E+03,.3126653E+03,.3089740E+03,&
     & .3316375E+03,.3284418E+03,.3250265E+03,.3214442E+03,.3176680E+03,&
     & .3390155E+03,.3360705E+03,.3328242E+03,.3291721E+03,.3252655E+03,&
     & .3449713E+03,.3421655E+03,.3390332E+03,.3355221E+03,.3318374E+03,&
     & .3493969E+03,.3468038E+03,.3437556E+03,.3405122E+03,.3368916E+03,&
     & .3522054E+03,.3499115E+03,.3472051E+03,.3440347E+03,.3404337E+03,&
     & .3535539E+03,.3513906E+03,.3491088E+03,.3465269E+03,.3430700E+03,&
     & .3545730E+03,.3522557E+03,.3497503E+03,.3472267E+03,.3443498E+03,&
     & .3550447E+03,.3525630E+03,.3499373E+03,.3472388E+03,.3443261E+03,&
     & .3552526E+03,.3524843E+03,.3496510E+03,.3469402E+03,.3437094E+03,&
     & .3555960E+03,.3526031E+03,.3495395E+03,.3465118E+03,.3432573E+03,&
     & .3557797E+03,.3525640E+03,.3494123E+03,.3460217E+03,.3426940E+03,&
     & .3564043E+03,.3530960E+03,.3498075E+03,.3463239E+03,.3427123E+03,&
     & .3568211E+03,.3535487E+03,.3500420E+03,.3465557E+03,.3428692E+03,&
     & .3576037E+03,.3541394E+03,.3507368E+03,.3471845E+03,.3434088E+03,&
     & .3580188E+03,.3547035E+03,.3511129E+03,.3473748E+03,.3438292E+03,&
     & .3581982E+03,.3548803E+03,.3511249E+03,.3474255E+03,.3435849E+03,&
     & .3580668E+03,.3546083E+03,.3508286E+03,.3468622E+03,.3427171E+03,&
     & .3579811E+03,.3542519E+03,.3503648E+03,.3462908E+03,.3414558E+03,&
     & .3576096E+03,.3539086E+03,.3499965E+03,.3456525E+03,.3403969E+03/

      data   absb(121:235, 6)   /                                       &
     & .3586604E+03,.3550335E+03,.3510399E+03,.3464481E+03,.3408584E+03,&
     & .3598800E+03,.3563594E+03,.3522228E+03,.3469653E+03,.3413783E+03,&
     & .3612682E+03,.3576057E+03,.3531275E+03,.3475159E+03,.3418607E+03,&
     & .3631576E+03,.3595023E+03,.3547940E+03,.3491532E+03,.3434219E+03,&
     & .3650737E+03,.3613720E+03,.3564614E+03,.3509635E+03,.3452500E+03,&
     & .3670026E+03,.3632776E+03,.3582252E+03,.3527572E+03,.3469855E+03,&
     & .3692557E+03,.3655325E+03,.3603844E+03,.3550208E+03,.3492473E+03,&
     & .3718397E+03,.3678903E+03,.3626881E+03,.3574203E+03,.3517227E+03,&
     & .3744095E+03,.3701124E+03,.3649258E+03,.3597189E+03,.3542337E+03,&
     & .3767816E+03,.3723343E+03,.3673166E+03,.3620799E+03,.3567366E+03,&
     & .3791321E+03,.3746009E+03,.3697970E+03,.3646097E+03,.3594172E+03,&
     & .3813238E+03,.3769336E+03,.3722063E+03,.3670953E+03,.3619918E+03,&
     & .3834849E+03,.3791057E+03,.3745237E+03,.3694845E+03,.3645086E+03,&
     & .3854281E+03,.3811015E+03,.3766032E+03,.3716952E+03,.3667988E+03,&
     & .3872332E+03,.3829179E+03,.3785031E+03,.3738180E+03,.3689635E+03,&
     & .3889780E+03,.3847170E+03,.3803790E+03,.3758553E+03,.3711179E+03,&
     & .3907123E+03,.3865429E+03,.3822340E+03,.3778545E+03,.3731887E+03,&
     & .3921534E+03,.3881790E+03,.3839339E+03,.3796445E+03,.3751081E+03,&
     & .3934377E+03,.3896848E+03,.3855532E+03,.3813151E+03,.3769936E+03,&
     & .3947370E+03,.3911513E+03,.3871310E+03,.3829493E+03,.3787443E+03,&
     & .3959555E+03,.3924751E+03,.3887103E+03,.3846245E+03,.3804545E+03,&
     & .3971711E+03,.3937408E+03,.3901626E+03,.3862220E+03,.3821316E+03,&
     & .3970289E+03,.3941655E+03,.3908061E+03,.3869572E+03,.3829149E+03/

!  ---
      data   forref(1:3,1: 6)  / .6585271E-05,.9433300E-05,.7977931E-04,&
     & .8154837E-04,.1197092E-03,.1424472E-03,.2535780E-03,.2329964E-03,&
     & .1885349E-03,.3380323E-03,.3052715E-03,.2150602E-03,.3685495E-03,&
     & .3314392E-03,.2543625E-03,.3806938E-03,.3823779E-03,.2995716E-03/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 6).

      data   selfref(1:10,1: 6)  /                                      &
     & .1352797E-02,.1118049E-02,.9240558E-03,.7637369E-03,.6312454E-03,&
     & .5217486E-03,.4312539E-03,.3564616E-03,.2946471E-03,.2435560E-03,&
     & .3427292E-02,.2856171E-02,.2390076E-02,.2008096E-02,.1693734E-02,&
     & .1433911E-02,.1218251E-02,.1038495E-02,.8880442E-03,.7616162E-03,&
     & .4545629E-02,.4185794E-02,.3854444E-02,.3549326E-02,.3268367E-02,&
     & .3009646E-02,.2771406E-02,.2552033E-02,.2350020E-02,.2163998E-02,&
     & .5477838E-02,.5082542E-02,.4715844E-02,.4375667E-02,.4060089E-02,&
     & .3767331E-02,.3495734E-02,.3243771E-02,.3010013E-02,.2793137E-02,&
     & .5682360E-02,.5292443E-02,.4929650E-02,.4592058E-02,.4277891E-02,&
     & .3985497E-02,.3713349E-02,.3460014E-02,.3224177E-02,.3004612E-02,&
     & .7004786E-02,.6372419E-02,.5797303E-02,.5274236E-02,.4798502E-02,&
     & .4365805E-02,.3972237E-02,.3614256E-02,.3288634E-02,.2992436E-02/

!........................................!
      end module module_radsw_kgb16      !
!========================================!


!========================================!
      module module_radsw_kgb17          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 17:  3250-4000 cm-1 (low - h2o,co2; high - h2o, co2)       !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG17

!
      implicit none
!
      private
!
      integer,  public :: MSA17, MSB17, MSF17, MFR17
      parameter (MSA17=585, MSB17=1175, MSF17=10, MFR17=4)

      real (kind=kind_phys), public ::           selfref(MSF17,NG17),   &
     &       absa(MSA17,NG17), absb(MSB17,NG17), forref(MFR17,NG17)

!  ---  rayleigh extinction coefficient at v = 3625 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 6.86e-10

!     the array absa(585,NG17) (ka((9,5,13,NG17)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 12, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .1113400E-05,.3209200E-03,.5566300E-03,.7822700E-03,.9942100E-03,&
     & .1204500E-02,.1411600E-02,.1606900E-02,.1933000E-02,.1054400E-05,&
     & .3543400E-03,.6148000E-03,.8642400E-03,.1105500E-02,.1343100E-02,&
     & .1575400E-02,.1795100E-02,.2128000E-02,.9834500E-06,.3898900E-03,&
     & .6774700E-03,.9528900E-03,.1219600E-02,.1486400E-02,.1744900E-02,&
     & .1989700E-02,.2356400E-02,.9501700E-06,.4258200E-03,.7441900E-03,&
     & .1046300E-02,.1342400E-02,.1635300E-02,.1921500E-02,.2193900E-02,&
     & .2599300E-02,.9187000E-06,.4637100E-03,.8139400E-03,.1146400E-02,&
     & .1471500E-02,.1793700E-02,.2107200E-02,.2407700E-02,.2854000E-02,&
     & .9418400E-06,.2653200E-03,.4615700E-03,.6377600E-03,.8108700E-03,&
     & .9772800E-03,.1141900E-02,.1299100E-02,.1512100E-02,.8906000E-06,&
     & .2941700E-03,.5096700E-03,.7075900E-03,.9020300E-03,.1092000E-02,&
     & .1276400E-02,.1454400E-02,.1679500E-02,.8272000E-06,.3236100E-03,&
     & .5612400E-03,.7829900E-03,.9971800E-03,.1209400E-02,.1417200E-02,&
     & .1616500E-02,.1863700E-02,.7884200E-06,.3549500E-03,.6168400E-03,&
     & .8617600E-03,.1099500E-02,.1334500E-02,.1565200E-02,.1785600E-02,&
     & .2062100E-02,.7661000E-06,.3872800E-03,.6753800E-03,.9461300E-03,&
     & .1208200E-02,.1467500E-02,.1720600E-02,.1964000E-02,.2272000E-02,&
     & .7966300E-06,.2136900E-03,.3708800E-03,.5122200E-03,.6454900E-03,&
     & .7732200E-03,.8993900E-03,.1020500E-02,.1156800E-02,.7595500E-06,&
     & .2378000E-03,.4115300E-03,.5690100E-03,.7200900E-03,.8673500E-03,&
     & .1009200E-02,.1147600E-02,.1291300E-02,.7169200E-06,.2627600E-03,&
     & .4548800E-03,.6292200E-03,.7983600E-03,.9625900E-03,.1123600E-02,&
     & .1279600E-02,.1439800E-02,.6634800E-06,.2889600E-03,.5022100E-03,&
     & .6941100E-03,.8822600E-03,.1065800E-02,.1245600E-02,.1419100E-02,&
     & .1599100E-02,.6401300E-06,.3164300E-03,.5516600E-03,.7643400E-03,&
     & .9721600E-03,.1174500E-02,.1373100E-02,.1565700E-02,.1767100E-02,&
     & .6621700E-06,.1696600E-03,.2935800E-03,.4068800E-03,.5111400E-03,&
     & .6073900E-03,.7019100E-03,.7944600E-03,.8883200E-03,.6411100E-06,&
     & .1890200E-03,.3274200E-03,.4536400E-03,.5708600E-03,.6840100E-03,&
     & .7911600E-03,.8950700E-03,.9965500E-03,.6072900E-06,.2100700E-03,&
     & .3635400E-03,.5035400E-03,.6345900E-03,.7611300E-03,.8847300E-03,&
     & .1003500E-02,.1116300E-02,.5725000E-06,.2320300E-03,.4024500E-03,&
     & .5575800E-03,.7029500E-03,.8451500E-03,.9837900E-03,.1116500E-02,&
     & .1244400E-02,.5352400E-06,.2552900E-03,.4439900E-03,.6156400E-03,&
     & .7762600E-03,.9340500E-03,.1089200E-02,.1236900E-02,.1379500E-02/

      data   absa(181:315, 1)   /                                       &
     & .5511200E-06,.1342200E-03,.2316600E-03,.3211900E-03,.4026500E-03,&
     & .4781200E-03,.5479700E-03,.6187300E-03,.6891200E-03,.5366900E-06,&
     & .1499000E-03,.2594900E-03,.3588600E-03,.4523900E-03,.5393600E-03,&
     & .6189400E-03,.6971800E-03,.7773200E-03,.5149500E-06,.1670600E-03,&
     & .2896000E-03,.4002000E-03,.5046100E-03,.6024900E-03,.6967700E-03,&
     & .7846400E-03,.8733400E-03,.4863300E-06,.1854300E-03,.3215900E-03,&
     & .4449900E-03,.5609500E-03,.6705500E-03,.7757500E-03,.8771700E-03,&
     & .9769400E-03,.4516200E-06,.2047200E-03,.3559100E-03,.4935000E-03,&
     & .6216300E-03,.7425300E-03,.8624700E-03,.9751800E-03,.1086400E-02,&
     & .4505500E-06,.1055600E-03,.1817600E-03,.2513700E-03,.3143800E-03,&
     & .3736300E-03,.4275900E-03,.4788300E-03,.5343700E-03,.4450200E-06,&
     & .1181400E-03,.2038300E-03,.2816200E-03,.3554700E-03,.4226100E-03,&
     & .4834000E-03,.5406600E-03,.6035200E-03,.4306600E-06,.1321700E-03,&
     & .2283600E-03,.3155200E-03,.3976200E-03,.4749300E-03,.5454300E-03,&
     & .6106600E-03,.6817600E-03,.4111600E-06,.1473300E-03,.2547200E-03,&
     & .3520400E-03,.4433400E-03,.5299700E-03,.6096600E-03,.6862200E-03,&
     & .7660700E-03,.3874800E-06,.1634400E-03,.2828100E-03,.3917300E-03,&
     & .4930500E-03,.5888000E-03,.6795400E-03,.7648500E-03,.8540100E-03,&
     & .3658200E-06,.8295800E-04,.1421000E-03,.1957400E-03,.2444000E-03,&
     & .2903900E-03,.3328100E-03,.3717700E-03,.4217800E-03,.3670700E-06,&
     & .9290900E-04,.1597700E-03,.2203400E-03,.2774200E-03,.3295500E-03,&
     & .3772700E-03,.4195600E-03,.4785900E-03,.3578700E-06,.1043200E-03,&
     & .1793000E-03,.2476900E-03,.3115500E-03,.3724200E-03,.4269900E-03,&
     & .4747400E-03,.5408300E-03,.3448200E-06,.1168500E-03,.2005500E-03,&
     & .2772500E-03,.3486600E-03,.4167700E-03,.4790300E-03,.5351800E-03,&
     & .6081200E-03,.3259800E-06,.1300100E-03,.2234100E-03,.3090000E-03,&
     & .3891900E-03,.4646500E-03,.5346700E-03,.5992600E-03,.6773300E-03/

      data   absa(316:450, 1)   /                                       &
     & .2983400E-06,.6500900E-04,.1118300E-03,.1520000E-03,.1896900E-03,&
     & .2251700E-03,.2583200E-03,.2873000E-03,.3395500E-03,.2974500E-06,&
     & .7287200E-04,.1248200E-03,.1722100E-03,.2158300E-03,.2559600E-03,&
     & .2932800E-03,.3252000E-03,.3862000E-03,.2942100E-06,.8203600E-04,&
     & .1401000E-03,.1934400E-03,.2436200E-03,.2907100E-03,.3330800E-03,&
     & .3686700E-03,.4390100E-03,.2859600E-06,.9225600E-04,.1573800E-03,&
     & .2175200E-03,.2732500E-03,.3264300E-03,.3757800E-03,.4171000E-03,&
     & .4935800E-03,.2729600E-06,.1032000E-03,.1759800E-03,.2433900E-03,&
     & .3059100E-03,.3650500E-03,.4206100E-03,.4687200E-03,.5499300E-03,&
     & .2425900E-06,.5062600E-04,.8694500E-04,.1177800E-03,.1469200E-03,&
     & .1743100E-03,.2005700E-03,.2213100E-03,.2991900E-03,.2405800E-06,&
     & .5696600E-04,.9746700E-04,.1340500E-03,.1675000E-03,.1981400E-03,&
     & .2273900E-03,.2528100E-03,.3402200E-03,.2409100E-06,.6432400E-04,&
     & .1097100E-03,.1506100E-03,.1899600E-03,.2258000E-03,.2588500E-03,&
     & .2869400E-03,.3804500E-03,.2355000E-06,.7245100E-04,.1234500E-03,&
     & .1699100E-03,.2136200E-03,.2548400E-03,.2934100E-03,.3252300E-03,&
     & .4283600E-03,.2269600E-06,.8135100E-04,.1383900E-03,.1909000E-03,&
     & .2398600E-03,.2860800E-03,.3291000E-03,.3663900E-03,.4792500E-03,&
     & .1967900E-06,.3976000E-04,.6805200E-04,.9229500E-04,.1149800E-03,&
     & .1365600E-03,.1560400E-03,.1717100E-03,.2967900E-03,.1956700E-06,&
     & .4484700E-04,.7682900E-04,.1051800E-03,.1310200E-03,.1550900E-03,&
     & .1776400E-03,.1984300E-03,.3286300E-03,.1957400E-06,.5056500E-04,&
     & .8646600E-04,.1187800E-03,.1492000E-03,.1769800E-03,.2025800E-03,&
     & .2252600E-03,.3695900E-03,.1926200E-06,.5712400E-04,.9754600E-04,&
     & .1339800E-03,.1680300E-03,.2004600E-03,.2302200E-03,.2557800E-03,&
     & .4154300E-03,.1874700E-06,.6426400E-04,.1097100E-03,.1508600E-03,&
     & .1892300E-03,.2256500E-03,.2593400E-03,.2892100E-03,.4646300E-03/

      data   absa(451:585, 1)   /                                       &
     & .1601500E-06,.3302500E-04,.5656200E-04,.7662500E-04,.9564500E-04,&
     & .1135800E-03,.1295500E-03,.1426900E-03,.2754500E-03,.1591800E-06,&
     & .3714800E-04,.6377500E-04,.8760000E-04,.1089700E-03,.1289300E-03,&
     & .1475600E-03,.1646500E-03,.3079900E-03,.1589700E-06,.4199300E-04,&
     & .7204000E-04,.9892800E-04,.1243300E-03,.1473300E-03,.1682400E-03,&
     & .1869400E-03,.3434000E-03,.1557800E-06,.4744100E-04,.8142600E-04,&
     & .1116500E-03,.1398500E-03,.1667300E-03,.1915600E-03,.2126800E-03,&
     & .3817600E-03,.1521000E-06,.5339800E-04,.9165700E-04,.1259200E-03,&
     & .1576900E-03,.1878100E-03,.2156900E-03,.2406900E-03,.4222800E-03,&
     & .1302100E-06,.2727200E-04,.4683200E-04,.6351100E-04,.7920500E-04,&
     & .9394900E-04,.1071900E-03,.1182700E-03,.2394400E-03,.1293000E-06,&
     & .3068600E-04,.5292500E-04,.7278200E-04,.9052700E-04,.1070800E-03,&
     & .1223600E-03,.1362000E-03,.2712100E-03,.1290000E-06,.3472300E-04,&
     & .5986700E-04,.8220500E-04,.1032600E-03,.1223800E-03,.1395800E-03,&
     & .1549800E-03,.2993000E-03,.1264800E-06,.3925200E-04,.6761800E-04,&
     & .9294900E-04,.1163600E-03,.1383500E-03,.1592400E-03,.1765300E-03,&
     & .3305000E-03,.1229900E-06,.4421300E-04,.7624400E-04,.1048800E-03,&
     & .1313500E-03,.1560100E-03,.1790200E-03,.1997600E-03,.3657900E-03,&
     & .1055400E-06,.2238800E-04,.3859700E-04,.5237800E-04,.6540400E-04,&
     & .7756700E-04,.8854300E-04,.9780100E-04,.1959900E-03,.1048000E-06,&
     & .2518100E-04,.4367000E-04,.6020500E-04,.7488300E-04,.8860500E-04,&
     & .1011700E-03,.1124200E-03,.2214600E-03,.1042900E-06,.2853000E-04,&
     & .4942200E-04,.6803900E-04,.8543500E-04,.1013600E-03,.1155700E-03,&
     & .1282800E-03,.2444700E-03,.1023200E-06,.3230100E-04,.5579700E-04,&
     & .7694800E-04,.9644700E-04,.1146200E-03,.1316600E-03,.1461600E-03,&
     & .2699900E-03,.9901400E-07,.3639400E-04,.6299400E-04,.8694600E-04,&
     & .1090000E-03,.1293700E-03,.1483300E-03,.1654000E-03,.2991600E-03/

      data   absa(  1:180, 2)   /                                       &
     & .2348800E-04,.1668400E-02,.2957500E-02,.4008300E-02,.4995300E-02,&
     & .5902100E-02,.6744600E-02,.7600800E-02,.8703200E-02,.2515600E-04,&
     & .1846200E-02,.3271100E-02,.4500800E-02,.5618200E-02,.6628700E-02,&
     & .7590900E-02,.8555800E-02,.9754300E-02,.2708400E-04,.2032500E-02,&
     & .3603400E-02,.4999200E-02,.6279600E-02,.7413400E-02,.8499400E-02,&
     & .9569400E-02,.1084700E-01,.2814700E-04,.2234700E-02,.3962800E-02,&
     & .5500500E-02,.6945300E-02,.8242900E-02,.9469900E-02,.1066000E-01,&
     & .1199100E-01,.2871900E-04,.2447500E-02,.4336900E-02,.6036000E-02,&
     & .7618300E-02,.9087800E-02,.1049300E-01,.1180500E-01,.1320400E-01,&
     & .2094700E-04,.1401500E-02,.2472800E-02,.3367600E-02,.4180100E-02,&
     & .4924400E-02,.5578700E-02,.6220600E-02,.6974100E-02,.2235200E-04,&
     & .1553500E-02,.2739600E-02,.3787000E-02,.4712100E-02,.5557100E-02,&
     & .6289200E-02,.7013200E-02,.7809200E-02,.2388200E-04,.1721300E-02,&
     & .3034500E-02,.4202000E-02,.5278200E-02,.6228500E-02,.7055600E-02,&
     & .7881900E-02,.8733000E-02,.2465000E-04,.1895400E-02,.3338400E-02,&
     & .4633600E-02,.5833300E-02,.6931100E-02,.7875100E-02,.8814900E-02,&
     & .9691100E-02,.2513600E-04,.2080600E-02,.3664100E-02,.5096600E-02,&
     & .6419300E-02,.7628100E-02,.8740300E-02,.9794300E-02,.1070400E-01,&
     & .1772000E-04,.1141400E-02,.2004500E-02,.2728800E-02,.3389100E-02,&
     & .3975500E-02,.4496700E-02,.4938900E-02,.5385800E-02,.1908700E-04,&
     & .1274500E-02,.2233600E-02,.3084100E-02,.3833300E-02,.4498300E-02,&
     & .5089500E-02,.5593000E-02,.6066600E-02,.2012500E-04,.1417800E-02,&
     & .2479500E-02,.3427800E-02,.4310600E-02,.5060700E-02,.5728000E-02,&
     & .6309200E-02,.6828500E-02,.2089500E-04,.1566600E-02,.2734800E-02,&
     & .3793400E-02,.4776900E-02,.5662100E-02,.6404600E-02,.7086400E-02,&
     & .7602600E-02,.2150800E-04,.1725100E-02,.3015300E-02,.4180900E-02,&
     & .5268900E-02,.6246200E-02,.7112500E-02,.7910400E-02,.8438900E-02,&
     & .1466100E-04,.9226100E-03,.1608800E-02,.2178800E-02,.2695600E-02,&
     & .3161200E-02,.3562600E-02,.3887800E-02,.4152500E-02,.1560700E-04,&
     & .1032900E-02,.1797300E-02,.2473500E-02,.3069600E-02,.3595300E-02,&
     & .4051100E-02,.4416200E-02,.4699000E-02,.1655900E-04,.1155000E-02,&
     & .2001900E-02,.2757700E-02,.3462500E-02,.4066200E-02,.4583100E-02,&
     & .4999500E-02,.5307300E-02,.1726300E-04,.1279400E-02,.2218400E-02,&
     & .3061400E-02,.3851000E-02,.4566400E-02,.5153200E-02,.5629000E-02,&
     & .5943600E-02,.1789500E-04,.1413000E-02,.2452600E-02,.3380500E-02,&
     & .4262200E-02,.5055600E-02,.5748900E-02,.6309400E-02,.6625600E-02/

      data   absa(181:315, 2)   /                                       &
     & .1188500E-04,.7488800E-03,.1282600E-02,.1725800E-02,.2134600E-02,&
     & .2497200E-02,.2804600E-02,.3060600E-02,.3232300E-02,.1245200E-04,&
     & .8408400E-03,.1438000E-02,.1969800E-02,.2438600E-02,.2853600E-02,&
     & .3207500E-02,.3493700E-02,.3656200E-02,.1343900E-04,.9427200E-03,&
     & .1606500E-02,.2207900E-02,.2766000E-02,.3238900E-02,.3644000E-02,&
     & .3967300E-02,.4139300E-02,.1405300E-04,.1047300E-02,.1789900E-02,&
     & .2460300E-02,.3086100E-02,.3652100E-02,.4115600E-02,.4482500E-02,&
     & .4652900E-02,.1458600E-04,.1158500E-02,.1988600E-02,.2725700E-02,&
     & .3427200E-02,.4059900E-02,.4611500E-02,.5028600E-02,.5208500E-02,&
     & .9487200E-05,.5979300E-03,.1021000E-02,.1357200E-02,.1671600E-02,&
     & .1951200E-02,.2189000E-02,.2382700E-02,.2534700E-02,.1006000E-04,&
     & .6740000E-03,.1143200E-02,.1556400E-02,.1918800E-02,.2238900E-02,&
     & .2516100E-02,.2730800E-02,.2877000E-02,.1074300E-04,.7594800E-03,&
     & .1280600E-02,.1755100E-02,.2186200E-02,.2554500E-02,.2872900E-02,&
     & .3114900E-02,.3271600E-02,.1132200E-04,.8513400E-03,.1432000E-02,&
     & .1963000E-02,.2451400E-02,.2893000E-02,.3257800E-02,.3537500E-02,&
     & .3691200E-02,.1175500E-04,.9461100E-03,.1598600E-02,.2180400E-02,&
     & .2729100E-02,.3230600E-02,.3669500E-02,.3991300E-02,.4143900E-02,&
     & .7593100E-05,.4713000E-03,.8121100E-03,.1065800E-02,.1300400E-02,&
     & .1517500E-02,.1700200E-02,.1847300E-02,.1986100E-02,.8072400E-05,&
     & .5335800E-03,.9116300E-03,.1226100E-02,.1501400E-02,.1748300E-02,&
     & .1961200E-02,.2120100E-02,.2259100E-02,.8440400E-05,.6051700E-03,&
     & .1023600E-02,.1388700E-02,.1720400E-02,.2003100E-02,.2249700E-02,&
     & .2432100E-02,.2582300E-02,.9051100E-05,.6803700E-03,.1147500E-02,&
     & .1559600E-02,.1940200E-02,.2278900E-02,.2563300E-02,.2775300E-02,&
     & .2924700E-02,.9440500E-05,.7610300E-03,.1283600E-02,.1742200E-02,&
     & .2169800E-02,.2558500E-02,.2898900E-02,.3144400E-02,.3301600E-02/

      data   absa(316:450, 2)   /                                       &
     & .5808000E-05,.3678200E-03,.6351600E-03,.8379900E-03,.1011900E-02,&
     & .1175400E-02,.1316100E-02,.1438200E-02,.1583700E-02,.6315600E-05,&
     & .4190900E-03,.7197400E-03,.9695200E-03,.1171100E-02,.1361700E-02,&
     & .1523400E-02,.1644100E-02,.1811800E-02,.6677600E-05,.4782900E-03,&
     & .8115000E-03,.1105600E-02,.1349300E-02,.1564600E-02,.1752400E-02,&
     & .1893700E-02,.2069800E-02,.7058200E-05,.5401700E-03,.9133000E-03,&
     & .1243400E-02,.1541100E-02,.1788400E-02,.2005700E-02,.2170400E-02,&
     & .2354800E-02,.7490200E-05,.6071700E-03,.1026700E-02,.1390800E-02,&
     & .1723100E-02,.2028500E-02,.2281000E-02,.2471000E-02,.2663100E-02,&
     & .4506100E-05,.2862600E-03,.4944800E-03,.6531700E-03,.7908100E-03,&
     & .9106600E-03,.1018200E-02,.1110700E-02,.1459200E-02,.4939200E-05,&
     & .3273400E-03,.5655800E-03,.7607700E-03,.9184600E-03,.1057800E-02,&
     & .1177600E-02,.1272000E-02,.1677800E-02,.5258000E-05,.3752200E-03,&
     & .6384500E-03,.8780500E-03,.1061900E-02,.1219400E-02,.1360600E-02,&
     & .1468000E-02,.1925900E-02,.5499100E-05,.4265900E-03,.7210300E-03,&
     & .9875600E-03,.1216300E-02,.1400700E-02,.1564700E-02,.1690000E-02,&
     & .2177200E-02,.5876900E-05,.4819900E-03,.8146600E-03,.1107200E-02,&
     & .1368400E-02,.1598400E-02,.1788700E-02,.1933300E-02,.2433100E-02,&
     & .3415400E-05,.2253400E-03,.3878800E-03,.5111300E-03,.6219400E-03,&
     & .7143200E-03,.7974300E-03,.8628900E-03,.1475800E-02,.3808500E-05,&
     & .2583200E-03,.4474800E-03,.5983500E-03,.7251400E-03,.8328400E-03,&
     & .9194100E-03,.9958300E-03,.1735800E-02,.4157800E-05,.2965500E-03,&
     & .5055300E-03,.6951100E-03,.8410700E-03,.9637400E-03,.1066600E-02,&
     & .1147800E-02,.1962500E-02,.4391700E-05,.3390800E-03,.5735100E-03,&
     & .7864000E-03,.9686200E-03,.1110900E-02,.1230900E-02,.1326600E-02,&
     & .2210300E-02,.4544600E-05,.3854800E-03,.6488300E-03,.8840100E-03,&
     & .1095200E-02,.1270600E-02,.1413600E-02,.1523900E-02,.2475400E-02/

      data   absa(451:585, 2)   /                                       &
     & .2760400E-05,.1884800E-03,.3253200E-03,.4294900E-03,.5235000E-03,&
     & .6008400E-03,.6664000E-03,.7207000E-03,.1369200E-02,.3090800E-05,&
     & .2167900E-03,.3725900E-03,.5036600E-03,.6107400E-03,.7015800E-03,&
     & .7727100E-03,.8303700E-03,.1568900E-02,.3367400E-05,.2496500E-03,&
     & .4224400E-03,.5835300E-03,.7092300E-03,.8133000E-03,.8975000E-03,&
     & .9613100E-03,.1797100E-02,.3556800E-05,.2860200E-03,.4804400E-03,&
     & .6586800E-03,.8181600E-03,.9385700E-03,.1035700E-02,.1111900E-02,&
     & .2050900E-02,.3692100E-05,.3248400E-03,.5462300E-03,.7447500E-03,&
     & .9230700E-03,.1075800E-02,.1189500E-02,.1277700E-02,.2325300E-02,&
     & .2227200E-05,.1580700E-03,.2716300E-03,.3593500E-03,.4378100E-03,&
     & .5032100E-03,.5577600E-03,.6004600E-03,.1185200E-02,.2489800E-05,&
     & .1820100E-03,.3100400E-03,.4220600E-03,.5122600E-03,.5882400E-03,&
     & .6491900E-03,.6929100E-03,.1362300E-02,.2702100E-05,.2095800E-03,&
     & .3532200E-03,.4867600E-03,.5958700E-03,.6836400E-03,.7552700E-03,&
     & .8043600E-03,.1565400E-02,.2852600E-05,.2403800E-03,.4026200E-03,&
     & .5509600E-03,.6867100E-03,.7910300E-03,.8721100E-03,.9312400E-03,&
     & .1789800E-02,.2989900E-05,.2728900E-03,.4590400E-03,.6244800E-03,&
     & .7749600E-03,.9021200E-03,.1001800E-02,.1070600E-02,.2026500E-02,&
     & .1757600E-05,.1315100E-03,.2251800E-03,.2989000E-03,.3639800E-03,&
     & .4188400E-03,.4650500E-03,.5008600E-03,.9791600E-03,.1952400E-05,&
     & .1518900E-03,.2568600E-03,.3512300E-03,.4272300E-03,.4906100E-03,&
     & .5429300E-03,.5781500E-03,.1126000E-02,.2113400E-05,.1755800E-03,&
     & .2938200E-03,.4045700E-03,.4977700E-03,.5715300E-03,.6318500E-03,&
     & .6721300E-03,.1296000E-02,.2268100E-05,.2010600E-03,.3358300E-03,&
     & .4597700E-03,.5709400E-03,.6627500E-03,.7314900E-03,.7779900E-03,&
     & .1485600E-02,.2394000E-05,.2285300E-03,.3832300E-03,.5212600E-03,&
     & .6468400E-03,.7545500E-03,.8411500E-03,.8957500E-03,.1681600E-02/

      data   absa(  1:180, 3)   /                                       &
     & .8715400E-04,.7045600E-02,.1162400E-01,.1600700E-01,.2027700E-01,&
     & .2434900E-01,.2790200E-01,.3095600E-01,.3315900E-01,.9267800E-04,&
     & .7532400E-02,.1255200E-01,.1744400E-01,.2196700E-01,.2625000E-01,&
     & .3024100E-01,.3334800E-01,.3616400E-01,.9795500E-04,.8015300E-02,&
     & .1350200E-01,.1881200E-01,.2386200E-01,.2848000E-01,.3251100E-01,&
     & .3571300E-01,.3834900E-01,.1040400E-03,.8472400E-02,.1442900E-01,&
     & .2013600E-01,.2567200E-01,.3068800E-01,.3501100E-01,.3837900E-01,&
     & .4067300E-01,.1111000E-03,.8912300E-02,.1534300E-01,.2147900E-01,&
     & .2729000E-01,.3275100E-01,.3745500E-01,.4093100E-01,.4246300E-01,&
     & .7321400E-04,.6141000E-02,.1012500E-01,.1365000E-01,.1715800E-01,&
     & .2062700E-01,.2362800E-01,.2625100E-01,.2748100E-01,.7793600E-04,&
     & .6575100E-02,.1096600E-01,.1487700E-01,.1869300E-01,.2227200E-01,&
     & .2563000E-01,.2839500E-01,.2989400E-01,.8306100E-04,.6982800E-02,&
     & .1177400E-01,.1606800E-01,.2031700E-01,.2424400E-01,.2774800E-01,&
     & .3043200E-01,.3185100E-01,.8856400E-04,.7398500E-02,.1258100E-01,&
     & .1722600E-01,.2184200E-01,.2617700E-01,.2996900E-01,.3274800E-01,&
     & .3380500E-01,.9493300E-04,.7798800E-02,.1337200E-01,.1836700E-01,&
     & .2332300E-01,.2792800E-01,.3196400E-01,.3501300E-01,.3532700E-01,&
     & .6094700E-04,.5243600E-02,.8617400E-02,.1152000E-01,.1423100E-01,&
     & .1702700E-01,.1946900E-01,.2163400E-01,.2198100E-01,.6492100E-04,&
     & .5597500E-02,.9351200E-02,.1252600E-01,.1551600E-01,.1847100E-01,&
     & .2127000E-01,.2351200E-01,.2411400E-01,.6952100E-04,.5964800E-02,&
     & .1006000E-01,.1358200E-01,.1690300E-01,.2017700E-01,.2305500E-01,&
     & .2538000E-01,.2582200E-01,.7434800E-04,.6310400E-02,.1076700E-01,&
     & .1459900E-01,.1827300E-01,.2183600E-01,.2498400E-01,.2735800E-01,&
     & .2745000E-01,.7978000E-04,.6667100E-02,.1145800E-01,.1559000E-01,&
     & .1957900E-01,.2335800E-01,.2680600E-01,.2933500E-01,.2878600E-01,&
     & .5079900E-04,.4414800E-02,.7222500E-02,.9648300E-02,.1176700E-01,&
     & .1382800E-01,.1584400E-01,.1758800E-01,.1739100E-01,.5422700E-04,&
     & .4723100E-02,.7852700E-02,.1049300E-01,.1283500E-01,.1512600E-01,&
     & .1746600E-01,.1921900E-01,.1935100E-01,.5815100E-04,.5027700E-02,&
     & .8482300E-02,.1141800E-01,.1396900E-01,.1657600E-01,.1890300E-01,&
     & .2084000E-01,.2085300E-01,.6243300E-04,.5337200E-02,.9081300E-02,&
     & .1232500E-01,.1516300E-01,.1797100E-01,.2059000E-01,.2252900E-01,&
     & .2227300E-01,.6685200E-04,.5642900E-02,.9683600E-02,.1318900E-01,&
     & .1628000E-01,.1931200E-01,.2213600E-01,.2426300E-01,.2346600E-01/

      data   absa(181:315, 3)   /                                       &
     & .4259300E-04,.3676000E-02,.5991300E-02,.7981200E-02,.9772900E-02,&
     & .1126500E-01,.1287400E-01,.1415000E-01,.1365100E-01,.4577900E-04,&
     & .3934400E-02,.6540100E-02,.8703000E-02,.1065000E-01,.1237400E-01,&
     & .1413500E-01,.1552900E-01,.1550700E-01,.4886800E-04,.4201300E-02,&
     & .7085900E-02,.9491200E-02,.1160900E-01,.1355700E-01,.1538100E-01,&
     & .1699300E-01,.1678500E-01,.5250900E-04,.4467400E-02,.7602200E-02,&
     & .1027100E-01,.1263000E-01,.1473600E-01,.1683000E-01,.1838700E-01,&
     & .1804000E-01,.5619600E-04,.4738100E-02,.8100500E-02,.1102600E-01,&
     & .1358600E-01,.1594000E-01,.1816000E-01,.1992000E-01,.1909200E-01,&
     & .3611200E-04,.3031600E-02,.4903800E-02,.6532500E-02,.7960600E-02,&
     & .9170600E-02,.1028500E-01,.1125200E-01,.1055600E-01,.3850200E-04,&
     & .3257500E-02,.5385500E-02,.7130200E-02,.8720000E-02,.1013800E-01,&
     & .1131900E-01,.1250400E-01,.1210900E-01,.4107000E-04,.3486900E-02,&
     & .5864300E-02,.7802000E-02,.9523200E-02,.1108100E-01,.1241300E-01,&
     & .1370000E-01,.1335000E-01,.4403400E-04,.3714200E-02,.6292400E-02,&
     & .8486700E-02,.1040000E-01,.1209100E-01,.1361400E-01,.1485000E-01,&
     & .1439600E-01,.4702100E-04,.3938700E-02,.6717100E-02,.9131900E-02,&
     & .1123100E-01,.1312900E-01,.1474500E-01,.1615500E-01,.1539200E-01,&
     & .3000100E-04,.2488400E-02,.3978700E-02,.5326200E-02,.6425400E-02,&
     & .7442200E-02,.8241800E-02,.8765100E-02,.8324100E-02,.3206700E-04,&
     & .2683100E-02,.4387300E-02,.5800900E-02,.7088100E-02,.8230300E-02,&
     & .9111500E-02,.9953000E-02,.9584300E-02,.3441400E-04,.2876200E-02,&
     & .4798100E-02,.6365000E-02,.7755500E-02,.9013000E-02,.1005100E-01,&
     & .1094400E-01,.1068700E-01,.3654300E-04,.3074900E-02,.5169700E-02,&
     & .6954900E-02,.8493900E-02,.9862100E-02,.1101200E-01,.1195000E-01,&
     & .1159600E-01,.3898600E-04,.3267200E-02,.5526800E-02,.7495000E-02,&
     & .9220600E-02,.1075000E-01,.1197000E-01,.1302900E-01,.1246100E-01/

      data   absa(316:450, 3)   /                                       &
     & .2471800E-04,.2030400E-02,.3225300E-02,.4295700E-02,.5162100E-02,&
     & .5947700E-02,.6554300E-02,.6790100E-02,.6657800E-02,.2622600E-04,&
     & .2202700E-02,.3552800E-02,.4684400E-02,.5746300E-02,.6607000E-02,&
     & .7356600E-02,.7908600E-02,.7680700E-02,.2809500E-04,.2361400E-02,&
     & .3896000E-02,.5152900E-02,.6277300E-02,.7291900E-02,.8147500E-02,&
     & .8734400E-02,.8650900E-02,.2985600E-04,.2534600E-02,.4218000E-02,&
     & .5650700E-02,.6886700E-02,.7994800E-02,.8916900E-02,.9622600E-02,&
     & .9421200E-02,.3143200E-04,.2698900E-02,.4528800E-02,.6115800E-02,&
     & .7523100E-02,.8731900E-02,.9744700E-02,.1047000E-01,.1017200E-01,&
     & .2015100E-04,.1662200E-02,.2610200E-02,.3434500E-02,.4151800E-02,&
     & .4735600E-02,.5119800E-02,.5290100E-02,.5560800E-02,.2129000E-04,&
     & .1811200E-02,.2862200E-02,.3777000E-02,.4609800E-02,.5282500E-02,&
     & .5871900E-02,.6211500E-02,.6428700E-02,.2260400E-04,.1948300E-02,&
     & .3153600E-02,.4141800E-02,.5051300E-02,.5888300E-02,.6519900E-02,&
     & .7000000E-02,.7164100E-02,.2403900E-04,.2090900E-02,.3432100E-02,&
     & .4563300E-02,.5557500E-02,.6445300E-02,.7180000E-02,.7706200E-02,&
     & .7782100E-02,.2527500E-04,.2232500E-02,.3697600E-02,.4972700E-02,&
     & .6094500E-02,.7063200E-02,.7869600E-02,.8404800E-02,.8457100E-02,&
     & .1652000E-04,.1357900E-02,.2121700E-02,.2765000E-02,.3320900E-02,&
     & .3765700E-02,.4026300E-02,.4163200E-02,.5901900E-02,.1730700E-04,&
     & .1486800E-02,.2323000E-02,.3061600E-02,.3700000E-02,.4262900E-02,&
     & .4695800E-02,.4895600E-02,.6472900E-02,.1821000E-04,.1608600E-02,&
     & .2563500E-02,.3348000E-02,.4082700E-02,.4759000E-02,.5242400E-02,&
     & .5608800E-02,.7107200E-02,.1926300E-04,.1730800E-02,.2803800E-02,&
     & .3701600E-02,.4493300E-02,.5211400E-02,.5822400E-02,.6205300E-02,&
     & .7644500E-02,.2045000E-04,.1851400E-02,.3028600E-02,.4056900E-02,&
     & .4941700E-02,.5729200E-02,.6380900E-02,.6816200E-02,.8180000E-02/

      data   absa(451:585, 3)   /                                       &
     & .1374300E-04,.1154400E-02,.1800500E-02,.2338000E-02,.2812700E-02,&
     & .3201900E-02,.3418400E-02,.3513700E-02,.5513200E-02,.1431400E-04,&
     & .1264800E-02,.1981100E-02,.2579400E-02,.3128300E-02,.3583500E-02,&
     & .3963500E-02,.4152400E-02,.6064000E-02,.1498000E-04,.1366700E-02,&
     & .2184600E-02,.2831600E-02,.3439000E-02,.4006900E-02,.4428300E-02,&
     & .4728200E-02,.6566200E-02,.1584300E-04,.1469100E-02,.2384400E-02,&
     & .3134600E-02,.3791500E-02,.4392600E-02,.4910800E-02,.5243100E-02,&
     & .7052100E-02,.1680100E-04,.1570900E-02,.2573800E-02,.3425000E-02,&
     & .4178100E-02,.4825800E-02,.5377600E-02,.5740000E-02,.7491300E-02,&
     & .1126500E-04,.9747300E-03,.1519000E-02,.1977900E-02,.2370800E-02,&
     & .2700200E-02,.2895400E-02,.2968900E-02,.4905200E-02,.1170300E-04,&
     & .1066000E-02,.1680600E-02,.2174200E-02,.2637000E-02,.3018400E-02,&
     & .3346100E-02,.3508100E-02,.5351500E-02,.1227500E-04,.1152400E-02,&
     & .1853200E-02,.2401400E-02,.2893200E-02,.3361500E-02,.3733200E-02,&
     & .3975900E-02,.5780600E-02,.1296300E-04,.1240500E-02,.2020000E-02,&
     & .2657300E-02,.3197800E-02,.3690400E-02,.4122500E-02,.4415100E-02,&
     & .6195600E-02,.1369600E-04,.1326600E-02,.2181400E-02,.2893900E-02,&
     & .3528400E-02,.4062900E-02,.4519100E-02,.4828300E-02,.6580000E-02,&
     & .9117900E-05,.8153100E-03,.1275400E-02,.1659800E-02,.1990000E-02,&
     & .2263400E-02,.2444100E-02,.2498400E-02,.4057600E-02,.9503900E-05,&
     & .8909300E-03,.1415000E-02,.1828700E-02,.2214300E-02,.2532900E-02,&
     & .2809200E-02,.2954600E-02,.4437400E-02,.9976200E-05,.9649400E-03,&
     & .1562500E-02,.2028800E-02,.2431900E-02,.2810800E-02,.3137300E-02,&
     & .3317700E-02,.4809400E-02,.1050900E-04,.1039800E-02,.1699600E-02,&
     & .2244300E-02,.2692500E-02,.3094200E-02,.3448100E-02,.3700300E-02,&
     & .5155000E-02,.1103300E-04,.1113400E-02,.1837200E-02,.2440800E-02,&
     & .2968200E-02,.3411400E-02,.3785900E-02,.4042400E-02,.5489300E-02/

      data   absa(  1:180, 4)   /                                       &
     & .2556400E-03,.2394000E-01,.4153500E-01,.5627600E-01,.6856100E-01,&
     & .7863000E-01,.8699300E-01,.9179400E-01,.8422900E-01,.2919400E-03,&
     & .2517000E-01,.4365900E-01,.5901600E-01,.7230200E-01,.8316700E-01,&
     & .9173900E-01,.9677400E-01,.8683800E-01,.3393400E-03,.2643000E-01,&
     & .4583500E-01,.6192200E-01,.7561400E-01,.8728500E-01,.9660600E-01,&
     & .1015600E+00,.9008400E-01,.3887100E-03,.2773300E-01,.4805800E-01,&
     & .6497300E-01,.7909000E-01,.9115600E-01,.1008400E+00,.1058900E+00,&
     & .9258300E-01,.4447300E-03,.2902800E-01,.5028700E-01,.6801000E-01,&
     & .8280500E-01,.9519300E-01,.1049200E+00,.1099500E+00,.9541900E-01,&
     & .2340000E-03,.2075300E-01,.3574200E-01,.4870900E-01,.5954800E-01,&
     & .6820600E-01,.7504500E-01,.7891500E-01,.7018000E-01,.2677500E-03,&
     & .2183200E-01,.3761100E-01,.5115600E-01,.6276800E-01,.7226700E-01,&
     & .7942200E-01,.8326200E-01,.7273500E-01,.3003300E-03,.2296100E-01,&
     & .3961800E-01,.5377400E-01,.6577300E-01,.7581900E-01,.8353200E-01,&
     & .8749600E-01,.7545900E-01,.3436600E-03,.2411600E-01,.4159700E-01,&
     & .5650600E-01,.6894400E-01,.7923100E-01,.8724500E-01,.9133600E-01,&
     & .7766900E-01,.3917900E-03,.2528900E-01,.4357500E-01,.5928200E-01,&
     & .7213200E-01,.8295600E-01,.9103800E-01,.9493700E-01,.8027200E-01,&
     & .2073000E-03,.1804400E-01,.3022700E-01,.4120600E-01,.5062600E-01,&
     & .5809500E-01,.6372500E-01,.6672100E-01,.5778000E-01,.2400700E-03,&
     & .1897200E-01,.3188300E-01,.4348600E-01,.5357200E-01,.6174200E-01,&
     & .6748900E-01,.7055200E-01,.6000900E-01,.2693100E-03,.1993400E-01,&
     & .3368100E-01,.4577500E-01,.5625500E-01,.6486900E-01,.7125400E-01,&
     & .7423200E-01,.6235000E-01,.3006400E-03,.2096500E-01,.3544800E-01,&
     & .4819900E-01,.5903600E-01,.6792100E-01,.7459400E-01,.7768900E-01,&
     & .6441000E-01,.3396700E-03,.2197200E-01,.3719200E-01,.5067600E-01,&
     & .6188400E-01,.7120700E-01,.7788800E-01,.8094700E-01,.6666700E-01,&
     & .1820100E-03,.1559500E-01,.2537200E-01,.3443100E-01,.4240100E-01,&
     & .4891500E-01,.5361300E-01,.5595300E-01,.4772700E-01,.2088300E-03,&
     & .1642200E-01,.2685100E-01,.3648100E-01,.4507400E-01,.5207800E-01,&
     & .5686000E-01,.5928700E-01,.4955300E-01,.2387400E-03,.1728500E-01,&
     & .2842500E-01,.3851400E-01,.4754200E-01,.5490400E-01,.6028400E-01,&
     & .6256100E-01,.5159500E-01,.2670700E-03,.1820000E-01,.3000700E-01,&
     & .4061300E-01,.4998200E-01,.5766200E-01,.6323100E-01,.6566600E-01,&
     & .5345000E-01,.3010200E-03,.1911300E-01,.3155200E-01,.4280500E-01,&
     & .5253500E-01,.6055900E-01,.6619500E-01,.6854900E-01,.5541500E-01/

      data   absa(181:315, 4)   /                                       &
     & .1580100E-03,.1329500E-01,.2152400E-01,.2866600E-01,.3514200E-01,&
     & .4075700E-01,.4471500E-01,.4674000E-01,.3938400E-01,.1799400E-03,&
     & .1405700E-01,.2278600E-01,.3048700E-01,.3756300E-01,.4350900E-01,&
     & .4767600E-01,.4963800E-01,.4084900E-01,.2053500E-03,.1481100E-01,&
     & .2411600E-01,.3226600E-01,.3976400E-01,.4612200E-01,.5066900E-01,&
     & .5241300E-01,.4268300E-01,.2351100E-03,.1559800E-01,.2548100E-01,&
     & .3410300E-01,.4192400E-01,.4859200E-01,.5328300E-01,.5523500E-01,&
     & .4433500E-01,.2664000E-03,.1640200E-01,.2684400E-01,.3600000E-01,&
     & .4418000E-01,.5108600E-01,.5591800E-01,.5779600E-01,.4599100E-01,&
     & .1380500E-03,.1122200E-01,.1812500E-01,.2387700E-01,.2893900E-01,&
     & .3350600E-01,.3694400E-01,.3867200E-01,.3233500E-01,.1543900E-03,&
     & .1188300E-01,.1926200E-01,.2552100E-01,.3106800E-01,.3588200E-01,&
     & .3958900E-01,.4113200E-01,.3368400E-01,.1756100E-03,.1255800E-01,&
     & .2040900E-01,.2703300E-01,.3305600E-01,.3826000E-01,.4215500E-01,&
     & .4363900E-01,.3517500E-01,.1996700E-03,.1323600E-01,.2162800E-01,&
     & .2855400E-01,.3493900E-01,.4045900E-01,.4455000E-01,.4614700E-01,&
     & .3664300E-01,.2311700E-03,.1393600E-01,.2283600E-01,.3018300E-01,&
     & .3690900E-01,.4265200E-01,.4686400E-01,.4844000E-01,.3799200E-01,&
     & .1198700E-03,.9398300E-02,.1513000E-01,.1980800E-01,.2379600E-01,&
     & .2728500E-01,.3017600E-01,.3190600E-01,.2619300E-01,.1316400E-03,&
     & .9974300E-02,.1611800E-01,.2131900E-01,.2560000E-01,.2940400E-01,&
     & .3248500E-01,.3390900E-01,.2743900E-01,.1486200E-03,.1055900E-01,&
     & .1712400E-01,.2267400E-01,.2737300E-01,.3150100E-01,.3470500E-01,&
     & .3611200E-01,.2866700E-01,.1694100E-03,.1114200E-01,.1816300E-01,&
     & .2399200E-01,.2902300E-01,.3346100E-01,.3688200E-01,.3828800E-01,&
     & .2999000E-01,.1944400E-03,.1174000E-01,.1923100E-01,.2541000E-01,&
     & .3070200E-01,.3538000E-01,.3893300E-01,.4032500E-01,.3116300E-01/

      data   absa(316:450, 4)   /                                       &
     & .1026600E-03,.7813200E-02,.1249500E-01,.1629400E-01,.1955200E-01,&
     & .2220300E-01,.2452200E-01,.2615800E-01,.2147700E-01,.1130800E-03,&
     & .8299600E-02,.1338500E-01,.1765000E-01,.2111300E-01,.2402600E-01,&
     & .2642400E-01,.2777900E-01,.2266100E-01,.1243100E-03,.8811800E-02,&
     & .1425400E-01,.1885200E-01,.2271700E-01,.2582200E-01,.2835200E-01,&
     & .2966700E-01,.2367400E-01,.1414600E-03,.9308000E-02,.1515200E-01,&
     & .2001300E-01,.2415800E-01,.2756900E-01,.3030600E-01,.3149500E-01,&
     & .2483800E-01,.1627500E-03,.9828700E-02,.1608000E-01,.2122700E-01,&
     & .2559200E-01,.2922600E-01,.3207500E-01,.3335700E-01,.2588700E-01,&
     & .8579700E-04,.6441200E-02,.1023100E-01,.1332700E-01,.1592300E-01,&
     & .1810300E-01,.1990300E-01,.2118900E-01,.1797500E-01,.9437000E-04,&
     & .6851400E-02,.1104100E-01,.1447200E-01,.1729800E-01,.1966100E-01,&
     & .2142100E-01,.2263100E-01,.1899000E-01,.1060300E-03,.7287000E-02,&
     & .1178600E-01,.1557000E-01,.1871000E-01,.2119500E-01,.2309200E-01,&
     & .2415700E-01,.1993400E-01,.1181900E-03,.7721700E-02,.1255900E-01,&
     & .1658900E-01,.1999100E-01,.2273800E-01,.2476900E-01,.2577100E-01,&
     & .2094600E-01,.1350000E-03,.8162400E-02,.1334600E-01,.1761000E-01,&
     & .2122700E-01,.2414000E-01,.2633900E-01,.2738800E-01,.2181900E-01,&
     & .7327500E-04,.5314700E-02,.8383200E-02,.1089000E-01,.1299800E-01,&
     & .1477200E-01,.1623700E-01,.1716500E-01,.1597200E-01,.8030300E-04,&
     & .5659400E-02,.9096400E-02,.1186900E-01,.1418100E-01,.1606500E-01,&
     & .1749300E-01,.1844700E-01,.1707500E-01,.8882100E-04,.6028600E-02,&
     & .9749100E-02,.1285200E-01,.1538700E-01,.1739400E-01,.1890900E-01,&
     & .1970800E-01,.1807800E-01,.1014200E-03,.6396200E-02,.1039200E-01,&
     & .1373000E-01,.1652200E-01,.1877100E-01,.2030800E-01,.2109600E-01,&
     & .1903700E-01,.1140500E-03,.6775400E-02,.1106500E-01,.1460500E-01,&
     & .1758100E-01,.1999100E-01,.2168700E-01,.2245800E-01,.1987500E-01/

      data   absa(451:585, 4)   /                                       &
     & .6451700E-04,.4513500E-02,.7116500E-02,.9238400E-02,.1099800E-01,&
     & .1247400E-01,.1370000E-01,.1438800E-01,.1463100E-01,.7153500E-04,&
     & .4807000E-02,.7713700E-02,.1009200E-01,.1202300E-01,.1362300E-01,&
     & .1479200E-01,.1544900E-01,.1548100E-01,.7950400E-04,.5119900E-02,&
     & .8260000E-02,.1091100E-01,.1307600E-01,.1475500E-01,.1600200E-01,&
     & .1654600E-01,.1629800E-01,.9043300E-04,.5435100E-02,.8810100E-02,&
     & .1164700E-01,.1401800E-01,.1592200E-01,.1720600E-01,.1771500E-01,&
     & .1699100E-01,.1014600E-03,.5759600E-02,.9386500E-02,.1240700E-01,&
     & .1492200E-01,.1695100E-01,.1838400E-01,.1889300E-01,.1766600E-01,&
     & .5691300E-04,.3836100E-02,.6023800E-02,.7810100E-02,.9294300E-02,&
     & .1051500E-01,.1151400E-01,.1205100E-01,.1270800E-01,.6298400E-04,&
     & .4088600E-02,.6515300E-02,.8546400E-02,.1016800E-01,.1149800E-01,&
     & .1245200E-01,.1293800E-01,.1347300E-01,.7058800E-04,.4356100E-02,&
     & .6976100E-02,.9213700E-02,.1106800E-01,.1248200E-01,.1349700E-01,&
     & .1389200E-01,.1409000E-01,.7955600E-04,.4622200E-02,.7447200E-02,&
     & .9838200E-02,.1185200E-01,.1346100E-01,.1453900E-01,.1486800E-01,&
     & .1468400E-01,.8933000E-04,.4903700E-02,.7933200E-02,.1049200E-01,&
     & .1261300E-01,.1433100E-01,.1552300E-01,.1586700E-01,.1526600E-01,&
     & .4904000E-04,.3247000E-02,.5078200E-02,.6581300E-02,.7825400E-02,&
     & .8839800E-02,.9648300E-02,.1009000E-01,.1061400E-01,.5422500E-04,&
     & .3461600E-02,.5490600E-02,.7195100E-02,.8571100E-02,.9678100E-02,&
     & .1046000E-01,.1084000E-01,.1120600E-01,.6113600E-04,.3687800E-02,&
     & .5877700E-02,.7745800E-02,.9318300E-02,.1052800E-01,.1134700E-01,&
     & .1168400E-01,.1173000E-01,.6878800E-04,.3921300E-02,.6279100E-02,&
     & .8277400E-02,.9976100E-02,.1133200E-01,.1224700E-01,.1249500E-01,&
     & .1225000E-01,.7742700E-04,.4164000E-02,.6689000E-02,.8835800E-02,&
     & .1062400E-01,.1206900E-01,.1306400E-01,.1334100E-01,.1273000E-01/

      data   absa(  1:180, 5)   /                                       &
     & .6909800E-02,.8465000E-01,.1300600E+00,.1665200E+00,.1960800E+00,&
     & .2197500E+00,.2349600E+00,.2327700E+00,.1800700E+00,.7820000E-02,&
     & .8927900E-01,.1363000E+00,.1740900E+00,.2039800E+00,.2271800E+00,&
     & .2412900E+00,.2375000E+00,.1843800E+00,.8978700E-02,.9442800E-01,&
     & .1429600E+00,.1817000E+00,.2120500E+00,.2345000E+00,.2472600E+00,&
     & .2417200E+00,.1887800E+00,.1042600E-01,.1000300E+00,.1500900E+00,&
     & .1894300E+00,.2201500E+00,.2420600E+00,.2534100E+00,.2457100E+00,&
     & .1932500E+00,.1220900E-01,.1060300E+00,.1578100E+00,.1975600E+00,&
     & .2283200E+00,.2496300E+00,.2595700E+00,.2495800E+00,.1974800E+00,&
     & .6402000E-02,.7459700E-01,.1145800E+00,.1458100E+00,.1710300E+00,&
     & .1910500E+00,.2041200E+00,.2013300E+00,.1524200E+00,.7258600E-02,&
     & .7885700E-01,.1202400E+00,.1525700E+00,.1779600E+00,.1976500E+00,&
     & .2097200E+00,.2054800E+00,.1562000E+00,.8290300E-02,.8360200E-01,&
     & .1262300E+00,.1593100E+00,.1851200E+00,.2042300E+00,.2150300E+00,&
     & .2093700E+00,.1599300E+00,.9457800E-02,.8866800E-01,.1327700E+00,&
     & .1663500E+00,.1923300E+00,.2109800E+00,.2205000E+00,.2129400E+00,&
     & .1639700E+00,.1086500E-01,.9411300E-01,.1398700E+00,.1737500E+00,&
     & .1997500E+00,.2177100E+00,.2259200E+00,.2163900E+00,.1674500E+00,&
     & .5694300E-02,.6408400E-01,.9929400E-01,.1261400E+00,.1474600E+00,&
     & .1641400E+00,.1750300E+00,.1725500E+00,.1273800E+00,.6470200E-02,&
     & .6796700E-01,.1042700E+00,.1319700E+00,.1535000E+00,.1698500E+00,&
     & .1800800E+00,.1763100E+00,.1308700E+00,.7420000E-02,.7225200E-01,&
     & .1095600E+00,.1380100E+00,.1597800E+00,.1756700E+00,.1847600E+00,&
     & .1797700E+00,.1342100E+00,.8490700E-02,.7677200E-01,.1153900E+00,&
     & .1442700E+00,.1661500E+00,.1816800E+00,.1894700E+00,.1830000E+00,&
     & .1376100E+00,.9707800E-02,.8167100E-01,.1217200E+00,.1508200E+00,&
     & .1727700E+00,.1876900E+00,.1942700E+00,.1860800E+00,.1406700E+00,&
     & .4882900E-02,.5438300E-01,.8507500E-01,.1081500E+00,.1263800E+00,&
     & .1402000E+00,.1490300E+00,.1467000E+00,.1061400E+00,.5574000E-02,&
     & .5777900E-01,.8943600E-01,.1132300E+00,.1316000E+00,.1451400E+00,&
     & .1534900E+00,.1502700E+00,.1094900E+00,.6401500E-02,.6154600E-01,&
     & .9413900E-01,.1185000E+00,.1370100E+00,.1501700E+00,.1575700E+00,&
     & .1534400E+00,.1125700E+00,.7371900E-02,.6552100E-01,.9923200E-01,&
     & .1240300E+00,.1425900E+00,.1554100E+00,.1617500E+00,.1563400E+00,&
     & .1155700E+00,.8478900E-02,.6981000E-01,.1047800E+00,.1298500E+00,&
     & .1484000E+00,.1607100E+00,.1659700E+00,.1591000E+00,.1183300E+00/

      data   absa(181:315, 5)   /                                       &
     & .4135700E-02,.4587800E-01,.7186600E-01,.9213000E-01,.1077200E+00,&
     & .1193500E+00,.1262700E+00,.1238200E+00,.8849000E-01,.4727800E-02,&
     & .4880900E-01,.7572400E-01,.9648500E-01,.1121800E+00,.1236300E+00,&
     & .1302300E+00,.1272300E+00,.9164600E-01,.5423400E-02,.5210800E-01,&
     & .7992500E-01,.1010500E+00,.1168400E+00,.1279500E+00,.1338900E+00,&
     & .1302400E+00,.9438400E-01,.6267200E-02,.5564700E-01,.8443800E-01,&
     & .1059100E+00,.1216700E+00,.1325500E+00,.1375900E+00,.1328800E+00,&
     & .9703700E-01,.7238700E-02,.5939300E-01,.8932600E-01,.1110200E+00,&
     & .1268000E+00,.1372300E+00,.1412800E+00,.1353000E+00,.9958500E-01,&
     & .3482000E-02,.3845500E-01,.6010600E-01,.7751500E-01,.9114300E-01,&
     & .1009400E+00,.1064900E+00,.1040300E+00,.7350900E-01,.3950700E-02,&
     & .4092700E-01,.6340200E-01,.8124000E-01,.9496800E-01,.1046900E+00,&
     & .1100100E+00,.1071700E+00,.7642600E-01,.4526600E-02,.4372600E-01,&
     & .6704100E-01,.8523400E-01,.9891100E-01,.1084500E+00,.1132900E+00,&
     & .1099600E+00,.7886100E-01,.5224200E-02,.4680700E-01,.7093900E-01,&
     & .8954700E-01,.1031200E+00,.1124700E+00,.1165300E+00,.1123700E+00,&
     & .8125000E-01,.6044900E-02,.5008300E-01,.7517500E-01,.9404100E-01,&
     & .1076000E+00,.1165700E+00,.1198000E+00,.1144900E+00,.8354900E-01,&
     & .2871100E-02,.3252300E-01,.4997100E-01,.6459700E-01,.7636000E-01,&
     & .8491800E-01,.8952300E-01,.8720000E-01,.6061800E-01,.3255000E-02,&
     & .3461400E-01,.5280300E-01,.6774000E-01,.7971300E-01,.8820700E-01,&
     & .9268100E-01,.9006000E-01,.6333100E-01,.3726300E-02,.3696800E-01,&
     & .5590400E-01,.7115300E-01,.8314300E-01,.9150900E-01,.9560700E-01,&
     & .9256400E-01,.6559500E-01,.4295300E-02,.3953800E-01,.5928100E-01,&
     & .7491200E-01,.8682200E-01,.9495600E-01,.9843800E-01,.9472800E-01,&
     & .6768000E-01,.4978800E-02,.4226400E-01,.6295300E-01,.7884200E-01,&
     & .9074200E-01,.9851000E-01,.1012900E+00,.9664100E-01,.6974600E-01/

      data   absa(316:450, 5)   /                                       &
     & .2348000E-02,.2723000E-01,.4167100E-01,.5356500E-01,.6343600E-01,&
     & .7099900E-01,.7502000E-01,.7290600E-01,.4938600E-01,.2651700E-02,&
     & .2906500E-01,.4401700E-01,.5622500E-01,.6634100E-01,.7393500E-01,&
     & .7787900E-01,.7546800E-01,.5175000E-01,.3028500E-02,.3111200E-01,&
     & .4659800E-01,.5916100E-01,.6930200E-01,.7680900E-01,.8043900E-01,&
     & .7772900E-01,.5390400E-01,.3481600E-02,.3334800E-01,.4944800E-01,&
     & .6238000E-01,.7250200E-01,.7978600E-01,.8288700E-01,.7968300E-01,&
     & .5573100E-01,.4044400E-02,.3573000E-01,.5254500E-01,.6579400E-01,&
     & .7587800E-01,.8288900E-01,.8537900E-01,.8139100E-01,.5750900E-01,&
     & .1924700E-02,.2263000E-01,.3475200E-01,.4438700E-01,.5245500E-01,&
     & .5883300E-01,.6246800E-01,.6071900E-01,.4188000E-01,.2155700E-02,&
     & .2420400E-01,.3675000E-01,.4663600E-01,.5497600E-01,.6142100E-01,&
     & .6505200E-01,.6299400E-01,.4401000E-01,.2447700E-02,.2595800E-01,&
     & .3898100E-01,.4910800E-01,.5752600E-01,.6390700E-01,.6735100E-01,&
     & .6503800E-01,.4592900E-01,.2819600E-02,.2788200E-01,.4143100E-01,&
     & .5180200E-01,.6024900E-01,.6648000E-01,.6946700E-01,.6678800E-01,&
     & .4755500E-01,.3265400E-02,.2993900E-01,.4407200E-01,.5471400E-01,&
     & .6316300E-01,.6920700E-01,.7160000E-01,.6832800E-01,.4907700E-01,&
     & .1620000E-02,.1881400E-01,.2891100E-01,.3702500E-01,.4347900E-01,&
     & .4869100E-01,.5176700E-01,.5051700E-01,.3656500E-01,.1803100E-02,&
     & .2015000E-01,.3063200E-01,.3896700E-01,.4560900E-01,.5094000E-01,&
     & .5404600E-01,.5251500E-01,.3842500E-01,.2023200E-02,.2164000E-01,&
     & .3257200E-01,.4107900E-01,.4778800E-01,.5310100E-01,.5611400E-01,&
     & .5433800E-01,.4016200E-01,.2303800E-02,.2329800E-01,.3470300E-01,&
     & .4339300E-01,.5011000E-01,.5529800E-01,.5798200E-01,.5587300E-01,&
     & .4178200E-01,.2664900E-02,.2507300E-01,.3698300E-01,.4586800E-01,&
     & .5261700E-01,.5764100E-01,.5983300E-01,.5722200E-01,.4324300E-01/

      data   absa(451:585, 5)   /                                       &
     & .1424100E-02,.1606500E-01,.2458600E-01,.3146500E-01,.3696300E-01,&
     & .4109300E-01,.4361700E-01,.4267300E-01,.3234300E-01,.1587800E-02,&
     & .1726800E-01,.2614400E-01,.3316900E-01,.3881600E-01,.4298800E-01,&
     & .4554600E-01,.4432700E-01,.3405400E-01,.1800300E-02,.1860900E-01,&
     & .2788800E-01,.3506600E-01,.4071700E-01,.4486100E-01,.4727700E-01,&
     & .4584500E-01,.3556200E-01,.2051300E-02,.2006900E-01,.2979400E-01,&
     & .3714700E-01,.4278100E-01,.4677400E-01,.4886900E-01,.4710600E-01,&
     & .3688100E-01,.2361000E-02,.2163600E-01,.3178600E-01,.3934100E-01,&
     & .4496400E-01,.4883200E-01,.5047800E-01,.4823700E-01,.3813600E-01,&
     & .1236700E-02,.1366400E-01,.2084100E-01,.2661900E-01,.3125800E-01,&
     & .3476600E-01,.3666500E-01,.3587300E-01,.2781100E-01,.1389800E-02,&
     & .1474700E-01,.2223800E-01,.2813400E-01,.3287600E-01,.3639200E-01,&
     & .3828600E-01,.3725700E-01,.2921200E-01,.1582200E-02,.1592000E-01,&
     & .2380800E-01,.2983000E-01,.3455800E-01,.3801400E-01,.3972800E-01,&
     & .3849300E-01,.3051700E-01,.1811700E-02,.1719500E-01,.2547900E-01,&
     & .3166400E-01,.3637800E-01,.3967500E-01,.4109800E-01,.3955300E-01,&
     & .3170500E-01,.2099600E-02,.1856700E-01,.2722500E-01,.3357100E-01,&
     & .3830200E-01,.4143700E-01,.4250100E-01,.4052500E-01,.3274800E-01,&
     & .1060600E-02,.1159400E-01,.1761600E-01,.2245600E-01,.2633700E-01,&
     & .2928200E-01,.3083100E-01,.3003700E-01,.2325100E-01,.1197500E-02,&
     & .1254100E-01,.1885000E-01,.2379500E-01,.2774500E-01,.3067600E-01,&
     & .3219100E-01,.3118600E-01,.2446100E-01,.1368600E-02,.1355700E-01,&
     & .2023600E-01,.2528700E-01,.2922400E-01,.3208500E-01,.3341100E-01,&
     & .3219000E-01,.2553900E-01,.1578700E-02,.1465500E-01,.2168500E-01,&
     & .2688700E-01,.3082100E-01,.3353300E-01,.3458100E-01,.3309100E-01,&
     & .2648300E-01,.1842200E-02,.1584700E-01,.2320100E-01,.2853500E-01,&
     & .3248600E-01,.3505900E-01,.3579100E-01,.3394200E-01,.2734000E-01/

      data   absa(  1:180, 6)   /                                       &
     & .3839378E+00,.6279180E+00,.7318272E+00,.7920608E+00,.8216785E+00,&
     & .8230008E+00,.7947126E+00,.7302326E+00,.5811177E+00,.4128871E+00,&
     & .6485557E+00,.7485293E+00,.8043995E+00,.8308183E+00,.8293298E+00,&
     & .7988096E+00,.7334673E+00,.5866991E+00,.4408435E+00,.6690805E+00,&
     & .7652913E+00,.8177871E+00,.8406473E+00,.8361296E+00,.8033672E+00,&
     & .7370020E+00,.5912609E+00,.4672892E+00,.6894163E+00,.7820258E+00,&
     & .8313814E+00,.8509556E+00,.8436351E+00,.8085369E+00,.7402632E+00,&
     & .5956103E+00,.4927709E+00,.7092005E+00,.7986459E+00,.8452246E+00,&
     & .8615155E+00,.8516325E+00,.8138231E+00,.7437317E+00,.5997054E+00,&
     & .3325679E+00,.5548358E+00,.6461479E+00,.6995525E+00,.7253118E+00,&
     & .7266814E+00,.7024839E+00,.6480527E+00,.5065275E+00,.3575484E+00,&
     & .5737759E+00,.6620339E+00,.7123154E+00,.7348811E+00,.7336335E+00,&
     & .7074669E+00,.6519501E+00,.5119171E+00,.3814205E+00,.5923353E+00,&
     & .6775822E+00,.7253565E+00,.7450096E+00,.7412995E+00,.7131653E+00,&
     & .6558395E+00,.5167702E+00,.4042236E+00,.6103975E+00,.6930426E+00,&
     & .7383493E+00,.7554787E+00,.7494988E+00,.7190075E+00,.6598135E+00,&
     & .5211434E+00,.4265822E+00,.6282698E+00,.7082747E+00,.7513798E+00,&
     & .7663755E+00,.7576413E+00,.7252801E+00,.6640475E+00,.5252205E+00,&
     & .2819028E+00,.4836690E+00,.5649400E+00,.6121709E+00,.6347883E+00,&
     & .6367827E+00,.6170448E+00,.5707116E+00,.4371090E+00,.3032609E+00,&
     & .5006309E+00,.5795105E+00,.6243936E+00,.6444057E+00,.6442898E+00,&
     & .6228572E+00,.5750819E+00,.4422485E+00,.3236861E+00,.5170268E+00,&
     & .5937689E+00,.6365222E+00,.6544181E+00,.6524051E+00,.6289736E+00,&
     & .5795020E+00,.4469729E+00,.3434421E+00,.5330792E+00,.6078864E+00,&
     & .6485227E+00,.6647491E+00,.6605735E+00,.6354043E+00,.5840256E+00,&
     & .4512681E+00,.3630007E+00,.5490413E+00,.6217655E+00,.6606764E+00,&
     & .6752251E+00,.6688803E+00,.6420320E+00,.5886154E+00,.4550732E+00,&
     & .2364425E+00,.4182373E+00,.4906262E+00,.5320011E+00,.5523509E+00,&
     & .5548703E+00,.5383899E+00,.4987899E+00,.3768887E+00,.2546801E+00,&
     & .4331987E+00,.5037554E+00,.5432869E+00,.5616903E+00,.5626459E+00,&
     & .5445887E+00,.5034773E+00,.3818393E+00,.2721315E+00,.4476600E+00,&
     & .5164741E+00,.5543752E+00,.5713544E+00,.5708581E+00,.5510757E+00,&
     & .5081946E+00,.3862452E+00,.2892043E+00,.4618692E+00,.5290503E+00,&
     & .5654304E+00,.5811711E+00,.5789470E+00,.5577046E+00,.5130234E+00,&
     & .3902090E+00,.3062774E+00,.4761272E+00,.5414198E+00,.5765335E+00,&
     & .5909044E+00,.5871126E+00,.5644694E+00,.5178294E+00,.3937875E+00/

      data   absa(181:315, 6)   /                                       &
     & .1976097E+00,.3599556E+00,.4238261E+00,.4601500E+00,.4784937E+00,&
     & .4808783E+00,.4668807E+00,.4328773E+00,.3243113E+00,.2131404E+00,&
     & .3730680E+00,.4353477E+00,.4703740E+00,.4874113E+00,.4885689E+00,&
     & .4731711E+00,.4377759E+00,.3288869E+00,.2280961E+00,.3857179E+00,&
     & .4465292E+00,.4804726E+00,.4964950E+00,.4964579E+00,.4796823E+00,&
     & .4426578E+00,.3330323E+00,.2427415E+00,.3981437E+00,.4574729E+00,&
     & .4906220E+00,.5055377E+00,.5041976E+00,.4863363E+00,.4474608E+00,&
     & .3367502E+00,.2574916E+00,.4108035E+00,.4685214E+00,.5005586E+00,&
     & .5144711E+00,.5119125E+00,.4929884E+00,.4523415E+00,.3400646E+00,&
     & .1644599E+00,.3082155E+00,.3641066E+00,.3958133E+00,.4120577E+00,&
     & .4144583E+00,.4024002E+00,.3731548E+00,.2772332E+00,.1776815E+00,&
     & .3195976E+00,.3741572E+00,.4049902E+00,.4203131E+00,.4217397E+00,&
     & .4086238E+00,.3780822E+00,.2815278E+00,.1904154E+00,.3305911E+00,&
     & .3838882E+00,.4140681E+00,.4286082E+00,.4290731E+00,.4150352E+00,&
     & .3829532E+00,.2854673E+00,.2029594E+00,.3414094E+00,.3935449E+00,&
     & .4230885E+00,.4367885E+00,.4362952E+00,.4213065E+00,.3877874E+00,&
     & .2889571E+00,.2156141E+00,.3525176E+00,.4033758E+00,.4319990E+00,&
     & .4448634E+00,.4434057E+00,.4275549E+00,.3926902E+00,.2921021E+00,&
     & .1365354E+00,.2627938E+00,.3116032E+00,.3390470E+00,.3531982E+00,&
     & .3553931E+00,.3450681E+00,.3195057E+00,.2355642E+00,.1476945E+00,&
     & .2725931E+00,.3203697E+00,.3471668E+00,.3606270E+00,.3621563E+00,&
     & .3510351E+00,.3243608E+00,.2395644E+00,.1584917E+00,.2820807E+00,&
     & .3288732E+00,.3551503E+00,.3679817E+00,.3688657E+00,.3570843E+00,&
     & .3290971E+00,.2432894E+00,.1692002E+00,.2914586E+00,.3373668E+00,&
     & .3630273E+00,.3752511E+00,.3754063E+00,.3629045E+00,.3338612E+00,&
     & .2465948E+00,.1800234E+00,.3011255E+00,.3460817E+00,.3709814E+00,&
     & .3824342E+00,.3817964E+00,.3686830E+00,.3386098E+00,.2495277E+00/

      data   absa(316:450, 6)   /                                       &
     & .1130524E+00,.2235733E+00,.2655973E+00,.2894880E+00,.3015165E+00,&
     & .3032589E+00,.2943333E+00,.2719817E+00,.1988483E+00,.1224152E+00,&
     & .2318861E+00,.2733317E+00,.2965534E+00,.3081331E+00,.3094246E+00,&
     & .2999446E+00,.2766238E+00,.2026511E+00,.1315214E+00,.2399609E+00,&
     & .2807287E+00,.3034743E+00,.3145900E+00,.3154081E+00,.3054644E+00,&
     & .2811905E+00,.2060707E+00,.1406060E+00,.2479709E+00,.2881554E+00,&
     & .3103325E+00,.3209441E+00,.3212333E+00,.3107742E+00,.2857474E+00,&
     & .2091635E+00,.1498138E+00,.2563308E+00,.2958019E+00,.3173622E+00,&
     & .3271974E+00,.3269371E+00,.3159983E+00,.2902250E+00,.2119359E+00,&
     & .9301188E-01,.1895129E+00,.2254431E+00,.2461449E+00,.2564128E+00,&
     & .2576877E+00,.2498578E+00,.2303767E+00,.1659328E+00,.1009065E+00,&
     & .1965526E+00,.2321100E+00,.2522877E+00,.2622065E+00,.2632244E+00,&
     & .2549826E+00,.2347053E+00,.1694537E+00,.1085578E+00,.2034287E+00,&
     & .2384465E+00,.2583058E+00,.2678405E+00,.2684545E+00,.2599040E+00,&
     & .2390279E+00,.1725782E+00,.1162137E+00,.2102907E+00,.2448387E+00,&
     & .2642935E+00,.2733148E+00,.2735764E+00,.2646745E+00,.2432888E+00,&
     & .1754745E+00,.1240561E+00,.2175246E+00,.2515388E+00,.2703899E+00,&
     & .2788204E+00,.2785885E+00,.2693407E+00,.2473703E+00,.1781073E+00,&
     & .7640796E-01,.1603808E+00,.1911818E+00,.2086778E+00,.2175552E+00,&
     & .2186307E+00,.2116034E+00,.1944515E+00,.1456540E+00,.8306947E-01,&
     & .1663339E+00,.1968047E+00,.2139806E+00,.2226600E+00,.2234720E+00,&
     & .2162217E+00,.1984391E+00,.1487611E+00,.8952871E-01,.1721996E+00,&
     & .2022331E+00,.2191642E+00,.2275327E+00,.2280233E+00,.2205353E+00,&
     & .2024797E+00,.1513787E+00,.9600912E-01,.1780570E+00,.2077289E+00,&
     & .2243376E+00,.2322476E+00,.2324865E+00,.2247285E+00,.2063451E+00,&
     & .1537312E+00,.1026970E+00,.1842930E+00,.2135640E+00,.2296382E+00,&
     & .2370510E+00,.2368348E+00,.2288014E+00,.2100442E+00,.1559266E+00/

      data   absa(451:585, 6)   /                                       &
     & .6498651E-01,.1372893E+00,.1636307E+00,.1784508E+00,.1858229E+00,&
     & .1867364E+00,.1805030E+00,.1651282E+00,.1253243E+00,.7051877E-01,&
     & .1422924E+00,.1682882E+00,.1829691E+00,.1901293E+00,.1907854E+00,&
     & .1844056E+00,.1688531E+00,.1278858E+00,.7594246E-01,.1472381E+00,&
     & .1729539E+00,.1873767E+00,.1942330E+00,.1947083E+00,.1880933E+00,&
     & .1724213E+00,.1301325E+00,.8152013E-01,.1524039E+00,.1778017E+00,&
     & .1918315E+00,.1983471E+00,.1985237E+00,.1916908E+00,.1757953E+00,&
     & .1322663E+00,.8732427E-01,.1579555E+00,.1829727E+00,.1965737E+00,&
     & .2026361E+00,.2023437E+00,.1952484E+00,.1791155E+00,.1342170E+00,&
     & .5506321E-01,.1168681E+00,.1392111E+00,.1518638E+00,.1581057E+00,&
     & .1587078E+00,.1534476E+00,.1400850E+00,.1080712E+00,.5967938E-01,&
     & .1210769E+00,.1431802E+00,.1556769E+00,.1617277E+00,.1621446E+00,&
     & .1567559E+00,.1434036E+00,.1102933E+00,.6432180E-01,.1253570E+00,&
     & .1472032E+00,.1594570E+00,.1652341E+00,.1654796E+00,.1599072E+00,&
     & .1464712E+00,.1123711E+00,.6912124E-01,.1299527E+00,.1515000E+00,&
     & .1634309E+00,.1688516E+00,.1687937E+00,.1630152E+00,.1494484E+00,&
     & .1143749E+00,.7411322E-01,.1349524E+00,.1561843E+00,.1676813E+00,&
     & .1726787E+00,.1722628E+00,.1662240E+00,.1523717E+00,.1162707E+00,&
     & .4671501E-01,.9906607E-01,.1179218E+00,.1285633E+00,.1338684E+00,&
     & .1344469E+00,.1299950E+00,.1186902E+00,.9160652E-01,.5062345E-01,&
     & .1026419E+00,.1213067E+00,.1318081E+00,.1369356E+00,.1373840E+00,&
     & .1328006E+00,.1215065E+00,.9359446E-01,.5459974E-01,.1064175E+00,&
     & .1248279E+00,.1351247E+00,.1399943E+00,.1402452E+00,.1355160E+00,&
     & .1241771E+00,.9556170E-01,.5873868E-01,.1105265E+00,.1286741E+00,&
     & .1386392E+00,.1431974E+00,.1431960E+00,.1382828E+00,.1267833E+00,&
     & .9738167E-01,.6302475E-01,.1149971E+00,.1329313E+00,.1425338E+00,&
     & .1467358E+00,.1463643E+00,.1411782E+00,.1293360E+00,.9929140E-01/

      data   absa(  1:180, 7)   /                                       &
     & .2084400E+01,.2334400E+01,.2557600E+01,.2670100E+01,.2701000E+01,&
     & .2668100E+01,.2568300E+01,.2365500E+01,.2081000E+01,.2080700E+01,&
     & .2330400E+01,.2548900E+01,.2661100E+01,.2692000E+01,.2660900E+01,&
     & .2564700E+01,.2364500E+01,.2085900E+01,.2087700E+01,.2334200E+01,&
     & .2547700E+01,.2654800E+01,.2684300E+01,.2654100E+01,.2560500E+01,&
     & .2363700E+01,.2090300E+01,.2105200E+01,.2345400E+01,.2552000E+01,&
     & .2652900E+01,.2677800E+01,.2647800E+01,.2556500E+01,.2363000E+01,&
     & .2093500E+01,.2130300E+01,.2363300E+01,.2561900E+01,.2654600E+01,&
     & .2674900E+01,.2642900E+01,.2554200E+01,.2361900E+01,.2095800E+01,&
     & .1812000E+01,.2110100E+01,.2320400E+01,.2426600E+01,.2465700E+01,&
     & .2446900E+01,.2370200E+01,.2193100E+01,.1866900E+01,.1818700E+01,&
     & .2113700E+01,.2318800E+01,.2422300E+01,.2462000E+01,.2443600E+01,&
     & .2368700E+01,.2194400E+01,.1872600E+01,.1835600E+01,.2125500E+01,&
     & .2325600E+01,.2423200E+01,.2459000E+01,.2440700E+01,.2367400E+01,&
     & .2196000E+01,.1877600E+01,.1861400E+01,.2144700E+01,.2338600E+01,&
     & .2428700E+01,.2459500E+01,.2438700E+01,.2367200E+01,.2197000E+01,&
     & .1881600E+01,.1892700E+01,.2168300E+01,.2357900E+01,.2439200E+01,&
     & .2463600E+01,.2440700E+01,.2366500E+01,.2197400E+01,.1886400E+01,&
     & .1570200E+01,.1894200E+01,.2083700E+01,.2185400E+01,.2232400E+01,&
     & .2226600E+01,.2164900E+01,.2013400E+01,.1661700E+01,.1582700E+01,&
     & .1903800E+01,.2088600E+01,.2186500E+01,.2232900E+01,.2226600E+01,&
     & .2164900E+01,.2016700E+01,.1669500E+01,.1604600E+01,.1921300E+01,&
     & .2101900E+01,.2194200E+01,.2236000E+01,.2227100E+01,.2166500E+01,&
     & .2019500E+01,.1676200E+01,.1632900E+01,.1944300E+01,.2121300E+01,&
     & .2207000E+01,.2242400E+01,.2231100E+01,.2168300E+01,.2021400E+01,&
     & .1683000E+01,.1665800E+01,.1971700E+01,.2145200E+01,.2225100E+01,&
     & .2253000E+01,.2237100E+01,.2170800E+01,.2023800E+01,.1690400E+01,&
     & .1356400E+01,.1686900E+01,.1855000E+01,.1953700E+01,.2004900E+01,&
     & .2008500E+01,.1960100E+01,.1828700E+01,.1475600E+01,.1372000E+01,&
     & .1700000E+01,.1865500E+01,.1960700E+01,.2010000E+01,.2010500E+01,&
     & .1962200E+01,.1833900E+01,.1485400E+01,.1395200E+01,.1720400E+01,&
     & .1883700E+01,.1974000E+01,.2018200E+01,.2015400E+01,.1965400E+01,&
     & .1838400E+01,.1494800E+01,.1422900E+01,.1746100E+01,.1907200E+01,&
     & .1992600E+01,.2030300E+01,.2023400E+01,.1969100E+01,.1842000E+01,&
     & .1504000E+01,.1453400E+01,.1775100E+01,.1934500E+01,.2015400E+01,&
     & .2046700E+01,.2033700E+01,.1974200E+01,.1846500E+01,.1513100E+01/

      data   absa(181:315, 7)   /                                       &
     & .1167500E+01,.1490600E+01,.1642200E+01,.1736400E+01,.1787500E+01,&
     & .1797000E+01,.1758200E+01,.1645900E+01,.1305700E+01,.1183900E+01,&
     & .1506200E+01,.1657300E+01,.1748300E+01,.1795600E+01,.1802000E+01,&
     & .1762300E+01,.1652900E+01,.1317600E+01,.1206100E+01,.1528100E+01,&
     & .1679000E+01,.1766000E+01,.1808200E+01,.1810600E+01,.1767800E+01,&
     & .1658700E+01,.1328900E+01,.1231100E+01,.1554900E+01,.1704800E+01,&
     & .1787700E+01,.1825000E+01,.1822200E+01,.1774200E+01,.1664400E+01,&
     & .1339800E+01,.1258200E+01,.1583600E+01,.1733800E+01,.1813100E+01,&
     & .1845700E+01,.1835900E+01,.1782600E+01,.1671000E+01,.1350700E+01,&
     & .9998100E+00,.1308000E+01,.1447700E+01,.1535600E+01,.1582900E+01,&
     & .1593600E+01,.1563600E+01,.1468300E+01,.1148200E+01,.1015100E+01,&
     & .1325100E+01,.1465500E+01,.1550400E+01,.1594100E+01,.1601500E+01,&
     & .1569800E+01,.1476700E+01,.1161900E+01,.1034400E+01,.1347600E+01,&
     & .1488700E+01,.1570200E+01,.1609700E+01,.1613000E+01,.1577200E+01,&
     & .1484700E+01,.1174300E+01,.1056000E+01,.1373500E+01,.1515500E+01,&
     & .1593900E+01,.1629300E+01,.1627600E+01,.1587000E+01,.1492400E+01,&
     & .1186400E+01,.1079400E+01,.1399600E+01,.1543800E+01,.1620500E+01,&
     & .1652100E+01,.1645100E+01,.1599100E+01,.1500300E+01,.1198200E+01,&
     & .8523900E+00,.1141900E+01,.1271400E+01,.1350700E+01,.1393000E+01,&
     & .1403900E+01,.1379000E+01,.1297100E+01,.1003400E+01,.8654200E+00,&
     & .1159100E+01,.1289800E+01,.1367000E+01,.1406200E+01,.1413900E+01,&
     & .1387100E+01,.1307200E+01,.1017900E+01,.8810800E+00,.1180900E+01,&
     & .1313000E+01,.1388000E+01,.1424100E+01,.1427800E+01,.1397500E+01,&
     & .1317200E+01,.1031000E+01,.8990000E+00,.1204100E+01,.1339000E+01,&
     & .1412300E+01,.1445500E+01,.1445300E+01,.1410500E+01,.1326600E+01,&
     & .1043400E+01,.9186200E+00,.1227900E+01,.1364900E+01,.1438600E+01,&
     & .1469200E+01,.1466400E+01,.1425400E+01,.1336700E+01,.1055800E+01/

      data   absa(316:450, 7)   /                                       &
     & .7249700E+00,.9920100E+00,.1110700E+01,.1181300E+01,.1219500E+01,&
     & .1228500E+01,.1205800E+01,.1135900E+01,.8711000E+00,.7351700E+00,&
     & .1008600E+01,.1128500E+01,.1198800E+01,.1234200E+01,.1240500E+01,&
     & .1216000E+01,.1147900E+01,.8856800E+00,.7475200E+00,.1028500E+01,&
     & .1150700E+01,.1219800E+01,.1253200E+01,.1257000E+01,.1228900E+01,&
     & .1158900E+01,.8992700E+00,.7621300E+00,.1048900E+01,.1174700E+01,&
     & .1243600E+01,.1275700E+01,.1276900E+01,.1244500E+01,.1170000E+01,&
     & .9120300E+00,.7785400E+00,.1069700E+01,.1198000E+01,.1267900E+01,&
     & .1300200E+01,.1299400E+01,.1262100E+01,.1181600E+01,.9239200E+00,&
     & .6168200E+00,.8584100E+00,.9640300E+00,.1027200E+01,.1060600E+01,&
     & .1067900E+01,.1047200E+01,.9871500E+00,.7482500E+00,.6241500E+00,&
     & .8740100E+00,.9814000E+00,.1044600E+01,.1076300E+01,.1081100E+01,&
     & .1059300E+01,.1000300E+01,.7629400E+00,.6338700E+00,.8910400E+00,&
     & .1002200E+01,.1065100E+01,.1096200E+01,.1099700E+01,.1074000E+01,&
     & .1012000E+01,.7765900E+00,.6454200E+00,.9087200E+00,.1023000E+01,&
     & .1087800E+01,.1118800E+01,.1120700E+01,.1091100E+01,.1024300E+01,&
     & .7890500E+00,.6590000E+00,.9267400E+00,.1043600E+01,.1110300E+01,&
     & .1142400E+01,.1143500E+01,.1110200E+01,.1037500E+01,.8003400E+00,&
     & .5254700E+00,.7422000E+00,.8343300E+00,.8889000E+00,.9174100E+00,&
     & .9233600E+00,.9053800E+00,.8531100E+00,.6469600E+00,.5306400E+00,&
     & .7560400E+00,.8510700E+00,.9058400E+00,.9336100E+00,.9383600E+00,&
     & .9186100E+00,.8662600E+00,.6606200E+00,.5380900E+00,.7703700E+00,&
     & .8694200E+00,.9256200E+00,.9540900E+00,.9577400E+00,.9342300E+00,&
     & .8787100E+00,.6739100E+00,.5473400E+00,.7857600E+00,.8873400E+00,&
     & .9463400E+00,.9761100E+00,.9789000E+00,.9525300E+00,.8919100E+00,&
     & .6852600E+00,.5584200E+00,.8012900E+00,.9054200E+00,.9667500E+00,&
     & .9978100E+00,.1001300E+01,.9725300E+00,.9059700E+00,.6958300E+00/

      data   absa(451:585, 7)   /                                       &
     & .4490200E+00,.6464100E+00,.7268200E+00,.7729500E+00,.7967900E+00,&
     & .8009500E+00,.7847800E+00,.7388000E+00,.5750000E+00,.4534300E+00,&
     & .6581900E+00,.7425900E+00,.7903000E+00,.8150900E+00,.8186600E+00,&
     & .7996000E+00,.7513700E+00,.5878400E+00,.4596800E+00,.6709400E+00,&
     & .7581200E+00,.8089800E+00,.8352200E+00,.8385400E+00,.8171100E+00,&
     & .7646100E+00,.5993900E+00,.4676400E+00,.6841800E+00,.7740200E+00,&
     & .8271100E+00,.8551800E+00,.8596800E+00,.8363400E+00,.7785900E+00,&
     & .6095400E+00,.4775000E+00,.6979800E+00,.7902100E+00,.8453400E+00,&
     & .8748000E+00,.8801100E+00,.8559500E+00,.7926900E+00,.6189600E+00,&
     & .3825200E+00,.5612300E+00,.6322100E+00,.6720700E+00,.6919100E+00,&
     & .6945100E+00,.6785300E+00,.6367100E+00,.4962400E+00,.3861000E+00,&
     & .5715200E+00,.6455900E+00,.6885700E+00,.7099200E+00,.7125000E+00,&
     & .6944000E+00,.6494800E+00,.5077200E+00,.3913500E+00,.5825100E+00,&
     & .6595300E+00,.7045900E+00,.7279300E+00,.7318800E+00,.7125100E+00,&
     & .6631400E+00,.5177100E+00,.3983800E+00,.5940600E+00,.6735700E+00,&
     & .7206600E+00,.7457000E+00,.7507500E+00,.7310800E+00,.6769600E+00,&
     & .5268000E+00,.4078500E+00,.6062800E+00,.6880200E+00,.7368500E+00,&
     & .7634200E+00,.7684200E+00,.7485700E+00,.6913500E+00,.5340800E+00,&
     & .3229700E+00,.4855600E+00,.5476000E+00,.5835000E+00,.6007900E+00,&
     & .6011600E+00,.5851800E+00,.5469900E+00,.4227600E+00,.3259100E+00,&
     & .4944500E+00,.5596300E+00,.5976400E+00,.6165400E+00,.6184400E+00,&
     & .6014900E+00,.5597300E+00,.4328000E+00,.3305700E+00,.5040500E+00,&
     & .5718400E+00,.6118600E+00,.6323900E+00,.6355700E+00,.6186400E+00,&
     & .5730500E+00,.4415100E+00,.3370500E+00,.5142700E+00,.5842800E+00,&
     & .6261400E+00,.6482500E+00,.6517500E+00,.6349200E+00,.5868900E+00,&
     & .4489300E+00,.3463600E+00,.5254400E+00,.5974000E+00,.6405500E+00,&
     & .6634200E+00,.6673600E+00,.6500200E+00,.6005500E+00,.4548700E+00/

      data   absa(  1:180, 8)   /                                       &
     & .8454475E+01,.7946981E+01,.8106902E+01,.8293415E+01,.8302744E+01,&
     & .8132473E+01,.7748332E+01,.7379908E+01,.7837216E+01,.8369020E+01,&
     & .7875895E+01,.8045177E+01,.8244563E+01,.8270709E+01,.8116300E+01,&
     & .7749859E+01,.7393894E+01,.7847366E+01,.8285211E+01,.7804773E+01,&
     & .7981608E+01,.8194058E+01,.8236878E+01,.8095655E+01,.7745523E+01,&
     & .7401744E+01,.7853486E+01,.8206631E+01,.7736200E+01,.7920325E+01,&
     & .8142313E+01,.8201279E+01,.8071188E+01,.7732988E+01,.7403543E+01,&
     & .7855384E+01,.8131353E+01,.7669212E+01,.7859419E+01,.8089006E+01,&
     & .8160968E+01,.8041936E+01,.7714112E+01,.7402770E+01,.7854506E+01,&
     & .8035329E+01,.7711984E+01,.8054326E+01,.8312552E+01,.8382878E+01,&
     & .8255716E+01,.7865615E+01,.7362355E+01,.7659280E+01,.7952578E+01,&
     & .7639845E+01,.7991063E+01,.8263177E+01,.8347848E+01,.8239605E+01,&
     & .7870529E+01,.7384107E+01,.7682523E+01,.7876313E+01,.7571250E+01,&
     & .7929061E+01,.8212430E+01,.8313538E+01,.8219780E+01,.7866208E+01,&
     & .7395265E+01,.7697467E+01,.7805455E+01,.7505993E+01,.7868255E+01,&
     & .8161157E+01,.8275309E+01,.8195732E+01,.7853413E+01,.7398979E+01,&
     & .7704855E+01,.7742281E+01,.7447306E+01,.7808392E+01,.8111167E+01,&
     & .8233033E+01,.8164826E+01,.7836884E+01,.7399520E+01,.7708591E+01,&
     & .7532706E+01,.7422731E+01,.7921038E+01,.8241749E+01,.8361773E+01,&
     & .8264953E+01,.7883764E+01,.7273843E+01,.7366162E+01,.7456853E+01,&
     & .7352022E+01,.7858416E+01,.8192991E+01,.8327704E+01,.8251231E+01,&
     & .7894283E+01,.7297668E+01,.7395152E+01,.7388143E+01,.7286773E+01,&
     & .7797671E+01,.8142220E+01,.8291520E+01,.8232710E+01,.7892900E+01,&
     & .7312768E+01,.7415601E+01,.7329897E+01,.7229298E+01,.7738967E+01,&
     & .8093725E+01,.8253763E+01,.8209025E+01,.7884574E+01,.7322979E+01,&
     & .7429711E+01,.7281243E+01,.7179080E+01,.7684292E+01,.8043638E+01,&
     & .8212576E+01,.8180221E+01,.7872560E+01,.7328406E+01,.7440260E+01,&
     & .6960506E+01,.7084860E+01,.7692419E+01,.8072395E+01,.8231420E+01,&
     & .8158941E+01,.7802620E+01,.7115961E+01,.6985525E+01,.6890460E+01,&
     & .7017186E+01,.7632657E+01,.8022920E+01,.8197430E+01,.8148687E+01,&
     & .7816871E+01,.7144322E+01,.7020806E+01,.6833009E+01,.6958106E+01,&
     & .7575897E+01,.7974036E+01,.8162581E+01,.8133025E+01,.7819855E+01,&
     & .7167278E+01,.7050824E+01,.6788341E+01,.6907823E+01,.7522755E+01,&
     & .7926643E+01,.8125775E+01,.8111753E+01,.7815827E+01,.7185676E+01,&
     & .7075641E+01,.6757259E+01,.6867282E+01,.7475452E+01,.7880340E+01,&
     & .8087458E+01,.8084863E+01,.7806464E+01,.7198807E+01,.7097443E+01/

      data   absa(181:315, 8)   /                                       &
     & .6340185E+01,.6707242E+01,.7374216E+01,.7801587E+01,.7986307E+01,&
     & .7935884E+01,.7614843E+01,.6902656E+01,.6547120E+01,.6278416E+01,&
     & .6644090E+01,.7317234E+01,.7752832E+01,.7957040E+01,.7930429E+01,&
     & .7630569E+01,.6936644E+01,.6589898E+01,.6232993E+01,.6592439E+01,&
     & .7264591E+01,.7707509E+01,.7925533E+01,.7918826E+01,.7635078E+01,&
     & .6967951E+01,.6630153E+01,.6204934E+01,.6551194E+01,.7219795E+01,&
     & .7664391E+01,.7893004E+01,.7899675E+01,.7635240E+01,.6992659E+01,&
     & .6663746E+01,.6191620E+01,.6521893E+01,.7181052E+01,.7625747E+01,&
     & .7860409E+01,.7878371E+01,.7631859E+01,.7008584E+01,.6691435E+01,&
     & .5701027E+01,.6282840E+01,.6983217E+01,.7436614E+01,.7637531E+01,&
     & .7613654E+01,.7322878E+01,.6631103E+01,.6068872E+01,.5649363E+01,&
     & .6225960E+01,.6931874E+01,.7392920E+01,.7615331E+01,.7612305E+01,&
     & .7340268E+01,.6671266E+01,.6121220E+01,.5616215E+01,.6181108E+01,&
     & .6886667E+01,.7353790E+01,.7592183E+01,.7603387E+01,.7349200E+01,&
     & .6709222E+01,.6171471E+01,.5602211E+01,.6150585E+01,.6849555E+01,&
     & .7318623E+01,.7568758E+01,.7589509E+01,.7355024E+01,.6738001E+01,&
     & .6212573E+01,.5606495E+01,.6135466E+01,.6822413E+01,.7289641E+01,&
     & .7542848E+01,.7572934E+01,.7357114E+01,.6757793E+01,.6246128E+01,&
     & .5069814E+01,.5819946E+01,.6539483E+01,.6990273E+01,.7204566E+01,&
     & .7201912E+01,.6945088E+01,.6305330E+01,.5574348E+01,.5026679E+01,&
     & .5770074E+01,.6492443E+01,.6954397E+01,.7190065E+01,.7204039E+01,&
     & .6965377E+01,.6352224E+01,.5636607E+01,.5006597E+01,.5734952E+01,&
     & .6454389E+01,.6924197E+01,.7174636E+01,.7200149E+01,.6978142E+01,&
     & .6394579E+01,.5692813E+01,.5006429E+01,.5716643E+01,.6426495E+01,&
     & .6900048E+01,.7157117E+01,.7191321E+01,.6989772E+01,.6426021E+01,&
     & .5738457E+01,.5025396E+01,.5715867E+01,.6411783E+01,.6882764E+01,&
     & .7139311E+01,.7183117E+01,.6997950E+01,.6450090E+01,.5778441E+01/

      data   absa(316:450, 8)   /                                       &
     & .4462234E+01,.5335655E+01,.6051411E+01,.6480333E+01,.6706714E+01,&
     & .6719742E+01,.6501878E+01,.5937436E+01,.5082003E+01,.4429084E+01,&
     & .5293651E+01,.6011247E+01,.6455537E+01,.6695967E+01,.6727139E+01,&
     & .6525443E+01,.5987665E+01,.5149466E+01,.4418913E+01,.5268426E+01,&
     & .5981934E+01,.6436217E+01,.6685886E+01,.6728405E+01,.6544585E+01,&
     & .6031729E+01,.5208404E+01,.4431057E+01,.5262879E+01,.5965884E+01,&
     & .6423698E+01,.6675347E+01,.6726852E+01,.6562473E+01,.6066482E+01,&
     & .5259098E+01,.4463604E+01,.5273803E+01,.5964332E+01,.6418269E+01,&
     & .6667516E+01,.6727383E+01,.6575984E+01,.6094410E+01,.5304372E+01,&
     & .3893017E+01,.4852214E+01,.5530986E+01,.5941479E+01,.6161053E+01,&
     & .6192794E+01,.6013485E+01,.5527651E+01,.4603839E+01,.3868292E+01,&
     & .4816432E+01,.5499970E+01,.5924581E+01,.6155187E+01,.6204798E+01,&
     & .6043116E+01,.5578514E+01,.4673568E+01,.3867315E+01,.4800668E+01,&
     & .5481603E+01,.5912694E+01,.6151550E+01,.6211560E+01,.6068531E+01,&
     & .5623715E+01,.4735833E+01,.3889089E+01,.4804843E+01,.5478401E+01,&
     & .5908385E+01,.6149321E+01,.6219065E+01,.6091671E+01,.5661853E+01,&
     & .4790546E+01,.3931954E+01,.4826464E+01,.5491042E+01,.5912928E+01,&
     & .6152912E+01,.6229107E+01,.6110084E+01,.5693512E+01,.4840222E+01,&
     & .3370800E+01,.4371741E+01,.4996837E+01,.5386824E+01,.5594880E+01,&
     & .5641906E+01,.5505250E+01,.5094350E+01,.4102630E+01,.3354024E+01,&
     & .4346187E+01,.4976577E+01,.5374389E+01,.5596429E+01,.5658447E+01,&
     & .5539483E+01,.5146897E+01,.4175662E+01,.3361375E+01,.4340181E+01,&
     & .4971228E+01,.5370928E+01,.5598179E+01,.5672261E+01,.5570116E+01,&
     & .5193502E+01,.4240227E+01,.3392497E+01,.4352744E+01,.4980679E+01,&
     & .5377112E+01,.5606443E+01,.5688563E+01,.5596813E+01,.5234384E+01,&
     & .4299382E+01,.3445749E+01,.4384734E+01,.5004059E+01,.5393218E+01,&
     & .5623134E+01,.5706956E+01,.5620707E+01,.5270254E+01,.4350295E+01/

      data   absa(451:585, 8)   /                                       &
     & .2896690E+01,.3891858E+01,.4467847E+01,.4828220E+01,.5035892E+01,&
     & .5103952E+01,.5010639E+01,.4674849E+01,.3670107E+01,.2897792E+01,&
     & .3883888E+01,.4462186E+01,.4828076E+01,.5043823E+01,.5124270E+01,&
     & .5046967E+01,.4727706E+01,.3740688E+01,.2923572E+01,.3895453E+01,&
     & .4471923E+01,.4837551E+01,.5057813E+01,.5145441E+01,.5079272E+01,&
     & .4774390E+01,.3805013E+01,.2972461E+01,.3927241E+01,.4497456E+01,&
     & .4859388E+01,.5080493E+01,.5168972E+01,.5109386E+01,.4816374E+01,&
     & .3861922E+01,.3043384E+01,.3977838E+01,.4537492E+01,.4893037E+01,&
     & .5111190E+01,.5197160E+01,.5139056E+01,.4854089E+01,.3912075E+01,&
     & .2490281E+01,.3445349E+01,.3966257E+01,.4299151E+01,.4503210E+01,&
     & .4585981E+01,.4527887E+01,.4253375E+01,.3285333E+01,.2507718E+01,&
     & .3454661E+01,.3975474E+01,.4311153E+01,.4520893E+01,.4611552E+01,&
     & .4565739E+01,.4306696E+01,.3353145E+01,.2549112E+01,.3484385E+01,&
     & .4000658E+01,.4335871E+01,.4546759E+01,.4639747E+01,.4601392E+01,&
     & .4354901E+01,.3414904E+01,.2613534E+01,.3533116E+01,.4041205E+01,&
     & .4373302E+01,.4580520E+01,.4672532E+01,.4636635E+01,.4399031E+01,&
     & .3469339E+01,.2696279E+01,.3599603E+01,.4096892E+01,.4422225E+01,&
     & .4622150E+01,.4711302E+01,.4672921E+01,.4436995E+01,.3520495E+01,&
     & .2147723E+01,.3043091E+01,.3506523E+01,.3813045E+01,.4007672E+01,&
     & .4097762E+01,.4063337E+01,.3839253E+01,.2937518E+01,.2178826E+01,&
     & .3066802E+01,.3529383E+01,.3837562E+01,.4035183E+01,.4129716E+01,&
     & .4103957E+01,.3892792E+01,.3003047E+01,.2233352E+01,.3109540E+01,&
     & .3568948E+01,.3874858E+01,.4071147E+01,.4166460E+01,.4144730E+01,&
     & .3941453E+01,.3061753E+01,.2307024E+01,.3171315E+01,.3623581E+01,&
     & .3923493E+01,.4115241E+01,.4208646E+01,.4185941E+01,.3984848E+01,&
     & .3115620E+01,.2388613E+01,.3250521E+01,.3691760E+01,.3982688E+01,&
     & .4168532E+01,.4255448E+01,.4227404E+01,.4025674E+01,.3167770E+01/

      data   absa(  1:180, 9)   /                                       &
     & .2529000E+02,.2237800E+02,.1968200E+02,.1909200E+02,.1840100E+02,&
     & .1733000E+02,.1719400E+02,.1951300E+02,.2154600E+02,.2515300E+02,&
     & .2225100E+02,.1957400E+02,.1900200E+02,.1830900E+02,.1734600E+02,&
     & .1716800E+02,.1946900E+02,.2151600E+02,.2499000E+02,.2210800E+02,&
     & .1945700E+02,.1889100E+02,.1819700E+02,.1734400E+02,.1715300E+02,&
     & .1943500E+02,.2151600E+02,.2478500E+02,.2192600E+02,.1931200E+02,&
     & .1877400E+02,.1811100E+02,.1730000E+02,.1716800E+02,.1942600E+02,&
     & .2152400E+02,.2456200E+02,.2172900E+02,.1914500E+02,.1865700E+02,&
     & .1804300E+02,.1723900E+02,.1719000E+02,.1940000E+02,.2151700E+02,&
     & .2673800E+02,.2363500E+02,.2116400E+02,.2088200E+02,.2013300E+02,&
     & .1895200E+02,.1839100E+02,.2039700E+02,.2275200E+02,.2657900E+02,&
     & .2349300E+02,.2102600E+02,.2077000E+02,.2004600E+02,.1892400E+02,&
     & .1835100E+02,.2030200E+02,.2267300E+02,.2636900E+02,.2330400E+02,&
     & .2084700E+02,.2063400E+02,.1995500E+02,.1886700E+02,.1835500E+02,&
     & .2027400E+02,.2264300E+02,.2614700E+02,.2310800E+02,.2066300E+02,&
     & .2049100E+02,.1987000E+02,.1879000E+02,.1836500E+02,.2027100E+02,&
     & .2263800E+02,.2587600E+02,.2286700E+02,.2045700E+02,.2030800E+02,&
     & .1978800E+02,.1872300E+02,.1835900E+02,.2024200E+02,.2261200E+02,&
     & .2780100E+02,.2455800E+02,.2262500E+02,.2245600E+02,.2176600E+02,&
     & .2072300E+02,.1978500E+02,.2119300E+02,.2381100E+02,.2758600E+02,&
     & .2436600E+02,.2244400E+02,.2234100E+02,.2167000E+02,.2065100E+02,&
     & .1973200E+02,.2116300E+02,.2377800E+02,.2736200E+02,.2416600E+02,&
     & .2223600E+02,.2220000E+02,.2157500E+02,.2055700E+02,.1971800E+02,&
     & .2115500E+02,.2378300E+02,.2708400E+02,.2391900E+02,.2201300E+02,&
     & .2201800E+02,.2147100E+02,.2045700E+02,.1970800E+02,.2113600E+02,&
     & .2376900E+02,.2679500E+02,.2366600E+02,.2179400E+02,.2182700E+02,&
     & .2135500E+02,.2037400E+02,.1965500E+02,.2109200E+02,.2372400E+02,&
     & .2832700E+02,.2502600E+02,.2391000E+02,.2372000E+02,.2326500E+02,&
     & .2239600E+02,.2108600E+02,.2185900E+02,.2464900E+02,.2809500E+02,&
     & .2481600E+02,.2368300E+02,.2359400E+02,.2315600E+02,.2230400E+02,&
     & .2102300E+02,.2187300E+02,.2467400E+02,.2781700E+02,.2456900E+02,&
     & .2345400E+02,.2344600E+02,.2303100E+02,.2218200E+02,.2102200E+02,&
     & .2187800E+02,.2468000E+02,.2752500E+02,.2431000E+02,.2322000E+02,&
     & .2326800E+02,.2290800E+02,.2209000E+02,.2102200E+02,.2182400E+02,&
     & .2462000E+02,.2719100E+02,.2402300E+02,.2299600E+02,.2307200E+02,&
     & .2276900E+02,.2202200E+02,.2098200E+02,.2173700E+02,.2451400E+02/

      data   absa(181:315, 9)   /                                       &
     & .2825900E+02,.2501400E+02,.2490700E+02,.2470600E+02,.2464200E+02,&
     & .2391800E+02,.2233000E+02,.2221900E+02,.2509100E+02,.2799300E+02,&
     & .2477100E+02,.2466600E+02,.2459600E+02,.2448700E+02,.2381800E+02,&
     & .2234800E+02,.2226900E+02,.2514900E+02,.2769100E+02,.2449900E+02,&
     & .2442100E+02,.2443200E+02,.2434000E+02,.2370500E+02,.2242000E+02,&
     & .2224200E+02,.2510900E+02,.2735700E+02,.2421400E+02,.2417800E+02,&
     & .2425300E+02,.2419100E+02,.2364200E+02,.2242200E+02,.2217400E+02,&
     & .2503000E+02,.2700800E+02,.2391600E+02,.2393500E+02,.2406300E+02,&
     & .2402000E+02,.2357200E+02,.2235500E+02,.2213200E+02,.2495700E+02,&
     & .2762700E+02,.2484100E+02,.2541400E+02,.2551300E+02,.2578800E+02,&
     & .2516100E+02,.2356700E+02,.2238400E+02,.2509100E+02,.2731700E+02,&
     & .2455000E+02,.2516500E+02,.2538400E+02,.2557800E+02,.2508300E+02,&
     & .2366200E+02,.2242300E+02,.2510200E+02,.2699400E+02,.2425900E+02,&
     & .2492600E+02,.2520600E+02,.2539100E+02,.2501500E+02,.2375800E+02,&
     & .2236300E+02,.2502500E+02,.2664000E+02,.2394500E+02,.2469200E+02,&
     & .2500200E+02,.2519300E+02,.2497300E+02,.2373300E+02,.2232800E+02,&
     & .2496500E+02,.2626500E+02,.2362500E+02,.2444600E+02,.2478500E+02,&
     & .2503700E+02,.2490600E+02,.2365900E+02,.2235200E+02,.2496500E+02,&
     & .2643000E+02,.2458100E+02,.2543900E+02,.2614000E+02,.2652400E+02,&
     & .2597400E+02,.2455200E+02,.2244100E+02,.2449800E+02,.2610500E+02,&
     & .2426300E+02,.2522500E+02,.2597400E+02,.2631500E+02,.2594600E+02,&
     & .2467700E+02,.2247100E+02,.2449000E+02,.2575200E+02,.2393800E+02,&
     & .2499600E+02,.2574900E+02,.2611300E+02,.2592500E+02,.2475800E+02,&
     & .2244200E+02,.2445400E+02,.2538900E+02,.2360900E+02,.2476100E+02,&
     & .2549900E+02,.2594000E+02,.2591300E+02,.2472000E+02,.2247300E+02,&
     & .2447500E+02,.2501000E+02,.2326000E+02,.2451000E+02,.2526200E+02,&
     & .2580400E+02,.2581100E+02,.2464000E+02,.2254700E+02,.2453400E+02/

      data   absa(316:450, 9)   /                                       &
     & .2476200E+02,.2399700E+02,.2520900E+02,.2646700E+02,.2674800E+02,&
     & .2636500E+02,.2509400E+02,.2235700E+02,.2340600E+02,.2442100E+02,&
     & .2367200E+02,.2498100E+02,.2621700E+02,.2659500E+02,.2637500E+02,&
     & .2519500E+02,.2243500E+02,.2345300E+02,.2406700E+02,.2333600E+02,&
     & .2475400E+02,.2595600E+02,.2645500E+02,.2636100E+02,.2524100E+02,&
     & .2245800E+02,.2347500E+02,.2369100E+02,.2299600E+02,.2450200E+02,&
     & .2569800E+02,.2633000E+02,.2634300E+02,.2519700E+02,.2253300E+02,&
     & .2357500E+02,.2331100E+02,.2266000E+02,.2423400E+02,.2545000E+02,&
     & .2621500E+02,.2620900E+02,.2512300E+02,.2264200E+02,.2370900E+02,&
     & .2280000E+02,.2301100E+02,.2479900E+02,.2618100E+02,.2653300E+02,&
     & .2630100E+02,.2504400E+02,.2207700E+02,.2190700E+02,.2243600E+02,&
     & .2269700E+02,.2454600E+02,.2594500E+02,.2646000E+02,.2631400E+02,&
     & .2512200E+02,.2220400E+02,.2202900E+02,.2207900E+02,.2236700E+02,&
     & .2429500E+02,.2570600E+02,.2637000E+02,.2628800E+02,.2516200E+02,&
     & .2231400E+02,.2215100E+02,.2171900E+02,.2202900E+02,.2401700E+02,&
     & .2548700E+02,.2627300E+02,.2621700E+02,.2511000E+02,.2243500E+02,&
     & .2233400E+02,.2134100E+02,.2170700E+02,.2370200E+02,.2528200E+02,&
     & .2615900E+02,.2608600E+02,.2506500E+02,.2257800E+02,.2250600E+02,&
     & .2060100E+02,.2167800E+02,.2405800E+02,.2535200E+02,.2593100E+02,&
     & .2571800E+02,.2443200E+02,.2160700E+02,.2026200E+02,.2025400E+02,&
     & .2134500E+02,.2379600E+02,.2518300E+02,.2587500E+02,.2569100E+02,&
     & .2450800E+02,.2177400E+02,.2044500E+02,.1989900E+02,.2104800E+02,&
     & .2349400E+02,.2500000E+02,.2580000E+02,.2566200E+02,.2453300E+02,&
     & .2192200E+02,.2063800E+02,.1954400E+02,.2076600E+02,.2320700E+02,&
     & .2482600E+02,.2568600E+02,.2559300E+02,.2452700E+02,.2208500E+02,&
     & .2083200E+02,.1919300E+02,.2047000E+02,.2293600E+02,.2467300E+02,&
     & .2554400E+02,.2548000E+02,.2453700E+02,.2221800E+02,.2104400E+02/

      data   absa(451:585, 9)   /                                       &
     & .1817000E+02,.2015500E+02,.2272100E+02,.2414500E+02,.2479800E+02,&
     & .2455900E+02,.2340300E+02,.2089500E+02,.1874000E+02,.1783000E+02,&
     & .1986000E+02,.2247900E+02,.2401800E+02,.2472100E+02,.2455600E+02,&
     & .2346600E+02,.2106900E+02,.1894900E+02,.1749600E+02,.1957600E+02,&
     & .2224200E+02,.2388600E+02,.2462100E+02,.2452900E+02,.2352500E+02,&
     & .2123100E+02,.1914700E+02,.1716800E+02,.1929200E+02,.2200500E+02,&
     & .2373400E+02,.2451400E+02,.2447600E+02,.2358000E+02,.2137600E+02,&
     & .1936800E+02,.1684400E+02,.1901300E+02,.2177100E+02,.2357000E+02,&
     & .2439000E+02,.2444000E+02,.2365100E+02,.2151700E+02,.1961200E+02,&
     & .1576200E+02,.1861500E+02,.2113300E+02,.2263900E+02,.2321400E+02,&
     & .2302200E+02,.2206000E+02,.1995800E+02,.1731400E+02,.1545500E+02,&
     & .1834100E+02,.2094400E+02,.2251000E+02,.2314800E+02,.2304200E+02,&
     & .2216400E+02,.2012700E+02,.1754300E+02,.1515600E+02,.1806800E+02,&
     & .2076800E+02,.2237400E+02,.2308400E+02,.2306000E+02,.2225700E+02,&
     & .2027500E+02,.1777000E+02,.1486500E+02,.1779400E+02,.2058200E+02,&
     & .2223700E+02,.2301800E+02,.2307900E+02,.2235000E+02,.2042700E+02,&
     & .1802600E+02,.1461200E+02,.1753400E+02,.2039900E+02,.2208600E+02,&
     & .2295600E+02,.2311200E+02,.2244300E+02,.2060700E+02,.1828000E+02,&
     & .1351700E+02,.1691400E+02,.1942500E+02,.2079000E+02,.2136700E+02,&
     & .2131500E+02,.2060700E+02,.1887100E+02,.1596700E+02,.1324900E+02,&
     & .1668800E+02,.1927900E+02,.2069300E+02,.2134400E+02,.2137200E+02,&
     & .2072500E+02,.1904300E+02,.1621700E+02,.1299400E+02,.1646700E+02,&
     & .1910800E+02,.2060200E+02,.2134300E+02,.2143600E+02,.2083900E+02,&
     & .1921200E+02,.1649400E+02,.1278400E+02,.1625000E+02,.1895100E+02,&
     & .2050600E+02,.2133500E+02,.2150300E+02,.2096100E+02,.1941500E+02,&
     & .1677000E+02,.1273400E+02,.1604900E+02,.1881900E+02,.2043800E+02,&
     & .2132900E+02,.2157300E+02,.2110400E+02,.1962300E+02,.1698300E+02/

      data   absa(  1:180,10)   /                                       &
     & .3587728E+02,.3163298E+02,.2744141E+02,.2457089E+02,.2397093E+02,&
     & .2424633E+02,.2723997E+02,.3121916E+02,.3444458E+02,.3583320E+02,&
     & .3160263E+02,.2742318E+02,.2450739E+02,.2389581E+02,.2409779E+02,&
     & .2700065E+02,.3092695E+02,.3408907E+02,.3574205E+02,.3152702E+02,&
     & .2736303E+02,.2442532E+02,.2382519E+02,.2395943E+02,.2677589E+02,&
     & .3065092E+02,.3375501E+02,.3559971E+02,.3140018E+02,.2725046E+02,&
     & .2431487E+02,.2372351E+02,.2384507E+02,.2657448E+02,.3041293E+02,&
     & .3346529E+02,.3540621E+02,.3122776E+02,.2710481E+02,.2417902E+02,&
     & .2360279E+02,.2374616E+02,.2639615E+02,.3021059E+02,.3322457E+02,&
     & .4028856E+02,.3550073E+02,.3077033E+02,.2797279E+02,.2711240E+02,&
     & .2689686E+02,.2944080E+02,.3377959E+02,.3768881E+02,.4023214E+02,&
     & .3545689E+02,.3073830E+02,.2787552E+02,.2703385E+02,.2679085E+02,&
     & .2921417E+02,.3349222E+02,.3733705E+02,.4012534E+02,.3536466E+02,&
     & .3066284E+02,.2776145E+02,.2693419E+02,.2667684E+02,.2898061E+02,&
     & .3321866E+02,.3700637E+02,.3994773E+02,.3520594E+02,.3052931E+02,&
     & .2761549E+02,.2681235E+02,.2656436E+02,.2878066E+02,.3297874E+02,&
     & .3672473E+02,.3972581E+02,.3501021E+02,.3036408E+02,.2745552E+02,&
     & .2667155E+02,.2645065E+02,.2861110E+02,.3278003E+02,.3649274E+02,&
     & .4476125E+02,.3941959E+02,.3422706E+02,.3167904E+02,.3052481E+02,&
     & .2973002E+02,.3165313E+02,.3629435E+02,.4090093E+02,.4471130E+02,&
     & .3937886E+02,.3418630E+02,.3155677E+02,.3043113E+02,.2965728E+02,&
     & .3143493E+02,.3600307E+02,.4054030E+02,.4457943E+02,.3926295E+02,&
     & .3408654E+02,.3140866E+02,.3032055E+02,.2958250E+02,.3123380E+02,&
     & .3573596E+02,.4022035E+02,.4439236E+02,.3909796E+02,.3394316E+02,&
     & .3123639E+02,.3018674E+02,.2947741E+02,.3105394E+02,.3550064E+02,&
     & .3995406E+02,.4412856E+02,.3886401E+02,.3374559E+02,.3104184E+02,&
     & .3002428E+02,.2936062E+02,.3091031E+02,.3530951E+02,.3974076E+02,&
     & .4910432E+02,.4321511E+02,.3779078E+02,.3556693E+02,.3409226E+02,&
     & .3283110E+02,.3395610E+02,.3866004E+02,.4376828E+02,.4902356E+02,&
     & .4314962E+02,.3771527E+02,.3544117E+02,.3399674E+02,.3277233E+02,&
     & .3377629E+02,.3837960E+02,.4342838E+02,.4888388E+02,.4302756E+02,&
     & .3758282E+02,.3527975E+02,.3389250E+02,.3269755E+02,.3359135E+02,&
     & .3810582E+02,.4311667E+02,.4865644E+02,.4282619E+02,.3740141E+02,&
     & .3507775E+02,.3374861E+02,.3258381E+02,.3342561E+02,.3789331E+02,&
     & .4287514E+02,.4836706E+02,.4257131E+02,.3716473E+02,.3485086E+02,&
     & .3357022E+02,.3245299E+02,.3329940E+02,.3773038E+02,.4269789E+02/

      data   absa(181:315,10)   /                                       &
     & .5307594E+02,.4667483E+02,.4130685E+02,.3949369E+02,.3769718E+02,&
     & .3615626E+02,.3643522E+02,.4090265E+02,.4638012E+02,.5296984E+02,&
     & .4659097E+02,.4119161E+02,.3933623E+02,.3762920E+02,.3609076E+02,&
     & .3626818E+02,.4063453E+02,.4606837E+02,.5278655E+02,.4643245E+02,&
     & .4102506E+02,.3914923E+02,.3752838E+02,.3600030E+02,.3607944E+02,&
     & .4039533E+02,.4580658E+02,.5252049E+02,.4620146E+02,.4079758E+02,&
     & .3892166E+02,.3738707E+02,.3586925E+02,.3592354E+02,.4020885E+02,&
     & .4560089E+02,.5218739E+02,.4591040E+02,.4053117E+02,.3866135E+02,&
     & .3721768E+02,.3572048E+02,.3579033E+02,.4004272E+02,.4542333E+02,&
     & .5645060E+02,.4959535E+02,.4476419E+02,.4329540E+02,.4136555E+02,&
     & .3958949E+02,.3901626E+02,.4295054E+02,.4873997E+02,.5630605E+02,&
     & .4947453E+02,.4458064E+02,.4311329E+02,.4131649E+02,.3950972E+02,&
     & .3886403E+02,.4274257E+02,.4850859E+02,.5608434E+02,.4928505E+02,&
     & .4434892E+02,.4288710E+02,.4120465E+02,.3940634E+02,.3867749E+02,&
     & .4253840E+02,.4828479E+02,.5577797E+02,.4902368E+02,.4406590E+02,&
     & .4263870E+02,.4106511E+02,.3925611E+02,.3851716E+02,.4234391E+02,&
     & .4807363E+02,.5538906E+02,.4869020E+02,.4373996E+02,.4235043E+02,&
     & .4087593E+02,.3908727E+02,.3836821E+02,.4215132E+02,.4786703E+02,&
     & .5901771E+02,.5183601E+02,.4798533E+02,.4682270E+02,.4499590E+02,&
     & .4308761E+02,.4172136E+02,.4474695E+02,.5079194E+02,.5882554E+02,&
     & .5166631E+02,.4772612E+02,.4661016E+02,.4492409E+02,.4299263E+02,&
     & .4159694E+02,.4458623E+02,.5061389E+02,.5854583E+02,.5141979E+02,&
     & .4742883E+02,.4637327E+02,.4480255E+02,.4286883E+02,.4142281E+02,&
     & .4438577E+02,.5039888E+02,.5817816E+02,.5110231E+02,.4708389E+02,&
     & .4609738E+02,.4463949E+02,.4268850E+02,.4126044E+02,.4418534E+02,&
     & .5018101E+02,.5773974E+02,.5072435E+02,.4670644E+02,.4577129E+02,&
     & .4443478E+02,.4250650E+02,.4111702E+02,.4397705E+02,.4997641E+02/

      data   absa(316:450,10)   /                                       &
     & .6060573E+02,.5344936E+02,.5077793E+02,.4997885E+02,.4845492E+02,&
     & .4650628E+02,.4444661E+02,.4611186E+02,.5223587E+02,.6035565E+02,&
     & .5321162E+02,.5047679E+02,.4976412E+02,.4835297E+02,.4640277E+02,&
     & .4437648E+02,.4597962E+02,.5210068E+02,.6001386E+02,.5290034E+02,&
     & .5010907E+02,.4949538E+02,.4818206E+02,.4628144E+02,.4423845E+02,&
     & .4582350E+02,.5193692E+02,.5960595E+02,.5253493E+02,.4971769E+02,&
     & .4917510E+02,.4797986E+02,.4609629E+02,.4410837E+02,.4565103E+02,&
     & .5175207E+02,.5910403E+02,.5209755E+02,.4928800E+02,.4883659E+02,&
     & .4774249E+02,.4591174E+02,.4398757E+02,.4546040E+02,.5157175E+02,&
     & .6109714E+02,.5440970E+02,.5300408E+02,.5268782E+02,.5157437E+02,&
     & .4964597E+02,.4705595E+02,.4702696E+02,.5286716E+02,.6078367E+02,&
     & .5409377E+02,.5266455E+02,.5243664E+02,.5140681E+02,.4958118E+02,&
     & .4706770E+02,.4700927E+02,.5284723E+02,.6037380E+02,.5371502E+02,&
     & .5225748E+02,.5215540E+02,.5119129E+02,.4948761E+02,.4697878E+02,&
     & .4691301E+02,.5274423E+02,.5989651E+02,.5328949E+02,.5183758E+02,&
     & .5181450E+02,.5096397E+02,.4932968E+02,.4689935E+02,.4677701E+02,&
     & .5261482E+02,.5936966E+02,.5280219E+02,.5138276E+02,.5145230E+02,&
     & .5070975E+02,.4914571E+02,.4680586E+02,.4662819E+02,.5250407E+02,&
     & .6042976E+02,.5467511E+02,.5454288E+02,.5481038E+02,.5409097E+02,&
     & .5235706E+02,.4927445E+02,.4742032E+02,.5252935E+02,.6001878E+02,&
     & .5427532E+02,.5413495E+02,.5452142E+02,.5389531E+02,.5233375E+02,&
     & .4936959E+02,.4752824E+02,.5263857E+02,.5955852E+02,.5382395E+02,&
     & .5371774E+02,.5419930E+02,.5366829E+02,.5225894E+02,.4936460E+02,&
     & .4753260E+02,.5265555E+02,.5902075E+02,.5331468E+02,.5326153E+02,&
     & .5383822E+02,.5344237E+02,.5210269E+02,.4932635E+02,.4747318E+02,&
     & .5266176E+02,.5843636E+02,.5278414E+02,.5278267E+02,.5345038E+02,&
     & .5319472E+02,.5193593E+02,.4925399E+02,.4743021E+02,.5267090E+02/

      data   absa(451:585,10)   /                                       &
     & .5841299E+02,.5394652E+02,.5521374E+02,.5605247E+02,.5585908E+02,&
     & .5438236E+02,.5102018E+02,.4757287E+02,.5144594E+02,.5792304E+02,&
     & .5345752E+02,.5477573E+02,.5572296E+02,.5566296E+02,.5437881E+02,&
     & .5115010E+02,.4772352E+02,.5165224E+02,.5737807E+02,.5293116E+02,&
     & .5431199E+02,.5537393E+02,.5546638E+02,.5430287E+02,.5118313E+02,&
     & .4781700E+02,.5183934E+02,.5678698E+02,.5237535E+02,.5381877E+02,&
     & .5500766E+02,.5523934E+02,.5416503E+02,.5117650E+02,.4791639E+02,&
     & .5201561E+02,.5615143E+02,.5178837E+02,.5328954E+02,.5460865E+02,&
     & .5497459E+02,.5398659E+02,.5113103E+02,.4798503E+02,.5214722E+02,&
     & .5536220E+02,.5249053E+02,.5510606E+02,.5651628E+02,.5687411E+02,&
     & .5558754E+02,.5214421E+02,.4740469E+02,.4972236E+02,.5479927E+02,&
     & .5192667E+02,.5465501E+02,.5620526E+02,.5671006E+02,.5563285E+02,&
     & .5229549E+02,.4766073E+02,.5010788E+02,.5421071E+02,.5135180E+02,&
     & .5415133E+02,.5586559E+02,.5652158E+02,.5556841E+02,.5239599E+02,&
     & .4791820E+02,.5046754E+02,.5358519E+02,.5076397E+02,.5362234E+02,&
     & .5548685E+02,.5629675E+02,.5544739E+02,.5247314E+02,.4811189E+02,&
     & .5074908E+02,.5294036E+02,.5016394E+02,.5307965E+02,.5511021E+02,&
     & .5600283E+02,.5528953E+02,.5251126E+02,.4826838E+02,.5100487E+02,&
     & .5148338E+02,.5053874E+02,.5414066E+02,.5624827E+02,.5702373E+02,&
     & .5589687E+02,.5255754E+02,.4705268E+02,.4767006E+02,.5089925E+02,&
     & .4993871E+02,.5366605E+02,.5594429E+02,.5690194E+02,.5597842E+02,&
     & .5278229E+02,.4742535E+02,.4816570E+02,.5028772E+02,.4933321E+02,&
     & .5319737E+02,.5560811E+02,.5674811E+02,.5596622E+02,.5298860E+02,&
     & .4773920E+02,.4858121E+02,.4966712E+02,.4872563E+02,.5271026E+02,&
     & .5528046E+02,.5651701E+02,.5590277E+02,.5315749E+02,.4798366E+02,&
     & .4895540E+02,.4904239E+02,.4811862E+02,.5220900E+02,.5494835E+02,&
     & .5626643E+02,.5584337E+02,.5327664E+02,.4820949E+02,.4934904E+02/

      data   absa(  1:180,11)   /                                       &
     & .5140400E+02,.4521200E+02,.3907800E+02,.3297800E+02,.3121300E+02,&
     & .3568900E+02,.4223900E+02,.4910900E+02,.5355500E+02,.5145600E+02,&
     & .4526300E+02,.3908500E+02,.3297500E+02,.3103400E+02,.3523700E+02,&
     & .4175600E+02,.4853100E+02,.5287700E+02,.5141900E+02,.4522600E+02,&
     & .3901300E+02,.3293200E+02,.3082600E+02,.3487100E+02,.4137200E+02,&
     & .4807000E+02,.5231900E+02,.5131500E+02,.4512200E+02,.3892500E+02,&
     & .3288600E+02,.3064300E+02,.3456500E+02,.4105400E+02,.4769000E+02,&
     & .5185300E+02,.5113400E+02,.4495800E+02,.3878700E+02,.3281900E+02,&
     & .3047600E+02,.3433000E+02,.4079900E+02,.4738400E+02,.5151300E+02,&
     & .5957500E+02,.5234300E+02,.4516800E+02,.3815600E+02,.3654900E+02,&
     & .4079800E+02,.4843300E+02,.5632000E+02,.6249900E+02,.5965600E+02,&
     & .5241500E+02,.4519600E+02,.3815900E+02,.3635600E+02,.4023200E+02,&
     & .4780600E+02,.5560300E+02,.6169200E+02,.5963500E+02,.5239500E+02,&
     & .4514600E+02,.3811600E+02,.3613900E+02,.3980500E+02,.4733900E+02,&
     & .5504600E+02,.6106800E+02,.5952200E+02,.5228500E+02,.4505100E+02,&
     & .3806600E+02,.3588900E+02,.3947000E+02,.4698100E+02,.5462100E+02,&
     & .6059200E+02,.5930700E+02,.5209400E+02,.4488900E+02,.3796000E+02,&
     & .3567700E+02,.3921600E+02,.4668700E+02,.5426800E+02,.6019300E+02,&
     & .6880900E+02,.6039700E+02,.5202500E+02,.4416100E+02,.4270100E+02,&
     & .4666700E+02,.5552000E+02,.6449300E+02,.7266900E+02,.6892400E+02,&
     & .6049900E+02,.5209800E+02,.4413300E+02,.4252100E+02,.4596600E+02,&
     & .5473200E+02,.6359400E+02,.7170500E+02,.6895000E+02,.6053100E+02,&
     & .5210100E+02,.4410000E+02,.4228500E+02,.4541800E+02,.5409500E+02,&
     & .6288300E+02,.7091900E+02,.6885700E+02,.6043400E+02,.5201400E+02,&
     & .4404100E+02,.4203100E+02,.4500500E+02,.5356400E+02,.6229100E+02,&
     & .7024700E+02,.6866600E+02,.6026500E+02,.5187100E+02,.4394200E+02,&
     & .4184200E+02,.4469000E+02,.5313400E+02,.6180600E+02,.6968600E+02,&
     & .7931100E+02,.6956100E+02,.5988100E+02,.5127600E+02,.4976100E+02,&
     & .5319300E+02,.6339600E+02,.7365600E+02,.8365200E+02,.7951100E+02,&
     & .6973600E+02,.6001900E+02,.5117300E+02,.4959400E+02,.5246600E+02,&
     & .6246100E+02,.7257800E+02,.8242600E+02,.7956200E+02,.6978900E+02,&
     & .6004300E+02,.5107500E+02,.4934600E+02,.5184700E+02,.6163900E+02,&
     & .7163200E+02,.8136900E+02,.7951200E+02,.6973300E+02,.5998900E+02,&
     & .5097900E+02,.4908800E+02,.5136000E+02,.6094700E+02,.7083600E+02,&
     & .8046800E+02,.7932500E+02,.6956900E+02,.5985400E+02,.5087100E+02,&
     & .4888200E+02,.5099100E+02,.6036200E+02,.7016100E+02,.7970100E+02/

      data   absa(181:315,11)   /                                       &
     & .9111200E+02,.7989200E+02,.6878100E+02,.5949600E+02,.5798800E+02,&
     & .6041500E+02,.7159500E+02,.8322700E+02,.9485400E+02,.9144700E+02,&
     & .8017300E+02,.6902300E+02,.5941600E+02,.5774300E+02,.5967300E+02,&
     & .7048300E+02,.8194200E+02,.9339400E+02,.9158300E+02,.8028800E+02,&
     & .6910600E+02,.5927600E+02,.5741600E+02,.5903500E+02,.6952800E+02,&
     & .8082900E+02,.9213100E+02,.9156600E+02,.8025700E+02,.6907300E+02,&
     & .5913600E+02,.5709200E+02,.5852700E+02,.6872400E+02,.7988500E+02,&
     & .9105300E+02,.9139000E+02,.8010400E+02,.6893700E+02,.5897300E+02,&
     & .5677600E+02,.5810700E+02,.6807900E+02,.7907200E+02,.9012200E+02,&
     & .1041900E+03,.9139400E+02,.7865600E+02,.6891600E+02,.6712800E+02,&
     & .6846100E+02,.7989400E+02,.9292600E+02,.1060700E+03,.1046300E+03,&
     & .9178200E+02,.7900200E+02,.6881900E+02,.6684500E+02,.6773800E+02,&
     & .7862800E+02,.9145800E+02,.1044000E+03,.1048600E+03,.9197000E+02,&
     & .7915600E+02,.6869800E+02,.6647000E+02,.6706600E+02,.7762800E+02,&
     & .9024200E+02,.1030200E+03,.1049400E+03,.9202500E+02,.7920700E+02,&
     & .6855500E+02,.6608100E+02,.6654300E+02,.7684300E+02,.8925300E+02,&
     & .1018900E+03,.1048100E+03,.9189200E+02,.7910500E+02,.6834400E+02,&
     & .6566800E+02,.6610900E+02,.7619000E+02,.8838600E+02,.1009000E+03,&
     & .1185000E+03,.1039400E+03,.8942300E+02,.7950900E+02,.7720700E+02,&
     & .7719600E+02,.8794300E+02,.1023600E+03,.1169400E+03,.1190500E+03,&
     & .1044300E+03,.8986400E+02,.7952800E+02,.7690500E+02,.7649400E+02,&
     & .8665000E+02,.1007800E+03,.1151500E+03,.1193900E+03,.1047300E+03,&
     & .9011500E+02,.7940100E+02,.7652300E+02,.7585300E+02,.8571500E+02,&
     & .9959500E+02,.1138000E+03,.1195100E+03,.1048400E+03,.9022200E+02,&
     & .7922000E+02,.7609000E+02,.7540500E+02,.8502200E+02,.9865300E+02,&
     & .1127100E+03,.1194200E+03,.1047600E+03,.9016900E+02,.7897600E+02,&
     & .7560700E+02,.7501800E+02,.8435200E+02,.9778300E+02,.1116700E+03/

      data   absa(316:450,11)   /                                       &
     & .1338100E+03,.1173500E+03,.1009500E+03,.9122600E+02,.8818800E+02,&
     & .8637900E+02,.9582800E+02,.1115500E+03,.1275500E+03,.1344600E+03,&
     & .1179400E+03,.1014500E+03,.9127100E+02,.8794700E+02,.8576600E+02,&
     & .9465200E+02,.1100600E+03,.1258300E+03,.1349200E+03,.1183400E+03,&
     & .1018000E+03,.9123300E+02,.8761900E+02,.8522800E+02,.9381100E+02,&
     & .1089200E+03,.1245200E+03,.1350500E+03,.1184600E+03,.1019200E+03,&
     & .9109000E+02,.8715000E+02,.8488700E+02,.9311100E+02,.1079700E+03,&
     & .1233700E+03,.1350000E+03,.1184400E+03,.1019200E+03,.9079700E+02,&
     & .8662600E+02,.8457000E+02,.9242100E+02,.1071000E+03,.1222600E+03,&
     & .1495200E+03,.1311000E+03,.1131600E+03,.1041000E+03,.9992700E+02,&
     & .9607400E+02,.1037400E+03,.1206700E+03,.1380100E+03,.1502800E+03,&
     & .1317900E+03,.1136000E+03,.1041500E+03,.9987000E+02,.9558900E+02,&
     & .1025700E+03,.1191200E+03,.1362300E+03,.1508200E+03,.1322700E+03,&
     & .1139200E+03,.1040500E+03,.9957100E+02,.9518900E+02,.1018200E+03,&
     & .1180300E+03,.1349600E+03,.1510300E+03,.1324700E+03,.1140300E+03,&
     & .1038500E+03,.9912600E+02,.9498000E+02,.1012200E+03,.1172000E+03,&
     & .1338800E+03,.1509400E+03,.1324500E+03,.1140100E+03,.1034800E+03,&
     & .9855000E+02,.9474600E+02,.1005500E+03,.1163600E+03,.1327600E+03,&
     & .1652400E+03,.1448600E+03,.1259400E+03,.1178500E+03,.1122500E+03,&
     & .1065400E+03,.1120200E+03,.1296500E+03,.1482900E+03,.1661000E+03,&
     & .1456400E+03,.1263600E+03,.1178700E+03,.1123300E+03,.1063000E+03,&
     & .1109300E+03,.1281100E+03,.1465300E+03,.1665900E+03,.1460800E+03,&
     & .1265600E+03,.1176900E+03,.1121400E+03,.1060900E+03,.1103200E+03,&
     & .1271300E+03,.1453200E+03,.1668100E+03,.1463000E+03,.1265900E+03,&
     & .1173600E+03,.1116900E+03,.1059700E+03,.1097300E+03,.1262900E+03,&
     & .1441600E+03,.1667100E+03,.1462300E+03,.1263900E+03,.1168800E+03,&
     & .1111100E+03,.1057600E+03,.1090900E+03,.1254200E+03,.1430300E+03/

      data   absa(451:585,11)   /                                       &
     & .1806500E+03,.1583200E+03,.1390300E+03,.1320100E+03,.1251900E+03,&
     & .1182000E+03,.1204700E+03,.1376000E+03,.1573900E+03,.1813100E+03,&
     & .1589300E+03,.1391600E+03,.1318300E+03,.1252000E+03,.1180000E+03,&
     & .1197900E+03,.1366000E+03,.1561200E+03,.1816200E+03,.1592500E+03,&
     & .1391200E+03,.1314500E+03,.1248700E+03,.1178900E+03,.1192200E+03,&
     & .1356600E+03,.1548100E+03,.1816100E+03,.1592700E+03,.1388800E+03,&
     & .1309100E+03,.1243200E+03,.1177000E+03,.1185800E+03,.1346200E+03,&
     & .1535000E+03,.1812900E+03,.1590100E+03,.1384900E+03,.1302700E+03,&
     & .1237100E+03,.1173800E+03,.1180000E+03,.1337800E+03,.1524800E+03,&
     & .1943900E+03,.1703100E+03,.1518100E+03,.1457700E+03,.1382800E+03,&
     & .1307000E+03,.1297200E+03,.1454600E+03,.1661800E+03,.1948300E+03,&
     & .1707300E+03,.1515600E+03,.1454300E+03,.1381200E+03,.1304300E+03,&
     & .1291300E+03,.1444300E+03,.1647600E+03,.1949400E+03,.1708700E+03,&
     & .1512400E+03,.1449300E+03,.1377100E+03,.1302000E+03,.1284700E+03,&
     & .1433000E+03,.1633500E+03,.1947400E+03,.1707300E+03,.1507800E+03,&
     & .1442500E+03,.1371800E+03,.1298600E+03,.1277900E+03,.1424000E+03,&
     & .1622900E+03,.1942000E+03,.1702900E+03,.1501100E+03,.1433700E+03,&
     & .1366600E+03,.1294200E+03,.1272500E+03,.1417100E+03,.1614700E+03,&
     & .2056200E+03,.1801400E+03,.1639500E+03,.1588500E+03,.1513100E+03,&
     & .1434400E+03,.1392800E+03,.1520100E+03,.1733600E+03,.2058300E+03,&
     & .1803400E+03,.1634600E+03,.1584400E+03,.1510200E+03,.1431000E+03,&
     & .1386700E+03,.1510100E+03,.1721000E+03,.2057900E+03,.1803400E+03,&
     & .1628200E+03,.1578200E+03,.1505100E+03,.1427100E+03,.1380300E+03,&
     & .1502600E+03,.1711900E+03,.2053900E+03,.1800200E+03,.1619900E+03,&
     & .1569600E+03,.1500500E+03,.1422600E+03,.1374600E+03,.1497700E+03,&
     & .1706100E+03,.2047100E+03,.1794600E+03,.1610100E+03,.1558000E+03,&
     & .1494200E+03,.1416300E+03,.1368500E+03,.1492200E+03,.1699800E+03/

      data   absa(  1:180,12)   /                                       &
     & .6180979E+02,.5415023E+02,.4652334E+02,.3927453E+02,.3636930E+02,&
     & .4458440E+02,.5332753E+02,.6199779E+02,.6686984E+02,.6201467E+02,&
     & .5432712E+02,.4674137E+02,.3945307E+02,.3621804E+02,.4416739E+02,&
     & .5281294E+02,.6138502E+02,.6622974E+02,.6212587E+02,.5443223E+02,&
     & .4690627E+02,.3958570E+02,.3609178E+02,.4380068E+02,.5235331E+02,&
     & .6084698E+02,.6567797E+02,.6210514E+02,.5448391E+02,.4698891E+02,&
     & .3965214E+02,.3594726E+02,.4347273E+02,.5194539E+02,.6034581E+02,&
     & .6519221E+02,.6202833E+02,.5446118E+02,.4698444E+02,.3963823E+02,&
     & .3578608E+02,.4315548E+02,.5153608E+02,.5985270E+02,.6466696E+02,&
     & .7408612E+02,.6488258E+02,.5568066E+02,.4672809E+02,.4299401E+02,&
     & .5300946E+02,.6345952E+02,.7381337E+02,.8120038E+02,.7431987E+02,&
     & .6508647E+02,.5591094E+02,.4696983E+02,.4270620E+02,.5245020E+02,&
     & .6277983E+02,.7301412E+02,.8031789E+02,.7445016E+02,.6522132E+02,&
     & .5612423E+02,.4716489E+02,.4247167E+02,.5192560E+02,.6214199E+02,&
     & .7225979E+02,.7950285E+02,.7449398E+02,.6532501E+02,.5623682E+02,&
     & .4725560E+02,.4229442E+02,.5140668E+02,.6150959E+02,.7151980E+02,&
     & .7870069E+02,.7445650E+02,.6532487E+02,.5624935E+02,.4725682E+02,&
     & .4210652E+02,.5092387E+02,.6091509E+02,.7081995E+02,.7792811E+02,&
     & .8862835E+02,.7760021E+02,.6657195E+02,.5566578E+02,.5128990E+02,&
     & .6327373E+02,.7579325E+02,.8820522E+02,.9907097E+02,.8897752E+02,&
     & .7790504E+02,.6685198E+02,.5596711E+02,.5087545E+02,.6255790E+02,&
     & .7492859E+02,.8717970E+02,.9792393E+02,.8915502E+02,.7806522E+02,&
     & .6707372E+02,.5618767E+02,.5053795E+02,.6185942E+02,.7408065E+02,&
     & .8619344E+02,.9681481E+02,.8920224E+02,.7817532E+02,.6720276E+02,&
     & .5630420E+02,.5024670E+02,.6121136E+02,.7329693E+02,.8526792E+02,&
     & .9576831E+02,.8916248E+02,.7817526E+02,.6722423E+02,.5632955E+02,&
     & .4997369E+02,.6057311E+02,.7252164E+02,.8436173E+02,.9475516E+02,&
     & .1056142E+03,.9245626E+02,.7929792E+02,.6618040E+02,.6147280E+02,&
     & .7551544E+02,.9049492E+02,.1053424E+03,.1195913E+03,.1061542E+03,&
     & .9293152E+02,.7971544E+02,.6661696E+02,.6092308E+02,.7453632E+02,&
     & .8931434E+02,.1039588E+03,.1180265E+03,.1064863E+03,.9323304E+02,&
     & .8004289E+02,.6692374E+02,.6046019E+02,.7370615E+02,.8831649E+02,&
     & .1027830E+03,.1166845E+03,.1066437E+03,.9342567E+02,.8024158E+02,&
     & .6710433E+02,.6006174E+02,.7289075E+02,.8732560E+02,.1016334E+03,&
     & .1153732E+03,.1066187E+03,.9342970E+02,.8026290E+02,.6712941E+02,&
     & .5968287E+02,.7205721E+02,.8632392E+02,.1004504E+03,.1140396E+03/

      data   absa(181:315,12)   /                                       &
     & .1255319E+03,.1098733E+03,.9422394E+02,.7856981E+02,.7364879E+02,&
     & .8993455E+02,.1078090E+03,.1255219E+03,.1431747E+03,.1261993E+03,&
     & .1104607E+03,.9472274E+02,.7906163E+02,.7299959E+02,.8871931E+02,&
     & .1063433E+03,.1238040E+03,.1412246E+03,.1266478E+03,.1108818E+03,&
     & .9515186E+02,.7946770E+02,.7245329E+02,.8760934E+02,.1050151E+03,&
     & .1222508E+03,.1394428E+03,.1269276E+03,.1111655E+03,.9543063E+02,&
     & .7971950E+02,.7193392E+02,.8655075E+02,.1037357E+03,.1207616E+03,&
     & .1377350E+03,.1269774E+03,.1112303E+03,.9549436E+02,.7977777E+02,&
     & .7145906E+02,.8552064E+02,.1024909E+03,.1193094E+03,.1360690E+03,&
     & .1486316E+03,.1300858E+03,.1115387E+03,.9299465E+02,.8823269E+02,&
     & .1068891E+03,.1281501E+03,.1492401E+03,.1705961E+03,.1495263E+03,&
     & .1308668E+03,.1122085E+03,.9359013E+02,.8736883E+02,.1053484E+03,&
     & .1263026E+03,.1470657E+03,.1681173E+03,.1501396E+03,.1314414E+03,&
     & .1127520E+03,.9410033E+02,.8669616E+02,.1039401E+03,.1246072E+03,&
     & .1450931E+03,.1658588E+03,.1504832E+03,.1317688E+03,.1130645E+03,&
     & .9437770E+02,.8607984E+02,.1025821E+03,.1229795E+03,.1431843E+03,&
     & .1636739E+03,.1505906E+03,.1318738E+03,.1131670E+03,.9446367E+02,&
     & .8553451E+02,.1012614E+03,.1213902E+03,.1413315E+03,.1615525E+03,&
     & .1749535E+03,.1531197E+03,.1312770E+03,.1098627E+03,.1054310E+03,&
     & .1267118E+03,.1519465E+03,.1769451E+03,.2025356E+03,.1761867E+03,&
     & .1541979E+03,.1321991E+03,.1103506E+03,.1043907E+03,.1247740E+03,&
     & .1496095E+03,.1742366E+03,.1994180E+03,.1770334E+03,.1549823E+03,&
     & .1329299E+03,.1109139E+03,.1035393E+03,.1229433E+03,.1474132E+03,&
     & .1716583E+03,.1964792E+03,.1775830E+03,.1554757E+03,.1333697E+03,&
     & .1112711E+03,.1027772E+03,.1211079E+03,.1452057E+03,.1690988E+03,&
     & .1935313E+03,.1778130E+03,.1556757E+03,.1335484E+03,.1114211E+03,&
     & .1020906E+03,.1193757E+03,.1431352E+03,.1666861E+03,.1907553E+03/

      data   absa(316:450,12)   /                                       &
     & .2050280E+03,.1794206E+03,.1538132E+03,.1296698E+03,.1256113E+03,&
     & .1498126E+03,.1795895E+03,.2091417E+03,.2395094E+03,.2065326E+03,&
     & .1807328E+03,.1549517E+03,.1300986E+03,.1242724E+03,.1472448E+03,&
     & .1765192E+03,.2055840E+03,.2354240E+03,.2076189E+03,.1817305E+03,&
     & .1558434E+03,.1304997E+03,.1232100E+03,.1448216E+03,.1736269E+03,&
     & .2021988E+03,.2315773E+03,.2083544E+03,.1823886E+03,.1564141E+03,&
     & .1307551E+03,.1223058E+03,.1424212E+03,.1707823E+03,.1989036E+03,&
     & .2278090E+03,.2086842E+03,.1826772E+03,.1566702E+03,.1308824E+03,&
     & .1214212E+03,.1402293E+03,.1681510E+03,.1958442E+03,.2243003E+03,&
     & .2393883E+03,.2094845E+03,.1795806E+03,.1526649E+03,.1492182E+03,&
     & .1761707E+03,.2111935E+03,.2459442E+03,.2817236E+03,.2413366E+03,&
     & .2111878E+03,.1810490E+03,.1530843E+03,.1474621E+03,.1729020E+03,&
     & .2072851E+03,.2414036E+03,.2765008E+03,.2426640E+03,.2123804E+03,&
     & .1821044E+03,.1534075E+03,.1461365E+03,.1697315E+03,.2034720E+03,&
     & .2369666E+03,.2714312E+03,.2435891E+03,.2131944E+03,.1828085E+03,&
     & .1535710E+03,.1449502E+03,.1666087E+03,.1997639E+03,.2326282E+03,&
     & .2664788E+03,.2439749E+03,.2135440E+03,.1831044E+03,.1535327E+03,&
     & .1438260E+03,.1637945E+03,.1963970E+03,.2287448E+03,.2620139E+03,&
     & .2786275E+03,.2438158E+03,.2090742E+03,.1791980E+03,.1764044E+03,&
     & .2051521E+03,.2459740E+03,.2864840E+03,.3281621E+03,.2809503E+03,&
     & .2458500E+03,.2108110E+03,.1796731E+03,.1742226E+03,.2010652E+03,&
     & .2410665E+03,.2807744E+03,.3216182E+03,.2826742E+03,.2473903E+03,&
     & .2121402E+03,.1799408E+03,.1725339E+03,.1970313E+03,.2361865E+03,&
     & .2750847E+03,.3150950E+03,.2837444E+03,.2483293E+03,.2129494E+03,&
     & .1799627E+03,.1710146E+03,.1933057E+03,.2316484E+03,.2698242E+03,&
     & .3090609E+03,.2842653E+03,.2487778E+03,.2133179E+03,.1797427E+03,&
     & .1695361E+03,.1899175E+03,.2275176E+03,.2649935E+03,.3035329E+03/

      data   absa(451:585,12)   /                                       &
     & .3246164E+03,.2841571E+03,.2437591E+03,.2099009E+03,.2061338E+03,&
     & .2346079E+03,.2807869E+03,.3270860E+03,.3746416E+03,.3270160E+03,&
     & .2862343E+03,.2455753E+03,.2101750E+03,.2037891E+03,.2298452E+03,&
     & .2746768E+03,.3199854E+03,.3664519E+03,.3287071E+03,.2876768E+03,&
     & .2468055E+03,.2102385E+03,.2017300E+03,.2252550E+03,.2691398E+03,&
     & .3135254E+03,.3590802E+03,.3296547E+03,.2884880E+03,.2474892E+03,&
     & .2100045E+03,.1997651E+03,.2211596E+03,.2640071E+03,.3075507E+03,&
     & .3522449E+03,.3298874E+03,.2886932E+03,.2476393E+03,.2093907E+03,&
     & .1977676E+03,.2173310E+03,.2589645E+03,.3016760E+03,.3454882E+03,&
     & .3764185E+03,.3295629E+03,.2827412E+03,.2448362E+03,.2395515E+03,&
     & .2667523E+03,.3170130E+03,.3693427E+03,.4229547E+03,.3787898E+03,&
     & .3316408E+03,.2845544E+03,.2447605E+03,.2368444E+03,.2613950E+03,&
     & .3103293E+03,.3615554E+03,.4140351E+03,.3801676E+03,.3328413E+03,&
     & .2856113E+03,.2443688E+03,.2342033E+03,.2564579E+03,.3041029E+03,&
     & .3543252E+03,.4057425E+03,.3807335E+03,.3332994E+03,.2860069E+03,&
     & .2436761E+03,.2315207E+03,.2518766E+03,.2980292E+03,.3472163E+03,&
     & .3976049E+03,.3804795E+03,.3330315E+03,.2857763E+03,.2425900E+03,&
     & .2287390E+03,.2473661E+03,.2919475E+03,.3401341E+03,.3894698E+03,&
     & .4337936E+03,.3798110E+03,.3258724E+03,.2839162E+03,.2765829E+03,&
     & .3019915E+03,.3551500E+03,.4138455E+03,.4738475E+03,.4358679E+03,&
     & .3816444E+03,.3274735E+03,.2832262E+03,.2732493E+03,.2960468E+03,&
     & .3477195E+03,.4051747E+03,.4639479E+03,.4367543E+03,.3824308E+03,&
     & .3281687E+03,.2821996E+03,.2697997E+03,.2905362E+03,.3403748E+03,&
     & .3966217E+03,.4541170E+03,.4367336E+03,.3824073E+03,.3281688E+03,&
     & .2807593E+03,.2662071E+03,.2851390E+03,.3330291E+03,.3880528E+03,&
     & .4442749E+03,.4356281E+03,.3814241E+03,.3273328E+03,.2789760E+03,&
     & .2624596E+03,.2797763E+03,.3260499E+03,.3798827E+03,.4349494E+03/

!     -----------------------------------------------------------------
!     the array absb(1175,12) (kb(5,5,13:59,12)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 12,
!     and tells us which g-interval the absorption coefficients are for.
!     -----------------------------------------------------------------

      data   absb(  1:175, 1)   /                                       &
     & .1055400E-06,.3859600E-04,.6540200E-04,.8854000E-04,.1960100E-03,&
     & .1047900E-06,.4367200E-04,.7488900E-04,.1011800E-03,.2214600E-03,&
     & .1042900E-06,.4941800E-04,.8542900E-04,.1155700E-03,.2444800E-03,&
     & .1023200E-06,.5580200E-04,.9645900E-04,.1316700E-03,.2700100E-03,&
     & .9901300E-07,.6299600E-04,.1090000E-03,.1483400E-03,.2991700E-03,&
     & .8513600E-07,.3194300E-04,.5440200E-04,.7388700E-04,.1589500E-03,&
     & .8459700E-07,.3619600E-04,.6242900E-04,.8447900E-04,.1774900E-03,&
     & .8362900E-07,.4098200E-04,.7112600E-04,.9657300E-04,.1974000E-03,&
     & .8231000E-07,.4634400E-04,.8045800E-04,.1098400E-03,.2184700E-03,&
     & .7870900E-07,.5225200E-04,.9098900E-04,.1239400E-03,.2428100E-03,&
     & .6849700E-07,.2639300E-04,.4516400E-04,.6157000E-04,.1249300E-03,&
     & .6806200E-07,.2993600E-04,.5191900E-04,.7038700E-04,.1395300E-03,&
     & .6676600E-07,.3386300E-04,.5905600E-04,.8069300E-04,.1565300E-03,&
     & .6563600E-07,.3829400E-04,.6688500E-04,.9155500E-04,.1751000E-03,&
     & .6271100E-07,.4318900E-04,.7575600E-04,.1035500E-03,.1943900E-03,&
     & .5508300E-07,.2176100E-04,.3748400E-04,.5130100E-04,.9787500E-04,&
     & .5409500E-07,.2467000E-04,.4308400E-04,.5863900E-04,.1099600E-03,&
     & .5334000E-07,.2792500E-04,.4897600E-04,.6723400E-04,.1241500E-03,&
     & .5141200E-07,.3158800E-04,.5557600E-04,.7635200E-04,.1393900E-03,&
     & .4934100E-07,.3567700E-04,.6293700E-04,.8636300E-04,.1554300E-03,&
     & .4402200E-07,.1796300E-04,.3110200E-04,.4260400E-04,.7829000E-04,&
     & .4316700E-07,.2038200E-04,.3575400E-04,.4887500E-04,.8794100E-04,&
     & .4221000E-07,.2309100E-04,.4067400E-04,.5605900E-04,.9925700E-04,&
     & .4040200E-07,.2612500E-04,.4616200E-04,.6358600E-04,.1116600E-03,&
     & .3853200E-07,.2949400E-04,.5232700E-04,.7196000E-04,.1248000E-03,&
     & .3493700E-07,.1490300E-04,.2586800E-04,.3550300E-04,.6336800E-04,&
     & .3430800E-07,.1692300E-04,.2978100E-04,.4078900E-04,.7111500E-04,&
     & .3299500E-07,.1917100E-04,.3387200E-04,.4681000E-04,.8030200E-04,&
     & .3165400E-07,.2167900E-04,.3846800E-04,.5304800E-04,.9043100E-04,&
     & .3028200E-07,.2449100E-04,.4359600E-04,.6007800E-04,.1013300E-03,&
     & .2797100E-07,.1238500E-04,.2154900E-04,.2961800E-04,.5173200E-04,&
     & .2712800E-07,.1404200E-04,.2482600E-04,.3404700E-04,.5806400E-04,&
     & .2601800E-07,.1592000E-04,.2822100E-04,.3907200E-04,.6549500E-04,&
     & .2471700E-07,.1801700E-04,.3205600E-04,.4428500E-04,.7385900E-04,&
     & .2385900E-07,.2036000E-04,.3634100E-04,.5017000E-04,.8294400E-04/

      data   absb(176:350, 1)   /                                       &
     & .2252900E-07,.1033600E-04,.1799900E-04,.2477600E-04,.4260200E-04,&
     & .2173300E-07,.1168500E-04,.2076900E-04,.2850300E-04,.4783300E-04,&
     & .2079100E-07,.1328000E-04,.2357100E-04,.3265400E-04,.5399500E-04,&
     & .1966300E-07,.1502900E-04,.2678800E-04,.3706600E-04,.6094400E-04,&
     & .1916900E-07,.1697500E-04,.3035300E-04,.4199600E-04,.6834400E-04,&
     & .1814800E-07,.8603900E-05,.1504100E-04,.2071700E-04,.3514300E-04,&
     & .1746500E-07,.9728700E-05,.1734800E-04,.2386200E-04,.3950800E-04,&
     & .1653700E-07,.1107300E-04,.1968700E-04,.2727100E-04,.4465200E-04,&
     & .1586000E-07,.1252600E-04,.2238800E-04,.3101500E-04,.5026900E-04,&
     & .1545200E-07,.1415800E-04,.2533900E-04,.3514100E-04,.5632700E-04,&
     & .1454200E-07,.7180700E-05,.1268200E-04,.1747300E-04,.2917900E-04,&
     & .1396700E-07,.8157400E-05,.1455000E-04,.2014700E-04,.3279500E-04,&
     & .1324100E-07,.9283900E-05,.1654700E-04,.2297100E-04,.3703600E-04,&
     & .1274600E-07,.1052200E-04,.1884000E-04,.2611400E-04,.4161700E-04,&
     & .1242600E-07,.1188600E-04,.2131300E-04,.2960400E-04,.4670900E-04,&
     & .1172400E-07,.6019600E-05,.1070300E-04,.1475300E-04,.2414300E-04,&
     & .1112400E-07,.6849900E-05,.1223300E-04,.1700900E-04,.2714700E-04,&
     & .1055200E-07,.7789700E-05,.1392700E-04,.1936200E-04,.3066100E-04,&
     & .1021300E-07,.8839800E-05,.1585000E-04,.2202400E-04,.3451900E-04,&
     & .9962700E-08,.9975500E-05,.1791900E-04,.2494400E-04,.3873000E-04,&
     & .9383500E-08,.5050500E-05,.9029800E-05,.1245000E-04,.1994000E-04,&
     & .8872200E-08,.5749400E-05,.1028700E-04,.1435100E-04,.2245000E-04,&
     & .8471900E-08,.6537400E-05,.1172700E-04,.1632900E-04,.2537800E-04,&
     & .8192400E-08,.7418600E-05,.1333400E-04,.1856700E-04,.2856000E-04,&
     & .7919400E-08,.8371900E-05,.1505600E-04,.2100300E-04,.3199000E-04,&
     & .7453500E-08,.4243700E-05,.7622000E-05,.1052300E-04,.1653100E-04,&
     & .7051600E-08,.4830000E-05,.8666700E-05,.1208900E-04,.1861600E-04,&
     & .6774600E-08,.5494100E-05,.9892700E-05,.1379000E-04,.2102400E-04,&
     & .6537500E-08,.6235700E-05,.1123300E-04,.1566200E-04,.2364200E-04,&
     & .6300100E-08,.7028200E-05,.1266900E-04,.1769500E-04,.2647800E-04,&
     & .5952000E-08,.3573900E-05,.6435000E-05,.8933000E-05,.1375400E-04,&
     & .5664400E-08,.4079700E-05,.7329100E-05,.1022800E-04,.1548900E-04,&
     & .5422900E-08,.4636600E-05,.8366400E-05,.1165900E-04,.1748600E-04,&
     & .5215500E-08,.5254600E-05,.9482900E-05,.1324600E-04,.1964300E-04,&
     & .5002600E-08,.5913400E-05,.1067600E-04,.1493300E-04,.2197900E-04/

      data   absb(351:525, 1)   /                                       &
     & .4734700E-08,.3011700E-05,.5422300E-05,.7581000E-05,.1142400E-04,&
     & .4521500E-08,.3442100E-05,.6195900E-05,.8661900E-05,.1288400E-04,&
     & .4333400E-08,.3914300E-05,.7070200E-05,.9876100E-05,.1454300E-04,&
     & .4135000E-08,.4427500E-05,.8000500E-05,.1119500E-04,.1631900E-04,&
     & .3965300E-08,.4976900E-05,.8993100E-05,.1259800E-04,.1816000E-04,&
     & .3794800E-08,.2541500E-05,.4581900E-05,.6433600E-05,.9495600E-05,&
     & .3608300E-08,.2903800E-05,.5242600E-05,.7340000E-05,.1071900E-04,&
     & .3441800E-08,.3301700E-05,.5973300E-05,.8357500E-05,.1206600E-04,&
     & .3263600E-08,.3729700E-05,.6749300E-05,.9455800E-05,.1346800E-04,&
     & .3134000E-08,.4187300E-05,.7575900E-05,.1062000E-04,.1495300E-04,&
     & .3012100E-08,.2153600E-05,.3884100E-05,.5449100E-05,.7909000E-05,&
     & .2877500E-08,.2455700E-05,.4446700E-05,.6224700E-05,.8914800E-05,&
     & .2719600E-08,.2790000E-05,.5054300E-05,.7078200E-05,.9994300E-05,&
     & .2579800E-08,.3147300E-05,.5701100E-05,.7993600E-05,.1113500E-04,&
     & .2478500E-08,.3528700E-05,.6389400E-05,.8962400E-05,.1235700E-04,&
     & .2403400E-08,.1823100E-05,.3294500E-05,.4618300E-05,.6583700E-05,&
     & .2270200E-08,.2076600E-05,.3767100E-05,.5273500E-05,.7387500E-05,&
     & .2155100E-08,.2355900E-05,.4274400E-05,.5993200E-05,.8266500E-05,&
     & .2039700E-08,.2654100E-05,.4813700E-05,.6754900E-05,.9207100E-05,&
     & .1920800E-08,.2971100E-05,.5388000E-05,.7559800E-05,.1021100E-04,&
     & .1911600E-08,.1545400E-05,.2795700E-05,.3922800E-05,.5466400E-05,&
     & .1804500E-08,.1757900E-05,.3193800E-05,.4478000E-05,.6120500E-05,&
     & .1701500E-08,.1991100E-05,.3616700E-05,.5075300E-05,.6852300E-05,&
     & .1604200E-08,.2239300E-05,.4066600E-05,.5709400E-05,.7626000E-05,&
     & .1518900E-08,.2503500E-05,.4543900E-05,.6378500E-05,.8456200E-05,&
     & .1513400E-08,.1308000E-05,.2376300E-05,.3336000E-05,.4538200E-05,&
     & .1425400E-08,.1488600E-05,.2708600E-05,.3799500E-05,.5093800E-05,&
     & .1347800E-08,.1683100E-05,.3060500E-05,.4297100E-05,.5693300E-05,&
     & .1272600E-08,.1890400E-05,.3435300E-05,.4824700E-05,.6335300E-05,&
     & .1202700E-08,.2109800E-05,.3831800E-05,.5380900E-05,.7030200E-05,&
     & .1202400E-08,.1108400E-05,.2019300E-05,.2831000E-05,.3784900E-05,&
     & .1133800E-08,.1260500E-05,.2294900E-05,.3222100E-05,.4245800E-05,&
     & .1067800E-08,.1423100E-05,.2588100E-05,.3636500E-05,.4746000E-05,&
     & .1005200E-08,.1596300E-05,.2901000E-05,.4072800E-05,.5280400E-05,&
     & .9594600E-09,.1777900E-05,.3230100E-05,.4538100E-05,.5854700E-05/

      data   absb(526:700, 1)   /                                       &
     & .9575000E-09,.9366500E-06,.1706800E-05,.2395400E-05,.3161100E-05,&
     & .8999100E-09,.1063600E-05,.1936600E-05,.2720900E-05,.3544000E-05,&
     & .8494400E-09,.1199400E-05,.2180400E-05,.3065100E-05,.3960000E-05,&
     & .8086300E-09,.1343100E-05,.2440600E-05,.3428500E-05,.4404500E-05,&
     & .7669700E-09,.1493700E-05,.2713800E-05,.3815300E-05,.4873900E-05,&
     & .7693100E-09,.7814500E-06,.1424300E-05,.2000400E-05,.2608800E-05,&
     & .7255400E-09,.8865200E-06,.1614000E-05,.2268400E-05,.2925500E-05,&
     & .6867700E-09,.9987200E-06,.1815900E-05,.2553700E-05,.3267200E-05,&
     & .6498600E-09,.1117800E-05,.2031200E-05,.2854200E-05,.3630400E-05,&
     & .6217700E-09,.1242000E-05,.2256600E-05,.3173700E-05,.4018700E-05,&
     & .6224500E-09,.6423500E-06,.1170900E-05,.1644500E-05,.2121300E-05,&
     & .5869600E-09,.7287600E-06,.1326600E-05,.1864600E-05,.2379800E-05,&
     & .5569300E-09,.8208900E-06,.1492700E-05,.2099100E-05,.2660000E-05,&
     & .5293400E-09,.9187400E-06,.1669500E-05,.2345700E-05,.2957500E-05,&
     & .5058500E-09,.1021000E-05,.1854900E-05,.2608500E-05,.3275800E-05,&
     & .5115600E-09,.5187200E-06,.9455000E-06,.1328700E-05,.1708900E-05,&
     & .4825800E-09,.5890800E-06,.1072200E-05,.1507900E-05,.1918700E-05,&
     & .4569400E-09,.6642100E-06,.1207800E-05,.1699200E-05,.2145600E-05,&
     & .4350200E-09,.7440800E-06,.1352000E-05,.1900000E-05,.2387600E-05,&
     & .4151700E-09,.8275800E-06,.1503400E-05,.2114600E-05,.2647300E-05,&
     & .4209400E-09,.4183700E-06,.7626000E-06,.1071900E-05,.1375600E-05,&
     & .3970000E-09,.4755700E-06,.8657200E-06,.1218000E-05,.1546200E-05,&
     & .3756300E-09,.5368400E-06,.9761300E-06,.1373800E-05,.1728900E-05,&
     & .3578300E-09,.6019300E-06,.1093600E-05,.1537400E-05,.1925200E-05,&
     & .3410700E-09,.6701400E-06,.1217300E-05,.1712700E-05,.2136900E-05,&
     & .3463500E-09,.3373200E-06,.6149200E-06,.8644700E-06,.1105500E-05,&
     & .3265400E-09,.3838300E-06,.6986900E-06,.9835500E-06,.1244000E-05,&
     & .3090900E-09,.4337100E-06,.7886900E-06,.1110500E-05,.1392400E-05,&
     & .2942500E-09,.4868100E-06,.8844600E-06,.1243300E-05,.1552600E-05,&
     & .2800000E-09,.5425600E-06,.9855000E-06,.1386800E-05,.1723800E-05,&
     & .2873100E-09,.2684300E-06,.4891500E-06,.6881000E-06,.8786700E-06,&
     & .2708900E-09,.3060000E-06,.5569800E-06,.7843000E-06,.9907500E-06,&
     & .2560700E-09,.3462400E-06,.6296600E-06,.8867700E-06,.1110700E-05,&
     & .2437200E-09,.3892600E-06,.7073200E-06,.9943000E-06,.1240200E-05,&
     & .2317600E-09,.4346500E-06,.7894300E-06,.1110700E-05,.1378700E-05/

      data   absb(701:875, 1)   /                                       &
     & .2390300E-09,.2132300E-06,.3885500E-06,.5470100E-06,.6978100E-06,&
     & .2248700E-09,.2433200E-06,.4432800E-06,.6239700E-06,.7877400E-06,&
     & .2123600E-09,.2758100E-06,.5017000E-06,.7066400E-06,.8842700E-06,&
     & .2020400E-09,.3106600E-06,.5643500E-06,.7938400E-06,.9889900E-06,&
     & .1920900E-09,.3475300E-06,.6311900E-06,.8878700E-06,.1100700E-05,&
     & .1986800E-09,.1695000E-06,.3088500E-06,.4342600E-06,.5533600E-06,&
     & .1868100E-09,.1933800E-06,.3525000E-06,.4962200E-06,.6262800E-06,&
     & .1764900E-09,.2196300E-06,.3995000E-06,.5626900E-06,.7039600E-06,&
     & .1675300E-09,.2477800E-06,.4501300E-06,.6334200E-06,.7877000E-06,&
     & .1593000E-09,.2777200E-06,.5043900E-06,.7094500E-06,.8778300E-06,&
     & .1657700E-09,.1333200E-06,.2432300E-06,.3418400E-06,.4359400E-06,&
     & .1559700E-09,.1524500E-06,.2780100E-06,.3913000E-06,.4946000E-06,&
     & .1472100E-09,.1735400E-06,.3158400E-06,.4449900E-06,.5570500E-06,&
     & .1393600E-09,.1962500E-06,.3566100E-06,.5018900E-06,.6239800E-06,&
     & .1325500E-09,.2204400E-06,.4003400E-06,.5630800E-06,.6963400E-06,&
     & .1386100E-09,.1044500E-06,.1905800E-06,.2684100E-06,.3423300E-06,&
     & .1302700E-09,.1197400E-06,.2183600E-06,.3074400E-06,.3889700E-06,&
     & .1229000E-09,.1366200E-06,.2486400E-06,.3506500E-06,.4391600E-06,&
     & .1161100E-09,.1548400E-06,.2815500E-06,.3962700E-06,.4927300E-06,&
     & .1104200E-09,.1743100E-06,.3166600E-06,.4454000E-06,.5508700E-06,&
     & .1158100E-09,.8191500E-07,.1491500E-06,.2114000E-06,.2698700E-06,&
     & .1090600E-09,.9407700E-07,.1714300E-06,.2417300E-06,.3058600E-06,&
     & .1025400E-09,.1074500E-06,.1958200E-06,.2760600E-06,.3459300E-06,&
     & .9675700E-10,.1220500E-06,.2220100E-06,.3127100E-06,.3887600E-06,&
     & .9197900E-10,.1377200E-06,.2501500E-06,.3520800E-06,.4356100E-06,&
     & .9688900E-10,.6411600E-07,.1170800E-06,.1645500E-06,.2103600E-06,&
     & .9128100E-10,.7364100E-07,.1343600E-06,.1889500E-06,.2395100E-06,&
     & .8583000E-10,.8416100E-07,.1536000E-06,.2164300E-06,.2717600E-06,&
     & .8104800E-10,.9586700E-07,.1744700E-06,.2458200E-06,.3061300E-06,&
     & .7671200E-10,.1084400E-06,.1970600E-06,.2774300E-06,.3432900E-06,&
     & .8151600E-10,.4969800E-07,.9059600E-07,.1272700E-06,.1631300E-06,&
     & .7660900E-10,.5713200E-07,.1043300E-06,.1469400E-06,.1867200E-06,&
     & .7197500E-10,.6553400E-07,.1195500E-06,.1685000E-06,.2121500E-06,&
     & .6790800E-10,.7482900E-07,.1361900E-06,.1920900E-06,.2395900E-06,&
     & .6411700E-10,.8487200E-07,.1543300E-06,.2173300E-06,.2691500E-06/

      data   absb(876:1050, 1)   /                                      &
     & .6842900E-10,.3871500E-07,.6985400E-07,.9837700E-07,.1261900E-06,&
     & .6423600E-10,.4437500E-07,.8089500E-07,.1147100E-06,.1460400E-06,&
     & .6051000E-10,.5102500E-07,.9297700E-07,.1312300E-06,.1654200E-06,&
     & .5685100E-10,.5834500E-07,.1063300E-06,.1499100E-06,.1874500E-06,&
     & .5367400E-10,.6634600E-07,.1207000E-06,.1700100E-06,.2109500E-06,&
     & .5738700E-10,.2980000E-07,.5381700E-07,.7590600E-07,.9771000E-07,&
     & .5396000E-10,.3450000E-07,.6320500E-07,.8865500E-07,.1131000E-06,&
     & .5074000E-10,.3968400E-07,.7250200E-07,.1019300E-06,.1289200E-06,&
     & .4771000E-10,.4541800E-07,.8294100E-07,.1169200E-06,.1464900E-06,&
     & .4505000E-10,.5179700E-07,.9432000E-07,.1329600E-06,.1652600E-06,&
     & .4801000E-10,.2299900E-07,.4168200E-07,.5897500E-07,.7589800E-07,&
     & .4528200E-10,.2691700E-07,.4892900E-07,.6876500E-07,.8798300E-07,&
     & .4253100E-10,.3087900E-07,.5640600E-07,.7959300E-07,.1010900E-06,&
     & .4000200E-10,.3547000E-07,.6470600E-07,.9131200E-07,.1147300E-06,&
     & .3767400E-10,.4054200E-07,.7383400E-07,.1041200E-06,.1296700E-06,&
     & .4009400E-10,.1781000E-07,.3241200E-07,.4598500E-07,.5904100E-07,&
     & .3790100E-10,.2101300E-07,.3792100E-07,.5347900E-07,.6847900E-07,&
     & .3558400E-10,.2410000E-07,.4398800E-07,.6240900E-07,.7931100E-07,&
     & .3351600E-10,.2773700E-07,.5054400E-07,.7143600E-07,.8991200E-07,&
     & .3150300E-10,.3173800E-07,.5788900E-07,.8159100E-07,.1019500E-06,&
     & .3346200E-10,.1382900E-07,.2522700E-07,.3549800E-07,.4579300E-07,&
     & .3170300E-10,.1627200E-07,.2939300E-07,.4147600E-07,.5334000E-07,&
     & .2985600E-10,.1883600E-07,.3453700E-07,.4849000E-07,.6179100E-07,&
     & .2803800E-10,.2168400E-07,.3957800E-07,.5575100E-07,.7042200E-07,&
     & .2637500E-10,.2482400E-07,.4538900E-07,.6389800E-07,.8007100E-07,&
     & .2785900E-10,.1075700E-07,.1943600E-07,.2736800E-07,.3540100E-07,&
     & .2648200E-10,.1257500E-07,.2278600E-07,.3227500E-07,.4150400E-07,&
     & .2501300E-10,.1471800E-07,.2678600E-07,.3765700E-07,.4815000E-07,&
     & .2348100E-10,.1689000E-07,.3087500E-07,.4356100E-07,.5528400E-07,&
     & .2210300E-10,.1940500E-07,.3543500E-07,.5004600E-07,.6282400E-07,&
     & .2326200E-10,.8353400E-08,.1503300E-07,.2114200E-07,.2749100E-07,&
     & .2207500E-10,.9782100E-08,.1779100E-07,.2529800E-07,.3244700E-07,&
     & .2088700E-10,.1154100E-07,.2085200E-07,.2939200E-07,.3764300E-07,&
     & .1962900E-10,.1322800E-07,.2416400E-07,.3433000E-07,.4362200E-07,&
     & .1847500E-10,.1523300E-07,.2777900E-07,.3930400E-07,.4943200E-07/

      data   absb(1051:1175, 1)   /                                     &
     & .1936000E-10,.6488800E-08,.1164400E-07,.1638700E-07,.2134000E-07,&
     & .1837200E-10,.7636700E-08,.1394700E-07,.1967200E-07,.2540100E-07,&
     & .1742100E-10,.9008300E-08,.1627500E-07,.2297100E-07,.2952200E-07,&
     & .1641300E-10,.1040200E-07,.1908100E-07,.2685400E-07,.3423300E-07,&
     & .1541700E-10,.1198400E-07,.2183500E-07,.3085600E-07,.3895400E-07,&
     & .1609400E-10,.5017400E-08,.9028300E-08,.1271800E-07,.1657600E-07,&
     & .1526500E-10,.5974700E-08,.1085000E-07,.1529700E-07,.1978300E-07,&
     & .1451500E-10,.7014200E-08,.1269900E-07,.1796000E-07,.2317100E-07,&
     & .1371500E-10,.8162600E-08,.1494000E-07,.2099400E-07,.2686500E-07,&
     & .1288000E-10,.9405400E-08,.1720600E-07,.2422100E-07,.3073900E-07,&
     & .1336800E-10,.3883800E-08,.6999100E-08,.9837200E-08,.1282100E-07,&
     & .1274300E-10,.4675100E-08,.8424300E-08,.1185600E-07,.1540600E-07,&
     & .1208700E-10,.5464200E-08,.9929500E-08,.1406900E-07,.1814400E-07,&
     & .1144200E-10,.6446100E-08,.1165700E-07,.1640700E-07,.2107200E-07,&
     & .1075900E-10,.7371100E-08,.1347800E-07,.1910600E-07,.2431400E-07,&
     & .1108100E-10,.3014800E-08,.5416500E-08,.7642100E-08,.9949600E-08,&
     & .1059800E-10,.3633700E-08,.6537500E-08,.9204100E-08,.1202500E-07,&
     & .1005200E-10,.4271200E-08,.7784100E-08,.1105300E-07,.1430600E-07,&
     & .9530800E-11,.5042900E-08,.9113300E-08,.1286500E-07,.1655400E-07,&
     & .8979500E-11,.5797900E-08,.1061500E-07,.1505100E-07,.1921600E-07,&
     & .9119400E-11,.2421400E-08,.4349600E-08,.6153100E-08,.8013600E-08,&
     & .8728700E-11,.2923200E-08,.5253500E-08,.7396600E-08,.9683000E-08,&
     & .8276500E-11,.3441200E-08,.6278800E-08,.8899200E-08,.1153400E-07,&
     & .7860500E-11,.4063200E-08,.7345400E-08,.1038200E-07,.1337200E-07,&
     & .7412700E-11,.4680100E-08,.8591000E-08,.1213700E-07,.1553000E-07/

      data   absb(  1:175, 2)   /                                       &
     & .1757600E-05,.2251800E-03,.3639800E-03,.4650500E-03,.9792000E-03,&
     & .1952200E-05,.2568800E-03,.4272400E-03,.5429300E-03,.1126100E-02,&
     & .2113400E-05,.2938200E-03,.4977800E-03,.6318600E-03,.1296100E-02,&
     & .2268000E-05,.3358500E-03,.5709500E-03,.7314900E-03,.1485700E-02,&
     & .2393900E-05,.3832600E-03,.6468800E-03,.8412000E-03,.1681500E-02,&
     & .1386600E-05,.1881600E-03,.3054100E-03,.3908200E-03,.8061400E-03,&
     & .1482500E-05,.2152500E-03,.3590600E-03,.4577200E-03,.9297600E-03,&
     & .1642800E-05,.2466400E-03,.4189100E-03,.5334100E-03,.1068900E-02,&
     & .1780700E-05,.2821800E-03,.4802100E-03,.6186100E-03,.1223600E-02,&
     & .1868500E-05,.3220400E-03,.5438500E-03,.7122300E-03,.1384400E-02,&
     & .1048300E-05,.1569900E-03,.2557100E-03,.3280300E-03,.6587300E-03,&
     & .1131000E-05,.1801600E-03,.3007100E-03,.3853400E-03,.7591600E-03,&
     & .1233000E-05,.2067200E-03,.3518100E-03,.4494000E-03,.8704100E-03,&
     & .1342600E-05,.2365000E-03,.4029200E-03,.5224600E-03,.9944500E-03,&
     & .1451400E-05,.2697900E-03,.4568400E-03,.6019500E-03,.1117300E-02,&
     & .8066300E-06,.1309800E-03,.2138200E-03,.2753900E-03,.5372700E-03,&
     & .8574700E-06,.1505800E-03,.2520100E-03,.3240600E-03,.6157300E-03,&
     & .9243500E-06,.1728300E-03,.2954200E-03,.3787900E-03,.7014000E-03,&
     & .9974700E-06,.1979300E-03,.3380700E-03,.4407600E-03,.7946800E-03,&
     & .1081900E-05,.2258200E-03,.3835400E-03,.5066300E-03,.8898000E-03,&
     & .6177200E-06,.1094100E-03,.1792600E-03,.2314400E-03,.4355400E-03,&
     & .6571800E-06,.1257600E-03,.2117400E-03,.2727500E-03,.4975000E-03,&
     & .6994900E-06,.1446100E-03,.2485200E-03,.3193700E-03,.5646200E-03,&
     & .7538700E-06,.1658000E-03,.2837400E-03,.3721300E-03,.6350700E-03,&
     & .7971500E-06,.1892600E-03,.3223600E-03,.4258800E-03,.7072800E-03,&
     & .4718800E-06,.9195200E-04,.1510000E-03,.1949500E-03,.3554300E-03,&
     & .5093700E-06,.1051300E-03,.1785200E-03,.2299900E-03,.4025000E-03,&
     & .5420600E-06,.1213000E-03,.2095900E-03,.2697900E-03,.4566200E-03,&
     & .5732800E-06,.1393600E-03,.2381500E-03,.3145100E-03,.5111300E-03,&
     & .6107700E-06,.1591600E-03,.2707500E-03,.3596700E-03,.5707600E-03,&
     & .3600500E-06,.7698400E-04,.1274300E-03,.1642200E-03,.2888500E-03,&
     & .3898800E-06,.8802900E-04,.1506700E-03,.1939600E-03,.3285400E-03,&
     & .4198900E-06,.1017200E-03,.1753900E-03,.2278400E-03,.3704500E-03,&
     & .4409200E-06,.1172700E-03,.2000400E-03,.2659200E-03,.4159800E-03,&
     & .4688300E-06,.1338600E-03,.2276900E-03,.3028500E-03,.4655000E-03/

      data   absb(176:350, 2)   /                                       &
     & .2790800E-06,.6445000E-04,.1079500E-03,.1388900E-03,.2387800E-03,&
     & .3093400E-06,.7418700E-04,.1277300E-03,.1641700E-03,.2701700E-03,&
     & .3308600E-06,.8580100E-04,.1478200E-03,.1930700E-03,.3050800E-03,&
     & .3505800E-06,.9886100E-04,.1686800E-03,.2250800E-03,.3424900E-03,&
     & .3706900E-06,.1130600E-03,.1918800E-03,.2560100E-03,.3851900E-03,&
     & .2241800E-06,.5409500E-04,.9141400E-04,.1175700E-03,.1969700E-03,&
     & .2455900E-06,.6256700E-04,.1081900E-03,.1390800E-03,.2228400E-03,&
     & .2616900E-06,.7238300E-04,.1248100E-03,.1636800E-03,.2520500E-03,&
     & .2788500E-06,.8344600E-04,.1425800E-03,.1891000E-03,.2838600E-03,&
     & .2934400E-06,.9554000E-04,.1619800E-03,.2160900E-03,.3194700E-03,&
     & .1794100E-06,.4583000E-04,.7817600E-04,.1006100E-03,.1631300E-03,&
     & .1939700E-06,.5313900E-04,.9255600E-04,.1190100E-03,.1848200E-03,&
     & .2070700E-06,.6159300E-04,.1057900E-03,.1400800E-03,.2096500E-03,&
     & .2207100E-06,.7101600E-04,.1210000E-03,.1612800E-03,.2366100E-03,&
     & .2314800E-06,.8128600E-04,.1379500E-03,.1840100E-03,.2658000E-03,&
     & .1429100E-06,.3882500E-04,.6686600E-04,.8614000E-04,.1351100E-03,&
     & .1517900E-06,.4513500E-04,.7845000E-04,.1019400E-03,.1536900E-03,&
     & .1632600E-06,.5239900E-04,.8991900E-04,.1200100E-03,.1744100E-03,&
     & .1730700E-06,.6044000E-04,.1029300E-03,.1373500E-03,.1966800E-03,&
     & .1813700E-06,.6918900E-04,.1175900E-03,.1565100E-03,.2199800E-03,&
     & .1124500E-06,.3286200E-04,.5716600E-04,.7382800E-04,.1118500E-03,&
     & .1195800E-06,.3828100E-04,.6663500E-04,.8739000E-04,.1274000E-03,&
     & .1270200E-06,.4451900E-04,.7660600E-04,.1019000E-03,.1441800E-03,&
     & .1346700E-06,.5137300E-04,.8762200E-04,.1170000E-03,.1620500E-03,&
     & .1412000E-06,.5887400E-04,.1001900E-03,.1334800E-03,.1818800E-03,&
     & .8824200E-07,.2795000E-04,.4901000E-04,.6343400E-04,.9287700E-04,&
     & .9438300E-07,.3261000E-04,.5669000E-04,.7511400E-04,.1053100E-03,&
     & .9971200E-07,.3792900E-04,.6520400E-04,.8711300E-04,.1193100E-03,&
     & .1047300E-06,.4377800E-04,.7480500E-04,.9995000E-04,.1342700E-03,&
     & .1098800E-06,.5022900E-04,.8559800E-04,.1140600E-03,.1505000E-03,&
     & .6969600E-07,.2389500E-04,.4216600E-04,.5473300E-04,.7714600E-04,&
     & .7439000E-07,.2792100E-04,.4838700E-04,.6481700E-04,.8760700E-04,&
     & .7885600E-07,.3249700E-04,.5569200E-04,.7469100E-04,.9938700E-04,&
     & .8325300E-07,.3749100E-04,.6413000E-04,.8560900E-04,.1119300E-03,&
     & .8767500E-07,.4301100E-04,.7339800E-04,.9782800E-04,.1257400E-03/

      data   absb(351:525, 2)   /                                       &
     & .5461900E-07,.2046900E-04,.3586900E-04,.4725900E-04,.6415100E-04,&
     & .5865700E-07,.2392500E-04,.4146800E-04,.5549900E-04,.7291100E-04,&
     & .6253200E-07,.2783900E-04,.4773500E-04,.6398500E-04,.8295500E-04,&
     & .6618400E-07,.3214100E-04,.5500100E-04,.7345600E-04,.9356700E-04,&
     & .6930500E-07,.3660500E-04,.6296800E-04,.8393800E-04,.1051700E-03,&
     & .4334800E-07,.1747600E-04,.3069200E-04,.4080300E-04,.5327100E-04,&
     & .4634800E-07,.2049600E-04,.3549800E-04,.4762900E-04,.6080900E-04,&
     & .4950700E-07,.2385700E-04,.4097400E-04,.5494700E-04,.6922200E-04,&
     & .5241900E-07,.2756000E-04,.4720500E-04,.6304400E-04,.7812300E-04,&
     & .5484200E-07,.3109400E-04,.5402800E-04,.7211600E-04,.8783400E-04,&
     & .3448000E-07,.1502700E-04,.2626800E-04,.3535800E-04,.4471100E-04,&
     & .3698500E-07,.1763800E-04,.3040500E-04,.4093800E-04,.5105400E-04,&
     & .3947600E-07,.2051200E-04,.3523700E-04,.4720700E-04,.5803400E-04,&
     & .4165000E-07,.2369400E-04,.4058600E-04,.5428600E-04,.6570800E-04,&
     & .4369300E-07,.2643800E-04,.4646700E-04,.6203500E-04,.7387200E-04,&
     & .2740200E-07,.1293200E-04,.2258500E-04,.3036500E-04,.3747100E-04,&
     & .2937700E-07,.1518300E-04,.2615800E-04,.3519500E-04,.4286700E-04,&
     & .3135300E-07,.1765800E-04,.3034300E-04,.4060600E-04,.4885800E-04,&
     & .3305600E-07,.2028600E-04,.3492400E-04,.4673500E-04,.5532900E-04,&
     & .3467300E-07,.2245100E-04,.3971700E-04,.5332900E-04,.6215100E-04,&
     & .2171600E-07,.1115600E-04,.1942700E-04,.2616900E-04,.3151200E-04,&
     & .2331300E-07,.1309100E-04,.2256800E-04,.3037700E-04,.3621600E-04,&
     & .2480300E-07,.1522100E-04,.2615700E-04,.3505000E-04,.4125600E-04,&
     & .2626000E-07,.1727700E-04,.3009100E-04,.4028800E-04,.4669300E-04,&
     & .2733400E-07,.1915900E-04,.3377300E-04,.4589200E-04,.5231400E-04,&
     & .1712300E-07,.9645300E-05,.1668800E-04,.2252800E-04,.2664800E-04,&
     & .1848900E-07,.1130500E-04,.1946400E-04,.2613700E-04,.3061700E-04,&
     & .1962100E-07,.1314300E-04,.2255400E-04,.3023800E-04,.3487600E-04,&
     & .2065300E-07,.1469800E-04,.2592800E-04,.3471200E-04,.3939800E-04,&
     & .2150000E-07,.1631500E-04,.2865100E-04,.3930200E-04,.4409700E-04,&
     & .1341400E-07,.8335500E-05,.1440900E-04,.1946100E-04,.2258200E-04,&
     & .1445600E-07,.9764700E-05,.1681800E-04,.2257800E-04,.2592300E-04,&
     & .1541600E-07,.1125000E-04,.1945100E-04,.2611900E-04,.2948200E-04,&
     & .1618400E-07,.1252000E-04,.2206400E-04,.2991100E-04,.3323000E-04,&
     & .1686100E-07,.1381000E-04,.2433200E-04,.3336300E-04,.3713500E-04/

      data   absb(526:700, 2)   /                                       &
     & .1057500E-07,.7178600E-05,.1240700E-04,.1671900E-04,.1915800E-04,&
     & .1132800E-07,.8409300E-05,.1445700E-04,.1943300E-04,.2194000E-04,&
     & .1204900E-07,.9586000E-05,.1671800E-04,.2242500E-04,.2493200E-04,&
     & .1268900E-07,.1065500E-04,.1877100E-04,.2564700E-04,.2806500E-04,&
     & .1324100E-07,.1170300E-04,.2062800E-04,.2818400E-04,.3132300E-04,&
     & .8248900E-08,.6085000E-05,.1050500E-04,.1414000E-04,.1600400E-04,&
     & .8853200E-08,.7130600E-05,.1225000E-04,.1646300E-04,.1833300E-04,&
     & .9346800E-08,.8026900E-05,.1415800E-04,.1898900E-04,.2083500E-04,&
     & .9847800E-08,.8945000E-05,.1569100E-04,.2157600E-04,.2342800E-04,&
     & .1033200E-07,.9835400E-05,.1729300E-04,.2361800E-04,.2612800E-04,&
     & .6395100E-08,.5054500E-05,.8737800E-05,.1177800E-04,.1317600E-04,&
     & .6825200E-08,.5934800E-05,.1019300E-04,.1373000E-04,.1510400E-04,&
     & .7244500E-08,.6648000E-05,.1178300E-04,.1583200E-04,.1716000E-04,&
     & .7587100E-08,.7396000E-05,.1300500E-04,.1786600E-04,.1929500E-04,&
     & .7907500E-08,.8155800E-05,.1430600E-04,.1961700E-04,.2153200E-04,&
     & .5057400E-08,.4121500E-05,.7124900E-05,.9592300E-05,.1068600E-04,&
     & .5396700E-08,.4851800E-05,.8328600E-05,.1120600E-04,.1226900E-04,&
     & .5751700E-08,.5441100E-05,.9638300E-05,.1294200E-04,.1395800E-04,&
     & .6027300E-08,.6059000E-05,.1064400E-04,.1461900E-04,.1571400E-04,&
     & .6277500E-08,.6690000E-05,.1172300E-04,.1607700E-04,.1756000E-04,&
     & .3994800E-08,.3353500E-05,.5797600E-05,.7798900E-05,.8654800E-05,&
     & .4264200E-08,.3958400E-05,.6793200E-05,.9129000E-05,.9950700E-05,&
     & .4552900E-08,.4447800E-05,.7874600E-05,.1056300E-04,.1133800E-04,&
     & .4791100E-08,.4959200E-05,.8701900E-05,.1195000E-04,.1278300E-04,&
     & .4995900E-08,.5482000E-05,.9596400E-05,.1315500E-04,.1430800E-04,&
     & .3160700E-08,.2726000E-05,.4714300E-05,.6338100E-05,.7009400E-05,&
     & .3377100E-08,.3226300E-05,.5535200E-05,.7432200E-05,.8069500E-05,&
     & .3590700E-08,.3631600E-05,.6425600E-05,.8613300E-05,.9201900E-05,&
     & .3792000E-08,.4055500E-05,.7112800E-05,.9767700E-05,.1039200E-04,&
     & .3968700E-08,.4488200E-05,.7849900E-05,.1075300E-04,.1164700E-04,&
     & .2513700E-08,.2178200E-05,.3770000E-05,.5063600E-05,.5587200E-05,&
     & .2695300E-08,.2587200E-05,.4441100E-05,.5953800E-05,.6469300E-05,&
     & .2873800E-08,.2941500E-05,.5169700E-05,.6921700E-05,.7380600E-05,&
     & .3036500E-08,.3288700E-05,.5757400E-05,.7911500E-05,.8354700E-05,&
     & .3182700E-08,.3639100E-05,.6373000E-05,.8711400E-05,.9385400E-05/

      data   absb(701:875, 2)   /                                       &
     & .2004900E-08,.1735000E-05,.3008900E-05,.4034200E-05,.4443700E-05,&
     & .2152200E-08,.2068500E-05,.3553200E-05,.4754200E-05,.5157700E-05,&
     & .2297800E-08,.2371600E-05,.4149000E-05,.5545900E-05,.5902900E-05,&
     & .2427700E-08,.2660800E-05,.4658500E-05,.6392800E-05,.6701500E-05,&
     & .2559200E-08,.2943400E-05,.5161400E-05,.7037600E-05,.7547200E-05,&
     & .1600700E-08,.1379300E-05,.2393900E-05,.3220300E-05,.3534700E-05,&
     & .1718200E-08,.1650900E-05,.2838800E-05,.3795000E-05,.4106900E-05,&
     & .1836400E-08,.1905800E-05,.3325000E-05,.4438600E-05,.4716900E-05,&
     & .1945900E-08,.2143600E-05,.3767600E-05,.5138900E-05,.5371000E-05,&
     & .2055500E-08,.2378100E-05,.4172200E-05,.5691500E-05,.6061300E-05,&
     & .1276300E-08,.1085500E-05,.1884200E-05,.2535500E-05,.2789400E-05,&
     & .1369300E-08,.1303800E-05,.2243400E-05,.2999100E-05,.3245200E-05,&
     & .1466900E-08,.1522700E-05,.2639400E-05,.3519500E-05,.3740300E-05,&
     & .1562100E-08,.1714500E-05,.3018800E-05,.4088100E-05,.4275700E-05,&
     & .1646700E-08,.1909300E-05,.3348100E-05,.4576800E-05,.4835700E-05,&
     & .1016400E-08,.8490800E-06,.1474100E-05,.1986400E-05,.2174400E-05,&
     & .1092600E-08,.1023200E-05,.1764200E-05,.2353600E-05,.2556200E-05,&
     & .1171700E-08,.1215600E-05,.2083600E-05,.2775000E-05,.2954700E-05,&
     & .1250200E-08,.1367100E-05,.2414900E-05,.3237400E-05,.3385600E-05,&
     & .1320400E-08,.1529000E-05,.2680100E-05,.3663100E-05,.3843000E-05,&
     & .8096700E-09,.6624700E-06,.1152500E-05,.1555700E-05,.1691100E-05,&
     & .8709100E-09,.8011400E-06,.1383800E-05,.1845700E-05,.2007100E-05,&
     & .9347100E-09,.9606300E-06,.1641400E-05,.2186200E-05,.2335700E-05,&
     & .9999600E-09,.1089800E-05,.1928700E-05,.2558700E-05,.2676500E-05,&
     & .1059400E-08,.1225500E-05,.2143700E-05,.2923900E-05,.3049100E-05,&
     & .6443800E-09,.5140800E-06,.8969200E-06,.1217600E-05,.1309400E-05,&
     & .6933800E-09,.6233500E-06,.1078300E-05,.1440800E-05,.1569700E-05,&
     & .7424900E-09,.7519700E-06,.1285900E-05,.1711600E-05,.1828600E-05,&
     & .7971000E-09,.8678500E-06,.1518200E-05,.2011700E-05,.2107200E-05,&
     & .8496600E-09,.9792300E-06,.1715600E-05,.2335400E-05,.2409600E-05,&
     & .5123600E-09,.3948300E-06,.6910000E-06,.9487100E-06,.1005000E-05,&
     & .5507500E-09,.4807500E-06,.8332600E-06,.1117100E-05,.1216900E-05,&
     & .5915300E-09,.5827900E-06,.9992200E-06,.1330700E-05,.1422700E-05,&
     & .6348500E-09,.6834700E-06,.1184500E-05,.1569600E-05,.1647400E-05,&
     & .6783300E-09,.7735900E-06,.1359300E-05,.1832900E-05,.1893300E-05/

      data   absb(876:1050, 2)   /                                      &
     & .4074900E-09,.3016200E-06,.5340800E-06,.7277700E-06,.7676800E-06,&
     & .4368200E-09,.3697500E-06,.6420100E-06,.8617400E-06,.9333400E-06,&
     & .4698100E-09,.4501500E-06,.7746000E-06,.1030600E-05,.1109200E-05,&
     & .5030200E-09,.5391200E-06,.9217100E-06,.1222500E-05,.1287400E-05,&
     & .5399900E-09,.6101300E-06,.1079000E-05,.1433400E-05,.1482800E-05,&
     & .3228800E-09,.2311500E-06,.4162500E-06,.5531800E-06,.5847600E-06,&
     & .3473100E-09,.2835500E-06,.4949300E-06,.6685000E-06,.7160000E-06,&
     & .3731000E-09,.3468200E-06,.5982000E-06,.7961700E-06,.8598500E-06,&
     & .3997500E-09,.4203400E-06,.7152700E-06,.9504600E-06,.1004100E-05,&
     & .4297000E-09,.4834100E-06,.8482900E-06,.1119100E-05,.1158200E-05,&
     & .2564600E-09,.1774300E-06,.3242300E-06,.4217500E-06,.4475400E-06,&
     & .2766900E-09,.2185600E-06,.3822000E-06,.5210200E-06,.5514100E-06,&
     & .2965400E-09,.2680800E-06,.4637500E-06,.6192100E-06,.6702500E-06,&
     & .3187200E-09,.3266200E-06,.5568700E-06,.7398500E-06,.7829100E-06,&
     & .3419800E-09,.3813600E-06,.6638500E-06,.8759100E-06,.9077100E-06,&
     & .2047200E-09,.1369400E-06,.2472600E-06,.3221600E-06,.3422900E-06,&
     & .2205400E-09,.1684200E-06,.2964600E-06,.4063500E-06,.4240200E-06,&
     & .2365300E-09,.2073100E-06,.3591300E-06,.4802700E-06,.5171400E-06,&
     & .2540200E-09,.2537900E-06,.4337700E-06,.5768900E-06,.6122800E-06,&
     & .2723300E-09,.3020200E-06,.5194000E-06,.6862600E-06,.7114600E-06,&
     & .1642800E-09,.1068700E-06,.1880300E-06,.2451700E-06,.2609000E-06,&
     & .1752000E-09,.1297800E-06,.2302500E-06,.3110600E-06,.3251200E-06,&
     & .1886500E-09,.1599400E-06,.2772600E-06,.3726100E-06,.3988800E-06,&
     & .2026500E-09,.1967000E-06,.3370800E-06,.4484500E-06,.4777900E-06,&
     & .2171100E-09,.2373900E-06,.4052800E-06,.5366800E-06,.5582800E-06,&
     & .1320600E-09,.8267700E-07,.1428400E-06,.1864000E-06,.1989800E-06,&
     & .1393800E-09,.9959700E-07,.1809500E-06,.2372500E-06,.2491100E-06,&
     & .1505600E-09,.1230900E-06,.2149400E-06,.2902600E-06,.3072000E-06,&
     & .1613600E-09,.1520500E-06,.2614700E-06,.3477400E-06,.3733100E-06,&
     & .1731800E-09,.1854700E-06,.3156600E-06,.4185700E-06,.4364900E-06,&
     & .1062800E-09,.6352900E-07,.1093800E-06,.1428500E-06,.1528100E-06,&
     & .1118100E-09,.7706800E-07,.1403600E-06,.1820100E-06,.1916300E-06,&
     & .1199200E-09,.9538700E-07,.1669200E-06,.2287800E-06,.2382400E-06,&
     & .1290000E-09,.1180900E-06,.2040600E-06,.2716300E-06,.2907800E-06,&
     & .1382800E-09,.1451000E-06,.2467800E-06,.3275300E-06,.3421700E-06/

      data   absb(1051:1175, 2)   /                                     &
     & .8566400E-10,.4844400E-07,.8393200E-07,.1095000E-06,.1178500E-06,&
     & .8999900E-10,.5988500E-07,.1077600E-06,.1402300E-06,.1476300E-06,&
     & .9583600E-10,.7394100E-07,.1302300E-06,.1787600E-06,.1842600E-06,&
     & .1032300E-09,.9179100E-07,.1588200E-06,.2120200E-06,.2261100E-06,&
     & .1108100E-09,.1133600E-06,.1931600E-06,.2564700E-06,.2691800E-06,&
     & .6909300E-10,.3683200E-07,.6412300E-07,.8403000E-07,.9097400E-07,&
     & .7243300E-10,.4708400E-07,.8261300E-07,.1075800E-06,.1134600E-06,&
     & .7661600E-10,.5738800E-07,.1018200E-06,.1376800E-06,.1422400E-06,&
     & .8270900E-10,.7129400E-07,.1235500E-06,.1653000E-06,.1754900E-06,&
     & .8863600E-10,.8844500E-07,.1509400E-06,.2006700E-06,.2111800E-06,&
     & .5598000E-10,.2801800E-07,.4886700E-07,.6485700E-07,.7005500E-07,&
     & .5839700E-10,.3662000E-07,.6323200E-07,.8242800E-07,.8745700E-07,&
     & .6150900E-10,.4444600E-07,.8051200E-07,.1058000E-06,.1098800E-06,&
     & .6601500E-10,.5522200E-07,.9640100E-07,.1293500E-06,.1361400E-06,&
     & .7097600E-10,.6882100E-07,.1178000E-06,.1563200E-06,.1658600E-06,&
     & .4572200E-10,.2150600E-07,.3732600E-07,.5033300E-07,.5485500E-07,&
     & .4709000E-10,.2824900E-07,.4861200E-07,.6344200E-07,.6755000E-07,&
     & .4955000E-10,.3443000E-07,.6276900E-07,.8150900E-07,.8491700E-07,&
     & .5281700E-10,.4295800E-07,.7517300E-07,.1020400E-06,.1060300E-06,&
     & .5686400E-10,.5363700E-07,.9204400E-07,.1222900E-06,.1301100E-06,&
     & .3749600E-10,.1725500E-07,.2996700E-07,.4039400E-07,.4405900E-07,&
     & .3836700E-10,.2278200E-07,.3914400E-07,.5101900E-07,.5440400E-07,&
     & .4034600E-10,.2784200E-07,.5069200E-07,.6571600E-07,.6844000E-07,&
     & .4290800E-10,.3476100E-07,.6084600E-07,.8283400E-07,.8571300E-07,&
     & .4621300E-10,.4346700E-07,.7462600E-07,.9914400E-07,.1051800E-06/

      data   absb(  1:175, 3)   /                                       &
     & .9118000E-05,.1275400E-02,.1990000E-02,.2444000E-02,.4057600E-02,&
     & .9503700E-05,.1415000E-02,.2214500E-02,.2809400E-02,.4437500E-02,&
     & .9975700E-05,.1562600E-02,.2432000E-02,.3137500E-02,.4809700E-02,&
     & .1050800E-04,.1699700E-02,.2692600E-02,.3448300E-02,.5155400E-02,&
     & .1103300E-04,.1837300E-02,.2968400E-02,.3786100E-02,.5489600E-02,&
     & .7320900E-05,.1072700E-02,.1678700E-02,.2088100E-02,.3304100E-02,&
     & .7679500E-05,.1193700E-02,.1868300E-02,.2361600E-02,.3624600E-02,&
     & .8041700E-05,.1317700E-02,.2056700E-02,.2646500E-02,.3943800E-02,&
     & .8429600E-05,.1431700E-02,.2282600E-02,.2902200E-02,.4236800E-02,&
     & .8895000E-05,.1549500E-02,.2508400E-02,.3190100E-02,.4499200E-02,&
     & .5827500E-05,.8989500E-03,.1416100E-02,.1776000E-02,.2650800E-02,&
     & .6116600E-05,.1002500E-02,.1570600E-02,.1989300E-02,.2921900E-02,&
     & .6434000E-05,.1105500E-02,.1737500E-02,.2223700E-02,.3171800E-02,&
     & .6750100E-05,.1202800E-02,.1929100E-02,.2443300E-02,.3401500E-02,&
     & .7084300E-05,.1303400E-02,.2115200E-02,.2689200E-02,.3635000E-02,&
     & .4577600E-05,.7523300E-03,.1190600E-02,.1495700E-02,.2107600E-02,&
     & .4826200E-05,.8405100E-03,.1319200E-02,.1678000E-02,.2326700E-02,&
     & .5102400E-05,.9260300E-03,.1465300E-02,.1867100E-02,.2524200E-02,&
     & .5385600E-05,.1009300E-02,.1628800E-02,.2059900E-02,.2713400E-02,&
     & .5662500E-05,.1093900E-02,.1783400E-02,.2269300E-02,.2909200E-02,&
     & .3597800E-05,.6305500E-03,.1000100E-02,.1261800E-02,.1675900E-02,&
     & .3809000E-05,.7051900E-03,.1108000E-02,.1414200E-02,.1856300E-02,&
     & .4036500E-05,.7759300E-03,.1235200E-02,.1569200E-02,.2021500E-02,&
     & .4269200E-05,.8464800E-03,.1375300E-02,.1736100E-02,.2188200E-02,&
     & .4532500E-05,.9181500E-03,.1503100E-02,.1915600E-02,.2351100E-02,&
     & .2857700E-05,.5292300E-03,.8406000E-03,.1064100E-02,.1352400E-02,&
     & .3018900E-05,.5927500E-03,.9326400E-03,.1192500E-02,.1505100E-02,&
     & .3197100E-05,.6514000E-03,.1042200E-02,.1317400E-02,.1641300E-02,&
     & .3398100E-05,.7114400E-03,.1159400E-02,.1461300E-02,.1785100E-02,&
     & .3612700E-05,.7720000E-03,.1268500E-02,.1616400E-02,.1916600E-02,&
     & .2283200E-05,.4448900E-03,.7052300E-03,.8915000E-03,.1105400E-02,&
     & .2400500E-05,.4986000E-03,.7859600E-03,.1003300E-02,.1231000E-02,&
     & .2546500E-05,.5474600E-03,.8809600E-03,.1108400E-02,.1347900E-02,&
     & .2716700E-05,.5976000E-03,.9768500E-03,.1230500E-02,.1466800E-02,&
     & .2892700E-05,.6484400E-03,.1069500E-02,.1363200E-02,.1574600E-02/

      data   absb(176:350, 3)   /                                       &
     & .1842600E-05,.3756700E-03,.5927600E-03,.7500200E-03,.9115900E-03,&
     & .1932400E-05,.4204900E-03,.6637600E-03,.8430400E-03,.1017500E-02,&
     & .2056300E-05,.4610400E-03,.7456100E-03,.9356800E-03,.1115300E-02,&
     & .2195200E-05,.5030200E-03,.8249100E-03,.1041000E-02,.1213100E-02,&
     & .2342900E-05,.5457000E-03,.9032400E-03,.1151200E-02,.1300200E-02,&
     & .1491400E-05,.3172700E-03,.4987900E-03,.6314100E-03,.7548000E-03,&
     & .1571100E-05,.3536700E-03,.5610500E-03,.7085900E-03,.8427800E-03,&
     & .1672000E-05,.3877600E-03,.6312200E-03,.7906600E-03,.9249200E-03,&
     & .1783000E-05,.4232700E-03,.6965500E-03,.8831100E-03,.1004100E-02,&
     & .1906500E-05,.4590500E-03,.7623800E-03,.9732000E-03,.1076700E-02,&
     & .1200700E-05,.2697600E-03,.4231600E-03,.5375000E-03,.6274700E-03,&
     & .1274400E-05,.2988000E-03,.4784000E-03,.6009400E-03,.7013400E-03,&
     & .1357200E-05,.3281300E-03,.5367200E-03,.6734100E-03,.7691600E-03,&
     & .1445900E-05,.3576100E-03,.5915400E-03,.7542100E-03,.8337100E-03,&
     & .1553800E-05,.3879200E-03,.6466300E-03,.8287700E-03,.8952900E-03,&
     & .9686100E-06,.2291800E-03,.3601500E-03,.4571900E-03,.5240400E-03,&
     & .1033600E-05,.2532100E-03,.4083400E-03,.5116100E-03,.5835100E-03,&
     & .1098500E-05,.2776700E-03,.4557100E-03,.5745900E-03,.6398500E-03,&
     & .1176800E-05,.3025200E-03,.5019500E-03,.6419000E-03,.6930300E-03,&
     & .1261500E-05,.3279400E-03,.5483100E-03,.7057900E-03,.7454000E-03,&
     & .7756000E-06,.1943700E-03,.3074800E-03,.3882800E-03,.4352900E-03,&
     & .8258000E-06,.2143800E-03,.3483500E-03,.4361700E-03,.4837300E-03,&
     & .8820000E-06,.2349200E-03,.3868000E-03,.4916900E-03,.5318900E-03,&
     & .9494700E-06,.2557300E-03,.4263500E-03,.5464100E-03,.5763700E-03,&
     & .1018200E-05,.2774500E-03,.4660700E-03,.6004600E-03,.6199800E-03,&
     & .6196300E-06,.1645400E-03,.2628400E-03,.3301600E-03,.3628400E-03,&
     & .6614600E-06,.1816200E-03,.2971400E-03,.3731500E-03,.4037000E-03,&
     & .7111100E-06,.1988200E-03,.3294200E-03,.4213000E-03,.4438200E-03,&
     & .7657300E-06,.2166300E-03,.3622800E-03,.4664000E-03,.4805200E-03,&
     & .8210000E-06,.2350300E-03,.3962700E-03,.5113200E-03,.5177600E-03,&
     & .4971800E-06,.1398200E-03,.2252300E-03,.2827500E-03,.3055800E-03,&
     & .5336500E-06,.1543100E-03,.2536800E-03,.3204400E-03,.3389500E-03,&
     & .5770500E-06,.1688400E-03,.2808200E-03,.3601900E-03,.3717300E-03,&
     & .6206100E-06,.1839300E-03,.3086500E-03,.3986600E-03,.4024300E-03,&
     & .6642700E-06,.1996800E-03,.3375700E-03,.4366300E-03,.4330900E-03/

      data   absb(351:525, 3)   /                                       &
     & .3996100E-06,.1188900E-03,.1936100E-03,.2427000E-03,.2569300E-03,&
     & .4313600E-06,.1310000E-03,.2163200E-03,.2759100E-03,.2846300E-03,&
     & .4649000E-06,.1433300E-03,.2396600E-03,.3084900E-03,.3110600E-03,&
     & .5000900E-06,.1563000E-03,.2634600E-03,.3408300E-03,.3371300E-03,&
     & .5380800E-06,.1699900E-03,.2876800E-03,.3732300E-03,.3627900E-03,&
     & .3198600E-06,.1010600E-03,.1658500E-03,.2089100E-03,.2155800E-03,&
     & .3454000E-06,.1111900E-03,.1850000E-03,.2376000E-03,.2385400E-03,&
     & .3722600E-06,.1218300E-03,.2045600E-03,.2641900E-03,.2605300E-03,&
     & .4031700E-06,.1329000E-03,.2248700E-03,.2915600E-03,.2826200E-03,&
     & .4327100E-06,.1448200E-03,.2454100E-03,.3193400E-03,.3041900E-03,&
     & .2567700E-06,.8606000E-04,.1420600E-03,.1802300E-03,.1814100E-03,&
     & .2773600E-06,.9467700E-04,.1584100E-03,.2037800E-03,.2008800E-03,&
     & .3010400E-06,.1037200E-03,.1751500E-03,.2266300E-03,.2192100E-03,&
     & .3258400E-06,.1134000E-03,.1923200E-03,.2495300E-03,.2378900E-03,&
     & .3489100E-06,.1236800E-03,.2097600E-03,.2736400E-03,.2557000E-03,&
     & .2055600E-06,.7327400E-04,.1217000E-03,.1558800E-03,.1531200E-03,&
     & .2237600E-06,.8065100E-04,.1356800E-03,.1752400E-03,.1694800E-03,&
     & .2431300E-06,.8848100E-04,.1499200E-03,.1944700E-03,.1847700E-03,&
     & .2623300E-06,.9682400E-04,.1645800E-03,.2140900E-03,.2002700E-03,&
     & .2824900E-06,.1058000E-03,.1798700E-03,.2343500E-03,.2153200E-03,&
     & .1655300E-06,.6253600E-04,.1045700E-03,.1344300E-03,.1296800E-03,&
     & .1807800E-06,.6886300E-04,.1164300E-03,.1506000E-03,.1430400E-03,&
     & .1961900E-06,.7567000E-04,.1286400E-03,.1668100E-03,.1561500E-03,&
     & .2115800E-06,.8289900E-04,.1411400E-03,.1838500E-03,.1691100E-03,&
     & .2291500E-06,.9062900E-04,.1545600E-03,.2009900E-03,.1814600E-03,&
     & .1338000E-06,.5340700E-04,.8993000E-04,.1159100E-03,.1100500E-03,&
     & .1460400E-06,.5888700E-04,.1000100E-03,.1296300E-03,.1210700E-03,&
     & .1584800E-06,.6479000E-04,.1104000E-03,.1434900E-03,.1323000E-03,&
     & .1719000E-06,.7116900E-04,.1212500E-03,.1579200E-03,.1429700E-03,&
     & .1858700E-06,.7779200E-04,.1329200E-03,.1726500E-03,.1530800E-03,&
     & .1083000E-06,.4568600E-04,.7733900E-04,.9986700E-04,.9356200E-04,&
     & .1176500E-06,.5052600E-04,.8595900E-04,.1115300E-03,.1026900E-03,&
     & .1282400E-06,.5562700E-04,.9490000E-04,.1235400E-03,.1120500E-03,&
     & .1395800E-06,.6115800E-04,.1044000E-03,.1357300E-03,.1209900E-03,&
     & .1507800E-06,.6690900E-04,.1143100E-03,.1486100E-03,.1291800E-03/

      data   absb(526:700, 3)   /                                       &
     & .8753700E-07,.3907300E-04,.6633000E-04,.8586000E-04,.7929600E-04,&
     & .9527000E-07,.4328200E-04,.7374300E-04,.9573300E-04,.8708200E-04,&
     & .1042300E-06,.4770100E-04,.8144700E-04,.1060900E-03,.9485700E-04,&
     & .1134800E-06,.5248400E-04,.8962400E-04,.1163800E-03,.1021100E-03,&
     & .1228900E-06,.5743100E-04,.9812200E-04,.1276000E-03,.1086600E-03,&
     & .6989100E-07,.3312900E-04,.5632300E-04,.7303600E-04,.6656800E-04,&
     & .7646500E-07,.3672400E-04,.6262700E-04,.8152000E-04,.7317600E-04,&
     & .8383300E-07,.4061900E-04,.6929300E-04,.9016100E-04,.7954800E-04,&
     & .9143600E-07,.4467100E-04,.7636200E-04,.9907500E-04,.8549700E-04,&
     & .9926300E-07,.4888600E-04,.8355000E-04,.1084900E-03,.9072000E-04,&
     & .5508000E-07,.2779600E-04,.4726100E-04,.6129900E-04,.5523700E-04,&
     & .6056200E-07,.3085000E-04,.5263500E-04,.6851500E-04,.6079400E-04,&
     & .6644000E-07,.3420000E-04,.5829800E-04,.7584700E-04,.6609700E-04,&
     & .7291300E-07,.3765000E-04,.6431000E-04,.8353000E-04,.7103200E-04,&
     & .7954500E-07,.4121600E-04,.7039700E-04,.9130400E-04,.7519300E-04,&
     & .4364400E-07,.2299700E-04,.3908800E-04,.5071400E-04,.4537400E-04,&
     & .4818200E-07,.2558500E-04,.4366100E-04,.5681300E-04,.5005700E-04,&
     & .5306700E-07,.2842900E-04,.4846500E-04,.6302000E-04,.5449000E-04,&
     & .5834900E-07,.3136500E-04,.5360300E-04,.6951600E-04,.5858500E-04,&
     & .6372700E-07,.3439800E-04,.5871000E-04,.7603200E-04,.6204600E-04,&
     & .3466100E-07,.1900300E-04,.3229500E-04,.4189800E-04,.3720700E-04,&
     & .3841400E-07,.2120000E-04,.3617000E-04,.4704500E-04,.4115000E-04,&
     & .4237800E-07,.2360500E-04,.4024700E-04,.5230600E-04,.4487200E-04,&
     & .4659800E-07,.2611000E-04,.4462000E-04,.5780000E-04,.4829300E-04,&
     & .5099500E-07,.2868300E-04,.4893500E-04,.6325000E-04,.5115500E-04,&
     & .2754500E-07,.1570000E-04,.2666100E-04,.3460300E-04,.3048700E-04,&
     & .3055300E-07,.1756800E-04,.2994800E-04,.3893500E-04,.3381300E-04,&
     & .3377200E-07,.1960100E-04,.3341100E-04,.4340300E-04,.3692200E-04,&
     & .3716400E-07,.2172600E-04,.3711300E-04,.4803000E-04,.3977100E-04,&
     & .4079500E-07,.2390100E-04,.4076500E-04,.5260700E-04,.4215200E-04,&
     & .2185200E-07,.1281900E-04,.2175700E-04,.2822100E-04,.2474500E-04,&
     & .2420700E-07,.1439500E-04,.2452700E-04,.3186700E-04,.2750200E-04,&
     & .2680600E-07,.1609200E-04,.2746200E-04,.3564600E-04,.3014500E-04,&
     & .2961200E-07,.1789400E-04,.3057200E-04,.3952700E-04,.3254300E-04,&
     & .3257700E-07,.1974600E-04,.3367200E-04,.4339900E-04,.3455600E-04/

      data   absb(701:875, 3)   /                                       &
     & .1733600E-07,.1044100E-04,.1769500E-04,.2295400E-04,.2004200E-04,&
     & .1918700E-07,.1177100E-04,.2002500E-04,.2602900E-04,.2234600E-04,&
     & .2131100E-07,.1319000E-04,.2251600E-04,.2920100E-04,.2457700E-04,&
     & .2359700E-07,.1471200E-04,.2512200E-04,.3246600E-04,.2658400E-04,&
     & .2595800E-07,.1628000E-04,.2774400E-04,.3576200E-04,.2829500E-04,&
     & .1375200E-07,.8497000E-05,.1437900E-04,.1864000E-04,.1620500E-04,&
     & .1524000E-07,.9608200E-05,.1632500E-04,.2122500E-04,.1812400E-04,&
     & .1692400E-07,.1080200E-04,.1843000E-04,.2389600E-04,.2002400E-04,&
     & .1876700E-07,.1208700E-04,.2060900E-04,.2665100E-04,.2169800E-04,&
     & .2066600E-07,.1340900E-04,.2283700E-04,.2942700E-04,.2315300E-04,&
     & .1082700E-07,.6862400E-05,.1158600E-04,.1499900E-04,.1297500E-04,&
     & .1203000E-07,.7784700E-05,.1320500E-04,.1716400E-04,.1460500E-04,&
     & .1335200E-07,.8777900E-05,.1497400E-04,.1941000E-04,.1623100E-04,&
     & .1482900E-07,.9862000E-05,.1679500E-04,.2175100E-04,.1763400E-04,&
     & .1639600E-07,.1097500E-04,.1868200E-04,.2406600E-04,.1887900E-04,&
     & .8573100E-08,.5513200E-05,.9286200E-05,.1200000E-04,.1034700E-04,&
     & .9503300E-08,.6281600E-05,.1062900E-04,.1381700E-04,.1171200E-04,&
     & .1053700E-07,.7096400E-05,.1210800E-04,.1570000E-04,.1309500E-04,&
     & .1172200E-07,.8012000E-05,.1362100E-04,.1767900E-04,.1429600E-04,&
     & .1299100E-07,.8945700E-05,.1522000E-04,.1961400E-04,.1535000E-04,&
     & .6751500E-08,.4421200E-05,.7424400E-05,.9577600E-05,.8242900E-05,&
     & .7499900E-08,.5060300E-05,.8536000E-05,.1109500E-04,.9379100E-05,&
     & .8318200E-08,.5741100E-05,.9768600E-05,.1266700E-04,.1052800E-04,&
     & .9252600E-08,.6494200E-05,.1102800E-04,.1432800E-04,.1156600E-04,&
     & .1026900E-07,.7276900E-05,.1237500E-04,.1596400E-04,.1247200E-04,&
     & .5303100E-08,.3530100E-05,.5905000E-05,.7596500E-05,.6545200E-05,&
     & .5890700E-08,.4056100E-05,.6823300E-05,.8862100E-05,.7477100E-05,&
     & .6542200E-08,.4627000E-05,.7839300E-05,.1016900E-04,.8437000E-05,&
     & .7266900E-08,.5243300E-05,.8906100E-05,.1156300E-04,.9326800E-05,&
     & .8088000E-08,.5896700E-05,.1001600E-04,.1293000E-04,.1010600E-04,&
     & .4145900E-08,.2796500E-05,.4652900E-05,.5964800E-05,.5155200E-05,&
     & .4632400E-08,.3225300E-05,.5410000E-05,.7009400E-05,.5911000E-05,&
     & .5143300E-08,.3701400E-05,.6239500E-05,.8097600E-05,.6710000E-05,&
     & .5704500E-08,.4209100E-05,.7138100E-05,.9253700E-05,.7489600E-05,&
     & .6352600E-08,.4754100E-05,.8057300E-05,.1042800E-04,.8154000E-05/

      data   absb(876:1050, 3)   /                                      &
     & .3254900E-08,.2208900E-05,.3657100E-05,.4666300E-05,.4038700E-05,&
     & .3619200E-08,.2558100E-05,.4275200E-05,.5526500E-05,.4658200E-05,&
     & .4021500E-08,.2951500E-05,.4955300E-05,.6432700E-05,.5323600E-05,&
     & .4473600E-08,.3369000E-05,.5701500E-05,.7385900E-05,.5996000E-05,&
     & .4976000E-08,.3827600E-05,.6459600E-05,.8384400E-05,.6564700E-05,&
     & .2543200E-08,.1739000E-05,.2860700E-05,.3643800E-05,.3142500E-05,&
     & .2836400E-08,.2021900E-05,.3369800E-05,.4341200E-05,.3665400E-05,&
     & .3160100E-08,.2347200E-05,.3923200E-05,.5096400E-05,.4214200E-05,&
     & .3510100E-08,.2694800E-05,.4542900E-05,.5879700E-05,.4771500E-05,&
     & .3898900E-08,.3071800E-05,.5173100E-05,.6719000E-05,.5272000E-05,&
     & .2009300E-08,.1370400E-05,.2237300E-05,.2837500E-05,.2455000E-05,&
     & .2227000E-08,.1602000E-05,.2662600E-05,.3414400E-05,.2896300E-05,&
     & .2479700E-08,.1868600E-05,.3113100E-05,.4034800E-05,.3339800E-05,&
     & .2761400E-08,.2159300E-05,.3619900E-05,.4685600E-05,.3801900E-05,&
     & .3067400E-08,.2469900E-05,.4151700E-05,.5384400E-05,.4232200E-05,&
     & .1590700E-08,.1080300E-05,.1751800E-05,.2206400E-05,.1925400E-05,&
     & .1750900E-08,.1269100E-05,.2100000E-05,.2679400E-05,.2283200E-05,&
     & .1950900E-08,.1486100E-05,.2472600E-05,.3195700E-05,.2641800E-05,&
     & .2170300E-08,.1728400E-05,.2883800E-05,.3732500E-05,.3025700E-05,&
     & .2412400E-08,.1984400E-05,.3330500E-05,.4312800E-05,.3401500E-05,&
     & .1256400E-08,.8466900E-06,.1371700E-05,.1714700E-05,.1494700E-05,&
     & .1379400E-08,.1002400E-05,.1652300E-05,.2099300E-05,.1787000E-05,&
     & .1532000E-08,.1179000E-05,.1957700E-05,.2522000E-05,.2084500E-05,&
     & .1713000E-08,.1379100E-05,.2295200E-05,.2967700E-05,.2404000E-05,&
     & .1904100E-08,.1591400E-05,.2663500E-05,.3443900E-05,.2724700E-05,&
     & .9952300E-09,.6621900E-06,.1063200E-05,.1334100E-05,.1155000E-05,&
     & .1090300E-08,.7902200E-06,.1291400E-05,.1640100E-05,.1392900E-05,&
     & .1208200E-08,.9326600E-06,.1544500E-05,.1984100E-05,.1645200E-05,&
     & .1343700E-08,.1096900E-05,.1821200E-05,.2351700E-05,.1902900E-05,&
     & .1497200E-08,.1273400E-05,.2127600E-05,.2743100E-05,.2172200E-05,&
     & .7878200E-09,.5199300E-06,.8275200E-06,.1042100E-05,.8863200E-06,&
     & .8664200E-09,.6244000E-06,.1013800E-05,.1282000E-05,.1095800E-05,&
     & .9514100E-09,.7405100E-06,.1222100E-05,.1562600E-05,.1301400E-05,&
     & .1062200E-08,.8743800E-06,.1448100E-05,.1867500E-05,.1513800E-05,&
     & .1182900E-08,.1021700E-05,.1702500E-05,.2190000E-05,.1734900E-05/

      data   absb(1051:1175, 3)   /                                     &
     & .6273400E-09,.4084400E-06,.6467500E-06,.8116800E-06,.6801500E-06,&
     & .6867000E-09,.4943700E-06,.7982200E-06,.1003100E-05,.8595500E-06,&
     & .7555100E-09,.5881300E-06,.9664000E-06,.1230500E-05,.1029000E-05,&
     & .8389600E-09,.6976200E-06,.1152900E-05,.1484500E-05,.1202500E-05,&
     & .9372700E-09,.8199200E-06,.1361800E-05,.1748900E-05,.1385600E-05,&
     & .5021400E-09,.3202800E-06,.5052400E-06,.6314700E-06,.5190300E-06,&
     & .5477500E-09,.3884100E-06,.6258900E-06,.7844500E-06,.6709900E-06,&
     & .5997100E-09,.4660200E-06,.7635600E-06,.9697800E-06,.8091200E-06,&
     & .6646600E-09,.5550900E-06,.9153300E-06,.1176600E-05,.9528400E-06,&
     & .7393100E-09,.6562500E-06,.1086400E-05,.1394600E-05,.1105500E-05,&
     & .4017100E-09,.2500900E-06,.3921100E-06,.4925000E-06,.3933200E-06,&
     & .4337900E-09,.3047300E-06,.4893200E-06,.6134200E-06,.5207600E-06,&
     & .4772900E-09,.3687400E-06,.5997800E-06,.7624600E-06,.6355600E-06,&
     & .5245400E-09,.4407500E-06,.7248800E-06,.9307900E-06,.7544800E-06,&
     & .5854600E-09,.5238900E-06,.8656500E-06,.1110700E-05,.8785800E-06,&
     & .3222200E-09,.1953600E-06,.3048400E-06,.3812300E-06,.2979800E-06,&
     & .3455000E-09,.2400300E-06,.3820600E-06,.4803300E-06,.4019300E-06,&
     & .3793000E-09,.2917900E-06,.4722600E-06,.5993800E-06,.5005600E-06,&
     & .4167500E-09,.3503500E-06,.5750000E-06,.7364200E-06,.5983600E-06,&
     & .4632000E-09,.4185400E-06,.6904100E-06,.8854800E-06,.7000300E-06,&
     & .2616600E-09,.1586800E-06,.2469700E-06,.3089200E-06,.2396400E-06,&
     & .2809500E-09,.1955200E-06,.3106800E-06,.3903700E-06,.3246900E-06,&
     & .3073000E-09,.2385600E-06,.3855400E-06,.4890200E-06,.4073800E-06,&
     & .3378100E-09,.2874900E-06,.4715700E-06,.6028600E-06,.4880300E-06,&
     & .3752600E-09,.3443900E-06,.5678900E-06,.7274200E-06,.5723800E-06/

      data   absb(  1:175, 4)   /                                       &
     & .4903800E-04,.5078100E-02,.7825400E-02,.9648400E-02,.1061400E-01,&
     & .5422500E-04,.5490500E-02,.8571000E-02,.1046000E-01,.1120600E-01,&
     & .6113400E-04,.5877300E-02,.9318300E-02,.1134700E-01,.1172900E-01,&
     & .6879300E-04,.6279400E-02,.9976800E-02,.1224800E-01,.1225000E-01,&
     & .7742900E-04,.6689300E-02,.1062400E-01,.1306400E-01,.1273000E-01,&
     & .4070600E-04,.4304500E-02,.6624000E-02,.8122700E-02,.8710500E-02,&
     & .4578900E-04,.4640400E-02,.7254700E-02,.8850800E-02,.9218300E-02,&
     & .5135900E-04,.4967000E-02,.7864900E-02,.9592800E-02,.9659000E-02,&
     & .5844800E-04,.5311200E-02,.8416500E-02,.1035600E-01,.1008500E-01,&
     & .6658500E-04,.5659000E-02,.8977000E-02,.1102200E-01,.1054000E-01,&
     & .3316100E-04,.3637400E-02,.5600700E-02,.6840800E-02,.7039900E-02,&
     & .3731600E-04,.3914500E-02,.6131800E-02,.7481200E-02,.7453300E-02,&
     & .4218400E-04,.4194900E-02,.6627200E-02,.8109700E-02,.7839300E-02,&
     & .4824900E-04,.4486100E-02,.7093400E-02,.8733000E-02,.8226200E-02,&
     & .5491200E-04,.4782900E-02,.7568800E-02,.9283000E-02,.8605800E-02,&
     & .2659800E-04,.3064900E-02,.4732300E-02,.5774200E-02,.5664400E-02,&
     & .3009600E-04,.3295000E-02,.5177800E-02,.6314500E-02,.6019300E-02,&
     & .3408500E-04,.3535000E-02,.5580600E-02,.6844700E-02,.6370800E-02,&
     & .3891900E-04,.3783100E-02,.5971900E-02,.7346500E-02,.6708800E-02,&
     & .4449200E-04,.4039300E-02,.6375000E-02,.7812000E-02,.7024900E-02,&
     & .2144900E-04,.2575600E-02,.3993600E-02,.4866200E-02,.4620500E-02,&
     & .2410300E-04,.2770600E-02,.4361900E-02,.5316400E-02,.4920200E-02,&
     & .2745100E-04,.2976500E-02,.4695900E-02,.5763200E-02,.5212200E-02,&
     & .3138100E-04,.3188400E-02,.5022200E-02,.6173500E-02,.5479000E-02,&
     & .3595300E-04,.3409900E-02,.5364500E-02,.6565900E-02,.5741900E-02,&
     & .1746800E-04,.2164200E-02,.3364200E-02,.4097200E-02,.3792100E-02,&
     & .1950600E-04,.2330800E-02,.3668300E-02,.4469600E-02,.4038500E-02,&
     & .2232600E-04,.2507200E-02,.3945400E-02,.4850700E-02,.4277300E-02,&
     & .2555800E-04,.2690100E-02,.4224000E-02,.5185500E-02,.4500000E-02,&
     & .2951900E-04,.2881600E-02,.4513500E-02,.5516800E-02,.4725900E-02,&
     & .1416200E-04,.1816500E-02,.2831100E-02,.3449100E-02,.3126300E-02,&
     & .1588000E-04,.1959400E-02,.3079200E-02,.3757700E-02,.3327700E-02,&
     & .1825000E-04,.2112400E-02,.3311000E-02,.4071700E-02,.3525500E-02,&
     & .2094800E-04,.2271100E-02,.3551200E-02,.4354100E-02,.3711400E-02,&
     & .2427900E-04,.2436000E-02,.3797200E-02,.4633600E-02,.3906300E-02/

      data   absb(176:350, 4)   /                                       &
     & .1167700E-04,.1526800E-02,.2385400E-02,.2905400E-02,.2592700E-02,&
     & .1312700E-04,.1650200E-02,.2588400E-02,.3164200E-02,.2758400E-02,&
     & .1514500E-04,.1782700E-02,.2785400E-02,.3419800E-02,.2922800E-02,&
     & .1742600E-04,.1920800E-02,.2991900E-02,.3656300E-02,.3083600E-02,&
     & .2018700E-04,.2063800E-02,.3202700E-02,.3895500E-02,.3251700E-02,&
     & .9624800E-05,.1283900E-02,.2006900E-02,.2443900E-02,.2152100E-02,&
     & .1088400E-04,.1390500E-02,.2175300E-02,.2663400E-02,.2290100E-02,&
     & .1257600E-04,.1505700E-02,.2343000E-02,.2871400E-02,.2427100E-02,&
     & .1449200E-04,.1624500E-02,.2522900E-02,.3070900E-02,.2565300E-02,&
     & .1678500E-04,.1748700E-02,.2705300E-02,.3276900E-02,.2704100E-02,&
     & .7898700E-05,.1084100E-02,.1696300E-02,.2064600E-02,.1792000E-02,&
     & .9068800E-05,.1178300E-02,.1836400E-02,.2252300E-02,.1905400E-02,&
     & .1043600E-04,.1277600E-02,.1983200E-02,.2422000E-02,.2020000E-02,&
     & .1205100E-04,.1380000E-02,.2139100E-02,.2590400E-02,.2136400E-02,&
     & .1399200E-04,.1487700E-02,.2298300E-02,.2769000E-02,.2251900E-02,&
     & .6514300E-05,.9162900E-03,.1433700E-02,.1746800E-02,.1489000E-02,&
     & .7529500E-05,.9988000E-03,.1553200E-02,.1901700E-02,.1584400E-02,&
     & .8617200E-05,.1084300E-02,.1682800E-02,.2045400E-02,.1682200E-02,&
     & .1001700E-04,.1172900E-02,.1817600E-02,.2191200E-02,.1780000E-02,&
     & .1164000E-04,.1266800E-02,.1954900E-02,.2344700E-02,.1876500E-02,&
     & .5343200E-05,.7757100E-03,.1211400E-02,.1480500E-02,.1237200E-02,&
     & .6151500E-05,.8471800E-03,.1315300E-02,.1607800E-02,.1318100E-02,&
     & .7092300E-05,.9202400E-03,.1429300E-02,.1730000E-02,.1398900E-02,&
     & .8268400E-05,.9974000E-03,.1544700E-02,.1857300E-02,.1481100E-02,&
     & .9626800E-05,.1078900E-02,.1662700E-02,.1989200E-02,.1562100E-02,&
     & .4393200E-05,.6589800E-03,.1025900E-02,.1256800E-02,.1028700E-02,&
     & .5048300E-05,.7197300E-03,.1117400E-02,.1361000E-02,.1097300E-02,&
     & .5852700E-05,.7826900E-03,.1215700E-02,.1465800E-02,.1164800E-02,&
     & .6825800E-05,.8496800E-03,.1315800E-02,.1577400E-02,.1235400E-02,&
     & .7978000E-05,.9209300E-03,.1417100E-02,.1691700E-02,.1302900E-02,&
     & .3634900E-05,.5613900E-03,.8723200E-03,.1067100E-02,.8574700E-03,&
     & .4176300E-05,.6130700E-03,.9528400E-03,.1155900E-02,.9164000E-03,&
     & .4864300E-05,.6676900E-03,.1037700E-02,.1248000E-02,.9741700E-03,&
     & .5680600E-05,.7261800E-03,.1123800E-02,.1344300E-02,.1034100E-02,&
     & .6657400E-05,.7879100E-03,.1211800E-02,.1442600E-02,.1090800E-02/

      data   absb(351:525, 4)   /                                       &
     & .2984900E-05,.4784500E-03,.7432400E-03,.9069500E-03,.7165400E-03,&
     & .3452800E-05,.5230500E-03,.8140700E-03,.9834800E-03,.7662200E-03,&
     & .4038600E-05,.5706700E-03,.8863500E-03,.1064300E-02,.8161700E-03,&
     & .4734900E-05,.6215900E-03,.9604500E-03,.1147600E-02,.8666700E-03,&
     & .5550400E-05,.6748600E-03,.1037300E-02,.1232700E-02,.9143700E-03,&
     & .2447400E-05,.4081900E-03,.6346900E-03,.7718400E-03,.5995900E-03,&
     & .2852700E-05,.4469400E-03,.6955000E-03,.8380900E-03,.6415100E-03,&
     & .3344400E-05,.4880400E-03,.7579700E-03,.9092100E-03,.6846400E-03,&
     & .3933200E-05,.5322900E-03,.8217400E-03,.9810300E-03,.7268000E-03,&
     & .4624500E-05,.5786400E-03,.8887200E-03,.1054400E-02,.7672300E-03,&
     & .2025300E-05,.3492100E-03,.5436500E-03,.6586500E-03,.5033100E-03,&
     & .2372000E-05,.3827800E-03,.5957700E-03,.7172900E-03,.5386400E-03,&
     & .2794500E-05,.4188700E-03,.6490700E-03,.7782300E-03,.5758100E-03,&
     & .3293600E-05,.4569900E-03,.7048100E-03,.8407400E-03,.6108900E-03,&
     & .3888100E-05,.4975700E-03,.7632500E-03,.9035600E-03,.6456400E-03,&
     & .1679700E-05,.2991800E-03,.4659800E-03,.5631600E-03,.4227000E-03,&
     & .1976400E-05,.3283900E-03,.5107600E-03,.6143900E-03,.4530600E-03,&
     & .2338400E-05,.3598500E-03,.5568600E-03,.6671200E-03,.4844500E-03,&
     & .2766200E-05,.3930200E-03,.6052500E-03,.7209200E-03,.5142500E-03,&
     & .3268700E-05,.4283700E-03,.6561000E-03,.7757000E-03,.5439600E-03,&
     & .1399100E-05,.2568700E-03,.3999600E-03,.4830700E-03,.3557000E-03,&
     & .1656400E-05,.2824600E-03,.4385800E-03,.5274300E-03,.3821100E-03,&
     & .1966000E-05,.3097100E-03,.4785600E-03,.5730900E-03,.4082600E-03,&
     & .2333400E-05,.3388100E-03,.5206600E-03,.6191900E-03,.4336700E-03,&
     & .2758700E-05,.3696400E-03,.5649900E-03,.6666000E-03,.4596900E-03,&
     & .1171100E-05,.2210200E-03,.3439400E-03,.4151200E-03,.2998000E-03,&
     & .1392800E-05,.2433700E-03,.3771300E-03,.4535100E-03,.3225300E-03,&
     & .1658900E-05,.2670400E-03,.4120200E-03,.4927200E-03,.3444200E-03,&
     & .1974000E-05,.2926100E-03,.4485500E-03,.5327400E-03,.3663300E-03,&
     & .2332300E-05,.3197300E-03,.4872600E-03,.5738300E-03,.3891500E-03,&
     & .9830000E-06,.1904700E-03,.2959200E-03,.3573900E-03,.2530300E-03,&
     & .1175100E-05,.2098300E-03,.3247400E-03,.3904300E-03,.2725900E-03,&
     & .1402700E-05,.2306500E-03,.3550500E-03,.4240500E-03,.2912000E-03,&
     & .1669200E-05,.2530200E-03,.3869900E-03,.4588800E-03,.3101800E-03,&
     & .1971600E-05,.2771500E-03,.4211900E-03,.4946800E-03,.3298900E-03/

      data   absb(526:700, 4)   /                                       &
     & .8291500E-06,.1639600E-03,.2543300E-03,.3070000E-03,.2140600E-03,&
     & .9935200E-06,.1808300E-03,.2793500E-03,.3355200E-03,.2304800E-03,&
     & .1187700E-05,.1991500E-03,.3057800E-03,.3645500E-03,.2465900E-03,&
     & .1413900E-05,.2188600E-03,.3338400E-03,.3947600E-03,.2631700E-03,&
     & .1667000E-05,.2402100E-03,.3639000E-03,.4260300E-03,.2800500E-03,&
     & .6867000E-06,.1397900E-03,.2167300E-03,.2616100E-03,.1797400E-03,&
     & .8243700E-06,.1544800E-03,.2384600E-03,.2861100E-03,.1937700E-03,&
     & .9864900E-06,.1704400E-03,.2613900E-03,.3113200E-03,.2076500E-03,&
     & .1174000E-05,.1877300E-03,.2860900E-03,.3374000E-03,.2219700E-03,&
     & .1383300E-05,.2065200E-03,.3123300E-03,.3644900E-03,.2365800E-03,&
     & .5556400E-06,.1178600E-03,.1828300E-03,.2208900E-03,.1495100E-03,&
     & .6673100E-06,.1305300E-03,.2015100E-03,.2418900E-03,.1614600E-03,&
     & .8004800E-06,.1443200E-03,.2215400E-03,.2636400E-03,.1734800E-03,&
     & .9519500E-06,.1593700E-03,.2431000E-03,.2860900E-03,.1857700E-03,&
     & .1121700E-05,.1758000E-03,.2660100E-03,.3094900E-03,.1985000E-03,&
     & .4452600E-06,.9833400E-04,.1523600E-03,.1839500E-03,.1236900E-03,&
     & .5355100E-06,.1092700E-03,.1683700E-03,.2019400E-03,.1338700E-03,&
     & .6430700E-06,.1211000E-03,.1855600E-03,.2205900E-03,.1441800E-03,&
     & .7668800E-06,.1341100E-03,.2041400E-03,.2398500E-03,.1547700E-03,&
     & .9059100E-06,.1484200E-03,.2240500E-03,.2599600E-03,.1656600E-03,&
     & .3563000E-06,.8199100E-04,.1269000E-03,.1531200E-03,.1022600E-03,&
     & .4291100E-06,.9139100E-04,.1406400E-03,.1684900E-03,.1109500E-03,&
     & .5158300E-06,.1016300E-03,.1553800E-03,.1844800E-03,.1197700E-03,&
     & .6165400E-06,.1128700E-03,.1713800E-03,.2010100E-03,.1288400E-03,&
     & .7302800E-06,.1253000E-03,.1886200E-03,.2182600E-03,.1381600E-03,&
     & .2845500E-06,.6836200E-04,.1057200E-03,.1274700E-03,.8449400E-04,&
     & .3431300E-06,.7641300E-04,.1175100E-03,.1406000E-03,.9190500E-04,&
     & .4130500E-06,.8529900E-04,.1301600E-03,.1543100E-03,.9946300E-04,&
     & .4946600E-06,.9504700E-04,.1439600E-03,.1685100E-03,.1072500E-03,&
     & .5872100E-06,.1058300E-03,.1588600E-03,.1833200E-03,.1151600E-03,&
     & .2241600E-06,.5645100E-04,.8719200E-04,.1050300E-03,.6929900E-04,&
     & .2705800E-06,.6332500E-04,.9724100E-04,.1162500E-03,.7563500E-04,&
     & .3264000E-06,.7094200E-04,.1080100E-03,.1279200E-03,.8206100E-04,&
     & .3917800E-06,.7938100E-04,.1198500E-03,.1400500E-03,.8872200E-04,&
     & .4669400E-06,.8867300E-04,.1326100E-03,.1527400E-03,.9543700E-04/

      data   absb(701:875, 4)   /                                       &
     & .1762000E-06,.4649500E-04,.7182900E-04,.8635100E-04,.5670100E-04,&
     & .2129500E-06,.5238300E-04,.8038000E-04,.9593700E-04,.6210000E-04,&
     & .2572000E-06,.5890000E-04,.8954800E-04,.1059400E-03,.6756800E-04,&
     & .3095700E-06,.6613600E-04,.9964300E-04,.1162300E-03,.7328000E-04,&
     & .3700300E-06,.7420300E-04,.1106100E-03,.1270700E-03,.7897900E-04,&
     & .1384300E-06,.3823800E-04,.5912400E-04,.7098200E-04,.4633100E-04,&
     & .1672600E-06,.4326900E-04,.6642400E-04,.7912500E-04,.5095600E-04,&
     & .2023200E-06,.4887000E-04,.7424700E-04,.8766100E-04,.5556100E-04,&
     & .2440800E-06,.5511900E-04,.8282700E-04,.9645300E-04,.6044700E-04,&
     & .2927600E-06,.6202800E-04,.9222100E-04,.1057000E-03,.6528100E-04,&
     & .1079300E-06,.3119100E-04,.4830500E-04,.5794800E-04,.3766600E-04,&
     & .1301700E-06,.3546500E-04,.5448900E-04,.6484100E-04,.4155600E-04,&
     & .1577100E-06,.4023500E-04,.6117600E-04,.7208200E-04,.4545200E-04,&
     & .1906400E-06,.4559900E-04,.6848800E-04,.7957600E-04,.4959300E-04,&
     & .2294400E-06,.5155200E-04,.7643100E-04,.8746200E-04,.5371500E-04,&
     & .8387900E-07,.2532500E-04,.3929700E-04,.4711800E-04,.3051000E-04,&
     & .1008700E-06,.2892000E-04,.4453200E-04,.5292900E-04,.3377200E-04,&
     & .1223800E-06,.3297700E-04,.5018900E-04,.5907500E-04,.3706900E-04,&
     & .1481300E-06,.3755600E-04,.5645100E-04,.6544800E-04,.4056700E-04,&
     & .1787400E-06,.4267200E-04,.6319600E-04,.7215900E-04,.4408500E-04,&
     & .6526100E-07,.2051600E-04,.3189500E-04,.3827900E-04,.2465000E-04,&
     & .7814500E-07,.2354000E-04,.3631900E-04,.4316100E-04,.2739000E-04,&
     & .9480500E-07,.2696200E-04,.4113400E-04,.4835700E-04,.3019000E-04,&
     & .1149000E-06,.3085800E-04,.4642900E-04,.5378500E-04,.3313600E-04,&
     & .1390500E-06,.3521000E-04,.5221600E-04,.5948200E-04,.3612200E-04,&
     & .5066800E-07,.1652600E-04,.2575200E-04,.3095400E-04,.1981700E-04,&
     & .6036100E-07,.1905800E-04,.2947100E-04,.3506000E-04,.2212300E-04,&
     & .7310100E-07,.2193700E-04,.3356900E-04,.3946100E-04,.2449800E-04,&
     & .8874700E-07,.2522200E-04,.3804800E-04,.4403900E-04,.2697300E-04,&
     & .1076200E-06,.2893200E-04,.4296400E-04,.4887100E-04,.2949900E-04,&
     & .3918500E-07,.1320700E-04,.2062600E-04,.2483500E-04,.1582600E-04,&
     & .4633500E-07,.1530300E-04,.2372200E-04,.2827700E-04,.1776200E-04,&
     & .5589600E-07,.1770300E-04,.2716200E-04,.3197600E-04,.1978000E-04,&
     & .6796200E-07,.2045100E-04,.3094000E-04,.3586700E-04,.2183400E-04,&
     & .8253500E-07,.2358300E-04,.3512800E-04,.3993300E-04,.2397200E-04/

      data   absb(876:1050, 4)   /                                      &
     & .3043500E-07,.1052800E-04,.1645800E-04,.1987100E-04,.1263000E-04,&
     & .3565700E-07,.1225500E-04,.1903400E-04,.2275700E-04,.1423000E-04,&
     & .4277700E-07,.1424100E-04,.2191600E-04,.2585600E-04,.1591600E-04,&
     & .5197700E-07,.1653100E-04,.2509400E-04,.2914200E-04,.1763200E-04,&
     & .6324600E-07,.1916200E-04,.2863100E-04,.3258600E-04,.1944700E-04,&
     & .2372200E-07,.8365600E-05,.1308800E-04,.1583400E-04,.1006000E-04,&
     & .2750900E-07,.9786900E-05,.1522700E-04,.1825200E-04,.1136300E-04,&
     & .3276900E-07,.1142600E-04,.1762800E-04,.2085300E-04,.1276700E-04,&
     & .3971900E-07,.1332600E-04,.2028500E-04,.2363700E-04,.1422300E-04,&
     & .4841500E-07,.1552400E-04,.2326800E-04,.2653000E-04,.1573900E-04,&
     & .1862300E-07,.6662000E-05,.1042500E-04,.1263400E-04,.8013200E-05,&
     & .2139300E-07,.7826400E-05,.1218800E-04,.1464300E-04,.9075700E-05,&
     & .2528100E-07,.9178700E-05,.1418600E-04,.1682500E-04,.1024700E-04,&
     & .3053000E-07,.1075800E-04,.1641900E-04,.1917800E-04,.1148200E-04,&
     & .3721300E-07,.1259300E-04,.1892000E-04,.2164300E-04,.1274900E-04,&
     & .1468100E-07,.5304300E-05,.8295600E-05,.1006400E-04,.6366900E-05,&
     & .1671700E-07,.6250800E-05,.9750100E-05,.1173700E-04,.7247300E-05,&
     & .1958700E-07,.7364300E-05,.1140000E-04,.1355800E-04,.8229000E-05,&
     & .2355500E-07,.8674800E-05,.1326800E-04,.1553700E-04,.9263400E-05,&
     & .2867100E-07,.1020900E-04,.1537900E-04,.1763500E-04,.1031600E-04,&
     & .1162600E-07,.4214900E-05,.6574000E-05,.7983500E-05,.5059500E-05,&
     & .1310200E-07,.4988400E-05,.7775800E-05,.9384600E-05,.5785600E-05,&
     & .1520800E-07,.5895500E-05,.9139200E-05,.1089800E-04,.6593700E-05,&
     & .1817400E-07,.6975800E-05,.1069200E-04,.1255400E-04,.7456400E-05,&
     & .2205700E-07,.8250500E-05,.1246200E-04,.1433100E-04,.8330100E-05,&
     & .9213000E-08,.3337800E-05,.5199800E-05,.6300600E-05,.4009700E-05,&
     & .1029800E-07,.3970400E-05,.6183900E-05,.7477500E-05,.4607700E-05,&
     & .1183100E-07,.4708800E-05,.7306200E-05,.8734500E-05,.5262000E-05,&
     & .1403000E-07,.5594200E-05,.8592100E-05,.1011900E-04,.5983800E-05,&
     & .1697000E-07,.6644700E-05,.1006300E-04,.1162000E-04,.6720800E-05,&
     & .7344300E-08,.2649800E-05,.4122900E-05,.4978800E-05,.3192900E-05,&
     & .8145300E-08,.3168900E-05,.4932000E-05,.5970100E-05,.3670600E-05,&
     & .9285300E-08,.3774600E-05,.5856100E-05,.7013900E-05,.4210500E-05,&
     & .1091300E-07,.4498600E-05,.6919200E-05,.8173300E-05,.4808100E-05,&
     & .1315100E-07,.5368900E-05,.8146900E-05,.9437700E-05,.5432000E-05/

      data   absb(1051:1175, 4)   /                                     &
     & .5869000E-08,.2105600E-05,.3271000E-05,.3936200E-05,.2535700E-05,&
     & .6482400E-08,.2529200E-05,.3933500E-05,.4765200E-05,.2926600E-05,&
     & .7319500E-08,.3026000E-05,.4694200E-05,.5634600E-05,.3370000E-05,&
     & .8529800E-08,.3621600E-05,.5572700E-05,.6599000E-05,.3866200E-05,&
     & .1022700E-07,.4336600E-05,.6593500E-05,.7662700E-05,.4388900E-05,&
     & .4687900E-08,.1667400E-05,.2584800E-05,.3099200E-05,.2009700E-05,&
     & .5157900E-08,.2014500E-05,.3130700E-05,.3787500E-05,.2330400E-05,&
     & .5782900E-08,.2419900E-05,.3752900E-05,.4511900E-05,.2696200E-05,&
     & .6681700E-08,.2909200E-05,.4478800E-05,.5314200E-05,.3103700E-05,&
     & .7954800E-08,.3498500E-05,.5324800E-05,.6206600E-05,.3538700E-05,&
     & .3748700E-08,.1317700E-05,.2036200E-05,.2427800E-05,.1588800E-05,&
     & .4113200E-08,.1600600E-05,.2486300E-05,.3000100E-05,.1853000E-05,&
     & .4579600E-08,.1930100E-05,.2995700E-05,.3605300E-05,.2151500E-05,&
     & .5244700E-08,.2330100E-05,.3590200E-05,.4269000E-05,.2484400E-05,&
     & .6195200E-08,.2817500E-05,.4289400E-05,.5013300E-05,.2850300E-05,&
     & .3005300E-08,.1041900E-05,.1603400E-05,.1907100E-05,.1255000E-05,&
     & .3290600E-08,.1271700E-05,.1975000E-05,.2375100E-05,.1476100E-05,&
     & .3641200E-08,.1541300E-05,.2395000E-05,.2881700E-05,.1717300E-05,&
     & .4132700E-08,.1867300E-05,.2880000E-05,.3430100E-05,.1989300E-05,&
     & .4844800E-08,.2269900E-05,.3455900E-05,.4049600E-05,.2293800E-05,&
     & .2451700E-08,.8536700E-06,.1312100E-05,.1558700E-05,.1020900E-05,&
     & .2681400E-08,.1045900E-05,.1624100E-05,.1950500E-05,.1206700E-05,&
     & .2960400E-08,.1272600E-05,.1978600E-05,.2376500E-05,.1407100E-05,&
     & .3349000E-08,.1548600E-05,.2388400E-05,.2839300E-05,.1635300E-05,&
     & .3917700E-08,.1892300E-05,.2875100E-05,.3364800E-05,.1889600E-05/

      data   absb(  1:175, 5)   /                                       &
     & .1060600E-02,.1761600E-01,.2633700E-01,.3083200E-01,.2325000E-01,&
     & .1197500E-02,.1885100E-01,.2774600E-01,.3219100E-01,.2446100E-01,&
     & .1368600E-02,.2023600E-01,.2922400E-01,.3340900E-01,.2553800E-01,&
     & .1578800E-02,.2168600E-01,.3082200E-01,.3458100E-01,.2648300E-01,&
     & .1842200E-02,.2320100E-01,.3248500E-01,.3579100E-01,.2733900E-01,&
     & .9029900E-03,.1493900E-01,.2223500E-01,.2596900E-01,.1945900E-01,&
     & .1027600E-02,.1604300E-01,.2346400E-01,.2710200E-01,.2041800E-01,&
     & .1184800E-02,.1724400E-01,.2476600E-01,.2813700E-01,.2127900E-01,&
     & .1376300E-02,.1849400E-01,.2615900E-01,.2914100E-01,.2203500E-01,&
     & .1608700E-02,.1980100E-01,.2758600E-01,.3020800E-01,.2271200E-01,&
     & .7616900E-03,.1265700E-01,.1874400E-01,.2180700E-01,.1615500E-01,&
     & .8742900E-03,.1362200E-01,.1982000E-01,.2275300E-01,.1690800E-01,&
     & .1015300E-02,.1465000E-01,.2096700E-01,.2363700E-01,.1759700E-01,&
     & .1186000E-02,.1572800E-01,.2215900E-01,.2452100E-01,.1823600E-01,&
     & .1394000E-02,.1685500E-01,.2337700E-01,.2544200E-01,.1880000E-01,&
     & .6397900E-03,.1071600E-01,.1578200E-01,.1828000E-01,.1333000E-01,&
     & .7399200E-03,.1155000E-01,.1672100E-01,.1907400E-01,.1395500E-01,&
     & .8653500E-03,.1243200E-01,.1771500E-01,.1983500E-01,.1451900E-01,&
     & .1018000E-02,.1335800E-01,.1873300E-01,.2061200E-01,.1505300E-01,&
     & .1202000E-02,.1433000E-01,.1978000E-01,.2139000E-01,.1552300E-01,&
     & .5382700E-03,.9067500E-02,.1328200E-01,.1531000E-01,.1100200E-01,&
     & .6273300E-03,.9781800E-02,.1409300E-01,.1598500E-01,.1151700E-01,&
     & .7389400E-03,.1053600E-01,.1493700E-01,.1664200E-01,.1198200E-01,&
     & .8747100E-03,.1132900E-01,.1581300E-01,.1730800E-01,.1243100E-01,&
     & .1036900E-02,.1216700E-01,.1671700E-01,.1796600E-01,.1283300E-01,&
     & .4563200E-03,.7663100E-02,.1117500E-01,.1282100E-01,.9100700E-02,&
     & .5361800E-03,.8274100E-02,.1186700E-01,.1340200E-01,.9528400E-02,&
     & .6352000E-03,.8917900E-02,.1259100E-01,.1396000E-01,.9922700E-02,&
     & .7551300E-03,.9596600E-02,.1334500E-01,.1452900E-01,.1029400E-01,&
     & .8982800E-03,.1032500E-01,.1412700E-01,.1508200E-01,.1063700E-01,&
     & .3878200E-03,.6468600E-02,.9394700E-02,.1073800E-01,.7536600E-02,&
     & .4582600E-03,.6987200E-02,.9987800E-02,.1123000E-01,.7892000E-02,&
     & .5454400E-03,.7538600E-02,.1061300E-01,.1170600E-01,.8221900E-02,&
     & .6508500E-03,.8124300E-02,.1126100E-01,.1218300E-01,.8544700E-02,&
     & .7760100E-03,.8759800E-02,.1193300E-01,.1266000E-01,.8838800E-02/

      data   absb(176:350, 5)   /                                       &
     & .3330400E-03,.5462300E-02,.7902800E-02,.8996000E-02,.6263500E-02,&
     & .3955300E-03,.5905500E-02,.8415500E-02,.9411100E-02,.6561300E-02,&
     & .4718000E-03,.6377900E-02,.8950000E-02,.9817200E-02,.6845900E-02,&
     & .5645300E-03,.6886200E-02,.9504800E-02,.1022500E-01,.7124600E-02,&
     & .6748600E-03,.7441100E-02,.1008200E-01,.1063700E-01,.7378600E-02,&
     & .2869300E-03,.4618000E-02,.6650400E-02,.7537900E-02,.5207100E-02,&
     & .3418000E-03,.4994800E-02,.7089800E-02,.7886600E-02,.5464500E-02,&
     & .4081100E-03,.5400600E-02,.7547400E-02,.8233100E-02,.5712700E-02,&
     & .4887300E-03,.5843400E-02,.8021200E-02,.8584400E-02,.5947000E-02,&
     & .5859300E-03,.6326000E-02,.8518200E-02,.8940800E-02,.6166200E-02,&
     & .2497400E-03,.3924000E-02,.5620800E-02,.6334500E-02,.4343100E-02,&
     & .2974900E-03,.4252600E-02,.5999600E-02,.6626600E-02,.4562400E-02,&
     & .3560800E-03,.4604700E-02,.6391900E-02,.6925900E-02,.4773800E-02,&
     & .4269900E-03,.4992000E-02,.6799300E-02,.7227600E-02,.4971400E-02,&
     & .5119800E-03,.5416600E-02,.7230200E-02,.7538700E-02,.5166000E-02,&
     & .2171600E-03,.3337500E-02,.4752800E-02,.5323800E-02,.3625800E-02,&
     & .2588600E-03,.3624300E-02,.5079700E-02,.5574000E-02,.3810700E-02,&
     & .3102600E-03,.3935300E-02,.5416900E-02,.5830900E-02,.3987600E-02,&
     & .3723500E-03,.4275700E-02,.5771100E-02,.6091100E-02,.4158600E-02,&
     & .4456800E-03,.4646300E-02,.6147600E-02,.6363300E-02,.4327700E-02,&
     & .1873800E-03,.2841200E-02,.4025100E-02,.4475400E-02,.3023200E-02,&
     & .2242200E-03,.3093000E-02,.4306200E-02,.4692100E-02,.3178800E-02,&
     & .2694200E-03,.3369400E-02,.4598300E-02,.4912600E-02,.3328700E-02,&
     & .3231700E-03,.3670200E-02,.4909200E-02,.5139300E-02,.3477100E-02,&
     & .3863800E-03,.3995100E-02,.5238500E-02,.5381500E-02,.3623200E-02,&
     & .1622400E-03,.2423200E-02,.3414100E-02,.3766100E-02,.2526200E-02,&
     & .1946500E-03,.2646300E-02,.3657500E-02,.3954100E-02,.2657700E-02,&
     & .2339100E-03,.2893600E-02,.3913000E-02,.4144600E-02,.2784100E-02,&
     & .2804400E-03,.3159800E-02,.4185300E-02,.4344200E-02,.2909800E-02,&
     & .3348300E-03,.3443700E-02,.4473500E-02,.4562500E-02,.3035100E-02,&
     & .1413200E-03,.2073700E-02,.2903600E-02,.3176200E-02,.2115700E-02,&
     & .1697700E-03,.2274300E-02,.3115200E-02,.3338200E-02,.2225800E-02,&
     & .2040900E-03,.2494700E-02,.3340100E-02,.3504800E-02,.2333800E-02,&
     & .2447200E-03,.2729700E-02,.3579800E-02,.3683500E-02,.2442100E-02,&
     & .2913100E-03,.2980400E-02,.3830800E-02,.3879700E-02,.2550000E-02/

      data   absb(351:525, 5)   /                                       &
     & .1229300E-03,.1779700E-02,.2472500E-02,.2682200E-02,.1771800E-02,&
     & .1478900E-03,.1959500E-02,.2658400E-02,.2822200E-02,.1865500E-02,&
     & .1779000E-03,.2155200E-02,.2858400E-02,.2969500E-02,.1958900E-02,&
     & .2131500E-03,.2362400E-02,.3068800E-02,.3130900E-02,.2052000E-02,&
     & .2528500E-03,.2582500E-02,.3287600E-02,.3306800E-02,.2144500E-02,&
     & .1065700E-03,.1531900E-02,.2109600E-02,.2267800E-02,.1484400E-02,&
     & .1285600E-03,.1692000E-02,.2274300E-02,.2389900E-02,.1563900E-02,&
     & .1547100E-03,.1863600E-02,.2451100E-02,.2522100E-02,.1643600E-02,&
     & .1851200E-03,.2047200E-02,.2636400E-02,.2668100E-02,.1723800E-02,&
     & .2187200E-03,.2241100E-02,.2827000E-02,.2826000E-02,.1804800E-02,&
     & .9287600E-04,.1323700E-02,.1804900E-02,.1921400E-02,.1247100E-02,&
     & .1122600E-03,.1465900E-02,.1951800E-02,.2029500E-02,.1314300E-02,&
     & .1351900E-03,.1617100E-02,.2108900E-02,.2149800E-02,.1383300E-02,&
     & .1610200E-03,.1777800E-02,.2270300E-02,.2281000E-02,.1452800E-02,&
     & .1898100E-03,.1948400E-02,.2437400E-02,.2424900E-02,.1523600E-02,&
     & .8082800E-04,.1147100E-02,.1548600E-02,.1630700E-02,.1048200E-02,&
     & .9806300E-04,.1271800E-02,.1679700E-02,.1728700E-02,.1105300E-02,&
     & .1177200E-03,.1405000E-02,.1817700E-02,.1837900E-02,.1164800E-02,&
     & .1398100E-03,.1546900E-02,.1958900E-02,.1956500E-02,.1225800E-02,&
     & .1644600E-03,.1694700E-02,.2105500E-02,.2087400E-02,.1288500E-02,&
     & .7053900E-04,.9962100E-03,.1332500E-02,.1387900E-02,.8822000E-03,&
     & .8558900E-04,.1106300E-02,.1449200E-02,.1477600E-02,.9311300E-03,&
     & .1024400E-03,.1224000E-02,.1570100E-02,.1576400E-02,.9823700E-03,&
     & .1215100E-03,.1348800E-02,.1694200E-02,.1684600E-02,.1036400E-02,&
     & .1424500E-03,.1475300E-02,.1822900E-02,.1804200E-02,.1091000E-02,&
     & .6159800E-04,.8670800E-03,.1150600E-02,.1185700E-02,.7439600E-03,&
     & .7469800E-04,.9644300E-03,.1253300E-02,.1267300E-02,.7859000E-03,&
     & .8909200E-04,.1068400E-02,.1359000E-02,.1357100E-02,.8309000E-03,&
     & .1053700E-03,.1176600E-02,.1468200E-02,.1456000E-02,.8782700E-03,&
     & .1232700E-03,.1285000E-02,.1581300E-02,.1564300E-02,.9259900E-03,&
     & .5374800E-04,.7562900E-03,.9959500E-03,.1016300E-02,.6280900E-03,&
     & .6497200E-04,.8426500E-03,.1085600E-02,.1090800E-02,.6641600E-03,&
     & .7738600E-04,.9339700E-03,.1178700E-02,.1172700E-02,.7038700E-03,&
     & .9113800E-04,.1026600E-02,.1274900E-02,.1263000E-02,.7450700E-03,&
     & .1064000E-03,.1118900E-02,.1373800E-02,.1360800E-02,.7883000E-03/

      data   absb(526:700, 5)   /                                       &
     & .4666700E-04,.6588400E-03,.8604900E-03,.8721400E-03,.5309900E-03,&
     & .5626300E-04,.7349000E-03,.9390000E-03,.9394600E-03,.5627800E-03,&
     & .6685300E-04,.8138300E-03,.1020900E-02,.1014200E-02,.5977600E-03,&
     & .7859800E-04,.8931400E-03,.1105400E-02,.1096300E-02,.6341900E-03,&
     & .9150300E-04,.9719300E-03,.1192700E-02,.1184000E-02,.6732000E-03,&
     & .3959900E-04,.5686600E-03,.7380300E-03,.7449100E-03,.4468800E-03,&
     & .4767700E-04,.6349000E-03,.8068500E-03,.8054000E-03,.4746600E-03,&
     & .5663000E-04,.7027900E-03,.8786000E-03,.8733200E-03,.5047600E-03,&
     & .6650600E-04,.7706100E-03,.9526200E-03,.9467600E-03,.5373000E-03,&
     & .7734300E-04,.8387000E-03,.1029500E-02,.1025000E-02,.5720000E-03,&
     & .3276700E-04,.4852300E-03,.6275100E-03,.6318300E-03,.3735300E-03,&
     & .3945000E-04,.5426100E-03,.6874400E-03,.6858700E-03,.3974400E-03,&
     & .4686100E-04,.6009300E-03,.7497700E-03,.7462300E-03,.4231500E-03,&
     & .5495600E-04,.6591600E-03,.8146900E-03,.8115800E-03,.4514400E-03,&
     & .6390800E-04,.7182000E-03,.8821200E-03,.8810300E-03,.4816500E-03,&
     & .2673400E-04,.4075400E-03,.5264500E-03,.5301400E-03,.3111800E-03,&
     & .3227300E-04,.4566100E-03,.5782500E-03,.5776600E-03,.3320600E-03,&
     & .3848800E-04,.5069700E-03,.6324500E-03,.6305800E-03,.3547600E-03,&
     & .4522800E-04,.5570600E-03,.6891800E-03,.6880900E-03,.3794400E-03,&
     & .5267500E-04,.6080800E-03,.7479400E-03,.7493400E-03,.4059800E-03,&
     & .2175100E-04,.3420900E-03,.4414600E-03,.4445800E-03,.2592000E-03,&
     & .2632600E-04,.3841500E-03,.4860300E-03,.4863200E-03,.2773000E-03,&
     & .3151500E-04,.4273600E-03,.5332500E-03,.5326200E-03,.2971400E-03,&
     & .3721900E-04,.4706400E-03,.5826300E-03,.5832100E-03,.3189000E-03,&
     & .4334400E-04,.5149300E-03,.6340200E-03,.6372900E-03,.3421200E-03,&
     & .1764300E-04,.2871700E-03,.3702000E-03,.3728800E-03,.2158000E-03,&
     & .2141800E-04,.3234300E-03,.4087100E-03,.4095300E-03,.2315100E-03,&
     & .2572500E-04,.3604400E-03,.4496600E-03,.4500500E-03,.2488000E-03,&
     & .3047900E-04,.3978500E-03,.4926300E-03,.4944500E-03,.2678300E-03,&
     & .3565300E-04,.4364600E-03,.5377100E-03,.5423000E-03,.2882200E-03,&
     & .1410200E-04,.2381400E-03,.3073000E-03,.3098100E-03,.1786400E-03,&
     & .1718700E-04,.2693400E-03,.3403900E-03,.3414600E-03,.1923400E-03,&
     & .2073900E-04,.3013000E-03,.3757100E-03,.3767200E-03,.2073700E-03,&
     & .2467000E-04,.3333300E-03,.4128300E-03,.4153800E-03,.2240800E-03,&
     & .2904600E-04,.3669400E-03,.4521600E-03,.4572900E-03,.2418700E-03/

      data   absb(701:875, 5)   /                                       &
     & .1122100E-04,.1971000E-03,.2546500E-03,.2569600E-03,.1477300E-03,&
     & .1374700E-04,.2239000E-03,.2830500E-03,.2841300E-03,.1595500E-03,&
     & .1664900E-04,.2514500E-03,.3134600E-03,.3146700E-03,.1726900E-03,&
     & .1989900E-04,.2793500E-03,.3456300E-03,.3484500E-03,.1871800E-03,&
     & .2356800E-04,.3083000E-03,.3797400E-03,.3851200E-03,.2028100E-03,&
     & .8914700E-05,.1631400E-03,.2108800E-03,.2129400E-03,.1220500E-03,&
     & .1095700E-04,.1859600E-03,.2352300E-03,.2362800E-03,.1322500E-03,&
     & .1332700E-04,.2097400E-03,.2613400E-03,.2626300E-03,.1435600E-03,&
     & .1600500E-04,.2337800E-03,.2892500E-03,.2920200E-03,.1562100E-03,&
     & .1903800E-04,.2591400E-03,.3189700E-03,.3241500E-03,.1698200E-03,&
     & .6999500E-05,.1340500E-03,.1733800E-03,.1753500E-03,.1003600E-03,&
     & .8640600E-05,.1534500E-03,.1941600E-03,.1951400E-03,.1091000E-03,&
     & .1055600E-04,.1737300E-03,.2165900E-03,.2176700E-03,.1188700E-03,&
     & .1273900E-04,.1944500E-03,.2405300E-03,.2431100E-03,.1298000E-03,&
     & .1523200E-04,.2164000E-03,.2664500E-03,.2710700E-03,.1416900E-03,&
     & .5471200E-05,.1097000E-03,.1419500E-03,.1438600E-03,.8226000E-04,&
     & .6764400E-05,.1261200E-03,.1596300E-03,.1606100E-03,.8972700E-04,&
     & .8297300E-05,.1434800E-03,.1787800E-03,.1797500E-03,.9812200E-04,&
     & .1008200E-04,.1613200E-03,.1995200E-03,.2015500E-03,.1075500E-03,&
     & .1210700E-04,.1800800E-03,.2217900E-03,.2257600E-03,.1178700E-03,&
     & .4276700E-05,.8968000E-04,.1161100E-03,.1178100E-03,.6733800E-04,&
     & .5286000E-05,.1036000E-03,.1310700E-03,.1320300E-03,.7372200E-04,&
     & .6510700E-05,.1183800E-03,.1473500E-03,.1482900E-03,.8085200E-04,&
     & .7950100E-05,.1337400E-03,.1652000E-03,.1669300E-03,.8896900E-04,&
     & .9593700E-05,.1499300E-03,.1845600E-03,.1878100E-03,.9789700E-04,&
     & .3322600E-05,.7291800E-04,.9463500E-04,.9606900E-04,.5491200E-04,&
     & .4118500E-05,.8470500E-04,.1072600E-03,.1080900E-03,.6032500E-04,&
     & .5079600E-05,.9741400E-04,.1210600E-03,.1218400E-03,.6641500E-04,&
     & .6231100E-05,.1105500E-03,.1362500E-03,.1377200E-03,.7335400E-04,&
     & .7559400E-05,.1243900E-03,.1529700E-03,.1556900E-03,.8097200E-04,&
     & .2550700E-05,.5875300E-04,.7659900E-04,.7779800E-04,.4450800E-04,&
     & .3180000E-05,.6867900E-04,.8724000E-04,.8784300E-04,.4911500E-04,&
     & .3927300E-05,.7949700E-04,.9887100E-04,.9942100E-04,.5424900E-04,&
     & .4834600E-05,.9078200E-04,.1117300E-03,.1128200E-03,.6011300E-04,&
     & .5899500E-05,.1026700E-03,.1259200E-03,.1281500E-03,.6665600E-04/

      data   absb(876:1050, 5)   /                                      &
     & .1953900E-05,.4717400E-04,.6187400E-04,.6291600E-04,.3597300E-04,&
     & .2445200E-05,.5557500E-04,.7085300E-04,.7123800E-04,.3989800E-04,&
     & .3040200E-05,.6478300E-04,.8063300E-04,.8092100E-04,.4421400E-04,&
     & .3739000E-05,.7443900E-04,.9149800E-04,.9220800E-04,.4916200E-04,&
     & .4583600E-05,.8460800E-04,.1035800E-03,.1053100E-03,.5470200E-04,&
     & .1494000E-05,.3771600E-04,.4982900E-04,.5080400E-04,.2898700E-04,&
     & .1875800E-05,.4477700E-04,.5739200E-04,.5769700E-04,.3231700E-04,&
     & .2341300E-05,.5260400E-04,.6560800E-04,.6574800E-04,.3594600E-04,&
     & .2896100E-05,.6089100E-04,.7482400E-04,.7518200E-04,.4009900E-04,&
     & .3552400E-05,.6958700E-04,.8506800E-04,.8629500E-04,.4480400E-04,&
     & .1150000E-05,.3016900E-04,.4015700E-04,.4106900E-04,.2336200E-04,&
     & .1445300E-05,.3608900E-04,.4654900E-04,.4679900E-04,.2619300E-04,&
     & .1812900E-05,.4273500E-04,.5347400E-04,.5351400E-04,.2924900E-04,&
     & .2252900E-05,.4984300E-04,.6123400E-04,.6140400E-04,.3274800E-04,&
     & .2776200E-05,.5732800E-04,.7002700E-04,.7078300E-04,.3671200E-04,&
     & .8885100E-06,.2409500E-04,.3230300E-04,.3321000E-04,.1880500E-04,&
     & .1117700E-05,.2908200E-04,.3770500E-04,.3794400E-04,.2120900E-04,&
     & .1406000E-05,.3466700E-04,.4357800E-04,.4357000E-04,.2379200E-04,&
     & .1753900E-05,.4073900E-04,.5011400E-04,.5017200E-04,.2672000E-04,&
     & .2172900E-05,.4722400E-04,.5759800E-04,.5807000E-04,.3007900E-04,&
     & .6852500E-06,.1917800E-04,.2589600E-04,.2680900E-04,.1509700E-04,&
     & .8630100E-06,.2333600E-04,.3043400E-04,.3071300E-04,.1712000E-04,&
     & .1088900E-05,.2802800E-04,.3542200E-04,.3539700E-04,.1930100E-04,&
     & .1364900E-05,.3318700E-04,.4095400E-04,.4092100E-04,.2175200E-04,&
     & .1698200E-05,.3874700E-04,.4726000E-04,.4755700E-04,.2458800E-04,&
     & .5280900E-06,.1520600E-04,.2068800E-04,.2159200E-04,.1208400E-04,&
     & .6649300E-06,.1864700E-04,.2447900E-04,.2481900E-04,.1378100E-04,&
     & .8410200E-06,.2257400E-04,.2868300E-04,.2869000E-04,.1562100E-04,&
     & .1058500E-05,.2695500E-04,.3336400E-04,.3329800E-04,.1767700E-04,&
     & .1323900E-05,.3171000E-04,.3873300E-04,.3886400E-04,.2004900E-04,&
     & .4096900E-06,.1209300E-04,.1656400E-04,.1742100E-04,.9688200E-05,&
     & .5156900E-06,.1493700E-04,.1973400E-04,.2009400E-04,.1111200E-04,&
     & .6529600E-06,.1822900E-04,.2328200E-04,.2332100E-04,.1266400E-04,&
     & .8257300E-06,.2195000E-04,.2723100E-04,.2715000E-04,.1439300E-04,&
     & .1037300E-05,.2601300E-04,.3179100E-04,.3182800E-04,.1637900E-04/

      data   absb(1051:1175, 5)   /                                     &
     & .3192800E-06,.9616200E-05,.1325700E-04,.1405300E-04,.7774100E-05,&
     & .4011200E-06,.1196300E-04,.1590600E-04,.1625700E-04,.8957800E-05,&
     & .5081600E-06,.1471700E-04,.1888600E-04,.1894700E-04,.1026200E-04,&
     & .6446500E-06,.1786400E-04,.2220700E-04,.2215500E-04,.1171500E-04,&
     & .8132800E-06,.2132600E-04,.2607600E-04,.2606100E-04,.1337600E-04,&
     & .2482900E-06,.7620400E-05,.1057800E-04,.1130900E-04,.6220800E-05,&
     & .3117600E-06,.9548800E-05,.1278000E-04,.1312500E-04,.7201300E-05,&
     & .3950600E-06,.1185000E-04,.1527900E-04,.1534800E-04,.8294700E-05,&
     & .5026600E-06,.1449200E-04,.1807400E-04,.1804000E-04,.9515300E-05,&
     & .6363800E-06,.1744200E-04,.2133600E-04,.2131000E-04,.1090400E-04,&
     & .1930200E-06,.6018700E-05,.8416200E-05,.9079900E-05,.4964900E-05,&
     & .2422200E-06,.7595900E-05,.1023300E-04,.1057000E-04,.5770100E-05,&
     & .3068800E-06,.9508000E-05,.1231800E-04,.1240300E-04,.6691300E-05,&
     & .3908900E-06,.1171900E-04,.1467000E-04,.1464400E-04,.7716600E-05,&
     & .4967400E-06,.1421400E-04,.1740200E-04,.1738200E-04,.8872700E-05,&
     & .1504600E-06,.4766300E-05,.6699800E-05,.7290800E-05,.3963100E-05,&
     & .1885600E-06,.6045900E-05,.8190800E-05,.8520600E-05,.4627000E-05,&
     & .2386900E-06,.7630200E-05,.9924500E-05,.1002300E-04,.5397700E-05,&
     & .3045900E-06,.9476700E-05,.1191100E-04,.1189400E-04,.6257900E-05,&
     & .3882000E-06,.1158800E-04,.1420500E-04,.1418200E-04,.7222600E-05,&
     & .1221200E-06,.3933200E-05,.5523000E-05,.6009400E-05,.3249900E-05,&
     & .1532400E-06,.5017800E-05,.6787900E-05,.7056600E-05,.3810800E-05,&
     & .1941800E-06,.6367800E-05,.8264700E-05,.8344300E-05,.4467200E-05,&
     & .2482000E-06,.7954300E-05,.9967000E-05,.9951800E-05,.5202700E-05,&
     & .3174000E-06,.9770400E-05,.1196400E-04,.1193800E-04,.6027800E-05/

      data   absb(  1:175, 6)   /                                       &
     & .4672255E-01,.1179341E+00,.1338757E+00,.1299972E+00,.9161157E-01,&
     & .5062470E-01,.1213117E+00,.1369429E+00,.1328028E+00,.9360006E-01,&
     & .5459974E-01,.1248223E+00,.1399926E+00,.1355144E+00,.9556280E-01,&
     & .5873868E-01,.1286746E+00,.1431991E+00,.1382901E+00,.9738836E-01,&
     & .6302530E-01,.1329297E+00,.1467302E+00,.1411782E+00,.9929195E-01,&
     & .3968991E-01,.9976895E-01,.1131751E+00,.1100474E+00,.7710072E-01,&
     & .4301477E-01,.1026851E+00,.1158037E+00,.1124295E+00,.7893417E-01,&
     & .4646290E-01,.1058587E+00,.1184976E+00,.1148060E+00,.8065093E-01,&
     & .5002899E-01,.1093646E+00,.1214273E+00,.1173032E+00,.8238971E-01,&
     & .5370336E-01,.1132393E+00,.1247403E+00,.1199136E+00,.8410182E-01,&
     & .3361226E-01,.8419071E-01,.9544048E-01,.9285444E-01,.6446524E-01,&
     & .3646479E-01,.8678678E-01,.9771637E-01,.9491470E-01,.6610593E-01,&
     & .3943418E-01,.8966612E-01,.1001521E+00,.9704047E-01,.6769280E-01,&
     & .4249750E-01,.9286388E-01,.1028925E+00,.9929778E-01,.6917725E-01,&
     & .4559710E-01,.9635024E-01,.1059705E+00,.1017312E+00,.7059703E-01,&
     & .2841305E-01,.7093757E-01,.8032314E-01,.7813700E-01,.5373624E-01,&
     & .3087001E-01,.7327916E-01,.8235049E-01,.7995476E-01,.5513198E-01,&
     & .3341491E-01,.7589200E-01,.8459743E-01,.8187839E-01,.5650567E-01,&
     & .3600242E-01,.7879372E-01,.8716029E-01,.8394378E-01,.5783239E-01,&
     & .3862500E-01,.8189679E-01,.8997661E-01,.8623580E-01,.5901740E-01,&
     & .2401634E-01,.5974102E-01,.6747709E-01,.6564216E-01,.4478664E-01,&
     & .2611796E-01,.6185142E-01,.6932940E-01,.6726455E-01,.4600594E-01,&
     & .2827920E-01,.6420880E-01,.7144020E-01,.6899927E-01,.4720878E-01,&
     & .3044618E-01,.6678350E-01,.7379995E-01,.7092071E-01,.4834282E-01,&
     & .3265937E-01,.6948508E-01,.7635424E-01,.7306622E-01,.4936137E-01,&
     & .2029779E-01,.5028866E-01,.5661750E-01,.5508955E-01,.3739316E-01,&
     & .2208972E-01,.5221100E-01,.5834184E-01,.5654839E-01,.3848438E-01,&
     & .2389477E-01,.5432163E-01,.6030221E-01,.5814906E-01,.3951198E-01,&
     & .2571656E-01,.5658610E-01,.6243776E-01,.5994219E-01,.4048024E-01,&
     & .2759559E-01,.5894005E-01,.6471627E-01,.6189304E-01,.4136898E-01,&
     & .1714555E-01,.4232553E-01,.4751571E-01,.4621072E-01,.3128045E-01,&
     & .1865226E-01,.4405192E-01,.4911024E-01,.4754554E-01,.3222507E-01,&
     & .2016046E-01,.4594242E-01,.5088205E-01,.4903912E-01,.3313354E-01,&
     & .2170679E-01,.4792414E-01,.5278935E-01,.5068417E-01,.3395265E-01,&
     & .2331129E-01,.4999405E-01,.5480871E-01,.5244342E-01,.3473337E-01/

      data   absb(176:350, 6)   /                                       &
     & .1449397E-01,.3565245E-01,.3997027E-01,.3878295E-01,.2623894E-01,&
     & .1574907E-01,.3721054E-01,.4142074E-01,.4002092E-01,.2707613E-01,&
     & .1701927E-01,.3887652E-01,.4300733E-01,.4140562E-01,.2785469E-01,&
     & .1834230E-01,.4061477E-01,.4467996E-01,.4289260E-01,.2857492E-01,&
     & .1972332E-01,.4243979E-01,.4646142E-01,.4447765E-01,.2929449E-01,&
     & .1223686E-01,.3005384E-01,.3364453E-01,.3256819E-01,.2203383E-01,&
     & .1329184E-01,.3143872E-01,.3496344E-01,.3371716E-01,.2275179E-01,&
     & .1437272E-01,.3289059E-01,.3638637E-01,.3497888E-01,.2343303E-01,&
     & .1550047E-01,.3441489E-01,.3787367E-01,.3632111E-01,.2407709E-01,&
     & .1668336E-01,.3604499E-01,.3944792E-01,.3774085E-01,.2472138E-01,&
     & .1038282E-01,.2542597E-01,.2842132E-01,.2744137E-01,.1852562E-01,&
     & .1127312E-01,.2664076E-01,.2962239E-01,.2850810E-01,.1915442E-01,&
     & .1219323E-01,.2791660E-01,.3088571E-01,.2965335E-01,.1975054E-01,&
     & .1315724E-01,.2927559E-01,.3221079E-01,.3086581E-01,.2032424E-01,&
     & .1418884E-01,.3074537E-01,.3362124E-01,.3215085E-01,.2090413E-01,&
     & .8810984E-02,.2153212E-01,.2404863E-01,.2318381E-01,.1558634E-01,&
     & .9557713E-02,.2259858E-01,.2512473E-01,.2414921E-01,.1614064E-01,&
     & .1034460E-01,.2372894E-01,.2624401E-01,.2518234E-01,.1665912E-01,&
     & .1117906E-01,.2495670E-01,.2743338E-01,.2627714E-01,.1717734E-01,&
     & .1208564E-01,.2629206E-01,.2873101E-01,.2745336E-01,.1769705E-01,&
     & .7472179E-02,.1826124E-01,.2037977E-01,.1963051E-01,.1311820E-01,&
     & .8105914E-02,.1920286E-01,.2133520E-01,.2050264E-01,.1360311E-01,&
     & .8786371E-02,.2021445E-01,.2233873E-01,.2143465E-01,.1406558E-01,&
     & .9517597E-02,.2132628E-01,.2342388E-01,.2243468E-01,.1453007E-01,&
     & .1031444E-01,.2254987E-01,.2462788E-01,.2351153E-01,.1499827E-01,&
     & .6339327E-02,.1551891E-01,.1730563E-01,.1666327E-01,.1106485E-01,&
     & .6886916E-02,.1635981E-01,.1815633E-01,.1745076E-01,.1148631E-01,&
     & .7479578E-02,.1727214E-01,.1906789E-01,.1829842E-01,.1190414E-01,&
     & .8122799E-02,.1828117E-01,.2007236E-01,.1921662E-01,.1232324E-01,&
     & .8828155E-02,.1940872E-01,.2120238E-01,.2020565E-01,.1274767E-01,&
     & .5392272E-02,.1323113E-01,.1473701E-01,.1419217E-01,.9365787E-02,&
     & .5870200E-02,.1398439E-01,.1550589E-01,.1490579E-01,.9737495E-02,&
     & .6389745E-02,.1481106E-01,.1634605E-01,.1568477E-01,.1010973E-01,&
     & .6958634E-02,.1574163E-01,.1728569E-01,.1652721E-01,.1049045E-01,&
     & .7581630E-02,.1678655E-01,.1834875E-01,.1744786E-01,.1088430E-01/

      data   absb(351:525, 6)   /                                       &
     & .4592919E-02,.1130255E-01,.1258234E-01,.1211906E-01,.7936306E-02,&
     & .5013344E-02,.1198363E-01,.1328506E-01,.1277230E-01,.8268918E-02,&
     & .5471875E-02,.1274465E-01,.1406720E-01,.1349234E-01,.8605208E-02,&
     & .5975738E-02,.1360931E-01,.1495090E-01,.1427336E-01,.8951125E-02,&
     & .6524713E-02,.1458353E-01,.1594799E-01,.1514041E-01,.9313867E-02,&
     & .3921257E-02,.9679135E-02,.1077668E-01,.1037874E-01,.6732679E-02,&
     & .4289466E-02,.1030205E-01,.1142417E-01,.1098668E-01,.7034601E-02,&
     & .4696449E-02,.1101261E-01,.1215654E-01,.1165356E-01,.7339816E-02,&
     & .5140924E-02,.1181986E-01,.1298600E-01,.1238661E-01,.7655437E-02,&
     & .5627986E-02,.1273133E-01,.1391895E-01,.1320853E-01,.7987663E-02,&
     & .3358374E-02,.8318578E-02,.9266665E-02,.8925427E-02,.5731495E-02,&
     & .3681177E-02,.8898948E-02,.9870888E-02,.9491903E-02,.6003477E-02,&
     & .4042382E-02,.9566221E-02,.1056008E-01,.1011534E-01,.6284646E-02,&
     & .4437145E-02,.1032409E-01,.1134025E-01,.1081068E-01,.6574647E-02,&
     & .4869992E-02,.1118221E-01,.1221978E-01,.1159427E-01,.6878293E-02,&
     & .2880952E-02,.7177713E-02,.8000548E-02,.7709187E-02,.4888892E-02,&
     & .3168109E-02,.7724358E-02,.8567998E-02,.8237375E-02,.5136819E-02,&
     & .3487574E-02,.8352795E-02,.9218971E-02,.8825929E-02,.5394241E-02,&
     & .3839297E-02,.9067161E-02,.9957533E-02,.9491401E-02,.5660899E-02,&
     & .4223712E-02,.9875806E-02,.1079202E-01,.1023576E-01,.5939546E-02,&
     & .2478739E-02,.6226385E-02,.6940669E-02,.6691212E-02,.4181873E-02,&
     & .2734468E-02,.6743172E-02,.7478246E-02,.7187563E-02,.4408421E-02,&
     & .3017927E-02,.7337488E-02,.8096419E-02,.7749978E-02,.4643233E-02,&
     & .3331178E-02,.8012614E-02,.8797666E-02,.8384632E-02,.4887248E-02,&
     & .3674803E-02,.8776521E-02,.9606185E-02,.9089542E-02,.5147082E-02,&
     & .2139286E-02,.5432558E-02,.6053952E-02,.5837608E-02,.3588363E-02,&
     & .2366596E-02,.5923173E-02,.6566620E-02,.6309240E-02,.3794714E-02,&
     & .2619079E-02,.6485565E-02,.7155112E-02,.6847354E-02,.4009599E-02,&
     & .2899192E-02,.7125771E-02,.7834469E-02,.7451217E-02,.4235186E-02,&
     & .3206376E-02,.7850198E-02,.8612179E-02,.8123777E-02,.4476963E-02,&
     & .1850772E-02,.4769289E-02,.5310687E-02,.5121509E-02,.3086654E-02,&
     & .2053571E-02,.5233812E-02,.5801277E-02,.5573075E-02,.3276182E-02,&
     & .2279445E-02,.5767992E-02,.6371794E-02,.6087455E-02,.3472505E-02,&
     & .2530319E-02,.6378620E-02,.7028994E-02,.6663834E-02,.3681702E-02,&
     & .2802247E-02,.7072716E-02,.7774573E-02,.7310658E-02,.3905465E-02/

      data   absb(526:700, 6)   /                                       &
     & .1599613E-02,.4199571E-02,.4673794E-02,.4506393E-02,.2664028E-02,&
     & .1780622E-02,.4639345E-02,.5147271E-02,.4937749E-02,.2836730E-02,&
     & .1982209E-02,.5147946E-02,.5697272E-02,.5427726E-02,.3018582E-02,&
     & .2205837E-02,.5730040E-02,.6328408E-02,.5979873E-02,.3214093E-02,&
     & .2444633E-02,.6402438E-02,.7040553E-02,.6604978E-02,.3423730E-02,&
     & .1370767E-02,.3680434E-02,.4101387E-02,.3951642E-02,.2288026E-02,&
     & .1530214E-02,.4092955E-02,.4551071E-02,.4359160E-02,.2445148E-02,&
     & .1709076E-02,.4573254E-02,.5075448E-02,.4822350E-02,.2615383E-02,&
     & .1905572E-02,.5130202E-02,.5673701E-02,.5348975E-02,.2795090E-02,&
     & .2113656E-02,.5770683E-02,.6349792E-02,.5951205E-02,.2990809E-02,&
     & .1161344E-02,.3200182E-02,.3576104E-02,.3443194E-02,.1950452E-02,&
     & .1300611E-02,.3583463E-02,.3997263E-02,.3823311E-02,.2093149E-02,&
     & .1456618E-02,.4035349E-02,.4489225E-02,.4258002E-02,.2249394E-02,&
     & .1627537E-02,.4558261E-02,.5050496E-02,.4756449E-02,.2415485E-02,&
     & .1808199E-02,.5161226E-02,.5687359E-02,.5329814E-02,.2597049E-02,&
     & .9689423E-03,.2745553E-02,.3074976E-02,.2960730E-02,.1661770E-02,&
     & .1088391E-02,.3094206E-02,.3461113E-02,.3308475E-02,.1792192E-02,&
     & .1222284E-02,.3506981E-02,.3912501E-02,.3709778E-02,.1935066E-02,&
     & .1369638E-02,.3987964E-02,.4430278E-02,.4173459E-02,.2088437E-02,&
     & .1526024E-02,.4545546E-02,.5021863E-02,.4708792E-02,.2255640E-02,&
     & .8077252E-03,.2356556E-02,.2645654E-02,.2547112E-02,.1416723E-02,&
     & .9097668E-03,.2673661E-02,.2999603E-02,.2865604E-02,.1535808E-02,&
     & .1024815E-02,.3051199E-02,.3413691E-02,.3236220E-02,.1665481E-02,&
     & .1151477E-02,.3493356E-02,.3891808E-02,.3667727E-02,.1807129E-02,&
     & .1286842E-02,.4009109E-02,.4442138E-02,.4167563E-02,.1962379E-02,&
     & .6730887E-03,.2025445E-02,.2280105E-02,.2194379E-02,.1207918E-02,&
     & .7604151E-03,.2314345E-02,.2604175E-02,.2486551E-02,.1316161E-02,&
     & .8589949E-03,.2660424E-02,.2985221E-02,.2829802E-02,.1435082E-02,&
     & .9677787E-03,.3067804E-02,.3427622E-02,.3232026E-02,.1564953E-02,&
     & .1084841E-02,.3545518E-02,.3940911E-02,.3699314E-02,.1709431E-02,&
     & .5540015E-03,.1719029E-02,.1940291E-02,.1868227E-02,.1025773E-02,&
     & .6275728E-03,.1977116E-02,.2231569E-02,.2131986E-02,.1124006E-02,&
     & .7113040E-03,.2288889E-02,.2577000E-02,.2444792E-02,.1232036E-02,&
     & .8041626E-03,.2658785E-02,.2980992E-02,.2813689E-02,.1351842E-02,&
     & .9045868E-03,.3094678E-02,.3453278E-02,.3244781E-02,.1485313E-02/

      data   absb(701:875, 6)   /                                       &
     & .4548896E-03,.1456307E-02,.1647750E-02,.1587652E-02,.8696717E-03,&
     & .5169304E-03,.1686770E-02,.1909185E-02,.1825449E-02,.9593282E-03,&
     & .5876050E-03,.1966424E-02,.2220965E-02,.2109507E-02,.1057588E-02,&
     & .6666461E-03,.2301238E-02,.2589214E-02,.2446720E-02,.1167507E-02,&
     & .7525562E-03,.2698055E-02,.3023141E-02,.2843621E-02,.1289308E-02,&
     & .3734897E-03,.1232867E-02,.1398777E-02,.1349075E-02,.7359709E-03,&
     & .4253634E-03,.1438755E-02,.1632644E-02,.1562815E-02,.8174284E-03,&
     & .4850469E-03,.1689562E-02,.1914018E-02,.1820551E-02,.9077182E-03,&
     & .5522656E-03,.1992581E-02,.2249528E-02,.2128458E-02,.1008030E-02,&
     & .6258618E-03,.2353791E-02,.2648433E-02,.2493658E-02,.1119658E-02,&
     & .3045367E-03,.1033446E-02,.1175577E-02,.1135975E-02,.6197699E-03,&
     & .3473823E-03,.1215271E-02,.1382545E-02,.1325828E-02,.6926384E-03,&
     & .3970307E-03,.1437673E-02,.1633541E-02,.1556762E-02,.7746954E-03,&
     & .4539062E-03,.1708894E-02,.1936090E-02,.1834842E-02,.8669180E-03,&
     & .5165865E-03,.2034536E-02,.2299010E-02,.2167541E-02,.9692109E-03,&
     & .2472336E-03,.8613164E-03,.9823016E-03,.9514129E-03,.5196099E-03,&
     & .2828538E-03,.1020438E-02,.1163949E-02,.1118679E-02,.5847521E-03,&
     & .3237516E-03,.1216274E-02,.1386324E-02,.1324100E-02,.6591741E-03,&
     & .3714736E-03,.1458189E-02,.1657661E-02,.1574061E-02,.7425059E-03,&
     & .4245663E-03,.1749792E-02,.1985372E-02,.1874641E-02,.8368539E-03,&
     & .2006002E-03,.7166388E-03,.8191347E-03,.7953709E-03,.4345003E-03,&
     & .2300088E-03,.8550124E-03,.9779366E-03,.9420990E-03,.4927929E-03,&
     & .2640138E-03,.1027318E-02,.1174304E-02,.1124337E-02,.5593878E-03,&
     & .3036185E-03,.1241926E-02,.1416866E-02,.1348159E-02,.6350064E-03,&
     & .3483255E-03,.1503311E-02,.1712388E-02,.1619624E-02,.7206305E-03,&
     & .1620672E-03,.5924002E-03,.6788004E-03,.6609668E-03,.3615247E-03,&
     & .1863562E-03,.7120820E-03,.8162499E-03,.7885530E-03,.4130089E-03,&
     & .2144232E-03,.8626280E-03,.9887801E-03,.9492287E-03,.4724825E-03,&
     & .2473832E-03,.1051309E-02,.1203656E-02,.1147998E-02,.5403679E-03,&
     & .2847841E-03,.1284144E-02,.1468365E-02,.1391452E-02,.6179991E-03,&
     & .1299474E-03,.4846482E-03,.5565712E-03,.5436535E-03,.2982511E-03,&
     & .1498986E-03,.5864424E-03,.6736990E-03,.6529381E-03,.3434172E-03,&
     & .1729325E-03,.7163074E-03,.8229894E-03,.7924220E-03,.3957912E-03,&
     & .2000081E-03,.8797706E-03,.1010993E-02,.9670011E-03,.4564930E-03,&
     & .2313308E-03,.1085043E-02,.1245383E-02,.1182959E-02,.5264754E-03/

      data   absb(876:1050, 6)   /                                      &
     & .1041538E-03,.3951902E-03,.4548472E-03,.4454470E-03,.2452479E-03,&
     & .1203711E-03,.4813274E-03,.5542653E-03,.5387494E-03,.2845514E-03,&
     & .1392557E-03,.5925482E-03,.6823171E-03,.6591592E-03,.3305080E-03,&
     & .1615406E-03,.7342118E-03,.8463369E-03,.8118950E-03,.3843966E-03,&
     & .1875302E-03,.9137067E-03,.1052775E-02,.1002597E-02,.4471473E-03,&
     & .8339613E-04,.3212348E-03,.3702850E-03,.3635514E-03,.2009331E-03,&
     & .9663218E-04,.3936340E-03,.4542459E-03,.4427976E-03,.2348384E-03,&
     & .1119212E-03,.4884335E-03,.5635159E-03,.5461149E-03,.2749028E-03,&
     & .1301921E-03,.6106310E-03,.7057229E-03,.6791355E-03,.3226368E-03,&
     & .1518690E-03,.7666828E-03,.8867916E-03,.8470024E-03,.3788358E-03,&
     & .6703277E-04,.2615715E-03,.3020685E-03,.2972644E-03,.1647457E-03,&
     & .7781745E-04,.3226047E-03,.3728758E-03,.3643763E-03,.1940020E-03,&
     & .9035340E-04,.4034207E-03,.4665853E-03,.4531468E-03,.2290995E-03,&
     & .1051725E-03,.5090275E-03,.5895403E-03,.5690155E-03,.2711767E-03,&
     & .1231606E-03,.6451073E-03,.7487546E-03,.7170779E-03,.3213112E-03,&
     & .5388627E-04,.2129730E-03,.2462008E-03,.2426751E-03,.1352080E-03,&
     & .6277450E-04,.2640459E-03,.3057224E-03,.2995161E-03,.1599361E-03,&
     & .7304733E-04,.3329169E-03,.3859229E-03,.3754737E-03,.1906374E-03,&
     & .8515751E-04,.4241494E-03,.4919296E-03,.4761974E-03,.2276739E-03,&
     & .9991988E-04,.5432688E-03,.6320927E-03,.6069695E-03,.2723849E-03,&
     & .4326262E-04,.1729315E-03,.2000827E-03,.1973909E-03,.1105920E-03,&
     & .5057315E-04,.2155302E-03,.2498708E-03,.2454105E-03,.1317820E-03,&
     & .5902025E-04,.2738145E-03,.3180329E-03,.3099867E-03,.1580455E-03,&
     & .6892231E-04,.3521416E-03,.4093245E-03,.3971150E-03,.1905530E-03,&
     & .8105656E-04,.4558123E-03,.5314348E-03,.5116504E-03,.2300912E-03,&
     & .3466273E-04,.1400139E-03,.1619739E-03,.1598674E-03,.9004375E-04,&
     & .4063512E-04,.1754360E-03,.2034983E-03,.2003206E-03,.1082624E-03,&
     & .4757947E-04,.2243047E-03,.2609699E-03,.2549037E-03,.1307310E-03,&
     & .5575140E-04,.2912187E-03,.3392706E-03,.3296565E-03,.1588775E-03,&
     & .6571337E-04,.3810311E-03,.4450150E-03,.4296054E-03,.1937683E-03,&
     & .2781043E-04,.1138793E-03,.1316412E-03,.1298484E-03,.7350396E-04,&
     & .3277645E-04,.1433051E-03,.1663826E-03,.1640294E-03,.8913117E-04,&
     & .3850357E-04,.1846844E-03,.2150481E-03,.2104981E-03,.1086380E-03,&
     & .4522837E-04,.2419881E-03,.2824462E-03,.2747396E-03,.1329710E-03,&
     & .5347828E-04,.3200061E-03,.3744878E-03,.3622497E-03,.1637392E-03/

      data   absb(1051:1175, 6)   /                                     &
     & .2232236E-04,.9267720E-04,.1070487E-03,.1054862E-03,.5992194E-04,&
     & .2641653E-04,.1172861E-03,.1361391E-03,.1343624E-03,.7334096E-04,&
     & .3117369E-04,.1521353E-03,.1772789E-03,.1738983E-03,.9024944E-04,&
     & .3674804E-04,.2011031E-03,.2351698E-03,.2290276E-03,.1115235E-03,&
     & .4354793E-04,.2690881E-03,.3153806E-03,.3055401E-03,.1384935E-03,&
     & .1785498E-04,.7521527E-04,.8673443E-04,.8536647E-04,.4869020E-04,&
     & .2122523E-04,.9568767E-04,.1111031E-03,.1096456E-03,.6010776E-04,&
     & .2515713E-04,.1249154E-03,.1456399E-03,.1431298E-03,.7472628E-04,&
     & .2980978E-04,.1668004E-03,.1951256E-03,.1902764E-03,.9333750E-04,&
     & .3542498E-04,.2255442E-03,.2648513E-03,.2567580E-03,.1168654E-03,&
     & .1425709E-04,.6087198E-04,.7014411E-04,.6881006E-04,.3939206E-04,&
     & .1702198E-04,.7779210E-04,.9029022E-04,.8913052E-04,.4908012E-04,&
     & .2026100E-04,.1023006E-03,.1192918E-03,.1173391E-03,.6161544E-04,&
     & .2410865E-04,.1377899E-03,.1613227E-03,.1576056E-03,.7784401E-04,&
     & .2878722E-04,.1883410E-03,.2215045E-03,.2149526E-03,.9849260E-04,&
     & .1138303E-04,.4934557E-04,.5673238E-04,.5553184E-04,.3186568E-04,&
     & .1366146E-04,.6335560E-04,.7355595E-04,.7250807E-04,.4005918E-04,&
     & .1632332E-04,.8387900E-04,.9788807E-04,.9633384E-04,.5083176E-04,&
     & .1950274E-04,.1140523E-03,.1335383E-03,.1306879E-03,.6491699E-04,&
     & .2338070E-04,.1577480E-03,.1855710E-03,.1802136E-03,.8318615E-04,&
     & .9393833E-05,.4172083E-04,.4794329E-04,.4688225E-04,.2676670E-04,&
     & .1131203E-04,.5413446E-04,.6285587E-04,.6190797E-04,.3404469E-04,&
     & .1356891E-04,.7245624E-04,.8467100E-04,.8329182E-04,.4370019E-04,&
     & .1627959E-04,.9993465E-04,.1171212E-03,.1145831E-03,.5652037E-04,&
     & .1962119E-04,.1401179E-03,.1650762E-03,.1601642E-03,.7346404E-04/

      data   absb(  1:175, 7)   /                                       &
     & .3229900E+00,.5476000E+00,.6007900E+00,.5851700E+00,.4227600E+00,&
     & .3259400E+00,.5596300E+00,.6165300E+00,.6014700E+00,.4328100E+00,&
     & .3305600E+00,.5718200E+00,.6323600E+00,.6186300E+00,.4414900E+00,&
     & .3370600E+00,.5842600E+00,.6482200E+00,.6348900E+00,.4489300E+00,&
     & .3463500E+00,.5973900E+00,.6634300E+00,.6500300E+00,.4548600E+00,&
     & .2718500E+00,.4739600E+00,.5208700E+00,.5051800E+00,.3589800E+00,&
     & .2745500E+00,.4845400E+00,.5347300E+00,.5205800E+00,.3672500E+00,&
     & .2787800E+00,.4952300E+00,.5487600E+00,.5355600E+00,.3747700E+00,&
     & .2854000E+00,.5066000E+00,.5624400E+00,.5494800E+00,.3809500E+00,&
     & .2947400E+00,.5187600E+00,.5759200E+00,.5625500E+00,.3866000E+00,&
     & .2281800E+00,.4082500E+00,.4500300E+00,.4358200E+00,.3035000E+00,&
     & .2307400E+00,.4174900E+00,.4622000E+00,.4493000E+00,.3106400E+00,&
     & .2351000E+00,.4273700E+00,.4743400E+00,.4620200E+00,.3169600E+00,&
     & .2418600E+00,.4378300E+00,.4863700E+00,.4738700E+00,.3226500E+00,&
     & .2512100E+00,.4494300E+00,.4985500E+00,.4851900E+00,.3281600E+00,&
     & .1912000E+00,.3497500E+00,.3873200E+00,.3750900E+00,.2561700E+00,&
     & .1938500E+00,.3582600E+00,.3979900E+00,.3866600E+00,.2625900E+00,&
     & .1984600E+00,.3672700E+00,.4087200E+00,.3975000E+00,.2681500E+00,&
     & .2053700E+00,.3771200E+00,.4195400E+00,.4077700E+00,.2733000E+00,&
     & .2142700E+00,.3886800E+00,.4307100E+00,.4178400E+00,.2785300E+00,&
     & .1598800E+00,.2985400E+00,.3318000E+00,.3220100E+00,.2161100E+00,&
     & .1627900E+00,.3062800E+00,.3412900E+00,.3318800E+00,.2216200E+00,&
     & .1676300E+00,.3146300E+00,.3508500E+00,.3412200E+00,.2266400E+00,&
     & .1744400E+00,.3243200E+00,.3606800E+00,.3503200E+00,.2316100E+00,&
     & .1827400E+00,.3360900E+00,.3714000E+00,.3594400E+00,.2366500E+00,&
     & .1335200E+00,.2540400E+00,.2833000E+00,.2757700E+00,.1822500E+00,&
     & .1366700E+00,.2610300E+00,.2917100E+00,.2843100E+00,.1871200E+00,&
     & .1416400E+00,.2691000E+00,.3003000E+00,.2925400E+00,.1917600E+00,&
     & .1480900E+00,.2788600E+00,.3095900E+00,.3007300E+00,.1963700E+00,&
     & .1555100E+00,.2905000E+00,.3202000E+00,.3093000E+00,.2011500E+00,&
     & .1117700E+00,.2158000E+00,.2413600E+00,.2355200E+00,.1539200E+00,&
     & .1150400E+00,.2224300E+00,.2489000E+00,.2430400E+00,.1583300E+00,&
     & .1197500E+00,.2303100E+00,.2568700E+00,.2504800E+00,.1624400E+00,&
     & .1255300E+00,.2399400E+00,.2658800E+00,.2581200E+00,.1667100E+00,&
     & .1320400E+00,.2510700E+00,.2763900E+00,.2662900E+00,.1711000E+00/

      data   absb(176:350, 7)   /                                       &
     & .9381800E-01,.1833600E+00,.2053300E+00,.2008300E+00,.1306300E+00,&
     & .9715300E-01,.1897500E+00,.2122200E+00,.2075400E+00,.1344500E+00,&
     & .1015700E+00,.1975900E+00,.2198900E+00,.2143900E+00,.1381900E+00,&
     & .1065600E+00,.2069200E+00,.2288500E+00,.2217000E+00,.1420000E+00,&
     & .1121700E+00,.2174100E+00,.2391400E+00,.2297200E+00,.1459500E+00,&
     & .7886400E-01,.1558100E+00,.1746700E+00,.1711100E+00,.1109300E+00,&
     & .8211500E-01,.1621300E+00,.1811400E+00,.1772200E+00,.1144800E+00,&
     & .8609100E-01,.1698900E+00,.1885800E+00,.1836800E+00,.1179500E+00,&
     & .9054400E-01,.1788300E+00,.1974000E+00,.1907500E+00,.1214000E+00,&
     & .9553100E-01,.1885700E+00,.2071900E+00,.1987200E+00,.1249400E+00,&
     & .6653600E-01,.1329300E+00,.1491300E+00,.1461800E+00,.9442900E-01,&
     & .6963900E-01,.1393100E+00,.1553700E+00,.1518700E+00,.9768000E-01,&
     & .7319400E-01,.1468800E+00,.1628200E+00,.1580500E+00,.1008700E+00,&
     & .7720500E-01,.1553600E+00,.1713500E+00,.1650300E+00,.1041400E+00,&
     & .8166700E-01,.1642700E+00,.1806600E+00,.1729100E+00,.1074400E+00,&
     & .5626100E-01,.1138300E+00,.1276000E+00,.1251200E+00,.8055900E-01,&
     & .5908600E-01,.1201400E+00,.1338400E+00,.1304800E+00,.8348500E-01,&
     & .6228100E-01,.1273800E+00,.1412100E+00,.1364800E+00,.8646000E-01,&
     & .6590600E-01,.1351700E+00,.1493700E+00,.1433600E+00,.8946200E-01,&
     & .6999800E-01,.1433500E+00,.1580700E+00,.1509800E+00,.9258800E-01,&
     & .4764200E-01,.9793300E-01,.1096300E+00,.1074200E+00,.6875000E-01,&
     & .5019300E-01,.1039900E+00,.1158700E+00,.1125800E+00,.7143200E-01,&
     & .5308500E-01,.1107700E+00,.1229800E+00,.1184500E+00,.7419400E-01,&
     & .5638400E-01,.1179500E+00,.1307000E+00,.1251200E+00,.7697300E-01,&
     & .6017400E-01,.1255500E+00,.1387900E+00,.1323700E+00,.7992000E-01,&
     & .4043200E-01,.8464500E-01,.9472400E-01,.9263700E-01,.5880500E-01,&
     & .4274800E-01,.9040900E-01,.1007900E+00,.9765400E-01,.6128900E-01,&
     & .4539200E-01,.9668200E-01,.1075600E+00,.1034100E+00,.6381800E-01,&
     & .4843800E-01,.1033500E+00,.1147800E+00,.1097800E+00,.6648100E-01,&
     & .5201900E-01,.1105000E+00,.1223400E+00,.1166400E+00,.6920900E-01,&
     & .3443000E-01,.7356700E-01,.8234900E-01,.8034100E-01,.5048500E-01,&
     & .3655800E-01,.7899700E-01,.8816300E-01,.8529300E-01,.5283500E-01,&
     & .3900900E-01,.8481800E-01,.9452100E-01,.9085100E-01,.5518100E-01,&
     & .4187600E-01,.9107400E-01,.1012700E+00,.9692200E-01,.5764400E-01,&
     & .4529400E-01,.9787400E-01,.1084100E+00,.1034000E+00,.6025800E-01/

      data   absb(351:525, 7)   /                                       &
     & .2941800E-01,.6428900E-01,.7198400E-01,.7011200E-01,.4351400E-01,&
     & .3136700E-01,.6933300E-01,.7750000E-01,.7494500E-01,.4565400E-01,&
     & .3365000E-01,.7477900E-01,.8345100E-01,.8028400E-01,.4783800E-01,&
     & .3637300E-01,.8069800E-01,.8980300E-01,.8603500E-01,.5018800E-01,&
     & .3962500E-01,.8722700E-01,.9660500E-01,.9216300E-01,.5266100E-01,&
     & .2520600E-01,.5643600E-01,.6328200E-01,.6157100E-01,.3760200E-01,&
     & .2700900E-01,.6114700E-01,.6848100E-01,.6623700E-01,.3956200E-01,&
     & .2915000E-01,.6630200E-01,.7407400E-01,.7134100E-01,.4160100E-01,&
     & .3175000E-01,.7195500E-01,.8010400E-01,.7678500E-01,.4379400E-01,&
     & .3483400E-01,.7826800E-01,.8666200E-01,.8263200E-01,.4619300E-01,&
     & .2167200E-01,.4982100E-01,.5598500E-01,.5443100E-01,.3267700E-01,&
     & .2336000E-01,.5425700E-01,.6087100E-01,.5891800E-01,.3445300E-01,&
     & .2542000E-01,.5915900E-01,.6616800E-01,.6376500E-01,.3639800E-01,&
     & .2790800E-01,.6462900E-01,.7193500E-01,.6897700E-01,.3849000E-01,&
     & .3084300E-01,.7079100E-01,.7832300E-01,.7461000E-01,.4074700E-01,&
     & .1870300E-01,.4423600E-01,.4981900E-01,.4842300E-01,.2850300E-01,&
     & .2031500E-01,.4844500E-01,.5442500E-01,.5272000E-01,.3017400E-01,&
     & .2230700E-01,.5316300E-01,.5948100E-01,.5735900E-01,.3201800E-01,&
     & .2468700E-01,.5849500E-01,.6506400E-01,.6236800E-01,.3402200E-01,&
     & .2751000E-01,.6459700E-01,.7135200E-01,.6786000E-01,.3624300E-01,&
     & .1622700E-01,.3954600E-01,.4461300E-01,.4338400E-01,.2494700E-01,&
     & .1779100E-01,.4357600E-01,.4901000E-01,.4750100E-01,.2657700E-01,&
     & .1972000E-01,.4816300E-01,.5387700E-01,.5196200E-01,.2834700E-01,&
     & .2201000E-01,.5341900E-01,.5936800E-01,.5684200E-01,.3032600E-01,&
     & .2477800E-01,.5953300E-01,.6558200E-01,.6227900E-01,.3246400E-01,&
     & .1416800E-01,.3561000E-01,.4022800E-01,.3914500E-01,.2199800E-01,&
     & .1570100E-01,.3951800E-01,.4445500E-01,.4310800E-01,.2355100E-01,&
     & .1756800E-01,.4402800E-01,.4922800E-01,.4744200E-01,.2530300E-01,&
     & .1982200E-01,.4927400E-01,.5464400E-01,.5226900E-01,.2721900E-01,&
     & .2249200E-01,.5542900E-01,.6087800E-01,.5769600E-01,.2928900E-01,&
     & .1246900E-01,.3232100E-01,.3654800E-01,.3558600E-01,.1953500E-01,&
     & .1396700E-01,.3616000E-01,.4067000E-01,.3941400E-01,.2105700E-01,&
     & .1580400E-01,.4064600E-01,.4537100E-01,.4369700E-01,.2274800E-01,&
     & .1800700E-01,.4593500E-01,.5079000E-01,.4850400E-01,.2458900E-01,&
     & .2058300E-01,.5211700E-01,.5708300E-01,.5394900E-01,.2661200E-01/

      data   absb(526:700, 7)   /                                       &
     & .1103000E-01,.2946200E-01,.3333300E-01,.3245700E-01,.1744100E-01,&
     & .1248500E-01,.3325200E-01,.3737400E-01,.3620300E-01,.1890200E-01,&
     & .1428400E-01,.3773500E-01,.4203900E-01,.4044900E-01,.2053300E-01,&
     & .1640600E-01,.4305000E-01,.4747300E-01,.4525000E-01,.2231500E-01,&
     & .1888900E-01,.4922100E-01,.5383800E-01,.5070900E-01,.2429500E-01,&
     & .9704500E-02,.2676500E-01,.3029400E-01,.2949400E-01,.1552700E-01,&
     & .1111600E-01,.3047100E-01,.3423700E-01,.3315600E-01,.1693600E-01,&
     & .1283300E-01,.3490800E-01,.3882700E-01,.3733500E-01,.1849400E-01,&
     & .1485100E-01,.4016000E-01,.4422800E-01,.4209500E-01,.2023000E-01,&
     & .1722000E-01,.4627500E-01,.5060000E-01,.4752400E-01,.2213700E-01,&
     & .8478700E-02,.2413100E-01,.2733900E-01,.2661900E-01,.1375600E-01,&
     & .9803300E-02,.2770600E-01,.3114700E-01,.3016600E-01,.1510600E-01,&
     & .1140700E-01,.3201500E-01,.3561000E-01,.3423900E-01,.1659300E-01,&
     & .1330200E-01,.3714000E-01,.4090700E-01,.3890400E-01,.1825700E-01,&
     & .1553700E-01,.4312900E-01,.4718200E-01,.4423600E-01,.2007400E-01,&
     & .7268500E-02,.2139200E-01,.2430100E-01,.2367900E-01,.1208700E-01,&
     & .8473300E-02,.2475900E-01,.2789800E-01,.2704600E-01,.1334200E-01,&
     & .9935300E-02,.2884800E-01,.3215200E-01,.3094000E-01,.1474800E-01,&
     & .1167300E-01,.3373400E-01,.3722400E-01,.3542200E-01,.1631900E-01,&
     & .1374000E-01,.3948100E-01,.4325600E-01,.4056600E-01,.1804900E-01,&
     & .6232600E-02,.1899900E-01,.2164400E-01,.2110300E-01,.1062900E-01,&
     & .7326900E-02,.2217000E-01,.2504200E-01,.2429900E-01,.1180700E-01,&
     & .8658600E-02,.2604500E-01,.2909800E-01,.2801900E-01,.1313800E-01,&
     & .1025400E-01,.3069900E-01,.3395200E-01,.3231600E-01,.1462100E-01,&
     & .1216800E-01,.3621100E-01,.3974800E-01,.3728800E-01,.1626300E-01,&
     & .5352200E-02,.1692500E-01,.1933700E-01,.1886300E-01,.9369700E-02,&
     & .6343700E-02,.1991600E-01,.2255500E-01,.2190000E-01,.1047700E-01,&
     & .7560000E-02,.2359000E-01,.2642800E-01,.2545300E-01,.1173200E-01,&
     & .9027700E-02,.2802700E-01,.3108100E-01,.2958100E-01,.1314000E-01,&
     & .1080400E-01,.3332500E-01,.3665500E-01,.3439600E-01,.1471400E-01,&
     & .4518300E-02,.1486200E-01,.1704300E-01,.1664000E-01,.8190600E-02,&
     & .5396400E-02,.1763000E-01,.2004300E-01,.1948200E-01,.9233500E-02,&
     & .6483500E-02,.2104100E-01,.2367400E-01,.2282100E-01,.1040000E-01,&
     & .7810000E-02,.2519700E-01,.2805600E-01,.2673300E-01,.1171800E-01,&
     & .9431000E-02,.3020300E-01,.3332200E-01,.3132100E-01,.1320900E-01/

      data   absb(701:875, 7)   /                                       &
     & .3802000E-02,.1303500E-01,.1500700E-01,.1466500E-01,.7164500E-02,&
     & .4579600E-02,.1559200E-01,.1780200E-01,.1732000E-01,.8133200E-02,&
     & .5546400E-02,.1874900E-01,.2119100E-01,.2044800E-01,.9231200E-02,&
     & .6740800E-02,.2262900E-01,.2530900E-01,.2414300E-01,.1045800E-01,&
     & .8215500E-02,.2734700E-01,.3027700E-01,.2850600E-01,.1187100E-01,&
     & .3196200E-02,.1144400E-01,.1322900E-01,.1293700E-01,.6278800E-02,&
     & .3881500E-02,.1379800E-01,.1582200E-01,.1540700E-01,.7177400E-02,&
     & .4742200E-02,.1672200E-01,.1899000E-01,.1833900E-01,.8198600E-02,&
     & .5815400E-02,.2034200E-01,.2285700E-01,.2183000E-01,.9359400E-02,&
     & .7154600E-02,.2478700E-01,.2754500E-01,.2597300E-01,.1068400E-01,&
     & .2654200E-02,.9942100E-02,.1154500E-01,.1129700E-01,.5468100E-02,&
     & .3246700E-02,.1208500E-01,.1392600E-01,.1357500E-01,.6301700E-02,&
     & .4003700E-02,.1475800E-01,.1684800E-01,.1629600E-01,.7243400E-02,&
     & .4953400E-02,.1809400E-01,.2043800E-01,.1955600E-01,.8335400E-02,&
     & .6152500E-02,.2223000E-01,.2481700E-01,.2345000E-01,.9575100E-02,&
     & .2188100E-02,.8585700E-02,.1001900E-01,.9804300E-02,.4745600E-02,&
     & .2693700E-02,.1052500E-01,.1219200E-01,.1189800E-01,.5516800E-02,&
     & .3353100E-02,.1295200E-01,.1487000E-01,.1440800E-01,.6393800E-02,&
     & .4191600E-02,.1601600E-01,.1818700E-01,.1744100E-01,.7399500E-02,&
     & .5252900E-02,.1982500E-01,.2224500E-01,.2107400E-01,.8569300E-02,&
     & .1797400E-02,.7403600E-02,.8681900E-02,.8494900E-02,.4110000E-02,&
     & .2228300E-02,.9155600E-02,.1066400E-01,.1041400E-01,.4825700E-02,&
     & .2799300E-02,.1136000E-01,.1311500E-01,.1273100E-01,.5642300E-02,&
     & .3535800E-02,.1416400E-01,.1617000E-01,.1554300E-01,.6586700E-02,&
     & .4476200E-02,.1767100E-01,.1992800E-01,.1893100E-01,.7672500E-02,&
     & .1466600E-02,.6344300E-02,.7471000E-02,.7309900E-02,.3554000E-02,&
     & .1828600E-02,.7914000E-02,.9271900E-02,.9058800E-02,.4198100E-02,&
     & .2319600E-02,.9908900E-02,.1150800E-01,.1119200E-01,.4965600E-02,&
     & .2957500E-02,.1245100E-01,.1429600E-01,.1377900E-01,.5844100E-02,&
     & .3784200E-02,.1566200E-01,.1775800E-01,.1692100E-01,.6866000E-02,&
     & .1182200E-02,.5370000E-02,.6348300E-02,.6209000E-02,.3050000E-02,&
     & .1481700E-02,.6761600E-02,.7967800E-02,.7787400E-02,.3631700E-02,&
     & .1891800E-02,.8539200E-02,.9981800E-02,.9725400E-02,.4324800E-02,&
     & .2438200E-02,.1082000E-01,.1250300E-01,.1208800E-01,.5147700E-02,&
     & .3153300E-02,.1372500E-01,.1565600E-01,.1497000E-01,.6100000E-02/

      data   absb(876:1050, 7)   /                                      &
     & .9490600E-03,.4529800E-02,.5370600E-02,.5250600E-02,.2616200E-02,&
     & .1196500E-02,.5759700E-02,.6821300E-02,.6669800E-02,.3140300E-02,&
     & .1536900E-02,.7338000E-02,.8636800E-02,.8424600E-02,.3768700E-02,&
     & .2000800E-02,.9381400E-02,.1091100E-01,.1058000E-01,.4521300E-02,&
     & .2615900E-02,.1200200E-01,.1377400E-01,.1322100E-01,.5414200E-02,&
     & .7591300E-03,.3804300E-02,.4520300E-02,.4418100E-02,.2237900E-02,&
     & .9609700E-03,.4889900E-02,.5813800E-02,.5687800E-02,.2709800E-02,&
     & .1244700E-02,.6289500E-02,.7450100E-02,.7272700E-02,.3285900E-02,&
     & .1632200E-02,.8112200E-02,.9498900E-02,.9234900E-02,.3972000E-02,&
     & .2159700E-02,.1047000E-01,.1209200E-01,.1165000E-01,.4793800E-02,&
     & .6094900E-03,.3200900E-02,.3808000E-02,.3720400E-02,.1914800E-02,&
     & .7733200E-03,.4163700E-02,.4965800E-02,.4858800E-02,.2349800E-02,&
     & .1009900E-02,.5409500E-02,.6441600E-02,.6293800E-02,.2871300E-02,&
     & .1335300E-02,.7041700E-02,.8300400E-02,.8084400E-02,.3511400E-02,&
     & .1787300E-02,.9169100E-02,.1065800E-01,.1030300E-01,.4264100E-02,&
     & .4896000E-03,.2687300E-02,.3200800E-02,.3125900E-02,.1637800E-02,&
     & .6218700E-03,.3542300E-02,.4234900E-02,.4144300E-02,.2032500E-02,&
     & .8178800E-03,.4652900E-02,.5565600E-02,.5442400E-02,.2516400E-02,&
     & .1093500E-02,.6115500E-02,.7256500E-02,.7076500E-02,.3100700E-02,&
     & .1477600E-02,.8041600E-02,.9408700E-02,.9119700E-02,.3808500E-02,&
     & .3930200E-03,.2243500E-02,.2675400E-02,.2612200E-02,.1389800E-02,&
     & .4988900E-03,.3000600E-02,.3594300E-02,.3517200E-02,.1753200E-02,&
     & .6592200E-03,.3990600E-02,.4790100E-02,.4688000E-02,.2198000E-02,&
     & .8907500E-03,.5298000E-02,.6323700E-02,.6174800E-02,.2741200E-02,&
     & .1215000E-02,.7031200E-02,.8283700E-02,.8049700E-02,.3395600E-02,&
     & .3153300E-03,.1859800E-02,.2221600E-02,.2169000E-02,.1170500E-02,&
     & .3991900E-03,.2529000E-02,.3033800E-02,.2967900E-02,.1505100E-02,&
     & .5295800E-03,.3409600E-02,.4103000E-02,.4018900E-02,.1913700E-02,&
     & .7214700E-03,.4575600E-02,.5489400E-02,.5367800E-02,.2414700E-02,&
     & .9956300E-03,.6132100E-02,.7273900E-02,.7082600E-02,.3028600E-02,&
     & .2543400E-03,.1547700E-02,.1850200E-02,.1807500E-02,.9889100E-03,&
     & .3222100E-03,.2139800E-02,.2570800E-02,.2514800E-02,.1293500E-02,&
     & .4277500E-03,.2927200E-02,.3531500E-02,.3459900E-02,.1674100E-02,&
     & .5876200E-03,.3976100E-02,.4790700E-02,.4689600E-02,.2137600E-02,&
     & .8205300E-03,.5382400E-02,.6424400E-02,.6266400E-02,.2717300E-02/

      data   absb(1051:1175, 7)   /                                     &
     & .2061800E-03,.1286600E-02,.1540000E-02,.1505500E-02,.8357200E-03,&
     & .2605900E-03,.1809400E-02,.2178700E-02,.2130200E-02,.1109000E-02,&
     & .3465100E-03,.2514000E-02,.3039200E-02,.2978000E-02,.1462800E-02,&
     & .4788200E-03,.3460800E-02,.4183100E-02,.4100300E-02,.1899200E-02,&
     & .6767100E-03,.4733100E-02,.5681700E-02,.5550300E-02,.2436000E-02,&
     & .1668100E-03,.1062100E-02,.1272800E-02,.1246100E-02,.7016200E-03,&
     & .2106400E-03,.1521300E-02,.1835000E-02,.1794500E-02,.9482800E-03,&
     & .2802800E-03,.2150000E-02,.2604500E-02,.2552100E-02,.1272100E-02,&
     & .3890600E-03,.3002900E-02,.3638700E-02,.3571000E-02,.1683300E-02,&
     & .5556800E-03,.4153400E-02,.5011100E-02,.4902700E-02,.2184900E-02,&
     & .1348400E-03,.8716800E-03,.1044400E-02,.1024800E-02,.5851600E-03,&
     & .1700700E-03,.1271700E-02,.1536000E-02,.1502400E-02,.8080500E-03,&
     & .2260300E-03,.1829600E-02,.2221100E-02,.2175700E-02,.1100600E-02,&
     & .3156500E-03,.2596300E-02,.3153100E-02,.3095900E-02,.1485100E-02,&
     & .4541700E-03,.3636500E-02,.4405800E-02,.4315700E-02,.1962500E-02,&
     & .1093800E-03,.7144900E-03,.8565500E-03,.8424000E-03,.4877200E-03,&
     & .1375200E-03,.1062400E-02,.1284900E-02,.1258300E-02,.6868900E-03,&
     & .1830700E-03,.1557200E-02,.1895400E-02,.1856600E-02,.9534400E-03,&
     & .2567300E-03,.2248100E-02,.2736300E-02,.2687700E-02,.1307900E-02,&
     & .3723000E-03,.3192100E-02,.3881200E-02,.3807000E-02,.1757000E-02,&
     & .9215300E-04,.6279800E-03,.7537400E-03,.7424300E-03,.4316100E-03,&
     & .1167600E-03,.9501900E-03,.1152500E-02,.1129600E-02,.6195900E-03,&
     & .1568700E-03,.1416800E-02,.1728500E-02,.1693800E-02,.8746800E-03,&
     & .2227800E-03,.2074300E-02,.2530200E-02,.2486300E-02,.1216100E-02,&
     & .3272400E-03,.2979700E-02,.3631300E-02,.3566400E-02,.1657700E-02/

      data   absb(  1:175, 8)   /                                       &
     & .2147772E+01,.3506523E+01,.4007672E+01,.4063247E+01,.2937459E+01,&
     & .2178726E+01,.3529383E+01,.4034995E+01,.4103957E+01,.3003216E+01,&
     & .2233362E+01,.3568760E+01,.4070770E+01,.4144550E+01,.3061714E+01,&
     & .2307004E+01,.3623393E+01,.4115151E+01,.4185941E+01,.3115511E+01,&
     & .2388633E+01,.3691850E+01,.4168630E+01,.4227592E+01,.3167839E+01,&
     & .1860982E+01,.3098016E+01,.3558079E+01,.3627319E+01,.2627403E+01,&
     & .1905280E+01,.3134646E+01,.3595375E+01,.3671395E+01,.2687036E+01,&
     & .1969054E+01,.3186559E+01,.3641361E+01,.3716596E+01,.2741361E+01,&
     & .2041391E+01,.3252536E+01,.3696281E+01,.3763063E+01,.2795119E+01,&
     & .2116579E+01,.3332677E+01,.3758858E+01,.3812101E+01,.2847416E+01,&
     & .1617458E+01,.2741093E+01,.3152632E+01,.3221428E+01,.2339877E+01,&
     & .1670160E+01,.2788238E+01,.3198512E+01,.3269657E+01,.2396388E+01,&
     & .1732677E+01,.2849720E+01,.3253678E+01,.3319736E+01,.2449246E+01,&
     & .1797925E+01,.2926437E+01,.3317317E+01,.3372271E+01,.2501129E+01,&
     & .1862880E+01,.3012958E+01,.3388595E+01,.3427980E+01,.2552877E+01,&
     & .1410490E+01,.2430273E+01,.2789567E+01,.2851573E+01,.2070884E+01,&
     & .1462546E+01,.2486398E+01,.2843042E+01,.2903268E+01,.2125260E+01,&
     & .1517639E+01,.2557586E+01,.2906093E+01,.2957563E+01,.2178494E+01,&
     & .1573385E+01,.2638614E+01,.2977318E+01,.3015588E+01,.2231226E+01,&
     & .1630346E+01,.2723778E+01,.3055751E+01,.3077711E+01,.2281963E+01,&
     & .1230541E+01,.2156310E+01,.2468500E+01,.2517143E+01,.1827564E+01,&
     & .1276117E+01,.2220980E+01,.2528907E+01,.2572462E+01,.1880843E+01,&
     & .1322689E+01,.2296509E+01,.2597740E+01,.2631249E+01,.1932799E+01,&
     & .1370777E+01,.2376996E+01,.2674941E+01,.2694611E+01,.1983918E+01,&
     & .1421644E+01,.2457551E+01,.2758874E+01,.2763587E+01,.2034867E+01,&
     & .1070969E+01,.1915020E+01,.2186826E+01,.2218812E+01,.1611091E+01,&
     & .1109526E+01,.1984117E+01,.2252013E+01,.2276826E+01,.1661713E+01,&
     & .1149568E+01,.2059351E+01,.2326295E+01,.2340036E+01,.1710989E+01,&
     & .1192105E+01,.2135358E+01,.2407183E+01,.2409247E+01,.1761075E+01,&
     & .1238203E+01,.2211009E+01,.2491678E+01,.2482619E+01,.1811988E+01,&
     & .9283756E+00,.1702510E+01,.1938601E+01,.1955270E+01,.1419882E+01,&
     & .9616580E+00,.1771777E+01,.2008679E+01,.2016979E+01,.1468037E+01,&
     & .9971495E+00,.1842985E+01,.2085977E+01,.2084514E+01,.1515441E+01,&
     & .1035954E+01,.1914171E+01,.2167568E+01,.2156971E+01,.1563415E+01,&
     & .1079188E+01,.1986694E+01,.2252099E+01,.2233773E+01,.1612851E+01/

      data   absb(176:350, 8)   /                                       &
     & .8036031E+00,.1516186E+01,.1721627E+01,.1727213E+01,.1252674E+01,&
     & .8326793E+00,.1581949E+01,.1794758E+01,.1792292E+01,.1298932E+01,&
     & .8648349E+00,.1648188E+01,.1873088E+01,.1862805E+01,.1346221E+01,&
     & .9010472E+00,.1716012E+01,.1954493E+01,.1937771E+01,.1392209E+01,&
     & .9422352E+00,.1787156E+01,.2036790E+01,.2017682E+01,.1438445E+01,&
     & .6953625E+00,.1350659E+01,.1532717E+01,.1530267E+01,.1104610E+01,&
     & .7214726E+00,.1411984E+01,.1606492E+01,.1597505E+01,.1150564E+01,&
     & .7509696E+00,.1474817E+01,.1683945E+01,.1669864E+01,.1196729E+01,&
     & .7851546E+00,.1540757E+01,.1763201E+01,.1747500E+01,.1242539E+01,&
     & .8247478E+00,.1610749E+01,.1844223E+01,.1828234E+01,.1287882E+01,&
     & .6026917E+00,.1207364E+01,.1372521E+01,.1364577E+01,.9775652E+00,&
     & .6273178E+00,.1265727E+01,.1445916E+01,.1433870E+01,.1022600E+01,&
     & .6557573E+00,.1326639E+01,.1521041E+01,.1508141E+01,.1067263E+01,&
     & .6889575E+00,.1391759E+01,.1598467E+01,.1586453E+01,.1112837E+01,&
     & .7278457E+00,.1461987E+01,.1679430E+01,.1668167E+01,.1158214E+01,&
     & .5220814E+00,.1081272E+01,.1234021E+01,.1222809E+01,.8668184E+00,&
     & .5462161E+00,.1137925E+01,.1304901E+01,.1293790E+01,.9111319E+00,&
     & .5743245E+00,.1198422E+01,.1378329E+01,.1368895E+01,.9551067E+00,&
     & .6074624E+00,.1263493E+01,.1455169E+01,.1447720E+01,.9999031E+00,&
     & .6464214E+00,.1334732E+01,.1536800E+01,.1530362E+01,.1044710E+01,&
     & .4530676E+00,.9718752E+00,.1113625E+01,.1102459E+01,.7702607E+00,&
     & .4767827E+00,.1027819E+01,.1182580E+01,.1173836E+01,.8131121E+00,&
     & .5050006E+00,.1088357E+01,.1255221E+01,.1249573E+01,.8570364E+00,&
     & .5385629E+00,.1154487E+01,.1332423E+01,.1329078E+01,.9009956E+00,&
     & .5782279E+00,.1226660E+01,.1415277E+01,.1412212E+01,.9457995E+00,&
     & .3944211E+00,.8783220E+00,.1009831E+01,.1000358E+01,.6870193E+00,&
     & .4180031E+00,.9342002E+00,.1077916E+01,.1071914E+01,.7295172E+00,&
     & .4466039E+00,.9953298E+00,.1150465E+01,.1147983E+01,.7723351E+00,&
     & .4807452E+00,.1062397E+01,.1228665E+01,.1227974E+01,.8161121E+00,&
     & .5212184E+00,.1135828E+01,.1313016E+01,.1312663E+01,.8641655E+00,&
     & .3452462E+00,.7999293E+00,.9219804E+00,.9149658E+00,.6174465E+00,&
     & .3690573E+00,.8557897E+00,.9899741E+00,.9866927E+00,.6583574E+00,&
     & .3981461E+00,.9176898E+00,.1063178E+01,.1062821E+01,.7012026E+00,&
     & .4331021E+00,.9859104E+00,.1142768E+01,.1144162E+01,.7470311E+00,&
     & .4742396E+00,.1061101E+01,.1229416E+01,.1231158E+01,.7973609E+00/

      data   absb(351:525, 8)   /                                       &
     & .3040686E+00,.7336469E+00,.8476943E+00,.8430763E+00,.5574518E+00,&
     & .3283125E+00,.7901985E+00,.9160319E+00,.9148169E+00,.5982934E+00,&
     & .3580998E+00,.8530076E+00,.9905938E+00,.9917986E+00,.6421955E+00,&
     & .3937998E+00,.9226604E+00,.1072205E+01,.1075289E+01,.6908810E+00,&
     & .4350420E+00,.1000013E+01,.1161544E+01,.1165076E+01,.7420692E+00,&
     & .2699261E+00,.6782542E+00,.7854145E+00,.7829386E+00,.5064205E+00,&
     & .2947504E+00,.7357341E+00,.8549126E+00,.8551804E+00,.5487324E+00,&
     & .3252847E+00,.7997113E+00,.9311639E+00,.9339914E+00,.5953021E+00,&
     & .3613092E+00,.8712485E+00,.1015309E+01,.1020027E+01,.6445334E+00,&
     & .4023145E+00,.9510193E+00,.1107717E+01,.1113072E+01,.6967532E+00,&
     & .2420005E+00,.6327790E+00,.7343674E+00,.7337147E+00,.4659044E+00,&
     & .2675426E+00,.6914513E+00,.8056098E+00,.8073577E+00,.5100332E+00,&
     & .2985977E+00,.7570808E+00,.8841869E+00,.8886282E+00,.5568586E+00,&
     & .3345903E+00,.8307955E+00,.9712488E+00,.9777164E+00,.6069869E+00,&
     & .3752790E+00,.9132888E+00,.1067298E+01,.1074521E+01,.6606337E+00,&
     & .2193104E+00,.5956904E+00,.6933100E+00,.6941123E+00,.4338918E+00,&
     & .2454356E+00,.6558867E+00,.7665763E+00,.7700794E+00,.4783097E+00,&
     & .2766892E+00,.7234267E+00,.8479414E+00,.8542122E+00,.5258379E+00,&
     & .3125185E+00,.7996635E+00,.9383927E+00,.9468621E+00,.5771763E+00,&
     & .3530433E+00,.8851441E+00,.1038330E+01,.1047465E+01,.6323351E+00,&
     & .2011174E+00,.5662868E+00,.6614813E+00,.6638455E+00,.4081986E+00,&
     & .2276405E+00,.6282768E+00,.7371843E+00,.7426597E+00,.4531121E+00,&
     & .2588601E+00,.6982349E+00,.8218536E+00,.8302250E+00,.5017038E+00,&
     & .2946400E+00,.7772718E+00,.9161532E+00,.9266575E+00,.5545280E+00,&
     & .3351557E+00,.8661253E+00,.1020315E+01,.1031181E+01,.6114083E+00,&
     & .1865615E+00,.5436250E+00,.6378358E+00,.6421123E+00,.3878649E+00,&
     & .2133106E+00,.6078013E+00,.7166414E+00,.7241786E+00,.4336769E+00,&
     & .2445042E+00,.6804884E+00,.8049500E+00,.8154355E+00,.4837168E+00,&
     & .2803835E+00,.7627085E+00,.9033997E+00,.9157583E+00,.5381867E+00,&
     & .3210362E+00,.8552279E+00,.1011978E+01,.1024218E+01,.5970880E+00,&
     & .1749538E+00,.5269996E+00,.6215976E+00,.6279908E+00,.3723894E+00,&
     & .2017790E+00,.5937199E+00,.7039645E+00,.7136463E+00,.4194095E+00,&
     & .2331437E+00,.6694912E+00,.7962787E+00,.8088205E+00,.4711663E+00,&
     & .2692141E+00,.7551867E+00,.8989845E+00,.9129664E+00,.5275761E+00,&
     & .3102376E+00,.8516644E+00,.1011909E+01,.1025210E+01,.5885594E+00/

      data   absb(526:700, 8)   /                                       &
     & .1648200E+00,.5135795E+00,.6093601E+00,.6178369E+00,.3595625E+00,&
     & .1916764E+00,.5828141E+00,.6952491E+00,.7068257E+00,.4080742E+00,&
     & .2231330E+00,.6615194E+00,.7912997E+00,.8055439E+00,.4615211E+00,&
     & .2594329E+00,.7506484E+00,.8977476E+00,.9130454E+00,.5197857E+00,&
     & .3009092E+00,.8509191E+00,.1014517E+01,.1028676E+01,.5827702E+00,&
     & .1542942E+00,.4984661E+00,.5950172E+00,.6050995E+00,.3460966E+00,&
     & .1808778E+00,.5696361E+00,.6834611E+00,.6965516E+00,.3958165E+00,&
     & .2120868E+00,.6504539E+00,.7821549E+00,.7977925E+00,.4505357E+00,&
     & .2483187E+00,.7421169E+00,.8913076E+00,.9077047E+00,.5102050E+00,&
     & .2900951E+00,.8450562E+00,.1010731E+01,.1025679E+01,.5747009E+00,&
     & .1428838E+00,.4799142E+00,.5760617E+00,.5872644E+00,.3307220E+00,&
     & .1688185E+00,.5520986E+00,.6658687E+00,.6800494E+00,.3810810E+00,&
     & .1993677E+00,.6340252E+00,.7660136E+00,.7826397E+00,.4366263E+00,&
     & .2351442E+00,.7269414E+00,.8765519E+00,.8938786E+00,.4971268E+00,&
     & .2767765E+00,.8312265E+00,.9973781E+00,.1013193E+01,.5625664E+00,&
     & .1295255E+00,.4542258E+00,.5478587E+00,.5595381E+00,.3107326E+00,&
     & .1542935E+00,.5261193E+00,.6373454E+00,.6520435E+00,.3608896E+00,&
     & .1835749E+00,.6075514E+00,.7372141E+00,.7544331E+00,.4164074E+00,&
     & .2182062E+00,.6998975E+00,.8474540E+00,.8656087E+00,.4768929E+00,&
     & .2589005E+00,.8036978E+00,.9679606E+00,.9848442E+00,.5424313E+00,&
     & .1175401E+00,.4307071E+00,.5219632E+00,.5338978E+00,.2926245E+00,&
     & .1412629E+00,.5022212E+00,.6108981E+00,.6259401E+00,.3424732E+00,&
     & .1693284E+00,.5830370E+00,.7102630E+00,.7278824E+00,.3978669E+00,&
     & .2028442E+00,.6746033E+00,.8199129E+00,.8387792E+00,.4582588E+00,&
     & .2426035E+00,.7776740E+00,.9399588E+00,.9577662E+00,.5237233E+00,&
     & .1069057E+00,.4095696E+00,.4986606E+00,.5107070E+00,.2765051E+00,&
     & .1297183E+00,.4806709E+00,.5869321E+00,.6022485E+00,.3260232E+00,&
     & .1566724E+00,.5608628E+00,.6856622E+00,.7036230E+00,.3812308E+00,&
     & .1891795E+00,.6516100E+00,.7946672E+00,.8140707E+00,.4414800E+00,&
     & .2280943E+00,.7538152E+00,.9140427E+00,.9326912E+00,.5067756E+00,&
     & .9538147E-01,.3836728E+00,.4692157E+00,.4807161E+00,.2573361E+00,&
     & .1169920E+00,.4534590E+00,.5555647E+00,.5707684E+00,.3058432E+00,&
     & .1425205E+00,.5320323E+00,.6525202E+00,.6704208E+00,.3602199E+00,&
     & .1734620E+00,.6207733E+00,.7596824E+00,.7794089E+00,.4197447E+00,&
     & .2108876E+00,.7208589E+00,.8772631E+00,.8967522E+00,.4842904E+00/

      data   absb(701:875, 8)   /                                       &
     & .8483947E-01,.3588563E+00,.4408601E+00,.4516281E+00,.2391711E+00,&
     & .1052687E+00,.4273398E+00,.5251982E+00,.5400986E+00,.2865821E+00,&
     & .1294958E+00,.5040617E+00,.6199729E+00,.6376780E+00,.3398708E+00,&
     & .1588835E+00,.5906162E+00,.7250743E+00,.7449273E+00,.3985409E+00,&
     & .1946890E+00,.6883248E+00,.8405775E+00,.8607468E+00,.4621965E+00,&
     & .7543551E-01,.3358861E+00,.4145122E+00,.4243832E+00,.2224114E+00,&
     & .9465525E-01,.4026844E+00,.4964574E+00,.5108780E+00,.2686287E+00,&
     & .1177290E+00,.4776992E+00,.5890965E+00,.6064776E+00,.3207812E+00,&
     & .1456605E+00,.5620963E+00,.6920814E+00,.7118635E+00,.3784909E+00,&
     & .1798767E+00,.6573786E+00,.8054007E+00,.8260421E+00,.4411895E+00,&
     & .6608792E-01,.3109105E+00,.3855264E+00,.3941304E+00,.2044580E+00,&
     & .8394591E-01,.3756301E+00,.4645764E+00,.4780384E+00,.2491451E+00,&
     & .1057032E+00,.4484352E+00,.5543433E+00,.5711787E+00,.2997657E+00,&
     & .1320096E+00,.5302167E+00,.6546096E+00,.6740082E+00,.3560965E+00,&
     & .1642833E+00,.6224082E+00,.7650984E+00,.7859469E+00,.4175347E+00,&
     & .5738271E-01,.2860275E+00,.3565059E+00,.3636546E+00,.1867469E+00,&
     & .7381553E-01,.3484044E+00,.4324451E+00,.4446271E+00,.2297128E+00,&
     & .9417454E-01,.4188318E+00,.5189475E+00,.5351023E+00,.2785779E+00,&
     & .1189809E+00,.4980563E+00,.6163102E+00,.6351253E+00,.3333660E+00,&
     & .1491626E+00,.5867414E+00,.7235000E+00,.7442712E+00,.3933218E+00,&
     & .4966189E-01,.2627287E+00,.3290966E+00,.3348944E+00,.1702478E+00,&
     & .6472660E-01,.3227122E+00,.4021344E+00,.4129120E+00,.2114833E+00,&
     & .8369235E-01,.3907371E+00,.4852534E+00,.5005355E+00,.2585650E+00,&
     & .1070502E+00,.4673220E+00,.5794833E+00,.5976216E+00,.3116401E+00,&
     & .1353378E+00,.5528051E+00,.6835756E+00,.7040363E+00,.3701096E+00,&
     & .4256723E-01,.2397851E+00,.3016717E+00,.3062819E+00,.1540481E+00,&
     & .5622816E-01,.2970953E+00,.3718496E+00,.3810025E+00,.1934507E+00,&
     & .7377185E-01,.3626587E+00,.4515842E+00,.4655582E+00,.2385992E+00,&
     & .9559465E-01,.4363178E+00,.5420972E+00,.5594484E+00,.2897050E+00,&
     & .1220368E+00,.5185878E+00,.6428277E+00,.6627182E+00,.3464809E+00,&
     & .3589415E-01,.2161673E+00,.2729648E+00,.2765338E+00,.1375028E+00,&
     & .4805408E-01,.2703938E+00,.3401678E+00,.3474504E+00,.1747652E+00,&
     & .6397943E-01,.3329344E+00,.4160369E+00,.4283381E+00,.2177681E+00,&
     & .8411883E-01,.4035911E+00,.5024152E+00,.5187692E+00,.2666244E+00,&
     & .1087145E+00,.4824389E+00,.5992777E+00,.6183786E+00,.3213119E+00/

      data   absb(876:1050, 8)   /                                      &
     & .3011941E-01,.1941366E+00,.2458479E+00,.2487746E+00,.1221988E+00,&
     & .4084506E-01,.2453806E+00,.3101120E+00,.3157807E+00,.1572649E+00,&
     & .5518255E-01,.3048452E+00,.3825091E+00,.3929551E+00,.1981390E+00,&
     & .7368328E-01,.3724217E+00,.4646897E+00,.4797945E+00,.2447415E+00,&
     & .9652937E-01,.4480322E+00,.5576115E+00,.5757517E+00,.2972212E+00,&
     & .2515607E-01,.1736587E+00,.2203553E+00,.2229593E+00,.1080693E+00,&
     & .3453399E-01,.2219693E+00,.2815682E+00,.2859887E+00,.1409424E+00,&
     & .4732652E-01,.2782830E+00,.3508111E+00,.3593155E+00,.1796231E+00,&
     & .6419246E-01,.3427924E+00,.4290016E+00,.4424221E+00,.2240016E+00,&
     & .8533710E-01,.4152474E+00,.5177134E+00,.5349045E+00,.2742416E+00,&
     & .2108838E-01,.1557171E+00,.1978594E+00,.2004336E+00,.9577831E-01,&
     & .2927578E-01,.2012690E+00,.2560716E+00,.2596510E+00,.1265930E+00,&
     & .4069811E-01,.2546968E+00,.3225058E+00,.3293032E+00,.1632049E+00,&
     & .5604639E-01,.3162711E+00,.3971951E+00,.4089321E+00,.2055380E+00,&
     & .7561968E-01,.3858696E+00,.4819614E+00,.4980595E+00,.2536384E+00,&
     & .1767518E-01,.1394513E+00,.1774120E+00,.1800232E+00,.8475686E-01,&
     & .2479537E-01,.1824043E+00,.2325611E+00,.2356543E+00,.1136169E+00,&
     & .3495024E-01,.2330902E+00,.2962474E+00,.3017090E+00,.1481769E+00,&
     & .4886078E-01,.2917985E+00,.3678777E+00,.3778687E+00,.1885266E+00,&
     & .6700377E-01,.3587323E+00,.4490277E+00,.4637480E+00,.2345858E+00,&
     & .1476895E-01,.1243742E+00,.1583798E+00,.1609596E+00,.7459372E-01,&
     & .2091694E-01,.1647602E+00,.2103847E+00,.2132485E+00,.1015375E+00,&
     & .2988238E-01,.2127462E+00,.2712282E+00,.2756073E+00,.1340822E+00,&
     & .4238686E-01,.2685463E+00,.3400097E+00,.3482087E+00,.1724055E+00,&
     & .5904346E-01,.3326286E+00,.4175653E+00,.4306952E+00,.2164369E+00,&
     & .1230374E-01,.1104196E+00,.1406329E+00,.1430549E+00,.6528664E-01,&
     & .1757050E-01,.1482205E+00,.1894519E+00,.1922843E+00,.9031455E-01,&
     & .2540539E-01,.1934889E+00,.2472646E+00,.2509359E+00,.1208463E+00,&
     & .3658763E-01,.2464677E+00,.3133038E+00,.3198840E+00,.1571069E+00,&
     & .5175125E-01,.3076675E+00,.3875310E+00,.3989625E+00,.1991350E+00,&
     & .1033543E-01,.9851647E-01,.1254426E+00,.1276686E+00,.5744944E-01,&
     & .1487034E-01,.1339448E+00,.1713721E+00,.1742024E+00,.8071988E-01,&
     & .2174783E-01,.1768075E+00,.2263355E+00,.2295753E+00,.1094349E+00,&
     & .3178517E-01,.2272599E+00,.2898005E+00,.2951977E+00,.1438116E+00,&
     & .4561407E-01,.2857740E+00,.3612374E+00,.3710210E+00,.1839585E+00/

      data   absb(1051:1175, 8)   /                                     &
     & .8709412E-02,.8794504E-01,.1119210E+00,.1139728E+00,.5066255E-01,&
     & .1262413E-01,.1212030E+00,.1551537E+00,.1579143E+00,.7214590E-01,&
     & .1865119E-01,.1616922E+00,.2072178E+00,.2102675E+00,.9913095E-01,&
     & .2764278E-01,.2097406E+00,.2681310E+00,.2726069E+00,.1317209E+00,&
     & .4023984E-01,.2656768E+00,.3370690E+00,.3452748E+00,.1700699E+00,&
     & .7309742E-02,.7805313E-01,.9935397E-01,.1011896E+00,.4454450E-01,&
     & .1068078E-01,.1092344E+00,.1398484E+00,.1424507E+00,.6420420E-01,&
     & .1594110E-01,.1474371E+00,.1891070E+00,.1921023E+00,.8951479E-01,&
     & .2393561E-01,.1930891E+00,.2473383E+00,.2511858E+00,.1203039E+00,&
     & .3536639E-01,.2465547E+00,.3138627E+00,.3206321E+00,.1568422E+00,&
     & .6111505E-02,.6882934E-01,.8775334E-01,.8932843E-01,.3912454E-01,&
     & .9015756E-02,.9808206E-01,.1255190E+00,.1279049E+00,.5696209E-01,&
     & .1357531E-01,.1339695E+00,.1719655E+00,.1749408E+00,.8050805E-01,&
     & .2064520E-01,.1772958E+00,.2274651E+00,.2308615E+00,.1095380E+00,&
     & .3095469E-01,.2282972E+00,.2914883E+00,.2970959E+00,.1442483E+00,&
     & .5121725E-02,.6067687E-01,.7763361E-01,.7890932E-01,.3444819E-01,&
     & .7639216E-02,.8816021E-01,.1127621E+00,.1149702E+00,.5070163E-01,&
     & .1160557E-01,.1219458E+00,.1566131E+00,.1595297E+00,.7248142E-01,&
     & .1786695E-01,.1630817E+00,.2094426E+00,.2126006E+00,.9988204E-01,&
     & .2716600E-01,.2117945E+00,.2710344E+00,.2757215E+00,.1328837E+00,&
     & .4599319E-02,.5738031E-01,.7361124E-01,.7479831E-01,.3257284E-01,&
     & .6965718E-02,.8418710E-01,.1077301E+00,.1098721E+00,.4821784E-01,&
     & .1072080E-01,.1171779E+00,.1505705E+00,.1534484E+00,.6933089E-01,&
     & .1668801E-01,.1574612E+00,.2023452E+00,.2054613E+00,.9606507E-01,&
     & .2561291E-01,.2052733E+00,.2629716E+00,.2673724E+00,.1283901E+00/

      data   absb(  1:175, 9)   /                                       &
     & .1351800E+02,.1942600E+02,.2136800E+02,.2060700E+02,.1596700E+02,&
     & .1324900E+02,.1927800E+02,.2134500E+02,.2072500E+02,.1621800E+02,&
     & .1299100E+02,.1910600E+02,.2134400E+02,.2083800E+02,.1649300E+02,&
     & .1278300E+02,.1895300E+02,.2133700E+02,.2095900E+02,.1676900E+02,&
     & .1273500E+02,.1882000E+02,.2132800E+02,.2110400E+02,.1698200E+02,&
     & .1150400E+02,.1753400E+02,.1945700E+02,.1913100E+02,.1462500E+02,&
     & .1128100E+02,.1739600E+02,.1949100E+02,.1927100E+02,.1493900E+02,&
     & .1111200E+02,.1729000E+02,.1952700E+02,.1941500E+02,.1525500E+02,&
     & .1111500E+02,.1721300E+02,.1956000E+02,.1958000E+02,.1550400E+02,&
     & .1134200E+02,.1713900E+02,.1961500E+02,.1975900E+02,.1570600E+02,&
     & .9817000E+01,.1559900E+02,.1762700E+02,.1766600E+02,.1335400E+02,&
     & .9674700E+01,.1553300E+02,.1769500E+02,.1784200E+02,.1368700E+02,&
     & .9698400E+01,.1550200E+02,.1775500E+02,.1803600E+02,.1400400E+02,&
     & .9952200E+01,.1548000E+02,.1782900E+02,.1823500E+02,.1426300E+02,&
     & .1034400E+02,.1552500E+02,.1792900E+02,.1841800E+02,.1447500E+02,&
     & .8433200E+01,.1380600E+02,.1592600E+02,.1621500E+02,.1222400E+02,&
     & .8458300E+01,.1380500E+02,.1602100E+02,.1643700E+02,.1255600E+02,&
     & .8706600E+01,.1382300E+02,.1610800E+02,.1666000E+02,.1283400E+02,&
     & .9082700E+01,.1392400E+02,.1622600E+02,.1688300E+02,.1307800E+02,&
     & .9526300E+01,.1415600E+02,.1638200E+02,.1707900E+02,.1333400E+02,&
     & .7306800E+01,.1225200E+02,.1434100E+02,.1481000E+02,.1112900E+02,&
     & .7529500E+01,.1229100E+02,.1446300E+02,.1505200E+02,.1143900E+02,&
     & .7885300E+01,.1240500E+02,.1460600E+02,.1530900E+02,.1172900E+02,&
     & .8304000E+01,.1264300E+02,.1477600E+02,.1554800E+02,.1200600E+02,&
     & .8712300E+01,.1303900E+02,.1498200E+02,.1576200E+02,.1228600E+02,&
     & .6428700E+01,.1093300E+02,.1287900E+02,.1345100E+02,.1005300E+02,&
     & .6744600E+01,.1103900E+02,.1304100E+02,.1374000E+02,.1037700E+02,&
     & .7127100E+01,.1126600E+02,.1322200E+02,.1401800E+02,.1069900E+02,&
     & .7506700E+01,.1165000E+02,.1345500E+02,.1426300E+02,.1101100E+02,&
     & .7879700E+01,.1214400E+02,.1376600E+02,.1451700E+02,.1132300E+02,&
     & .5716500E+01,.9819800E+01,.1156300E+02,.1221000E+02,.9028700E+01,&
     & .6050400E+01,.1003000E+02,.1175900E+02,.1251100E+02,.9377100E+01,&
     & .6387200E+01,.1039000E+02,.1200400E+02,.1278700E+02,.9738200E+01,&
     & .6726200E+01,.1085800E+02,.1232500E+02,.1307100E+02,.1010000E+02,&
     & .7063000E+01,.1139600E+02,.1272000E+02,.1337100E+02,.1043300E+02/

      data   absb(176:350, 9)   /                                       &
     & .5107700E+01,.8868000E+01,.1041500E+02,.1105300E+02,.8096300E+01,&
     & .5401700E+01,.9210700E+01,.1066200E+02,.1136100E+02,.8475800E+01,&
     & .5701500E+01,.9669800E+01,.1098200E+02,.1166800E+02,.8848100E+01,&
     & .6004100E+01,.1019600E+02,.1138400E+02,.1200000E+02,.9237600E+01,&
     & .6310600E+01,.1073200E+02,.1188400E+02,.1234900E+02,.9625500E+01,&
     & .4541400E+01,.8084100E+01,.9433300E+01,.1000100E+02,.7298700E+01,&
     & .4806000E+01,.8516900E+01,.9739900E+01,.1032400E+02,.7679900E+01,&
     & .5075400E+01,.9031100E+01,.1013400E+02,.1067300E+02,.8069200E+01,&
     & .5348900E+01,.9566000E+01,.1062600E+02,.1104600E+02,.8467100E+01,&
     & .5622600E+01,.1010000E+02,.1118800E+02,.1146700E+02,.8887800E+01,&
     & .4051200E+01,.7453700E+01,.8611300E+01,.9076000E+01,.6640600E+01,&
     & .4286100E+01,.7945500E+01,.8998800E+01,.9432600E+01,.7029800E+01,&
     & .4528400E+01,.8472300E+01,.9480600E+01,.9828200E+01,.7436500E+01,&
     & .4776500E+01,.9002200E+01,.1003600E+02,.1026700E+02,.7853000E+01,&
     & .5027300E+01,.9537500E+01,.1064400E+02,.1076200E+02,.8292500E+01,&
     & .3627200E+01,.6934500E+01,.7929200E+01,.8282100E+01,.6086200E+01,&
     & .3832400E+01,.7441200E+01,.8405800E+01,.8677900E+01,.6481400E+01,&
     & .4049600E+01,.7955600E+01,.8949700E+01,.9127700E+01,.6903100E+01,&
     & .4273600E+01,.8488100E+01,.9551200E+01,.9635300E+01,.7342300E+01,&
     & .4500800E+01,.9023100E+01,.1019900E+02,.1019600E+02,.7810700E+01,&
     & .3261700E+01,.6492700E+01,.7381400E+01,.7620400E+01,.5635100E+01,&
     & .3445500E+01,.6986400E+01,.7914600E+01,.8066800E+01,.6048200E+01,&
     & .3637500E+01,.7498200E+01,.8512400E+01,.8568800E+01,.6479300E+01,&
     & .3836100E+01,.8025300E+01,.9153000E+01,.9138400E+01,.6948800E+01,&
     & .4043400E+01,.8577000E+01,.9824900E+01,.9770100E+01,.7439200E+01,&
     & .2950400E+01,.6099600E+01,.6937800E+01,.7093300E+01,.5269600E+01,&
     & .3113400E+01,.6585900E+01,.7519300E+01,.7587400E+01,.5690800E+01,&
     & .3285000E+01,.7099100E+01,.8158300E+01,.8145500E+01,.6150400E+01,&
     & .3463900E+01,.7635700E+01,.8828900E+01,.8776600E+01,.6639100E+01,&
     & .3657500E+01,.8203100E+01,.9533500E+01,.9472800E+01,.7118100E+01,&
     & .2678500E+01,.5759200E+01,.6594400E+01,.6679300E+01,.4965900E+01,&
     & .2830300E+01,.6251200E+01,.7218000E+01,.7229200E+01,.5415800E+01,&
     & .2984500E+01,.6770000E+01,.7885300E+01,.7851700E+01,.5892900E+01,&
     & .3149800E+01,.7321800E+01,.8589200E+01,.8540500E+01,.6377900E+01,&
     & .3336500E+01,.7909500E+01,.9326000E+01,.9287600E+01,.6859300E+01/

      data   absb(351:525, 9)   /                                       &
     & .2438100E+01,.5471900E+01,.6329600E+01,.6369900E+01,.4737800E+01,&
     & .2583300E+01,.5970600E+01,.6983900E+01,.6980900E+01,.5202400E+01,&
     & .2727600E+01,.6503200E+01,.7682300E+01,.7662300E+01,.5684900E+01,&
     & .2884300E+01,.7076800E+01,.8422300E+01,.8397300E+01,.6161300E+01,&
     & .3077600E+01,.7683700E+01,.9192900E+01,.9187100E+01,.6669700E+01,&
     & .2223800E+01,.5236600E+01,.6133500E+01,.6157400E+01,.4570500E+01,&
     & .2364800E+01,.5745100E+01,.6815100E+01,.6826600E+01,.5034700E+01,&
     & .2506700E+01,.6298000E+01,.7546000E+01,.7554800E+01,.5506400E+01,&
     & .2668000E+01,.6893000E+01,.8322300E+01,.8330800E+01,.6010300E+01,&
     & .2876300E+01,.7523400E+01,.9126900E+01,.9154700E+01,.6544300E+01,&
     & .2034300E+01,.5054000E+01,.6002100E+01,.6036000E+01,.4430600E+01,&
     & .2173200E+01,.5578500E+01,.6713400E+01,.6754400E+01,.4892400E+01,&
     & .2321700E+01,.6155400E+01,.7482800E+01,.7521700E+01,.5389900E+01,&
     & .2499100E+01,.6772700E+01,.8294300E+01,.8334100E+01,.5921100E+01,&
     & .2729800E+01,.7430900E+01,.9129400E+01,.9187700E+01,.6477600E+01,&
     & .1867800E+01,.4920300E+01,.5929100E+01,.5987500E+01,.4320400E+01,&
     & .2010200E+01,.5466700E+01,.6674700E+01,.6743000E+01,.4802600E+01,&
     & .2171900E+01,.6068200E+01,.7482700E+01,.7547700E+01,.5328100E+01,&
     & .2372000E+01,.6712600E+01,.8325200E+01,.8391000E+01,.5882600E+01,&
     & .2626400E+01,.7397700E+01,.9187000E+01,.9272300E+01,.6459800E+01,&
     & .1724400E+01,.4837000E+01,.5915600E+01,.6000100E+01,.4255500E+01,&
     & .1875500E+01,.5410700E+01,.6699900E+01,.6790400E+01,.4764400E+01,&
     & .2057300E+01,.6036300E+01,.7540200E+01,.7627400E+01,.5315500E+01,&
     & .2282900E+01,.6709400E+01,.8407900E+01,.8496200E+01,.5890100E+01,&
     & .2559000E+01,.7419500E+01,.9290900E+01,.9399400E+01,.6487100E+01,&
     & .1606900E+01,.4803000E+01,.5957900E+01,.6063900E+01,.4235600E+01,&
     & .1770500E+01,.5403700E+01,.6778800E+01,.6887400E+01,.4771200E+01,&
     & .1974200E+01,.6057400E+01,.7644800E+01,.7749100E+01,.5343400E+01,&
     & .2224700E+01,.6756000E+01,.8531000E+01,.8638500E+01,.5938100E+01,&
     & .2523800E+01,.7488000E+01,.9429200E+01,.9557000E+01,.6549700E+01,&
     & .1512800E+01,.4814000E+01,.6050100E+01,.6171400E+01,.4255000E+01,&
     & .1694600E+01,.5441700E+01,.6900000E+01,.7021200E+01,.4815500E+01,&
     & .1920600E+01,.6121300E+01,.7783900E+01,.7902000E+01,.5406900E+01,&
     & .2194600E+01,.6842200E+01,.8682700E+01,.8807500E+01,.6016900E+01,&
     & .2517300E+01,.7591300E+01,.9592400E+01,.9737300E+01,.6639800E+01/

      data   absb(526:700, 9)   /                                       &
     & .1435600E+01,.4841800E+01,.6149500E+01,.6282800E+01,.4289600E+01,&
     & .1635700E+01,.5493500E+01,.7018400E+01,.7151600E+01,.4869100E+01,&
     & .1883800E+01,.6195700E+01,.7915100E+01,.8045700E+01,.5474200E+01,&
     & .2178800E+01,.6932500E+01,.8825600E+01,.8964300E+01,.6095600E+01,&
     & .2523800E+01,.7693000E+01,.9743900E+01,.9902100E+01,.6726400E+01,&
     & .1362000E+01,.4840000E+01,.6191100E+01,.6334100E+01,.4295000E+01,&
     & .1576500E+01,.5507700E+01,.7071700E+01,.7214900E+01,.4887200E+01,&
     & .1841600E+01,.6223800E+01,.7977800E+01,.8117300E+01,.5501400E+01,&
     & .2152800E+01,.6969400E+01,.8895100E+01,.9043000E+01,.6130400E+01,&
     & .2513500E+01,.7737000E+01,.9819100E+01,.9986500E+01,.6766200E+01,&
     & .1283600E+01,.4789200E+01,.6154900E+01,.6305000E+01,.4254000E+01,&
     & .1507000E+01,.5464900E+01,.7041600E+01,.7192300E+01,.4853600E+01,&
     & .1783000E+01,.6187200E+01,.7953100E+01,.8099400E+01,.5473400E+01,&
     & .2105500E+01,.6936900E+01,.8874500E+01,.9028800E+01,.6106600E+01,&
     & .2475400E+01,.7709000E+01,.9802300E+01,.9975900E+01,.6745900E+01,&
     & .1187800E+01,.4654100E+01,.5997500E+01,.6153100E+01,.4135700E+01,&
     & .1411400E+01,.5327900E+01,.6883600E+01,.7040700E+01,.4736600E+01,&
     & .1690700E+01,.6048800E+01,.7796100E+01,.7949200E+01,.5358700E+01,&
     & .2017100E+01,.6798700E+01,.8721400E+01,.8878200E+01,.5993100E+01,&
     & .2387900E+01,.7572100E+01,.9650500E+01,.9826300E+01,.6634600E+01,&
     & .1101900E+01,.4524100E+01,.5841100E+01,.6001500E+01,.4020600E+01,&
     & .1324000E+01,.5193800E+01,.6724800E+01,.6888100E+01,.4621500E+01,&
     & .1605000E+01,.5912700E+01,.7637700E+01,.7796900E+01,.5244400E+01,&
     & .1934400E+01,.6661400E+01,.8565600E+01,.8724100E+01,.5879700E+01,&
     & .2305300E+01,.7435200E+01,.9496100E+01,.9673000E+01,.6523200E+01,&
     & .1026100E+01,.4403500E+01,.5692500E+01,.5857100E+01,.3913000E+01,&
     & .1245800E+01,.5069400E+01,.6573500E+01,.6741100E+01,.4513000E+01,&
     & .1528100E+01,.5784400E+01,.7485300E+01,.7649700E+01,.5135300E+01,&
     & .1859000E+01,.6531000E+01,.8413900E+01,.8575200E+01,.5771100E+01,&
     & .2229600E+01,.7303100E+01,.9346100E+01,.9523300E+01,.6415500E+01,&
     & .9402000E+00,.4225200E+01,.5461800E+01,.5630500E+01,.3751400E+01,&
     & .1151000E+01,.4880800E+01,.6335900E+01,.6505100E+01,.4346400E+01,&
     & .1426700E+01,.5586500E+01,.7242900E+01,.7411600E+01,.4965900E+01,&
     & .1753400E+01,.6328400E+01,.8170600E+01,.8334600E+01,.5600400E+01,&
     & .2119600E+01,.7095400E+01,.9103800E+01,.9279000E+01,.6244300E+01/

      data   absb(701:875, 9)   /                                       &
     & .8612900E+00,.4045700E+01,.5226400E+01,.5397600E+01,.3587900E+01,&
     & .1063300E+01,.4691500E+01,.6092400E+01,.6263500E+01,.4176800E+01,&
     & .1328300E+01,.5384500E+01,.6992600E+01,.7164900E+01,.4792100E+01,&
     & .1648500E+01,.6120100E+01,.7917500E+01,.8084400E+01,.5424000E+01,&
     & .2009300E+01,.6881200E+01,.8852200E+01,.9024900E+01,.6066800E+01,&
     & .7912300E+00,.3873500E+01,.4996000E+01,.5169900E+01,.3428700E+01,&
     & .9829500E+00,.4507200E+01,.5853300E+01,.6025400E+01,.4010900E+01,&
     & .1236400E+01,.5187600E+01,.6745900E+01,.6921400E+01,.4621200E+01,&
     & .1548400E+01,.5915000E+01,.7666500E+01,.7837300E+01,.5250300E+01,&
     & .1902600E+01,.6671600E+01,.8601800E+01,.8772800E+01,.5891500E+01,&
     & .7198400E+00,.3672900E+01,.4723500E+01,.4897500E+01,.3239700E+01,&
     & .8980800E+00,.4291600E+01,.5569700E+01,.5744300E+01,.3815300E+01,&
     & .1136000E+01,.4957400E+01,.6451900E+01,.6628900E+01,.4418400E+01,&
     & .1434500E+01,.5672800E+01,.7366000E+01,.7541000E+01,.5043700E+01,&
     & .1779300E+01,.6422800E+01,.8299900E+01,.8470500E+01,.5682300E+01,&
     & .6528500E+00,.3466700E+01,.4437200E+01,.4608200E+01,.3039900E+01,&
     & .8165700E+00,.4067400E+01,.5270100E+01,.5446600E+01,.3609400E+01,&
     & .1037000E+01,.4717100E+01,.6140500E+01,.6317500E+01,.4203800E+01,&
     & .1319400E+01,.5418400E+01,.7047300E+01,.7224800E+01,.4823700E+01,&
     & .1651400E+01,.6158400E+01,.7976000E+01,.8147700E+01,.5458100E+01,&
     & .5932400E+00,.3268700E+01,.4161000E+01,.4327000E+01,.2843700E+01,&
     & .7429200E+00,.3850200E+01,.4975400E+01,.5152500E+01,.3408100E+01,&
     & .9460600E+00,.4483800E+01,.5834700E+01,.6011200E+01,.3993200E+01,&
     & .1210300E+01,.5169100E+01,.6731200E+01,.6911300E+01,.4606300E+01,&
     & .1528300E+01,.5897000E+01,.7652900E+01,.7828300E+01,.5236500E+01,&
     & .5366500E+00,.3066500E+01,.3878800E+01,.4034400E+01,.2639600E+01,&
     & .6732600E+00,.3627500E+01,.4668500E+01,.4844500E+01,.3197000E+01,&
     & .8596400E+00,.4245000E+01,.5516100E+01,.5694000E+01,.3774200E+01,&
     & .1102600E+01,.4911200E+01,.6399200E+01,.6579500E+01,.4378500E+01,&
     & .1403000E+01,.5624700E+01,.7312800E+01,.7491800E+01,.5003700E+01,&
     & .4796500E+00,.2851200E+01,.3577300E+01,.3719000E+01,.2417900E+01,&
     & .6049900E+00,.3389500E+01,.4335300E+01,.4507000E+01,.2965300E+01,&
     & .7721100E+00,.3985700E+01,.5165600E+01,.5343900E+01,.3534600E+01,&
     & .9928000E+00,.4632000E+01,.6034600E+01,.6213800E+01,.4127800E+01,&
     & .1271100E+01,.5327200E+01,.6937700E+01,.7118900E+01,.4746400E+01/

      data   absb(876:1050, 9)   /                                      &
     & .4265100E+00,.2644500E+01,.3289400E+01,.3414800E+01,.2203300E+01,&
     & .5438800E+00,.3161100E+01,.4016100E+01,.4179100E+01,.2737300E+01,&
     & .6936800E+00,.3734700E+01,.4820600E+01,.4998900E+01,.3299700E+01,&
     & .8927900E+00,.4362000E+01,.5676700E+01,.5855400E+01,.3882200E+01,&
     & .1148400E+01,.5036900E+01,.6566800E+01,.6749500E+01,.4491800E+01,&
     & .3771300E+00,.2443900E+01,.3016000E+01,.3120600E+01,.1997600E+01,&
     & .4876600E+00,.2943400E+01,.3710400E+01,.3859900E+01,.2514300E+01,&
     & .6240700E+00,.3493900E+01,.4484700E+01,.4659600E+01,.3068000E+01,&
     & .8029800E+00,.4100600E+01,.5325500E+01,.5504900E+01,.3641300E+01,&
     & .1035300E+01,.4755400E+01,.6201000E+01,.6382000E+01,.4240700E+01,&
     & .3333400E+00,.2257300E+01,.2773000E+01,.2853700E+01,.1814100E+01,&
     & .4382400E+00,.2747700E+01,.3435900E+01,.3571200E+01,.2311200E+01,&
     & .5655500E+00,.3276700E+01,.4180700E+01,.4349700E+01,.2854000E+01,&
     & .7266200E+00,.3863100E+01,.5000300E+01,.5179800E+01,.3420600E+01,&
     & .9381100E+00,.4499800E+01,.5863500E+01,.6043300E+01,.4009200E+01,&
     & .2934400E+00,.2079200E+01,.2549300E+01,.2604500E+01,.1644700E+01,&
     & .3932000E+00,.2563300E+01,.3181100E+01,.3299600E+01,.2120200E+01,&
     & .5133500E+00,.3073900E+01,.3896800E+01,.4055400E+01,.2649300E+01,&
     & .6599100E+00,.3639600E+01,.4691000E+01,.4869200E+01,.3209500E+01,&
     & .8530500E+00,.4260200E+01,.5542800E+01,.5722700E+01,.3789000E+01,&
     & .2563400E+00,.1906500E+01,.2337300E+01,.2367500E+01,.1484900E+01,&
     & .3508500E+00,.2383200E+01,.2938000E+01,.3036300E+01,.1937900E+01,&
     & .4646300E+00,.2880600E+01,.3624500E+01,.3770500E+01,.2449900E+01,&
     & .6001400E+00,.3424700E+01,.4390100E+01,.4564400E+01,.3001300E+01,&
     & .7750200E+00,.4026800E+01,.5226400E+01,.5406400E+01,.3572900E+01,&
     & .2219800E+00,.1738200E+01,.2135500E+01,.2144300E+01,.1332100E+01,&
     & .3108800E+00,.2203700E+01,.2707000E+01,.2780900E+01,.1763100E+01,&
     & .4186800E+00,.2693700E+01,.3362800E+01,.3493700E+01,.2255100E+01,&
     & .5459000E+00,.3217200E+01,.4099400E+01,.4266900E+01,.2795000E+01,&
     & .7038200E+00,.3799300E+01,.4913400E+01,.5093400E+01,.3360900E+01,&
     & .1927900E+00,.1588400E+01,.1957600E+01,.1951600E+01,.1199100E+01,&
     & .2761400E+00,.2042300E+01,.2505300E+01,.2555900E+01,.1610600E+01,&
     & .3790000E+00,.2526400E+01,.3132300E+01,.3247300E+01,.2082500E+01,&
     & .4995400E+00,.3034400E+01,.3842500E+01,.3999400E+01,.2609000E+01,&
     & .6448300E+00,.3596700E+01,.4632000E+01,.4810400E+01,.3168300E+01/

      data   absb(1051:1175, 9)   /                                     &
     & .1675200E+00,.1450600E+01,.1792400E+01,.1778000E+01,.1076600E+01,&
     & .2454600E+00,.1892400E+01,.2321500E+01,.2350200E+01,.1471800E+01,&
     & .3428200E+00,.2368600E+01,.2920300E+01,.3017100E+01,.1923800E+01,&
     & .4575100E+00,.2865700E+01,.3604800E+01,.3750400E+01,.2434700E+01,&
     & .5934500E+00,.3409100E+01,.4369200E+01,.4543600E+01,.2986000E+01,&
     & .1444200E+00,.1319100E+01,.1630700E+01,.1615800E+01,.9596600E+00,&
     & .2162900E+00,.1745600E+01,.2145400E+01,.2155300E+01,.1338400E+01,&
     & .3083800E+00,.2211700E+01,.2718400E+01,.2793900E+01,.1771000E+01,&
     & .4178000E+00,.2702400E+01,.3376100E+01,.3508300E+01,.2264700E+01,&
     & .5463300E+00,.3227700E+01,.4115300E+01,.4283500E+01,.2805700E+01,&
     & .1236900E+00,.1194200E+01,.1475100E+01,.1466000E+01,.8479400E+00,&
     & .1894300E+00,.1602500E+01,.1975200E+01,.1970600E+01,.1210400E+01,&
     & .2758000E+00,.2057700E+01,.2525300E+01,.2578500E+01,.1624900E+01,&
     & .3801500E+00,.2542500E+01,.3155400E+01,.3272400E+01,.2099500E+01,&
     & .5022000E+00,.3052800E+01,.3869400E+01,.4027500E+01,.2627900E+01,&
     & .1061900E+00,.1081500E+01,.1333800E+01,.1334000E+01,.7492400E+00,&
     & .1658600E+00,.1471300E+01,.1818300E+01,.1804700E+01,.1093300E+01,&
     & .2466800E+00,.1915000E+01,.2349800E+01,.2382400E+01,.1493100E+01,&
     & .3461300E+00,.2393600E+01,.2954200E+01,.3054500E+01,.1948900E+01,&
     & .4625100E+00,.2892800E+01,.3643900E+01,.3791400E+01,.2462800E+01,&
     & .9878100E-01,.1037400E+01,.1279000E+01,.1283400E+01,.7109100E+00,&
     & .1564300E+00,.1419900E+01,.1756000E+01,.1740600E+01,.1047500E+01,&
     & .2351200E+00,.1858500E+01,.2281500E+01,.2306100E+01,.1440800E+01,&
     & .3325000E+00,.2333800E+01,.2875500E+01,.2968600E+01,.1889800E+01,&
     & .4469700E+00,.2829800E+01,.3555500E+01,.3698400E+01,.2397500E+01/

      data   absb(  1:175,10)   /                                       &
     & .5148311E+02,.5413939E+02,.5702358E+02,.5255781E+02,.4767175E+02,&
     & .5089809E+02,.5366563E+02,.5690094E+02,.5278229E+02,.4816697E+02,&
     & .5028687E+02,.5319767E+02,.5674627E+02,.5298787E+02,.4858275E+02,&
     & .4966788E+02,.5270972E+02,.5651558E+02,.5315507E+02,.4895540E+02,&
     & .4904266E+02,.5220812E+02,.5626701E+02,.5327637E+02,.4934862E+02,&
     & .4695357E+02,.5244112E+02,.5623255E+02,.5228165E+02,.4528190E+02,&
     & .4636862E+02,.5203478E+02,.5620209E+02,.5261290E+02,.4583339E+02,&
     & .4578848E+02,.5161094E+02,.5610019E+02,.5291867E+02,.4633999E+02,&
     & .4522091E+02,.5118237E+02,.5596404E+02,.5316078E+02,.4686589E+02,&
     & .4464656E+02,.5075226E+02,.5579719E+02,.5336823E+02,.4742733E+02,&
     & .4214601E+02,.5023928E+02,.5465620E+02,.5137721E+02,.4265810E+02,&
     & .4159514E+02,.4989728E+02,.5473059E+02,.5180612E+02,.4330876E+02,&
     & .4107738E+02,.4954386E+02,.5475515E+02,.5218532E+02,.4396773E+02,&
     & .4057407E+02,.4919951E+02,.5475077E+02,.5253512E+02,.4466305E+02,&
     & .4017621E+02,.4885046E+02,.5474194E+02,.5287718E+02,.4535369E+02,&
     & .3729887E+02,.4762187E+02,.5248346E+02,.4992379E+02,.3996857E+02,&
     & .3682707E+02,.4735522E+02,.5268439E+02,.5045155E+02,.4074890E+02,&
     & .3639654E+02,.4710649E+02,.5286499E+02,.5096814E+02,.4156839E+02,&
     & .3608349E+02,.4686360E+02,.5304363E+02,.5147530E+02,.4241003E+02,&
     & .3594030E+02,.4664103E+02,.5320927E+02,.5198934E+02,.4322855E+02,&
     & .3267085E+02,.4471454E+02,.4990144E+02,.4806389E+02,.3737273E+02,&
     & .3229921E+02,.4458748E+02,.5027045E+02,.4875782E+02,.3829531E+02,&
     & .3205109E+02,.4447599E+02,.5065142E+02,.4944055E+02,.3925216E+02,&
     & .3198556E+02,.4439449E+02,.5103374E+02,.5013485E+02,.4020794E+02,&
     & .3212562E+02,.4431957E+02,.5139858E+02,.5083922E+02,.4116103E+02,&
     & .2840862E+02,.4164567E+02,.4712876E+02,.4597745E+02,.3496902E+02,&
     & .2819485E+02,.4169424E+02,.4770395E+02,.4685191E+02,.3600935E+02,&
     & .2816868E+02,.4176226E+02,.4829256E+02,.4773086E+02,.3708676E+02,&
     & .2836998E+02,.4184080E+02,.4888393E+02,.4862950E+02,.3815932E+02,&
     & .2876609E+02,.4198837E+02,.4946530E+02,.4954016E+02,.3924359E+02,&
     & .2458838E+02,.3854886E+02,.4433510E+02,.4378221E+02,.3278158E+02,&
     & .2458406E+02,.3879353E+02,.4511982E+02,.4485465E+02,.3393561E+02,&
     & .2481011E+02,.3904687E+02,.4591165E+02,.4596040E+02,.3509830E+02,&
     & .2524181E+02,.3936718E+02,.4670940E+02,.4707856E+02,.3628077E+02,&
     & .2588043E+02,.3975083E+02,.4751106E+02,.4819091E+02,.3749205E+02/

      data   absb(176:350,10)   /                                       &
     & .2128943E+02,.3559835E+02,.4162163E+02,.4164333E+02,.3082318E+02,&
     & .2150537E+02,.3602723E+02,.4259734E+02,.4292976E+02,.3205684E+02,&
     & .2193801E+02,.3650531E+02,.4361086E+02,.4425346E+02,.3332155E+02,&
     & .2259310E+02,.3706210E+02,.4463171E+02,.4558654E+02,.3461692E+02,&
     & .2342964E+02,.3772964E+02,.4566117E+02,.4688864E+02,.3593171E+02,&
     & .1852062E+02,.3287436E+02,.3903572E+02,.3964314E+02,.2907608E+02,&
     & .1891782E+02,.3350332E+02,.4024543E+02,.4114241E+02,.3039091E+02,&
     & .1954091E+02,.3421519E+02,.4148282E+02,.4268352E+02,.3174986E+02,&
     & .2036981E+02,.3502995E+02,.4273744E+02,.4421574E+02,.3314916E+02,&
     & .2138764E+02,.3596408E+02,.4400706E+02,.4569955E+02,.3457774E+02,&
     & .1624714E+02,.3047658E+02,.3680690E+02,.3795944E+02,.2761997E+02,&
     & .1682346E+02,.3133476E+02,.3822752E+02,.3967654E+02,.2904446E+02,&
     & .1761414E+02,.3229856E+02,.3969010E+02,.4139802E+02,.3050220E+02,&
     & .1860843E+02,.3338503E+02,.4119523E+02,.4312030E+02,.3200324E+02,&
     & .1979142E+02,.3455683E+02,.4269211E+02,.4477201E+02,.3355101E+02,&
     & .1441593E+02,.2844847E+02,.3488725E+02,.3654168E+02,.2639799E+02,&
     & .1513627E+02,.2951628E+02,.3652960E+02,.3844842E+02,.2793035E+02,&
     & .1607257E+02,.3073671E+02,.3822881E+02,.4036376E+02,.2949013E+02,&
     & .1721158E+02,.3204955E+02,.3993969E+02,.4224228E+02,.3109975E+02,&
     & .1853506E+02,.3344658E+02,.4166331E+02,.4405921E+02,.3276077E+02,&
     & .1295279E+02,.2677411E+02,.3332381E+02,.3540024E+02,.2541395E+02,&
     & .1380491E+02,.2807574E+02,.3519465E+02,.3748808E+02,.2704929E+02,&
     & .1487470E+02,.2950308E+02,.3708718E+02,.3957763E+02,.2871542E+02,&
     & .1613680E+02,.3102968E+02,.3900635E+02,.4159900E+02,.3042625E+02,&
     & .1756668E+02,.3261608E+02,.4093414E+02,.4356179E+02,.3219685E+02,&
     & .1181308E+02,.2545547E+02,.3213568E+02,.3454579E+02,.2465802E+02,&
     & .1278415E+02,.2697292E+02,.3418991E+02,.3680560E+02,.2639551E+02,&
     & .1396046E+02,.2859206E+02,.3627615E+02,.3903520E+02,.2816396E+02,&
     & .1530966E+02,.3030034E+02,.3838513E+02,.4119274E+02,.2998441E+02,&
     & .1680246E+02,.3204287E+02,.4046635E+02,.4326902E+02,.3183828E+02,&
     & .1095745E+02,.2448401E+02,.3130388E+02,.3400597E+02,.2413910E+02,&
     & .1202575E+02,.2618548E+02,.3353135E+02,.3640460E+02,.2597275E+02,&
     & .1327161E+02,.2798353E+02,.3579506E+02,.3875055E+02,.2784255E+02,&
     & .1467686E+02,.2984441E+02,.3805097E+02,.4101420E+02,.2975816E+02,&
     & .1623812E+02,.3172488E+02,.4025111E+02,.4318326E+02,.3168296E+02/

      data   absb(351:525,10)   /                                       &
     & .1031026E+02,.2379955E+02,.3077002E+02,.3371001E+02,.2381408E+02,&
     & .1143882E+02,.2566958E+02,.3315826E+02,.3621895E+02,.2574289E+02,&
     & .1273674E+02,.2762295E+02,.3556863E+02,.3865824E+02,.2770335E+02,&
     & .1420938E+02,.2960954E+02,.3792581E+02,.4100632E+02,.2969115E+02,&
     & .1584019E+02,.3161041E+02,.4022388E+02,.4324073E+02,.3167330E+02,&
     & .9809660E+01,.2337401E+02,.3050021E+02,.3361805E+02,.2366649E+02,&
     & .1099292E+02,.2539109E+02,.3302281E+02,.3621280E+02,.2568000E+02,&
     & .1234898E+02,.2746966E+02,.3553468E+02,.3873037E+02,.2771661E+02,&
     & .1389015E+02,.2956157E+02,.3797616E+02,.4114283E+02,.2975703E+02,&
     & .1559986E+02,.3165438E+02,.4034450E+02,.4341423E+02,.3178583E+02,&
     & .9444252E+01,.2318083E+02,.3046927E+02,.3371915E+02,.2368071E+02,&
     & .1068902E+02,.2532385E+02,.3309721E+02,.3637816E+02,.2576903E+02,&
     & .1210865E+02,.2749920E+02,.3567757E+02,.3895396E+02,.2786288E+02,&
     & .1372527E+02,.2967804E+02,.3817975E+02,.4140126E+02,.2994365E+02,&
     & .1551148E+02,.3183065E+02,.4058662E+02,.4368854E+02,.3200806E+02,&
     & .9197704E+01,.2317475E+02,.3061830E+02,.3396112E+02,.2382472E+02,&
     & .1050749E+02,.2541270E+02,.3332073E+02,.3667202E+02,.2597119E+02,&
     & .1200173E+02,.2766533E+02,.3594813E+02,.3928170E+02,.2810254E+02,&
     & .1369150E+02,.2990848E+02,.3849061E+02,.4174119E+02,.3021392E+02,&
     & .1554967E+02,.3210113E+02,.4091548E+02,.4403004E+02,.3230534E+02,&
     & .9070152E+01,.2333358E+02,.3092333E+02,.3432849E+02,.2408369E+02,&
     & .1044943E+02,.2564057E+02,.3366988E+02,.3707300E+02,.2627089E+02,&
     & .1201766E+02,.2795335E+02,.3632847E+02,.3969279E+02,.2842654E+02,&
     & .1377594E+02,.3023853E+02,.3888979E+02,.4215285E+02,.3056150E+02,&
     & .1570981E+02,.3246011E+02,.4132215E+02,.4443246E+02,.3266580E+02,&
     & .9050881E+01,.2361502E+02,.3134148E+02,.3479389E+02,.2443447E+02,&
     & .1049856E+02,.2597874E+02,.3411303E+02,.3755332E+02,.2664478E+02,&
     & .1213853E+02,.2833372E+02,.3679073E+02,.4016895E+02,.2881882E+02,&
     & .1396914E+02,.3064406E+02,.3935900E+02,.4261289E+02,.3096497E+02,&
     & .1597202E+02,.3287736E+02,.4178044E+02,.4487584E+02,.3307507E+02,&
     & .9121485E+01,.2399340E+02,.3184110E+02,.3533107E+02,.2484905E+02,&
     & .1063898E+02,.2639662E+02,.3462510E+02,.3808849E+02,.2707110E+02,&
     & .1235080E+02,.2877918E+02,.3730856E+02,.4068584E+02,.2925606E+02,&
     & .1425215E+02,.3109959E+02,.3987030E+02,.4310628E+02,.3140551E+02,&
     & .1631751E+02,.3333996E+02,.4227596E+02,.4534660E+02,.3351854E+02/

      data   absb(526:700,10)   /                                       &
     & .9221530E+01,.2435543E+02,.3229891E+02,.3581316E+02,.2522887E+02,&
     & .1079853E+02,.2679042E+02,.3509314E+02,.3856585E+02,.2745675E+02,&
     & .1257368E+02,.2918513E+02,.3777390E+02,.4114200E+02,.2964790E+02,&
     & .1453244E+02,.3151030E+02,.4032236E+02,.4354185E+02,.3180140E+02,&
     & .1664739E+02,.3375272E+02,.4271105E+02,.4575694E+02,.3391150E+02,&
     & .9234921E+01,.2453098E+02,.3252381E+02,.3605734E+02,.2542131E+02,&
     & .1085339E+02,.2698432E+02,.3532561E+02,.3880579E+02,.2765412E+02,&
     & .1266775E+02,.2939222E+02,.3801034E+02,.4137794E+02,.2985073E+02,&
     & .1466294E+02,.3172547E+02,.4055737E+02,.4377010E+02,.3200723E+02,&
     & .1680944E+02,.3397153E+02,.4293979E+02,.4597565E+02,.3411921E+02,&
     & .9122231E+01,.2446628E+02,.3246758E+02,.3601819E+02,.2538766E+02,&
     & .1075904E+02,.2694105E+02,.3528642E+02,.3878018E+02,.2762932E+02,&
     & .1258935E+02,.2936114E+02,.3798364E+02,.4136006E+02,.2983193E+02,&
     & .1459875E+02,.3170547E+02,.4053737E+02,.4375979E+02,.3199342E+02,&
     & .1675964E+02,.3396515E+02,.4293329E+02,.4597404E+02,.3411195E+02,&
     & .8803526E+01,.2404740E+02,.3199538E+02,.3556585E+02,.2502152E+02,&
     & .1042899E+02,.2653898E+02,.3484146E+02,.3835958E+02,.2727937E+02,&
     & .1224422E+02,.2898189E+02,.3757031E+02,.4097542E+02,.2949406E+02,&
     & .1424127E+02,.3135230E+02,.4015581E+02,.4341193E+02,.3166878E+02,&
     & .1639118E+02,.3362956E+02,.4258239E+02,.4565679E+02,.3379358E+02,&
     & .8495800E+01,.2362002E+02,.3150999E+02,.3509681E+02,.2464648E+02,&
     & .1010745E+02,.2612626E+02,.3438400E+02,.3792281E+02,.2691822E+02,&
     & .1190476E+02,.2858932E+02,.3713908E+02,.4057067E+02,.2914314E+02,&
     & .1388319E+02,.3098024E+02,.3975563E+02,.4304137E+02,.3132741E+02,&
     & .1601937E+02,.3327847E+02,.4221299E+02,.4532208E+02,.3346428E+02,&
     & .8209035E+01,.2320772E+02,.3103726E+02,.3463708E+02,.2427972E+02,&
     & .9807919E+01,.2572338E+02,.3393496E+02,.3749067E+02,.2656635E+02,&
     & .1158465E+02,.2820687E+02,.3671655E+02,.4017333E+02,.2879950E+02,&
     & .1354242E+02,.3061364E+02,.3936092E+02,.4267581E+02,.3099415E+02,&
     & .1566241E+02,.3293272E+02,.4184735E+02,.4498926E+02,.3314168E+02,&
     & .7784536E+01,.2254512E+02,.3026965E+02,.3388321E+02,.2367971E+02,&
     & .9354919E+01,.2507029E+02,.3319736E+02,.3677442E+02,.2598726E+02,&
     & .1108920E+02,.2757544E+02,.3601743E+02,.3950998E+02,.2823588E+02,&
     & .1300857E+02,.3000630E+02,.3870211E+02,.4206119E+02,.3044264E+02,&
     & .1509523E+02,.3235435E+02,.4123355E+02,.4442692E+02,.3260463E+02/

      data   absb(701:875,10)   /                                       &
     & .7359863E+01,.2185814E+02,.2946589E+02,.3308961E+02,.2305342E+02,&
     & .8903128E+01,.2438939E+02,.3241822E+02,.3601163E+02,.2537863E+02,&
     & .1058971E+02,.2690897E+02,.3527629E+02,.3879993E+02,.2764129E+02,&
     & .1246583E+02,.2936503E+02,.3800506E+02,.4140154E+02,.2986144E+02,&
     & .1451513E+02,.3174374E+02,.4058041E+02,.4382112E+02,.3203789E+02,&
     & .6956382E+01,.2118043E+02,.2866324E+02,.3229115E+02,.2242833E+02,&
     & .8461548E+01,.2370968E+02,.3163712E+02,.3524703E+02,.2476677E+02,&
     & .1010795E+02,.2624061E+02,.3452870E+02,.3808268E+02,.2704785E+02,&
     & .1193898E+02,.2872275E+02,.3729875E+02,.4073465E+02,.2928035E+02,&
     & .1394781E+02,.3112498E+02,.3991541E+02,.4320304E+02,.3147062E+02,&
     & .6487545E+01,.2036428E+02,.2769057E+02,.3132006E+02,.2167153E+02,&
     & .7946649E+01,.2289019E+02,.3068619E+02,.3430975E+02,.2402051E+02,&
     & .9549264E+01,.2542789E+02,.3361405E+02,.3719415E+02,.2632349E+02,&
     & .1132268E+02,.2793770E+02,.3642692E+02,.3990874E+02,.2857172E+02,&
     & .1327981E+02,.3036678E+02,.3909744E+02,.4243907E+02,.3077710E+02,&
     & .6005571E+01,.1948807E+02,.2664131E+02,.3026331E+02,.2085818E+02,&
     & .7413456E+01,.2201077E+02,.2965560E+02,.3329082E+02,.2321444E+02,&
     & .8970318E+01,.2455040E+02,.3261620E+02,.3621899E+02,.2554069E+02,&
     & .1068617E+02,.2708083E+02,.3547019E+02,.3899468E+02,.2780284E+02,&
     & .1258054E+02,.2953954E+02,.3819719E+02,.4159109E+02,.3002584E+02,&
     & .5548432E+01,.1861826E+02,.2559278E+02,.2919402E+02,.2004363E+02,&
     & .6900092E+01,.2113681E+02,.2862219E+02,.3226196E+02,.2240744E+02,&
     & .8410066E+01,.2367383E+02,.3160742E+02,.3522918E+02,.2474973E+02,&
     & .1007047E+02,.2621742E+02,.3450362E+02,.3806349E+02,.2703382E+02,&
     & .1190409E+02,.2870722E+02,.3728217E+02,.4072426E+02,.2927143E+02,&
     & .5089620E+01,.1770102E+02,.2447913E+02,.2805560E+02,.1917787E+02,&
     & .6378669E+01,.2021146E+02,.2751878E+02,.3115717E+02,.2154621E+02,&
     & .7837964E+01,.2274529E+02,.3052386E+02,.3415832E+02,.2390212E+02,&
     & .9443196E+01,.2529235E+02,.3346307E+02,.3705110E+02,.2620879E+02,&
     & .1120944E+02,.2781188E+02,.3628879E+02,.3978127E+02,.2846286E+02,&
     & .4610896E+01,.1669349E+02,.2324369E+02,.2677647E+02,.1820576E+02,&
     & .5824606E+01,.1918277E+02,.2628270E+02,.2990605E+02,.2058572E+02,&
     & .7220182E+01,.2171074E+02,.2930791E+02,.3295106E+02,.2294896E+02,&
     & .8769986E+01,.2425684E+02,.3228052E+02,.3589409E+02,.2528213E+02,&
     & .1046384E+02,.2679654E+02,.3515901E+02,.3869877E+02,.2755405E+02/

      data   absb(876:1050,10)   /                                      &
     & .4168305E+01,.1570562E+02,.2201962E+02,.2548007E+02,.1723083E+02,&
     & .5302277E+01,.1816020E+02,.2504538E+02,.2864169E+02,.1962267E+02,&
     & .6628824E+01,.2068096E+02,.2808483E+02,.3173022E+02,.2199260E+02,&
     & .8120143E+01,.2322187E+02,.3108353E+02,.3471491E+02,.2434405E+02,&
     & .9751318E+01,.2577042E+02,.3400727E+02,.3758542E+02,.2664002E+02,&
     & .3760997E+01,.1474503E+02,.2081030E+02,.2417490E+02,.1625573E+02,&
     & .4812522E+01,.1715003E+02,.2381102E+02,.2737145E+02,.1865721E+02,&
     & .6065394E+01,.1965574E+02,.2685618E+02,.3049364E+02,.2103561E+02,&
     & .7493897E+01,.2218813E+02,.2987458E+02,.3351954E+02,.2339558E+02,&
     & .9069722E+01,.2473888E+02,.3283783E+02,.3644425E+02,.2572001E+02,&
     & .3407008E+01,.1387327E+02,.1968672E+02,.2295130E+02,.1534229E+02,&
     & .4384559E+01,.1621715E+02,.2266121E+02,.2616930E+02,.1774766E+02,&
     & .5564493E+01,.1869821E+02,.2569991E+02,.2931565E+02,.2013618E+02,&
     & .6930044E+01,.2122343E+02,.2873366E+02,.3238405E+02,.2250272E+02,&
     & .8451961E+01,.2376957E+02,.3172435E+02,.3535077E+02,.2484828E+02,&
     & .3086932E+01,.1305420E+02,.1860890E+02,.2176702E+02,.1445866E+02,&
     & .3998495E+01,.1533239E+02,.2155796E+02,.2498933E+02,.1686364E+02,&
     & .5105023E+01,.1777463E+02,.2457856E+02,.2816630E+02,.1926213E+02,&
     & .6405638E+01,.2029281E+02,.2762298E+02,.3127387E+02,.2163511E+02,&
     & .7876512E+01,.2283418E+02,.3063295E+02,.3427110E+02,.2399217E+02,&
     & .2791590E+01,.1226154E+02,.1753948E+02,.2058143E+02,.1357966E+02,&
     & .3641206E+01,.1447202E+02,.2046653E+02,.2380567E+02,.1597990E+02,&
     & .4672585E+01,.1686375E+02,.2346271E+02,.2701461E+02,.1838557E+02,&
     & .5905294E+01,.1936440E+02,.2650933E+02,.3014596E+02,.2076820E+02,&
     & .7318334E+01,.2189961E+02,.2953666E+02,.3318455E+02,.2313340E+02,&
     & .2518774E+01,.1149827E+02,.1648112E+02,.1939248E+02,.1269754E+02,&
     & .3307338E+01,.1363661E+02,.1938353E+02,.2262314E+02,.1509794E+02,&
     & .4268652E+01,.1596732E+02,.2235360E+02,.2584534E+02,.1750525E+02,&
     & .5428748E+01,.1843909E+02,.2539018E+02,.2900296E+02,.1989679E+02,&
     & .6779426E+01,.2096706E+02,.2843121E+02,.3208295E+02,.2226801E+02,&
     & .2289464E+01,.1082893E+02,.1552622E+02,.1830474E+02,.1189008E+02,&
     & .3020620E+01,.1289378E+02,.1840077E+02,.2154161E+02,.1429147E+02,&
     & .3922374E+01,.1516305E+02,.2134768E+02,.2476500E+02,.1669686E+02,&
     & .5015015E+01,.1759724E+02,.2436604E+02,.2795197E+02,.1909808E+02,&
     & .6305470E+01,.2011705E+02,.2741465E+02,.3106251E+02,.2147332E+02/

      data   absb(1051:1175,10)   /                                     &
     & .2089789E+01,.1021602E+02,.1463344E+02,.1726758E+02,.1112336E+02,&
     & .2767245E+01,.1220573E+02,.1746541E+02,.2050030E+02,.1351996E+02,&
     & .3612314E+01,.1441119E+02,.2039079E+02,.2372523E+02,.1592114E+02,&
     & .4639139E+01,.1679873E+02,.2338586E+02,.2693757E+02,.1832633E+02,&
     & .5869463E+01,.1930265E+02,.2643475E+02,.3007091E+02,.2071169E+02,&
     & .1902771E+01,.9617449E+01,.1375735E+02,.1623578E+02,.1036934E+02,&
     & .2529334E+01,.1153875E+02,.1653836E+02,.1945888E+02,.1274803E+02,&
     & .3320906E+01,.1368161E+02,.1944359E+02,.2269151E+02,.1514962E+02,&
     & .4286297E+01,.1601548E+02,.2241651E+02,.2591604E+02,.1755694E+02,&
     & .5452967E+01,.1849373E+02,.2545654E+02,.2907074E+02,.1994983E+02,&
     & .1725440E+01,.9031381E+01,.1289432E+02,.1520765E+02,.9626329E+01,&
     & .2309750E+01,.1089872E+02,.1562812E+02,.1842270E+02,.1197707E+02,&
     & .3046266E+01,.1297125E+02,.1850543E+02,.2165803E+02,.1437884E+02,&
     & .3954352E+01,.1524573E+02,.2145498E+02,.2488428E+02,.1678481E+02,&
     & .5056606E+01,.1768754E+02,.2447686E+02,.2806579E+02,.1918626E+02,&
     & .1564823E+01,.8489334E+01,.1209006E+02,.1424280E+02,.8931475E+01,&
     & .2117660E+01,.1031347E+02,.1477913E+02,.1744092E+02,.1125085E+02,&
     & .2802981E+01,.1231464E+02,.1761845E+02,.2067534E+02,.1364984E+02,&
     & .3659632E+01,.1453293E+02,.2055035E+02,.2390260E+02,.1605194E+02,&
     & .4698820E+01,.1693318E+02,.2355149E+02,.2710920E+02,.1845815E+02,&
     & .1502352E+01,.8276308E+01,.1177375E+02,.1385967E+02,.8655479E+01,&
     & .2044007E+01,.1008294E+02,.1444326E+02,.1704897E+02,.1096188E+02,&
     & .2709296E+01,.1205664E+02,.1726418E+02,.2027874E+02,.1335631E+02,&
     & .3545610E+01,.1425196E+02,.2018817E+02,.2350704E+02,.1575791E+02,&
     & .4560655E+01,.1663292E+02,.2318230E+02,.2672443E+02,.1816710E+02/

      data   absb(  1:175,11)   /                                       &
     & .2056300E+03,.1639500E+03,.1513100E+03,.1392800E+03,.1733600E+03,&
     & .2058300E+03,.1634600E+03,.1510200E+03,.1386700E+03,.1720900E+03,&
     & .2057900E+03,.1628200E+03,.1505100E+03,.1380300E+03,.1711900E+03,&
     & .2053800E+03,.1619800E+03,.1500500E+03,.1374600E+03,.1706200E+03,&
     & .2047100E+03,.1610000E+03,.1494200E+03,.1368500E+03,.1699800E+03,&
     & .2138700E+03,.1748600E+03,.1641000E+03,.1489600E+03,.1792600E+03,&
     & .2139800E+03,.1740700E+03,.1635400E+03,.1484900E+03,.1787200E+03,&
     & .2137100E+03,.1730600E+03,.1630000E+03,.1479600E+03,.1783800E+03,&
     & .2132000E+03,.1719300E+03,.1623800E+03,.1474800E+03,.1780700E+03,&
     & .2123900E+03,.1706200E+03,.1615800E+03,.1466100E+03,.1772600E+03,&
     & .2186800E+03,.1839200E+03,.1759700E+03,.1587700E+03,.1840900E+03,&
     & .2186100E+03,.1828500E+03,.1753300E+03,.1584500E+03,.1841300E+03,&
     & .2183800E+03,.1816800E+03,.1746900E+03,.1581400E+03,.1842300E+03,&
     & .2178500E+03,.1802800E+03,.1739200E+03,.1574700E+03,.1839100E+03,&
     & .2171400E+03,.1788100E+03,.1728100E+03,.1564400E+03,.1833700E+03,&
     & .2198900E+03,.1909000E+03,.1866600E+03,.1682300E+03,.1871700E+03,&
     & .2200000E+03,.1898900E+03,.1860400E+03,.1682200E+03,.1879700E+03,&
     & .2198200E+03,.1885600E+03,.1853300E+03,.1679400E+03,.1885100E+03,&
     & .2195400E+03,.1870900E+03,.1843200E+03,.1671500E+03,.1885400E+03,&
     & .2189800E+03,.1854200E+03,.1830300E+03,.1660000E+03,.1883900E+03,&
     & .2179300E+03,.1959600E+03,.1960300E+03,.1770300E+03,.1885700E+03,&
     & .2182100E+03,.1949200E+03,.1954800E+03,.1772500E+03,.1902300E+03,&
     & .2185000E+03,.1936900E+03,.1946200E+03,.1770100E+03,.1913300E+03,&
     & .2185500E+03,.1922100E+03,.1934300E+03,.1761600E+03,.1919400E+03,&
     & .2183900E+03,.1905700E+03,.1920600E+03,.1749000E+03,.1922500E+03,&
     & .2131100E+03,.1993500E+03,.2038500E+03,.1849200E+03,.1886400E+03,&
     & .2139900E+03,.1983800E+03,.2033900E+03,.1853600E+03,.1911700E+03,&
     & .2147700E+03,.1972700E+03,.2025100E+03,.1851700E+03,.1929600E+03,&
     & .2153700E+03,.1960200E+03,.2013100E+03,.1843300E+03,.1942600E+03,&
     & .2159400E+03,.1946000E+03,.1998900E+03,.1829900E+03,.1951100E+03,&
     & .2062800E+03,.2013100E+03,.2101800E+03,.1918100E+03,.1878000E+03,&
     & .2078600E+03,.2006100E+03,.2098500E+03,.1924900E+03,.1911700E+03,&
     & .2093900E+03,.1997800E+03,.2091500E+03,.1924500E+03,.1937900E+03,&
     & .2107800E+03,.1987600E+03,.2080800E+03,.1915900E+03,.1957800E+03,&
     & .2119600E+03,.1976200E+03,.2066600E+03,.1902000E+03,.1972600E+03/

      data   absb(176:350,11)   /                                       &
     & .1980600E+03,.2021500E+03,.2152500E+03,.1978100E+03,.1864600E+03,&
     & .2005400E+03,.2018000E+03,.2152700E+03,.1987400E+03,.1907200E+03,&
     & .2029800E+03,.2013200E+03,.2147800E+03,.1987700E+03,.1941300E+03,&
     & .2052200E+03,.2007000E+03,.2138400E+03,.1979300E+03,.1968100E+03,&
     & .2072500E+03,.1997500E+03,.2124900E+03,.1965900E+03,.1989900E+03,&
     & .1891900E+03,.2021700E+03,.2194200E+03,.2029100E+03,.1849900E+03,&
     & .1926800E+03,.2023900E+03,.2197400E+03,.2041500E+03,.1901200E+03,&
     & .1960900E+03,.2023100E+03,.2194700E+03,.2042500E+03,.1943200E+03,&
     & .1992600E+03,.2020100E+03,.2186600E+03,.2034600E+03,.1977000E+03,&
     & .2022400E+03,.2013600E+03,.2174200E+03,.2021200E+03,.2004400E+03,&
     & .1806400E+03,.2018200E+03,.2227600E+03,.2074000E+03,.1840900E+03,&
     & .1851300E+03,.2025300E+03,.2234100E+03,.2087600E+03,.1898800E+03,&
     & .1895100E+03,.2028700E+03,.2232900E+03,.2089000E+03,.1947400E+03,&
     & .1936800E+03,.2028300E+03,.2225900E+03,.2081200E+03,.1986900E+03,&
     & .1975200E+03,.2024800E+03,.2214300E+03,.2067600E+03,.2018500E+03,&
     & .1726100E+03,.2012900E+03,.2256000E+03,.2112600E+03,.1836000E+03,&
     & .1781300E+03,.2025500E+03,.2264500E+03,.2127700E+03,.1899300E+03,&
     & .1834700E+03,.2032200E+03,.2264900E+03,.2128800E+03,.1953400E+03,&
     & .1885800E+03,.2034500E+03,.2259300E+03,.2121100E+03,.1997200E+03,&
     & .1932500E+03,.2033000E+03,.2247000E+03,.2106600E+03,.2031400E+03,&
     & .1655000E+03,.2007900E+03,.2279700E+03,.2145900E+03,.1835200E+03,&
     & .1719900E+03,.2024700E+03,.2289900E+03,.2161400E+03,.1903100E+03,&
     & .1782900E+03,.2034900E+03,.2292000E+03,.2162100E+03,.1961000E+03,&
     & .1842400E+03,.2039500E+03,.2286500E+03,.2154400E+03,.2008000E+03,&
     & .1896600E+03,.2039300E+03,.2273500E+03,.2138800E+03,.2043700E+03,&
     & .1594900E+03,.2004800E+03,.2299900E+03,.2175800E+03,.1839500E+03,&
     & .1669800E+03,.2024700E+03,.2311900E+03,.2190300E+03,.1910800E+03,&
     & .1741100E+03,.2037000E+03,.2314400E+03,.2190400E+03,.1971100E+03,&
     & .1808800E+03,.2043500E+03,.2308200E+03,.2181700E+03,.2019200E+03,&
     & .1871000E+03,.2045000E+03,.2295000E+03,.2164800E+03,.2055900E+03,&
     & .1547900E+03,.2003800E+03,.2318100E+03,.2201600E+03,.1849000E+03,&
     & .1632200E+03,.2025800E+03,.2330700E+03,.2214500E+03,.1922200E+03,&
     & .1711900E+03,.2040100E+03,.2332600E+03,.2214000E+03,.1983100E+03,&
     & .1786600E+03,.2047800E+03,.2325900E+03,.2203800E+03,.2031500E+03,&
     & .1855300E+03,.2049800E+03,.2312200E+03,.2185400E+03,.2068500E+03/

      data   absb(351:525,11)   /                                       &
     & .1514300E+03,.2004900E+03,.2334400E+03,.2224100E+03,.1861500E+03,&
     & .1607000E+03,.2027900E+03,.2346500E+03,.2235200E+03,.1935300E+03,&
     & .1693200E+03,.2043000E+03,.2347300E+03,.2233100E+03,.1996000E+03,&
     & .1773700E+03,.2051800E+03,.2340200E+03,.2221200E+03,.2044000E+03,&
     & .1847200E+03,.2053700E+03,.2325700E+03,.2201500E+03,.2080700E+03,&
     & .1493500E+03,.2007800E+03,.2348700E+03,.2243600E+03,.1876200E+03,&
     & .1592600E+03,.2031200E+03,.2359700E+03,.2252300E+03,.1949100E+03,&
     & .1683900E+03,.2046700E+03,.2359700E+03,.2248400E+03,.2009200E+03,&
     & .1768700E+03,.2055400E+03,.2351600E+03,.2234500E+03,.2056200E+03,&
     & .1845300E+03,.2057000E+03,.2335700E+03,.2213500E+03,.2092100E+03,&
     & .1484400E+03,.2011700E+03,.2361400E+03,.2260100E+03,.1892700E+03,&
     & .1587800E+03,.2035000E+03,.2370700E+03,.2266400E+03,.1964200E+03,&
     & .1682900E+03,.2050500E+03,.2369600E+03,.2260100E+03,.2022900E+03,&
     & .1770200E+03,.2058500E+03,.2360200E+03,.2244500E+03,.2068500E+03,&
     & .1848700E+03,.2059900E+03,.2343000E+03,.2222000E+03,.2103100E+03,&
     & .1484100E+03,.2016500E+03,.2372400E+03,.2274200E+03,.1910100E+03,&
     & .1590300E+03,.2039500E+03,.2379800E+03,.2277400E+03,.1979600E+03,&
     & .1687800E+03,.2054300E+03,.2377400E+03,.2268900E+03,.2036700E+03,&
     & .1776500E+03,.2061400E+03,.2366200E+03,.2251500E+03,.2080400E+03,&
     & .1855600E+03,.2062600E+03,.2347800E+03,.2227400E+03,.2113300E+03,&
     & .1491000E+03,.2021900E+03,.2381600E+03,.2285700E+03,.1928000E+03,&
     & .1598700E+03,.2044200E+03,.2387300E+03,.2286000E+03,.1995500E+03,&
     & .1697400E+03,.2057800E+03,.2383000E+03,.2275100E+03,.2050300E+03,&
     & .1787000E+03,.2064000E+03,.2370300E+03,.2255900E+03,.2091700E+03,&
     & .1865600E+03,.2064700E+03,.2350400E+03,.2230300E+03,.2123200E+03,&
     & .1503100E+03,.2027900E+03,.2389700E+03,.2294900E+03,.1945900E+03,&
     & .1611900E+03,.2048900E+03,.2393100E+03,.2292200E+03,.2011300E+03,&
     & .1711000E+03,.2061000E+03,.2386900E+03,.2279100E+03,.2063400E+03,&
     & .1800200E+03,.2066200E+03,.2372300E+03,.2258200E+03,.2102900E+03,&
     & .1877700E+03,.2065900E+03,.2351000E+03,.2230800E+03,.2132200E+03,&
     & .1519700E+03,.2034100E+03,.2396500E+03,.2301900E+03,.1963900E+03,&
     & .1628500E+03,.2053400E+03,.2397600E+03,.2296400E+03,.2026700E+03,&
     & .1727000E+03,.2063800E+03,.2389300E+03,.2281100E+03,.2075800E+03,&
     & .1815100E+03,.2068000E+03,.2372700E+03,.2258400E+03,.2113200E+03,&
     & .1891200E+03,.2066500E+03,.2350000E+03,.2229600E+03,.2140600E+03/

      data   absb(526:700,11)   /                                       &
     & .1535200E+03,.2039100E+03,.2401400E+03,.2306900E+03,.1979000E+03,&
     & .1643700E+03,.2056600E+03,.2400600E+03,.2299200E+03,.2039600E+03,&
     & .1741600E+03,.2066200E+03,.2390700E+03,.2282300E+03,.2086300E+03,&
     & .1828500E+03,.2069500E+03,.2372800E+03,.2258000E+03,.2121800E+03,&
     & .1903300E+03,.2066700E+03,.2348700E+03,.2228000E+03,.2147400E+03,&
     & .1542100E+03,.2041800E+03,.2405200E+03,.2311000E+03,.1987600E+03,&
     & .1650700E+03,.2058600E+03,.2403300E+03,.2302000E+03,.2046800E+03,&
     & .1748600E+03,.2067700E+03,.2392400E+03,.2284100E+03,.2092300E+03,&
     & .1835000E+03,.2070500E+03,.2373700E+03,.2258900E+03,.2126900E+03,&
     & .1909400E+03,.2066900E+03,.2348900E+03,.2228100E+03,.2151400E+03,&
     & .1538100E+03,.2042400E+03,.2408000E+03,.2314500E+03,.1988800E+03,&
     & .1647800E+03,.2059200E+03,.2406000E+03,.2305500E+03,.2048400E+03,&
     & .1746400E+03,.2068400E+03,.2395000E+03,.2287400E+03,.2093800E+03,&
     & .1833600E+03,.2071500E+03,.2376200E+03,.2261800E+03,.2128400E+03,&
     & .1908600E+03,.2067800E+03,.2351400E+03,.2231100E+03,.2153200E+03,&
     & .1517700E+03,.2039400E+03,.2409600E+03,.2317800E+03,.1980200E+03,&
     & .1629800E+03,.2057800E+03,.2409100E+03,.2310200E+03,.2041600E+03,&
     & .1730700E+03,.2068100E+03,.2399400E+03,.2293400E+03,.2089300E+03,&
     & .1820400E+03,.2072100E+03,.2381800E+03,.2268800E+03,.2125500E+03,&
     & .1897300E+03,.2069500E+03,.2357300E+03,.2238300E+03,.2151600E+03,&
     & .1496800E+03,.2035900E+03,.2410400E+03,.2320200E+03,.1970300E+03,&
     & .1611300E+03,.2056000E+03,.2411600E+03,.2314300E+03,.2034100E+03,&
     & .1714400E+03,.2067400E+03,.2403300E+03,.2298800E+03,.2083800E+03,&
     & .1806300E+03,.2072400E+03,.2386600E+03,.2275200E+03,.2121600E+03,&
     & .1885500E+03,.2071000E+03,.2363200E+03,.2245300E+03,.2149300E+03,&
     & .1476400E+03,.2032200E+03,.2410600E+03,.2322000E+03,.1960400E+03,&
     & .1593200E+03,.2054000E+03,.2413700E+03,.2318000E+03,.2026200E+03,&
     & .1698400E+03,.2066500E+03,.2406500E+03,.2303600E+03,.2078100E+03,&
     & .1792300E+03,.2072500E+03,.2391000E+03,.2281000E+03,.2117500E+03,&
     & .1873700E+03,.2072200E+03,.2368400E+03,.2251700E+03,.2146700E+03,&
     & .1443800E+03,.2025400E+03,.2409200E+03,.2322500E+03,.1942500E+03,&
     & .1563300E+03,.2049800E+03,.2415200E+03,.2321700E+03,.2011500E+03,&
     & .1671800E+03,.2064200E+03,.2410500E+03,.2309600E+03,.2067200E+03,&
     & .1769000E+03,.2072000E+03,.2396900E+03,.2288700E+03,.2109300E+03,&
     & .1853900E+03,.2073300E+03,.2375800E+03,.2260900E+03,.2141100E+03/

      data   absb(701:875,11)   /                                       &
     & .1409300E+03,.2017000E+03,.2406200E+03,.2321400E+03,.1922600E+03,&
     & .1530900E+03,.2044100E+03,.2415500E+03,.2324300E+03,.1995100E+03,&
     & .1643400E+03,.2061300E+03,.2413600E+03,.2314900E+03,.2054500E+03,&
     & .1743900E+03,.2070900E+03,.2402200E+03,.2296300E+03,.2100000E+03,&
     & .1832500E+03,.2073900E+03,.2382900E+03,.2270000E+03,.2134400E+03,&
     & .1374200E+03,.2007200E+03,.2401800E+03,.2318800E+03,.1901700E+03,&
     & .1498700E+03,.2038100E+03,.2415100E+03,.2326000E+03,.1977800E+03,&
     & .1614400E+03,.2058000E+03,.2415900E+03,.2319400E+03,.2040800E+03,&
     & .1718200E+03,.2069300E+03,.2406800E+03,.2303000E+03,.2089700E+03,&
     & .1810100E+03,.2073900E+03,.2389400E+03,.2278500E+03,.2126900E+03,&
     & .1331600E+03,.1994600E+03,.2395200E+03,.2313400E+03,.1874600E+03,&
     & .1459000E+03,.2029900E+03,.2413100E+03,.2326500E+03,.1955500E+03,&
     & .1578200E+03,.2053100E+03,.2417600E+03,.2323700E+03,.2022800E+03,&
     & .1686000E+03,.2066600E+03,.2411300E+03,.2309900E+03,.2076100E+03,&
     & .1781800E+03,.2073300E+03,.2396400E+03,.2287800E+03,.2116600E+03,&
     & .1285100E+03,.1979300E+03,.2386000E+03,.2305100E+03,.1844000E+03,&
     & .1415600E+03,.2019500E+03,.2409200E+03,.2324900E+03,.1929800E+03,&
     & .1538100E+03,.2046800E+03,.2418200E+03,.2326800E+03,.2001500E+03,&
     & .1649400E+03,.2062500E+03,.2414800E+03,.2316300E+03,.2059700E+03,&
     & .1749900E+03,.2071900E+03,.2402900E+03,.2296900E+03,.2104300E+03,&
     & .1237700E+03,.1962200E+03,.2374500E+03,.2293800E+03,.1811400E+03,&
     & .1371300E+03,.2007400E+03,.2403500E+03,.2321000E+03,.1902400E+03,&
     & .1496900E+03,.2038900E+03,.2417100E+03,.2328300E+03,.1978700E+03,&
     & .1612200E+03,.2058000E+03,.2417200E+03,.2321500E+03,.2041700E+03,&
     & .1716900E+03,.2069700E+03,.2408500E+03,.2305100E+03,.2090700E+03,&
     & .1187100E+03,.1941800E+03,.2360000E+03,.2279000E+03,.1774700E+03,&
     & .1323100E+03,.1992900E+03,.2395400E+03,.2314300E+03,.1871200E+03,&
     & .1451400E+03,.2028800E+03,.2413800E+03,.2328000E+03,.1952600E+03,&
     & .1571100E+03,.2052300E+03,.2418700E+03,.2325700E+03,.2020700E+03,&
     & .1680200E+03,.2066400E+03,.2413200E+03,.2312600E+03,.2074900E+03,&
     & .1129500E+03,.1915900E+03,.2339800E+03,.2258800E+03,.1731300E+03,&
     & .1268300E+03,.1974100E+03,.2383500E+03,.2303100E+03,.1834200E+03,&
     & .1399900E+03,.2015900E+03,.2408500E+03,.2325300E+03,.1921700E+03,&
     & .1523400E+03,.2044200E+03,.2418600E+03,.2328600E+03,.1994900E+03,&
     & .1637300E+03,.2061800E+03,.2416900E+03,.2319400E+03,.2054900E+03/

      data   absb(876:1050,11)   /                                      &
     & .1071200E+03,.1887100E+03,.2315900E+03,.2234700E+03,.1685000E+03,&
     & .1212400E+03,.1952900E+03,.2368700E+03,.2288200E+03,.1794600E+03,&
     & .1347300E+03,.2000700E+03,.2400500E+03,.2319100E+03,.1888300E+03,&
     & .1474200E+03,.2034200E+03,.2416400E+03,.2329400E+03,.1967000E+03,&
     & .1592800E+03,.2056200E+03,.2419200E+03,.2324700E+03,.2032500E+03,&
     & .1012000E+03,.1855000E+03,.2288000E+03,.2207100E+03,.1636300E+03,&
     & .1155600E+03,.1928500E+03,.2350400E+03,.2269700E+03,.1752400E+03,&
     & .1293200E+03,.1983200E+03,.2390100E+03,.2309700E+03,.1852400E+03,&
     & .1423800E+03,.2022600E+03,.2412000E+03,.2327800E+03,.1937000E+03,&
     & .1546000E+03,.2048800E+03,.2419900E+03,.2328300E+03,.2007800E+03,&
     & .9560300E+02,.1821600E+03,.2257600E+03,.2176800E+03,.1587800E+03,&
     & .1101500E+03,.1903100E+03,.2329800E+03,.2249000E+03,.1710500E+03,&
     & .1241500E+03,.1964600E+03,.2377500E+03,.2297500E+03,.1816600E+03,&
     & .1375200E+03,.2009400E+03,.2405800E+03,.2323600E+03,.1906900E+03,&
     & .1500700E+03,.2040300E+03,.2418700E+03,.2329900E+03,.1982600E+03,&
     & .9013700E+02,.1786800E+03,.2224000E+03,.2144200E+03,.1538400E+03,&
     & .1048400E+03,.1875700E+03,.2306600E+03,.2225800E+03,.1667700E+03,&
     & .1190700E+03,.1944200E+03,.2362800E+03,.2282400E+03,.1779600E+03,&
     & .1326900E+03,.1994800E+03,.2397600E+03,.2316700E+03,.1875600E+03,&
     & .1455200E+03,.2030200E+03,.2415400E+03,.2329600E+03,.1956500E+03,&
     & .8469000E+02,.1749600E+03,.2186400E+03,.2108000E+03,.1486200E+03,&
     & .9947100E+02,.1845300E+03,.2279800E+03,.2199200E+03,.1622500E+03,&
     & .1139100E+03,.1921300E+03,.2345100E+03,.2264400E+03,.1740600E+03,&
     & .1277700E+03,.1978100E+03,.2387200E+03,.2307000E+03,.1842400E+03,&
     & .1409100E+03,.2018900E+03,.2410700E+03,.2327300E+03,.1928600E+03,&
     & .7926200E+02,.1709500E+03,.2143600E+03,.2068100E+03,.1431700E+03,&
     & .9404700E+02,.1812600E+03,.2249400E+03,.2169000E+03,.1574800E+03,&
     & .1086500E+03,.1895800E+03,.2324100E+03,.2243400E+03,.1699400E+03,&
     & .1227400E+03,.1959500E+03,.2374200E+03,.2294100E+03,.1807000E+03,&
     & .1361800E+03,.2005600E+03,.2404000E+03,.2322400E+03,.1898800E+03,&
     & .7431200E+02,.1670100E+03,.2100000E+03,.2028300E+03,.1380100E+03,&
     & .8905900E+02,.1780200E+03,.2217700E+03,.2138100E+03,.1528900E+03,&
     & .1037900E+03,.1870100E+03,.2302100E+03,.2221400E+03,.1659500E+03,&
     & .1180800E+03,.1940300E+03,.2360200E+03,.2279700E+03,.1772800E+03,&
     & .1317400E+03,.1991800E+03,.2396100E+03,.2315500E+03,.1869700E+03/

      data   absb(1051:1175,11)   /                                     &
     & .6961500E+02,.1629600E+03,.2054000E+03,.1986600E+03,.1328800E+03,&
     & .8427600E+02,.1746600E+03,.2183500E+03,.2105600E+03,.1482800E+03,&
     & .9906600E+02,.1843300E+03,.2278200E+03,.2197700E+03,.1619700E+03,&
     & .1135400E+03,.1919900E+03,.2344200E+03,.2263600E+03,.1738300E+03,&
     & .1274100E+03,.1977000E+03,.2386600E+03,.2306500E+03,.1840300E+03,&
     & .6494400E+02,.1587700E+03,.2004400E+03,.1942100E+03,.1276200E+03,&
     & .7952400E+02,.1711400E+03,.2146100E+03,.2070700E+03,.1435100E+03,&
     & .9432300E+02,.1814400E+03,.2251400E+03,.2171100E+03,.1577800E+03,&
     & .1089400E+03,.1897600E+03,.2325700E+03,.2245200E+03,.1702200E+03,&
     & .1230100E+03,.1960500E+03,.2375200E+03,.2295400E+03,.1809400E+03,&
     & .6035300E+02,.1543100E+03,.1951400E+03,.1894500E+03,.1222500E+03,&
     & .7481500E+02,.1674200E+03,.2104900E+03,.2032900E+03,.1386000E+03,&
     & .8956700E+02,.1783700E+03,.2221400E+03,.2141900E+03,.1534300E+03,&
     & .1043100E+03,.1873300E+03,.2305000E+03,.2224400E+03,.1664400E+03,&
     & .1185600E+03,.1942300E+03,.2361800E+03,.2281600E+03,.1776700E+03,&
     & .5609800E+02,.1499100E+03,.1898000E+03,.1846500E+03,.1170600E+03,&
     & .7037700E+02,.1636700E+03,.2062200E+03,.1994200E+03,.1337800E+03,&
     & .8504900E+02,.1752800E+03,.2190200E+03,.2111900E+03,.1491000E+03,&
     & .9985700E+02,.1848200E+03,.2282700E+03,.2202200E+03,.1626800E+03,&
     & .1142800E+03,.1923300E+03,.2347000E+03,.2266600E+03,.1744400E+03,&
     & .5441800E+02,.1480900E+03,.1875400E+03,.1826100E+03,.1149500E+03,&
     & .6859800E+02,.1621000E+03,.2044100E+03,.1977800E+03,.1318000E+03,&
     & .8324600E+02,.1739800E+03,.2176800E+03,.2099300E+03,.1473300E+03,&
     & .9805500E+02,.1837500E+03,.2273200E+03,.2192700E+03,.1611300E+03,&
     & .1125500E+03,.1915300E+03,.2340600E+03,.2260100E+03,.1731000E+03/

      data   absb(  1:175,12)   /                                       &
     & .4338110E+03,.3258785E+03,.2765841E+03,.3551700E+03,.4738213E+03,&
     & .4358591E+03,.3274735E+03,.2732330E+03,.3477157E+03,.4639305E+03,&
     & .4367994E+03,.3282050E+03,.2698072E+03,.3403736E+03,.4541170E+03,&
     & .4367673E+03,.3281913E+03,.2662171E+03,.3330191E+03,.4442737E+03,&
     & .4356257E+03,.3273304E+03,.2624520E+03,.3260525E+03,.4349606E+03,&
     & .4965751E+03,.3730543E+03,.3165865E+03,.3940821E+03,.5257811E+03,&
     & .4977615E+03,.3739641E+03,.3123416E+03,.3853185E+03,.5140967E+03,&
     & .4978529E+03,.3740600E+03,.3078301E+03,.3766318E+03,.5024939E+03,&
     & .4965358E+03,.3731122E+03,.3030947E+03,.3681551E+03,.4911078E+03,&
     & .4942501E+03,.3714089E+03,.2982254E+03,.3606307E+03,.4809116E+03,&
     & .5631880E+03,.4233224E+03,.3594901E+03,.4335570E+03,.5784608E+03,&
     & .5634438E+03,.4233039E+03,.3539288E+03,.4233711E+03,.5648410E+03,&
     & .5619418E+03,.4221964E+03,.3480452E+03,.4132459E+03,.5512312E+03,&
     & .5592483E+03,.4202048E+03,.3418750E+03,.4039045E+03,.5386286E+03,&
     & .5551436E+03,.4171466E+03,.3356718E+03,.3953284E+03,.5268417E+03,&
     & .6327464E+03,.4759753E+03,.4043694E+03,.4738466E+03,.6318088E+03,&
     & .6309974E+03,.4742402E+03,.3971672E+03,.4618354E+03,.6156456E+03,&
     & .6278582E+03,.4716973E+03,.3895188E+03,.4502024E+03,.5999656E+03,&
     & .6230476E+03,.4681132E+03,.3817116E+03,.4394718E+03,.5854793E+03,&
     & .6168383E+03,.4634762E+03,.3737303E+03,.4293418E+03,.5716261E+03,&
     & .7031362E+03,.5296686E+03,.4502274E+03,.5144971E+03,.6844628E+03,&
     & .6994439E+03,.5260700E+03,.4409153E+03,.5003459E+03,.6654934E+03,&
     & .6938124E+03,.5214150E+03,.4313297E+03,.4869200E+03,.6474992E+03,&
     & .6865591E+03,.5157531E+03,.4215792E+03,.4743711E+03,.6306843E+03,&
     & .6778828E+03,.5092809E+03,.4115306E+03,.4623577E+03,.6144981E+03,&
     & .7731573E+03,.5833614E+03,.4960552E+03,.5546751E+03,.7351434E+03,&
     & .7667584E+03,.5774695E+03,.4843214E+03,.5381637E+03,.7133093E+03,&
     & .7584006E+03,.5703374E+03,.4724758E+03,.5225411E+03,.6927570E+03,&
     & .7484491E+03,.5623616E+03,.4604054E+03,.5079339E+03,.6733220E+03,&
     & .7367135E+03,.5534249E+03,.4480895E+03,.4939401E+03,.6546934E+03,&
     & .8409349E+03,.6358707E+03,.5406929E+03,.5933755E+03,.7826997E+03,&
     & .8314773E+03,.6270808E+03,.5263991E+03,.5744137E+03,.7580256E+03,&
     & .8200103E+03,.6173497E+03,.5119158E+03,.5563407E+03,.7348534E+03,&
     & .8068534E+03,.6066626E+03,.4972618E+03,.5395008E+03,.7127611E+03,&
     & .7922845E+03,.5951793E+03,.4826400E+03,.5234260E+03,.6916658E+03/

      data   absb(176:350,12)   /                                       &
     & .9048913E+03,.6856063E+03,.5829003E+03,.6293251E+03,.8259779E+03,&
     & .8920622E+03,.6739392E+03,.5656996E+03,.6076975E+03,.7986291E+03,&
     & .8773064E+03,.6613225E+03,.5484044E+03,.5872679E+03,.7727058E+03,&
     & .8609036E+03,.6477146E+03,.5311640E+03,.5681427E+03,.7481234E+03,&
     & .8431150E+03,.6336171E+03,.5140156E+03,.5498732E+03,.7244630E+03,&
     & .9636991E+03,.7316951E+03,.6218536E+03,.6620342E+03,.8647359E+03,&
     & .9475066E+03,.7168594E+03,.6016241E+03,.6376903E+03,.8346844E+03,&
     & .9293968E+03,.7013269E+03,.5815752E+03,.6149608E+03,.8061544E+03,&
     & .9098003E+03,.6849951E+03,.5617381E+03,.5935925E+03,.7790911E+03,&
     & .8886450E+03,.6681002E+03,.5420821E+03,.5733205E+03,.7530675E+03,&
     & .1015405E+04,.7723224E+03,.6554984E+03,.6893842E+03,.8965322E+03,&
     & .9957317E+03,.7543125E+03,.6322716E+03,.6625732E+03,.8641101E+03,&
     & .9742889E+03,.7357574E+03,.6095984E+03,.6377464E+03,.8332099E+03,&
     & .9513341E+03,.7167192E+03,.5872099E+03,.6143318E+03,.8038674E+03,&
     & .9271699E+03,.6973008E+03,.5653278E+03,.5922105E+03,.7757920E+03,&
     & .1060462E+04,.8075265E+03,.6843897E+03,.7124747E+03,.9230005E+03,&
     & .1037459E+04,.7864953E+03,.6584383E+03,.6833274E+03,.8883530E+03,&
     & .1012792E+04,.7651897E+03,.6331576E+03,.6565949E+03,.8552652E+03,&
     & .9866354E+03,.7436769E+03,.6084848E+03,.6313485E+03,.8239323E+03,&
     & .9595800E+03,.7219245E+03,.5846257E+03,.6076220E+03,.7941595E+03,&
     & .1098608E+04,.8371963E+03,.7083084E+03,.7311988E+03,.9441461E+03,&
     & .1072494E+04,.8134318E+03,.6798957E+03,.7001704E+03,.9076022E+03,&
     & .1044667E+04,.7896182E+03,.6523292E+03,.6716308E+03,.8727223E+03,&
     & .1015777E+04,.7658591E+03,.6257102E+03,.6448513E+03,.8396486E+03,&
     & .9862086E+03,.7420869E+03,.6001289E+03,.6196814E+03,.8085246E+03,&
     & .1130080E+04,.8613842E+03,.7273980E+03,.7455565E+03,.9603143E+03,&
     & .1100795E+04,.8351571E+03,.6967578E+03,.7129616E+03,.9220699E+03,&
     & .1070329E+04,.8092621E+03,.6672876E+03,.6829948E+03,.8856533E+03,&
     & .1038932E+04,.7834641E+03,.6390022E+03,.6549212E+03,.8512901E+03,&
     & .1007073E+04,.7578586E+03,.6119071E+03,.6285978E+03,.8190226E+03,&
     & .1154817E+04,.8801654E+03,.7415039E+03,.7556111E+03,.9714143E+03,&
     & .1122664E+04,.8518781E+03,.7090277E+03,.7218209E+03,.9318180E+03,&
     & .1089916E+04,.8241127E+03,.6780777E+03,.6907287E+03,.8943125E+03,&
     & .1056461E+04,.7965849E+03,.6484079E+03,.6616194E+03,.8589272E+03,&
     & .1022590E+04,.7695324E+03,.6201114E+03,.6344623E+03,.8258624E+03/

      data   absb(351:525,12)   /                                       &
     & .1173826E+04,.8944232E+03,.7516288E+03,.7622958E+03,.9786164E+03,&
     & .1139213E+04,.8644602E+03,.7176974E+03,.7275753E+03,.9379617E+03,&
     & .1104499E+04,.8350215E+03,.6855100E+03,.6956267E+03,.8996608E+03,&
     & .1069228E+04,.8060327E+03,.6547517E+03,.6658171E+03,.8636644E+03,&
     & .1033679E+04,.7777792E+03,.6254959E+03,.6380265E+03,.8299256E+03,&
     & .1187612E+04,.9045966E+03,.7582339E+03,.7660705E+03,.9825372E+03,&
     & .1151104E+04,.8732077E+03,.7232198E+03,.7307711E+03,.9412613E+03,&
     & .1114812E+04,.8425100E+03,.6900707E+03,.6982050E+03,.9022920E+03,&
     & .1077922E+04,.8124079E+03,.6584780E+03,.6679077E+03,.8658499E+03,&
     & .1040927E+04,.7831204E+03,.6285352E+03,.6396645E+03,.8317461E+03,&
     & .1196774E+04,.9111403E+03,.7616196E+03,.7671722E+03,.9834539E+03,&
     & .1158870E+04,.8786102E+03,.7258194E+03,.7315277E+03,.9417190E+03,&
     & .1121068E+04,.8468836E+03,.6919844E+03,.6985953E+03,.9023781E+03,&
     & .1082920E+04,.8159387E+03,.6597813E+03,.6679809E+03,.8657188E+03,&
     & .1044783E+04,.7858772E+03,.6294298E+03,.6395237E+03,.8313784E+03,&
     & .1202260E+04,.9148111E+03,.7624882E+03,.7661952E+03,.9820032E+03,&
     & .1163158E+04,.8813333E+03,.7261740E+03,.7304250E+03,.9399991E+03,&
     & .1124180E+04,.8488606E+03,.6918461E+03,.6972877E+03,.9005744E+03,&
     & .1084954E+04,.8172592E+03,.6592986E+03,.6665878E+03,.8638719E+03,&
     & .1045950E+04,.7865464E+03,.6286194E+03,.6380504E+03,.8294321E+03,&
     & .1204456E+04,.9157270E+03,.7610818E+03,.7633951E+03,.9784831E+03,&
     & .1164367E+04,.8816366E+03,.7244819E+03,.7276259E+03,.9364715E+03,&
     & .1124324E+04,.8486129E+03,.6898842E+03,.6945597E+03,.8971270E+03,&
     & .1084324E+04,.8165570E+03,.6571659E+03,.6638568E+03,.8604884E+03,&
     & .1044694E+04,.7853529E+03,.6263972E+03,.6354464E+03,.8261775E+03,&
     & .1204084E+04,.9146184E+03,.7579482E+03,.7592747E+03,.9734683E+03,&
     & .1162995E+04,.8799770E+03,.7211610E+03,.7236223E+03,.9315108E+03,&
     & .1122270E+04,.8466270E+03,.6864778E+03,.6906781E+03,.8923413E+03,&
     & .1081628E+04,.8142783E+03,.6537748E+03,.6601480E+03,.8559108E+03,&
     & .1041535E+04,.7827690E+03,.6229791E+03,.6318521E+03,.8217287E+03,&
     & .1201431E+04,.9117433E+03,.7533822E+03,.7540042E+03,.9671155E+03,&
     & .1159753E+04,.8768687E+03,.7166394E+03,.7186372E+03,.9254210E+03,&
     & .1118355E+04,.8432577E+03,.6819604E+03,.6859134E+03,.8865933E+03,&
     & .1077205E+04,.8106161E+03,.6493437E+03,.6556209E+03,.8503919E+03,&
     & .1036866E+04,.7790721E+03,.6187296E+03,.6275874E+03,.8165163E+03/

      data   absb(526:700,12)   /                                       &
     & .1198563E+04,.9088441E+03,.7491554E+03,.7492438E+03,.9613627E+03,&
     & .1156284E+04,.8737803E+03,.7124077E+03,.7140741E+03,.9199132E+03,&
     & .1114455E+04,.8399712E+03,.6778293E+03,.6816338E+03,.8814481E+03,&
     & .1073022E+04,.8072052E+03,.6453367E+03,.6515614E+03,.8454321E+03,&
     & .1032356E+04,.7755667E+03,.6148054E+03,.6237372E+03,.8118204E+03,&
     & .1198333E+04,.9083513E+03,.7477168E+03,.7473560E+03,.9590262E+03,&
     & .1155753E+04,.8731138E+03,.7109009E+03,.7122360E+03,.9176511E+03,&
     & .1113496E+04,.8390696E+03,.6762632E+03,.6798170E+03,.8792408E+03,&
     & .1071776E+04,.8061247E+03,.6437713E+03,.6498449E+03,.8432884E+03,&
     & .1030817E+04,.7743687E+03,.6132096E+03,.6220561E+03,.8097579E+03,&
     & .1201812E+04,.9109743E+03,.7497301E+03,.7489114E+03,.9607489E+03,&
     & .1158859E+04,.8754611E+03,.7126895E+03,.7135931E+03,.9191682E+03,&
     & .1116249E+04,.8411676E+03,.6778498E+03,.6810248E+03,.8806473E+03,&
     & .1074182E+04,.8079523E+03,.6451507E+03,.6509316E+03,.8445579E+03,&
     & .1033029E+04,.7760505E+03,.6144744E+03,.6230304E+03,.8108939E+03,&
     & .1211130E+04,.9186048E+03,.7571510E+03,.7557283E+03,.9686341E+03,&
     & .1167769E+04,.8825784E+03,.7195377E+03,.7197860E+03,.9264272E+03,&
     & .1124863E+04,.8479369E+03,.6841799E+03,.6866852E+03,.8873138E+03,&
     & .1082412E+04,.8143560E+03,.6510060E+03,.6560895E+03,.8507530E+03,&
     & .1040908E+04,.7820519E+03,.6199358E+03,.6277981E+03,.8166320E+03,&
     & .1220285E+04,.9261353E+03,.7646141E+03,.7626244E+03,.9766109E+03,&
     & .1176440E+04,.8896203E+03,.7264230E+03,.7260792E+03,.9337937E+03,&
     & .1133375E+04,.8546623E+03,.6905960E+03,.6924389E+03,.8940982E+03,&
     & .1090677E+04,.8207879E+03,.6569885E+03,.6613756E+03,.8570661E+03,&
     & .1048813E+04,.7881007E+03,.6254696E+03,.6327012E+03,.8225054E+03,&
     & .1228866E+04,.9332604E+03,.7717386E+03,.7692485E+03,.9842332E+03,&
     & .1184837E+04,.8964165E+03,.7330783E+03,.7321425E+03,.9408794E+03,&
     & .1141425E+04,.8610630E+03,.6967646E+03,.6979977E+03,.9006456E+03,&
     & .1098473E+04,.8268792E+03,.6626986E+03,.6664793E+03,.8631791E+03,&
     & .1056323E+04,.7938687E+03,.6308135E+03,.6374322E+03,.8281892E+03,&
     & .1241753E+04,.9440723E+03,.7828610E+03,.7798359E+03,.9964019E+03,&
     & .1197417E+04,.9066817E+03,.7434365E+03,.7418632E+03,.9522524E+03,&
     & .1153661E+04,.8708987E+03,.7064391E+03,.7069149E+03,.9111337E+03,&
     & .1110381E+04,.8362458E+03,.6717052E+03,.6746952E+03,.8729740E+03,&
     & .1067895E+04,.8028379E+03,.6392420E+03,.6449926E+03,.8372651E+03/

      data   absb(701:875,12)   /                                       &
     & .1254711E+04,.9552218E+03,.7944804E+03,.7910279E+03,.1009186E+04,&
     & .1210280E+04,.9173371E+03,.7543339E+03,.7521533E+03,.9642251E+03,&
     & .1166182E+04,.8810139E+03,.7165982E+03,.7163206E+03,.9222282E+03,&
     & .1122776E+04,.8459987E+03,.6811757E+03,.6833364E+03,.8832595E+03,&
     & .1079901E+04,.8122269E+03,.6480909E+03,.6529289E+03,.8468137E+03,&
     & .1267704E+04,.9665631E+03,.8063197E+03,.8024662E+03,.1022026E+04,&
     & .1222982E+04,.9279730E+03,.7652524E+03,.7624963E+03,.9762665E+03,&
     & .1178689E+04,.8911031E+03,.7268015E+03,.7258004E+03,.9333389E+03,&
     & .1134938E+04,.8556974E+03,.6907044E+03,.6920393E+03,.8935613E+03,&
     & .1091744E+04,.8215054E+03,.6569768E+03,.6609780E+03,.8565207E+03,&
     & .1282885E+04,.9799862E+03,.8205225E+03,.8164768E+03,.1037659E+04,&
     & .1238160E+04,.9407046E+03,.7785501E+03,.7751742E+03,.9909163E+03,&
     & .1193461E+04,.9032104E+03,.7391944E+03,.7374586E+03,.9470329E+03,&
     & .1149466E+04,.8673209E+03,.7023116E+03,.7027833E+03,.9062086E+03,&
     & .1106044E+04,.8327209E+03,.6678128E+03,.6708431E+03,.8683165E+03,&
     & .1299221E+04,.9946371E+03,.8360635E+03,.8320461E+03,.1054652E+04,&
     & .1254105E+04,.9544361E+03,.7930175E+03,.7891647E+03,.1006958E+04,&
     & .1209314E+04,.9163157E+03,.7527076E+03,.7503398E+03,.9620928E+03,&
     & .1165150E+04,.8800178E+03,.7151041E+03,.7146356E+03,.9201686E+03,&
     & .1121407E+04,.8448306E+03,.6796764E+03,.6817394E+03,.8813213E+03,&
     & .1315203E+04,.1009411E+04,.8519211E+03,.8481780E+03,.1072024E+04,&
     & .1270148E+04,.9683874E+03,.8077450E+03,.8035704E+03,.1023229E+04,&
     & .1225178E+04,.9296125E+03,.7664897E+03,.7634566E+03,.9773360E+03,&
     & .1180684E+04,.8926430E+03,.7280109E+03,.7267010E+03,.9343522E+03,&
     & .1136746E+04,.8570580E+03,.6917330E+03,.6928240E+03,.8944515E+03,&
     & .1332014E+04,.1025376E+04,.8690997E+03,.8658147E+03,.1090797E+04,&
     & .1287083E+04,.9833984E+03,.8237334E+03,.8194241E+03,.1040908E+04,&
     & .1241959E+04,.9438288E+03,.7814714E+03,.7777789E+03,.9938697E+03,&
     & .1197174E+04,.9061941E+03,.7419541E+03,.7398805E+03,.9497877E+03,&
     & .1152918E+04,.8700812E+03,.7048184E+03,.7049556E+03,.9087731E+03,&
     & .1350647E+04,.1043564E+04,.8888305E+03,.8860745E+03,.1112065E+04,&
     & .1305779E+04,.1000456E+04,.8419893E+03,.8378345E+03,.1060921E+04,&
     & .1260705E+04,.9599420E+03,.7984917E+03,.7943099E+03,.1012757E+04,&
     & .1215666E+04,.9215564E+03,.7578604E+03,.7550366E+03,.9674587E+03,&
     & .1171119E+04,.8847872E+03,.7197368E+03,.7188825E+03,.9251420E+03/

      data   absb(876:1050,12)   /                                      &
     & .1369025E+04,.1061988E+04,.9090576E+03,.9070507E+03,.1133822E+04,&
     & .1324283E+04,.1017792E+04,.8607146E+03,.8569957E+03,.1081438E+04,&
     & .1279221E+04,.9763247E+03,.8159408E+03,.8115038E+03,.1032081E+04,&
     & .1234106E+04,.9370840E+03,.7741603E+03,.7706394E+03,.9855998E+03,&
     & .1189255E+04,.8996323E+03,.7349893E+03,.7332176E+03,.9419966E+03,&
     & .1387296E+04,.1080897E+04,.9299250E+03,.9288068E+03,.1156040E+04,&
     & .1342857E+04,.1035650E+04,.8800130E+03,.8768348E+03,.1102421E+04,&
     & .1297820E+04,.9930678E+03,.8338259E+03,.8293976E+03,.1051801E+04,&
     & .1252579E+04,.9528599E+03,.7908416E+03,.7867574E+03,.1004167E+04,&
     & .1207574E+04,.9146924E+03,.7505619E+03,.7480415E+03,.9593335E+03,&
     & .1404104E+04,.1098935E+04,.9500783E+03,.9498735E+03,.1177271E+04,&
     & .1360008E+04,.1052670E+04,.8986275E+03,.8960550E+03,.1122511E+04,&
     & .1315196E+04,.1009091E+04,.8510987E+03,.8469719E+03,.1070745E+04,&
     & .1269965E+04,.9680021E+03,.8068947E+03,.8024578E+03,.1021945E+04,&
     & .1224670E+04,.9290678E+03,.7655748E+03,.7624022E+03,.9760562E+03,&
     & .1420396E+04,.1116667E+04,.9703151E+03,.9709136E+03,.1198267E+04,&
     & .1376519E+04,.1069496E+04,.9171148E+03,.9153343E+03,.1142380E+04,&
     & .1331895E+04,.1024943E+04,.8682037E+03,.8645664E+03,.1089458E+04,&
     & .1286713E+04,.9828640E+03,.8228042E+03,.8182606E+03,.1039599E+04,&
     & .1241514E+04,.9432594E+03,.7805968E+03,.7767631E+03,.9926584E+03,&
     & .1436295E+04,.1134602E+04,.9911820E+03,.9925055E+03,.1219679E+04,&
     & .1392861E+04,.1086682E+04,.9361507E+03,.9351807E+03,.1162507E+04,&
     & .1348464E+04,.1041116E+04,.8857605E+03,.8826630E+03,.1108509E+04,&
     & .1303445E+04,.9981438E+03,.8391267E+03,.8346717E+03,.1057512E+04,&
     & .1258155E+04,.9576321E+03,.7957736E+03,.7914987E+03,.1009530E+04,&
     & .1451947E+04,.1152803E+04,.1012912E+04,.1014725E+04,.1241375E+04,&
     & .1409171E+04,.1104184E+04,.9558739E+03,.9558293E+03,.1183310E+04,&
     & .1365042E+04,.1057601E+04,.9039098E+03,.9014495E+03,.1128065E+04,&
     & .1320122E+04,.1013646E+04,.8559028E+03,.8518298E+03,.1075944E+04,&
     & .1274918E+04,.9723353E+03,.8114299E+03,.8068514E+03,.1026857E+04,&
     & .1466066E+04,.1169745E+04,.1033600E+04,.1035678E+04,.1261517E+04,&
     & .1423885E+04,.1120363E+04,.9744091E+03,.9750701E+03,.1202414E+04,&
     & .1380086E+04,.1073062E+04,.9208922E+03,.9191581E+03,.1146254E+04,&
     & .1335390E+04,.1028184E+04,.8716013E+03,.8680259E+03,.1093132E+04,&
     & .1290197E+04,.9859694E+03,.8260174E+03,.8214000E+03,.1043057E+04/

      data   absb(1051:1175,12)   /                                     &
     & .1479052E+04,.1186138E+04,.1053992E+04,.1056277E+04,.1281061E+04,&
     & .1437721E+04,.1136174E+04,.9928663E+03,.9941031E+03,.1221167E+04,&
     & .1394340E+04,.1088094E+04,.9376415E+03,.9367024E+03,.1164059E+04,&
     & .1349933E+04,.1042434E+04,.8871475E+03,.8840194E+03,.1109960E+04,&
     & .1304838E+04,.9993418E+03,.8403647E+03,.8358995E+03,.1058827E+04,&
     & .1492071E+04,.1202807E+04,.1075082E+04,.1077418E+04,.1300751E+04,&
     & .1451378E+04,.1152078E+04,.1011888E+04,.1013598E+04,.1240207E+04,&
     & .1408590E+04,.1103420E+04,.9548463E+03,.9546446E+03,.1182081E+04,&
     & .1364310E+04,.1056805E+04,.9029595E+03,.9004074E+03,.1127003E+04,&
     & .1319414E+04,.1012901E+04,.8550197E+03,.8508617E+03,.1074911E+04,&
     & .1504643E+04,.1219744E+04,.1096579E+04,.1098934E+04,.1320376E+04,&
     & .1464791E+04,.1168191E+04,.1031579E+04,.1033563E+04,.1259497E+04,&
     & .1422597E+04,.1118810E+04,.9725173E+03,.9730206E+03,.1200352E+04,&
     & .1378559E+04,.1071486E+04,.9191235E+03,.9172829E+03,.1144372E+04,&
     & .1333898E+04,.1026715E+04,.8699929E+03,.8663091E+03,.1091312E+04,&
     & .1516339E+04,.1236030E+04,.1117522E+04,.1119831E+04,.1339023E+04,&
     & .1477239E+04,.1183650E+04,.1050741E+04,.1052932E+04,.1277904E+04,&
     & .1435679E+04,.1133650E+04,.9897694E+03,.9909248E+03,.1218096E+04,&
     & .1392107E+04,.1085680E+04,.9348858E+03,.9337357E+03,.1161095E+04,&
     & .1347658E+04,.1040131E+04,.8845784E+03,.8813290E+03,.1107099E+04,&
     & .1520825E+04,.1242563E+04,.1126048E+04,.1128336E+04,.1346460E+04,&
     & .1482173E+04,.1189885E+04,.1058578E+04,.1060811E+04,.1285327E+04,&
     & .1440948E+04,.1139665E+04,.9968499E+03,.9981671E+03,.1225222E+04,&
     & .1397470E+04,.1091462E+04,.9413190E+03,.9405053E+03,.1167923E+04,&
     & .1353151E+04,.1045578E+04,.8905298E+03,.8874634E+03,.1113520E+04/

!  ---
      data   forref(1:4,1:12) /  .5532580E-03,.5554860E-03,.6013390E-03,&
     & .7082800E-03,.1585580E-02,.1629570E-02,.2049910E-02,.4758810E-02,&
     & .7725420E-02,.7845620E-02,.1119790E-01,.2290160E-01,.2550970E-01,&
     & .2562720E-01,.2706910E-01,.2595050E-01,.3232630E-01,.3244950E-01,&
     & .3055350E-01,.2639930E-01,.3557280E-01,.3564191E-01,.3329601E-01,&
     & .2956313E-01,.3784510E-01,.3753410E-01,.3743690E-01,.3203340E-01,&
     & .4098704E-01,.3991769E-01,.3953723E-01,.3197576E-01,.4282540E-01,&
     & .4411510E-01,.4088870E-01,.3270770E-01,.4529070E-01,.4526648E-01,&
     & .4032414E-01,.3589280E-01,.4839280E-01,.4772840E-01,.3806840E-01,&
     & .3879400E-01,.5074618E-01,.4940565E-01,.4711472E-01,.3831108E-01/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to NG17).

      data   selfref(1:10,1:12)  /                                      &
     & .1605370E-01,.1490380E-01,.1383630E-01,.1284520E-01,.1192510E-01,&
     & .1107090E-01,.1027790E-01,.9541750E-02,.8858290E-02,.8223790E-02,&
     & .3657530E-01,.3422670E-01,.3202880E-01,.2997200E-01,.2804740E-01,&
     & .2624630E-01,.2456090E-01,.2298370E-01,.2150780E-01,.2012670E-01,&
     & .1274190E+00,.1185530E+00,.1103040E+00,.1026290E+00,.9548830E-01,&
     & .8884420E-01,.8266240E-01,.7691070E-01,.7155930E-01,.6658020E-01,&
     & .3786870E+00,.3489610E+00,.3215680E+00,.2963250E+00,.2730640E+00,&
     & .2516290E+00,.2318760E+00,.2136740E+00,.1969010E+00,.1814440E+00,&
     & .4728220E+00,.4350180E+00,.4002360E+00,.3682360E+00,.3387940E+00,&
     & .3117060E+00,.2867830E+00,.2638540E+00,.2427570E+00,.2233480E+00,&
     & .5168017E+00,.4753661E+00,.4372531E+00,.4021962E+00,.3699493E+00,&
     & .3402876E+00,.3130049E+00,.2879097E+00,.2648259E+00,.2435933E+00,&
     & .5402220E+00,.4977460E+00,.4586100E+00,.4225510E+00,.3893270E+00,&
     & .3587160E+00,.3305110E+00,.3045240E+00,.2805800E+00,.2585190E+00,&
     & .5700865E+00,.5267717E+00,.4867481E+00,.4497659E+00,.4155940E+00,&
     & .3840184E+00,.3548416E+00,.3278825E+00,.3029720E+00,.2799538E+00,&
     & .6451760E+00,.5889570E+00,.5376360E+00,.4907880E+00,.4480220E+00,&
     & .4089820E+00,.3733440E+00,.3408120E+00,.3111140E+00,.2840040E+00,&
     & .6556933E+00,.6013742E+00,.5515585E+00,.5058718E+00,.4639719E+00,&
     & .4255448E+00,.3903026E+00,.3579808E+00,.3283371E+00,.3011502E+00,&
     & .6925540E+00,.6355740E+00,.5832820E+00,.5352930E+00,.4912510E+00,&
     & .4508340E+00,.4137410E+00,.3797010E+00,.3484610E+00,.3197910E+00,&
     & .7229700E+00,.6637789E+00,.6094446E+00,.5595658E+00,.5137786E+00,&
     & .4717450E+00,.4331576E+00,.3977321E+00,.3652099E+00,.3353523E+00/

!........................................!
      end module module_radsw_kgb17      !
!========================================!


!========================================!
      module module_radsw_kgb18          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 18:  4000-4650 cm-1 (low - h2o,ch4; high - ch4)            !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG18

!
      implicit none
!
      private
!
      integer,  public :: MSA18, MSB18, MSF18, MFR18
      parameter (MSA18=585, MSB18=235, MSF18=10, MFR18=3)

      real (kind=kind_phys), public ::           selfref(MSF18,NG18),   &
     &       absa(MSA18,NG18), absb(MSB18,NG18), forref(MFR18,NG18)

!  ---  rayleigh extinction coefficient at v = 4325 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 1.39e-09

!     the array absa(585,NG18) (ka(9,5,13,NG18)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds
!     to different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, Jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 8, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .1481300E-04,.3984200E-04,.4336200E-04,.4309500E-04,.3981100E-04,&
     & .3499400E-04,.2898400E-04,.2060900E-04,.1884500E-05,.1443200E-04,&
     & .3874900E-04,.4211900E-04,.4183400E-04,.3890000E-04,.3436500E-04,&
     & .2857100E-04,.2050300E-04,.1471500E-05,.1396900E-04,.3766600E-04,&
     & .4094200E-04,.4070300E-04,.3798200E-04,.3373100E-04,.2815800E-04,&
     & .2035100E-04,.1084200E-05,.1336700E-04,.3660500E-04,.3972900E-04,&
     & .3952900E-04,.3703300E-04,.3309600E-04,.2769000E-04,.2015400E-04,&
     & .7870200E-06,.1276500E-04,.3558400E-04,.3853600E-04,.3836100E-04,&
     & .3607300E-04,.3238400E-04,.2724200E-04,.1993200E-04,.5829600E-06,&
     & .1229900E-04,.3417900E-04,.3740500E-04,.3753500E-04,.3471400E-04,&
     & .3033500E-04,.2489200E-04,.1752400E-04,.1330100E-05,.1206400E-04,&
     & .3322500E-04,.3635700E-04,.3651200E-04,.3387000E-04,.2978900E-04,&
     & .2457800E-04,.1743500E-04,.1021700E-05,.1169700E-04,.3230100E-04,&
     & .3531600E-04,.3548100E-04,.3304900E-04,.2922600E-04,.2423300E-04,&
     & .1730400E-04,.7215200E-06,.1120900E-04,.3139500E-04,.3426300E-04,&
     & .3443600E-04,.3217300E-04,.2861000E-04,.2384700E-04,.1716100E-04,&
     & .5178700E-06,.1070600E-04,.3049600E-04,.3331100E-04,.3339600E-04,&
     & .3129500E-04,.2800200E-04,.2344500E-04,.1697100E-04,.4023000E-06,&
     & .1027100E-04,.2927300E-04,.3204900E-04,.3236500E-04,.2985700E-04,&
     & .2617100E-04,.2135200E-04,.1481400E-04,.8047000E-06,.1005800E-04,&
     & .2845800E-04,.3119600E-04,.3144000E-04,.2913300E-04,.2564000E-04,&
     & .2105600E-04,.1475600E-04,.5301000E-06,.9817200E-05,.2765100E-04,&
     & .3035200E-04,.3060900E-04,.2839600E-04,.2511000E-04,.2073000E-04,&
     & .1465500E-04,.3865200E-06,.9457700E-05,.2687400E-04,.2947500E-04,&
     & .2974900E-04,.2766900E-04,.2455300E-04,.2040000E-04,.1453600E-04,&
     & .3323600E-06,.9050800E-05,.2612800E-04,.2863100E-04,.2883400E-04,&
     & .2695200E-04,.2401500E-04,.2004500E-04,.1438200E-04,.3065900E-06,&
     & .8608100E-05,.2489700E-04,.2731700E-04,.2763200E-04,.2546500E-04,&
     & .2232800E-04,.1811200E-04,.1246800E-04,.3928600E-06,.8379300E-05,&
     & .2427300E-04,.2659000E-04,.2686500E-04,.2487600E-04,.2186700E-04,&
     & .1784900E-04,.1242000E-04,.3226600E-06,.8229600E-05,.2358400E-04,&
     & .2586900E-04,.2615500E-04,.2425500E-04,.2140300E-04,.1757900E-04,&
     & .1233800E-04,.3077500E-06,.7973100E-05,.2291500E-04,.2516600E-04,&
     & .2543500E-04,.2362700E-04,.2091700E-04,.1728200E-04,.1224200E-04,&
     & .3041800E-06,.7655300E-05,.2225900E-04,.2444000E-04,.2467300E-04,&
     & .2300300E-04,.2043700E-04,.1698400E-04,.1211200E-04,.3063800E-06/

      data   absa(181:315, 1)   /                                       &
     & .7225900E-05,.2108900E-04,.2317700E-04,.2341900E-04,.2158400E-04,&
     & .1892600E-04,.1525100E-04,.1041700E-04,.2517000E-06,.7021500E-05,&
     & .2056700E-04,.2254100E-04,.2278800E-04,.2110200E-04,.1852600E-04,&
     & .1504900E-04,.1037900E-04,.2595800E-06,.6849000E-05,.2001800E-04,&
     & .2193800E-04,.2220600E-04,.2057900E-04,.1813900E-04,.1480600E-04,&
     & .1031300E-04,.2666600E-06,.6697300E-05,.1944400E-04,.2134000E-04,&
     & .2161500E-04,.2007000E-04,.1772800E-04,.1455400E-04,.1023900E-04,&
     & .2858900E-06,.6480800E-05,.1888300E-04,.2073800E-04,.2099400E-04,&
     & .1954600E-04,.1730400E-04,.1430600E-04,.1014800E-04,.3088600E-06,&
     & .6072700E-05,.1779400E-04,.1960900E-04,.1976900E-04,.1822800E-04,&
     & .1594600E-04,.1279300E-04,.8655900E-05,.1855900E-06,.5894900E-05,&
     & .1739000E-04,.1905900E-04,.1924400E-04,.1782000E-04,.1564700E-04,&
     & .1262600E-04,.8635400E-05,.1954400E-06,.5726800E-05,.1693500E-04,&
     & .1856400E-04,.1872400E-04,.1740200E-04,.1530400E-04,.1243300E-04,&
     & .8588200E-05,.2203300E-06,.5624300E-05,.1645900E-04,.1805400E-04,&
     & .1826300E-04,.1698400E-04,.1497100E-04,.1221500E-04,.8526800E-05,&
     & .2507200E-06,.5458400E-05,.1598400E-04,.1754800E-04,.1777300E-04,&
     & .1655100E-04,.1460100E-04,.1200400E-04,.8456800E-05,.2811000E-06,&
     & .5062400E-05,.1496300E-04,.1648200E-04,.1661100E-04,.1533900E-04,&
     & .1333300E-04,.1065600E-04,.7157000E-05,.1345300E-06,.4942800E-05,&
     & .1463600E-04,.1604000E-04,.1618100E-04,.1499200E-04,.1314400E-04,&
     & .1053300E-04,.7153200E-05,.1562000E-06,.4795500E-05,.1425600E-04,&
     & .1561100E-04,.1573500E-04,.1464300E-04,.1286500E-04,.1039000E-04,&
     & .7124800E-05,.1857100E-06,.4674300E-05,.1387600E-04,.1521300E-04,&
     & .1534200E-04,.1429300E-04,.1257600E-04,.1021500E-04,.7073500E-05,&
     & .2138200E-06,.4570800E-05,.1347300E-04,.1478200E-04,.1494700E-04,&
     & .1393300E-04,.1227400E-04,.1003600E-04,.7017700E-05,.2441900E-06/

      data   absa(316:450, 1)   /                                       &
     & .4228900E-05,.1252800E-04,.1381700E-04,.1392200E-04,.1286700E-04,&
     & .1112600E-04,.8846800E-05,.5901000E-05,.1060000E-06,.4131900E-05,&
     & .1228600E-04,.1348000E-04,.1358200E-04,.1258400E-04,.1097300E-04,&
     & .8764400E-05,.5904900E-05,.1295100E-06,.4031700E-05,.1199600E-04,&
     & .1311600E-04,.1320600E-04,.1229500E-04,.1078600E-04,.8653700E-05,&
     & .5889800E-05,.1573100E-06,.3910700E-05,.1166600E-04,.1277100E-04,&
     & .1287000E-04,.1201000E-04,.1054600E-04,.8524400E-05,.5853600E-05,&
     & .1871600E-06,.3838400E-05,.1134600E-04,.1243500E-04,.1254900E-04,&
     & .1170600E-04,.1028900E-04,.8375600E-05,.5810400E-05,.2167900E-06,&
     & .3524000E-05,.1041800E-04,.1155800E-04,.1168700E-04,.1074800E-04,&
     & .9230500E-05,.7319800E-05,.4851400E-05,.1017600E-06,.3447800E-05,&
     & .1029600E-04,.1129600E-04,.1136900E-04,.1053200E-04,.9137500E-05,&
     & .7273700E-05,.4863900E-05,.1269500E-06,.3371800E-05,.1006800E-04,&
     & .1099600E-04,.1106300E-04,.1029000E-04,.8995700E-05,.7184400E-05,&
     & .4858300E-05,.1548200E-06,.3280100E-05,.9799000E-05,.1071200E-04,&
     & .1076200E-04,.1005600E-04,.8825500E-05,.7090700E-05,.4836400E-05,&
     & .1812300E-06,.3196000E-05,.9535000E-05,.1042700E-04,.1050100E-04,&
     & .9805600E-05,.8615700E-05,.6972300E-05,.4801000E-05,.2073800E-06,&
     & .2919700E-05,.8618300E-05,.9610000E-05,.9744800E-05,.8918700E-05,&
     & .7635400E-05,.6031800E-05,.3973300E-05,.1031000E-06,.2867700E-05,&
     & .8556500E-05,.9419100E-05,.9468200E-05,.8765300E-05,.7572500E-05,&
     & .6008900E-05,.3995600E-05,.1295100E-06,.2803600E-05,.8412100E-05,&
     & .9190200E-05,.9225100E-05,.8581100E-05,.7465300E-05,.5945200E-05,&
     & .3996700E-05,.1568400E-06,.2742000E-05,.8203500E-05,.8944000E-05,&
     & .8974800E-05,.8381700E-05,.7342200E-05,.5872500E-05,.3984400E-05,&
     & .1856400E-06,.2659600E-05,.7973900E-05,.8707400E-05,.8750800E-05,&
     & .8178400E-05,.7178100E-05,.5781500E-05,.3956900E-05,.2147600E-06/

      data   absa(451:585, 1)   /                                       &
     & .2398800E-05,.7113500E-05,.7912100E-05,.8011800E-05,.7337000E-05,&
     & .6284400E-05,.4961000E-05,.3264100E-05,.9169700E-07,.2355200E-05,&
     & .7058700E-05,.7751900E-05,.7782100E-05,.7211300E-05,.6229700E-05,&
     & .4940700E-05,.3281500E-05,.1130700E-06,.2304600E-05,.6936700E-05,&
     & .7566200E-05,.7579000E-05,.7063100E-05,.6140900E-05,.4886700E-05,&
     & .3281200E-05,.1371500E-06,.2255700E-05,.6763200E-05,.7364000E-05,&
     & .7377200E-05,.6897200E-05,.6038100E-05,.4825400E-05,.3270500E-05,&
     & .1643600E-06,.2189900E-05,.6574800E-05,.7169500E-05,.7194600E-05,&
     & .6729500E-05,.5902200E-05,.4748800E-05,.3247600E-05,.1902500E-06,&
     & .1969900E-05,.5867000E-05,.6507500E-05,.6580800E-05,.6030000E-05,&
     & .5166500E-05,.4074900E-05,.2678800E-05,.7710000E-07,.1934800E-05,&
     & .5814600E-05,.6373000E-05,.6389100E-05,.5926000E-05,.5119600E-05,&
     & .4057700E-05,.2692600E-05,.9569300E-07,.1894700E-05,.5713400E-05,&
     & .6220700E-05,.6223800E-05,.5809000E-05,.5045900E-05,.4011300E-05,&
     & .2691600E-05,.1157200E-06,.1855200E-05,.5570900E-05,.6052200E-05,&
     & .6061100E-05,.5669300E-05,.4959400E-05,.3960600E-05,.2682400E-05,&
     & .1379700E-06,.1804000E-05,.5413700E-05,.5893500E-05,.5909600E-05,&
     & .5529400E-05,.4844300E-05,.3897700E-05,.2663300E-05,.1588400E-06,&
     & .1616700E-05,.4827200E-05,.5347500E-05,.5398300E-05,.4950300E-05,&
     & .4241300E-05,.3343100E-05,.2195600E-05,.6138100E-07,.1589500E-05,&
     & .4780900E-05,.5234200E-05,.5240900E-05,.4865200E-05,.4202100E-05,&
     & .3326300E-05,.2207200E-05,.7638200E-07,.1558200E-05,.4698200E-05,&
     & .5105500E-05,.5105800E-05,.4771600E-05,.4140600E-05,.3288600E-05,&
     & .2205800E-05,.9308300E-07,.1524200E-05,.4583100E-05,.4969600E-05,&
     & .4973500E-05,.4653400E-05,.4067800E-05,.3246900E-05,.2196900E-05,&
     & .1121100E-06,.1485900E-05,.4449200E-05,.4839200E-05,.4847900E-05,&
     & .4537800E-05,.3968900E-05,.3193300E-05,.2181300E-05,.1297000E-06/

      data   absa(  1:180, 2)   /                                       &
     & .1013800E-03,.2045600E-03,.2211700E-03,.2194800E-03,.2080500E-03,&
     & .1837300E-03,.1499900E-03,.9538200E-04,.3321300E-05,.1014100E-03,&
     & .2088900E-03,.2245600E-03,.2222300E-03,.2091200E-03,.1846700E-03,&
     & .1506000E-03,.9571300E-04,.2847900E-05,.1012500E-03,.2122300E-03,&
     & .2271200E-03,.2238800E-03,.2098300E-03,.1851300E-03,.1510000E-03,&
     & .9588100E-04,.2633800E-05,.1010500E-03,.2144200E-03,.2286300E-03,&
     & .2242100E-03,.2104600E-03,.1854000E-03,.1506000E-03,.9579800E-04,&
     & .2763800E-05,.1004000E-03,.2161200E-03,.2289900E-03,.2241600E-03,&
     & .2104600E-03,.1857000E-03,.1500100E-03,.9556900E-04,.2931300E-05,&
     & .8963900E-04,.1779700E-03,.1928400E-03,.1898600E-03,.1782400E-03,&
     & .1578900E-03,.1290600E-03,.8352400E-04,.2392600E-05,.8949900E-04,&
     & .1814400E-03,.1956300E-03,.1922700E-03,.1794100E-03,.1584400E-03,&
     & .1296100E-03,.8363200E-04,.2196900E-05,.8941100E-04,.1840100E-03,&
     & .1975300E-03,.1935000E-03,.1799900E-03,.1588300E-03,.1298900E-03,&
     & .8357700E-04,.2315800E-05,.8884100E-04,.1854100E-03,.1987200E-03,&
     & .1939800E-03,.1805300E-03,.1590700E-03,.1297200E-03,.8339600E-04,&
     & .2376000E-05,.8794900E-04,.1865900E-03,.1990200E-03,.1940700E-03,&
     & .1807300E-03,.1591000E-03,.1293700E-03,.8312000E-04,.2524700E-05,&
     & .7746600E-04,.1526400E-03,.1670400E-03,.1634200E-03,.1530000E-03,&
     & .1349900E-03,.1100700E-03,.7188100E-04,.1783900E-05,.7748300E-04,&
     & .1561100E-03,.1695000E-03,.1655100E-03,.1540800E-03,.1358000E-03,&
     & .1105500E-03,.7202000E-04,.1856400E-05,.7714200E-04,.1586700E-03,&
     & .1711200E-03,.1667200E-03,.1548100E-03,.1361600E-03,.1107700E-03,&
     & .7192200E-04,.2022800E-05,.7646000E-04,.1605800E-03,.1718600E-03,&
     & .1671700E-03,.1551900E-03,.1364100E-03,.1106000E-03,.7177500E-04,&
     & .2171700E-05,.7551000E-04,.1615800E-03,.1721000E-03,.1673000E-03,&
     & .1552100E-03,.1364900E-03,.1102200E-03,.7151600E-04,.2314000E-05,&
     & .6593100E-04,.1294000E-03,.1429500E-03,.1400500E-03,.1309500E-03,&
     & .1149400E-03,.9365600E-04,.6116700E-04,.1709400E-05,.6609000E-04,&
     & .1326900E-03,.1454200E-03,.1423300E-03,.1320100E-03,.1155800E-03,&
     & .9397500E-04,.6143600E-04,.1852700E-05,.6563000E-04,.1353200E-03,&
     & .1471700E-03,.1436400E-03,.1326600E-03,.1160400E-03,.9412600E-04,&
     & .6146700E-04,.1957600E-05,.6497100E-04,.1370800E-03,.1481800E-03,&
     & .1442700E-03,.1330400E-03,.1161600E-03,.9415200E-04,.6133400E-04,&
     & .2061400E-05,.6412200E-04,.1383400E-03,.1486400E-03,.1445100E-03,&
     & .1331200E-03,.1163800E-03,.9395500E-04,.6116200E-04,.2164000E-05/

      data   absa(181:315, 2)   /                                       &
     & .5559400E-04,.1083000E-03,.1206800E-03,.1186500E-03,.1112700E-03,&
     & .9792500E-04,.7920800E-04,.5184200E-04,.1595400E-05,.5566900E-04,&
     & .1111700E-03,.1229900E-03,.1209100E-03,.1124800E-03,.9864300E-04,&
     & .7943300E-04,.5212700E-04,.1696600E-05,.5543000E-04,.1134500E-03,&
     & .1247800E-03,.1223500E-03,.1133200E-03,.9911700E-04,.7982300E-04,&
     & .5228600E-04,.1830400E-05,.5480000E-04,.1154300E-03,.1260100E-03,&
     & .1230400E-03,.1138900E-03,.9934600E-04,.7993500E-04,.5227500E-04,&
     & .1947900E-05,.5402700E-04,.1166500E-03,.1266000E-03,.1233900E-03,&
     & .1141000E-03,.9953500E-04,.7984000E-04,.5216800E-04,.2058300E-05,&
     & .4650000E-04,.8988700E-04,.1004800E-03,.9916100E-04,.9313600E-04,&
     & .8227900E-04,.6687200E-04,.4361700E-04,.1473000E-05,.4663200E-04,&
     & .9244400E-04,.1027100E-03,.1012300E-03,.9455900E-04,.8313500E-04,&
     & .6720400E-04,.4404600E-04,.1621500E-05,.4649000E-04,.9441500E-04,&
     & .1044600E-03,.1026900E-03,.9538700E-04,.8367600E-04,.6757900E-04,&
     & .4427900E-04,.1735800E-05,.4600400E-04,.9619000E-04,.1057000E-03,&
     & .1034600E-03,.9588300E-04,.8403800E-04,.6775300E-04,.4434800E-04,&
     & .1809600E-05,.4536200E-04,.9751100E-04,.1064500E-03,.1038400E-03,&
     & .9619700E-04,.8434700E-04,.6775800E-04,.4430500E-04,.1888200E-05,&
     & .3859700E-04,.7401900E-04,.8288200E-04,.8180700E-04,.7693600E-04,&
     & .6827800E-04,.5574400E-04,.3650700E-04,.1340000E-05,.3872800E-04,&
     & .7635300E-04,.8503000E-04,.8382200E-04,.7852300E-04,.6922100E-04,&
     & .5619900E-04,.3693200E-04,.1476600E-05,.3868800E-04,.7816400E-04,&
     & .8669100E-04,.8526900E-04,.7942400E-04,.6984900E-04,.5650700E-04,&
     & .3721100E-04,.1607700E-05,.3839900E-04,.7965400E-04,.8788200E-04,&
     & .8614500E-04,.7991700E-04,.7021400E-04,.5669800E-04,.3730200E-04,&
     & .1736500E-05,.3789900E-04,.8101200E-04,.8870600E-04,.8658900E-04,&
     & .8025900E-04,.7042900E-04,.5681000E-04,.3728700E-04,.1859500E-05/

      data   absa(316:450, 2)   /                                       &
     & .3195900E-04,.6067100E-04,.6796300E-04,.6705300E-04,.6305700E-04,&
     & .5628700E-04,.4618700E-04,.3029800E-04,.1207100E-05,.3206200E-04,&
     & .6275300E-04,.6989200E-04,.6894100E-04,.6462400E-04,.5718700E-04,&
     & .4656600E-04,.3068700E-04,.1358200E-05,.3209800E-04,.6443500E-04,&
     & .7142300E-04,.7035900E-04,.6564600E-04,.5788200E-04,.4689700E-04,&
     & .3099300E-04,.1521700E-05,.3195600E-04,.6575800E-04,.7265000E-04,&
     & .7127500E-04,.6627500E-04,.5829800E-04,.4710000E-04,.3107900E-04,&
     & .1679400E-05,.3158400E-04,.6691600E-04,.7350200E-04,.7181900E-04,&
     & .6661100E-04,.5850500E-04,.4724300E-04,.3108800E-04,.1825000E-05,&
     & .2635500E-04,.4947200E-04,.5541500E-04,.5458400E-04,.5136100E-04,&
     & .4613500E-04,.3802700E-04,.2498300E-04,.1354900E-05,.2647200E-04,&
     & .5130200E-04,.5716500E-04,.5641700E-04,.5293600E-04,.4704800E-04,&
     & .3841700E-04,.2534700E-04,.1494100E-05,.2654500E-04,.5286600E-04,&
     & .5856300E-04,.5777600E-04,.5398800E-04,.4770600E-04,.3873900E-04,&
     & .2563800E-04,.1656300E-05,.2646900E-04,.5410500E-04,.5974900E-04,&
     & .5874600E-04,.5466800E-04,.4816100E-04,.3897000E-04,.2574400E-04,&
     & .1826200E-05,.2624800E-04,.5507900E-04,.6061900E-04,.5931700E-04,&
     & .5509900E-04,.4843500E-04,.3911500E-04,.2577800E-04,.2023800E-05,&
     & .2163100E-04,.4029700E-04,.4517000E-04,.4429800E-04,.4187600E-04,&
     & .3769000E-04,.3121200E-04,.2055200E-04,.1632800E-05,.2181400E-04,&
     & .4189600E-04,.4668100E-04,.4606500E-04,.4322700E-04,.3860800E-04,&
     & .3161400E-04,.2085700E-04,.1861000E-05,.2186400E-04,.4324700E-04,&
     & .4794500E-04,.4732700E-04,.4427300E-04,.3920100E-04,.3189400E-04,&
     & .2111900E-04,.2077500E-05,.2183500E-04,.4438600E-04,.4898600E-04,&
     & .4822800E-04,.4491600E-04,.3963700E-04,.3209300E-04,.2122800E-04,&
     & .2254100E-05,.2172800E-04,.4526300E-04,.4980400E-04,.4881800E-04,&
     & .4540900E-04,.3995000E-04,.3226900E-04,.2129000E-04,.2452500E-05/

      data   absa(451:585, 2)   /                                       &
     & .1782300E-04,.3337800E-04,.3733700E-04,.3661900E-04,.3459800E-04,&
     & .3111300E-04,.2576100E-04,.1695600E-04,.1504200E-05,.1797000E-04,&
     & .3467900E-04,.3858900E-04,.3806500E-04,.3569500E-04,.3186900E-04,&
     & .2604400E-04,.1721400E-04,.1684700E-05,.1799700E-04,.3579100E-04,&
     & .3961900E-04,.3909700E-04,.3653900E-04,.3237400E-04,.2629800E-04,&
     & .1742800E-04,.1888700E-05,.1796700E-04,.3671700E-04,.4046300E-04,&
     & .3982100E-04,.3707100E-04,.3271600E-04,.2646100E-04,.1749600E-04,&
     & .2054600E-05,.1786800E-04,.3742300E-04,.4112400E-04,.4030400E-04,&
     & .3747300E-04,.3297100E-04,.2661200E-04,.1753200E-04,.2224000E-05,&
     & .1466500E-04,.2760500E-04,.3081100E-04,.3025000E-04,.2852500E-04,&
     & .2564400E-04,.2118600E-04,.1395500E-04,.1258800E-05,.1477500E-04,&
     & .2865500E-04,.3185100E-04,.3140800E-04,.2941800E-04,.2626100E-04,&
     & .2143000E-04,.1417200E-04,.1418100E-05,.1479900E-04,.2957100E-04,&
     & .3268700E-04,.3224200E-04,.3009300E-04,.2668300E-04,.2163600E-04,&
     & .1433800E-04,.1561900E-05,.1477700E-04,.3032500E-04,.3337100E-04,&
     & .3282600E-04,.3055300E-04,.2694500E-04,.2177500E-04,.1438000E-04,&
     & .1691700E-05,.1468500E-04,.3090400E-04,.3390700E-04,.3321400E-04,&
     & .3087500E-04,.2715800E-04,.2188300E-04,.1441400E-04,.1842300E-05,&
     & .1204400E-04,.2277200E-04,.2537500E-04,.2492600E-04,.2345800E-04,&
     & .2105900E-04,.1738700E-04,.1145700E-04,.9912200E-06,.1212800E-04,&
     & .2363700E-04,.2624900E-04,.2584200E-04,.2418700E-04,.2158200E-04,&
     & .1759000E-04,.1163600E-04,.1103700E-05,.1215200E-04,.2438600E-04,&
     & .2691000E-04,.2653100E-04,.2473100E-04,.2192300E-04,.1776000E-04,&
     & .1175500E-04,.1202600E-05,.1213000E-04,.2499600E-04,.2746400E-04,&
     & .2700600E-04,.2511300E-04,.2213800E-04,.1787400E-04,.1179700E-04,&
     & .1302800E-05,.1205000E-04,.2547400E-04,.2789500E-04,.2729700E-04,&
     & .2537300E-04,.2230200E-04,.1794800E-04,.1181700E-04,.1401000E-05/

      data   absa(  1:180, 3)   /                                       &
     & .3682300E-03,.4961300E-03,.5181700E-03,.5053800E-03,.4571100E-03,&
     & .3960500E-03,.3148600E-03,.2147400E-03,.1093200E-04,.3700800E-03,&
     & .5077800E-03,.5336100E-03,.5177800E-03,.4699500E-03,.4062900E-03,&
     & .3230900E-03,.2187800E-03,.1142100E-04,.3698900E-03,.5181200E-03,&
     & .5469900E-03,.5292200E-03,.4803600E-03,.4155900E-03,.3296500E-03,&
     & .2222600E-03,.1239500E-04,.3672100E-03,.5276300E-03,.5574600E-03,&
     & .5395700E-03,.4889100E-03,.4231300E-03,.3354500E-03,.2258800E-03,&
     & .1351500E-04,.3631300E-03,.5365200E-03,.5668000E-03,.5479100E-03,&
     & .4966400E-03,.4290700E-03,.3404300E-03,.2291600E-03,.1473400E-04,&
     & .3241100E-03,.4281500E-03,.4472500E-03,.4381300E-03,.4015000E-03,&
     & .3487100E-03,.2791200E-03,.1892900E-03,.8074100E-05,.3256700E-03,&
     & .4378600E-03,.4606900E-03,.4502700E-03,.4131900E-03,.3581900E-03,&
     & .2863500E-03,.1932500E-03,.8734000E-05,.3242900E-03,.4463800E-03,&
     & .4722800E-03,.4607400E-03,.4227200E-03,.3660200E-03,.2920100E-03,&
     & .1965300E-03,.9611000E-05,.3213700E-03,.4553100E-03,.4814200E-03,&
     & .4695100E-03,.4299600E-03,.3725000E-03,.2970900E-03,.1997400E-03,&
     & .1069700E-04,.3177600E-03,.4620700E-03,.4888000E-03,.4765900E-03,&
     & .4355100E-03,.3777600E-03,.3013800E-03,.2027400E-03,.1170400E-04,&
     & .2848800E-03,.3676700E-03,.3808100E-03,.3743800E-03,.3452500E-03,&
     & .3015000E-03,.2433200E-03,.1641500E-03,.6397000E-05,.2853900E-03,&
     & .3750900E-03,.3919600E-03,.3851000E-03,.3556600E-03,.3099800E-03,&
     & .2496300E-03,.1676600E-03,.7200600E-05,.2839700E-03,.3821500E-03,&
     & .4015600E-03,.3942800E-03,.3641800E-03,.3172700E-03,.2544600E-03,&
     & .1710700E-03,.7957400E-05,.2813400E-03,.3886700E-03,.4095900E-03,&
     & .4023000E-03,.3709300E-03,.3230100E-03,.2589800E-03,.1744700E-03,&
     & .8713100E-05,.2788600E-03,.3936000E-03,.4164200E-03,.4085200E-03,&
     & .3760900E-03,.3274000E-03,.2631200E-03,.1773700E-03,.9470400E-05,&
     & .2492300E-03,.3155500E-03,.3242300E-03,.3173100E-03,.2941000E-03,&
     & .2567100E-03,.2076200E-03,.1407400E-03,.5587700E-05,.2497700E-03,&
     & .3212000E-03,.3337200E-03,.3265700E-03,.3029500E-03,.2647200E-03,&
     & .2130300E-03,.1439600E-03,.6157500E-05,.2486500E-03,.3266800E-03,&
     & .3418000E-03,.3344600E-03,.3103300E-03,.2714400E-03,.2177700E-03,&
     & .1471400E-03,.6791600E-05,.2470900E-03,.3313300E-03,.3484100E-03,&
     & .3413400E-03,.3160000E-03,.2764700E-03,.2217900E-03,.1501600E-03,&
     & .7465300E-05,.2445500E-03,.3349100E-03,.3538800E-03,.3470800E-03,&
     & .3206900E-03,.2800500E-03,.2252200E-03,.1524200E-03,.8161900E-05/

      data   absa(181:315, 3)   /                                       &
     & .2152600E-03,.2711500E-03,.2750500E-03,.2680200E-03,.2496800E-03,&
     & .2169000E-03,.1748100E-03,.1190600E-03,.5085000E-05,.2161100E-03,&
     & .2762100E-03,.2829800E-03,.2762000E-03,.2572200E-03,.2241100E-03,&
     & .1800400E-03,.1219700E-03,.5573800E-05,.2158300E-03,.2806000E-03,&
     & .2895500E-03,.2828200E-03,.2634600E-03,.2300500E-03,.1841500E-03,&
     & .1245100E-03,.6086400E-05,.2144600E-03,.2836300E-03,.2946900E-03,&
     & .2889600E-03,.2686000E-03,.2343500E-03,.1876800E-03,.1269000E-03,&
     & .6641700E-05,.2121700E-03,.2862600E-03,.2992900E-03,.2937000E-03,&
     & .2725000E-03,.2376300E-03,.1907300E-03,.1290900E-03,.7212200E-05,&
     & .1826200E-03,.2311200E-03,.2338800E-03,.2268100E-03,.2106200E-03,&
     & .1819600E-03,.1463400E-03,.9970700E-04,.4807800E-05,.1839300E-03,&
     & .2359100E-03,.2409400E-03,.2336400E-03,.2166300E-03,.1881400E-03,&
     & .1511400E-03,.1020500E-03,.5225200E-05,.1837900E-03,.2398400E-03,&
     & .2467600E-03,.2395600E-03,.2222100E-03,.1935600E-03,.1548900E-03,&
     & .1043400E-03,.5640500E-05,.1827000E-03,.2425800E-03,.2510200E-03,&
     & .2445800E-03,.2268500E-03,.1977100E-03,.1580500E-03,.1065500E-03,&
     & .6072800E-05,.1806700E-03,.2444600E-03,.2546700E-03,.2489400E-03,&
     & .2305100E-03,.2005600E-03,.1608800E-03,.1084400E-03,.6516000E-05,&
     & .1526900E-03,.1940700E-03,.1964000E-03,.1907300E-03,.1771500E-03,&
     & .1522400E-03,.1218100E-03,.8330200E-04,.4518500E-05,.1545000E-03,&
     & .1983400E-03,.2024600E-03,.1967500E-03,.1824200E-03,.1577600E-03,&
     & .1261000E-03,.8530600E-04,.4931500E-05,.1545900E-03,.2017800E-03,&
     & .2075900E-03,.2017600E-03,.1873700E-03,.1623300E-03,.1295100E-03,&
     & .8741700E-04,.5349100E-05,.1536800E-03,.2041000E-03,.2116000E-03,&
     & .2062300E-03,.1916100E-03,.1660300E-03,.1324600E-03,.8923700E-04,&
     & .5756200E-05,.1520700E-03,.2056700E-03,.2145800E-03,.2099100E-03,&
     & .1952500E-03,.1690600E-03,.1348100E-03,.9093200E-04,.6165900E-05/

      data   absa(316:450, 3)   /                                       &
     & .1267200E-03,.1615900E-03,.1632900E-03,.1582100E-03,.1474600E-03,&
     & .1262400E-03,.1011600E-03,.6930600E-04,.4665200E-05,.1285300E-03,&
     & .1653900E-03,.1685900E-03,.1635900E-03,.1520800E-03,.1312000E-03,&
     & .1051300E-03,.7100500E-04,.5139200E-05,.1290100E-03,.1684100E-03,&
     & .1730100E-03,.1681300E-03,.1561400E-03,.1353200E-03,.1082900E-03,&
     & .7286300E-04,.5524400E-05,.1283300E-03,.1705800E-03,.1765200E-03,&
     & .1718400E-03,.1597100E-03,.1386700E-03,.1108700E-03,.7447500E-04,&
     & .5819300E-05,.1271500E-03,.1719700E-03,.1791800E-03,.1751300E-03,&
     & .1629200E-03,.1414400E-03,.1130100E-03,.7596500E-04,.6170500E-05,&
     & .1042800E-03,.1334800E-03,.1346200E-03,.1299500E-03,.1215800E-03,&
     & .1037900E-03,.8300500E-04,.5737100E-04,.5596500E-05,.1061900E-03,&
     & .1371000E-03,.1393200E-03,.1346900E-03,.1253800E-03,.1080000E-03,&
     & .8663200E-04,.5895400E-04,.6196100E-05,.1069000E-03,.1397800E-03,&
     & .1431500E-03,.1387700E-03,.1288500E-03,.1116900E-03,.8964300E-04,&
     & .6056800E-04,.6852600E-05,.1066800E-03,.1417800E-03,.1462600E-03,&
     & .1421300E-03,.1320700E-03,.1146800E-03,.9198400E-04,.6207000E-04,&
     & .7512200E-05,.1058200E-03,.1431600E-03,.1487300E-03,.1450100E-03,&
     & .1347100E-03,.1170400E-03,.9387600E-04,.6338800E-04,.8076100E-05,&
     & .8547600E-04,.1098200E-03,.1103400E-03,.1064300E-03,.9970200E-04,&
     & .8496700E-04,.6800800E-04,.4722900E-04,.6614500E-05,.8724200E-04,&
     & .1132200E-03,.1146300E-03,.1104500E-03,.1029300E-03,.8858000E-04,&
     & .7111300E-04,.4863100E-04,.7545200E-05,.8811100E-04,.1156300E-03,&
     & .1179200E-03,.1140400E-03,.1058700E-03,.9187700E-04,.7379000E-04,&
     & .5004400E-04,.8488100E-05,.8820000E-04,.1174300E-03,.1206500E-03,&
     & .1170800E-03,.1087000E-03,.9448000E-04,.7589700E-04,.5138400E-04,&
     & .9513600E-05,.8758300E-04,.1187100E-03,.1229200E-03,.1195400E-03,&
     & .1110300E-03,.9652300E-04,.7752600E-04,.5251800E-04,.1053200E-04/

      data   absa(451:585, 3)   /                                       &
     & .7055600E-04,.9142800E-04,.9186700E-04,.8846300E-04,.8284600E-04,&
     & .7077400E-04,.5663200E-04,.3916700E-04,.6187200E-05,.7193300E-04,&
     & .9418200E-04,.9526900E-04,.9164600E-04,.8538300E-04,.7373800E-04,&
     & .5921900E-04,.4047400E-04,.6996500E-05,.7256700E-04,.9609800E-04,&
     & .9792700E-04,.9460600E-04,.8783700E-04,.7637000E-04,.6132600E-04,&
     & .4163300E-04,.7823500E-05,.7252900E-04,.9753200E-04,.1001100E-03,&
     & .9707600E-04,.9013400E-04,.7842800E-04,.6301600E-04,.4275400E-04,&
     & .8790400E-05,.7199000E-04,.9855200E-04,.1019300E-03,.9908000E-04,&
     & .9209100E-04,.8006800E-04,.6434900E-04,.4367400E-04,.9801100E-05,&
     & .5807200E-04,.7581500E-04,.7616800E-04,.7325200E-04,.6852100E-04,&
     & .5874800E-04,.4702900E-04,.3243000E-04,.5183800E-05,.5914900E-04,&
     & .7800500E-04,.7888900E-04,.7585300E-04,.7069100E-04,.6116000E-04,&
     & .4910500E-04,.3357200E-04,.5866900E-05,.5961500E-04,.7957200E-04,&
     & .8104300E-04,.7830100E-04,.7271600E-04,.6326500E-04,.5077700E-04,&
     & .3450600E-04,.6646400E-05,.5950600E-04,.8071200E-04,.8281100E-04,&
     & .8032100E-04,.7457100E-04,.6493500E-04,.5216500E-04,.3544700E-04,&
     & .7441200E-05,.5906700E-04,.8152000E-04,.8428800E-04,.8197700E-04,&
     & .7617200E-04,.6625900E-04,.5328100E-04,.3617800E-04,.8246500E-05,&
     & .4764000E-04,.6258300E-04,.6288700E-04,.6048500E-04,.5655900E-04,&
     & .4862400E-04,.3890400E-04,.2677300E-04,.4058200E-05,.4849800E-04,&
     & .6437900E-04,.6508700E-04,.6264100E-04,.5837700E-04,.5057200E-04,&
     & .4056600E-04,.2771700E-04,.4661600E-05,.4884400E-04,.6565600E-04,&
     & .6685100E-04,.6462800E-04,.6004200E-04,.5225400E-04,.4191700E-04,&
     & .2851000E-04,.5320400E-05,.4872200E-04,.6657400E-04,.6830300E-04,&
     & .6630000E-04,.6154700E-04,.5361400E-04,.4305800E-04,.2925400E-04,&
     & .5944200E-05,.4835500E-04,.6721700E-04,.6951100E-04,.6765100E-04,&
     & .6286400E-04,.5471200E-04,.4399800E-04,.2987500E-04,.6643600E-05/

      data   absa(  1:180, 4)   /                                       &
     & .8538700E-03,.9726800E-03,.9520100E-03,.8922600E-03,.8146500E-03,&
     & .6970700E-03,.5482900E-03,.3588200E-03,.3792000E-04,.8549600E-03,&
     & .9874600E-03,.9718800E-03,.9177500E-03,.8352900E-03,.7164900E-03,&
     & .5637600E-03,.3706100E-03,.4315500E-04,.8515400E-03,.9990800E-03,&
     & .9896900E-03,.9401400E-03,.8560400E-03,.7337900E-03,.5783200E-03,&
     & .3818300E-03,.4870200E-04,.8460600E-03,.1009000E-02,.1006800E-02,&
     & .9594400E-03,.8743400E-03,.7497500E-03,.5924300E-03,.3918800E-03,&
     & .5402200E-04,.8406100E-03,.1017100E-02,.1021800E-02,.9768400E-03,&
     & .8890600E-03,.7634500E-03,.6047100E-03,.4018000E-03,.5977800E-04,&
     & .7610600E-03,.8676300E-03,.8462000E-03,.7933700E-03,.7236800E-03,&
     & .6263300E-03,.4916800E-03,.3234600E-03,.3132200E-04,.7615300E-03,&
     & .8814800E-03,.8634900E-03,.8155900E-03,.7448000E-03,.6444800E-03,&
     & .5063500E-03,.3343000E-03,.3556600E-04,.7597700E-03,.8919500E-03,&
     & .8787100E-03,.8360500E-03,.7628600E-03,.6612700E-03,.5195900E-03,&
     & .3447200E-03,.3970500E-04,.7577100E-03,.8998700E-03,.8940600E-03,&
     & .8553000E-03,.7812300E-03,.6761200E-03,.5322100E-03,.3544700E-03,&
     & .4426600E-04,.7528900E-03,.9065700E-03,.9077100E-03,.8720000E-03,&
     & .7971500E-03,.6892600E-03,.5438200E-03,.3637200E-03,.4870800E-04,&
     & .6689300E-03,.7624600E-03,.7433400E-03,.6975200E-03,.6350100E-03,&
     & .5536200E-03,.4345500E-03,.2884300E-03,.2426000E-04,.6711800E-03,&
     & .7758800E-03,.7594400E-03,.7171000E-03,.6546100E-03,.5712200E-03,&
     & .4499100E-03,.2990600E-03,.2743400E-04,.6721800E-03,.7850500E-03,&
     & .7732800E-03,.7349700E-03,.6728600E-03,.5871800E-03,.4637500E-03,&
     & .3093300E-03,.3080800E-04,.6705800E-03,.7924200E-03,.7862700E-03,&
     & .7518100E-03,.6894600E-03,.6018300E-03,.4753200E-03,.3174300E-03,&
     & .3432400E-04,.6646700E-03,.7982200E-03,.7977600E-03,.7677900E-03,&
     & .7048100E-03,.6151000E-03,.4855300E-03,.3252600E-03,.3790500E-04,&
     & .5829700E-03,.6602000E-03,.6433700E-03,.6061000E-03,.5522200E-03,&
     & .4833800E-03,.3815400E-03,.2553700E-03,.2005400E-04,.5861600E-03,&
     & .6731100E-03,.6576400E-03,.6236200E-03,.5699600E-03,.4993900E-03,&
     & .3965700E-03,.2641800E-03,.2245700E-04,.5871900E-03,.6829100E-03,&
     & .6701400E-03,.6392600E-03,.5870300E-03,.5145300E-03,.4097600E-03,&
     & .2728000E-03,.2497400E-04,.5845100E-03,.6892500E-03,.6814400E-03,&
     & .6541000E-03,.6031200E-03,.5290300E-03,.4214700E-03,.2809600E-03,&
     & .2761300E-04,.5788500E-03,.6936900E-03,.6915300E-03,.6672500E-03,&
     & .6166600E-03,.5411400E-03,.4312300E-03,.2890700E-03,.3024300E-04/

      data   absa(181:315, 4)   /                                       &
     & .5063900E-03,.5689700E-03,.5541200E-03,.5212700E-03,.4734100E-03,&
     & .4166900E-03,.3329300E-03,.2221500E-03,.1705700E-04,.5106500E-03,&
     & .5811100E-03,.5674000E-03,.5364200E-03,.4897000E-03,.4316600E-03,&
     & .3467100E-03,.2311100E-03,.1919100E-04,.5113100E-03,.5892300E-03,&
     & .5780200E-03,.5498900E-03,.5051900E-03,.4452000E-03,.3586700E-03,&
     & .2395500E-03,.2123800E-04,.5088800E-03,.5943200E-03,.5870000E-03,&
     & .5623800E-03,.5189900E-03,.4578700E-03,.3696000E-03,.2473300E-03,&
     & .2344200E-04,.5041100E-03,.5980400E-03,.5949700E-03,.5734200E-03,&
     & .5315800E-03,.4692400E-03,.3788600E-03,.2545000E-03,.2575600E-04,&
     & .4367300E-03,.4878200E-03,.4749900E-03,.4462900E-03,.4035400E-03,&
     & .3545700E-03,.2860500E-03,.1908600E-03,.1470200E-04,.4410200E-03,&
     & .4983100E-03,.4863900E-03,.4594300E-03,.4187900E-03,.3680800E-03,&
     & .2980200E-03,.1998500E-03,.1647100E-04,.4422100E-03,.5051800E-03,&
     & .4953800E-03,.4707500E-03,.4318300E-03,.3799500E-03,.3088200E-03,&
     & .2076300E-03,.1842000E-04,.4401700E-03,.5099400E-03,.5032300E-03,&
     & .4814100E-03,.4436000E-03,.3911600E-03,.3184400E-03,.2146000E-03,&
     & .2047300E-04,.4365600E-03,.5124900E-03,.5100000E-03,.4906800E-03,&
     & .4542500E-03,.4010700E-03,.3265300E-03,.2210900E-03,.2260000E-04,&
     & .3762700E-03,.4152500E-03,.4036500E-03,.3792300E-03,.3425700E-03,&
     & .3014700E-03,.2432400E-03,.1609400E-03,.1340000E-04,.3805100E-03,&
     & .4248900E-03,.4140000E-03,.3906700E-03,.3555500E-03,.3125700E-03,&
     & .2535300E-03,.1694700E-03,.1490500E-04,.3819600E-03,.4315400E-03,&
     & .4216900E-03,.4003500E-03,.3670300E-03,.3231100E-03,.2629100E-03,&
     & .1767200E-03,.1656600E-04,.3810800E-03,.4358700E-03,.4280900E-03,&
     & .4089000E-03,.3769400E-03,.3324600E-03,.2709400E-03,.1830500E-03,&
     & .1835200E-04,.3785400E-03,.4388900E-03,.4340200E-03,.4169700E-03,&
     & .3853200E-03,.3407400E-03,.2781600E-03,.1887300E-03,.2022000E-04/

      data   absa(316:450, 4)   /                                       &
     & .3196800E-03,.3530100E-03,.3431000E-03,.3220600E-03,.2890400E-03,&
     & .2535600E-03,.2052300E-03,.1343200E-03,.1354800E-04,.3238600E-03,&
     & .3622900E-03,.3528800E-03,.3321400E-03,.3003300E-03,.2634500E-03,&
     & .2137400E-03,.1419800E-03,.1496600E-04,.3252100E-03,.3684800E-03,&
     & .3598600E-03,.3405600E-03,.3104200E-03,.2722200E-03,.2211500E-03,&
     & .1484100E-03,.1627800E-04,.3252700E-03,.3725800E-03,.3655300E-03,&
     & .3481100E-03,.3191600E-03,.2803400E-03,.2283700E-03,.1544300E-03,&
     & .1761400E-04,.3227200E-03,.3753300E-03,.3702500E-03,.3543800E-03,&
     & .3263200E-03,.2874000E-03,.2348400E-03,.1595200E-03,.1907500E-04,&
     & .2679400E-03,.2965500E-03,.2885100E-03,.2715500E-03,.2429000E-03,&
     & .2118200E-03,.1718700E-03,.1115800E-03,.1689300E-04,.2721500E-03,&
     & .3047700E-03,.2976500E-03,.2807500E-03,.2533600E-03,.2212000E-03,&
     & .1785800E-03,.1179400E-03,.1857600E-04,.2740000E-03,.3106100E-03,&
     & .3044700E-03,.2883600E-03,.2622400E-03,.2288600E-03,.1847400E-03,&
     & .1237200E-03,.2005600E-04,.2738500E-03,.3142400E-03,.3092300E-03,&
     & .2947700E-03,.2697700E-03,.2355400E-03,.1909300E-03,.1288400E-03,&
     & .2132100E-04,.2719600E-03,.3164700E-03,.3130800E-03,.3000600E-03,&
     & .2760400E-03,.2419100E-03,.1964900E-03,.1335200E-03,.2261100E-04,&
     & .2225500E-03,.2471600E-03,.2408700E-03,.2267200E-03,.2021600E-03,&
     & .1758200E-03,.1433900E-03,.9252800E-04,.2247100E-04,.2266300E-03,&
     & .2542900E-03,.2490800E-03,.2350600E-03,.2116700E-03,.1837700E-03,&
     & .1491000E-03,.9797200E-04,.2453600E-04,.2286700E-03,.2596300E-03,&
     & .2553200E-03,.2416900E-03,.2192300E-03,.1906700E-03,.1545800E-03,&
     & .1029300E-03,.2645300E-04,.2285300E-03,.2627300E-03,.2596000E-03,&
     & .2470400E-03,.2254900E-03,.1967900E-03,.1596800E-03,.1072800E-03,&
     & .2838300E-04,.2270400E-03,.2646900E-03,.2625900E-03,.2514100E-03,&
     & .2308000E-03,.2021600E-03,.1643300E-03,.1111600E-03,.3011100E-04/

      data   absa(451:585, 4)   /                                       &
     & .1857000E-03,.2079100E-03,.2031000E-03,.1910600E-03,.1706600E-03,&
     & .1479700E-03,.1209700E-03,.7870400E-04,.2072000E-04,.1886800E-03,&
     & .2135600E-03,.2096900E-03,.1980700E-03,.1782100E-03,.1543400E-03,&
     & .1258400E-03,.8317900E-04,.2326800E-04,.1897200E-03,.2173200E-03,&
     & .2144700E-03,.2030500E-03,.1841600E-03,.1599300E-03,.1300600E-03,&
     & .8713400E-04,.2578100E-04,.1892900E-03,.2195500E-03,.2177100E-03,&
     & .2071100E-03,.1890700E-03,.1649300E-03,.1342000E-03,.9069900E-04,&
     & .2818100E-04,.1876600E-03,.2209500E-03,.2198700E-03,.2104500E-03,&
     & .1931200E-03,.1693400E-03,.1381200E-03,.9384000E-04,.3048900E-04,&
     & .1540900E-03,.1739900E-03,.1701000E-03,.1600200E-03,.1431100E-03,&
     & .1237600E-03,.1012100E-03,.6642800E-04,.1781300E-04,.1559900E-03,&
     & .1783100E-03,.1752100E-03,.1655400E-03,.1489000E-03,.1288100E-03,&
     & .1052700E-03,.6996400E-04,.1991800E-04,.1566100E-03,.1811900E-03,&
     & .1789200E-03,.1693400E-03,.1536500E-03,.1333800E-03,.1086400E-03,&
     & .7314200E-04,.2212100E-04,.1560800E-03,.1829300E-03,.1814500E-03,&
     & .1725400E-03,.1574300E-03,.1374700E-03,.1120900E-03,.7597100E-04,&
     & .2442500E-04,.1545300E-03,.1838300E-03,.1830900E-03,.1751000E-03,&
     & .1606700E-03,.1410200E-03,.1152900E-03,.7860700E-04,.2679100E-04,&
     & .1270500E-03,.1448500E-03,.1415500E-03,.1331200E-03,.1191400E-03,&
     & .1029900E-03,.8441600E-04,.5559800E-04,.1442900E-04,.1284300E-03,&
     & .1482200E-03,.1455500E-03,.1374800E-03,.1237700E-03,.1070200E-03,&
     & .8752000E-04,.5840500E-04,.1614700E-04,.1287500E-03,.1504300E-03,&
     & .1485000E-03,.1405300E-03,.1275200E-03,.1108100E-03,.9034800E-04,&
     & .6093400E-04,.1799200E-04,.1282200E-03,.1517500E-03,.1504900E-03,&
     & .1430100E-03,.1305900E-03,.1141300E-03,.9317100E-04,.6329600E-04,&
     & .1988800E-04,.1268400E-03,.1523900E-03,.1517900E-03,.1451100E-03,&
     & .1332000E-03,.1170200E-03,.9578600E-04,.6546800E-04,.2170600E-04/

      data   absa(  1:180, 5)   /                                       &
     & .2275875E-02,.2332064E-02,.2216796E-02,.2037339E-02,.1810754E-02,&
     & .1542017E-02,.1221835E-02,.8225168E-03,.2378785E-03,.2274525E-02,&
     & .2355192E-02,.2253198E-02,.2079119E-02,.1858235E-02,.1590203E-02,&
     & .1266759E-02,.8576253E-03,.2590612E-03,.2266023E-02,.2375024E-02,&
     & .2283817E-02,.2117878E-02,.1902253E-02,.1637433E-02,.1310064E-02,&
     & .8925666E-03,.2839408E-03,.2251924E-02,.2389473E-02,.2311430E-02,&
     & .2154739E-02,.1945283E-02,.1682775E-02,.1352510E-02,.9268463E-03,&
     & .3108163E-03,.2232455E-02,.2399259E-02,.2336395E-02,.2189553E-02,&
     & .1986811E-02,.1727976E-02,.1393776E-02,.9592790E-03,.3382077E-03,&
     & .2106993E-02,.2154787E-02,.2043104E-02,.1868719E-02,.1654669E-02,&
     & .1404007E-02,.1111065E-02,.7507219E-03,.1949377E-03,.2109908E-02,&
     & .2178927E-02,.2078863E-02,.1908781E-02,.1697578E-02,.1448872E-02,&
     & .1153419E-02,.7832801E-03,.2139563E-03,.2105465E-02,.2199441E-02,&
     & .2109463E-02,.1944753E-02,.1739524E-02,.1492018E-02,.1195120E-02,&
     & .8155632E-03,.2349725E-03,.2093796E-02,.2214377E-02,.2136026E-02,&
     & .1977275E-02,.1778542E-02,.1533694E-02,.1236220E-02,.8467658E-03,&
     & .2569866E-03,.2076724E-02,.2225712E-02,.2159173E-02,.2008267E-02,&
     & .1816462E-02,.1574183E-02,.1275209E-02,.8771670E-03,.2791186E-03,&
     & .1940537E-02,.1975605E-02,.1864311E-02,.1695911E-02,.1496137E-02,&
     & .1263628E-02,.9971610E-03,.6738674E-03,.1565544E-03,.1948245E-02,&
     & .2001986E-02,.1900553E-02,.1736295E-02,.1537745E-02,.1305077E-02,&
     & .1036131E-02,.7035141E-03,.1728753E-03,.1946880E-02,.2024379E-02,&
     & .1930764E-02,.1771154E-02,.1575770E-02,.1344673E-02,.1074374E-02,&
     & .7332422E-03,.1899479E-03,.1938868E-02,.2041140E-02,.1955874E-02,&
     & .1802287E-02,.1611440E-02,.1382812E-02,.1112406E-02,.7633322E-03,&
     & .2072614E-03,.1926362E-02,.2054363E-02,.1977970E-02,.1829838E-02,&
     & .1645759E-02,.1419736E-02,.1149822E-02,.7922080E-03,.2267149E-03,&
     & .1777321E-02,.1802735E-02,.1693881E-02,.1532687E-02,.1345353E-02,&
     & .1130261E-02,.8895753E-03,.5968647E-03,.1247261E-03,.1788584E-02,&
     & .1831387E-02,.1729859E-02,.1572174E-02,.1384900E-02,.1169670E-02,&
     & .9252263E-03,.6260595E-03,.1385262E-03,.1791618E-02,.1854218E-02,&
     & .1759929E-02,.1606995E-02,.1420262E-02,.1206226E-02,.9598075E-03,&
     & .6546109E-03,.1526122E-03,.1788339E-02,.1872013E-02,.1786084E-02,&
     & .1636442E-02,.1452167E-02,.1240664E-02,.9939639E-03,.6826539E-03,&
     & .1672628E-03,.1778773E-02,.1885271E-02,.1807813E-02,.1662029E-02,&
     & .1483280E-02,.1274547E-02,.1027899E-02,.7097707E-03,.1836638E-03/

      data   absa(181:315, 5)   /                                       &
     & .1611039E-02,.1628826E-02,.1528886E-02,.1381805E-02,.1208375E-02,&
     & .1011477E-02,.7896015E-03,.5279159E-03,.9908775E-04,.1625251E-02,&
     & .1659059E-02,.1564750E-02,.1419820E-02,.1247291E-02,.1047725E-02,&
     & .8222728E-03,.5541873E-03,.1104539E-03,.1631934E-02,.1683911E-02,&
     & .1595157E-02,.1453710E-02,.1281311E-02,.1080902E-02,.8540548E-03,&
     & .5806197E-03,.1219897E-03,.1631403E-02,.1703448E-02,.1621164E-02,&
     & .1482847E-02,.1310904E-02,.1111385E-02,.8847073E-03,.6065309E-03,&
     & .1346152E-03,.1624079E-02,.1717008E-02,.1643088E-02,.1507761E-02,&
     & .1337964E-02,.1141306E-02,.9151343E-03,.6325088E-03,.1481383E-03,&
     & .1446415E-02,.1455765E-02,.1361590E-02,.1229351E-02,.1074912E-02,&
     & .9006186E-03,.7006813E-03,.4647234E-03,.8189242E-04,.1463779E-02,&
     & .1487367E-02,.1398211E-02,.1267666E-02,.1112416E-02,.9358187E-03,&
     & .7313361E-03,.4886301E-03,.9135341E-04,.1472351E-02,.1513138E-02,&
     & .1428682E-02,.1300953E-02,.1145840E-02,.9675370E-03,.7596020E-03,&
     & .5122126E-03,.1005443E-03,.1474221E-02,.1532600E-02,.1454339E-02,&
     & .1329459E-02,.1174906E-02,.9956373E-03,.7867852E-03,.5359551E-03,&
     & .1109812E-03,.1468517E-02,.1546507E-02,.1474790E-02,.1352688E-02,&
     & .1200227E-02,.1022681E-02,.8131587E-03,.5591171E-03,.1219361E-03,&
     & .1282142E-02,.1287927E-02,.1202141E-02,.1083733E-02,.9460514E-03,&
     & .7915341E-03,.6168183E-03,.4084185E-03,.6913010E-04,.1301052E-02,&
     & .1319819E-02,.1238509E-02,.1121340E-02,.9823064E-03,.8254094E-03,&
     & .6455635E-03,.4300976E-03,.7651588E-04,.1311903E-02,.1345238E-02,&
     & .1268573E-02,.1153718E-02,.1014080E-02,.8553954E-03,.6719282E-03,&
     & .4514177E-03,.8484352E-04,.1314609E-02,.1365010E-02,.1292722E-02,&
     & .1180653E-02,.1041774E-02,.8822963E-03,.6970864E-03,.4722480E-03,&
     & .9377363E-04,.1310264E-02,.1377648E-02,.1311780E-02,.1202781E-02,&
     & .1065878E-02,.9070933E-03,.7215140E-03,.4928747E-03,.1032168E-03/

      data   absa(316:450, 5)   /                                       &
     & .1123722E-02,.1125509E-02,.1049077E-02,.9463051E-03,.8264962E-03,&
     & .6910472E-03,.5381327E-03,.3566196E-03,.6074714E-04,.1144528E-02,&
     & .1157108E-02,.1084145E-02,.9824251E-03,.8612451E-03,.7227531E-03,&
     & .5650534E-03,.3769954E-03,.6752101E-04,.1156115E-02,.1182481E-02,&
     & .1113388E-02,.1013135E-02,.8909498E-03,.7510056E-03,.5899576E-03,&
     & .3965626E-03,.7509877E-04,.1159636E-02,.1200610E-02,.1136628E-02,&
     & .1038419E-02,.9163190E-03,.7757266E-03,.6129711E-03,.4156689E-03,&
     & .8319483E-04,.1156675E-02,.1212668E-02,.1153943E-02,.1059223E-02,&
     & .9387272E-03,.7983980E-03,.6348116E-03,.4345877E-03,.9149048E-04,&
     & .9788217E-03,.9762902E-03,.9076191E-03,.8180805E-03,.7140719E-03,&
     & .5979311E-03,.4657057E-03,.3080365E-03,.6376560E-04,.9995801E-03,&
     & .1006767E-02,.9409303E-03,.8518710E-03,.7465719E-03,.6272074E-03,&
     & .4915396E-03,.3271476E-03,.7043059E-04,.1011283E-02,.1030289E-02,&
     & .9679162E-03,.8806454E-03,.7743638E-03,.6533475E-03,.5145414E-03,&
     & .3451157E-03,.7748348E-04,.1015523E-02,.1047024E-02,.9891231E-03,&
     & .9038676E-03,.7980900E-03,.6765444E-03,.5353367E-03,.3623073E-03,&
     & .8485843E-04,.1014685E-02,.1058164E-02,.1005055E-02,.9228604E-03,&
     & .8183767E-03,.6971991E-03,.5549716E-03,.3793243E-03,.9265390E-04,&
     & .8467257E-03,.8439047E-03,.7839737E-03,.7069600E-03,.6168442E-03,&
     & .5161363E-03,.4002993E-03,.2648075E-03,.8538662E-04,.8663801E-03,&
     & .8726556E-03,.8143622E-03,.7377253E-03,.6460394E-03,.5429897E-03,&
     & .4242444E-03,.2822342E-03,.9232499E-04,.8777160E-03,.8939799E-03,&
     & .8388502E-03,.7633777E-03,.6710202E-03,.5661949E-03,.4453570E-03,&
     & .2983921E-03,.9968327E-04,.8822976E-03,.9090305E-03,.8576617E-03,&
     & .7836215E-03,.6918973E-03,.5865524E-03,.4643056E-03,.3139878E-03,&
     & .1075001E-03,.8816821E-03,.9194329E-03,.8721358E-03,.8002626E-03,&
     & .7096839E-03,.6047828E-03,.4818629E-03,.3291128E-03,.1153966E-03/

      data   absa(451:585, 5)   /                                       &
     & .7345259E-03,.7353571E-03,.6844124E-03,.6191407E-03,.5419865E-03,&
     & .4543535E-03,.3524790E-03,.2326534E-03,.8831490E-04,.7490617E-03,&
     & .7579739E-03,.7093615E-03,.6447658E-03,.5665622E-03,.4774427E-03,&
     & .3729073E-03,.2477509E-03,.9439904E-04,.7564292E-03,.7744541E-03,&
     & .7287630E-03,.6652977E-03,.5871248E-03,.4970065E-03,.3911667E-03,&
     & .2619945E-03,.9990772E-04,.7586881E-03,.7858289E-03,.7437324E-03,&
     & .6818515E-03,.6043581E-03,.5141276E-03,.4072910E-03,.2755984E-03,&
     & .1066252E-03,.7563639E-03,.7931595E-03,.7548375E-03,.6950726E-03,&
     & .6192222E-03,.5294685E-03,.4222590E-03,.2889243E-03,.1133924E-03,&
     & .6314226E-03,.6349627E-03,.5926562E-03,.5376114E-03,.4719251E-03,&
     & .3962308E-03,.3074879E-03,.2032569E-03,.8003266E-04,.6418725E-03,&
     & .6525898E-03,.6125061E-03,.5580757E-03,.4921735E-03,.4155327E-03,&
     & .3248794E-03,.2162533E-03,.8576530E-04,.6471266E-03,.6653221E-03,&
     & .6276452E-03,.5745768E-03,.5090609E-03,.4319430E-03,.3403545E-03,&
     & .2285539E-03,.9041149E-04,.6478227E-03,.6738338E-03,.6394020E-03,&
     & .5876220E-03,.5231812E-03,.4462724E-03,.3541410E-03,.2403090E-03,&
     & .9584348E-04,.6442291E-03,.6788200E-03,.6475947E-03,.5980228E-03,&
     & .5350971E-03,.4588656E-03,.3667841E-03,.2515799E-03,.1017699E-03,&
     & .5366413E-03,.5433012E-03,.5091779E-03,.4634801E-03,.4076173E-03,&
     & .3421937E-03,.2655508E-03,.1759732E-03,.6609799E-04,.5446297E-03,&
     & .5573704E-03,.5249762E-03,.4800512E-03,.4241853E-03,.3581272E-03,&
     & .2800981E-03,.1871731E-03,.7047861E-04,.5481742E-03,.5672989E-03,&
     & .5373039E-03,.4932643E-03,.4379422E-03,.3716071E-03,.2930461E-03,&
     & .1977626E-03,.7433299E-04,.5476171E-03,.5738401E-03,.5461812E-03,&
     & .5035685E-03,.4492907E-03,.3832995E-03,.3046021E-03,.2077508E-03,&
     & .7880238E-04,.5438946E-03,.5774436E-03,.5525251E-03,.5119901E-03,&
     & .4590263E-03,.3938170E-03,.3152705E-03,.2173888E-03,.8388184E-04/

      data   absa(  1:180, 6)   /                                       &
     & .7453129E-02,.7117405E-02,.6661341E-02,.6078589E-02,.5376308E-02,&
     & .4581666E-02,.3692876E-02,.2668674E-02,.1857553E-02,.7414650E-02,&
     & .7132841E-02,.6723850E-02,.6175876E-02,.5501579E-02,.4730655E-02,&
     & .3847363E-02,.2834084E-02,.2101417E-02,.7367697E-02,.7142450E-02,&
     & .6787572E-02,.6271236E-02,.5628812E-02,.4876081E-02,.4005906E-02,&
     & .3014246E-02,.2361444E-02,.7309381E-02,.7154158E-02,.6847198E-02,&
     & .6364336E-02,.5757194E-02,.5023915E-02,.4174265E-02,.3211257E-02,&
     & .2644919E-02,.7240917E-02,.7165375E-02,.6901545E-02,.6460272E-02,&
     & .5886974E-02,.5176789E-02,.4355003E-02,.3427381E-02,.2951988E-02,&
     & .7302417E-02,.6986029E-02,.6506078E-02,.5915241E-02,.5213714E-02,&
     & .4409454E-02,.3509969E-02,.2477433E-02,.1531293E-02,.7272446E-02,&
     & .7005207E-02,.6569159E-02,.6009028E-02,.5333024E-02,.4547186E-02,&
     & .3651478E-02,.2623452E-02,.1730635E-02,.7229818E-02,.7019119E-02,&
     & .6631639E-02,.6101575E-02,.5452906E-02,.4682444E-02,.3795174E-02,&
     & .2779605E-02,.1947925E-02,.7174295E-02,.7033174E-02,.6689925E-02,&
     & .6193261E-02,.5571003E-02,.4818428E-02,.3945240E-02,.2947728E-02,&
     & .2183570E-02,.7110805E-02,.7045732E-02,.6744602E-02,.6284811E-02,&
     & .5687916E-02,.4958393E-02,.4105561E-02,.3129372E-02,.2439844E-02,&
     & .7107732E-02,.6797516E-02,.6299565E-02,.5698070E-02,.4995391E-02,&
     & .4198481E-02,.3302300E-02,.2275195E-02,.1230993E-02,.7086152E-02,&
     & .6823070E-02,.6363343E-02,.5788270E-02,.5108424E-02,.4328271E-02,&
     & .3431427E-02,.2406277E-02,.1392637E-02,.7052294E-02,.6841431E-02,&
     & .6426576E-02,.5878051E-02,.5223227E-02,.4454560E-02,.3563303E-02,&
     & .2543558E-02,.1569089E-02,.7006893E-02,.6857640E-02,.6486440E-02,&
     & .5967434E-02,.5334883E-02,.4578334E-02,.3699649E-02,.2688020E-02,&
     & .1760440E-02,.6949333E-02,.6870773E-02,.6541587E-02,.6057335E-02,&
     & .5442169E-02,.4706411E-02,.3839426E-02,.2841732E-02,.1964963E-02,&
     & .6855130E-02,.6551944E-02,.6044314E-02,.5438518E-02,.4743060E-02,&
     & .3966419E-02,.3087108E-02,.2087789E-02,.9879948E-03,.6846892E-02,&
     & .6585398E-02,.6112347E-02,.5528377E-02,.4852610E-02,.4086708E-02,&
     & .3208051E-02,.2203856E-02,.1119742E-02,.6824941E-02,.6610372E-02,&
     & .6177682E-02,.5615844E-02,.4962743E-02,.4204184E-02,.3329715E-02,&
     & .2324960E-02,.1263158E-02,.6787743E-02,.6632257E-02,.6236180E-02,&
     & .5702032E-02,.5069502E-02,.4319861E-02,.3453836E-02,.2450583E-02,&
     & .1417217E-02,.6740085E-02,.6651560E-02,.6290312E-02,.5788524E-02,&
     & .5171266E-02,.4436812E-02,.3580405E-02,.2581552E-02,.1582148E-02/

      data   absa(181:315, 6)   /                                       &
     & .6558083E-02,.6261779E-02,.5751378E-02,.5145860E-02,.4467228E-02,&
     & .3714081E-02,.2872846E-02,.1912747E-02,.7974292E-03,.6563849E-02,&
     & .6305787E-02,.5824503E-02,.5238051E-02,.4571578E-02,.3828273E-02,&
     & .2985436E-02,.2017845E-02,.9044707E-03,.6552571E-02,.6339245E-02,&
     & .5892376E-02,.5324090E-02,.4676050E-02,.3939449E-02,.3097458E-02,&
     & .2125514E-02,.1021181E-02,.6528895E-02,.6367176E-02,.5952402E-02,&
     & .5408542E-02,.4777701E-02,.4049784E-02,.3211983E-02,.2236558E-02,&
     & .1145140E-02,.6494313E-02,.6390805E-02,.6006376E-02,.5490888E-02,&
     & .4876710E-02,.4160137E-02,.3328414E-02,.2349602E-02,.1279011E-02,&
     & .6217297E-02,.5928758E-02,.5426959E-02,.4833520E-02,.4174155E-02,&
     & .3449793E-02,.2650701E-02,.1741615E-02,.6370094E-03,.6238859E-02,&
     & .5985757E-02,.5505992E-02,.4926333E-02,.4277869E-02,.3557040E-02,&
     & .2755926E-02,.1837721E-02,.7241979E-03,.6246176E-02,.6029777E-02,&
     & .5577993E-02,.5013764E-02,.4377842E-02,.3662657E-02,.2861509E-02,&
     & .1935493E-02,.8193352E-03,.6236692E-02,.6066266E-02,.5641135E-02,&
     & .5097431E-02,.4474642E-02,.3767073E-02,.2967217E-02,.2035588E-02,&
     & .9198253E-03,.6214342E-02,.6096770E-02,.5698970E-02,.5179034E-02,&
     & .4569107E-02,.3869801E-02,.3075666E-02,.2136680E-02,.1028239E-02,&
     & .5843663E-02,.5561987E-02,.5073558E-02,.4502171E-02,.3871558E-02,&
     & .3184826E-02,.2432337E-02,.1579438E-02,.5066815E-03,.5883549E-02,&
     & .5632733E-02,.5159626E-02,.4598553E-02,.3975656E-02,.3289560E-02,&
     & .2531458E-02,.1667525E-02,.5778308E-03,.5906315E-02,.5689718E-02,&
     & .5238108E-02,.4688943E-02,.4074005E-02,.3390408E-02,.2630796E-02,&
     & .1755973E-02,.6537602E-03,.5911760E-02,.5735215E-02,.5308819E-02,&
     & .4775175E-02,.4168666E-02,.3489857E-02,.2728910E-02,.1847257E-02,&
     & .7347999E-03,.5905012E-02,.5775036E-02,.5371993E-02,.4856819E-02,&
     & .4260538E-02,.3587351E-02,.2827507E-02,.1938584E-02,.8225668E-03/

      data   absa(316:450, 6)   /                                       &
     & .5448799E-02,.5174237E-02,.4702978E-02,.4159096E-02,.3562636E-02,&
     & .2919042E-02,.2216707E-02,.1428474E-02,.4372080E-03,.5503786E-02,&
     & .5258826E-02,.4798801E-02,.4260222E-02,.3666742E-02,.3022053E-02,&
     & .2311790E-02,.1509380E-02,.4923566E-03,.5542089E-02,.5327495E-02,&
     & .4884619E-02,.4354537E-02,.3765908E-02,.3119877E-02,.2406348E-02,&
     & .1590203E-02,.5508055E-03,.5564630E-02,.5383808E-02,.4961019E-02,&
     & .4443926E-02,.3860688E-02,.3216549E-02,.2498792E-02,.1671379E-02,&
     & .6126355E-03,.5570361E-02,.5432506E-02,.5031105E-02,.4526985E-02,&
     & .3951398E-02,.3309750E-02,.2590269E-02,.1752856E-02,.6794564E-03,&
     & .5025183E-02,.4765836E-02,.4321116E-02,.3810537E-02,.3254592E-02,&
     & .2655950E-02,.2006115E-02,.1286407E-02,.4311014E-03,.5098265E-02,&
     & .4863995E-02,.4425250E-02,.3917786E-02,.3360874E-02,.2757245E-02,&
     & .2097126E-02,.1360829E-02,.4799419E-03,.5154179E-02,.4945399E-02,&
     & .4519161E-02,.4017193E-02,.3461670E-02,.2853712E-02,.2187196E-02,&
     & .1435268E-02,.5332265E-03,.5190969E-02,.5013790E-02,.4604355E-02,&
     & .4110643E-02,.3555937E-02,.2947656E-02,.2275382E-02,.1509403E-02,&
     & .5888012E-03,.5208138E-02,.5072165E-02,.4680561E-02,.4195990E-02,&
     & .3646519E-02,.3037397E-02,.2361829E-02,.1583627E-02,.6478322E-03,&
     & .4623391E-02,.4366790E-02,.3943485E-02,.3465227E-02,.2955259E-02,&
     & .2405643E-02,.1812616E-02,.1157161E-02,.4945874E-03,.4710125E-02,&
     & .4474615E-02,.4054953E-02,.3577026E-02,.3062985E-02,.2506113E-02,&
     & .1900613E-02,.1226503E-02,.5465245E-03,.4776716E-02,.4564405E-02,&
     & .4153594E-02,.3680995E-02,.3166029E-02,.2601953E-02,.1986110E-02,&
     & .1295516E-02,.6032750E-03,.4823514E-02,.4640328E-02,.4243823E-02,&
     & .3778649E-02,.3262225E-02,.2694009E-02,.2069881E-02,.1363711E-02,&
     & .6621313E-03,.4852467E-02,.4703395E-02,.4324910E-02,.3866669E-02,&
     & .3352109E-02,.2781681E-02,.2151024E-02,.1432036E-02,.7228720E-03/

      data   absa(451:585, 6)   /                                       &
     & .4268691E-02,.4040183E-02,.3646880E-02,.3203178E-02,.2726389E-02,&
     & .2213905E-02,.1666317E-02,.1066325E-02,.4983873E-03,.4358586E-02,&
     & .4148745E-02,.3757876E-02,.3313137E-02,.2831232E-02,.2309276E-02,&
     & .1749842E-02,.1130782E-02,.5479266E-03,.4428469E-02,.4241143E-02,&
     & .3857210E-02,.3414820E-02,.2929730E-02,.2400315E-02,.1831540E-02,&
     & .1194311E-02,.6008082E-03,.4479685E-02,.4318791E-02,.3947347E-02,&
     & .3507844E-02,.3019422E-02,.2487256E-02,.1910638E-02,.1257207E-02,&
     & .6538267E-03,.4513413E-02,.4384415E-02,.4026009E-02,.3591207E-02,&
     & .3103605E-02,.2569184E-02,.1986548E-02,.1320050E-02,.7081602E-03,&
     & .3915100E-02,.3711716E-02,.3349816E-02,.2944438E-02,.2506601E-02,&
     & .2037038E-02,.1533857E-02,.9781961E-03,.4622673E-03,.4005434E-02,&
     & .3819687E-02,.3461208E-02,.3053733E-02,.2609936E-02,.2129368E-02,&
     & .1612065E-02,.1038339E-02,.5051377E-03,.4076956E-02,.3913286E-02,&
     & .3561928E-02,.3154739E-02,.2705010E-02,.2215707E-02,.1687464E-02,&
     & .1097449E-02,.5512414E-03,.4130247E-02,.3993297E-02,.3650863E-02,&
     & .3245814E-02,.2792524E-02,.2297376E-02,.1759369E-02,.1155953E-02,&
     & .5980184E-03,.4167218E-02,.4060665E-02,.3729251E-02,.3327359E-02,&
     & .2873454E-02,.2374433E-02,.1829761E-02,.1214279E-02,.6444273E-03,&
     & .3565599E-02,.3385516E-02,.3057279E-02,.2688351E-02,.2290666E-02,&
     & .1864426E-02,.1405266E-02,.8966862E-03,.3986965E-03,.3656897E-02,&
     & .3493575E-02,.3167640E-02,.2795196E-02,.2390148E-02,.1952514E-02,&
     & .1480086E-02,.9522316E-03,.4352189E-03,.3729366E-02,.3587046E-02,&
     & .3266101E-02,.2893554E-02,.2481255E-02,.2035902E-02,.1551310E-02,&
     & .1006380E-02,.4737097E-03,.3784570E-02,.3667287E-02,.3354159E-02,&
     & .2981192E-02,.2566707E-02,.2114949E-02,.1619376E-02,.1060223E-02,&
     & .5128669E-03,.3823908E-02,.3735653E-02,.3431813E-02,.3061934E-02,&
     & .2645587E-02,.2189124E-02,.1685116E-02,.1114442E-02,.5524785E-03/

      data   absa(  1:180, 7)   /                                       &
     & .2337626E-01,.2175176E-01,.2063054E-01,.1953289E-01,.1869334E-01,&
     & .1836575E-01,.1866188E-01,.1986732E-01,.2252135E-01,.2315624E-01,&
     & .2164265E-01,.2067161E-01,.1978356E-01,.1921682E-01,.1922562E-01,&
     & .1992930E-01,.2155800E-01,.2450575E-01,.2293107E-01,.2154457E-01,&
     & .2074715E-01,.2011748E-01,.1985588E-01,.2024390E-01,.2137333E-01,&
     & .2342559E-01,.2668727E-01,.2270210E-01,.2145089E-01,.2088033E-01,&
     & .2054255E-01,.2062482E-01,.2143098E-01,.2299821E-01,.2549251E-01,&
     & .2907234E-01,.2247095E-01,.2137582E-01,.2108331E-01,.2107230E-01,&
     & .2155716E-01,.2281018E-01,.2484353E-01,.2780611E-01,.3172795E-01,&
     & .2458189E-01,.2285200E-01,.2156813E-01,.2024632E-01,.1900708E-01,&
     & .1819156E-01,.1796768E-01,.1857224E-01,.2076937E-01,.2435250E-01,&
     & .2275178E-01,.2160965E-01,.2048103E-01,.1947652E-01,.1898219E-01,&
     & .1913925E-01,.2015560E-01,.2268231E-01,.2412387E-01,.2265897E-01,&
     & .2168600E-01,.2079019E-01,.2005803E-01,.1992705E-01,.2048310E-01,&
     & .2192140E-01,.2477362E-01,.2389129E-01,.2256825E-01,.2181441E-01,&
     & .2118241E-01,.2077464E-01,.2103762E-01,.2201020E-01,.2389372E-01,&
     & .2708335E-01,.2365223E-01,.2248782E-01,.2200444E-01,.2167167E-01,&
     & .2165150E-01,.2233835E-01,.2375741E-01,.2611574E-01,.2966311E-01,&
     & .2579688E-01,.2397618E-01,.2249064E-01,.2092442E-01,.1929406E-01,&
     & .1790138E-01,.1706811E-01,.1698922E-01,.1852242E-01,.2557176E-01,&
     & .2388374E-01,.2253992E-01,.2114151E-01,.1969695E-01,.1860179E-01,&
     & .1812445E-01,.1843215E-01,.2030655E-01,.2534781E-01,.2379879E-01,&
     & .2261460E-01,.2141516E-01,.2020271E-01,.1944733E-01,.1934135E-01,&
     & .2004948E-01,.2227174E-01,.2511276E-01,.2371341E-01,.2273562E-01,&
     & .2176102E-01,.2083763E-01,.2045509E-01,.2073270E-01,.2187030E-01,&
     & .2445363E-01,.2487422E-01,.2363274E-01,.2290622E-01,.2219407E-01,&
     & .2162169E-01,.2162715E-01,.2232993E-01,.2392248E-01,.2688023E-01,&
     & .2698586E-01,.2506791E-01,.2338609E-01,.2157417E-01,.1958193E-01,&
     & .1763778E-01,.1616964E-01,.1538518E-01,.1621144E-01,.2676194E-01,&
     & .2498910E-01,.2344897E-01,.2177479E-01,.1993962E-01,.1825433E-01,&
     & .1710937E-01,.1668719E-01,.1786207E-01,.2654257E-01,.2491521E-01,&
     & .2352676E-01,.2201454E-01,.2038036E-01,.1899998E-01,.1819673E-01,&
     & .1814869E-01,.1968096E-01,.2631697E-01,.2483856E-01,.2364114E-01,&
     & .2232785E-01,.2093620E-01,.1989270E-01,.1944697E-01,.1980212E-01,&
     & .2170612E-01,.2608471E-01,.2475687E-01,.2379589E-01,.2271583E-01,&
     & .2161901E-01,.2093691E-01,.2088135E-01,.2166610E-01,.2395288E-01/

      data   absa(181:315, 7)   /                                       &
     & .2810778E-01,.2610373E-01,.2423479E-01,.2218757E-01,.1986177E-01,&
     & .1746306E-01,.1537001E-01,.1389630E-01,.1400744E-01,.2789745E-01,&
     & .2604100E-01,.2431619E-01,.2238153E-01,.2019516E-01,.1800500E-01,&
     & .1621009E-01,.1507015E-01,.1553290E-01,.2769341E-01,.2597967E-01,&
     & .2440064E-01,.2260000E-01,.2059359E-01,.1865773E-01,.1717719E-01,&
     & .1638502E-01,.1720845E-01,.2747581E-01,.2591687E-01,.2451639E-01,&
     & .2287475E-01,.2108787E-01,.1943977E-01,.1828677E-01,.1786928E-01,&
     & .1906661E-01,.2724525E-01,.2584524E-01,.2465979E-01,.2322623E-01,&
     & .2169174E-01,.2036043E-01,.1956477E-01,.1955161E-01,.2112946E-01,&
     & .2916121E-01,.2706131E-01,.2501249E-01,.2273351E-01,.2011979E-01,&
     & .1734949E-01,.1468202E-01,.1254397E-01,.1192697E-01,.2896641E-01,&
     & .2702014E-01,.2511575E-01,.2293374E-01,.2042776E-01,.1783904E-01,&
     & .1542709E-01,.1359860E-01,.1333064E-01,.2876765E-01,.2697723E-01,&
     & .2521010E-01,.2314222E-01,.2079486E-01,.1840512E-01,.1626966E-01,&
     & .1476028E-01,.1484637E-01,.2856294E-01,.2693274E-01,.2532642E-01,&
     & .2339329E-01,.2123982E-01,.1907661E-01,.1724154E-01,.1606921E-01,&
     & .1652405E-01,.2834370E-01,.2687419E-01,.2545751E-01,.2371200E-01,&
     & .2178226E-01,.1987947E-01,.1836893E-01,.1756873E-01,.1840173E-01,&
     & .3011114E-01,.2792230E-01,.2568609E-01,.2319257E-01,.2034293E-01,&
     & .1726900E-01,.1415286E-01,.1138485E-01,.1005692E-01,.2992968E-01,&
     & .2790487E-01,.2581306E-01,.2340902E-01,.2064353E-01,.1772380E-01,&
     & .1480077E-01,.1231510E-01,.1132697E-01,.2974561E-01,.2787980E-01,&
     & .2592483E-01,.2361776E-01,.2099074E-01,.1823014E-01,.1552253E-01,&
     & .1333086E-01,.1268422E-01,.2955745E-01,.2785343E-01,.2604522E-01,&
     & .2385432E-01,.2139366E-01,.1881776E-01,.1636294E-01,.1447960E-01,&
     & .1418800E-01,.2935104E-01,.2781242E-01,.2617843E-01,.2414962E-01,&
     & .2187815E-01,.1952002E-01,.1734737E-01,.1580301E-01,.1587593E-01/

      data   absa(316:450, 7)   /                                       &
     & .3090210E-01,.2863464E-01,.2623745E-01,.2353758E-01,.2049216E-01,&
     & .1719200E-01,.1374636E-01,.1041842E-01,.8306976E-02,.3074160E-01,&
     & .2864184E-01,.2639435E-01,.2377357E-01,.2080408E-01,.1762587E-01,&
     & .1431865E-01,.1122298E-01,.9441247E-02,.3058186E-01,.2864960E-01,&
     & .2652689E-01,.2399238E-01,.2114556E-01,.1809030E-01,.1494024E-01,&
     & .1210474E-01,.1065458E-01,.3041236E-01,.2865545E-01,.2666041E-01,&
     & .2422874E-01,.2152521E-01,.1861362E-01,.1566308E-01,.1310960E-01,&
     & .1200349E-01,.3023431E-01,.2863951E-01,.2679661E-01,.2451409E-01,&
     & .2196729E-01,.1923952E-01,.1651206E-01,.1426729E-01,.1351982E-01,&
     & .3150955E-01,.2917466E-01,.2663128E-01,.2375509E-01,.2054034E-01,&
     & .1706533E-01,.1339031E-01,.9619698E-02,.6589618E-02,.3137548E-01,&
     & .2921391E-01,.2682829E-01,.2401415E-01,.2086776E-01,.1749458E-01,&
     & .1391017E-01,.1031015E-01,.7573865E-02,.3124577E-01,.2926104E-01,&
     & .2699356E-01,.2424726E-01,.2121023E-01,.1794016E-01,.1446463E-01,&
     & .1107269E-01,.8630532E-02,.3111437E-01,.2930266E-01,.2714520E-01,&
     & .2449152E-01,.2158301E-01,.1842118E-01,.1509518E-01,.1193837E-01,&
     & .9801078E-02,.3097127E-01,.2932558E-01,.2729444E-01,.2477809E-01,&
     & .2199750E-01,.1898438E-01,.1582945E-01,.1293776E-01,.1111997E-01,&
     & .3182644E-01,.2948280E-01,.2685783E-01,.2385751E-01,.2050472E-01,&
     & .1690419E-01,.1307973E-01,.9016818E-02,.5889262E-02,.3174219E-01,&
     & .2957180E-01,.2709658E-01,.2415027E-01,.2085202E-01,.1733699E-01,&
     & .1357172E-01,.9617188E-02,.6575397E-02,.3166606E-01,.2967426E-01,&
     & .2730658E-01,.2441152E-01,.2120507E-01,.1777789E-01,.1408374E-01,&
     & .1027324E-01,.7358277E-02,.3158325E-01,.2976719E-01,.2749330E-01,&
     & .2467389E-01,.2157919E-01,.1824104E-01,.1464950E-01,.1102019E-01,&
     & .8229230E-02,.3148314E-01,.2984161E-01,.2766625E-01,.2497260E-01,&
     & .2198691E-01,.1876667E-01,.1530472E-01,.1188221E-01,.9182963E-02/

      data   absa(451:585, 7)   /                                       &
     & .3191734E-01,.2961741E-01,.2697787E-01,.2391284E-01,.2049970E-01,&
     & .1685955E-01,.1296507E-01,.8750098E-02,.6274631E-02,.3190439E-01,&
     & .2977990E-01,.2725815E-01,.2422984E-01,.2087607E-01,.1730510E-01,&
     & .1344617E-01,.9295005E-02,.7007386E-02,.3188423E-01,.2993640E-01,&
     & .2750670E-01,.2452542E-01,.2126330E-01,.1774973E-01,.1394711E-01,&
     & .9900539E-02,.7782927E-02,.3184397E-01,.3007747E-01,.2772343E-01,&
     & .2483444E-01,.2166507E-01,.1822702E-01,.1449986E-01,.1058694E-01,&
     & .8671897E-02,.3178483E-01,.3018389E-01,.2793133E-01,.2517191E-01,&
     & .2210215E-01,.1876481E-01,.1512017E-01,.1136237E-01,.9662908E-02,&
     & .3187853E-01,.2963766E-01,.2698548E-01,.2385689E-01,.2040357E-01,&
     & .1672870E-01,.1278988E-01,.8497981E-02,.6174837E-02,.3194590E-01,&
     & .2988270E-01,.2731430E-01,.2420369E-01,.2080895E-01,.1718277E-01,&
     & .1326475E-01,.9006730E-02,.6900214E-02,.3198870E-01,.3010121E-01,&
     & .2760049E-01,.2453900E-01,.2122452E-01,.1764036E-01,.1376646E-01,&
     & .9568560E-02,.7665374E-02,.3200932E-01,.3028722E-01,.2785313E-01,&
     & .2489082E-01,.2164826E-01,.1813119E-01,.1432073E-01,.1019444E-01,&
     & .8520390E-02,.3200022E-01,.3043673E-01,.2810016E-01,.2525761E-01,&
     & .2211496E-01,.1867947E-01,.1492276E-01,.1090133E-01,.9520500E-02,&
     & .3173815E-01,.2955655E-01,.2688573E-01,.2371393E-01,.2023390E-01,&
     & .1654567E-01,.1257985E-01,.8246315E-02,.5647340E-02,.3187913E-01,&
     & .2987574E-01,.2727046E-01,.2410527E-01,.2067778E-01,.1701522E-01,&
     & .1304871E-01,.8729674E-02,.6295753E-02,.3199367E-01,.3016037E-01,&
     & .2760193E-01,.2448927E-01,.2112455E-01,.1748698E-01,.1354982E-01,&
     & .9254652E-02,.7027536E-02,.3208158E-01,.3040579E-01,.2790109E-01,&
     & .2488587E-01,.2157632E-01,.1798983E-01,.1409732E-01,.9833255E-02,&
     & .7857223E-02,.3212706E-01,.3060928E-01,.2819618E-01,.2528322E-01,&
     & .2206831E-01,.1854255E-01,.1468812E-01,.1048268E-01,.8776360E-02/

      data   absa(  1:180, 8)   /                                       &
     & .6013431E-01,.5707436E-01,.7233980E-01,.9815382E-01,.1294324E+00,&
     & .1617466E+00,.1940373E+00,.2262581E+00,.2513644E+00,.5875631E-01,&
     & .5691751E-01,.7437061E-01,.1026345E+00,.1360233E+00,.1699724E+00,&
     & .2039042E+00,.2377536E+00,.2641912E+00,.5766566E-01,.5691471E-01,&
     & .7651533E-01,.1069690E+00,.1422529E+00,.1777591E+00,.2132449E+00,&
     & .2486476E+00,.2761623E+00,.5690082E-01,.5712785E-01,.7881629E-01,&
     & .1114369E+00,.1484198E+00,.1854598E+00,.2224755E+00,.2594138E+00,&
     & .2880248E+00,.5637766E-01,.5755994E-01,.8139096E-01,.1160220E+00,&
     & .1545869E+00,.1931597E+00,.2317210E+00,.2701851E+00,.2998888E+00,&
     & .6618075E-01,.6337862E-01,.7888769E-01,.1046010E+00,.1369770E+00,&
     & .1711778E+00,.2053496E+00,.2394477E+00,.2688156E+00,.6469675E-01,&
     & .6319075E-01,.8109464E-01,.1094837E+00,.1444540E+00,.1805151E+00,&
     & .2165594E+00,.2525366E+00,.2834257E+00,.6352136E-01,.6312417E-01,&
     & .8332160E-01,.1142143E+00,.1514669E+00,.1892772E+00,.2270740E+00,&
     & .2647762E+00,.2971560E+00,.6267746E-01,.6327579E-01,.8565769E-01,&
     & .1189657E+00,.1582452E+00,.1977516E+00,.2372313E+00,.2766038E+00,&
     & .3104162E+00,.6209896E-01,.6369481E-01,.8824714E-01,.1238764E+00,&
     & .1649199E+00,.2060757E+00,.2472064E+00,.2882573E+00,.3234086E+00,&
     & .7395912E-01,.7063428E-01,.8565208E-01,.1104255E+00,.1427485E+00,&
     & .1783683E+00,.2139880E+00,.2495178E+00,.2828639E+00,.7228712E-01,&
     & .7036097E-01,.8793923E-01,.1157395E+00,.1512413E+00,.1890002E+00,&
     & .2267276E+00,.2644122E+00,.2997266E+00,.7086841E-01,.7018107E-01,&
     & .9023900E-01,.1209004E+00,.1591022E+00,.1988304E+00,.2385408E+00,&
     & .2781769E+00,.3153056E+00,.6982927E-01,.7021452E-01,.9257013E-01,&
     & .1259826E+00,.1665897E+00,.2081673E+00,.2497368E+00,.2911947E+00,&
     & .3300931E+00,.6907982E-01,.7054544E-01,.9512295E-01,.1311315E+00,&
     & .1739073E+00,.2173068E+00,.2606881E+00,.3039811E+00,.3445288E+00,&
     & .8336981E-01,.7909588E-01,.9293217E-01,.1163289E+00,.1478222E+00,&
     & .1841644E+00,.2209570E+00,.2576676E+00,.2934903E+00,.8148502E-01,&
     & .7868615E-01,.9524447E-01,.1221028E+00,.1571946E+00,.1961923E+00,&
     & .2353851E+00,.2744893E+00,.3126755E+00,.7980558E-01,.7839855E-01,&
     & .9757525E-01,.1277440E+00,.1659210E+00,.2072782E+00,.2486777E+00,&
     & .2899782E+00,.3303130E+00,.7850723E-01,.7829446E-01,.9994420E-01,&
     & .1331287E+00,.1741803E+00,.2176536E+00,.2611345E+00,.3044935E+00,&
     & .3468364E+00,.7749949E-01,.7851124E-01,.1025029E+00,.1385272E+00,&
     & .1822356E+00,.2277238E+00,.2731983E+00,.3185592E+00,.3628488E+00/

      data   absa(181:315, 8)   /                                       &
     & .9446343E-01,.8887098E-01,.1010405E+00,.1226819E+00,.1528690E+00,&
     & .1887604E+00,.2264637E+00,.2640916E+00,.3014227E+00,.9223331E-01,&
     & .8827980E-01,.1033453E+00,.1288736E+00,.1629549E+00,.2021939E+00,&
     & .2425871E+00,.2829079E+00,.3229151E+00,.9024155E-01,.8783436E-01,&
     & .1057387E+00,.1349856E+00,.1724292E+00,.2146241E+00,.2574756E+00,&
     & .3002489E+00,.3427212E+00,.8862647E-01,.8754527E-01,.1081502E+00,&
     & .1408020E+00,.1814043E+00,.2262377E+00,.2714152E+00,.3165045E+00,&
     & .3612330E+00,.8735280E-01,.8760876E-01,.1107602E+00,.1464752E+00,&
     & .1900954E+00,.2373504E+00,.2847413E+00,.3320395E+00,.3789626E+00,&
     & .1071919E+00,.1001003E+00,.1101040E+00,.1294657E+00,.1574729E+00,&
     & .1915463E+00,.2292568E+00,.2673709E+00,.3054402E+00,.1045794E+00,&
     & .9926080E-01,.1123922E+00,.1359833E+00,.1683351E+00,.2062283E+00,&
     & .2471625E+00,.2882394E+00,.3292971E+00,.1022687E+00,.9862247E-01,&
     & .1148716E+00,.1424605E+00,.1785172E+00,.2200001E+00,.2638803E+00,&
     & .3077436E+00,.3515504E+00,.1003101E+00,.9811233E-01,.1173419E+00,&
     & .1486525E+00,.1881373E+00,.2329575E+00,.2794837E+00,.3259493E+00,&
     & .3723585E+00,.9873698E-01,.9795941E-01,.1200135E+00,.1545733E+00,&
     & .1973376E+00,.2452232E+00,.2942202E+00,.3431039E+00,.3919463E+00,&
     & .1214673E+00,.1126359E+00,.1204125E+00,.1370664E+00,.1621239E+00,&
     & .1935142E+00,.2296249E+00,.2677842E+00,.3060416E+00,.1185186E+00,&
     & .1115740E+00,.1227071E+00,.1438053E+00,.1736162E+00,.2092232E+00,&
     & .2494502E+00,.2909400E+00,.3324970E+00,.1158775E+00,.1107513E+00,&
     & .1251835E+00,.1505785E+00,.1844456E+00,.2241943E+00,.2681063E+00,&
     & .3126858E+00,.3573659E+00,.1135645E+00,.1100550E+00,.1276724E+00,&
     & .1570942E+00,.1947551E+00,.2383233E+00,.2854861E+00,.3329504E+00,&
     & .3805180E+00,.1116111E+00,.1096678E+00,.1303082E+00,.1632798E+00,&
     & .2045649E+00,.2516490E+00,.3017320E+00,.3519240E+00,.4021839E+00/

      data   absa(316:450, 8)   /                                       &
     & .1374520E+00,.1266748E+00,.1318516E+00,.1456457E+00,.1672523E+00,&
     & .1950666E+00,.2283281E+00,.2656918E+00,.3037056E+00,.1341802E+00,&
     & .1254071E+00,.1340698E+00,.1526186E+00,.1792338E+00,.2117540E+00,&
     & .2498928E+00,.2912047E+00,.3328745E+00,.1311544E+00,.1242569E+00,&
     & .1365313E+00,.1596294E+00,.1905813E+00,.2277714E+00,.2703124E+00,&
     & .3152132E+00,.3603201E+00,.1284296E+00,.1232723E+00,.1390451E+00,&
     & .1663760E+00,.2014538E+00,.2429694E+00,.2894262E+00,.3375633E+00,&
     & .3858525E+00,.1260194E+00,.1225869E+00,.1416527E+00,.1727500E+00,&
     & .2118169E+00,.2572538E+00,.3073512E+00,.3584426E+00,.4097428E+00,&
     & .1553761E+00,.1422412E+00,.1444711E+00,.1550385E+00,.1728286E+00,&
     & .1965109E+00,.2258295E+00,.2610663E+00,.2984504E+00,.1516669E+00,&
     & .1407304E+00,.1465588E+00,.1622080E+00,.1852388E+00,.2140019E+00,&
     & .2488334E+00,.2889075E+00,.3302659E+00,.1481522E+00,.1392528E+00,&
     & .1489452E+00,.1694320E+00,.1970843E+00,.2308530E+00,.2707638E+00,&
     & .3150768E+00,.3601714E+00,.1448954E+00,.1379226E+00,.1514075E+00,&
     & .1763709E+00,.2084024E+00,.2469639E+00,.2914326E+00,.3395756E+00,&
     & .3881786E+00,.1419554E+00,.1368141E+00,.1539816E+00,.1829006E+00,&
     & .2192687E+00,.2621639E+00,.3109195E+00,.3624990E+00,.4144086E+00,&
     & .1748198E+00,.1592727E+00,.1584450E+00,.1657904E+00,.1799080E+00,&
     & .1994836E+00,.2245418E+00,.2566689E+00,.2810339E+00,.1705294E+00,&
     & .1573886E+00,.1604084E+00,.1730449E+00,.1927005E+00,.2176246E+00,&
     & .2486329E+00,.2864375E+00,.3169857E+00,.1664134E+00,.1555110E+00,&
     & .1626512E+00,.1803894E+00,.2049406E+00,.2351258E+00,.2717671E+00,&
     & .3146346E+00,.3505565E+00,.1625805E+00,.1537499E+00,.1650044E+00,&
     & .1874575E+00,.2166562E+00,.2519400E+00,.2937443E+00,.3411186E+00,&
     & .3823263E+00,.1590426E+00,.1521966E+00,.1674556E+00,.1940812E+00,&
     & .2278853E+00,.2678905E+00,.3145120E+00,.3659844E+00,.4129266E+00/

      data   absa(451:585, 8)   /                                       &
     & .1935762E+00,.1766297E+00,.1744420E+00,.1806968E+00,.1935473E+00,&
     & .2114104E+00,.2347348E+00,.2655809E+00,.2757601E+00,.1885866E+00,&
     & .1741931E+00,.1762668E+00,.1880588E+00,.2064070E+00,.2298344E+00,&
     & .2593418E+00,.2963843E+00,.3112482E+00,.1838929E+00,.1718429E+00,&
     & .1783334E+00,.1953918E+00,.2186897E+00,.2477198E+00,.2829811E+00,&
     & .3255600E+00,.3451455E+00,.1794971E+00,.1696478E+00,.1805638E+00,&
     & .2022902E+00,.2305237E+00,.2647488E+00,.3054649E+00,.3531405E+00,&
     & .3769471E+00,.1753728E+00,.1677953E+00,.1828746E+00,.2087886E+00,&
     & .2417893E+00,.2809426E+00,.3270047E+00,.3794515E+00,.4073030E+00,&
     & .2131992E+00,.1947813E+00,.1912304E+00,.1963678E+00,.2076432E+00,&
     & .2236360E+00,.2450570E+00,.2741551E+00,.2789175E+00,.2074072E+00,&
     & .1916172E+00,.1927506E+00,.2036992E+00,.2204764E+00,.2423112E+00,&
     & .2701300E+00,.3057154E+00,.3147569E+00,.2019948E+00,.1886976E+00,&
     & .1945776E+00,.2108178E+00,.2327675E+00,.2604245E+00,.2941651E+00,&
     & .3357507E+00,.3491138E+00,.1968339E+00,.1860228E+00,.1965911E+00,&
     & .2174692E+00,.2445952E+00,.2776478E+00,.3170335E+00,.3643697E+00,&
     & .3816200E+00,.1920176E+00,.1837096E+00,.1986031E+00,.2238583E+00,&
     & .2557920E+00,.2940556E+00,.3391509E+00,.3918311E+00,.4121885E+00,&
     & .2332827E+00,.2134102E+00,.2085161E+00,.2124079E+00,.2219989E+00,&
     & .2359574E+00,.2553647E+00,.2822947E+00,.2887838E+00,.2266775E+00,&
     & .2095424E+00,.2096502E+00,.2195248E+00,.2347351E+00,.2548294E+00,&
     & .2808212E+00,.3144477E+00,.3259151E+00,.2203906E+00,.2059404E+00,&
     & .2111339E+00,.2263053E+00,.2469443E+00,.2730233E+00,.3051566E+00,&
     & .3452396E+00,.3610131E+00,.2143742E+00,.2026221E+00,.2127678E+00,&
     & .2326886E+00,.2586790E+00,.2903679E+00,.3283839E+00,.3747332E+00,&
     & .3941719E+00,.2087774E+00,.1996808E+00,.2143237E+00,.2388436E+00,&
     & .2697785E+00,.3069660E+00,.3509603E+00,.4031790E+00,.4260779E+00/

!     the array absb(235,8) (kb(5,13:59,8)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 8,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:120, 1)  /                                        &
     & .1281000E-03,.1289100E-03,.1289000E-03,.1271300E-03,.1247700E-03,&
     & .1065300E-03,.1066700E-03,.1064800E-03,.1047300E-03,.1029300E-03,&
     & .9017600E-04,.9007700E-04,.8928600E-04,.8787900E-04,.8598700E-04,&
     & .7770400E-04,.7730400E-04,.7587900E-04,.7424600E-04,.7207000E-04,&
     & .6718300E-04,.6590600E-04,.6461900E-04,.6300000E-04,.6105600E-04,&
     & .5874100E-04,.5764900E-04,.5602800E-04,.5429300E-04,.5250600E-04,&
     & .5183900E-04,.5071500E-04,.4905300E-04,.4719500E-04,.4536800E-04,&
     & .4432100E-04,.4329700E-04,.4192300E-04,.4029800E-04,.3874400E-04,&
     & .3752700E-04,.3651600E-04,.3538000E-04,.3403700E-04,.3295100E-04,&
     & .3175200E-04,.3070300E-04,.2958600E-04,.2849900E-04,.2751900E-04,&
     & .2663800E-04,.2564300E-04,.2469300E-04,.2383200E-04,.2291300E-04,&
     & .2232500E-04,.2148000E-04,.2062100E-04,.1986700E-04,.1906600E-04,&
     & .1876400E-04,.1799800E-04,.1726900E-04,.1652700E-04,.1582100E-04,&
     & .1565000E-04,.1499200E-04,.1435600E-04,.1371000E-04,.1311900E-04,&
     & .1305400E-04,.1247700E-04,.1192900E-04,.1139300E-04,.1094800E-04,&
     & .1085200E-04,.1039000E-04,.9919500E-05,.9502000E-05,.9199300E-05,&
     & .9099000E-05,.8720900E-05,.8338400E-05,.8002900E-05,.7724500E-05,&
     & .7716900E-05,.7366200E-05,.7055200E-05,.6802900E-05,.6574900E-05,&
     & .6525000E-05,.6236400E-05,.6017200E-05,.5780400E-05,.5550400E-05,&
     & .5516700E-05,.5302100E-05,.5108800E-05,.4911300E-05,.4715000E-05,&
     & .4726100E-05,.4553000E-05,.4364100E-05,.4172500E-05,.4007700E-05,&
     & .4064600E-05,.3906200E-05,.3747200E-05,.3600900E-05,.3454900E-05,&
     & .3547900E-05,.3405700E-05,.3270400E-05,.3142000E-05,.3021900E-05,&
     & .3135300E-05,.3027000E-05,.2902400E-05,.2777600E-05,.2660000E-05/

      data   absb(121:235, 1)   /                                       &
     & .2659500E-05,.2562100E-05,.2464600E-05,.2358200E-05,.2256300E-05,&
     & .2246800E-05,.2165100E-05,.2081200E-05,.2004300E-05,.1922100E-05,&
     & .1900900E-05,.1833500E-05,.1768500E-05,.1703900E-05,.1640600E-05,&
     & .1574600E-05,.1518800E-05,.1466900E-05,.1414800E-05,.1366800E-05,&
     & .1300100E-05,.1256100E-05,.1212200E-05,.1171800E-05,.1130600E-05,&
     & .1071600E-05,.1038000E-05,.1001800E-05,.9674500E-06,.9347500E-06,&
     & .8765100E-06,.8517200E-06,.8245300E-06,.7956000E-06,.7701100E-06,&
     & .7156500E-06,.6969900E-06,.6769900E-06,.6540300E-06,.6320200E-06,&
     & .5839100E-06,.5694600E-06,.5531800E-06,.5357500E-06,.5177400E-06,&
     & .4749000E-06,.4653100E-06,.4517500E-06,.4368600E-06,.4226000E-06,&
     & .3871800E-06,.3761000E-06,.3665000E-06,.3558000E-06,.3441700E-06,&
     & .3146400E-06,.3049400E-06,.2987200E-06,.2909600E-06,.2823000E-06,&
     & .2554200E-06,.2488500E-06,.2431400E-06,.2374500E-06,.2309200E-06,&
     & .2074100E-06,.2017500E-06,.1969900E-06,.1930500E-06,.1879100E-06,&
     & .1676000E-06,.1640600E-06,.1599300E-06,.1563100E-06,.1532700E-06,&
     & .1357900E-06,.1328700E-06,.1298500E-06,.1272500E-06,.1247000E-06,&
     & .1100600E-06,.1075700E-06,.1056800E-06,.1031400E-06,.1010100E-06,&
     & .8871100E-07,.8713100E-07,.8516200E-07,.8340500E-07,.8158500E-07,&
     & .7088900E-07,.6991700E-07,.6869200E-07,.6746300E-07,.6604700E-07,&
     & .5665900E-07,.5607700E-07,.5551800E-07,.5446200E-07,.5356300E-07,&
     & .4538000E-07,.4515900E-07,.4471800E-07,.4406300E-07,.4311600E-07,&
     & .3596200E-07,.3617700E-07,.3599700E-07,.3560000E-07,.3434000E-07,&
     & .2857800E-07,.2892400E-07,.2890100E-07,.2817600E-07,.2724600E-07/

      data   absb(  1:120, 2)   /                                       &
     & .6321500E-03,.6475600E-03,.6573400E-03,.6651500E-03,.6707700E-03,&
     & .5237800E-03,.5363100E-03,.5448500E-03,.5507100E-03,.5560500E-03,&
     & .4377700E-03,.4481400E-03,.4552100E-03,.4612400E-03,.4670000E-03,&
     & .3702300E-03,.3786900E-03,.3856800E-03,.3907000E-03,.3976000E-03,&
     & .3175300E-03,.3263200E-03,.3325400E-03,.3381700E-03,.3431200E-03,&
     & .2758000E-03,.2831000E-03,.2891500E-03,.2948800E-03,.2985500E-03,&
     & .2412900E-03,.2469700E-03,.2530200E-03,.2578500E-03,.2613700E-03,&
     & .2047000E-03,.2104000E-03,.2155200E-03,.2196300E-03,.2226000E-03,&
     & .1729000E-03,.1778200E-03,.1818800E-03,.1851300E-03,.1879400E-03,&
     & .1459900E-03,.1506300E-03,.1538900E-03,.1568200E-03,.1596500E-03,&
     & .1239000E-03,.1279700E-03,.1307100E-03,.1334700E-03,.1357100E-03,&
     & .1055300E-03,.1087500E-03,.1113400E-03,.1135100E-03,.1151300E-03,&
     & .9026300E-04,.9256000E-04,.9466700E-04,.9622500E-04,.9704100E-04,&
     & .7654200E-04,.7822700E-04,.7975500E-04,.8076800E-04,.8141200E-04,&
     & .6456200E-04,.6602400E-04,.6712100E-04,.6787800E-04,.6799200E-04,&
     & .5458700E-04,.5566800E-04,.5643900E-04,.5677800E-04,.5664900E-04,&
     & .4624800E-04,.4688500E-04,.4746900E-04,.4747600E-04,.4746400E-04,&
     & .3908400E-04,.3970100E-04,.3983600E-04,.3991900E-04,.3976200E-04,&
     & .3329700E-04,.3367000E-04,.3378100E-04,.3357100E-04,.3341800E-04,&
     & .2847500E-04,.2847900E-04,.2835200E-04,.2820200E-04,.2810700E-04,&
     & .2402300E-04,.2397700E-04,.2387400E-04,.2385800E-04,.2372900E-04,&
     & .2027600E-04,.2031400E-04,.2026200E-04,.2020500E-04,.2001300E-04,&
     & .1717700E-04,.1717900E-04,.1718100E-04,.1704800E-04,.1691500E-04,&
     & .1458600E-04,.1463200E-04,.1456900E-04,.1446500E-04,.1440400E-04/

      data   absb(121:235, 2)   /                                       &
     & .1216500E-04,.1222000E-04,.1216400E-04,.1209900E-04,.1203600E-04,&
     & .1017800E-04,.1022400E-04,.1020100E-04,.1013400E-04,.1006000E-04,&
     & .8562200E-05,.8582600E-05,.8552000E-05,.8510900E-05,.8437800E-05,&
     & .7026300E-05,.7054900E-05,.7033400E-05,.7000000E-05,.6950000E-05,&
     & .5751000E-05,.5774900E-05,.5769200E-05,.5738400E-05,.5712100E-05,&
     & .4703800E-05,.4726100E-05,.4727800E-05,.4702300E-05,.4686700E-05,&
     & .3824900E-05,.3833900E-05,.3849800E-05,.3836400E-05,.3812500E-05,&
     & .3097300E-05,.3112000E-05,.3116600E-05,.3120300E-05,.3102200E-05,&
     & .2508500E-05,.2526300E-05,.2529300E-05,.2532800E-05,.2524900E-05,&
     & .2026100E-05,.2047900E-05,.2056500E-05,.2057500E-05,.2058800E-05,&
     & .1631900E-05,.1663900E-05,.1674500E-05,.1674000E-05,.1675900E-05,&
     & .1309700E-05,.1344900E-05,.1361100E-05,.1362400E-05,.1358900E-05,&
     & .1052100E-05,.1084700E-05,.1105200E-05,.1110200E-05,.1108500E-05,&
     & .8466800E-06,.8723500E-06,.8954400E-06,.9060700E-06,.9061600E-06,&
     & .6806000E-06,.7018000E-06,.7218700E-06,.7356200E-06,.7380000E-06,&
     & .5463400E-06,.5648100E-06,.5825600E-06,.5954300E-06,.6012200E-06,&
     & .4386600E-06,.4557700E-06,.4695700E-06,.4817600E-06,.4894000E-06,&
     & .3506200E-06,.3664700E-06,.3793500E-06,.3897800E-06,.3969700E-06,&
     & .2788300E-06,.2951400E-06,.3039700E-06,.3125100E-06,.3200400E-06,&
     & .2209800E-06,.2355000E-06,.2428900E-06,.2501800E-06,.2570800E-06,&
     & .1729500E-06,.1862400E-06,.1933200E-06,.1993400E-06,.2058700E-06,&
     & .1362300E-06,.1468000E-06,.1540300E-06,.1591200E-06,.1649900E-06,&
     & .1086400E-06,.1168000E-06,.1235300E-06,.1284000E-06,.1335200E-06/

      data   absb(  1:120, 3)   /                                       &
     & .2106900E-02,.2157100E-02,.2194300E-02,.2206100E-02,.2212400E-02,&
     & .1741100E-02,.1783300E-02,.1809700E-02,.1822500E-02,.1825100E-02,&
     & .1444000E-02,.1479000E-02,.1496300E-02,.1506600E-02,.1506600E-02,&
     & .1199800E-02,.1226100E-02,.1239800E-02,.1250000E-02,.1248700E-02,&
     & .9998000E-03,.1020200E-02,.1033200E-02,.1039900E-02,.1039200E-02,&
     & .8380500E-03,.8562900E-03,.8682700E-03,.8730300E-03,.8754000E-03,&
     & .7074300E-03,.7238100E-03,.7325100E-03,.7355000E-03,.7393100E-03,&
     & .5934800E-03,.6057200E-03,.6113700E-03,.6149600E-03,.6176100E-03,&
     & .4955600E-03,.5048800E-03,.5098700E-03,.5142100E-03,.5158100E-03,&
     & .4146100E-03,.4214500E-03,.4258400E-03,.4296900E-03,.4305500E-03,&
     & .3468300E-03,.3518100E-03,.3566000E-03,.3591100E-03,.3595800E-03,&
     & .2900400E-03,.2938300E-03,.2978700E-03,.2996400E-03,.3004200E-03,&
     & .2414600E-03,.2456100E-03,.2487900E-03,.2505300E-03,.2513400E-03,&
     & .2016000E-03,.2053700E-03,.2077400E-03,.2092800E-03,.2102300E-03,&
     & .1691600E-03,.1722600E-03,.1738600E-03,.1752900E-03,.1763400E-03,&
     & .1418300E-03,.1441400E-03,.1460400E-03,.1474100E-03,.1484400E-03,&
     & .1195400E-03,.1216100E-03,.1228600E-03,.1238600E-03,.1244800E-03,&
     & .1006700E-03,.1020500E-03,.1033100E-03,.1040900E-03,.1047600E-03,&
     & .8501400E-04,.8624800E-04,.8702400E-04,.8795800E-04,.8857900E-04,&
     & .7193000E-04,.7307400E-04,.7398500E-04,.7493400E-04,.7484800E-04,&
     & .6120500E-04,.6225900E-04,.6320900E-04,.6338300E-04,.6346600E-04,&
     & .5204600E-04,.5289600E-04,.5342000E-04,.5351600E-04,.5375500E-04,&
     & .4438000E-04,.4512100E-04,.4527700E-04,.4560400E-04,.4576100E-04,&
     & .3805800E-04,.3840900E-04,.3865700E-04,.3881300E-04,.3862800E-04/

      data   absb(121:235, 3)   /                                       &
     & .3173500E-04,.3199000E-04,.3219900E-04,.3228800E-04,.3217900E-04,&
     & .2637800E-04,.2666700E-04,.2677800E-04,.2684000E-04,.2676000E-04,&
     & .2194900E-04,.2219500E-04,.2234000E-04,.2233000E-04,.2228100E-04,&
     & .1800100E-04,.1820900E-04,.1833800E-04,.1837600E-04,.1830900E-04,&
     & .1471500E-04,.1490600E-04,.1502500E-04,.1509800E-04,.1502600E-04,&
     & .1201900E-04,.1219700E-04,.1231200E-04,.1238000E-04,.1234600E-04,&
     & .9756600E-05,.9931800E-05,.1003600E-04,.1009600E-04,.1011800E-04,&
     & .7881000E-05,.8071600E-05,.8163700E-05,.8220200E-05,.8260800E-05,&
     & .6354100E-05,.6517100E-05,.6633000E-05,.6697700E-05,.6723900E-05,&
     & .5107700E-05,.5242800E-05,.5363600E-05,.5436300E-05,.5469300E-05,&
     & .4088300E-05,.4219600E-05,.4319900E-05,.4407400E-05,.4446200E-05,&
     & .3290900E-05,.3394100E-05,.3474600E-05,.3554600E-05,.3608900E-05,&
     & .2640100E-05,.2726900E-05,.2800300E-05,.2861600E-05,.2918800E-05,&
     & .2122200E-05,.2203200E-05,.2263800E-05,.2312700E-05,.2360300E-05,&
     & .1705600E-05,.1783800E-05,.1838600E-05,.1880400E-05,.1913400E-05,&
     & .1366200E-05,.1433300E-05,.1485500E-05,.1523800E-05,.1552100E-05,&
     & .1088900E-05,.1144500E-05,.1195500E-05,.1230000E-05,.1256000E-05,&
     & .8625100E-06,.9155600E-06,.9599900E-06,.9945400E-06,.1017100E-05,&
     & .6864000E-06,.7286800E-06,.7738900E-06,.8062700E-06,.8269600E-06,&
     & .5433400E-06,.5833900E-06,.6195300E-06,.6519000E-06,.6728500E-06,&
     & .4333600E-06,.4651100E-06,.4956100E-06,.5233700E-06,.5449100E-06,&
     & .3442800E-06,.3723600E-06,.3968700E-06,.4201700E-06,.4379000E-06,&
     & .2788300E-06,.3023700E-06,.3209600E-06,.3397800E-06,.3548500E-06/

      data   absb(  1:120, 4)   /                                       &
     & .5280600E-02,.5355100E-02,.5383600E-02,.5381000E-02,.5341300E-02,&
     & .4359400E-02,.4411400E-02,.4426800E-02,.4418300E-02,.4387700E-02,&
     & .3608000E-02,.3642100E-02,.3660300E-02,.3654000E-02,.3630700E-02,&
     & .3000700E-02,.3032400E-02,.3047900E-02,.3037700E-02,.3017500E-02,&
     & .2511300E-02,.2541500E-02,.2548900E-02,.2542800E-02,.2528100E-02,&
     & .2111900E-02,.2134000E-02,.2139600E-02,.2138100E-02,.2123200E-02,&
     & .1770500E-02,.1788400E-02,.1794200E-02,.1793100E-02,.1781900E-02,&
     & .1471400E-02,.1485400E-02,.1492100E-02,.1492000E-02,.1487400E-02,&
     & .1222400E-02,.1234900E-02,.1243200E-02,.1241100E-02,.1239500E-02,&
     & .1017000E-02,.1028100E-02,.1031900E-02,.1031000E-02,.1028600E-02,&
     & .8468100E-03,.8548400E-03,.8551700E-03,.8554500E-03,.8557100E-03,&
     & .7033000E-03,.7087500E-03,.7107800E-03,.7126100E-03,.7111200E-03,&
     & .5848100E-03,.5887000E-03,.5915700E-03,.5930500E-03,.5923400E-03,&
     & .4864000E-03,.4904400E-03,.4939500E-03,.4946800E-03,.4946600E-03,&
     & .4047200E-03,.4091500E-03,.4123500E-03,.4127800E-03,.4134500E-03,&
     & .3382600E-03,.3420500E-03,.3436400E-03,.3448000E-03,.3452800E-03,&
     & .2830800E-03,.2862700E-03,.2879200E-03,.2891000E-03,.2895800E-03,&
     & .2375900E-03,.2399400E-03,.2419600E-03,.2429500E-03,.2427000E-03,&
     & .1999300E-03,.2020200E-03,.2039000E-03,.2045600E-03,.2041100E-03,&
     & .1685100E-03,.1706000E-03,.1715100E-03,.1716900E-03,.1713300E-03,&
     & .1427500E-03,.1439200E-03,.1444500E-03,.1445200E-03,.1439900E-03,&
     & .1206000E-03,.1214200E-03,.1218100E-03,.1216500E-03,.1210500E-03,&
     & .1016800E-03,.1024000E-03,.1027600E-03,.1026100E-03,.1020200E-03,&
     & .8578000E-04,.8663600E-04,.8681500E-04,.8678000E-04,.8672200E-04/

      data   absb(121:235, 4)   /                                       &
     & .7154000E-04,.7220100E-04,.7242200E-04,.7250100E-04,.7239800E-04,&
     & .5960900E-04,.6015200E-04,.6044300E-04,.6054000E-04,.6053400E-04,&
     & .4968100E-04,.5019400E-04,.5048100E-04,.5064900E-04,.5064300E-04,&
     & .4081500E-04,.4133500E-04,.4156500E-04,.4172900E-04,.4177600E-04,&
     & .3347600E-04,.3393200E-04,.3417100E-04,.3430100E-04,.3439700E-04,&
     & .2741900E-04,.2782400E-04,.2808300E-04,.2822100E-04,.2829600E-04,&
     & .2234900E-04,.2275300E-04,.2297500E-04,.2310000E-04,.2316400E-04,&
     & .1819700E-04,.1848800E-04,.1874700E-04,.1887800E-04,.1894300E-04,&
     & .1481500E-04,.1508100E-04,.1528400E-04,.1540800E-04,.1549300E-04,&
     & .1204600E-04,.1230400E-04,.1247600E-04,.1259000E-04,.1265400E-04,&
     & .9783400E-05,.1001800E-04,.1018700E-04,.1029500E-04,.1036500E-04,&
     & .7905800E-05,.8158300E-05,.8315800E-05,.8425900E-05,.8479100E-05,&
     & .6384700E-05,.6621200E-05,.6773500E-05,.6881800E-05,.6938200E-05,&
     & .5161900E-05,.5369800E-05,.5523900E-05,.5623000E-05,.5693100E-05,&
     & .4176400E-05,.4354600E-05,.4501800E-05,.4592700E-05,.4660900E-05,&
     & .3369600E-05,.3530200E-05,.3658500E-05,.3754800E-05,.3809100E-05,&
     & .2714200E-05,.2858500E-05,.2970000E-05,.3063200E-05,.3116300E-05,&
     & .2197400E-05,.2317600E-05,.2416900E-05,.2498000E-05,.2555500E-05,&
     & .1782000E-05,.1882700E-05,.1964700E-05,.2034700E-05,.2089500E-05,&
     & .1444500E-05,.1527400E-05,.1599900E-05,.1655800E-05,.1700400E-05,&
     & .1161200E-05,.1239000E-05,.1302000E-05,.1350800E-05,.1388100E-05,&
     & .9360600E-06,.1001700E-05,.1059800E-05,.1101600E-05,.1136700E-05,&
     & .7668200E-06,.8231800E-06,.8728100E-06,.9120800E-06,.9403700E-06/

      data   absb(  1:120, 5)   /                                       &
     & .2162906E-01,.2201877E-01,.2224013E-01,.2232097E-01,.2226088E-01,&
     & .1828684E-01,.1857354E-01,.1872091E-01,.1873615E-01,.1863965E-01,&
     & .1535086E-01,.1556848E-01,.1564923E-01,.1561925E-01,.1551236E-01,&
     & .1285052E-01,.1299486E-01,.1303075E-01,.1299836E-01,.1289615E-01,&
     & .1075316E-01,.1084397E-01,.1086973E-01,.1082689E-01,.1072798E-01,&
     & .9002505E-02,.9072913E-02,.9081241E-02,.9034900E-02,.8955849E-02,&
     & .7562305E-02,.7609626E-02,.7609405E-02,.7571232E-02,.7507851E-02,&
     & .6329221E-02,.6362683E-02,.6359991E-02,.6325694E-02,.6271378E-02,&
     & .5284028E-02,.5311691E-02,.5305973E-02,.5285178E-02,.5242950E-02,&
     & .4403121E-02,.4424122E-02,.4421960E-02,.4403212E-02,.4371719E-02,&
     & .3670406E-02,.3685830E-02,.3685814E-02,.3671437E-02,.3647935E-02,&
     & .3056358E-02,.3068927E-02,.3067363E-02,.3057520E-02,.3043098E-02,&
     & .2542912E-02,.2553558E-02,.2555470E-02,.2549363E-02,.2539078E-02,&
     & .2116477E-02,.2124848E-02,.2126732E-02,.2124859E-02,.2121688E-02,&
     & .1763370E-02,.1771671E-02,.1775722E-02,.1777888E-02,.1775372E-02,&
     & .1472312E-02,.1480631E-02,.1485906E-02,.1489295E-02,.1490270E-02,&
     & .1235246E-02,.1243142E-02,.1250718E-02,.1254010E-02,.1256275E-02,&
     & .1040850E-02,.1049997E-02,.1055542E-02,.1060339E-02,.1065075E-02,&
     & .8811911E-03,.8895566E-03,.8956959E-03,.9025351E-03,.9097381E-03,&
     & .7497834E-03,.7578993E-03,.7659613E-03,.7738154E-03,.7827692E-03,&
     & .6406593E-03,.6501363E-03,.6581906E-03,.6679566E-03,.6771392E-03,&
     & .5512107E-03,.5601135E-03,.5694184E-03,.5792748E-03,.5886259E-03,&
     & .4749345E-03,.4844659E-03,.4947239E-03,.5042924E-03,.5143445E-03,&
     & .4104977E-03,.4207339E-03,.4309147E-03,.4409904E-03,.4509584E-03/

      data   absb(121:235, 5)   /                                       &
     & .3490867E-03,.3588313E-03,.3687089E-03,.3789368E-03,.3886881E-03,&
     & .2966250E-03,.3063019E-03,.3155889E-03,.3255266E-03,.3349869E-03,&
     & .2526321E-03,.2616358E-03,.2706359E-03,.2802750E-03,.2890766E-03,&
     & .2118746E-03,.2200243E-03,.2284269E-03,.2373129E-03,.2453378E-03,&
     & .1772518E-03,.1845848E-03,.1923345E-03,.2004084E-03,.2080501E-03,&
     & .1480145E-03,.1547136E-03,.1617671E-03,.1689561E-03,.1761243E-03,&
     & .1228565E-03,.1288933E-03,.1351904E-03,.1417161E-03,.1481713E-03,&
     & .1015563E-03,.1069125E-03,.1124515E-03,.1181336E-03,.1240966E-03,&
     & .8391796E-04,.8846774E-04,.9331366E-04,.9841094E-04,.1037701E-03,&
     & .6907800E-04,.7301768E-04,.7725329E-04,.8170854E-04,.8659904E-04,&
     & .5663202E-04,.6003002E-04,.6370177E-04,.6762785E-04,.7189491E-04,&
     & .4624238E-04,.4927219E-04,.5239730E-04,.5580819E-04,.5956741E-04,&
     & .3765713E-04,.4030391E-04,.4301446E-04,.4597554E-04,.4926441E-04,&
     & .3073045E-04,.3306204E-04,.3540699E-04,.3792633E-04,.4077919E-04,&
     & .2508246E-04,.2707468E-04,.2909111E-04,.3125371E-04,.3371262E-04,&
     & .2044678E-04,.2211599E-04,.2384933E-04,.2568808E-04,.2780633E-04,&
     & .1657839E-04,.1800707E-04,.1948311E-04,.2105650E-04,.2284205E-04,&
     & .1347071E-04,.1469642E-04,.1596158E-04,.1729776E-04,.1878962E-04,&
     & .1095998E-04,.1197888E-04,.1307499E-04,.1421130E-04,.1546214E-04,&
     & .8897744E-05,.9746282E-05,.1066740E-04,.1163972E-04,.1268761E-04,&
     & .7209215E-05,.7906994E-05,.8676190E-05,.9504972E-05,.1037954E-04,&
     & .5835869E-05,.6420526E-05,.7058145E-05,.7757162E-05,.8507066E-05,&
     & .4829548E-05,.5320639E-05,.5859975E-05,.6447762E-05,.7091888E-05/

      data   absb(  1:120, 6)   /                                       &
     & .1413456E+00,.1451118E+00,.1482471E+00,.1506848E+00,.1524351E+00,&
     & .1284824E+00,.1323235E+00,.1353715E+00,.1377662E+00,.1396511E+00,&
     & .1165087E+00,.1201098E+00,.1230983E+00,.1255529E+00,.1275427E+00,&
     & .1056253E+00,.1090989E+00,.1120377E+00,.1145066E+00,.1165271E+00,&
     & .9552257E-01,.9900700E-01,.1020287E+00,.1046013E+00,.1068915E+00,&
     & .8651031E-01,.9000117E-01,.9310752E-01,.9595637E-01,.9839325E-01,&
     & .7836450E-01,.8201222E-01,.8531971E-01,.8822345E-01,.9085876E-01,&
     & .7092238E-01,.7459607E-01,.7796568E-01,.8111548E-01,.8403243E-01,&
     & .6424556E-01,.6792949E-01,.7138543E-01,.7475690E-01,.7782387E-01,&
     & .5852648E-01,.6226349E-01,.6593117E-01,.6939257E-01,.7271605E-01,&
     & .5365315E-01,.5746839E-01,.6121888E-01,.6482400E-01,.6833360E-01,&
     & .4952470E-01,.5342082E-01,.5723031E-01,.6098038E-01,.6470372E-01,&
     & .4609148E-01,.5006244E-01,.5398056E-01,.5794359E-01,.6187053E-01,&
     & .4334627E-01,.4739302E-01,.5147575E-01,.5560970E-01,.5968009E-01,&
     & .4118624E-01,.4532777E-01,.4955345E-01,.5391898E-01,.5813420E-01,&
     & .3952662E-01,.4385566E-01,.4826024E-01,.5268550E-01,.5705181E-01,&
     & .3841724E-01,.4291105E-01,.4755308E-01,.5207099E-01,.5646855E-01,&
     & .3779958E-01,.4252491E-01,.4729707E-01,.5191028E-01,.5641217E-01,&
     & .3769831E-01,.4260967E-01,.4736941E-01,.5212121E-01,.5677123E-01,&
     & .3802439E-01,.4300132E-01,.4793508E-01,.5274887E-01,.5740482E-01,&
     & .3864580E-01,.4371453E-01,.4869077E-01,.5366066E-01,.5849903E-01,&
     & .3933702E-01,.4447547E-01,.4962428E-01,.5473829E-01,.5982359E-01,&
     & .3971900E-01,.4497057E-01,.5034745E-01,.5572486E-01,.6081806E-01,&
     & .3964192E-01,.4514283E-01,.5071851E-01,.5624884E-01,.6141856E-01/

      data   absb(121:235, 6)   /                                       &
     & .3866583E-01,.4430320E-01,.4992214E-01,.5555938E-01,.6080523E-01,&
     & .3776931E-01,.4345093E-01,.4914913E-01,.5483878E-01,.6021309E-01,&
     & .3697431E-01,.4267835E-01,.4846432E-01,.5419260E-01,.5971603E-01,&
     & .3540090E-01,.4109817E-01,.4690264E-01,.5270132E-01,.5824775E-01,&
     & .3376861E-01,.3944516E-01,.4521743E-01,.5105174E-01,.5670807E-01,&
     & .3217884E-01,.3777623E-01,.4359346E-01,.4943539E-01,.5515047E-01,&
     & .3023451E-01,.3578942E-01,.4155720E-01,.4739287E-01,.5322236E-01,&
     & .2814056E-01,.3368562E-01,.3938021E-01,.4520123E-01,.5100155E-01,&
     & .2613012E-01,.3159507E-01,.3723484E-01,.4301429E-01,.4881413E-01,&
     & .2404068E-01,.2939351E-01,.3504512E-01,.4079501E-01,.4656480E-01,&
     & .2179194E-01,.2706163E-01,.3260199E-01,.3830559E-01,.4403615E-01,&
     & .1964345E-01,.2472402E-01,.3015571E-01,.3586422E-01,.4157514E-01,&
     & .1761158E-01,.2244787E-01,.2779047E-01,.3341890E-01,.3908741E-01,&
     & .1578946E-01,.2041434E-01,.2560918E-01,.3115439E-01,.3682998E-01,&
     & .1410355E-01,.1854242E-01,.2353769E-01,.2895272E-01,.3464369E-01,&
     & .1247313E-01,.1674870E-01,.2151305E-01,.2681285E-01,.3241759E-01,&
     & .1093422E-01,.1501782E-01,.1957117E-01,.2466990E-01,.3022010E-01,&
     & .9611967E-02,.1347400E-01,.1786246E-01,.2277546E-01,.2821315E-01,&
     & .8428789E-02,.1205812E-01,.1629395E-01,.2103849E-01,.2627794E-01,&
     & .7314617E-02,.1072816E-01,.1475277E-01,.1935069E-01,.2443117E-01,&
     & .6277076E-02,.9462864E-02,.1326850E-01,.1769409E-01,.2260642E-01,&
     & .5370648E-02,.8332516E-02,.1192724E-01,.1617813E-01,.2094606E-01,&
     & .5000457E-02,.7850552E-02,.1134835E-01,.1553696E-01,.2024820E-01/

      data   absb(  1:120, 7)   /                                       &
     & .1233797E+01,.1240309E+01,.1245906E+01,.1249038E+01,.1250667E+01,&
     & .1227509E+01,.1237527E+01,.1245130E+01,.1250890E+01,.1254582E+01,&
     & .1220370E+01,.1233242E+01,.1243916E+01,.1252002E+01,.1257208E+01,&
     & .1209755E+01,.1226066E+01,.1239648E+01,.1251101E+01,.1262465E+01,&
     & .1198271E+01,.1218221E+01,.1236549E+01,.1253105E+01,.1264757E+01,&
     & .1187954E+01,.1214287E+01,.1237014E+01,.1254616E+01,.1268426E+01,&
     & .1184097E+01,.1214093E+01,.1238496E+01,.1259993E+01,.1276453E+01,&
     & .1180879E+01,.1213827E+01,.1241956E+01,.1265044E+01,.1281987E+01,&
     & .1178298E+01,.1215263E+01,.1245209E+01,.1269051E+01,.1287376E+01,&
     & .1182695E+01,.1219813E+01,.1250266E+01,.1274554E+01,.1293058E+01,&
     & .1187264E+01,.1225491E+01,.1256328E+01,.1280895E+01,.1299927E+01,&
     & .1193354E+01,.1231549E+01,.1262983E+01,.1287976E+01,.1307592E+01,&
     & .1201358E+01,.1239017E+01,.1270766E+01,.1295922E+01,.1315914E+01,&
     & .1210113E+01,.1248264E+01,.1279581E+01,.1304944E+01,.1322887E+01,&
     & .1220261E+01,.1258227E+01,.1289322E+01,.1313044E+01,.1333034E+01,&
     & .1231439E+01,.1268725E+01,.1298471E+01,.1322363E+01,.1344202E+01,&
     & .1243861E+01,.1279684E+01,.1309193E+01,.1334435E+01,.1354444E+01,&
     & .1256537E+01,.1292561E+01,.1320904E+01,.1346216E+01,.1368544E+01,&
     & .1270814E+01,.1305822E+01,.1336261E+01,.1360944E+01,.1384844E+01,&
     & .1288177E+01,.1321480E+01,.1350393E+01,.1377826E+01,.1400827E+01,&
     & .1302859E+01,.1337128E+01,.1367775E+01,.1395784E+01,.1419468E+01,&
     & .1319163E+01,.1354279E+01,.1387311E+01,.1414162E+01,.1436650E+01,&
     & .1332975E+01,.1371023E+01,.1402616E+01,.1431177E+01,.1453004E+01,&
     & .1345134E+01,.1383758E+01,.1415866E+01,.1445650E+01,.1471428E+01/

      data   absb(121:235, 7)   /                                       &
     & .1345082E+01,.1383743E+01,.1417311E+01,.1448976E+01,.1475650E+01,&
     & .1344235E+01,.1383125E+01,.1418137E+01,.1452392E+01,.1479460E+01,&
     & .1345148E+01,.1385709E+01,.1420152E+01,.1456438E+01,.1484592E+01,&
     & .1336140E+01,.1378290E+01,.1414883E+01,.1450315E+01,.1479273E+01,&
     & .1326907E+01,.1369209E+01,.1406929E+01,.1443061E+01,.1473182E+01,&
     & .1317970E+01,.1361334E+01,.1398756E+01,.1434897E+01,.1468648E+01,&
     & .1307925E+01,.1351090E+01,.1391766E+01,.1425847E+01,.1460105E+01,&
     & .1292135E+01,.1338462E+01,.1379193E+01,.1415171E+01,.1450080E+01,&
     & .1275926E+01,.1325616E+01,.1367315E+01,.1406267E+01,.1439728E+01,&
     & .1262168E+01,.1313046E+01,.1356870E+01,.1395704E+01,.1429802E+01,&
     & .1243844E+01,.1297984E+01,.1345253E+01,.1384776E+01,.1423560E+01,&
     & .1221932E+01,.1280865E+01,.1330001E+01,.1373395E+01,.1410813E+01,&
     & .1200347E+01,.1260855E+01,.1314395E+01,.1359376E+01,.1397674E+01,&
     & .1181546E+01,.1245642E+01,.1304804E+01,.1351665E+01,.1391744E+01,&
     & .1161296E+01,.1229915E+01,.1291786E+01,.1343999E+01,.1385754E+01,&
     & .1137476E+01,.1211401E+01,.1274402E+01,.1331651E+01,.1376935E+01,&
     & .1111820E+01,.1188448E+01,.1255900E+01,.1313802E+01,.1364916E+01,&
     & .1087055E+01,.1170159E+01,.1237638E+01,.1301159E+01,.1353229E+01,&
     & .1062808E+01,.1149118E+01,.1221983E+01,.1285669E+01,.1342137E+01,&
     & .1033591E+01,.1125023E+01,.1202761E+01,.1269609E+01,.1328922E+01,&
     & .1001992E+01,.1099491E+01,.1180679E+01,.1251796E+01,.1314844E+01,&
     & .9714260E+00,.1073483E+01,.1159480E+01,.1233092E+01,.1298532E+01,&
     & .9605430E+00,.1062574E+01,.1153238E+01,.1229435E+01,.1295998E+01/

      data   absb(  1:120, 8)   /                                       &
     & .8974716E+01,.8691653E+01,.8415802E+01,.8168444E+01,.7938743E+01,&
     & .9715620E+01,.9387697E+01,.9091159E+01,.8812007E+01,.8550103E+01,&
     & .1040539E+02,.1005124E+02,.9716540E+01,.9405217E+01,.9117647E+01,&
     & .1107237E+02,.1067879E+02,.1030786E+02,.9949266E+01,.9577784E+01,&
     & .1169578E+02,.1125483E+02,.1081775E+02,.1038726E+02,.9999236E+01,&
     & .1224439E+02,.1171912E+02,.1121989E+02,.1076358E+02,.1033881E+02,&
     & .1266035E+02,.1208097E+02,.1155062E+02,.1103907E+02,.1056995E+02,&
     & .1303081E+02,.1240926E+02,.1182594E+02,.1128091E+02,.1079129E+02,&
     & .1335386E+02,.1267661E+02,.1206291E+02,.1149646E+02,.1098015E+02,&
     & .1353946E+02,.1285236E+02,.1221793E+02,.1163647E+02,.1110309E+02,&
     & .1368295E+02,.1297221E+02,.1232355E+02,.1172711E+02,.1117398E+02,&
     & .1377130E+02,.1305174E+02,.1238638E+02,.1177241E+02,.1119857E+02,&
     & .1380231E+02,.1307928E+02,.1239887E+02,.1176960E+02,.1117753E+02,&
     & .1379128E+02,.1305118E+02,.1236306E+02,.1171797E+02,.1114040E+02,&
     & .1373516E+02,.1298613E+02,.1228873E+02,.1164833E+02,.1103314E+02,&
     & .1364257E+02,.1288771E+02,.1219339E+02,.1154194E+02,.1089134E+02,&
     & .1350900E+02,.1275874E+02,.1205091E+02,.1137192E+02,.1073702E+02,&
     & .1335003E+02,.1258061E+02,.1187564E+02,.1118588E+02,.1051162E+02,&
     & .1314747E+02,.1237631E+02,.1163728E+02,.1094357E+02,.1023812E+02,&
     & .1288648E+02,.1212623E+02,.1139312E+02,.1065542E+02,.9954987E+01,&
     & .1264609E+02,.1186159E+02,.1109721E+02,.1034029E+02,.9619395E+01,&
     & .1238368E+02,.1157958E+02,.1077263E+02,.1001950E+02,.9300261E+01,&
     & .1217819E+02,.1132895E+02,.1052829E+02,.9741813E+01,.9027899E+01,&
     & .1202662E+02,.1116103E+02,.1034657E+02,.9538267E+01,.8771268E+01/

      data   absb(121:235, 8)   /                                       &
     & .1208641E+02,.1121978E+02,.1038937E+02,.9556683E+01,.8777908E+01,&
     & .1215415E+02,.1128733E+02,.1043945E+02,.9575262E+01,.8790855E+01,&
     & .1219484E+02,.1131019E+02,.1047069E+02,.9582740E+01,.8782235E+01,&
     & .1240319E+02,.1150569E+02,.1064514E+02,.9772131E+01,.8965666E+01,&
     & .1261524E+02,.1172536E+02,.1085921E+02,.9982767E+01,.9163514E+01,&
     & .1282029E+02,.1192789E+02,.1107155E+02,.1020133E+02,.9340664E+01,&
     & .1305607E+02,.1217814E+02,.1129130E+02,.1045671E+02,.9593408E+01,&
     & .1336869E+02,.1246260E+02,.1158864E+02,.1074016E+02,.9877648E+01,&
     & .1367966E+02,.1274532E+02,.1187204E+02,.1099717E+02,.1016259E+02,&
     & .1395988E+02,.1302676E+02,.1213900E+02,.1127751E+02,.1044562E+02,&
     & .1430124E+02,.1334454E+02,.1243000E+02,.1157468E+02,.1069646E+02,&
     & .1467926E+02,.1368264E+02,.1276261E+02,.1187105E+02,.1102298E+02,&
     & .1504278E+02,.1404941E+02,.1309066E+02,.1219578E+02,.1135077E+02,&
     & .1535675E+02,.1433897E+02,.1332735E+02,.1242447E+02,.1156923E+02,&
     & .1567971E+02,.1462376E+02,.1359863E+02,.1264514E+02,.1177949E+02,&
     & .1604096E+02,.1493692E+02,.1391760E+02,.1291724E+02,.1202255E+02,&
     & .1641711E+02,.1529760E+02,.1424404E+02,.1325595E+02,.1230184E+02,&
     & .1677154E+02,.1558947E+02,.1455293E+02,.1351142E+02,.1256144E+02,&
     & .1711050E+02,.1590689E+02,.1481896E+02,.1379270E+02,.1280515E+02,&
     & .1750874E+02,.1625687E+02,.1512545E+02,.1407639E+02,.1306970E+02,&
     & .1793109E+02,.1661941E+02,.1546240E+02,.1437682E+02,.1334037E+02,&
     & .1833366E+02,.1698072E+02,.1578011E+02,.1467905E+02,.1362875E+02,&
     & .1847759E+02,.1713162E+02,.1587877E+02,.1475220E+02,.1369335E+02/

!  ---
      data   forref(1:3,1: 8)    /                                      &
     & .8605600E-06,.1304390E-05,.3823780E-05,.8179260E-06,.1585990E-05,&
     & .6587710E-04,.1293690E-05,.8244060E-05,.9527780E-04,.4389180E-05,&
     & .3753560E-04,.1191110E-03,.5773510E-04,.7309656E-04,.8352278E-04,&
     & .2517591E-03,.2051344E-03,.5589648E-04,.3341543E-03,.2978433E-03,&
     & .5905554E-04,.4375873E-03,.4018119E-03,.2342177E-03/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 8).

      data   selfref(1:10,1: 8)  /                                      &
     & .7503700E-03,.6449380E-03,.5543210E-03,.4764360E-03,.4094940E-03,&
     & .3519570E-03,.3025050E-03,.2600020E-03,.2234700E-03,.1920710E-03,&
     & .1361350E-02,.1131870E-02,.9410760E-03,.7824400E-03,.6505460E-03,&
     & .5408850E-03,.4497090E-03,.3739020E-03,.3108740E-03,.2584710E-03,&
     & .3339500E-02,.2563910E-02,.1968450E-02,.1511290E-02,.1160300E-02,&
     & .8908240E-03,.6839340E-03,.5250930E-03,.4031430E-03,.3095150E-03,&
     & .7933920E-02,.5898650E-02,.4385480E-02,.3260480E-02,.2424080E-02,&
     & .1802230E-02,.1339910E-02,.9961860E-03,.7406360E-03,.5506420E-03,&
     & .8309571E-02,.7388342E-02,.6584621E-02,.5882080E-02,.5266743E-02,&
     & .4726659E-02,.4251616E-02,.3832861E-02,.3462890E-02,.3135269E-02,&
     & .1297838E-01,.1251031E-01,.1206401E-01,.1163845E-01,.1123247E-01,&
     & .1084516E-01,.1047551E-01,.1012259E-01,.9785517E-02,.9463545E-02,&
     & .1916794E-01,.1802151E-01,.1694407E-01,.1593151E-01,.1497992E-01,&
     & .1408552E-01,.1324501E-01,.1245490E-01,.1171229E-01,.1101426E-01,&
     & .2508831E-01,.2343262E-01,.2189176E-01,.2045740E-01,.1912177E-01,&
     & .1787778E-01,.1671886E-01,.1563887E-01,.1463222E-01,.1369363E-01/

!........................................!
      end module module_radsw_kgb18      !
!========================================!


!========================================!
      module module_radsw_kgb19          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 19:  4650-5150 cm-1 (low - h2o,co2; high - co2)            !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG19

!
      implicit none
!
      private
!
      integer,  public :: MSA19, MSB19, MSF19, MFR19
      parameter (MSA19=585, MSB19=235, MSF19=10, MFR19=3)

      real (kind=kind_phys), public ::           selfref(MSF19,NG19),   &
     &       absa(MSA19,NG19), absb(MSB19,NG19), forref(MFR19,NG19)

!  ---  rayleigh extinction coefficient at v = 4900 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 2.29e-09

!     the array absa(585,NG19) (ka(9,5,13,NG19)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 8, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .1498100E-05,.2665900E-05,.3187400E-05,.3550900E-05,.3759300E-05,&
     & .3851400E-05,.3736900E-05,.3501100E-05,.2389400E-05,.1510300E-05,&
     & .2734800E-05,.3269200E-05,.3655800E-05,.3892600E-05,.3973700E-05,&
     & .3879800E-05,.3656500E-05,.2407200E-05,.1523300E-05,.2809000E-05,&
     & .3374400E-05,.3772900E-05,.4025800E-05,.4110700E-05,.4038900E-05,&
     & .3822600E-05,.2470400E-05,.1521900E-05,.2897800E-05,.3501500E-05,&
     & .3898400E-05,.4156900E-05,.4277100E-05,.4211000E-05,.3996300E-05,&
     & .2550400E-05,.1525400E-05,.2963300E-05,.3622400E-05,.4043500E-05,&
     & .4297500E-05,.4441000E-05,.4384900E-05,.4184700E-05,.2642000E-05,&
     & .1302400E-05,.2369900E-05,.2837000E-05,.3144300E-05,.3332600E-05,&
     & .3379800E-05,.3279700E-05,.3007800E-05,.1881900E-05,.1324900E-05,&
     & .2429900E-05,.2910300E-05,.3233600E-05,.3430800E-05,.3501900E-05,&
     & .3404600E-05,.3132300E-05,.1951100E-05,.1324100E-05,.2505900E-05,&
     & .3001900E-05,.3331700E-05,.3551600E-05,.3633300E-05,.3537200E-05,&
     & .3261500E-05,.2025400E-05,.1332500E-05,.2584000E-05,.3100900E-05,&
     & .3439900E-05,.3669900E-05,.3772700E-05,.3681100E-05,.3401600E-05,&
     & .2061100E-05,.1332100E-05,.2660100E-05,.3218900E-05,.3564000E-05,&
     & .3801200E-05,.3911300E-05,.3827100E-05,.3549900E-05,.2136800E-05,&
     & .1107900E-05,.2075700E-05,.2484600E-05,.2751100E-05,.2888300E-05,&
     & .2914200E-05,.2820300E-05,.2570600E-05,.1512400E-05,.1129800E-05,&
     & .2127800E-05,.2555100E-05,.2830600E-05,.2981400E-05,.3012600E-05,&
     & .2932800E-05,.2672100E-05,.1578400E-05,.1140500E-05,.2181200E-05,&
     & .2623700E-05,.2913000E-05,.3081600E-05,.3126500E-05,.3048000E-05,&
     & .2778400E-05,.1646800E-05,.1134700E-05,.2247200E-05,.2709400E-05,&
     & .3005700E-05,.3182100E-05,.3253600E-05,.3171400E-05,.2883700E-05,&
     & .1718900E-05,.1138300E-05,.2315200E-05,.2801600E-05,.3112600E-05,&
     & .3305100E-05,.3375900E-05,.3296100E-05,.2999300E-05,.1791900E-05,&
     & .9310400E-06,.1800500E-05,.2169300E-05,.2381900E-05,.2482900E-05,&
     & .2486000E-05,.2386000E-05,.2158600E-05,.1219800E-05,.9459100E-06,&
     & .1830900E-05,.2215300E-05,.2448700E-05,.2557400E-05,.2572100E-05,&
     & .2480200E-05,.2245800E-05,.1310500E-05,.9563800E-06,.1875300E-05,&
     & .2278800E-05,.2514900E-05,.2649900E-05,.2667100E-05,.2579700E-05,&
     & .2333500E-05,.1401700E-05,.9576300E-06,.1937900E-05,.2327100E-05,&
     & .2585800E-05,.2743700E-05,.2779200E-05,.2679000E-05,.2426800E-05,&
     & .1487200E-05,.9566800E-06,.1987700E-05,.2399700E-05,.2672100E-05,&
     & .2838200E-05,.2880600E-05,.2785700E-05,.2522500E-05,.1569800E-05/

      data   absa(181:315, 1)   /                                       &
     & .7732900E-06,.1535400E-05,.1872300E-05,.2041100E-05,.2109500E-05,&
     & .2101700E-05,.1999800E-05,.1801700E-05,.9762900E-06,.7824100E-06,&
     & .1566000E-05,.1909100E-05,.2097000E-05,.2178600E-05,.2173000E-05,&
     & .2079400E-05,.1873100E-05,.1064500E-05,.7946300E-06,.1594800E-05,&
     & .1945800E-05,.2152900E-05,.2255700E-05,.2257600E-05,.2163100E-05,&
     & .1946200E-05,.1150700E-05,.7977900E-06,.1638900E-05,.1987700E-05,&
     & .2208700E-05,.2327400E-05,.2346200E-05,.2250100E-05,.2020400E-05,&
     & .1236600E-05,.7987500E-06,.1686500E-05,.2041000E-05,.2278900E-05,&
     & .2404900E-05,.2428300E-05,.2338800E-05,.2100600E-05,.1315500E-05,&
     & .6379900E-06,.1307900E-05,.1587300E-05,.1718700E-05,.1768900E-05,&
     & .1750900E-05,.1662500E-05,.1495000E-05,.7762400E-06,.6442300E-06,&
     & .1318000E-05,.1617300E-05,.1764800E-05,.1825300E-05,.1815200E-05,&
     & .1729900E-05,.1548200E-05,.8545700E-06,.6521400E-06,.1348100E-05,&
     & .1647600E-05,.1815800E-05,.1886700E-05,.1885900E-05,.1799900E-05,&
     & .1608000E-05,.9396800E-06,.6594900E-06,.1372800E-05,.1679500E-05,&
     & .1861800E-05,.1950700E-05,.1956800E-05,.1869300E-05,.1671000E-05,&
     & .1025600E-05,.6601200E-06,.1405100E-05,.1717400E-05,.1915400E-05,&
     & .2011100E-05,.2023000E-05,.1942100E-05,.1736100E-05,.1107200E-05,&
     & .5265200E-06,.1099500E-05,.1322100E-05,.1428200E-05,.1462700E-05,&
     & .1449400E-05,.1375400E-05,.1235900E-05,.6239900E-06,.5263100E-06,&
     & .1104700E-05,.1349800E-05,.1464600E-05,.1509400E-05,.1496100E-05,&
     & .1427200E-05,.1279100E-05,.6969200E-06,.5326300E-06,.1123100E-05,&
     & .1376300E-05,.1505900E-05,.1560200E-05,.1553300E-05,.1482700E-05,&
     & .1325700E-05,.7695900E-06,.5378800E-06,.1144800E-05,.1402600E-05,&
     & .1547800E-05,.1612200E-05,.1613700E-05,.1539800E-05,.1374800E-05,&
     & .8448800E-06,.5424300E-06,.1166000E-05,.1428800E-05,.1589100E-05,&
     & .1661900E-05,.1667200E-05,.1597800E-05,.1426500E-05,.9243900E-06/

      data   absa(316:450, 1)   /                                       &
     & .4286000E-06,.9120800E-06,.1088700E-05,.1173200E-05,.1200000E-05,&
     & .1190100E-05,.1130000E-05,.1018100E-05,.5136000E-06,.4260500E-06,&
     & .9135500E-06,.1110500E-05,.1200400E-05,.1235900E-05,.1223800E-05,&
     & .1167400E-05,.1049600E-05,.5859300E-06,.4291000E-06,.9245800E-06,&
     & .1134100E-05,.1235500E-05,.1275400E-05,.1266700E-05,.1210000E-05,&
     & .1087700E-05,.6520900E-06,.4334500E-06,.9402400E-06,.1154000E-05,&
     & .1266900E-05,.1318600E-05,.1316300E-05,.1256700E-05,.1128000E-05,&
     & .7205100E-06,.4374400E-06,.9584500E-06,.1176400E-05,.1302200E-05,&
     & .1359500E-05,.1361800E-05,.1303400E-05,.1167100E-05,.7937300E-06,&
     & .3445800E-06,.7484400E-06,.8921700E-06,.9587500E-06,.9824300E-06,&
     & .9756600E-06,.9293600E-06,.8361100E-06,.4418100E-06,.3479700E-06,&
     & .7530400E-06,.9081600E-06,.9826400E-06,.1008000E-05,.9988900E-06,&
     & .9531800E-06,.8604100E-06,.5101300E-06,.3459300E-06,.7602000E-06,&
     & .9270700E-06,.1006200E-05,.1039100E-05,.1029600E-05,.9853000E-06,&
     & .8901100E-06,.5807000E-06,.3497100E-06,.7708800E-06,.9462300E-06,&
     & .1034200E-05,.1072600E-05,.1068900E-05,.1021000E-05,.9202000E-06,&
     & .6569700E-06,.3528200E-06,.7851700E-06,.9624500E-06,.1060900E-05,&
     & .1107100E-05,.1107300E-05,.1058300E-05,.9505400E-06,.7295600E-06,&
     & .2784000E-06,.6103500E-06,.7293000E-06,.7841500E-06,.8045100E-06,&
     & .7992400E-06,.7623100E-06,.6872700E-06,.4020800E-06,.2817100E-06,&
     & .6179800E-06,.7419700E-06,.8011500E-06,.8218300E-06,.8162100E-06,&
     & .7803000E-06,.7042500E-06,.4670000E-06,.2802300E-06,.6226900E-06,&
     & .7568500E-06,.8210100E-06,.8453800E-06,.8382100E-06,.8030100E-06,&
     & .7266300E-06,.5378100E-06,.2816300E-06,.6315600E-06,.7721400E-06,&
     & .8424400E-06,.8718400E-06,.8675200E-06,.8302400E-06,.7500000E-06,&
     & .6047800E-06,.2851800E-06,.6407000E-06,.7862800E-06,.8643900E-06,&
     & .8996900E-06,.8991000E-06,.8593300E-06,.7737600E-06,.6701600E-06/

      data   absa(451:585, 1)   /                                       &
     & .2255500E-06,.5004200E-06,.5996000E-06,.6453700E-06,.6632400E-06,&
     & .6592400E-06,.6300300E-06,.5689000E-06,.3446600E-06,.2285800E-06,&
     & .5070500E-06,.6099700E-06,.6594800E-06,.6768800E-06,.6731100E-06,&
     & .6446200E-06,.5833300E-06,.4030000E-06,.2267700E-06,.5117800E-06,&
     & .6229900E-06,.6757200E-06,.6962400E-06,.6912600E-06,.6628500E-06,&
     & .6009000E-06,.4606600E-06,.2282500E-06,.5190700E-06,.6350000E-06,&
     & .6931100E-06,.7179600E-06,.7148400E-06,.6841600E-06,.6185900E-06,&
     & .5196600E-06,.2312200E-06,.5259800E-06,.6457400E-06,.7111600E-06,&
     & .7404600E-06,.7397900E-06,.7067200E-06,.6370800E-06,.5819600E-06,&
     & .1814900E-06,.4082900E-06,.4902200E-06,.5291000E-06,.5440800E-06,&
     & .5416300E-06,.5186700E-06,.4693300E-06,.2849600E-06,.1843300E-06,&
     & .4137800E-06,.4990800E-06,.5402700E-06,.5552200E-06,.5528600E-06,&
     & .5303800E-06,.4810300E-06,.3333600E-06,.1824600E-06,.4179900E-06,&
     & .5096500E-06,.5534700E-06,.5713400E-06,.5678700E-06,.5447700E-06,&
     & .4946200E-06,.3802300E-06,.1840900E-06,.4234700E-06,.5193600E-06,&
     & .5674900E-06,.5887700E-06,.5864800E-06,.5614900E-06,.5082500E-06,&
     & .4308100E-06,.1858200E-06,.4288800E-06,.5283000E-06,.5824000E-06,&
     & .6064700E-06,.6059800E-06,.5793300E-06,.5227700E-06,.4856500E-06,&
     & .1434500E-06,.3292700E-06,.3975000E-06,.4300000E-06,.4431800E-06,&
     & .4420100E-06,.4246900E-06,.3853800E-06,.2337700E-06,.1457100E-06,&
     & .3335500E-06,.4043000E-06,.4395400E-06,.4525600E-06,.4515900E-06,&
     & .4339900E-06,.3946700E-06,.2733800E-06,.1444200E-06,.3371900E-06,&
     & .4131600E-06,.4499600E-06,.4657500E-06,.4636400E-06,.4452800E-06,&
     & .4051000E-06,.3115400E-06,.1460000E-06,.3414100E-06,.4213600E-06,&
     & .4609600E-06,.4793700E-06,.4781700E-06,.4585000E-06,.4159800E-06,&
     & .3528900E-06,.1466700E-06,.3460800E-06,.4285300E-06,.4729600E-06,&
     & .4930000E-06,.4935200E-06,.4728000E-06,.4275700E-06,.3975200E-06/

      data   absa(  1:180, 2)   /                                       &
     & .1628800E-04,.2581700E-04,.2959700E-04,.3131900E-04,.3122000E-04,&
     & .2953900E-04,.2562700E-04,.1975800E-04,.7294400E-05,.1812300E-04,&
     & .2856500E-04,.3263300E-04,.3442700E-04,.3421900E-04,.3222300E-04,&
     & .2798700E-04,.2148500E-04,.7955400E-05,.1995700E-04,.3160700E-04,&
     & .3603300E-04,.3788700E-04,.3769400E-04,.3539900E-04,.3071800E-04,&
     & .2341200E-04,.8529300E-05,.2200500E-04,.3506400E-04,.3976300E-04,&
     & .4169400E-04,.4149000E-04,.3889100E-04,.3371500E-04,.2551200E-04,&
     & .9123200E-05,.2424000E-04,.3890600E-04,.4389900E-04,.4587000E-04,&
     & .4558000E-04,.4261600E-04,.3691200E-04,.2772800E-04,.9779200E-05,&
     & .1494400E-04,.2322300E-04,.2579200E-04,.2701900E-04,.2696300E-04,&
     & .2543100E-04,.2206900E-04,.1693300E-04,.5907100E-05,.1661500E-04,&
     & .2545200E-04,.2832200E-04,.2980000E-04,.2959300E-04,.2777600E-04,&
     & .2412300E-04,.1844400E-04,.6314200E-05,.1835500E-04,.2799600E-04,&
     & .3133800E-04,.3289200E-04,.3257500E-04,.3047800E-04,.2650400E-04,&
     & .2013200E-04,.6789800E-05,.2009200E-04,.3093800E-04,.3468000E-04,&
     & .3625500E-04,.3583600E-04,.3350000E-04,.2910600E-04,.2195000E-04,&
     & .7334300E-05,.2189700E-04,.3415000E-04,.3827400E-04,.3993100E-04,&
     & .3933700E-04,.3675200E-04,.3183100E-04,.2388200E-04,.7870500E-05,&
     & .1313000E-04,.2036300E-04,.2237700E-04,.2300300E-04,.2268300E-04,&
     & .2142900E-04,.1862100E-04,.1417700E-04,.4789200E-05,.1461000E-04,&
     & .2229900E-04,.2448600E-04,.2514000E-04,.2484200E-04,.2341300E-04,&
     & .2036100E-04,.1546900E-04,.5225000E-05,.1613300E-04,.2448200E-04,&
     & .2687600E-04,.2765300E-04,.2735700E-04,.2566900E-04,.2234100E-04,&
     & .1689700E-04,.5664700E-05,.1770800E-04,.2688700E-04,.2952200E-04,&
     & .3057700E-04,.3015100E-04,.2822100E-04,.2454500E-04,.1845500E-04,&
     & .6073500E-05,.1930500E-04,.2936400E-04,.3252000E-04,.3374100E-04,&
     & .3316200E-04,.3097600E-04,.2680000E-04,.2010300E-04,.6486400E-05,&
     & .1116300E-04,.1749800E-04,.1906900E-04,.1956100E-04,.1915400E-04,&
     & .1790600E-04,.1552200E-04,.1177800E-04,.4046100E-05,.1250500E-04,&
     & .1923800E-04,.2091700E-04,.2136500E-04,.2091300E-04,.1947100E-04,&
     & .1698600E-04,.1285600E-04,.4393500E-05,.1384800E-04,.2113200E-04,&
     & .2297500E-04,.2343300E-04,.2289300E-04,.2132700E-04,.1866200E-04,&
     & .1405600E-04,.4701500E-05,.1521900E-04,.2315900E-04,.2525000E-04,&
     & .2575300E-04,.2508800E-04,.2343700E-04,.2045500E-04,.1536600E-04,&
     & .5023900E-05,.1663100E-04,.2531100E-04,.2765100E-04,.2824300E-04,&
     & .2756600E-04,.2577500E-04,.2234400E-04,.1675800E-04,.5359400E-05/

      data   absa(181:315, 2)   /                                       &
     & .9268800E-05,.1482900E-04,.1603600E-04,.1638800E-04,.1608700E-04,&
     & .1504900E-04,.1293900E-04,.9724300E-05,.3506400E-05,.1042800E-04,&
     & .1628900E-04,.1759600E-04,.1792800E-04,.1754700E-04,.1634700E-04,&
     & .1410800E-04,.1062100E-04,.3789300E-05,.1160000E-04,.1792200E-04,&
     & .1940300E-04,.1970100E-04,.1922700E-04,.1783500E-04,.1543900E-04,&
     & .1162700E-04,.4042600E-05,.1279500E-04,.1967200E-04,.2136000E-04,&
     & .2167700E-04,.2107800E-04,.1953500E-04,.1691900E-04,.1274600E-04,&
     & .4301500E-05,.1402900E-04,.2155100E-04,.2341000E-04,.2374600E-04,&
     & .2310600E-04,.2139700E-04,.1850000E-04,.1391800E-04,.4566300E-05,&
     & .7535800E-05,.1228400E-04,.1332100E-04,.1358000E-04,.1332200E-04,&
     & .1245900E-04,.1075700E-04,.8001200E-05,.2859500E-05,.8480100E-05,&
     & .1354100E-04,.1460000E-04,.1485700E-04,.1451900E-04,.1352800E-04,&
     & .1174700E-04,.8719900E-05,.3125000E-05,.9479700E-05,.1489800E-04,&
     & .1608800E-04,.1632900E-04,.1592200E-04,.1478600E-04,.1283900E-04,&
     & .9540600E-05,.3392500E-05,.1051000E-04,.1638700E-04,.1770900E-04,&
     & .1796800E-04,.1746500E-04,.1620500E-04,.1399900E-04,.1046500E-04,&
     & .3678600E-05,.1155400E-04,.1797300E-04,.1944200E-04,.1973600E-04,&
     & .1917000E-04,.1775100E-04,.1528800E-04,.1144100E-04,.3961800E-05,&
     & .5999400E-05,.1006800E-04,.1090300E-04,.1110000E-04,.1091000E-04,&
     & .1020700E-04,.8830100E-05,.6575500E-05,.2353700E-05,.6791100E-05,&
     & .1109800E-04,.1196400E-04,.1216100E-04,.1191000E-04,.1111300E-04,&
     & .9647500E-05,.7173300E-05,.2602100E-05,.7601200E-05,.1220500E-04,&
     & .1316700E-04,.1336800E-04,.1305800E-04,.1213600E-04,.1057100E-04,&
     & .7845200E-05,.2841600E-05,.8458200E-05,.1341200E-04,.1451800E-04,&
     & .1472100E-04,.1432800E-04,.1332300E-04,.1152700E-04,.8599500E-05,&
     & .3092500E-05,.9334600E-05,.1474900E-04,.1597300E-04,.1618000E-04,&
     & .1573000E-04,.1459700E-04,.1260100E-04,.9397300E-05,.3328500E-05/

      data   absa(316:450, 2)   /                                       &
     & .4717400E-05,.8136700E-05,.8818400E-05,.9000700E-05,.8836300E-05,&
     & .8305900E-05,.7210100E-05,.5349600E-05,.1886200E-05,.5354300E-05,&
     & .8985200E-05,.9690400E-05,.9866600E-05,.9650900E-05,.9036800E-05,&
     & .7870000E-05,.5834100E-05,.2114700E-05,.6008800E-05,.9885500E-05,&
     & .1066600E-04,.1084900E-04,.1059700E-04,.9882600E-05,.8624300E-05,&
     & .6394900E-05,.2367200E-05,.6695800E-05,.1087300E-04,.1175100E-04,&
     & .1194500E-04,.1163600E-04,.1085200E-04,.9410100E-05,.7026200E-05,&
     & .2632900E-05,.7419100E-05,.1196000E-04,.1294300E-04,.1315200E-04,&
     & .1278800E-04,.1188900E-04,.1029700E-04,.7697700E-05,.2901900E-05,&
     & .3688600E-05,.6514000E-05,.7068400E-05,.7237700E-05,.7108000E-05,&
     & .6707600E-05,.5835300E-05,.4329700E-05,.1641900E-05,.4177200E-05,&
     & .7210400E-05,.7779300E-05,.7933500E-05,.7768200E-05,.7290300E-05,&
     & .6364000E-05,.4721000E-05,.1872100E-05,.4712300E-05,.7947100E-05,&
     & .8566200E-05,.8739600E-05,.8531900E-05,.7976900E-05,.6969400E-05,&
     & .5179300E-05,.2101500E-05,.5261200E-05,.8731900E-05,.9436500E-05,&
     & .9619100E-05,.9378300E-05,.8758100E-05,.7627700E-05,.5697400E-05,&
     & .2350500E-05,.5840100E-05,.9610600E-05,.1040400E-04,.1059400E-04,&
     & .1030800E-04,.9603000E-05,.8340200E-05,.6262700E-05,.2617000E-05,&
     & .2902300E-05,.5223000E-05,.5666900E-05,.5815800E-05,.5713700E-05,&
     & .5424100E-05,.4741600E-05,.3509800E-05,.1422300E-05,.3275200E-05,&
     & .5790300E-05,.6254800E-05,.6385900E-05,.6252800E-05,.5888100E-05,&
     & .5154200E-05,.3825700E-05,.1655900E-05,.3704700E-05,.6389200E-05,&
     & .6892800E-05,.7031200E-05,.6873400E-05,.6435700E-05,.5636100E-05,&
     & .4200300E-05,.1924800E-05,.4147100E-05,.7017900E-05,.7592000E-05,&
     & .7749600E-05,.7550800E-05,.7062400E-05,.6177800E-05,.4627100E-05,&
     & .2207400E-05,.4606100E-05,.7721500E-05,.8369900E-05,.8530000E-05,&
     & .8312100E-05,.7749300E-05,.6755100E-05,.5093100E-05,.2504300E-05/

      data   absa(451:585, 2)   /                                       &
     & .2406000E-05,.4375200E-05,.4737200E-05,.4859800E-05,.4768900E-05,&
     & .4529000E-05,.3965300E-05,.2944400E-05,.1225500E-05,.2712300E-05,&
     & .4847900E-05,.5233900E-05,.5348200E-05,.5234200E-05,.4927500E-05,&
     & .4323900E-05,.3219600E-05,.1428200E-05,.3063200E-05,.5340200E-05,&
     & .5766000E-05,.5893600E-05,.5751500E-05,.5387300E-05,.4734400E-05,&
     & .3541000E-05,.1663600E-05,.3424500E-05,.5865300E-05,.6355400E-05,&
     & .6487800E-05,.6324400E-05,.5914700E-05,.5185500E-05,.3906800E-05,&
     & .1885800E-05,.3798400E-05,.6455700E-05,.7003200E-05,.7143200E-05,&
     & .6959100E-05,.6493600E-05,.5674800E-05,.4288900E-05,.2152900E-05,&
     & .1982200E-05,.3643800E-05,.3943900E-05,.4042800E-05,.3965600E-05,&
     & .3762700E-05,.3304800E-05,.2462000E-05,.1034900E-05,.2234700E-05,&
     & .4032900E-05,.4355900E-05,.4453500E-05,.4355600E-05,.4101800E-05,&
     & .3615600E-05,.2697600E-05,.1207800E-05,.2519600E-05,.4440500E-05,&
     & .4796500E-05,.4902500E-05,.4785000E-05,.4485400E-05,.3954200E-05,&
     & .2973400E-05,.1390800E-05,.2814200E-05,.4879200E-05,.5288200E-05,&
     & .5396200E-05,.5264600E-05,.4920200E-05,.4322900E-05,.3281700E-05,&
     & .1589500E-05,.3118200E-05,.5368400E-05,.5825600E-05,.5941300E-05,&
     & .5785500E-05,.5404300E-05,.4729800E-05,.3592500E-05,.1805000E-05,&
     & .1620600E-05,.3012200E-05,.3261200E-05,.3345100E-05,.3279400E-05,&
     & .3114400E-05,.2743000E-05,.2048800E-05,.8526700E-06,.1826700E-05,&
     & .3333100E-05,.3600200E-05,.3681600E-05,.3605400E-05,.3395000E-05,&
     & .3002100E-05,.2248700E-05,.9949000E-06,.2058700E-05,.3668100E-05,&
     & .3965100E-05,.4053100E-05,.3956400E-05,.3712700E-05,.3281500E-05,&
     & .2479500E-05,.1142200E-05,.2297300E-05,.4032500E-05,.4372800E-05,&
     & .4460000E-05,.4354000E-05,.4071800E-05,.3584800E-05,.2736600E-05,&
     & .1305100E-05,.2545200E-05,.4437900E-05,.4816400E-05,.4910000E-05,&
     & .4783300E-05,.4467400E-05,.3919400E-05,.2990400E-05,.1481700E-05/

      data   absa(  1:180, 3)   /                                       &
     & .6664100E-04,.9299000E-04,.9627700E-04,.9518200E-04,.9142400E-04,&
     & .8544500E-04,.7701400E-04,.6209100E-04,.1828400E-04,.7535000E-04,&
     & .1039100E-03,.1075100E-03,.1060600E-03,.1017400E-03,.9475600E-04,&
     & .8448700E-04,.6726700E-04,.2003700E-04,.8528600E-04,.1156300E-03,&
     & .1196600E-03,.1177700E-03,.1125900E-03,.1043500E-03,.9207600E-04,&
     & .7249200E-04,.2222900E-04,.9613700E-04,.1283600E-03,.1326500E-03,&
     & .1303100E-03,.1240400E-03,.1140900E-03,.9975600E-04,.7789700E-04,&
     & .2465600E-04,.1073900E-03,.1420900E-03,.1465200E-03,.1436300E-03,&
     & .1357900E-03,.1240800E-03,.1074900E-03,.8327300E-04,.2699800E-04,&
     & .5700500E-04,.8068800E-04,.8419900E-04,.8306700E-04,.7948000E-04,&
     & .7409600E-04,.6632800E-04,.5353200E-04,.1453400E-04,.6451800E-04,&
     & .9027400E-04,.9414900E-04,.9249600E-04,.8837400E-04,.8204800E-04,&
     & .7269800E-04,.5787100E-04,.1620200E-04,.7301400E-04,.1006500E-03,&
     & .1046600E-03,.1027100E-03,.9788000E-04,.9037200E-04,.7936100E-04,&
     & .6235700E-04,.1805600E-04,.8229700E-04,.1119500E-03,.1158600E-03,&
     & .1136500E-03,.1079900E-03,.9882900E-04,.8592300E-04,.6689100E-04,&
     & .1989000E-04,.9217500E-04,.1239900E-03,.1279400E-03,.1253100E-03,&
     & .1182500E-03,.1076400E-03,.9274400E-04,.7160800E-04,.2196800E-04,&
     & .4732400E-04,.6816700E-04,.7106900E-04,.7029300E-04,.6731300E-04,&
     & .6245600E-04,.5564600E-04,.4493300E-04,.1137400E-04,.5365800E-04,&
     & .7634200E-04,.7956000E-04,.7854700E-04,.7499700E-04,.6926400E-04,&
     & .6111900E-04,.4858700E-04,.1263900E-04,.6090500E-04,.8524700E-04,&
     & .8873800E-04,.8738700E-04,.8309800E-04,.7643200E-04,.6673300E-04,&
     & .5227500E-04,.1387100E-04,.6883100E-04,.9496000E-04,.9863800E-04,&
     & .9677800E-04,.9181600E-04,.8374700E-04,.7240300E-04,.5616500E-04,&
     & .1542300E-04,.7725900E-04,.1054900E-03,.1090900E-03,.1067200E-03,&
     & .1006500E-03,.9132000E-04,.7840600E-04,.6015700E-04,.1714100E-04,&
     & .4010200E-04,.5699000E-04,.5944400E-04,.5873200E-04,.5609100E-04,&
     & .5190200E-04,.4624700E-04,.3744200E-04,.9402200E-05,.4493000E-04,&
     & .6389300E-04,.6662900E-04,.6569400E-04,.6262700E-04,.5782200E-04,&
     & .5081900E-04,.4042100E-04,.1047500E-04,.5053200E-04,.7142400E-04,&
     & .7435200E-04,.7322100E-04,.6965100E-04,.6403400E-04,.5543000E-04,&
     & .4352900E-04,.1172800E-04,.5695300E-04,.7974000E-04,.8276900E-04,&
     & .8131400E-04,.7727800E-04,.7032400E-04,.6039600E-04,.4674700E-04,&
     & .1306300E-04,.6401900E-04,.8864700E-04,.9176800E-04,.8992600E-04,&
     & .8495800E-04,.7674000E-04,.6556100E-04,.5006600E-04,.1443500E-04/

      data   absa(181:315, 3)   /                                       &
     & .3380600E-04,.4782600E-04,.4944300E-04,.4877500E-04,.4647700E-04,&
     & .4279100E-04,.3815500E-04,.3117500E-04,.7788600E-05,.3797300E-04,&
     & .5339200E-04,.5552600E-04,.5471900E-04,.5206200E-04,.4776300E-04,&
     & .4202100E-04,.3351000E-04,.8740400E-05,.4275800E-04,.5966600E-04,&
     & .6205800E-04,.6112700E-04,.5801200E-04,.5307100E-04,.4597100E-04,&
     & .3611400E-04,.9849900E-05,.4800600E-04,.6667900E-04,.6923000E-04,&
     & .6798900E-04,.6440000E-04,.5852100E-04,.5019600E-04,.3871000E-04,&
     & .1102200E-04,.5365700E-04,.7424400E-04,.7681300E-04,.7530300E-04,&
     & .7101100E-04,.6409200E-04,.5459900E-04,.4148100E-04,.1222500E-04,&
     & .2795300E-04,.4022300E-04,.4133100E-04,.4033200E-04,.3811400E-04,&
     & .3496700E-04,.3115800E-04,.2570100E-04,.6760400E-05,.3159500E-04,&
     & .4497500E-04,.4637400E-04,.4522900E-04,.4277500E-04,.3916200E-04,&
     & .3437300E-04,.2761600E-04,.7555800E-05,.3564900E-04,.5026300E-04,&
     & .5171300E-04,.5051600E-04,.4785500E-04,.4360500E-04,.3770500E-04,&
     & .2980000E-04,.8406300E-05,.4008600E-04,.5590500E-04,.5758400E-04,&
     & .5634800E-04,.5320000E-04,.4828300E-04,.4133100E-04,.3193600E-04,&
     & .9268600E-05,.4493100E-04,.6202500E-04,.6394200E-04,.6249800E-04,&
     & .5878800E-04,.5303200E-04,.4508500E-04,.3422900E-04,.1017500E-04,&
     & .2282900E-04,.3327800E-04,.3419600E-04,.3338100E-04,.3144400E-04,&
     & .2861500E-04,.2533100E-04,.2109600E-04,.6073200E-05,.2591700E-04,&
     & .3742400E-04,.3852700E-04,.3758000E-04,.3530200E-04,.3202000E-04,&
     & .2795700E-04,.2266000E-04,.6818000E-05,.2938600E-04,.4193300E-04,&
     & .4321300E-04,.4203900E-04,.3944500E-04,.3565900E-04,.3078000E-04,&
     & .2443200E-04,.7629600E-05,.3318600E-04,.4681100E-04,.4821800E-04,&
     & .4684900E-04,.4384800E-04,.3956700E-04,.3385800E-04,.2625700E-04,&
     & .8263400E-05,.3722700E-04,.5206000E-04,.5359200E-04,.5194800E-04,&
     & .4847100E-04,.4359500E-04,.3702200E-04,.2813000E-04,.9005300E-05/

      data   absa(316:450, 3)   /                                       &
     & .1833300E-04,.2725900E-04,.2792900E-04,.2723000E-04,.2567200E-04,&
     & .2337400E-04,.2057100E-04,.1733300E-04,.5415700E-05,.2094800E-04,&
     & .3077400E-04,.3160400E-04,.3082300E-04,.2900200E-04,.2629200E-04,&
     & .2273400E-04,.1859300E-04,.6166900E-05,.2388300E-04,.3464200E-04,&
     & .3565600E-04,.3467000E-04,.3250700E-04,.2930300E-04,.2511100E-04,&
     & .1998500E-04,.6904800E-05,.2709800E-04,.3880200E-04,.4001300E-04,&
     & .3877700E-04,.3621400E-04,.3254200E-04,.2767400E-04,.2152600E-04,&
     & .7659100E-05,.3046200E-04,.4330700E-04,.4459600E-04,.4312700E-04,&
     & .4014200E-04,.3589000E-04,.3027600E-04,.2306500E-04,.8448000E-05,&
     & .1451000E-04,.2204900E-04,.2259600E-04,.2202100E-04,.2075000E-04,&
     & .1889500E-04,.1668500E-04,.1415800E-04,.5028400E-05,.1671000E-04,&
     & .2501200E-04,.2568600E-04,.2501700E-04,.2353700E-04,.2135000E-04,&
     & .1851700E-04,.1517000E-04,.5901600E-05,.1913800E-04,.2827100E-04,&
     & .2910400E-04,.2827200E-04,.2649300E-04,.2391600E-04,.2053000E-04,&
     & .1629700E-04,.6841300E-05,.2179800E-04,.3185900E-04,.3280200E-04,&
     & .3176200E-04,.2960300E-04,.2664300E-04,.2265000E-04,.1755900E-04,&
     & .7809300E-05,.2459900E-04,.3561800E-04,.3673600E-04,.3545600E-04,&
     & .3298100E-04,.2945900E-04,.2487700E-04,.1882100E-04,.8814900E-05,&
     & .1145500E-04,.1779600E-04,.1828200E-04,.1783300E-04,.1681400E-04,&
     & .1530500E-04,.1352600E-04,.1162800E-04,.4763700E-05,.1329500E-04,&
     & .2029000E-04,.2086200E-04,.2031600E-04,.1912500E-04,.1734800E-04,&
     & .1504200E-04,.1245500E-04,.5649500E-05,.1529700E-04,.2299400E-04,&
     & .2374000E-04,.2307600E-04,.2162600E-04,.1952900E-04,.1676600E-04,&
     & .1339900E-04,.6598800E-05,.1748800E-04,.2598600E-04,.2688200E-04,&
     & .2602100E-04,.2424600E-04,.2182000E-04,.1855400E-04,.1442700E-04,&
     & .7464400E-05,.1980000E-04,.2916000E-04,.3020800E-04,.2916100E-04,&
     & .2708600E-04,.2418900E-04,.2046200E-04,.1547400E-04,.8439500E-05/

      data   absa(451:585, 3)   /                                       &
     & .9636600E-05,.1516800E-04,.1561100E-04,.1523000E-04,.1438100E-04,&
     & .1305800E-04,.1145700E-04,.9819500E-05,.4321000E-05,.1122100E-04,&
     & .1730400E-04,.1785900E-04,.1739000E-04,.1638300E-04,.1483700E-04,&
     & .1282000E-04,.1054100E-04,.5091500E-05,.1292200E-04,.1961100E-04,&
     & .2037600E-04,.1976900E-04,.1851000E-04,.1673300E-04,.1429700E-04,&
     & .1137500E-04,.5934900E-05,.1475900E-04,.2222600E-04,.2308700E-04,&
     & .2231900E-04,.2078900E-04,.1867700E-04,.1585400E-04,.1223300E-04,&
     & .6864600E-05,.1668700E-04,.2490100E-04,.2589400E-04,.2502000E-04,&
     & .2323400E-04,.2074600E-04,.1745000E-04,.1314300E-04,.7794500E-05,&
     & .8072800E-05,.1283300E-04,.1324100E-04,.1293000E-04,.1220900E-04,&
     & .1107700E-04,.9687400E-05,.8251800E-05,.3698700E-05,.9409600E-05,&
     & .1464700E-04,.1517900E-04,.1478900E-04,.1392900E-04,.1261400E-04,&
     & .1086300E-04,.8883900E-05,.4364200E-05,.1082700E-04,.1662500E-04,&
     & .1733600E-04,.1682500E-04,.1575700E-04,.1421500E-04,.1213500E-04,&
     & .9570200E-05,.5127600E-05,.1234600E-04,.1882300E-04,.1962600E-04,&
     & .1901100E-04,.1771500E-04,.1589100E-04,.1347200E-04,.1029700E-04,&
     & .5881800E-05,.1395400E-04,.2107100E-04,.2198500E-04,.2127800E-04,&
     & .1977300E-04,.1764200E-04,.1481300E-04,.1110900E-04,.6648500E-05,&
     & .6719800E-05,.1077700E-04,.1115300E-04,.1090000E-04,.1030100E-04,&
     & .9334500E-05,.8143300E-05,.6915600E-05,.3049800E-05,.7828300E-05,&
     & .1228300E-04,.1280500E-04,.1249200E-04,.1175700E-04,.1064900E-04,&
     & .9151900E-05,.7449500E-05,.3602800E-05,.8996600E-05,.1398300E-04,&
     & .1462900E-04,.1421800E-04,.1331500E-04,.1199000E-04,.1023000E-04,&
     & .8023700E-05,.4238200E-05,.1025000E-04,.1579400E-04,.1653600E-04,&
     & .1605500E-04,.1497900E-04,.1342100E-04,.1135500E-04,.8637100E-05,&
     & .4851100E-05,.1157500E-04,.1768900E-04,.1850200E-04,.1795000E-04,&
     & .1668900E-04,.1488300E-04,.1247600E-04,.9353700E-05,.5479900E-05/

      data   absa(  1:180, 4)   /                                       &
     & .2031300E-03,.2641000E-03,.2705400E-03,.2584200E-03,.2379400E-03,&
     & .2103300E-03,.1773000E-03,.1383100E-03,.5299500E-04,.2272200E-03,&
     & .2905800E-03,.2957200E-03,.2820000E-03,.2589800E-03,.2281400E-03,&
     & .1920900E-03,.1495700E-03,.5909600E-04,.2505400E-03,.3158000E-03,&
     & .3199100E-03,.3048600E-03,.2795500E-03,.2463400E-03,.2078700E-03,&
     & .1617900E-03,.6511800E-04,.2719700E-03,.3397100E-03,.3431000E-03,&
     & .3267700E-03,.3000200E-03,.2657400E-03,.2250200E-03,.1752000E-03,&
     & .7171100E-04,.2917800E-03,.3621300E-03,.3650300E-03,.3481500E-03,&
     & .3211400E-03,.2859900E-03,.2432800E-03,.1895800E-03,.7852300E-04,&
     & .1754700E-03,.2301200E-03,.2363900E-03,.2260600E-03,.2079900E-03,&
     & .1832600E-03,.1538500E-03,.1188800E-03,.4363300E-04,.1960100E-03,&
     & .2526900E-03,.2575100E-03,.2462900E-03,.2262300E-03,.1992600E-03,&
     & .1671200E-03,.1287700E-03,.4838200E-04,.2159800E-03,.2742300E-03,&
     & .2781200E-03,.2657600E-03,.2440300E-03,.2155200E-03,.1814100E-03,&
     & .1398200E-03,.5356000E-04,.2344900E-03,.2948200E-03,.2982000E-03,&
     & .2845400E-03,.2617100E-03,.2325400E-03,.1968100E-03,.1519200E-03,&
     & .5905400E-04,.2519800E-03,.3140800E-03,.3172300E-03,.3034100E-03,&
     & .2804000E-03,.2502200E-03,.2133300E-03,.1648700E-03,.6477900E-04,&
     & .1473600E-03,.1948300E-03,.2006500E-03,.1924200E-03,.1772800E-03,&
     & .1562700E-03,.1307800E-03,.1003700E-03,.3421700E-04,.1649200E-03,&
     & .2144100E-03,.2189200E-03,.2098000E-03,.1928900E-03,.1700000E-03,&
     & .1424900E-03,.1090200E-03,.3835200E-04,.1820000E-03,.2329400E-03,&
     & .2365800E-03,.2265800E-03,.2082200E-03,.1839100E-03,.1548400E-03,&
     & .1188800E-03,.4270000E-04,.1981100E-03,.2509400E-03,.2538500E-03,&
     & .2427600E-03,.2233800E-03,.1984100E-03,.1682200E-03,.1294300E-03,&
     & .4725700E-04,.2135200E-03,.2678400E-03,.2702700E-03,.2591700E-03,&
     & .2396600E-03,.2139900E-03,.1823900E-03,.1405800E-03,.5199700E-04,&
     & .1207000E-03,.1628200E-03,.1687000E-03,.1621500E-03,.1496600E-03,&
     & .1319500E-03,.1100300E-03,.8374300E-04,.2684100E-04,.1361600E-03,&
     & .1797900E-03,.1842000E-03,.1769200E-03,.1628800E-03,.1435100E-03,&
     & .1200000E-03,.9132400E-04,.2995600E-04,.1514200E-03,.1960600E-03,&
     & .1993300E-03,.1913200E-03,.1758200E-03,.1550600E-03,.1306100E-03,&
     & .9988100E-04,.3317500E-04,.1659100E-03,.2117000E-03,.2142100E-03,&
     & .2052500E-03,.1888800E-03,.1674700E-03,.1419200E-03,.1090700E-03,&
     & .3662500E-04,.1795600E-03,.2265500E-03,.2283300E-03,.2194900E-03,&
     & .2028300E-03,.1808600E-03,.1539300E-03,.1186500E-03,.4051300E-04/

      data   absa(181:315, 4)   /                                       &
     & .9862500E-04,.1348000E-03,.1409900E-03,.1365300E-03,.1260000E-03,&
     & .1110700E-03,.9210200E-04,.6941400E-04,.2259700E-04,.1116500E-03,&
     & .1497500E-03,.1542900E-03,.1488600E-03,.1371000E-03,.1207600E-03,&
     & .1005100E-03,.7604600E-04,.2526500E-04,.1246100E-03,.1639100E-03,&
     & .1672500E-03,.1609100E-03,.1479600E-03,.1304700E-03,.1094100E-03,&
     & .8319300E-04,.2796200E-04,.1373800E-03,.1776100E-03,.1798200E-03,&
     & .1728100E-03,.1591600E-03,.1408700E-03,.1189200E-03,.9097200E-04,&
     & .3085500E-04,.1496700E-03,.1906600E-03,.1923100E-03,.1850900E-03,&
     & .1710900E-03,.1522700E-03,.1290600E-03,.9913600E-04,.3401700E-04,&
     & .7986300E-04,.1103400E-03,.1162200E-03,.1140000E-03,.1053200E-03,&
     & .9297800E-04,.7666100E-04,.5728700E-04,.1866400E-04,.9075300E-04,&
     & .1231800E-03,.1276600E-03,.1244000E-03,.1148200E-03,.1011000E-03,&
     & .8369700E-04,.6266300E-04,.2092800E-04,.1018300E-03,.1353200E-03,&
     & .1390700E-03,.1346400E-03,.1239600E-03,.1092500E-03,.9098400E-04,&
     & .6851300E-04,.2326400E-04,.1129300E-03,.1474200E-03,.1499500E-03,&
     & .1448100E-03,.1335400E-03,.1179700E-03,.9884500E-04,.7496500E-04,&
     & .2585000E-04,.1236000E-03,.1591400E-03,.1609000E-03,.1552100E-03,&
     & .1436100E-03,.1275500E-03,.1073900E-03,.8191500E-04,.2863500E-04,&
     & .6567900E-04,.9008400E-04,.9568100E-04,.9456700E-04,.8733800E-04,&
     & .7727500E-04,.6364200E-04,.4709700E-04,.1550800E-04,.7455100E-04,&
     & .1008900E-03,.1053100E-03,.1032700E-03,.9536400E-04,.8415400E-04,&
     & .6956900E-04,.5142600E-04,.1723400E-04,.8358400E-04,.1113700E-03,&
     & .1146900E-03,.1119300E-03,.1032500E-03,.9107500E-04,.7547900E-04,&
     & .5622800E-04,.1912700E-04,.9276200E-04,.1217100E-03,.1242000E-03,&
     & .1206000E-03,.1114700E-03,.9846500E-04,.8196900E-04,.6152900E-04,&
     & .2145200E-04,.1018900E-03,.1318200E-03,.1336400E-03,.1296000E-03,&
     & .1200800E-03,.1065200E-03,.8904500E-04,.6733400E-04,.2371100E-04/

      data   absa(316:450, 4)   /                                       &
     & .5365600E-04,.7410900E-04,.7862900E-04,.7824500E-04,.7232200E-04,&
     & .6391500E-04,.5262500E-04,.3859100E-04,.1441600E-04,.6116300E-04,&
     & .8310400E-04,.8683200E-04,.8556200E-04,.7899200E-04,.6966600E-04,&
     & .5757400E-04,.4209800E-04,.1592100E-04,.6891400E-04,.9190900E-04,&
     & .9459400E-04,.9281600E-04,.8559000E-04,.7549100E-04,.6244500E-04,&
     & .4610900E-04,.1720900E-04,.7684700E-04,.1005900E-03,.1025600E-03,&
     & .1001900E-03,.9249600E-04,.8172900E-04,.6776300E-04,.5044500E-04,&
     & .1858900E-04,.8484700E-04,.1090200E-03,.1107100E-03,.1078400E-03,&
     & .9981200E-04,.8851600E-04,.7371800E-04,.5525000E-04,.2019400E-04,&
     & .4334600E-04,.6060800E-04,.6481800E-04,.6468100E-04,.5965500E-04,&
     & .5261700E-04,.4320600E-04,.3157300E-04,.1469600E-04,.4966900E-04,&
     & .6838000E-04,.7180000E-04,.7086000E-04,.6526000E-04,.5749600E-04,&
     & .4731300E-04,.3440500E-04,.1616400E-04,.5627400E-04,.7604000E-04,&
     & .7845400E-04,.7694400E-04,.7066500E-04,.6234700E-04,.5138300E-04,&
     & .3766500E-04,.1784100E-04,.6312700E-04,.8345100E-04,.8517800E-04,&
     & .8308200E-04,.7643700E-04,.6753800E-04,.5579700E-04,.4126400E-04,&
     & .1950400E-04,.7012400E-04,.9080700E-04,.9196000E-04,.8943200E-04,&
     & .8252700E-04,.7317600E-04,.6066800E-04,.4517900E-04,.2099200E-04,&
     & .3505900E-04,.4953100E-04,.5337600E-04,.5366500E-04,.4964400E-04,&
     & .4359200E-04,.3560900E-04,.2584500E-04,.1323100E-04,.4037600E-04,&
     & .5617400E-04,.5950200E-04,.5904000E-04,.5439900E-04,.4768700E-04,&
     & .3900700E-04,.2818900E-04,.1522300E-04,.4600600E-04,.6289900E-04,&
     & .6521400E-04,.6417800E-04,.5897600E-04,.5172600E-04,.4238600E-04,&
     & .3084300E-04,.1724100E-04,.5190500E-04,.6939500E-04,.7098800E-04,&
     & .6946100E-04,.6380100E-04,.5604000E-04,.4605900E-04,.3383100E-04,&
     & .1971400E-04,.5799000E-04,.7573700E-04,.7682600E-04,.7471800E-04,&
     & .6883900E-04,.6066400E-04,.5010500E-04,.3704900E-04,.2230800E-04/

      data   absa(451:585, 4)   /                                       &
     & .3015000E-04,.4273800E-04,.4601100E-04,.4633000E-04,.4290700E-04,&
     & .3760500E-04,.3070200E-04,.2196200E-04,.1179100E-04,.3471400E-04,&
     & .4851600E-04,.5112000E-04,.5072600E-04,.4691200E-04,.4103300E-04,&
     & .3350500E-04,.2395200E-04,.1324200E-04,.3964900E-04,.5432500E-04,&
     & .5614600E-04,.5527000E-04,.5090200E-04,.4455900E-04,.3635100E-04,&
     & .2626600E-04,.1507500E-04,.4477900E-04,.5989300E-04,.6116600E-04,&
     & .5976600E-04,.5507000E-04,.4830300E-04,.3949100E-04,.2884700E-04,&
     & .1716400E-04,.4994600E-04,.6541100E-04,.6625600E-04,.6434500E-04,&
     & .5947400E-04,.5228700E-04,.4300300E-04,.3161200E-04,.1954200E-04,&
     & .2568100E-04,.3662200E-04,.3942800E-04,.3958200E-04,.3678600E-04,&
     & .3221000E-04,.2627000E-04,.1862900E-04,.1003500E-04,.2963600E-04,&
     & .4162000E-04,.4370100E-04,.4342500E-04,.4015000E-04,.3510500E-04,&
     & .2863700E-04,.2036500E-04,.1141600E-04,.3392100E-04,.4662800E-04,&
     & .4806200E-04,.4730100E-04,.4363300E-04,.3818900E-04,.3113300E-04,&
     & .2240500E-04,.1294600E-04,.3834100E-04,.5140100E-04,.5245700E-04,&
     & .5112900E-04,.4723800E-04,.4142900E-04,.3390500E-04,.2454600E-04,&
     & .1477000E-04,.4257600E-04,.5615300E-04,.5686300E-04,.5518200E-04,&
     & .5111300E-04,.4494000E-04,.3698700E-04,.2689800E-04,.1686500E-04,&
     & .2170700E-04,.3114200E-04,.3348700E-04,.3365000E-04,.3130100E-04,&
     & .2739600E-04,.2235200E-04,.1574400E-04,.8300500E-05,.2511200E-04,&
     & .3547400E-04,.3715800E-04,.3697000E-04,.3417000E-04,.2987100E-04,&
     & .2436100E-04,.1731200E-04,.9355900E-05,.2876200E-04,.3971200E-04,&
     & .4092400E-04,.4019900E-04,.3716900E-04,.3256500E-04,.2656600E-04,&
     & .1907400E-04,.1067000E-04,.3244800E-04,.4380800E-04,.4473400E-04,&
     & .4353400E-04,.4032300E-04,.3536700E-04,.2901100E-04,.2089700E-04,&
     & .1224000E-04,.3590500E-04,.4777100E-04,.4848200E-04,.4709400E-04,&
     & .4370300E-04,.3847400E-04,.3171500E-04,.2289900E-04,.1395800E-04/

      data   absa(  1:180, 5)   /                                       &
     & .7617673E-03,.8971104E-03,.9278378E-03,.9076002E-03,.8545237E-03,&
     & .7758218E-03,.6640038E-03,.5009860E-03,.2100718E-03,.7925828E-03,&
     & .9442955E-03,.9755110E-03,.9542425E-03,.9014791E-03,.8188875E-03,&
     & .7022179E-03,.5337239E-03,.2374824E-03,.8249630E-03,.9924062E-03,&
     & .1024145E-02,.1003594E-02,.9497958E-03,.8633171E-03,.7423153E-03,&
     & .5703062E-03,.2679147E-03,.8583524E-03,.1040661E-02,.1074578E-02,&
     & .1055443E-02,.9993194E-03,.9103749E-03,.7856031E-03,.6103198E-03,&
     & .3005890E-03,.8914210E-03,.1089831E-02,.1127450E-02,.1108978E-02,&
     & .1051150E-02,.9598170E-03,.8317411E-03,.6540146E-03,.3372384E-03,&
     & .6627779E-03,.7779969E-03,.8008718E-03,.7838334E-03,.7398886E-03,&
     & .6733388E-03,.5758040E-03,.4394197E-03,.1791672E-03,.6931434E-03,&
     & .8216003E-03,.8453718E-03,.8264838E-03,.7819403E-03,.7110987E-03,&
     & .6107270E-03,.4698192E-03,.2021868E-03,.7240368E-03,.8657540E-03,&
     & .8908257E-03,.8720081E-03,.8257095E-03,.7512371E-03,.6480081E-03,&
     & .5029834E-03,.2276890E-03,.7549662E-03,.9109098E-03,.9378553E-03,&
     & .9196391E-03,.8709512E-03,.7935808E-03,.6875784E-03,.5392493E-03,&
     & .2558951E-03,.7854858E-03,.9570998E-03,.9870226E-03,.9682145E-03,&
     & .9174446E-03,.8393444E-03,.7305754E-03,.5787675E-03,.2867225E-03,&
     & .5698218E-03,.6646945E-03,.6811412E-03,.6675254E-03,.6306449E-03,&
     & .5735521E-03,.4911267E-03,.3756753E-03,.1468372E-03,.5979398E-03,&
     & .7039422E-03,.7219077E-03,.7061823E-03,.6682445E-03,.6070832E-03,&
     & .5222789E-03,.4034172E-03,.1660478E-03,.6260630E-03,.7441821E-03,&
     & .7640366E-03,.7472492E-03,.7070503E-03,.6422790E-03,.5549202E-03,&
     & .4333711E-03,.1875462E-03,.6541096E-03,.7851727E-03,.8072018E-03,&
     & .7903360E-03,.7475103E-03,.6804148E-03,.5905641E-03,.4657632E-03,&
     & .2111058E-03,.6816820E-03,.8277472E-03,.8519631E-03,.8340047E-03,&
     & .7892681E-03,.7212366E-03,.6290190E-03,.5008742E-03,.2367354E-03,&
     & .4861342E-03,.5638913E-03,.5756770E-03,.5644357E-03,.5327681E-03,&
     & .4835474E-03,.4138863E-03,.3179456E-03,.1189626E-03,.5113753E-03,&
     & .5987845E-03,.6124007E-03,.5992699E-03,.5662793E-03,.5134428E-03,&
     & .4408138E-03,.3421165E-03,.1351360E-03,.5363872E-03,.6346295E-03,&
     & .6504470E-03,.6359055E-03,.6008677E-03,.5448237E-03,.4696582E-03,&
     & .3682585E-03,.1527590E-03,.5608616E-03,.6715761E-03,.6895903E-03,&
     & .6743859E-03,.6365185E-03,.5786033E-03,.5010073E-03,.3967389E-03,&
     & .1724827E-03,.5849723E-03,.7098200E-03,.7303204E-03,.7137716E-03,&
     & .6742570E-03,.6146787E-03,.5348228E-03,.4271745E-03,.1936713E-03/

      data   absa(181:315, 5)   /                                       &
     & .4124715E-03,.4761738E-03,.4853020E-03,.4745565E-03,.4479021E-03,&
     & .4057915E-03,.3474465E-03,.2667416E-03,.9516165E-04,.4344911E-03,&
     & .5069050E-03,.5177953E-03,.5062569E-03,.4775319E-03,.4319312E-03,&
     & .3702139E-03,.2876629E-03,.1085617E-03,.4559645E-03,.5386154E-03,&
     & .5518018E-03,.5389479E-03,.5081875E-03,.4599769E-03,.3953892E-03,&
     & .3104428E-03,.1229340E-03,.4769668E-03,.5709955E-03,.5869421E-03,&
     & .5732638E-03,.5400999E-03,.4895927E-03,.4226193E-03,.3347530E-03,&
     & .1391112E-03,.4977864E-03,.6042702E-03,.6230627E-03,.6084073E-03,&
     & .5737828E-03,.5212317E-03,.4520350E-03,.3599815E-03,.1565328E-03,&
     & .3481462E-03,.3997927E-03,.4064302E-03,.3958879E-03,.3740189E-03,&
     & .3386598E-03,.2894388E-03,.2220176E-03,.7721520E-04,.3667478E-03,&
     & .4264134E-03,.4347817E-03,.4238152E-03,.3999290E-03,.3615359E-03,&
     & .3088615E-03,.2396985E-03,.8791758E-04,.3848509E-03,.4536800E-03,&
     & .4643234E-03,.4533341E-03,.4271237E-03,.3859069E-03,.3305190E-03,&
     & .2589672E-03,.9917704E-04,.4025904E-03,.4814459E-03,.4948753E-03,&
     & .4836881E-03,.4553829E-03,.4119222E-03,.3539441E-03,.2788595E-03,&
     & .1117935E-03,.4204129E-03,.5099243E-03,.5262683E-03,.5145801E-03,&
     & .4847600E-03,.4394867E-03,.3794179E-03,.3009917E-03,.1256044E-03,&
     & .2923436E-03,.3350198E-03,.3390235E-03,.3289566E-03,.3102593E-03,&
     & .2807257E-03,.2400716E-03,.1841474E-03,.6514967E-04,.3079436E-03,&
     & .3575834E-03,.3635135E-03,.3531948E-03,.3332576E-03,.3010207E-03,&
     & .2570614E-03,.1988560E-03,.7422428E-04,.3232435E-03,.3805545E-03,&
     & .3886434E-03,.3787663E-03,.3570394E-03,.3221858E-03,.2755102E-03,&
     & .2150719E-03,.8445731E-04,.3383325E-03,.4040182E-03,.4144616E-03,&
     & .4051543E-03,.3816543E-03,.3448001E-03,.2956857E-03,.2316464E-03,&
     & .9529395E-04,.3535369E-03,.4281879E-03,.4413816E-03,.4318028E-03,&
     & .4070455E-03,.3690091E-03,.3175244E-03,.2504699E-03,.1070304E-03/

      data   absa(316:450, 5)   /                                       &
     & .2453512E-03,.2800882E-03,.2825252E-03,.2730919E-03,.2566169E-03,&
     & .2318802E-03,.1983126E-03,.1522973E-03,.5516642E-04,.2581699E-03,&
     & .2989905E-03,.3031376E-03,.2937127E-03,.2763027E-03,.2495537E-03,&
     & .2126952E-03,.1646454E-03,.6300898E-04,.2707693E-03,.3182237E-03,&
     & .3242581E-03,.3153146E-03,.2972863E-03,.2680977E-03,.2289101E-03,&
     & .1781625E-03,.7205544E-04,.2834359E-03,.3379200E-03,.3459773E-03,&
     & .3376659E-03,.3181431E-03,.2874562E-03,.2463233E-03,.1919211E-03,&
     & .8179890E-04,.2961968E-03,.3584988E-03,.3687767E-03,.3605384E-03,&
     & .3397168E-03,.3080263E-03,.2651369E-03,.2080999E-03,.9227521E-04,&
     & .2054310E-03,.2336020E-03,.2346874E-03,.2262676E-03,.2118556E-03,&
     & .1911126E-03,.1628566E-03,.1251317E-03,.5265615E-04,.2158983E-03,&
     & .2491904E-03,.2518957E-03,.2435142E-03,.2286421E-03,.2061588E-03,&
     & .1755758E-03,.1356394E-03,.5986972E-04,.2262897E-03,.2650278E-03,&
     & .2694531E-03,.2615131E-03,.2463445E-03,.2220951E-03,.1892217E-03,&
     & .1469564E-03,.6762150E-04,.2368500E-03,.2815163E-03,.2876091E-03,&
     & .2802849E-03,.2643048E-03,.2385402E-03,.2043353E-03,.1585617E-03,&
     & .7608500E-04,.2475469E-03,.2987922E-03,.3069398E-03,.2998091E-03,&
     & .2825176E-03,.2560367E-03,.2200683E-03,.1724086E-03,.8542444E-04,&
     & .1724065E-03,.1956039E-03,.1956892E-03,.1881402E-03,.1756197E-03,&
     & .1579428E-03,.1342625E-03,.1031020E-03,.6401734E-04,.1809875E-03,&
     & .2084355E-03,.2098304E-03,.2024866E-03,.1896267E-03,.1710181E-03,&
     & .1452408E-03,.1119637E-03,.7219467E-04,.1895575E-03,.2214531E-03,&
     & .2244155E-03,.2174338E-03,.2045039E-03,.1844632E-03,.1570651E-03,&
     & .1214344E-03,.8097348E-04,.1983280E-03,.2351430E-03,.2396300E-03,&
     & .2331386E-03,.2197000E-03,.1984522E-03,.1696161E-03,.1313673E-03,&
     & .9012464E-04,.2073235E-03,.2496450E-03,.2559261E-03,.2497870E-03,&
     & .2354805E-03,.2133323E-03,.1830084E-03,.1431258E-03,.9946132E-04/

      data   absa(451:585, 5)   /                                       &
     & .1474531E-03,.1682762E-03,.1683793E-03,.1616613E-03,.1506254E-03,&
     & .1352532E-03,.1147895E-03,.8797863E-04,.6244135E-04,.1544785E-03,&
     & .1789686E-03,.1802145E-03,.1739110E-03,.1625411E-03,.1464475E-03,&
     & .1243412E-03,.9584180E-04,.7111930E-04,.1616375E-03,.1899537E-03,&
     & .1924972E-03,.1864886E-03,.1751900E-03,.1580145E-03,.1346723E-03,&
     & .1036729E-03,.7851804E-04,.1690889E-03,.2016073E-03,.2056052E-03,&
     & .2000516E-03,.1882465E-03,.1701850E-03,.1455146E-03,.1127735E-03,&
     & .8728573E-04,.1768424E-03,.2142601E-03,.2196970E-03,.2143761E-03,&
     & .2020742E-03,.1832716E-03,.1571724E-03,.1229344E-03,.9627397E-04,&
     & .1259909E-03,.1440828E-03,.1441655E-03,.1384718E-03,.1288147E-03,&
     & .1155429E-03,.9784118E-04,.7511447E-04,.5542102E-04,.1317506E-03,&
     & .1530088E-03,.1541902E-03,.1487423E-03,.1390011E-03,.1250725E-03,&
     & .1060741E-03,.8202844E-04,.6380787E-04,.1377366E-03,.1623281E-03,&
     & .1647109E-03,.1596366E-03,.1497584E-03,.1350689E-03,.1149756E-03,&
     & .8862399E-04,.7101181E-04,.1440326E-03,.1724497E-03,.1760537E-03,&
     & .1714078E-03,.1610900E-03,.1456787E-03,.1244204E-03,.9651500E-04,&
     & .7800418E-04,.1507641E-03,.1835052E-03,.1883108E-03,.1838011E-03,&
     & .1732376E-03,.1570042E-03,.1346136E-03,.1051140E-03,.8602649E-04,&
     & .1072501E-03,.1233653E-03,.1231303E-03,.1180489E-03,.1097712E-03,&
     & .9835244E-04,.8317681E-04,.6399482E-04,.4621221E-04,.1121185E-03,&
     & .1307584E-03,.1315510E-03,.1267530E-03,.1184196E-03,.1064469E-03,&
     & .9028401E-04,.6947188E-04,.5321306E-04,.1172881E-03,.1387038E-03,&
     & .1405630E-03,.1362742E-03,.1276910E-03,.1151082E-03,.9795080E-04,&
     & .7537889E-04,.5873871E-04,.1228106E-03,.1473997E-03,.1504434E-03,&
     & .1464443E-03,.1375481E-03,.1243899E-03,.1061458E-03,.8236870E-04,&
     & .6477734E-04,.1289014E-03,.1570159E-03,.1610714E-03,.1572623E-03,&
     & .1482077E-03,.1342864E-03,.1150946E-03,.8978401E-04,.7158089E-04/

      data   absa(  1:180, 6)   /                                       &
     & .3618426E-02,.3641770E-02,.3630477E-02,.3560259E-02,.3416323E-02,&
     & .3176047E-02,.2830864E-02,.2373983E-02,.1742217E-02,.3598081E-02,&
     & .3703258E-02,.3743793E-02,.3712218E-02,.3589778E-02,.3370737E-02,&
     & .3037900E-02,.2596162E-02,.1991398E-02,.3584083E-02,.3774324E-02,&
     & .3866788E-02,.3867624E-02,.3769761E-02,.3570448E-02,.3255037E-02,&
     & .2830185E-02,.2261505E-02,.3577298E-02,.3852926E-02,.3993744E-02,&
     & .4026505E-02,.3954894E-02,.3775808E-02,.3481871E-02,.3078695E-02,&
     & .2554303E-02,.3579835E-02,.3939097E-02,.4125092E-02,.4189431E-02,&
     & .4145315E-02,.3990234E-02,.3720434E-02,.3340287E-02,.2865995E-02,&
     & .3248394E-02,.3297351E-02,.3295356E-02,.3225177E-02,.3084542E-02,&
     & .2861202E-02,.2553948E-02,.2123300E-02,.1461572E-02,.3235583E-02,&
     & .3361291E-02,.3404532E-02,.3369041E-02,.3247031E-02,.3043618E-02,&
     & .2745319E-02,.2323230E-02,.1677107E-02,.3231488E-02,.3436144E-02,&
     & .3523056E-02,.3516391E-02,.3416868E-02,.3231131E-02,.2944369E-02,&
     & .2533343E-02,.1912294E-02,.3237872E-02,.3517820E-02,.3646256E-02,&
     & .3668618E-02,.3593949E-02,.3426895E-02,.3151603E-02,.2756947E-02,&
     & .2166557E-02,.3253132E-02,.3606661E-02,.3773434E-02,.3826500E-02,&
     & .3776601E-02,.3630583E-02,.3369967E-02,.2994163E-02,.2437904E-02,&
     & .2886205E-02,.2942447E-02,.2941032E-02,.2863367E-02,.2725094E-02,&
     & .2518760E-02,.2241939E-02,.1855734E-02,.1188420E-02,.2880736E-02,&
     & .3006642E-02,.3042538E-02,.2997000E-02,.2874870E-02,.2685370E-02,&
     & .2415558E-02,.2030078E-02,.1369150E-02,.2886508E-02,.3082007E-02,&
     & .3154756E-02,.3134344E-02,.3033232E-02,.2858755E-02,.2598268E-02,&
     & .2216396E-02,.1566621E-02,.2902943E-02,.3166339E-02,.3272849E-02,&
     & .3277933E-02,.3197754E-02,.3039090E-02,.2789791E-02,.2414716E-02,&
     & .1779400E-02,.2928623E-02,.3256730E-02,.3396972E-02,.3427808E-02,&
     & .3370428E-02,.3228051E-02,.2990498E-02,.2624579E-02,.2010439E-02,&
     & .2540902E-02,.2597254E-02,.2589360E-02,.2508382E-02,.2375438E-02,&
     & .2188928E-02,.1938803E-02,.1600312E-02,.9604968E-03,.2542011E-02,&
     & .2659586E-02,.2684388E-02,.2628952E-02,.2511349E-02,.2338234E-02,&
     & .2094432E-02,.1754453E-02,.1111332E-02,.2555832E-02,.2734832E-02,&
     & .2789039E-02,.2755882E-02,.2656417E-02,.2495319E-02,.2258255E-02,&
     & .1919151E-02,.1276371E-02,.2581259E-02,.2819185E-02,.2901637E-02,&
     & .2889786E-02,.2808889E-02,.2659442E-02,.2431576E-02,.2093659E-02,&
     & .1455003E-02,.2614546E-02,.2911442E-02,.3020621E-02,.3031206E-02,&
     & .2967469E-02,.2832291E-02,.2614781E-02,.2279966E-02,.1647719E-02/

      data   absa(181:315, 6)   /                                       &
     & .2219972E-02,.2275081E-02,.2258189E-02,.2178017E-02,.2052724E-02,&
     & .1883431E-02,.1661334E-02,.1366404E-02,.7744816E-03,.2227260E-02,&
     & .2335504E-02,.2345620E-02,.2284987E-02,.2174042E-02,.2016070E-02,&
     & .1799386E-02,.1503145E-02,.9005459E-03,.2248029E-02,.2409026E-02,&
     & .2443618E-02,.2400559E-02,.2304822E-02,.2156227E-02,.1945494E-02,&
     & .1648525E-02,.1039558E-02,.2279277E-02,.2492598E-02,.2549202E-02,&
     & .2524778E-02,.2443050E-02,.2304879E-02,.2101498E-02,.1803888E-02,&
     & .1189595E-02,.2314653E-02,.2584274E-02,.2663118E-02,.2656729E-02,&
     & .2589557E-02,.2462949E-02,.2266082E-02,.1970673E-02,.1351586E-02,&
     & .1926874E-02,.1977274E-02,.1952607E-02,.1874742E-02,.1757624E-02,&
     & .1604309E-02,.1409103E-02,.1152190E-02,.6162171E-03,.1939310E-02,&
     & .2034841E-02,.2032252E-02,.1970439E-02,.1864280E-02,.1719652E-02,&
     & .1530483E-02,.1272154E-02,.7211166E-03,.1964743E-02,.2105464E-02,&
     & .2122862E-02,.2074082E-02,.1979741E-02,.1844073E-02,.1659546E-02,&
     & .1399922E-02,.8374058E-03,.1996506E-02,.2186938E-02,.2222376E-02,&
     & .2187288E-02,.2103966E-02,.1977177E-02,.1797677E-02,.1537669E-02,&
     & .9633722E-03,.2032972E-02,.2273283E-02,.2330136E-02,.2309382E-02,&
     & .2237782E-02,.2120007E-02,.1943914E-02,.1683301E-02,.1099973E-02,&
     & .1663865E-02,.1707960E-02,.1678656E-02,.1605070E-02,.1496908E-02,&
     & .1359728E-02,.1186597E-02,.9637506E-03,.4837199E-03,.1679462E-02,&
     & .1762213E-02,.1749710E-02,.1688780E-02,.1589709E-02,.1459287E-02,&
     & .1291495E-02,.1067463E-02,.5692182E-03,.1705626E-02,.1829062E-02,&
     & .1833331E-02,.1782521E-02,.1691254E-02,.1568704E-02,.1405411E-02,&
     & .1178417E-02,.6634981E-03,.1737149E-02,.1904246E-02,.1926414E-02,&
     & .1885283E-02,.1802747E-02,.1686750E-02,.1526571E-02,.1298759E-02,&
     & .7673774E-03,.1774180E-02,.1984243E-02,.2025542E-02,.1997561E-02,&
     & .1924977E-02,.1813742E-02,.1655518E-02,.1426328E-02,.8813749E-03/

      data   absa(316:450, 6)   /                                       &
     & .1431715E-02,.1468745E-02,.1437010E-02,.1368212E-02,.1270165E-02,&
     & .1147038E-02,.9947349E-03,.8008574E-03,.3951453E-03,.1448558E-02,&
     & .1518890E-02,.1500478E-02,.1441411E-02,.1351002E-02,.1233436E-02,&
     & .1085630E-02,.8898114E-03,.4614759E-03,.1473293E-02,.1580156E-02,&
     & .1576676E-02,.1525332E-02,.1440556E-02,.1328775E-02,.1183815E-02,&
     & .9858304E-03,.5346545E-03,.1503819E-02,.1648007E-02,.1660678E-02,&
     & .1618866E-02,.1540977E-02,.1433665E-02,.1289444E-02,.1090469E-02,&
     & .6144704E-03,.1540870E-02,.1721704E-02,.1750156E-02,.1719303E-02,&
     & .1651341E-02,.1546906E-02,.1402428E-02,.1200202E-02,.7022918E-03,&
     & .1229566E-02,.1259757E-02,.1226783E-02,.1162063E-02,.1073623E-02,&
     & .9638267E-03,.8304962E-03,.6619654E-03,.3708964E-03,.1245677E-02,&
     & .1304848E-02,.1282510E-02,.1226105E-02,.1143293E-02,.1038470E-02,&
     & .9077754E-03,.7376031E-03,.4310410E-03,.1268666E-02,.1359022E-02,&
     & .1349687E-02,.1300919E-02,.1222896E-02,.1121551E-02,.9925926E-03,&
     & .8196689E-03,.4969197E-03,.1297981E-02,.1419642E-02,.1423650E-02,&
     & .1383567E-02,.1312384E-02,.1213886E-02,.1083936E-02,.9092041E-03,&
     & .5687180E-03,.1333382E-02,.1486555E-02,.1503607E-02,.1472905E-02,&
     & .1409828E-02,.1314124E-02,.1183141E-02,.1003566E-02,.6475735E-03,&
     & .1053427E-02,.1080875E-02,.1049457E-02,.9889791E-03,.9098076E-03,&
     & .8127675E-03,.6957123E-03,.5493509E-03,.3851070E-03,.1068299E-02,&
     & .1121177E-02,.1098614E-02,.1045290E-02,.9707095E-03,.8766135E-03,&
     & .7618622E-03,.6136871E-03,.4400535E-03,.1090018E-02,.1169156E-02,&
     & .1157141E-02,.1111356E-02,.1041184E-02,.9497327E-03,.8348360E-03,&
     & .6839361E-03,.4994168E-03,.1118137E-02,.1223841E-02,.1222357E-02,&
     & .1184095E-02,.1119621E-02,.1031067E-02,.9147253E-03,.7606201E-03,&
     & .5637416E-03,.1150470E-02,.1284547E-02,.1293867E-02,.1263779E-02,&
     & .1204832E-02,.1118950E-02,.1001368E-02,.8420516E-03,.6348671E-03/

      data   absa(451:585, 6)   /                                       &
     & .9046486E-03,.9397347E-03,.9133460E-03,.8617363E-03,.7934190E-03,&
     & .7090706E-03,.6066044E-03,.4794255E-03,.3743553E-03,.9212914E-03,&
     & .9782734E-03,.9602049E-03,.9151820E-03,.8507753E-03,.7678173E-03,&
     & .6660920E-03,.5356152E-03,.4226059E-03,.9442639E-03,.1023620E-02,&
     & .1014658E-02,.9758938E-03,.9151114E-03,.8345845E-03,.7318442E-03,&
     & .5984128E-03,.4774443E-03,.9713828E-03,.1075284E-02,.1075663E-02,&
     & .1042999E-02,.9862816E-03,.9076184E-03,.8041084E-03,.6658818E-03,&
     & .5353482E-03,.1002245E-02,.1131170E-02,.1142136E-02,.1116667E-02,&
     & .1063668E-02,.9860813E-03,.8820903E-03,.7392501E-03,.5986885E-03,&
     & .7757001E-03,.8156004E-03,.7935608E-03,.7501868E-03,.6919875E-03,&
     & .6187418E-03,.5293689E-03,.4171913E-03,.3400983E-03,.7936331E-03,&
     & .8519754E-03,.8374063E-03,.7995658E-03,.7443953E-03,.6725458E-03,&
     & .5825365E-03,.4667964E-03,.3809920E-03,.8158134E-03,.8946178E-03,&
     & .8881924E-03,.8553581E-03,.8027948E-03,.7321369E-03,.6414490E-03,&
     & .5225120E-03,.4287453E-03,.8413662E-03,.9418592E-03,.9443588E-03,&
     & .9171257E-03,.8676347E-03,.7974095E-03,.7054434E-03,.5826005E-03,&
     & .4804212E-03,.8686293E-03,.9928428E-03,.1005484E-02,.9842193E-03,&
     & .9373691E-03,.8680623E-03,.7750971E-03,.6486142E-03,.5361541E-03,&
     & .6656671E-03,.7063220E-03,.6882893E-03,.6520290E-03,.6021008E-03,&
     & .5393669E-03,.4616943E-03,.3624013E-03,.2876110E-03,.6827709E-03,&
     & .7403802E-03,.7293362E-03,.6972791E-03,.6496824E-03,.5872587E-03,&
     & .5091135E-03,.4069669E-03,.3235500E-03,.7030563E-03,.7795754E-03,&
     & .7759040E-03,.7481102E-03,.7027456E-03,.6406035E-03,.5612231E-03,&
     & .4560596E-03,.3644783E-03,.7249585E-03,.8222986E-03,.8269765E-03,&
     & .8039916E-03,.7610904E-03,.6992438E-03,.6179241E-03,.5093407E-03,&
     & .4084015E-03,.7487037E-03,.8678454E-03,.8821100E-03,.8646953E-03,&
     & .8237630E-03,.7629048E-03,.6800190E-03,.5680363E-03,.4564484E-03/

      data   absa(  1:180, 7)   /                                       &
     & .1944086E-01,.1794384E-01,.1665104E-01,.1551773E-01,.1445828E-01,&
     & .1338923E-01,.1239141E-01,.1193783E-01,.1267749E-01,.1929939E-01,&
     & .1802401E-01,.1696529E-01,.1609074E-01,.1524109E-01,.1436017E-01,&
     & .1365895E-01,.1355010E-01,.1460849E-01,.1914240E-01,.1812275E-01,&
     & .1733603E-01,.1672488E-01,.1609343E-01,.1543656E-01,.1506976E-01,&
     & .1532896E-01,.1671755E-01,.1896990E-01,.1824026E-01,.1775719E-01,&
     & .1740823E-01,.1702225E-01,.1660281E-01,.1659697E-01,.1723872E-01,&
     & .1897711E-01,.1878230E-01,.1836854E-01,.1822350E-01,.1813959E-01,&
     & .1798727E-01,.1786818E-01,.1824983E-01,.1930509E-01,.2141343E-01,&
     & .2002304E-01,.1846205E-01,.1712703E-01,.1593023E-01,.1477768E-01,&
     & .1360259E-01,.1233974E-01,.1147779E-01,.1184062E-01,.1987095E-01,&
     & .1854946E-01,.1743597E-01,.1649501E-01,.1555867E-01,.1455749E-01,&
     & .1351955E-01,.1298854E-01,.1367056E-01,.1970539E-01,.1864562E-01,&
     & .1779988E-01,.1712573E-01,.1640112E-01,.1560634E-01,.1483953E-01,&
     & .1466071E-01,.1566922E-01,.1952211E-01,.1876024E-01,.1821996E-01,&
     & .1780079E-01,.1732241E-01,.1670897E-01,.1627296E-01,.1646775E-01,&
     & .1781807E-01,.1932576E-01,.1888733E-01,.1868624E-01,.1852437E-01,&
     & .1828787E-01,.1789646E-01,.1782466E-01,.1840912E-01,.2012300E-01,&
     & .2046316E-01,.1881445E-01,.1742883E-01,.1614035E-01,.1486365E-01,&
     & .1356768E-01,.1209778E-01,.1077331E-01,.1065343E-01,.2030717E-01,&
     & .1890498E-01,.1772206E-01,.1668034E-01,.1562712E-01,.1448379E-01,&
     & .1318643E-01,.1216891E-01,.1235738E-01,.2013274E-01,.1900626E-01,&
     & .1806511E-01,.1729725E-01,.1644190E-01,.1549571E-01,.1438909E-01,&
     & .1370013E-01,.1420354E-01,.1994378E-01,.1911368E-01,.1847290E-01,&
     & .1795539E-01,.1733637E-01,.1657610E-01,.1569548E-01,.1535991E-01,&
     & .1619433E-01,.1974273E-01,.1923315E-01,.1892487E-01,.1866106E-01,&
     & .1828690E-01,.1769725E-01,.1711524E-01,.1715953E-01,.1834492E-01,&
     & .2071926E-01,.1899869E-01,.1758640E-01,.1621993E-01,.1481733E-01,&
     & .1340442E-01,.1178457E-01,.1005441E-01,.9415201E-02,.2056070E-01,&
     & .1908468E-01,.1786585E-01,.1672760E-01,.1555529E-01,.1427366E-01,&
     & .1280976E-01,.1132568E-01,.1098244E-01,.2038159E-01,.1918328E-01,&
     & .1818981E-01,.1732353E-01,.1633951E-01,.1523974E-01,.1393712E-01,&
     & .1273530E-01,.1270208E-01,.2019166E-01,.1930154E-01,.1857522E-01,&
     & .1795884E-01,.1719737E-01,.1628902E-01,.1513446E-01,.1425698E-01,&
     & .1454259E-01,.1999244E-01,.1941900E-01,.1900880E-01,.1863941E-01,&
     & .1812618E-01,.1738164E-01,.1642114E-01,.1589227E-01,.1651351E-01/

      data   absa(181:315, 7)   /                                       &
     & .2078596E-01,.1902221E-01,.1759185E-01,.1618479E-01,.1468286E-01,&
     & .1316356E-01,.1141915E-01,.9423027E-02,.8269424E-02,.2061989E-01,&
     & .1909317E-01,.1787631E-01,.1666093E-01,.1538845E-01,.1399494E-01,&
     & .1238597E-01,.1057539E-01,.9705846E-02,.2044247E-01,.1918749E-01,&
     & .1817839E-01,.1723098E-01,.1614137E-01,.1490895E-01,.1346159E-01,&
     & .1185395E-01,.1127991E-01,.2025356E-01,.1930375E-01,.1854460E-01,&
     & .1784554E-01,.1696147E-01,.1591307E-01,.1460469E-01,.1324520E-01,&
     & .1297427E-01,.2007410E-01,.1943195E-01,.1896380E-01,.1849843E-01,&
     & .1785367E-01,.1697392E-01,.1579435E-01,.1474526E-01,.1480343E-01,&
     & .2067653E-01,.1889520E-01,.1744869E-01,.1600835E-01,.1443542E-01,&
     & .1280811E-01,.1096768E-01,.8811489E-02,.7161785E-02,.2051191E-01,&
     & .1895371E-01,.1772735E-01,.1645848E-01,.1509553E-01,.1360134E-01,&
     & .1187179E-01,.9849619E-02,.8463973E-02,.2033498E-01,.1904213E-01,&
     & .1803233E-01,.1698898E-01,.1581982E-01,.1446262E-01,.1288709E-01,&
     & .1101445E-01,.9901719E-02,.2016531E-01,.1915657E-01,.1837300E-01,&
     & .1758375E-01,.1659315E-01,.1541743E-01,.1398290E-01,.1227676E-01,&
     & .1145224E-01,.2000011E-01,.1929718E-01,.1877422E-01,.1820930E-01,&
     & .1743917E-01,.1643489E-01,.1512599E-01,.1364249E-01,.1312614E-01,&
     & .2038344E-01,.1861656E-01,.1717360E-01,.1569740E-01,.1410542E-01,&
     & .1238154E-01,.1046720E-01,.8234826E-02,.6157987E-02,.2021852E-01,&
     & .1866243E-01,.1743952E-01,.1614631E-01,.1471423E-01,.1312828E-01,&
     & .1131255E-01,.9181663E-02,.7325752E-02,.2004782E-01,.1874347E-01,&
     & .1774105E-01,.1664594E-01,.1539580E-01,.1393665E-01,.1225989E-01,&
     & .1024134E-01,.8627463E-02,.1988902E-01,.1886277E-01,.1808084E-01,&
     & .1721183E-01,.1612179E-01,.1483288E-01,.1330233E-01,.1139021E-01,&
     & .1003993E-01,.1973638E-01,.1901550E-01,.1847625E-01,.1780825E-01,&
     & .1691910E-01,.1580549E-01,.1439570E-01,.1263517E-01,.1157007E-01/

      data   absa(316:450, 7)   /                                       &
     & .1987279E-01,.1815516E-01,.1673357E-01,.1526588E-01,.1367570E-01,&
     & .1190873E-01,.9931491E-02,.7668521E-02,.5208898E-02,.1970849E-01,&
     & .1819142E-01,.1699015E-01,.1570273E-01,.1425525E-01,.1259893E-01,&
     & .1071895E-01,.8538062E-02,.6255037E-02,.1954825E-01,.1826821E-01,&
     & .1728456E-01,.1619427E-01,.1489632E-01,.1335233E-01,.1159816E-01,&
     & .9521153E-02,.7436933E-02,.1939994E-01,.1839394E-01,.1763047E-01,&
     & .1674380E-01,.1557766E-01,.1418956E-01,.1258113E-01,.1057697E-01,&
     & .8728022E-02,.1926214E-01,.1855724E-01,.1803115E-01,.1732234E-01,&
     & .1632534E-01,.1511381E-01,.1362321E-01,.1171409E-01,.1013851E-01,&
     & .1916880E-01,.1752174E-01,.1613409E-01,.1469612E-01,.1312851E-01,&
     & .1136007E-01,.9364186E-02,.7102147E-02,.4213687E-02,.1900554E-01,&
     & .1754861E-01,.1637886E-01,.1511063E-01,.1368165E-01,.1201315E-01,&
     & .1009580E-01,.7893303E-02,.5110717E-02,.1885316E-01,.1762136E-01,&
     & .1666679E-01,.1558750E-01,.1430162E-01,.1272233E-01,.1090848E-01,&
     & .8800048E-02,.6141449E-02,.1871378E-01,.1775166E-01,.1701481E-01,&
     & .1613266E-01,.1495234E-01,.1350195E-01,.1182791E-01,.9791825E-02,&
     & .7281500E-02,.1858911E-01,.1792329E-01,.1741640E-01,.1670595E-01,&
     & .1566347E-01,.1437003E-01,.1281383E-01,.1084750E-01,.8535050E-02,&
     & .1826944E-01,.1672464E-01,.1541276E-01,.1403848E-01,.1252666E-01,&
     & .1078514E-01,.8823245E-02,.6591625E-02,.4177613E-02,.1810805E-01,&
     & .1674909E-01,.1564423E-01,.1443382E-01,.1304370E-01,.1140669E-01,&
     & .9515283E-02,.7318294E-02,.4841751E-02,.1796423E-01,.1682298E-01,&
     & .1593052E-01,.1489393E-01,.1363432E-01,.1209430E-01,.1027133E-01,&
     & .8153051E-02,.5674979E-02,.1783274E-01,.1695370E-01,.1627489E-01,&
     & .1542379E-01,.1427771E-01,.1283692E-01,.1112848E-01,.9081882E-02,&
     & .6511275E-02,.1772600E-01,.1713577E-01,.1668031E-01,.1599272E-01,&
     & .1497596E-01,.1366160E-01,.1205854E-01,.1007221E-01,.7398941E-02/

      data   absa(451:585, 7)   /                                       &
     & .1712738E-01,.1578625E-01,.1465104E-01,.1343124E-01,.1204735E-01,&
     & .1040471E-01,.8547650E-02,.6380408E-02,.4682263E-02,.1697749E-01,&
     & .1582950E-01,.1490046E-01,.1383672E-01,.1256494E-01,.1102596E-01,&
     & .9230857E-02,.7091836E-02,.5424321E-02,.1684963E-01,.1592973E-01,&
     & .1520871E-01,.1431627E-01,.1316095E-01,.1170953E-01,.9981803E-02,&
     & .7904548E-02,.6291977E-02,.1674859E-01,.1609441E-01,.1557937E-01,&
     & .1485696E-01,.1380508E-01,.1246505E-01,.1081864E-01,.8796469E-02,&
     & .7234547E-02,.1667150E-01,.1630711E-01,.1601588E-01,.1543527E-01,&
     & .1451902E-01,.1330051E-01,.1171274E-01,.9750545E-02,.8163171E-02,&
     & .1585812E-01,.1473647E-01,.1378249E-01,.1272207E-01,.1146005E-01,&
     & .9946309E-02,.8216259E-02,.6139489E-02,.4795601E-02,.1573042E-01,&
     & .1480849E-01,.1405564E-01,.1313767E-01,.1198402E-01,.1056976E-01,&
     & .8883654E-02,.6832310E-02,.5569244E-02,.1562948E-01,.1493848E-01,&
     & .1438406E-01,.1362526E-01,.1258929E-01,.1125212E-01,.9629528E-02,&
     & .7616188E-02,.6452525E-02,.1556093E-01,.1513902E-01,.1478476E-01,&
     & .1418150E-01,.1324034E-01,.1201861E-01,.1045886E-01,.8467010E-02,&
     & .7448495E-02,.1552707E-01,.1538861E-01,.1525660E-01,.1477573E-01,&
     & .1397485E-01,.1285192E-01,.1132600E-01,.9383049E-02,.8380165E-02,&
     & .1451815E-01,.1362323E-01,.1284326E-01,.1193288E-01,.1079141E-01,&
     & .9422285E-02,.7828749E-02,.5876279E-02,.4564233E-02,.1441454E-01,&
     & .1372037E-01,.1313620E-01,.1235490E-01,.1132640E-01,.1004968E-01,&
     & .8482915E-02,.6542489E-02,.5277165E-02,.1434814E-01,.1388758E-01,&
     & .1349254E-01,.1286003E-01,.1193607E-01,.1073461E-01,.9223888E-02,&
     & .7293736E-02,.6181241E-02,.1432181E-01,.1412637E-01,.1392568E-01,&
     & .1342872E-01,.1260305E-01,.1150145E-01,.1004198E-01,.8104002E-02,&
     & .7114973E-02,.1432961E-01,.1441453E-01,.1442859E-01,.1404464E-01,&
     & .1335348E-01,.1232881E-01,.1089297E-01,.8984991E-02,.8052181E-02/

      data   absa(  1:180, 8)   /                                       &
     & .7758517E-01,.6971744E-01,.6856234E-01,.7085104E-01,.7686968E-01,&
     & .8798079E-01,.1038826E+00,.1198350E+00,.1359548E+00,.7760111E-01,&
     & .7121281E-01,.7343034E-01,.7934822E-01,.9017738E-01,.1066736E+00,&
     & .1269235E+00,.1465907E+00,.1664556E+00,.7752743E-01,.7302330E-01,&
     & .7902576E-01,.8954413E-01,.1059545E+00,.1282979E+00,.1531287E+00,&
     & .1770168E+00,.2010905E+00,.7732969E-01,.7518701E-01,.8549302E-01,&
     & .1015046E+00,.1240275E+00,.1528286E+00,.1826322E+00,.2112286E+00,&
     & .2400654E+00,.7704248E-01,.7773694E-01,.9279924E-01,.1151544E+00,&
     & .1447190E+00,.1798906E+00,.2150779E+00,.2488116E+00,.2828065E+00,&
     & .8996337E-01,.8019925E-01,.7649483E-01,.7619075E-01,.7950947E-01,&
     & .8744378E-01,.1013048E+00,.1167488E+00,.1333495E+00,.9002593E-01,&
     & .8150228E-01,.8115314E-01,.8427387E-01,.9208357E-01,.1052256E+00,&
     & .1239442E+00,.1430385E+00,.1634839E+00,.8993043E-01,.8319207E-01,&
     & .8655274E-01,.9399363E-01,.1071792E+00,.1260258E+00,.1498926E+00,&
     & .1732190E+00,.1980122E+00,.8971818E-01,.8524906E-01,.9277644E-01,&
     & .1055693E+00,.1245408E+00,.1502414E+00,.1793335E+00,.2072943E+00,&
     & .2370216E+00,.8937589E-01,.8770760E-01,.9989309E-01,.1188472E+00,&
     & .1444723E+00,.1773019E+00,.2118836E+00,.2451113E+00,.2803148E+00,&
     & .1035251E+00,.9170912E-01,.8469424E-01,.8124545E-01,.8127330E-01,&
     & .8520134E-01,.9536063E-01,.1094892E+00,.1259212E+00,.1036314E+00,&
     & .9276163E-01,.8901665E-01,.8866608E-01,.9260241E-01,.1015125E+00,&
     & .1166863E+00,.1344219E+00,.1546172E+00,.1035650E+00,.9415806E-01,&
     & .9411336E-01,.9760915E-01,.1066039E+00,.1208833E+00,.1416737E+00,&
     & .1635528E+00,.1881599E+00,.1033569E+00,.9604690E-01,.9993466E-01,&
     & .1084464E+00,.1229492E+00,.1435592E+00,.1703234E+00,.1968385E+00,&
     & .2265370E+00,.1029798E+00,.9835979E-01,.1066697E+00,.1210700E+00,&
     & .1417416E+00,.1696560E+00,.2023878E+00,.2340132E+00,.2693038E+00,&
     & .1182256E+00,.1042918E+00,.9385612E-01,.8708706E-01,.8392123E-01,&
     & .8386827E-01,.8993733E-01,.1021139E+00,.1178700E+00,.1183624E+00,&
     & .1051802E+00,.9770525E-01,.9387938E-01,.9397963E-01,.9866810E-01,&
     & .1093764E+00,.1253639E+00,.1447078E+00,.1183448E+00,.1063559E+00,&
     & .1024326E+00,.1020095E+00,.1067911E+00,.1165316E+00,.1326917E+00,&
     & .1528199E+00,.1764343E+00,.1181015E+00,.1078331E+00,.1078798E+00,&
     & .1120521E+00,.1219908E+00,.1374912E+00,.1601454E+00,.1848559E+00,&
     & .2134132E+00,.1177256E+00,.1098961E+00,.1142273E+00,.1239094E+00,&
     & .1395842E+00,.1620158E+00,.1913771E+00,.2212322E+00,.2554318E+00/

      data   absa(181:315, 8)   /                                       &
     & .1337645E+00,.1176975E+00,.1042174E+00,.9408829E-01,.8780506E-01,&
     & .8428291E-01,.8648117E-01,.9578625E-01,.1107310E+00,.1340095E+00,&
     & .1185612E+00,.1073992E+00,.1002667E+00,.9676787E-01,.9754447E-01,&
     & .1041226E+00,.1173533E+00,.1356505E+00,.1339933E+00,.1195717E+00,&
     & .1117737E+00,.1076768E+00,.1084293E+00,.1139755E+00,.1254459E+00,&
     & .1433024E+00,.1656569E+00,.1337898E+00,.1208507E+00,.1168604E+00,&
     & .1168941E+00,.1225882E+00,.1334602E+00,.1509971E+00,.1738575E+00,&
     & .2010332E+00,.1333402E+00,.1225783E+00,.1227702E+00,.1280534E+00,&
     & .1390939E+00,.1563754E+00,.1810270E+00,.2089873E+00,.2415778E+00,&
     & .1499294E+00,.1316511E+00,.1153149E+00,.1019770E+00,.9253012E-01,&
     & .8589064E-01,.8430058E-01,.8998453E-01,.1039889E+00,.1502123E+00,&
     & .1324781E+00,.1179942E+00,.1074361E+00,.1005474E+00,.9758469E-01,&
     & .1000820E+00,.1097714E+00,.1269049E+00,.1502509E+00,.1333737E+00,&
     & .1216948E+00,.1142169E+00,.1109667E+00,.1125131E+00,.1194607E+00,&
     & .1338501E+00,.1548471E+00,.1500164E+00,.1344800E+00,.1264420E+00,&
     & .1225692E+00,.1240810E+00,.1304661E+00,.1429117E+00,.1627629E+00,&
     & .1883115E+00,.1495747E+00,.1359816E+00,.1319394E+00,.1329504E+00,&
     & .1395630E+00,.1517775E+00,.1709091E+00,.1964203E+00,.2271878E+00,&
     & .1667524E+00,.1461966E+00,.1270922E+00,.1109576E+00,.9817555E-01,&
     & .8876777E-01,.8374701E-01,.8543513E-01,.9818294E-01,.1671368E+00,&
     & .1470313E+00,.1293919E+00,.1154650E+00,.1053506E+00,.9907389E-01,&
     & .9777459E-01,.1033433E+00,.1192365E+00,.1672667E+00,.1478601E+00,&
     & .1325480E+00,.1215247E+00,.1147604E+00,.1125732E+00,.1153357E+00,&
     & .1255724E+00,.1452073E+00,.1670804E+00,.1488123E+00,.1366626E+00,&
     & .1291483E+00,.1268401E+00,.1291153E+00,.1368015E+00,.1526760E+00,&
     & .1766400E+00,.1666531E+00,.1500830E+00,.1416654E+00,.1387987E+00,&
     & .1413161E+00,.1488921E+00,.1627763E+00,.1846442E+00,.2135966E+00/

      data   absa(316:450, 8)   /                                       &
     & .1846850E+00,.1617527E+00,.1399797E+00,.1208986E+00,.1049700E+00,&
     & .9268890E-01,.8468015E-01,.8250239E-01,.9346009E-01,.1852004E+00,&
     & .1626022E+00,.1419177E+00,.1246679E+00,.1111616E+00,.1018673E+00,&
     & .9711144E-01,.9858998E-01,.1128165E+00,.1854002E+00,.1633815E+00,&
     & .1446073E+00,.1298548E+00,.1195851E+00,.1140911E+00,.1130781E+00,&
     & .1187616E+00,.1368729E+00,.1852584E+00,.1641980E+00,.1481385E+00,&
     & .1365970E+00,.1306723E+00,.1293064E+00,.1327425E+00,.1439504E+00,&
     & .1663864E+00,.1848096E+00,.1652487E+00,.1525559E+00,.1454722E+00,&
     & .1442002E+00,.1476584E+00,.1567932E+00,.1741988E+00,.2014170E+00,&
     & .2035660E+00,.1782138E+00,.1538641E+00,.1319292E+00,.1129603E+00,&
     & .9769919E-01,.8663861E-01,.8077073E-01,.8932903E-01,.2042261E+00,&
     & .1790867E+00,.1555132E+00,.1351430E+00,.1182459E+00,.1056510E+00,&
     & .9764716E-01,.9524493E-01,.1071831E+00,.2045296E+00,.1798786E+00,&
     & .1578247E+00,.1395652E+00,.1254969E+00,.1165399E+00,.1121007E+00,&
     & .1135498E+00,.1294885E+00,.2044541E+00,.1805616E+00,.1608548E+00,&
     & .1453947E+00,.1354455E+00,.1305073E+00,.1300663E+00,.1365302E+00,&
     & .1570562E+00,.2040215E+00,.1814068E+00,.1647421E+00,.1533244E+00,&
     & .1479662E+00,.1475364E+00,.1523061E+00,.1647332E+00,.1901427E+00,&
     & .2235492E+00,.1956768E+00,.1686716E+00,.1440048E+00,.1221516E+00,&
     & .1041925E+00,.9012153E-01,.8095605E-01,.7543929E-01,.2243439E+00,&
     & .1965703E+00,.1701921E+00,.1468360E+00,.1269493E+00,.1112681E+00,&
     & .9999583E-01,.9424336E-01,.9351393E-01,.2246860E+00,.1973265E+00,&
     & .1722057E+00,.1507546E+00,.1334737E+00,.1209496E+00,.1133134E+00,&
     & .1112100E+00,.1153114E+00,.2246584E+00,.1979469E+00,.1749030E+00,&
     & .1560431E+00,.1423719E+00,.1337802E+00,.1300644E+00,.1325688E+00,&
     & .1435519E+00,.2242020E+00,.1986013E+00,.1783455E+00,.1632177E+00,&
     & .1538706E+00,.1497467E+00,.1509387E+00,.1590812E+00,.1779531E+00/

      data   absa(451:585, 8)   /                                       &
     & .2450204E+00,.2144676E+00,.1850038E+00,.1579688E+00,.1339651E+00,&
     & .1141008E+00,.9793249E-01,.8695958E-01,.6939988E-01,.2456964E+00,&
     & .2153041E+00,.1865121E+00,.1608968E+00,.1389868E+00,.1214503E+00,&
     & .1082108E+00,.1007628E+00,.8668012E-01,.2459004E+00,.2159604E+00,&
     & .1885239E+00,.1648976E+00,.1457743E+00,.1314989E+00,.1219175E+00,&
     & .1183238E+00,.1082310E+00,.2456185E+00,.2163907E+00,.1911660E+00,&
     & .1703594E+00,.1550454E+00,.1445551E+00,.1392768E+00,.1404833E+00,&
     & .1353141E+00,.2449307E+00,.2169326E+00,.1945158E+00,.1777645E+00,&
     & .1667152E+00,.1608041E+00,.1609850E+00,.1678148E+00,.1694632E+00,&
     & .2671603E+00,.2338498E+00,.2019245E+00,.1725008E+00,.1464684E+00,&
     & .1244209E+00,.1059693E+00,.9304783E-01,.6959000E-01,.2676057E+00,&
     & .2345222E+00,.2033654E+00,.1755662E+00,.1517012E+00,.1320205E+00,&
     & .1167556E+00,.1073595E+00,.8673211E-01,.2675630E+00,.2350038E+00,&
     & .2053552E+00,.1797362E+00,.1586943E+00,.1424342E+00,.1308446E+00,&
     & .1256134E+00,.1083518E+00,.2670071E+00,.2352413E+00,.2078987E+00,&
     & .1853108E+00,.1681964E+00,.1557131E+00,.1486669E+00,.1486318E+00,&
     & .1350793E+00,.2659779E+00,.2356017E+00,.2111172E+00,.1928578E+00,&
     & .1799895E+00,.1723719E+00,.1711124E+00,.1767808E+00,.1695167E+00,&
     & .2893519E+00,.2532903E+00,.2190193E+00,.1873805E+00,.1594254E+00,&
     & .1350214E+00,.1142355E+00,.9903594E-01,.7490192E-01,.2895758E+00,&
     & .2538124E+00,.2204052E+00,.1906072E+00,.1647749E+00,.1428610E+00,&
     & .1254558E+00,.1139403E+00,.9307717E-01,.2892263E+00,.2540433E+00,&
     & .2223015E+00,.1948310E+00,.1719923E+00,.1535461E+00,.1399201E+00,&
     & .1328863E+00,.1147608E+00,.2883124E+00,.2540521E+00,.2247025E+00,&
     & .2005493E+00,.1816141E+00,.1671436E+00,.1582248E+00,.1567001E+00,&
     & .1426012E+00,.2869377E+00,.2542235E+00,.2278499E+00,.2081519E+00,&
     & .1935477E+00,.1842067E+00,.1812086E+00,.1856280E+00,.1773106E+00/

!     the array absb(235,8) (kb(5,13:59,8)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 8,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:120, 1)   /                                       &
     & .8084900E-06,.8136200E-06,.8139100E-06,.8183200E-06,.8100000E-06,&
     & .6259300E-06,.6309000E-06,.6315800E-06,.6282100E-06,.6250200E-06,&
     & .4782500E-06,.4785200E-06,.4771300E-06,.4768800E-06,.4798900E-06,&
     & .3603900E-06,.3590500E-06,.3606400E-06,.3629800E-06,.3686000E-06,&
     & .2767900E-06,.2760800E-06,.2774200E-06,.2811200E-06,.2875900E-06,&
     & .2177100E-06,.2182600E-06,.2201900E-06,.2242900E-06,.2299600E-06,&
     & .1733400E-06,.1744100E-06,.1768100E-06,.1805500E-06,.1853700E-06,&
     & .1397400E-06,.1407600E-06,.1432600E-06,.1469400E-06,.1507000E-06,&
     & .1128900E-06,.1139400E-06,.1163100E-06,.1196300E-06,.1228400E-06,&
     & .9057400E-07,.9191400E-07,.9404600E-07,.9699800E-07,.9984100E-07,&
     & .7248300E-07,.7406900E-07,.7618600E-07,.7848200E-07,.8096400E-07,&
     & .5746300E-07,.5913000E-07,.6080200E-07,.6290300E-07,.6521800E-07,&
     & .4576700E-07,.4718400E-07,.4869600E-07,.5049400E-07,.5252700E-07,&
     & .3663600E-07,.3779400E-07,.3921700E-07,.4076800E-07,.4246500E-07,&
     & .2917300E-07,.3018300E-07,.3146900E-07,.3283400E-07,.3421300E-07,&
     & .2322500E-07,.2413700E-07,.2516700E-07,.2637800E-07,.2753400E-07,&
     & .1867500E-07,.1947200E-07,.2034000E-07,.2134000E-07,.2228600E-07,&
     & .1502000E-07,.1569000E-07,.1646800E-07,.1725800E-07,.1806000E-07,&
     & .1214900E-07,.1272800E-07,.1337500E-07,.1403500E-07,.1471100E-07,&
     & .9859600E-08,.1034400E-07,.1087900E-07,.1142700E-07,.1200300E-07,&
     & .8012700E-08,.8419900E-08,.8861000E-08,.9318800E-08,.9810200E-08,&
     & .6547300E-08,.6885000E-08,.7247000E-08,.7637900E-08,.8032900E-08,&
     & .5310700E-08,.5593000E-08,.5898400E-08,.6216100E-08,.6551100E-08,&
     & .4274900E-08,.4507200E-08,.4759200E-08,.5024400E-08,.5303400E-08/

      data   absb(121:235, 1)   /                                       &
     & .3459300E-08,.3647900E-08,.3855700E-08,.4071400E-08,.4303100E-08,&
     & .2798100E-08,.2952500E-08,.3121200E-08,.3298400E-08,.3489000E-08,&
     & .2261600E-08,.2387900E-08,.2526600E-08,.2671600E-08,.2829500E-08,&
     & .1823200E-08,.1924500E-08,.2036200E-08,.2155400E-08,.2283100E-08,&
     & .1467900E-08,.1551100E-08,.1640200E-08,.1737600E-08,.1841300E-08,&
     & .1182800E-08,.1248800E-08,.1320000E-08,.1400800E-08,.1484100E-08,&
     & .9497000E-09,.1002800E-08,.1061000E-08,.1125400E-08,.1193300E-08,&
     & .7621000E-09,.8040100E-09,.8509800E-09,.9020300E-09,.9579400E-09,&
     & .6124200E-09,.6447100E-09,.6820500E-09,.7233600E-09,.7684600E-09,&
     & .4921400E-09,.5164200E-09,.5463300E-09,.5791600E-09,.6150600E-09,&
     & .3955200E-09,.4134300E-09,.4365800E-09,.4626600E-09,.4912100E-09,&
     & .3177300E-09,.3316100E-09,.3487800E-09,.3697000E-09,.3923100E-09,&
     & .2552600E-09,.2661300E-09,.2790600E-09,.2951000E-09,.3134600E-09,&
     & .2057100E-09,.2141900E-09,.2237900E-09,.2362200E-09,.2507600E-09,&
     & .1657300E-09,.1725100E-09,.1800200E-09,.1892500E-09,.2007400E-09,&
     & .1331200E-09,.1388800E-09,.1448000E-09,.1517000E-09,.1606700E-09,&
     & .1071700E-09,.1120000E-09,.1165300E-09,.1218700E-09,.1286400E-09,&
     & .8539500E-10,.9019000E-10,.9397800E-10,.9811500E-10,.1032400E-09,&
     & .6793300E-10,.7268200E-10,.7585700E-10,.7907900E-10,.8294200E-10,&
     & .5357200E-10,.5809500E-10,.6123100E-10,.6373800E-10,.6673800E-10,&
     & .4172900E-10,.4638000E-10,.4935700E-10,.5140900E-10,.5374300E-10,&
     & .3231900E-10,.3673700E-10,.3957200E-10,.4148700E-10,.4332700E-10,&
     & .2582000E-10,.2954100E-10,.3210700E-10,.3376200E-10,.3525000E-10/

      data   absb(  1:120, 2)   /                                       &
     & .9380900E-05,.1053600E-04,.1177700E-04,.1309800E-04,.1448600E-04,&
     & .7705000E-05,.8652000E-05,.9674500E-05,.1075900E-04,.1189100E-04,&
     & .6311200E-05,.7091400E-05,.7932000E-05,.8817900E-05,.9734400E-05,&
     & .5167200E-05,.5817200E-05,.6496100E-05,.7213300E-05,.7959500E-05,&
     & .4247000E-05,.4781700E-05,.5339600E-05,.5922900E-05,.6538500E-05,&
     & .3510500E-05,.3951300E-05,.4404200E-05,.4886500E-05,.5398300E-05,&
     & .2905700E-05,.3268000E-05,.3640900E-05,.4041900E-05,.4464600E-05,&
     & .2413200E-05,.2711100E-05,.3019200E-05,.3349800E-05,.3698500E-05,&
     & .2004100E-05,.2248400E-05,.2505200E-05,.2777000E-05,.3066100E-05,&
     & .1676200E-05,.1877500E-05,.2090000E-05,.2315800E-05,.2554300E-05,&
     & .1401300E-05,.1566500E-05,.1743300E-05,.1930800E-05,.2129000E-05,&
     & .1168000E-05,.1303900E-05,.1451500E-05,.1608500E-05,.1773300E-05,&
     & .9749100E-06,.1088600E-05,.1211400E-05,.1342800E-05,.1480600E-05,&
     & .8160800E-06,.9119800E-06,.1014800E-05,.1124400E-05,.1239200E-05,&
     & .6840700E-06,.7643200E-06,.8504800E-06,.9421000E-06,.1039300E-05,&
     & .5734000E-06,.6407100E-06,.7131100E-06,.7898800E-06,.8730400E-06,&
     & .4823400E-06,.5388600E-06,.5999200E-06,.6655700E-06,.7356500E-06,&
     & .4062800E-06,.4538400E-06,.5063800E-06,.5617500E-06,.6208600E-06,&
     & .3427600E-06,.3840900E-06,.4281700E-06,.4755100E-06,.5250800E-06,&
     & .2902400E-06,.3254800E-06,.3629100E-06,.4026300E-06,.4442300E-06,&
     & .2462300E-06,.2759600E-06,.3074800E-06,.3410400E-06,.3757100E-06,&
     & .2081400E-06,.2331800E-06,.2597500E-06,.2874600E-06,.3167400E-06,&
     & .1739900E-06,.1947600E-06,.2169800E-06,.2402100E-06,.2646300E-06,&
     & .1435200E-06,.1607900E-06,.1792000E-06,.1983800E-06,.2185200E-06/

      data   absb(121:235, 2)   /                                       &
     & .1164400E-06,.1306100E-06,.1456900E-06,.1614800E-06,.1779400E-06,&
     & .9432700E-07,.1059700E-06,.1182700E-06,.1312100E-06,.1447100E-06,&
     & .7638600E-07,.8593200E-07,.9600400E-07,.1067500E-06,.1178500E-06,&
     & .6109300E-07,.6885600E-07,.7708800E-07,.8583100E-07,.9487800E-07,&
     & .4878100E-07,.5505800E-07,.6172500E-07,.6885200E-07,.7623400E-07,&
     & .3890900E-07,.4398700E-07,.4939100E-07,.5516100E-07,.6119700E-07,&
     & .3077900E-07,.3486600E-07,.3922200E-07,.4387000E-07,.4878500E-07,&
     & .2424900E-07,.2751400E-07,.3101400E-07,.3477400E-07,.3877200E-07,&
     & .1907600E-07,.2169300E-07,.2451400E-07,.2752200E-07,.3075600E-07,&
     & .1494900E-07,.1704600E-07,.1930300E-07,.2172100E-07,.2432000E-07,&
     & .1162700E-07,.1329400E-07,.1510200E-07,.1704000E-07,.1911500E-07,&
     & .9020800E-08,.1035900E-07,.1180300E-07,.1334900E-07,.1500300E-07,&
     & .6997900E-08,.8062500E-08,.9207500E-08,.1044500E-07,.1176800E-07,&
     & .5450700E-08,.6277700E-08,.7200400E-08,.8189900E-08,.9248800E-08,&
     & .4254100E-08,.4893300E-08,.5630400E-08,.6421400E-08,.7272500E-08,&
     & .3316100E-08,.3812100E-08,.4397600E-08,.5030100E-08,.5714000E-08,&
     & .2581700E-08,.2969100E-08,.3429100E-08,.3936800E-08,.4483600E-08,&
     & .2020300E-08,.2322800E-08,.2682300E-08,.3090000E-08,.3528500E-08,&
     & .1583300E-08,.1819600E-08,.2102300E-08,.2427600E-08,.2778700E-08,&
     & .1240300E-08,.1425000E-08,.1646100E-08,.1904500E-08,.2185500E-08,&
     & .9724900E-09,.1116400E-08,.1288100E-08,.1492300E-08,.1717500E-08,&
     & .7643100E-09,.8769900E-09,.1010800E-08,.1170900E-08,.1351600E-08,&
     & .6155300E-09,.7064900E-09,.8146300E-09,.9431500E-09,.1090700E-08/

      data   absb(  1:120, 3)   /                                       &
     & .3770800E-04,.4394200E-04,.5058600E-04,.5765600E-04,.6513900E-04,&
     & .3169400E-04,.3686200E-04,.4235000E-04,.4819700E-04,.5434100E-04,&
     & .2656400E-04,.3082400E-04,.3537200E-04,.4018000E-04,.4523100E-04,&
     & .2223200E-04,.2574000E-04,.2951200E-04,.3344700E-04,.3758800E-04,&
     & .1857800E-04,.2146700E-04,.2458200E-04,.2783200E-04,.3122100E-04,&
     & .1553000E-04,.1792200E-04,.2049800E-04,.2317000E-04,.2597300E-04,&
     & .1297200E-04,.1495200E-04,.1705900E-04,.1927000E-04,.2158000E-04,&
     & .1085800E-04,.1249800E-04,.1422800E-04,.1606100E-04,.1795800E-04,&
     & .9081900E-05,.1044600E-04,.1188000E-04,.1339400E-04,.1495100E-04,&
     & .7664800E-05,.8797900E-05,.9993100E-05,.1125500E-04,.1254200E-04,&
     & .6463400E-05,.7414000E-05,.8408600E-05,.9454600E-05,.1052700E-04,&
     & .5452300E-05,.6246400E-05,.7080900E-05,.7950800E-05,.8844600E-05,&
     & .4611000E-05,.5271600E-05,.5969300E-05,.6690400E-05,.7432200E-05,&
     & .3913400E-05,.4462000E-05,.5042500E-05,.5643900E-05,.6257700E-05,&
     & .3320800E-05,.3777700E-05,.4262900E-05,.4760700E-05,.5271400E-05,&
     & .2817700E-05,.3200800E-05,.3603100E-05,.4018700E-05,.4435800E-05,&
     & .2393200E-05,.2715800E-05,.3051600E-05,.3394500E-05,.3741600E-05,&
     & .2034000E-05,.2303500E-05,.2582700E-05,.2867200E-05,.3155900E-05,&
     & .1730800E-05,.1954700E-05,.2186900E-05,.2424300E-05,.2664900E-05,&
     & .1473400E-05,.1659300E-05,.1853600E-05,.2051800E-05,.2251400E-05,&
     & .1253800E-05,.1409700E-05,.1571800E-05,.1736900E-05,.1901300E-05,&
     & .1062600E-05,.1193200E-05,.1329000E-05,.1466200E-05,.1602500E-05,&
     & .8899000E-06,.9987800E-06,.1112200E-05,.1226600E-05,.1339500E-05,&
     & .7353500E-06,.8269000E-06,.9209900E-06,.1015900E-05,.1109900E-05/

      data   absb(121:235, 3)   /                                       &
     & .5972800E-06,.6730100E-06,.7512700E-06,.8302100E-06,.9084500E-06,&
     & .4848400E-06,.5473700E-06,.6122900E-06,.6776200E-06,.7425800E-06,&
     & .3934700E-06,.4448500E-06,.4988600E-06,.5529000E-06,.6067400E-06,&
     & .3148800E-06,.3570300E-06,.4014800E-06,.4464000E-06,.4909100E-06,&
     & .2513000E-06,.2856100E-06,.3222400E-06,.3595300E-06,.3961800E-06,&
     & .2003000E-06,.2284000E-06,.2583400E-06,.2893000E-06,.3196500E-06,&
     & .1580700E-06,.1810900E-06,.2054200E-06,.2309400E-06,.2563500E-06,&
     & .1241800E-06,.1428200E-06,.1626100E-06,.1836400E-06,.2046500E-06,&
     & .9738300E-07,.1124500E-06,.1285500E-06,.1457200E-06,.1630700E-06,&
     & .7601500E-07,.8806700E-07,.1012000E-06,.1151000E-06,.1293500E-06,&
     & .5881300E-07,.6849300E-07,.7910200E-07,.9027300E-07,.1020000E-06,&
     & .4544000E-07,.5314400E-07,.6159700E-07,.7063900E-07,.8025500E-07,&
     & .3501500E-07,.4113600E-07,.4786900E-07,.5521600E-07,.6299400E-07,&
     & .2705600E-07,.3191000E-07,.3726700E-07,.4321000E-07,.4952200E-07,&
     & .2090800E-07,.2475600E-07,.2904400E-07,.3379100E-07,.3890400E-07,&
     & .1613100E-07,.1918100E-07,.2258900E-07,.2637800E-07,.3050400E-07,&
     & .1241800E-07,.1483500E-07,.1753700E-07,.2054700E-07,.2386000E-07,&
     & .9599400E-08,.1151400E-07,.1366200E-07,.1606500E-07,.1871700E-07,&
     & .7423900E-08,.8945300E-08,.1065600E-07,.1256900E-07,.1469300E-07,&
     & .5730000E-08,.6944100E-08,.8303000E-08,.9819300E-08,.1152300E-07,&
     & .4415200E-08,.5381100E-08,.6458100E-08,.7661000E-08,.9020100E-08,&
     & .3407200E-08,.4170800E-08,.5029700E-08,.5987600E-08,.7068400E-08,&
     & .2730200E-08,.3349200E-08,.4048100E-08,.4826600E-08,.5704600E-08/

      data   absb(  1:120, 4)   /                                       &
     & .1201500E-03,.1392000E-03,.1593700E-03,.1800100E-03,.1992900E-03,&
     & .1025600E-03,.1188300E-03,.1358700E-03,.1525900E-03,.1681200E-03,&
     & .8720300E-04,.1009900E-03,.1149800E-03,.1284700E-03,.1411400E-03,&
     & .7385900E-04,.8530400E-04,.9681600E-04,.1077400E-03,.1180500E-03,&
     & .6233400E-04,.7178600E-04,.8122500E-04,.9009100E-04,.9845400E-04,&
     & .5258100E-04,.6034400E-04,.6801800E-04,.7523700E-04,.8190300E-04,&
     & .4419700E-04,.5057300E-04,.5684000E-04,.6265600E-04,.6796800E-04,&
     & .3715100E-04,.4238400E-04,.4749400E-04,.5219300E-04,.5646700E-04,&
     & .3119300E-04,.3548000E-04,.3960000E-04,.4342700E-04,.4687000E-04,&
     & .2637400E-04,.2992500E-04,.3322400E-04,.3626100E-04,.3904000E-04,&
     & .2231500E-04,.2519800E-04,.2783300E-04,.3029000E-04,.3250900E-04,&
     & .1888300E-04,.2118400E-04,.2329100E-04,.2526800E-04,.2704600E-04,&
     & .1597700E-04,.1780800E-04,.1952900E-04,.2110600E-04,.2253300E-04,&
     & .1352400E-04,.1499600E-04,.1639100E-04,.1765800E-04,.1879600E-04,&
     & .1143300E-04,.1263100E-04,.1375700E-04,.1476900E-04,.1567700E-04,&
     & .9655800E-05,.1064000E-04,.1153700E-04,.1235700E-04,.1308600E-04,&
     & .8163200E-05,.8956000E-05,.9680100E-05,.1033900E-04,.1091900E-04,&
     & .6897200E-05,.7532100E-05,.8118900E-05,.8641800E-05,.9112000E-05,&
     & .5825500E-05,.6342700E-05,.6811500E-05,.7229100E-05,.7608800E-05,&
     & .4923500E-05,.5340400E-05,.5712800E-05,.6045000E-05,.6354000E-05,&
     & .4160200E-05,.4490400E-05,.4787300E-05,.5056700E-05,.5308500E-05,&
     & .3499300E-05,.3764000E-05,.4003300E-05,.4221700E-05,.4432100E-05,&
     & .2916800E-05,.3133100E-05,.3326400E-05,.3507200E-05,.3680800E-05,&
     & .2410500E-05,.2586300E-05,.2745900E-05,.2896900E-05,.3043300E-05/

      data   absb(121:235, 4)   /                                       &
     & .1969900E-05,.2116700E-05,.2249900E-05,.2375100E-05,.2498400E-05,&
     & .1608500E-05,.1731600E-05,.1843100E-05,.1948200E-05,.2052000E-05,&
     & .1314000E-05,.1417200E-05,.1510200E-05,.1598900E-05,.1686200E-05,&
     & .1064300E-05,.1151800E-05,.1230000E-05,.1305200E-05,.1378900E-05,&
     & .8610300E-06,.9349400E-06,.1001300E-05,.1064800E-05,.1127500E-05,&
     & .6956600E-06,.7580300E-06,.8146800E-06,.8681700E-06,.9211300E-06,&
     & .5585600E-06,.6113500E-06,.6599200E-06,.7050100E-06,.7491500E-06,&
     & .4464400E-06,.4911300E-06,.5328400E-06,.5706600E-06,.6077600E-06,&
     & .3563800E-06,.3938600E-06,.4294700E-06,.4619200E-06,.4929500E-06,&
     & .2831700E-06,.3149000E-06,.3450300E-06,.3727600E-06,.3989400E-06,&
     & .2234100E-06,.2500200E-06,.2756000E-06,.2995700E-06,.3214400E-06,&
     & .1755400E-06,.1979800E-06,.2196200E-06,.2401000E-06,.2587600E-06,&
     & .1373700E-06,.1564200E-06,.1745100E-06,.1919200E-06,.2079500E-06,&
     & .1075400E-06,.1235700E-06,.1387900E-06,.1534300E-06,.1671200E-06,&
     & .8412100E-07,.9744100E-07,.1101800E-06,.1225600E-06,.1341900E-06,&
     & .6560400E-07,.7656400E-07,.8735600E-07,.9770300E-07,.1074500E-06,&
     & .5094300E-07,.5998800E-07,.6904800E-07,.7762900E-07,.8593300E-07,&
     & .3967900E-07,.4712100E-07,.5461400E-07,.6176600E-07,.6878300E-07,&
     & .3093400E-07,.3698000E-07,.4311300E-07,.4914200E-07,.5500500E-07,&
     & .2405000E-07,.2892300E-07,.3399300E-07,.3904700E-07,.4392800E-07,&
     & .1866000E-07,.2255700E-07,.2674800E-07,.3095100E-07,.3500000E-07,&
     & .1448800E-07,.1762500E-07,.2104300E-07,.2451100E-07,.2788400E-07,&
     & .1167400E-07,.1423400E-07,.1704400E-07,.1990200E-07,.2270200E-07/

      data   absb(  1:120, 5)   /                                       &
     & .5907500E-03,.6176547E-03,.6462139E-03,.6766147E-03,.7105858E-03,&
     & .5021733E-03,.5254919E-03,.5505928E-03,.5781750E-03,.6090718E-03,&
     & .4251753E-03,.4458740E-03,.4682385E-03,.4935183E-03,.5206768E-03,&
     & .3591813E-03,.3774559E-03,.3977574E-03,.4201300E-03,.4431469E-03,&
     & .3027495E-03,.3191476E-03,.3371282E-03,.3563107E-03,.3755935E-03,&
     & .2550227E-03,.2694860E-03,.2850784E-03,.3010196E-03,.3174204E-03,&
     & .2145715E-03,.2270374E-03,.2400302E-03,.2534708E-03,.2670980E-03,&
     & .1804017E-03,.1910788E-03,.2019239E-03,.2130155E-03,.2241997E-03,&
     & .1515116E-03,.1603628E-03,.1693253E-03,.1785842E-03,.1878199E-03,&
     & .1274750E-03,.1347358E-03,.1422851E-03,.1500079E-03,.1574823E-03,&
     & .1070376E-03,.1130951E-03,.1194785E-03,.1258411E-03,.1318753E-03,&
     & .8978466E-04,.9494123E-04,.1002546E-03,.1054132E-03,.1102246E-03,&
     & .7533356E-04,.7966324E-04,.8405144E-04,.8817180E-04,.9207527E-04,&
     & .6331538E-04,.6693246E-04,.7046596E-04,.7375138E-04,.7692004E-04,&
     & .5321123E-04,.5620271E-04,.5902917E-04,.6164644E-04,.6430450E-04,&
     & .4466516E-04,.4710753E-04,.4935907E-04,.5155918E-04,.5379983E-04,&
     & .3751313E-04,.3945958E-04,.4130911E-04,.4316167E-04,.4502731E-04,&
     & .3148456E-04,.3306301E-04,.3457534E-04,.3614273E-04,.3767792E-04,&
     & .2643306E-04,.2770164E-04,.2898435E-04,.3028820E-04,.3158349E-04,&
     & .2217901E-04,.2324093E-04,.2432947E-04,.2540558E-04,.2651303E-04,&
     & .1860493E-04,.1950781E-04,.2040165E-04,.2133522E-04,.2227687E-04,&
     & .1560186E-04,.1635369E-04,.1712128E-04,.1791364E-04,.1869262E-04,&
     & .1303087E-04,.1366605E-04,.1431103E-04,.1497004E-04,.1563111E-04,&
     & .1082591E-04,.1136497E-04,.1190838E-04,.1246591E-04,.1301791E-04/

      data   absb(121:235, 5)   /                                       &
     & .8949161E-05,.9402237E-05,.9857226E-05,.1033892E-04,.1080399E-04,&
     & .7398494E-05,.7779511E-05,.8166568E-05,.8566423E-05,.8964480E-05,&
     & .6113971E-05,.6437029E-05,.6769198E-05,.7108135E-05,.7445280E-05,&
     & .5026369E-05,.5297079E-05,.5578033E-05,.5865750E-05,.6156368E-05,&
     & .4126175E-05,.4355500E-05,.4592219E-05,.4836238E-05,.5085641E-05,&
     & .3390696E-05,.3580871E-05,.3780710E-05,.3987163E-05,.4198069E-05,&
     & .2776281E-05,.2936183E-05,.3106181E-05,.3277157E-05,.3458786E-05,&
     & .2266975E-05,.2401731E-05,.2543341E-05,.2688159E-05,.2843911E-05,&
     & .1851175E-05,.1964040E-05,.2082434E-05,.2205585E-05,.2334866E-05,&
     & .1508390E-05,.1603576E-05,.1702375E-05,.1806116E-05,.1913393E-05,&
     & .1224515E-05,.1304492E-05,.1387169E-05,.1474342E-05,.1565346E-05,&
     & .9927416E-06,.1060100E-05,.1128545E-05,.1203015E-05,.1278997E-05,&
     & .8040118E-06,.8600684E-06,.9178680E-06,.9793761E-06,.1043884E-05,&
     & .6511957E-06,.6983405E-06,.7470839E-06,.7982212E-06,.8522692E-06,&
     & .5274519E-06,.5672397E-06,.6079811E-06,.6509056E-06,.6959894E-06,&
     & .4266819E-06,.4601105E-06,.4940630E-06,.5297410E-06,.5677419E-06,&
     & .3444399E-06,.3727641E-06,.4009432E-06,.4307494E-06,.4625489E-06,&
     & .2784617E-06,.3023078E-06,.3258711E-06,.3507791E-06,.3773299E-06,&
     & .2249763E-06,.2450479E-06,.2650681E-06,.2856452E-06,.3078613E-06,&
     & .1813509E-06,.1984089E-06,.2151828E-06,.2323405E-06,.2508149E-06,&
     & .1459558E-06,.1603926E-06,.1745826E-06,.1886604E-06,.2039625E-06,&
     & .1175460E-06,.1296812E-06,.1415814E-06,.1532665E-06,.1658461E-06,&
     & .9640155E-07,.1066253E-06,.1166034E-06,.1263881E-06,.1369759E-06/

      data   absb(  1:120, 6)   /                                       &
     & .3651810E-02,.3746595E-02,.3858789E-02,.3979164E-02,.4109370E-02,&
     & .3141811E-02,.3230210E-02,.3325595E-02,.3429577E-02,.3547976E-02,&
     & .2701370E-02,.2775610E-02,.2858077E-02,.2953528E-02,.3061412E-02,&
     & .2315393E-02,.2378831E-02,.2454205E-02,.2541014E-02,.2642100E-02,&
     & .1977791E-02,.2036050E-02,.2104700E-02,.2186085E-02,.2283190E-02,&
     & .1685324E-02,.1740054E-02,.1805015E-02,.1884094E-02,.1978199E-02,&
     & .1437021E-02,.1487251E-02,.1550029E-02,.1626786E-02,.1715576E-02,&
     & .1225237E-02,.1273610E-02,.1334615E-02,.1407663E-02,.1492701E-02,&
     & .1044735E-02,.1092958E-02,.1152692E-02,.1223437E-02,.1303990E-02,&
     & .8942887E-03,.9428463E-03,.1001793E-02,.1070477E-02,.1149176E-02,&
     & .7694049E-03,.8171499E-03,.8745104E-03,.9407106E-03,.1016851E-02,&
     & .6645252E-03,.7113905E-03,.7673264E-03,.8320640E-03,.9036498E-03,&
     & .5770389E-03,.6234065E-03,.6779963E-03,.7399304E-03,.8071342E-03,&
     & .5049575E-03,.5502834E-03,.6030293E-03,.6613349E-03,.7246181E-03,&
     & .4447202E-03,.4886958E-03,.5389704E-03,.5940660E-03,.6530676E-03,&
     & .3944259E-03,.4369189E-03,.4843540E-03,.5362101E-03,.5917015E-03,&
     & .3520821E-03,.3928555E-03,.4377636E-03,.4866798E-03,.5390983E-03,&
     & .3162045E-03,.3551094E-03,.3978195E-03,.4440662E-03,.4940346E-03,&
     & .2859841E-03,.3231854E-03,.3639308E-03,.4079919E-03,.4558740E-03,&
     & .2600638E-03,.2959725E-03,.3350934E-03,.3776361E-03,.4237286E-03,&
     & .2380660E-03,.2725920E-03,.3104257E-03,.3517378E-03,.3964521E-03,&
     & .2181478E-03,.2514965E-03,.2881695E-03,.3284820E-03,.3717262E-03,&
     & .1983067E-03,.2303075E-03,.2658491E-03,.3047978E-03,.3465710E-03,&
     & .1780704E-03,.2085530E-03,.2427653E-03,.2802542E-03,.3204639E-03/

      data   absb(121:235, 6)   /                                       &
     & .1565144E-03,.1849367E-03,.2172813E-03,.2529291E-03,.2913913E-03,&
     & .1374906E-03,.1639820E-03,.1944800E-03,.2286994E-03,.2656361E-03,&
     & .1209235E-03,.1455643E-03,.1744351E-03,.2072121E-03,.2429831E-03,&
     & .1042521E-03,.1269284E-03,.1536021E-03,.1846991E-03,.2190618E-03,&
     & .8956332E-04,.1101797E-03,.1349815E-03,.1642251E-03,.1972149E-03,&
     & .7682060E-04,.9563184E-04,.1183845E-03,.1457632E-03,.1774806E-03,&
     & .6483539E-04,.8163448E-04,.1023214E-03,.1276117E-03,.1579098E-03,&
     & .5421096E-04,.6906822E-04,.8771080E-04,.1108467E-03,.1392070E-03,&
     & .4516018E-04,.5817822E-04,.7490186E-04,.9595379E-04,.1222179E-03,&
     & .3720506E-04,.4849290E-04,.6322139E-04,.8217544E-04,.1061847E-03,&
     & .3014760E-04,.3978783E-04,.5256468E-04,.6924860E-04,.9085729E-04,&
     & .2427491E-04,.3242037E-04,.4333198E-04,.5801079E-04,.7732452E-04,&
     & .1942608E-04,.2622750E-04,.3554119E-04,.4824771E-04,.6532610E-04,&
     & .1560503E-04,.2127119E-04,.2922144E-04,.4016492E-04,.5526442E-04,&
     & .1246448E-04,.1717739E-04,.2388696E-04,.3336674E-04,.4667005E-04,&
     & .9928517E-05,.1381768E-04,.1943442E-04,.2753949E-04,.3915061E-04,&
     & .7845842E-05,.1104661E-04,.1568945E-04,.2259744E-04,.3259294E-04,&
     & .6241865E-05,.8874763E-05,.1274389E-04,.1859414E-04,.2728085E-04,&
     & .4962728E-05,.7110134E-05,.1034291E-04,.1527272E-04,.2280240E-04,&
     & .3921680E-05,.5664841E-05,.8348994E-05,.1247617E-04,.1894519E-04,&
     & .3092625E-05,.4493204E-05,.6711569E-05,.1013101E-04,.1563245E-04,&
     & .2446444E-05,.3564130E-05,.5373137E-05,.8228205E-05,.1289787E-04,&
     & .2061392E-05,.3034672E-05,.4641777E-05,.7229029E-05,.1154580E-04/

      data   absb(  1:120, 7)   /                                       &
     & .7974511E-01,.7917223E-01,.7880571E-01,.7865688E-01,.7869742E-01,&
     & .7216761E-01,.7180196E-01,.7168710E-01,.7178858E-01,.7210934E-01,&
     & .6471287E-01,.6460046E-01,.6473991E-01,.6511587E-01,.6570290E-01,&
     & .5758865E-01,.5773359E-01,.5813143E-01,.5877105E-01,.5965013E-01,&
     & .5098748E-01,.5136835E-01,.5201772E-01,.5291936E-01,.5405095E-01,&
     & .4498346E-01,.4559189E-01,.4647453E-01,.4761481E-01,.4899661E-01,&
     & .3962956E-01,.4044591E-01,.4154636E-01,.4290895E-01,.4451971E-01,&
     & .3494623E-01,.3595631E-01,.3725553E-01,.3882539E-01,.4065371E-01,&
     & .3090317E-01,.3209598E-01,.3356918E-01,.3532088E-01,.3734086E-01,&
     & .2753735E-01,.2889680E-01,.3055328E-01,.3249469E-01,.3469930E-01,&
     & .2472194E-01,.2624583E-01,.2807624E-01,.3018719E-01,.3256894E-01,&
     & .2241246E-01,.2409593E-01,.2607770E-01,.2835897E-01,.3091024E-01,&
     & .2056186E-01,.2238553E-01,.2452455E-01,.2695966E-01,.2967202E-01,&
     & .1912250E-01,.2109409E-01,.2338231E-01,.2597361E-01,.2884315E-01,&
     & .1803069E-01,.2014547E-01,.2257560E-01,.2531768E-01,.2833188E-01,&
     & .1723757E-01,.1948465E-01,.2205955E-01,.2494239E-01,.2808185E-01,&
     & .1671584E-01,.1909967E-01,.2181124E-01,.2481820E-01,.2807426E-01,&
     & .1642188E-01,.1893762E-01,.2177522E-01,.2489493E-01,.2825602E-01,&
     & .1633447E-01,.1897666E-01,.2192891E-01,.2515422E-01,.2861454E-01,&
     & .1642117E-01,.1918126E-01,.2224051E-01,.2556670E-01,.2911713E-01,&
     & .1665212E-01,.1952573E-01,.2268617E-01,.2610433E-01,.2973637E-01,&
     & .1690445E-01,.1987409E-01,.2312114E-01,.2661658E-01,.3031656E-01,&
     & .1697413E-01,.2000260E-01,.2330589E-01,.2685241E-01,.3059806E-01,&
     & .1679058E-01,.1984390E-01,.2316985E-01,.2673928E-01,.3050650E-01/

      data   absb(121:235, 7)   /                                       &
     & .1621314E-01,.1924283E-01,.2254989E-01,.2610500E-01,.2986511E-01,&
     & .1565005E-01,.1865129E-01,.2193291E-01,.2546985E-01,.2921685E-01,&
     & .1512354E-01,.1809306E-01,.2134739E-01,.2486405E-01,.2859408E-01,&
     & .1434388E-01,.1724600E-01,.2044363E-01,.2391201E-01,.2760136E-01,&
     & .1356639E-01,.1639374E-01,.1952578E-01,.2293718E-01,.2658026E-01,&
     & .1282426E-01,.1557142E-01,.1863464E-01,.2198740E-01,.2557922E-01,&
     & .1197741E-01,.1462053E-01,.1760067E-01,.2086625E-01,.2439863E-01,&
     & .1111477E-01,.1364234E-01,.1651789E-01,.1969313E-01,.2315038E-01,&
     & .1030179E-01,.1271031E-01,.1547643E-01,.1856508E-01,.2193139E-01,&
     & .9491085E-02,.1177021E-01,.1441481E-01,.1739504E-01,.2066837E-01,&
     & .8639429E-02,.1078206E-01,.1328441E-01,.1613831E-01,.1930011E-01,&
     & .7839892E-02,.9857468E-02,.1221482E-01,.1493348E-01,.1797877E-01,&
     & .7092072E-02,.8991607E-02,.1120568E-01,.1378380E-01,.1670474E-01,&
     & .6432463E-02,.8224187E-02,.1031515E-01,.1275809E-01,.1555586E-01,&
     & .5832717E-02,.7522561E-02,.9503860E-02,.1181296E-01,.1448506E-01,&
     & .5280145E-02,.6862909E-02,.8736900E-02,.1091927E-01,.1346213E-01,&
     & .4760950E-02,.6239258E-02,.8009269E-02,.1007450E-01,.1248495E-01,&
     & .4319927E-02,.5706660E-02,.7378235E-02,.9343778E-02,.1163217E-01,&
     & .3927425E-02,.5223469E-02,.6805075E-02,.8675949E-02,.1085346E-01,&
     & .3566312E-02,.4771326E-02,.6260588E-02,.8040438E-02,.1011504E-01,&
     & .3235298E-02,.4349610E-02,.5751274E-02,.7436457E-02,.9414970E-02,&
     & .2948268E-02,.3977194E-02,.5291955E-02,.6891886E-02,.8780537E-02,&
     & .2838304E-02,.3833360E-02,.5113920E-02,.6679829E-02,.8533277E-02/

      data   absb(  1:120, 8)   /                                       &
     & .1589380E+01,.1590617E+01,.1588657E+01,.1583627E+01,.1576103E+01,&
     & .1709558E+01,.1708607E+01,.1704305E+01,.1697075E+01,.1686632E+01,&
     & .1824948E+01,.1821482E+01,.1814617E+01,.1804445E+01,.1791378E+01,&
     & .1933407E+01,.1927272E+01,.1917585E+01,.1904625E+01,.1888292E+01,&
     & .2032788E+01,.2024036E+01,.2011680E+01,.1995779E+01,.1976689E+01,&
     & .2122514E+01,.2111187E+01,.2096103E+01,.2077495E+01,.2055661E+01,&
     & .2201894E+01,.2188149E+01,.2170688E+01,.2149637E+01,.2125223E+01,&
     & .2271095E+01,.2255113E+01,.2235383E+01,.2211893E+01,.2185029E+01,&
     & .2330637E+01,.2312532E+01,.2290760E+01,.2265233E+01,.2236248E+01,&
     & .2380235E+01,.2360023E+01,.2336137E+01,.2308453E+01,.2277303E+01,&
     & .2421520E+01,.2399497E+01,.2373596E+01,.2343962E+01,.2310850E+01,&
     & .2455548E+01,.2431616E+01,.2403985E+01,.2372410E+01,.2337341E+01,&
     & .2483000E+01,.2457533E+01,.2427896E+01,.2394579E+01,.2357794E+01,&
     & .2504497E+01,.2477271E+01,.2446062E+01,.2410900E+01,.2372279E+01,&
     & .2521140E+01,.2492125E+01,.2459283E+01,.2422484E+01,.2382142E+01,&
     & .2533448E+01,.2502926E+01,.2468416E+01,.2429941E+01,.2388304E+01,&
     & .2541996E+01,.2509801E+01,.2473737E+01,.2433957E+01,.2391015E+01,&
     & .2547277E+01,.2513671E+01,.2476134E+01,.2434937E+01,.2390872E+01,&
     & .2549698E+01,.2514609E+01,.2475824E+01,.2433436E+01,.2388147E+01,&
     & .2549776E+01,.2513277E+01,.2473300E+01,.2429667E+01,.2383220E+01,&
     & .2547817E+01,.2510019E+01,.2468803E+01,.2424113E+01,.2376687E+01,&
     & .2545438E+01,.2506563E+01,.2464177E+01,.2418709E+01,.2370529E+01,&
     & .2545437E+01,.2505887E+01,.2462992E+01,.2416876E+01,.2368167E+01,&
     & .2548620E+01,.2508763E+01,.2465654E+01,.2419269E+01,.2370306E+01/

      data   absb(121:235, 8)   /                                       &
     & .2556882E+01,.2517384E+01,.2474533E+01,.2428347E+01,.2379571E+01,&
     & .2564751E+01,.2525708E+01,.2483281E+01,.2437321E+01,.2388806E+01,&
     & .2572195E+01,.2533600E+01,.2491357E+01,.2445820E+01,.2397529E+01,&
     & .2582733E+01,.2544997E+01,.2503670E+01,.2458754E+01,.2411027E+01,&
     & .2593208E+01,.2556450E+01,.2515941E+01,.2471723E+01,.2424698E+01,&
     & .2603291E+01,.2567534E+01,.2527827E+01,.2484500E+01,.2438025E+01,&
     & .2614485E+01,.2580122E+01,.2541480E+01,.2499317E+01,.2453682E+01,&
     & .2626088E+01,.2593101E+01,.2555788E+01,.2514770E+01,.2470119E+01,&
     & .2636975E+01,.2605452E+01,.2569448E+01,.2529581E+01,.2486026E+01,&
     & .2647757E+01,.2617959E+01,.2583514E+01,.2544917E+01,.2502605E+01,&
     & .2659180E+01,.2630964E+01,.2598269E+01,.2561345E+01,.2520508E+01,&
     & .2669920E+01,.2643199E+01,.2612358E+01,.2577040E+01,.2537623E+01,&
     & .2680036E+01,.2654848E+01,.2625604E+01,.2592088E+01,.2554283E+01,&
     & .2688888E+01,.2664859E+01,.2637463E+01,.2605558E+01,.2569275E+01,&
     & .2696948E+01,.2674379E+01,.2648210E+01,.2617842E+01,.2583127E+01,&
     & .2704362E+01,.2683131E+01,.2658263E+01,.2629633E+01,.2596479E+01,&
     & .2711382E+01,.2691630E+01,.2667973E+01,.2640779E+01,.2609393E+01,&
     & .2717487E+01,.2698684E+01,.2676350E+01,.2650456E+01,.2620490E+01,&
     & .2722676E+01,.2705192E+01,.2684099E+01,.2659244E+01,.2630640E+01,&
     & .2727533E+01,.2711288E+01,.2691406E+01,.2667688E+01,.2640330E+01,&
     & .2731955E+01,.2717083E+01,.2698177E+01,.2675744E+01,.2649670E+01,&
     & .2735798E+01,.2722099E+01,.2704431E+01,.2682979E+01,.2657966E+01,&
     & .2737188E+01,.2723903E+01,.2706774E+01,.2685895E+01,.2661240E+01/

!  ---

      data   forref(1:3,1: 8)    /                                      &
     & .1062750E-05,.1041850E-05,.4201540E-05,.1543430E-05,.6531930E-05,&
     & .1745960E-04,.3489170E-05,.1084200E-04,.5408490E-04,.1458220E-04,&
     & .1560270E-04,.8812630E-04,.3256123E-04,.1001071E-03,.8710486E-04,&
     & .1958190E-03,.1469074E-03,.7616317E-04,.2991387E-03,.1425125E-03,&
     & .6636773E-04,.4406950E-03,.1586718E-03,.3817266E-04/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 8).

      data   selfref(1:10,1: 8)  /                                      &
     & .3317280E-03,.2874800E-03,.2491350E-03,.2159040E-03,.1871060E-03,&
     & .1621490E-03,.1405200E-03,.1217770E-03,.1055340E-03,.9145730E-04,&
     & .8826280E-03,.6989140E-03,.5534390E-03,.4382440E-03,.3470260E-03,&
     & .2747950E-03,.2175980E-03,.1723060E-03,.1364420E-03,.1080420E-03,&
     & .1154610E-02,.9372030E-03,.7607300E-03,.6174860E-03,.5012150E-03,&
     & .4068370E-03,.3302310E-03,.2680490E-03,.2175760E-03,.1766070E-03,&
     & .1034500E-02,.9602680E-03,.8913600E-03,.8273970E-03,.7680240E-03,&
     & .7129110E-03,.6617540E-03,.6142670E-03,.5701880E-03,.5292720E-03,&
     & .3227190E-02,.2709139E-02,.2274479E-02,.1909745E-02,.1603665E-02,&
     & .1346766E-02,.1131137E-02,.9501252E-03,.7981584E-03,.6705633E-03,&
     & .3142835E-02,.3104435E-02,.3068990E-02,.3036443E-02,.3006728E-02,&
     & .2979763E-02,.2955519E-02,.2933917E-02,.2914907E-02,.2898444E-02,&
     & .2729233E-02,.2892026E-02,.3065501E-02,.3250452E-02,.3447726E-02,&
     & .3658264E-02,.3883086E-02,.4123283E-02,.4380054E-02,.4654719E-02,&
     & .2594476E-02,.2804964E-02,.3065165E-02,.3386332E-02,.3782724E-02,&
     & .4272483E-02,.4878732E-02,.5631073E-02,.6567477E-02,.7736768E-02/

!........................................!
      end module module_radsw_kgb19      !
!========================================!


!========================================!
      module module_radsw_kgb20          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 20:  5150-6150 cm-1 (low - h2o; high - h2o)                !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG20

!
      implicit none
!
      private
!
      integer,  public :: MSA20, MSB20, MSF20, MFR20
      parameter (MSA20=65, MSB20=235, MSF20=10, MFR20=4)

      real (kind=kind_phys), public :: forref(MFR20,NG20), absch4(NG20),&
     &       absa(MSA20,NG20), absb(MSB20,NG20), selfref(MSF20,NG20)

!  ---  rayleigh extinction coefficient at v = 5670 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 4.12e-09

!     the array absa(65,NG20) (ka(5,13,NG20)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 10, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1: 65, 1)   /                                       &
     & .7838300E-06,.8622000E-06,.9535900E-06,.1059000E-05,.1178200E-05,&
     & .6504000E-06,.7251000E-06,.8131800E-06,.9005900E-06,.9978600E-06,&
     & .5807200E-06,.6588800E-06,.7426500E-06,.8185400E-06,.9064500E-06,&
     & .5360100E-06,.6076500E-06,.6808800E-06,.7574100E-06,.8380100E-06,&
     & .5014200E-06,.5695100E-06,.6402800E-06,.7194400E-06,.7981300E-06,&
     & .4716400E-06,.5400800E-06,.6104000E-06,.6800600E-06,.7503400E-06,&
     & .4933700E-06,.5617800E-06,.6221500E-06,.6912700E-06,.7606000E-06,&
     & .6158100E-06,.7011700E-06,.7894200E-06,.8714500E-06,.9464700E-06,&
     & .1415400E-05,.1575400E-05,.1726100E-05,.1873900E-05,.1977400E-05,&
     & .3418600E-05,.3701200E-05,.3960200E-05,.4223500E-05,.4425600E-05,&
     & .3857400E-05,.4208000E-05,.4470100E-05,.4745400E-05,.5000200E-05,&
     & .3579400E-05,.3868600E-05,.4144300E-05,.4393900E-05,.4650000E-05,&
     & .2958600E-05,.3193800E-05,.3416700E-05,.3625300E-05,.3825000E-05/

      data   absa(  1: 65, 2)   /                                       &
     & .5709800E-05,.6463000E-05,.7311700E-05,.8243600E-05,.9194700E-05,&
     & .4638400E-05,.5312500E-05,.6030100E-05,.6802800E-05,.7582400E-05,&
     & .4065700E-05,.4648600E-05,.5245900E-05,.5895500E-05,.6554600E-05,&
     & .3889500E-05,.4425800E-05,.4975900E-05,.5566300E-05,.6170200E-05,&
     & .3897100E-05,.4435900E-05,.4993300E-05,.5552300E-05,.6123400E-05,&
     & .3953200E-05,.4464400E-05,.4973400E-05,.5487500E-05,.6010200E-05,&
     & .4106800E-05,.4583200E-05,.5069800E-05,.5552100E-05,.6017500E-05,&
     & .4792200E-05,.5215600E-05,.5681700E-05,.6157600E-05,.6631300E-05,&
     & .8319900E-05,.8831700E-05,.9368800E-05,.9975400E-05,.1062000E-04,&
     & .1683600E-04,.1852600E-04,.1988700E-04,.2116800E-04,.2210400E-04,&
     & .1888200E-04,.2100500E-04,.2289600E-04,.2477700E-04,.2611500E-04,&
     & .1774400E-04,.1978000E-04,.2160000E-04,.2352300E-04,.2512800E-04,&
     & .1473600E-04,.1635600E-04,.1795500E-04,.1953300E-04,.2086100E-04/

      data   absa(  1: 65, 3)   /                                       &
     & .4177600E-04,.4815000E-04,.5509700E-04,.6266100E-04,.7092000E-04,&
     & .3390900E-04,.3917600E-04,.4462200E-04,.5075400E-04,.5733600E-04,&
     & .2743800E-04,.3147300E-04,.3612200E-04,.4113400E-04,.4645600E-04,&
     & .2322200E-04,.2653500E-04,.3014100E-04,.3409900E-04,.3841600E-04,&
     & .2111400E-04,.2388800E-04,.2688300E-04,.3034000E-04,.3378500E-04,&
     & .2075000E-04,.2292600E-04,.2553600E-04,.2849200E-04,.3167600E-04,&
     & .2158400E-04,.2411200E-04,.2687200E-04,.2979400E-04,.3294000E-04,&
     & .2419400E-04,.2698100E-04,.3013700E-04,.3354600E-04,.3718200E-04,&
     & .3746100E-04,.4215800E-04,.4671800E-04,.5104800E-04,.5515400E-04,&
     & .7239100E-04,.7716400E-04,.8401600E-04,.8965800E-04,.9551100E-04,&
     & .9173600E-04,.9910700E-04,.1046300E-03,.1095200E-03,.1154900E-03,&
     & .9120000E-04,.9881200E-04,.1043200E-03,.1089300E-03,.1134900E-03,&
     & .7621700E-04,.8270200E-04,.8750000E-04,.9134900E-04,.9534900E-04/

      data   absa(  1: 65, 4)   /                                       &
     & .7170500E-03,.8274300E-03,.9470000E-03,.1067000E-02,.1190200E-02,&
     & .5790900E-03,.6709600E-03,.7672400E-03,.8658200E-03,.9716700E-03,&
     & .4477100E-03,.5199700E-03,.5986100E-03,.6796700E-03,.7667600E-03,&
     & .3408500E-03,.3983300E-03,.4622600E-03,.5288900E-03,.6000800E-03,&
     & .2667800E-03,.3123400E-03,.3634100E-03,.4166300E-03,.4741800E-03,&
     & .2090300E-03,.2471800E-03,.2892200E-03,.3335300E-03,.3817600E-03,&
     & .1717300E-03,.2033000E-03,.2371100E-03,.2740100E-03,.3144500E-03,&
     & .1672200E-03,.1939600E-03,.2230700E-03,.2534700E-03,.2856100E-03,&
     & .2504300E-03,.2882400E-03,.3254700E-03,.3619300E-03,.3980600E-03,&
     & .4332300E-03,.4866700E-03,.5346200E-03,.5787900E-03,.6204000E-03,&
     & .4771900E-03,.5227100E-03,.5640200E-03,.6038800E-03,.6352000E-03,&
     & .4506100E-03,.4901400E-03,.5239100E-03,.5527600E-03,.5776000E-03,&
     & .3766400E-03,.4095200E-03,.4369000E-03,.4605500E-03,.4801800E-03/

      data   absa(  1: 65, 5)   /                                       &
     & .6646600E-02,.6868600E-02,.7089700E-02,.7301900E-02,.7491400E-02,&
     & .5402300E-02,.5601000E-02,.5800800E-02,.5998700E-02,.6168500E-02,&
     & .4321800E-02,.4512700E-02,.4684800E-02,.4864700E-02,.5026700E-02,&
     & .3463900E-02,.3647300E-02,.3795100E-02,.3957000E-02,.4104900E-02,&
     & .2774500E-02,.2937000E-02,.3075400E-02,.3214800E-02,.3349200E-02,&
     & .2202000E-02,.2352000E-02,.2483600E-02,.2596500E-02,.2711900E-02,&
     & .1762400E-02,.1896300E-02,.2016100E-02,.2115300E-02,.2211600E-02,&
     & .1416900E-02,.1533200E-02,.1635000E-02,.1723100E-02,.1812100E-02,&
     & .1303000E-02,.1390400E-02,.1469000E-02,.1550000E-02,.1627500E-02,&
     & .1715500E-02,.1816900E-02,.1894300E-02,.1962000E-02,.2021500E-02,&
     & .1715600E-02,.1780800E-02,.1841900E-02,.1892500E-02,.1931300E-02,&
     & .1526900E-02,.1581100E-02,.1628300E-02,.1668600E-02,.1706100E-02,&
     & .1276800E-02,.1320700E-02,.1355500E-02,.1392400E-02,.1423400E-02/

      data   absa(  1: 65, 6)   /                                       &
     & .1919100E-01,.1946300E-01,.1969200E-01,.1990000E-01,.2011200E-01,&
     & .1596700E-01,.1621900E-01,.1642600E-01,.1661300E-01,.1680500E-01,&
     & .1318600E-01,.1341100E-01,.1361400E-01,.1378500E-01,.1394400E-01,&
     & .1093500E-01,.1113100E-01,.1131800E-01,.1147500E-01,.1162000E-01,&
     & .9054100E-02,.9234400E-02,.9403500E-02,.9553700E-02,.9686200E-02,&
     & .7472400E-02,.7632000E-02,.7779400E-02,.7929400E-02,.8052800E-02,&
     & .6118300E-02,.6264400E-02,.6396100E-02,.6530300E-02,.6647800E-02,&
     & .5063800E-02,.5198400E-02,.5322200E-02,.5441200E-02,.5546500E-02,&
     & .4287000E-02,.4400900E-02,.4510900E-02,.4597600E-02,.4679600E-02,&
     & .4523900E-02,.4607300E-02,.4696800E-02,.4797900E-02,.4895600E-02,&
     & .4408900E-02,.4500600E-02,.4588100E-02,.4672100E-02,.4801500E-02,&
     & .3959800E-02,.4029000E-02,.4110800E-02,.4189900E-02,.4267200E-02,&
     & .3342500E-02,.3414800E-02,.3480800E-02,.3540500E-02,.3598300E-02/

      data   absa(  1: 65, 7)   /                                       &
     & .5016500E-01,.5034300E-01,.5045200E-01,.5042400E-01,.5032900E-01,&
     & .4272300E-01,.4288000E-01,.4293900E-01,.4291000E-01,.4286100E-01,&
     & .3611700E-01,.3625500E-01,.3630100E-01,.3629400E-01,.3628800E-01,&
     & .3058500E-01,.3072000E-01,.3078700E-01,.3081600E-01,.3084200E-01,&
     & .2587900E-01,.2602900E-01,.2611600E-01,.2617900E-01,.2622500E-01,&
     & .2182200E-01,.2197800E-01,.2209400E-01,.2217800E-01,.2224400E-01,&
     & .1830400E-01,.1847600E-01,.1860600E-01,.1870500E-01,.1878000E-01,&
     & .1522400E-01,.1539400E-01,.1551900E-01,.1561900E-01,.1569000E-01,&
     & .1283500E-01,.1302900E-01,.1316900E-01,.1328100E-01,.1336900E-01,&
     & .1163200E-01,.1171400E-01,.1176000E-01,.1177600E-01,.1180600E-01,&
     & .1117000E-01,.1125200E-01,.1128800E-01,.1135700E-01,.1139700E-01,&
     & .1040400E-01,.1053300E-01,.1070500E-01,.1083300E-01,.1094700E-01,&
     & .9104100E-02,.9267400E-02,.9406800E-02,.9541200E-02,.9676900E-02/

      data   absa(  1: 65, 8)   /                                       &
     & .1452700E+00,.1448300E+00,.1443200E+00,.1438900E+00,.1434900E+00,&
     & .1273900E+00,.1270200E+00,.1266100E+00,.1262400E+00,.1258400E+00,&
     & .1109100E+00,.1105800E+00,.1102400E+00,.1099000E+00,.1095300E+00,&
     & .9659000E-01,.9633500E-01,.9610500E-01,.9581800E-01,.9553000E-01,&
     & .8379900E-01,.8370800E-01,.8357200E-01,.8334700E-01,.8312600E-01,&
     & .7236900E-01,.7238800E-01,.7228900E-01,.7214300E-01,.7198100E-01,&
     & .6215800E-01,.6224700E-01,.6218500E-01,.6211100E-01,.6200700E-01,&
     & .5299800E-01,.5314200E-01,.5316200E-01,.5315400E-01,.5310100E-01,&
     & .4477600E-01,.4487300E-01,.4492000E-01,.4494600E-01,.4488500E-01,&
     & .3823700E-01,.3851400E-01,.3871600E-01,.3887700E-01,.3896600E-01,&
     & .3351100E-01,.3372100E-01,.3384300E-01,.3384000E-01,.3383900E-01,&
     & .2970400E-01,.2982500E-01,.2980800E-01,.2978700E-01,.2979400E-01,&
     & .2597300E-01,.2600400E-01,.2603200E-01,.2602300E-01,.2610700E-01/

      data   absa(  1: 65, 9)   /                                       &
     & .6574795E+00,.6554552E+00,.6533155E+00,.6509209E+00,.6484065E+00,&
     & .6255494E+00,.6246438E+00,.6232566E+00,.6216554E+00,.6197426E+00,&
     & .5862362E+00,.5862909E+00,.5857885E+00,.5847743E+00,.5835312E+00,&
     & .5431781E+00,.5440621E+00,.5441314E+00,.5437951E+00,.5431448E+00,&
     & .4982859E+00,.4996650E+00,.5003975E+00,.5006596E+00,.5007368E+00,&
     & .4522079E+00,.4542740E+00,.4556478E+00,.4567195E+00,.4575025E+00,&
     & .4066660E+00,.4093121E+00,.4113888E+00,.4131704E+00,.4142502E+00,&
     & .3626680E+00,.3658157E+00,.3685601E+00,.3707145E+00,.3720343E+00,&
     & .3201627E+00,.3237696E+00,.3268366E+00,.3290751E+00,.3306950E+00,&
     & .2769076E+00,.2802971E+00,.2831727E+00,.2851341E+00,.2866169E+00,&
     & .2450947E+00,.2482141E+00,.2507025E+00,.2525044E+00,.2537402E+00,&
     & .2182159E+00,.2209177E+00,.2229032E+00,.2242398E+00,.2253965E+00,&
     & .1913957E+00,.1940531E+00,.1961425E+00,.1981273E+00,.1993420E+00/

      data   absa(  1: 65,10)   /                                       &
     & .3268900E+01,.3251984E+01,.3235834E+01,.3221079E+01,.3207393E+01,&
     & .3622351E+01,.3601154E+01,.3582181E+01,.3564354E+01,.3547975E+01,&
     & .3989098E+01,.3964764E+01,.3942540E+01,.3922453E+01,.3903201E+01,&
     & .4343054E+01,.4315865E+01,.4291453E+01,.4268631E+01,.4246808E+01,&
     & .4682358E+01,.4653365E+01,.4626500E+01,.4601251E+01,.4576531E+01,&
     & .5008731E+01,.4977636E+01,.4948849E+01,.4920625E+01,.4892993E+01,&
     & .5317015E+01,.5284091E+01,.5253355E+01,.5222597E+01,.5194092E+01,&
     & .5603209E+01,.5569632E+01,.5536865E+01,.5505362E+01,.5476476E+01,&
     & .5865108E+01,.5831464E+01,.5798284E+01,.5767336E+01,.5737919E+01,&
     & .6093085E+01,.6061308E+01,.6030013E+01,.6000723E+01,.5972079E+01,&
     & .6260183E+01,.6230003E+01,.6201202E+01,.6173485E+01,.6146273E+01,&
     & .6409424E+01,.6381513E+01,.6354613E+01,.6328684E+01,.6301983E+01,&
     & .6564707E+01,.6537322E+01,.6510214E+01,.6481775E+01,.6454592E+01/

!     the array absb(235,10) (kb(5,13:59,10)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 10,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:120, 1)   /                                       &
     & .2959700E-05,.3192900E-05,.3415900E-05,.3625600E-05,.3826400E-05,&
     & .2423300E-05,.2615100E-05,.2786600E-05,.2950200E-05,.3117000E-05,&
     & .1920600E-05,.2078900E-05,.2209900E-05,.2338300E-05,.2450500E-05,&
     & .1493300E-05,.1607100E-05,.1703900E-05,.1799400E-05,.1881600E-05,&
     & .1142900E-05,.1214400E-05,.1289500E-05,.1350300E-05,.1417400E-05,&
     & .8413400E-06,.8926400E-06,.9426000E-06,.9900300E-06,.1039100E-05,&
     & .6068300E-06,.6414200E-06,.6801800E-06,.7204200E-06,.7547900E-06,&
     & .4580900E-06,.4881300E-06,.5193400E-06,.5482300E-06,.5726100E-06,&
     & .3521900E-06,.3783500E-06,.4032200E-06,.4277200E-06,.4473600E-06,&
     & .2741900E-06,.2954200E-06,.3161700E-06,.3339800E-06,.3493200E-06,&
     & .2138500E-06,.2312700E-06,.2467000E-06,.2595100E-06,.2695200E-06,&
     & .1676700E-06,.1817900E-06,.1928200E-06,.2013600E-06,.2111300E-06,&
     & .1336100E-06,.1430100E-06,.1498000E-06,.1577800E-06,.1661500E-06,&
     & .1061500E-06,.1121800E-06,.1188900E-06,.1257400E-06,.1311100E-06,&
     & .8366000E-07,.8929600E-07,.9501900E-07,.1000700E-06,.1039700E-06,&
     & .6666700E-07,.7121700E-07,.7561500E-07,.7923700E-07,.8223900E-07,&
     & .5277600E-07,.5608600E-07,.5982800E-07,.6251300E-07,.6421800E-07,&
     & .4142900E-07,.4437000E-07,.4673100E-07,.4855300E-07,.5014300E-07,&
     & .3243700E-07,.3452300E-07,.3620800E-07,.3750000E-07,.3857100E-07,&
     & .2523700E-07,.2673000E-07,.2786800E-07,.2882900E-07,.2994300E-07,&
     & .1954500E-07,.2061400E-07,.2146700E-07,.2228000E-07,.2314100E-07,&
     & .1514600E-07,.1586900E-07,.1650900E-07,.1717900E-07,.1798000E-07,&
     & .1148200E-07,.1203900E-07,.1254700E-07,.1325500E-07,.1387400E-07,&
     & .8617200E-08,.9002300E-08,.9560700E-08,.1010600E-07,.1059500E-07/

      data   absb(121:235, 1)   /                                       &
     & .6741100E-08,.7078100E-08,.7555300E-08,.7973900E-08,.8401100E-08,&
     & .5260400E-08,.5574300E-08,.5941000E-08,.6303100E-08,.6651600E-08,&
     & .4112900E-08,.4385400E-08,.4685100E-08,.4977100E-08,.5246300E-08,&
     & .3308900E-08,.3528900E-08,.3770200E-08,.4012500E-08,.4241500E-08,&
     & .2679400E-08,.2848800E-08,.3042800E-08,.3243700E-08,.3442000E-08,&
     & .2167000E-08,.2303000E-08,.2458900E-08,.2625200E-08,.2786700E-08,&
     & .1769900E-08,.1885300E-08,.2000100E-08,.2133000E-08,.2265700E-08,&
     & .1444600E-08,.1547100E-08,.1639700E-08,.1742300E-08,.1852600E-08,&
     & .1173400E-08,.1275400E-08,.1354200E-08,.1424200E-08,.1519300E-08,&
     & .9574300E-09,.1047000E-08,.1118300E-08,.1172600E-08,.1238900E-08,&
     & .7734500E-09,.8519700E-09,.9218400E-09,.9736800E-09,.1014900E-08,&
     & .6250000E-09,.6928800E-09,.7554800E-09,.8044600E-09,.8442800E-09,&
     & .5009500E-09,.5596700E-09,.6190400E-09,.6644100E-09,.7033900E-09,&
     & .4080600E-09,.4571200E-09,.5053000E-09,.5497400E-09,.5824600E-09,&
     & .3310200E-09,.3713000E-09,.4112100E-09,.4502400E-09,.4843600E-09,&
     & .2690600E-09,.3073900E-09,.3384500E-09,.3688400E-09,.4002400E-09,&
     & .2220200E-09,.2510800E-09,.2793700E-09,.3059200E-09,.3301200E-09,&
     & .1866200E-09,.2076100E-09,.2307200E-09,.2534200E-09,.2744500E-09,&
     & .1574100E-09,.1714700E-09,.1914600E-09,.2102900E-09,.2285700E-09,&
     & .1315100E-09,.1463700E-09,.1597700E-09,.1754700E-09,.1897400E-09,&
     & .1087400E-09,.1244700E-09,.1363400E-09,.1475200E-09,.1602200E-09,&
     & .9020600E-10,.1048400E-09,.1161700E-09,.1254500E-09,.1347400E-09,&
     & .7675900E-10,.8886600E-10,.9934000E-10,.1070600E-09,.1144400E-09/

      data   absb(  1:120, 2)   /                                       &
     & .1472700E-04,.1635700E-04,.1796100E-04,.1953000E-04,.2085200E-04,&
     & .1214800E-04,.1347800E-04,.1481800E-04,.1604900E-04,.1710000E-04,&
     & .9690200E-05,.1078700E-04,.1183500E-04,.1277000E-04,.1351500E-04,&
     & .7623500E-05,.8457500E-05,.9221700E-05,.9852600E-05,.1044900E-04,&
     & .5804900E-05,.6423900E-05,.6973300E-05,.7413100E-05,.7808200E-05,&
     & .4320400E-05,.4751000E-05,.5116500E-05,.5457900E-05,.5743400E-05,&
     & .3221100E-05,.3532700E-05,.3817500E-05,.4053500E-05,.4224500E-05,&
     & .2526500E-05,.2765200E-05,.2964400E-05,.3133100E-05,.3273000E-05,&
     & .1992100E-05,.2174900E-05,.2334300E-05,.2437700E-05,.2545800E-05,&
     & .1585900E-05,.1725900E-05,.1823600E-05,.1912400E-05,.1994200E-05,&
     & .1259100E-05,.1356500E-05,.1430900E-05,.1503200E-05,.1564200E-05,&
     & .1001200E-05,.1070100E-05,.1124600E-05,.1181900E-05,.1227900E-05,&
     & .7898500E-06,.8407400E-06,.8902900E-06,.9314500E-06,.9708500E-06,&
     & .6299600E-06,.6684700E-06,.7063700E-06,.7351300E-06,.7713500E-06,&
     & .5028300E-06,.5311100E-06,.5565300E-06,.5830600E-06,.6179900E-06,&
     & .3996200E-06,.4232000E-06,.4426500E-06,.4671500E-06,.4951600E-06,&
     & .3181000E-06,.3359700E-06,.3519100E-06,.3744800E-06,.3976800E-06,&
     & .2524200E-06,.2644700E-06,.2810900E-06,.3008900E-06,.3154300E-06,&
     & .1983700E-06,.2095400E-06,.2244200E-06,.2376200E-06,.2518300E-06,&
     & .1559900E-06,.1670100E-06,.1778900E-06,.1888000E-06,.2003000E-06,&
     & .1230700E-06,.1321100E-06,.1404600E-06,.1511100E-06,.1598800E-06,&
     & .9842900E-07,.1049800E-06,.1132600E-06,.1207400E-06,.1277900E-06,&
     & .7774300E-07,.8395100E-07,.9047300E-07,.9627700E-07,.1020400E-06,&
     & .6122000E-07,.6639900E-07,.7110200E-07,.7594600E-07,.8080300E-07/

      data   absb(121:235, 2)   /                                       &
     & .4881900E-07,.5296200E-07,.5666900E-07,.6081700E-07,.6470900E-07,&
     & .3887100E-07,.4222600E-07,.4537600E-07,.4864100E-07,.5185500E-07,&
     & .3086900E-07,.3348500E-07,.3623300E-07,.3886800E-07,.4152300E-07,&
     & .2479800E-07,.2698300E-07,.2918000E-07,.3143700E-07,.3360000E-07,&
     & .1995100E-07,.2178400E-07,.2351800E-07,.2546000E-07,.2721900E-07,&
     & .1609800E-07,.1754200E-07,.1897900E-07,.2054700E-07,.2201600E-07,&
     & .1302000E-07,.1414400E-07,.1541400E-07,.1661000E-07,.1787000E-07,&
     & .1054800E-07,.1150000E-07,.1249200E-07,.1352000E-07,.1452400E-07,&
     & .8560200E-08,.9324500E-08,.1009600E-07,.1097800E-07,.1181900E-07,&
     & .6914900E-08,.7565300E-08,.8215400E-08,.8882200E-08,.9600200E-08,&
     & .5570800E-08,.6116400E-08,.6667800E-08,.7192200E-08,.7763300E-08,&
     & .4480400E-08,.4931200E-08,.5396700E-08,.5866200E-08,.6278000E-08,&
     & .3646700E-08,.3986600E-08,.4362800E-08,.4758900E-08,.5120800E-08,&
     & .2952400E-08,.3254000E-08,.3547100E-08,.3863800E-08,.4188700E-08,&
     & .2388700E-08,.2652600E-08,.2895900E-08,.3150400E-08,.3411200E-08,&
     & .1932100E-08,.2164800E-08,.2362500E-08,.2583900E-08,.2817300E-08,&
     & .1568400E-08,.1772500E-08,.1947600E-08,.2124000E-08,.2311800E-08,&
     & .1279900E-08,.1446700E-08,.1607300E-08,.1749400E-08,.1916000E-08,&
     & .1044200E-08,.1191100E-08,.1321600E-08,.1453700E-08,.1577100E-08,&
     & .8574100E-09,.9793700E-09,.1093600E-08,.1205600E-08,.1316700E-08,&
     & .7035800E-09,.8033400E-09,.9082400E-09,.1005600E-08,.1101000E-08,&
     & .5742800E-09,.6644100E-09,.7542000E-09,.8362700E-09,.9248500E-09,&
     & .4792200E-09,.5559200E-09,.6349900E-09,.7073100E-09,.7867800E-09/

      data   absb(  1:120, 3)   /                                       &
     & .7620100E-04,.8269900E-04,.8749900E-04,.9135200E-04,.9534800E-04,&
     & .6282100E-04,.6786100E-04,.7186800E-04,.7510800E-04,.7855700E-04,&
     & .4977500E-04,.5336700E-04,.5663900E-04,.5900100E-04,.6230300E-04,&
     & .3800500E-04,.4088300E-04,.4326900E-04,.4540300E-04,.4813900E-04,&
     & .2855000E-04,.3075000E-04,.3233100E-04,.3448900E-04,.3668700E-04,&
     & .2097600E-04,.2232200E-04,.2374300E-04,.2555400E-04,.2722300E-04,&
     & .1538900E-04,.1642200E-04,.1779200E-04,.1912700E-04,.2029500E-04,&
     & .1188300E-04,.1281600E-04,.1396200E-04,.1487200E-04,.1572500E-04,&
     & .9294800E-05,.1018700E-04,.1098600E-04,.1171300E-04,.1242600E-04,&
     & .7396900E-05,.8084000E-05,.8726300E-05,.9385600E-05,.9953300E-05,&
     & .5896100E-05,.6459100E-05,.6964800E-05,.7449400E-05,.7938500E-05,&
     & .4744500E-05,.5174200E-05,.5603600E-05,.5975800E-05,.6372400E-05,&
     & .3844200E-05,.4179100E-05,.4483700E-05,.4807800E-05,.5129400E-05,&
     & .3133500E-05,.3392300E-05,.3640000E-05,.3896100E-05,.4148700E-05,&
     & .2549900E-05,.2743200E-05,.2956000E-05,.3154500E-05,.3343300E-05,&
     & .2063500E-05,.2223100E-05,.2389700E-05,.2551600E-05,.2693800E-05,&
     & .1663500E-05,.1802400E-05,.1927900E-05,.2049800E-05,.2162700E-05,&
     & .1335900E-05,.1447400E-05,.1547000E-05,.1634600E-05,.1745700E-05,&
     & .1069700E-05,.1147700E-05,.1225400E-05,.1309700E-05,.1391400E-05,&
     & .8495800E-06,.9146400E-06,.9818900E-06,.1048900E-05,.1115000E-05,&
     & .6817800E-06,.7335600E-06,.7890500E-06,.8422600E-06,.8947600E-06,&
     & .5473700E-06,.5899500E-06,.6337400E-06,.6773200E-06,.7218200E-06,&
     & .4368000E-06,.4708600E-06,.5077100E-06,.5420900E-06,.5768700E-06,&
     & .3461200E-06,.3746600E-06,.4037000E-06,.4307300E-06,.4597900E-06/

      data   absb(121:235, 3)   /                                       &
     & .2772500E-06,.3004800E-06,.3245100E-06,.3463800E-06,.3710500E-06,&
     & .2222800E-06,.2411400E-06,.2604900E-06,.2787100E-06,.2993500E-06,&
     & .1779700E-06,.1931700E-06,.2087800E-06,.2241100E-06,.2410200E-06,&
     & .1434600E-06,.1560100E-06,.1691300E-06,.1819700E-06,.1956600E-06,&
     & .1155900E-06,.1261500E-06,.1369700E-06,.1478500E-06,.1587400E-06,&
     & .9335800E-07,.1021400E-06,.1108900E-06,.1199800E-06,.1290400E-06,&
     & .7559900E-07,.8269000E-07,.8980800E-07,.9752400E-07,.1051100E-06,&
     & .6128500E-07,.6678600E-07,.7293700E-07,.7908700E-07,.8552500E-07,&
     & .4972500E-07,.5429400E-07,.5926300E-07,.6421300E-07,.6958300E-07,&
     & .4033600E-07,.4418600E-07,.4803400E-07,.5218400E-07,.5653400E-07,&
     & .3290400E-07,.3563300E-07,.3895900E-07,.4240200E-07,.4588300E-07,&
     & .2672500E-07,.2910400E-07,.3169300E-07,.3431200E-07,.3730400E-07,&
     & .2175100E-07,.2384900E-07,.2580300E-07,.2802400E-07,.3035600E-07,&
     & .1758700E-07,.1960900E-07,.2121600E-07,.2302200E-07,.2494200E-07,&
     & .1437600E-07,.1599000E-07,.1749300E-07,.1887200E-07,.2046300E-07,&
     & .1181100E-07,.1310600E-07,.1448900E-07,.1561500E-07,.1683200E-07,&
     & .9600100E-08,.1078800E-07,.1192100E-07,.1298500E-07,.1390400E-07,&
     & .7824300E-08,.8917800E-08,.9843000E-08,.1084000E-07,.1161900E-07,&
     & .6363800E-08,.7367800E-08,.8221400E-08,.8999700E-08,.9756500E-08,&
     & .5175300E-08,.6049700E-08,.6841900E-08,.7456000E-08,.8236200E-08,&
     & .4216400E-08,.4991600E-08,.5706600E-08,.6330800E-08,.6895000E-08,&
     & .3461800E-08,.4142400E-08,.4805400E-08,.5375700E-08,.5809500E-08,&
     & .2908300E-08,.3529100E-08,.4093200E-08,.4628500E-08,.4986500E-08/

      data   absb(  1:120, 4)   /                                       &
     & .3767400E-03,.4095300E-03,.4369000E-03,.4605900E-03,.4800700E-03,&
     & .3120200E-03,.3390600E-03,.3606600E-03,.3800300E-03,.3962700E-03,&
     & .2513500E-03,.2727100E-03,.2906900E-03,.3066100E-03,.3200500E-03,&
     & .1979700E-03,.2152700E-03,.2295500E-03,.2432800E-03,.2545600E-03,&
     & .1535400E-03,.1667900E-03,.1793300E-03,.1910100E-03,.2004100E-03,&
     & .1174200E-03,.1279700E-03,.1392300E-03,.1483600E-03,.1565400E-03,&
     & .9051900E-04,.9938600E-04,.1081900E-03,.1160100E-03,.1231900E-03,&
     & .7257900E-04,.7991000E-04,.8707500E-04,.9372800E-04,.9965600E-04,&
     & .5886600E-04,.6480500E-04,.7070500E-04,.7615300E-04,.8099600E-04,&
     & .4815700E-04,.5288300E-04,.5774000E-04,.6213000E-04,.6610000E-04,&
     & .3933700E-04,.4328900E-04,.4727500E-04,.5090600E-04,.5416100E-04,&
     & .3228900E-04,.3557700E-04,.3880800E-04,.4181400E-04,.4443300E-04,&
     & .2661500E-04,.2930100E-04,.3198400E-04,.3439600E-04,.3650900E-04,&
     & .2206000E-04,.2427000E-04,.2646000E-04,.2845000E-04,.3020300E-04,&
     & .1834400E-04,.2017700E-04,.2194800E-04,.2359500E-04,.2500100E-04,&
     & .1525600E-04,.1678700E-04,.1824200E-04,.1956100E-04,.2069600E-04,&
     & .1267900E-04,.1395200E-04,.1517000E-04,.1621100E-04,.1716300E-04,&
     & .1055000E-04,.1160500E-04,.1257900E-04,.1345100E-04,.1421000E-04,&
     & .8751200E-05,.9632200E-05,.1043300E-04,.1113300E-04,.1176200E-04,&
     & .7300100E-05,.8020300E-05,.8674500E-05,.9229200E-05,.9729500E-05,&
     & .6095800E-05,.6686200E-05,.7200600E-05,.7635900E-05,.8057600E-05,&
     & .5085500E-05,.5567100E-05,.5967000E-05,.6339200E-05,.6686900E-05,&
     & .4203300E-05,.4590500E-05,.4925000E-05,.5238400E-05,.5536900E-05,&
     & .3438400E-05,.3761800E-05,.4043000E-05,.4306000E-05,.4557700E-05/

      data   absb(121:235, 4)   /                                       &
     & .2803900E-05,.3074900E-05,.3313500E-05,.3534400E-05,.3745800E-05,&
     & .2284200E-05,.2510400E-05,.2712600E-05,.2896600E-05,.3076800E-05,&
     & .1858500E-05,.2051000E-05,.2219400E-05,.2373100E-05,.2524900E-05,&
     & .1504100E-05,.1667600E-05,.1809700E-05,.1940200E-05,.2068100E-05,&
     & .1215400E-05,.1354100E-05,.1474900E-05,.1585200E-05,.1693400E-05,&
     & .9803300E-06,.1097500E-05,.1201100E-05,.1294500E-05,.1385900E-05,&
     & .7872400E-06,.8862500E-06,.9748600E-06,.1055400E-05,.1130700E-05,&
     & .6305900E-06,.7138200E-06,.7894800E-06,.8575500E-06,.9204400E-06,&
     & .5042600E-06,.5739100E-06,.6380800E-06,.6971800E-06,.7497200E-06,&
     & .4013300E-06,.4585800E-06,.5141400E-06,.5640000E-06,.6096400E-06,&
     & .3158000E-06,.3644000E-06,.4112000E-06,.4543300E-06,.4934800E-06,&
     & .2484700E-06,.2890700E-06,.3287900E-06,.3653700E-06,.3990300E-06,&
     & .1953000E-06,.2285800E-06,.2624700E-06,.2933100E-06,.3221700E-06,&
     & .1536400E-06,.1811300E-06,.2091100E-06,.2358900E-06,.2599400E-06,&
     & .1207200E-06,.1441300E-06,.1665600E-06,.1895200E-06,.2098300E-06,&
     & .9492500E-07,.1139700E-06,.1325500E-06,.1519300E-06,.1696000E-06,&
     & .7447300E-07,.9019700E-07,.1057400E-06,.1215300E-06,.1366100E-06,&
     & .5900900E-07,.7159400E-07,.8478100E-07,.9775800E-07,.1101800E-06,&
     & .4700800E-07,.5699700E-07,.6783800E-07,.7875300E-07,.8911600E-07,&
     & .3741500E-07,.4564200E-07,.5442000E-07,.6346600E-07,.7191000E-07,&
     & .2989600E-07,.3663500E-07,.4369700E-07,.5103600E-07,.5826700E-07,&
     & .2394600E-07,.2952300E-07,.3531600E-07,.4118300E-07,.4739100E-07,&
     & .1988100E-07,.2464200E-07,.2939000E-07,.3409900E-07,.3923400E-07/

      data   absb(  1:120, 5)   /                                       &
     & .1277100E-02,.1320200E-02,.1355700E-02,.1392700E-02,.1423300E-02,&
     & .1060800E-02,.1095100E-02,.1124900E-02,.1154900E-02,.1178800E-02,&
     & .8675000E-03,.8967900E-03,.9198600E-03,.9439900E-03,.9640500E-03,&
     & .7007300E-03,.7250400E-03,.7463000E-03,.7649500E-03,.7816500E-03,&
     & .5628000E-03,.5832100E-03,.6015300E-03,.6154400E-03,.6288500E-03,&
     & .4494000E-03,.4659800E-03,.4788700E-03,.4905800E-03,.5022300E-03,&
     & .3575300E-03,.3706400E-03,.3829000E-03,.3939000E-03,.4038700E-03,&
     & .2901300E-03,.3009900E-03,.3117900E-03,.3211700E-03,.3301100E-03,&
     & .2365700E-03,.2460200E-03,.2550400E-03,.2634000E-03,.2714500E-03,&
     & .1942300E-03,.2022400E-03,.2098700E-03,.2171100E-03,.2243900E-03,&
     & .1596300E-03,.1664300E-03,.1730800E-03,.1794900E-03,.1858000E-03,&
     & .1316600E-03,.1374500E-03,.1433200E-03,.1487100E-03,.1542800E-03,&
     & .1088500E-03,.1139400E-03,.1189600E-03,.1237100E-03,.1284600E-03,&
     & .9040300E-04,.9491400E-04,.9915700E-04,.1031000E-03,.1072900E-03,&
     & .7526500E-04,.7916600E-04,.8264800E-04,.8620900E-04,.8988200E-04,&
     & .6287000E-04,.6605000E-04,.6911100E-04,.7228900E-04,.7545100E-04,&
     & .5246500E-04,.5519200E-04,.5788700E-04,.6066000E-04,.6336700E-04,&
     & .4388700E-04,.4619100E-04,.4858800E-04,.5091000E-04,.5327800E-04,&
     & .3676000E-04,.3876600E-04,.4078300E-04,.4278300E-04,.4478000E-04,&
     & .3083100E-04,.3256100E-04,.3426700E-04,.3600000E-04,.3759400E-04,&
     & .2590800E-04,.2738900E-04,.2883200E-04,.3029000E-04,.3158100E-04,&
     & .2177600E-04,.2302600E-04,.2426600E-04,.2544400E-04,.2651000E-04,&
     & .1823300E-04,.1928000E-04,.2032400E-04,.2126600E-04,.2214100E-04,&
     & .1516000E-04,.1604200E-04,.1691800E-04,.1768600E-04,.1840600E-04/

      data   absb(121:235, 5)   /                                       &
     & .1253800E-04,.1328500E-04,.1401500E-04,.1465700E-04,.1526900E-04,&
     & .1035700E-04,.1099000E-04,.1159500E-04,.1214800E-04,.1265100E-04,&
     & .8556600E-05,.9085600E-05,.9598600E-05,.1005700E-04,.1047800E-04,&
     & .7039000E-05,.7482600E-05,.7920000E-05,.8306100E-05,.8663400E-05,&
     & .5782700E-05,.6157200E-05,.6526900E-05,.6852800E-05,.7159500E-05,&
     & .4747700E-05,.5065600E-05,.5370500E-05,.5648600E-05,.5910000E-05,&
     & .3880200E-05,.4153300E-05,.4408900E-05,.4651500E-05,.4871900E-05,&
     & .3165800E-05,.3397100E-05,.3617700E-05,.3820500E-05,.4008900E-05,&
     & .2579600E-05,.2774000E-05,.2963500E-05,.3133600E-05,.3298600E-05,&
     & .2094500E-05,.2263600E-05,.2421400E-05,.2569800E-05,.2707700E-05,&
     & .1693100E-05,.1838300E-05,.1972900E-05,.2099600E-05,.2217200E-05,&
     & .1365000E-05,.1488900E-05,.1604300E-05,.1713900E-05,.1813600E-05,&
     & .1098100E-05,.1204200E-05,.1302800E-05,.1396300E-05,.1482400E-05,&
     & .8840700E-06,.9737000E-06,.1058900E-05,.1138300E-05,.1212400E-05,&
     & .7115000E-06,.7868200E-06,.8604800E-06,.9271100E-06,.9912800E-06,&
     & .5714000E-06,.6351300E-06,.6979000E-06,.7546600E-06,.8094600E-06,&
     & .4587100E-06,.5118300E-06,.5649400E-06,.6137600E-06,.6602200E-06,&
     & .3688900E-06,.4131900E-06,.4575900E-06,.4997300E-06,.5393300E-06,&
     & .2968300E-06,.3339600E-06,.3711000E-06,.4067100E-06,.4402600E-06,&
     & .2382800E-06,.2697300E-06,.3004800E-06,.3309900E-06,.3594900E-06,&
     & .1908300E-06,.2178200E-06,.2435000E-06,.2688300E-06,.2930900E-06,&
     & .1533400E-06,.1759200E-06,.1975300E-06,.2186900E-06,.2390300E-06,&
     & .1263500E-06,.1450400E-06,.1632100E-06,.1807400E-06,.1975900E-06/

      data   absb(  1:120, 6)   /                                       &
     & .3343300E-02,.3415900E-02,.3479800E-02,.3539700E-02,.3599500E-02,&
     & .2803900E-02,.2858900E-02,.2909000E-02,.2959500E-02,.3010800E-02,&
     & .2313500E-02,.2357900E-02,.2401200E-02,.2447000E-02,.2490000E-02,&
     & .1892500E-02,.1928100E-02,.1967100E-02,.2009000E-02,.2044500E-02,&
     & .1536500E-02,.1570300E-02,.1603900E-02,.1642400E-02,.1676000E-02,&
     & .1242400E-02,.1274600E-02,.1306700E-02,.1339700E-02,.1368700E-02,&
     & .1011300E-02,.1041700E-02,.1066400E-02,.1095800E-02,.1123100E-02,&
     & .8337500E-03,.8586300E-03,.8809800E-03,.9068000E-03,.9309300E-03,&
     & .6884900E-03,.7105600E-03,.7316100E-03,.7540200E-03,.7743200E-03,&
     & .5707600E-03,.5900100E-03,.6092500E-03,.6281600E-03,.6453100E-03,&
     & .4746200E-03,.4913800E-03,.5081800E-03,.5242100E-03,.5393000E-03,&
     & .3955200E-03,.4101100E-03,.4246000E-03,.4386500E-03,.4520600E-03,&
     & .3304100E-03,.3431700E-03,.3557300E-03,.3680000E-03,.3800500E-03,&
     & .2769900E-03,.2880600E-03,.2989200E-03,.3098300E-03,.3203800E-03,&
     & .2325300E-03,.2422900E-03,.2520800E-03,.2614400E-03,.2703900E-03,&
     & .1957300E-03,.2044200E-03,.2127600E-03,.2208700E-03,.2286900E-03,&
     & .1652800E-03,.1727100E-03,.1797400E-03,.1868100E-03,.1937300E-03,&
     & .1394900E-03,.1459900E-03,.1521400E-03,.1583400E-03,.1644800E-03,&
     & .1179300E-03,.1234500E-03,.1289100E-03,.1343200E-03,.1397500E-03,&
     & .9984500E-04,.1046400E-03,.1094100E-03,.1142500E-03,.1189600E-03,&
     & .8472300E-04,.8880800E-04,.9305400E-04,.9725700E-04,.1014400E-03,&
     & .7185700E-04,.7547800E-04,.7924500E-04,.8287800E-04,.8652700E-04,&
     & .6074100E-04,.6398700E-04,.6719000E-04,.7044500E-04,.7363400E-04,&
     & .5109400E-04,.5395300E-04,.5678700E-04,.5964400E-04,.6242100E-04/

      data   absb(121:235, 6)   /                                       &
     & .4280900E-04,.4532400E-04,.4780500E-04,.5031400E-04,.5279700E-04,&
     & .3586200E-04,.3808200E-04,.4024900E-04,.4245900E-04,.4469100E-04,&
     & .3004700E-04,.3197000E-04,.3387700E-04,.3584100E-04,.3779200E-04,&
     & .2505100E-04,.2673400E-04,.2842900E-04,.3017700E-04,.3185900E-04,&
     & .2085800E-04,.2233700E-04,.2383700E-04,.2534800E-04,.2684500E-04,&
     & .1735800E-04,.1865300E-04,.1994700E-04,.2127900E-04,.2261400E-04,&
     & .1438200E-04,.1550700E-04,.1663700E-04,.1780800E-04,.1897500E-04,&
     & .1187500E-04,.1285100E-04,.1384700E-04,.1486500E-04,.1588600E-04,&
     & .9782400E-05,.1064100E-04,.1149700E-04,.1238600E-04,.1328800E-04,&
     & .8030500E-05,.8771200E-05,.9517000E-05,.1028200E-04,.1107300E-04,&
     & .6555400E-05,.7188100E-05,.7833100E-05,.8502200E-05,.9179900E-05,&
     & .5339900E-05,.5881800E-05,.6436400E-05,.7009300E-05,.7599200E-05,&
     & .4339600E-05,.4799000E-05,.5272700E-05,.5763300E-05,.6278500E-05,&
     & .3527200E-05,.3913900E-05,.4323000E-05,.4742300E-05,.5181900E-05,&
     & .2862400E-05,.3191300E-05,.3537900E-05,.3897900E-05,.4274400E-05,&
     & .2318900E-05,.2596100E-05,.2890700E-05,.3197600E-05,.3520000E-05,&
     & .1873600E-05,.2107200E-05,.2357600E-05,.2617900E-05,.2894300E-05,&
     & .1515500E-05,.1713500E-05,.1924500E-05,.2145100E-05,.2380800E-05,&
     & .1225000E-05,.1392300E-05,.1571400E-05,.1757800E-05,.1959700E-05,&
     & .9901200E-06,.1129600E-05,.1280100E-05,.1439500E-05,.1610700E-05,&
     & .7987500E-06,.9141800E-06,.1040800E-05,.1177700E-05,.1321200E-05,&
     & .6447400E-06,.7410900E-06,.8466400E-06,.9632800E-06,.1084900E-05,&
     & .5329600E-06,.6142700E-06,.7038500E-06,.8037500E-06,.9086300E-06/

      data   absb(  1:120, 7)   /                                       &
     & .9106300E-02,.9268100E-02,.9404500E-02,.9538800E-02,.9673500E-02,&
     & .7848300E-02,.7980600E-02,.8123800E-02,.8252800E-02,.8360300E-02,&
     & .6626900E-02,.6744500E-02,.6884500E-02,.6991100E-02,.7085400E-02,&
     & .5518300E-02,.5641400E-02,.5757400E-02,.5848700E-02,.5940700E-02,&
     & .4569300E-02,.4678000E-02,.4774300E-02,.4862500E-02,.4946700E-02,&
     & .3761100E-02,.3853500E-02,.3940800E-02,.4023400E-02,.4100700E-02,&
     & .3099100E-02,.3180900E-02,.3264700E-02,.3338800E-02,.3404200E-02,&
     & .2585800E-02,.2658500E-02,.2734700E-02,.2799000E-02,.2857900E-02,&
     & .2167200E-02,.2232600E-02,.2297300E-02,.2354700E-02,.2405800E-02,&
     & .1821100E-02,.1880900E-02,.1936400E-02,.1985900E-02,.2035200E-02,&
     & .1531400E-02,.1584400E-02,.1633200E-02,.1676000E-02,.1724300E-02,&
     & .1292400E-02,.1338500E-02,.1379300E-02,.1420900E-02,.1465900E-02,&
     & .1092600E-02,.1132200E-02,.1168900E-02,.1208100E-02,.1249300E-02,&
     & .9275600E-03,.9613600E-03,.9960100E-03,.1032400E-02,.1069500E-02,&
     & .7883200E-03,.8189300E-03,.8513700E-03,.8846500E-03,.9188500E-03,&
     & .6709700E-03,.6997900E-03,.7298500E-03,.7603800E-03,.7921300E-03,&
     & .5723100E-03,.5993400E-03,.6272700E-03,.6547100E-03,.6838900E-03,&
     & .4899600E-03,.5144100E-03,.5394500E-03,.5656400E-03,.5916400E-03,&
     & .4199800E-03,.4423700E-03,.4655100E-03,.4892400E-03,.5132200E-03,&
     & .3613300E-03,.3816800E-03,.4029700E-03,.4244100E-03,.4472200E-03,&
     & .3115300E-03,.3305500E-03,.3498000E-03,.3697200E-03,.3908600E-03,&
     & .2694100E-03,.2865700E-03,.3043600E-03,.3229500E-03,.3425600E-03,&
     & .2321000E-03,.2478200E-03,.2641100E-03,.2812100E-03,.2993000E-03,&
     & .1990400E-03,.2131100E-03,.2279700E-03,.2438000E-03,.2603000E-03/

      data   absb(121:235, 7)   /                                       &
     & .1700000E-03,.1830000E-03,.1966600E-03,.2112800E-03,.2264100E-03,&
     & .1452800E-03,.1570400E-03,.1697500E-03,.1832000E-03,.1973500E-03,&
     & .1241400E-03,.1349600E-03,.1465400E-03,.1589900E-03,.1721100E-03,&
     & .1056700E-03,.1154700E-03,.1261800E-03,.1376500E-03,.1499400E-03,&
     & .8977400E-04,.9875100E-04,.1085200E-03,.1191800E-03,.1305700E-03,&
     & .7623200E-04,.8438300E-04,.9333600E-04,.1031400E-03,.1138100E-03,&
     & .6437000E-04,.7168400E-04,.7979700E-04,.8885000E-04,.9873300E-04,&
     & .5412500E-04,.6068000E-04,.6804200E-04,.7631300E-04,.8543200E-04,&
     & .4537400E-04,.5128000E-04,.5793200E-04,.6542900E-04,.7385200E-04,&
     & .3786100E-04,.4307800E-04,.4901600E-04,.5584400E-04,.6353800E-04,&
     & .3134200E-04,.3587900E-04,.4117500E-04,.4725200E-04,.5428600E-04,&
     & .2582100E-04,.2977700E-04,.3445800E-04,.3988000E-04,.4621800E-04,&
     & .2118000E-04,.2461700E-04,.2871900E-04,.3350700E-04,.3918400E-04,&
     & .1737700E-04,.2034500E-04,.2391600E-04,.2817700E-04,.3323700E-04,&
     & .1422300E-04,.1679000E-04,.1986800E-04,.2365600E-04,.2816300E-04,&
     & .1159300E-04,.1378800E-04,.1647900E-04,.1977300E-04,.2376500E-04,&
     & .9414100E-05,.1127900E-04,.1359100E-04,.1644700E-04,.1996900E-04,&
     & .7662200E-05,.9241700E-05,.1123300E-04,.1370900E-04,.1681900E-04,&
     & .6231500E-05,.7567100E-05,.9269900E-05,.1142900E-04,.1418600E-04,&
     & .5047300E-05,.6176600E-05,.7631600E-05,.9497900E-05,.1189000E-04,&
     & .4077300E-05,.5024200E-05,.6254000E-05,.7859400E-05,.9937800E-05,&
     & .3293700E-05,.4086200E-05,.5122800E-05,.6499200E-05,.8311200E-05,&
     & .2759300E-05,.3450500E-05,.4371600E-05,.5603500E-05,.7260900E-05/

      data   absb(  1:120, 8)   /                                       &
     & .2599000E-01,.2599700E-01,.2602400E-01,.2603800E-01,.2613200E-01,&
     & .2264800E-01,.2273100E-01,.2280300E-01,.2286100E-01,.2302000E-01,&
     & .1964100E-01,.1981200E-01,.1988000E-01,.1999800E-01,.2014100E-01,&
     & .1690900E-01,.1703000E-01,.1714800E-01,.1729600E-01,.1750200E-01,&
     & .1436500E-01,.1450400E-01,.1464300E-01,.1480800E-01,.1499600E-01,&
     & .1207200E-01,.1220700E-01,.1234600E-01,.1251300E-01,.1274200E-01,&
     & .1013500E-01,.1026300E-01,.1040200E-01,.1060200E-01,.1079400E-01,&
     & .8650800E-02,.8795100E-02,.8976500E-02,.9153000E-02,.9344700E-02,&
     & .7422800E-02,.7596400E-02,.7780500E-02,.7944100E-02,.8126600E-02,&
     & .6394900E-02,.6592000E-02,.6758400E-02,.6929800E-02,.7096800E-02,&
     & .5538900E-02,.5716500E-02,.5883400E-02,.6049000E-02,.6216300E-02,&
     & .4798000E-02,.4969600E-02,.5133300E-02,.5305500E-02,.5469500E-02,&
     & .4167600E-02,.4335700E-02,.4499900E-02,.4674900E-02,.4817900E-02,&
     & .3639100E-02,.3807900E-02,.3983400E-02,.4137400E-02,.4275400E-02,&
     & .3191700E-02,.3358000E-02,.3523700E-02,.3673700E-02,.3798200E-02,&
     & .2809600E-02,.2970900E-02,.3124200E-02,.3261300E-02,.3382900E-02,&
     & .2478600E-02,.2627200E-02,.2770600E-02,.2898600E-02,.3013000E-02,&
     & .2190700E-02,.2328200E-02,.2457400E-02,.2577800E-02,.2680600E-02,&
     & .1934600E-02,.2056900E-02,.2175700E-02,.2280500E-02,.2374200E-02,&
     & .1710100E-02,.1824700E-02,.1930800E-02,.2025600E-02,.2117600E-02,&
     & .1517100E-02,.1616800E-02,.1712100E-02,.1803500E-02,.1891100E-02,&
     & .1346600E-02,.1439000E-02,.1524600E-02,.1608300E-02,.1692100E-02,&
     & .1187100E-02,.1269800E-02,.1353400E-02,.1432000E-02,.1507100E-02,&
     & .1038800E-02,.1117900E-02,.1193400E-02,.1264700E-02,.1339100E-02/

      data   absb(121:235, 8)   /                                       &
     & .9223300E-03,.9985400E-03,.1072200E-02,.1142600E-02,.1211800E-02,&
     & .8184300E-03,.8923600E-03,.9660500E-03,.1034000E-02,.1100300E-02,&
     & .7273800E-03,.7977700E-03,.8699700E-03,.9349600E-03,.9985200E-03,&
     & .6514900E-03,.7209800E-03,.7923500E-03,.8611600E-03,.9247400E-03,&
     & .5831800E-03,.6515400E-03,.7216000E-03,.7940000E-03,.8597300E-03,&
     & .5212700E-03,.5877000E-03,.6578100E-03,.7311000E-03,.8016500E-03,&
     & .4640400E-03,.5296000E-03,.5988600E-03,.6736600E-03,.7472700E-03,&
     & .4122900E-03,.4760900E-03,.5442400E-03,.6191800E-03,.6936600E-03,&
     & .3653400E-03,.4262400E-03,.4937400E-03,.5666800E-03,.6436400E-03,&
     & .3213400E-03,.3798700E-03,.4459700E-03,.5166900E-03,.5946900E-03,&
     & .2785500E-03,.3353100E-03,.3985500E-03,.4683200E-03,.5456200E-03,&
     & .2409400E-03,.2944000E-03,.3547700E-03,.4228400E-03,.4976700E-03,&
     & .2071000E-03,.2571200E-03,.3142500E-03,.3809800E-03,.4546500E-03,&
     & .1775700E-03,.2242200E-03,.2790600E-03,.3423100E-03,.4141900E-03,&
     & .1519800E-03,.1953800E-03,.2475800E-03,.3079600E-03,.3776500E-03,&
     & .1294300E-03,.1690700E-03,.2181200E-03,.2758500E-03,.3428400E-03,&
     & .1094600E-03,.1456000E-03,.1910300E-03,.2463300E-03,.3104900E-03,&
     & .9295000E-04,.1259900E-03,.1684800E-03,.2210900E-03,.2826600E-03,&
     & .7872200E-04,.1088300E-03,.1483100E-03,.1979000E-03,.2577000E-03,&
     & .6632300E-04,.9365100E-04,.1298300E-03,.1763800E-03,.2339300E-03,&
     & .5564400E-04,.8008500E-04,.1134900E-03,.1574100E-03,.2127400E-03,&
     & .4653700E-04,.6844200E-04,.9919000E-04,.1404500E-03,.1932700E-03,&
     & .4155800E-04,.6250300E-04,.9237700E-04,.1325800E-03,.1852800E-03/

      data   absb(  1:120, 9)   /                                       &
     & .1913712E+00,.1940307E+00,.1961583E+00,.1981395E+00,.1992614E+00,&
     & .1688802E+00,.1712855E+00,.1734248E+00,.1752061E+00,.1762731E+00,&
     & .1486065E+00,.1508840E+00,.1528820E+00,.1544423E+00,.1558272E+00,&
     & .1299204E+00,.1323469E+00,.1341015E+00,.1357898E+00,.1371436E+00,&
     & .1134080E+00,.1156327E+00,.1174880E+00,.1190475E+00,.1205541E+00,&
     & .9849405E-01,.1005410E+00,.1023631E+00,.1040376E+00,.1053612E+00,&
     & .8553764E-01,.8752614E-01,.8929406E-01,.9077432E-01,.9251699E-01,&
     & .7472428E-01,.7670302E-01,.7818691E-01,.7985497E-01,.8155359E-01,&
     & .6533778E-01,.6717463E-01,.6889333E-01,.7050656E-01,.7229544E-01,&
     & .5740823E-01,.5919933E-01,.6095743E-01,.6261078E-01,.6444982E-01,&
     & .5056393E-01,.5229127E-01,.5411951E-01,.5593955E-01,.5779249E-01,&
     & .4478827E-01,.4654763E-01,.4835743E-01,.5018840E-01,.5221163E-01,&
     & .3995582E-01,.4170732E-01,.4352502E-01,.4537017E-01,.4745831E-01,&
     & .3584681E-01,.3763708E-01,.3940778E-01,.4140061E-01,.4366003E-01,&
     & .3241328E-01,.3414397E-01,.3597457E-01,.3798590E-01,.4030971E-01,&
     & .2947190E-01,.3126308E-01,.3314326E-01,.3529197E-01,.3770177E-01,&
     & .2705105E-01,.2882380E-01,.3077801E-01,.3304753E-01,.3559583E-01,&
     & .2495554E-01,.2671043E-01,.2883204E-01,.3115120E-01,.3377072E-01,&
     & .2314399E-01,.2503929E-01,.2721446E-01,.2967647E-01,.3240495E-01,&
     & .2168424E-01,.2363375E-01,.2593698E-01,.2846534E-01,.3128868E-01,&
     & .2052923E-01,.2253442E-01,.2489053E-01,.2751815E-01,.3055501E-01,&
     & .1954963E-01,.2161307E-01,.2412847E-01,.2681518E-01,.3006419E-01,&
     & .1858710E-01,.2078212E-01,.2330820E-01,.2617072E-01,.2946078E-01,&
     & .1751615E-01,.1979783E-01,.2246608E-01,.2526516E-01,.2885677E-01/

      data   absb(121:235, 9)   /                                       &
     & .1645401E-01,.1873164E-01,.2142687E-01,.2424787E-01,.2779912E-01,&
     & .1549653E-01,.1781459E-01,.2042772E-01,.2337789E-01,.2680098E-01,&
     & .1464498E-01,.1698778E-01,.1958951E-01,.2255141E-01,.2592896E-01,&
     & .1374930E-01,.1598681E-01,.1858564E-01,.2145636E-01,.2483064E-01,&
     & .1297376E-01,.1509265E-01,.1767753E-01,.2043917E-01,.2378184E-01,&
     & .1225680E-01,.1431909E-01,.1681124E-01,.1957355E-01,.2273600E-01,&
     & .1154426E-01,.1346806E-01,.1585020E-01,.1858172E-01,.2161718E-01,&
     & .1073729E-01,.1273625E-01,.1500546E-01,.1763766E-01,.2052634E-01,&
     & .1003661E-01,.1202377E-01,.1415118E-01,.1671879E-01,.1956038E-01,&
     & .9280320E-02,.1124463E-01,.1334817E-01,.1580509E-01,.1857998E-01,&
     & .8562651E-02,.1045470E-01,.1255035E-01,.1487256E-01,.1749111E-01,&
     & .7862421E-02,.9650932E-02,.1175305E-01,.1399227E-01,.1656834E-01,&
     & .7176180E-02,.8968905E-02,.1094204E-01,.1318820E-01,.1566044E-01,&
     & .6553611E-02,.8323164E-02,.1026176E-01,.1243120E-01,.1484228E-01,&
     & .6016277E-02,.7685907E-02,.9617875E-02,.1169791E-01,.1410813E-01,&
     & .5488131E-02,.7123060E-02,.9006212E-02,.1103169E-01,.1339416E-01,&
     & .5039153E-02,.6572314E-02,.8368057E-02,.1042322E-01,.1261228E-01,&
     & .4678553E-02,.6104245E-02,.7858028E-02,.9862751E-02,.1203004E-01,&
     & .4397629E-02,.5698528E-02,.7389611E-02,.9348370E-02,.1154387E-01,&
     & .4125069E-02,.5371631E-02,.6928103E-02,.8831411E-02,.1098275E-01,&
     & .3904687E-02,.5099382E-02,.6546356E-02,.8396888E-02,.1047269E-01,&
     & .3677700E-02,.4862089E-02,.6220196E-02,.7951650E-02,.1006273E-01,&
     & .3646869E-02,.4855547E-02,.6218200E-02,.7940987E-02,.1000532E-01/

      data   absb(  1:120,10)   /                                       &
     & .6563434E+01,.6536173E+01,.6509225E+01,.6480775E+01,.6453855E+01,&
     & .6694993E+01,.6668788E+01,.6641684E+01,.6613791E+01,.6586857E+01,&
     & .6814067E+01,.6788196E+01,.6761389E+01,.6734157E+01,.6706224E+01,&
     & .6922709E+01,.6896320E+01,.6870266E+01,.6842505E+01,.6814432E+01,&
     & .7018842E+01,.6992813E+01,.6966297E+01,.6939182E+01,.6910339E+01,&
     & .7104680E+01,.7079554E+01,.7053120E+01,.7025060E+01,.6997053E+01,&
     & .7178134E+01,.7153173E+01,.7126816E+01,.7099394E+01,.7069733E+01,&
     & .7237652E+01,.7212460E+01,.7187023E+01,.7158873E+01,.7129293E+01,&
     & .7288276E+01,.7263276E+01,.7236743E+01,.7208943E+01,.7178798E+01,&
     & .7329577E+01,.7304627E+01,.7277602E+01,.7249474E+01,.7218919E+01,&
     & .7364634E+01,.7339549E+01,.7311831E+01,.7282930E+01,.7252217E+01,&
     & .7393382E+01,.7367435E+01,.7340013E+01,.7310737E+01,.7279210E+01,&
     & .7416720E+01,.7390784E+01,.7362566E+01,.7332879E+01,.7300963E+01,&
     & .7435429E+01,.7408936E+01,.7380806E+01,.7350034E+01,.7317321E+01,&
     & .7450559E+01,.7423883E+01,.7395076E+01,.7363982E+01,.7330904E+01,&
     & .7462543E+01,.7435279E+01,.7405991E+01,.7374365E+01,.7340536E+01,&
     & .7471315E+01,.7443887E+01,.7414078E+01,.7381879E+01,.7347478E+01,&
     & .7478800E+01,.7450609E+01,.7419846E+01,.7387182E+01,.7352811E+01,&
     & .7483666E+01,.7454950E+01,.7423611E+01,.7390401E+01,.7355051E+01,&
     & .7487253E+01,.7457765E+01,.7425717E+01,.7391895E+01,.7356050E+01,&
     & .7488506E+01,.7458765E+01,.7426508E+01,.7392054E+01,.7355156E+01,&
     & .7489850E+01,.7459250E+01,.7426035E+01,.7391305E+01,.7353645E+01,&
     & .7492443E+01,.7461624E+01,.7428244E+01,.7392628E+01,.7354878E+01,&
     & .7497726E+01,.7466429E+01,.7432747E+01,.7397279E+01,.7358181E+01/

      data   absb(121:235,10)   /                                       &
     & .7505737E+01,.7474800E+01,.7440929E+01,.7405596E+01,.7366801E+01,&
     & .7513970E+01,.7482707E+01,.7449119E+01,.7413672E+01,.7375439E+01,&
     & .7520741E+01,.7489941E+01,.7456502E+01,.7420983E+01,.7383218E+01,&
     & .7530440E+01,.7499850E+01,.7466874E+01,.7431912E+01,.7393870E+01,&
     & .7538723E+01,.7509468E+01,.7476827E+01,.7442592E+01,.7405041E+01,&
     & .7547204E+01,.7518633E+01,.7486628E+01,.7452286E+01,.7415815E+01,&
     & .7556213E+01,.7528691E+01,.7497855E+01,.7463955E+01,.7428134E+01,&
     & .7565953E+01,.7538872E+01,.7508898E+01,.7475761E+01,.7440938E+01,&
     & .7575062E+01,.7548586E+01,.7519907E+01,.7487622E+01,.7453098E+01,&
     & .7584131E+01,.7558667E+01,.7530653E+01,.7499618E+01,.7465741E+01,&
     & .7593779E+01,.7569486E+01,.7542361E+01,.7512145E+01,.7479280E+01,&
     & .7602657E+01,.7580065E+01,.7553432E+01,.7524179E+01,.7492239E+01,&
     & .7610916E+01,.7589199E+01,.7564289E+01,.7535843E+01,.7505070E+01,&
     & .7618668E+01,.7597891E+01,.7574135E+01,.7546664E+01,.7516426E+01,&
     & .7625181E+01,.7605859E+01,.7582588E+01,.7556608E+01,.7527157E+01,&
     & .7630829E+01,.7612911E+01,.7590839E+01,.7566252E+01,.7537672E+01,&
     & .7636310E+01,.7619845E+01,.7598957E+01,.7575011E+01,.7547925E+01,&
     & .7640632E+01,.7625839E+01,.7606042E+01,.7583011E+01,.7556788E+01,&
     & .7644312E+01,.7630300E+01,.7612179E+01,.7589767E+01,.7564669E+01,&
     & .7647263E+01,.7634539E+01,.7617801E+01,.7596900E+01,.7572581E+01,&
     & .7650044E+01,.7638295E+01,.7623237E+01,.7603393E+01,.7580008E+01,&
     & .7651936E+01,.7641531E+01,.7627671E+01,.7609561E+01,.7586309E+01,&
     & .7652501E+01,.7642169E+01,.7629030E+01,.7611100E+01,.7588572E+01/

!  ---
      data   forref(1:4,1:10)    /                                      &
     & .2145040E-06,.4604180E-06,.3576080E-05,.1920370E-05,.1425760E-05,&
     & .3644630E-05,.1170330E-04,.1120850E-04,.1015360E-04,.1240960E-04,&
     & .5091900E-04,.5652820E-04,.1433940E-03,.1547000E-03,.4664980E-03,&
     & .9188290E-03,.2516310E-02,.2417290E-02,.2400570E-02,.3504080E-02,&
     & .4103090E-02,.4168510E-02,.3909250E-02,.3836940E-02,.4453870E-02,&
     & .4486570E-02,.4323100E-02,.3707390E-02,.4581500E-02,.4600140E-02,&
     & .4502450E-02,.3367180E-02,.4682248E-02,.4680093E-02,.4684726E-02,&
     & .3680135E-02,.5243173E-02,.5159096E-02,.5132896E-02,.3931064E-02/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 10).

      data   selfref(1:10,1:10)  /                                      &
     & .2170580E-03,.1763910E-03,.1433420E-03,.1164860E-03,.9466140E-04,&
     & .7692570E-04,.6251310E-04,.5080070E-04,.4128280E-04,.3354810E-04,&
     & .5980550E-03,.4848050E-03,.3930000E-03,.3185800E-03,.2582520E-03,&
     & .2093480E-03,.1697050E-03,.1375690E-03,.1115180E-03,.9040080E-04,&
     & .1026910E-02,.9302810E-03,.8427400E-03,.7634370E-03,.6915960E-03,&
     & .6265160E-03,.5675600E-03,.5141520E-03,.4657690E-03,.4219400E-03,&
     & .3885690E-02,.3650980E-02,.3430450E-02,.3223240E-02,.3028540E-02,&
     & .2845610E-02,.2673720E-02,.2512220E-02,.2360470E-02,.2217890E-02,&
     & .3498450E-01,.3266780E-01,.3050450E-01,.2848450E-01,.2659820E-01,&
     & .2483690E-01,.2319210E-01,.2165630E-01,.2022220E-01,.1888310E-01,&
     & .6137050E-01,.5626760E-01,.5158900E-01,.4729940E-01,.4336650E-01,&
     & .3976060E-01,.3645450E-01,.3342330E-01,.3064420E-01,.2809610E-01,&
     & .6569810E-01,.6026600E-01,.5528300E-01,.5071200E-01,.4651900E-01,&
     & .4267260E-01,.3914430E-01,.3590770E-01,.3293870E-01,.3021530E-01,&
     & .6717820E-01,.6164610E-01,.5656950E-01,.5191100E-01,.4763610E-01,&
     & .4371320E-01,.4011340E-01,.3681000E-01,.3377870E-01,.3099700E-01,&
     & .6790842E-01,.6238861E-01,.5731740E-01,.5265845E-01,.4837825E-01,&
     & .4444589E-01,.4083320E-01,.3751416E-01,.3446487E-01,.3166347E-01,&
     & .7454175E-01,.6857684E-01,.6309064E-01,.5804463E-01,.5340338E-01,&
     & .4913438E-01,.4520762E-01,.4159562E-01,.3827306E-01,.3521673E-01/

!  ---  ch4
      data   absch4(1:10)        /                                      &
     & .1013810E-02,.6336920E-02,.1941850E-01,.4832100E-01,.2365740E-02,&
     & .6619730E-03,.5645520E-03,.2831830E-03,.6714756E-04,.2647642E-06/

!........................................!
      end module module_radsw_kgb20      !
!========================================!


!========================================!
      module module_radsw_kgb21          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 21:  6150-7700 cm-1 (low - j2o,co2; high - h2o,co2)        !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG21

!
      implicit none
!
      private
!
      integer,  public :: MSA21, MSB21, MSF21, MFR21
      parameter (MSA21=585, MSB21=1175, MSF21=10, MFR21=4)

      real (kind=kind_phys), public ::           forref(MFR21,NG21),    &
     &       absa(MSA21,NG21), absb(MSB21,NG21), selfref(MSF21,NG21)

!  ---  rayleigh extinction coefficient at v = 6925 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 9.41e-09

!     the array absa(585,NG21) (ka((9,5,13,NG21)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 10, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .3148200E-07,.6401000E-05,.9501700E-05,.1173500E-04,.1356100E-04,&
     & .1507700E-04,.1612100E-04,.1613800E-04,.4168700E-06,.2979100E-07,&
     & .6820600E-05,.1026000E-04,.1276400E-04,.1488000E-04,.1666500E-04,&
     & .1794400E-04,.1817000E-04,.4424600E-06,.2827200E-07,.7244800E-05,&
     & .1108300E-04,.1386600E-04,.1625600E-04,.1832300E-04,.1988600E-04,&
     & .2022900E-04,.4747200E-06,.2690000E-07,.7693500E-05,.1193400E-04,&
     & .1501500E-04,.1765700E-04,.2003800E-04,.2190200E-04,.2236100E-04,&
     & .5077000E-06,.2565600E-07,.8166100E-05,.1276900E-04,.1618100E-04,&
     & .1912700E-04,.2175500E-04,.2396300E-04,.2442100E-04,.5420500E-06,&
     & .2639600E-07,.5222500E-05,.7722100E-05,.9515000E-05,.1100400E-04,&
     & .1219500E-04,.1303700E-04,.1298500E-04,.3209700E-06,.2494500E-07,&
     & .5573700E-05,.8361200E-05,.1037500E-04,.1211600E-04,.1356900E-04,&
     & .1456100E-04,.1471100E-04,.3457900E-06,.2364600E-07,.5933300E-05,&
     & .9053100E-05,.1130100E-04,.1324700E-04,.1495600E-04,.1619300E-04,&
     & .1639600E-04,.3720000E-06,.2247600E-07,.6314400E-05,.9773200E-05,&
     & .1227400E-04,.1443600E-04,.1637700E-04,.1787900E-04,.1822300E-04,&
     & .4005100E-06,.2141500E-07,.6713300E-05,.1047100E-04,.1326400E-04,&
     & .1568100E-04,.1784300E-04,.1962700E-04,.1995400E-04,.4292900E-06,&
     & .2239700E-07,.4193000E-05,.6155900E-05,.7554400E-05,.8690200E-05,&
     & .9589600E-05,.1026900E-04,.1011400E-04,.2414800E-06,.2112400E-07,&
     & .4483700E-05,.6678600E-05,.8247500E-05,.9635300E-05,.1074200E-04,&
     & .1146900E-04,.1155600E-04,.2630400E-06,.1998800E-07,.4783900E-05,&
     & .7249900E-05,.9022900E-05,.1056000E-04,.1190000E-04,.1285900E-04,&
     & .1293500E-04,.2861800E-06,.1896800E-07,.5104000E-05,.7858600E-05,&
     & .9830800E-05,.1155100E-04,.1309400E-04,.1424000E-04,.1445500E-04,&
     & .3098300E-06,.1804700E-07,.5436600E-05,.8446200E-05,.1067000E-04,&
     & .1258500E-04,.1431300E-04,.1569500E-04,.1593400E-04,.3325500E-06,&
     & .1908000E-07,.3340900E-05,.4873800E-05,.5935400E-05,.6787700E-05,&
     & .7490000E-05,.7914200E-05,.7720000E-05,.1846600E-06,.1795400E-07,&
     & .3591000E-05,.5296000E-05,.6503700E-05,.7574400E-05,.8408200E-05,&
     & .8942200E-05,.8930200E-05,.2029200E-06,.1695400E-07,.3831700E-05,&
     & .5753600E-05,.7132200E-05,.8342400E-05,.9378800E-05,.1006300E-04,&
     & .1009800E-04,.2217300E-06,.1605900E-07,.4093200E-05,.6258600E-05,&
     & .7796100E-05,.9147500E-05,.1034700E-04,.1122200E-04,.1131200E-04,&
     & .2400500E-06,.1525400E-07,.4367800E-05,.6752500E-05,.8487300E-05,&
     & .9999500E-05,.1135300E-04,.1241600E-04,.1258600E-04,.2593900E-06/

      data   absa(181:315, 1)   /                                       &
     & .1626100E-07,.2654000E-05,.3854100E-05,.4644300E-05,.5296100E-05,&
     & .5805400E-05,.6071800E-05,.5854000E-05,.1427400E-06,.1526500E-07,&
     & .2859800E-05,.4183100E-05,.5122100E-05,.5911000E-05,.6520700E-05,&
     & .6976100E-05,.6868500E-05,.1572500E-06,.1438400E-07,.3062300E-05,&
     & .4552800E-05,.5612400E-05,.6565000E-05,.7343600E-05,.7820800E-05,&
     & .7852300E-05,.1727100E-06,.1359900E-07,.3272000E-05,.4962200E-05,&
     & .6160500E-05,.7224400E-05,.8158200E-05,.8797100E-05,.8809600E-05,&
     & .1876600E-06,.1289500E-07,.3498500E-05,.5383300E-05,.6724000E-05,&
     & .7909100E-05,.8968800E-05,.9769800E-05,.9863900E-05,.2037500E-06,&
     & .1389800E-07,.2098100E-05,.3038400E-05,.3604100E-05,.4104000E-05,&
     & .4433400E-05,.4576100E-05,.4395000E-05,.1102500E-06,.1301100E-07,&
     & .2264500E-05,.3289000E-05,.4004200E-05,.4585100E-05,.5047300E-05,&
     & .5325000E-05,.5185400E-05,.1222700E-06,.1223100E-07,.2432200E-05,&
     & .3584200E-05,.4391500E-05,.5125900E-05,.5690300E-05,.6034300E-05,&
     & .6016600E-05,.1350600E-06,.1153900E-07,.2603200E-05,.3910700E-05,&
     & .4832100E-05,.5652500E-05,.6362500E-05,.6807800E-05,.6802400E-05,&
     & .1471900E-06,.1092100E-07,.2786600E-05,.4257800E-05,.5291900E-05,&
     & .6218100E-05,.7036200E-05,.7614000E-05,.7651900E-05,.1605200E-06,&
     & .1188600E-07,.1656300E-05,.2386000E-05,.2827600E-05,.3144900E-05,&
     & .3377100E-05,.3430500E-05,.3282400E-05,.8494300E-07,.1109600E-07,&
     & .1787400E-05,.2586100E-05,.3110000E-05,.3542500E-05,.3873700E-05,&
     & .4046800E-05,.3893300E-05,.9473400E-07,.1040500E-07,.1928600E-05,&
     & .2816600E-05,.3441700E-05,.3978600E-05,.4380400E-05,.4675900E-05,&
     & .4583500E-05,.1053100E-06,.9794800E-08,.2068000E-05,.3074100E-05,&
     & .3780300E-05,.4416300E-05,.4942000E-05,.5247700E-05,.5250900E-05,&
     & .1151700E-06,.9252200E-08,.2214000E-05,.3355800E-05,.4154400E-05,&
     & .4871900E-05,.5496800E-05,.5922500E-05,.5902700E-05,.1261800E-06/

      data   absa(316:450, 1)   /                                       &
     & .1016400E-07,.1305600E-05,.1869000E-05,.2194000E-05,.2421000E-05,&
     & .2550400E-05,.2567600E-05,.2444400E-05,.6602100E-07,.9461200E-08,&
     & .1410600E-05,.2039800E-05,.2413600E-05,.2742400E-05,.2957800E-05,&
     & .3044600E-05,.2918200E-05,.7398500E-07,.8848900E-08,.1525600E-05,&
     & .2212300E-05,.2689400E-05,.3070400E-05,.3380800E-05,.3552100E-05,&
     & .3452600E-05,.8266600E-07,.8311100E-08,.1639600E-05,.2415900E-05,&
     & .2956300E-05,.3450400E-05,.3819600E-05,.4042500E-05,.4009300E-05,&
     & .9094000E-07,.7834900E-08,.1757000E-05,.2639800E-05,.3254700E-05,&
     & .3806000E-05,.4281300E-05,.4573200E-05,.4546500E-05,.1002200E-06,&
     & .8698300E-08,.1028400E-05,.1473400E-05,.1697300E-05,.1855100E-05,&
     & .1922300E-05,.1922200E-05,.1828900E-05,.5346000E-07,.8071200E-08,&
     & .1114100E-05,.1601600E-05,.1890200E-05,.2102700E-05,.2250400E-05,&
     & .2280000E-05,.2174900E-05,.6018000E-07,.7528400E-08,.1203300E-05,&
     & .1737600E-05,.2084900E-05,.2371900E-05,.2590600E-05,.2697700E-05,&
     & .2591000E-05,.6688600E-07,.7054100E-08,.1297900E-05,.1895900E-05,&
     & .2309000E-05,.2669500E-05,.2936700E-05,.3123900E-05,.3057400E-05,&
     & .7397800E-07,.6635900E-08,.1393700E-05,.2072000E-05,.2544800E-05,&
     & .2968000E-05,.3320800E-05,.3519100E-05,.3509700E-05,.8172400E-07,&
     & .7400600E-08,.8170500E-06,.1161000E-05,.1332500E-05,.1430500E-05,&
     & .1466800E-05,.1464200E-05,.1376500E-05,.4472400E-07,.6847700E-08,&
     & .8830600E-06,.1264000E-05,.1476800E-05,.1632900E-05,.1720900E-05,&
     & .1730600E-05,.1644100E-05,.5068800E-07,.6371700E-08,.9552200E-06,&
     & .1378300E-05,.1630300E-05,.1854600E-05,.1996300E-05,.2057000E-05,&
     & .1967800E-05,.5605200E-07,.5957600E-08,.1033100E-05,.1497500E-05,&
     & .1818000E-05,.2077000E-05,.2281300E-05,.2402100E-05,.2335000E-05,&
     & .6241200E-07,.5594000E-08,.1110200E-05,.1637000E-05,.1999900E-05,&
     & .2334900E-05,.2590000E-05,.2732800E-05,.2705700E-05,.6946600E-07/

      data   absa(451:585, 1)   /                                       &
     & .6061700E-08,.6716700E-06,.9559400E-06,.1093900E-05,.1176700E-05,&
     & .1205900E-05,.1203200E-05,.1131400E-05,.3709300E-07,.5608700E-08,&
     & .7270400E-06,.1040600E-05,.1213700E-05,.1343400E-05,.1417100E-05,&
     & .1424600E-05,.1352100E-05,.4191800E-07,.5218700E-08,.7868100E-06,&
     & .1133800E-05,.1341900E-05,.1525400E-05,.1642600E-05,.1695000E-05,&
     & .1620300E-05,.4633900E-07,.4879300E-08,.8501300E-06,.1232700E-05,&
     & .1497100E-05,.1710900E-05,.1877600E-05,.1978400E-05,.1924000E-05,&
     & .5172500E-07,.4581500E-08,.9142000E-06,.1342700E-05,.1645800E-05,&
     & .1922100E-05,.2132300E-05,.2249000E-05,.2226300E-05,.5755500E-07,&
     & .4962900E-08,.5522300E-06,.7867700E-06,.8978900E-06,.9677800E-06,&
     & .9914800E-06,.9888300E-06,.9302800E-06,.3061000E-07,.4592000E-08,&
     & .5983200E-06,.8563200E-06,.9973000E-06,.1105200E-05,.1166500E-05,&
     & .1172900E-05,.1112500E-05,.3455300E-07,.4272700E-08,.6476000E-06,&
     & .9331500E-06,.1104100E-05,.1255200E-05,.1352100E-05,.1395600E-05,&
     & .1333300E-05,.3817600E-07,.3994900E-08,.6992600E-06,.1011000E-05,&
     & .1230400E-05,.1408300E-05,.1544200E-05,.1628300E-05,.1583700E-05,&
     & .4264800E-07,.3751000E-08,.7523900E-06,.1100600E-05,.1353500E-05,&
     & .1578500E-05,.1754700E-05,.1850100E-05,.1829800E-05,.4745200E-07,&
     & .4063300E-08,.4537800E-06,.6471900E-06,.7363700E-06,.7957100E-06,&
     & .8150300E-06,.8126400E-06,.7648800E-06,.2508000E-07,.3759600E-08,&
     & .4920800E-06,.7043000E-06,.8189400E-06,.9085600E-06,.9591900E-06,&
     & .9643500E-06,.9141100E-06,.2829300E-07,.3498200E-08,.5325900E-06,&
     & .7637000E-06,.9076100E-06,.1031600E-05,.1111600E-05,.1147200E-05,&
     & .1095600E-05,.3126700E-07,.3270700E-08,.5748800E-06,.8283300E-06,&
     & .1009600E-05,.1157700E-05,.1268800E-05,.1338800E-05,.1301500E-05,&
     & .3493000E-07,.3071000E-08,.6183200E-06,.9025000E-06,.1112000E-05,&
     & .1296200E-05,.1442400E-05,.1520600E-05,.1503200E-05,.3886100E-07/

      data   absa(  1:180, 2)   /                                       &
     & .1421500E-06,.6572600E-04,.1132700E-03,.1532400E-03,.1847300E-03,&
     & .2065800E-03,.2160300E-03,.2061700E-03,.1325200E-04,.1345200E-06,&
     & .6952100E-04,.1205900E-03,.1634200E-03,.1961300E-03,.2198700E-03,&
     & .2294600E-03,.2187900E-03,.1481600E-04,.1276600E-06,.7331300E-04,&
     & .1278600E-03,.1730300E-03,.2075400E-03,.2332600E-03,.2432400E-03,&
     & .2295600E-03,.1653600E-04,.1214700E-06,.7708700E-04,.1352100E-03,&
     & .1826300E-03,.2190400E-03,.2461400E-03,.2563700E-03,.2406000E-03,&
     & .1809200E-04,.1158400E-06,.8094600E-04,.1423300E-03,.1923000E-03,&
     & .2304700E-03,.2586900E-03,.2690400E-03,.2513900E-03,.1976800E-04,&
     & .1191800E-06,.5516700E-04,.9455700E-04,.1276000E-03,.1532200E-03,&
     & .1714800E-03,.1789400E-03,.1704900E-03,.1071000E-04,.1126400E-06,&
     & .5858100E-04,.1011300E-03,.1364800E-03,.1633500E-03,.1828800E-03,&
     & .1905900E-03,.1813100E-03,.1206700E-04,.1067800E-06,.6193500E-04,&
     & .1075600E-03,.1449700E-03,.1733500E-03,.1943700E-03,.2024500E-03,&
     & .1907000E-03,.1355300E-04,.1014900E-06,.6526100E-04,.1140400E-03,&
     & .1533100E-03,.1832300E-03,.2055900E-03,.2138500E-03,.2002400E-03,&
     & .1480100E-04,.9669900E-07,.6869800E-04,.1203800E-03,.1616700E-03,&
     & .1930800E-03,.2163500E-03,.2247900E-03,.2096400E-03,.1624500E-04,&
     & .1011200E-06,.4539900E-04,.7722600E-04,.1039700E-03,.1246200E-03,&
     & .1395600E-03,.1453100E-03,.1381600E-03,.8363500E-05,.9538100E-07,&
     & .4835700E-04,.8298600E-04,.1117300E-03,.1333800E-03,.1489800E-03,&
     & .1553400E-03,.1478600E-03,.9491500E-05,.9025700E-07,.5135500E-04,&
     & .8872700E-04,.1192000E-03,.1421500E-03,.1585900E-03,.1654100E-03,&
     & .1562200E-03,.1068300E-04,.8565300E-07,.5432200E-04,.9436900E-04,&
     & .1264600E-03,.1507200E-03,.1683800E-03,.1753600E-03,.1643300E-03,&
     & .1176400E-04,.8149200E-07,.5730000E-04,.9994000E-04,.1335500E-03,&
     & .1592900E-03,.1780000E-03,.1849600E-03,.1723400E-03,.1297900E-04,&
     & .8613200E-07,.3691000E-04,.6234600E-04,.8362800E-04,.1002500E-03,&
     & .1122400E-03,.1170600E-03,.1110900E-03,.6471500E-05,.8106200E-07,&
     & .3949000E-04,.6727400E-04,.9038400E-04,.1078500E-03,.1202600E-03,&
     & .1254300E-03,.1194400E-03,.7399100E-05,.7655300E-07,.4205500E-04,&
     & .7226900E-04,.9695600E-04,.1152800E-03,.1283300E-03,.1339600E-03,&
     & .1269700E-03,.8389900E-05,.7251600E-07,.4469600E-04,.7723500E-04,&
     & .1032100E-03,.1227600E-03,.1367400E-03,.1426000E-03,.1338400E-03,&
     & .9305200E-05,.6888100E-07,.4727300E-04,.8211800E-04,.1093500E-03,&
     & .1301600E-03,.1450600E-03,.1508000E-03,.1406200E-03,.1031400E-04/

      data   absa(181:315, 2)   /                                       &
     & .7340100E-07,.2977200E-04,.4995700E-04,.6672000E-04,.8010400E-04,&
     & .8976100E-04,.9385300E-04,.8895300E-04,.5017100E-05,.6891600E-07,&
     & .3197600E-04,.5421700E-04,.7257000E-04,.8667100E-04,.9665300E-04,&
     & .1007300E-03,.9587500E-04,.5774600E-05,.6494500E-07,.3421900E-04,&
     & .5841700E-04,.7832700E-04,.9295000E-04,.1035800E-03,.1080300E-03,&
     & .1027400E-03,.6619700E-05,.6140500E-07,.3645700E-04,.6269400E-04,&
     & .8375600E-04,.9935600E-04,.1105800E-03,.1154200E-03,.1085800E-03,&
     & .7372900E-05,.5822900E-07,.3871300E-04,.6705900E-04,.8903300E-04,&
     & .1057300E-03,.1176400E-03,.1224300E-03,.1144000E-03,.8214900E-05,&
     & .6272300E-07,.2375000E-04,.3964300E-04,.5279600E-04,.6347800E-04,&
     & .7115100E-04,.7464000E-04,.7019300E-04,.3889900E-05,.5873500E-07,&
     & .2565700E-04,.4327400E-04,.5771900E-04,.6912400E-04,.7737500E-04,&
     & .8049800E-04,.7654300E-04,.4483300E-05,.5522200E-07,.2756300E-04,&
     & .4686600E-04,.6269900E-04,.7449400E-04,.8294600E-04,.8665300E-04,&
     & .8250900E-04,.5163500E-05,.5210300E-07,.2952800E-04,.5047000E-04,&
     & .6747300E-04,.7989900E-04,.8900400E-04,.9278600E-04,.8762800E-04,&
     & .5810400E-05,.4931700E-07,.3144400E-04,.5417500E-04,.7200000E-04,&
     & .8532200E-04,.9488500E-04,.9882600E-04,.9264800E-04,.6499700E-05,&
     & .5363400E-07,.1886200E-04,.3132500E-04,.4157500E-04,.5014100E-04,&
     & .5619000E-04,.5888200E-04,.5502600E-04,.2979500E-05,.5008400E-07,&
     & .2047400E-04,.3434200E-04,.4573200E-04,.5488400E-04,.6149400E-04,&
     & .6418700E-04,.6085700E-04,.3458900E-05,.4697300E-07,.2206300E-04,&
     & .3741100E-04,.4989300E-04,.5947500E-04,.6627900E-04,.6912900E-04,&
     & .6575800E-04,.3998300E-05,.4422400E-07,.2376400E-04,.4048100E-04,&
     & .5410700E-04,.6402600E-04,.7135000E-04,.7432800E-04,.7051300E-04,&
     & .4552600E-05,.4177700E-07,.2543400E-04,.4355500E-04,.5795800E-04,&
     & .6859900E-04,.7630400E-04,.7945600E-04,.7464900E-04,.5104600E-05/

      data   absa(316:450, 2)   /                                       &
     & .4585700E-07,.1494500E-04,.2459400E-04,.3269600E-04,.3957100E-04,&
     & .4434800E-04,.4609000E-04,.4303400E-04,.2283300E-05,.4269700E-07,&
     & .1626500E-04,.2719800E-04,.3610800E-04,.4338100E-04,.4864200E-04,&
     & .5094300E-04,.4784800E-04,.2666900E-05,.3994400E-07,.1761600E-04,&
     & .2977500E-04,.3957700E-04,.4736600E-04,.5279100E-04,.5499200E-04,&
     & .5223600E-04,.3093200E-05,.3752200E-07,.1904200E-04,.3235100E-04,&
     & .4312600E-04,.5115300E-04,.5698700E-04,.5931600E-04,.5643800E-04,&
     & .3566600E-05,.3537600E-07,.2047600E-04,.3494600E-04,.4652100E-04,&
     & .5500900E-04,.6122100E-04,.6371400E-04,.6003500E-04,.4003700E-05,&
     & .3923300E-07,.1181700E-04,.1925400E-04,.2559600E-04,.3084300E-04,&
     & .3457400E-04,.3603500E-04,.3342500E-04,.1767300E-05,.3641800E-07,&
     & .1291400E-04,.2140400E-04,.2837200E-04,.3416100E-04,.3832100E-04,&
     & .4005300E-04,.3743500E-04,.2075100E-05,.3397900E-07,.1402600E-04,&
     & .2357400E-04,.3131600E-04,.3751400E-04,.4197700E-04,.4374700E-04,&
     & .4134600E-04,.2415100E-05,.3184400E-07,.1520500E-04,.2577000E-04,&
     & .3423100E-04,.4072300E-04,.4531500E-04,.4720200E-04,.4486900E-04,&
     & .2815100E-05,.2996100E-07,.1642000E-04,.2795500E-04,.3718700E-04,&
     & .4394100E-04,.4887300E-04,.5085600E-04,.4817000E-04,.3173300E-05,&
     & .3337300E-07,.9350500E-05,.1521600E-04,.2019800E-04,.2424400E-04,&
     & .2707700E-04,.2818600E-04,.2599600E-04,.1394100E-05,.3089300E-07,&
     & .1028400E-04,.1690200E-04,.2246800E-04,.2707200E-04,.3038200E-04,&
     & .3160300E-04,.2945600E-04,.1635200E-05,.2875400E-07,.1120700E-04,&
     & .1871500E-04,.2484900E-04,.2981500E-04,.3337900E-04,.3490100E-04,&
     & .3283600E-04,.1918800E-05,.2689200E-07,.1217200E-04,.2057800E-04,&
     & .2726900E-04,.3253600E-04,.3618200E-04,.3770300E-04,.3575000E-04,&
     & .2246700E-05,.2525500E-07,.1319500E-04,.2242200E-04,.2978100E-04,&
     & .3522000E-04,.3915700E-04,.4071900E-04,.3872900E-04,.2553500E-05/

      data   absa(451:585, 2)   /                                       &
     & .2733500E-07,.7749100E-05,.1260700E-04,.1669500E-04,.2009300E-04,&
     & .2243100E-04,.2334500E-04,.2152200E-04,.1156900E-05,.2530300E-07,&
     & .8532200E-05,.1402200E-04,.1859300E-04,.2237800E-04,.2513300E-04,&
     & .2616100E-04,.2437900E-04,.1370900E-05,.2355100E-07,.9291200E-05,&
     & .1553600E-04,.2061800E-04,.2472200E-04,.2764700E-04,.2887400E-04,&
     & .2719600E-04,.1609900E-05,.2202500E-07,.1011700E-04,.1711500E-04,&
     & .2264500E-04,.2697400E-04,.2994700E-04,.3121000E-04,.2958100E-04,&
     & .1884500E-05,.2068400E-07,.1097700E-04,.1864500E-04,.2473400E-04,&
     & .2920300E-04,.3245300E-04,.3373300E-04,.3205600E-04,.2132600E-05,&
     & .2238000E-07,.6414400E-05,.1043600E-04,.1380200E-04,.1662200E-04,&
     & .1855700E-04,.1931400E-04,.1779300E-04,.9590600E-06,.2071600E-07,&
     & .7060300E-05,.1162500E-04,.1538500E-04,.1850300E-04,.2077600E-04,&
     & .2163000E-04,.2015200E-04,.1136300E-05,.1928200E-07,.7696000E-05,&
     & .1288600E-04,.1708700E-04,.2047500E-04,.2286700E-04,.2385600E-04,&
     & .2250600E-04,.1340000E-05,.1803200E-07,.8398300E-05,.1420600E-04,&
     & .1878300E-04,.2233400E-04,.2476400E-04,.2581500E-04,.2446100E-04,&
     & .1563700E-05,.1693400E-07,.9118600E-05,.1549100E-04,.2051300E-04,&
     & .2420100E-04,.2686400E-04,.2791700E-04,.2649100E-04,.1770700E-05,&
     & .1832300E-07,.5302200E-05,.8627300E-05,.1138500E-04,.1372900E-04,&
     & .1531800E-04,.1595200E-04,.1469300E-04,.7892700E-06,.1696100E-07,&
     & .5831600E-05,.9620400E-05,.1271700E-04,.1527500E-04,.1714100E-04,&
     & .1786300E-04,.1663900E-04,.9365900E-06,.1578700E-07,.6371000E-05,&
     & .1067500E-04,.1414000E-04,.1692500E-04,.1888100E-04,.1967500E-04,&
     & .1856400E-04,.1105000E-05,.1476400E-07,.6960700E-05,.1177900E-04,&
     & .1555400E-04,.1847500E-04,.2046100E-04,.2132000E-04,.2020700E-04,&
     & .1286800E-05,.1386500E-07,.7561100E-05,.1284300E-04,.1699100E-04,&
     & .2002900E-04,.2220800E-04,.2307900E-04,.2183800E-04,.1456500E-05/

      data   absa(  1:180, 3)   /                                       &
     & .5594800E-06,.4347200E-03,.6568900E-03,.8023700E-03,.9032600E-03,&
     & .9599500E-03,.9424200E-03,.8180100E-03,.1838000E-03,.5317500E-06,&
     & .4539500E-03,.6877900E-03,.8404000E-03,.9459800E-03,.1001100E-02,&
     & .9848100E-03,.8574800E-03,.1956700E-03,.5064800E-06,.4683500E-03,&
     & .7140300E-03,.8768400E-03,.9852000E-03,.1039300E-02,.1023300E-02,&
     & .8972900E-03,.2072500E-03,.4833000E-06,.4814300E-03,.7397900E-03,&
     & .9106300E-03,.1021200E-02,.1076200E-02,.1060000E-02,.9350700E-03,&
     & .2181400E-03,.4619500E-06,.4941300E-03,.7647000E-03,.9420800E-03,&
     & .1054700E-02,.1109900E-02,.1096000E-02,.9716400E-03,.2305600E-03,&
     & .4681300E-06,.3671100E-03,.5574300E-03,.6807000E-03,.7661400E-03,&
     & .8144200E-03,.7994000E-03,.6969100E-03,.1498800E-03,.4445000E-06,&
     & .3841300E-03,.5850200E-03,.7150600E-03,.8040900E-03,.8519100E-03,&
     & .8372200E-03,.7323100E-03,.1598500E-03,.4230100E-06,.3974400E-03,&
     & .6085200E-03,.7474100E-03,.8392000E-03,.8858800E-03,.8720600E-03,&
     & .7667700E-03,.1696800E-03,.4033500E-06,.4092700E-03,.6320000E-03,&
     & .7783500E-03,.8719000E-03,.9184300E-03,.9046700E-03,.7985900E-03,&
     & .1794800E-03,.3852900E-06,.4203400E-03,.6535300E-03,.8075800E-03,&
     & .9024800E-03,.9492900E-03,.9371500E-03,.8313600E-03,.1901800E-03,&
     & .3959100E-06,.3045900E-03,.4636400E-03,.5659000E-03,.6368000E-03,&
     & .6768100E-03,.6651700E-03,.5810600E-03,.1193600E-03,.3753800E-06,&
     & .3203600E-03,.4887400E-03,.5964900E-03,.6717000E-03,.7116100E-03,&
     & .6994300E-03,.6115900E-03,.1280400E-03,.3567500E-06,.3332100E-03,&
     & .5099900E-03,.6259100E-03,.7031900E-03,.7433300E-03,.7301500E-03,&
     & .6407000E-03,.1364200E-03,.3397800E-06,.3438200E-03,.5310200E-03,&
     & .6539100E-03,.7327800E-03,.7730300E-03,.7599000E-03,.6697900E-03,&
     & .1447700E-03,.3242200E-06,.3541600E-03,.5507100E-03,.6804200E-03,&
     & .7604100E-03,.8011800E-03,.7891400E-03,.6992300E-03,.1536500E-03,&
     & .3359800E-06,.2497000E-03,.3805300E-03,.4646400E-03,.5223400E-03,&
     & .5548800E-03,.5467500E-03,.4779400E-03,.9527500E-04,.3180000E-06,&
     & .2641500E-03,.4031300E-03,.4921000E-03,.5535900E-03,.5866100E-03,&
     & .5769400E-03,.5039300E-03,.1029200E-03,.3017700E-06,.2763800E-03,&
     & .4226800E-03,.5182300E-03,.5834800E-03,.6159100E-03,.6049700E-03,&
     & .5294700E-03,.1101600E-03,.2870300E-06,.2861200E-03,.4416500E-03,&
     & .5432600E-03,.6101000E-03,.6429900E-03,.6315100E-03,.5555800E-03,&
     & .1170800E-03,.2735600E-06,.2955000E-03,.4590300E-03,.5670300E-03,&
     & .6352200E-03,.6683900E-03,.6581200E-03,.5814200E-03,.1244400E-03/

      data   absa(181:315, 3)   /                                       &
     & .2851700E-06,.2030500E-03,.3094000E-03,.3785400E-03,.4251600E-03,&
     & .4516700E-03,.4461700E-03,.3900600E-03,.7639400E-04,.2694300E-06,&
     & .2163100E-03,.3294200E-03,.4027900E-03,.4528100E-03,.4795900E-03,&
     & .4727400E-03,.4121900E-03,.8309300E-04,.2552600E-06,.2276000E-03,&
     & .3473600E-03,.4260300E-03,.4792800E-03,.5057300E-03,.4975300E-03,&
     & .4342300E-03,.8913700E-04,.2424500E-06,.2366100E-03,.3642300E-03,&
     & .4483200E-03,.5037300E-03,.5302100E-03,.5209200E-03,.4571500E-03,&
     & .9502600E-04,.2307900E-06,.2449000E-03,.3797900E-03,.4695800E-03,&
     & .5267100E-03,.5536000E-03,.5442200E-03,.4798200E-03,.1010000E-03,&
     & .2425800E-06,.1637400E-03,.2488900E-03,.3052600E-03,.3425600E-03,&
     & .3643700E-03,.3605400E-03,.3164300E-03,.6098300E-04,.2287300E-06,&
     & .1755700E-03,.2667400E-03,.3266200E-03,.3669800E-03,.3886200E-03,&
     & .3839500E-03,.3348100E-03,.6671700E-04,.2163200E-06,.1859600E-03,&
     & .2831500E-03,.3471000E-03,.3903200E-03,.4118600E-03,.4059700E-03,&
     & .3534700E-03,.7180500E-04,.2051500E-06,.1941200E-03,.2982500E-03,&
     & .3667200E-03,.4122300E-03,.4337000E-03,.4265400E-03,.3730800E-03,&
     & .7695000E-04,.1950000E-06,.2016300E-03,.3117700E-03,.3857300E-03,&
     & .4329800E-03,.4547400E-03,.4465900E-03,.3927300E-03,.8185400E-04,&
     & .2064400E-06,.1308800E-03,.1988300E-03,.2445300E-03,.2737900E-03,&
     & .2913300E-03,.2897200E-03,.2552700E-03,.4848300E-04,.1942200E-06,&
     & .1412300E-03,.2146400E-03,.2630700E-03,.2952000E-03,.3126200E-03,&
     & .3097600E-03,.2705700E-03,.5318900E-04,.1833400E-06,.1505600E-03,&
     & .2294500E-03,.2810200E-03,.3157000E-03,.3333600E-03,.3292100E-03,&
     & .2865700E-03,.5766400E-04,.1735900E-06,.1584500E-03,.2425400E-03,&
     & .2981200E-03,.3353300E-03,.3527300E-03,.3473700E-03,.3029000E-03,&
     & .6208000E-04,.1647600E-06,.1651500E-03,.2547200E-03,.3149200E-03,&
     & .3536800E-03,.3713400E-03,.3646600E-03,.3197100E-03,.6628300E-04/

      data   absa(316:450, 3)   /                                       &
     & .1756400E-06,.1037700E-03,.1579900E-03,.1944300E-03,.2177500E-03,&
     & .2322100E-03,.2317800E-03,.2051800E-03,.3861300E-04,.1648600E-06,&
     & .1129100E-03,.1717700E-03,.2106000E-03,.2360500E-03,.2502300E-03,&
     & .2486000E-03,.2184000E-03,.4257400E-04,.1553200E-06,.1216000E-03,&
     & .1847200E-03,.2261300E-03,.2538300E-03,.2681300E-03,.2656900E-03,&
     & .2314900E-03,.4643800E-04,.1467900E-06,.1287100E-03,.1962900E-03,&
     & .2411700E-03,.2710100E-03,.2854600E-03,.2814800E-03,.2451100E-03,&
     & .5013200E-04,.1391300E-06,.1347000E-03,.2071100E-03,.2558300E-03,&
     & .2873500E-03,.3017100E-03,.2965300E-03,.2593100E-03,.5389200E-04,&
     & .1495000E-06,.8194100E-04,.1247300E-03,.1536300E-03,.1725700E-03,&
     & .1840900E-03,.1842400E-03,.1643900E-03,.3077200E-04,.1399800E-06,&
     & .8970200E-04,.1365500E-03,.1676300E-03,.1877900E-03,.1991900E-03,&
     & .1987700E-03,.1758100E-03,.3402200E-04,.1316000E-06,.9726300E-04,&
     & .1477600E-03,.1808900E-03,.2029200E-03,.2145100E-03,.2131700E-03,&
     & .1864000E-03,.3738100E-04,.1241500E-06,.1038900E-03,.1581700E-03,&
     & .1940400E-03,.2179400E-03,.2298000E-03,.2271500E-03,.1978400E-03,&
     & .4045700E-04,.1174900E-06,.1093400E-03,.1676800E-03,.2068000E-03,&
     & .2323400E-03,.2441400E-03,.2402600E-03,.2095900E-03,.4358600E-04,&
     & .1265800E-06,.6476300E-04,.9869000E-04,.1217000E-03,.1370900E-03,&
     & .1464400E-03,.1470900E-03,.1321500E-03,.2473700E-04,.1182600E-06,&
     & .7140500E-04,.1087500E-03,.1336800E-03,.1497800E-03,.1590100E-03,&
     & .1593700E-03,.1415300E-03,.2748700E-04,.1109700E-06,.7785700E-04,&
     & .1184200E-03,.1451000E-03,.1626100E-03,.1720400E-03,.1712900E-03,&
     & .1504100E-03,.3025000E-04,.1045100E-06,.8376400E-04,.1276200E-03,&
     & .1562500E-03,.1754400E-03,.1851600E-03,.1834500E-03,.1601000E-03,&
     & .3289100E-04,.9876700E-07,.8883300E-04,.1359500E-03,.1672300E-03,&
     & .1879800E-03,.1976600E-03,.1947700E-03,.1697900E-03,.3554700E-04/

      data   absa(451:585, 3)   /                                       &
     & .1036800E-06,.5371400E-04,.8174800E-04,.1008500E-03,.1134900E-03,&
     & .1212600E-03,.1218000E-03,.1095000E-03,.2078700E-04,.9685400E-07,&
     & .5917500E-04,.9013100E-04,.1108500E-03,.1242500E-03,.1316600E-03,&
     & .1320700E-03,.1173300E-03,.2302800E-04,.9088200E-07,.6459300E-04,&
     & .9827500E-04,.1204100E-03,.1349700E-03,.1427300E-03,.1421500E-03,&
     & .1247900E-03,.2536200E-04,.8560200E-07,.6954500E-04,.1059200E-03,&
     & .1298200E-03,.1458300E-03,.1538800E-03,.1522800E-03,.1329700E-03,&
     & .2758500E-04,.8088900E-07,.7373100E-04,.1129000E-03,.1391400E-03,&
     & .1563700E-03,.1643100E-03,.1618900E-03,.1411600E-03,.2982800E-04,&
     & .8488600E-07,.4441800E-04,.6764100E-04,.8349000E-04,.9390600E-04,&
     & .1003400E-03,.1007500E-03,.9061400E-04,.1735000E-04,.7929700E-07,&
     & .4898100E-04,.7465600E-04,.9182100E-04,.1029900E-03,.1089100E-03,&
     & .1093300E-03,.9722100E-04,.1923700E-04,.7440800E-07,.5355300E-04,&
     & .8150300E-04,.9987200E-04,.1119600E-03,.1183200E-03,.1178200E-03,&
     & .1034100E-03,.2118900E-04,.7008500E-07,.5769800E-04,.8788700E-04,&
     & .1078100E-03,.1210700E-03,.1277500E-03,.1262400E-03,.1103400E-03,&
     & .2303800E-04,.6622600E-07,.6117700E-04,.9369400E-04,.1156700E-03,&
     & .1299500E-03,.1364500E-03,.1343600E-03,.1172600E-03,.2491300E-04,&
     & .6949900E-07,.3671600E-04,.5590200E-04,.6906700E-04,.7763500E-04,&
     & .8293400E-04,.8330200E-04,.7490500E-04,.1441900E-04,.6492300E-07,&
     & .4051100E-04,.6177900E-04,.7598400E-04,.8523900E-04,.9009000E-04,&
     & .9045600E-04,.8026400E-04,.1599600E-04,.6092000E-07,.4435600E-04,&
     & .6753300E-04,.8274600E-04,.9281600E-04,.9801700E-04,.9759000E-04,&
     & .8562100E-04,.1762300E-04,.5738100E-07,.4782600E-04,.7287400E-04,&
     & .8944300E-04,.1004000E-03,.1058800E-03,.1045600E-03,.9142900E-04,&
     & .1914400E-04,.5422100E-07,.5075000E-04,.7769800E-04,.9600500E-04,&
     & .1078400E-03,.1131800E-03,.1113900E-03,.9725800E-04,.2067100E-04/

      data   absa(  1:180, 4)   /                                       &
     & .3665400E-05,.1368900E-02,.2110800E-02,.2631400E-02,.2945800E-02,&
     & .3083100E-02,.3020700E-02,.2600300E-02,.9368900E-03,.3530400E-05,&
     & .1402500E-02,.2171600E-02,.2707800E-02,.3055100E-02,.3194800E-02,&
     & .3164900E-02,.2725800E-02,.9798300E-03,.3405900E-05,.1434000E-02,&
     & .2221400E-02,.2777300E-02,.3134800E-02,.3304900E-02,.3283500E-02,&
     & .2846100E-02,.1025100E-02,.3289500E-05,.1461000E-02,.2259600E-02,&
     & .2831400E-02,.3211300E-02,.3413100E-02,.3392100E-02,.2950800E-02,&
     & .1073000E-02,.3181400E-05,.1481400E-02,.2290500E-02,.2878900E-02,&
     & .3278500E-02,.3499600E-02,.3495200E-02,.3052800E-02,.1120500E-02,&
     & .3029300E-05,.1160000E-02,.1794600E-02,.2237000E-02,.2520500E-02,&
     & .2643200E-02,.2617200E-02,.2269100E-02,.7846600E-03,.2917300E-05,&
     & .1191600E-02,.1846800E-02,.2307200E-02,.2609400E-02,.2747700E-02,&
     & .2738800E-02,.2380900E-02,.8235600E-03,.2815200E-05,.1220300E-02,&
     & .1893900E-02,.2369100E-02,.2681400E-02,.2850600E-02,.2841400E-02,&
     & .2482500E-02,.8616100E-03,.2720700E-05,.1245400E-02,.1929100E-02,&
     & .2418000E-02,.2748900E-02,.2938000E-02,.2934500E-02,.2580800E-02,&
     & .9032800E-03,.2631600E-05,.1264400E-02,.1958600E-02,.2462400E-02,&
     & .2814600E-02,.3012500E-02,.3023700E-02,.2669500E-02,.9467800E-03,&
     & .2528800E-05,.9734800E-03,.1507000E-02,.1879500E-02,.2120600E-02,&
     & .2236200E-02,.2221000E-02,.1936900E-02,.6430800E-03,.2435300E-05,&
     & .1003100E-02,.1555200E-02,.1944100E-02,.2201300E-02,.2325700E-02,&
     & .2328600E-02,.2036200E-02,.6752900E-03,.2349500E-05,.1029500E-02,&
     & .1598600E-02,.2001900E-02,.2266300E-02,.2416000E-02,.2420400E-02,&
     & .2127600E-02,.7099100E-03,.2269900E-05,.1053300E-02,.1633700E-02,&
     & .2048900E-02,.2328800E-02,.2491100E-02,.2501600E-02,.2214800E-02,&
     & .7463500E-03,.2195100E-05,.1072400E-02,.1662500E-02,.2089100E-02,&
     & .2389600E-02,.2558200E-02,.2581200E-02,.2292500E-02,.7843200E-03,&
     & .2116900E-05,.8105300E-03,.1256800E-02,.1564700E-02,.1765400E-02,&
     & .1866200E-02,.1856800E-02,.1626900E-02,.5260800E-03,.2040700E-05,&
     & .8387500E-03,.1301900E-02,.1625200E-02,.1838300E-02,.1944100E-02,&
     & .1953800E-02,.1713900E-02,.5532500E-03,.1967400E-05,.8632900E-03,&
     & .1341700E-02,.1678800E-02,.1898500E-02,.2024600E-02,.2031500E-02,&
     & .1797800E-02,.5825300E-03,.1899400E-05,.8855100E-03,.1375300E-02,&
     & .1723800E-02,.1956600E-02,.2092300E-02,.2103500E-02,.1872400E-02,&
     & .6143800E-03,.1835800E-05,.9044200E-03,.1404200E-02,.1761900E-02,&
     & .2011900E-02,.2154200E-02,.2174900E-02,.1942500E-02,.6476200E-03/

      data   absa(181:315, 4)   /                                       &
     & .1772500E-05,.6714600E-03,.1039600E-02,.1293500E-02,.1457600E-02,&
     & .1543700E-02,.1534100E-02,.1353200E-02,.4298100E-03,.1709500E-05,&
     & .6977400E-03,.1082300E-02,.1347700E-02,.1522700E-02,.1611700E-02,&
     & .1622200E-02,.1430200E-02,.4528600E-03,.1647800E-05,.7204400E-03,&
     & .1119000E-02,.1396700E-02,.1578300E-02,.1681700E-02,.1689700E-02,&
     & .1504000E-02,.4778200E-03,.1589600E-05,.7413600E-03,.1151200E-02,&
     & .1439400E-02,.1630700E-02,.1743500E-02,.1753600E-02,.1568700E-02,&
     & .5046800E-03,.1535300E-05,.7597700E-03,.1179000E-02,.1475500E-02,&
     & .1681100E-02,.1799000E-02,.1817500E-02,.1627900E-02,.5335300E-03,&
     & .1485600E-05,.5516300E-03,.8518200E-03,.1058300E-02,.1193300E-02,&
     & .1263300E-02,.1251900E-02,.1112600E-02,.3496900E-03,.1432300E-05,&
     & .5753400E-03,.8921100E-03,.1108400E-02,.1251000E-02,.1324800E-02,&
     & .1330200E-02,.1181500E-02,.3689300E-03,.1381800E-05,.5970500E-03,&
     & .9255200E-03,.1153000E-02,.1301800E-02,.1386000E-02,.1394200E-02,&
     & .1243700E-02,.3899100E-03,.1331700E-05,.6167100E-03,.9556800E-03,&
     & .1192200E-02,.1348100E-02,.1442600E-02,.1450800E-02,.1300100E-02,&
     & .4124800E-03,.1285100E-05,.6341400E-03,.9822700E-03,.1226500E-02,&
     & .1393600E-02,.1491300E-02,.1506400E-02,.1351200E-02,.4369600E-03,&
     & .1245300E-05,.4498200E-03,.6928600E-03,.8585700E-03,.9711500E-03,&
     & .1026800E-02,.1013200E-02,.9066100E-03,.2833300E-03,.1199700E-05,&
     & .4716300E-03,.7298400E-03,.9052500E-03,.1020900E-02,.1083000E-02,&
     & .1081500E-02,.9682000E-03,.2999100E-03,.1157100E-05,.4917000E-03,&
     & .7606400E-03,.9459700E-03,.1068300E-02,.1133700E-02,.1141800E-02,&
     & .1021000E-02,.3173800E-03,.1115700E-05,.5096400E-03,.7885900E-03,&
     & .9823600E-03,.1108800E-02,.1185900E-02,.1192000E-02,.1069000E-02,&
     & .3360000E-03,.1075700E-05,.5257800E-03,.8133000E-03,.1014100E-02,&
     & .1149300E-02,.1228500E-02,.1239900E-02,.1115100E-02,.3563100E-03/

      data   absa(316:450, 4)   /                                       &
     & .1043600E-05,.3647500E-03,.5594800E-03,.6939900E-03,.7869900E-03,&
     & .8273300E-03,.8154600E-03,.7344300E-03,.2287400E-03,.1004300E-05,&
     & .3846100E-03,.5936500E-03,.7363100E-03,.8290200E-03,.8799600E-03,&
     & .8758900E-03,.7874500E-03,.2428100E-03,.9678500E-06,.4022700E-03,&
     & .6219100E-03,.7718300E-03,.8706700E-03,.9233800E-03,.9311600E-03,&
     & .8333400E-03,.2573500E-03,.9336700E-06,.4188100E-03,.6469700E-03,&
     & .8047400E-03,.9075400E-03,.9680600E-03,.9743400E-03,.8743900E-03,&
     & .2728600E-03,.9001700E-06,.4337700E-03,.6702600E-03,.8339800E-03,&
     & .9426900E-03,.1008000E-02,.1015400E-02,.9147900E-03,.2894800E-03,&
     & .8746900E-06,.2934500E-03,.4485700E-03,.5565300E-03,.6337100E-03,&
     & .6625300E-03,.6522900E-03,.5900900E-03,.1847500E-03,.8406200E-06,&
     & .3115200E-03,.4797500E-03,.5942200E-03,.6707700E-03,.7116800E-03,&
     & .7055000E-03,.6367400E-03,.1966900E-03,.8094700E-06,.3277900E-03,&
     & .5058200E-03,.6270700E-03,.7070500E-03,.7493600E-03,.7525200E-03,&
     & .6768200E-03,.2089600E-03,.7805200E-06,.3426200E-03,.5282000E-03,&
     & .6563100E-03,.7400200E-03,.7873100E-03,.7925700E-03,.7115500E-03,&
     & .2219700E-03,.7530000E-06,.3561800E-03,.5494000E-03,.6823600E-03,&
     & .7697800E-03,.8227100E-03,.8280600E-03,.7465800E-03,.2358800E-03,&
     & .7308000E-06,.2361900E-03,.3600200E-03,.4476900E-03,.5100100E-03,&
     & .5329400E-03,.5231900E-03,.4749200E-03,.1500500E-03,.7014200E-06,&
     & .2522500E-03,.3874200E-03,.4803900E-03,.5428500E-03,.5740700E-03,&
     & .5676500E-03,.5144400E-03,.1600000E-03,.6748400E-06,.2668700E-03,&
     & .4111900E-03,.5092400E-03,.5737200E-03,.6091200E-03,.6081300E-03,&
     & .5494500E-03,.1703600E-03,.6503200E-06,.2800200E-03,.4310400E-03,&
     & .5351300E-03,.6029400E-03,.6399800E-03,.6448500E-03,.5788800E-03,&
     & .1812000E-03,.6275200E-06,.2918200E-03,.4495000E-03,.5583600E-03,&
     & .6284900E-03,.6714800E-03,.6757000E-03,.6091800E-03,.1927100E-03/

      data   absa(451:585, 4)   /                                       &
     & .5984600E-06,.1960300E-03,.2990200E-03,.3715900E-03,.4236700E-03,&
     & .4424100E-03,.4355000E-03,.3961800E-03,.1249800E-03,.5744700E-06,&
     & .2095300E-03,.3218700E-03,.3988200E-03,.4502100E-03,.4771700E-03,&
     & .4724000E-03,.4289400E-03,.1336200E-03,.5526500E-06,.2218500E-03,&
     & .3413300E-03,.4228300E-03,.4761600E-03,.5048100E-03,.5055300E-03,&
     & .4565400E-03,.1424600E-03,.5325900E-06,.2330100E-03,.3581600E-03,&
     & .4444800E-03,.5004800E-03,.5314700E-03,.5357400E-03,.4816900E-03,&
     & .1516800E-03,.5137900E-06,.2430200E-03,.3738300E-03,.4635200E-03,&
     & .5220300E-03,.5578300E-03,.5617500E-03,.5073700E-03,.1615300E-03,&
     & .4900600E-06,.1625900E-03,.2480200E-03,.3080400E-03,.3511100E-03,&
     & .3667200E-03,.3616600E-03,.3296800E-03,.1040100E-03,.4703300E-06,&
     & .1737900E-03,.2669700E-03,.3306900E-03,.3729200E-03,.3958700E-03,&
     & .3923900E-03,.3567200E-03,.1113800E-03,.4524700E-06,.1841700E-03,&
     & .2829500E-03,.3504500E-03,.3946500E-03,.4182500E-03,.4198400E-03,&
     & .3786700E-03,.1188200E-03,.4360500E-06,.1935100E-03,.2971600E-03,&
     & .3685000E-03,.4148900E-03,.4407400E-03,.4446000E-03,.4003100E-03,&
     & .1266700E-03,.4206600E-06,.2020600E-03,.3103800E-03,.3846400E-03,&
     & .4333700E-03,.4629700E-03,.4666700E-03,.4220200E-03,.1349800E-03,&
     & .4012200E-06,.1346500E-03,.2054100E-03,.2548800E-03,.2900600E-03,&
     & .3035800E-03,.2998400E-03,.2737500E-03,.8640600E-04,.3850200E-06,&
     & .1439800E-03,.2208600E-03,.2737600E-03,.3083200E-03,.3279400E-03,&
     & .3251700E-03,.2956200E-03,.9261400E-04,.3704500E-06,.1526600E-03,&
     & .2341600E-03,.2901300E-03,.3267500E-03,.3462200E-03,.3481600E-03,&
     & .3140100E-03,.9888000E-04,.3570200E-06,.1605000E-03,.2460800E-03,&
     & .3052100E-03,.3433600E-03,.3652200E-03,.3685700E-03,.3324600E-03,&
     & .1055900E-03,.3444000E-06,.1677100E-03,.2574200E-03,.3190400E-03,&
     & .3594300E-03,.3840500E-03,.3871500E-03,.3508700E-03,.1126200E-03/

      data   absa(  1:180, 5)   /                                       &
     & .1558300E-03,.3754600E-02,.5353600E-02,.6416100E-02,.7122300E-02,&
     & .7498200E-02,.7359600E-02,.6513500E-02,.3517500E-02,.1614800E-03,&
     & .3856900E-02,.5525600E-02,.6646100E-02,.7375700E-02,.7753000E-02,&
     & .7566500E-02,.6649600E-02,.3684000E-02,.1648400E-03,.3935300E-02,&
     & .5674500E-02,.6836400E-02,.7616600E-02,.7971100E-02,.7771700E-02,&
     & .6784600E-02,.3848500E-02,.1662100E-03,.3994600E-02,.5794700E-02,&
     & .7007100E-02,.7819300E-02,.8158100E-02,.7959200E-02,.6913200E-02,&
     & .3976900E-02,.1651800E-03,.4038600E-02,.5892500E-02,.7153000E-02,&
     & .7994600E-02,.8325800E-02,.8110700E-02,.7022600E-02,.4113400E-02,&
     & .1248200E-03,.3220400E-02,.4610200E-02,.5547500E-02,.6173000E-02,&
     & .6500000E-02,.6394200E-02,.5639400E-02,.3000200E-02,.1302700E-03,&
     & .3309900E-02,.4761700E-02,.5749700E-02,.6403100E-02,.6718400E-02,&
     & .6582900E-02,.5772100E-02,.3144200E-02,.1337000E-03,.3380800E-02,&
     & .4884500E-02,.5923500E-02,.6607400E-02,.6904800E-02,.6766000E-02,&
     & .5901800E-02,.3278800E-02,.1354900E-03,.3434000E-02,.4988500E-02,&
     & .6074500E-02,.6787700E-02,.7078000E-02,.6920200E-02,.6013100E-02,&
     & .3394600E-02,.1354300E-03,.3477800E-02,.5074500E-02,.6198100E-02,&
     & .6928200E-02,.7222900E-02,.7052100E-02,.6121800E-02,.3509700E-02,&
     & .9843700E-04,.2720900E-02,.3903500E-02,.4718500E-02,.5270600E-02,&
     & .5557900E-02,.5484600E-02,.4846900E-02,.2507700E-02,.1039600E-03,&
     & .2803100E-02,.4038400E-02,.4899300E-02,.5475300E-02,.5757500E-02,&
     & .5652500E-02,.4968400E-02,.2636600E-02,.1077000E-03,.2870200E-02,&
     & .4151300E-02,.5050400E-02,.5664000E-02,.5924500E-02,.5812500E-02,&
     & .5092200E-02,.2747800E-02,.1099600E-03,.2923200E-02,.4245100E-02,&
     & .5183800E-02,.5825700E-02,.6079300E-02,.5949800E-02,.5194900E-02,&
     & .2849000E-02,.1108800E-03,.2964500E-02,.4325100E-02,.5299200E-02,&
     & .5946800E-02,.6209600E-02,.6064800E-02,.5290400E-02,.2949500E-02,&
     & .7658700E-04,.2279200E-02,.3273100E-02,.3972400E-02,.4456100E-02,&
     & .4710900E-02,.4670600E-02,.4144400E-02,.2081000E-02,.8222500E-04,&
     & .2354800E-02,.3394300E-02,.4132500E-02,.4641500E-02,.4893900E-02,&
     & .4820000E-02,.4259100E-02,.2192500E-02,.8620300E-04,.2418000E-02,&
     & .3499100E-02,.4268400E-02,.4807400E-02,.5041600E-02,.4965100E-02,&
     & .4364300E-02,.2293800E-02,.8876600E-04,.2469400E-02,.3587400E-02,&
     & .4387500E-02,.4946900E-02,.5181000E-02,.5085000E-02,.4460600E-02,&
     & .2379100E-02,.9021800E-04,.2511000E-02,.3661300E-02,.4493700E-02,&
     & .5054800E-02,.5298200E-02,.5188300E-02,.4549800E-02,.2465800E-02/

      data   absa(181:315, 5)   /                                       &
     & .5908000E-04,.1897200E-02,.2729400E-02,.3319500E-02,.3739500E-02,&
     & .3968300E-02,.3952300E-02,.3516100E-02,.1719500E-02,.6450800E-04,&
     & .1967300E-02,.2837600E-02,.3463200E-02,.3903300E-02,.4134300E-02,&
     & .4082600E-02,.3619500E-02,.1817300E-02,.6853800E-04,.2026400E-02,&
     & .2934200E-02,.3585600E-02,.4048800E-02,.4266400E-02,.4214100E-02,&
     & .3714400E-02,.1908100E-02,.7132800E-04,.2075000E-02,.3016500E-02,&
     & .3693300E-02,.4171900E-02,.4388000E-02,.4322100E-02,.3800600E-02,&
     & .1981800E-02,.7307300E-04,.2115500E-02,.3086100E-02,.3789200E-02,&
     & .4269000E-02,.4490100E-02,.4416100E-02,.3886400E-02,.2058300E-02,&
     & .4506900E-04,.1567300E-02,.2259600E-02,.2756600E-02,.3111700E-02,&
     & .3316200E-02,.3313500E-02,.2957400E-02,.1408900E-02,.4995900E-04,&
     & .1634200E-02,.2357700E-02,.2882800E-02,.3257200E-02,.3462600E-02,&
     & .3430200E-02,.3047600E-02,.1495500E-02,.5401300E-04,.1689100E-02,&
     & .2445700E-02,.2992600E-02,.3386500E-02,.3580900E-02,.3542700E-02,&
     & .3135800E-02,.1576300E-02,.5691000E-04,.1735600E-02,.2522000E-02,&
     & .3090300E-02,.3496800E-02,.3685300E-02,.3644300E-02,.3214000E-02,&
     & .1642800E-02,.5885200E-04,.1774300E-02,.2586800E-02,.3177100E-02,&
     & .3585700E-02,.3778200E-02,.3728400E-02,.3290500E-02,.1707900E-02,&
     & .3413100E-04,.1287300E-02,.1861900E-02,.2274500E-02,.2568800E-02,&
     & .2749400E-02,.2757400E-02,.2465800E-02,.1144400E-02,.3837900E-04,&
     & .1350300E-02,.1949000E-02,.2386900E-02,.2698500E-02,.2876100E-02,&
     & .2860700E-02,.2544300E-02,.1224200E-02,.4214600E-04,.1401800E-02,&
     & .2028700E-02,.2484300E-02,.2811900E-02,.2984800E-02,.2956900E-02,&
     & .2625800E-02,.1295600E-02,.4511600E-04,.1445400E-02,.2098900E-02,&
     & .2571800E-02,.2911500E-02,.3074000E-02,.3048200E-02,.2696700E-02,&
     & .1353700E-02,.4717800E-04,.1481800E-02,.2159100E-02,.2650900E-02,&
     & .2994500E-02,.3159000E-02,.3123500E-02,.2764100E-02,.1410000E-02/

      data   absa(316:450, 5)   /                                       &
     & .2565600E-04,.1052500E-02,.1526600E-02,.1864800E-02,.2106000E-02,&
     & .2266000E-02,.2280700E-02,.2041500E-02,.9248200E-03,.2931300E-04,&
     & .1109600E-02,.1602700E-02,.1962800E-02,.2222400E-02,.2373000E-02,&
     & .2368300E-02,.2111100E-02,.9936600E-03,.3264300E-04,.1157900E-02,&
     & .1673200E-02,.2050300E-02,.2323100E-02,.2472100E-02,.2451200E-02,&
     & .2184200E-02,.1057000E-02,.3550900E-04,.1198500E-02,.1737900E-02,&
     & .2128400E-02,.2411000E-02,.2551900E-02,.2533900E-02,.2249600E-02,&
     & .1111600E-02,.3761600E-04,.1232700E-02,.1793300E-02,.2199600E-02,&
     & .2488000E-02,.2625800E-02,.2602900E-02,.2308800E-02,.1159500E-02,&
     & .1916600E-04,.8555900E-03,.1244600E-02,.1521300E-02,.1716000E-02,&
     & .1852700E-02,.1872300E-02,.1679400E-02,.7431600E-03,.2225100E-04,&
     & .9070100E-03,.1310700E-02,.1606100E-02,.1817700E-02,.1945200E-02,&
     & .1949600E-02,.1742000E-02,.8016800E-03,.2512300E-04,.9510600E-03,&
     & .1372400E-02,.1682600E-02,.1906400E-02,.2035200E-02,.2024100E-02,&
     & .1806400E-02,.8565900E-03,.2771300E-04,.9890300E-03,.1430100E-02,&
     & .1752300E-02,.1986200E-02,.2107900E-02,.2094400E-02,.1865600E-02,&
     & .9065500E-03,.2982100E-04,.1020700E-02,.1481300E-02,.1816300E-02,&
     & .2057000E-02,.2173700E-02,.2157400E-02,.1915300E-02,.9488200E-03,&
     & .1445500E-04,.6950800E-03,.1014300E-02,.1238200E-02,.1397700E-02,&
     & .1510100E-02,.1533900E-02,.1379400E-02,.6006400E-03,.1697500E-04,&
     & .7405500E-03,.1071500E-02,.1311700E-02,.1485100E-02,.1593800E-02,&
     & .1602600E-02,.1435400E-02,.6503400E-03,.1942100E-04,.7799600E-03,&
     & .1124300E-02,.1379100E-02,.1563600E-02,.1670700E-02,.1667800E-02,&
     & .1490800E-02,.6987500E-03,.2165900E-04,.8136900E-03,.1174600E-02,&
     & .1440800E-02,.1634000E-02,.1737600E-02,.1726700E-02,.1541800E-02,&
     & .7433600E-03,.2361600E-04,.8431600E-03,.1221500E-02,.1497200E-02,&
     & .1697100E-02,.1794500E-02,.1783400E-02,.1584400E-02,.7804500E-03/

      data   absa(451:585, 5)   /                                       &
     & .1181200E-04,.5806700E-03,.8465800E-03,.1034900E-02,.1168100E-02,&
     & .1262200E-02,.1280600E-02,.1151800E-02,.5019700E-03,.1387000E-04,&
     & .6187000E-03,.8936400E-03,.1095700E-02,.1241100E-02,.1330700E-02,&
     & .1336600E-02,.1199000E-02,.5441900E-03,.1587200E-04,.6513100E-03,&
     & .9384300E-03,.1152000E-02,.1306800E-02,.1396600E-02,.1392700E-02,&
     & .1245900E-02,.5841200E-03,.1770600E-04,.6795700E-03,.9812800E-03,&
     & .1203700E-02,.1365300E-02,.1451000E-02,.1442300E-02,.1287300E-02,&
     & .6208600E-03,.1930700E-04,.7047700E-03,.1020800E-02,.1252200E-02,&
     & .1418700E-02,.1500000E-02,.1488200E-02,.1322300E-02,.6527300E-03,&
     & .9663300E-05,.4838400E-03,.7046400E-03,.8621400E-03,.9732500E-03,&
     & .1052500E-02,.1066100E-02,.9581800E-03,.4183800E-03,.1134600E-04,&
     & .5154400E-03,.7437500E-03,.9125500E-03,.1034600E-02,.1108800E-02,&
     & .1112400E-02,.9978700E-03,.4543600E-03,.1298300E-04,.5425800E-03,&
     & .7819100E-03,.9599400E-03,.1089300E-02,.1163600E-02,.1159500E-02,&
     & .1037900E-02,.4876700E-03,.1448300E-04,.5666200E-03,.8179500E-03,&
     & .1003600E-02,.1138500E-02,.1209100E-02,.1201200E-02,.1070700E-02,&
     & .5181000E-03,.1579000E-04,.5880500E-03,.8516500E-03,.1044700E-02,&
     & .1183000E-02,.1250000E-02,.1238900E-02,.1101200E-02,.5450800E-03,&
     & .7905700E-05,.4021500E-03,.5851600E-03,.7166600E-03,.8096900E-03,&
     & .8751800E-03,.8858100E-03,.7956900E-03,.3482500E-03,.9280800E-05,&
     & .4285300E-03,.6181800E-03,.7584700E-03,.8608200E-03,.9216000E-03,&
     & .9243600E-03,.8289400E-03,.3787100E-03,.1062100E-04,.4513400E-03,&
     & .6502800E-03,.7981500E-03,.9062200E-03,.9675200E-03,.9629100E-03,&
     & .8617700E-03,.4064200E-03,.1184900E-04,.4716100E-03,.6809000E-03,&
     & .8353000E-03,.9477000E-03,.1005400E-02,.9981500E-03,.8890800E-03,&
     & .4314900E-03,.1289700E-04,.4900000E-03,.7094400E-03,.8698200E-03,&
     & .9843000E-03,.1039800E-02,.1029700E-02,.9148300E-03,.4545900E-03/

      data   absa(  1:180, 6)   /                                       &
     & .2357500E-02,.9645700E-02,.1312800E-01,.1536100E-01,.1666200E-01,&
     & .1699900E-01,.1630300E-01,.1436800E-01,.1018500E-01,.2450800E-02,&
     & .9976800E-02,.1356200E-01,.1578600E-01,.1704600E-01,.1731600E-01,&
     & .1653900E-01,.1462700E-01,.1044700E-01,.2517600E-02,.1026200E-01,&
     & .1394700E-01,.1618200E-01,.1739000E-01,.1758800E-01,.1675200E-01,&
     & .1485700E-01,.1069900E-01,.2562700E-02,.1051000E-01,.1427100E-01,&
     & .1651700E-01,.1767800E-01,.1781500E-01,.1694100E-01,.1507300E-01,&
     & .1097000E-01,.2590400E-02,.1071200E-01,.1452800E-01,.1678800E-01,&
     & .1791200E-01,.1801300E-01,.1712600E-01,.1528200E-01,.1121500E-01,&
     & .1942200E-02,.8517900E-02,.1160300E-01,.1356100E-01,.1469400E-01,&
     & .1495200E-01,.1427400E-01,.1261600E-01,.8691000E-02,.2023100E-02,&
     & .8820800E-02,.1199200E-01,.1394200E-01,.1504300E-01,.1524100E-01,&
     & .1450300E-01,.1284900E-01,.8920300E-02,.2082400E-02,.9089200E-02,&
     & .1233300E-01,.1428300E-01,.1534300E-01,.1549300E-01,.1470400E-01,&
     & .1306500E-01,.9141200E-02,.2123500E-02,.9319300E-02,.1261300E-01,&
     & .1457300E-01,.1558600E-01,.1569600E-01,.1490000E-01,.1326900E-01,&
     & .9373100E-02,.2150100E-02,.9506400E-02,.1284400E-01,.1480200E-01,&
     & .1578900E-01,.1587200E-01,.1507900E-01,.1345900E-01,.9592200E-02,&
     & .1571000E-02,.7373900E-02,.1008400E-01,.1179500E-01,.1279100E-01,&
     & .1300200E-01,.1240800E-01,.1094800E-01,.7325600E-02,.1644500E-02,&
     & .7660300E-02,.1044100E-01,.1213600E-01,.1309800E-01,.1327200E-01,&
     & .1262900E-01,.1116900E-01,.7527700E-02,.1701600E-02,.7912000E-02,&
     & .1073800E-01,.1244400E-01,.1336500E-01,.1350200E-01,.1282700E-01,&
     & .1136500E-01,.7727500E-02,.1741900E-02,.8120400E-02,.1099200E-01,&
     & .1270400E-01,.1357900E-01,.1368700E-01,.1302200E-01,.1156500E-01,&
     & .7932700E-02,.1769300E-02,.8289600E-02,.1120100E-01,.1290300E-01,&
     & .1376100E-01,.1385400E-01,.1319300E-01,.1176000E-01,.8132400E-02,&
     & .1256800E-02,.6292500E-02,.8657000E-02,.1015600E-01,.1102000E-01,&
     & .1122000E-01,.1071000E-01,.9432900E-02,.6181200E-02,.1324200E-02,&
     & .6553200E-02,.8975600E-02,.1046000E-01,.1129200E-01,.1146300E-01,&
     & .1092400E-01,.9644900E-02,.6362000E-02,.1378200E-02,.6780700E-02,&
     & .9244700E-02,.1073900E-01,.1153100E-01,.1167600E-01,.1111500E-01,&
     & .9838400E-02,.6541700E-02,.1418500E-02,.6972800E-02,.9472400E-02,&
     & .1096700E-01,.1173200E-01,.1185300E-01,.1129900E-01,.1002900E-01,&
     & .6731400E-02,.1446600E-02,.7129400E-02,.9655100E-02,.1115000E-01,&
     & .1189700E-01,.1200300E-01,.1145700E-01,.1021200E-01,.6913400E-02/

      data   absa(181:315, 6)   /                                       &
     & .9981500E-03,.5318400E-02,.7366800E-02,.8666100E-02,.9418900E-02,&
     & .9608800E-02,.9186000E-02,.8085400E-02,.5214700E-02,.1059400E-02,&
     & .5550200E-02,.7648000E-02,.8938200E-02,.9667500E-02,.9828100E-02,&
     & .9390800E-02,.8290400E-02,.5380500E-02,.1109300E-02,.5753400E-02,&
     & .7882000E-02,.9187300E-02,.9886000E-02,.1002700E-01,.9573900E-02,&
     & .8477400E-02,.5539800E-02,.1148700E-02,.5926400E-02,.8074600E-02,&
     & .9395200E-02,.1007000E-01,.1019600E-01,.9746000E-02,.8658600E-02,&
     & .5711800E-02,.1177000E-02,.6071500E-02,.8232200E-02,.9559000E-02,&
     & .1022500E-01,.1033700E-01,.9889100E-02,.8828200E-02,.5875600E-02,&
     & .7848400E-03,.4455700E-02,.6208800E-02,.7323800E-02,.7975800E-02,&
     & .8156500E-02,.7823400E-02,.6894900E-02,.4372100E-02,.8410400E-03,&
     & .4658400E-02,.6454800E-02,.7572000E-02,.8204900E-02,.8360300E-02,&
     & .8013900E-02,.7091200E-02,.4520800E-02,.8870300E-03,.4838600E-02,&
     & .6658500E-02,.7793900E-02,.8407700E-02,.8549400E-02,.8187800E-02,&
     & .7267300E-02,.4666400E-02,.9242900E-03,.4993900E-02,.6825500E-02,&
     & .7979300E-02,.8581100E-02,.8712300E-02,.8342300E-02,.7432400E-02,&
     & .4819300E-02,.9528700E-03,.5126900E-02,.6966400E-02,.8121600E-02,&
     & .8725600E-02,.8840400E-02,.8479300E-02,.7588500E-02,.4968400E-02,&
     & .6117600E-03,.3710500E-02,.5192100E-02,.6149500E-02,.6708600E-02,&
     & .6873700E-02,.6619600E-02,.5849900E-02,.3639800E-02,.6635500E-03,&
     & .3884500E-02,.5409200E-02,.6368400E-02,.6919100E-02,.7063800E-02,&
     & .6796700E-02,.6032900E-02,.3772500E-02,.7058400E-03,.4042900E-02,&
     & .5589100E-02,.6566500E-02,.7106100E-02,.7239200E-02,.6956600E-02,&
     & .6195800E-02,.3904000E-02,.7401600E-03,.4180400E-02,.5736200E-02,&
     & .6730600E-02,.7265600E-02,.7393900E-02,.7100200E-02,.6345800E-02,&
     & .4042200E-02,.7679300E-03,.4301300E-02,.5863400E-02,.6860500E-02,&
     & .7391700E-02,.7516000E-02,.7229200E-02,.6486700E-02,.4175300E-02/

      data   absa(316:450, 6)   /                                       &
     & .4738300E-03,.3071900E-02,.4315700E-02,.5129900E-02,.5611300E-02,&
     & .5760400E-02,.5558400E-02,.4926300E-02,.3009900E-02,.5203000E-03,&
     & .3222400E-02,.4508800E-02,.5325600E-02,.5804900E-02,.5936900E-02,&
     & .5724900E-02,.5095000E-02,.3133500E-02,.5589700E-03,.3360100E-02,&
     & .4669000E-02,.5500800E-02,.5973300E-02,.6098200E-02,.5870300E-02,&
     & .5241600E-02,.3250500E-02,.5905100E-03,.3482300E-02,.4800300E-02,&
     & .5649200E-02,.6114700E-02,.6240700E-02,.6001100E-02,.5377600E-02,&
     & .3369100E-02,.6163800E-03,.3590800E-02,.4915100E-02,.5768200E-02,&
     & .6226900E-02,.6350500E-02,.6122600E-02,.5506100E-02,.3489300E-02,&
     & .3645200E-03,.2530100E-02,.3565500E-02,.4255800E-02,.4666800E-02,&
     & .4799500E-02,.4639100E-02,.4117600E-02,.2472300E-02,.4052100E-03,&
     & .2660900E-02,.3736800E-02,.4431300E-02,.4842200E-02,.4960500E-02,&
     & .4787600E-02,.4268200E-02,.2585400E-02,.4404200E-03,.2781100E-02,&
     & .3882200E-02,.4587100E-02,.4995100E-02,.5104800E-02,.4918900E-02,&
     & .4398800E-02,.2692200E-02,.4693400E-03,.2889000E-02,.4001500E-02,&
     & .4720400E-02,.5119500E-02,.5232900E-02,.5039800E-02,.4521100E-02,&
     & .2794700E-02,.4929600E-03,.2986300E-02,.4105200E-02,.4829600E-02,&
     & .5221200E-02,.5332000E-02,.5151600E-02,.4641000E-02,.2901400E-02,&
     & .2814100E-03,.2083900E-02,.2943400E-02,.3525000E-02,.3870500E-02,&
     & .3990700E-02,.3857400E-02,.3428000E-02,.2027600E-02,.3164200E-03,&
     & .2196200E-02,.3094700E-02,.3679800E-02,.4027900E-02,.4131700E-02,&
     & .3990500E-02,.3559100E-02,.2130400E-02,.3474300E-03,.2300200E-02,&
     & .3225100E-02,.3816700E-02,.4163200E-02,.4258800E-02,.4107700E-02,&
     & .3673700E-02,.2224600E-02,.3733700E-03,.2395800E-02,.3332600E-02,&
     & .3935100E-02,.4272700E-02,.4369900E-02,.4216700E-02,.3785000E-02,&
     & .2317100E-02,.3946400E-03,.2481800E-02,.3424500E-02,.4034100E-02,&
     & .4363500E-02,.4459200E-02,.4313300E-02,.3893100E-02,.2412300E-02/

      data   absa(451:585, 6)   /                                       &
     & .2304600E-03,.1756100E-02,.2482100E-02,.2969700E-02,.3260300E-02,&
     & .3359000E-02,.3246500E-02,.2891100E-02,.1700000E-02,.2590000E-03,&
     & .1849900E-02,.2607900E-02,.3098300E-02,.3391400E-02,.3479000E-02,&
     & .3359800E-02,.3000000E-02,.1787900E-02,.2843100E-03,.1937100E-02,&
     & .2715400E-02,.3212300E-02,.3500900E-02,.3581000E-02,.3458200E-02,&
     & .3098900E-02,.1870900E-02,.3055400E-03,.2017500E-02,.2804700E-02,&
     & .3309400E-02,.3589700E-02,.3670800E-02,.3550000E-02,.3196400E-02,&
     & .1952400E-02,.3229000E-03,.2090600E-02,.2882700E-02,.3390700E-02,&
     & .3663800E-02,.3744800E-02,.3632200E-02,.3292100E-02,.2034200E-02,&
     & .1887300E-03,.1475300E-02,.2085900E-02,.2493200E-02,.2736200E-02,&
     & .2816300E-02,.2723100E-02,.2428700E-02,.1422600E-02,.2120600E-03,&
     & .1554200E-02,.2190500E-02,.2599700E-02,.2843600E-02,.2914600E-02,&
     & .2818200E-02,.2519600E-02,.1496400E-02,.2327100E-03,.1628200E-02,&
     & .2278900E-02,.2693700E-02,.2932000E-02,.2999500E-02,.2900400E-02,&
     & .2606300E-02,.1568400E-02,.2500500E-03,.1695900E-02,.2353900E-02,&
     & .2773900E-02,.3005200E-02,.3073800E-02,.2978100E-02,.2692400E-02,&
     & .1638800E-02,.2642400E-03,.1757900E-02,.2419800E-02,.2841800E-02,&
     & .3068800E-02,.3137000E-02,.3047800E-02,.2773400E-02,.1709000E-02,&
     & .1545100E-03,.1236200E-02,.1747100E-02,.2085600E-02,.2287000E-02,&
     & .2353000E-02,.2275800E-02,.2031800E-02,.1186800E-02,.1735700E-03,&
     & .1302700E-02,.1833700E-02,.2173600E-02,.2375200E-02,.2433900E-02,&
     & .2354900E-02,.2110400E-02,.1249200E-02,.1904500E-03,.1365100E-02,&
     & .1907100E-02,.2252000E-02,.2447900E-02,.2505200E-02,.2425500E-02,&
     & .2186200E-02,.1311400E-02,.2045900E-03,.1422500E-02,.1970200E-02,&
     & .2319500E-02,.2510900E-02,.2568000E-02,.2490600E-02,.2259200E-02,&
     & .1371700E-02,.2162100E-03,.1475100E-02,.2026700E-02,.2377500E-02,&
     & .2565400E-02,.2622100E-02,.2551300E-02,.2328300E-02,.1432100E-02/

      data   absa(  1:180, 7)   /                                       &
     & .1455100E-01,.2628200E-01,.3205300E-01,.3518900E-01,.3660000E-01,&
     & .3664600E-01,.3555200E-01,.3321500E-01,.2730900E-01,.1536000E-01,&
     & .2684700E-01,.3241100E-01,.3542600E-01,.3680300E-01,.3699400E-01,&
     & .3603700E-01,.3367400E-01,.2788100E-01,.1605200E-01,.2735800E-01,&
     & .3271100E-01,.3560200E-01,.3700200E-01,.3732200E-01,.3649500E-01,&
     & .3409200E-01,.2836500E-01,.1663400E-01,.2776500E-01,.3297500E-01,&
     & .3575700E-01,.3719800E-01,.3762900E-01,.3687100E-01,.3448900E-01,&
     & .2881700E-01,.1709200E-01,.2808800E-01,.3317300E-01,.3589900E-01,&
     & .3736900E-01,.3789400E-01,.3720600E-01,.3485600E-01,.2923600E-01,&
     & .1267500E-01,.2375700E-01,.2882700E-01,.3147500E-01,.3264700E-01,&
     & .3277900E-01,.3187400E-01,.2965900E-01,.2373500E-01,.1336300E-01,&
     & .2426100E-01,.2916500E-01,.3173400E-01,.3290500E-01,.3313700E-01,&
     & .3235700E-01,.3011400E-01,.2424100E-01,.1395200E-01,.2467300E-01,&
     & .2945800E-01,.3194000E-01,.3316700E-01,.3348600E-01,.3279400E-01,&
     & .3054000E-01,.2470400E-01,.1444000E-01,.2501000E-01,.2969200E-01,&
     & .3213100E-01,.3340200E-01,.3381600E-01,.3318500E-01,.3094400E-01,&
     & .2512300E-01,.1481000E-01,.2524600E-01,.2986500E-01,.3232300E-01,&
     & .3362400E-01,.3412500E-01,.3353000E-01,.3129900E-01,.2550100E-01,&
     & .1076400E-01,.2118000E-01,.2564500E-01,.2793400E-01,.2889100E-01,&
     & .2897800E-01,.2821500E-01,.2625000E-01,.2034000E-01,.1137100E-01,&
     & .2161100E-01,.2597600E-01,.2822100E-01,.2920400E-01,.2934700E-01,&
     & .2868900E-01,.2668700E-01,.2080300E-01,.1188200E-01,.2197700E-01,&
     & .2625800E-01,.2846700E-01,.2949800E-01,.2973000E-01,.2911300E-01,&
     & .2711500E-01,.2124100E-01,.1230200E-01,.2227300E-01,.2647700E-01,&
     & .2869600E-01,.2976600E-01,.3009900E-01,.2950700E-01,.2750300E-01,&
     & .2162000E-01,.1262400E-01,.2248100E-01,.2663100E-01,.2890000E-01,&
     & .3003200E-01,.3042500E-01,.2985900E-01,.2784200E-01,.2196600E-01,&
     & .8986000E-02,.1865900E-01,.2260500E-01,.2458000E-01,.2538300E-01,&
     & .2543500E-01,.2476100E-01,.2303700E-01,.1741400E-01,.9520700E-02,&
     & .1904800E-01,.2292400E-01,.2488800E-01,.2573900E-01,.2582300E-01,&
     & .2520000E-01,.2345000E-01,.1784700E-01,.9972300E-02,.1938000E-01,&
     & .2318200E-01,.2515500E-01,.2606100E-01,.2620900E-01,.2561100E-01,&
     & .2386200E-01,.1825100E-01,.1035400E-01,.1964400E-01,.2339700E-01,&
     & .2540100E-01,.2634500E-01,.2658000E-01,.2600000E-01,.2423300E-01,&
     & .1860000E-01,.1063700E-01,.1982200E-01,.2355600E-01,.2560500E-01,&
     & .2661900E-01,.2691300E-01,.2635900E-01,.2456200E-01,.1892500E-01/

      data   absa(181:315, 7)   /                                       &
     & .7410400E-02,.1626400E-01,.1971700E-01,.2145200E-01,.2217200E-01,&
     & .2220700E-01,.2159800E-01,.2007500E-01,.1488000E-01,.7877800E-02,&
     & .1662400E-01,.2002200E-01,.2176500E-01,.2253100E-01,.2259600E-01,&
     & .2199900E-01,.2045800E-01,.1527900E-01,.8269400E-02,.1692800E-01,&
     & .2027700E-01,.2203600E-01,.2285900E-01,.2297800E-01,.2238400E-01,&
     & .2083400E-01,.1565000E-01,.8591900E-02,.1716800E-01,.2049800E-01,&
     & .2227400E-01,.2314300E-01,.2332600E-01,.2275400E-01,.2118900E-01,&
     & .1598000E-01,.8833700E-02,.1733200E-01,.2066400E-01,.2247700E-01,&
     & .2340700E-01,.2364800E-01,.2311100E-01,.2150900E-01,.1628300E-01,&
     & .6030500E-02,.1400800E-01,.1702800E-01,.1857400E-01,.1923600E-01,&
     & .1926900E-01,.1872000E-01,.1734700E-01,.1265600E-01,.6435600E-02,&
     & .1433900E-01,.1731800E-01,.1887200E-01,.1958000E-01,.1964500E-01,&
     & .1909500E-01,.1770500E-01,.1302500E-01,.6770800E-02,.1462600E-01,&
     & .1757500E-01,.1913500E-01,.1988300E-01,.2000400E-01,.1945400E-01,&
     & .1805500E-01,.1336600E-01,.7044900E-02,.1485300E-01,.1779500E-01,&
     & .1936300E-01,.2015600E-01,.2032400E-01,.1981100E-01,.1839200E-01,&
     & .1367900E-01,.7258600E-02,.1501400E-01,.1795400E-01,.1956900E-01,&
     & .2040100E-01,.2062200E-01,.2015500E-01,.1870600E-01,.1396300E-01,&
     & .4861500E-02,.1193600E-01,.1457400E-01,.1596300E-01,.1657800E-01,&
     & .1661500E-01,.1612500E-01,.1490400E-01,.1071800E-01,.5209300E-02,&
     & .1224700E-01,.1484500E-01,.1623900E-01,.1689000E-01,.1696500E-01,&
     & .1647700E-01,.1523800E-01,.1105000E-01,.5498100E-02,.1252000E-01,&
     & .1508900E-01,.1648500E-01,.1716300E-01,.1729600E-01,.1682300E-01,&
     & .1556100E-01,.1136700E-01,.5734100E-02,.1273200E-01,.1530200E-01,&
     & .1670900E-01,.1741200E-01,.1758700E-01,.1715600E-01,.1588200E-01,&
     & .1165700E-01,.5924500E-02,.1287900E-01,.1546000E-01,.1690000E-01,&
     & .1764600E-01,.1786200E-01,.1747500E-01,.1618900E-01,.1192400E-01/

      data   absa(316:450, 7)   /                                       &
     & .3890300E-02,.1008600E-01,.1239300E-01,.1361700E-01,.1417900E-01,&
     & .1423000E-01,.1381000E-01,.1274100E-01,.9040200E-02,.4188300E-02,&
     & .1037900E-01,.1264200E-01,.1386600E-01,.1445400E-01,.1454900E-01,&
     & .1414000E-01,.1305000E-01,.9340700E-02,.4437400E-02,.1062800E-01,&
     & .1286600E-01,.1409300E-01,.1470200E-01,.1484700E-01,.1446600E-01,&
     & .1335400E-01,.9628900E-02,.4641700E-02,.1082300E-01,.1306700E-01,&
     & .1429700E-01,.1493600E-01,.1511300E-01,.1477700E-01,.1365900E-01,&
     & .9894600E-02,.4810600E-02,.1095800E-01,.1321200E-01,.1447700E-01,&
     & .1515500E-01,.1536900E-01,.1506500E-01,.1395400E-01,.1013700E-01,&
     & .3090600E-02,.8471600E-02,.1047500E-01,.1153300E-01,.1203700E-01,&
     & .1210200E-01,.1175700E-01,.1084000E-01,.7579000E-02,.3345600E-02,&
     & .8738000E-02,.1069800E-01,.1175900E-01,.1228100E-01,.1238400E-01,&
     & .1206300E-01,.1112400E-01,.7852400E-02,.3562400E-02,.8963600E-02,&
     & .1090800E-01,.1196400E-01,.1250300E-01,.1265100E-01,.1236600E-01,&
     & .1141300E-01,.8111600E-02,.3740900E-02,.9139400E-02,.1108500E-01,&
     & .1215600E-01,.1271700E-01,.1289500E-01,.1264600E-01,.1170300E-01,&
     & .8354300E-02,.3889900E-02,.9266300E-02,.1121000E-01,.1232300E-01,&
     & .1292300E-01,.1313500E-01,.1290900E-01,.1197900E-01,.8574800E-02,&
     & .2458800E-02,.7099700E-02,.8815600E-02,.9728000E-02,.1017500E-01,&
     & .1024600E-01,.9971200E-02,.9195100E-02,.6332400E-02,.2674300E-02,&
     & .7339300E-02,.9018300E-02,.9928900E-02,.1038700E-01,.1049600E-01,&
     & .1024900E-01,.9456600E-02,.6577000E-02,.2861000E-02,.7535200E-02,&
     & .9202800E-02,.1011700E-01,.1058600E-01,.1073000E-01,.1052400E-01,&
     & .9730700E-02,.6807700E-02,.3014700E-02,.7689900E-02,.9355000E-02,&
     & .1029100E-01,.1078100E-01,.1095400E-01,.1077800E-01,.1000100E-01,&
     & .7020100E-02,.3144400E-02,.7805000E-02,.9466800E-02,.1043300E-01,&
     & .1097400E-01,.1117300E-01,.1101900E-01,.1025300E-01,.7218900E-02/

      data   absa(451:585, 7)   /                                       &
     & .2034500E-02,.6024900E-02,.7471600E-02,.8249600E-02,.8645800E-02,&
     & .8734000E-02,.8522200E-02,.7865700E-02,.5371400E-02,.2207100E-02,&
     & .6217600E-02,.7644000E-02,.8425100E-02,.8827300E-02,.8946100E-02,&
     & .8768900E-02,.8112500E-02,.5583500E-02,.2355400E-02,.6372900E-02,&
     & .7793200E-02,.8586800E-02,.9005800E-02,.9154800E-02,.9007500E-02,&
     & .8366000E-02,.5782700E-02,.2478000E-02,.6494400E-02,.7910600E-02,&
     & .8729100E-02,.9185700E-02,.9356500E-02,.9230200E-02,.8603600E-02,&
     & .5969500E-02,.2582000E-02,.6580200E-02,.7997400E-02,.8842700E-02,&
     & .9348200E-02,.9553900E-02,.9444300E-02,.8818000E-02,.6151200E-02,&
     & .1678900E-02,.5090600E-02,.6307800E-02,.6975000E-02,.7317200E-02,&
     & .7407800E-02,.7250400E-02,.6701500E-02,.4539400E-02,.1817000E-02,&
     & .5243700E-02,.6451200E-02,.7125500E-02,.7479000E-02,.7595900E-02,&
     & .7465500E-02,.6930700E-02,.4723700E-02,.1935600E-02,.5368900E-02,&
     & .6570100E-02,.7261000E-02,.7641500E-02,.7781100E-02,.7670300E-02,&
     & .7150800E-02,.4895300E-02,.2034200E-02,.5464400E-02,.6664000E-02,&
     & .7379200E-02,.7792900E-02,.7963500E-02,.7866000E-02,.7353200E-02,&
     & .5060700E-02,.2118200E-02,.5532800E-02,.6735200E-02,.7473600E-02,&
     & .7928200E-02,.8130700E-02,.8054200E-02,.7540700E-02,.5223400E-02,&
     & .1381900E-02,.4282300E-02,.5309600E-02,.5877100E-02,.6174500E-02,&
     & .6260100E-02,.6141100E-02,.5689300E-02,.3820100E-02,.1492700E-02,&
     & .4407800E-02,.5426100E-02,.6005500E-02,.6316700E-02,.6427800E-02,&
     & .6326800E-02,.5889100E-02,.3978900E-02,.1588400E-02,.4509000E-02,&
     & .5524600E-02,.6121700E-02,.6457700E-02,.6590300E-02,.6504400E-02,&
     & .6075800E-02,.4128800E-02,.1668300E-02,.4585800E-02,.5602900E-02,&
     & .6219300E-02,.6585900E-02,.6745400E-02,.6676800E-02,.6250600E-02,&
     & .4276500E-02,.1736400E-02,.4641900E-02,.5664400E-02,.6299600E-02,&
     & .6699800E-02,.6888800E-02,.6840400E-02,.6418800E-02,.4419800E-02/

      data   absa(  1:180, 8)   /                                       &
     & .5776000E-01,.6943200E-01,.7640200E-01,.8151700E-01,.8512500E-01,&
     & .8728900E-01,.8740800E-01,.8331700E-01,.7472000E-01,.5825600E-01,&
     & .6966700E-01,.7672300E-01,.8191100E-01,.8536200E-01,.8728900E-01,&
     & .8730600E-01,.8335700E-01,.7517700E-01,.5871000E-01,.6985900E-01,&
     & .7697300E-01,.8232800E-01,.8569000E-01,.8742000E-01,.8726400E-01,&
     & .8343500E-01,.7570100E-01,.5905200E-01,.7005000E-01,.7717600E-01,&
     & .8264200E-01,.8607100E-01,.8766000E-01,.8734100E-01,.8354400E-01,&
     & .7623000E-01,.5929800E-01,.7019800E-01,.7737400E-01,.8288300E-01,&
     & .8646900E-01,.8797700E-01,.8747300E-01,.8369100E-01,.7676900E-01,&
     & .5090800E-01,.6284800E-01,.7000300E-01,.7526300E-01,.7867000E-01,&
     & .8059400E-01,.8069400E-01,.7704800E-01,.6704800E-01,.5146900E-01,&
     & .6317800E-01,.7039500E-01,.7576300E-01,.7903300E-01,.8077500E-01,&
     & .8068600E-01,.7719900E-01,.6760500E-01,.5195600E-01,.6351900E-01,&
     & .7074800E-01,.7622100E-01,.7951200E-01,.8106700E-01,.8079700E-01,&
     & .7737000E-01,.6818200E-01,.5232900E-01,.6383100E-01,.7111500E-01,&
     & .7659100E-01,.8003200E-01,.8144500E-01,.8100100E-01,.7756800E-01,&
     & .6876800E-01,.5264800E-01,.6410800E-01,.7144100E-01,.7694100E-01,&
     & .8046400E-01,.8186600E-01,.8128200E-01,.7779600E-01,.6937700E-01,&
     & .4463600E-01,.5657500E-01,.6366700E-01,.6880400E-01,.7214600E-01,&
     & .7394000E-01,.7375600E-01,.7035900E-01,.5944600E-01,.4521600E-01,&
     & .5700500E-01,.6411100E-01,.6932800E-01,.7262000E-01,.7422800E-01,&
     & .7385400E-01,.7060000E-01,.6008600E-01,.4571600E-01,.5744800E-01,&
     & .6459100E-01,.6981600E-01,.7317800E-01,.7461600E-01,.7409300E-01,&
     & .7084300E-01,.6073800E-01,.4613700E-01,.5786200E-01,.6506000E-01,&
     & .7028300E-01,.7372400E-01,.7506700E-01,.7440800E-01,.7113000E-01,&
     & .6138500E-01,.4649000E-01,.5824000E-01,.6553700E-01,.7075200E-01,&
     & .7418700E-01,.7554500E-01,.7481700E-01,.7147900E-01,.6199800E-01,&
     & .3891000E-01,.5082300E-01,.5750500E-01,.6236400E-01,.6555900E-01,&
     & .6715400E-01,.6679800E-01,.6353100E-01,.5240100E-01,.3948800E-01,&
     & .5130700E-01,.5802200E-01,.6291200E-01,.6607400E-01,.6751800E-01,&
     & .6702500E-01,.6384000E-01,.5309300E-01,.3999100E-01,.5181200E-01,&
     & .5856600E-01,.6345100E-01,.6663200E-01,.6801000E-01,.6734800E-01,&
     & .6416900E-01,.5378200E-01,.4038700E-01,.5229100E-01,.5911700E-01,&
     & .6400100E-01,.6719600E-01,.6851400E-01,.6778900E-01,.6455500E-01,&
     & .5447000E-01,.4071900E-01,.5273900E-01,.5969100E-01,.6457200E-01,&
     & .6772500E-01,.6903100E-01,.6830700E-01,.6498500E-01,.5509100E-01/

      data   absa(181:315, 8)   /                                       &
     & .3369200E-01,.4545900E-01,.5170500E-01,.5611100E-01,.5898400E-01,&
     & .6038400E-01,.5995100E-01,.5677000E-01,.4593900E-01,.3425400E-01,&
     & .4597900E-01,.5225200E-01,.5666400E-01,.5951500E-01,.6081600E-01,&
     & .6027300E-01,.5714600E-01,.4664700E-01,.3473800E-01,.4653200E-01,&
     & .5284600E-01,.5723800E-01,.6007700E-01,.6135200E-01,.6071400E-01,&
     & .5756600E-01,.4735900E-01,.3511700E-01,.4704700E-01,.5345900E-01,&
     & .5784900E-01,.6066800E-01,.6190200E-01,.6125900E-01,.5803000E-01,&
     & .4803600E-01,.3542800E-01,.4750800E-01,.5408300E-01,.5849300E-01,&
     & .6125300E-01,.6248100E-01,.6185100E-01,.5852300E-01,.4864800E-01,&
     & .2891000E-01,.4042900E-01,.4617600E-01,.5010800E-01,.5260900E-01,&
     & .5375000E-01,.5328900E-01,.5028600E-01,.4006100E-01,.2943300E-01,&
     & .4096700E-01,.4673100E-01,.5065300E-01,.5314900E-01,.5423200E-01,&
     & .5368700E-01,.5071400E-01,.4075600E-01,.2988300E-01,.4152000E-01,&
     & .4734900E-01,.5125100E-01,.5373100E-01,.5479600E-01,.5420700E-01,&
     & .5118800E-01,.4144300E-01,.3025200E-01,.4202400E-01,.4799800E-01,&
     & .5191800E-01,.5433900E-01,.5538700E-01,.5482000E-01,.5171100E-01,&
     & .4208500E-01,.3054200E-01,.4244700E-01,.4863500E-01,.5259800E-01,&
     & .5498600E-01,.5603800E-01,.5543000E-01,.5223600E-01,.4267500E-01,&
     & .2460000E-01,.3572500E-01,.4095200E-01,.4439800E-01,.4655800E-01,&
     & .4746200E-01,.4695000E-01,.4422700E-01,.3475100E-01,.2506000E-01,&
     & .3625000E-01,.4149800E-01,.4495000E-01,.4709700E-01,.4797700E-01,&
     & .4739900E-01,.4469000E-01,.3541000E-01,.2546600E-01,.3676900E-01,&
     & .4211900E-01,.4557200E-01,.4769600E-01,.4854800E-01,.4796500E-01,&
     & .4519800E-01,.3605000E-01,.2580400E-01,.3723200E-01,.4276400E-01,&
     & .4624900E-01,.4834900E-01,.4917300E-01,.4860300E-01,.4573700E-01,&
     & .3664900E-01,.2607500E-01,.3764500E-01,.4337100E-01,.4695000E-01,&
     & .4904500E-01,.4985600E-01,.4922300E-01,.4627900E-01,.3719800E-01/

      data   absa(316:450, 8)   /                                       &
     & .2074700E-01,.3132500E-01,.3601600E-01,.3904700E-01,.4092100E-01,&
     & .4165600E-01,.4109700E-01,.3868200E-01,.2995500E-01,.2114500E-01,&
     & .3181100E-01,.3654100E-01,.3959700E-01,.4145500E-01,.4216900E-01,&
     & .4157500E-01,.3914500E-01,.3057700E-01,.2149700E-01,.3229000E-01,&
     & .3715500E-01,.4023800E-01,.4206700E-01,.4274400E-01,.4215400E-01,&
     & .3964800E-01,.3117800E-01,.2179600E-01,.3271700E-01,.3776600E-01,&
     & .4092300E-01,.4273800E-01,.4339400E-01,.4276800E-01,.4018500E-01,&
     & .3173300E-01,.2204400E-01,.3311200E-01,.3833800E-01,.4159700E-01,&
     & .4346200E-01,.4408700E-01,.4338500E-01,.4074000E-01,.3225400E-01,&
     & .1736000E-01,.2721800E-01,.3140100E-01,.3408400E-01,.3574100E-01,&
     & .3637300E-01,.3579800E-01,.3359400E-01,.2566400E-01,.1769300E-01,&
     & .2766200E-01,.3190800E-01,.3462200E-01,.3626400E-01,.3687700E-01,&
     & .3628500E-01,.3404900E-01,.2625100E-01,.1799200E-01,.2808900E-01,&
     & .3247700E-01,.3525500E-01,.3687400E-01,.3744400E-01,.3683100E-01,&
     & .3454400E-01,.2681400E-01,.1825800E-01,.2848100E-01,.3305000E-01,&
     & .3590900E-01,.3755200E-01,.3807800E-01,.3742100E-01,.3507300E-01,&
     & .2733500E-01,.1848400E-01,.2884900E-01,.3360100E-01,.3653600E-01,&
     & .3825400E-01,.3875600E-01,.3802600E-01,.3561600E-01,.2782900E-01,&
     & .1446200E-01,.2346300E-01,.2718600E-01,.2958800E-01,.3107900E-01,&
     & .3162500E-01,.3105300E-01,.2901200E-01,.2191600E-01,.1473700E-01,&
     & .2385300E-01,.2766200E-01,.3010900E-01,.3158100E-01,.3211600E-01,&
     & .3152700E-01,.2946100E-01,.2246000E-01,.1498300E-01,.2423800E-01,&
     & .2818700E-01,.3071000E-01,.3217600E-01,.3266400E-01,.3204400E-01,&
     & .2994800E-01,.2298700E-01,.1522800E-01,.2459300E-01,.2871800E-01,&
     & .3131200E-01,.3282600E-01,.3328200E-01,.3261500E-01,.3046700E-01,&
     & .2347800E-01,.1541100E-01,.2493200E-01,.2922800E-01,.3190300E-01,&
     & .3345500E-01,.3394300E-01,.3320700E-01,.3099400E-01,.2393900E-01/

      data   absa(451:585, 8)   /                                       &
     & .1209700E-01,.2023000E-01,.2355400E-01,.2573700E-01,.2707000E-01,&
     & .2752600E-01,.2698700E-01,.2512500E-01,.1884300E-01,.1231600E-01,&
     & .2056400E-01,.2401400E-01,.2626500E-01,.2758900E-01,.2802100E-01,&
     & .2746200E-01,.2558000E-01,.1934100E-01,.1252500E-01,.2089300E-01,&
     & .2449400E-01,.2681700E-01,.2817200E-01,.2857800E-01,.2798500E-01,&
     & .2607300E-01,.1981100E-01,.1269800E-01,.2121200E-01,.2497100E-01,&
     & .2735800E-01,.2875200E-01,.2919600E-01,.2855400E-01,.2657500E-01,&
     & .2025400E-01,.1281300E-01,.2152700E-01,.2542800E-01,.2788600E-01,&
     & .2931600E-01,.2979400E-01,.2914400E-01,.2709500E-01,.2066900E-01,&
     & .1007900E-01,.1733200E-01,.2032600E-01,.2228300E-01,.2345200E-01,&
     & .2384000E-01,.2335100E-01,.2170100E-01,.1615000E-01,.1025300E-01,&
     & .1762400E-01,.2073800E-01,.2277200E-01,.2396300E-01,.2434300E-01,&
     & .2383000E-01,.2215900E-01,.1660500E-01,.1041200E-01,.1791400E-01,&
     & .2116500E-01,.2325700E-01,.2449100E-01,.2490700E-01,.2436400E-01,&
     & .2263300E-01,.1703100E-01,.1052800E-01,.1820300E-01,.2158500E-01,&
     & .2373200E-01,.2501900E-01,.2546600E-01,.2492500E-01,.2312300E-01,&
     & .1744000E-01,.1059500E-01,.1849100E-01,.2198700E-01,.2420500E-01,&
     & .2552600E-01,.2601300E-01,.2549700E-01,.2363300E-01,.1782500E-01,&
     & .8367500E-02,.1477800E-01,.1745500E-01,.1920100E-01,.2021400E-01,&
     & .2054500E-01,.2013300E-01,.1870300E-01,.1381000E-01,.8503700E-02,&
     & .1503300E-01,.1782400E-01,.1962100E-01,.2068400E-01,.2105200E-01,&
     & .2061600E-01,.1914500E-01,.1421800E-01,.8612900E-02,.1530300E-01,&
     & .1819700E-01,.2004300E-01,.2116300E-01,.2157700E-01,.2114300E-01,&
     & .1960000E-01,.1461700E-01,.8691200E-02,.1556800E-01,.1856500E-01,&
     & .2047200E-01,.2163600E-01,.2209100E-01,.2167400E-01,.2009000E-01,&
     & .1499100E-01,.8730200E-02,.1580000E-01,.1891700E-01,.2088700E-01,&
     & .2210300E-01,.2259400E-01,.2219800E-01,.2058600E-01,.1535100E-01/

      data   absa(  1:180, 9)   /                                       &
     & .2875600E+00,.2846608E+00,.3007336E+00,.3116711E+00,.3128429E+00,&
     & .3061657E+00,.2955131E+00,.2798330E+00,.2885174E+00,.2846791E+00,&
     & .2818630E+00,.2979418E+00,.3088891E+00,.3105850E+00,.3045729E+00,&
     & .2947321E+00,.2798508E+00,.2884665E+00,.2823420E+00,.2795426E+00,&
     & .2956237E+00,.3062904E+00,.3083329E+00,.3027922E+00,.2937895E+00,&
     & .2797232E+00,.2884416E+00,.2804602E+00,.2775936E+00,.2936396E+00,&
     & .3040736E+00,.3061042E+00,.3009290E+00,.2927341E+00,.2793614E+00,&
     & .2882076E+00,.2789708E+00,.2760060E+00,.2919045E+00,.3021032E+00,&
     & .3039633E+00,.2991521E+00,.2914947E+00,.2788660E+00,.2878374E+00,&
     & .2691839E+00,.2738289E+00,.2925806E+00,.3047303E+00,.3076965E+00,&
     & .3021735E+00,.2903704E+00,.2740484E+00,.2753391E+00,.2667388E+00,&
     & .2713965E+00,.2901694E+00,.3021752E+00,.3055593E+00,.3006242E+00,&
     & .2897230E+00,.2740378E+00,.2755566E+00,.2649469E+00,.2695060E+00,&
     & .2881919E+00,.2999513E+00,.3034581E+00,.2989484E+00,.2889087E+00,&
     & .2738977E+00,.2755970E+00,.2636807E+00,.2680315E+00,.2865238E+00,&
     & .2980684E+00,.3014061E+00,.2973517E+00,.2879069E+00,.2735423E+00,&
     & .2754315E+00,.2627764E+00,.2669319E+00,.2851547E+00,.2963620E+00,&
     & .2995768E+00,.2957673E+00,.2866862E+00,.2730982E+00,.2752015E+00,&
     & .2490195E+00,.2608954E+00,.2819694E+00,.2948392E+00,.2994181E+00,&
     & .2960253E+00,.2839092E+00,.2667913E+00,.2610712E+00,.2470483E+00,&
     & .2588078E+00,.2799097E+00,.2926407E+00,.2974527E+00,.2945959E+00,&
     & .2834853E+00,.2668456E+00,.2614061E+00,.2457732E+00,.2572386E+00,&
     & .2782150E+00,.2907979E+00,.2955550E+00,.2931132E+00,.2827947E+00,&
     & .2667547E+00,.2614625E+00,.2450220E+00,.2561881E+00,.2768616E+00,&
     & .2892386E+00,.2937867E+00,.2916947E+00,.2819208E+00,.2664583E+00,&
     & .2614477E+00,.2447576E+00,.2555647E+00,.2758063E+00,.2878616E+00,&
     & .2922788E+00,.2903473E+00,.2808432E+00,.2661024E+00,.2614835E+00,&
     & .2278433E+00,.2459460E+00,.2687748E+00,.2823420E+00,.2883606E+00,&
     & .2867773E+00,.2761135E+00,.2583114E+00,.2469847E+00,.2262821E+00,&
     & .2442492E+00,.2669664E+00,.2804797E+00,.2866383E+00,.2856021E+00,&
     & .2758521E+00,.2584755E+00,.2474068E+00,.2254367E+00,.2430836E+00,&
     & .2656062E+00,.2789372E+00,.2850299E+00,.2843218E+00,.2753572E+00,&
     & .2584303E+00,.2476169E+00,.2252826E+00,.2424728E+00,.2646308E+00,&
     & .2777122E+00,.2836401E+00,.2831157E+00,.2746060E+00,.2582669E+00,&
     & .2478333E+00,.2256551E+00,.2423529E+00,.2639731E+00,.2766779E+00,&
     & .2825116E+00,.2820352E+00,.2736912E+00,.2580734E+00,.2481106E+00/

      data   absa(181:315, 9)   /                                       &
     & .2062751E+00,.2298044E+00,.2532990E+00,.2676292E+00,.2748784E+00,&
     & .2747996E+00,.2666545E+00,.2487411E+00,.2325541E+00,.2051043E+00,&
     & .2284554E+00,.2518239E+00,.2660870E+00,.2734853E+00,.2738569E+00,&
     & .2665350E+00,.2490789E+00,.2332102E+00,.2047919E+00,.2277211E+00,&
     & .2507972E+00,.2649403E+00,.2721940E+00,.2728699E+00,.2661103E+00,&
     & .2492251E+00,.2337220E+00,.2051961E+00,.2275744E+00,.2501902E+00,&
     & .2640418E+00,.2711870E+00,.2720181E+00,.2654582E+00,.2492504E+00,&
     & .2342238E+00,.2061900E+00,.2279645E+00,.2499630E+00,.2633833E+00,&
     & .2704230E+00,.2712077E+00,.2647461E+00,.2492510E+00,.2347478E+00,&
     & .1853388E+00,.2130833E+00,.2365145E+00,.2513589E+00,.2593134E+00,&
     & .2609466E+00,.2549939E+00,.2376988E+00,.2170867E+00,.1845601E+00,&
     & .2121024E+00,.2353552E+00,.2502019E+00,.2582784E+00,.2602288E+00,&
     & .2550486E+00,.2382771E+00,.2180680E+00,.1846602E+00,.2117219E+00,&
     & .2346887E+00,.2493886E+00,.2574166E+00,.2595177E+00,.2547983E+00,&
     & .2386926E+00,.2189256E+00,.1855458E+00,.2119966E+00,.2344799E+00,&
     & .2488166E+00,.2567948E+00,.2589356E+00,.2543373E+00,.2389459E+00,&
     & .2197901E+00,.1870772E+00,.2128598E+00,.2346986E+00,.2485625E+00,&
     & .2564000E+00,.2584353E+00,.2538996E+00,.2391912E+00,.2205482E+00,&
     & .1654511E+00,.1961135E+00,.2190853E+00,.2338982E+00,.2425818E+00,&
     & .2456184E+00,.2412218E+00,.2252744E+00,.2008814E+00,.1650292E+00,&
     & .1954131E+00,.2182427E+00,.2330965E+00,.2418982E+00,.2451802E+00,&
     & .2414790E+00,.2260855E+00,.2021875E+00,.1654771E+00,.1953910E+00,&
     & .2179136E+00,.2326080E+00,.2414414E+00,.2447571E+00,.2414612E+00,&
     & .2267328E+00,.2033987E+00,.1667613E+00,.1960735E+00,.2180521E+00,&
     & .2324354E+00,.2411480E+00,.2444710E+00,.2413228E+00,.2272824E+00,&
     & .2045050E+00,.1686834E+00,.1973572E+00,.2187006E+00,.2325944E+00,&
     & .2410697E+00,.2443064E+00,.2412853E+00,.2277903E+00,.2054612E+00/

      data   absa(316:450, 9)   /                                       &
     & .1466987E+00,.1792921E+00,.2014461E+00,.2160765E+00,.2251959E+00,&
     & .2288858E+00,.2256137E+00,.2115906E+00,.1843005E+00,.1465091E+00,&
     & .1788078E+00,.2009639E+00,.2155790E+00,.2248838E+00,.2288389E+00,&
     & .2260797E+00,.2126564E+00,.1858920E+00,.1472457E+00,.1790577E+00,&
     & .2009403E+00,.2154072E+00,.2247365E+00,.2287839E+00,.2263437E+00,&
     & .2135828E+00,.1874072E+00,.1487936E+00,.1800885E+00,.2014536E+00,&
     & .2156062E+00,.2248091E+00,.2288471E+00,.2265932E+00,.2144085E+00,&
     & .1887703E+00,.1510004E+00,.1817540E+00,.2025068E+00,.2162005E+00,&
     & .2250829E+00,.2290843E+00,.2269522E+00,.2151187E+00,.1899588E+00,&
     & .1290166E+00,.1628134E+00,.1841393E+00,.1984523E+00,.2074091E+00,&
     & .2111610E+00,.2086918E+00,.1966022E+00,.1675581E+00,.1290151E+00,&
     & .1624988E+00,.1838874E+00,.1982834E+00,.2074512E+00,.2114622E+00,&
     & .2094660E+00,.1978766E+00,.1694333E+00,.1299542E+00,.1629858E+00,&
     & .1841354E+00,.1984427E+00,.2076315E+00,.2117841E+00,.2100747E+00,&
     & .1990529E+00,.1712079E+00,.1317221E+00,.1642880E+00,.1849872E+00,&
     & .1989586E+00,.2080569E+00,.2122421E+00,.2107272E+00,.2001600E+00,&
     & .1728028E+00,.1340336E+00,.1662556E+00,.1864180E+00,.1999647E+00,&
     & .2087569E+00,.2128849E+00,.2114836E+00,.2011527E+00,.1742183E+00,&
     & .1126804E+00,.1466556E+00,.1672889E+00,.1811021E+00,.1894716E+00,&
     & .1930711E+00,.1911967E+00,.1807203E+00,.1511468E+00,.1128902E+00,&
     & .1466324E+00,.1672756E+00,.1812574E+00,.1898492E+00,.1936951E+00,&
     & .1922455E+00,.1822647E+00,.1532592E+00,.1140256E+00,.1474289E+00,&
     & .1678506E+00,.1817516E+00,.1903705E+00,.1944050E+00,.1932330E+00,&
     & .1836902E+00,.1552103E+00,.1159334E+00,.1490190E+00,.1690437E+00,&
     & .1826551E+00,.1911836E+00,.1952461E+00,.1942671E+00,.1850452E+00,&
     & .1569828E+00,.1181595E+00,.1512541E+00,.1708193E+00,.1840599E+00,&
     & .1923574E+00,.1963008E+00,.1954232E+00,.1863167E+00,.1585924E+00/

      data   absa(451:585, 9)   /                                       &
     & .9798189E-01,.1313265E+00,.1510348E+00,.1641642E+00,.1721410E+00,&
     & .1755759E+00,.1741644E+00,.1652309E+00,.1361983E+00,.9876591E-01,&
     & .1319241E+00,.1515858E+00,.1647606E+00,.1728744E+00,.1765573E+00,&
     & .1754929E+00,.1669557E+00,.1384236E+00,.1003841E+00,.1333210E+00,&
     & .1527464E+00,.1657657E+00,.1738505E+00,.1776840E+00,.1768404E+00,&
     & .1686021E+00,.1404407E+00,.1023815E+00,.1354349E+00,.1545144E+00,&
     & .1673004E+00,.1752277E+00,.1790049E+00,.1782869E+00,.1701818E+00,&
     & .1423084E+00,.1045182E+00,.1378387E+00,.1568227E+00,.1693189E+00,&
     & .1770176E+00,.1806205E+00,.1797803E+00,.1716817E+00,.1441206E+00,&
     & .8513105E-01,.1174272E+00,.1357845E+00,.1479723E+00,.1555369E+00,&
     & .1588474E+00,.1576920E+00,.1498764E+00,.1219425E+00,.8639013E-01,&
     & .1185546E+00,.1369085E+00,.1490184E+00,.1566903E+00,.1601862E+00,&
     & .1592810E+00,.1518099E+00,.1242064E+00,.8811019E-01,.1204518E+00,&
     & .1385898E+00,.1506015E+00,.1581843E+00,.1617403E+00,.1609764E+00,&
     & .1536670E+00,.1263021E+00,.9004252E-01,.1227432E+00,.1408853E+00,&
     & .1527101E+00,.1601143E+00,.1635634E+00,.1627369E+00,.1554561E+00,&
     & .1283211E+00,.9212719E-01,.1251177E+00,.1434973E+00,.1553078E+00,&
     & .1624820E+00,.1656416E+00,.1645751E+00,.1572251E+00,.1303753E+00,&
     & .7386256E-01,.1049065E+00,.1217281E+00,.1327938E+00,.1397977E+00,&
     & .1430454E+00,.1419673E+00,.1350204E+00,.1084738E+00,.7529440E-01,&
     & .1065145E+00,.1232849E+00,.1343471E+00,.1414069E+00,.1447364E+00,&
     & .1438645E+00,.1371204E+00,.1108041E+00,.7697705E-01,.1086621E+00,&
     & .1254864E+00,.1364483E+00,.1434325E+00,.1467317E+00,.1458386E+00,&
     & .1392068E+00,.1129989E+00,.7887272E-01,.1109542E+00,.1281014E+00,&
     & .1390876E+00,.1459050E+00,.1489849E+00,.1479204E+00,.1412350E+00,&
     & .1152445E+00,.8092106E-01,.1133621E+00,.1308019E+00,.1420681E+00,&
     & .1487342E+00,.1514606E+00,.1501482E+00,.1433560E+00,.1175338E+00/

      data   absa(  1:180,10)   /                                       &
     & .1892134E+01,.1658490E+01,.1482317E+01,.1379089E+01,.1354260E+01,&
     & .1396373E+01,.1493971E+01,.1670802E+01,.1862417E+01,.1891097E+01,&
     & .1657654E+01,.1481328E+01,.1378847E+01,.1353670E+01,.1394913E+01,&
     & .1490545E+01,.1665288E+01,.1856307E+01,.1886928E+01,.1654075E+01,&
     & .1478255E+01,.1377182E+01,.1351932E+01,.1393126E+01,.1486947E+01,&
     & .1659721E+01,.1849481E+01,.1880181E+01,.1648189E+01,.1473312E+01,&
     & .1373916E+01,.1349297E+01,.1390619E+01,.1483307E+01,.1654665E+01,&
     & .1843360E+01,.1870673E+01,.1639925E+01,.1466606E+01,.1369235E+01,&
     & .1345693E+01,.1387158E+01,.1479894E+01,.1649791E+01,.1837634E+01,&
     & .2033645E+01,.1786076E+01,.1609696E+01,.1508649E+01,.1481540E+01,&
     & .1520376E+01,.1619882E+01,.1790721E+01,.2011948E+01,.2032041E+01,&
     & .1784578E+01,.1608196E+01,.1508007E+01,.1480683E+01,.1518803E+01,&
     & .1616344E+01,.1785487E+01,.2005460E+01,.2027050E+01,.1780171E+01,&
     & .1604546E+01,.1505776E+01,.1478425E+01,.1516595E+01,.1612478E+01,&
     & .1780218E+01,.1999063E+01,.2019259E+01,.1773344E+01,.1599018E+01,&
     & .1501972E+01,.1475347E+01,.1513286E+01,.1608726E+01,.1775367E+01,&
     & .1993239E+01,.2008510E+01,.1763923E+01,.1591459E+01,.1496491E+01,&
     & .1471124E+01,.1509095E+01,.1605098E+01,.1770446E+01,.1987403E+01,&
     & .2178944E+01,.1920080E+01,.1745231E+01,.1648443E+01,.1618386E+01,&
     & .1649487E+01,.1748723E+01,.1914679E+01,.2160515E+01,.2176816E+01,&
     & .1918278E+01,.1743380E+01,.1647385E+01,.1617218E+01,.1647942E+01,&
     & .1744763E+01,.1909779E+01,.2154360E+01,.2171431E+01,.1913602E+01,&
     & .1739533E+01,.1644632E+01,.1614855E+01,.1645433E+01,.1740900E+01,&
     & .1904856E+01,.2148715E+01,.2162892E+01,.1906049E+01,.1733600E+01,&
     & .1640053E+01,.1611602E+01,.1641860E+01,.1736869E+01,.1900085E+01,&
     & .2142847E+01,.2151134E+01,.1895842E+01,.1725453E+01,.1633876E+01,&
     & .1606822E+01,.1637313E+01,.1732726E+01,.1894852E+01,.2136670E+01,&
     & .2323030E+01,.2056823E+01,.1886477E+01,.1793946E+01,.1761503E+01,&
     & .1786099E+01,.1877376E+01,.2039031E+01,.2297025E+01,.2320669E+01,&
     & .2054539E+01,.1884669E+01,.1792680E+01,.1760212E+01,.1784371E+01,&
     & .1873523E+01,.2034631E+01,.2291545E+01,.2314935E+01,.2049383E+01,&
     & .1880494E+01,.1789605E+01,.1757829E+01,.1781704E+01,.1869518E+01,&
     & .2030102E+01,.2286263E+01,.2305612E+01,.2041188E+01,.1873956E+01,&
     & .1784458E+01,.1754036E+01,.1777972E+01,.1865353E+01,.2025199E+01,&
     & .2280190E+01,.2293056E+01,.2030193E+01,.1865150E+01,.1777683E+01,&
     & .1748548E+01,.1773179E+01,.1860971E+01,.2019743E+01,.2273813E+01/

      data   absa(181:315,10)   /                                       &
     & .2463042E+01,.2192642E+01,.2030621E+01,.1942197E+01,.1908343E+01,&
     & .1927535E+01,.2006081E+01,.2161944E+01,.2425159E+01,.2460420E+01,&
     & .2190039E+01,.2028628E+01,.1940843E+01,.1906973E+01,.1925788E+01,&
     & .2002593E+01,.2157807E+01,.2420144E+01,.2453905E+01,.2184091E+01,&
     & .2024115E+01,.1937152E+01,.1904425E+01,.1922817E+01,.1998960E+01,&
     & .2153199E+01,.2414552E+01,.2443881E+01,.2175206E+01,.2017059E+01,&
     & .1931637E+01,.1900107E+01,.1918693E+01,.1994945E+01,.2148178E+01,&
     & .2408426E+01,.2430382E+01,.2163388E+01,.2007456E+01,.1924180E+01,&
     & .1893992E+01,.1913520E+01,.1990266E+01,.2142434E+01,.2401933E+01,&
     & .2595361E+01,.2325394E+01,.2174062E+01,.2090012E+01,.2056871E+01,&
     & .2069942E+01,.2136930E+01,.2284007E+01,.2549107E+01,.2592280E+01,&
     & .2322210E+01,.2171892E+01,.2088399E+01,.2055272E+01,.2068312E+01,&
     & .2133783E+01,.2280033E+01,.2544255E+01,.2585394E+01,.2315981E+01,&
     & .2166893E+01,.2084474E+01,.2052146E+01,.2065367E+01,.2130298E+01,&
     & .2275321E+01,.2538577E+01,.2574759E+01,.2306564E+01,.2159165E+01,&
     & .2078513E+01,.2047302E+01,.2061084E+01,.2126258E+01,.2270085E+01,&
     & .2532268E+01,.2560534E+01,.2294295E+01,.2148978E+01,.2070421E+01,&
     & .2040584E+01,.2055565E+01,.2121352E+01,.2264122E+01,.2525794E+01,&
     & .2717730E+01,.2453250E+01,.2313374E+01,.2235402E+01,.2202356E+01,&
     & .2210413E+01,.2268848E+01,.2403823E+01,.2668217E+01,.2714408E+01,&
     & .2449934E+01,.2311069E+01,.2233513E+01,.2200700E+01,.2208877E+01,&
     & .2266063E+01,.2400211E+01,.2663785E+01,.2707312E+01,.2443349E+01,&
     & .2305816E+01,.2229419E+01,.2197247E+01,.2206137E+01,.2262718E+01,&
     & .2395786E+01,.2658274E+01,.2696315E+01,.2433710E+01,.2297781E+01,&
     & .2222901E+01,.2191947E+01,.2201655E+01,.2258338E+01,.2390352E+01,&
     & .2652236E+01,.2681738E+01,.2420849E+01,.2287046E+01,.2214166E+01,&
     & .2184829E+01,.2195666E+01,.2252886E+01,.2384216E+01,.2646214E+01/

      data   absa(316:450,10)   /                                       &
     & .2830476E+01,.2574810E+01,.2446984E+01,.2374964E+01,.2342639E+01,&
     & .2348427E+01,.2400182E+01,.2520854E+01,.2781859E+01,.2827137E+01,&
     & .2571641E+01,.2444361E+01,.2373065E+01,.2340938E+01,.2346711E+01,&
     & .2397962E+01,.2517827E+01,.2778017E+01,.2819904E+01,.2565126E+01,&
     & .2438846E+01,.2368655E+01,.2337305E+01,.2343718E+01,.2394597E+01,&
     & .2513695E+01,.2772893E+01,.2808916E+01,.2555136E+01,.2430441E+01,&
     & .2361733E+01,.2331640E+01,.2338952E+01,.2389996E+01,.2508304E+01,&
     & .2767177E+01,.2794379E+01,.2541925E+01,.2419332E+01,.2352428E+01,&
     & .2324012E+01,.2332369E+01,.2384042E+01,.2502259E+01,.2761425E+01,&
     & .2933800E+01,.2689436E+01,.2572674E+01,.2506396E+01,.2476838E+01,&
     & .2482312E+01,.2528761E+01,.2636078E+01,.2889382E+01,.2930816E+01,&
     & .2686703E+01,.2570265E+01,.2504494E+01,.2475090E+01,.2480774E+01,&
     & .2526766E+01,.2633891E+01,.2886327E+01,.2923537E+01,.2680192E+01,&
     & .2564715E+01,.2499735E+01,.2471291E+01,.2477512E+01,.2523484E+01,&
     & .2630022E+01,.2881726E+01,.2912511E+01,.2670142E+01,.2556117E+01,&
     & .2492614E+01,.2465215E+01,.2472392E+01,.2518562E+01,.2624696E+01,&
     & .2876581E+01,.2898553E+01,.2656790E+01,.2544582E+01,.2482931E+01,&
     & .2457049E+01,.2465309E+01,.2512229E+01,.2618696E+01,.2871034E+01,&
     & .3027058E+01,.2797212E+01,.2690074E+01,.2629551E+01,.2604067E+01,&
     & .2610197E+01,.2652331E+01,.2748363E+01,.2990031E+01,.3023852E+01,&
     & .2794164E+01,.2687619E+01,.2627303E+01,.2602246E+01,.2608704E+01,&
     & .2650569E+01,.2746538E+01,.2987720E+01,.3016507E+01,.2787386E+01,&
     & .2681752E+01,.2622344E+01,.2598218E+01,.2605186E+01,.2647180E+01,&
     & .2742958E+01,.2983973E+01,.3005577E+01,.2777090E+01,.2672822E+01,&
     & .2614852E+01,.2591693E+01,.2599702E+01,.2642041E+01,.2737917E+01,&
     & .2979411E+01,.2992905E+01,.2763507E+01,.2660993E+01,.2604652E+01,&
     & .2582864E+01,.2591914E+01,.2635114E+01,.2731878E+01,.2974194E+01/

      data   absa(451:585,10)   /                                       &
     & .3108634E+01,.2895420E+01,.2798425E+01,.2744368E+01,.2722653E+01,&
     & .2730122E+01,.2769336E+01,.2855617E+01,.3083249E+01,.3103552E+01,&
     & .2890636E+01,.2794237E+01,.2740840E+01,.2719834E+01,.2727815E+01,&
     & .2767022E+01,.2853516E+01,.3081043E+01,.3094635E+01,.2882058E+01,&
     & .2786673E+01,.2734481E+01,.2714488E+01,.2723042E+01,.2762618E+01,&
     & .2849365E+01,.3077793E+01,.3083887E+01,.2870120E+01,.2776133E+01,&
     & .2725197E+01,.2706459E+01,.2715971E+01,.2756337E+01,.2843995E+01,&
     & .3073502E+01,.3072208E+01,.2856377E+01,.2762710E+01,.2713207E+01,&
     & .2695854E+01,.2706674E+01,.2748406E+01,.2837477E+01,.3067985E+01,&
     & .3179487E+01,.2983632E+01,.2897614E+01,.2850758E+01,.2832739E+01,&
     & .2841435E+01,.2878587E+01,.2957109E+01,.3169588E+01,.3172853E+01,&
     & .2977291E+01,.2891777E+01,.2845999E+01,.2828739E+01,.2838054E+01,&
     & .2875642E+01,.2954402E+01,.3167982E+01,.3164067E+01,.2967272E+01,&
     & .2882837E+01,.2838175E+01,.2822030E+01,.2832002E+01,.2870360E+01,&
     & .2950082E+01,.3164988E+01,.3154001E+01,.2955025E+01,.2870648E+01,&
     & .2827230E+01,.2812357E+01,.2823496E+01,.2863128E+01,.2944286E+01,&
     & .3160501E+01,.3142799E+01,.2941633E+01,.2856439E+01,.2813383E+01,&
     & .2800017E+01,.2812937E+01,.2854322E+01,.2937121E+01,.3154451E+01,&
     & .3241097E+01,.3062217E+01,.2987360E+01,.2948157E+01,.2934292E+01,&
     & .2943847E+01,.2979640E+01,.3051660E+01,.3249168E+01,.3234006E+01,&
     & .3054284E+01,.2980255E+01,.2942054E+01,.2929013E+01,.2939386E+01,&
     & .2975962E+01,.3048876E+01,.3247803E+01,.3225888E+01,.3043668E+01,&
     & .2969796E+01,.2932743E+01,.2920751E+01,.2932040E+01,.2969899E+01,&
     & .3044051E+01,.3244864E+01,.3216320E+01,.3031794E+01,.2956859E+01,&
     & .2920139E+01,.2909436E+01,.2922425E+01,.2961874E+01,.3037648E+01,&
     & .3239868E+01,.3205483E+01,.3018784E+01,.2942726E+01,.2905212E+01,&
     & .2895638E+01,.2910719E+01,.2952008E+01,.3029378E+01,.3233244E+01/

!     the array absb(1175,10) (kb(5,5,13:59,10)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 10,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:175, 1)   /                                       &
     & .4063300E-08,.6471800E-06,.7956700E-06,.8126400E-06,.2508000E-07,&
     & .3759600E-08,.7042900E-06,.9085500E-06,.9643200E-06,.2829200E-07,&
     & .3498200E-08,.7637000E-06,.1031600E-05,.1147200E-05,.3126500E-07,&
     & .3270700E-08,.8283400E-06,.1157700E-05,.1338700E-05,.3492800E-07,&
     & .3071000E-08,.9025200E-06,.1296200E-05,.1520600E-05,.3885900E-07,&
     & .3302600E-08,.5359200E-06,.6617100E-06,.6763300E-06,.2076700E-07,&
     & .3057400E-08,.5795900E-06,.7538600E-06,.8039500E-06,.2333400E-07,&
     & .2846100E-08,.6289700E-06,.8548600E-06,.9574500E-06,.2584100E-07,&
     & .2662100E-08,.6835200E-06,.9613200E-06,.1114800E-05,.2886200E-07,&
     & .2500500E-08,.7451700E-06,.1073900E-05,.1263200E-05,.3209300E-07,&
     & .2682100E-08,.4408100E-06,.5509300E-06,.5646500E-06,.1718600E-07,&
     & .2484500E-08,.4777300E-06,.6263600E-06,.6718000E-06,.1923400E-07,&
     & .2314000E-08,.5181100E-06,.7079100E-06,.7965800E-06,.2134800E-07,&
     & .2165400E-08,.5641600E-06,.7989300E-06,.9287300E-06,.2383200E-07,&
     & .2034700E-08,.6153500E-06,.8898500E-06,.1049900E-05,.2648500E-07,&
     & .2177500E-08,.3625400E-06,.4587000E-06,.4712500E-06,.1421100E-07,&
     & .2018300E-08,.3937600E-06,.5205300E-06,.5616300E-06,.1586900E-07,&
     & .1880800E-08,.4268700E-06,.5869900E-06,.6638600E-06,.1762700E-07,&
     & .1760800E-08,.4655900E-06,.6638800E-06,.7706300E-06,.1966400E-07,&
     & .1655200E-08,.5074700E-06,.7369900E-06,.8724900E-06,.2184400E-07,&
     & .1768200E-08,.2983600E-06,.3810400E-06,.3931400E-06,.1173500E-07,&
     & .1639900E-08,.3249300E-06,.4321500E-06,.4689700E-06,.1307700E-07,&
     & .1529000E-08,.3520300E-06,.4866500E-06,.5532800E-06,.1453500E-07,&
     & .1432100E-08,.3838300E-06,.5503300E-06,.6384400E-06,.1620800E-07,&
     & .1346800E-08,.4182200E-06,.6095000E-06,.7237600E-06,.1799500E-07,&
     & .1436200E-08,.2456000E-06,.3166700E-06,.3279700E-06,.9683100E-08,&
     & .1332700E-08,.2678400E-06,.3587200E-06,.3913400E-06,.1077300E-07,&
     & .1243200E-08,.2900800E-06,.4033400E-06,.4602600E-06,.1198000E-07,&
     & .1165000E-08,.3163700E-06,.4558300E-06,.5288000E-06,.1335000E-07,&
     & .1096000E-08,.3450300E-06,.5036300E-06,.5996700E-06,.1481500E-07,&
     & .1166700E-08,.2023100E-06,.2631300E-06,.2735900E-06,.7993400E-08,&
     & .1083300E-08,.2208500E-06,.2976000E-06,.3262000E-06,.8880700E-08,&
     & .1011000E-08,.2391200E-06,.3342300E-06,.3827100E-06,.9877100E-08,&
     & .9477800E-09,.2607300E-06,.3768500E-06,.4379100E-06,.1100200E-07,&
     & .8919800E-09,.2842500E-06,.4158900E-06,.4968200E-06,.1220400E-07/

      data   absb(176:350, 1)   /                                       &
     & .9466800E-09,.1673500E-06,.2182200E-06,.2291400E-06,.6623700E-08,&
     & .8795600E-09,.1825500E-06,.2477000E-06,.2729000E-06,.7347000E-08,&
     & .8213200E-09,.1977600E-06,.2778600E-06,.3191300E-06,.8173200E-08,&
     & .7703200E-09,.2155700E-06,.3119700E-06,.3638000E-06,.9101900E-08,&
     & .7252800E-09,.2345800E-06,.3444900E-06,.4125700E-06,.1008400E-07,&
     & .7680900E-09,.1385800E-06,.1813800E-06,.1918800E-06,.5492900E-08,&
     & .7140800E-09,.1508800E-06,.2061400E-06,.2283500E-06,.6082500E-08,&
     & .6671700E-09,.1636600E-06,.2311100E-06,.2662200E-06,.6768000E-08,&
     & .6260500E-09,.1781600E-06,.2583400E-06,.3024200E-06,.7534400E-08,&
     & .5897000E-09,.1937400E-06,.2854200E-06,.3422200E-06,.8339400E-08,&
     & .6201300E-09,.1154700E-06,.1521200E-06,.1622700E-06,.4592600E-08,&
     & .5770900E-09,.1255200E-06,.1721100E-06,.1926500E-06,.5072800E-08,&
     & .5396400E-09,.1361900E-06,.1936300E-06,.2240200E-06,.5645400E-08,&
     & .5067500E-09,.1480500E-06,.2155700E-06,.2532000E-06,.6280500E-08,&
     & .4776400E-09,.1609700E-06,.2379900E-06,.2857400E-06,.6941400E-08,&
     & .5005500E-09,.9616500E-07,.1277600E-06,.1375200E-06,.3793300E-08,&
     & .4662700E-09,.1042100E-06,.1438700E-06,.1622600E-06,.4215400E-08,&
     & .4363800E-09,.1132000E-06,.1624000E-06,.1880600E-06,.4676100E-08,&
     & .4100900E-09,.1231200E-06,.1800200E-06,.2121400E-06,.5172400E-08,&
     & .3867900E-09,.1338000E-06,.1983500E-06,.2383600E-06,.5686300E-08,&
     & .4038400E-09,.8017400E-07,.1074200E-06,.1166500E-06,.3143500E-08,&
     & .3765600E-09,.8661100E-07,.1206200E-06,.1369400E-06,.3474600E-08,&
     & .3527300E-09,.9420300E-07,.1361000E-06,.1572900E-06,.3858600E-08,&
     & .3317400E-09,.1024600E-06,.1503500E-06,.1777500E-06,.4270700E-08,&
     & .3131000E-09,.1112200E-06,.1654300E-06,.1991700E-06,.4702000E-08,&
     & .3256200E-09,.6678500E-07,.9044700E-07,.9905900E-07,.2613400E-08,&
     & .3039300E-09,.7213300E-07,.1013900E-06,.1157100E-06,.2887200E-08,&
     & .2849500E-09,.7844800E-07,.1138500E-06,.1317800E-06,.3206100E-08,&
     & .2682100E-09,.8533900E-07,.1256300E-06,.1491400E-06,.3548900E-08,&
     & .2533200E-09,.9244500E-07,.1381400E-06,.1667400E-06,.3906000E-08,&
     & .2622400E-09,.5568000E-07,.7636900E-07,.8440000E-07,.2175900E-08,&
     & .2450400E-09,.6020400E-07,.8542300E-07,.9788500E-07,.2411800E-08,&
     & .2299600E-09,.6547700E-07,.9532100E-07,.1109300E-06,.2677400E-08,&
     & .2166300E-09,.7118800E-07,.1052900E-06,.1254400E-06,.2961500E-08,&
     & .2047500E-09,.7694000E-07,.1155600E-06,.1397700E-06,.3256900E-08/

      data   absb(351:525, 1)   /                                       &
     & .2112000E-09,.4638200E-07,.6389400E-07,.7137900E-07,.1817800E-08,&
     & .1975600E-09,.5027800E-07,.7203300E-07,.8290900E-07,.2018100E-08,&
     & .1855700E-09,.5467000E-07,.7995300E-07,.9350500E-07,.2239600E-08,&
     & .1749600E-09,.5940800E-07,.8817900E-07,.1055300E-06,.2473700E-08,&
     & .1654900E-09,.6402500E-07,.9669600E-07,.1170400E-06,.2709200E-08,&
     & .1701000E-09,.3863100E-07,.5377300E-07,.6057900E-07,.1522000E-08,&
     & .1592800E-09,.4200100E-07,.6066000E-07,.6972700E-07,.1690100E-08,&
     & .1497600E-09,.4566100E-07,.6706500E-07,.7884500E-07,.1869600E-08,&
     & .1413100E-09,.4957100E-07,.7380900E-07,.8848100E-07,.2061100E-08,&
     & .1337600E-09,.5330400E-07,.8091000E-07,.9801700E-07,.2256500E-08,&
     & .1369600E-09,.3227000E-07,.4539200E-07,.5143800E-07,.1272100E-08,&
     & .1283900E-09,.3507900E-07,.5094600E-07,.5872600E-07,.1411800E-08,&
     & .1208200E-09,.3815000E-07,.5625800E-07,.6649300E-07,.1562800E-08,&
     & .1141000E-09,.4132400E-07,.6184300E-07,.7438700E-07,.1720500E-08,&
     & .1080800E-09,.4438700E-07,.6773900E-07,.8221600E-07,.1874500E-08,&
     & .1102900E-09,.2697000E-07,.3833000E-07,.4369800E-07,.1065500E-08,&
     & .1034900E-09,.2931900E-07,.4273900E-07,.4955200E-07,.1182300E-08,&
     & .9748300E-10,.3186900E-07,.4721000E-07,.5605700E-07,.1303800E-08,&
     & .9213300E-10,.3443600E-07,.5181200E-07,.6248500E-07,.1428500E-08,&
     & .8734000E-10,.3695300E-07,.5670900E-07,.6892100E-07,.1556300E-08,&
     & .8880600E-10,.2255500E-07,.3238600E-07,.3712900E-07,.8902900E-09,&
     & .8341700E-10,.2451900E-07,.3592100E-07,.4187000E-07,.9854800E-09,&
     & .7864400E-10,.2663800E-07,.3960900E-07,.4727300E-07,.1085400E-08,&
     & .7438800E-10,.2869400E-07,.4342300E-07,.5243500E-07,.1188000E-08,&
     & .7056800E-10,.3077600E-07,.4741100E-07,.5775000E-07,.1293100E-08,&
     & .7150700E-10,.1887000E-07,.2733200E-07,.3130900E-07,.7443600E-09,&
     & .6723500E-10,.2051200E-07,.3018900E-07,.3540700E-07,.8229400E-09,&
     & .6344400E-10,.2225900E-07,.3321800E-07,.3973000E-07,.9048300E-09,&
     & .6005800E-10,.2392300E-07,.3640500E-07,.4397600E-07,.9890800E-09,&
     & .5701500E-10,.2563400E-07,.3954800E-07,.4834600E-07,.1075000E-08,&
     & .5758600E-10,.1577900E-07,.2295000E-07,.2641500E-07,.6225600E-09,&
     & .5419900E-10,.1715300E-07,.2535400E-07,.2990400E-07,.6873400E-09,&
     & .5118800E-10,.1856900E-07,.2786900E-07,.3340400E-07,.7546700E-09,&
     & .4849300E-10,.1993500E-07,.3043700E-07,.3685600E-07,.8236900E-09,&
     & .4606900E-10,.2134500E-07,.3298800E-07,.4047800E-07,.8940200E-09/

      data   absb(526:700, 1)   /                                       &
     & .4648700E-10,.1316100E-07,.1921600E-07,.2221200E-07,.5191000E-09,&
     & .4378900E-10,.1430100E-07,.2118200E-07,.2509000E-07,.5724700E-09,&
     & .4138600E-10,.1544700E-07,.2320800E-07,.2793800E-07,.6276700E-09,&
     & .3923400E-10,.1657100E-07,.2534700E-07,.3079500E-07,.6842600E-09,&
     & .3729400E-10,.1772500E-07,.2739200E-07,.3378100E-07,.7417900E-09,&
     & .3775600E-10,.1089000E-07,.1588200E-07,.1844100E-07,.4287100E-09,&
     & .3558100E-10,.1183000E-07,.1751400E-07,.2081000E-07,.4725100E-09,&
     & .3364300E-10,.1275700E-07,.1918400E-07,.2314800E-07,.5176900E-09,&
     & .3190500E-10,.1368200E-07,.2094100E-07,.2547700E-07,.5639400E-09,&
     & .3033800E-10,.1462800E-07,.2261300E-07,.2794800E-07,.6110000E-09,&
     & .3087200E-10,.8929000E-08,.1300700E-07,.1510800E-07,.3503400E-09,&
     & .2909600E-10,.9699500E-08,.1434300E-07,.1705100E-07,.3860900E-09,&
     & .2751300E-10,.1045600E-07,.1571300E-07,.1896800E-07,.4229600E-09,&
     & .2609300E-10,.1121300E-07,.1715300E-07,.2087500E-07,.4607300E-09,&
     & .2481200E-10,.1198700E-07,.1852200E-07,.2289900E-07,.4991600E-09,&
     & .2547900E-10,.7233800E-08,.1050800E-07,.1216500E-07,.2826800E-09,&
     & .2400200E-10,.7859300E-08,.1159400E-07,.1374700E-07,.3117300E-09,&
     & .2268700E-10,.8484500E-08,.1271200E-07,.1531500E-07,.3417800E-09,&
     & .2150800E-10,.9099700E-08,.1389200E-07,.1688600E-07,.3726000E-09,&
     & .2044600E-10,.9732200E-08,.1501200E-07,.1852600E-07,.4039800E-09,&
     & .2104100E-10,.5856500E-08,.8483100E-08,.9789200E-08,.2279100E-09,&
     & .1981100E-10,.6363600E-08,.9361900E-08,.1107200E-07,.2514800E-09,&
     & .1871700E-10,.6878400E-08,.1027600E-07,.1236000E-07,.2759500E-09,&
     & .1773800E-10,.7379200E-08,.1123400E-07,.1363800E-07,.3010900E-09,&
     & .1685600E-10,.7897100E-08,.1216600E-07,.1497700E-07,.3267100E-09,&
     & .1737700E-10,.4741800E-08,.6856100E-08,.7879500E-08,.1837300E-09,&
     & .1635300E-10,.5153200E-08,.7562000E-08,.8912800E-08,.2028400E-09,&
     & .1544300E-10,.5575200E-08,.8306900E-08,.9966600E-08,.2227900E-09,&
     & .1462900E-10,.5985700E-08,.9087200E-08,.1101100E-07,.2432700E-09,&
     & .1389600E-10,.6408600E-08,.9861600E-08,.1210400E-07,.2642000E-09,&
     & .1444500E-10,.3809100E-08,.5479900E-08,.6275400E-08,.1467900E-09,&
     & .1358100E-10,.4139100E-08,.6053000E-08,.7090900E-08,.1622000E-09,&
     & .1281500E-10,.4488800E-08,.6655500E-08,.7953200E-08,.1784200E-09,&
     & .1213100E-10,.4820700E-08,.7285500E-08,.8807100E-08,.1950900E-09,&
     & .1151600E-10,.5164600E-08,.7926800E-08,.9693200E-08,.2121400E-09/

      data   absb(701:875, 1)   /                                       &
     & .1202100E-10,.3056600E-08,.4360600E-08,.4992100E-08,.1171300E-09,&
     & .1129100E-10,.3320500E-08,.4834000E-08,.5625600E-08,.1295400E-09,&
     & .1064500E-10,.3605300E-08,.5326100E-08,.6347300E-08,.1426900E-09,&
     & .1006900E-10,.3878200E-08,.5834700E-08,.7038600E-08,.1562400E-09,&
     & .9552000E-11,.4158000E-08,.6364700E-08,.7751500E-08,.1701300E-09,&
     & .1000700E-10,.2451600E-08,.3465300E-08,.3939800E-08,.9347500E-10,&
     & .9390200E-11,.2663100E-08,.3858800E-08,.4463500E-08,.1034200E-09,&
     & .8844900E-11,.2892600E-08,.4260000E-08,.5046600E-08,.1140600E-09,&
     & .8359400E-11,.3119800E-08,.4671900E-08,.5621800E-08,.1250700E-09,&
     & .7924400E-11,.3346500E-08,.5106500E-08,.6199900E-08,.1364000E-09,&
     & .8364700E-11,.1957800E-08,.2734100E-08,.3082700E-08,.7427700E-10,&
     & .7838900E-11,.2126300E-08,.3069000E-08,.3521200E-08,.8211800E-10,&
     & .7375200E-11,.2310300E-08,.3386500E-08,.3983600E-08,.9067200E-10,&
     & .6963300E-11,.2498400E-08,.3719800E-08,.4456900E-08,.9962800E-10,&
     & .6595000E-11,.2683300E-08,.4070900E-08,.4926500E-08,.1088300E-09,&
     & .7007000E-11,.1562800E-08,.2152100E-08,.2401700E-08,.5911600E-10,&
     & .6557000E-11,.1695400E-08,.2425400E-08,.2777200E-08,.6502700E-10,&
     & .6161200E-11,.1841300E-08,.2684500E-08,.3131000E-08,.7189200E-10,&
     & .5810600E-11,.1997700E-08,.2954200E-08,.3521100E-08,.7914400E-10,&
     & .5497600E-11,.2146400E-08,.3236000E-08,.3904500E-08,.8662900E-10,&
     & .5872800E-11,.1250000E-08,.1707900E-08,.1874300E-08,.4664600E-10,&
     & .5487300E-11,.1350500E-08,.1910500E-08,.2171900E-08,.5149900E-10,&
     & .5149200E-11,.1467000E-08,.2125000E-08,.2459000E-08,.5698200E-10,&
     & .4850400E-11,.1593100E-08,.2346400E-08,.2780200E-08,.6283900E-10,&
     & .4584400E-11,.1716700E-08,.2573100E-08,.3095400E-08,.6890800E-10,&
     & .4932800E-11,.9981100E-09,.1339700E-08,.1445700E-08,.3669400E-10,&
     & .4601200E-11,.1074200E-08,.1498600E-08,.1687500E-08,.4074200E-10,&
     & .4311300E-11,.1166500E-08,.1681300E-08,.1928100E-08,.4504900E-10,&
     & .4055800E-11,.1267300E-08,.1856700E-08,.2181700E-08,.4975100E-10,&
     & .3828800E-11,.1370200E-08,.2039900E-08,.2441900E-08,.5467900E-10,&
     & .4159200E-11,.7920500E-09,.1043800E-08,.1105500E-08,.2870900E-10,&
     & .3871700E-11,.8535700E-09,.1171000E-08,.1301500E-08,.3219800E-10,&
     & .3621500E-11,.9251500E-09,.1319600E-08,.1510300E-08,.3544800E-10,&
     & .3401600E-11,.1004600E-08,.1461900E-08,.1701000E-08,.3920300E-10,&
     & .3206900E-11,.1090400E-08,.1610000E-08,.1916800E-08,.4318700E-10/

      data   absb(876:1050, 1)   /                                      &
     & .3510000E-11,.6284900E-09,.8131900E-09,.8456700E-09,.2245600E-10,&
     & .3260500E-11,.6802100E-09,.9216900E-09,.1004000E-08,.2522900E-10,&
     & .3044100E-11,.7329400E-09,.1031400E-08,.1166500E-08,.2790300E-10,&
     & .2854700E-11,.7961500E-09,.1149500E-08,.1325100E-08,.3087900E-10,&
     & .2687400E-11,.8646300E-09,.1270300E-08,.1499400E-08,.3408200E-10,&
     & .2964900E-11,.4986700E-09,.6357200E-09,.6455700E-09,.1755400E-10,&
     & .2748000E-11,.5412400E-09,.7187100E-09,.7684000E-09,.1974900E-10,&
     & .2560600E-11,.5814000E-09,.8041300E-09,.9005400E-09,.2201200E-10,&
     & .2397200E-11,.6309700E-09,.9063700E-09,.1034900E-08,.2430800E-10,&
     & .2253400E-11,.6854500E-09,.1000600E-08,.1170400E-08,.2687200E-10,&
     & .2501900E-11,.3985300E-09,.4937400E-09,.4961500E-09,.1376200E-10,&
     & .2313600E-11,.4295200E-09,.5614100E-09,.5898000E-09,.1550000E-10,&
     & .2151700E-11,.4633400E-09,.6328400E-09,.6978800E-09,.1739000E-10,&
     & .2011000E-11,.5013000E-09,.7112400E-09,.8090300E-09,.1918400E-10,&
     & .1887500E-11,.5444600E-09,.7890800E-09,.9143500E-09,.2123000E-10,&
     & .2111100E-11,.3153400E-09,.3855200E-09,.3832400E-09,.1085100E-10,&
     & .1947800E-11,.3417900E-09,.4398600E-09,.4535500E-09,.1217600E-10,&
     & .1807900E-11,.3701600E-09,.4983500E-09,.5391900E-09,.1368300E-10,&
     & .1686800E-11,.3984600E-09,.5574600E-09,.6280500E-09,.1516400E-10,&
     & .1580900E-11,.4327200E-09,.6231000E-09,.7158700E-09,.1678500E-10,&
     & .1783000E-11,.2493900E-09,.3003300E-09,.2970800E-09,.8548000E-11,&
     & .1641000E-11,.2724400E-09,.3446700E-09,.3487000E-09,.9564200E-11,&
     & .1520000E-11,.2950700E-09,.3902000E-09,.4150500E-09,.1076400E-10,&
     & .1415600E-11,.3173600E-09,.4369300E-09,.4875800E-09,.1201600E-10,&
     & .1324600E-11,.3440900E-09,.4924300E-09,.5618100E-09,.1326700E-10,&
     & .1507400E-11,.1974600E-09,.2344200E-09,.2302700E-09,.6694200E-11,&
     & .1383800E-11,.2175400E-09,.2681200E-09,.2686500E-09,.7509500E-11,&
     & .1278900E-11,.2344500E-09,.3051000E-09,.3191600E-09,.8462600E-11,&
     & .1188800E-11,.2530500E-09,.3449600E-09,.3790600E-09,.9496300E-11,&
     & .1110600E-11,.2735500E-09,.3868000E-09,.4388200E-09,.1048600E-10,&
     & .1272200E-11,.1567600E-09,.1843600E-09,.1818800E-09,.5260100E-11,&
     & .1164900E-11,.1725600E-09,.2103000E-09,.2087600E-09,.5946500E-11,&
     & .1074300E-11,.1870600E-09,.2402200E-09,.2465700E-09,.6675000E-11,&
     & .9968300E-12,.2026300E-09,.2720600E-09,.2933700E-09,.7501600E-11,&
     & .9297500E-12,.2179800E-09,.3042000E-09,.3420300E-09,.8318200E-11/

      data   absb(1051:1175, 1)   /                                     &
     & .1073300E-11,.1255200E-09,.1466800E-09,.1447700E-09,.4144000E-11,&
     & .9802700E-12,.1370500E-09,.1648000E-09,.1629800E-09,.4713700E-11,&
     & .9020900E-12,.1496600E-09,.1891100E-09,.1911700E-09,.5273000E-11,&
     & .8354600E-12,.1621000E-09,.2141000E-09,.2275800E-09,.5934900E-11,&
     & .7780000E-12,.1742300E-09,.2397200E-09,.2672500E-09,.6616300E-11,&
     & .9062400E-12,.9978600E-10,.1167400E-09,.1128600E-09,.3277600E-11,&
     & .8254700E-12,.1089700E-09,.1293800E-09,.1270500E-09,.3716100E-11,&
     & .7579100E-12,.1200900E-09,.1480400E-09,.1482700E-09,.4164600E-11,&
     & .7005800E-12,.1292500E-09,.1683700E-09,.1763000E-09,.4695700E-11,&
     & .6513100E-12,.1393700E-09,.1899900E-09,.2089900E-09,.5274200E-11,&
     & .7659500E-12,.7882200E-10,.9386300E-10,.8752900E-10,.2603300E-11,&
     & .6956800E-12,.8655500E-10,.1019000E-09,.1002200E-09,.2928800E-11,&
     & .6372100E-12,.9538500E-10,.1164000E-09,.1153300E-09,.3299400E-11,&
     & .5878200E-12,.1032500E-09,.1326400E-09,.1366300E-09,.3718600E-11,&
     & .5455200E-12,.1117300E-09,.1504200E-09,.1625300E-09,.4181900E-11,&
     & .6469300E-12,.6242100E-10,.7617400E-10,.6832400E-10,.2083300E-11,&
     & .5858800E-12,.6927800E-10,.8119700E-10,.7969100E-10,.2313000E-11,&
     & .5353500E-12,.7587300E-10,.9141400E-10,.9039800E-10,.2635000E-11,&
     & .4928500E-12,.8265300E-10,.1049300E-09,.1061700E-09,.2947300E-11,&
     & .4566000E-12,.8964300E-10,.1186100E-09,.1264700E-09,.3323100E-11,&
     & .5364500E-12,.5055700E-10,.6224000E-10,.5485400E-10,.1693000E-11,&
     & .4852300E-12,.5646800E-10,.6560400E-10,.6482700E-10,.1870200E-11,&
     & .4429400E-12,.6144200E-10,.7366700E-10,.7268800E-10,.2129200E-11,&
     & .4074400E-12,.6718400E-10,.8449000E-10,.8521700E-10,.2383200E-11,&
     & .3772000E-12,.7265700E-10,.9569200E-10,.1014400E-09,.2685200E-11/

      data   absb(  1:175, 2)   /                                       &
     & .1832300E-07,.8626600E-05,.1372800E-04,.1595200E-04,.7892700E-06,&
     & .1696100E-07,.9620300E-05,.1527500E-04,.1786300E-04,.9365900E-06,&
     & .1578700E-07,.1067500E-04,.1692400E-04,.1967500E-04,.1105000E-05,&
     & .1476400E-07,.1177900E-04,.1847400E-04,.2132000E-04,.1286800E-05,&
     & .1386500E-07,.1284300E-04,.2002900E-04,.2308000E-04,.1456500E-05,&
     & .1489300E-07,.7188200E-05,.1144300E-04,.1327100E-04,.6583500E-06,&
     & .1379300E-07,.8027600E-05,.1273200E-04,.1489000E-04,.7813200E-06,&
     & .1284400E-07,.8918900E-05,.1410100E-04,.1633900E-04,.9223300E-06,&
     & .1201700E-07,.9830900E-05,.1538400E-04,.1772100E-04,.1067600E-05,&
     & .1128900E-07,.1070400E-04,.1667900E-04,.1919400E-04,.1209100E-05,&
     & .1209600E-07,.5993400E-05,.9544900E-05,.1105800E-04,.5496900E-06,&
     & .1120900E-07,.6693200E-05,.1062300E-04,.1240700E-04,.6519700E-06,&
     & .1044300E-07,.7444900E-05,.1174600E-04,.1357100E-04,.7694000E-06,&
     & .9774700E-08,.8194800E-05,.1281400E-04,.1473200E-04,.8849600E-06,&
     & .9186400E-08,.8918800E-05,.1388300E-04,.1594600E-04,.1002300E-05,&
     & .9820500E-08,.5002700E-05,.7964400E-05,.9224800E-05,.4587600E-06,&
     & .9106100E-08,.5584600E-05,.8859300E-05,.1033100E-04,.5433400E-06,&
     & .8488200E-08,.6218300E-05,.9775700E-05,.1127900E-04,.6405000E-06,&
     & .7948600E-08,.6827600E-05,.1066400E-04,.1224400E-04,.7334400E-06,&
     & .7473200E-08,.7426800E-05,.1154500E-04,.1322400E-04,.8295500E-06,&
     & .7975300E-08,.4171300E-05,.6615300E-05,.7689200E-05,.3824700E-06,&
     & .7399400E-08,.4656200E-05,.7375300E-05,.8592200E-05,.4518800E-06,&
     & .6900800E-08,.5182900E-05,.8128000E-05,.9363800E-05,.5321100E-06,&
     & .6465000E-08,.5676500E-05,.8856000E-05,.1016800E-04,.6064700E-06,&
     & .6080700E-08,.6175800E-05,.9589400E-05,.1094900E-04,.6854900E-06,&
     & .6477900E-08,.3472700E-05,.5487500E-05,.6400000E-05,.3185100E-06,&
     & .6013500E-08,.3875100E-05,.6130800E-05,.7137200E-05,.3755200E-06,&
     & .5611100E-08,.4313200E-05,.6747800E-05,.7769800E-05,.4414100E-06,&
     & .5259000E-08,.4715000E-05,.7344400E-05,.8437000E-05,.5007500E-06,&
     & .4948300E-08,.5134500E-05,.7949500E-05,.9063700E-05,.5660100E-06,&
     & .5262700E-08,.2890800E-05,.4558600E-05,.5326000E-05,.2651600E-06,&
     & .4888200E-08,.3226000E-05,.5094000E-05,.5920000E-05,.3119000E-06,&
     & .4563200E-08,.3584300E-05,.5599200E-05,.6439700E-05,.3660800E-06,&
     & .4278600E-08,.3916700E-05,.6092300E-05,.6997200E-05,.4136100E-06,&
     & .4027300E-08,.4266700E-05,.6590400E-05,.7502100E-05,.4674100E-06/

      data   absb(176:350, 2)   /                                       &
     & .4270400E-08,.2407400E-05,.3799000E-05,.4445800E-05,.2210300E-06,&
     & .3969000E-08,.2694200E-05,.4239300E-05,.4912000E-05,.2601400E-06,&
     & .3707100E-08,.2982700E-05,.4657400E-05,.5348700E-05,.3041600E-06,&
     & .3477600E-08,.3260100E-05,.5063500E-05,.5807600E-05,.3430400E-06,&
     & .3274700E-08,.3551800E-05,.5470000E-05,.6219700E-05,.3875200E-06,&
     & .3464900E-08,.2009600E-05,.3168900E-05,.3707000E-05,.1842400E-06,&
     & .3222400E-08,.2250300E-05,.3527100E-05,.4078400E-05,.2170500E-06,&
     & .3011400E-08,.2483400E-05,.3873700E-05,.4443600E-05,.2522900E-06,&
     & .2826300E-08,.2714900E-05,.4208300E-05,.4806100E-05,.2846400E-06,&
     & .2662600E-08,.2955600E-05,.4539400E-05,.5156500E-05,.3214800E-06,&
     & .2797600E-08,.1691300E-05,.2663200E-05,.3110800E-05,.1553700E-06,&
     & .2604300E-08,.1890800E-05,.2955300E-05,.3406700E-05,.1831100E-06,&
     & .2435900E-08,.2081100E-05,.3239000E-05,.3711000E-05,.2109400E-06,&
     & .2287800E-08,.2274600E-05,.3514400E-05,.4000900E-05,.2381600E-06,&
     & .2156700E-08,.2471400E-05,.3783900E-05,.4291000E-05,.2688600E-06,&
     & .2258300E-08,.1423400E-05,.2238800E-05,.2608800E-05,.1314300E-06,&
     & .2104300E-08,.1586400E-05,.2477600E-05,.2849300E-05,.1546000E-06,&
     & .1969900E-08,.1743600E-05,.2707200E-05,.3100900E-05,.1766900E-06,&
     & .1851500E-08,.1905700E-05,.2935500E-05,.3330800E-05,.1995300E-06,&
     & .1746500E-08,.2065300E-05,.3154200E-05,.3566300E-05,.2250100E-06,&
     & .1822100E-08,.1200100E-05,.1880800E-05,.2186800E-05,.1113800E-06,&
     & .1699500E-08,.1331500E-05,.2077900E-05,.2382500E-05,.1306800E-06,&
     & .1592300E-08,.1462500E-05,.2264700E-05,.2588400E-05,.1482000E-06,&
     & .1497800E-08,.1597200E-05,.2451200E-05,.2777100E-05,.1673400E-06,&
     & .1413800E-08,.1725400E-05,.2629600E-05,.2964900E-05,.1884800E-06,&
     & .1469300E-08,.1011100E-05,.1580900E-05,.1828600E-05,.9436600E-07,&
     & .1371800E-08,.1119900E-05,.1743000E-05,.1995200E-05,.1104500E-06,&
     & .1286400E-08,.1227300E-05,.1895700E-05,.2157200E-05,.1244500E-06,&
     & .1211000E-08,.1337300E-05,.2046200E-05,.2316200E-05,.1404200E-06,&
     & .1143800E-08,.1441000E-05,.2191600E-05,.2464800E-05,.1576700E-06,&
     & .1183400E-08,.8533100E-06,.1331600E-05,.1532900E-05,.8021200E-07,&
     & .1106100E-08,.9419200E-06,.1462600E-05,.1672900E-05,.9277500E-07,&
     & .1038200E-08,.1031700E-05,.1589000E-05,.1800600E-05,.1046200E-06,&
     & .9781000E-09,.1120500E-05,.1709900E-05,.1931400E-05,.1180000E-06,&
     & .9245700E-09,.1203400E-05,.1827600E-05,.2050400E-05,.1320900E-06/

      data   absb(351:525, 2)   /                                       &
     & .9531100E-09,.7189000E-06,.1121800E-05,.1286700E-05,.6808400E-07,&
     & .8917700E-09,.7925800E-06,.1227500E-05,.1401900E-05,.7799900E-07,&
     & .8378100E-09,.8660500E-06,.1330800E-05,.1504000E-05,.8802700E-07,&
     & .7899800E-09,.9381500E-06,.1428500E-05,.1609100E-05,.9905700E-07,&
     & .7472900E-09,.1004600E-05,.1523600E-05,.1705900E-05,.1107400E-06,&
     & .7677100E-09,.6062300E-06,.9442700E-06,.1079600E-05,.5784400E-07,&
     & .7190400E-09,.6665300E-06,.1029800E-05,.1169900E-05,.6572600E-07,&
     & .6761400E-09,.7268900E-06,.1113100E-05,.1256800E-05,.7400900E-07,&
     & .6380600E-09,.7848200E-06,.1192700E-05,.1340600E-05,.8320000E-07,&
     & .6040100E-09,.8380600E-06,.1269500E-05,.1418600E-05,.9230600E-07,&
     & .6181700E-09,.5105300E-06,.7945000E-06,.9068700E-06,.4907800E-07,&
     & .5795800E-09,.5607100E-06,.8640300E-06,.9769000E-06,.5525700E-07,&
     & .5455100E-09,.6100900E-06,.9307700E-06,.1049500E-05,.6228600E-07,&
     & .5152000E-09,.6563100E-06,.9956700E-06,.1115900E-05,.6995400E-07,&
     & .4880600E-09,.6983000E-06,.1057600E-05,.1179000E-05,.7722800E-07,&
     & .4978300E-09,.4301400E-06,.6673200E-06,.7605800E-06,.4130600E-07,&
     & .4672300E-09,.4714800E-06,.7243400E-06,.8168200E-06,.4653700E-07,&
     & .4401500E-09,.5114600E-06,.7780900E-06,.8753700E-06,.5245100E-07,&
     & .4160300E-09,.5487700E-06,.8307300E-06,.9288400E-06,.5879300E-07,&
     & .3944000E-09,.5815900E-06,.8805400E-06,.9803000E-06,.6463000E-07,&
     & .4008700E-09,.3625600E-06,.5606200E-06,.6354900E-06,.3477800E-07,&
     & .3766000E-09,.3962000E-06,.6067400E-06,.6831300E-06,.3921300E-07,&
     & .3551000E-09,.4285500E-06,.6503700E-06,.7299900E-06,.4417600E-07,&
     & .3359000E-09,.4585200E-06,.6930800E-06,.7733800E-06,.4942400E-07,&
     & .3186600E-09,.4845900E-06,.7329700E-06,.8148400E-06,.5413700E-07,&
     & .3228000E-09,.3054700E-06,.4708300E-06,.5314700E-06,.2935300E-07,&
     & .3035600E-09,.3327100E-06,.5077000E-06,.5711200E-06,.3307800E-07,&
     & .2864700E-09,.3587600E-06,.5433900E-06,.6085200E-06,.3721100E-07,&
     & .2712000E-09,.3821400E-06,.5778100E-06,.6435400E-06,.4129500E-07,&
     & .2574600E-09,.4037400E-06,.6099000E-06,.6771100E-06,.4536200E-07,&
     & .2599700E-09,.2572300E-06,.3950500E-06,.4446700E-06,.2475400E-07,&
     & .2447100E-09,.2792600E-06,.4246900E-06,.4770200E-06,.2790700E-07,&
     & .2311300E-09,.3002400E-06,.4538400E-06,.5067200E-06,.3134400E-07,&
     & .2189800E-09,.3186200E-06,.4814200E-06,.5351500E-06,.3459000E-07,&
     & .2080300E-09,.3362400E-06,.5072100E-06,.5624200E-06,.3795700E-07/

      data   absb(526:700, 2)   /                                       &
     & .2098700E-09,.2156100E-06,.3303500E-06,.3709300E-06,.2080100E-07,&
     & .1977100E-09,.2335600E-06,.3543300E-06,.3972300E-06,.2344300E-07,&
     & .1868800E-09,.2504200E-06,.3780400E-06,.4212700E-06,.2627100E-07,&
     & .1771600E-09,.2651400E-06,.4002900E-06,.4443300E-06,.2886700E-07,&
     & .1684000E-09,.2794900E-06,.4210200E-06,.4661200E-06,.3162000E-07,&
     & .1704600E-09,.1790900E-06,.2739700E-06,.3073000E-06,.1728300E-07,&
     & .1606600E-09,.1937800E-06,.2936700E-06,.3286900E-06,.1945600E-07,&
     & .1519100E-09,.2071600E-06,.3128700E-06,.3482000E-06,.2175900E-07,&
     & .1440700E-09,.2193800E-06,.3309000E-06,.3670300E-06,.2385400E-07,&
     & .1369900E-09,.2311500E-06,.3477900E-06,.3845900E-06,.2612800E-07,&
     & .1393800E-09,.1474200E-06,.2253500E-06,.2526100E-06,.1414500E-07,&
     & .1313700E-09,.1594900E-06,.2415500E-06,.2701100E-06,.1591700E-07,&
     & .1242300E-09,.1703700E-06,.2572100E-06,.2860600E-06,.1780900E-07,&
     & .1178200E-09,.1804400E-06,.2720000E-06,.3014900E-06,.1953000E-07,&
     & .1120400E-09,.1901100E-06,.2858200E-06,.3158100E-06,.2139100E-07,&
     & .1150300E-09,.1199100E-06,.1833900E-06,.2054000E-06,.1139800E-07,&
     & .1083700E-09,.1298500E-06,.1967000E-06,.2199100E-06,.1282900E-07,&
     & .1024400E-09,.1389500E-06,.2096400E-06,.2331000E-06,.1437000E-07,&
     & .9712200E-10,.1472500E-06,.2218300E-06,.2458200E-06,.1579000E-07,&
     & .9232500E-10,.1552600E-06,.2332700E-06,.2576700E-06,.1730800E-07,&
     & .9499000E-10,.9743400E-07,.1491200E-06,.1669900E-06,.9174500E-08,&
     & .8944900E-10,.1056300E-06,.1600700E-06,.1789000E-06,.1033000E-07,&
     & .8451600E-10,.1132200E-06,.1707100E-06,.1898200E-06,.1158500E-07,&
     & .8009600E-10,.1200900E-06,.1808400E-06,.2003000E-06,.1275900E-07,&
     & .7611300E-10,.1267200E-06,.1902800E-06,.2101200E-06,.1399200E-07,&
     & .7844400E-10,.7914700E-07,.1212200E-06,.1357300E-06,.7384600E-08,&
     & .7383200E-10,.8590100E-07,.1302200E-06,.1455200E-06,.8318000E-08,&
     & .6972900E-10,.9223000E-07,.1389900E-06,.1545900E-06,.9337400E-08,&
     & .6605600E-10,.9791900E-07,.1473700E-06,.1632500E-06,.1030700E-07,&
     & .6274900E-10,.1034100E-06,.1551800E-06,.1713300E-06,.1131200E-07,&
     & .6520700E-10,.6371500E-07,.9775500E-07,.1094500E-06,.5883100E-08,&
     & .6131900E-10,.6928900E-07,.1052400E-06,.1175700E-06,.6623700E-08,&
     & .5786600E-10,.7469400E-07,.1124500E-06,.1251500E-06,.7445500E-08,&
     & .5477900E-10,.7935900E-07,.1194100E-06,.1322900E-06,.8259500E-08,&
     & .5200300E-10,.8395700E-07,.1259300E-06,.1390900E-06,.9069800E-08/

      data   absb(701:875, 2)   /                                       &
     & .5426300E-10,.5117200E-07,.7871400E-07,.8818300E-07,.4674000E-08,&
     & .5097800E-10,.5583800E-07,.8493900E-07,.9482800E-07,.5265500E-08,&
     & .4806600E-10,.6032800E-07,.9085900E-07,.1011800E-06,.5927300E-08,&
     & .4546700E-10,.6426000E-07,.9666700E-07,.1071000E-06,.6627900E-08,&
     & .4313400E-10,.6808900E-07,.1020800E-06,.1127900E-06,.7256000E-08,&
     & .4517000E-10,.4107300E-07,.6333300E-07,.7106700E-07,.3718500E-08,&
     & .4239300E-10,.4496400E-07,.6850700E-07,.7645500E-07,.4186400E-08,&
     & .3993600E-10,.4866700E-07,.7338200E-07,.8177200E-07,.4715700E-08,&
     & .3774700E-10,.5199600E-07,.7822000E-07,.8668900E-07,.5283400E-08,&
     & .3578400E-10,.5517800E-07,.8273900E-07,.9143400E-07,.5804000E-08,&
     & .3775400E-10,.3277000E-07,.5066200E-07,.5697900E-07,.2940500E-08,&
     & .3538700E-10,.3599700E-07,.5499300E-07,.6135700E-07,.3306600E-08,&
     & .3329900E-10,.3907300E-07,.5903600E-07,.6579300E-07,.3726500E-08,&
     & .3144200E-10,.4190300E-07,.6301400E-07,.6992000E-07,.4184800E-08,&
     & .2978100E-10,.4454600E-07,.6682200E-07,.7384900E-07,.4620400E-08,&
     & .3162300E-10,.2605400E-07,.4040500E-07,.4569200E-07,.2291800E-08,&
     & .2959900E-10,.2871300E-07,.4401000E-07,.4912200E-07,.2605300E-08,&
     & .2781700E-10,.3129600E-07,.4739100E-07,.5281500E-07,.2935300E-08,&
     & .2623600E-10,.3367700E-07,.5065500E-07,.5626400E-07,.3302300E-08,&
     & .2482500E-10,.3589600E-07,.5386300E-07,.5953800E-07,.3673000E-08,&
     & .2650200E-10,.2070500E-07,.3215200E-07,.3644800E-07,.1784100E-08,&
     & .2476800E-10,.2286800E-07,.3518500E-07,.3932900E-07,.2051500E-08,&
     & .2324700E-10,.2502700E-07,.3799900E-07,.4233800E-07,.2311800E-08,&
     & .2190000E-10,.2706300E-07,.4069400E-07,.4525800E-07,.2604800E-08,&
     & .2070100E-10,.2890100E-07,.4336100E-07,.4796900E-07,.2917400E-08,&
     & .2225800E-10,.1642200E-07,.2548000E-07,.2899100E-07,.1383000E-08,&
     & .2076700E-10,.1815800E-07,.2803300E-07,.3144200E-07,.1614400E-08,&
     & .1946200E-10,.1995300E-07,.3040200E-07,.3385000E-07,.1816200E-08,&
     & .1831200E-10,.2165800E-07,.3263900E-07,.3630800E-07,.2047500E-08,&
     & .1728900E-10,.2321100E-07,.3483900E-07,.3858300E-07,.2299400E-08,&
     & .1876400E-10,.1295100E-07,.2007200E-07,.2298200E-07,.1064500E-08,&
     & .1747300E-10,.1433500E-07,.2221400E-07,.2508500E-07,.1247900E-08,&
     & .1634700E-10,.1581300E-07,.2420500E-07,.2697100E-07,.1420700E-08,&
     & .1535700E-10,.1725500E-07,.2607500E-07,.2901500E-07,.1600800E-08,&
     & .1448000E-10,.1855800E-07,.2788500E-07,.3092200E-07,.1801800E-08/

      data   absb(876:1050, 2)   /                                      &
     & .1583300E-10,.1018600E-07,.1578200E-07,.1817400E-07,.8171800E-09,&
     & .1471300E-10,.1131200E-07,.1755500E-07,.1989100E-07,.9609800E-09,&
     & .1374000E-10,.1250900E-07,.1924800E-07,.2148200E-07,.1111200E-08,&
     & .1288700E-10,.1371900E-07,.2081100E-07,.2314000E-07,.1251500E-08,&
     & .1213400E-10,.1484000E-07,.2231300E-07,.2476900E-07,.1410500E-08,&
     & .1337200E-10,.8005000E-08,.1240300E-07,.1428300E-07,.6311700E-09,&
     & .1239800E-10,.8939300E-08,.1384000E-07,.1575100E-07,.7390700E-09,&
     & .1155600E-10,.9882100E-08,.1526100E-07,.1710400E-07,.8664500E-09,&
     & .1082100E-10,.1088400E-07,.1658200E-07,.1844100E-07,.9782900E-09,&
     & .1017400E-10,.1184100E-07,.1782800E-07,.1980400E-07,.1103400E-08,&
     & .1128200E-10,.6272600E-08,.9747800E-08,.1122600E-07,.4839800E-09,&
     & .1043700E-10,.7054600E-08,.1093200E-07,.1251800E-07,.5705100E-09,&
     & .9710100E-11,.7819300E-08,.1211400E-07,.1367100E-07,.6699700E-09,&
     & .9077200E-11,.8637900E-08,.1321800E-07,.1471200E-07,.7662500E-09,&
     & .8521600E-11,.9440500E-08,.1425800E-07,.1584400E-07,.8648800E-09,&
     & .9517900E-11,.4925100E-08,.7635600E-08,.8776100E-08,.3721300E-09,&
     & .8785600E-11,.5568100E-08,.8622000E-08,.9915900E-08,.4407400E-09,&
     & .8157600E-11,.6193000E-08,.9598900E-08,.1086500E-07,.5184300E-09,&
     & .7613100E-11,.6851800E-08,.1053900E-07,.1174600E-07,.6021000E-09,&
     & .7136600E-11,.7520600E-08,.1140300E-07,.1265900E-07,.6784100E-09,&
     & .8036800E-11,.3860800E-08,.6001000E-08,.6844100E-08,.2884800E-09,&
     & .7400700E-11,.4390300E-08,.6797400E-08,.7812700E-08,.3424500E-09,&
     & .6857700E-11,.4908600E-08,.7595900E-08,.8627900E-08,.4010600E-09,&
     & .6388600E-11,.5429700E-08,.8380100E-08,.9377800E-08,.4705800E-09,&
     & .5979500E-11,.5982900E-08,.9110300E-08,.1011100E-07,.5321800E-09,&
     & .6793200E-11,.3010200E-08,.4726900E-08,.5339500E-08,.2243200E-09,&
     & .6239700E-11,.3445000E-08,.5345800E-08,.6144900E-08,.2628200E-09,&
     & .5769300E-11,.3876800E-08,.6000600E-08,.6861300E-08,.3102000E-09,&
     & .5364700E-11,.4300200E-08,.6653300E-08,.7499000E-08,.3644600E-09,&
     & .5012900E-11,.4752700E-08,.7263800E-08,.8071700E-08,.4175900E-09,&
     & .5732000E-11,.2358200E-08,.3683700E-08,.4131700E-08,.1747700E-09,&
     & .5251800E-11,.2716300E-08,.4204800E-08,.4820800E-08,.2032100E-09,&
     & .4845700E-11,.3067800E-08,.4747200E-08,.5447200E-08,.2408100E-09,&
     & .4497800E-11,.3414800E-08,.5285100E-08,.5973500E-08,.2834500E-09,&
     & .4196300E-11,.3780400E-08,.5800100E-08,.6457000E-08,.3292300E-09/

      data   absb(1051:1175, 2)   /                                     &
     & .4834700E-11,.1853400E-08,.2853900E-08,.3182800E-08,.1349200E-09,&
     & .4418600E-11,.2139200E-08,.3319000E-08,.3781700E-08,.1584800E-09,&
     & .4068300E-11,.2432000E-08,.3759200E-08,.4315600E-08,.1883500E-09,&
     & .3769300E-11,.2715000E-08,.4199900E-08,.4762500E-08,.2207600E-09,&
     & .3511100E-11,.3008200E-08,.4634500E-08,.5173600E-08,.2591600E-09,&
     & .4081300E-11,.1454300E-08,.2203800E-08,.2448800E-08,.1032800E-09,&
     & .3720100E-11,.1681300E-08,.2637100E-08,.2964000E-08,.1244600E-09,&
     & .3417500E-11,.1919100E-08,.2974700E-08,.3411500E-08,.1460400E-09,&
     & .3160400E-11,.2157500E-08,.3334800E-08,.3801800E-08,.1720200E-09,&
     & .2939100E-11,.2393200E-08,.3693000E-08,.4147200E-08,.2023500E-09,&
     & .3448600E-11,.1148000E-08,.1703200E-08,.1877100E-08,.7878700E-10,&
     & .3134500E-11,.1318900E-08,.2061500E-08,.2312100E-08,.9683200E-10,&
     & .2872800E-11,.1514400E-08,.2344000E-08,.2686300E-08,.1131700E-09,&
     & .2651300E-11,.1710200E-08,.2643600E-08,.3024600E-08,.1339300E-09,&
     & .2461500E-11,.1902200E-08,.2939400E-08,.3313500E-08,.1577200E-09,&
     & .2911900E-11,.8950000E-09,.1302000E-08,.1438300E-08,.6025300E-10,&
     & .2639200E-11,.1036900E-08,.1603600E-08,.1789100E-08,.7576500E-10,&
     & .2413200E-11,.1197500E-08,.1852300E-08,.2111400E-08,.8826000E-10,&
     & .2222700E-11,.1357000E-08,.2095900E-08,.2404500E-08,.1047800E-09,&
     & .2060100E-11,.1513900E-08,.2340600E-08,.2645600E-08,.1233100E-09,&
     & .2414300E-11,.7185100E-09,.1036200E-08,.1151600E-08,.4808600E-10,&
     & .2185700E-11,.8373200E-09,.1289200E-08,.1433500E-08,.6063000E-10,&
     & .1996500E-11,.9681500E-09,.1499000E-08,.1703900E-08,.7120800E-10,&
     & .1837400E-11,.1098200E-08,.1697300E-08,.1945500E-08,.8451400E-10,&
     & .1701700E-11,.1227700E-08,.1898100E-08,.2147200E-08,.9928900E-10/

      data   absb(  1:175, 3)   /                                       &
     & .6949700E-07,.5590000E-04,.7763400E-04,.8330000E-04,.1441900E-04,&
     & .6492300E-07,.6177900E-04,.8523800E-04,.9045800E-04,.1599600E-04,&
     & .6092000E-07,.6753200E-04,.9281100E-04,.9758800E-04,.1762200E-04,&
     & .5737900E-07,.7287600E-04,.1004000E-03,.1045500E-03,.1914400E-04,&
     & .5422000E-07,.7769900E-04,.1078400E-03,.1113900E-03,.2067100E-04,&
     & .5653800E-07,.4662700E-04,.6467800E-04,.6939900E-04,.1208600E-04,&
     & .5283900E-07,.5153600E-04,.7108000E-04,.7529200E-04,.1341400E-04,&
     & .4960000E-07,.5633400E-04,.7738400E-04,.8127700E-04,.1473200E-04,&
     & .4673100E-07,.6075100E-04,.8375300E-04,.8706700E-04,.1597300E-04,&
     & .4417100E-07,.6479200E-04,.8996300E-04,.9261700E-04,.1721400E-04,&
     & .4596300E-07,.3892000E-04,.5392100E-04,.5782400E-04,.1008900E-04,&
     & .4297600E-07,.4298700E-04,.5928600E-04,.6261500E-04,.1123800E-04,&
     & .4035800E-07,.4701400E-04,.6455700E-04,.6765800E-04,.1229300E-04,&
     & .3803700E-07,.5064100E-04,.6987900E-04,.7248700E-04,.1331700E-04,&
     & .3596400E-07,.5400300E-04,.7506400E-04,.7708300E-04,.1432100E-04,&
     & .3735400E-07,.3247900E-04,.4495300E-04,.4814600E-04,.8407900E-05,&
     & .3494400E-07,.3587300E-04,.4945400E-04,.5212000E-04,.9340500E-05,&
     & .3282900E-07,.3922000E-04,.5386100E-04,.5629300E-04,.1020900E-04,&
     & .3095200E-07,.4215500E-04,.5830900E-04,.6026800E-04,.1108000E-04,&
     & .2927400E-07,.4501400E-04,.6261000E-04,.6418400E-04,.1188700E-04,&
     & .3036400E-07,.2708600E-04,.3745100E-04,.3999500E-04,.7004900E-05,&
     & .2842000E-07,.2991900E-04,.4119400E-04,.4336700E-04,.7773600E-05,&
     & .2671000E-07,.3265000E-04,.4490200E-04,.4681700E-04,.8483800E-05,&
     & .2519200E-07,.3509800E-04,.4860500E-04,.5010300E-04,.9203800E-05,&
     & .2383300E-07,.3750700E-04,.5218800E-04,.5341300E-04,.9860700E-05,&
     & .2468700E-07,.2258100E-04,.3121100E-04,.3322000E-04,.5836100E-05,&
     & .2311600E-07,.2494000E-04,.3430800E-04,.3607800E-04,.6464000E-05,&
     & .2173400E-07,.2718100E-04,.3741400E-04,.3893300E-04,.7034400E-05,&
     & .2050600E-07,.2921600E-04,.4051700E-04,.4165300E-04,.7627600E-05,&
     & .1940600E-07,.3123700E-04,.4349000E-04,.4439700E-04,.8175300E-05,&
     & .2007400E-07,.1881800E-04,.2599900E-04,.2759700E-04,.4866200E-05,&
     & .1880500E-07,.2078400E-04,.2856200E-04,.2999300E-04,.5372300E-05,&
     & .1768800E-07,.2262600E-04,.3115100E-04,.3236300E-04,.5837700E-05,&
     & .1669300E-07,.2430000E-04,.3376000E-04,.3462900E-04,.6317700E-05,&
     & .1580200E-07,.2598500E-04,.3622600E-04,.3690300E-04,.6774200E-05/

      data   absb(176:350, 3)   /                                       &
     & .1630600E-07,.1571700E-04,.2168600E-04,.2295900E-04,.4068000E-05,&
     & .1528300E-07,.1735500E-04,.2382600E-04,.2496900E-04,.4470600E-05,&
     & .1438100E-07,.1885600E-04,.2599200E-04,.2691200E-04,.4855000E-05,&
     & .1357800E-07,.2026500E-04,.2816200E-04,.2881700E-04,.5235500E-05,&
     & .1285700E-07,.2163800E-04,.3021500E-04,.3070700E-04,.5622400E-05,&
     & .1324500E-07,.1312800E-04,.1809100E-04,.1910700E-04,.3388000E-05,&
     & .1242000E-07,.1449500E-04,.1988700E-04,.2079400E-04,.3714400E-05,&
     & .1169200E-07,.1570400E-04,.2170300E-04,.2239700E-04,.4041500E-05,&
     & .1104300E-07,.1690400E-04,.2348900E-04,.2398400E-04,.4345500E-05,&
     & .1046000E-07,.1803700E-04,.2519700E-04,.2555800E-04,.4666200E-05,&
     & .1071100E-07,.1104400E-04,.1519900E-04,.1600800E-04,.2840900E-05,&
     & .1005200E-07,.1216500E-04,.1669900E-04,.1741900E-04,.3104100E-05,&
     & .9469000E-08,.1315800E-04,.1822400E-04,.1873300E-04,.3376800E-05,&
     & .8948400E-08,.1416600E-04,.1969100E-04,.2005000E-04,.3623700E-05,&
     & .8480400E-08,.1508500E-04,.2110500E-04,.2135600E-04,.3887600E-05,&
     & .8660600E-08,.9300200E-05,.1277500E-04,.1342000E-04,.2381500E-05,&
     & .8134000E-08,.1021000E-04,.1403400E-04,.1455200E-04,.2595500E-05,&
     & .7667100E-08,.1103500E-04,.1530200E-04,.1565700E-04,.2815000E-05,&
     & .7249700E-08,.1186300E-04,.1651300E-04,.1677000E-04,.3019800E-05,&
     & .6873900E-08,.1261600E-04,.1766400E-04,.1784800E-04,.3238700E-05,&
     & .6999500E-08,.7839200E-05,.1075400E-04,.1125000E-04,.1998600E-05,&
     & .6579000E-08,.8567400E-05,.1180600E-04,.1218600E-04,.2173800E-05,&
     & .6205500E-08,.9266400E-05,.1285000E-04,.1310300E-04,.2346900E-05,&
     & .5871000E-08,.9937600E-05,.1385400E-04,.1402700E-04,.2519500E-05,&
     & .5569600E-08,.1052700E-04,.1479500E-04,.1491700E-04,.2698800E-05,&
     & .5653800E-08,.6611800E-05,.9061000E-05,.9442200E-05,.1675300E-05,&
     & .5318400E-08,.7198600E-05,.9938900E-05,.1020900E-04,.1822300E-05,&
     & .5019900E-08,.7786300E-05,.1080000E-04,.1098100E-04,.1959000E-05,&
     & .4752200E-08,.8341500E-05,.1162600E-04,.1173700E-04,.2102000E-05,&
     & .4510500E-08,.8801700E-05,.1237300E-04,.1246800E-04,.2249800E-05,&
     & .4562000E-08,.5574900E-05,.7646300E-05,.7928500E-05,.1403200E-05,&
     & .4295000E-08,.6059000E-05,.8382200E-05,.8565200E-05,.1525500E-05,&
     & .4056900E-08,.6551000E-05,.9088800E-05,.9211600E-05,.1637300E-05,&
     & .3842900E-08,.6991800E-05,.9760400E-05,.9832900E-05,.1756100E-05,&
     & .3649500E-08,.7356600E-05,.1036700E-04,.1042900E-04,.1876900E-05/

      data   absb(351:525, 3)   /                                       &
     & .3680900E-08,.4696700E-05,.6458000E-05,.6664900E-05,.1178600E-05,&
     & .3468300E-08,.5107200E-05,.7066100E-05,.7190200E-05,.1275900E-05,&
     & .3278400E-08,.5505300E-05,.7647000E-05,.7725000E-05,.1369100E-05,&
     & .3107400E-08,.5847400E-05,.8189800E-05,.8234000E-05,.1466600E-05,&
     & .2952600E-08,.6149300E-05,.8681500E-05,.8719900E-05,.1565000E-05,&
     & .2970100E-08,.3958200E-05,.5457300E-05,.5595900E-05,.9903700E-06,&
     & .2800800E-08,.4304800E-05,.5954600E-05,.6040600E-05,.1065900E-05,&
     & .2649300E-08,.4627700E-05,.6428600E-05,.6474500E-05,.1144200E-05,&
     & .2512700E-08,.4894000E-05,.6861200E-05,.6891800E-05,.1225000E-05,&
     & .2388800E-08,.5142800E-05,.7265400E-05,.7286500E-05,.1304600E-05,&
     & .2395800E-08,.3340900E-05,.4611500E-05,.4705400E-05,.8302100E-06,&
     & .2261100E-08,.3626700E-05,.5019200E-05,.5075400E-05,.8915700E-06,&
     & .2140200E-08,.3880500E-05,.5405300E-05,.5429100E-05,.9566300E-06,&
     & .2031100E-08,.4092900E-05,.5752900E-05,.5768500E-05,.1023200E-05,&
     & .1932000E-08,.4301400E-05,.6068400E-05,.6086800E-05,.1087400E-05,&
     & .1932800E-08,.2821700E-05,.3895500E-05,.3956200E-05,.6946400E-06,&
     & .1825500E-08,.3051500E-05,.4229300E-05,.4260500E-05,.7458800E-06,&
     & .1729100E-08,.3249600E-05,.4539000E-05,.4550100E-05,.7995700E-06,&
     & .1641900E-08,.3423500E-05,.4819100E-05,.4824900E-05,.8537300E-06,&
     & .1562600E-08,.3599200E-05,.5063200E-05,.5080500E-05,.9058200E-06,&
     & .1559000E-08,.2384100E-05,.3289000E-05,.3328200E-05,.5810800E-06,&
     & .1473600E-08,.2568300E-05,.3560500E-05,.3575600E-05,.6240100E-06,&
     & .1396700E-08,.2722300E-05,.3805500E-05,.3811000E-05,.6686200E-06,&
     & .1327100E-08,.2865200E-05,.4029500E-05,.4033100E-05,.7121600E-06,&
     & .1263700E-08,.3006900E-05,.4225700E-05,.4239300E-05,.7552700E-06,&
     & .1257500E-08,.2009900E-05,.2776400E-05,.2798800E-05,.4861400E-06,&
     & .1189500E-08,.2155100E-05,.2996100E-05,.2999600E-05,.5219900E-06,&
     & .1128200E-08,.2277100E-05,.3190100E-05,.3189700E-05,.5586700E-06,&
     & .1072600E-08,.2396900E-05,.3365700E-05,.3369600E-05,.5941000E-06,&
     & .1021800E-08,.2507600E-05,.3523200E-05,.3536600E-05,.6296500E-06,&
     & .1014400E-08,.1694000E-05,.2342100E-05,.2351900E-05,.4070100E-06,&
     & .9602400E-09,.1806600E-05,.2514600E-05,.2514800E-05,.4365700E-06,&
     & .9113400E-09,.1905300E-05,.2673800E-05,.2669000E-05,.4665600E-06,&
     & .8668900E-09,.2003800E-05,.2810400E-05,.2813900E-05,.4951600E-06,&
     & .8263000E-09,.2086000E-05,.2929400E-05,.2948500E-05,.5246200E-06/

      data   absb(526:700, 3)   /                                       &
     & .8200000E-09,.1423100E-05,.1966100E-05,.1970000E-05,.3399600E-06,&
     & .7767200E-09,.1510500E-05,.2104700E-05,.2101800E-05,.3643500E-06,&
     & .7375700E-09,.1590800E-05,.2230700E-05,.2227200E-05,.3885100E-06,&
     & .7019300E-09,.1670600E-05,.2342100E-05,.2344800E-05,.4121900E-06,&
     & .6693600E-09,.1733000E-05,.2434200E-05,.2453800E-05,.4360700E-06,&
     & .6665200E-09,.1184200E-05,.1638300E-05,.1638200E-05,.2818300E-06,&
     & .6315600E-09,.1254100E-05,.1749900E-05,.1745700E-05,.3019500E-06,&
     & .5999000E-09,.1320900E-05,.1850400E-05,.1848400E-05,.3217400E-06,&
     & .5710700E-09,.1385000E-05,.1941800E-05,.1943800E-05,.3412400E-06,&
     & .5447000E-09,.1434800E-05,.2013500E-05,.2033500E-05,.3605000E-06,&
     & .5450600E-09,.9787000E-06,.1354100E-05,.1352000E-05,.2319000E-06,&
     & .5165000E-09,.1035400E-05,.1445500E-05,.1440400E-05,.2484500E-06,&
     & .4906300E-09,.1091000E-05,.1527000E-05,.1524700E-05,.2646400E-06,&
     & .4670700E-09,.1142700E-05,.1601200E-05,.1603300E-05,.2807200E-06,&
     & .4455300E-09,.1183300E-05,.1660400E-05,.1677400E-05,.2964300E-06,&
     & .4494900E-09,.8010400E-06,.1106600E-05,.1104500E-05,.1888800E-06,&
     & .4257900E-09,.8481000E-06,.1182600E-05,.1177700E-05,.2024200E-06,&
     & .4043500E-09,.8940400E-06,.1250400E-05,.1247700E-05,.2157800E-06,&
     & .3848300E-09,.9373700E-06,.1312400E-05,.1312900E-05,.2289200E-06,&
     & .3669900E-09,.9711800E-06,.1361700E-05,.1374400E-05,.2419000E-06,&
     & .3708800E-09,.6546800E-06,.9034700E-06,.9014000E-06,.1537100E-06,&
     & .3512000E-09,.6942100E-06,.9667600E-06,.9622200E-06,.1648400E-06,&
     & .3334000E-09,.7321100E-06,.1023300E-05,.1020300E-05,.1757800E-06,&
     & .3172100E-09,.7683800E-06,.1075000E-05,.1074500E-05,.1865600E-06,&
     & .3024200E-09,.7967600E-06,.1116400E-05,.1125600E-05,.1973000E-06,&
     & .3060300E-09,.5351800E-06,.7375500E-06,.7356000E-06,.1250900E-06,&
     & .2896700E-09,.5682700E-06,.7901700E-06,.7860700E-06,.1342000E-06,&
     & .2749000E-09,.5994600E-06,.8372900E-06,.8341800E-06,.1432000E-06,&
     & .2614800E-09,.6296800E-06,.8804500E-06,.8791900E-06,.1520600E-06,&
     & .2492200E-09,.6535800E-06,.9151100E-06,.9218700E-06,.1609200E-06,&
     & .2540100E-09,.4349100E-06,.5970600E-06,.5958300E-06,.1010300E-06,&
     & .2402700E-09,.4624300E-06,.6412700E-06,.6378000E-06,.1085400E-06,&
     & .2278800E-09,.4878900E-06,.6811100E-06,.6778500E-06,.1159600E-06,&
     & .2166400E-09,.5137000E-06,.7177600E-06,.7155700E-06,.1231800E-06,&
     & .2063900E-09,.5341300E-06,.7473800E-06,.7511600E-06,.1305500E-06/

      data   absb(701:875, 3)   /                                       &
     & .2110300E-09,.3516200E-06,.4824500E-06,.4818100E-06,.8155100E-07,&
     & .1994700E-09,.3757700E-06,.5197500E-06,.5168300E-06,.8762400E-07,&
     & .1890600E-09,.3968100E-06,.5537200E-06,.5501600E-06,.9377500E-07,&
     & .1796300E-09,.4185600E-06,.5840500E-06,.5817700E-06,.9967700E-07,&
     & .1710500E-09,.4361000E-06,.6095700E-06,.6115000E-06,.1057800E-06,&
     & .1753600E-09,.2841200E-06,.3894800E-06,.3893300E-06,.6579100E-07,&
     & .1656300E-09,.3050400E-06,.4209700E-06,.4185400E-06,.7075200E-07,&
     & .1568800E-09,.3226500E-06,.4495700E-06,.4462400E-06,.7579000E-07,&
     & .1489700E-09,.3408000E-06,.4750200E-06,.4726800E-06,.8064800E-07,&
     & .1417800E-09,.3558400E-06,.4969600E-06,.4974700E-06,.8568500E-07,&
     & .1462500E-09,.2282800E-06,.3126900E-06,.3130100E-06,.5285800E-07,&
     & .1380000E-09,.2466000E-06,.3394500E-06,.3373600E-06,.5687200E-07,&
     & .1306000E-09,.2615000E-06,.3638200E-06,.3605200E-06,.6100500E-07,&
     & .1239200E-09,.2763700E-06,.3849600E-06,.3826200E-06,.6506400E-07,&
     & .1178600E-09,.2896000E-06,.4043200E-06,.4034600E-06,.6913400E-07,&
     & .1222000E-09,.1829900E-06,.2500300E-06,.2507500E-06,.4242600E-07,&
     & .1151800E-09,.1985800E-06,.2727100E-06,.2712400E-06,.4560500E-07,&
     & .1089000E-09,.2114900E-06,.2930600E-06,.2905900E-06,.4898900E-07,&
     & .1032400E-09,.2236100E-06,.3112800E-06,.3090900E-06,.5235700E-07,&
     & .9811900E-10,.2354000E-06,.3278200E-06,.3265700E-06,.5565000E-07,&
     & .1021500E-09,.1460400E-06,.1995700E-06,.2007900E-06,.3400900E-07,&
     & .9616800E-10,.1597600E-06,.2185900E-06,.2178200E-06,.3656500E-07,&
     & .9083200E-10,.1709200E-06,.2359400E-06,.2340300E-06,.3933600E-07,&
     & .8603900E-10,.1807600E-06,.2514800E-06,.2494000E-06,.4212000E-07,&
     & .8170500E-10,.1909100E-06,.2657900E-06,.2641700E-06,.4480600E-07,&
     & .8554800E-10,.1160600E-06,.1585800E-06,.1602500E-06,.2713800E-07,&
     & .8043000E-10,.1277700E-06,.1746500E-06,.1743900E-06,.2926000E-07,&
     & .7588200E-10,.1377100E-06,.1895600E-06,.1880100E-06,.3151900E-07,&
     & .7180700E-10,.1459700E-06,.2028200E-06,.2008500E-06,.3379000E-07,&
     & .6813100E-10,.1545400E-06,.2148500E-06,.2132100E-06,.3601600E-07,&
     & .7188000E-10,.9167700E-07,.1251800E-06,.1271700E-06,.2154800E-07,&
     & .6747400E-10,.1018200E-06,.1388500E-06,.1388700E-06,.2336600E-07,&
     & .6357300E-10,.1103700E-06,.1515500E-06,.1503700E-06,.2514000E-07,&
     & .6008900E-10,.1175700E-06,.1629200E-06,.1611500E-06,.2702300E-07,&
     & .5695500E-10,.1246000E-06,.1730300E-06,.1715000E-06,.2888600E-07/

      data   absb(876:1050, 3)   /                                      &
     & .6043800E-10,.7237800E-07,.9858600E-07,.1007500E-06,.1706800E-07,&
     & .5663900E-10,.8066200E-07,.1100300E-06,.1105200E-06,.1864300E-07,&
     & .5328800E-10,.8839100E-07,.1206900E-06,.1200700E-06,.2005000E-07,&
     & .5030600E-10,.9462400E-07,.1304500E-06,.1291600E-06,.2158700E-07,&
     & .4763200E-10,.1002600E-06,.1391400E-06,.1377800E-06,.2313600E-07,&
     & .5085600E-10,.5704600E-07,.7756500E-07,.7974100E-07,.1351000E-07,&
     & .4757400E-10,.6373000E-07,.8693900E-07,.8782600E-07,.1481100E-07,&
     & .4469200E-10,.7036700E-07,.9597400E-07,.9570700E-07,.1599200E-07,&
     & .4213600E-10,.7595900E-07,.1044100E-06,.1033600E-06,.1723500E-07,&
     & .3985100E-10,.8066200E-07,.1117900E-06,.1106100E-06,.1850100E-07,&
     & .4275200E-10,.4499400E-07,.6093300E-07,.6317000E-07,.1073300E-07,&
     & .3992000E-10,.5041700E-07,.6869500E-07,.6977600E-07,.1177800E-07,&
     & .3744500E-10,.5612900E-07,.7638100E-07,.7629200E-07,.1279600E-07,&
     & .3525800E-10,.6091600E-07,.8353300E-07,.8274600E-07,.1377000E-07,&
     & .3330900E-10,.6501000E-07,.8993300E-07,.8882800E-07,.1480900E-07,&
     & .3593500E-10,.3527800E-07,.4790100E-07,.5005600E-07,.8523900E-08,&
     & .3349400E-10,.3989700E-07,.5424500E-07,.5543900E-07,.9356200E-08,&
     & .3136800E-10,.4457700E-07,.6068700E-07,.6085400E-07,.1022900E-07,&
     & .2949700E-10,.4890600E-07,.6666800E-07,.6621700E-07,.1100900E-07,&
     & .2783400E-10,.5234800E-07,.7215400E-07,.7132200E-07,.1185600E-07,&
     & .3022700E-10,.2757200E-07,.3752400E-07,.3975400E-07,.6719800E-08,&
     & .2811800E-10,.3156000E-07,.4281000E-07,.4403100E-07,.7426700E-08,&
     & .2629000E-10,.3531900E-07,.4807900E-07,.4848600E-07,.8151100E-08,&
     & .2468700E-10,.3904100E-07,.5314100E-07,.5289800E-07,.8804400E-08,&
     & .2326700E-10,.4212300E-07,.5783500E-07,.5717700E-07,.9493400E-08,&
     & .2544800E-10,.2145600E-07,.2932400E-07,.3145000E-07,.5303500E-08,&
     & .2362300E-10,.2492200E-07,.3370600E-07,.3490300E-07,.5902500E-08,&
     & .2204800E-10,.2793100E-07,.3800900E-07,.3854800E-07,.6484200E-08,&
     & .2067300E-10,.3111700E-07,.4229300E-07,.4218000E-07,.7045400E-08,&
     & .1945900E-10,.3380000E-07,.4631600E-07,.4577600E-07,.7588600E-08,&
     & .2139000E-10,.1674300E-07,.2303100E-07,.2487700E-07,.4177700E-08,&
     & .1981500E-10,.1961900E-07,.2655600E-07,.2772200E-07,.4703100E-08,&
     & .1846300E-10,.2216700E-07,.3010100E-07,.3070800E-07,.5164600E-08,&
     & .1728600E-10,.2480400E-07,.3370400E-07,.3371600E-07,.5644800E-08,&
     & .1625100E-10,.2718300E-07,.3704900E-07,.3669600E-07,.6078800E-08/

      data   absb(1051:1175, 3)   /                                     &
     & .1797300E-10,.1303700E-07,.1811500E-07,.1972700E-07,.3291200E-08,&
     & .1661500E-10,.1540200E-07,.2094200E-07,.2204300E-07,.3730100E-08,&
     & .1545400E-10,.1759200E-07,.2384800E-07,.2448500E-07,.4112900E-08,&
     & .1444800E-10,.1973300E-07,.2681500E-07,.2695400E-07,.4514600E-08,&
     & .1356500E-10,.2178500E-07,.2963300E-07,.2941400E-07,.4876200E-08,&
     & .1511200E-10,.1012700E-07,.1426200E-07,.1563700E-07,.2598600E-08,&
     & .1393900E-10,.1208200E-07,.1645800E-07,.1756700E-07,.2951000E-08,&
     & .1294200E-10,.1398000E-07,.1887600E-07,.1949900E-07,.3280300E-08,&
     & .1208000E-10,.1567700E-07,.2129800E-07,.2151600E-07,.3607800E-08,&
     & .1132800E-10,.1742100E-07,.2367400E-07,.2355200E-07,.3914200E-08,&
     & .1271600E-10,.7842300E-08,.1121800E-07,.1236800E-07,.2057200E-08,&
     & .1170300E-10,.9446800E-08,.1294400E-07,.1393900E-07,.2344900E-08,&
     & .1084400E-10,.1105400E-07,.1493300E-07,.1550400E-07,.2616100E-08,&
     & .1010500E-10,.1244500E-07,.1689500E-07,.1716700E-07,.2878900E-08,&
     & .9462900E-11,.1392900E-07,.1888400E-07,.1883700E-07,.3139100E-08,&
     & .1069400E-10,.6108400E-08,.8827800E-08,.9780100E-08,.1619200E-08,&
     & .9818000E-11,.7385900E-08,.1020600E-07,.1106400E-07,.1848600E-08,&
     & .9079700E-11,.8711300E-08,.1180100E-07,.1234800E-07,.2087200E-08,&
     & .8447800E-11,.9900300E-08,.1340900E-07,.1370700E-07,.2296700E-08,&
     & .7899800E-11,.1110800E-07,.1506900E-07,.1508200E-07,.2518200E-08,&
     & .8851300E-11,.4923900E-08,.7138600E-08,.7917000E-08,.1308900E-08,&
     & .8118700E-11,.5954100E-08,.8253900E-08,.8975800E-08,.1495200E-08,&
     & .7502100E-11,.7045000E-08,.9556700E-08,.1002400E-07,.1691900E-08,&
     & .6975300E-11,.8039100E-08,.1087400E-07,.1114300E-07,.1864600E-08,&
     & .6519100E-11,.9026500E-08,.1224000E-07,.1226800E-07,.2047100E-08/

      data   absb(  1:175, 4)   /                                       &
     & .4011500E-06,.2054200E-03,.2900600E-03,.2998300E-03,.8640400E-04,&
     & .3850300E-06,.2208600E-03,.3083300E-03,.3251800E-03,.9261100E-04,&
     & .3704200E-06,.2341500E-03,.3267400E-03,.3481400E-03,.9887800E-04,&
     & .3569900E-06,.2460700E-03,.3433500E-03,.3685800E-03,.1055900E-03,&
     & .3443900E-06,.2574100E-03,.3594300E-03,.3871600E-03,.1126200E-03,&
     & .3271700E-06,.1708600E-03,.2412100E-03,.2500100E-03,.7212000E-04,&
     & .3141100E-06,.1835800E-03,.2559900E-03,.2708900E-03,.7735100E-04,&
     & .3022500E-06,.1945600E-03,.2714900E-03,.2900200E-03,.8267600E-04,&
     & .2913200E-06,.2046000E-03,.2854400E-03,.3067800E-03,.8833100E-04,&
     & .2810500E-06,.2141000E-03,.2991900E-03,.3227100E-03,.9445100E-04,&
     & .2667300E-06,.1422100E-03,.2004800E-03,.2084300E-03,.6021300E-04,&
     & .2561500E-06,.1526800E-03,.2126400E-03,.2253100E-03,.6453100E-04,&
     & .2465200E-06,.1617000E-03,.2256700E-03,.2415700E-03,.6910200E-04,&
     & .2376400E-06,.1702300E-03,.2375000E-03,.2554300E-03,.7389100E-04,&
     & .2292600E-06,.1781500E-03,.2491200E-03,.2689200E-03,.7916700E-04,&
     & .2174000E-06,.1182400E-03,.1664400E-03,.1738100E-03,.5022700E-04,&
     & .2088400E-06,.1269600E-03,.1768300E-03,.1878000E-03,.5390300E-04,&
     & .2010400E-06,.1343400E-03,.1876400E-03,.2009400E-03,.5777700E-04,&
     & .1938100E-06,.1417700E-03,.1975700E-03,.2125900E-03,.6182100E-04,&
     & .1869800E-06,.1482800E-03,.2074700E-03,.2241600E-03,.6632600E-04,&
     & .1772400E-06,.9828700E-04,.1381300E-03,.1449600E-03,.4185100E-04,&
     & .1702900E-06,.1054800E-03,.1470100E-03,.1563600E-03,.4494300E-04,&
     & .1639600E-06,.1116900E-03,.1560500E-03,.1672700E-03,.4817600E-04,&
     & .1580900E-06,.1179500E-03,.1643500E-03,.1768400E-03,.5171600E-04,&
     & .1525100E-06,.1233200E-03,.1726500E-03,.1868300E-03,.5548000E-04,&
     & .1445000E-06,.8170300E-04,.1146400E-03,.1207900E-03,.3484900E-04,&
     & .1388700E-06,.8758000E-04,.1222400E-03,.1302800E-03,.3746500E-04,&
     & .1337300E-06,.9286900E-04,.1296600E-03,.1390600E-03,.4022500E-04,&
     & .1289600E-06,.9803400E-04,.1366800E-03,.1471200E-03,.4322700E-04,&
     & .1243900E-06,.1025000E-03,.1436700E-03,.1555700E-03,.4638300E-04,&
     & .1178200E-06,.6792200E-04,.9512100E-04,.1005900E-03,.2901400E-04,&
     & .1132500E-06,.7275900E-04,.1016600E-03,.1084300E-03,.3123900E-04,&
     & .1090900E-06,.7721300E-04,.1076900E-03,.1156300E-03,.3359400E-04,&
     & .1052100E-06,.8150700E-04,.1136700E-03,.1224400E-03,.3608700E-04,&
     & .1014600E-06,.8522900E-04,.1195100E-03,.1295500E-03,.3876000E-04/

      data   absb(176:350, 4)   /                                       &
     & .9600500E-07,.5653900E-04,.7904800E-04,.8363800E-04,.2418200E-04,&
     & .9231100E-07,.6052200E-04,.8450600E-04,.9030800E-04,.2605200E-04,&
     & .8892700E-07,.6430500E-04,.8960800E-04,.9622900E-04,.2807100E-04,&
     & .8577500E-07,.6781100E-04,.9465900E-04,.1020300E-03,.3018500E-04,&
     & .8270900E-07,.7096700E-04,.9955100E-04,.1079600E-03,.3243800E-04,&
     & .7822500E-07,.4709400E-04,.6573600E-04,.6970600E-04,.2016800E-04,&
     & .7523500E-07,.5038300E-04,.7030900E-04,.7521000E-04,.2174800E-04,&
     & .7249100E-07,.5358100E-04,.7457900E-04,.8010600E-04,.2344200E-04,&
     & .6992900E-07,.5644800E-04,.7886400E-04,.8510800E-04,.2524800E-04,&
     & .6741900E-07,.5909000E-04,.8293200E-04,.8987000E-04,.2709800E-04,&
     & .6357200E-07,.3942600E-04,.5497900E-04,.5844300E-04,.1691300E-04,&
     & .6116600E-07,.4209700E-04,.5877900E-04,.6293300E-04,.1825900E-04,&
     & .5895100E-07,.4479400E-04,.6233000E-04,.6695200E-04,.1967000E-04,&
     & .5687200E-07,.4714700E-04,.6596100E-04,.7123000E-04,.2122100E-04,&
     & .5482200E-07,.4937700E-04,.6923400E-04,.7503800E-04,.2274300E-04,&
     & .5165600E-07,.3295000E-04,.4605200E-04,.4908400E-04,.1418100E-04,&
     & .4972100E-07,.3525000E-04,.4910600E-04,.5267500E-04,.1533000E-04,&
     & .4793000E-07,.3743800E-04,.5215200E-04,.5609700E-04,.1653200E-04,&
     & .4624200E-07,.3941800E-04,.5516000E-04,.5967400E-04,.1781100E-04,&
     & .4457000E-07,.4128000E-04,.5789900E-04,.6265400E-04,.1910000E-04,&
     & .4196200E-07,.2760800E-04,.3852700E-04,.4116700E-04,.1190800E-04,&
     & .4040500E-07,.2954700E-04,.4109900E-04,.4408100E-04,.1288000E-04,&
     & .3895900E-07,.3132100E-04,.4368700E-04,.4705100E-04,.1391200E-04,&
     & .3758300E-07,.3299400E-04,.4619000E-04,.4989000E-04,.1496100E-04,&
     & .3622700E-07,.3456000E-04,.4847300E-04,.5235800E-04,.1603900E-04,&
     & .3407500E-07,.2315300E-04,.3230500E-04,.3456600E-04,.1001400E-04,&
     & .3282200E-07,.2476900E-04,.3443500E-04,.3696800E-04,.1082200E-04,&
     & .3165500E-07,.2624200E-04,.3663500E-04,.3948400E-04,.1169900E-04,&
     & .3052800E-07,.2762400E-04,.3864800E-04,.4177000E-04,.1258800E-04,&
     & .2943400E-07,.2895100E-04,.4063200E-04,.4377400E-04,.1347900E-04,&
     & .2765200E-07,.1946000E-04,.2711400E-04,.2905200E-04,.8437200E-05,&
     & .2664500E-07,.2078500E-04,.2893300E-04,.3109100E-04,.9130500E-05,&
     & .2570300E-07,.2201800E-04,.3074900E-04,.3317300E-04,.9856000E-05,&
     & .2478000E-07,.2316600E-04,.3240500E-04,.3497100E-04,.1059300E-04,&
     & .2390000E-07,.2428600E-04,.3404800E-04,.3666300E-04,.1134600E-04/

      data   absb(351:525, 4)   /                                       &
     & .2243900E-07,.1637000E-04,.2276300E-04,.2439700E-04,.7108400E-05,&
     & .2162800E-07,.1746100E-04,.2431800E-04,.2614000E-04,.7696100E-05,&
     & .2086500E-07,.1849300E-04,.2582600E-04,.2782400E-04,.8308000E-05,&
     & .2011300E-07,.1945200E-04,.2721500E-04,.2928700E-04,.8920700E-05,&
     & .1940500E-07,.2037600E-04,.2854300E-04,.3071000E-04,.9547700E-05,&
     & .1820800E-07,.1377700E-04,.1913800E-04,.2052400E-04,.5995200E-05,&
     & .1755500E-07,.1467900E-04,.2044800E-04,.2199600E-04,.6490700E-05,&
     & .1693600E-07,.1552200E-04,.2166100E-04,.2335700E-04,.6996600E-05,&
     & .1632500E-07,.1633400E-04,.2284600E-04,.2453700E-04,.7512400E-05,&
     & .1575500E-07,.1708800E-04,.2393100E-04,.2572200E-04,.8030500E-05,&
     & .1477200E-07,.1159800E-04,.1611700E-04,.1729400E-04,.5060200E-05,&
     & .1424600E-07,.1234600E-04,.1719800E-04,.1852100E-04,.5479900E-05,&
     & .1373900E-07,.1304700E-04,.1820500E-04,.1956600E-04,.5899200E-05,&
     & .1324700E-07,.1372900E-04,.1917300E-04,.2057400E-04,.6325100E-05,&
     & .1278900E-07,.1434100E-04,.2006800E-04,.2153600E-04,.6749600E-05,&
     & .1198400E-07,.9767900E-05,.1356800E-04,.1456500E-04,.4272100E-05,&
     & .1156000E-07,.1038900E-04,.1446100E-04,.1556000E-04,.4618200E-05,&
     & .1114400E-07,.1097300E-04,.1530200E-04,.1641000E-04,.4971500E-05,&
     & .1075000E-07,.1153200E-04,.1609100E-04,.1724400E-04,.5321200E-05,&
     & .1038100E-07,.1203400E-04,.1682100E-04,.1802300E-04,.5671200E-05,&
     & .9720500E-08,.8229600E-05,.1144000E-04,.1228800E-04,.3607900E-05,&
     & .9377700E-08,.8743100E-05,.1215800E-04,.1306700E-04,.3897400E-05,&
     & .9039100E-08,.9230000E-05,.1286200E-04,.1376900E-04,.4188500E-05,&
     & .8721900E-08,.9685400E-05,.1350800E-04,.1445400E-04,.4475600E-05,&
     & .8425100E-08,.1010100E-04,.1409000E-04,.1506400E-04,.4764300E-05,&
     & .7884200E-08,.6937900E-05,.9637800E-05,.1035000E-04,.3046200E-05,&
     & .7605800E-08,.7360500E-05,.1023000E-04,.1096400E-04,.3287900E-05,&
     & .7331200E-08,.7768500E-05,.1080600E-04,.1155200E-04,.3525800E-05,&
     & .7076100E-08,.8139300E-05,.1132900E-04,.1210900E-04,.3764200E-05,&
     & .6837500E-08,.8472300E-05,.1180600E-04,.1258200E-04,.4002700E-05,&
     & .6394600E-08,.5848400E-05,.8113200E-05,.8713800E-05,.2572200E-05,&
     & .6166800E-08,.6198500E-05,.8615200E-05,.9204300E-05,.2771600E-05,&
     & .5946000E-08,.6532900E-05,.9076100E-05,.9691400E-05,.2967200E-05,&
     & .5741000E-08,.6834900E-05,.9499100E-05,.1012800E-04,.3164900E-05,&
     & .5549000E-08,.7109800E-05,.9897200E-05,.1049400E-04,.3361300E-05/

      data   absb(526:700, 4)   /                                       &
     & .5193100E-08,.4915500E-05,.6812200E-05,.7300300E-05,.2164800E-05,&
     & .5006600E-08,.5210800E-05,.7229900E-05,.7712800E-05,.2328000E-05,&
     & .4828900E-08,.5483800E-05,.7607600E-05,.8107700E-05,.2490600E-05,&
     & .4663600E-08,.5727100E-05,.7946000E-05,.8449400E-05,.2653400E-05,&
     & .4508900E-08,.5954300E-05,.8275100E-05,.8742200E-05,.2815900E-05,&
     & .4217700E-08,.4110400E-05,.5691400E-05,.6085200E-05,.1807800E-05,&
     & .4079500E-08,.4358000E-05,.6033400E-05,.6427400E-05,.1942100E-05,&
     & .3935300E-08,.4581400E-05,.6340700E-05,.6744300E-05,.2076700E-05,&
     & .3801200E-08,.4780300E-05,.6619600E-05,.7014400E-05,.2211400E-05,&
     & .3675600E-08,.4968200E-05,.6888400E-05,.7253900E-05,.2345800E-05,&
     & .3416200E-08,.3414100E-05,.4722400E-05,.5041200E-05,.1496500E-05,&
     & .3319600E-08,.3621500E-05,.5003700E-05,.5324100E-05,.1607800E-05,&
     & .3219500E-08,.3806400E-05,.5258300E-05,.5581700E-05,.1718800E-05,&
     & .3109900E-08,.3971500E-05,.5488100E-05,.5800600E-05,.1830700E-05,&
     & .3007200E-08,.4127000E-05,.5706500E-05,.5997400E-05,.1941900E-05,&
     & .2800800E-08,.2809400E-05,.3880100E-05,.4139300E-05,.1224800E-05,&
     & .2712100E-08,.2983600E-05,.4115500E-05,.4374800E-05,.1316800E-05,&
     & .2648200E-08,.3139700E-05,.4328200E-05,.4588500E-05,.1408700E-05,&
     & .2557600E-08,.3277900E-05,.4520000E-05,.4770600E-05,.1501300E-05,&
     & .2472700E-08,.3408200E-05,.4702300E-05,.4935400E-05,.1593500E-05,&
     & .2298400E-08,.2310600E-05,.3185600E-05,.3396100E-05,.1001600E-05,&
     & .2222000E-08,.2456300E-05,.3382600E-05,.3591500E-05,.1077600E-05,&
     & .2158800E-08,.2588200E-05,.3560000E-05,.3769600E-05,.1153800E-05,&
     & .2104000E-08,.2703300E-05,.3719900E-05,.3921300E-05,.1230400E-05,&
     & .2033900E-08,.2812200E-05,.3872800E-05,.4059700E-05,.1306700E-05,&
     & .1886700E-08,.1899000E-05,.2614700E-05,.2785500E-05,.8189500E-06,&
     & .1821900E-08,.2021100E-05,.2778900E-05,.2948200E-05,.8816800E-06,&
     & .1767100E-08,.2132400E-05,.2927200E-05,.3096200E-05,.9447500E-06,&
     & .1720100E-08,.2228700E-05,.3060700E-05,.3223100E-05,.1008000E-05,&
     & .1672900E-08,.2319700E-05,.3188500E-05,.3338900E-05,.1071300E-05,&
     & .1556700E-08,.1548600E-05,.2129900E-05,.2269700E-05,.6641800E-06,&
     & .1503000E-08,.1652500E-05,.2268600E-05,.2406300E-05,.7159100E-06,&
     & .1455900E-08,.1746800E-05,.2393800E-05,.2530400E-05,.7680400E-06,&
     & .1414700E-08,.1828400E-05,.2506200E-05,.2637700E-05,.8205300E-06,&
     & .1380800E-08,.1905100E-05,.2614200E-05,.2736200E-05,.8728200E-06/

      data   absb(701:875, 4)   /                                       &
     & .1285500E-08,.1262200E-05,.1732600E-05,.1847200E-05,.5377800E-06,&
     & .1241400E-08,.1348600E-05,.1849400E-05,.1961200E-05,.5806200E-06,&
     & .1200600E-08,.1429000E-05,.1955100E-05,.2066500E-05,.6236500E-06,&
     & .1165500E-08,.1498400E-05,.2050600E-05,.2157000E-05,.6670700E-06,&
     & .1136600E-08,.1563000E-05,.2141600E-05,.2240400E-05,.7103500E-06,&
     & .1061600E-08,.1027300E-05,.1408500E-05,.1503400E-05,.4351400E-06,&
     & .1025000E-08,.1099700E-05,.1507000E-05,.1597300E-05,.4706100E-06,&
     & .9903000E-09,.1167900E-05,.1595300E-05,.1686100E-05,.5061300E-06,&
     & .9606200E-09,.1227200E-05,.1676600E-05,.1764200E-05,.5421600E-06,&
     & .9344300E-09,.1281700E-05,.1752900E-05,.1833300E-05,.5778400E-06,&
     & .8788300E-09,.8319900E-06,.1140200E-05,.1216800E-05,.3502500E-06,&
     & .8482900E-09,.8925400E-06,.1222500E-05,.1295800E-05,.3796000E-06,&
     & .8190300E-09,.9507200E-06,.1296600E-05,.1370800E-05,.4089300E-06,&
     & .7936900E-09,.1002000E-05,.1366500E-05,.1437900E-05,.4386200E-06,&
     & .7710800E-09,.1047800E-05,.1430200E-05,.1496200E-05,.4681700E-06,&
     & .7283800E-09,.6708000E-06,.9196200E-06,.9810200E-06,.2810700E-06,&
     & .7027900E-09,.7227100E-06,.9885300E-06,.1049100E-05,.3054000E-06,&
     & .6785800E-09,.7720300E-06,.1052200E-05,.1112200E-05,.3295800E-06,&
     & .6565100E-09,.8162500E-06,.1111500E-05,.1169700E-05,.3540700E-06,&
     & .6372700E-09,.8549500E-06,.1165100E-05,.1219200E-05,.3785000E-06,&
     & .6037900E-09,.5404700E-06,.7412800E-06,.7910000E-06,.2253500E-06,&
     & .5822800E-09,.5842900E-06,.7985400E-06,.8484100E-06,.2454800E-06,&
     & .5622000E-09,.6260600E-06,.8528300E-06,.9015700E-06,.2654100E-06,&
     & .5432000E-09,.6642300E-06,.9029600E-06,.9501500E-06,.2855800E-06,&
     & .5268600E-09,.6971900E-06,.9483000E-06,.9925400E-06,.3058000E-06,&
     & .5010800E-09,.4336100E-06,.5955400E-06,.6343200E-06,.1802300E-06,&
     & .4829100E-09,.4712700E-06,.6434900E-06,.6851000E-06,.1967700E-06,&
     & .4661200E-09,.5062100E-06,.6894200E-06,.7292900E-06,.2132500E-06,&
     & .4500400E-09,.5390300E-06,.7316900E-06,.7709500E-06,.2298600E-06,&
     & .4360000E-09,.5674000E-06,.7708100E-06,.8071400E-06,.2465400E-06,&
     & .4166500E-09,.3461400E-06,.4756500E-06,.5059800E-06,.1434500E-06,&
     & .4011900E-09,.3777900E-06,.5164200E-06,.5501500E-06,.1569700E-06,&
     & .3870200E-09,.4075200E-06,.5552500E-06,.5875900E-06,.1706400E-06,&
     & .3737200E-09,.4356300E-06,.5907600E-06,.6231800E-06,.1843000E-06,&
     & .3613900E-09,.4604100E-06,.6247400E-06,.6551500E-06,.1980000E-06/

      data   absb(876:1050, 4)   /                                      &
     & .3465800E-09,.2749400E-06,.3792400E-06,.4034900E-06,.1140700E-06,&
     & .3334000E-09,.3024300E-06,.4134000E-06,.4406600E-06,.1250800E-06,&
     & .3214300E-09,.3275600E-06,.4463500E-06,.4730800E-06,.1363600E-06,&
     & .3103200E-09,.3514900E-06,.4766900E-06,.5030900E-06,.1476300E-06,&
     & .2996900E-09,.3732300E-06,.5054200E-06,.5306200E-06,.1589200E-06,&
     & .2884600E-09,.2177800E-06,.3016800E-06,.3215800E-06,.9060700E-07,&
     & .2771600E-09,.2413900E-06,.3307100E-06,.3521400E-06,.9957700E-07,&
     & .2670000E-09,.2630400E-06,.3580400E-06,.3804900E-06,.1088100E-06,&
     & .2576700E-09,.2831300E-06,.3839400E-06,.4056600E-06,.1181400E-06,&
     & .2487500E-09,.3018300E-06,.4083200E-06,.4290600E-06,.1274200E-06,&
     & .2399400E-09,.1727600E-06,.2403900E-06,.2564400E-06,.7212400E-07,&
     & .2302800E-09,.1927500E-06,.2642900E-06,.2811700E-06,.7938800E-07,&
     & .2216200E-09,.2110600E-06,.2875800E-06,.3060600E-06,.8692200E-07,&
     & .2137700E-09,.2280700E-06,.3094700E-06,.3271400E-06,.9462800E-07,&
     & .2064100E-09,.2440000E-06,.3296200E-06,.3471500E-06,.1022800E-06,&
     & .1996000E-09,.1370900E-06,.1910100E-06,.2040700E-06,.5756600E-07,&
     & .1913200E-09,.1535200E-06,.2112000E-06,.2245500E-06,.6330800E-07,&
     & .1839500E-09,.1692400E-06,.2306800E-06,.2456300E-06,.6942100E-07,&
     & .1773200E-09,.1835200E-06,.2492600E-06,.2638300E-06,.7577300E-07,&
     & .1711700E-09,.1971500E-06,.2663000E-06,.2806800E-06,.8209700E-07,&
     & .1661600E-09,.1085200E-06,.1519100E-06,.1613700E-06,.4597500E-07,&
     & .1590100E-09,.1218800E-06,.1685500E-06,.1794300E-06,.5045400E-07,&
     & .1527200E-09,.1353500E-06,.1850200E-06,.1970500E-06,.5542400E-07,&
     & .1471100E-09,.1476500E-06,.2004200E-06,.2125200E-06,.6061600E-07,&
     & .1419400E-09,.1590700E-06,.2148400E-06,.2267300E-06,.6582600E-07,&
     & .1383900E-09,.8570400E-07,.1202300E-06,.1274400E-06,.3652300E-07,&
     & .1322300E-09,.9660000E-07,.1343800E-06,.1431200E-06,.4015200E-07,&
     & .1268400E-09,.1080400E-06,.1478400E-06,.1573500E-06,.4420700E-07,&
     & .1220800E-09,.1185100E-06,.1609300E-06,.1710800E-06,.4843500E-07,&
     & .1177200E-09,.1281100E-06,.1731300E-06,.1829300E-06,.5273300E-07,&
     & .1151600E-09,.6780200E-07,.9540000E-07,.1006500E-06,.2904000E-07,&
     & .1098600E-09,.7692700E-07,.1071200E-06,.1142800E-06,.3204400E-07,&
     & .1052600E-09,.8629500E-07,.1183300E-06,.1258000E-06,.3533500E-07,&
     & .1012200E-09,.9519000E-07,.1293600E-06,.1376300E-06,.3876400E-07,&
     & .9754500E-10,.1032400E-06,.1396900E-06,.1476800E-06,.4231200E-07/

      data   absb(1051:1175, 4)   /                                     &
     & .9583200E-10,.5361000E-07,.7598100E-07,.7966700E-07,.2311700E-07,&
     & .9128100E-10,.6113000E-07,.8532900E-07,.9100500E-07,.2565700E-07,&
     & .8733500E-10,.6882500E-07,.9473600E-07,.1007500E-06,.2826400E-07,&
     & .8389700E-10,.7643500E-07,.1039600E-06,.1107000E-06,.3105700E-07,&
     & .8081000E-10,.8327800E-07,.1126900E-06,.1192800E-06,.3397200E-07,&
     & .7981000E-10,.4232200E-07,.6037400E-07,.6292600E-07,.1840400E-07,&
     & .7586400E-10,.4855900E-07,.6803900E-07,.7224100E-07,.2054800E-07,&
     & .7248500E-10,.5477300E-07,.7579900E-07,.8075700E-07,.2258100E-07,&
     & .6956000E-10,.6122400E-07,.8355400E-07,.8896700E-07,.2485900E-07,&
     & .6695100E-10,.6708400E-07,.9079700E-07,.9624600E-07,.2724500E-07,&
     & .6652400E-10,.3332000E-07,.4751600E-07,.4954100E-07,.1462000E-07,&
     & .6308100E-10,.3844300E-07,.5402700E-07,.5714200E-07,.1637000E-07,&
     & .6018700E-10,.4358700E-07,.6061100E-07,.6460800E-07,.1803600E-07,&
     & .5768900E-10,.4897000E-07,.6691500E-07,.7123300E-07,.1989500E-07,&
     & .5547800E-10,.5392500E-07,.7304500E-07,.7761800E-07,.2183100E-07,&
     & .5544700E-10,.2624000E-07,.3743500E-07,.3898900E-07,.1160600E-07,&
     & .5244200E-10,.3046000E-07,.4295500E-07,.4523300E-07,.1304700E-07,&
     & .4996200E-10,.3476100E-07,.4841800E-07,.5168000E-07,.1443100E-07,&
     & .4782600E-10,.3915200E-07,.5362600E-07,.5704700E-07,.1593400E-07,&
     & .4595400E-10,.4338000E-07,.5878600E-07,.6253800E-07,.1751100E-07,&
     & .4574000E-10,.2124800E-07,.3026100E-07,.3157700E-07,.9422100E-08,&
     & .4321000E-10,.2473000E-07,.3494800E-07,.3672200E-07,.1060800E-07,&
     & .4114100E-10,.2828800E-07,.3941000E-07,.4205200E-07,.1175400E-07,&
     & .3936100E-10,.3190400E-07,.4374000E-07,.4652700E-07,.1298600E-07,&
     & .3780500E-10,.3544200E-07,.4802000E-07,.5108300E-07,.1427800E-07/

      data   absb(  1:175, 5)   /                                       &
     & .7904500E-05,.5851200E-03,.8096700E-03,.8857800E-03,.3482400E-03,&
     & .9280700E-05,.6181800E-03,.8608300E-03,.9243700E-03,.3787000E-03,&
     & .1062000E-04,.6502500E-03,.9061700E-03,.9629000E-03,.4063900E-03,&
     & .1184700E-04,.6808600E-03,.9476700E-03,.9981500E-03,.4314900E-03,&
     & .1289700E-04,.7094100E-03,.9842600E-03,.1029600E-02,.4545900E-03,&
     & .6568300E-05,.4879800E-03,.6757000E-03,.7375500E-03,.2917200E-03,&
     & .7695700E-05,.5155500E-03,.7185600E-03,.7697800E-03,.3170900E-03,&
     & .8785200E-05,.5424300E-03,.7560400E-03,.8014200E-03,.3406400E-03,&
     & .9762200E-05,.5681700E-03,.7903800E-03,.8309100E-03,.3609500E-03,&
     & .1061400E-04,.5920800E-03,.8201800E-03,.8569100E-03,.3809500E-03,&
     & .5464300E-05,.4066000E-03,.5637100E-03,.6128600E-03,.2443500E-03,&
     & .6388300E-05,.4297300E-03,.5991400E-03,.6409300E-03,.2647500E-03,&
     & .7259300E-05,.4521800E-03,.6300400E-03,.6665900E-03,.2848600E-03,&
     & .8052600E-05,.4737300E-03,.6584700E-03,.6912800E-03,.3020000E-03,&
     & .8736400E-05,.4939300E-03,.6831800E-03,.7132100E-03,.3193000E-03,&
     & .4548200E-05,.3387200E-03,.4700300E-03,.5092000E-03,.2047300E-03,&
     & .5289300E-05,.3578500E-03,.4989800E-03,.5330600E-03,.2217100E-03,&
     & .5998900E-05,.3767300E-03,.5246700E-03,.5546100E-03,.2380300E-03,&
     & .6642600E-05,.3947000E-03,.5482700E-03,.5751500E-03,.2524400E-03,&
     & .7175900E-05,.4119300E-03,.5690900E-03,.5936100E-03,.2677300E-03,&
     & .3769000E-05,.2817700E-03,.3915300E-03,.4226500E-03,.1715500E-03,&
     & .4379000E-05,.2977800E-03,.4153500E-03,.4428800E-03,.1856300E-03,&
     & .4954500E-05,.3137100E-03,.4366400E-03,.4609000E-03,.1990800E-03,&
     & .5456300E-05,.3288600E-03,.4563700E-03,.4785400E-03,.2109700E-03,&
     & .5887900E-05,.3435900E-03,.4741600E-03,.4938700E-03,.2244000E-03,&
     & .3126000E-05,.2343200E-03,.3259600E-03,.3508200E-03,.1434800E-03,&
     & .3618500E-05,.2478600E-03,.3456100E-03,.3676500E-03,.1554600E-03,&
     & .4073200E-05,.2612000E-03,.3634300E-03,.3832500E-03,.1662300E-03,&
     & .4483900E-05,.2740300E-03,.3799700E-03,.3981100E-03,.1765900E-03,&
     & .4830400E-05,.2867400E-03,.3951300E-03,.4111400E-03,.1881300E-03,&
     & .2585300E-05,.1948300E-03,.2712800E-03,.2912600E-03,.1196900E-03,&
     & .2983400E-05,.2062300E-03,.2874900E-03,.3054100E-03,.1297800E-03,&
     & .3354600E-05,.2175000E-03,.3026700E-03,.3187100E-03,.1387400E-03,&
     & .3686300E-05,.2284200E-03,.3165600E-03,.3311500E-03,.1478900E-03,&
     & .3963400E-05,.2394200E-03,.3294400E-03,.3422000E-03,.1574200E-03/

      data   absb(176:350, 5)   /                                       &
     & .2148600E-05,.1622100E-03,.2260900E-03,.2423600E-03,.1001400E-03,&
     & .2473500E-05,.1717800E-03,.2394900E-03,.2539900E-03,.1085700E-03,&
     & .2774400E-05,.1813600E-03,.2522600E-03,.2653500E-03,.1160400E-03,&
     & .3041400E-05,.1907600E-03,.2639700E-03,.2756600E-03,.1239800E-03,&
     & .3261000E-05,.2001900E-03,.2748800E-03,.2851000E-03,.1319900E-03,&
     & .1787700E-05,.1350700E-03,.1884500E-03,.2016300E-03,.8387900E-04,&
     & .2051800E-05,.1431900E-03,.1996800E-03,.2113400E-03,.9085800E-04,&
     & .2295300E-05,.1513700E-03,.2103800E-03,.2209400E-03,.9709800E-04,&
     & .2509800E-05,.1594700E-03,.2201900E-03,.2294800E-03,.1040200E-03,&
     & .2683000E-05,.1674900E-03,.2295000E-03,.2376100E-03,.1106900E-03,&
     & .1502200E-05,.1129700E-03,.1577300E-03,.1681900E-03,.7080800E-04,&
     & .1715700E-05,.1199800E-03,.1671400E-03,.1764400E-03,.7634500E-04,&
     & .1911100E-05,.1269000E-03,.1761100E-03,.1845400E-03,.8180400E-04,&
     & .2081200E-05,.1338500E-03,.1842500E-03,.1916400E-03,.8759100E-04,&
     & .2214400E-05,.1406600E-03,.1923500E-03,.1986700E-03,.9322800E-04,&
     & .1262100E-05,.9468100E-04,.1321000E-03,.1403200E-03,.5963000E-04,&
     & .1434600E-05,.1006100E-03,.1400900E-03,.1475100E-03,.6414200E-04,&
     & .1590800E-05,.1065500E-03,.1474900E-03,.1541300E-03,.6898800E-04,&
     & .1725300E-05,.1124600E-03,.1543700E-03,.1601300E-03,.7389300E-04,&
     & .1827200E-05,.1182500E-03,.1613000E-03,.1662700E-03,.7864900E-04,&
     & .1061100E-05,.7941900E-04,.1107900E-03,.1173600E-03,.5024400E-04,&
     & .1199600E-05,.8448600E-04,.1174900E-03,.1234500E-03,.5405400E-04,&
     & .1324100E-05,.8956500E-04,.1236400E-03,.1288300E-03,.5825900E-04,&
     & .1430000E-05,.9459200E-04,.1294900E-03,.1340600E-03,.6230000E-04,&
     & .1507000E-05,.9954500E-04,.1353800E-03,.1392800E-03,.6648500E-04,&
     & .8924800E-06,.6675700E-04,.9304300E-04,.9826900E-04,.4239600E-04,&
     & .1003400E-05,.7108200E-04,.9865200E-04,.1033200E-03,.4571900E-04,&
     & .1102400E-05,.7540400E-04,.1037500E-03,.1078200E-03,.4922800E-04,&
     & .1183500E-05,.7969400E-04,.1088200E-03,.1123400E-03,.5264900E-04,&
     & .1240600E-05,.8391500E-04,.1137600E-03,.1167700E-03,.5626400E-04,&
     & .7514700E-06,.5624800E-04,.7829100E-04,.8244100E-04,.3578100E-04,&
     & .8402500E-06,.5992900E-04,.8291700E-04,.8656200E-04,.3871500E-04,&
     & .9172800E-06,.6361800E-04,.8723100E-04,.9044200E-04,.4168200E-04,&
     & .9794700E-06,.6728500E-04,.9160600E-04,.9431100E-04,.4461000E-04,&
     & .1022200E-05,.7087900E-04,.9575900E-04,.9795400E-04,.4767800E-04/

      data   absb(351:525, 5)   /                                       &
     & .6322800E-06,.4744800E-04,.6592600E-04,.6921100E-04,.3031200E-04,&
     & .7018700E-06,.5058300E-04,.6975200E-04,.7262200E-04,.3284400E-04,&
     & .7626300E-06,.5372300E-04,.7343100E-04,.7594300E-04,.3528300E-04,&
     & .8097500E-06,.5688400E-04,.7713900E-04,.7923500E-04,.3783700E-04,&
     & .8409900E-06,.5993700E-04,.8067000E-04,.8219800E-04,.4039800E-04,&
     & .5306200E-06,.4005400E-04,.5553600E-04,.5810500E-04,.2568300E-04,&
     & .5858100E-06,.4274800E-04,.5871900E-04,.6098800E-04,.2782900E-04,&
     & .6328300E-06,.4544500E-04,.6192700E-04,.6380800E-04,.2991400E-04,&
     & .6684100E-06,.4814100E-04,.6503300E-04,.6656700E-04,.3210100E-04,&
     & .6913200E-06,.5074400E-04,.6800200E-04,.6902400E-04,.3419600E-04,&
     & .4448200E-06,.3387600E-04,.4681300E-04,.4882000E-04,.2182200E-04,&
     & .4883000E-06,.3617300E-04,.4953400E-04,.5126700E-04,.2359800E-04,&
     & .5252000E-06,.3850700E-04,.5226400E-04,.5370000E-04,.2540700E-04,&
     & .5514200E-06,.4079000E-04,.5487900E-04,.5593400E-04,.2723300E-04,&
     & .5682100E-06,.4300500E-04,.5737400E-04,.5799500E-04,.2893900E-04,&
     & .3724000E-06,.2868800E-04,.3949800E-04,.4108200E-04,.1854200E-04,&
     & .4069700E-06,.3066200E-04,.4183200E-04,.4313900E-04,.2003700E-04,&
     & .4351800E-06,.3265700E-04,.4413700E-04,.4519800E-04,.2159200E-04,&
     & .4545900E-06,.3460000E-04,.4634200E-04,.4703000E-04,.2309900E-04,&
     & .4667400E-06,.3646800E-04,.4842700E-04,.4875200E-04,.2452400E-04,&
     & .3116700E-06,.2432800E-04,.3336500E-04,.3457700E-04,.1577500E-04,&
     & .3388000E-06,.2603300E-04,.3537600E-04,.3634300E-04,.1703700E-04,&
     & .3602200E-06,.2772800E-04,.3730600E-04,.3802500E-04,.1833400E-04,&
     & .3744300E-06,.2938100E-04,.3915300E-04,.3955100E-04,.1955900E-04,&
     & .3830400E-06,.3094000E-04,.4088800E-04,.4101200E-04,.2073500E-04,&
     & .2606500E-06,.2066600E-04,.2822600E-04,.2912800E-04,.1340300E-04,&
     & .2817900E-06,.2213000E-04,.2993400E-04,.3064200E-04,.1449500E-04,&
     & .2979100E-06,.2357200E-04,.3155400E-04,.3200500E-04,.1557600E-04,&
     & .3081300E-06,.2496300E-04,.3310100E-04,.3327800E-04,.1658200E-04,&
     & .3141200E-06,.2626100E-04,.3451000E-04,.3449700E-04,.1753300E-04,&
     & .2176800E-06,.1758100E-04,.2390300E-04,.2455800E-04,.1140500E-04,&
     & .2340900E-06,.1882700E-04,.2533300E-04,.2581200E-04,.1233100E-04,&
     & .2459600E-06,.2005300E-04,.2669600E-04,.2694200E-04,.1320000E-04,&
     & .2533800E-06,.2121100E-04,.2797800E-04,.2801200E-04,.1403700E-04,&
     & .2573500E-06,.2229200E-04,.2911300E-04,.2901900E-04,.1478900E-04/

      data   absb(526:700, 5)   /                                       &
     & .1811100E-06,.1493300E-04,.2022100E-04,.2069100E-04,.9685300E-05,&
     & .1937300E-06,.1599300E-04,.2141300E-04,.2170800E-04,.1045900E-04,&
     & .2026200E-06,.1702500E-04,.2255100E-04,.2265100E-04,.1117000E-04,&
     & .2080200E-06,.1799200E-04,.2360300E-04,.2354000E-04,.1184700E-04,&
     & .2106800E-06,.1889200E-04,.2452200E-04,.2437000E-04,.1244400E-04,&
     & .1494000E-06,.1260400E-04,.1700100E-04,.1733900E-04,.8163000E-05,&
     & .1593400E-06,.1350500E-04,.1800800E-04,.1818300E-04,.8789000E-05,&
     & .1662600E-06,.1437400E-04,.1896300E-04,.1897600E-04,.9383800E-05,&
     & .1704000E-06,.1518700E-04,.1982400E-04,.1972000E-04,.9922900E-05,&
     & .1722900E-06,.1593900E-04,.2058200E-04,.2039400E-04,.1041200E-04,&
     & .1222400E-06,.1056000E-04,.1420300E-04,.1444700E-04,.6819600E-05,&
     & .1303000E-06,.1132600E-04,.1505500E-04,.1515600E-04,.7332100E-05,&
     & .1359000E-06,.1206200E-04,.1585600E-04,.1582300E-04,.7823000E-05,&
     & .1392900E-06,.1275000E-04,.1657300E-04,.1644900E-04,.8263200E-05,&
     & .1408100E-06,.1338400E-04,.1720800E-04,.1700600E-04,.8665600E-05,&
     & .9897900E-07,.8750000E-05,.1175500E-04,.1194300E-04,.5626900E-05,&
     & .1058300E-06,.9403500E-05,.1248000E-04,.1254400E-04,.6052800E-05,&
     & .1106800E-06,.1003100E-04,.1316100E-04,.1311000E-04,.6460400E-05,&
     & .1136500E-06,.1061900E-04,.1376700E-04,.1364200E-04,.6825300E-05,&
     & .1151000E-06,.1116200E-04,.1430600E-04,.1410800E-04,.7161500E-05,&
     & .8005000E-07,.7243600E-05,.9722500E-05,.9865700E-05,.4637500E-05,&
     & .8588600E-07,.7800600E-05,.1033800E-04,.1037800E-04,.4991900E-05,&
     & .9010800E-07,.8335500E-05,.1091600E-04,.1085600E-04,.5330300E-05,&
     & .9269000E-07,.8838000E-05,.1142900E-04,.1130600E-04,.5634900E-05,&
     & .9404600E-07,.9301900E-05,.1188500E-04,.1169600E-04,.5915200E-05,&
     & .6472000E-07,.5995500E-05,.8038900E-05,.8148800E-05,.3819200E-05,&
     & .6966300E-07,.6469900E-05,.8561300E-05,.8581800E-05,.4115900E-05,&
     & .7330500E-07,.6924600E-05,.9050600E-05,.8986100E-05,.4396800E-05,&
     & .7558800E-07,.7353000E-05,.9485700E-05,.9363700E-05,.4650500E-05,&
     & .7682500E-07,.7749200E-05,.9870800E-05,.9689800E-05,.4884100E-05,&
     & .5187100E-07,.4919100E-05,.6599400E-05,.6689000E-05,.3118600E-05,&
     & .5612700E-07,.5322700E-05,.7043500E-05,.7057800E-05,.3367800E-05,&
     & .5939500E-07,.5712400E-05,.7460900E-05,.7401800E-05,.3602500E-05,&
     & .6150100E-07,.6079000E-05,.7833300E-05,.7721000E-05,.3816400E-05,&
     & .6271000E-07,.6419400E-05,.8161300E-05,.7998000E-05,.4012200E-05/

      data   absb(701:875, 5)   /                                       &
     & .4147100E-07,.4027600E-05,.5407600E-05,.5481500E-05,.2540200E-05,&
     & .4512700E-07,.4371100E-05,.5784700E-05,.5797100E-05,.2750700E-05,&
     & .4801800E-07,.4704000E-05,.6140700E-05,.6088300E-05,.2947900E-05,&
     & .4998300E-07,.5017700E-05,.6459500E-05,.6358700E-05,.3128600E-05,&
     & .5115300E-07,.5309800E-05,.6738900E-05,.6595000E-05,.3292800E-05,&
     & .3310800E-07,.3294500E-05,.4426100E-05,.4487100E-05,.2067400E-05,&
     & .3622800E-07,.3586100E-05,.4746200E-05,.4757100E-05,.2245200E-05,&
     & .3877600E-07,.3870100E-05,.5050500E-05,.5004600E-05,.2410600E-05,&
     & .4058000E-07,.4137800E-05,.5321900E-05,.5232300E-05,.2562200E-05,&
     & .4168900E-07,.4388100E-05,.5560700E-05,.5435600E-05,.2701500E-05,&
     & .2624300E-07,.2678000E-05,.3602100E-05,.3657800E-05,.1672200E-05,&
     & .2890300E-07,.2925400E-05,.3875400E-05,.3888600E-05,.1823500E-05,&
     & .3115800E-07,.3167400E-05,.4136300E-05,.4099800E-05,.1962800E-05,&
     & .3282800E-07,.3396600E-05,.4368700E-05,.4293500E-05,.2090600E-05,&
     & .3389600E-07,.3611200E-05,.4574500E-05,.4468000E-05,.2208800E-05,&
     & .2069200E-07,.2168800E-05,.2922600E-05,.2973800E-05,.1348600E-05,&
     & .2296100E-07,.2377600E-05,.3155600E-05,.3169900E-05,.1475900E-05,&
     & .2493400E-07,.2583400E-05,.3377900E-05,.3350700E-05,.1592600E-05,&
     & .2647200E-07,.2780000E-05,.3577900E-05,.3516200E-05,.1701700E-05,&
     & .2750400E-07,.2963700E-05,.3755800E-05,.3666600E-05,.1801700E-05,&
     & .1626900E-07,.1753700E-05,.2367200E-05,.2413300E-05,.1085600E-05,&
     & .1819400E-07,.1929100E-05,.2566300E-05,.2580900E-05,.1192700E-05,&
     & .1990300E-07,.2103900E-05,.2755200E-05,.2736000E-05,.1290900E-05,&
     & .2129300E-07,.2272100E-05,.2927000E-05,.2877500E-05,.1383900E-05,&
     & .2228300E-07,.2429200E-05,.3080200E-05,.3006700E-05,.1468600E-05,&
     & .1271200E-07,.1412800E-05,.1911300E-05,.1954800E-05,.8711400E-06,&
     & .1435100E-07,.1559600E-05,.2080200E-05,.2094900E-05,.9611800E-06,&
     & .1581500E-07,.1707600E-05,.2240400E-05,.2228900E-05,.1043600E-05,&
     & .1706200E-07,.1851100E-05,.2388900E-05,.2349200E-05,.1122500E-05,&
     & .1799300E-07,.1985500E-05,.2520600E-05,.2460900E-05,.1194800E-05,&
     & .9836400E-08,.1130600E-05,.1535300E-05,.1574600E-05,.6955400E-06,&
     & .1122600E-07,.1253400E-05,.1676500E-05,.1694400E-05,.7692500E-06,&
     & .1248300E-07,.1378400E-05,.1813000E-05,.1808700E-05,.8403600E-06,&
     & .1358700E-07,.1500700E-05,.1941800E-05,.1912200E-05,.9071200E-06,&
     & .1445700E-07,.1615800E-05,.2055700E-05,.2007300E-05,.9684700E-06/

      data   absb(876:1050, 5)   /                                      &
     & .7582400E-08,.9032900E-06,.1230700E-05,.1265300E-05,.5534600E-06,&
     & .8745300E-08,.1005000E-05,.1349000E-05,.1367600E-05,.6147000E-06,&
     & .9819400E-08,.1110100E-05,.1465000E-05,.1465100E-05,.6749200E-06,&
     & .1077900E-07,.1214100E-05,.1575600E-05,.1554400E-05,.7316300E-06,&
     & .1157900E-07,.1312500E-05,.1674300E-05,.1635900E-05,.7840500E-06,&
     & .5821900E-08,.7201500E-06,.9837900E-06,.1014300E-05,.4393300E-06,&
     & .6781700E-08,.8042300E-06,.1083200E-05,.1102300E-05,.4901800E-06,&
     & .7696600E-08,.8920500E-06,.1181900E-05,.1184600E-05,.5413100E-06,&
     & .8522400E-08,.9800500E-06,.1276000E-05,.1261700E-05,.5885800E-06,&
     & .9235000E-08,.1064400E-05,.1361500E-05,.1331900E-05,.6335800E-06,&
     & .4479000E-08,.5741800E-06,.7863500E-06,.8129600E-06,.3488100E-06,&
     & .5257900E-08,.6438800E-06,.8705600E-06,.8889600E-06,.3913500E-06,&
     & .6027900E-08,.7172200E-06,.9534800E-06,.9580900E-06,.4344400E-06,&
     & .6732400E-08,.7913400E-06,.1033100E-05,.1024300E-05,.4739100E-06,&
     & .7355100E-08,.8633000E-06,.1107300E-05,.1084200E-05,.5123400E-06,&
     & .3445000E-08,.4575300E-06,.6281400E-06,.6515800E-06,.2762500E-06,&
     & .4069000E-08,.5153000E-06,.6989200E-06,.7155500E-06,.3126000E-06,&
     & .4708700E-08,.5762400E-06,.7684000E-06,.7747700E-06,.3480400E-06,&
     & .5308600E-08,.6384300E-06,.8358000E-06,.8309200E-06,.3818100E-06,&
     & .5843900E-08,.6995600E-06,.8996200E-06,.8822500E-06,.4141200E-06,&
     & .2645400E-08,.3638700E-06,.5000700E-06,.5217900E-06,.2189400E-06,&
     & .3140300E-08,.4117300E-06,.5599700E-06,.5747900E-06,.2487000E-06,&
     & .3665500E-08,.4621100E-06,.6181900E-06,.6251100E-06,.2784000E-06,&
     & .4173000E-08,.5141500E-06,.6754600E-06,.6730700E-06,.3069500E-06,&
     & .4631100E-08,.5658200E-06,.7297300E-06,.7169700E-06,.3340700E-06,&
     & .2027500E-08,.2887300E-06,.3974900E-06,.4164700E-06,.1735500E-06,&
     & .2419000E-08,.3282100E-06,.4473000E-06,.4606300E-06,.1977400E-06,&
     & .2844000E-08,.3698100E-06,.4967700E-06,.5041100E-06,.2221800E-06,&
     & .3269000E-08,.4131500E-06,.5448800E-06,.5440800E-06,.2464700E-06,&
     & .3659200E-08,.4568000E-06,.5909000E-06,.5818900E-06,.2690200E-06,&
     & .1561100E-08,.2296200E-06,.3161300E-06,.3330000E-06,.1373000E-06,&
     & .1869600E-08,.2618700E-06,.3580200E-06,.3695400E-06,.1570100E-06,&
     & .2210000E-08,.2964100E-06,.3997000E-06,.4068300E-06,.1776400E-06,&
     & .2561900E-08,.3325500E-06,.4400900E-06,.4406300E-06,.1980800E-06,&
     & .2892800E-08,.3690800E-06,.4789200E-06,.4727000E-06,.2169600E-06/

      data   absb(1051:1175, 5)   /                                     &
     & .1204100E-08,.1825900E-06,.2510800E-06,.2660800E-06,.1084200E-06,&
     & .1446600E-08,.2091800E-06,.2863300E-06,.2967700E-06,.1246800E-06,&
     & .1717600E-08,.2376500E-06,.3213900E-06,.3279900E-06,.1421300E-06,&
     & .2005800E-08,.2676600E-06,.3552300E-06,.3566400E-06,.1590700E-06,&
     & .2285200E-08,.2981800E-06,.3880600E-06,.3838100E-06,.1751200E-06,&
     & .9293400E-09,.1449600E-06,.1988700E-06,.2121500E-06,.8518500E-07,&
     & .1118000E-08,.1666800E-06,.2284100E-06,.2379900E-06,.9917200E-07,&
     & .1333300E-08,.1902400E-06,.2579000E-06,.2638500E-06,.1133100E-06,&
     & .1566800E-08,.2151200E-06,.2862600E-06,.2882000E-06,.1275800E-06,&
     & .1800000E-08,.2405500E-06,.3140500E-06,.3113400E-06,.1410600E-06,&
     & .7174700E-09,.1149100E-06,.1576000E-06,.1690700E-06,.6676200E-07,&
     & .8631800E-09,.1326600E-06,.1819800E-06,.1905100E-06,.7884400E-07,&
     & .1033200E-08,.1519900E-06,.2064600E-06,.2118500E-06,.9031700E-07,&
     & .1220500E-08,.1725500E-06,.2304900E-06,.2328000E-06,.1021100E-06,&
     & .1413700E-08,.1937400E-06,.2537700E-06,.2520900E-06,.1134700E-06,&
     & .5553400E-09,.9116100E-07,.1249500E-06,.1348100E-06,.5252400E-07,&
     & .6679600E-09,.1056400E-06,.1449000E-06,.1524600E-06,.6249100E-07,&
     & .8020800E-09,.1214200E-06,.1654000E-06,.1701300E-06,.7192300E-07,&
     & .9515500E-09,.1384600E-06,.1856200E-06,.1880300E-06,.8176000E-07,&
     & .1110000E-08,.1560800E-06,.2051300E-06,.2042500E-06,.9140500E-07,&
     & .4448300E-09,.7420100E-07,.1017500E-06,.1098600E-06,.4262600E-07,&
     & .5347800E-09,.8625700E-07,.1181800E-06,.1245300E-06,.5081600E-07,&
     & .6426500E-09,.9939000E-07,.1353500E-06,.1392800E-06,.5865800E-07,&
     & .7636900E-09,.1136000E-06,.1522400E-06,.1541700E-06,.6684500E-07,&
     & .8931600E-09,.1282600E-06,.1685000E-06,.1677500E-06,.7484100E-07/

      data   absb(  1:175, 6)   /                                       &
     & .1545200E-03,.1747100E-02,.2287000E-02,.2275700E-02,.1186800E-02,&
     & .1735800E-03,.1833700E-02,.2375200E-02,.2354900E-02,.1249200E-02,&
     & .1904300E-03,.1907000E-02,.2447900E-02,.2425500E-02,.1311400E-02,&
     & .2045900E-03,.1970200E-02,.2510900E-02,.2490500E-02,.1371700E-02,&
     & .2162100E-03,.2026600E-02,.2565300E-02,.2551200E-02,.1432000E-02,&
     & .1279300E-03,.1466200E-02,.1914100E-02,.1903200E-02,.9940400E-03,&
     & .1433500E-03,.1536700E-02,.1985200E-02,.1969100E-02,.1046200E-02,&
     & .1569800E-03,.1597400E-02,.2045400E-02,.2028500E-02,.1098600E-02,&
     & .1683700E-03,.1650500E-02,.2098200E-02,.2082800E-02,.1150900E-02,&
     & .1776500E-03,.1699000E-02,.2145200E-02,.2135700E-02,.1202000E-02,&
     & .1060000E-03,.1228000E-02,.1598700E-02,.1589900E-02,.8316000E-03,&
     & .1184600E-03,.1285800E-02,.1657000E-02,.1643800E-02,.8764100E-03,&
     & .1294700E-03,.1336200E-02,.1707300E-02,.1693700E-02,.9203700E-03,&
     & .1386000E-03,.1381800E-02,.1751900E-02,.1740100E-02,.9651400E-03,&
     & .1460500E-03,.1423100E-02,.1792200E-02,.1784900E-02,.1008800E-02,&
     & .8784400E-04,.1026800E-02,.1333700E-02,.1325800E-02,.6947100E-03,&
     & .9792100E-04,.1074700E-02,.1381600E-02,.1370400E-02,.7330900E-03,&
     & .1067800E-03,.1117300E-02,.1424000E-02,.1412200E-02,.7712600E-03,&
     & .1141000E-03,.1156100E-02,.1461900E-02,.1452500E-02,.8098300E-03,&
     & .1200800E-03,.1191300E-02,.1495700E-02,.1490400E-02,.8468500E-03,&
     & .7273600E-04,.8577000E-03,.1111000E-02,.1104300E-02,.5798800E-03,&
     & .8087000E-04,.8973700E-03,.1150800E-02,.1141200E-02,.6131000E-03,&
     & .8798400E-04,.9333400E-03,.1186500E-02,.1176900E-02,.6456300E-03,&
     & .9390000E-04,.9663700E-03,.1218800E-02,.1211500E-02,.6794600E-03,&
     & .9866900E-04,.9968200E-03,.1247700E-02,.1244400E-02,.7112900E-03,&
     & .6019600E-04,.7158500E-03,.9248400E-03,.9188500E-03,.4842800E-03,&
     & .6675900E-04,.7490200E-03,.9580600E-03,.9500700E-03,.5125400E-03,&
     & .7248600E-04,.7793600E-03,.9882800E-03,.9803600E-03,.5411800E-03,&
     & .7721000E-04,.8075700E-03,.1015700E-02,.1010100E-02,.5699500E-03,&
     & .8104600E-04,.8334400E-03,.1040500E-02,.1039000E-02,.5976200E-03,&
     & .4979700E-04,.5971400E-03,.7695100E-03,.7640800E-03,.4048700E-03,&
     & .5509800E-04,.6250500E-03,.7972800E-03,.7907100E-03,.4290400E-03,&
     & .5969000E-04,.6507100E-03,.8229400E-03,.8167100E-03,.4537500E-03,&
     & .6345700E-04,.6749000E-03,.8463500E-03,.8424500E-03,.4783300E-03,&
     & .6653800E-04,.6968300E-03,.8680500E-03,.8681200E-03,.5023800E-03/

      data   absb(176:350, 6)   /                                       &
     & .4127700E-04,.4985400E-03,.6404300E-03,.6358200E-03,.3388500E-03,&
     & .4555700E-04,.5219700E-03,.6640800E-03,.6585900E-03,.3596000E-03,&
     & .4922600E-04,.5438000E-03,.6858200E-03,.6811100E-03,.3810800E-03,&
     & .5223800E-04,.5644200E-03,.7059800E-03,.7036400E-03,.4019900E-03,&
     & .5469400E-04,.5832000E-03,.7250400E-03,.7264500E-03,.4228200E-03,&
     & .3421800E-04,.4162400E-03,.5331200E-03,.5291200E-03,.2837600E-03,&
     & .3767000E-04,.4359700E-03,.5531600E-03,.5489300E-03,.3016100E-03,&
     & .4060200E-04,.4546700E-03,.5717800E-03,.5686100E-03,.3202300E-03,&
     & .4300900E-04,.4722000E-03,.5893100E-03,.5885000E-03,.3380500E-03,&
     & .4496800E-04,.4885000E-03,.6062400E-03,.6086700E-03,.3563300E-03,&
     & .2855400E-04,.3487200E-03,.4451900E-03,.4420300E-03,.2386500E-03,&
     & .3131200E-04,.3654300E-03,.4621500E-03,.4589600E-03,.2542900E-03,&
     & .3362300E-04,.3813600E-03,.4780800E-03,.4762500E-03,.2702000E-03,&
     & .3551800E-04,.3963800E-03,.4935200E-03,.4939300E-03,.2856700E-03,&
     & .3706000E-04,.4104600E-03,.5083300E-03,.5115200E-03,.3016900E-03,&
     & .2382500E-04,.2924100E-03,.3720200E-03,.3695700E-03,.2012000E-03,&
     & .2601900E-04,.3066400E-03,.3865000E-03,.3843300E-03,.2148400E-03,&
     & .2784000E-04,.3202700E-03,.4003800E-03,.3995700E-03,.2282900E-03,&
     & .2932900E-04,.3331700E-03,.4139600E-03,.4149800E-03,.2416900E-03,&
     & .3054200E-04,.3454700E-03,.4268100E-03,.4304800E-03,.2556800E-03,&
     & .1987900E-04,.2455200E-03,.3113500E-03,.3093600E-03,.1698600E-03,&
     & .2161700E-04,.2576700E-03,.3237600E-03,.3222800E-03,.1817300E-03,&
     & .2305000E-04,.2693700E-03,.3359000E-03,.3357900E-03,.1932000E-03,&
     & .2421700E-04,.2805400E-03,.3477200E-03,.3492200E-03,.2050100E-03,&
     & .2516400E-04,.2912100E-03,.3588300E-03,.3628600E-03,.2170800E-03,&
     & .1658900E-04,.2064500E-03,.2608700E-03,.2594400E-03,.1436900E-03,&
     & .1796100E-04,.2168800E-03,.2716200E-03,.2708300E-03,.1538600E-03,&
     & .1908300E-04,.2269700E-03,.2822900E-03,.2825800E-03,.1638300E-03,&
     & .1999800E-04,.2366800E-03,.2924500E-03,.2943800E-03,.1741800E-03,&
     & .2073700E-04,.2458900E-03,.3021600E-03,.3064700E-03,.1846500E-03,&
     & .1385800E-04,.1740200E-03,.2190600E-03,.2180400E-03,.1220500E-03,&
     & .1493000E-04,.1830000E-03,.2284300E-03,.2281300E-03,.1306500E-03,&
     & .1580700E-04,.1917400E-03,.2377000E-03,.2383000E-03,.1393300E-03,&
     & .1652200E-04,.2001000E-03,.2464000E-03,.2487600E-03,.1483700E-03,&
     & .1709000E-04,.2079800E-03,.2550000E-03,.2594300E-03,.1574800E-03/

      data   absb(351:525, 6)   /                                       &
     & .1156500E-04,.1468900E-03,.1842300E-03,.1835800E-03,.1037200E-03,&
     & .1240400E-04,.1546600E-03,.1924200E-03,.1924200E-03,.1110400E-03,&
     & .1308800E-04,.1622500E-03,.2003500E-03,.2013600E-03,.1187700E-03,&
     & .1364400E-04,.1693800E-03,.2079300E-03,.2105400E-03,.1265800E-03,&
     & .1407900E-04,.1761400E-03,.2155300E-03,.2200400E-03,.1345500E-03,&
     & .9642900E-05,.1242200E-03,.1551800E-03,.1548600E-03,.8824000E-04,&
     & .1029700E-04,.1309300E-03,.1622900E-03,.1625500E-03,.9466400E-04,&
     & .1082900E-04,.1374400E-03,.1691100E-03,.1704300E-03,.1013700E-03,&
     & .1126300E-04,.1435500E-03,.1757700E-03,.1786100E-03,.1081800E-03,&
     & .1159300E-04,.1493600E-03,.1824500E-03,.1869800E-03,.1151700E-03,&
     & .8035700E-05,.1052500E-03,.1309800E-03,.1308700E-03,.7519900E-04,&
     & .8545700E-05,.1110600E-03,.1370900E-03,.1375600E-03,.8087700E-04,&
     & .8958500E-05,.1165900E-03,.1429800E-03,.1445800E-03,.8670300E-04,&
     & .9293600E-05,.1218500E-03,.1488700E-03,.1518400E-03,.9267900E-04,&
     & .9541300E-05,.1268500E-03,.1547600E-03,.1592600E-03,.9877000E-04,&
     & .6689100E-05,.8932600E-04,.1107000E-03,.1107500E-03,.6425400E-04,&
     & .7085500E-05,.9428000E-04,.1159500E-03,.1166800E-03,.6921500E-04,&
     & .7406800E-05,.9902900E-04,.1211200E-03,.1229000E-03,.7427800E-04,&
     & .7663400E-05,.1035600E-03,.1263100E-03,.1293300E-03,.7948900E-04,&
     & .7849900E-05,.1078500E-03,.1315200E-03,.1358800E-03,.8476200E-04,&
     & .5564700E-05,.7594300E-04,.9369500E-04,.9390600E-04,.5500200E-04,&
     & .5871900E-05,.8016700E-04,.9822900E-04,.9920400E-04,.5932900E-04,&
     & .6121200E-05,.8424900E-04,.1028100E-03,.1047100E-03,.6377800E-04,&
     & .6316200E-05,.8812200E-04,.1073800E-03,.1104100E-03,.6831800E-04,&
     & .6455300E-05,.9183500E-04,.1120400E-03,.1161400E-03,.7290700E-04,&
     & .4624600E-05,.6463300E-04,.7939800E-04,.7983100E-04,.4719000E-04,&
     & .4862800E-05,.6825800E-04,.8338300E-04,.8452100E-04,.5095300E-04,&
     & .5055700E-05,.7175800E-04,.8741800E-04,.8941700E-04,.5482300E-04,&
     & .5203500E-05,.7509900E-04,.9148100E-04,.9442300E-04,.5878500E-04,&
     & .5306800E-05,.7831700E-04,.9564600E-04,.9945200E-04,.6276600E-04,&
     & .3840300E-05,.5507700E-04,.6741000E-04,.6802400E-04,.4053800E-04,&
     & .4024400E-05,.5819900E-04,.7092400E-04,.7218000E-04,.4383900E-04,&
     & .4173000E-05,.6120600E-04,.7447500E-04,.7650200E-04,.4722200E-04,&
     & .4283500E-05,.6409100E-04,.7811400E-04,.8088300E-04,.5065200E-04,&
     & .4360400E-05,.6689800E-04,.8179800E-04,.8529300E-04,.5410200E-04/

      data   absb(526:700, 6)   /                                       &
     & .3180100E-05,.4688800E-04,.5723300E-04,.5794500E-04,.3478100E-04,&
     & .3324400E-05,.4958600E-04,.6031900E-04,.6162500E-04,.3764600E-04,&
     & .3438900E-05,.5218300E-04,.6346500E-04,.6542200E-04,.4060600E-04,&
     & .3522800E-05,.5469600E-04,.6669700E-04,.6926600E-04,.4357100E-04,&
     & .3580400E-05,.5716200E-04,.6996400E-04,.7312000E-04,.4656200E-04,&
     & .2620000E-05,.3974200E-04,.4841000E-04,.4914100E-04,.2963900E-04,&
     & .2735100E-05,.4207500E-04,.5112200E-04,.5238200E-04,.3214600E-04,&
     & .2825300E-05,.4432500E-04,.5391600E-04,.5569800E-04,.3470100E-04,&
     & .2890700E-05,.4653000E-04,.5676900E-04,.5906800E-04,.3728000E-04,&
     & .2935400E-05,.4870700E-04,.5964900E-04,.6246100E-04,.3986400E-04,&
     & .2146900E-05,.3349800E-04,.4074400E-04,.4142700E-04,.2504500E-04,&
     & .2240700E-05,.3551300E-04,.4312400E-04,.4426200E-04,.2722700E-04,&
     & .2314000E-05,.3748100E-04,.4558800E-04,.4715700E-04,.2943300E-04,&
     & .2367300E-05,.3941900E-04,.4810900E-04,.5010700E-04,.3167200E-04,&
     & .2403500E-05,.4134500E-04,.5064000E-04,.5308900E-04,.3391100E-04,&
     & .1746400E-05,.2798400E-04,.3400700E-04,.3458400E-04,.2089900E-04,&
     & .1825200E-05,.2973700E-04,.3608800E-04,.3705100E-04,.2278500E-04,&
     & .1887700E-05,.3146000E-04,.3824700E-04,.3957300E-04,.2468700E-04,&
     & .1933300E-05,.3316500E-04,.4046000E-04,.4214600E-04,.2662600E-04,&
     & .1964700E-05,.3486900E-04,.4269100E-04,.4476500E-04,.2856100E-04,&
     & .1419800E-05,.2336400E-04,.2836700E-04,.2885400E-04,.1742200E-04,&
     & .1486200E-05,.2489200E-04,.3018600E-04,.3099400E-04,.1904700E-04,&
     & .1539400E-05,.2639900E-04,.3207600E-04,.3319200E-04,.2068800E-04,&
     & .1578600E-05,.2789800E-04,.3402000E-04,.3544100E-04,.2236000E-04,&
     & .1605800E-05,.2940100E-04,.3598300E-04,.3773500E-04,.2403300E-04,&
     & .1154100E-05,.1950900E-04,.2366300E-04,.2407200E-04,.1452100E-04,&
     & .1210000E-05,.2083700E-04,.2525200E-04,.2592900E-04,.1591500E-04,&
     & .1255100E-05,.2215500E-04,.2690200E-04,.2784200E-04,.1733100E-04,&
     & .1288800E-05,.2347000E-04,.2860300E-04,.2980600E-04,.1877300E-04,&
     & .1312300E-05,.2479400E-04,.3033100E-04,.3181500E-04,.2021600E-04,&
     & .9324400E-06,.1617100E-04,.1959900E-04,.1991800E-04,.1197700E-04,&
     & .9805900E-06,.1732400E-04,.2097600E-04,.2151900E-04,.1316900E-04,&
     & .1019600E-05,.1847500E-04,.2241100E-04,.2317800E-04,.1438700E-04,&
     & .1049600E-05,.1962600E-04,.2389700E-04,.2488700E-04,.1562500E-04,&
     & .1070600E-05,.2079200E-04,.2541100E-04,.2664100E-04,.1687100E-04/

      data   absb(701:875, 6)   /                                       &
     & .7520600E-06,.1338100E-04,.1620400E-04,.1644800E-04,.9858200E-05,&
     & .7936800E-06,.1438000E-04,.1739500E-04,.1782400E-04,.1087300E-04,&
     & .8274200E-06,.1538200E-04,.1864000E-04,.1925900E-04,.1191600E-04,&
     & .8539000E-06,.1638800E-04,.1993500E-04,.2074500E-04,.1297900E-04,&
     & .8728100E-06,.1741200E-04,.2126100E-04,.2227100E-04,.1405300E-04,&
     & .6059100E-06,.1106100E-04,.1338500E-04,.1356800E-04,.8104500E-05,&
     & .6418200E-06,.1192600E-04,.1441200E-04,.1475100E-04,.8966100E-05,&
     & .6710100E-06,.1279700E-04,.1548900E-04,.1598700E-04,.9857800E-05,&
     & .6942700E-06,.1367500E-04,.1661600E-04,.1727400E-04,.1077000E-04,&
     & .7112200E-06,.1457300E-04,.1777400E-04,.1860100E-04,.1169000E-04,&
     & .4856500E-06,.9092900E-05,.1099800E-04,.1112700E-04,.6613600E-05,&
     & .5170300E-06,.9840100E-05,.1187800E-04,.1213600E-04,.7343100E-05,&
     & .5424600E-06,.1059400E-04,.1280700E-04,.1319700E-04,.8101900E-05,&
     & .5630900E-06,.1135900E-04,.1378200E-04,.1430600E-04,.8881200E-05,&
     & .5785400E-06,.1214400E-04,.1479100E-04,.1545600E-04,.9668500E-05,&
     & .3877700E-06,.7447800E-05,.9006200E-05,.9092100E-05,.5370900E-05,&
     & .4152200E-06,.8091900E-05,.9757000E-05,.9947800E-05,.5988900E-05,&
     & .4376100E-06,.8743300E-05,.1055500E-04,.1085600E-04,.6633200E-05,&
     & .4558400E-06,.9406300E-05,.1139500E-04,.1180800E-04,.7295500E-05,&
     & .4699000E-06,.1009100E-04,.1226900E-04,.1279900E-04,.7969600E-05,&
     & .3088800E-06,.6089800E-05,.7363200E-05,.7418100E-05,.4353500E-05,&
     & .3329100E-06,.6644000E-05,.8002300E-05,.8141600E-05,.4874100E-05,&
     & .3525500E-06,.7205200E-05,.8685500E-05,.8915400E-05,.5420700E-05,&
     & .3685400E-06,.7778700E-05,.9406800E-05,.9730800E-05,.5983300E-05,&
     & .3812500E-06,.8374500E-05,.1016300E-04,.1058300E-04,.6557800E-05,&
     & .2449400E-06,.4959800E-05,.5998700E-05,.6028900E-05,.3512300E-05,&
     & .2659500E-06,.5435500E-05,.6541100E-05,.6638300E-05,.3949400E-05,&
     & .2832800E-06,.5917300E-05,.7123300E-05,.7294300E-05,.4411300E-05,&
     & .2973500E-06,.6412300E-05,.7740600E-05,.7990400E-05,.4888300E-05,&
     & .3087600E-06,.6928300E-05,.8391300E-05,.8721100E-05,.5376600E-05,&
     & .1928700E-06,.4013400E-05,.4858600E-05,.4871800E-05,.2811400E-05,&
     & .2112300E-06,.4420800E-05,.5317800E-05,.5379400E-05,.3178100E-05,&
     & .2265800E-06,.4833200E-05,.5810400E-05,.5933500E-05,.3564600E-05,&
     & .2391200E-06,.5258200E-05,.6335400E-05,.6523000E-05,.3967600E-05,&
     & .2493100E-06,.5702500E-05,.6892000E-05,.7145800E-05,.4382600E-05/

      data   absb(876:1050, 6)   /                                      &
     & .1513200E-06,.3239500E-05,.3926900E-05,.3928900E-05,.2245800E-05,&
     & .1671600E-06,.3586600E-05,.4314400E-05,.4351100E-05,.2551200E-05,&
     & .1807600E-06,.3939100E-05,.4729400E-05,.4816100E-05,.2874600E-05,&
     & .1919400E-06,.4303000E-05,.5174800E-05,.5314200E-05,.3213500E-05,&
     & .2009900E-06,.4684200E-05,.5648800E-05,.5843000E-05,.3564400E-05,&
     & .1182800E-06,.2607800E-05,.3167200E-05,.3161700E-05,.1789400E-05,&
     & .1318500E-06,.2902900E-05,.3492900E-05,.3512500E-05,.2042300E-05,&
     & .1437800E-06,.3203000E-05,.3840900E-05,.3900000E-05,.2312100E-05,&
     & .1536900E-06,.3513900E-05,.4217900E-05,.4319800E-05,.2597100E-05,&
     & .1617600E-06,.3839100E-05,.4619800E-05,.4766700E-05,.2892900E-05,&
     & .9236200E-07,.2099700E-05,.2555600E-05,.2547100E-05,.1426700E-05,&
     & .1039000E-06,.2349600E-05,.2828600E-05,.2836800E-05,.1635600E-05,&
     & .1142100E-06,.2604900E-05,.3121300E-05,.3160600E-05,.1860900E-05,&
     & .1229700E-06,.2869400E-05,.3439200E-05,.3512500E-05,.2099600E-05,&
     & .1301200E-06,.3147100E-05,.3779400E-05,.3890000E-05,.2348600E-05,&
     & .7202900E-07,.1687900E-05,.2060600E-05,.2050600E-05,.1136500E-05,&
     & .8171700E-07,.1899400E-05,.2288600E-05,.2290200E-05,.1308100E-05,&
     & .9054700E-07,.2116200E-05,.2534700E-05,.2558800E-05,.1496400E-05,&
     & .9823100E-07,.2340700E-05,.2801700E-05,.2853700E-05,.1695800E-05,&
     & .1045400E-06,.2577000E-05,.3089100E-05,.3171900E-05,.1905300E-05,&
     & .5599400E-07,.1353500E-05,.1658300E-05,.1648300E-05,.9021500E-06,&
     & .6406500E-07,.1531800E-05,.1848300E-05,.1845200E-05,.1044700E-05,&
     & .7160100E-07,.1715600E-05,.2054100E-05,.2067900E-05,.1200500E-05,&
     & .7826500E-07,.1905500E-05,.2278600E-05,.2315100E-05,.1367300E-05,&
     & .8382700E-07,.2106200E-05,.2520100E-05,.2581200E-05,.1542600E-05,&
     & .4337600E-07,.1082300E-05,.1331600E-05,.1322900E-05,.7138700E-06,&
     & .5007800E-07,.1232000E-05,.1489800E-05,.1484300E-05,.8319600E-06,&
     & .5644000E-07,.1387300E-05,.1661200E-05,.1667800E-05,.9608500E-06,&
     & .6218800E-07,.1548100E-05,.1848900E-05,.1873600E-05,.1099500E-05,&
     & .6707200E-07,.1717900E-05,.2052000E-05,.2096400E-05,.1246500E-05,&
     & .3366200E-07,.8668500E-06,.1071200E-05,.1063800E-05,.5667500E-06,&
     & .3918800E-07,.9923700E-06,.1202400E-05,.1196500E-05,.6640700E-06,&
     & .4451700E-07,.1123400E-05,.1345300E-05,.1347700E-05,.7701500E-06,&
     & .4941000E-07,.1259100E-05,.1502200E-05,.1518600E-05,.8856800E-06,&
     & .5366200E-07,.1402800E-05,.1673200E-05,.1705400E-05,.1008800E-05/

      data   absb(1051:1175, 6)   /                                     &
     & .2610700E-07,.6940900E-06,.8614800E-06,.8553500E-06,.4499300E-06,&
     & .3064800E-07,.7987400E-06,.9705000E-06,.9643200E-06,.5298700E-06,&
     & .3507000E-07,.9091900E-06,.1089300E-05,.1088900E-05,.6172700E-06,&
     & .3922300E-07,.1023700E-05,.1220600E-05,.1230800E-05,.7134500E-06,&
     & .4289700E-07,.1145000E-05,.1364100E-05,.1387800E-05,.8161400E-06,&
     & .2018300E-07,.5546100E-06,.6918700E-06,.6868600E-06,.3563400E-06,&
     & .2390600E-07,.6414300E-06,.7819300E-06,.7761500E-06,.4213300E-06,&
     & .2757300E-07,.7342200E-06,.8805900E-06,.8784800E-06,.4938700E-06,&
     & .3106600E-07,.8308300E-06,.9900100E-06,.9960400E-06,.5736200E-06,&
     & .3421300E-07,.9330200E-06,.1110300E-05,.1127100E-05,.6590600E-06,&
     & .1555300E-07,.4422100E-06,.5547500E-06,.5505100E-06,.2815800E-06,&
     & .1859700E-07,.5138700E-06,.6288300E-06,.6238100E-06,.3343200E-06,&
     & .2162700E-07,.5916500E-06,.7107900E-06,.7076800E-06,.3942300E-06,&
     & .2454800E-07,.6728600E-06,.8015900E-06,.8045100E-06,.4600900E-06,&
     & .2722300E-07,.7587000E-06,.9019600E-06,.9137200E-06,.5312600E-06,&
     & .1198900E-07,.3527100E-06,.4450400E-06,.4416400E-06,.2224700E-06,&
     & .1447300E-07,.4116700E-06,.5059300E-06,.5017900E-06,.2655700E-06,&
     & .1695900E-07,.4768100E-06,.5738900E-06,.5705500E-06,.3148800E-06,&
     & .1938600E-07,.5450300E-06,.6493300E-06,.6504300E-06,.3692700E-06,&
     & .2165500E-07,.6170500E-06,.7328500E-06,.7409200E-06,.4282300E-06,&
     & .9574500E-08,.2890900E-06,.3650400E-06,.3622100E-06,.1814300E-06,&
     & .1160200E-07,.3386900E-06,.4162500E-06,.4126700E-06,.2175400E-06,&
     & .1363900E-07,.3936300E-06,.4734500E-06,.4705300E-06,.2587300E-06,&
     & .1563500E-07,.4513200E-06,.5372200E-06,.5377900E-06,.3043600E-06,&
     & .1751500E-07,.5125500E-06,.6080400E-06,.6144500E-06,.3539500E-06/

      data   absb(  1:175, 7)   /                                       &
     & .1381800E-02,.5309500E-02,.6174400E-02,.6140900E-02,.3820100E-02,&
     & .1492700E-02,.5426200E-02,.6316900E-02,.6326800E-02,.3978700E-02,&
     & .1588200E-02,.5524500E-02,.6457500E-02,.6504200E-02,.4128500E-02,&
     & .1668300E-02,.5602800E-02,.6585900E-02,.6676400E-02,.4276600E-02,&
     & .1736400E-02,.5664200E-02,.6699600E-02,.6840000E-02,.4419600E-02,&
     & .1143900E-02,.4464200E-02,.5205200E-02,.5195400E-02,.3218500E-02,&
     & .1232200E-02,.4561200E-02,.5329500E-02,.5355400E-02,.3354400E-02,&
     & .1308200E-02,.4642500E-02,.5447400E-02,.5511100E-02,.3486900E-02,&
     & .1372300E-02,.4707800E-02,.5556800E-02,.5660500E-02,.3617800E-02,&
     & .1427300E-02,.4761400E-02,.5653600E-02,.5800400E-02,.3744500E-02,&
     & .9460000E-03,.3745200E-02,.4378600E-02,.4383900E-02,.2706400E-02,&
     & .1016600E-02,.3827200E-02,.4484500E-02,.4523800E-02,.2825900E-02,&
     & .1077200E-02,.3895100E-02,.4585200E-02,.4656900E-02,.2942400E-02,&
     & .1128700E-02,.3950800E-02,.4677600E-02,.4785600E-02,.3057300E-02,&
     & .1172700E-02,.3998400E-02,.4763900E-02,.4909400E-02,.3170700E-02,&
     & .7817000E-03,.3138600E-02,.3675300E-02,.3692500E-02,.2274900E-02,&
     & .8382800E-03,.3206700E-02,.3765500E-02,.3810800E-02,.2379100E-02,&
     & .8867700E-03,.3263600E-02,.3852400E-02,.3926700E-02,.2480900E-02,&
     & .9281400E-03,.3312700E-02,.3933200E-02,.4038900E-02,.2581900E-02,&
     & .9634200E-03,.3356100E-02,.4012300E-02,.4149300E-02,.2682500E-02,&
     & .6451300E-03,.2625800E-02,.3079200E-02,.3102700E-02,.1910900E-02,&
     & .6906500E-03,.2683000E-02,.3157300E-02,.3205600E-02,.2001300E-02,&
     & .7295100E-03,.2732100E-02,.3232600E-02,.3306100E-02,.2090500E-02,&
     & .7627900E-03,.2775900E-02,.3305700E-02,.3404900E-02,.2179100E-02,&
     & .7912700E-03,.2816400E-02,.3377600E-02,.3503700E-02,.2268000E-02,&
     & .5321600E-03,.2194500E-02,.2576800E-02,.2604100E-02,.1604700E-02,&
     & .5687800E-03,.2243100E-02,.2645300E-02,.2693400E-02,.1682900E-02,&
     & .6000200E-03,.2286400E-02,.2712100E-02,.2781200E-02,.1760500E-02,&
     & .6266600E-03,.2326500E-02,.2777800E-02,.2868900E-02,.1838900E-02,&
     & .6495600E-03,.2364500E-02,.2843600E-02,.2958700E-02,.1917500E-02,&
     & .4387800E-03,.1832800E-02,.2155500E-02,.2184400E-02,.1347000E-02,&
     & .4683200E-03,.1874900E-02,.2216000E-02,.2261900E-02,.1414700E-02,&
     & .4933700E-03,.1913600E-02,.2275200E-02,.2339500E-02,.1483400E-02,&
     & .5148300E-03,.1950900E-02,.2334400E-02,.2417700E-02,.1552600E-02,&
     & .5332100E-03,.1986300E-02,.2395000E-02,.2498300E-02,.1622800E-02/

      data   absb(176:350, 7)   /                                       &
     & .3621800E-03,.1531000E-02,.1804400E-02,.1833300E-02,.1132000E-02,&
     & .3858700E-03,.1568100E-02,.1857300E-02,.1901000E-02,.1191100E-02,&
     & .4059900E-03,.1603400E-02,.1910500E-02,.1969300E-02,.1251600E-02,&
     & .4232700E-03,.1637200E-02,.1964200E-02,.2040000E-02,.1313300E-02,&
     & .4380000E-03,.1670200E-02,.2019600E-02,.2112000E-02,.1376000E-02,&
     & .2989400E-03,.1279400E-02,.1510900E-02,.1539200E-02,.9519600E-03,&
     & .3179500E-03,.1312600E-02,.1557800E-02,.1598500E-02,.1004100E-02,&
     & .3340900E-03,.1344400E-02,.1605400E-02,.1659000E-02,.1057500E-02,&
     & .3479800E-03,.1375200E-02,.1654200E-02,.1722800E-02,.1112400E-02,&
     & .3598200E-03,.1406200E-02,.1704600E-02,.1787200E-02,.1168800E-02,&
     & .2477900E-03,.1072000E-02,.1268700E-02,.1295900E-02,.8042800E-03,&
     & .2628700E-03,.1101200E-02,.1310400E-02,.1348300E-02,.8504600E-03,&
     & .2757000E-03,.1130000E-02,.1353200E-02,.1403200E-02,.8979600E-03,&
     & .2867600E-03,.1158300E-02,.1397800E-02,.1460200E-02,.9472200E-03,&
     & .2961200E-03,.1187300E-02,.1443900E-02,.1518500E-02,.9981200E-03,&
     & .2053600E-03,.8990100E-03,.1066600E-02,.1092500E-02,.6807700E-03,&
     & .2173200E-03,.9251400E-03,.1103700E-02,.1139400E-02,.7218900E-03,&
     & .2275200E-03,.9510900E-03,.1142600E-02,.1188800E-02,.7643700E-03,&
     & .2363200E-03,.9772800E-03,.1183200E-02,.1240300E-02,.8087700E-03,&
     & .2436900E-03,.1004000E-02,.1226000E-02,.1293400E-02,.8548900E-03,&
     & .1702100E-03,.7550600E-03,.8980000E-03,.9226500E-03,.5778800E-03,&
     & .1796900E-03,.7785300E-03,.9314900E-03,.9649700E-03,.6144300E-03,&
     & .1877800E-03,.8020700E-03,.9668000E-03,.1009700E-02,.6527800E-03,&
     & .1947700E-03,.8262200E-03,.1004100E-02,.1056400E-02,.6929300E-03,&
     & .2005500E-03,.8510400E-03,.1043900E-02,.1105300E-02,.7342400E-03,&
     & .1411000E-03,.6353100E-03,.7574600E-03,.7810900E-03,.4919500E-03,&
     & .1486000E-03,.6562900E-03,.7880100E-03,.8195100E-03,.5247200E-03,&
     & .1550200E-03,.6780200E-03,.8201800E-03,.8600500E-03,.5592900E-03,&
     & .1605400E-03,.7002300E-03,.8548700E-03,.9027000E-03,.5954600E-03,&
     & .1650600E-03,.7233300E-03,.8921300E-03,.9479400E-03,.6329600E-03,&
     & .1170400E-03,.5356300E-03,.6408900E-03,.6634800E-03,.4203600E-03,&
     & .1229600E-03,.5548200E-03,.6685900E-03,.6984200E-03,.4498600E-03,&
     & .1280400E-03,.5746000E-03,.6983800E-03,.7353200E-03,.4811500E-03,&
     & .1323700E-03,.5952800E-03,.7308100E-03,.7747200E-03,.5137100E-03,&
     & .1358700E-03,.6170800E-03,.7656200E-03,.8164900E-03,.5480000E-03/

      data   absb(351:525, 7)   /                                       &
     & .9703700E-04,.4526300E-03,.5435300E-03,.5652800E-03,.3603100E-03,&
     & .1017100E-03,.4701800E-03,.5689900E-03,.5971000E-03,.3869800E-03,&
     & .1057200E-03,.4883300E-03,.5967500E-03,.6309600E-03,.4150500E-03,&
     & .1091100E-03,.5076700E-03,.6271300E-03,.6673600E-03,.4446600E-03,&
     & .1118200E-03,.5283000E-03,.6594900E-03,.7060000E-03,.4762500E-03,&
     & .8041900E-04,.3834900E-03,.4622400E-03,.4830800E-03,.3098200E-03,&
     & .8411000E-04,.3994300E-03,.4858400E-03,.5121400E-03,.3338600E-03,&
     & .8728300E-04,.4163600E-03,.5118900E-03,.5435300E-03,.3592400E-03,&
     & .8991400E-04,.4345600E-03,.5402000E-03,.5769800E-03,.3863500E-03,&
     & .9200300E-04,.4540600E-03,.5703000E-03,.6128600E-03,.4155300E-03,&
     & .6663700E-04,.3258000E-03,.3945300E-03,.4143900E-03,.2673800E-03,&
     & .6955200E-04,.3405200E-03,.4166000E-03,.4411600E-03,.2890400E-03,&
     & .7204200E-04,.3563300E-03,.4410200E-03,.4700500E-03,.3121800E-03,&
     & .7408200E-04,.3735100E-03,.4673100E-03,.5010800E-03,.3371400E-03,&
     & .7569200E-04,.3919000E-03,.4952900E-03,.5344700E-03,.3642100E-03,&
     & .5519400E-04,.2775700E-03,.3380400E-03,.3567000E-03,.2314800E-03,&
     & .5748900E-04,.2913000E-03,.3587600E-03,.3814300E-03,.2511500E-03,&
     & .5944500E-04,.3061900E-03,.3814800E-03,.4081000E-03,.2724200E-03,&
     & .6102400E-04,.3223300E-03,.4057800E-03,.4369800E-03,.2955200E-03,&
     & .6225100E-04,.3396900E-03,.4318500E-03,.4683000E-03,.3205200E-03,&
     & .4570100E-04,.2373600E-03,.2908600E-03,.3083400E-03,.2011400E-03,&
     & .4751600E-04,.2502000E-03,.3103200E-03,.3311300E-03,.2191600E-03,&
     & .4903400E-04,.2642400E-03,.3313100E-03,.3559300E-03,.2387400E-03,&
     & .5024700E-04,.2794300E-03,.3539500E-03,.3829900E-03,.2602400E-03,&
     & .5118600E-04,.2957800E-03,.3782900E-03,.4125000E-03,.2834100E-03,&
     & .3782900E-04,.2037900E-03,.2514500E-03,.2676100E-03,.1754900E-03,&
     & .3925600E-04,.2158500E-03,.2695100E-03,.2887100E-03,.1920700E-03,&
     & .4043400E-04,.2290400E-03,.2890300E-03,.3119200E-03,.2103400E-03,&
     & .4136700E-04,.2433200E-03,.3102000E-03,.3374400E-03,.2302400E-03,&
     & .4207200E-04,.2587300E-03,.3330900E-03,.3652700E-03,.2517900E-03,&
     & .3130200E-04,.1757200E-03,.2183000E-03,.2332100E-03,.1537500E-03,&
     & .3241900E-04,.1870600E-03,.2350400E-03,.2529100E-03,.1691500E-03,&
     & .3332700E-04,.1994000E-03,.2532800E-03,.2747900E-03,.1861700E-03,&
     & .3403900E-04,.2128300E-03,.2731500E-03,.2988700E-03,.2046500E-03,&
     & .3456800E-04,.2273100E-03,.2947400E-03,.3251900E-03,.2248500E-03/

      data   absb(526:700, 7)   /                                       &
     & .2585100E-04,.1518200E-03,.1897600E-03,.2035300E-03,.1348300E-03,&
     & .2673100E-04,.1624100E-03,.2053100E-03,.2219900E-03,.1491800E-03,&
     & .2743700E-04,.1739500E-03,.2223700E-03,.2425800E-03,.1649100E-03,&
     & .2798800E-04,.1865600E-03,.2411300E-03,.2652900E-03,.1821700E-03,&
     & .2838400E-04,.2001700E-03,.2614400E-03,.2902000E-03,.2011600E-03,&
     & .2127100E-04,.1307800E-03,.1642600E-03,.1768500E-03,.1175600E-03,&
     & .2197500E-04,.1405800E-03,.1786500E-03,.1940300E-03,.1307200E-03,&
     & .2253300E-04,.1513200E-03,.1945600E-03,.2132300E-03,.1452300E-03,&
     & .2296600E-04,.1630700E-03,.2120600E-03,.2345400E-03,.1613300E-03,&
     & .2327400E-04,.1757900E-03,.2311200E-03,.2580000E-03,.1791100E-03,&
     & .1743100E-04,.1120500E-03,.1412800E-03,.1526000E-03,.1016100E-03,&
     & .1800400E-04,.1210300E-03,.1544800E-03,.1684000E-03,.1135600E-03,&
     & .1845800E-04,.1309500E-03,.1691800E-03,.1861600E-03,.1268700E-03,&
     & .1881000E-04,.1418200E-03,.1853900E-03,.2059600E-03,.1417900E-03,&
     & .1905900E-04,.1536300E-03,.2031700E-03,.2278800E-03,.1583100E-03,&
     & .1420400E-04,.9501000E-04,.1200800E-03,.1299600E-03,.8648200E-04,&
     & .1468500E-04,.1031300E-03,.1320000E-03,.1442600E-03,.9717900E-04,&
     & .1506900E-04,.1121400E-03,.1453600E-03,.1604200E-03,.1092200E-03,&
     & .1536900E-04,.1221100E-03,.1602000E-03,.1785500E-03,.1228200E-03,&
     & .1558600E-04,.1329600E-03,.1766100E-03,.1987600E-03,.1379700E-03,&
     & .1157100E-04,.8054500E-04,.1020600E-03,.1106300E-03,.7356300E-04,&
     & .1197600E-04,.8787500E-04,.1128100E-03,.1235700E-03,.8314900E-04,&
     & .1230100E-04,.9606300E-04,.1249500E-03,.1382900E-03,.9402600E-04,&
     & .1255700E-04,.1051800E-03,.1385100E-03,.1548700E-03,.1064200E-03,&
     & .1274400E-04,.1151500E-03,.1536400E-03,.1735200E-03,.1202900E-03,&
     & .9425400E-05,.6833300E-04,.8679800E-04,.9424700E-04,.6260400E-04,&
     & .9765700E-05,.7494500E-04,.9649500E-04,.1059500E-03,.7118500E-04,&
     & .1004100E-04,.8237600E-04,.1075200E-03,.1193400E-03,.8102200E-04,&
     & .1025800E-04,.9068700E-04,.1199200E-03,.1345300E-03,.9231500E-04,&
     & .1041900E-04,.9985200E-04,.1338700E-03,.1517700E-03,.1050400E-03,&
     & .7645200E-05,.5742800E-04,.7302100E-04,.7933300E-04,.5256800E-04,&
     & .7935400E-05,.6331500E-04,.8165000E-04,.8975200E-04,.6014400E-04,&
     & .8173900E-05,.6996100E-04,.9154200E-04,.1017600E-03,.6890500E-04,&
     & .8362600E-05,.7745600E-04,.1027500E-03,.1155200E-03,.7905900E-04,&
     & .8506400E-05,.8579500E-04,.1154200E-03,.1312800E-03,.9057800E-04/

      data   absb(701:875, 7)   /                                       &
     & .6195300E-05,.4816300E-04,.6128700E-04,.6661600E-04,.4401100E-04,&
     & .6441800E-05,.5338300E-04,.6893400E-04,.7584200E-04,.5067000E-04,&
     & .6648400E-05,.5931200E-04,.7778500E-04,.8658000E-04,.5844800E-04,&
     & .6813400E-05,.6605400E-04,.8787600E-04,.9899500E-04,.6754300E-04,&
     & .6940200E-05,.7361400E-04,.9937800E-04,.1133700E-03,.7794200E-04,&
     & .5017000E-05,.4035800E-04,.5137800E-04,.5587500E-04,.3679400E-04,&
     & .5226600E-05,.4497100E-04,.5814100E-04,.6402400E-04,.4263900E-04,&
     & .5404600E-05,.5025000E-04,.6603400E-04,.7360100E-04,.4952700E-04,&
     & .5548400E-05,.5630200E-04,.7511100E-04,.8478800E-04,.5765400E-04,&
     & .5660000E-05,.6315300E-04,.8555100E-04,.9789800E-04,.6704000E-04,&
     & .4049800E-05,.3356300E-04,.4271600E-04,.4643300E-04,.3044700E-04,&
     & .4229100E-05,.3760300E-04,.4861900E-04,.5355100E-04,.3551600E-04,&
     & .4383300E-05,.4226000E-04,.5558500E-04,.6199900E-04,.4154900E-04,&
     & .4509900E-05,.4765000E-04,.6366800E-04,.7196400E-04,.4872800E-04,&
     & .4609600E-05,.5379000E-04,.7305300E-04,.8377900E-04,.5710500E-04,&
     & .3261600E-05,.2778100E-04,.3533500E-04,.3836600E-04,.2503200E-04,&
     & .3416400E-05,.3129600E-04,.4044400E-04,.4453100E-04,.2939900E-04,&
     & .3549100E-05,.3537500E-04,.4654000E-04,.5192500E-04,.3463300E-04,&
     & .3661000E-05,.4014600E-04,.5368700E-04,.6073800E-04,.4093500E-04,&
     & .3750000E-05,.4561800E-04,.6206100E-04,.7129800E-04,.4835600E-04,&
     & .2623800E-05,.2295200E-04,.2916200E-04,.3163000E-04,.2052200E-04,&
     & .2757100E-05,.2599200E-04,.3356700E-04,.3693800E-04,.2426800E-04,&
     & .2871300E-05,.2954900E-04,.3888100E-04,.4338300E-04,.2879200E-04,&
     & .2969300E-05,.3376000E-04,.4517700E-04,.5114900E-04,.3430300E-04,&
     & .3048200E-05,.3862700E-04,.5262400E-04,.6055100E-04,.4085700E-04,&
     & .2105100E-05,.1886700E-04,.2393500E-04,.2591700E-04,.1671000E-04,&
     & .2220600E-05,.2148000E-04,.2770500E-04,.3044900E-04,.1989300E-04,&
     & .2319100E-05,.2455700E-04,.3229800E-04,.3601900E-04,.2376900E-04,&
     & .2404600E-05,.2823900E-04,.3779800E-04,.4280700E-04,.2854500E-04,&
     & .2474600E-05,.3254300E-04,.4437400E-04,.5110900E-04,.3429900E-04,&
     & .1681700E-05,.1538600E-04,.1946400E-04,.2102400E-04,.1345500E-04,&
     & .1782200E-05,.1760500E-04,.2265200E-04,.2484000E-04,.1612400E-04,&
     & .1868000E-05,.2023600E-04,.2656700E-04,.2958500E-04,.1940100E-04,&
     & .1942300E-05,.2341300E-04,.3131700E-04,.3543300E-04,.2347600E-04,&
     & .2004900E-05,.2718100E-04,.3705400E-04,.4266800E-04,.2846600E-04/

      data   absb(876:1050, 7)   /                                      &
     & .1340500E-05,.1251400E-04,.1577900E-04,.1699300E-04,.1079100E-04,&
     & .1428000E-05,.1438500E-04,.1846000E-04,.2018700E-04,.1301700E-04,&
     & .1502600E-05,.1662300E-04,.2177300E-04,.2420500E-04,.1576800E-04,&
     & .1567100E-05,.1935400E-04,.2585400E-04,.2921800E-04,.1922700E-04,&
     & .1622400E-05,.2262900E-04,.3083300E-04,.3547900E-04,.2352500E-04,&
     & .1066400E-05,.1014900E-04,.1275100E-04,.1368400E-04,.8618800E-05,&
     & .1141800E-05,.1172000E-04,.1499900E-04,.1635200E-04,.1046700E-04,&
     & .1206900E-05,.1361200E-04,.1778000E-04,.1971600E-04,.1275900E-04,&
     & .1263000E-05,.1594500E-04,.2126200E-04,.2399000E-04,.1567600E-04,&
     & .1311300E-05,.1877500E-04,.2556000E-04,.2938200E-04,.1935200E-04,&
     & .8480800E-06,.8240100E-05,.1031800E-04,.1103600E-04,.6893900E-05,&
     & .9126900E-06,.9551300E-05,.1218900E-04,.1324700E-04,.8420900E-05,&
     & .9692800E-06,.1116000E-04,.1453900E-04,.1608400E-04,.1033600E-04,&
     & .1017700E-05,.1314600E-04,.1749800E-04,.1971100E-04,.1279000E-04,&
     & .1059500E-05,.1559200E-04,.2120700E-04,.2435600E-04,.1592900E-04,&
     & .6737800E-06,.6682600E-05,.8338100E-05,.8882800E-05,.5505200E-05,&
     & .7287300E-06,.7775300E-05,.9890500E-05,.1071400E-04,.6763600E-05,&
     & .7776300E-06,.9136800E-05,.1186600E-04,.1309100E-04,.8355900E-05,&
     & .8195300E-06,.1082400E-04,.1437800E-04,.1616800E-04,.1041700E-04,&
     & .8556100E-06,.1293300E-04,.1757100E-04,.2015900E-04,.1308700E-04,&
     & .5344400E-06,.5406600E-05,.6719700E-05,.7129700E-05,.4381400E-05,&
     & .5809500E-06,.6312400E-05,.7998900E-05,.8634600E-05,.5412200E-05,&
     & .6228200E-06,.7456100E-05,.9648900E-05,.1061400E-04,.6728900E-05,&
     & .6591300E-06,.8891700E-05,.1178000E-04,.1322000E-04,.8449700E-05,&
     & .6903200E-06,.1069100E-04,.1450400E-04,.1661500E-04,.1070600E-04,&
     & .4230900E-06,.4363500E-05,.5401700E-05,.5704700E-05,.3473300E-05,&
     & .4622300E-06,.5109700E-05,.6446500E-05,.6932200E-05,.4312000E-05,&
     & .4979100E-06,.6064300E-05,.7816600E-05,.8569000E-05,.5394800E-05,&
     & .5294200E-06,.7272600E-05,.9604200E-05,.1074800E-04,.6821100E-05,&
     & .5563500E-06,.8807400E-05,.1192400E-04,.1363200E-04,.8717600E-05,&
     & .3352500E-06,.3529100E-05,.4353300E-05,.4578300E-05,.2761800E-05,&
     & .3680100E-06,.4146400E-05,.5209800E-05,.5582100E-05,.3446800E-05,&
     & .3983000E-06,.4944200E-05,.6349500E-05,.6939300E-05,.4339000E-05,&
     & .4253000E-06,.5963800E-05,.7852000E-05,.8765200E-05,.5523800E-05,&
     & .4485000E-06,.7271100E-05,.9825600E-05,.1121300E-04,.7118000E-05/

      data   absb(1051:1175, 7)   /                                     &
     & .2656100E-06,.2853500E-05,.3509100E-05,.3674700E-05,.2195300E-05,&
     & .2929900E-06,.3363000E-05,.4209300E-05,.4493500E-05,.2754300E-05,&
     & .3185500E-06,.4028600E-05,.5155500E-05,.5616000E-05,.3488500E-05,&
     & .3414900E-06,.4888400E-05,.6416600E-05,.7143200E-05,.4471200E-05,&
     & .3614900E-06,.6008200E-05,.8105300E-05,.9233800E-05,.5811400E-05,&
     & .2101700E-06,.2301900E-05,.2821700E-05,.2942000E-05,.1740300E-05,&
     & .2329400E-06,.2721500E-05,.3392200E-05,.3607000E-05,.2194300E-05,&
     & .2543600E-06,.3273200E-05,.4172000E-05,.4529100E-05,.2795000E-05,&
     & .2738300E-06,.3995100E-05,.5225200E-05,.5799000E-05,.3605400E-05,&
     & .2910600E-06,.4942900E-05,.6651800E-05,.7561300E-05,.4724600E-05,&
     & .1660800E-06,.1853100E-05,.2263900E-05,.2350400E-05,.1376000E-05,&
     & .1848800E-06,.2197100E-05,.2727100E-05,.2887000E-05,.1742100E-05,&
     & .2027900E-06,.2652000E-05,.3365900E-05,.3639200E-05,.2231600E-05,&
     & .2193100E-06,.3254300E-05,.4239100E-05,.4689100E-05,.2894900E-05,&
     & .2340400E-06,.4052700E-05,.5438000E-05,.6164900E-05,.3823400E-05,&
     & .1313100E-06,.1492700E-05,.1817900E-05,.1878700E-05,.1089300E-05,&
     & .1467700E-06,.1774800E-05,.2194600E-05,.2312200E-05,.1383900E-05,&
     & .1617100E-06,.2149400E-05,.2717100E-05,.2925600E-05,.1782700E-05,&
     & .1756700E-06,.2651400E-05,.3440700E-05,.3792300E-05,.2326900E-05,&
     & .1881600E-06,.3323700E-05,.4446600E-05,.5028000E-05,.3095800E-05,&
     & .1059900E-06,.1237400E-05,.1506000E-05,.1554300E-05,.8975000E-06,&
     & .1186800E-06,.1479500E-05,.1828500E-05,.1925100E-05,.1147900E-05,&
     & .1309800E-06,.1804200E-05,.2280700E-05,.2455200E-05,.1489700E-05,&
     & .1425200E-06,.2243900E-05,.2915200E-05,.3216300E-05,.1962200E-05,&
     & .1529100E-06,.2839600E-05,.3808700E-05,.4313800E-05,.2638700E-05/

      data   absb(  1:175, 8)   /                                       &
     & .8367300E-02,.1745400E-01,.2021300E-01,.2013200E-01,.1381000E-01,&
     & .8503900E-02,.1782400E-01,.2068400E-01,.2061600E-01,.1421900E-01,&
     & .8612900E-02,.1819600E-01,.2116300E-01,.2114300E-01,.1461600E-01,&
     & .8691300E-02,.1856300E-01,.2163500E-01,.2167400E-01,.1499100E-01,&
     & .8730100E-02,.1891600E-01,.2210300E-01,.2219800E-01,.1535100E-01,&
     & .6934600E-02,.1495200E-01,.1738000E-01,.1734200E-01,.1180300E-01,&
     & .7032800E-02,.1527600E-01,.1780800E-01,.1783000E-01,.1218100E-01,&
     & .7111300E-02,.1560400E-01,.1824000E-01,.1832100E-01,.1254400E-01,&
     & .7158000E-02,.1593700E-01,.1866700E-01,.1881400E-01,.1288400E-01,&
     & .7178500E-02,.1622800E-01,.1909300E-01,.1929900E-01,.1323000E-01,&
     & .5730700E-02,.1276300E-01,.1489000E-01,.1491200E-01,.1007300E-01,&
     & .5801700E-02,.1305200E-01,.1527800E-01,.1537000E-01,.1041700E-01,&
     & .5853300E-02,.1334700E-01,.1566600E-01,.1582800E-01,.1074200E-01,&
     & .5882600E-02,.1362800E-01,.1605700E-01,.1628900E-01,.1106600E-01,&
     & .5893300E-02,.1387300E-01,.1644400E-01,.1673900E-01,.1138800E-01,&
     & .4724700E-02,.1086300E-01,.1272300E-01,.1278800E-01,.8582200E-02,&
     & .4775500E-02,.1112400E-01,.1307600E-01,.1321100E-01,.8890700E-02,&
     & .4809500E-02,.1138500E-01,.1343100E-01,.1364100E-01,.9190300E-02,&
     & .4828300E-02,.1162200E-01,.1379000E-01,.1406600E-01,.9491200E-02,&
     & .4833300E-02,.1184100E-01,.1413500E-01,.1448700E-01,.9793800E-02,&
     & .3888100E-02,.9231300E-02,.1084800E-01,.1094200E-01,.7300600E-02,&
     & .3923800E-02,.9460100E-02,.1117100E-01,.1133300E-01,.7579200E-02,&
     & .3946800E-02,.9687400E-02,.1150000E-01,.1173200E-01,.7856000E-02,&
     & .3959000E-02,.9896200E-02,.1182600E-01,.1212900E-01,.8137600E-02,&
     & .3961200E-02,.1009400E-01,.1214400E-01,.1252100E-01,.8429300E-02,&
     & .3195300E-02,.7830900E-02,.9236100E-02,.9352300E-02,.6204800E-02,&
     & .3220800E-02,.8037200E-02,.9535000E-02,.9714800E-02,.6459000E-02,&
     & .3236500E-02,.8234200E-02,.9838200E-02,.1008200E-01,.6715400E-02,&
     & .3244700E-02,.8422900E-02,.1013800E-01,.1045000E-01,.6983800E-02,&
     & .3245000E-02,.8606100E-02,.1042700E-01,.1081400E-01,.7262600E-02,&
     & .2623500E-02,.6639900E-02,.7859900E-02,.7987500E-02,.5272800E-02,&
     & .2641400E-02,.6823100E-02,.8135100E-02,.8323000E-02,.5507000E-02,&
     & .2653000E-02,.7000400E-02,.8413800E-02,.8662900E-02,.5746800E-02,&
     & .2658300E-02,.7175600E-02,.8688700E-02,.9002900E-02,.6000600E-02,&
     & .2657300E-02,.7345700E-02,.8954900E-02,.9342200E-02,.6265800E-02/

      data   absb(176:350, 8)   /                                       &
     & .2152800E-02,.5630000E-02,.6692800E-02,.6827500E-02,.4488700E-02,&
     & .2165900E-02,.5796500E-02,.6948300E-02,.7137400E-02,.4704400E-02,&
     & .2174200E-02,.5961700E-02,.7203600E-02,.7450300E-02,.4931400E-02,&
     & .2177700E-02,.6125100E-02,.7456200E-02,.7765700E-02,.5170000E-02,&
     & .2175700E-02,.6285000E-02,.7706500E-02,.8089100E-02,.5422500E-02,&
     & .1765900E-02,.4777800E-02,.5705400E-02,.5841700E-02,.3827100E-02,&
     & .1775400E-02,.4929900E-02,.5939300E-02,.6125700E-02,.4026400E-02,&
     & .1781700E-02,.5083600E-02,.6175000E-02,.6414800E-02,.4239700E-02,&
     & .1783600E-02,.5237000E-02,.6408300E-02,.6712500E-02,.4464700E-02,&
     & .1781100E-02,.5387600E-02,.6648000E-02,.7020100E-02,.4707600E-02,&
     & .1448600E-02,.4067900E-02,.4882600E-02,.5020500E-02,.3280500E-02,&
     & .1455400E-02,.4208500E-02,.5098000E-02,.5282700E-02,.3466900E-02,&
     & .1459700E-02,.4351500E-02,.5315300E-02,.5553500E-02,.3667400E-02,&
     & .1460600E-02,.4495700E-02,.5535300E-02,.5837600E-02,.3881600E-02,&
     & .1457700E-02,.4638300E-02,.5767200E-02,.6130200E-02,.4115600E-02,&
     & .1188000E-02,.3469800E-02,.4186100E-02,.4324800E-02,.2819900E-02,&
     & .1192900E-02,.3601000E-02,.4386200E-02,.4568900E-02,.2995600E-02,&
     & .1195800E-02,.3734400E-02,.4587700E-02,.4827300E-02,.3184300E-02,&
     & .1195900E-02,.3869700E-02,.4800700E-02,.5096500E-02,.3390300E-02,&
     & .1192900E-02,.4006800E-02,.5023200E-02,.5379100E-02,.3613600E-02,&
     & .9740600E-03,.2967500E-02,.3600600E-02,.3737900E-02,.2434400E-02,&
     & .9775600E-03,.3089600E-02,.3785000E-02,.3968600E-02,.2599400E-02,&
     & .9794600E-03,.3215000E-02,.3977500E-02,.4213500E-02,.2777600E-02,&
     & .9789900E-03,.3343000E-02,.4182500E-02,.4470800E-02,.2975300E-02,&
     & .9761500E-03,.3476100E-02,.4396600E-02,.4745700E-02,.3190000E-02,&
     & .7985600E-03,.2545900E-02,.3107700E-02,.3244700E-02,.2111400E-02,&
     & .8011000E-03,.2660500E-02,.3280500E-02,.3464000E-02,.2266400E-02,&
     & .8022200E-03,.2777800E-02,.3466600E-02,.3695800E-02,.2437600E-02,&
     & .8012900E-03,.2901400E-02,.3662300E-02,.3944400E-02,.2626900E-02,&
     & .7985700E-03,.3030200E-02,.3869800E-02,.4212600E-02,.2833500E-02,&
     & .6546000E-03,.2193300E-02,.2694400E-02,.2833400E-02,.1841700E-02,&
     & .6563900E-03,.2300300E-02,.2859900E-02,.3040900E-02,.1988600E-02,&
     & .6568500E-03,.2413200E-02,.3038400E-02,.3263900E-02,.2154500E-02,&
     & .6557700E-03,.2532300E-02,.3227500E-02,.3505500E-02,.2336600E-02,&
     & .6532500E-03,.2657200E-02,.3429400E-02,.3768700E-02,.2536500E-02/

      data   absb(351:525, 8)   /                                       &
     & .5365500E-03,.1896600E-02,.2347400E-02,.2487300E-02,.1614200E-02,&
     & .5377700E-03,.1997900E-02,.2506900E-02,.2684900E-02,.1757000E-02,&
     & .5378300E-03,.2107000E-02,.2677800E-02,.2901000E-02,.1916300E-02,&
     & .5366100E-03,.2221800E-02,.2860900E-02,.3137300E-02,.2092500E-02,&
     & .5342600E-03,.2343700E-02,.3059100E-02,.3395300E-02,.2287100E-02,&
     & .4397300E-03,.1646700E-02,.2056400E-02,.2195000E-02,.1424300E-02,&
     & .4404600E-03,.1744500E-02,.2209400E-02,.2385400E-02,.1562200E-02,&
     & .4402700E-03,.1849500E-02,.2373700E-02,.2595400E-02,.1716000E-02,&
     & .4390200E-03,.1960700E-02,.2552700E-02,.2828000E-02,.1887500E-02,&
     & .4369000E-03,.2079700E-02,.2748300E-02,.3080900E-02,.2078300E-02,&
     & .3603500E-03,.1437700E-02,.1812900E-02,.1950200E-02,.1266500E-02,&
     & .3607800E-03,.1532200E-02,.1959300E-02,.2134600E-02,.1399300E-02,&
     & .3603500E-03,.1633300E-02,.2119400E-02,.2340700E-02,.1548900E-02,&
     & .3591700E-03,.1741600E-02,.2295800E-02,.2569300E-02,.1716900E-02,&
     & .3572200E-03,.1858300E-02,.2489300E-02,.2819900E-02,.1905800E-02,&
     & .2952200E-03,.1262500E-02,.1607300E-02,.1744400E-02,.1133700E-02,&
     & .2954400E-03,.1353400E-02,.1749300E-02,.1924800E-02,.1262900E-02,&
     & .2948900E-03,.1451300E-02,.1906400E-02,.2128100E-02,.1409300E-02,&
     & .2937700E-03,.1557500E-02,.2081300E-02,.2353600E-02,.1575100E-02,&
     & .2920100E-03,.1672500E-02,.2272800E-02,.2603100E-02,.1763300E-02,&
     & .2418600E-03,.1115500E-02,.1435300E-02,.1572900E-02,.1023200E-02,&
     & .2418900E-03,.1203200E-02,.1573900E-02,.1750800E-02,.1149800E-02,&
     & .2413000E-03,.1298800E-02,.1729600E-02,.1951600E-02,.1294200E-02,&
     & .2402500E-03,.1403400E-02,.1903000E-02,.2176400E-02,.1459400E-02,&
     & .2386600E-03,.1517000E-02,.2094000E-02,.2427000E-02,.1647100E-02,&
     & .1981100E-03,.9918800E-03,.1291400E-02,.1431100E-02,.9315700E-03,&
     & .1979900E-03,.1077100E-02,.1428200E-02,.1607300E-02,.1056200E-02,&
     & .1974100E-03,.1171100E-02,.1582800E-02,.1806800E-02,.1199900E-02,&
     & .1964500E-03,.1274600E-02,.1755900E-02,.2032000E-02,.1365400E-02,&
     & .1950300E-03,.1387100E-02,.1947300E-02,.2285100E-02,.1553300E-02,&
     & .1622500E-03,.8880300E-03,.1171700E-02,.1313800E-02,.8555600E-03,&
     & .1620500E-03,.9717400E-03,.1307700E-02,.1489000E-02,.9793000E-03,&
     & .1614800E-03,.1064800E-02,.1462100E-02,.1689100E-02,.1123700E-02,&
     & .1605900E-03,.1167300E-02,.1635700E-02,.1916500E-02,.1289600E-02,&
     & .1593400E-03,.1279500E-02,.1828800E-02,.2173600E-02,.1479100E-02/

      data   absb(526:700, 8)   /                                       &
     & .1328600E-03,.7983500E-03,.1068300E-02,.1211700E-02,.7890800E-03,&
     & .1326100E-03,.8806600E-03,.1203600E-02,.1386500E-02,.9125200E-03,&
     & .1320900E-03,.9727000E-03,.1358000E-02,.1587100E-02,.1057000E-02,&
     & .1313000E-03,.1074800E-02,.1531800E-02,.1817100E-02,.1223600E-02,&
     & .1302200E-03,.1186600E-02,.1726400E-02,.2078600E-02,.1414800E-02,&
     & .1087900E-03,.7156700E-03,.9707500E-03,.1111800E-02,.7235300E-03,&
     & .1085500E-03,.7962500E-03,.1104100E-02,.1284500E-02,.8454000E-03,&
     & .1081100E-03,.8868600E-03,.1256900E-02,.1484700E-02,.9883100E-03,&
     & .1074300E-03,.9876100E-03,.1430000E-02,.1715700E-02,.1154100E-02,&
     & .1065000E-03,.1098700E-02,.1625100E-02,.1979900E-02,.1346000E-02,&
     & .8909100E-04,.6377300E-03,.8752500E-03,.1010600E-02,.6564300E-03,&
     & .8889200E-04,.7157200E-03,.1005000E-02,.1179400E-02,.7748200E-03,&
     & .8851000E-04,.8039000E-03,.1154400E-02,.1376800E-02,.9143700E-03,&
     & .8795100E-04,.9025300E-03,.1325500E-02,.1606300E-02,.1077900E-02,&
     & .8717500E-04,.1012000E-02,.1518700E-02,.1870300E-02,.1267700E-02,&
     & .7296700E-04,.5605100E-03,.7758600E-03,.9013900E-03,.5831900E-03,&
     & .7281800E-04,.6345900E-03,.8996200E-03,.1062700E-02,.6954500E-03,&
     & .7252300E-04,.7190900E-03,.1043300E-02,.1253300E-02,.8289600E-03,&
     & .7208700E-04,.8142700E-03,.1209200E-02,.1476600E-02,.9867100E-03,&
     & .7148600E-04,.9209300E-03,.1397700E-02,.1735500E-02,.1170800E-02,&
     & .5976500E-04,.4932000E-03,.6884400E-03,.8047800E-03,.5184900E-03,&
     & .5966000E-04,.5635400E-03,.8064000E-03,.9587600E-03,.6246600E-03,&
     & .5943600E-04,.6444000E-03,.9445400E-03,.1142600E-02,.7523600E-03,&
     & .5910200E-04,.7363100E-03,.1105200E-02,.1359900E-02,.9043300E-03,&
     & .5863500E-04,.8400900E-03,.1289100E-02,.1613400E-02,.1082900E-02,&
     & .4895600E-04,.4349900E-03,.6123100E-03,.7201700E-03,.4618900E-03,&
     & .4889000E-04,.5018100E-03,.7247800E-03,.8673000E-03,.5624400E-03,&
     & .4871700E-04,.5792300E-03,.8577800E-03,.1044800E-02,.6846400E-03,&
     & .4845100E-04,.6679900E-03,.1013200E-02,.1256300E-02,.8311200E-03,&
     & .4808500E-04,.7689800E-03,.1192800E-02,.1504700E-02,.1004400E-02,&
     & .4010700E-04,.3786000E-03,.5359600E-03,.6330200E-03,.4035000E-03,&
     & .4006800E-04,.4409900E-03,.6412600E-03,.7710500E-03,.4969700E-03,&
     & .3994500E-04,.5139900E-03,.7669400E-03,.9393400E-03,.6116600E-03,&
     & .3975200E-04,.5985100E-03,.9152800E-03,.1141600E-02,.7503700E-03,&
     & .3946900E-04,.6954400E-03,.1088000E-02,.1380900E-02,.9159300E-03/

      data   absb(701:875, 8)   /                                       &
     & .3285100E-04,.3287800E-03,.4678600E-03,.5546900E-03,.3512000E-03,&
     & .3283900E-04,.3868300E-03,.5660700E-03,.6839400E-03,.4376900E-03,&
     & .3274900E-04,.4554200E-03,.6844400E-03,.8425900E-03,.5449000E-03,&
     & .3260700E-04,.5356400E-03,.8254700E-03,.1035200E-02,.6758500E-03,&
     & .3239500E-04,.6284100E-03,.9913300E-03,.1265200E-02,.8333200E-03,&
     & .2690600E-04,.2853400E-03,.4080500E-03,.4853500E-03,.3052600E-03,&
     & .2690800E-04,.3392200E-03,.4995000E-03,.6060300E-03,.3850500E-03,&
     & .2684600E-04,.4036100E-03,.6107600E-03,.7556400E-03,.4852400E-03,&
     & .2673900E-04,.4795300E-03,.7445400E-03,.9388200E-03,.6086100E-03,&
     & .2658500E-04,.5681600E-03,.9036300E-03,.1159700E-02,.7582900E-03,&
     & .2203400E-04,.2449400E-03,.3512600E-03,.4186200E-03,.2613300E-03,&
     & .2204500E-04,.2942800E-03,.4352600E-03,.5296000E-03,.3337900E-03,&
     & .2200900E-04,.3539400E-03,.5383800E-03,.6689300E-03,.4259900E-03,&
     & .2193300E-04,.4251000E-03,.6639200E-03,.8409000E-03,.5406100E-03,&
     & .2182000E-04,.5087800E-03,.8147400E-03,.1050300E-02,.6813900E-03,&
     & .1804200E-04,.2087800E-03,.2998800E-03,.3578200E-03,.2216000E-03,&
     & .1805800E-04,.2536000E-03,.3762100E-03,.4587500E-03,.2867000E-03,&
     & .1804100E-04,.3084500E-03,.4712300E-03,.5872700E-03,.3707000E-03,&
     & .1798900E-04,.3745900E-03,.5880400E-03,.7473900E-03,.4765000E-03,&
     & .1790600E-04,.4531700E-03,.7300300E-03,.9445400E-03,.6076800E-03,&
     & .1477000E-04,.1774300E-03,.2551800E-03,.3046100E-03,.1871800E-03,&
     & .1479000E-04,.2179400E-03,.3242400E-03,.3960300E-03,.2453400E-03,&
     & .1478500E-04,.2682200E-03,.4113200E-03,.5140600E-03,.3215800E-03,&
     & .1475100E-04,.3294800E-03,.5196800E-03,.6628800E-03,.4188900E-03,&
     & .1469200E-04,.4030900E-03,.6529800E-03,.8479500E-03,.5407800E-03,&
     & .1209100E-04,.1495600E-03,.2150700E-03,.2566400E-03,.1564100E-03,&
     & .1211200E-04,.1858500E-03,.2768900E-03,.3385300E-03,.2077800E-03,&
     & .1211300E-04,.2315600E-03,.3559700E-03,.4457500E-03,.2761900E-03,&
     & .1209400E-04,.2878900E-03,.4557600E-03,.5831300E-03,.3649200E-03,&
     & .1205400E-04,.3562800E-03,.5798000E-03,.7552300E-03,.4772100E-03,&
     & .9895800E-05,.1243600E-03,.1783800E-03,.2124100E-03,.1284100E-03,&
     & .9915700E-05,.1563300E-03,.2327900E-03,.2844300E-03,.1729100E-03,&
     & .9922400E-05,.1972400E-03,.3035000E-03,.3801700E-03,.2332300E-03,&
     & .9915000E-05,.2483100E-03,.3939600E-03,.5048800E-03,.3127400E-03,&
     & .9888100E-05,.3111000E-03,.5078800E-03,.6630400E-03,.4148500E-03/

      data   absb(876:1050, 8)   /                                      &
     & .8098100E-05,.1028400E-03,.1470100E-03,.1745700E-03,.1046400E-03,&
     & .8117000E-05,.1308300E-03,.1945200E-03,.2373400E-03,.1429800E-03,&
     & .8125400E-05,.1671600E-03,.2572700E-03,.3223000E-03,.1956500E-03,&
     & .8125300E-05,.2132800E-03,.3388700E-03,.4347500E-03,.2665200E-03,&
     & .8109200E-05,.2706200E-03,.4430100E-03,.5797200E-03,.3588700E-03,&
     & .6626200E-05,.8454200E-04,.1203400E-03,.1424500E-03,.8463600E-04,&
     & .6643500E-05,.1090300E-03,.1616800E-03,.1968600E-03,.1174000E-03,&
     & .6652700E-05,.1409200E-03,.2168500E-03,.2715600E-03,.1630900E-03,&
     & .6656700E-05,.1822900E-03,.2898900E-03,.3721400E-03,.2257200E-03,&
     & .6648900E-05,.2344300E-03,.3846100E-03,.5041800E-03,.3088300E-03,&
     & .5420200E-05,.6960800E-04,.9868000E-04,.1164600E-03,.6851000E-04,&
     & .5437400E-05,.9080200E-04,.1343400E-03,.1633000E-03,.9649900E-04,&
     & .5447500E-05,.1191600E-03,.1832700E-03,.2293200E-03,.1362000E-03,&
     & .5452100E-05,.1560800E-03,.2484500E-03,.3192800E-03,.1915400E-03,&
     & .5449800E-05,.2034800E-03,.3346600E-03,.4393700E-03,.2662600E-03,&
     & .4435100E-05,.5708800E-04,.8058200E-04,.9479000E-04,.5525000E-04,&
     & .4450400E-05,.7543500E-04,.1113100E-03,.1350300E-03,.7902400E-04,&
     & .4459900E-05,.1003700E-03,.1542800E-03,.1929100E-03,.1134300E-03,&
     & .4465100E-05,.1333600E-03,.2125200E-03,.2733500E-03,.1621800E-03,&
     & .4465300E-05,.1764000E-03,.2907500E-03,.3822900E-03,.2292300E-03,&
     & .3629000E-05,.4656500E-04,.6536400E-04,.7659800E-04,.4424000E-04,&
     & .3642000E-05,.6231400E-04,.9164400E-04,.1109000E-03,.6423400E-04,&
     & .3650900E-05,.8410200E-04,.1291000E-03,.1612500E-03,.9383300E-04,&
     & .3655500E-05,.1136200E-03,.1811500E-03,.2329700E-03,.1365700E-03,&
     & .3657900E-05,.1522600E-03,.2513600E-03,.3309900E-03,.1962100E-03,&
     & .2969200E-05,.3774600E-04,.5263200E-04,.6139700E-04,.3516500E-04,&
     & .2980800E-05,.5114900E-04,.7487900E-04,.9033500E-04,.5180700E-04,&
     & .2988000E-05,.7002800E-04,.1072400E-03,.1337500E-03,.7701000E-04,&
     & .2993100E-05,.9610500E-04,.1531400E-03,.1968900E-03,.1141700E-03,&
     & .2995900E-05,.1307400E-03,.2161200E-03,.2849200E-03,.1669700E-03,&
     & .2429600E-05,.3069800E-04,.4254400E-04,.4940600E-04,.2807200E-04,&
     & .2439300E-05,.4212200E-04,.6145400E-04,.7392700E-04,.4196400E-04,&
     & .2446000E-05,.5852800E-04,.8950000E-04,.1114600E-03,.6350300E-04,&
     & .2450700E-05,.8161100E-04,.1300900E-03,.1672800E-03,.9594700E-04,&
     & .2453500E-05,.1127600E-03,.1868200E-03,.2465400E-03,.1428600E-03/

      data   absb(1051:1175, 8)   /                                     &
     & .1988300E-05,.2492100E-04,.3431200E-04,.3966600E-04,.2237500E-04,&
     & .1996300E-05,.3464100E-04,.5034400E-04,.6038100E-04,.3394200E-04,&
     & .2002000E-05,.4886800E-04,.7461800E-04,.9280900E-04,.5228000E-04,&
     & .2006500E-05,.6925500E-04,.1104200E-03,.1420300E-03,.8056600E-04,&
     & .2008900E-05,.9752700E-04,.1618200E-03,.2137300E-03,.1222800E-03,&
     & .1626800E-05,.2012200E-04,.2750600E-04,.3162900E-04,.1771800E-04,&
     & .1633700E-05,.2832100E-04,.4096600E-04,.4896000E-04,.2727100E-04,&
     & .1638500E-05,.4057900E-04,.6182800E-04,.7674700E-04,.4273900E-04,&
     & .1642400E-05,.5849000E-04,.9322000E-04,.1199100E-03,.6723200E-04,&
     & .1645000E-05,.8377600E-04,.1391700E-03,.1840100E-03,.1040600E-03,&
     & .1331100E-05,.1615100E-04,.2190100E-04,.2504400E-04,.1393400E-04,&
     & .1336900E-05,.2300500E-04,.3308300E-04,.3937700E-04,.2175100E-04,&
     & .1341100E-05,.3350100E-04,.5088800E-04,.6301400E-04,.3469600E-04,&
     & .1344300E-05,.4911100E-04,.7819100E-04,.1005100E-03,.5571100E-04,&
     & .1346700E-05,.7158600E-04,.1190200E-03,.1574400E-03,.8802500E-04,&
     & .1089200E-05,.1296200E-04,.1744300E-04,.1982500E-04,.1095500E-04,&
     & .1094200E-05,.1868800E-04,.2671600E-04,.3166400E-04,.1734800E-04,&
     & .1097600E-05,.2765000E-04,.4186400E-04,.5171600E-04,.2815900E-04,&
     & .1100400E-05,.4125100E-04,.6563000E-04,.8429900E-04,.4615600E-04,&
     & .1102500E-05,.6126100E-04,.1019600E-03,.1349600E-03,.7453600E-04,&
     & .8919700E-06,.1104700E-04,.1489900E-04,.1696400E-04,.9299900E-05,&
     & .8960500E-06,.1621500E-04,.2329100E-04,.2768600E-04,.1501900E-04,&
     & .8987500E-06,.2446200E-04,.3726700E-04,.4624300E-04,.2490500E-04,&
     & .9011700E-06,.3717500E-04,.5959400E-04,.7696900E-04,.4167200E-04,&
     & .9031000E-06,.5611500E-04,.9418700E-04,.1252800E-03,.6849700E-04/

      data   absb(  1:175, 9)   /                                       &
     & .7386370E-01,.1217270E+00,.1397997E+00,.1419673E+00,.1084738E+00,&
     & .7529449E-01,.1232830E+00,.1414079E+00,.1438635E+00,.1108050E+00,&
     & .7697992E-01,.1254793E+00,.1434365E+00,.1458286E+00,.1129988E+00,&
     & .7887100E-01,.1280855E+00,.1459070E+00,.1479085E+00,.1152446E+00,&
     & .8092090E-01,.1307909E+00,.1487251E+00,.1501501E+00,.1175320E+00,&
     & .6403379E-01,.1090204E+00,.1253282E+00,.1274699E+00,.9632574E-01,&
     & .6547459E-01,.1110735E+00,.1274271E+00,.1295924E+00,.9864560E-01,&
     & .6715540E-01,.1136407E+00,.1299702E+00,.1318704E+00,.1010107E+00,&
     & .6906380E-01,.1163504E+00,.1328994E+00,.1343146E+00,.1034582E+00,&
     & .7105787E-01,.1191957E+00,.1361107E+00,.1370057E+00,.1059074E+00,&
     & .5530200E-01,.9757076E-01,.1121740E+00,.1140513E+00,.8533242E-01,&
     & .5675787E-01,.1000348E+00,.1147292E+00,.1164809E+00,.8776543E-01,&
     & .5846659E-01,.1027322E+00,.1176838E+00,.1190849E+00,.9029882E-01,&
     & .6032734E-01,.1055957E+00,.1209546E+00,.1219366E+00,.9286960E-01,&
     & .6209121E-01,.1085948E+00,.1243416E+00,.1250550E+00,.9547001E-01,&
     & .4762395E-01,.8734512E-01,.1003708E+00,.1018642E+00,.7561273E-01,&
     & .4911368E-01,.8996446E-01,.1032731E+00,.1045503E+00,.7815395E-01,&
     & .5080031E-01,.9279755E-01,.1065652E+00,.1074922E+00,.8078702E-01,&
     & .5243449E-01,.9581412E-01,.1100211E+00,.1107497E+00,.8346128E-01,&
     & .5394988E-01,.9894050E-01,.1135959E+00,.1142797E+00,.8622867E-01,&
     & .4090347E-01,.7810647E-01,.8979429E-01,.9092391E-01,.6709977E-01,&
     & .4239633E-01,.8087453E-01,.9303629E-01,.9387987E-01,.6971100E-01,&
     & .4389973E-01,.8384796E-01,.9650459E-01,.9716037E-01,.7239385E-01,&
     & .4532474E-01,.8698861E-01,.1001236E+00,.1007738E+00,.7517902E-01,&
     & .4664725E-01,.9022266E-01,.1038715E+00,.1046269E+00,.7808115E-01,&
     & .3507354E-01,.6984031E-01,.8045794E-01,.8126652E-01,.5967808E-01,&
     & .3641404E-01,.7272845E-01,.8387260E-01,.8449727E-01,.6232326E-01,&
     & .3773547E-01,.7581798E-01,.8747042E-01,.8807986E-01,.6506146E-01,&
     & .3899381E-01,.7905992E-01,.9126223E-01,.9196437E-01,.6793314E-01,&
     & .4015940E-01,.8236350E-01,.9521155E-01,.9607923E-01,.7090836E-01,&
     & .2996758E-01,.6248612E-01,.7222878E-01,.7286184E-01,.5319451E-01,&
     & .3114015E-01,.6548203E-01,.7573235E-01,.7633337E-01,.5587473E-01,&
     & .3229126E-01,.6867140E-01,.7946891E-01,.8015608E-01,.5867307E-01,&
     & .3340366E-01,.7198946E-01,.8342015E-01,.8425577E-01,.6160468E-01,&
     & .3444385E-01,.7527768E-01,.8753864E-01,.8855726E-01,.6465244E-01/

      data   absb(176:350, 9)   /                                       &
     & .2552897E-01,.5604549E-01,.6504953E-01,.6568338E-01,.4759673E-01,&
     & .2655263E-01,.5914810E-01,.6865615E-01,.6934990E-01,.5031583E-01,&
     & .2756945E-01,.6242191E-01,.7252447E-01,.7335652E-01,.5316631E-01,&
     & .2855171E-01,.6573444E-01,.7661794E-01,.7759839E-01,.5616029E-01,&
     & .2946709E-01,.6902393E-01,.8088256E-01,.8205689E-01,.5928927E-01,&
     & .2169323E-01,.5042927E-01,.5880926E-01,.5953518E-01,.4276904E-01,&
     & .2259137E-01,.5361379E-01,.6252841E-01,.6336797E-01,.4551161E-01,&
     & .2349223E-01,.5691109E-01,.6652380E-01,.6748411E-01,.4841701E-01,&
     & .2435866E-01,.6022552E-01,.7072962E-01,.7184630E-01,.5147673E-01,&
     & .2516486E-01,.6352693E-01,.7511834E-01,.7646009E-01,.5467556E-01,&
     & .1845909E-01,.4576813E-01,.5367465E-01,.5454792E-01,.3878639E-01,&
     & .1925599E-01,.4899168E-01,.5752566E-01,.5849933E-01,.4157610E-01,&
     & .2005017E-01,.5229447E-01,.6163578E-01,.6272679E-01,.4454755E-01,&
     & .2080611E-01,.5562768E-01,.6596785E-01,.6724008E-01,.4767687E-01,&
     & .2150826E-01,.5895362E-01,.7048564E-01,.7202217E-01,.5095808E-01,&
     & .1570422E-01,.4178498E-01,.4935434E-01,.5034621E-01,.3542605E-01,&
     & .1640986E-01,.4501386E-01,.5331082E-01,.5440870E-01,.3827459E-01,&
     & .1710795E-01,.4833320E-01,.5753856E-01,.5876524E-01,.4130508E-01,&
     & .1776552E-01,.5169642E-01,.6199404E-01,.6343553E-01,.4450482E-01,&
     & .1836301E-01,.5505656E-01,.6662521E-01,.6837233E-01,.4786425E-01,&
     & .1336950E-01,.3839896E-01,.4576222E-01,.4685100E-01,.3263474E-01,&
     & .1399497E-01,.4164334E-01,.4982575E-01,.5104297E-01,.3553968E-01,&
     & .1459622E-01,.4499292E-01,.5417389E-01,.5554832E-01,.3863403E-01,&
     & .1515950E-01,.4838565E-01,.5872737E-01,.6036693E-01,.4189693E-01,&
     & .1567094E-01,.5178762E-01,.6347431E-01,.6547102E-01,.4533998E-01,&
     & .1139415E-01,.3556539E-01,.4282652E-01,.4400955E-01,.3034823E-01,&
     & .1193609E-01,.3883632E-01,.4700523E-01,.4834132E-01,.3330647E-01,&
     & .1245410E-01,.4221330E-01,.5145145E-01,.5300744E-01,.3646560E-01,&
     & .1293803E-01,.4565202E-01,.5613109E-01,.5799082E-01,.3980753E-01,&
     & .1337501E-01,.4910924E-01,.6099932E-01,.6327343E-01,.4334453E-01,&
     & .9722913E-02,.3326032E-01,.4052121E-01,.4180726E-01,.2852809E-01,&
     & .1018992E-01,.3656568E-01,.4481036E-01,.4630022E-01,.3155388E-01,&
     & .1063595E-01,.3998111E-01,.4937235E-01,.5113650E-01,.3479040E-01,&
     & .1105185E-01,.4347100E-01,.5418973E-01,.5629833E-01,.3822398E-01,&
     & .1143041E-01,.4699477E-01,.5918778E-01,.6176195E-01,.4186182E-01/

      data   absb(351:525, 9)   /                                       &
     & .8300709E-02,.3138262E-01,.3871373E-01,.4011993E-01,.2708755E-01,&
     & .8705608E-02,.3472930E-01,.4312192E-01,.4478609E-01,.3018835E-01,&
     & .9091211E-02,.3820073E-01,.4781870E-01,.4980017E-01,.3350792E-01,&
     & .9448828E-02,.4175226E-01,.5277360E-01,.5514398E-01,.3704207E-01,&
     & .9783765E-02,.4534922E-01,.5791589E-01,.6079477E-01,.4078994E-01,&
     & .7094791E-02,.2988609E-01,.3735115E-01,.3890287E-01,.2597883E-01,&
     & .7443776E-02,.3328590E-01,.4189354E-01,.4374887E-01,.2916268E-01,&
     & .7777473E-02,.3682175E-01,.4673676E-01,.4894363E-01,.3257840E-01,&
     & .8091821E-02,.4045146E-01,.5183182E-01,.5446905E-01,.3621756E-01,&
     & .8393762E-02,.4412277E-01,.5711772E-01,.6030049E-01,.4008338E-01,&
     & .6072721E-02,.2875406E-01,.3642428E-01,.3814167E-01,.2519233E-01,&
     & .6375450E-02,.3222477E-01,.4111499E-01,.4316950E-01,.2846448E-01,&
     & .6665401E-02,.3583825E-01,.4610908E-01,.4855053E-01,.3198856E-01,&
     & .6947813E-02,.3954583E-01,.5134775E-01,.5425463E-01,.3574183E-01,&
     & .7224449E-02,.4329363E-01,.5677163E-01,.6025181E-01,.3971948E-01,&
     & .5204996E-02,.2793567E-01,.3586995E-01,.3776366E-01,.2467285E-01,&
     & .5467917E-02,.3148948E-01,.4071531E-01,.4297096E-01,.2804493E-01,&
     & .5726383E-02,.3518386E-01,.4585588E-01,.4852812E-01,.3168056E-01,&
     & .5982991E-02,.3897236E-01,.5123260E-01,.5440198E-01,.3555049E-01,&
     & .6240892E-02,.4279660E-01,.5678660E-01,.6055093E-01,.3963932E-01,&
     & .4469682E-02,.2742048E-01,.3567286E-01,.3774779E-01,.2440899E-01,&
     & .4700641E-02,.3106355E-01,.4067206E-01,.4312796E-01,.2789294E-01,&
     & .4936385E-02,.3484450E-01,.4595839E-01,.4885735E-01,.3164155E-01,&
     & .5173162E-02,.3871026E-01,.5146122E-01,.5488738E-01,.3562372E-01,&
     & .5417514E-02,.4260570E-01,.5713175E-01,.6117015E-01,.3982399E-01,&
     & .3844994E-02,.2717409E-01,.3578885E-01,.3804312E-01,.2437272E-01,&
     & .4053349E-02,.3090840E-01,.4093856E-01,.4359092E-01,.2797192E-01,&
     & .4269342E-02,.3477112E-01,.4635566E-01,.4947518E-01,.3183081E-01,&
     & .4494036E-02,.3870776E-01,.5197315E-01,.5564894E-01,.3592188E-01,&
     & .4730771E-02,.4267219E-01,.5774228E-01,.6204827E-01,.4022778E-01,&
     & .3316155E-02,.2716439E-01,.3617214E-01,.3860092E-01,.2453709E-01,&
     & .3507741E-02,.3098644E-01,.4145934E-01,.4430670E-01,.2825000E-01,&
     & .3708780E-02,.3492455E-01,.4699683E-01,.5032871E-01,.3221778E-01,&
     & .3925552E-02,.3892413E-01,.5271553E-01,.5663089E-01,.3641334E-01,&
     & .4158892E-02,.4294570E-01,.5857114E-01,.6313194E-01,.4081644E-01/

      data   absb(526:700, 9)   /                                       &
     & .2863791E-02,.2722174E-01,.3659228E-01,.3917713E-01,.2474498E-01,&
     & .3040588E-02,.3112320E-01,.4199421E-01,.4501754E-01,.2855786E-01,&
     & .3233150E-02,.3512719E-01,.4763348E-01,.5115797E-01,.3261841E-01,&
     & .3445381E-02,.3917826E-01,.5343694E-01,.5756145E-01,.3690166E-01,&
     & .3677438E-02,.4324272E-01,.5935861E-01,.6414300E-01,.4138900E-01,&
     & .2465329E-02,.2707028E-01,.3666695E-01,.3936557E-01,.2472691E-01,&
     & .2632781E-02,.3103171E-01,.4215043E-01,.4529694E-01,.2860992E-01,&
     & .2818872E-02,.3508603E-01,.4786198E-01,.5151711E-01,.3273833E-01,&
     & .3025017E-02,.3917768E-01,.5372478E-01,.5798985E-01,.3708084E-01,&
     & .3257127E-02,.4327518E-01,.5969457E-01,.6462649E-01,.4162399E-01,&
     & .2114507E-02,.2661661E-01,.3628160E-01,.3903834E-01,.2439799E-01,&
     & .2271568E-02,.3061795E-01,.4180884E-01,.4501716E-01,.2831780E-01,&
     & .2449593E-02,.3470815E-01,.4756236E-01,.5128126E-01,.3248201E-01,&
     & .2650872E-02,.3883456E-01,.5346450E-01,.5779235E-01,.3685837E-01,&
     & .2883010E-02,.4296394E-01,.5947166E-01,.6446457E-01,.4143251E-01,&
     & .1797225E-02,.2566144E-01,.3516915E-01,.3791294E-01,.2356973E-01,&
     & .1942471E-02,.2967546E-01,.4069640E-01,.4388416E-01,.2748224E-01,&
     & .2110732E-02,.3378859E-01,.4645705E-01,.5014523E-01,.3164309E-01,&
     & .2305322E-02,.3794341E-01,.5237303E-01,.5666115E-01,.3601691E-01,&
     & .2533979E-02,.4209991E-01,.5839699E-01,.6333729E-01,.4058728E-01,&
     & .1528499E-02,.2474850E-01,.3408418E-01,.3680675E-01,.2276935E-01,&
     & .1662648E-02,.2877135E-01,.3960487E-01,.4276395E-01,.2666990E-01,&
     & .1822270E-02,.3290004E-01,.4536537E-01,.4901380E-01,.3081993E-01,&
     & .2010407E-02,.3707876E-01,.5128684E-01,.5551957E-01,.3518382E-01,&
     & .2236121E-02,.4125795E-01,.5732398E-01,.6219988E-01,.3974976E-01,&
     & .1301157E-02,.2390332E-01,.3306486E-01,.3576088E-01,.2202215E-01,&
     & .1425753E-02,.2792846E-01,.3857359E-01,.4169850E-01,.2590697E-01,&
     & .1577002E-02,.3206803E-01,.4432843E-01,.4793360E-01,.3004339E-01,&
     & .1759961E-02,.3626411E-01,.5025187E-01,.5442732E-01,.3439556E-01,&
     & .1982828E-02,.4046418E-01,.5629663E-01,.6110344E-01,.3895158E-01,&
     & .1098183E-02,.2271194E-01,.3154937E-01,.3417342E-01,.2093772E-01,&
     & .1211979E-02,.2671363E-01,.3701515E-01,.4005573E-01,.2477551E-01,&
     & .1352163E-02,.3085276E-01,.4273410E-01,.4624396E-01,.2887103E-01,&
     & .1526376E-02,.3505659E-01,.4864130E-01,.5269139E-01,.3318689E-01,&
     & .1741576E-02,.3927458E-01,.5467901E-01,.5935019E-01,.3770824E-01/

      data   absb(701:875, 9)   /                                       &
     & .9256636E-03,.2151927E-01,.3001800E-01,.3256386E-01,.1984971E-01,&
     & .1029330E-02,.2548720E-01,.3542880E-01,.3837882E-01,.2363194E-01,&
     & .1157935E-02,.2961670E-01,.4110275E-01,.4450903E-01,.2767804E-01,&
     & .1322566E-02,.3382477E-01,.4698627E-01,.5090557E-01,.3195340E-01,&
     & .1529747E-02,.3805510E-01,.5300873E-01,.5753600E-01,.3643451E-01,&
     & .7800682E-03,.2037045E-01,.2853508E-01,.3100101E-01,.1880237E-01,&
     & .8728720E-03,.2429814E-01,.3387777E-01,.3673821E-01,.2252410E-01,&
     & .9913989E-03,.2840935E-01,.3950510E-01,.4280696E-01,.2651603E-01,&
     & .1147051E-02,.3261525E-01,.4535763E-01,.4915180E-01,.3074712E-01,&
     & .1345265E-02,.3685628E-01,.5135640E-01,.5573969E-01,.3518635E-01,&
     & .6520564E-03,.1905366E-01,.2681157E-01,.2917864E-01,.1759490E-01,&
     & .7355199E-03,.2291988E-01,.3206051E-01,.3480883E-01,.2123646E-01,&
     & .8430968E-03,.2699435E-01,.3762006E-01,.4079097E-01,.2515653E-01,&
     & .9861085E-03,.3118915E-01,.4342129E-01,.4706516E-01,.2932868E-01,&
     & .1172376E-02,.3543571E-01,.4939030E-01,.5358864E-01,.3371077E-01,&
     & .5429238E-03,.1769119E-01,.2501427E-01,.2727445E-01,.1634645E-01,&
     & .6168245E-03,.2147865E-01,.3015275E-01,.3277935E-01,.1989356E-01,&
     & .7121998E-03,.2550313E-01,.3563092E-01,.3866373E-01,.2373486E-01,&
     & .8424551E-03,.2967789E-01,.4136324E-01,.4485540E-01,.2783187E-01,&
     & .1016095E-02,.3392096E-01,.4729347E-01,.5130218E-01,.3215264E-01,&
     & .4520417E-03,.1638027E-01,.2327058E-01,.2542894E-01,.1514441E-01,&
     & .5155432E-03,.2007947E-01,.2829736E-01,.3080698E-01,.1859499E-01,&
     & .6014451E-03,.2404182E-01,.3368120E-01,.3657952E-01,.2235065E-01,&
     & .7188950E-03,.2818500E-01,.3934025E-01,.4268031E-01,.2636784E-01,&
     & .8790319E-03,.3241918E-01,.4522294E-01,.4905072E-01,.3062540E-01,&
     & .3739048E-03,.1504727E-01,.2147687E-01,.2353542E-01,.1391939E-01,&
     & .4299766E-03,.1864024E-01,.2638349E-01,.2876868E-01,.1726321E-01,&
     & .5043505E-03,.2252541E-01,.3165039E-01,.3441216E-01,.2091919E-01,&
     & .6094823E-03,.2662193E-01,.3722724E-01,.4040986E-01,.2485188E-01,&
     & .7547889E-03,.3083607E-01,.4304298E-01,.4669398E-01,.2903395E-01,&
     & .3059529E-03,.1362998E-01,.1954590E-01,.2150308E-01,.1261761E-01,&
     & .3544511E-03,.1709153E-01,.2431072E-01,.2656287E-01,.1583432E-01,&
     & .4182967E-03,.2087582E-01,.2943979E-01,.3205269E-01,.1937141E-01,&
     & .5101848E-03,.2490407E-01,.3491246E-01,.3792397E-01,.2320561E-01,&
     & .6398861E-03,.2908833E-01,.4064101E-01,.4410450E-01,.2729138E-01/

      data   absb(876:1050, 9)   /                                      &
     & .2498082E-03,.1228614E-01,.1769311E-01,.1955495E-01,.1137919E-01,&
     & .2913249E-03,.1560283E-01,.2230066E-01,.2443072E-01,.1445974E-01,&
     & .3471061E-03,.1927312E-01,.2729375E-01,.2976141E-01,.1787798E-01,&
     & .4260536E-03,.2321801E-01,.3264138E-01,.3549044E-01,.2160039E-01,&
     & .5402840E-03,.2735520E-01,.3827674E-01,.4155616E-01,.2559086E-01,&
     & .2032306E-03,.1100929E-01,.1592358E-01,.1768796E-01,.1020153E-01,&
     & .2392377E-03,.1418860E-01,.2036714E-01,.2238632E-01,.1314703E-01,&
     & .2862724E-03,.1772046E-01,.2521008E-01,.2753806E-01,.1643822E-01,&
     & .3532839E-03,.2156941E-01,.3041971E-01,.3311704E-01,.2004337E-01,&
     & .4535630E-03,.2564413E-01,.3595639E-01,.3905830E-01,.2393631E-01,&
     & .1658721E-03,.9881414E-02,.1435034E-01,.1601976E-01,.9156258E-02,&
     & .1953876E-03,.1291102E-01,.1860859E-01,.2053398E-01,.1196964E-01,&
     & .2372144E-03,.1632300E-01,.2331887E-01,.2552322E-01,.1513690E-01,&
     & .2956183E-03,.2005853E-01,.2839017E-01,.3094782E-01,.1862793E-01,&
     & .3822967E-03,.2406104E-01,.3381785E-01,.3676346E-01,.2242186E-01,&
     & .1347727E-03,.8839179E-02,.1288890E-01,.1446932E-01,.8192478E-02,&
     & .1593422E-03,.1172817E-01,.1696668E-01,.1880252E-01,.1087547E-01,&
     & .1948108E-03,.1500685E-01,.2152813E-01,.2362440E-01,.1391796E-01,&
     & .2456981E-03,.1863185E-01,.2647287E-01,.2889888E-01,.1729864E-01,&
     & .3210495E-03,.2255415E-01,.3178001E-01,.3458170E-01,.2098803E-01,&
     & .1090472E-03,.7861512E-02,.1150652E-01,.1299630E-01,.7284963E-02,&
     & .1294974E-03,.1060140E-01,.1539628E-01,.1714133E-01,.9834191E-02,&
     & .1592049E-03,.1374264E-01,.1979296E-01,.2179392E-01,.1275057E-01,&
     & .2045251E-03,.1726001E-01,.2461884E-01,.2691705E-01,.1601660E-01,&
     & .2700821E-03,.2107733E-01,.2978897E-01,.3245179E-01,.1959460E-01,&
     & .8802434E-04,.6944882E-02,.1020032E-01,.1159221E-01,.6430862E-02,&
     & .1046855E-03,.9527594E-02,.1389317E-01,.1554646E-01,.8842774E-02,&
     & .1292946E-03,.1253134E-01,.1811521E-01,.2002416E-01,.1163053E-01,&
     & .1677549E-03,.1591776E-01,.2279809E-01,.2497825E-01,.1477209E-01,&
     & .2257215E-03,.1963067E-01,.2784013E-01,.3036876E-01,.1824002E-01,&
     & .7112381E-04,.6158932E-02,.9074677E-02,.1037208E-01,.5697312E-02,&
     & .8497772E-04,.8594221E-02,.1257729E-01,.1414858E-01,.7977017E-02,&
     & .1052109E-03,.1146594E-01,.1663179E-01,.1845837E-01,.1064379E-01,&
     & .1377962E-03,.1472932E-01,.2117504E-01,.2325811E-01,.1367082E-01,&
     & .1885595E-03,.1833924E-01,.2610150E-01,.2850850E-01,.1703634E-01/

      data   absb(1051:1175, 9)   /                                     &
     & .5743054E-04,.5454299E-02,.8062707E-02,.9263120E-02,.5037907E-02,&
     & .6891997E-04,.7748836E-02,.1137635E-01,.1286665E-01,.7190454E-02,&
     & .8575160E-04,.1048793E-01,.1526380E-01,.1700893E-01,.9738840E-02,&
     & .1130459E-03,.1362883E-01,.1965917E-01,.2165939E-01,.1265333E-01,&
     & .1594488E-03,.1714474E-01,.2448136E-01,.2677687E-01,.1591770E-01,&
     & .4626830E-04,.4800265E-02,.7116803E-02,.8217649E-02,.4424823E-02,&
     & .5574968E-04,.6951784E-02,.1023739E-01,.1164073E-01,.6446800E-02,&
     & .6940184E-04,.9552514E-02,.1395187E-01,.1561652E-01,.8874544E-02,&
     & .9239592E-04,.1257225E-01,.1819349E-01,.2011229E-01,.1167577E-01,&
     & .1322365E-03,.1597315E-01,.2289106E-01,.2508204E-01,.1483008E-01,&
     & .3713644E-04,.4193801E-02,.6237852E-02,.7236320E-02,.3854523E-02,&
     & .4495045E-04,.6202948E-02,.9160942E-02,.1047397E-01,.5746268E-02,&
     & .5606539E-04,.8661882E-02,.1269376E-01,.1427915E-01,.8047359E-02,&
     & .7499774E-04,.1155560E-01,.1677574E-01,.1861487E-01,.1073338E-01,&
     & .1090623E-03,.1483830E-01,.2133895E-01,.2343538E-01,.1377716E-01,&
     & .2983624E-04,.3660809E-02,.5463957E-02,.6366210E-02,.3353712E-02,&
     & .3622972E-04,.5534921E-02,.8199583E-02,.9421109E-02,.5120953E-02,&
     & .4528199E-04,.7860566E-02,.1155366E-01,.1306274E-01,.7301227E-02,&
     & .6078597E-04,.1062904E-01,.1547773E-01,.1723998E-01,.9874672E-02,&
     & .8960328E-04,.1379622E-01,.1990262E-01,.2191965E-01,.1281258E-01,&
     & .2476815E-04,.3454037E-02,.5166124E-02,.6030808E-02,.3159940E-02,&
     & .3026086E-04,.5275928E-02,.7826736E-02,.9011759E-02,.4878738E-02,&
     & .3827593E-04,.7548106E-02,.1110916E-01,.1258647E-01,.7009912E-02,&
     & .5239001E-04,.1026550E-01,.1496950E-01,.1670101E-01,.9539209E-02,&
     & .7936022E-04,.1338634E-01,.1933796E-01,.2132326E-01,.1243426E-01/

      data   absb(  1:175,10)   /                                       &
     & .3241099E+01,.2987317E+01,.2934217E+01,.2979591E+01,.3249070E+01,&
     & .3234075E+01,.2980277E+01,.2928977E+01,.2975887E+01,.3247797E+01,&
     & .3225854E+01,.2969842E+01,.2920752E+01,.2969916E+01,.3244885E+01,&
     & .3216237E+01,.2956861E+01,.2909451E+01,.2961989E+01,.3239941E+01,&
     & .3205570E+01,.2942860E+01,.2895750E+01,.2952083E+01,.3233269E+01,&
     & .3294407E+01,.3067418E+01,.3026193E+01,.3071528E+01,.3320951E+01,&
     & .3287555E+01,.3058712E+01,.3019273E+01,.3067009E+01,.3319914E+01,&
     & .3279570E+01,.3047161E+01,.3009370E+01,.3060158E+01,.3316506E+01,&
     & .3270203E+01,.3034142E+01,.2996595E+01,.3051009E+01,.3310913E+01,&
     & .3259714E+01,.3019806E+01,.2981647E+01,.3039634E+01,.3303820E+01,&
     & .3341126E+01,.3138576E+01,.3108484E+01,.3154777E+01,.3385204E+01,&
     & .3334442E+01,.3128468E+01,.3100116E+01,.3149324E+01,.3383891E+01,&
     & .3326558E+01,.3116752E+01,.3088845E+01,.3141443E+01,.3379937E+01,&
     & .3317437E+01,.3103447E+01,.3075001E+01,.3130952E+01,.3374048E+01,&
     & .3307931E+01,.3088783E+01,.3059530E+01,.3118062E+01,.3366502E+01,&
     & .3381819E+01,.3201482E+01,.3181519E+01,.3229413E+01,.3441758E+01,&
     & .3375218E+01,.3191080E+01,.3171998E+01,.3223235E+01,.3440199E+01,&
     & .3367503E+01,.3178917E+01,.3159508E+01,.3214061E+01,.3435864E+01,&
     & .3359300E+01,.3165234E+01,.3145138E+01,.3202140E+01,.3429679E+01,&
     & .3350904E+01,.3150191E+01,.3129217E+01,.3187795E+01,.3421572E+01,&
     & .3416967E+01,.3257138E+01,.3245938E+01,.3295538E+01,.3491078E+01,&
     & .3410526E+01,.3246460E+01,.3235248E+01,.3288412E+01,.3489256E+01,&
     & .3403411E+01,.3233801E+01,.3222173E+01,.3278081E+01,.3484896E+01,&
     & .3396503E+01,.3220051E+01,.3207563E+01,.3264988E+01,.3478422E+01,&
     & .3388687E+01,.3204490E+01,.3190965E+01,.3249372E+01,.3469667E+01,&
     & .3447347E+01,.3306390E+01,.3302358E+01,.3353515E+01,.3533939E+01,&
     & .3441677E+01,.3295397E+01,.3291151E+01,.3345399E+01,.3532046E+01,&
     & .3435433E+01,.3282539E+01,.3277801E+01,.3334036E+01,.3527652E+01,&
     & .3428932E+01,.3268231E+01,.3262441E+01,.3319876E+01,.3520689E+01,&
     & .3421802E+01,.3252582E+01,.3245243E+01,.3303411E+01,.3511826E+01,&
     & .3473639E+01,.3349610E+01,.3351573E+01,.3403647E+01,.3571220E+01,&
     & .3468683E+01,.3338245E+01,.3340115E+01,.3394675E+01,.3569195E+01,&
     & .3463191E+01,.3325117E+01,.3326306E+01,.3382460E+01,.3564603E+01,&
     & .3457226E+01,.3310536E+01,.3310441E+01,.3367641E+01,.3557528E+01,&
     & .3450573E+01,.3294967E+01,.3292594E+01,.3350498E+01,.3548320E+01/

      data   absb(176:350,10)   /                                       &
     & .3496427E+01,.3387213E+01,.3394307E+01,.3446696E+01,.3603551E+01,&
     & .3492015E+01,.3375447E+01,.3382527E+01,.3437054E+01,.3601524E+01,&
     & .3487054E+01,.3362003E+01,.3368240E+01,.3424123E+01,.3596558E+01,&
     & .3481600E+01,.3347475E+01,.3351850E+01,.3408660E+01,.3589083E+01,&
     & .3475345E+01,.3331832E+01,.3333426E+01,.3390953E+01,.3579691E+01,&
     & .3515729E+01,.3419608E+01,.3431233E+01,.3483605E+01,.3631724E+01,&
     & .3512042E+01,.3407688E+01,.3419052E+01,.3473258E+01,.3629302E+01,&
     & .3507568E+01,.3394146E+01,.3404283E+01,.3459906E+01,.3623996E+01,&
     & .3502487E+01,.3379596E+01,.3387427E+01,.3443990E+01,.3616232E+01,&
     & .3496406E+01,.3363737E+01,.3368380E+01,.3425639E+01,.3606532E+01,&
     & .3532277E+01,.3446949E+01,.3462235E+01,.3514489E+01,.3655808E+01,&
     & .3528770E+01,.3434733E+01,.3449373E+01,.3503513E+01,.3652945E+01,&
     & .3524634E+01,.3421100E+01,.3434125E+01,.3489643E+01,.3647282E+01,&
     & .3519795E+01,.3406260E+01,.3416576E+01,.3472980E+01,.3639037E+01,&
     & .3514104E+01,.3390336E+01,.3397008E+01,.3453854E+01,.3628856E+01,&
     & .3546078E+01,.3470188E+01,.3488411E+01,.3540723E+01,.3676433E+01,&
     & .3542944E+01,.3457842E+01,.3475086E+01,.3529136E+01,.3673056E+01,&
     & .3538969E+01,.3443995E+01,.3459112E+01,.3514554E+01,.3666861E+01,&
     & .3534492E+01,.3429012E+01,.3441008E+01,.3497202E+01,.3658321E+01,&
     & .3528936E+01,.3412721E+01,.3420721E+01,.3477224E+01,.3647530E+01,&
     & .3557812E+01,.3489957E+01,.3510431E+01,.3562900E+01,.3693951E+01,&
     & .3554915E+01,.3477501E+01,.3496575E+01,.3550650E+01,.3690213E+01,&
     & .3551290E+01,.3463442E+01,.3480042E+01,.3535257E+01,.3683497E+01,&
     & .3546841E+01,.3448056E+01,.3461256E+01,.3517075E+01,.3674340E+01,&
     & .3541285E+01,.3431269E+01,.3440276E+01,.3496294E+01,.3663228E+01,&
     & .3567647E+01,.3506550E+01,.3528762E+01,.3581376E+01,.3708814E+01,&
     & .3564997E+01,.3493846E+01,.3514220E+01,.3568244E+01,.3704420E+01,&
     & .3561496E+01,.3479527E+01,.3497088E+01,.3552055E+01,.3697250E+01,&
     & .3557192E+01,.3463748E+01,.3477590E+01,.3533010E+01,.3687653E+01,&
     & .3551616E+01,.3446545E+01,.3455968E+01,.3511360E+01,.3675961E+01,&
     & .3576050E+01,.3520359E+01,.3543745E+01,.3596477E+01,.3721344E+01,&
     & .3573439E+01,.3507280E+01,.3528498E+01,.3582455E+01,.3716375E+01,&
     & .3570110E+01,.3492608E+01,.3510656E+01,.3565258E+01,.3708511E+01,&
     & .3565744E+01,.3476407E+01,.3490353E+01,.3545222E+01,.3698329E+01,&
     & .3559803E+01,.3458419E+01,.3467812E+01,.3522516E+01,.3686100E+01/

      data   absb(351:525,10)   /                                       &
     & .3582887E+01,.3531577E+01,.3555722E+01,.3608504E+01,.3731686E+01,&
     & .3580475E+01,.3518245E+01,.3539855E+01,.3593592E+01,.3726109E+01,&
     & .3577189E+01,.3503133E+01,.3521212E+01,.3575393E+01,.3717607E+01,&
     & .3572639E+01,.3486267E+01,.3500027E+01,.3554276E+01,.3706744E+01,&
     & .3566732E+01,.3467823E+01,.3476736E+01,.3530687E+01,.3693895E+01,&
     & .3588948E+01,.3540932E+01,.3565482E+01,.3618178E+01,.3740395E+01,&
     & .3586372E+01,.3527032E+01,.3548694E+01,.3602144E+01,.3734037E+01,&
     & .3582904E+01,.3511279E+01,.3529122E+01,.3582894E+01,.3724850E+01,&
     & .3578253E+01,.3493847E+01,.3507118E+01,.3560863E+01,.3713367E+01,&
     & .3571896E+01,.3474633E+01,.3482868E+01,.3536238E+01,.3699809E+01,&
     & .3593753E+01,.3548111E+01,.3572749E+01,.3625342E+01,.3747338E+01,&
     & .3591161E+01,.3533619E+01,.3554981E+01,.3608138E+01,.3740209E+01,&
     & .3587644E+01,.3517376E+01,.3534583E+01,.3587946E+01,.3730305E+01,&
     & .3582824E+01,.3499345E+01,.3511738E+01,.3564886E+01,.3718106E+01,&
     & .3576090E+01,.3479444E+01,.3486584E+01,.3539307E+01,.3703918E+01,&
     & .3597761E+01,.3553599E+01,.3577948E+01,.3630395E+01,.3752803E+01,&
     & .3595135E+01,.3538555E+01,.3559293E+01,.3612190E+01,.3744937E+01,&
     & .3591361E+01,.3521618E+01,.3537925E+01,.3590860E+01,.3734257E+01,&
     & .3586342E+01,.3502980E+01,.3514239E+01,.3566848E+01,.3721332E+01,&
     & .3579098E+01,.3482302E+01,.3488295E+01,.3540497E+01,.3706615E+01,&
     & .3601000E+01,.3557421E+01,.3581302E+01,.3633532E+01,.3756941E+01,&
     & .3598425E+01,.3541845E+01,.3561786E+01,.3614297E+01,.3748264E+01,&
     & .3594258E+01,.3524164E+01,.3539472E+01,.3591996E+01,.3736861E+01,&
     & .3588813E+01,.3504757E+01,.3514868E+01,.3567024E+01,.3723189E+01,&
     & .3581159E+01,.3483405E+01,.3488066E+01,.3539801E+01,.3707789E+01,&
     & .3603721E+01,.3559860E+01,.3582917E+01,.3634951E+01,.3759790E+01,&
     & .3600695E+01,.3543471E+01,.3562334E+01,.3614594E+01,.3750203E+01,&
     & .3596558E+01,.3525354E+01,.3539343E+01,.3591476E+01,.3738184E+01,&
     & .3590489E+01,.3505086E+01,.3513885E+01,.3565600E+01,.3723853E+01,&
     & .3582281E+01,.3482974E+01,.3486236E+01,.3537560E+01,.3707796E+01,&
     & .3606012E+01,.3561147E+01,.3583252E+01,.3635062E+01,.3761716E+01,&
     & .3602690E+01,.3544023E+01,.3561757E+01,.3613695E+01,.3751349E+01,&
     & .3598158E+01,.3525175E+01,.3537873E+01,.3589589E+01,.3738443E+01,&
     & .3591521E+01,.3504190E+01,.3511600E+01,.3562948E+01,.3723509E+01,&
     & .3582782E+01,.3481406E+01,.3483237E+01,.3534221E+01,.3706854E+01/

      data   absb(526:700,10)   /                                       &
     & .3607593E+01,.3561716E+01,.3582936E+01,.3634631E+01,.3762977E+01,&
     & .3604207E+01,.3544117E+01,.3560767E+01,.3612484E+01,.3751904E+01,&
     & .3599186E+01,.3524501E+01,.3536083E+01,.3587617E+01,.3738408E+01,&
     & .3592335E+01,.3503119E+01,.3509231E+01,.3560224E+01,.3722888E+01,&
     & .3583023E+01,.3479645E+01,.3480307E+01,.3530962E+01,.3705868E+01,&
     & .3609388E+01,.3563205E+01,.3583835E+01,.3635279E+01,.3764323E+01,&
     & .3605668E+01,.3545073E+01,.3561130E+01,.3612628E+01,.3752963E+01,&
     & .3600435E+01,.3525079E+01,.3535962E+01,.3587210E+01,.3739092E+01,&
     & .3593253E+01,.3503218E+01,.3508581E+01,.3559430E+01,.3723196E+01,&
     & .3583674E+01,.3479411E+01,.3479316E+01,.3529774E+01,.3705733E+01,&
     & .3610877E+01,.3565753E+01,.3586334E+01,.3637630E+01,.3766564E+01,&
     & .3607388E+01,.3547593E+01,.3563510E+01,.3614777E+01,.3754879E+01,&
     & .3601974E+01,.3527292E+01,.3538093E+01,.3589236E+01,.3740900E+01,&
     & .3594896E+01,.3505409E+01,.3510693E+01,.3561326E+01,.3724918E+01,&
     & .3585207E+01,.3481394E+01,.3481186E+01,.3531504E+01,.3707454E+01,&
     & .3612619E+01,.3570439E+01,.3591639E+01,.3642767E+01,.3769831E+01,&
     & .3609315E+01,.3552400E+01,.3569038E+01,.3620211E+01,.3758468E+01,&
     & .3604385E+01,.3532422E+01,.3543881E+01,.3594866E+01,.3744636E+01,&
     & .3597500E+01,.3510559E+01,.3516551E+01,.3567108E+01,.3728771E+01,&
     & .3588141E+01,.3486709E+01,.3487212E+01,.3537489E+01,.3711452E+01,&
     & .3614294E+01,.3574997E+01,.3596711E+01,.3647701E+01,.3772830E+01,&
     & .3611060E+01,.3557018E+01,.3574287E+01,.3625347E+01,.3761707E+01,&
     & .3606301E+01,.3537069E+01,.3549260E+01,.3600210E+01,.3747990E+01,&
     & .3599986E+01,.3515632E+01,.3522343E+01,.3572792E+01,.3732484E+01,&
     & .3590746E+01,.3491810E+01,.3493076E+01,.3543289E+01,.3715323E+01,&
     & .3615385E+01,.3578875E+01,.3601241E+01,.3652114E+01,.3775522E+01,&
     & .3612653E+01,.3561267E+01,.3579150E+01,.3630121E+01,.3764687E+01,&
     & .3608225E+01,.3541594E+01,.3554508E+01,.3605343E+01,.3751304E+01,&
     & .3601890E+01,.3520037E+01,.3527598E+01,.3578113E+01,.3735943E+01,&
     & .3593282E+01,.3496658E+01,.3498659E+01,.3548727E+01,.3718739E+01,&
     & .3616729E+01,.3584234E+01,.3607568E+01,.3658316E+01,.3778888E+01,&
     & .3614336E+01,.3567065E+01,.3586083E+01,.3636983E+01,.3768550E+01,&
     & .3610281E+01,.3547775E+01,.3561945E+01,.3612790E+01,.3755757E+01,&
     & .3604549E+01,.3526723E+01,.3535518E+01,.3586033E+01,.3740710E+01,&
     & .3596403E+01,.3503693E+01,.3506997E+01,.3557144E+01,.3724043E+01/

      data   absb(701:875,10)   /                                       &
     & .3617849E+01,.3589442E+01,.3613858E+01,.3664479E+01,.3782091E+01,&
     & .3615818E+01,.3572805E+01,.3592899E+01,.3643677E+01,.3772222E+01,&
     & .3612124E+01,.3553880E+01,.3569311E+01,.3620106E+01,.3759976E+01,&
     & .3606887E+01,.3533209E+01,.3543433E+01,.3593996E+01,.3745475E+01,&
     & .3599450E+01,.3510687E+01,.3515389E+01,.3565549E+01,.3729132E+01,&
     & .3618859E+01,.3594390E+01,.3619692E+01,.3670127E+01,.3784822E+01,&
     & .3617031E+01,.3578150E+01,.3599460E+01,.3650217E+01,.3775829E+01,&
     & .3613773E+01,.3559780E+01,.3576444E+01,.3627182E+01,.3764034E+01,&
     & .3609186E+01,.3539677E+01,.3551204E+01,.3601833E+01,.3750137E+01,&
     & .3602259E+01,.3517530E+01,.3523661E+01,.3573794E+01,.3734189E+01,&
     & .3619642E+01,.3599861E+01,.3626247E+01,.3676465E+01,.3787596E+01,&
     & .3618160E+01,.3584226E+01,.3606851E+01,.3657509E+01,.3779539E+01,&
     & .3615477E+01,.3566520E+01,.3584677E+01,.3635460E+01,.3768627E+01,&
     & .3611247E+01,.3546802E+01,.3559989E+01,.3610646E+01,.3755225E+01,&
     & .3605074E+01,.3525201E+01,.3533059E+01,.3583351E+01,.3739819E+01,&
     & .3620312E+01,.3605283E+01,.3632856E+01,.3682831E+01,.3790274E+01,&
     & .3619186E+01,.3590447E+01,.3614470E+01,.3665009E+01,.3783275E+01,&
     & .3616882E+01,.3573318E+01,.3593065E+01,.3643837E+01,.3773166E+01,&
     & .3613451E+01,.3554344E+01,.3569198E+01,.3619779E+01,.3760340E+01,&
     & .3607869E+01,.3533334E+01,.3542978E+01,.3593388E+01,.3745727E+01,&
     & .3620861E+01,.3610352E+01,.3639014E+01,.3688623E+01,.3792389E+01,&
     & .3619907E+01,.3596190E+01,.3621534E+01,.3671961E+01,.3786487E+01,&
     & .3618144E+01,.3579813E+01,.3601034E+01,.3651766E+01,.3777268E+01,&
     & .3615214E+01,.3561501E+01,.3578086E+01,.3628673E+01,.3765343E+01,&
     & .3610346E+01,.3541102E+01,.3552515E+01,.3602980E+01,.3751243E+01,&
     & .3621171E+01,.3615299E+01,.3645122E+01,.3694327E+01,.3794232E+01,&
     & .3620700E+01,.3602063E+01,.3628647E+01,.3678772E+01,.3789342E+01,&
     & .3619172E+01,.3586367E+01,.3609119E+01,.3659731E+01,.3781216E+01,&
     & .3616635E+01,.3568642E+01,.3586973E+01,.3637668E+01,.3770251E+01,&
     & .3612602E+01,.3549063E+01,.3562386E+01,.3612958E+01,.3756942E+01,&
     & .3621585E+01,.3620432E+01,.3651384E+01,.3699855E+01,.3795401E+01,&
     & .3621229E+01,.3608077E+01,.3635997E+01,.3685814E+01,.3792009E+01,&
     & .3620075E+01,.3593269E+01,.3617681E+01,.3668163E+01,.3785187E+01,&
     & .3618192E+01,.3576439E+01,.3596584E+01,.3647184E+01,.3775165E+01,&
     & .3614853E+01,.3557603E+01,.3572939E+01,.3623466E+01,.3762716E+01/

      data   absb(876:1050,10)   /                                      &
     & .3621531E+01,.3624874E+01,.3656941E+01,.3704801E+01,.3796091E+01,&
     & .3621578E+01,.3613665E+01,.3642948E+01,.3692378E+01,.3794280E+01,&
     & .3620925E+01,.3599898E+01,.3625750E+01,.3676030E+01,.3788596E+01,&
     & .3619395E+01,.3583738E+01,.3605662E+01,.3656221E+01,.3779663E+01,&
     & .3616452E+01,.3565615E+01,.3582989E+01,.3633553E+01,.3768255E+01,&
     & .3621699E+01,.3629071E+01,.3661985E+01,.3709070E+01,.3796100E+01,&
     & .3621985E+01,.3618898E+01,.3649290E+01,.3698155E+01,.3795661E+01,&
     & .3621415E+01,.3605926E+01,.3633265E+01,.3683249E+01,.3791501E+01,&
     & .3620359E+01,.3590761E+01,.3614336E+01,.3664785E+01,.3783789E+01,&
     & .3618117E+01,.3573457E+01,.3592656E+01,.3643288E+01,.3773301E+01,&
     & .3621904E+01,.3632715E+01,.3666350E+01,.3712572E+01,.3795707E+01,&
     & .3622024E+01,.3623236E+01,.3654773E+01,.3703040E+01,.3796613E+01,&
     & .3621869E+01,.3611272E+01,.3639856E+01,.3689474E+01,.3793663E+01,&
     & .3621052E+01,.3597007E+01,.3622051E+01,.3672345E+01,.3787216E+01,&
     & .3619289E+01,.3580478E+01,.3601357E+01,.3651927E+01,.3777750E+01,&
     & .3621852E+01,.3635792E+01,.3669888E+01,.3715191E+01,.3794554E+01,&
     & .3622079E+01,.3627138E+01,.3659507E+01,.3707091E+01,.3796772E+01,&
     & .3622144E+01,.3616154E+01,.3645824E+01,.3695017E+01,.3795270E+01,&
     & .3621572E+01,.3602644E+01,.3629021E+01,.3679124E+01,.3790058E+01,&
     & .3620273E+01,.3586904E+01,.3609375E+01,.3659834E+01,.3781548E+01,&
     & .3621524E+01,.3638153E+01,.3672641E+01,.3717019E+01,.3792489E+01,&
     & .3622011E+01,.3630710E+01,.3663819E+01,.3710696E+01,.3796596E+01,&
     & .3622140E+01,.3620549E+01,.3651350E+01,.3700088E+01,.3796517E+01,&
     & .3621964E+01,.3607992E+01,.3635644E+01,.3685439E+01,.3792448E+01,&
     & .3620930E+01,.3593051E+01,.3617045E+01,.3667379E+01,.3785155E+01,&
     & .3621285E+01,.3640375E+01,.3675144E+01,.3718598E+01,.3790284E+01,&
     & .3621997E+01,.3633935E+01,.3667680E+01,.3713679E+01,.3795786E+01,&
     & .3622336E+01,.3624705E+01,.3656432E+01,.3704538E+01,.3797068E+01,&
     & .3622150E+01,.3612954E+01,.3641755E+01,.3691352E+01,.3794423E+01,&
     & .3621542E+01,.3598955E+01,.3624346E+01,.3674512E+01,.3788265E+01,&
     & .3620995E+01,.3641953E+01,.3676700E+01,.3719069E+01,.3787249E+01,&
     & .3621905E+01,.3636516E+01,.3670740E+01,.3715933E+01,.3794540E+01,&
     & .3622165E+01,.3628082E+01,.3660699E+01,.3708147E+01,.3797220E+01,&
     & .3622324E+01,.3617255E+01,.3647220E+01,.3696381E+01,.3795917E+01,&
     & .3621972E+01,.3604029E+01,.3630657E+01,.3680650E+01,.3790740E+01/

      data   absb(1051:1175,10)   /                                     &
     & .3620526E+01,.3643008E+01,.3677709E+01,.3719079E+01,.3783909E+01,&
     & .3621663E+01,.3638625E+01,.3673147E+01,.3717490E+01,.3792683E+01,&
     & .3622181E+01,.3631146E+01,.3664388E+01,.3711198E+01,.3796901E+01,&
     & .3622567E+01,.3621247E+01,.3652054E+01,.3700619E+01,.3796719E+01,&
     & .3622221E+01,.3608539E+01,.3636273E+01,.3686109E+01,.3792815E+01,&
     & .3620197E+01,.3643890E+01,.3678333E+01,.3718665E+01,.3780167E+01,&
     & .3621325E+01,.3640368E+01,.3675148E+01,.3718577E+01,.3790537E+01,&
     & .3622155E+01,.3633956E+01,.3667672E+01,.3713768E+01,.3796050E+01,&
     & .3622497E+01,.3624766E+01,.3656479E+01,.3704595E+01,.3797402E+01,&
     & .3622419E+01,.3612956E+01,.3641709E+01,.3691233E+01,.3794545E+01,&
     & .3619809E+01,.3644465E+01,.3678494E+01,.3717752E+01,.3775862E+01,&
     & .3621149E+01,.3642028E+01,.3676823E+01,.3719299E+01,.3787818E+01,&
     & .3621940E+01,.3636328E+01,.3670542E+01,.3715827E+01,.3794751E+01,&
     & .3622399E+01,.3627949E+01,.3660463E+01,.3707973E+01,.3797438E+01,&
     & .3622573E+01,.3617046E+01,.3646813E+01,.3695952E+01,.3795817E+01,&
     & .3619235E+01,.3644575E+01,.3678035E+01,.3716081E+01,.3771129E+01,&
     & .3620784E+01,.3643128E+01,.3677789E+01,.3719247E+01,.3784648E+01,&
     & .3621727E+01,.3638446E+01,.3672940E+01,.3717422E+01,.3793308E+01,&
     & .3622318E+01,.3630870E+01,.3663967E+01,.3710850E+01,.3797063E+01,&
     & .3622631E+01,.3620709E+01,.3651341E+01,.3700062E+01,.3796727E+01,&
     & .3618949E+01,.3644500E+01,.3677804E+01,.3715402E+01,.3769243E+01,&
     & .3620600E+01,.3643472E+01,.3678108E+01,.3719204E+01,.3783320E+01,&
     & .3621596E+01,.3639124E+01,.3673749E+01,.3717908E+01,.3792537E+01,&
     & .3622327E+01,.3631967E+01,.3665327E+01,.3711976E+01,.3796952E+01,&
     & .3622725E+01,.3622155E+01,.3653104E+01,.3701619E+01,.3797027E+01/

!  ---
      data   forref(1:4,1:10)    /                                      &
     & .1100080E-06,.6309120E-06,.3631590E-05,.6168920E-05,.4297090E-05,&
     & .7891740E-05,.2174160E-04,.6393930E-04,.4362830E-04,.5262470E-04,&
     & .1163410E-03,.2056160E-03,.2156270E-03,.2345220E-03,.2804970E-03,&
     & .8386680E-03,.5292830E-03,.6208480E-03,.9355610E-03,.1712520E-02,&
     & .2122670E-02,.2185640E-02,.2222270E-02,.1996500E-02,.2911200E-02,&
     & .2811680E-02,.2595430E-02,.2101590E-02,.3162490E-02,.3106950E-02,&
     & .2795010E-02,.2080760E-02,.3591895E-02,.3399762E-02,.3018810E-02,&
     & .1801670E-02,.4302254E-02,.4229914E-02,.3954639E-02,.2291698E-02/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 10).

      data   selfref(1:10,1:10)  /                                      &
     & .1158870E-03,.9265370E-04,.7407830E-04,.5922700E-04,.4735300E-04,&
     & .3785960E-04,.3026940E-04,.2420100E-04,.1934910E-04,.1547000E-04,&
     & .4595570E-03,.3819620E-03,.3174690E-03,.2638660E-03,.2193130E-03,&
     & .1822830E-03,.1515050E-03,.1259240E-03,.1046620E-03,.8699040E-04,&
     & .1668210E-02,.1511030E-02,.1368660E-02,.1239700E-02,.1122900E-02,&
     & .1017100E-02,.9212660E-03,.8344630E-03,.7558390E-03,.6846230E-03,&
     & .4601750E-02,.4213720E-02,.3858420E-02,.3533070E-02,.3235160E-02,&
     & .2962360E-02,.2712570E-02,.2483850E-02,.2274400E-02,.2082620E-02,&
     & .1015890E-01,.9247420E-02,.8417720E-02,.7662470E-02,.6974970E-02,&
     & .6349170E-02,.5779510E-02,.5260960E-02,.4788930E-02,.4359260E-02,&
     & .3280430E-01,.3008530E-01,.2759170E-01,.2530480E-01,.2320750E-01,&
     & .2128390E-01,.1951980E-01,.1790200E-01,.1641820E-01,.1505740E-01,&
     & .4059360E-01,.3760320E-01,.3483310E-01,.3226710E-01,.2989010E-01,&
     & .2768830E-01,.2564860E-01,.2375910E-01,.2200890E-01,.2038760E-01,&
     & .4483620E-01,.4138110E-01,.3819230E-01,.3524920E-01,.3253290E-01,&
     & .3002590E-01,.2771210E-01,.2557660E-01,.2360560E-01,.2178660E-01,&
     & .4836264E-01,.4491932E-01,.4172126E-01,.3875097E-01,.3599212E-01,&
     & .3342979E-01,.3105000E-01,.2883953E-01,.2678655E-01,.2487982E-01,&
     & .6056851E-01,.5576215E-01,.5133769E-01,.4726472E-01,.4351530E-01,&
     & .4006365E-01,.3688611E-01,.3396089E-01,.3126793E-01,.2878878E-01/

!........................................!
      end module module_radsw_kgb21      !
!========================================!


!========================================!
      module module_radsw_kgb22          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 22:  7700-8050 cm-1 (low - h2o,o2; high - o2)              !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG22

!
      implicit none
!
      private
!
      integer,  public :: MSA22, MSB22, MSF22, MFR22
      parameter (MSA22=585, MSB22=235, MSF22=10, MFR22=3)

      real (kind=kind_phys), public ::           forref(MFR22,NG22),    &
     &       absa(MSA22,NG22), absb(MSB22,NG22), selfref(MSF22,NG22)

!  ---  rayleigh extinction coefficient at v = 8000 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 1.54e-08

!     the array absa(585,NG22) (ka(9,5,13,NG22)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 2, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .4811305E-05,.6129178E-05,.6975295E-05,.7459097E-05,.7681764E-05,&
     & .7669132E-05,.7386319E-05,.6757954E-05,.4773633E-05,.4852970E-05,&
     & .6482051E-05,.7514931E-05,.8141914E-05,.8481015E-05,.8536849E-05,&
     & .8278644E-05,.7661509E-05,.5661537E-05,.4887964E-05,.6857744E-05,&
     & .8089331E-05,.8885017E-05,.9332288E-05,.9453783E-05,.9249515E-05,&
     & .8659290E-05,.6666955E-05,.4916026E-05,.7226977E-05,.8662139E-05,&
     & .9602646E-05,.1014228E-04,.1032751E-04,.1017396E-04,.9601762E-05,&
     & .7631048E-05,.4939202E-05,.7613809E-05,.9263194E-05,.1033860E-04,&
     & .1098728E-04,.1125104E-04,.1116365E-04,.1062721E-04,.8702900E-05,&
     & .4106730E-05,.5324296E-05,.6058728E-05,.6475552E-05,.6675502E-05,&
     & .6660409E-05,.6407153E-05,.5864093E-05,.3937516E-05,.4142531E-05,&
     & .5645457E-05,.6543640E-05,.7093457E-05,.7392197E-05,.7432767E-05,&
     & .7202851E-05,.6679036E-05,.4695930E-05,.4172412E-05,.5983858E-05,&
     & .7068059E-05,.7761470E-05,.8155332E-05,.8259215E-05,.8079259E-05,&
     & .7583378E-05,.5569182E-05,.4197861E-05,.6325920E-05,.7598301E-05,&
     & .8416188E-05,.8900428E-05,.9074421E-05,.8942307E-05,.8472099E-05,&
     & .6440222E-05,.4216312E-05,.6670040E-05,.8121375E-05,.9056370E-05,&
     & .9638048E-05,.9875647E-05,.9796244E-05,.9352752E-05,.7315977E-05,&
     & .3467259E-05,.4530486E-05,.5130556E-05,.5469514E-05,.5627152E-05,&
     & .5606389E-05,.5378296E-05,.4903857E-05,.3080725E-05,.3499079E-05,&
     & .4815823E-05,.5557660E-05,.6015497E-05,.6256261E-05,.6280454E-05,&
     & .6078238E-05,.5616791E-05,.3711088E-05,.3526114E-05,.5110612E-05,&
     & .6012398E-05,.6587675E-05,.6901419E-05,.6980572E-05,.6815694E-05,&
     & .6375015E-05,.4396734E-05,.3547696E-05,.5422743E-05,.6499204E-05,&
     & .7185087E-05,.7588601E-05,.7739029E-05,.7620329E-05,.7212753E-05,&
     & .5186395E-05,.3563976E-05,.5729219E-05,.6960032E-05,.7748552E-05,&
     & .8240995E-05,.8448645E-05,.8371310E-05,.7991578E-05,.5921336E-05,&
     & .2903406E-05,.3807717E-05,.4284968E-05,.4550532E-05,.4671249E-05,&
     & .4645256E-05,.4435665E-05,.4014615E-05,.2368786E-05,.2932650E-05,&
     & .4056866E-05,.4656270E-05,.5027012E-05,.5216501E-05,.5228575E-05,&
     & .5042545E-05,.4630358E-05,.2886709E-05,.2956965E-05,.4316547E-05,&
     & .5055602E-05,.5525523E-05,.5778913E-05,.5837818E-05,.5685140E-05,&
     & .5291024E-05,.3454518E-05,.2976066E-05,.4585564E-05,.5471875E-05,&
     & .6031035E-05,.6358030E-05,.6472254E-05,.6355066E-05,.5985167E-05,&
     & .4072052E-05,.2992113E-05,.4870065E-05,.5900799E-05,.6563641E-05,&
     & .6978014E-05,.7155318E-05,.7090308E-05,.6756111E-05,.4779983E-05/

      data   absa(181:315, 1)   /                                       &
     & .2416301E-05,.3175817E-05,.3551611E-05,.3757653E-05,.3845981E-05,&
     & .3806815E-05,.3615925E-05,.3248989E-05,.1807537E-05,.2442982E-05,&
     & .3391649E-05,.3872022E-05,.4168023E-05,.4314273E-05,.4308869E-05,&
     & .4135096E-05,.3773612E-05,.2229249E-05,.2464689E-05,.3616511E-05,&
     & .4219290E-05,.4597739E-05,.4800153E-05,.4835853E-05,.4687880E-05,&
     & .4341846E-05,.2697930E-05,.2482346E-05,.3851272E-05,.4579688E-05,&
     & .5036754E-05,.5303465E-05,.5384276E-05,.5269123E-05,.4942816E-05,&
     & .3211571E-05,.2496958E-05,.4103453E-05,.4953618E-05,.5507193E-05,&
     & .5845841E-05,.5983458E-05,.5915896E-05,.5620275E-05,.3806808E-05,&
     & .1999899E-05,.2625031E-05,.2915420E-05,.3071867E-05,.3124395E-05,&
     & .3072646E-05,.2900498E-05,.2587050E-05,.1352887E-05,.2024083E-05,&
     & .2809833E-05,.3188693E-05,.3420314E-05,.3522509E-05,.3496553E-05,&
     & .3339109E-05,.3027660E-05,.1693245E-05,.2044050E-05,.3003017E-05,&
     & .3487914E-05,.3787092E-05,.3938052E-05,.3945979E-05,.3809484E-05,&
     & .3508701E-05,.2075300E-05,.2060639E-05,.3207415E-05,.3796719E-05,&
     & .4165268E-05,.4370415E-05,.4417121E-05,.4305496E-05,.4021839E-05,&
     & .2497828E-05,.2073109E-05,.3422126E-05,.4107505E-05,.4555166E-05,&
     & .4814366E-05,.4903671E-05,.4826510E-05,.4567702E-05,.2956470E-05,&
     & .1648342E-05,.2157145E-05,.2379934E-05,.2491203E-05,.2516412E-05,&
     & .2455538E-05,.2301740E-05,.2037661E-05,.1000718E-05,.1669893E-05,&
     & .2314044E-05,.2612275E-05,.2784581E-05,.2848144E-05,.2810417E-05,&
     & .2667632E-05,.2403320E-05,.1271614E-05,.1688235E-05,.2479334E-05,&
     & .2867128E-05,.3096477E-05,.3197978E-05,.3188985E-05,.3063442E-05,&
     & .2806027E-05,.1579154E-05,.1703269E-05,.2655822E-05,.3129870E-05,&
     & .3418130E-05,.3565529E-05,.3589051E-05,.3483463E-05,.3238464E-05,&
     & .1922786E-05,.1714256E-05,.2841908E-05,.3397528E-05,.3751189E-05,&
     & .3946139E-05,.4003586E-05,.3926825E-05,.3703912E-05,.2299592E-05/

      data   absa(316:450, 1)   /                                       &
     & .1353974E-05,.1765068E-05,.1932966E-05,.2007855E-05,.2012902E-05,&
     & .1948449E-05,.1811997E-05,.1592898E-05,.7335145E-06,.1373071E-05,&
     & .1897413E-05,.2129239E-05,.2252513E-05,.2286864E-05,.2242429E-05,&
     & .2114952E-05,.1891560E-05,.9471789E-06,.1389649E-05,.2038030E-05,&
     & .2344379E-05,.2513362E-05,.2580054E-05,.2558580E-05,.2443905E-05,&
     & .2225644E-05,.1192247E-05,.1403159E-05,.2190301E-05,.2568252E-05,&
     & .2784775E-05,.2889879E-05,.2895016E-05,.2795830E-05,.2587464E-05,&
     & .1469124E-05,.1413141E-05,.2349730E-05,.2797042E-05,.3067316E-05,&
     & .3212459E-05,.3245440E-05,.3170497E-05,.2979658E-05,.1775814E-05,&
     & .1109176E-05,.1438260E-05,.1560320E-05,.1608170E-05,.1598482E-05,&
     & .1534740E-05,.1415276E-05,.1235245E-05,.5310552E-06,.1125890E-05,&
     & .1549645E-05,.1724605E-05,.1809358E-05,.1823301E-05,.1775680E-05,&
     & .1662906E-05,.1476017E-05,.6978386E-06,.1140697E-05,.1669212E-05,&
     & .1904678E-05,.2026271E-05,.2067267E-05,.2037250E-05,.1934040E-05,&
     & .1748711E-05,.8911666E-06,.1152696E-05,.1799505E-05,.2093197E-05,&
     & .2254146E-05,.2325995E-05,.2316830E-05,.2226050E-05,.2048536E-05,&
     & .1112322E-05,.1161759E-05,.1935095E-05,.2286766E-05,.2492651E-05,&
     & .2596669E-05,.2611477E-05,.2539447E-05,.2375149E-05,.1359322E-05,&
     & .9077717E-06,.1174065E-05,.1263812E-05,.1292739E-05,.1275331E-05,&
     & .1215719E-05,.1112922E-05,.9650216E-06,.3899969E-06,.9222425E-06,&
     & .1268083E-05,.1400764E-05,.1458340E-05,.1460261E-05,.1413218E-05,&
     & .1314688E-05,.1159764E-05,.5200394E-06,.9352421E-06,.1370113E-05,&
     & .1550495E-05,.1639021E-05,.1662747E-05,.1629076E-05,.1537446E-05,&
     & .1381442E-05,.6733322E-06,.9455894E-06,.1481006E-05,.1708811E-05,&
     & .1830099E-05,.1878238E-05,.1861201E-05,.1779442E-05,.1628569E-05,&
     & .8501168E-06,.9536626E-06,.1596411E-05,.1872605E-05,.2030381E-05,&
     & .2104312E-05,.2107801E-05,.2041174E-05,.1900067E-05,.1049009E-05/

      data   absa(451:585, 1)   /                                       &
     & .7476406E-06,.9904317E-06,.1070509E-05,.1095431E-05,.1080587E-05,&
     & .1030695E-05,.9437084E-06,.8190527E-06,.3284667E-06,.7596122E-06,&
     & .1072730E-05,.1188660E-05,.1237753E-05,.1239815E-05,.1198979E-05,&
     & .1115702E-05,.9855200E-06,.4385665E-06,.7699182E-06,.1162790E-05,&
     & .1317244E-05,.1392618E-05,.1412049E-05,.1383498E-05,.1305642E-05,&
     & .1175399E-05,.5683962E-06,.7780771E-06,.1258890E-05,.1452488E-05,&
     & .1555737E-05,.1595474E-05,.1580904E-05,.1512840E-05,.1387631E-05,&
     & .7175693E-06,.7842272E-06,.1356608E-05,.1593107E-05,.1725648E-05,&
     & .1788723E-05,.1792615E-05,.1737571E-05,.1621220E-05,.8851099E-06,&
     & .6151816E-06,.8344601E-06,.9039065E-06,.9253730E-06,.9133751E-06,&
     & .8709904E-06,.7977024E-06,.6919990E-06,.2763894E-06,.6250011E-06,&
     & .9064917E-06,.1005419E-05,.1047668E-05,.1048849E-05,.1014123E-05,&
     & .9437407E-06,.8343490E-06,.3694765E-06,.6331236E-06,.9841850E-06,&
     & .1115823E-05,.1179325E-05,.1194996E-05,.1170425E-05,.1105130E-05,&
     & .9965785E-06,.4789252E-06,.6395688E-06,.1066132E-05,.1231039E-05,&
     & .1317746E-05,.1350903E-05,.1338577E-05,.1281845E-05,.1177960E-05,&
     & .6042803E-06,.6438610E-06,.1149508E-05,.1350173E-05,.1462385E-05,&
     & .1516252E-05,.1519539E-05,.1474251E-05,.1378037E-05,.7447887E-06,&
     & .5057177E-06,.7019314E-06,.7608992E-06,.7796527E-06,.7694622E-06,&
     & .7335795E-06,.6717508E-06,.5829027E-06,.2320809E-06,.5135675E-06,&
     & .7639636E-06,.8482237E-06,.8838364E-06,.8842773E-06,.8547719E-06,&
     & .7955572E-06,.7038419E-06,.3103090E-06,.5201276E-06,.8305264E-06,&
     & .9421352E-06,.9952533E-06,.1007989E-05,.9869406E-06,.9321635E-06,&
     & .8418452E-06,.4021416E-06,.5249168E-06,.9003622E-06,.1040066E-05,&
     & .1112264E-05,.1140134E-05,.1130238E-05,.1082715E-05,.9969161E-06,&
     & .5069884E-06,.5279947E-06,.9718501E-06,.1141014E-05,.1235721E-05,&
     & .1280968E-05,.1284659E-05,.1246723E-05,.1167012E-05,.6247983E-06/

      data   absa(  1:180, 2)   /                                       &
     & .3134144E-03,.2768907E-03,.2479948E-03,.2249616E-03,.2061518E-03,&
     & .1911399E-03,.1804902E-03,.1753720E-03,.1923166E-03,.3125459E-03,&
     & .2772983E-03,.2516939E-03,.2326523E-03,.2182683E-03,.2084544E-03,&
     & .2037087E-03,.2047079E-03,.2282142E-03,.3117491E-03,.2781161E-03,&
     & .2564198E-03,.2417736E-03,.2327659E-03,.2290275E-03,.2305224E-03,&
     & .2382043E-03,.2687208E-03,.3110029E-03,.2794674E-03,.2621069E-03,&
     & .2527413E-03,.2498662E-03,.2527146E-03,.2610136E-03,.2760286E-03,&
     & .3138217E-03,.3102439E-03,.2812588E-03,.2688570E-03,.2657349E-03,&
     & .2695233E-03,.2795220E-03,.2951889E-03,.3180254E-03,.3635083E-03,&
     & .3250728E-03,.2877719E-03,.2582845E-03,.2339301E-03,.2130875E-03,&
     & .1957091E-03,.1821765E-03,.1732917E-03,.1869083E-03,.3243276E-03,&
     & .2884264E-03,.2623057E-03,.2418236E-03,.2253898E-03,.2131266E-03,&
     & .2052326E-03,.2020436E-03,.2226017E-03,.3236251E-03,.2895634E-03,&
     & .2672790E-03,.2513043E-03,.2401735E-03,.2337268E-03,.2318615E-03,&
     & .2350471E-03,.2629556E-03,.3229411E-03,.2911957E-03,.2732898E-03,&
     & .2627372E-03,.2575851E-03,.2574466E-03,.2622274E-03,.2724290E-03,&
     & .3080495E-03,.3222901E-03,.2933218E-03,.2805641E-03,.2761631E-03,&
     & .2774714E-03,.2843448E-03,.2963172E-03,.3141300E-03,.3578779E-03,&
     & .3356807E-03,.2975818E-03,.2669711E-03,.2405827E-03,.2171280E-03,&
     & .1965563E-03,.1793297E-03,.1660515E-03,.1747079E-03,.3350220E-03,&
     & .2984105E-03,.2711015E-03,.2483929E-03,.2291853E-03,.2134785E-03,&
     & .2014226E-03,.1935279E-03,.2091221E-03,.3343949E-03,.2997564E-03,&
     & .2761602E-03,.2578485E-03,.2437572E-03,.2334625E-03,.2271012E-03,&
     & .2251640E-03,.2482498E-03,.3338041E-03,.3015868E-03,.2822818E-03,&
     & .2693005E-03,.2608828E-03,.2565529E-03,.2565695E-03,.2612161E-03,&
     & .2921963E-03,.3332193E-03,.3039287E-03,.2897558E-03,.2827271E-03,&
     & .2804955E-03,.2828580E-03,.2898279E-03,.3016339E-03,.3409420E-03,&
     & .3450280E-03,.3062487E-03,.2743678E-03,.2459208E-03,.2198163E-03,&
     & .1960751E-03,.1752958E-03,.1579201E-03,.1604421E-03,.3444259E-03,&
     & .3072374E-03,.2785358E-03,.2535346E-03,.2314569E-03,.2122613E-03,&
     & .1962491E-03,.1838764E-03,.1931422E-03,.3438771E-03,.3087236E-03,&
     & .2835940E-03,.2628376E-03,.2455747E-03,.2314574E-03,.2207572E-03,&
     & .2139307E-03,.2305496E-03,.3433424E-03,.3107056E-03,.2897639E-03,&
     & .2741158E-03,.2622176E-03,.2537611E-03,.2490381E-03,.2483736E-03,&
     & .2727973E-03,.3428143E-03,.3132078E-03,.2972996E-03,.2873501E-03,&
     & .2814114E-03,.2793015E-03,.2811140E-03,.2872362E-03,.3200560E-03/

      data   absa(181:315, 2)   /                                       &
     & .3531053E-03,.3137584E-03,.2806342E-03,.2502419E-03,.2217600E-03,&
     & .1953442E-03,.1713669E-03,.1502203E-03,.1465365E-03,.3525451E-03,&
     & .3148795E-03,.2847860E-03,.2576829E-03,.2329973E-03,.2107590E-03,&
     & .1912366E-03,.1747357E-03,.1774476E-03,.3520500E-03,.3165091E-03,&
     & .2898443E-03,.2668091E-03,.2466298E-03,.2291220E-03,.2146066E-03,&
     & .2032519E-03,.2129838E-03,.3515817E-03,.3186337E-03,.2960481E-03,&
     & .2778468E-03,.2627291E-03,.2506136E-03,.2416613E-03,.2361063E-03,&
     & .2533676E-03,.3511001E-03,.3212433E-03,.3036225E-03,.2908003E-03,&
     & .2814464E-03,.2753399E-03,.2725583E-03,.2734290E-03,.2989606E-03,&
     & .3599961E-03,.3201297E-03,.2856670E-03,.2533754E-03,.2227636E-03,&
     & .1938352E-03,.1668527E-03,.1421432E-03,.1323787E-03,.3595027E-03,&
     & .3213444E-03,.2897706E-03,.2605819E-03,.2334763E-03,.2084445E-03,&
     & .1855378E-03,.1651107E-03,.1612872E-03,.3590452E-03,.3230619E-03,&
     & .2947399E-03,.2694228E-03,.2465064E-03,.2258970E-03,.2076219E-03,&
     & .1919813E-03,.1947351E-03,.3586303E-03,.3252517E-03,.3009081E-03,&
     & .2801177E-03,.2619774E-03,.2463994E-03,.2333786E-03,.2231210E-03,&
     & .2329972E-03,.3581989E-03,.3279250E-03,.3083841E-03,.2926945E-03,&
     & .2800511E-03,.2701445E-03,.2629339E-03,.2586312E-03,.2762878E-03,&
     & .3658182E-03,.3254390E-03,.2896752E-03,.2557287E-03,.2231665E-03,&
     & .1919902E-03,.1623152E-03,.1344214E-03,.1190659E-03,.3653707E-03,&
     & .3267289E-03,.2936760E-03,.2626621E-03,.2334132E-03,.2057935E-03,&
     & .1798675E-03,.1559010E-03,.1460080E-03,.3649726E-03,.3284984E-03,&
     & .2985492E-03,.2711719E-03,.2458593E-03,.2223271E-03,.2006879E-03,&
     & .1811555E-03,.1773614E-03,.3645857E-03,.3307244E-03,.3046099E-03,&
     & .2814987E-03,.2606681E-03,.2418349E-03,.2250909E-03,.2105867E-03,&
     & .2134590E-03,.3642185E-03,.3334525E-03,.3119355E-03,.2937036E-03,&
     & .2780069E-03,.2645516E-03,.2532548E-03,.2443282E-03,.2545442E-03/

      data   absa(316:450, 2)   /                                       &
     & .3706916E-03,.3298152E-03,.2928733E-03,.2574333E-03,.2231173E-03,&
     & .1899407E-03,.1579051E-03,.1272072E-03,.1069023E-03,.3702964E-03,&
     & .3311594E-03,.2967583E-03,.2641125E-03,.2329078E-03,.2029732E-03,&
     & .1743745E-03,.1473211E-03,.1319665E-03,.3699285E-03,.3329585E-03,&
     & .3015284E-03,.2723221E-03,.2447616E-03,.2186223E-03,.1939959E-03,&
     & .1710321E-03,.1612919E-03,.3695766E-03,.3351892E-03,.3074184E-03,&
     & .2822618E-03,.2589063E-03,.2371561E-03,.2170927E-03,.1987943E-03,&
     & .1952513E-03,.3692655E-03,.3379605E-03,.3145686E-03,.2940369E-03,&
     & .2755227E-03,.2588395E-03,.2438914E-03,.2307984E-03,.2341519E-03,&
     & .3747630E-03,.3333819E-03,.2953456E-03,.2585076E-03,.2226014E-03,&
     & .1875711E-03,.1534395E-03,.1202799E-03,.9562602E-04,.3743949E-03,&
     & .3347353E-03,.2991026E-03,.2649281E-03,.2319014E-03,.1998657E-03,&
     & .1688739E-03,.1390792E-03,.1188772E-03,.3740704E-03,.3365341E-03,&
     & .3037434E-03,.2727874E-03,.2431388E-03,.2146370E-03,.1873092E-03,&
     & .1613140E-03,.1462150E-03,.3737686E-03,.3387595E-03,.3094625E-03,&
     & .2823102E-03,.2566053E-03,.2321963E-03,.2091128E-03,.1874298E-03,&
     & .1780428E-03,.3734709E-03,.3415160E-03,.3163831E-03,.2936218E-03,&
     & .2724956E-03,.2528123E-03,.2345043E-03,.2177162E-03,.2147257E-03,&
     & .3781153E-03,.3363497E-03,.2974396E-03,.2595086E-03,.2223300E-03,&
     & .1858397E-03,.1500339E-03,.1149605E-03,.8681636E-04,.3777843E-03,&
     & .3377331E-03,.3011323E-03,.2657468E-03,.2312660E-03,.1975718E-03,&
     & .1646999E-03,.1327485E-03,.1086418E-03,.3774794E-03,.3395301E-03,&
     & .3057000E-03,.2733530E-03,.2420489E-03,.2116788E-03,.1822379E-03,&
     & .1538334E-03,.1343822E-03,.3772048E-03,.3417633E-03,.3112954E-03,&
     & .2825617E-03,.2549956E-03,.2284826E-03,.2030187E-03,.1786623E-03,&
     & .1644876E-03,.3769456E-03,.3445443E-03,.3180728E-03,.2935143E-03,&
     & .2703103E-03,.2482498E-03,.2273178E-03,.2075828E-03,.1993535E-03/

      data   absa(451:585, 2)   /                                       &
     & .3807336E-03,.3393314E-03,.3005620E-03,.2626883E-03,.2254498E-03,&
     & .1887780E-03,.1527070E-03,.1172356E-03,.8770656E-04,.3804457E-03,&
     & .3409096E-03,.3045588E-03,.2692926E-03,.2347827E-03,.2009743E-03,&
     & .1678340E-03,.1354561E-03,.1098393E-03,.3801856E-03,.3428944E-03,&
     & .3094604E-03,.2773048E-03,.2460506E-03,.2155751E-03,.1858896E-03,&
     & .1570450E-03,.1359353E-03,.3799496E-03,.3453641E-03,.3154279E-03,&
     & .2869512E-03,.2594977E-03,.2329282E-03,.2072109E-03,.1824185E-03,&
     & .1664584E-03,.3797085E-03,.3484177E-03,.3225651E-03,.2983903E-03,&
     & .2753339E-03,.2532382E-03,.2320873E-03,.2119269E-03,.2018178E-03,&
     & .3829042E-03,.3418780E-03,.3032789E-03,.2654560E-03,.2281697E-03,&
     & .1913858E-03,.1550847E-03,.1193084E-03,.8855999E-04,.3826538E-03,&
     & .3436246E-03,.3075479E-03,.2723898E-03,.2378998E-03,.2039790E-03,&
     & .1706332E-03,.1379197E-03,.1109681E-03,.3824225E-03,.3458050E-03,&
     & .3127369E-03,.2807746E-03,.2495908E-03,.2190408E-03,.1891472E-03,&
     & .1599572E-03,.1373987E-03,.3822117E-03,.3485078E-03,.3190368E-03,&
     & .2908289E-03,.2634888E-03,.2368723E-03,.2109681E-03,.1858311E-03,&
     & .1683087E-03,.3819986E-03,.3517908E-03,.3265146E-03,.3026775E-03,&
     & .2797758E-03,.2576864E-03,.2363700E-03,.2158850E-03,.2040952E-03,&
     & .3846876E-03,.3440378E-03,.3056059E-03,.2678315E-03,.2305157E-03,&
     & .1936247E-03,.1571344E-03,.1210869E-03,.8928501E-04,.3844746E-03,&
     & .3459486E-03,.3101125E-03,.2750561E-03,.2405776E-03,.2065779E-03,&
     & .1730455E-03,.1400443E-03,.1119334E-03,.3842745E-03,.3483142E-03,&
     & .3155650E-03,.2837694E-03,.2526251E-03,.2220255E-03,.1919644E-03,&
     & .1624719E-03,.1386492E-03,.3840848E-03,.3512033E-03,.3221367E-03,&
     & .2941685E-03,.2669103E-03,.2402530E-03,.2142043E-03,.1887786E-03,&
     & .1698933E-03,.3839069E-03,.3546918E-03,.3299208E-03,.3063646E-03,&
     & .2835973E-03,.2614972E-03,.2400614E-03,.2193165E-03,.2060528E-03/

!     the array iabsb(235,2) (kb(5,13:59,2)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 2,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:120, 1)   /                                       &
     & .1148431E-07,.1166076E-07,.1181018E-07,.1191914E-07,.1198918E-07,&
     & .9447302E-08,.9588853E-08,.9702321E-08,.9781170E-08,.9829657E-08,&
     & .7768804E-08,.7881241E-08,.7964406E-08,.8027339E-08,.8064535E-08,&
     & .6387323E-08,.6478389E-08,.6541760E-08,.6584671E-08,.6611940E-08,&
     & .5258216E-08,.5325260E-08,.5372095E-08,.5406858E-08,.5433075E-08,&
     & .4325046E-08,.4376631E-08,.4414720E-08,.4442940E-08,.4468040E-08,&
     & .3558904E-08,.3599790E-08,.3631084E-08,.3656572E-08,.3675668E-08,&
     & .2928994E-08,.2962935E-08,.2992609E-08,.3011945E-08,.3028560E-08,&
     & .2410261E-08,.2440986E-08,.2461561E-08,.2481755E-08,.2502470E-08,&
     & .1987605E-08,.2009776E-08,.2031171E-08,.2053325E-08,.2074842E-08,&
     & .1638935E-08,.1660456E-08,.1681270E-08,.1702299E-08,.1720110E-08,&
     & .1354484E-08,.1373198E-08,.1393541E-08,.1411146E-08,.1425870E-08,&
     & .1120107E-08,.1139127E-08,.1154912E-08,.1169427E-08,.1184272E-08,&
     & .9286718E-09,.9430320E-09,.9571872E-09,.9708633E-09,.9838995E-09,&
     & .7699187E-09,.7826246E-09,.7946201E-09,.8067476E-09,.8192210E-09,&
     & .6396450E-09,.6506736E-09,.6618526E-09,.6737570E-09,.6838369E-09,&
     & .5326710E-09,.5429777E-09,.5542978E-09,.5638603E-09,.5735975E-09,&
     & .4444457E-09,.4553219E-09,.4646430E-09,.4739935E-09,.4848399E-09,&
     & .3733850E-09,.3825372E-09,.3906037E-09,.4005917E-09,.4095546E-09,&
     & .3137446E-09,.3209559E-09,.3301769E-09,.3382360E-09,.3459685E-09,&
     & .2630519E-09,.2710057E-09,.2790274E-09,.2856675E-09,.2926209E-09,&
     & .2217405E-09,.2292329E-09,.2353810E-09,.2416136E-09,.2479216E-09,&
     & .1867928E-09,.1929235E-09,.1984309E-09,.2043657E-09,.2098748E-09,&
     & .1569163E-09,.1620869E-09,.1668054E-09,.1724993E-09,.1773310E-09/

      data   absb(121:235, 1)   /                                       &
     & .1308096E-09,.1354063E-09,.1397323E-09,.1448444E-09,.1490380E-09,&
     & .1091276E-09,.1131669E-09,.1173393E-09,.1217436E-09,.1254286E-09,&
     & .9109868E-10,.9461754E-10,.9833961E-10,.1018699E-09,.1054553E-09,&
     & .7563445E-10,.7868380E-10,.8193156E-10,.8518848E-10,.8844146E-10,&
     & .6263641E-10,.6531415E-10,.6821057E-10,.7108715E-10,.7394727E-10,&
     & .5187154E-10,.5419548E-10,.5674107E-10,.5926221E-10,.6180436E-10,&
     & .4271996E-10,.4475106E-10,.4699608E-10,.4922324E-10,.5127378E-10,&
     & .3507508E-10,.3687109E-10,.3881362E-10,.4074018E-10,.4258146E-10,&
     & .2877982E-10,.3031418E-10,.3201489E-10,.3369063E-10,.3530266E-10,&
     & .2354780E-10,.2487550E-10,.2632461E-10,.2777491E-10,.2921257E-10,&
     & .1917857E-10,.2030852E-10,.2155848E-10,.2281480E-10,.2406339E-10,&
     & .1560083E-10,.1657804E-10,.1762140E-10,.1871016E-10,.1980295E-10,&
     & .1265679E-10,.1349811E-10,.1439672E-10,.1532257E-10,.1626259E-10,&
     & .1025893E-10,.1099795E-10,.1174551E-10,.1253606E-10,.1335795E-10,&
     & .8310093E-11,.8943449E-11,.9572090E-11,.1024901E-10,.1095019E-10,&
     & .6730157E-11,.7257352E-11,.7793431E-11,.8371987E-11,.8975155E-11,&
     & .5435177E-11,.5888298E-11,.6335711E-11,.6830724E-11,.7343180E-11,&
     & .4401117E-11,.4770132E-11,.5149810E-11,.5568023E-11,.6001500E-11,&
     & .3553186E-11,.3856211E-11,.4189341E-11,.4542349E-11,.4912188E-11,&
     & .2869971E-11,.3121146E-11,.3397253E-11,.3695718E-11,.4009005E-11,&
     & .2316841E-11,.2524409E-11,.2751777E-11,.3000391E-11,.3264281E-11,&
     & .1868320E-11,.2038026E-11,.2227552E-11,.2436194E-11,.2658837E-11,&
     & .1531264E-11,.1674283E-11,.1835257E-11,.2010670E-11,.2200480E-11/

      data   absb(  1:120, 2)   /                                       &
     & .8735620E-05,.8730295E-05,.8725985E-05,.8721791E-05,.8717868E-05,&
     & .8768835E-05,.8764701E-05,.8760332E-05,.8756520E-05,.8752933E-05,&
     & .8796329E-05,.8792037E-05,.8788589E-05,.8785089E-05,.8781325E-05,&
     & .8818502E-05,.8814986E-05,.8812031E-05,.8808408E-05,.8804794E-05,&
     & .8836927E-05,.8833555E-05,.8830851E-05,.8827746E-05,.8824444E-05,&
     & .8852216E-05,.8849478E-05,.8846711E-05,.8842936E-05,.8839425E-05,&
     & .8864508E-05,.8861820E-05,.8858820E-05,.8855571E-05,.8852261E-05,&
     & .8874870E-05,.8872004E-05,.8869799E-05,.8866369E-05,.8862109E-05,&
     & .8883129E-05,.8880356E-05,.8877848E-05,.8874404E-05,.8870894E-05,&
     & .8890022E-05,.8886869E-05,.8884553E-05,.8881040E-05,.8876761E-05,&
     & .8894947E-05,.8892776E-05,.8889874E-05,.8886360E-05,.8882208E-05,&
     & .8899699E-05,.8896942E-05,.8893922E-05,.8890629E-05,.8886217E-05,&
     & .8903090E-05,.8900055E-05,.8897299E-05,.8893628E-05,.8888999E-05,&
     & .8905522E-05,.8902992E-05,.8899724E-05,.8895896E-05,.8891197E-05,&
     & .8907716E-05,.8904994E-05,.8901656E-05,.8897848E-05,.8892455E-05,&
     & .8909085E-05,.8906056E-05,.8903176E-05,.8899099E-05,.8893887E-05,&
     & .8910421E-05,.8907592E-05,.8904343E-05,.8899582E-05,.8894302E-05,&
     & .8911657E-05,.8908291E-05,.8904615E-05,.8899966E-05,.8893877E-05,&
     & .8912045E-05,.8908750E-05,.8904978E-05,.8899891E-05,.8893523E-05,&
     & .8912007E-05,.8908554E-05,.8905008E-05,.8899598E-05,.8893271E-05,&
     & .8912113E-05,.8908681E-05,.8904415E-05,.8899216E-05,.8892224E-05,&
     & .8911996E-05,.8908643E-05,.8903701E-05,.8898193E-05,.8891198E-05,&
     & .8912323E-05,.8908424E-05,.8904030E-05,.8897838E-05,.8890898E-05,&
     & .8912745E-05,.8908995E-05,.8904478E-05,.8898387E-05,.8891021E-05/

      data   absb(121:235, 2)   /                                       &
     & .8913438E-05,.8910133E-05,.8905684E-05,.8899336E-05,.8892605E-05,&
     & .8914238E-05,.8910808E-05,.8906957E-05,.8900965E-05,.8894403E-05,&
     & .8915350E-05,.8911585E-05,.8907534E-05,.8901916E-05,.8895407E-05,&
     & .8916208E-05,.8912694E-05,.8909062E-05,.8903707E-05,.8897419E-05,&
     & .8916842E-05,.8914357E-05,.8910350E-05,.8905322E-05,.8899560E-05,&
     & .8918148E-05,.8915052E-05,.8911721E-05,.8906637E-05,.8901314E-05,&
     & .8918763E-05,.8916311E-05,.8912939E-05,.8908528E-05,.8903349E-05,&
     & .8919788E-05,.8917267E-05,.8914201E-05,.8910219E-05,.8904947E-05,&
     & .8920650E-05,.8918085E-05,.8915570E-05,.8911600E-05,.8907370E-05,&
     & .8921233E-05,.8919303E-05,.8916881E-05,.8913302E-05,.8909080E-05,&
     & .8922720E-05,.8920411E-05,.8917844E-05,.8914538E-05,.8910801E-05,&
     & .8923377E-05,.8921620E-05,.8918941E-05,.8916117E-05,.8912493E-05,&
     & .8924238E-05,.8922269E-05,.8920225E-05,.8917590E-05,.8913869E-05,&
     & .8925304E-05,.8923279E-05,.8920969E-05,.8918470E-05,.8915390E-05,&
     & .8926224E-05,.8924279E-05,.8921984E-05,.8919554E-05,.8916354E-05,&
     & .8926896E-05,.8924592E-05,.8922547E-05,.8920115E-05,.8917801E-05,&
     & .8927945E-05,.8925423E-05,.8923860E-05,.8921064E-05,.8918600E-05,&
     & .8928033E-05,.8926417E-05,.8923949E-05,.8921931E-05,.8919379E-05,&
     & .8929706E-05,.8927053E-05,.8924754E-05,.8922728E-05,.8920444E-05,&
     & .8929850E-05,.8927620E-05,.8925048E-05,.8923373E-05,.8920887E-05,&
     & .8931261E-05,.8928697E-05,.8925909E-05,.8924267E-05,.8921512E-05,&
     & .8931652E-05,.8929478E-05,.8926979E-05,.8924651E-05,.8922436E-05,&
     & .8932405E-05,.8929827E-05,.8927149E-05,.8924663E-05,.8922710E-05/

!  ---
      data   forref(1:3,1: 2)    /                         .8005249E-06,&
     & .8929867E-06,.9310644E-06,.2367132E-05,.8712965E-06,.2553141E-06/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 2).

      data   selfref(1:10,1: 2)  /                                      &
     & .1890754E-03,.1626696E-03,.1404954E-03,.1218179E-03,.1060361E-03,&
     & .9265747E-04,.8127769E-04,.7156485E-04,.6324571E-04,.5609450E-04,&
     & .1435463E-03,.1416034E-03,.1400831E-03,.1390223E-03,.1384643E-03,&
     & .1384633E-03,.1390821E-03,.1403951E-03,.1424925E-03,.1454806E-03/

!........................................!
      end module module_radsw_kgb22      !
!========================================!


!========================================!
      module module_radsw_kgb23          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 23:  8050-12850 cm-1 (low - h2o; high - nothing)           !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG23

!
      implicit none
!
      private
!
      integer,  public :: MSA23, MSF23, MFR23
      parameter (MSA23=65, MSF23=10, MFR23=3)

      real (kind=kind_phys), public ::              forref(MFR23,NG23), &
     &       absa(MSA23,NG23), selfref(MSF23,NG23), rayl(NG23)

!  ---  average giver et al. correction factor for this band.
      real (kind=kind_phys), parameter, public :: givfac = 1.029

!  ---  rayleigh extinction coefficient at all v
      data   rayl (1:10)         /                                      &
     & .5828588E-07,.5935776E-07,.5255710E-07,.4733880E-07,.4174660E-07,&
     & .3980970E-07,.4007860E-07,.3674780E-07,.3456143E-07,.3212082E-07/

!     the array absa(65,NG23) (ka(5,13,NG23)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 10, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1: 65, 1)   /                                       &
     & .4349017E-06,.4470059E-06,.4658775E-06,.4619598E-06,.4690252E-06,&
     & .3442571E-06,.3522645E-06,.3709106E-06,.3777774E-06,.3744779E-06,&
     & .2678674E-06,.2763444E-06,.2843822E-06,.2996831E-06,.2973083E-06,&
     & .2115438E-06,.2202997E-06,.2278028E-06,.2342144E-06,.2458529E-06,&
     & .1770743E-06,.1850806E-06,.1926042E-06,.1976887E-06,.2082632E-06,&
     & .2400161E-06,.2495213E-06,.2584593E-06,.2672377E-06,.2743845E-06,&
     & .2965188E-06,.3124621E-06,.3263585E-06,.3361788E-06,.3465234E-06,&
     & .5867176E-06,.6150627E-06,.6378628E-06,.6634773E-06,.6794765E-06,&
     & .2613851E-05,.2661762E-05,.2697551E-05,.2717845E-05,.2731026E-05,&
     & .4391054E-05,.4625930E-05,.4834719E-05,.5033254E-05,.5183516E-05,&
     & .5804333E-05,.6050090E-05,.6263521E-05,.6493135E-05,.6671565E-05,&
     & .6793586E-05,.7078673E-05,.7345537E-05,.7567607E-05,.7770683E-05,&
     & .7689857E-05,.8029923E-05,.8306724E-05,.8523862E-05,.8740922E-05/

      data   absa(  1: 65, 2)   /                                       &
     & .1671495E-04,.1698172E-04,.1762477E-04,.1736035E-04,.1748475E-04,&
     & .1392235E-04,.1415328E-04,.1470623E-04,.1483132E-04,.1457076E-04,&
     & .1145651E-04,.1165559E-04,.1181059E-04,.1229573E-04,.1205623E-04,&
     & .9472038E-05,.9654257E-05,.9802561E-05,.9927636E-05,.1031608E-04,&
     & .7831263E-05,.7996876E-05,.8135117E-05,.8252891E-05,.8598083E-05,&
     & .6358154E-05,.6506960E-05,.6630804E-05,.6734304E-05,.6813372E-05,&
     & .5148405E-05,.5280721E-05,.5396333E-05,.5487849E-05,.5553536E-05,&
     & .5133743E-05,.5259110E-05,.5361676E-05,.5436170E-05,.5502268E-05,&
     & .1218538E-04,.1263140E-04,.1302060E-04,.1326091E-04,.1346663E-04,&
     & .4141871E-04,.4267769E-04,.4394235E-04,.4496715E-04,.4594083E-04,&
     & .7068660E-04,.7240671E-04,.7377895E-04,.7506963E-04,.7606084E-04,&
     & .8723266E-04,.8870892E-04,.9006588E-04,.9131091E-04,.9244792E-04,&
     & .9151119E-04,.9291720E-04,.9438789E-04,.9568004E-04,.9699178E-04/

      data   absa(  1: 65, 3)   /                                       &
     & .8285900E-04,.8481700E-04,.8905600E-04,.8805700E-04,.8941000E-04,&
     & .7093700E-04,.7268500E-04,.7679600E-04,.7795500E-04,.7673500E-04,&
     & .5987600E-04,.6144800E-04,.6283700E-04,.6629300E-04,.6499600E-04,&
     & .5059800E-04,.5205400E-04,.5329300E-04,.5433200E-04,.5733300E-04,&
     & .4274200E-04,.4403500E-04,.4516400E-04,.4613400E-04,.4893500E-04,&
     & .3576900E-04,.3697500E-04,.3803800E-04,.3891700E-04,.3968100E-04,&
     & .2974700E-04,.3082400E-04,.3175600E-04,.3258900E-04,.3331400E-04,&
     & .2199400E-04,.2294500E-04,.2378600E-04,.2451700E-04,.2515500E-04,&
     & .2229800E-04,.2268800E-04,.2312700E-04,.2380300E-04,.2433500E-04,&
     & .8889800E-04,.9128000E-04,.9333300E-04,.9522400E-04,.9629800E-04,&
     & .1229900E-03,.1240700E-03,.1253600E-03,.1264200E-03,.1281300E-03,&
     & .1453900E-03,.1485100E-03,.1502200E-03,.1515700E-03,.1520400E-03,&
     & .1594900E-03,.1623900E-03,.1646700E-03,.1666700E-03,.1680100E-03/

      data   absa(  1: 65, 4)   /                                       &
     & .2533900E-03,.2599500E-03,.2717000E-03,.2696300E-03,.2741300E-03,&
     & .2190800E-03,.2240400E-03,.2352200E-03,.2390200E-03,.2371700E-03,&
     & .1861100E-03,.1907600E-03,.1951800E-03,.2050300E-03,.2022200E-03,&
     & .1576900E-03,.1621000E-03,.1659700E-03,.1694800E-03,.1781700E-03,&
     & .1340200E-03,.1379200E-03,.1415300E-03,.1443400E-03,.1526700E-03,&
     & .1139000E-03,.1174300E-03,.1206500E-03,.1231600E-03,.1254300E-03,&
     & .9641700E-04,.9961200E-04,.1023300E-03,.1045300E-03,.1065200E-03,&
     & .8139500E-04,.8420500E-04,.8634600E-04,.8840600E-04,.9012200E-04,&
     & .4777600E-04,.4897100E-04,.4973600E-04,.4991700E-04,.5028900E-04,&
     & .1069800E-03,.1081500E-03,.1081700E-03,.1079900E-03,.1085100E-03,&
     & .2022000E-03,.2072700E-03,.2124100E-03,.2167500E-03,.2198900E-03,&
     & .2347400E-03,.2360100E-03,.2397400E-03,.2438300E-03,.2487600E-03,&
     & .2341000E-03,.2380900E-03,.2418500E-03,.2455400E-03,.2495200E-03/

      data   absa(  1: 65, 5)   /                                       &
     & .6702400E-03,.6802600E-03,.7041900E-03,.7015900E-03,.7108900E-03,&
     & .5872900E-03,.5977800E-03,.6209700E-03,.6291200E-03,.6242300E-03,&
     & .5096700E-03,.5190000E-03,.5276500E-03,.5479400E-03,.5426600E-03,&
     & .4416700E-03,.4500600E-03,.4579300E-03,.4646900E-03,.4833500E-03,&
     & .3809600E-03,.3888100E-03,.3957600E-03,.4025900E-03,.4208600E-03,&
     & .3281800E-03,.3353900E-03,.3419200E-03,.3482900E-03,.3540500E-03,&
     & .2825900E-03,.2894600E-03,.2958400E-03,.3020300E-03,.3074200E-03,&
     & .2427300E-03,.2491200E-03,.2554600E-03,.2611000E-03,.2660700E-03,&
     & .1993700E-03,.2065300E-03,.2131400E-03,.2196800E-03,.2252000E-03,&
     & .1330600E-03,.1333100E-03,.1339300E-03,.1353800E-03,.1361600E-03,&
     & .1623600E-03,.1615400E-03,.1618700E-03,.1611300E-03,.1620900E-03,&
     & .1787200E-03,.1835500E-03,.1861200E-03,.1879200E-03,.1874500E-03,&
     & .1897000E-03,.1938400E-03,.1977300E-03,.2026100E-03,.2037700E-03/

      data   absa(  1: 65, 6)   /                                       &
     & .1813000E-02,.1830500E-02,.1871600E-02,.1865500E-02,.1881400E-02,&
     & .1642000E-02,.1660000E-02,.1700600E-02,.1715600E-02,.1710800E-02,&
     & .1468700E-02,.1487000E-02,.1504200E-02,.1542300E-02,.1537600E-02,&
     & .1306800E-02,.1324800E-02,.1342100E-02,.1359200E-02,.1396000E-02,&
     & .1157400E-02,.1175300E-02,.1192800E-02,.1209700E-02,.1246700E-02,&
     & .1016700E-02,.1034200E-02,.1051500E-02,.1068100E-02,.1084000E-02,&
     & .8899200E-03,.9066200E-03,.9229900E-03,.9384400E-03,.9534700E-03,&
     & .7844500E-03,.8003100E-03,.8163900E-03,.8313100E-03,.8463100E-03,&
     & .6981200E-03,.7131200E-03,.7280100E-03,.7422200E-03,.7560500E-03,&
     & .3252100E-03,.3383500E-03,.3519400E-03,.3638100E-03,.3759400E-03,&
     & .3140600E-03,.3201300E-03,.3245600E-03,.3326100E-03,.3374100E-03,&
     & .2813200E-03,.2873200E-03,.2967400E-03,.3050900E-03,.3139300E-03,&
     & .2570400E-03,.2631600E-03,.2719500E-03,.2773200E-03,.2890500E-03/

      data   absa(  1: 65, 7)   /                                       &
     & .6737000E-02,.6787300E-02,.6889600E-02,.6881900E-02,.6926800E-02,&
     & .6311100E-02,.6362200E-02,.6462300E-02,.6506000E-02,.6502700E-02,&
     & .5883400E-02,.5936100E-02,.5985800E-02,.6081100E-02,.6080600E-02,&
     & .5475300E-02,.5530900E-02,.5582300E-02,.5630600E-02,.5722900E-02,&
     & .5078100E-02,.5137300E-02,.5189500E-02,.5239100E-02,.5332800E-02,&
     & .4679100E-02,.4740800E-02,.4794900E-02,.4847000E-02,.4899300E-02,&
     & .4272400E-02,.4338100E-02,.4395100E-02,.4450300E-02,.4506100E-02,&
     & .3856800E-02,.3925400E-02,.3984800E-02,.4043100E-02,.4100500E-02,&
     & .3565700E-02,.3636200E-02,.3699400E-02,.3760800E-02,.3821500E-02,&
     & .3377400E-02,.3450100E-02,.3517100E-02,.3581100E-02,.3645000E-02,&
     & .2392300E-02,.2462300E-02,.2526300E-02,.2589000E-02,.2651700E-02,&
     & .1695900E-02,.1754200E-02,.1807700E-02,.1861400E-02,.1918700E-02,&
     & .1173200E-02,.1223200E-02,.1271200E-02,.1322100E-02,.1367700E-02/

      data   absa(  1: 65, 8)   /                                       &
     & .1960400E-01,.1969800E-01,.1993800E-01,.1985400E-01,.1995000E-01,&
     & .1871400E-01,.1880300E-01,.1903500E-01,.1913100E-01,.1910200E-01,&
     & .1767600E-01,.1778500E-01,.1790400E-01,.1818900E-01,.1815600E-01,&
     & .1666200E-01,.1677300E-01,.1690800E-01,.1705600E-01,.1730300E-01,&
     & .1565500E-01,.1577500E-01,.1594200E-01,.1610300E-01,.1635900E-01,&
     & .1469400E-01,.1483800E-01,.1502900E-01,.1520000E-01,.1532000E-01,&
     & .1379700E-01,.1395900E-01,.1417400E-01,.1435000E-01,.1447100E-01,&
     & .1290200E-01,.1308900E-01,.1331300E-01,.1348900E-01,.1362600E-01,&
     & .1189700E-01,.1210500E-01,.1233300E-01,.1251200E-01,.1265600E-01,&
     & .1183400E-01,.1207200E-01,.1233100E-01,.1254800E-01,.1272100E-01,&
     & .1141600E-01,.1169900E-01,.1196800E-01,.1214100E-01,.1231100E-01,&
     & .1077600E-01,.1107000E-01,.1130900E-01,.1150700E-01,.1170200E-01,&
     & .9957700E-02,.1026300E-01,.1049200E-01,.1071900E-01,.1095100E-01/

      data   absa(  1: 65, 9)   /                                       &
     & .3722573E-01,.3735001E-01,.3766995E-01,.3757402E-01,.3768499E-01,&
     & .3662238E-01,.3678396E-01,.3719958E-01,.3732433E-01,.3724068E-01,&
     & .3562867E-01,.3578960E-01,.3595765E-01,.3640607E-01,.3634033E-01,&
     & .3435375E-01,.3454391E-01,.3475193E-01,.3497256E-01,.3549406E-01,&
     & .3293358E-01,.3315823E-01,.3338119E-01,.3362894E-01,.3419310E-01,&
     & .3144491E-01,.3168016E-01,.3191274E-01,.3218704E-01,.3249824E-01,&
     & .2994750E-01,.3018681E-01,.3043038E-01,.3073495E-01,.3106162E-01,&
     & .2842901E-01,.2866936E-01,.2893935E-01,.2926623E-01,.2960819E-01,&
     & .2613286E-01,.2638746E-01,.2667815E-01,.2703098E-01,.2739097E-01,&
     & .2645921E-01,.2674553E-01,.2704316E-01,.2742667E-01,.2777404E-01,&
     & .2621020E-01,.2647702E-01,.2689048E-01,.2732045E-01,.2775716E-01,&
     & .2539997E-01,.2572727E-01,.2619059E-01,.2665103E-01,.2709386E-01,&
     & .2408096E-01,.2452980E-01,.2508382E-01,.2552259E-01,.2603640E-01/

      data   absa(  1: 65,10)   /                                       &
     & .1157665E+00,.1155684E+00,.1154876E+00,.1152633E+00,.1151482E+00,&
     & .1272768E+00,.1270495E+00,.1269626E+00,.1267696E+00,.1265036E+00,&
     & .1391213E+00,.1389149E+00,.1387068E+00,.1386130E+00,.1382518E+00,&
     & .1505134E+00,.1502913E+00,.1500415E+00,.1497761E+00,.1496606E+00,&
     & .1614616E+00,.1612169E+00,.1609519E+00,.1606590E+00,.1605876E+00,&
     & .1719962E+00,.1717604E+00,.1715010E+00,.1711819E+00,.1708343E+00,&
     & .1820676E+00,.1818515E+00,.1815935E+00,.1812537E+00,.1808999E+00,&
     & .1917067E+00,.1915304E+00,.1912634E+00,.1909221E+00,.1905620E+00,&
     & .2008731E+00,.2007265E+00,.2004847E+00,.2001356E+00,.1997760E+00,&
     & .2030412E+00,.2029033E+00,.2026654E+00,.2022945E+00,.2019604E+00,&
     & .2081847E+00,.2080855E+00,.2077116E+00,.2073153E+00,.2068695E+00,&
     & .2148279E+00,.2146757E+00,.2142582E+00,.2138204E+00,.2133431E+00,&
     & .2225531E+00,.2222204E+00,.2216768E+00,.2212495E+00,.2206397E+00/

!  ---
      data   forref(1:3,1:10)    /                                      &
     & .1709528E-06,.1749320E-06,.2303093E-05,.3273721E-05,.3572838E-05,&
     & .7664943E-05,.9640070E-05,.1071100E-04,.1044860E-04,.3027750E-04,&
     & .3575300E-04,.3407240E-04,.1024370E-03,.1084750E-03,.1052450E-03,&
     & .1460540E-03,.1414900E-03,.1330710E-03,.1639780E-03,.1502080E-03,&
     & .1428640E-03,.2204120E-03,.1829430E-03,.1509410E-03,.2312169E-03,&
     & .2065332E-03,.1728525E-03,.2740557E-03,.2606475E-03,.2434367E-03/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 10).

      data   selfref(1:10,1:10)  /                                      &
     & .1042197E-04,.8992829E-05,.7807679E-05,.6819782E-05,.5991772E-05,&
     & .5293756E-05,.4701789E-05,.4196660E-05,.3762952E-05,.3388235E-05,&
     & .7734423E-04,.6990340E-04,.6317978E-04,.5710393E-04,.5161343E-04,&
     & .4665177E-04,.4216797E-04,.3811593E-04,.3445388E-04,.3114436E-04,&
     & .2064340E-03,.1874350E-03,.1701850E-03,.1545220E-03,.1403010E-03,&
     & .1273880E-03,.1156640E-03,.1050190E-03,.9535400E-04,.8657830E-04,&
     & .5906450E-03,.5331090E-03,.4811770E-03,.4343050E-03,.3919980E-03,&
     & .3538120E-03,.3193460E-03,.2882380E-03,.2601600E-03,.2348170E-03,&
     & .1630290E-02,.1487730E-02,.1357630E-02,.1238910E-02,.1130570E-02,&
     & .1031700E-02,.9414830E-03,.8591530E-03,.7840230E-03,.7154620E-03,&
     & .2045280E-02,.1892580E-02,.1751280E-02,.1620530E-02,.1499540E-02,&
     & .1387580E-02,.1283980E-02,.1188120E-02,.1099410E-02,.1017330E-02,&
     & .2105890E-02,.1970780E-02,.1844340E-02,.1726010E-02,.1615280E-02,&
     & .1511640E-02,.1414660E-02,.1323900E-02,.1238960E-02,.1159470E-02,&
     & .2450980E-02,.2337450E-02,.2229180E-02,.2125920E-02,.2027450E-02,&
     & .1933530E-02,.1843970E-02,.1758560E-02,.1677100E-02,.1599410E-02,&
     & .2838174E-02,.2668321E-02,.2508780E-02,.2358929E-02,.2218161E-02,&
     & .2085920E-02,.1961685E-02,.1844964E-02,.1735293E-02,.1632237E-02,&
     & .3679539E-02,.3409274E-02,.3159083E-02,.2927474E-02,.2713037E-02,&
     & .2514491E-02,.2330645E-02,.2160398E-02,.2002729E-02,.1856699E-02/

!........................................!
      end module module_radsw_kgb23      !
!========================================!


!========================================!
      module module_radsw_kgb24          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 24: 12850-16000 cm-1 (low - h2o,o2; high - o2)             !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG24

!
      implicit none
!
      private
!
      integer,  public :: MSA24, MSB24, MSF24, MFR24, MFX24
      parameter (MSA24=585, MSB24=235, MSF24=10, MFR24=3, MFX24=9)

      real (kind=kind_phys), public ::           forref(MFR24,NG24),    &
     &       absa(MSA24,NG24), absb(MSB24,NG24), selfref(MSF24,NG24),   &
     &       abso3a(NG24), abso3b(NG24), rayla(NG24,MFX24), raylb(NG24)

!  ---  rayleigh extinction coefficient at all v
      data   rayla (1: 8,1:9)    /            .1368516E-06,.1805397E-06,&
     & .2243572E-06,.2191201E-06,.1450164E-06,.1344697E-06,.1214120E-06,&
     & .1199659E-06,.1672218E-06,.2134978E-06,.1793655E-06,.1624268E-06,&
     & .1442232E-06,.1377599E-06,.1214250E-06,.1199661E-06,.1708948E-06,&
     & .2099293E-06,.1784014E-06,.1622850E-06,.1458217E-06,.1400260E-06,&
     & .1241789E-06,.1199661E-06,.1737959E-06,.2064310E-06,.1785495E-06,&
     & .1617341E-06,.1480687E-06,.1414056E-06,.1280616E-06,.1199661E-06,&
     & .1751957E-06,.2045877E-06,.1784314E-06,.1614815E-06,.1507242E-06,&
     & .1421961E-06,.1320991E-06,.1199661E-06,.1757802E-06,.2032567E-06,&
     & .1789106E-06,.1613267E-06,.1525379E-06,.1429132E-06,.1360017E-06,&
     & .1204062E-06,.1757127E-06,.2027785E-06,.1790739E-06,.1616034E-06,&
     & .1535303E-06,.1446526E-06,.1386231E-06,.1317693E-06,.1746465E-06,&
     & .2026682E-06,.1793777E-06,.1632574E-06,.1537427E-06,.1462557E-06,&
     & .1419684E-06,.1489889E-06,.1524502E-06,.2170624E-06,.1873095E-06,&
     & .1679364E-06,.1595036E-06,.1513078E-06,.1490206E-06,.1492883E-06/

      data   raylb (1: 8)      / .1320456E-06,.1720100E-06,.2114786E-06,&
     & .2446587E-06,.2048151E-06,.1382257E-06,.1353522E-06,.1198919E-06/

!  ---  o3
      data   abso3a(1: 8)      / .1300197E+00,.1988798E+00,.1272878E+00,&
     & .8885802E-01,.6659378E-01,.5166209E-01,.3931020E-01,.2659650E-01/

      data   abso3b(1: 8)      / .3634220E-01,.8528316E-01,.1832493E+00,&
     & .2884644E+00,.3568897E+00,.3534033E+00,.1043294E+00,.2575196E-01/

!     the array absa(585,NG24) (ka(9,5,13,NG24)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 8, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .9478488E-09,.2006042E-07,.2228363E-07,.2263495E-07,.2547446E-07,&
     & .2936576E-07,.3475032E-07,.4537659E-07,.1269078E-07,.9492310E-09,&
     & .2077413E-07,.2280572E-07,.2371246E-07,.2688328E-07,.3058537E-07,&
     & .3544994E-07,.4495120E-07,.1274292E-07,.1009719E-08,.2560505E-07,&
     & .2989703E-07,.3567437E-07,.4195170E-07,.4903334E-07,.5753744E-07,&
     & .7103506E-07,.2087177E-07,.1021243E-08,.2157727E-07,.2428543E-07,&
     & .2545064E-07,.2876447E-07,.3263409E-07,.3724139E-07,.4541822E-07,&
     & .1284090E-07,.1028299E-08,.2206984E-07,.2462863E-07,.2608214E-07,&
     & .2944973E-07,.3351924E-07,.3801002E-07,.4608904E-07,.1282141E-07,&
     & .1105064E-08,.2024352E-07,.2144429E-07,.2292737E-07,.2205869E-07,&
     & .2461858E-07,.2882375E-07,.3780175E-07,.1335949E-07,.1115462E-08,&
     & .2242119E-07,.2253968E-07,.2374626E-07,.2336119E-07,.2589110E-07,&
     & .2968126E-07,.3712386E-07,.1367759E-07,.1127330E-08,.2593823E-07,&
     & .3036245E-07,.3217154E-07,.3601214E-07,.4106337E-07,.4749265E-07,&
     & .5871813E-07,.2194477E-07,.1151546E-08,.2687708E-07,.3062911E-07,&
     & .3274636E-07,.3661706E-07,.4151034E-07,.4800693E-07,.5860615E-07,&
     & .2166646E-07,.1215036E-08,.2423972E-07,.2411833E-07,.2469573E-07,&
     & .2645282E-07,.2869316E-07,.3165709E-07,.3778384E-07,.1423468E-07,&
     & .1327285E-08,.1823139E-07,.2252831E-07,.2249874E-07,.2224138E-07,&
     & .2137212E-07,.2379889E-07,.3066259E-07,.3075095E-07,.1374278E-08,&
     & .2003317E-07,.2357640E-07,.2309863E-07,.2282640E-07,.2242184E-07,&
     & .2479411E-07,.3108145E-07,.3095887E-07,.1409564E-08,.2175633E-07,&
     & .2468093E-07,.2423024E-07,.2334201E-07,.2354485E-07,.2574451E-07,&
     & .3082158E-07,.3128347E-07,.1450696E-08,.2746727E-07,.3099897E-07,&
     & .3259416E-07,.3329255E-07,.3570574E-07,.3986596E-07,.4846921E-07,&
     & .4021097E-07,.1442717E-08,.2412402E-07,.2625917E-07,.2602265E-07,&
     & .2512855E-07,.2541577E-07,.2680625E-07,.3103534E-07,.3137157E-07,&
     & .1649543E-08,.1970815E-07,.2150649E-07,.2298869E-07,.2180025E-07,&
     & .2008423E-07,.1973561E-07,.2448138E-07,.8556873E-07,.1735034E-08,&
     & .2060802E-07,.2356664E-07,.2443517E-07,.2299440E-07,.2108029E-07,&
     & .2105873E-07,.2549524E-07,.8685059E-07,.1736792E-08,.2188572E-07,&
     & .2575240E-07,.2521604E-07,.2423738E-07,.2219800E-07,.2218711E-07,&
     & .2606412E-07,.8913315E-07,.1802505E-08,.2311836E-07,.2795390E-07,&
     & .2628918E-07,.2520889E-07,.2295160E-07,.2272776E-07,.2590098E-07,&
     & .9188038E-07,.1819519E-08,.2703557E-07,.3363986E-07,.3350051E-07,&
     & .3297237E-07,.3293271E-07,.3417284E-07,.4007015E-07,.1097390E-06/

      data   absa(181:315, 1)   /                                       &
     & .2076399E-08,.2299543E-07,.2011683E-07,.2214187E-07,.2287048E-07,&
     & .2069490E-07,.1709628E-07,.1987151E-07,.2613410E-06,.2054496E-08,&
     & .2464912E-07,.2181049E-07,.2435469E-07,.2449140E-07,.2201224E-07,&
     & .1834284E-07,.2047387E-07,.2681452E-06,.2212265E-08,.2619392E-07,&
     & .2373276E-07,.2675398E-07,.2535082E-07,.2318480E-07,.1957713E-07,&
     & .2126646E-07,.2766404E-06,.2373248E-08,.2771181E-07,.2568041E-07,&
     & .2923537E-07,.2639253E-07,.2388604E-07,.2054018E-07,.2213864E-07,&
     & .2841552E-06,.2343337E-08,.3064130E-07,.3236485E-07,.3551409E-07,&
     & .3395783E-07,.3172537E-07,.3073988E-07,.3350972E-07,.3192722E-06,&
     & .2376949E-08,.2980194E-07,.2387416E-07,.2049156E-07,.2087214E-07,&
     & .2167267E-07,.1825923E-07,.1653693E-07,.6372515E-06,.2667041E-08,&
     & .3135808E-07,.2549292E-07,.2250603E-07,.2333514E-07,.2247609E-07,&
     & .1950044E-07,.1700265E-07,.6563665E-06,.2966669E-08,.3250866E-07,&
     & .2727873E-07,.2457664E-07,.2584670E-07,.2376895E-07,.2074333E-07,&
     & .1753015E-07,.6745009E-06,.3145538E-08,.3465169E-07,.2866168E-07,&
     & .2652981E-07,.2797349E-07,.2493029E-07,.2162154E-07,.1839404E-07,&
     & .6921558E-06,.3048309E-08,.3670297E-07,.3037871E-07,.2846443E-07,&
     & .3043302E-07,.2606580E-07,.2240677E-07,.1923357E-07,.7083018E-06,&
     & .2954559E-08,.3887753E-07,.2869351E-07,.2420710E-07,.2014166E-07,&
     & .1897808E-07,.1853848E-07,.1389583E-07,.1017915E-05,.3249121E-08,&
     & .4105480E-07,.3073517E-07,.2582420E-07,.2186903E-07,.2106748E-07,&
     & .1950220E-07,.1466410E-07,.1065309E-05,.3560837E-08,.4307434E-07,&
     & .3307124E-07,.2749773E-07,.2349186E-07,.2343715E-07,.2046075E-07,&
     & .1541227E-07,.1097022E-05,.3768057E-08,.4535221E-07,.3522003E-07,&
     & .2913787E-07,.2516808E-07,.2572932E-07,.2163504E-07,.1608168E-07,&
     & .1134264E-05,.4127508E-08,.4772213E-07,.3727530E-07,.3084310E-07,&
     & .2724223E-07,.2780339E-07,.2276541E-07,.1676406E-07,.1166442E-05/

      data   absa(316:450, 1)   /                                       &
     & .3755131E-08,.5425837E-07,.4118146E-07,.2970906E-07,.2360125E-07,&
     & .1910884E-07,.1685495E-07,.1393367E-07,.1706962E-05,.3837129E-08,&
     & .5684497E-07,.4375453E-07,.3243058E-07,.2577570E-07,.2032717E-07,&
     & .1872850E-07,.1474043E-07,.1765060E-05,.4347070E-08,.5965052E-07,&
     & .4627956E-07,.3497038E-07,.2747824E-07,.2198991E-07,.2061735E-07,&
     & .1576341E-07,.1825492E-05,.4988663E-08,.6165437E-07,.4887741E-07,&
     & .3754106E-07,.2965672E-07,.2382663E-07,.2246982E-07,.1698318E-07,&
     & .1871583E-05,.5424215E-08,.6390254E-07,.5129696E-07,.3972602E-07,&
     & .3163844E-07,.2555544E-07,.2420387E-07,.1814940E-07,.1924155E-05,&
     & .4442173E-08,.7197256E-07,.5819392E-07,.4306671E-07,.2953792E-07,&
     & .2178359E-07,.1602856E-07,.1525612E-07,.4752921E-05,.5074649E-08,&
     & .7446008E-07,.6175165E-07,.4628955E-07,.3209104E-07,.2411698E-07,&
     & .1784882E-07,.1645798E-07,.4869510E-05,.5728645E-08,.7723379E-07,&
     & .6474879E-07,.4922829E-07,.3503195E-07,.2630379E-07,.1935594E-07,&
     & .1753886E-07,.4973365E-05,.6351806E-08,.8000072E-07,.6740072E-07,&
     & .5210377E-07,.3796839E-07,.2830818E-07,.2112504E-07,.1881638E-07,&
     & .5058268E-05,.6912437E-08,.8315033E-07,.7068889E-07,.5412660E-07,&
     & .4056832E-07,.3019131E-07,.2281027E-07,.1946796E-07,.5120717E-05,&
     & .6424864E-08,.9702677E-07,.8541300E-07,.6566892E-07,.4587460E-07,&
     & .2900504E-07,.1935626E-07,.1385212E-07,.8949001E-05,.7095455E-08,&
     & .1023503E-06,.8901069E-07,.7031849E-07,.5021834E-07,.3251464E-07,&
     & .2150478E-07,.1541222E-07,.9105756E-05,.7837477E-08,.1065872E-06,&
     & .9339944E-07,.7381282E-07,.5418142E-07,.3553682E-07,.2351485E-07,&
     & .1667046E-07,.9321311E-05,.8700945E-08,.1108036E-06,.9742466E-07,&
     & .7752400E-07,.5759385E-07,.3839799E-07,.2580432E-07,.1788370E-07,&
     & .9568035E-05,.9372952E-08,.1149751E-06,.1012313E-06,.8204487E-07,&
     & .6076544E-07,.4108120E-07,.2793641E-07,.1911302E-07,.9731271E-05/

      data   absa(451:585, 1)   /                                       &
     & .8979819E-08,.1058443E-06,.1200210E-06,.9695637E-07,.7036848E-07,&
     & .4678128E-07,.2686372E-07,.1541405E-07,.1220035E-04,.9805661E-08,&
     & .1118530E-06,.1255675E-06,.1030716E-06,.7739733E-07,.5201243E-07,&
     & .2992503E-07,.1670143E-07,.1258780E-04,.1069760E-07,.1173469E-06,&
     & .1304830E-06,.1095539E-06,.8282101E-07,.5625194E-07,.3263649E-07,&
     & .1827566E-07,.1292889E-04,.1119525E-07,.1227001E-06,.1373018E-06,&
     & .1148947E-06,.8789054E-07,.5978610E-07,.3529093E-07,.1964769E-07,&
     & .1327840E-04,.1201126E-07,.1273789E-06,.1421903E-06,.1200244E-06,&
     & .9264785E-07,.6378835E-07,.3808366E-07,.2094784E-07,.1367923E-04,&
     & .9719070E-08,.9999501E-07,.1384487E-06,.1213259E-06,.9225490E-07,&
     & .6132161E-07,.3331774E-07,.1686558E-07,.1343339E-04,.1109778E-07,&
     & .1062708E-06,.1461547E-06,.1279839E-06,.9860632E-07,.6579031E-07,&
     & .3628776E-07,.1826740E-07,.1383474E-04,.1211127E-07,.1133042E-06,&
     & .1521046E-06,.1333579E-06,.1044442E-06,.7051604E-07,.3966511E-07,&
     & .1978957E-07,.1430481E-04,.1342211E-07,.1191516E-06,.1571868E-06,&
     & .1380165E-06,.1093802E-06,.7496750E-07,.4327741E-07,.2126680E-07,&
     & .1475217E-04,.1467534E-07,.1241612E-06,.1623203E-06,.1431277E-06,&
     & .1135740E-06,.7963856E-07,.4685852E-07,.2271310E-07,.1522177E-04,&
     & .1161087E-07,.1041677E-06,.1364162E-06,.1423574E-06,.1160046E-06,&
     & .7917598E-07,.4352117E-07,.1902079E-07,.1388023E-04,.1297072E-07,&
     & .1100490E-06,.1431349E-06,.1479064E-06,.1210891E-06,.8468921E-07,&
     & .4817011E-07,.2075879E-07,.1442933E-04,.1434007E-07,.1155767E-06,&
     & .1497075E-06,.1533287E-06,.1261775E-06,.8986500E-07,.5204569E-07,&
     & .2239805E-07,.1502654E-04,.1566851E-07,.1199822E-06,.1550418E-06,&
     & .1587526E-06,.1308995E-06,.9466747E-07,.5568553E-07,.2384695E-07,&
     & .1555344E-04,.1683829E-07,.1243538E-06,.1604785E-06,.1641767E-06,&
     & .1360688E-06,.9947999E-07,.5921603E-07,.2546256E-07,.1596362E-04/

      data   absa(  1:180, 2)   /                                       &
     & .4977875E-08,.2921108E-06,.5418060E-06,.7727212E-06,.9911457E-06,&
     & .1214800E-05,.1465814E-05,.1829868E-05,.7823624E-06,.4994907E-08,&
     & .3018351E-06,.5578764E-06,.7940989E-06,.1016090E-05,.1239397E-05,&
     & .1487931E-05,.1836976E-05,.7862442E-06,.5208110E-08,.3874388E-06,&
     & .7181819E-06,.1020477E-05,.1309594E-05,.1599138E-05,.1909038E-05,&
     & .2327755E-05,.1153932E-05,.5319118E-08,.3178392E-06,.5822238E-06,&
     & .8242852E-06,.1050880E-05,.1277685E-05,.1520218E-05,.1852230E-05,&
     & .7871951E-06,.5556151E-08,.3235154E-06,.5911725E-06,.8338245E-06,&
     & .1062185E-05,.1291143E-05,.1532737E-05,.1857528E-05,.7845580E-06,&
     & .5432943E-08,.2394007E-06,.4471522E-06,.6385952E-06,.8269131E-06,&
     & .1017096E-05,.1232743E-05,.1559228E-05,.6244771E-06,.5892629E-08,&
     & .2459631E-06,.4597979E-06,.6563874E-06,.8463292E-06,.1038052E-05,&
     & .1251153E-05,.1563781E-05,.6288295E-06,.5915709E-08,.3242534E-06,&
     & .6031237E-06,.8632595E-06,.1113129E-05,.1365831E-05,.1640492E-05,&
     & .2027362E-05,.9462585E-06,.6171888E-08,.3261724E-06,.6054889E-06,&
     & .8640996E-06,.1112880E-05,.1361041E-05,.1630551E-05,.2003512E-05,&
     & .9272718E-06,.6400658E-08,.2632985E-06,.4872328E-06,.6900240E-06,&
     & .8829870E-06,.1077466E-05,.1285821E-05,.1578521E-05,.6307027E-06,&
     & .6718273E-08,.1967264E-06,.3613932E-06,.5196750E-06,.6756207E-06,&
     & .8399391E-06,.1024899E-05,.1319338E-05,.4959268E-06,.7019662E-08,&
     & .2019256E-06,.3715815E-06,.5349054E-06,.6929146E-06,.8570698E-06,&
     & .1040099E-05,.1317826E-05,.5026796E-06,.6849002E-08,.2066083E-06,&
     & .3800896E-06,.5461625E-06,.7072097E-06,.8714979E-06,.1051566E-05,&
     & .1319363E-05,.5066881E-06,.7348552E-08,.2698050E-06,.5015846E-06,&
     & .7193981E-06,.9328462E-06,.1147717E-05,.1384165E-05,.1724206E-05,&
     & .7547895E-06,.7364360E-08,.2140724E-06,.3941245E-06,.5620667E-06,&
     & .7253341E-06,.8905787E-06,.1068093E-05,.1327568E-05,.5107295E-06,&
     & .7188660E-08,.1608987E-06,.2919364E-06,.4183428E-06,.5474637E-06,&
     & .6862098E-06,.8454628E-06,.1112417E-05,.7753418E-06,.7923510E-08,&
     & .1660346E-06,.2994130E-06,.4302319E-06,.5615672E-06,.7001230E-06,&
     & .8570141E-06,.1102211E-05,.7947637E-06,.8710415E-08,.1699772E-06,&
     & .3051256E-06,.4401873E-06,.5730835E-06,.7124193E-06,.8673643E-06,&
     & .1099678E-05,.8076872E-06,.9379724E-08,.1733439E-06,.3101037E-06,&
     & .4482693E-06,.5824117E-06,.7220472E-06,.8751372E-06,.1103747E-05,&
     & .8157169E-06,.9693296E-08,.2255703E-06,.4099284E-06,.5915659E-06,&
     & .7704284E-06,.9518875E-06,.1153151E-05,.1447965E-05,.9509966E-06/

      data   absa(181:315, 2)   /                                       &
     & .8742219E-08,.1401619E-06,.2365810E-06,.3367232E-06,.4389881E-06,&
     & .5544262E-06,.6941176E-06,.9221962E-06,.1547996E-05,.9825526E-08,&
     & .1439063E-06,.2438132E-06,.3455625E-06,.4507360E-06,.5666773E-06,&
     & .7024141E-06,.9173596E-06,.1577601E-05,.1032729E-07,.1470757E-06,&
     & .2492266E-06,.3522677E-06,.4609870E-06,.5767649E-06,.7112768E-06,&
     & .9130431E-06,.1601942E-05,.1052928E-07,.1495088E-06,.2530720E-06,&
     & .3574969E-06,.4695458E-06,.5854586E-06,.7174989E-06,.9137192E-06,&
     & .1618037E-05,.1150457E-07,.1903978E-06,.3372032E-06,.4819982E-06,&
     & .6322987E-06,.7870872E-06,.9614418E-06,.1220624E-05,.1804705E-05,&
     & .1081727E-07,.1563034E-06,.1929738E-06,.2720043E-06,.3525847E-06,&
     & .4442876E-06,.5631051E-06,.7626249E-06,.2334044E-05,.1150244E-07,&
     & .1599274E-06,.1989339E-06,.2804224E-06,.3618620E-06,.4559776E-06,&
     & .5702232E-06,.7643768E-06,.2382319E-05,.1175189E-07,.1620605E-06,&
     & .2044243E-06,.2860895E-06,.3693875E-06,.4646579E-06,.5769806E-06,&
     & .7565793E-06,.2416745E-05,.1306718E-07,.1630619E-06,.2085618E-06,&
     & .2905226E-06,.3757709E-06,.4717225E-06,.5824128E-06,.7540595E-06,&
     & .2439861E-05,.1426020E-07,.1645534E-06,.2117139E-06,.2947798E-06,&
     & .3794331E-06,.4775901E-06,.5875284E-06,.7558601E-06,.2466640E-05,&
     & .1207434E-07,.1674269E-06,.1791693E-06,.2187708E-06,.2826358E-06,&
     & .3574471E-06,.4560189E-06,.6292048E-06,.4950228E-05,.1327450E-07,&
     & .1717629E-06,.1844629E-06,.2269350E-06,.2920013E-06,.3674358E-06,&
     & .4607900E-06,.6289581E-06,.4970699E-05,.1458123E-07,.1756793E-06,&
     & .1880198E-06,.2332289E-06,.2991457E-06,.3736616E-06,.4662710E-06,&
     & .6243034E-06,.5001187E-05,.1616738E-07,.1790958E-06,.1914327E-06,&
     & .2381938E-06,.3052274E-06,.3787455E-06,.4719405E-06,.6206030E-06,&
     & .5028994E-05,.1738758E-07,.1816052E-06,.1943182E-06,.2415142E-06,&
     & .3091524E-06,.3827622E-06,.4755721E-06,.6213090E-06,.5041895E-05/

      data   absa(316:450, 2)   /                                       &
     & .1665196E-07,.1938979E-06,.2113958E-06,.2003810E-06,.2278096E-06,&
     & .2865973E-06,.3718779E-06,.5164350E-06,.1076729E-04,.1817122E-07,&
     & .1993569E-06,.2175471E-06,.2075727E-06,.2358089E-06,.2950307E-06,&
     & .3720816E-06,.5129796E-06,.1080841E-04,.1956687E-07,.2035852E-06,&
     & .2222226E-06,.2131321E-06,.2435564E-06,.3019781E-06,.3754017E-06,&
     & .5134335E-06,.1082806E-04,.2070597E-07,.2074456E-06,.2260920E-06,&
     & .2168666E-06,.2488899E-06,.3073148E-06,.3798904E-06,.5077018E-06,&
     & .1084943E-04,.2173248E-07,.2098779E-06,.2294201E-06,.2205251E-06,&
     & .2538736E-06,.3118254E-06,.3821669E-06,.5055703E-06,.1085222E-04,&
     & .2191248E-07,.2365055E-06,.2390465E-06,.2374263E-06,.2072159E-06,&
     & .2312165E-06,.3013057E-06,.4184984E-06,.2723092E-04,.2361382E-07,&
     & .2462069E-06,.2472867E-06,.2475309E-06,.2196618E-06,.2383569E-06,&
     & .3008372E-06,.4161238E-06,.2736074E-04,.2508833E-07,.2530970E-06,&
     & .2541614E-06,.2555112E-06,.2280678E-06,.2448032E-06,.3033101E-06,&
     & .4166889E-06,.2760212E-04,.2600366E-07,.2555124E-06,.2601272E-06,&
     & .2621345E-06,.2330125E-06,.2502924E-06,.3081806E-06,.4117555E-06,&
     & .2781471E-04,.2625556E-07,.2585939E-06,.2658891E-06,.2667422E-06,&
     & .2373911E-06,.2553542E-06,.3106119E-06,.4101077E-06,.2799279E-04,&
     & .2786381E-07,.2663930E-06,.3136530E-06,.2903642E-06,.2699185E-06,&
     & .2154845E-06,.2415255E-06,.3414221E-06,.6623836E-04,.3081478E-07,&
     & .2743516E-06,.3245697E-06,.3003308E-06,.2804576E-06,.2242044E-06,&
     & .2415067E-06,.3388596E-06,.6704072E-04,.3288814E-07,.2818415E-06,&
     & .3322102E-06,.3116516E-06,.2890397E-06,.2345975E-06,.2455989E-06,&
     & .3380705E-06,.6784378E-04,.3384756E-07,.2878021E-06,.3383139E-06,&
     & .3187847E-06,.2960257E-06,.2435067E-06,.2492429E-06,.3358934E-06,&
     & .6873920E-04,.3644271E-07,.2922944E-06,.3431136E-06,.3228649E-06,&
     & .3009446E-06,.2515472E-06,.2527789E-06,.3321823E-06,.6964582E-04/

      data   absa(451:585, 2)   /                                       &
     & .4098843E-07,.3384011E-06,.3624859E-06,.3751623E-06,.3279197E-06,&
     & .2825050E-06,.2084273E-06,.2761760E-06,.8850341E-04,.4683147E-07,&
     & .3426868E-06,.3709702E-06,.3853158E-06,.3371326E-06,.2929673E-06,&
     & .2151462E-06,.2724341E-06,.8924994E-04,.4984644E-07,.3465890E-06,&
     & .3781183E-06,.3930485E-06,.3467252E-06,.3026766E-06,.2220474E-06,&
     & .2731323E-06,.9000607E-04,.5357426E-07,.3496935E-06,.3823149E-06,&
     & .4006097E-06,.3557600E-06,.3109094E-06,.2298146E-06,.2691423E-06,&
     & .9077335E-04,.5649178E-07,.3523119E-06,.3864299E-06,.4065303E-06,&
     & .3608387E-06,.3158353E-06,.2374848E-06,.2683036E-06,.9190915E-04,&
     & .5606804E-07,.3942048E-06,.3696286E-06,.3914481E-06,.3666506E-06,&
     & .3045442E-06,.2355668E-06,.2246247E-06,.9830605E-04,.5792873E-07,&
     & .3951960E-06,.3767630E-06,.4000468E-06,.3765494E-06,.3169486E-06,&
     & .2468239E-06,.2208278E-06,.1003406E-03,.6144867E-07,.3964744E-06,&
     & .3840637E-06,.4079564E-06,.3868875E-06,.3269661E-06,.2582322E-06,&
     & .2208739E-06,.1013645E-03,.6347446E-07,.3944624E-06,.3901494E-06,&
     & .4142722E-06,.3972735E-06,.3364436E-06,.2683503E-06,.2181227E-06,&
     & .1025965E-03,.6571022E-07,.3935800E-06,.3934698E-06,.4190604E-06,&
     & .4047063E-06,.3426999E-06,.2756876E-06,.2187860E-06,.1036640E-03,&
     & .6947980E-07,.4277234E-06,.4260311E-06,.3985021E-06,.3938855E-06,&
     & .3382988E-06,.2596779E-06,.1845908E-06,.1007433E-03,.7232340E-07,&
     & .4309643E-06,.4284737E-06,.4112915E-06,.4041636E-06,.3526595E-06,&
     & .2709767E-06,.1835141E-06,.1027545E-03,.7385675E-07,.4323241E-06,&
     & .4337810E-06,.4231404E-06,.4150882E-06,.3638010E-06,.2833177E-06,&
     & .1825627E-06,.1048033E-03,.7472322E-07,.4340229E-06,.4375142E-06,&
     & .4293950E-06,.4227926E-06,.3709701E-06,.2917057E-06,.1845645E-06,&
     & .1071308E-03,.7710708E-07,.4377958E-06,.4436351E-06,.4362045E-06,&
     & .4261614E-06,.3773780E-06,.2982907E-06,.1879122E-06,.1086672E-03/

      data   absa(  1:180, 3)   /                                       &
     & .7885147E-08,.2480177E-05,.4465798E-05,.6288079E-05,.8021775E-05,&
     & .9713012E-05,.1149293E-04,.1389792E-04,.9583620E-05,.8337260E-08,&
     & .2533549E-05,.4538671E-05,.6376335E-05,.8116846E-05,.9823186E-05,&
     & .1160281E-04,.1396043E-04,.9711327E-05,.8347835E-08,.2875505E-05,&
     & .5166441E-05,.7269794E-05,.9276602E-05,.1119934E-04,.1322573E-04,&
     & .1573805E-04,.1164942E-04,.8341010E-08,.2624899E-05,.4665526E-05,&
     & .6527056E-05,.8285471E-05,.9970463E-05,.1177728E-04,.1416190E-04,&
     & .9853793E-05,.8441087E-08,.2662384E-05,.4716519E-05,.6584497E-05,&
     & .8347227E-05,.1002746E-04,.1181802E-04,.1420893E-04,.9876490E-05,&
     & .9538043E-08,.2172518E-05,.3916041E-05,.5520408E-05,.7047709E-05,&
     & .8561064E-05,.1016253E-04,.1235053E-04,.8228063E-05,.9896959E-08,&
     & .2218811E-05,.3982498E-05,.5600469E-05,.7138336E-05,.8647352E-05,&
     & .1025838E-04,.1240949E-04,.8337951E-05,.9725183E-08,.2562004E-05,&
     & .4611611E-05,.6497303E-05,.8305798E-05,.1005129E-04,.1190483E-04,&
     & .1422914E-04,.1028529E-04,.9815185E-08,.2580610E-05,.4626595E-05,&
     & .6503599E-05,.8291966E-05,.1002549E-04,.1185886E-04,.1419713E-04,&
     & .1021346E-04,.9353012E-08,.2330951E-05,.4132274E-05,.5770881E-05,&
     & .7327013E-05,.8826693E-05,.1043691E-04,.1262749E-04,.8444115E-05,&
     & .1055499E-07,.1876951E-05,.3395021E-05,.4794118E-05,.6129975E-05,&
     & .7457686E-05,.8897848E-05,.1089769E-04,.6935281E-05,.1082740E-07,&
     & .1919857E-05,.3454800E-05,.4863748E-05,.6212902E-05,.7539732E-05,&
     & .8988633E-05,.1093165E-04,.7014326E-05,.1237586E-07,.1957445E-05,&
     & .3508502E-05,.4924591E-05,.6283591E-05,.7606837E-05,.9064706E-05,&
     & .1103621E-04,.7066774E-05,.1310556E-07,.2280479E-05,.4102438E-05,&
     & .5774148E-05,.7371455E-05,.8939771E-05,.1061982E-04,.1277638E-04,&
     & .8882241E-05,.1381953E-07,.2018379E-05,.3585970E-05,.5012832E-05,&
     & .6367746E-05,.7692141E-05,.9142910E-05,.1113079E-04,.7092611E-05,&
     & .1302165E-07,.1600407E-05,.2913854E-05,.4120799E-05,.5279357E-05,&
     & .6430192E-05,.7712566E-05,.9529299E-05,.5293062E-05,.1389129E-07,&
     & .1639258E-05,.2967097E-05,.4184307E-05,.5353035E-05,.6512557E-05,&
     & .7798666E-05,.9560399E-05,.5349139E-05,.1483348E-07,.1673536E-05,&
     & .3015174E-05,.4237461E-05,.5413790E-05,.6575943E-05,.7862585E-05,&
     & .9658800E-05,.5382522E-05,.1563210E-07,.1703299E-05,.3054374E-05,&
     & .4280442E-05,.5458053E-05,.6620462E-05,.7906329E-05,.9704216E-05,&
     & .5391051E-05,.1599402E-07,.2006384E-05,.3612837E-05,.5084142E-05,&
     & .6481925E-05,.7864478E-05,.9387955E-05,.1136686E-04,.7116652E-05/

      data   absa(181:315, 3)   /                                       &
     & .1759652E-07,.1338435E-05,.2477474E-05,.3510671E-05,.4506288E-05,&
     & .5498399E-05,.6620804E-05,.8293824E-05,.3301450E-05,.1839690E-07,&
     & .1372726E-05,.2524704E-05,.3567406E-05,.4571892E-05,.5574072E-05,&
     & .6701396E-05,.8294132E-05,.3333569E-05,.1844120E-07,.1403022E-05,&
     & .2565887E-05,.3614241E-05,.4624775E-05,.5628533E-05,.6758496E-05,&
     & .8374936E-05,.3343845E-05,.2053628E-07,.1429066E-05,.2600703E-05,&
     & .3651806E-05,.4660425E-05,.5666649E-05,.6798791E-05,.8414165E-05,&
     & .3333421E-05,.2112872E-07,.1738328E-05,.3156880E-05,.4451390E-05,&
     & .5679788E-05,.6911108E-05,.8277220E-05,.1010231E-04,.4801846E-05,&
     & .2138930E-07,.1060078E-05,.2077209E-05,.2959831E-05,.3807513E-05,&
     & .4658382E-05,.5629044E-05,.7177701E-05,.5287833E-05,.2197261E-07,&
     & .1090324E-05,.2118891E-05,.3009588E-05,.3867726E-05,.4725436E-05,&
     & .5702333E-05,.7135044E-05,.5295206E-05,.2370908E-07,.1117799E-05,&
     & .2154519E-05,.3051661E-05,.3914248E-05,.4773139E-05,.5753687E-05,&
     & .7202983E-05,.5282002E-05,.2418585E-07,.1140836E-05,.2184495E-05,&
     & .3085157E-05,.3945767E-05,.4806508E-05,.5792297E-05,.7235984E-05,&
     & .5240017E-05,.2563587E-07,.1157256E-05,.2206672E-05,.3110571E-05,&
     & .3966388E-05,.4824447E-05,.5812975E-05,.7247817E-05,.5173282E-05,&
     & .2761854E-07,.8192162E-06,.1695308E-05,.2467470E-05,.3187895E-05,&
     & .3913885E-05,.4745695E-05,.6132573E-05,.8445327E-05,.2791826E-07,&
     & .8431674E-06,.1731404E-05,.2511477E-05,.3242547E-05,.3969814E-05,&
     & .4810419E-05,.6100692E-05,.8456733E-05,.2941384E-07,.8642535E-06,&
     & .1763447E-05,.2547427E-05,.3283616E-05,.4013509E-05,.4856118E-05,&
     & .6137849E-05,.8444823E-05,.3093995E-07,.8803612E-06,.1788691E-05,&
     & .2576812E-05,.3311827E-05,.4043744E-05,.4889006E-05,.6164513E-05,&
     & .8438944E-05,.3175649E-07,.8926786E-06,.1807355E-05,.2599811E-05,&
     & .3329744E-05,.4060525E-05,.4908054E-05,.6172563E-05,.8415014E-05/

      data   absa(316:450, 3)   /                                       &
     & .3594320E-07,.5898798E-06,.1300990E-05,.2005624E-05,.2639841E-05,&
     & .3258662E-05,.3970239E-05,.5168834E-05,.1389043E-04,.3842574E-07,&
     & .6079368E-06,.1330795E-05,.2043110E-05,.2689852E-05,.3308616E-05,&
     & .4024395E-05,.5188513E-05,.1404319E-04,.4003510E-07,.6246941E-06,&
     & .1356682E-05,.2074572E-05,.2724323E-05,.3347997E-05,.4066594E-05,&
     & .5184194E-05,.1421675E-04,.4109691E-07,.6372206E-06,.1377478E-05,&
     & .2099971E-05,.2748886E-05,.3373994E-05,.4094365E-05,.5206120E-05,&
     & .1431172E-04,.4160609E-07,.6467677E-06,.1391926E-05,.2119005E-05,&
     & .2763924E-05,.3389851E-05,.4111985E-05,.5213753E-05,.1436877E-04,&
     & .4460131E-07,.3996422E-06,.9634371E-06,.1535190E-05,.2135126E-05,&
     & .2688150E-05,.3299750E-05,.4333189E-05,.2704843E-04,.4831887E-07,&
     & .4098677E-06,.9845303E-06,.1564750E-05,.2173466E-05,.2731313E-05,&
     & .3343042E-05,.4388007E-05,.2787395E-04,.5033440E-07,.4173500E-06,&
     & .1002962E-05,.1587816E-05,.2202008E-05,.2765444E-05,.3380831E-05,&
     & .4346875E-05,.2836708E-04,.5144378E-07,.4256035E-06,.1017999E-05,&
     & .1606384E-05,.2223268E-05,.2789015E-05,.3404990E-05,.4364321E-05,&
     & .2885698E-04,.5376703E-07,.4301287E-06,.1025925E-05,.1621665E-05,&
     & .2236412E-05,.2802339E-05,.3420850E-05,.4370319E-05,.2935788E-04,&
     & .7159604E-07,.4940214E-06,.6385949E-06,.1108740E-05,.1601203E-05,&
     & .2163439E-05,.2727326E-05,.3598975E-05,.4606013E-04,.7586834E-07,&
     & .4934774E-06,.6516736E-06,.1132261E-05,.1634648E-05,.2197928E-05,&
     & .2762190E-05,.3675388E-05,.4725942E-04,.7940266E-07,.4992704E-06,&
     & .6618015E-06,.1146976E-05,.1656272E-05,.2222531E-05,.2791013E-05,&
     & .3629707E-05,.4765531E-04,.8216318E-07,.4976479E-06,.6706045E-06,&
     & .1160539E-05,.1671805E-05,.2236388E-05,.2810390E-05,.3635786E-05,&
     & .4814584E-04,.8519042E-07,.4933370E-06,.6756937E-06,.1170724E-05,&
     & .1681781E-05,.2243295E-05,.2821977E-05,.3642627E-05,.4872656E-04/

      data   absa(451:585, 3)   /                                       &
     & .1081928E-06,.5818788E-06,.6386596E-06,.7432114E-06,.1166448E-05,&
     & .1632769E-05,.2234855E-05,.3011606E-05,.5082881E-04,.1059761E-06,&
     & .5865974E-06,.6439649E-06,.7553534E-06,.1187103E-05,.1654747E-05,&
     & .2252083E-05,.3058057E-05,.5151287E-04,.1087338E-06,.5874771E-06,&
     & .6413861E-06,.7655310E-06,.1198176E-05,.1670032E-05,.2266997E-05,&
     & .3012908E-05,.5289813E-04,.1114046E-06,.5874841E-06,.6406983E-06,&
     & .7727642E-06,.1203345E-05,.1677472E-05,.2275155E-05,.3017584E-05,&
     & .5372932E-04,.1149838E-06,.5849412E-06,.6431792E-06,.7741413E-06,&
     & .1208047E-05,.1681178E-05,.2276094E-05,.3015110E-05,.5370886E-04,&
     & .1308163E-06,.5923091E-06,.7055074E-06,.6422362E-06,.8303543E-06,&
     & .1235001E-05,.1751640E-05,.2496860E-05,.5055594E-04,.1362223E-06,&
     & .6024702E-06,.7146505E-06,.6458685E-06,.8419302E-06,.1248937E-05,&
     & .1758098E-05,.2520128E-05,.4923848E-04,.1383439E-06,.6090485E-06,&
     & .7121358E-06,.6578181E-06,.8468564E-06,.1256934E-05,.1760117E-05,&
     & .2491841E-05,.5060877E-04,.1418196E-06,.6173069E-06,.7165570E-06,&
     & .6647670E-06,.8470868E-06,.1257404E-05,.1760260E-05,.2490114E-05,&
     & .5072850E-04,.1467094E-06,.6274341E-06,.7144183E-06,.6594283E-06,&
     & .8460663E-06,.1255755E-05,.1758322E-05,.2483648E-05,.5190473E-04,&
     & .1679150E-06,.6319279E-06,.7600366E-06,.7654140E-06,.6345031E-06,&
     & .8895555E-06,.1341549E-05,.2061106E-05,.5170435E-04,.1687483E-06,&
     & .6398709E-06,.7658349E-06,.7599068E-06,.6394553E-06,.8948407E-06,&
     & .1341176E-05,.2062854E-05,.5191565E-04,.1856866E-06,.6469542E-06,&
     & .7612484E-06,.7510875E-06,.6399811E-06,.8962538E-06,.1338671E-05,&
     & .2043994E-05,.5247877E-04,.1946851E-06,.6555257E-06,.7574664E-06,&
     & .7531901E-06,.6416353E-06,.8973155E-06,.1338204E-05,.2036318E-05,&
     & .5249060E-04,.1959063E-06,.6536393E-06,.7490042E-06,.7486915E-06,&
     & .6419812E-06,.8937247E-06,.1335333E-05,.2025525E-05,.5294532E-04/

      data   absa(  1:180, 4)   /                                       &
     & .1305351E-05,.1815013E-04,.2979305E-04,.3999652E-04,.4905169E-04,&
     & .5693488E-04,.6409806E-04,.7177995E-04,.6208906E-04,.1399449E-05,&
     & .1830100E-04,.3003956E-04,.4034832E-04,.4948205E-04,.5741995E-04,&
     & .6471188E-04,.7264387E-04,.6277389E-04,.1500703E-05,.1907688E-04,&
     & .3148533E-04,.4244547E-04,.5212651E-04,.6064266E-04,.6853163E-04,&
     & .7718017E-04,.6793589E-04,.1602781E-05,.1859572E-04,.3046861E-04,&
     & .4090220E-04,.5010077E-04,.5818133E-04,.6567877E-04,.7391640E-04,&
     & .6394695E-04,.1701735E-05,.1874689E-04,.3064865E-04,.4112164E-04,&
     & .5031423E-04,.5845409E-04,.6607977E-04,.7447772E-04,.6440095E-04,&
     & .1064649E-05,.1659392E-04,.2728440E-04,.3661825E-04,.4488879E-04,&
     & .5216469E-04,.5901946E-04,.6679678E-04,.5636200E-04,.1144486E-05,&
     & .1674885E-04,.2754084E-04,.3698113E-04,.4529662E-04,.5265143E-04,&
     & .5963365E-04,.6763659E-04,.5703090E-04,.1229283E-05,.1755590E-04,&
     & .2904804E-04,.3914394E-04,.4801072E-04,.5596306E-04,.6357576E-04,&
     & .7232186E-04,.6241105E-04,.1311915E-05,.1765175E-04,.2914597E-04,&
     & .3924495E-04,.4810414E-04,.5607476E-04,.6377001E-04,.7264059E-04,&
     & .6255768E-04,.1393667E-05,.1719840E-04,.2815613E-04,.3775091E-04,&
     & .4611483E-04,.5365819E-04,.6097830E-04,.6951518E-04,.5857723E-04,&
     & .8510123E-06,.1505886E-04,.2479646E-04,.3323319E-04,.4067061E-04,&
     & .4732001E-04,.5376955E-04,.6151348E-04,.5054458E-04,.9176962E-06,&
     & .1522049E-04,.2506909E-04,.3359909E-04,.4107044E-04,.4781093E-04,&
     & .5439483E-04,.6241934E-04,.5123952E-04,.9852056E-06,.1537278E-04,&
     & .2530125E-04,.3390810E-04,.4140644E-04,.4822544E-04,.5493151E-04,&
     & .6311719E-04,.5182421E-04,.1051221E-05,.1616403E-04,.2674998E-04,&
     & .3597079E-04,.4404185E-04,.5143084E-04,.5876151E-04,.6767562E-04,&
     & .5703403E-04,.1116703E-05,.1566305E-04,.2568848E-04,.3436963E-04,&
     & .4192613E-04,.4885053E-04,.5577076E-04,.6434502E-04,.5271676E-04,&
     & .6706972E-06,.1356834E-04,.2234016E-04,.2988013E-04,.3652444E-04,&
     & .4257018E-04,.4855089E-04,.5610533E-04,.4513334E-04,.7245362E-06,&
     & .1373634E-04,.2262553E-04,.3024618E-04,.3692628E-04,.4305715E-04,&
     & .4917507E-04,.5705200E-04,.4580726E-04,.7792176E-06,.1389124E-04,&
     & .2286472E-04,.3055718E-04,.3727285E-04,.4346813E-04,.4973107E-04,&
     & .5777942E-04,.4638437E-04,.8325154E-06,.1403502E-04,.2307317E-04,&
     & .3081856E-04,.3756897E-04,.4382456E-04,.5020863E-04,.5845074E-04,&
     & .4687437E-04,.8857411E-06,.1481254E-04,.2449491E-04,.3281306E-04,&
     & .4013490E-04,.4697409E-04,.5391258E-04,.6287210E-04,.5200378E-04/

      data   absa(181:315, 4)   /                                       &
     & .5167508E-06,.1213015E-04,.1994761E-04,.2666478E-04,.3257689E-04,&
     & .3801490E-04,.4351481E-04,.5071048E-04,.3970568E-04,.5611450E-06,&
     & .1230180E-04,.2023486E-04,.2701994E-04,.3297257E-04,.3850691E-04,&
     & .4413473E-04,.5168590E-04,.4035843E-04,.6074942E-06,.1245708E-04,&
     & .2048237E-04,.2733236E-04,.3332457E-04,.3893535E-04,.4469849E-04,&
     & .5243452E-04,.4091681E-04,.6497091E-06,.1260117E-04,.2070243E-04,&
     & .2759760E-04,.3363226E-04,.3930008E-04,.4517120E-04,.5311341E-04,&
     & .4140540E-04,.6918234E-06,.1342830E-04,.2222680E-04,.2973432E-04,&
     & .3639360E-04,.4267235E-04,.4915864E-04,.5781325E-04,.4703066E-04,&
     & .3889111E-06,.1075212E-04,.1766146E-04,.2361401E-04,.2885114E-04,&
     & .3370300E-04,.3869751E-04,.4540779E-04,.2843576E-04,.4255971E-06,&
     & .1092624E-04,.1794903E-04,.2396544E-04,.2925012E-04,.3420204E-04,&
     & .3932564E-04,.4642134E-04,.2904572E-04,.4617245E-06,.1108153E-04,&
     & .1820339E-04,.2427801E-04,.2960841E-04,.3464423E-04,.3989616E-04,&
     & .4717937E-04,.2958696E-04,.4963196E-06,.1122268E-04,.1843067E-04,&
     & .2454397E-04,.2992123E-04,.3501424E-04,.4036504E-04,.4787143E-04,&
     & .3007661E-04,.5288712E-06,.1136116E-04,.1862698E-04,.2476416E-04,&
     & .3018301E-04,.3532236E-04,.4075287E-04,.4844677E-04,.3048386E-04,&
     & .2818201E-06,.9454122E-05,.1552584E-04,.2075905E-04,.2537253E-04,&
     & .2967483E-04,.3416421E-04,.4036992E-04,.1358631E-04,.3116980E-06,&
     & .9630115E-05,.1581178E-04,.2110576E-04,.2577640E-04,.3017903E-04,&
     & .3479450E-04,.4135899E-04,.1411977E-04,.3391436E-06,.9785821E-05,&
     & .1606901E-04,.2141705E-04,.2613954E-04,.3061951E-04,.3535944E-04,&
     & .4214515E-04,.1459742E-04,.3649969E-06,.9929644E-05,.1629735E-04,&
     & .2168157E-04,.2645134E-04,.3099353E-04,.3582904E-04,.4283092E-04,&
     & .1493885E-04,.3909595E-06,.1006778E-04,.1649654E-04,.2190347E-04,&
     & .2671526E-04,.3130880E-04,.3622231E-04,.4340255E-04,.1525050E-04/

      data   absa(316:450, 4)   /                                       &
     & .1885550E-06,.8237115E-05,.1355115E-04,.1812203E-04,.2216406E-04,&
     & .2595910E-04,.2995607E-04,.3567120E-04,.1531929E-04,.2087836E-06,&
     & .8413745E-05,.1383332E-04,.1846337E-04,.2256351E-04,.2645687E-04,&
     & .3058162E-04,.3655441E-04,.1558368E-04,.2296238E-06,.8571555E-05,&
     & .1408726E-04,.1876607E-04,.2292379E-04,.2689093E-04,.3112883E-04,&
     & .3737211E-04,.1557312E-04,.2500010E-06,.8719796E-05,.1431412E-04,&
     & .1902641E-04,.2323430E-04,.2726681E-04,.3159471E-04,.3804543E-04,&
     & .1578427E-04,.2711853E-06,.8859111E-05,.1451202E-04,.1924827E-04,&
     & .2349894E-04,.2758553E-04,.3199143E-04,.3860292E-04,.1588112E-04,&
     & .1230406E-06,.7071075E-05,.1173600E-04,.1570803E-04,.1922851E-04,&
     & .2255710E-04,.2608910E-04,.3126942E-04,.3216989E-04,.1333225E-06,&
     & .7245254E-05,.1201186E-04,.1604140E-04,.1961962E-04,.2304330E-04,&
     & .2669918E-04,.3205734E-04,.3251376E-04,.1467059E-06,.7408812E-05,&
     & .1226050E-04,.1633677E-04,.1997347E-04,.2347166E-04,.2723239E-04,&
     & .3289453E-04,.3282901E-04,.1619556E-06,.7561287E-05,.1248084E-04,&
     & .1659177E-04,.2028185E-04,.2384470E-04,.2769227E-04,.3354727E-04,&
     & .3315058E-04,.1762633E-06,.7701606E-05,.1267408E-04,.1681320E-04,&
     & .2054736E-04,.2416501E-04,.2808628E-04,.3409678E-04,.3312584E-04,&
     & .1086826E-06,.5647058E-05,.1006035E-04,.1354169E-04,.1659919E-04,&
     & .1950529E-04,.2261606E-04,.2728891E-04,.4618372E-04,.1056253E-06,&
     & .5824652E-05,.1032706E-04,.1386088E-04,.1697733E-04,.1997451E-04,&
     & .2320443E-04,.2799400E-04,.4673866E-04,.1091752E-06,.5977988E-05,&
     & .1056852E-04,.1414636E-04,.1732210E-04,.2039088E-04,.2372048E-04,&
     & .2879946E-04,.4859587E-04,.1162781E-06,.6130784E-05,.1078173E-04,&
     & .1439326E-04,.1762377E-04,.2075664E-04,.2416964E-04,.2942961E-04,&
     & .4852916E-04,.1200840E-06,.6274589E-05,.1096931E-04,.1461013E-04,&
     & .1788556E-04,.2107397E-04,.2455553E-04,.2995813E-04,.4904614E-04/

      data   absa(451:585, 4)   /                                       &
     & .1675748E-06,.4418138E-05,.8301726E-05,.1168525E-04,.1439259E-04,&
     & .1696072E-04,.1973556E-04,.2395464E-04,.4468651E-04,.1734702E-06,&
     & .4573506E-05,.8548681E-05,.1197974E-04,.1474159E-04,.1738826E-04,&
     & .2027162E-04,.2462762E-04,.4554179E-04,.1739972E-06,.4723577E-05,&
     & .8779827E-05,.1223847E-04,.1505819E-04,.1776729E-04,.2074409E-04,&
     & .2535087E-04,.4445044E-04,.1750928E-06,.4863503E-05,.8978592E-05,&
     & .1246324E-04,.1533399E-04,.1810410E-04,.2114969E-04,.2591012E-04,&
     & .4445933E-04,.1808266E-06,.4993518E-05,.9148942E-05,.1266696E-04,&
     & .1557959E-04,.1839978E-04,.2150457E-04,.2639122E-04,.4563702E-04,&
     & .2010952E-06,.3480775E-05,.6740064E-05,.9826566E-05,.1239046E-04,&
     & .1466668E-04,.1712740E-04,.2092359E-04,.4828846E-04,.2004906E-06,&
     & .3617594E-05,.6950190E-05,.1010039E-04,.1271291E-04,.1505595E-04,&
     & .1761624E-04,.2155447E-04,.4984876E-04,.2193992E-06,.3747107E-05,&
     & .7152981E-05,.1032882E-04,.1300048E-04,.1539966E-04,.1804326E-04,&
     & .2217884E-04,.4995284E-04,.2317770E-06,.3868823E-05,.7322475E-05,&
     & .1053445E-04,.1325413E-04,.1570896E-04,.1841202E-04,.2268552E-04,&
     & .5167058E-04,.2292152E-06,.3975147E-05,.7484804E-05,.1073491E-04,&
     & .1348680E-04,.1598438E-04,.1873862E-04,.2312606E-04,.4992244E-04,&
     & .2573677E-06,.2629386E-05,.5319732E-05,.7892318E-05,.1048730E-04,&
     & .1260461E-04,.1478749E-04,.1816822E-04,.4948787E-04,.2535694E-06,&
     & .2751139E-05,.5519585E-05,.8143370E-05,.1078554E-04,.1295471E-04,&
     & .1522905E-04,.1876027E-04,.5038619E-04,.2296209E-06,.2871168E-05,&
     & .5705427E-05,.8368314E-05,.1104763E-04,.1326931E-04,.1561214E-04,&
     & .1930238E-04,.4875010E-04,.2285283E-06,.2976458E-05,.5870383E-05,&
     & .8562471E-05,.1128721E-04,.1355655E-04,.1595638E-04,.1975997E-04,&
     & .4655490E-04,.2277949E-06,.3086384E-05,.6023967E-05,.8743382E-05,&
     & .1150669E-04,.1381686E-04,.1626204E-04,.2015870E-04,.4599833E-04/

      data   absa(  1:180, 5)   /                                       &
     & .1258379E-03,.1792232E-03,.2263834E-03,.2643095E-03,.2960118E-03,&
     & .3236055E-03,.3486226E-03,.3708136E-03,.3537227E-03,.1252867E-03,&
     & .1784744E-03,.2256677E-03,.2637463E-03,.2957115E-03,.3235207E-03,&
     & .3487202E-03,.3710916E-03,.3547494E-03,.1246492E-03,.1788252E-03,&
     & .2273405E-03,.2669495E-03,.3004867E-03,.3297723E-03,.3562380E-03,&
     & .3799251E-03,.3658619E-03,.1240169E-03,.1766881E-03,.2240641E-03,&
     & .2627788E-03,.2953183E-03,.3234639E-03,.3488432E-03,.3713321E-03,&
     & .3567172E-03,.1233679E-03,.1757608E-03,.2232662E-03,.2622089E-03,&
     & .2951315E-03,.3234287E-03,.3488259E-03,.3713523E-03,.3576976E-03,&
     & .1069251E-03,.1638851E-03,.2111585E-03,.2498003E-03,.2830029E-03,&
     & .3120257E-03,.3378034E-03,.3624640E-03,.3424007E-03,.1064316E-03,&
     & .1632513E-03,.2106727E-03,.2494364E-03,.2829718E-03,.3122311E-03,&
     & .3381458E-03,.3630252E-03,.3438497E-03,.1058739E-03,.1637860E-03,&
     & .2127118E-03,.2531195E-03,.2882500E-03,.3190771E-03,.3463354E-03,&
     & .3723861E-03,.3557833E-03,.1053157E-03,.1629041E-03,.2119399E-03,&
     & .2525668E-03,.2878735E-03,.3188283E-03,.3460548E-03,.3719927E-03,&
     & .3563658E-03,.1048030E-03,.1609177E-03,.2089982E-03,.2486370E-03,&
     & .2829942E-03,.3129743E-03,.3391967E-03,.3638899E-03,.3480124E-03,&
     & .9052393E-04,.1496647E-03,.1963928E-03,.2353138E-03,.2692949E-03,&
     & .2994397E-03,.3259337E-03,.3522926E-03,.3288949E-03,.9011012E-04,&
     & .1491318E-03,.1961079E-03,.2352015E-03,.2695824E-03,.2999606E-03,&
     & .3266081E-03,.3531618E-03,.3308028E-03,.8966771E-04,.1485392E-03,&
     & .1958028E-03,.2351815E-03,.2698429E-03,.3005448E-03,.3273009E-03,&
     & .3538438E-03,.3326818E-03,.8920657E-04,.1491611E-03,.1980348E-03,&
     & .2389898E-03,.2753310E-03,.3075620E-03,.3356528E-03,.3632137E-03,&
     & .3449423E-03,.8880078E-04,.1472741E-03,.1951287E-03,.2350641E-03,&
     & .2704561E-03,.3016661E-03,.3287211E-03,.3549932E-03,.3361806E-03,&
     & .7642425E-04,.1365161E-03,.1822415E-03,.2208431E-03,.2550453E-03,&
     & .2857390E-03,.3131524E-03,.3407797E-03,.3140906E-03,.7610890E-04,&
     & .1361051E-03,.1821158E-03,.2209982E-03,.2556705E-03,.2866395E-03,&
     & .3142723E-03,.3419173E-03,.3164057E-03,.7573557E-04,.1356867E-03,&
     & .1820099E-03,.2212242E-03,.2562294E-03,.2875830E-03,.3152966E-03,&
     & .3429191E-03,.3186642E-03,.7538404E-04,.1352458E-03,.1818638E-03,&
     & .2214386E-03,.2568082E-03,.2884658E-03,.3163349E-03,.3438669E-03,&
     & .3207779E-03,.7505591E-04,.1360347E-03,.1841939E-03,.2254174E-03,&
     & .2624835E-03,.2957188E-03,.3250526E-03,.3534819E-03,.3333120E-03/

      data   absa(181:315, 5)   /                                       &
     & .6441960E-04,.1243382E-03,.1686492E-03,.2063321E-03,.2401962E-03,&
     & .2709669E-03,.2991358E-03,.3279206E-03,.2981254E-03,.6417820E-04,&
     & .1241020E-03,.1687386E-03,.2068102E-03,.2412076E-03,.2722842E-03,&
     & .3007606E-03,.3294871E-03,.3009391E-03,.6388399E-04,.1238576E-03,&
     & .1687898E-03,.2073172E-03,.2421411E-03,.2736197E-03,.3022572E-03,&
     & .3309669E-03,.3036272E-03,.6360964E-04,.1235961E-03,.1688422E-03,&
     & .2077864E-03,.2431068E-03,.2749041E-03,.3037508E-03,.3323669E-03,&
     & .3061923E-03,.6334715E-04,.1246342E-03,.1715166E-03,.2122785E-03,&
     & .2493866E-03,.2829267E-03,.3132804E-03,.3430618E-03,.3198672E-03,&
     & .5428305E-04,.1131178E-03,.1555172E-03,.1918587E-03,.2248108E-03,&
     & .2552688E-03,.2837909E-03,.3135092E-03,.2808167E-03,.5410275E-04,&
     & .1130139E-03,.1558251E-03,.1926789E-03,.2262062E-03,.2569617E-03,&
     & .2858483E-03,.3156324E-03,.2840877E-03,.5388978E-04,.1129118E-03,&
     & .1560839E-03,.1935035E-03,.2275082E-03,.2586750E-03,.2877475E-03,&
     & .3176409E-03,.2872619E-03,.5366157E-04,.1128154E-03,.1563700E-03,&
     & .1943089E-03,.2288065E-03,.2603020E-03,.2896110E-03,.3195144E-03,&
     & .2902915E-03,.5344784E-04,.1126980E-03,.1566320E-03,.1951184E-03,&
     & .2300546E-03,.2619121E-03,.2914792E-03,.3214494E-03,.2932874E-03,&
     & .4571342E-04,.1026942E-03,.1428236E-03,.1775296E-03,.2091830E-03,&
     & .2387595E-03,.2671739E-03,.2978545E-03,.2594784E-03,.4560282E-04,&
     & .1027218E-03,.1433756E-03,.1786580E-03,.2109254E-03,.2408706E-03,&
     & .2696813E-03,.3004723E-03,.2634130E-03,.4544692E-04,.1027687E-03,&
     & .1438888E-03,.1797866E-03,.2125279E-03,.2429678E-03,.2720358E-03,&
     & .3029173E-03,.2670716E-03,.4528151E-04,.1028069E-03,.1444180E-03,&
     & .1808751E-03,.2141242E-03,.2449535E-03,.2743172E-03,.3052177E-03,&
     & .2706270E-03,.4511742E-04,.1028521E-03,.1449287E-03,.1819408E-03,&
     & .2156718E-03,.2468811E-03,.2765286E-03,.3074942E-03,.2741063E-03/

      data   absa(316:450, 5)   /                                       &
     & .3845347E-04,.9301429E-04,.1306852E-03,.1633456E-03,.1934332E-03,&
     & .2218378E-03,.2497592E-03,.2809541E-03,.1575108E-03,.3839587E-04,&
     & .9318412E-04,.1314652E-03,.1648012E-03,.1954890E-03,.2243498E-03,&
     & .2526899E-03,.2840733E-03,.1617842E-03,.3829010E-04,.9337649E-04,&
     & .1322053E-03,.1662204E-03,.1974070E-03,.2268068E-03,.2554975E-03,&
     & .2869785E-03,.1663258E-03,.3818495E-04,.9357010E-04,.1329542E-03,&
     & .1675979E-03,.1993416E-03,.2291758E-03,.2582113E-03,.2897317E-03,&
     & .1701293E-03,.3807196E-04,.9375146E-04,.1336930E-03,.1689218E-03,&
     & .2012206E-03,.2314740E-03,.2608244E-03,.2924501E-03,.1741563E-03,&
     & .3222272E-04,.8402664E-04,.1190004E-03,.1495500E-03,.1778606E-03,&
     & .2048436E-03,.2318785E-03,.2632322E-03,.2491014E-04,.3221362E-04,&
     & .8435285E-04,.1199991E-03,.1512485E-03,.1802135E-03,.2077441E-03,&
     & .2352663E-03,.2668367E-03,.2569053E-04,.3215953E-04,.8468829E-04,&
     & .1209883E-03,.1529182E-03,.1824779E-03,.2105732E-03,.2385311E-03,&
     & .2702367E-03,.2701309E-04,.3210851E-04,.8503241E-04,.1219626E-03,&
     & .1545750E-03,.1847085E-03,.2133101E-03,.2416857E-03,.2734763E-03,&
     & .2805465E-04,.3204318E-04,.8537283E-04,.1229412E-03,.1561853E-03,&
     & .1869065E-03,.2159588E-03,.2447099E-03,.2766143E-03,.2860049E-04,&
     & .2668365E-04,.7570181E-04,.1080203E-03,.1364318E-03,.1629303E-03,&
     & .1883981E-03,.2142724E-03,.2453798E-03,.3073231E-04,.2671397E-04,&
     & .7619513E-04,.1092103E-03,.1383829E-03,.1655646E-03,.1916417E-03,&
     & .2180830E-03,.2494081E-03,.2860411E-04,.2670321E-04,.7667716E-04,&
     & .1104090E-03,.1402846E-03,.1681221E-03,.1948231E-03,.2217702E-03,&
     & .2532299E-03,.2852470E-04,.2669101E-04,.7717197E-04,.1115742E-03,&
     & .1422043E-03,.1706685E-03,.1979160E-03,.2253063E-03,.2569544E-03,&
     & .3387605E-04,.2666179E-04,.7765343E-04,.1127755E-03,.1440858E-03,&
     & .1731700E-03,.2008739E-03,.2287122E-03,.2605352E-03,.3169336E-04/

      data   absa(451:585, 5)   /                                       &
     & .2167692E-04,.6827587E-04,.9836512E-04,.1250201E-03,.1498886E-03,&
     & .1739600E-03,.1987214E-03,.2293404E-03,.2440081E-04,.2169387E-04,&
     & .6891366E-04,.9974241E-04,.1271668E-03,.1528340E-03,.1776053E-03,&
     & .2029862E-03,.2337700E-03,.2638768E-04,.2170600E-04,.6954070E-04,&
     & .1011255E-03,.1293455E-03,.1556958E-03,.1811079E-03,.2069944E-03,&
     & .2380400E-03,.3200312E-04,.2170812E-04,.7016990E-04,.1025105E-03,&
     & .1314880E-03,.1585164E-03,.1844769E-03,.2108627E-03,.2421072E-03,&
     & .3275541E-04,.2168923E-04,.7080350E-04,.1038948E-03,.1335789E-03,&
     & .1612308E-03,.1877502E-03,.2145673E-03,.2460134E-03,.3270936E-04,&
     & .1752912E-04,.6154969E-04,.8955810E-04,.1143831E-03,.1376209E-03,&
     & .1602880E-03,.1838222E-03,.2134820E-03,.2872965E-04,.1755385E-04,&
     & .6231518E-04,.9115429E-04,.1168314E-03,.1408712E-03,.1642185E-03,&
     & .1883638E-03,.2183575E-03,.3183547E-04,.1752199E-04,.6309426E-04,&
     & .9273525E-04,.1192604E-03,.1440348E-03,.1680514E-03,.1927365E-03,&
     & .2230115E-03,.3077672E-04,.1749575E-04,.6385017E-04,.9430365E-04,&
     & .1216307E-03,.1471390E-03,.1717409E-03,.1969541E-03,.2274695E-03,&
     & .2312932E-04,.1752070E-04,.6462019E-04,.9584906E-04,.1239388E-03,&
     & .1501161E-03,.1753754E-03,.2010501E-03,.2317004E-03,.2383092E-04,&
     & .1387461E-04,.5546622E-04,.8152797E-04,.1045929E-03,.1262746E-03,&
     & .1474566E-03,.1696496E-03,.1981242E-03,.3139074E-04,.1392397E-04,&
     & .5636393E-04,.8334885E-04,.1072746E-03,.1297503E-03,.1516626E-03,&
     & .1744739E-03,.2033126E-03,.2572972E-04,.1396545E-04,.5726788E-04,&
     & .8511400E-04,.1099575E-03,.1332042E-03,.1557803E-03,.1791529E-03,&
     & .2083642E-03,.2669035E-04,.1397377E-04,.5816790E-04,.8688926E-04,&
     & .1125444E-03,.1365495E-03,.1597819E-03,.1837038E-03,.2131552E-03,&
     & .2848091E-04,.1401956E-04,.5906486E-04,.8860992E-04,.1150938E-03,&
     & .1398565E-03,.1638183E-03,.1882562E-03,.2178384E-03,.2719011E-04/

      data   absa(  1:180, 6)   /                                       &
     & .1127779E-02,.1042084E-02,.1103730E-02,.1177585E-02,.1245882E-02,&
     & .1305946E-02,.1334440E-02,.1306277E-02,.1318375E-02,.1125070E-02,&
     & .1037761E-02,.1098032E-02,.1171270E-02,.1239267E-02,.1299657E-02,&
     & .1328359E-02,.1299803E-02,.1315607E-02,.1123280E-02,.1035695E-02,&
     & .1095422E-02,.1170386E-02,.1240633E-02,.1304457E-02,.1337002E-02,&
     & .1309998E-02,.1335448E-02,.1122248E-02,.1032813E-02,.1088190E-02,&
     & .1159040E-02,.1227090E-02,.1287160E-02,.1316759E-02,.1286817E-02,&
     & .1309802E-02,.1121332E-02,.1030797E-02,.1084488E-02,.1154404E-02,&
     & .1221682E-02,.1281593E-02,.1311385E-02,.1280520E-02,.1306178E-02,&
     & .1019965E-02,.9810459E-03,.1068764E-02,.1149672E-02,.1228423E-02,&
     & .1296985E-02,.1342592E-02,.1331587E-02,.1337457E-02,.1017770E-02,&
     & .9768453E-03,.1063034E-02,.1145111E-02,.1222358E-02,.1291726E-02,&
     & .1337570E-02,.1326202E-02,.1335613E-02,.1016813E-02,.9753083E-03,&
     & .1061473E-02,.1145518E-02,.1226142E-02,.1297946E-02,.1347626E-02,&
     & .1338886E-02,.1357689E-02,.1015843E-02,.9725487E-03,.1057441E-02,&
     & .1140180E-02,.1220522E-02,.1291191E-02,.1340908E-02,.1331183E-02,&
     & .1353078E-02,.1015463E-02,.9698700E-03,.1050819E-02,.1130971E-02,&
     & .1207619E-02,.1275709E-02,.1322301E-02,.1309378E-02,.1327704E-02,&
     & .9119475E-03,.9235847E-03,.1031890E-02,.1126261E-02,.1212118E-02,&
     & .1287381E-02,.1346323E-02,.1352648E-02,.1351799E-02,.9104425E-03,&
     & .9200649E-03,.1026543E-02,.1122353E-02,.1206597E-02,.1282710E-02,&
     & .1341997E-02,.1348233E-02,.1350149E-02,.9096645E-03,.9174474E-03,&
     & .1022607E-02,.1117149E-02,.1202222E-02,.1277201E-02,.1337011E-02,&
     & .1343242E-02,.1348493E-02,.9088794E-03,.9162707E-03,.1022764E-02,&
     & .1119302E-02,.1206504E-02,.1284741E-02,.1348507E-02,.1357512E-02,&
     & .1369635E-02,.9088680E-03,.9133197E-03,.1016712E-02,.1109724E-02,&
     & .1192813E-02,.1268948E-02,.1329585E-02,.1335432E-02,.1344303E-02,&
     & .8030336E-03,.8629808E-03,.9905516E-03,.1102706E-02,.1194581E-02,&
     & .1276097E-02,.1344985E-02,.1368489E-02,.1358472E-02,.8020685E-03,&
     & .8599499E-03,.9864952E-03,.1098943E-02,.1190085E-02,.1271988E-02,&
     & .1340997E-02,.1365627E-02,.1358480E-02,.8014347E-03,.8575084E-03,&
     & .9835460E-03,.1094344E-02,.1186524E-02,.1267351E-02,.1337492E-02,&
     & .1362554E-02,.1358055E-02,.8012884E-03,.8553201E-03,.9810718E-03,&
     & .1090482E-02,.1182154E-02,.1264203E-02,.1334958E-02,.1360018E-02,&
     & .1357215E-02,.8017028E-03,.8553036E-03,.9828813E-03,.1094204E-02,&
     & .1188558E-02,.1274654E-02,.1347715E-02,.1376787E-02,.1377563E-02/

      data   absa(181:315, 6)   /                                       &
     & .6989341E-03,.8008136E-03,.9474693E-03,.1073693E-02,.1176718E-02,&
     & .1263281E-02,.1337495E-02,.1377174E-02,.1357373E-02,.6982803E-03,&
     & .7984718E-03,.9443526E-03,.1071067E-02,.1172943E-02,.1259834E-02,&
     & .1334341E-02,.1375767E-02,.1358133E-02,.6978266E-03,.7963865E-03,&
     & .9422629E-03,.1067324E-02,.1169642E-02,.1256400E-02,.1331961E-02,&
     & .1374337E-02,.1358589E-02,.6981011E-03,.7946817E-03,.9401344E-03,&
     & .1064498E-02,.1165906E-02,.1254292E-02,.1329925E-02,.1372816E-02,&
     & .1358794E-02,.6987926E-03,.7955223E-03,.9431432E-03,.1070202E-02,&
     & .1174838E-02,.1266995E-02,.1345241E-02,.1390924E-02,.1382115E-02,&
     & .6011592E-03,.7388959E-03,.9023729E-03,.1040717E-02,.1155626E-02,&
     & .1247524E-02,.1326820E-02,.1381148E-02,.1347234E-02,.6009151E-03,&
     & .7373240E-03,.9002892E-03,.1038951E-02,.1152565E-02,.1245208E-02,&
     & .1324569E-02,.1380312E-02,.1349694E-02,.6008914E-03,.7360400E-03,&
     & .8989538E-03,.1035946E-02,.1150472E-02,.1242758E-02,.1323179E-02,&
     & .1379317E-02,.1351734E-02,.6017350E-03,.7352405E-03,.8972176E-03,&
     & .1033783E-02,.1147905E-02,.1241675E-02,.1321813E-02,.1377843E-02,&
     & .1353588E-02,.6024658E-03,.7352400E-03,.8961216E-03,.1032038E-02,&
     & .1146289E-02,.1240281E-02,.1320537E-02,.1375799E-02,.1354849E-02,&
     & .5131365E-03,.6802953E-03,.8558547E-03,.1004257E-02,.1129015E-02,&
     & .1228658E-02,.1311670E-02,.1377495E-02,.1326794E-02,.5128417E-03,&
     & .6796430E-03,.8551433E-03,.1003333E-02,.1127459E-02,.1227610E-02,&
     & .1310680E-02,.1377165E-02,.1331440E-02,.5128844E-03,.6791129E-03,&
     & .8543752E-03,.1001545E-02,.1126702E-02,.1226436E-02,.1310287E-02,&
     & .1377210E-02,.1336197E-02,.5138776E-03,.6790044E-03,.8533106E-03,&
     & .1000677E-02,.1125665E-02,.1226017E-02,.1310194E-02,.1376877E-02,&
     & .1340793E-02,.5148503E-03,.6794602E-03,.8527717E-03,.1000386E-02,&
     & .1125230E-02,.1225940E-02,.1310446E-02,.1376564E-02,.1344823E-02/

      data   absa(316:450, 6)   /                                       &
     & .4354937E-03,.6258347E-03,.8095219E-03,.9652907E-03,.1096244E-02,&
     & .1204241E-02,.1290647E-02,.1366018E-02,.1291769E-02,.4351834E-03,&
     & .6260531E-03,.8097110E-03,.9658306E-03,.1096299E-02,.1204907E-02,&
     & .1291676E-02,.1367386E-02,.1299495E-02,.4354926E-03,.6262211E-03,&
     & .8099139E-03,.9655329E-03,.1097699E-02,.1205112E-02,.1292665E-02,&
     & .1368366E-02,.1307945E-02,.4366061E-03,.6265142E-03,.8100645E-03,&
     & .9659489E-03,.1098184E-02,.1206245E-02,.1294118E-02,.1369694E-02,&
     & .1316015E-02,.4374936E-03,.6275003E-03,.8106097E-03,.9669869E-03,&
     & .1099303E-02,.1207811E-02,.1295643E-02,.1370889E-02,.1323956E-02,&
     & .3685128E-03,.5745214E-03,.7639287E-03,.9223177E-03,.1057771E-02,&
     & .1170619E-02,.1261579E-02,.1344082E-02,.3978360E-03,.3683928E-03,&
     & .5756454E-03,.7650351E-03,.9248369E-03,.1060206E-02,.1174221E-02,&
     & .1265274E-02,.1347856E-02,.4071014E-03,.3688375E-03,.5764859E-03,&
     & .7661665E-03,.9267124E-03,.1063374E-02,.1176667E-02,.1268148E-02,&
     & .1350882E-02,.4149611E-03,.3697059E-03,.5774958E-03,.7676652E-03,&
     & .9288451E-03,.1065840E-02,.1179962E-02,.1271481E-02,.1354201E-02,&
     & .4230889E-03,.3705971E-03,.5790606E-03,.7693590E-03,.9312187E-03,&
     & .1068730E-02,.1183767E-02,.1275156E-02,.1357593E-02,.4395701E-03,&
     & .3118513E-03,.5278670E-03,.7179922E-03,.8762847E-03,.1013004E-02,&
     & .1129188E-02,.1225380E-02,.1313014E-02,.2823336E-04,.3119642E-03,&
     & .5295040E-03,.7205085E-03,.8810719E-03,.1018490E-02,.1135842E-02,&
     & .1231411E-02,.1319687E-02,.2591174E-04,.3123480E-03,.5313833E-03,&
     & .7231295E-03,.8852977E-03,.1024173E-02,.1141110E-02,.1237009E-02,&
     & .1325493E-02,.2490050E-04,.3131033E-03,.5329730E-03,.7261876E-03,&
     & .8890594E-03,.1028824E-02,.1147600E-02,.1243500E-02,.1331641E-02,&
     & .2358085E-04,.3141200E-03,.5351677E-03,.7291337E-03,.8933695E-03,&
     & .1034263E-02,.1154416E-02,.1249926E-02,.1337840E-02,.1914261E-04/

      data   absa(451:585, 6)   /                                       &
     & .2645518E-03,.4865074E-03,.6737735E-03,.8307780E-03,.9684326E-03,&
     & .1087714E-02,.1187390E-02,.1279536E-02,.3096728E-04,.2646171E-03,&
     & .4891259E-03,.6781457E-03,.8380014E-03,.9763776E-03,.1095878E-02,&
     & .1195388E-02,.1287784E-02,.1937897E-04,.2653275E-03,.4912812E-03,&
     & .6826723E-03,.8439569E-03,.9843890E-03,.1104414E-02,.1204820E-02,&
     & .1296209E-02,.2310672E-04,.2660999E-03,.4938062E-03,.6869132E-03,&
     & .8501984E-03,.9923360E-03,.1114850E-02,.1214879E-02,.1305902E-02,&
     & .1551854E-04,.2672744E-03,.4963201E-03,.6913518E-03,.8568207E-03,&
     & .1001054E-02,.1124616E-02,.1224913E-02,.1315531E-02,.1902280E-04,&
     & .2249381E-03,.4491543E-03,.6314125E-03,.7873093E-03,.9244395E-03,&
     & .1044841E-02,.1147218E-02,.1242720E-02,.1795873E-04,.2253436E-03,&
     & .4522769E-03,.6374135E-03,.7959199E-03,.9349022E-03,.1056532E-02,&
     & .1159671E-02,.1253800E-02,.3013456E-04,.2260901E-03,.4552587E-03,&
     & .6435905E-03,.8039747E-03,.9457286E-03,.1069229E-02,.1173356E-02,&
     & .1266548E-02,.1199840E-04,.2269143E-03,.4587430E-03,.6495108E-03,&
     & .8129332E-03,.9570070E-03,.1082883E-02,.1186741E-02,.1279448E-02,&
     & .1794789E-04,.2280730E-03,.4618575E-03,.6559639E-03,.8219928E-03,&
     & .9685634E-03,.1095437E-02,.1200070E-02,.1292864E-02,.3604270E-04,&
     & .1921277E-03,.4151006E-03,.5920383E-03,.7457881E-03,.8813168E-03,&
     & .1001800E-02,.1105493E-02,.1202692E-02,.2297239E-04,.1928316E-03,&
     & .4191322E-03,.5996165E-03,.7563148E-03,.8956200E-03,.1018058E-02,&
     & .1123450E-02,.1218805E-02,.7650963E-05,.1934147E-03,.4229466E-03,&
     & .6073039E-03,.7668297E-03,.9096828E-03,.1035442E-02,.1141697E-02,&
     & .1235773E-02,.1552682E-04,.1943616E-03,.4270842E-03,.6149144E-03,&
     & .7783265E-03,.9240174E-03,.1052153E-02,.1158854E-02,.1252683E-02,&
     & .1480436E-06,.1954216E-03,.4310524E-03,.6232033E-03,.7895004E-03,&
     & .9380216E-03,.1067280E-02,.1175445E-02,.1269539E-02,.1885871E-06/

      data   absa(  1:180, 7)   /                                       &
     & .6726057E-02,.5885484E-02,.5089667E-02,.4459280E-02,.3971837E-02,&
     & .3599783E-02,.3258569E-02,.2761804E-02,.3295159E-02,.6741033E-02,&
     & .5898281E-02,.5098508E-02,.4461017E-02,.3965454E-02,.3586408E-02,&
     & .3239758E-02,.2739181E-02,.3264504E-02,.6753803E-02,.5909939E-02,&
     & .5108959E-02,.4465734E-02,.3967543E-02,.3586504E-02,.3237724E-02,&
     & .2736408E-02,.3262274E-02,.6761981E-02,.5916777E-02,.5112425E-02,&
     & .4461186E-02,.3950994E-02,.3560504E-02,.3206228E-02,.2700475E-02,&
     & .3204452E-02,.6766641E-02,.5920869E-02,.5113796E-02,.4457946E-02,&
     & .3942736E-02,.3547005E-02,.3190693E-02,.2682249E-02,.3178026E-02,&
     & .6602172E-02,.5776872E-02,.5061456E-02,.4534366E-02,.4128541E-02,&
     & .3833906E-02,.3516765E-02,.3008287E-02,.3509874E-02,.6617721E-02,&
     & .5790525E-02,.5069114E-02,.4531714E-02,.4120691E-02,.3818183E-02,&
     & .3497546E-02,.2984974E-02,.3478859E-02,.6628661E-02,.5799925E-02,&
     & .5076114E-02,.4533842E-02,.4120334E-02,.3817738E-02,.3496943E-02,&
     & .2984746E-02,.3479524E-02,.6637051E-02,.5807576E-02,.5078438E-02,&
     & .4529659E-02,.4109342E-02,.3801331E-02,.3478883E-02,.2965136E-02,&
     & .3448943E-02,.6644129E-02,.5813792E-02,.5079055E-02,.4521155E-02,&
     & .4092528E-02,.3771957E-02,.3445403E-02,.2926792E-02,.3395993E-02,&
     & .6403543E-02,.5608393E-02,.4995394E-02,.4569211E-02,.4267045E-02,&
     & .4052917E-02,.3769832E-02,.3262352E-02,.3718785E-02,.6418428E-02,&
     & .5620905E-02,.5000985E-02,.4564662E-02,.4256975E-02,.4036131E-02,&
     & .3749359E-02,.3237816E-02,.3690475E-02,.6431526E-02,.5631313E-02,&
     & .5004466E-02,.4561740E-02,.4245515E-02,.4020306E-02,.3730187E-02,&
     & .3216366E-02,.3662511E-02,.6443000E-02,.5641170E-02,.5008963E-02,&
     & .4562706E-02,.4244648E-02,.4017451E-02,.3728288E-02,.3215678E-02,&
     & .3669714E-02,.6454834E-02,.5651225E-02,.5010564E-02,.4555174E-02,&
     & .4226789E-02,.3985960E-02,.3691258E-02,.3172291E-02,.3616086E-02,&
     & .6136735E-02,.5394093E-02,.4895063E-02,.4573199E-02,.4385068E-02,&
     & .4249509E-02,.4003690E-02,.3510981E-02,.3909452E-02,.6155380E-02,&
     & .5408904E-02,.4900856E-02,.4569405E-02,.4373257E-02,.4232877E-02,&
     & .3984093E-02,.3485462E-02,.3882943E-02,.6172980E-02,.5422475E-02,&
     & .4904945E-02,.4567515E-02,.4361079E-02,.4216322E-02,.3963626E-02,&
     & .3461480E-02,.3857723E-02,.6190567E-02,.5436595E-02,.4910048E-02,&
     & .4565398E-02,.4351183E-02,.4198401E-02,.3942356E-02,.3437743E-02,&
     & .3835996E-02,.6207595E-02,.5450671E-02,.4917790E-02,.4569064E-02,&
     & .4350510E-02,.4193413E-02,.3939962E-02,.3434045E-02,.3850556E-02/

      data   absa(181:315, 7)   /                                       &
     & .5805911E-02,.5141666E-02,.4763038E-02,.4558815E-02,.4481648E-02,&
     & .4418775E-02,.4218480E-02,.3750496E-02,.4085582E-02,.5829599E-02,&
     & .5159492E-02,.4770932E-02,.4555269E-02,.4469546E-02,.4402981E-02,&
     & .4198610E-02,.3724260E-02,.4061967E-02,.5854540E-02,.5177705E-02,&
     & .4778238E-02,.4554312E-02,.4457721E-02,.4386787E-02,.4178282E-02,&
     & .3698112E-02,.4040118E-02,.5879038E-02,.5196952E-02,.4787326E-02,&
     & .4553873E-02,.4447674E-02,.4369633E-02,.4158545E-02,.3673692E-02,&
     & .4021141E-02,.5902275E-02,.5214049E-02,.4797276E-02,.4557481E-02,&
     & .4445243E-02,.4367290E-02,.4159851E-02,.3673710E-02,.4041450E-02,&
     & .5423597E-02,.4859083E-02,.4609862E-02,.4529795E-02,.4557832E-02,&
     & .4560070E-02,.4409076E-02,.3975219E-02,.4252602E-02,.5454365E-02,&
     & .4881607E-02,.4620352E-02,.4527565E-02,.4548530E-02,.4548237E-02,&
     & .4391661E-02,.3949326E-02,.4232009E-02,.5488044E-02,.4905751E-02,&
     & .4632103E-02,.4528708E-02,.4538967E-02,.4534788E-02,.4373488E-02,&
     & .3924420E-02,.4212910E-02,.5518713E-02,.4928892E-02,.4643930E-02,&
     & .4529113E-02,.4529842E-02,.4520746E-02,.4357460E-02,.3901482E-02,&
     & .4195693E-02,.5549581E-02,.4949387E-02,.4654691E-02,.4528154E-02,&
     & .4518717E-02,.4507320E-02,.4341984E-02,.3878945E-02,.4179215E-02,&
     & .5010662E-02,.4562289E-02,.4450406E-02,.4497408E-02,.4612122E-02,&
     & .4668231E-02,.4573752E-02,.4184611E-02,.4407478E-02,.5048488E-02,&
     & .4588791E-02,.4462195E-02,.4495999E-02,.4605287E-02,.4662092E-02,&
     & .4561288E-02,.4161583E-02,.4389563E-02,.5087790E-02,.4615970E-02,&
     & .4475379E-02,.4496880E-02,.4598974E-02,.4654027E-02,.4547703E-02,&
     & .4139097E-02,.4372125E-02,.5124133E-02,.4641212E-02,.4488105E-02,&
     & .4495983E-02,.4593289E-02,.4645512E-02,.4535541E-02,.4118063E-02,&
     & .4355876E-02,.5162759E-02,.4667582E-02,.4502006E-02,.4496029E-02,&
     & .4585626E-02,.4636559E-02,.4523214E-02,.4097252E-02,.4340154E-02/

      data   absa(316:450, 7)   /                                       &
     & .4587688E-02,.4265942E-02,.4293117E-02,.4465668E-02,.4646838E-02,&
     & .4746553E-02,.4710731E-02,.4377367E-02,.4543398E-02,.4630881E-02,&
     & .4294688E-02,.4306044E-02,.4466382E-02,.4646169E-02,.4747955E-02,&
     & .4704166E-02,.4357348E-02,.4528127E-02,.4671866E-02,.4321409E-02,&
     & .4317765E-02,.4465517E-02,.4644351E-02,.4746523E-02,.4697148E-02,&
     & .4338778E-02,.4512456E-02,.4713989E-02,.4350596E-02,.4331423E-02,&
     & .4464363E-02,.4642472E-02,.4743704E-02,.4690020E-02,.4320460E-02,&
     & .4498985E-02,.4761723E-02,.4382334E-02,.4348263E-02,.4465420E-02,&
     & .4638543E-02,.4740624E-02,.4683278E-02,.4302003E-02,.4484275E-02,&
     & .4164717E-02,.3979215E-02,.4144193E-02,.4422962E-02,.4657806E-02,&
     & .4800744E-02,.4824067E-02,.4555259E-02,.4602286E-02,.4211716E-02,&
     & .4008521E-02,.4157412E-02,.4428976E-02,.4663384E-02,.4809552E-02,&
     & .4822353E-02,.4539339E-02,.4590712E-02,.4257203E-02,.4037513E-02,&
     & .4169430E-02,.4432371E-02,.4669584E-02,.4816112E-02,.4822125E-02,&
     & .4523735E-02,.4581771E-02,.4307089E-02,.4070222E-02,.4184488E-02,&
     & .4434611E-02,.4676070E-02,.4820311E-02,.4822098E-02,.4508261E-02,&
     & .4573178E-02,.4362715E-02,.4106878E-02,.4203483E-02,.4440385E-02,&
     & .4678627E-02,.4823705E-02,.4820573E-02,.4492558E-02,.4565516E-02,&
     & .3757322E-02,.3709073E-02,.4010509E-02,.4365375E-02,.4646644E-02,&
     & .4836629E-02,.4911390E-02,.4716037E-02,.2246564E-04,.3804915E-02,&
     & .3737287E-02,.4021585E-02,.4379566E-02,.4660941E-02,.4852517E-02,&
     & .4918338E-02,.4703233E-02,.1560965E-04,.3856320E-02,.3768752E-02,&
     & .4034329E-02,.4388713E-02,.4676333E-02,.4865802E-02,.4925596E-02,&
     & .4691751E-02,.2028569E-04,.3915081E-02,.3807123E-02,.4052103E-02,&
     & .4397870E-02,.4691010E-02,.4877070E-02,.4930793E-02,.4679143E-02,&
     & .8984221E-05,.3978571E-02,.3848618E-02,.4073285E-02,.4408428E-02,&
     & .4700559E-02,.4888012E-02,.4935723E-02,.4666034E-02,.2377631E-04/

      data   absa(451:585, 7)   /                                       &
     & .3392928E-02,.3472731E-02,.3891540E-02,.4305406E-02,.4626003E-02,&
     & .4865579E-02,.4987193E-02,.4850053E-02,.7140743E-05,.3445972E-02,&
     & .3503144E-02,.3906610E-02,.4325733E-02,.4652048E-02,.4888232E-02,&
     & .5003402E-02,.4843920E-02,.1439870E-04,.3506494E-02,.3541411E-02,&
     & .3924176E-02,.4343703E-02,.4677805E-02,.4908831E-02,.5016389E-02,&
     & .4837310E-02,.2945034E-04,.3573835E-02,.3584076E-02,.3945325E-02,&
     & .4362029E-02,.4698214E-02,.4926791E-02,.5027125E-02,.4828102E-02,&
     & .4370988E-04,.3640920E-02,.3627224E-02,.3965033E-02,.4380312E-02,&
     & .4714810E-02,.4945090E-02,.5036845E-02,.4818793E-02,.2945488E-04,&
     & .3062423E-02,.3266343E-02,.3779589E-02,.4238657E-02,.4602824E-02,&
     & .4884734E-02,.5053548E-02,.4961911E-02,.2279693E-04,.3122111E-02,&
     & .3300450E-02,.3801982E-02,.4268791E-02,.4639422E-02,.4914708E-02,&
     & .5076081E-02,.4964486E-02,.3897689E-12,.3189802E-02,.3342291E-02,&
     & .3827671E-02,.4296813E-02,.4673472E-02,.4942459E-02,.5094747E-02,&
     & .4963101E-02,.2288291E-04,.3258443E-02,.3383882E-02,.3851683E-02,&
     & .4323754E-02,.4700766E-02,.4969560E-02,.5112385E-02,.4958514E-02,&
     & .5725606E-12,.3332531E-02,.3430641E-02,.3873322E-02,.4350068E-02,&
     & .4725169E-02,.4995061E-02,.5126654E-02,.4953566E-02,.2246441E-04,&
     & .2769676E-02,.3091600E-02,.3674265E-02,.4169149E-02,.4577937E-02,&
     & .4901246E-02,.5107166E-02,.5055949E-02,.2616949E-12,.2834654E-02,&
     & .3128534E-02,.3705750E-02,.4210247E-02,.4623099E-02,.4938500E-02,&
     & .5137406E-02,.5065069E-02,.1482104E-04,.2902742E-02,.3167333E-02,&
     & .3738974E-02,.4249285E-02,.4663578E-02,.4972092E-02,.5164558E-02,&
     & .5069435E-02,.4171247E-12,.2976402E-02,.3210834E-02,.3767797E-02,&
     & .4287152E-02,.4699067E-02,.5006489E-02,.5189897E-02,.5072755E-02,&
     & .2892462E-04,.3057599E-02,.3261050E-02,.3794966E-02,.4320752E-02,&
     & .4731977E-02,.5038479E-02,.5208571E-02,.5071823E-02,.5748148E-12/

      data   absa(  1:180, 8)   /                                       &
     & .2384831E-01,.2086796E-01,.1788674E-01,.1490488E-01,.1192466E-01,&
     & .8942720E-02,.6435132E-02,.5216105E-02,.8046861E-02,.2383260E-01,&
     & .2085323E-01,.1787386E-01,.1489524E-01,.1191574E-01,.8936646E-02,&
     & .6410723E-02,.5172302E-02,.7958914E-02,.2381817E-01,.2084152E-01,&
     & .1786363E-01,.1488699E-01,.1190859E-01,.8931517E-02,.6409801E-02,&
     & .5184894E-02,.7931542E-02,.2382505E-01,.2084563E-01,.1786745E-01,&
     & .1488939E-01,.1191159E-01,.8933625E-02,.6363063E-02,.5080538E-02,&
     & .7797373E-02,.2384370E-01,.2086411E-01,.1788240E-01,.1490168E-01,&
     & .1192085E-01,.8941188E-02,.6342996E-02,.5038349E-02,.7721574E-02,&
     & .2736703E-01,.2394425E-01,.2052396E-01,.1710455E-01,.1368277E-01,&
     & .1026855E-01,.7533161E-02,.6102509E-02,.9457556E-02,.2733895E-01,&
     & .2392100E-01,.2050455E-01,.1708673E-01,.1366891E-01,.1025512E-01,&
     & .7493273E-02,.6042133E-02,.9346029E-02,.2732514E-01,.2390916E-01,&
     & .2049354E-01,.1707793E-01,.1366282E-01,.1024661E-01,.7477603E-02,&
     & .6033817E-02,.9291851E-02,.2732521E-01,.2390831E-01,.2049329E-01,&
     & .1707676E-01,.1366198E-01,.1024619E-01,.7441129E-02,.5964001E-02,&
     & .9183864E-02,.2732055E-01,.2390460E-01,.2048954E-01,.1707397E-01,&
     & .1365928E-01,.1024450E-01,.7386349E-02,.5857452E-02,.9032475E-02,&
     & .3111827E-01,.2722823E-01,.2333792E-01,.1944912E-01,.1556020E-01,&
     & .1174017E-01,.8774373E-02,.7126843E-02,.1107681E-01,.3108435E-01,&
     & .2719812E-01,.2331276E-01,.1942815E-01,.1554155E-01,.1171142E-01,&
     & .8719646E-02,.7046216E-02,.1093045E-01,.3105719E-01,.2717391E-01,&
     & .2328911E-01,.1940895E-01,.1552703E-01,.1168858E-01,.8669582E-02,&
     & .6965164E-02,.1078986E-01,.3102639E-01,.2714667E-01,.2326970E-01,&
     & .1939161E-01,.1551301E-01,.1167462E-01,.8637664E-02,.6932578E-02,&
     & .1069100E-01,.3098455E-01,.2711278E-01,.2323876E-01,.1936649E-01,&
     & .1549184E-01,.1165288E-01,.8564686E-02,.6812398E-02,.1050557E-01,&
     & .3509245E-01,.3070408E-01,.2631947E-01,.2193322E-01,.1754460E-01,&
     & .1336963E-01,.1019024E-01,.8304991E-02,.1279060E-01,.3502835E-01,&
     & .3065102E-01,.2627132E-01,.2189262E-01,.1751355E-01,.1331920E-01,&
     & .1010855E-01,.8200658E-02,.1260628E-01,.3496387E-01,.3059309E-01,&
     & .2622344E-01,.2185116E-01,.1748300E-01,.1327541E-01,.1003370E-01,&
     & .8097466E-02,.1242619E-01,.3489136E-01,.3053088E-01,.2616852E-01,&
     & .2180854E-01,.1744581E-01,.1323085E-01,.9958124E-02,.7996001E-02,&
     & .1224573E-01,.3480910E-01,.3045842E-01,.2610586E-01,.2175555E-01,&
     & .1740387E-01,.1318718E-01,.9898605E-02,.7943117E-02,.1209514E-01/

      data   absa(181:315, 8)   /                                       &
     & .3922337E-01,.3432089E-01,.2941855E-01,.2451621E-01,.1961212E-01,&
     & .1515625E-01,.1177220E-01,.9640074E-02,.1459421E-01,.3912061E-01,&
     & .3423066E-01,.2933971E-01,.2445001E-01,.1956031E-01,.1507474E-01,&
     & .1166291E-01,.9508349E-02,.1436943E-01,.3900929E-01,.3413223E-01,&
     & .2925716E-01,.2437959E-01,.1950452E-01,.1499589E-01,.1155252E-01,&
     & .9377833E-02,.1414677E-01,.3888582E-01,.3402602E-01,.2916246E-01,&
     & .2430215E-01,.1944160E-01,.1491429E-01,.1144124E-01,.9247519E-02,&
     & .1391910E-01,.3876452E-01,.3391846E-01,.2907242E-01,.2422862E-01,&
     & .1938320E-01,.1483471E-01,.1134199E-01,.9163542E-02,.1372356E-01,&
     & .4343721E-01,.3800576E-01,.3257544E-01,.2714774E-01,.2177640E-01,&
     & .1709399E-01,.1352071E-01,.1113282E-01,.1649120E-01,.4327842E-01,&
     & .3786995E-01,.3246159E-01,.2705100E-01,.2168123E-01,.1696268E-01,&
     & .1336889E-01,.1097194E-01,.1622011E-01,.4310942E-01,.3772166E-01,&
     & .3233366E-01,.2694253E-01,.2158320E-01,.1684054E-01,.1321804E-01,&
     & .1080800E-01,.1594714E-01,.4294323E-01,.3757194E-01,.3220476E-01,&
     & .2683910E-01,.2148483E-01,.1671415E-01,.1306213E-01,.1064614E-01,&
     & .1567355E-01,.4277920E-01,.3742913E-01,.3208316E-01,.2673469E-01,&
     & .2139400E-01,.1658867E-01,.1290640E-01,.1049169E-01,.1540259E-01,&
     & .4758398E-01,.4163822E-01,.3568958E-01,.2974106E-01,.2404453E-01,&
     & .1920360E-01,.1543848E-01,.1277750E-01,.1847724E-01,.4738640E-01,&
     & .4146564E-01,.3554040E-01,.2961803E-01,.2390579E-01,.1900947E-01,&
     & .1522766E-01,.1257734E-01,.1815308E-01,.4718171E-01,.4128335E-01,&
     & .3538475E-01,.2948927E-01,.2375761E-01,.1882582E-01,.1502284E-01,&
     & .1237488E-01,.1783112E-01,.4697763E-01,.4110594E-01,.3523272E-01,&
     & .2936065E-01,.2361055E-01,.1864217E-01,.1481297E-01,.1217429E-01,&
     & .1750726E-01,.4675913E-01,.4091320E-01,.3506988E-01,.2922357E-01,&
     & .2347293E-01,.1846131E-01,.1460514E-01,.1197957E-01,.1718605E-01/

      data   absa(316:450, 8)   /                                       &
     & .5157307E-01,.4512456E-01,.3867815E-01,.3225755E-01,.2639876E-01,&
     & .2145781E-01,.1750537E-01,.1456533E-01,.2056891E-01,.5133970E-01,&
     & .4492267E-01,.3850493E-01,.3209518E-01,.2619464E-01,.2118929E-01,&
     & .1723033E-01,.1431879E-01,.2018920E-01,.5111942E-01,.4472821E-01,&
     & .3833961E-01,.3195026E-01,.2598683E-01,.2093775E-01,.1695719E-01,&
     & .1406885E-01,.1981018E-01,.5087613E-01,.4451558E-01,.3815604E-01,&
     & .3179899E-01,.2578609E-01,.2068553E-01,.1668237E-01,.1381989E-01,&
     & .1942343E-01,.5060089E-01,.4427189E-01,.3795104E-01,.3162743E-01,&
     & .2558999E-01,.2043394E-01,.1640721E-01,.1357794E-01,.1904337E-01,&
     & .5534675E-01,.4842752E-01,.4150943E-01,.3476993E-01,.2883356E-01,&
     & .2383857E-01,.1970071E-01,.1648151E-01,.2277280E-01,.5509028E-01,&
     & .4820517E-01,.4131655E-01,.3454903E-01,.2856187E-01,.2348302E-01,&
     & .1935846E-01,.1617643E-01,.2231934E-01,.5483513E-01,.4798329E-01,&
     & .4112957E-01,.3434599E-01,.2827236E-01,.2314776E-01,.1900981E-01,&
     & .1587603E-01,.2186150E-01,.5455249E-01,.4773592E-01,.4091413E-01,&
     & .3413989E-01,.2798959E-01,.2281411E-01,.1865486E-01,.1557181E-01,&
     & .2140320E-01,.5423071E-01,.4745272E-01,.4067235E-01,.3391765E-01,&
     & .2771937E-01,.2248276E-01,.1830805E-01,.1527494E-01,.2095422E-01,&
     & .5884200E-01,.5148486E-01,.4413720E-01,.3726346E-01,.3131906E-01,&
     & .2625246E-01,.2197329E-01,.1846472E-01,.2390318E-01,.5857990E-01,&
     & .5125704E-01,.4393628E-01,.3696356E-01,.3095564E-01,.2581323E-01,&
     & .2154391E-01,.1810523E-01,.2329308E-01,.5829302E-01,.5100372E-01,&
     & .4371805E-01,.3669137E-01,.3057614E-01,.2539516E-01,.2111008E-01,&
     & .1774335E-01,.2234009E-01,.5795260E-01,.5070680E-01,.4346190E-01,&
     & .3641971E-01,.3020199E-01,.2497397E-01,.2067529E-01,.1737760E-01,&
     & .2139251E-01,.5758016E-01,.5038153E-01,.4318130E-01,.3612879E-01,&
     & .2984663E-01,.2454856E-01,.2024342E-01,.1701514E-01,.2073609E-01/

      data   absa(451:585, 8)   /                                       &
     & .6192199E-01,.5417932E-01,.4648962E-01,.3955997E-01,.3362055E-01,&
     & .2844545E-01,.2405185E-01,.2035174E-01,.1147313E-01,.6162491E-01,&
     & .5392502E-01,.4624284E-01,.3918782E-01,.3314307E-01,.2793088E-01,&
     & .2352660E-01,.1992303E-01,.1059037E-01,.6128035E-01,.5361653E-01,&
     & .4596296E-01,.3883034E-01,.3266054E-01,.2741823E-01,.2300155E-01,&
     & .1948403E-01,.9386733E-02,.6089037E-01,.5327663E-01,.4566324E-01,&
     & .3846602E-01,.3220217E-01,.2689963E-01,.2248042E-01,.1904677E-01,&
     & .8579093E-02,.6048928E-01,.5292930E-01,.4536869E-01,.3808824E-01,&
     & .3175943E-01,.2638003E-01,.2196410E-01,.1861176E-01,.7679777E-02,&
     & .6465333E-01,.5657427E-01,.4867177E-01,.4175377E-01,.3579281E-01,&
     & .3056479E-01,.2606485E-01,.2228565E-01,.5410855E-02,.6431550E-01,&
     & .5627490E-01,.4834862E-01,.4128883E-01,.3521242E-01,.2995578E-01,&
     & .2544242E-01,.2175694E-01,.4314129E-02,.6391863E-01,.5593558E-01,&
     & .4799090E-01,.4083615E-01,.3463083E-01,.2933897E-01,.2482046E-01,&
     & .2122362E-01,.3358717E-02,.6351746E-01,.5557930E-01,.4765095E-01,&
     & .4037369E-01,.3408214E-01,.2871059E-01,.2420299E-01,.2070479E-01,&
     & .2857525E-02,.6306892E-01,.5519351E-01,.4730897E-01,.3991011E-01,&
     & .3354065E-01,.2809407E-01,.2359936E-01,.2018705E-01,.1777796E-02,&
     & .6703446E-01,.5865676E-01,.5067634E-01,.4381050E-01,.3782219E-01,&
     & .3255454E-01,.2802206E-01,.2421920E-01,.3263487E-02,.6665902E-01,&
     & .5832721E-01,.5026612E-01,.4325080E-01,.3713778E-01,.3183964E-01,&
     & .2728129E-01,.2358311E-01,.2472960E-02,.6626616E-01,.5798723E-01,&
     & .4984365E-01,.4269195E-01,.3646735E-01,.3111990E-01,.2654294E-01,&
     & .2295618E-01,.1479400E-02,.6583606E-01,.5760454E-01,.4944157E-01,&
     & .4212139E-01,.3581890E-01,.3039518E-01,.2581655E-01,.2233056E-01,&
     & .6227714E-03,.6534867E-01,.5717837E-01,.4902999E-01,.4157439E-01,&
     & .3517603E-01,.2968487E-01,.2512196E-01,.2172312E-01,.0000000E+00/

!     the array iabsb(235,8) (kb(5,13:59,8)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 8,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:120, 1)   /                                       &
     & .1504485E-08,.1655173E-08,.1834066E-08,.2005073E-08,.2225228E-08,&
     & .2825609E-08,.3120559E-08,.3367595E-08,.3608461E-08,.3837790E-08,&
     & .4601061E-08,.4908305E-08,.5119376E-08,.5374147E-08,.5818231E-08,&
     & .6064902E-08,.6787357E-08,.7300746E-08,.8100588E-08,.8763709E-08,&
     & .9117383E-08,.9716817E-08,.1020886E-07,.1060372E-07,.1107971E-07,&
     & .1180416E-07,.1231184E-07,.1271306E-07,.1305533E-07,.1325092E-07,&
     & .1466240E-07,.1530788E-07,.1602711E-07,.1636818E-07,.1670250E-07,&
     & .1693173E-07,.1796180E-07,.1893120E-07,.2002152E-07,.2083754E-07,&
     & .2023944E-07,.2129216E-07,.2220720E-07,.2278656E-07,.2334708E-07,&
     & .2244645E-07,.2315295E-07,.2356590E-07,.2386446E-07,.2427953E-07,&
     & .2319360E-07,.2372368E-07,.2396595E-07,.2414683E-07,.2427181E-07,&
     & .2327042E-07,.2360101E-07,.2379715E-07,.2393742E-07,.2409578E-07,&
     & .2292965E-07,.2319441E-07,.2326480E-07,.2333873E-07,.2344359E-07,&
     & .2207835E-07,.2228167E-07,.2230807E-07,.2237370E-07,.2237045E-07,&
     & .2063469E-07,.2078315E-07,.2069998E-07,.2077522E-07,.2076445E-07,&
     & .1888890E-07,.1895087E-07,.1888552E-07,.1896680E-07,.1893800E-07,&
     & .1661684E-07,.1660531E-07,.1660816E-07,.1663392E-07,.1655720E-07,&
     & .1439584E-07,.1436756E-07,.1440958E-07,.1432625E-07,.1427447E-07,&
     & .1206857E-07,.1208380E-07,.1201089E-07,.1199160E-07,.1194784E-07,&
     & .1009433E-07,.1006531E-07,.1001838E-07,.9943710E-08,.9961010E-08,&
     & .8315238E-08,.8268144E-08,.8236950E-08,.8211572E-08,.8220796E-08,&
     & .6957233E-08,.6909506E-08,.6862800E-08,.6872511E-08,.6883916E-08,&
     & .5813067E-08,.5757273E-08,.5742704E-08,.5742958E-08,.5758679E-08,&
     & .4871874E-08,.4830498E-08,.4812558E-08,.4818784E-08,.4830929E-08/

      data   absb(121:235, 1)   /                                       &
     & .4145901E-08,.4106507E-08,.4098022E-08,.4099861E-08,.4120380E-08,&
     & .3494406E-08,.3465963E-08,.3462820E-08,.3456337E-08,.3467275E-08,&
     & .2913432E-08,.2889628E-08,.2883621E-08,.2884851E-08,.2891869E-08,&
     & .2518380E-08,.2496362E-08,.2483940E-08,.2484762E-08,.2483586E-08,&
     & .2170390E-08,.2145546E-08,.2143039E-08,.2142370E-08,.2144634E-08,&
     & .1858046E-08,.1844624E-08,.1839065E-08,.1839091E-08,.1839162E-08,&
     & .1593689E-08,.1577154E-08,.1573460E-08,.1568484E-08,.1569186E-08,&
     & .1355958E-08,.1341138E-08,.1331384E-08,.1328120E-08,.1330151E-08,&
     & .1137083E-08,.1126680E-08,.1120107E-08,.1113883E-08,.1115922E-08,&
     & .9476994E-09,.9397984E-09,.9335405E-09,.9287986E-09,.9311504E-09,&
     & .7932191E-09,.7894802E-09,.7806572E-09,.7786638E-09,.7792733E-09,&
     & .6576028E-09,.6532665E-09,.6444048E-09,.6406832E-09,.6417746E-09,&
     & .5317563E-09,.5279691E-09,.5215721E-09,.5171123E-09,.5157522E-09,&
     & .4412974E-09,.4375477E-09,.4332457E-09,.4272818E-09,.4275019E-09,&
     & .3706852E-09,.3662403E-09,.3636864E-09,.3584788E-09,.3586079E-09,&
     & .3049173E-09,.3029969E-09,.3014427E-09,.2964519E-09,.2955262E-09,&
     & .2463755E-09,.2449361E-09,.2438107E-09,.2391687E-09,.2384375E-09,&
     & .2163852E-09,.2148951E-09,.2136504E-09,.2102694E-09,.2093443E-09,&
     & .1986154E-09,.1986504E-09,.1964941E-09,.1948428E-09,.1924552E-09,&
     & .1827696E-09,.1814666E-09,.1809023E-09,.1806150E-09,.1773916E-09,&
     & .1651185E-09,.1676846E-09,.1675465E-09,.1672040E-09,.1645195E-09,&
     & .1529359E-09,.1572349E-09,.1577673E-09,.1563134E-09,.1551000E-09,&
     & .1662330E-09,.1742843E-09,.1737219E-09,.1735156E-09,.1726745E-09/

      data   absb(  1:120, 2)   /                                       &
     & .8620280E-08,.8996610E-08,.9179512E-08,.9273313E-08,.9496698E-08,&
     & .1372714E-07,.1415234E-07,.1568014E-07,.1633632E-07,.1660623E-07,&
     & .2431506E-07,.2468338E-07,.2511484E-07,.2544479E-07,.2520146E-07,&
     & .3107454E-07,.3086269E-07,.3175603E-07,.3173983E-07,.3139772E-07,&
     & .3808811E-07,.3902271E-07,.4062594E-07,.4223852E-07,.4271310E-07,&
     & .4796946E-07,.4857088E-07,.4939986E-07,.5006781E-07,.5049110E-07,&
     & .5417823E-07,.5385088E-07,.5369254E-07,.5418624E-07,.5425686E-07,&
     & .5417471E-07,.5315715E-07,.5265850E-07,.5227965E-07,.5182433E-07,&
     & .5084925E-07,.4952405E-07,.4900280E-07,.4911242E-07,.4895983E-07,&
     & .4638127E-07,.4550613E-07,.4547382E-07,.4602960E-07,.4609687E-07,&
     & .4282135E-07,.4200746E-07,.4233769E-07,.4314509E-07,.4364517E-07,&
     & .4009028E-07,.3966390E-07,.4021359E-07,.4114239E-07,.4138132E-07,&
     & .3718799E-07,.3696495E-07,.3760314E-07,.3889000E-07,.3926156E-07,&
     & .3428008E-07,.3436242E-07,.3521050E-07,.3663400E-07,.3706092E-07,&
     & .3131405E-07,.3161472E-07,.3286669E-07,.3437652E-07,.3468898E-07,&
     & .2838426E-07,.2902508E-07,.3042525E-07,.3175777E-07,.3176690E-07,&
     & .2508366E-07,.2604667E-07,.2745713E-07,.2796496E-07,.2814461E-07,&
     & .2198601E-07,.2304237E-07,.2412502E-07,.2435339E-07,.2460182E-07,&
     & .1894551E-07,.1992262E-07,.2052438E-07,.2042507E-07,.2067876E-07,&
     & .1617250E-07,.1693348E-07,.1699496E-07,.1708234E-07,.1690581E-07,&
     & .1380557E-07,.1391757E-07,.1389405E-07,.1394496E-07,.1376301E-07,&
     & .1155663E-07,.1155203E-07,.1168102E-07,.1152309E-07,.1127853E-07,&
     & .9697158E-08,.9695956E-08,.9680578E-08,.9559403E-08,.9302754E-08,&
     & .8154203E-08,.8120989E-08,.8036412E-08,.7883272E-08,.7698274E-08/

      data   absb(121:235, 2)   /                                       &
     & .6967330E-08,.6906568E-08,.6813175E-08,.6653576E-08,.6523040E-08,&
     & .5881983E-08,.5845620E-08,.5707887E-08,.5564188E-08,.5496845E-08,&
     & .4867982E-08,.4842877E-08,.4723792E-08,.4610581E-08,.4535252E-08,&
     & .4220891E-08,.4197573E-08,.4091011E-08,.3984631E-08,.3910118E-08,&
     & .3693929E-08,.3663715E-08,.3538644E-08,.3460285E-08,.3368615E-08,&
     & .3187896E-08,.3159197E-08,.3046331E-08,.2969790E-08,.2900429E-08,&
     & .2727771E-08,.2707740E-08,.2609750E-08,.2537780E-08,.2466859E-08,&
     & .2336416E-08,.2288611E-08,.2243157E-08,.2156881E-08,.2077391E-08,&
     & .1962744E-08,.1900100E-08,.1900600E-08,.1810939E-08,.1740140E-08,&
     & .1641370E-08,.1585025E-08,.1598322E-08,.1519198E-08,.1446996E-08,&
     & .1369667E-08,.1331096E-08,.1332974E-08,.1271075E-08,.1212575E-08,&
     & .1134932E-08,.1101458E-08,.1088461E-08,.1059785E-08,.1000704E-08,&
     & .9094399E-09,.8846586E-09,.8692260E-09,.8514667E-09,.8067748E-09,&
     & .7475365E-09,.7309474E-09,.7248197E-09,.7107080E-09,.6704647E-09,&
     & .6155314E-09,.6185778E-09,.6128860E-09,.5976466E-09,.5727505E-09,&
     & .5007571E-09,.5092447E-09,.5016075E-09,.4940536E-09,.4741181E-09,&
     & .3942016E-09,.4100999E-09,.4008001E-09,.3983216E-09,.3787150E-09,&
     & .3377646E-09,.3579863E-09,.3566328E-09,.3468912E-09,.3368833E-09,&
     & .3036220E-09,.3322234E-09,.3325357E-09,.3256853E-09,.3158305E-09,&
     & .2804988E-09,.3017266E-09,.3108715E-09,.2978439E-09,.2936541E-09,&
     & .2609790E-09,.2661588E-09,.2860275E-09,.2792342E-09,.2602529E-09,&
     & .2458827E-09,.2411410E-09,.2673329E-09,.2554838E-09,.2352146E-09,&
     & .2526218E-09,.2394002E-09,.2533229E-09,.2725054E-09,.2570953E-09/

      data   absb(  1:120, 3)   /                                       &
     & .2094454E-07,.2103723E-07,.2317421E-07,.2429927E-07,.2443259E-07,&
     & .3496380E-07,.3490941E-07,.3349223E-07,.3343521E-07,.3310576E-07,&
     & .4585045E-07,.4658060E-07,.4768562E-07,.5006330E-07,.5247674E-07,&
     & .6284991E-07,.6579582E-07,.6633160E-07,.6800469E-07,.6989589E-07,&
     & .7248123E-07,.7171694E-07,.7074783E-07,.6969505E-07,.7027375E-07,&
     & .6908485E-07,.6849817E-07,.6716994E-07,.6730050E-07,.6828500E-07,&
     & .6934186E-07,.6902567E-07,.6864856E-07,.6846652E-07,.6943860E-07,&
     & .6706792E-07,.6726913E-07,.6653898E-07,.6592318E-07,.6689835E-07,&
     & .6315693E-07,.6388301E-07,.6354862E-07,.6322257E-07,.6392275E-07,&
     & .6021001E-07,.6063825E-07,.6004737E-07,.5982331E-07,.6053882E-07,&
     & .5662743E-07,.5727057E-07,.5683287E-07,.5634445E-07,.5728420E-07,&
     & .5345713E-07,.5415602E-07,.5376082E-07,.5388461E-07,.5551090E-07,&
     & .5097174E-07,.5174221E-07,.5185410E-07,.5170391E-07,.5343406E-07,&
     & .4885071E-07,.4964076E-07,.4957246E-07,.4927493E-07,.5181847E-07,&
     & .4607099E-07,.4641337E-07,.4642465E-07,.4657013E-07,.4906130E-07,&
     & .4289388E-07,.4288894E-07,.4267067E-07,.4328417E-07,.4696503E-07,&
     & .3872644E-07,.3855498E-07,.3845341E-07,.4062810E-07,.4336730E-07,&
     & .3420706E-07,.3415253E-07,.3481498E-07,.3737966E-07,.3894101E-07,&
     & .2944343E-07,.2985002E-07,.3132611E-07,.3395546E-07,.3326394E-07,&
     & .2542129E-07,.2622059E-07,.2846268E-07,.2890366E-07,.2912570E-07,&
     & .2147717E-07,.2336132E-07,.2481620E-07,.2489857E-07,.2493064E-07,&
     & .1865200E-07,.2074963E-07,.2073473E-07,.2113566E-07,.2126999E-07,&
     & .1634412E-07,.1718199E-07,.1732143E-07,.1735331E-07,.1757109E-07,&
     & .1379461E-07,.1435679E-07,.1452171E-07,.1458681E-07,.1440034E-07/

      data   absb(121:235, 3)   /                                       &
     & .1173430E-07,.1207008E-07,.1230757E-07,.1227543E-07,.1218723E-07,&
     & .9814963E-08,.1008580E-07,.1002846E-07,.1018936E-07,.9936886E-08,&
     & .8116338E-08,.8224445E-08,.8137160E-08,.8175235E-08,.7809894E-08,&
     & .6934363E-08,.6954010E-08,.7028701E-08,.6954052E-08,.6730920E-08,&
     & .6044805E-08,.6051282E-08,.6043548E-08,.5955009E-08,.5737156E-08,&
     & .5148115E-08,.5173181E-08,.5143460E-08,.5091858E-08,.4844251E-08,&
     & .4357708E-08,.4427891E-08,.4341154E-08,.4374327E-08,.4087965E-08,&
     & .3575010E-08,.3758880E-08,.3640315E-08,.3685335E-08,.3462657E-08,&
     & .2951838E-08,.3149133E-08,.2949301E-08,.3004140E-08,.2803498E-08,&
     & .2384856E-08,.2593067E-08,.2441925E-08,.2455559E-08,.2313264E-08,&
     & .1941929E-08,.2123023E-08,.2023449E-08,.2026484E-08,.1914693E-08,&
     & .1543067E-08,.1703838E-08,.1718628E-08,.1615796E-08,.1571495E-08,&
     & .1183980E-08,.1356359E-08,.1356083E-08,.1273436E-08,.1237516E-08,&
     & .9485975E-09,.1113593E-08,.1117504E-08,.1051262E-08,.1016169E-08,&
     & .7815940E-09,.9032580E-09,.9357674E-09,.8676745E-09,.8421725E-09,&
     & .6380256E-09,.7279036E-09,.7623641E-09,.7144892E-09,.6975727E-09,&
     & .5107804E-09,.5598705E-09,.6119439E-09,.5744789E-09,.5583942E-09,&
     & .4463278E-09,.4765332E-09,.5199521E-09,.5043420E-09,.4788736E-09,&
     & .4250688E-09,.4156415E-09,.4855086E-09,.4508491E-09,.4411532E-09,&
     & .3832275E-09,.3697062E-09,.4153029E-09,.4390022E-09,.4199681E-09,&
     & .3471974E-09,.3281596E-09,.3530108E-09,.4078183E-09,.4102159E-09,&
     & .3136929E-09,.3137942E-09,.3117708E-09,.4176296E-09,.4265890E-09,&
     & .3736120E-09,.4038289E-09,.4253194E-09,.5208493E-09,.6353775E-09/

      data   absb(  1:120, 4)   /                                       &
     & .3207017E-07,.3159502E-07,.2858870E-07,.2844242E-07,.2837738E-07,&
     & .3793390E-07,.3899797E-07,.4004916E-07,.4219287E-07,.4324826E-07,&
     & .6367926E-07,.6755417E-07,.7008340E-07,.7122591E-07,.7118042E-07,&
     & .7271716E-07,.6967355E-07,.6852180E-07,.6761184E-07,.6663393E-07,&
     & .6791256E-07,.6626521E-07,.6475029E-07,.6335519E-07,.6341090E-07,&
     & .6702275E-07,.6576791E-07,.6566342E-07,.6383917E-07,.6283841E-07,&
     & .6421842E-07,.6328175E-07,.6271542E-07,.6152237E-07,.6029747E-07,&
     & .6063945E-07,.6050892E-07,.6120783E-07,.6022239E-07,.5821503E-07,&
     & .5811689E-07,.5640629E-07,.5625783E-07,.5511738E-07,.5487818E-07,&
     & .5348844E-07,.5166798E-07,.5260953E-07,.5132573E-07,.5125687E-07,&
     & .5056797E-07,.4945797E-07,.5018027E-07,.5055972E-07,.5015391E-07,&
     & .4934735E-07,.4858752E-07,.4903720E-07,.4872055E-07,.4906669E-07,&
     & .4696374E-07,.4689614E-07,.4670931E-07,.4783716E-07,.4808487E-07,&
     & .4579724E-07,.4485585E-07,.4523316E-07,.4654221E-07,.4697737E-07,&
     & .4177011E-07,.4195636E-07,.4315538E-07,.4366803E-07,.4370471E-07,&
     & .3826636E-07,.3900949E-07,.4040767E-07,.4050401E-07,.3926292E-07,&
     & .3417724E-07,.3398437E-07,.3497769E-07,.3504542E-07,.3635089E-07,&
     & .2972685E-07,.3050472E-07,.3019202E-07,.2941542E-07,.3300928E-07,&
     & .2569885E-07,.2510078E-07,.2547159E-07,.2603840E-07,.3137309E-07,&
     & .2253041E-07,.2113115E-07,.2134413E-07,.2537857E-07,.2492397E-07,&
     & .1910416E-07,.1866822E-07,.2015910E-07,.2178797E-07,.2103215E-07,&
     & .1751768E-07,.1752356E-07,.1997915E-07,.1876099E-07,.1868247E-07,&
     & .1601254E-07,.1739373E-07,.1791673E-07,.1773318E-07,.1719741E-07,&
     & .1475039E-07,.1681135E-07,.1649793E-07,.1611369E-07,.1570679E-07/

      data   absb(121:235, 4)   /                                       &
     & .1303339E-07,.1487026E-07,.1490773E-07,.1478564E-07,.1430577E-07,&
     & .1163810E-07,.1261966E-07,.1306776E-07,.1264305E-07,.1269552E-07,&
     & .9753745E-08,.1052021E-07,.1077719E-07,.1076181E-07,.1057069E-07,&
     & .8428703E-08,.9412717E-08,.9437343E-08,.9161081E-08,.9398817E-08,&
     & .6734710E-08,.7798478E-08,.8144691E-08,.8022473E-08,.8252301E-08,&
     & .5704222E-08,.6683171E-08,.6786990E-08,.6802772E-08,.7124318E-08,&
     & .4641578E-08,.5477035E-08,.5637589E-08,.5540870E-08,.5878158E-08,&
     & .3894186E-08,.4572580E-08,.4683803E-08,.4627519E-08,.4835245E-08,&
     & .3094846E-08,.3631163E-08,.3872510E-08,.3801876E-08,.4051812E-08,&
     & .2505988E-08,.2813430E-08,.3115255E-08,.3108868E-08,.3368799E-08,&
     & .1974147E-08,.2337742E-08,.2632663E-08,.2536104E-08,.2818301E-08,&
     & .1571250E-08,.1795999E-08,.2019658E-08,.2053809E-08,.2214852E-08,&
     & .1279646E-08,.1412932E-08,.1556847E-08,.1606893E-08,.1672149E-08,&
     & .1031072E-08,.1097428E-08,.1196985E-08,.1290650E-08,.1288385E-08,&
     & .8938603E-09,.8837689E-09,.9745065E-09,.1061662E-08,.1059312E-08,&
     & .7322346E-09,.7033864E-09,.7896888E-09,.8643607E-09,.8131766E-09,&
     & .5805907E-09,.5690340E-09,.6243145E-09,.6418920E-09,.6258469E-09,&
     & .5029656E-09,.4690698E-09,.5424865E-09,.6080127E-09,.5841233E-09,&
     & .4250789E-09,.4197107E-09,.4660663E-09,.6124726E-09,.5623562E-09,&
     & .4019151E-09,.3994626E-09,.4466929E-09,.5705557E-09,.5871247E-09,&
     & .3606061E-09,.4105570E-09,.4275902E-09,.5426140E-09,.6398303E-09,&
     & .3779486E-09,.4060351E-09,.4385552E-09,.5030377E-09,.6537476E-09,&
     & .5016060E-09,.5443259E-09,.6140966E-09,.6419200E-09,.8282210E-09/

      data   absb(  1:120, 5)   /                                       &
     & .1729944E-05,.1736124E-05,.1741333E-05,.1742362E-05,.1748113E-05,&
     & .1274396E-05,.1276178E-05,.1276894E-05,.1273619E-05,.1277683E-05,&
     & .7962882E-06,.7847962E-06,.7761880E-06,.7674539E-06,.7644962E-06,&
     & .4154425E-06,.4161228E-06,.4168582E-06,.4153229E-06,.4178350E-06,&
     & .1529712E-06,.1587454E-06,.1627009E-06,.1686135E-06,.1698471E-06,&
     & .5915086E-07,.5907368E-07,.5815373E-07,.5755800E-07,.5672556E-07,&
     & .5797453E-07,.5829676E-07,.5862911E-07,.5886891E-07,.5612632E-07,&
     & .5902708E-07,.5607408E-07,.5455860E-07,.5499325E-07,.5710234E-07,&
     & .5283276E-07,.5163077E-07,.5070747E-07,.5268528E-07,.5249233E-07,&
     & .4705563E-07,.5073384E-07,.5033114E-07,.5234450E-07,.5415567E-07,&
     & .4780445E-07,.4964787E-07,.5059932E-07,.4884527E-07,.5386605E-07,&
     & .4658544E-07,.4857945E-07,.4939821E-07,.4877906E-07,.5115413E-07,&
     & .4761843E-07,.4933275E-07,.5350810E-07,.4890684E-07,.5018725E-07,&
     & .4263688E-07,.5007044E-07,.5014879E-07,.4749372E-07,.4465415E-07,&
     & .4455329E-07,.4855181E-07,.4368603E-07,.4298662E-07,.4551816E-07,&
     & .4053097E-07,.4563140E-07,.3772562E-07,.4071168E-07,.4236715E-07,&
     & .3588889E-07,.3889871E-07,.3518841E-07,.3620149E-07,.3217659E-07,&
     & .3083266E-07,.2840331E-07,.3135215E-07,.3222827E-07,.3059802E-07,&
     & .2609754E-07,.2597536E-07,.2416454E-07,.2407412E-07,.2851713E-07,&
     & .1942653E-07,.2176047E-07,.1994188E-07,.2100237E-07,.2564799E-07,&
     & .1872337E-07,.1873896E-07,.1659557E-07,.1939157E-07,.1979997E-07,&
     & .1520850E-07,.1331818E-07,.1631543E-07,.1634361E-07,.1553305E-07,&
     & .1209783E-07,.1336203E-07,.1561210E-07,.1467477E-07,.1431486E-07,&
     & .1182141E-07,.1046697E-07,.1289913E-07,.1276273E-07,.1359344E-07/

      data   absb(121:235, 5)   /                                       &
     & .1184794E-07,.1311106E-07,.1238478E-07,.1217861E-07,.1239514E-07,&
     & .9765005E-08,.1222800E-07,.1250535E-07,.1265256E-07,.1275629E-07,&
     & .8730087E-08,.1006998E-07,.1108201E-07,.1107370E-07,.1128404E-07,&
     & .7184544E-08,.8280430E-08,.9337916E-08,.1097275E-07,.9422199E-08,&
     & .7139569E-08,.8403293E-08,.8805228E-08,.8723704E-08,.8768754E-08,&
     & .5638776E-08,.6617168E-08,.8131084E-08,.7952481E-08,.7843380E-08,&
     & .4925723E-08,.6108668E-08,.6899038E-08,.7204770E-08,.7687590E-08,&
     & .4080853E-08,.4506541E-08,.5062322E-08,.5570520E-08,.6245092E-08,&
     & .3631086E-08,.3803154E-08,.4237952E-08,.4459341E-08,.4641922E-08,&
     & .2798333E-08,.3196813E-08,.3468125E-08,.3333398E-08,.3634127E-08,&
     & .2078749E-08,.2197782E-08,.2749626E-08,.2961556E-08,.2737758E-08,&
     & .1606200E-08,.1840574E-08,.2290696E-08,.2323292E-08,.2090580E-08,&
     & .1242108E-08,.1231152E-08,.1604400E-08,.1622018E-08,.1621244E-08,&
     & .1142268E-08,.1161378E-08,.1312609E-08,.1332037E-08,.1336670E-08,&
     & .7937740E-09,.9156464E-09,.9774718E-09,.1301830E-08,.1022826E-08,&
     & .6792043E-09,.5672534E-09,.8065168E-09,.9174893E-09,.8812439E-09,&
     & .4994562E-09,.4116159E-09,.6437120E-09,.8032581E-09,.6371815E-09,&
     & .3965500E-09,.4502722E-09,.5469149E-09,.6195100E-09,.6194173E-09,&
     & .4642266E-09,.4594787E-09,.4914294E-09,.5762278E-09,.7042388E-09,&
     & .3468720E-09,.4302644E-09,.4377266E-09,.5298753E-09,.6737359E-09,&
     & .4090614E-09,.3868913E-09,.4712038E-09,.5590598E-09,.5935130E-09,&
     & .3717901E-09,.5393381E-09,.5139211E-09,.5362056E-09,.6355660E-09,&
     & .5106858E-09,.5935961E-09,.6679782E-09,.7444804E-09,.8364933E-09/

      data   absb(  1:120, 6)   /                                       &
     & .2395686E-04,.2404438E-04,.2411693E-04,.2423607E-04,.2436738E-04,&
     & .2056849E-04,.2065059E-04,.2074045E-04,.2087079E-04,.2099978E-04,&
     & .1768295E-04,.1777975E-04,.1792091E-04,.1803977E-04,.1817680E-04,&
     & .1516668E-04,.1531217E-04,.1547416E-04,.1561878E-04,.1578350E-04,&
     & .1290092E-04,.1310023E-04,.1328029E-04,.1346289E-04,.1365293E-04,&
     & .1024835E-04,.1050381E-04,.1075123E-04,.1101880E-04,.1126531E-04,&
     & .7489506E-05,.7760773E-05,.8019168E-05,.8297504E-05,.8573388E-05,&
     & .5464496E-05,.5734262E-05,.6016157E-05,.6302015E-05,.6577927E-05,&
     & .3936259E-05,.4224419E-05,.4513834E-05,.4789178E-05,.5070700E-05,&
     & .2805984E-05,.3072729E-05,.3338950E-05,.3623166E-05,.3891777E-05,&
     & .1900823E-05,.2167276E-05,.2408743E-05,.2685381E-05,.2917449E-05,&
     & .1174025E-05,.1409556E-05,.1643976E-05,.1906921E-05,.2111832E-05,&
     & .6636735E-06,.8537937E-06,.1064189E-05,.1305203E-05,.1520120E-05,&
     & .3317132E-06,.4631507E-06,.6718687E-06,.8755380E-06,.1099835E-05,&
     & .1412547E-06,.2795370E-06,.4605960E-06,.6358157E-06,.8299133E-06,&
     & .5886507E-07,.1686485E-06,.3427727E-06,.5009229E-06,.7048885E-06,&
     & .5375997E-07,.1921376E-06,.3466112E-06,.4979867E-06,.7234630E-06,&
     & .9833696E-07,.2424778E-06,.3777826E-06,.5767946E-06,.7689130E-06,&
     & .1780398E-06,.3286727E-06,.5106210E-06,.7063377E-06,.9191132E-06,&
     & .2791479E-06,.4463444E-06,.6419014E-06,.8393519E-06,.1186341E-05,&
     & .3815600E-06,.5566532E-06,.7712321E-06,.1041457E-05,.1452473E-05,&
     & .4683750E-06,.6571130E-06,.8750377E-06,.1244250E-05,.1672199E-05,&
     & .5302781E-06,.7255338E-06,.9960660E-06,.1375837E-05,.1817809E-05,&
     & .5718819E-06,.7824923E-06,.1082650E-05,.1463943E-05,.1914798E-05/

      data   absb(121:235, 6)   /                                       &
     & .5779563E-06,.7793008E-06,.1089554E-05,.1465980E-05,.1912492E-05,&
     & .6012058E-06,.8101817E-06,.1114671E-05,.1481773E-05,.1917882E-05,&
     & .6359830E-06,.8577926E-06,.1156572E-05,.1515261E-05,.1953664E-05,&
     & .6218324E-06,.8312575E-06,.1120830E-05,.1466837E-05,.1889580E-05,&
     & .6020577E-06,.7988481E-06,.1076839E-05,.1420463E-05,.1818710E-05,&
     & .5888633E-06,.7776144E-06,.1041644E-05,.1370194E-05,.1754483E-05,&
     & .5631597E-06,.7415089E-06,.9956498E-06,.1305861E-05,.1669685E-05,&
     & .5356640E-06,.7073841E-06,.9466091E-06,.1238098E-05,.1585048E-05,&
     & .5104786E-06,.6741957E-06,.8973165E-06,.1175203E-05,.1505003E-05,&
     & .4827379E-06,.6379585E-06,.8434834E-06,.1105762E-05,.1414222E-05,&
     & .4471224E-06,.5931371E-06,.7785484E-06,.1020135E-05,.1311082E-05,&
     & .4093237E-06,.5500658E-06,.7202141E-06,.9432841E-06,.1215064E-05,&
     & .3743207E-06,.5084612E-06,.6692669E-06,.8748475E-06,.1125655E-05,&
     & .3375380E-06,.4645308E-06,.6183099E-06,.8066744E-06,.1040697E-05,&
     & .3011207E-06,.4225809E-06,.5687082E-06,.7426667E-06,.9612401E-06,&
     & .2668279E-06,.3832441E-06,.5209688E-06,.6850474E-06,.8874070E-06,&
     & .2381612E-06,.3456426E-06,.4754255E-06,.6316630E-06,.8198579E-06,&
     & .2073586E-06,.3073931E-06,.4304397E-06,.5791758E-06,.7545941E-06,&
     & .1768447E-06,.2704831E-06,.3869748E-06,.5275204E-06,.6934370E-06,&
     & .1503327E-06,.2353970E-06,.3469204E-06,.4790892E-06,.6362293E-06,&
     & .1255561E-06,.2028854E-06,.3081471E-06,.4327060E-06,.5822767E-06,&
     & .1038629E-06,.1726647E-06,.2718683E-06,.3906697E-06,.5319483E-06,&
     & .8678777E-07,.1522598E-06,.2477843E-06,.3639270E-06,.4979000E-06/

      data   absb(  1:120, 7)   /                                       &
     & .3453565E-03,.3534617E-03,.3619531E-03,.3711305E-03,.3812639E-03,&
     & .3141656E-03,.3221962E-03,.3311595E-03,.3411912E-03,.3521288E-03,&
     & .2869233E-03,.2952510E-03,.3048137E-03,.3155602E-03,.3272615E-03,&
     & .2633665E-03,.2725298E-03,.2826467E-03,.2940184E-03,.3062037E-03,&
     & .2432615E-03,.2533698E-03,.2641385E-03,.2758932E-03,.2885698E-03,&
     & .2263129E-03,.2373054E-03,.2487516E-03,.2608205E-03,.2739867E-03,&
     & .2120588E-03,.2239554E-03,.2359421E-03,.2484613E-03,.2621061E-03,&
     & .2005107E-03,.2131852E-03,.2256599E-03,.2387045E-03,.2528263E-03,&
     & .1911925E-03,.2045282E-03,.2175666E-03,.2310156E-03,.2455442E-03,&
     & .1846431E-03,.1985047E-03,.2120615E-03,.2260435E-03,.2409818E-03,&
     & .1798623E-03,.1940866E-03,.2081664E-03,.2226065E-03,.2379772E-03,&
     & .1766175E-03,.1912018E-03,.2057461E-03,.2205236E-03,.2363363E-03,&
     & .1746678E-03,.1895715E-03,.2044754E-03,.2196431E-03,.2358377E-03,&
     & .1739647E-03,.1892010E-03,.2043935E-03,.2199752E-03,.2365860E-03,&
     & .1742702E-03,.1897414E-03,.2051758E-03,.2211667E-03,.2381552E-03,&
     & .1753432E-03,.1911018E-03,.2067274E-03,.2230734E-03,.2404076E-03,&
     & .1772791E-03,.1932058E-03,.2090684E-03,.2257815E-03,.2433940E-03,&
     & .1798122E-03,.1958563E-03,.2119375E-03,.2289267E-03,.2468559E-03,&
     & .1828739E-03,.1989598E-03,.2152598E-03,.2325938E-03,.2507241E-03,&
     & .1862774E-03,.2023367E-03,.2189782E-03,.2365487E-03,.2548744E-03,&
     & .1898946E-03,.2060048E-03,.2229518E-03,.2407447E-03,.2592115E-03,&
     & .1930147E-03,.2092865E-03,.2264529E-03,.2444439E-03,.2630292E-03,&
     & .1947320E-03,.2110719E-03,.2284152E-03,.2464695E-03,.2651495E-03,&
     & .1947235E-03,.2111115E-03,.2284595E-03,.2465787E-03,.2652695E-03/

      data   absb(121:235, 7)   /                                       &
     & .1923135E-03,.2086721E-03,.2258790E-03,.2439421E-03,.2625178E-03,&
     & .1898304E-03,.2061203E-03,.2232301E-03,.2411648E-03,.2597217E-03,&
     & .1873764E-03,.2036289E-03,.2206075E-03,.2384662E-03,.2569599E-03,&
     & .1832260E-03,.1995592E-03,.2162785E-03,.2339279E-03,.2523120E-03,&
     & .1788996E-03,.1952895E-03,.2118287E-03,.2292618E-03,.2475232E-03,&
     & .1745665E-03,.1910561E-03,.2074156E-03,.2246436E-03,.2426726E-03,&
     & .1693616E-03,.1858699E-03,.2021660E-03,.2190895E-03,.2369106E-03,&
     & .1636984E-03,.1802123E-03,.1965993E-03,.2131937E-03,.2307370E-03,&
     & .1579848E-03,.1745716E-03,.1910412E-03,.2074387E-03,.2246686E-03,&
     & .1517765E-03,.1686193E-03,.1851339E-03,.2014696E-03,.2183450E-03,&
     & .1446931E-03,.1619128E-03,.1784332E-03,.1948757E-03,.2113871E-03,&
     & .1374469E-03,.1550648E-03,.1717760E-03,.1882836E-03,.2046026E-03,&
     & .1301193E-03,.1480729E-03,.1651093E-03,.1816194E-03,.1979742E-03,&
     & .1232387E-03,.1413641E-03,.1587786E-03,.1753667E-03,.1918502E-03,&
     & .1165074E-03,.1347306E-03,.1525223E-03,.1693426E-03,.1858642E-03,&
     & .1098514E-03,.1281199E-03,.1461473E-03,.1632796E-03,.1797964E-03,&
     & .1032338E-03,.1214316E-03,.1396126E-03,.1571081E-03,.1737446E-03,&
     & .9707472E-04,.1152633E-03,.1335133E-03,.1513622E-03,.1682314E-03,&
     & .9122100E-04,.1094451E-03,.1276962E-03,.1457224E-03,.1629077E-03,&
     & .8538680E-04,.1036587E-03,.1218392E-03,.1400148E-03,.1575026E-03,&
     & .7959199E-04,.9778645E-04,.1159632E-03,.1342022E-03,.1520145E-03,&
     & .7417050E-04,.9223069E-04,.1104637E-03,.1287103E-03,.1467060E-03,&
     & .7201794E-04,.9003123E-04,.1082584E-03,.1264928E-03,.1445780E-03/

      data   absb(  1:120, 8)   /                                       &
     & .8358600E-02,.8311878E-02,.8262807E-02,.8209212E-02,.8148456E-02,&
     & .8611422E-02,.8564644E-02,.8512562E-02,.8452401E-02,.8387225E-02,&
     & .8829188E-02,.8780796E-02,.8724483E-02,.8660460E-02,.8590523E-02,&
     & .9017237E-02,.8963386E-02,.8903457E-02,.8835037E-02,.8761588E-02,&
     & .9177720E-02,.9117835E-02,.9052597E-02,.8982454E-02,.8905848E-02,&
     & .9312725E-02,.9247622E-02,.9178554E-02,.9105643E-02,.9025534E-02,&
     & .9426857E-02,.9355560E-02,.9283636E-02,.9207311E-02,.9124846E-02,&
     & .9520420E-02,.9444026E-02,.9368533E-02,.9289838E-02,.9203532E-02,&
     & .9596404E-02,.9516161E-02,.9437583E-02,.9356012E-02,.9267042E-02,&
     & .9652643E-02,.9569500E-02,.9487039E-02,.9401969E-02,.9310716E-02,&
     & .9695422E-02,.9609692E-02,.9524686E-02,.9435966E-02,.9342924E-02,&
     & .9727439E-02,.9637865E-02,.9550739E-02,.9460899E-02,.9364315E-02,&
     & .9747348E-02,.9657887E-02,.9567816E-02,.9475420E-02,.9377002E-02,&
     & .9759778E-02,.9668908E-02,.9576933E-02,.9481742E-02,.9380067E-02,&
     & .9764468E-02,.9671597E-02,.9578772E-02,.9481823E-02,.9378210E-02,&
     & .9763831E-02,.9670378E-02,.9575566E-02,.9476824E-02,.9370080E-02,&
     & .9759671E-02,.9663673E-02,.9567437E-02,.9466585E-02,.9358136E-02,&
     & .9749029E-02,.9653783E-02,.9555706E-02,.9452450E-02,.9342273E-02,&
     & .9736435E-02,.9639659E-02,.9540533E-02,.9435413E-02,.9323567E-02,&
     & .9720078E-02,.9623847E-02,.9522512E-02,.9415460E-02,.9302259E-02,&
     & .9703088E-02,.9605298E-02,.9501404E-02,.9392140E-02,.9278916E-02,&
     & .9686283E-02,.9587583E-02,.9483967E-02,.9371995E-02,.9257294E-02,&
     & .9678111E-02,.9578966E-02,.9473567E-02,.9361585E-02,.9246698E-02,&
     & .9679790E-02,.9580995E-02,.9474493E-02,.9363423E-02,.9247588E-02/

      data   absb(121:235, 8)   /                                       &
     & .9697036E-02,.9597631E-02,.9491980E-02,.9381268E-02,.9266303E-02,&
     & .9712567E-02,.9614449E-02,.9509960E-02,.9399294E-02,.9284125E-02,&
     & .9728855E-02,.9630143E-02,.9527100E-02,.9417372E-02,.9302616E-02,&
     & .9755136E-02,.9656175E-02,.9554532E-02,.9445454E-02,.9332069E-02,&
     & .9781450E-02,.9682450E-02,.9581889E-02,.9475267E-02,.9362746E-02,&
     & .9808399E-02,.9708959E-02,.9609987E-02,.9504452E-02,.9392809E-02,&
     & .9839580E-02,.9740865E-02,.9642287E-02,.9538676E-02,.9428802E-02,&
     & .9874370E-02,.9776400E-02,.9676193E-02,.9575430E-02,.9467798E-02,&
     & .9908878E-02,.9809623E-02,.9710657E-02,.9611071E-02,.9505151E-02,&
     & .9946962E-02,.9845757E-02,.9746231E-02,.9647315E-02,.9544139E-02,&
     & .9988261E-02,.9885906E-02,.9786939E-02,.9687236E-02,.9586655E-02,&
     & .1003101E-01,.9927336E-02,.9827361E-02,.9728363E-02,.9628271E-02,&
     & .1007558E-01,.9968398E-02,.9867081E-02,.9768397E-02,.9669498E-02,&
     & .1011578E-01,.1000878E-01,.9905269E-02,.9805950E-02,.9706828E-02,&
     & .1015569E-01,.1004819E-01,.9942285E-02,.9841944E-02,.9742563E-02,&
     & .1019452E-01,.1008719E-01,.9980576E-02,.9878353E-02,.9779624E-02,&
     & .1023378E-01,.1012695E-01,.1001916E-01,.9915168E-02,.9815063E-02,&
     & .1026952E-01,.1016297E-01,.1005517E-01,.9949717E-02,.9848721E-02,&
     & .1030346E-01,.1019748E-01,.1008930E-01,.9983003E-02,.9880655E-02,&
     & .1033774E-01,.1023094E-01,.1012441E-01,.1001809E-01,.9912916E-02,&
     & .1037133E-01,.1026548E-01,.1015955E-01,.1005208E-01,.9945433E-02,&
     & .1040238E-01,.1029750E-01,.1019014E-01,.1008328E-01,.9977541E-02,&
     & .1041506E-01,.1031116E-01,.1020458E-01,.1009725E-01,.9990140E-02/

!  ---
      data   forref(1:3,1: 8)    /                                      &
     & .1889348E-07,.2790121E-06,.2442243E-05,.3917008E-06,.7957867E-06,&
     & .1584630E-05,.1690886E-05,.1737327E-05,.5814421E-06,.4276674E-05,&
     & .3754692E-05,.3297487E-06,.4845389E-05,.3046027E-05,.8469466E-06,&
     & .5624835E-05,.1676442E-05,.3856211E-07,.3141158E-05,.1459015E-07,&
     & .2568816E-07,.8749816E-12,.8199214E-12,.8132942E-12/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  for instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 8).

      data   selfref(1:10,1: 8)  /                                      &
     & .1216369E-04,.7423490E-05,.4532882E-05,.2769295E-05,.1692770E-05,&
     & .1035295E-05,.6335399E-06,.3879104E-06,.2376510E-06,.1456812E-06,&
     & .1738951E-04,.1350041E-04,.1061362E-04,.8452065E-05,.6817644E-05,&
     & .5568311E-05,.4602063E-05,.3845424E-05,.3245295E-05,.2763136E-05,&
     & .2573945E-04,.2373710E-04,.2191428E-04,.2025397E-04,.1874100E-04,&
     & .1736167E-04,.1610330E-04,.1495474E-04,.1390567E-04,.1294697E-04,&
     & .5188697E-04,.4899932E-04,.4628324E-04,.4372800E-04,.4132363E-04,&
     & .3906076E-04,.3693055E-04,.3492493E-04,.3303618E-04,.3125709E-04,&
     & .3645284E-04,.3672976E-04,.3702492E-04,.3733989E-04,.3767626E-04,&
     & .3803589E-04,.3842086E-04,.3883327E-04,.3927557E-04,.3975032E-04,&
     & .1526953E-04,.1771468E-04,.2057485E-04,.2392454E-04,.2785266E-04,&
     & .3246491E-04,.3788738E-04,.4427053E-04,.5179416E-04,.6067315E-04,&
     & .1304688E-06,.2753468E-06,.5813077E-06,.1227665E-05,.2593570E-05,&
     & .5480882E-05,.1158601E-04,.2449863E-04,.5181665E-04,.1096252E-03,&
     & .2493662E-06,.2172185E-06,.1892149E-06,.1648213E-06,.1435736E-06,&
     & .1250641E-06,.1089414E-06,.9489759E-07,.8266404E-07,.7200766E-07/

!........................................!
      end module module_radsw_kgb24      !
!========================================!


!========================================!
      module module_radsw_kgb25          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 25: 16000-22650 cm-1 (low - h2o; high - nothing)           !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG25

!
      implicit none
!
      private
!
      integer,  public :: MSA25
      parameter (MSA25=65)

      real (kind=kind_phys), public ::                                  &
     &       absa(MSA25,NG25), rayl(NG25), abso3a(NG25), abso3b(NG25)

!  ---  rayleigh extinction coefficient at v =      cm-1.
      data   rayl  (1: 6)        /                         .9811320E-06,&
     & .8256050E-06,.6146670E-06,.3838724E-06,.4417251E-06,.3613850E-06/

!  ---  o3
      data   abso3a(1: 6)        /                         .2326640E-01,&
     & .5761540E-01,.1854021E+00,.3896100E+00,.3610619E+00,.4362158E+00/

      data   abso3b(1: 6)        /                         .1769170E-01,&
     & .4641850E-01,.1449232E+00,.3484920E+00,.4676420E+00,.5178092E+00/

!     the array absa(65,NG25) (ka(5,13,NG25)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 6, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1: 65, 1)   /                                       &
     & .1646100E-08,.1678200E-08,.1933900E-08,.1710000E-08,.1704500E-08,&
     & .2875900E-08,.2946900E-08,.3378900E-08,.3435700E-08,.2883300E-08,&
     & .5514800E-08,.5480800E-08,.5419000E-08,.6826000E-08,.5197200E-08,&
     & .9533600E-08,.9455200E-08,.9300100E-08,.9096100E-08,.1445100E-07,&
     & .1493000E-07,.1473600E-07,.1443200E-07,.1407400E-07,.2410200E-07,&
     & .2277000E-07,.2230100E-07,.2177800E-07,.2119400E-07,.2056900E-07,&
     & .3469900E-07,.3395100E-07,.3312400E-07,.3214400E-07,.3122000E-07,&
     & .6233900E-07,.6040500E-07,.5954800E-07,.5821400E-07,.5697700E-07,&
     & .1741100E-06,.1765400E-06,.1831500E-06,.1810000E-06,.1783900E-06,&
     & .2352600E-06,.2272900E-06,.2194700E-06,.2118800E-06,.2045400E-06,&
     & .2353500E-06,.2273700E-06,.2195600E-06,.2119600E-06,.2046100E-06,&
     & .2353900E-06,.2274000E-06,.2195900E-06,.2119900E-06,.2046500E-06,&
     & .2354300E-06,.2274400E-06,.2196200E-06,.2120200E-06,.2046700E-06/

      data   absa(  1: 65, 2)   /                                       &
     & .6291200E-08,.6155900E-08,.8464000E-08,.5924000E-08,.5821700E-08,&
     & .8374900E-08,.8075600E-08,.1162300E-07,.1127200E-07,.7363600E-08,&
     & .1330400E-07,.1279500E-07,.1234300E-07,.2123500E-07,.1157700E-07,&
     & .2070400E-07,.1973600E-07,.1890000E-07,.1822800E-07,.3160100E-07,&
     & .3114900E-07,.2966900E-07,.2831800E-07,.2710100E-07,.4964900E-07,&
     & .4571300E-07,.4351900E-07,.4148800E-07,.3991800E-07,.3829100E-07,&
     & .7726500E-07,.7384800E-07,.7043700E-07,.6794500E-07,.6612700E-07,&
     & .1575400E-06,.1566400E-06,.1537800E-06,.1502700E-06,.1463300E-06,&
     & .1643900E-06,.1467800E-06,.1261000E-06,.1153200E-06,.1059100E-06,&
     & .1436600E-06,.1350600E-06,.1258300E-06,.1177400E-06,.1101100E-06,&
     & .1452100E-06,.1376600E-06,.1307200E-06,.1221800E-06,.1140000E-06,&
     & .1452400E-06,.1376900E-06,.1307400E-06,.1224100E-06,.1155200E-06,&
     & .1452500E-06,.1377000E-06,.1307500E-06,.1225200E-06,.1155300E-06/

      data   absa(  1: 65, 3)   /                                       &
     & .2304255E-07,.2212848E-07,.4109493E-07,.2019160E-07,.1941596E-07,&
     & .2701108E-07,.2633924E-07,.5665284E-07,.5516586E-07,.2490900E-07,&
     & .4992253E-07,.4901988E-07,.4802034E-07,.7658263E-07,.4679826E-07,&
     & .9936387E-07,.9737083E-07,.9572112E-07,.9398584E-07,.1435192E-06,&
     & .1737489E-06,.1690329E-06,.1649946E-06,.1608759E-06,.2616677E-06,&
     & .2850168E-06,.2771528E-06,.2703079E-06,.2636367E-06,.2572958E-06,&
     & .4363192E-06,.4270658E-06,.4142017E-06,.4031191E-06,.3921783E-06,&
     & .6694834E-06,.6560782E-06,.6423565E-06,.6305881E-06,.6197580E-06,&
     & .1127398E-05,.1109264E-05,.1096811E-05,.1079010E-05,.1060076E-05,&
     & .1142385E-05,.1123159E-05,.1103836E-05,.1083587E-05,.1063058E-05,&
     & .1141922E-05,.1122126E-05,.1101516E-05,.1081516E-05,.1061329E-05,&
     & .1142101E-05,.1122316E-05,.1101648E-05,.1081539E-05,.1060666E-05,&
     & .1142284E-05,.1122439E-05,.1101810E-05,.1081628E-05,.1060802E-05/

      data   absa(  1: 65, 4)   /                                       &
     & .5358826E-07,.5191800E-07,.1241794E-06,.4941069E-07,.4826537E-07,&
     & .6634855E-07,.6497957E-07,.1011451E-06,.9587780E-07,.6034307E-07,&
     & .1109051E-06,.1079549E-06,.1052181E-06,.1673772E-06,.9938209E-07,&
     & .1837864E-06,.1778823E-06,.1716021E-06,.1661914E-06,.2696765E-06,&
     & .2944786E-06,.2858136E-06,.2767250E-06,.2693989E-06,.4648876E-06,&
     & .4455888E-06,.4336889E-06,.4201096E-06,.4076205E-06,.3956245E-06,&
     & .6742110E-06,.6611647E-06,.6510830E-06,.6399340E-06,.6276101E-06,&
     & .8969775E-06,.8864724E-06,.8790142E-06,.8698967E-06,.8588636E-06,&
     & .7342734E-06,.7160851E-06,.6891067E-06,.6701000E-06,.6510138E-06,&
     & .7386682E-06,.7215150E-06,.7046031E-06,.6851616E-06,.6677641E-06,&
     & .7580174E-06,.7411749E-06,.7247261E-06,.7009916E-06,.6816196E-06,&
     & .7649827E-06,.7506295E-06,.7339233E-06,.7090960E-06,.6870193E-06,&
     & .7676832E-06,.7529800E-06,.7370035E-06,.7118011E-06,.6888372E-06/

      data   absa(  1: 65, 5)   /                                       &
     & .7117654E-05,.7054566E-05,.8530292E-05,.6912687E-05,.6834460E-05,&
     & .6579625E-05,.6521231E-05,.8074960E-05,.7892008E-05,.6316120E-05,&
     & .5972427E-05,.5925015E-05,.5871889E-05,.7336016E-05,.5749214E-05,&
     & .5324376E-05,.5290991E-05,.5251560E-05,.5205073E-05,.6531106E-05,&
     & .4622548E-05,.4605694E-05,.4579758E-05,.4546281E-05,.5745046E-05,&
     & .3863569E-05,.3864350E-05,.3857382E-05,.3842378E-05,.3819669E-05,&
     & .3076464E-05,.3091729E-05,.3102707E-05,.3099905E-05,.3090770E-05,&
     & .2816484E-05,.2825815E-05,.2821070E-05,.2804759E-05,.2789727E-05,&
     & .6454490E-05,.6508164E-05,.6551126E-05,.6559912E-05,.6545487E-05,&
     & .1257978E-04,.1269684E-04,.1274689E-04,.1276159E-04,.1276158E-04,&
     & .1419438E-04,.1414710E-04,.1407662E-04,.1400149E-04,.1389726E-04,&
     & .1419357E-04,.1414993E-04,.1408845E-04,.1402116E-04,.1393645E-04,&
     & .1419415E-04,.1414965E-04,.1408763E-04,.1402335E-04,.1394694E-04/

      data   absa(  1: 65, 6)   /                                       &
     & .1587872E-03,.1576306E-03,.1591069E-03,.1553587E-03,.1542553E-03,&
     & .1690035E-03,.1677837E-03,.1693876E-03,.1678861E-03,.1641727E-03,&
     & .1796181E-03,.1782899E-03,.1769631E-03,.1785679E-03,.1743591E-03,&
     & .1896678E-03,.1882560E-03,.1868296E-03,.1854223E-03,.1871320E-03,&
     & .1991122E-03,.1976085E-03,.1961276E-03,.1946438E-03,.1968553E-03,&
     & .2076995E-03,.2061273E-03,.2045536E-03,.2029803E-03,.2014317E-03,&
     & .2138594E-03,.2121941E-03,.2104844E-03,.2088605E-03,.2072504E-03,&
     & .2079296E-03,.2064979E-03,.2051095E-03,.2037779E-03,.2023372E-03,&
     & .1359973E-03,.1343382E-03,.1327011E-03,.1314602E-03,.1305188E-03,&
     & .2924225E-04,.2660282E-04,.2493377E-04,.2376549E-04,.2266769E-04,&
     & .7730806E-06,.9895626E-06,.1411477E-05,.1894061E-05,.2654902E-05,&
     & .6900584E-06,.7953827E-06,.1063045E-05,.1425467E-05,.1894041E-05,&
     & .6604518E-06,.7863470E-06,.1049399E-05,.1366331E-05,.1711475E-05/

!........................................!
      end module module_radsw_kgb25      !
!========================================!


!========================================!
      module module_radsw_kgb26          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 26: 22650-29000 cm-1 (low - nothing; high - nothing)       !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG26

!
      implicit none
!
      private
!
      real (kind=kind_phys), public ::  rayl(NG26)

!  ---  rayleigh extinction coefficient at all v
      data   rayl  (1: 6)        /                         .1212630E-05,&
     & .1434280E-05,.1799798E-05,.2307617E-05,.2814376E-05,.3092339E-05/

!........................................!
      end module module_radsw_kgb26      !
!========================================!


!========================================!
      module module_radsw_kgb27          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 27: 29000-38000 cm-1 (low - o3; high - o3)
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG27

!
      implicit none
!
      private
!
      integer,  public :: MSA27, MSB27
      parameter (MSA27=65, MSB27=235)

      real (kind=kind_phys), public ::                                  &
     &       absa(MSA27,NG27), absb(MSB27,NG27), rayl(NG27)

!  ---  rayleigh extinction coefficient at v =      cm-1.
      data   rayl (1: 8)       / .3445340E-05,.4144800E-05,.4950690E-05,&
     & .5812040E-05,.6697480E-05,.7564880E-05,.8886761E-05,.9744758E-05/

!     the array absa(65,NG27) (ka(5,13,NG27)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 8, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1: 65, 1)   /                                       &
     & .2290700E+00,.2562500E+00,.2877900E+00,.3237600E+00,.3642600E+00,&
     & .2191300E+00,.2444500E+00,.2742200E+00,.3083200E+00,.3469400E+00,&
     & .2060200E+00,.2286400E+00,.2557400E+00,.2872100E+00,.3231000E+00,&
     & .1937900E+00,.2135200E+00,.2377100E+00,.2664300E+00,.2994000E+00,&
     & .1836900E+00,.2006100E+00,.2220000E+00,.2478600E+00,.2781600E+00,&
     & .1751700E+00,.1892000E+00,.2077100E+00,.2306900E+00,.2581700E+00,&
     & .1688800E+00,.1798800E+00,.1955900E+00,.2157700E+00,.2404200E+00,&
     & .1644200E+00,.1727300E+00,.1856500E+00,.2031500E+00,.2251200E+00,&
     & .1615900E+00,.1673800E+00,.1775200E+00,.1924200E+00,.2117900E+00,&
     & .1603700E+00,.1640200E+00,.1720400E+00,.1846300E+00,.2018300E+00,&
     & .1603600E+00,.1639900E+00,.1719800E+00,.1845500E+00,.2017300E+00,&
     & .1603600E+00,.1639900E+00,.1719800E+00,.1845500E+00,.2017300E+00,&
     & .1603600E+00,.1639900E+00,.1719800E+00,.1845500E+00,.2017300E+00/

      data   absa(  1: 65, 2)   /                                       &
     & .2071600E+01,.2192500E+01,.2332700E+01,.2492100E+01,.2672600E+01,&
     & .2027200E+01,.2140100E+01,.2272300E+01,.2423800E+01,.2595000E+01,&
     & .1968400E+01,.2069700E+01,.2190200E+01,.2330100E+01,.2489200E+01,&
     & .1913300E+01,.2002100E+01,.2110100E+01,.2237500E+01,.2384200E+01,&
     & .1867300E+01,.1944100E+01,.2040000E+01,.2155300E+01,.2289800E+01,&
     & .1827700E+01,.1892500E+01,.1976000E+01,.2078800E+01,.2200900E+01,&
     & .1796300E+01,.1849700E+01,.1921400E+01,.2012100E+01,.2122200E+01,&
     & .1774900E+01,.1815600E+01,.1876300E+01,.1955500E+01,.2054000E+01,&
     & .1760000E+01,.1788500E+01,.1838800E+01,.1907100E+01,.1994300E+01,&
     & .1752600E+01,.1772900E+01,.1812100E+01,.1871600E+01,.1949600E+01,&
     & .1752500E+01,.1772700E+01,.1811900E+01,.1871200E+01,.1949100E+01,&
     & .1752500E+01,.1772700E+01,.1811900E+01,.1871200E+01,.1949100E+01,&
     & .1752500E+01,.1772700E+01,.1811900E+01,.1871200E+01,.1949100E+01/

      data   absa(  1: 65, 3)   /                                       &
     & .1218900E+02,.1261900E+02,.1311000E+02,.1366300E+02,.1427500E+02,&
     & .1203000E+02,.1243400E+02,.1289900E+02,.1342700E+02,.1401500E+02,&
     & .1181600E+02,.1218300E+02,.1261100E+02,.1310100E+02,.1365300E+02,&
     & .1161100E+02,.1193800E+02,.1232700E+02,.1277700E+02,.1328900E+02,&
     & .1143700E+02,.1172600E+02,.1207600E+02,.1248800E+02,.1296100E+02,&
     & .1128400E+02,.1153300E+02,.1184300E+02,.1221500E+02,.1264900E+02,&
     & .1116000E+02,.1137000E+02,.1164100E+02,.1197500E+02,.1237000E+02,&
     & .1106300E+02,.1123700E+02,.1147100E+02,.1176800E+02,.1212600E+02,&
     & .1099000E+02,.1112900E+02,.1132700E+02,.1158800E+02,.1191000E+02,&
     & .1094500E+02,.1105300E+02,.1122300E+02,.1145400E+02,.1174600E+02,&
     & .1094500E+02,.1105200E+02,.1122200E+02,.1145200E+02,.1174400E+02,&
     & .1094500E+02,.1105200E+02,.1122200E+02,.1145200E+02,.1174400E+02,&
     & .1094500E+02,.1105200E+02,.1122200E+02,.1145200E+02,.1174400E+02/

      data   absa(  1: 65, 4)   /                                       &
     & .5505700E+02,.5614300E+02,.5736400E+02,.5871900E+02,.6020800E+02,&
     & .5464700E+02,.5567700E+02,.5684200E+02,.5814200E+02,.5957500E+02,&
     & .5409000E+02,.5503900E+02,.5612300E+02,.5734200E+02,.5869400E+02,&
     & .5354800E+02,.5441000E+02,.5540800E+02,.5653900E+02,.5780500E+02,&
     & .5307400E+02,.5385300E+02,.5476600E+02,.5581300E+02,.5699500E+02,&
     & .5264400E+02,.5333600E+02,.5416200E+02,.5512300E+02,.5621800E+02,&
     & .5227900E+02,.5288700E+02,.5362900E+02,.5450500E+02,.5551600E+02,&
     & .5197900E+02,.5250700E+02,.5316800E+02,.5396500E+02,.5489500E+02,&
     & .5173400E+02,.5218300E+02,.5276700E+02,.5348500E+02,.5433700E+02,&
     & .5156500E+02,.5194800E+02,.5246700E+02,.5312000E+02,.5390700E+02,&
     & .5156300E+02,.5194600E+02,.5246400E+02,.5311600E+02,.5390200E+02,&
     & .5156300E+02,.5194600E+02,.5246400E+02,.5311600E+02,.5390200E+02,&
     & .5156300E+02,.5194600E+02,.5246400E+02,.5311600E+02,.5390200E+02/

      data   absa(  1: 65, 5)   /                                       &
     & .1779400E+03,.1797300E+03,.1816400E+03,.1836600E+03,.1858100E+03,&
     & .1772400E+03,.1789800E+03,.1808300E+03,.1828100E+03,.1849100E+03,&
     & .1762400E+03,.1779100E+03,.1797000E+03,.1816000E+03,.1836300E+03,&
     & .1752300E+03,.1768200E+03,.1785300E+03,.1803600E+03,.1823100E+03,&
     & .1742900E+03,.1758100E+03,.1774400E+03,.1792000E+03,.1810700E+03,&
     & .1733800E+03,.1748200E+03,.1763800E+03,.1780500E+03,.1798500E+03,&
     & .1725300E+03,.1739000E+03,.1753900E+03,.1769900E+03,.1787100E+03,&
     & .1717700E+03,.1730700E+03,.1744800E+03,.1760100E+03,.1776700E+03,&
     & .1710700E+03,.1723000E+03,.1736400E+03,.1751100E+03,.1766900E+03,&
     & .1705200E+03,.1716900E+03,.1729800E+03,.1743800E+03,.1759100E+03,&
     & .1705100E+03,.1716800E+03,.1729700E+03,.1743700E+03,.1759000E+03,&
     & .1705100E+03,.1716800E+03,.1729700E+03,.1743700E+03,.1759000E+03,&
     & .1705100E+03,.1716800E+03,.1729700E+03,.1743700E+03,.1759000E+03/

      data   absa(  1: 65, 6)   /                                       &
     & .3768000E+05,.3680100E+05,.3601100E+05,.3529800E+05,.3471500E+05,&
     & .3928600E+03,.3946300E+03,.3961600E+03,.3974800E+03,.3985800E+03,&
     & .3916900E+03,.3935900E+03,.3952600E+03,.3967100E+03,.3979400E+03,&
     & .3903400E+03,.3923900E+03,.3942100E+03,.3958000E+03,.3971700E+03,&
     & .3889500E+03,.3911300E+03,.3930900E+03,.3948300E+03,.3963400E+03,&
     & .3874200E+03,.3897500E+03,.3918500E+03,.3937300E+03,.3953900E+03,&
     & .3858500E+03,.3883200E+03,.3905600E+03,.3925800E+03,.3943800E+03,&
     & .3842600E+03,.3868600E+03,.3892400E+03,.3914000E+03,.3933300E+03,&
     & .3826400E+03,.3853800E+03,.3878900E+03,.3901700E+03,.3922300E+03,&
     & .3812300E+03,.3840800E+03,.3867000E+03,.3890900E+03,.3912600E+03,&
     & .3812200E+03,.3840600E+03,.3866800E+03,.3890800E+03,.3912500E+03,&
     & .3812200E+03,.3840600E+03,.3866800E+03,.3890800E+03,.3912500E+03,&
     & .3812200E+03,.3840600E+03,.3866800E+03,.3890800E+03,.3912500E+03/

      data   absa(  1: 65, 7)   /                                       &
     & .5138014E+07,.5012023E+07,.4897696E+07,.4788104E+07,.4690601E+07,&
     & .7716789E+03,.7727576E+03,.7738234E+03,.7748824E+03,.7760097E+03,&
     & .7709970E+03,.7720920E+03,.7731683E+03,.7742329E+03,.7752795E+03,&
     & .7702703E+03,.7713814E+03,.7724638E+03,.7735352E+03,.7745939E+03,&
     & .7695623E+03,.7706840E+03,.7717805E+03,.7728581E+03,.7739261E+03,&
     & .7688269E+03,.7699602E+03,.7710739E+03,.7721618E+03,.7732324E+03,&
     & .7681233E+03,.7692532E+03,.7703760E+03,.7714779E+03,.7725630E+03,&
     & .7674388E+03,.7685763E+03,.7697000E+03,.7708216E+03,.7719092E+03,&
     & .7667706E+03,.7679148E+03,.7690491E+03,.7701722E+03,.7712802E+03,&
     & .7662119E+03,.7673655E+03,.7684992E+03,.7696303E+03,.7707457E+03,&
     & .7662075E+03,.7673571E+03,.7684953E+03,.7696203E+03,.7707387E+03,&
     & .7662135E+03,.7673563E+03,.7684924E+03,.7696200E+03,.7707387E+03,&
     & .7662080E+03,.7673567E+03,.7684935E+03,.7696200E+03,.7707387E+03/

      data   absa(  1: 65, 8)   /                                       &
     & .1263473E+08,.1241774E+08,.1224143E+08,.1219450E+08,.1211569E+08,&
     & .1001941E+04,.1000716E+04,.9984375E+03,.9948240E+03,.9885902E+03,&
     & .1002595E+04,.1001895E+04,.1000220E+04,.9973344E+03,.9934343E+03,&
     & .1002661E+04,.1002592E+04,.1001621E+04,.9995163E+03,.9962347E+03,&
     & .1002279E+04,.1002873E+04,.1002423E+04,.1001059E+04,.9984745E+03,&
     & .1001370E+04,.1002671E+04,.1002871E+04,.1002175E+04,.1000396E+04,&
     & .9999024E+03,.1002012E+04,.1002937E+04,.1002823E+04,.1001683E+04,&
     & .9980806E+03,.1000892E+04,.1002512E+04,.1003003E+04,.1002533E+04,&
     & .9959107E+03,.9994193E+03,.1001722E+04,.1002845E+04,.1002953E+04,&
     & .9937920E+03,.9978415E+03,.1000716E+04,.1002445E+04,.1003004E+04,&
     & .9937640E+03,.9978135E+03,.1000658E+04,.1002444E+04,.1003004E+04,&
     & .9936612E+03,.9978143E+03,.1000698E+04,.1002407E+04,.1003004E+04,&
     & .9937589E+03,.9978125E+03,.1000708E+04,.1002433E+04,.1003006E+04/

!     the array absb(235,8) (kb(5,13:59,8)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 8,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:120, 1)   /                                       &
     & .1603600E+00,.1639900E+00,.1719800E+00,.1845500E+00,.2017300E+00,&
     & .1605000E+00,.1645400E+00,.1729200E+00,.1859200E+00,.2035100E+00,&
     & .1607100E+00,.1651900E+00,.1740000E+00,.1875000E+00,.2055400E+00,&
     & .1609800E+00,.1659300E+00,.1751700E+00,.1892100E+00,.2077200E+00,&
     & .1612900E+00,.1667000E+00,.1764200E+00,.1909300E+00,.2099100E+00,&
     & .1616400E+00,.1675000E+00,.1777100E+00,.1926700E+00,.2121100E+00,&
     & .1620300E+00,.1683200E+00,.1790100E+00,.1944300E+00,.2143100E+00,&
     & .1625500E+00,.1693400E+00,.1805900E+00,.1965300E+00,.2169400E+00,&
     & .1631300E+00,.1704400E+00,.1822500E+00,.1987300E+00,.2196700E+00,&
     & .1641600E+00,.1722700E+00,.1849800E+00,.2022900E+00,.2240600E+00,&
     & .1653900E+00,.1743100E+00,.1879600E+00,.2061200E+00,.2287500E+00,&
     & .1668400E+00,.1766600E+00,.1912500E+00,.2103200E+00,.2338500E+00,&
     & .1685600E+00,.1793900E+00,.1949300E+00,.2149500E+00,.2394300E+00,&
     & .1706700E+00,.1826000E+00,.1991900E+00,.2202400E+00,.2457800E+00,&
     & .1730600E+00,.1861300E+00,.2037700E+00,.2258800E+00,.2524800E+00,&
     & .1757000E+00,.1899400E+00,.2086500E+00,.2318400E+00,.2595200E+00,&
     & .1788200E+00,.1941700E+00,.2139900E+00,.2382800E+00,.2670800E+00,&
     & .1822300E+00,.1987000E+00,.2196400E+00,.2450500E+00,.2749200E+00,&
     & .1860200E+00,.2036400E+00,.2257200E+00,.2522900E+00,.2832500E+00,&
     & .1901800E+00,.2089500E+00,.2321900E+00,.2599400E+00,.2920000E+00,&
     & .1946600E+00,.2146100E+00,.2390200E+00,.2679500E+00,.3011400E+00,&
     & .1988000E+00,.2197600E+00,.2452000E+00,.2750900E+00,.3093200E+00,&
     & .2012900E+00,.2228300E+00,.2488600E+00,.2793100E+00,.3141200E+00,&
     & .2017000E+00,.2233400E+00,.2494600E+00,.2800000E+00,.3149200E+00/

      data   absb(121:235, 1)   /                                       &
     & .1992000E+00,.2202600E+00,.2457900E+00,.2757800E+00,.3101000E+00,&
     & .1966200E+00,.2170600E+00,.2419700E+00,.2713500E+00,.3050400E+00,&
     & .1941400E+00,.2139600E+00,.2382400E+00,.2670400E+00,.3001000E+00,&
     & .1900300E+00,.2087700E+00,.2319700E+00,.2596800E+00,.2917100E+00,&
     & .1859700E+00,.2035700E+00,.2256400E+00,.2521900E+00,.2831400E+00,&
     & .1822000E+00,.1986500E+00,.2195800E+00,.2449800E+00,.2748400E+00,&
     & .1779800E+00,.1930400E+00,.2125700E+00,.2365700E+00,.2651000E+00,&
     & .1739700E+00,.1874600E+00,.2054800E+00,.2279800E+00,.2549600E+00,&
     & .1705300E+00,.1823900E+00,.1989000E+00,.2198900E+00,.2453500E+00,&
     & .1673900E+00,.1775400E+00,.1924500E+00,.2118300E+00,.2356800E+00,&
     & .1645400E+00,.1729200E+00,.1859200E+00,.2035100E+00,.2255600E+00,&
     & .1623900E+00,.1690300E+00,.1801100E+00,.1959000E+00,.2161500E+00,&
     & .1609500E+00,.1658400E+00,.1750400E+00,.1890200E+00,.2074700E+00,&
     & .1602500E+00,.1634800E+00,.1710900E+00,.1832200E+00,.2000000E+00,&
     & .1602100E+00,.1617900E+00,.1678100E+00,.1782100E+00,.1933500E+00,&
     & .1607800E+00,.1606800E+00,.1651100E+00,.1738700E+00,.1873100E+00,&
     & .1619700E+00,.1601800E+00,.1629900E+00,.1701700E+00,.1818600E+00,&
     & .1636200E+00,.1602600E+00,.1615500E+00,.1672900E+00,.1773800E+00,&
     & .1657000E+00,.1608300E+00,.1606400E+00,.1649800E+00,.1736500E+00,&
     & .1682500E+00,.1618700E+00,.1602000E+00,.1631100E+00,.1704100E+00,&
     & .1712900E+00,.1634000E+00,.1602300E+00,.1616900E+00,.1676100E+00,&
     & .1746200E+00,.1652900E+00,.1606900E+00,.1607700E+00,.1653500E+00,&
     & .1760700E+00,.1661700E+00,.1610000E+00,.1605100E+00,.1645600E+00/

      data   absb(  1:120, 2)   /                                       &
     & .1752500E+01,.1772700E+01,.1811900E+01,.1871200E+01,.1949100E+01,&
     & .1753600E+01,.1775500E+01,.1816500E+01,.1877500E+01,.1957100E+01,&
     & .1754900E+01,.1778700E+01,.1821900E+01,.1884700E+01,.1966200E+01,&
     & .1756500E+01,.1781900E+01,.1827800E+01,.1892500E+01,.1976100E+01,&
     & .1758300E+01,.1785000E+01,.1833600E+01,.1900300E+01,.1985900E+01,&
     & .1760300E+01,.1789100E+01,.1839600E+01,.1908200E+01,.1995700E+01,&
     & .1762400E+01,.1793400E+01,.1845700E+01,.1916200E+01,.2005600E+01,&
     & .1765200E+01,.1798600E+01,.1853000E+01,.1925700E+01,.2017400E+01,&
     & .1768300E+01,.1804200E+01,.1860700E+01,.1935600E+01,.2029600E+01,&
     & .1773600E+01,.1813300E+01,.1873200E+01,.1951600E+01,.2049200E+01,&
     & .1779600E+01,.1823400E+01,.1886800E+01,.1968900E+01,.2070200E+01,&
     & .1785700E+01,.1834700E+01,.1901800E+01,.1987700E+01,.2092900E+01,&
     & .1794600E+01,.1847500E+01,.1918500E+01,.2008500E+01,.2117800E+01,&
     & .1805300E+01,.1862300E+01,.1937700E+01,.2032200E+01,.2146000E+01,&
     & .1817200E+01,.1878500E+01,.1958300E+01,.2057400E+01,.2175700E+01,&
     & .1830300E+01,.1895900E+01,.1980200E+01,.2083900E+01,.2206900E+01,&
     & .1844800E+01,.1915000E+01,.2004200E+01,.2112600E+01,.2240400E+01,&
     & .1860600E+01,.1935500E+01,.2029500E+01,.2142800E+01,.2275400E+01,&
     & .1878000E+01,.1957700E+01,.2056600E+01,.2174900E+01,.2312500E+01,&
     & .1896900E+01,.1981600E+01,.2085500E+01,.2208800E+01,.2351400E+01,&
     & .1917200E+01,.2006900E+01,.2116000E+01,.2244300E+01,.2392000E+01,&
     & .1935900E+01,.2030000E+01,.2143400E+01,.2276200E+01,.2428200E+01,&
     & .1947100E+01,.2043700E+01,.2159700E+01,.2294900E+01,.2449500E+01,&
     & .1949000E+01,.2046000E+01,.2162400E+01,.2298000E+01,.2453000E+01/

      data   absb(121:235, 2)   /                                       &
     & .1937700E+01,.2032200E+01,.2146100E+01,.2279200E+01,.2431700E+01,&
     & .1926100E+01,.2017900E+01,.2129100E+01,.2259500E+01,.2409300E+01,&
     & .1914900E+01,.2004000E+01,.2112500E+01,.2240200E+01,.2387300E+01,&
     & .1896300E+01,.1980700E+01,.2084500E+01,.2207600E+01,.2350100E+01,&
     & .1877800E+01,.1957400E+01,.2056300E+01,.2174500E+01,.2312000E+01,&
     & .1860400E+01,.1935200E+01,.2029200E+01,.2142500E+01,.2275000E+01,&
     & .1840900E+01,.1909900E+01,.1997800E+01,.2105000E+01,.2231500E+01,&
     & .1821700E+01,.1884500E+01,.1966000E+01,.2066700E+01,.2186700E+01,&
     & .1804600E+01,.1861300E+01,.1936400E+01,.2030600E+01,.2144100E+01,&
     & .1788600E+01,.1838900E+01,.1907200E+01,.1994500E+01,.2101100E+01,&
     & .1775500E+01,.1816500E+01,.1877500E+01,.1957100E+01,.2055900E+01,&
     & .1764400E+01,.1797000E+01,.1850800E+01,.1922800E+01,.2013900E+01,&
     & .1756300E+01,.1781600E+01,.1827100E+01,.1891600E+01,.1974900E+01,&
     & .1751700E+01,.1770200E+01,.1807400E+01,.1865100E+01,.1941300E+01,&
     & .1750400E+01,.1761100E+01,.1790800E+01,.1842000E+01,.1911300E+01,&
     & .1752300E+01,.1754700E+01,.1778300E+01,.1821200E+01,.1883900E+01,&
     & .1756600E+01,.1751000E+01,.1767500E+01,.1802800E+01,.1858900E+01,&
     & .1762300E+01,.1750700E+01,.1759800E+01,.1788100E+01,.1838100E+01,&
     & .1769100E+01,.1752400E+01,.1754400E+01,.1777700E+01,.1820100E+01,&
     & .1777600E+01,.1756200E+01,.1751200E+01,.1768200E+01,.1804000E+01,&
     & .1788100E+01,.1761600E+01,.1750500E+01,.1760600E+01,.1789700E+01,&
     & .1799600E+01,.1767800E+01,.1752000E+01,.1755200E+01,.1779400E+01,&
     & .1804500E+01,.1770700E+01,.1753000E+01,.1753600E+01,.1775600E+01/

      data   absb(  1:120, 3)   /                                       &
     & .1094500E+02,.1105200E+02,.1122200E+02,.1145200E+02,.1174400E+02,&
     & .1095200E+02,.1106500E+02,.1124000E+02,.1147600E+02,.1177400E+02,&
     & .1096100E+02,.1108000E+02,.1126100E+02,.1150300E+02,.1180700E+02,&
     & .1097100E+02,.1109700E+02,.1128400E+02,.1153300E+02,.1184400E+02,&
     & .1098100E+02,.1111500E+02,.1130700E+02,.1156200E+02,.1187900E+02,&
     & .1099200E+02,.1113200E+02,.1133100E+02,.1159200E+02,.1191500E+02,&
     & .1100300E+02,.1114900E+02,.1135400E+02,.1162200E+02,.1195100E+02,&
     & .1101700E+02,.1117000E+02,.1138200E+02,.1165700E+02,.1199400E+02,&
     & .1103100E+02,.1119200E+02,.1141200E+02,.1169400E+02,.1203800E+02,&
     & .1105600E+02,.1122800E+02,.1146000E+02,.1175400E+02,.1210900E+02,&
     & .1108500E+02,.1126700E+02,.1151100E+02,.1181700E+02,.1218400E+02,&
     & .1111800E+02,.1131200E+02,.1156800E+02,.1188600E+02,.1226600E+02,&
     & .1115400E+02,.1136100E+02,.1163000E+02,.1196200E+02,.1235400E+02,&
     & .1119600E+02,.1141800E+02,.1170200E+02,.1204800E+02,.1245500E+02,&
     & .1124300E+02,.1148000E+02,.1177800E+02,.1213800E+02,.1256000E+02,&
     & .1129400E+02,.1154500E+02,.1185900E+02,.1223400E+02,.1267000E+02,&
     & .1135100E+02,.1161700E+02,.1194600E+02,.1233600E+02,.1278800E+02,&
     & .1141100E+02,.1169400E+02,.1203800E+02,.1244300E+02,.1291000E+02,&
     & .1147800E+02,.1177600E+02,.1213600E+02,.1255700E+02,.1304000E+02,&
     & .1154900E+02,.1186400E+02,.1223900E+02,.1267700E+02,.1317500E+02,&
     & .1162600E+02,.1195600E+02,.1234800E+02,.1280100E+02,.1331600E+02,&
     & .1169500E+02,.1204000E+02,.1244600E+02,.1291300E+02,.1344200E+02,&
     & .1173700E+02,.1208900E+02,.1250300E+02,.1297900E+02,.1351500E+02,&
     & .1174400E+02,.1209800E+02,.1251300E+02,.1298900E+02,.1352800E+02/

      data   absb(121:235, 3)   /                                       &
     & .1170200E+02,.1204800E+02,.1245500E+02,.1292400E+02,.1345400E+02,&
     & .1165900E+02,.1199600E+02,.1239500E+02,.1285500E+02,.1337600E+02,&
     & .1161700E+02,.1194600E+02,.1233600E+02,.1278700E+02,.1330000E+02,&
     & .1154700E+02,.1186100E+02,.1223600E+02,.1267300E+02,.1317100E+02,&
     & .1147700E+02,.1177500E+02,.1213400E+02,.1255500E+02,.1303800E+02,&
     & .1141100E+02,.1169300E+02,.1203700E+02,.1244200E+02,.1290900E+02,&
     & .1133500E+02,.1159800E+02,.1192300E+02,.1230900E+02,.1275700E+02,&
     & .1126100E+02,.1150300E+02,.1180700E+02,.1217200E+02,.1259900E+02,&
     & .1119300E+02,.1141400E+02,.1169700E+02,.1204200E+02,.1244800E+02,&
     & .1112900E+02,.1132800E+02,.1158800E+02,.1191100E+02,.1229500E+02,&
     & .1106500E+02,.1124000E+02,.1147600E+02,.1177400E+02,.1213300E+02,&
     & .1101200E+02,.1116300E+02,.1137400E+02,.1164700E+02,.1198100E+02,&
     & .1096900E+02,.1109500E+02,.1128200E+02,.1153000E+02,.1183900E+02,&
     & .1093800E+02,.1104000E+02,.1120400E+02,.1142900E+02,.1171500E+02,&
     & .1091600E+02,.1099600E+02,.1113800E+02,.1134000E+02,.1160400E+02,&
     & .1090000E+02,.1096000E+02,.1107900E+02,.1125900E+02,.1150000E+02,&
     & .1089400E+02,.1093200E+02,.1102800E+02,.1118600E+02,.1140500E+02,&
     & .1089500E+02,.1091200E+02,.1098900E+02,.1112700E+02,.1132500E+02,&
     & .1090300E+02,.1090000E+02,.1095800E+02,.1107500E+02,.1125500E+02,&
     & .1092100E+02,.1089400E+02,.1093300E+02,.1103100E+02,.1119100E+02,&
     & .1094400E+02,.1089400E+02,.1091400E+02,.1099300E+02,.1113400E+02,&
     & .1097200E+02,.1090100E+02,.1090200E+02,.1096300E+02,.1108400E+02,&
     & .1098400E+02,.1090600E+02,.1089800E+02,.1095200E+02,.1106600E+02/

      data   absb(  1:120, 4)   /                                       &
     & .5156300E+02,.5194600E+02,.5246400E+02,.5311600E+02,.5390200E+02,&
     & .5159300E+02,.5198700E+02,.5251700E+02,.5318200E+02,.5398000E+02,&
     & .5162600E+02,.5203500E+02,.5257900E+02,.5325600E+02,.5406900E+02,&
     & .5166400E+02,.5208700E+02,.5264400E+02,.5333700E+02,.5416300E+02,&
     & .5170100E+02,.5213800E+02,.5271000E+02,.5341600E+02,.5425700E+02,&
     & .5174000E+02,.5219100E+02,.5277600E+02,.5349600E+02,.5435100E+02,&
     & .5177900E+02,.5224400E+02,.5284300E+02,.5357600E+02,.5444400E+02,&
     & .5182700E+02,.5230700E+02,.5292200E+02,.5367100E+02,.5455500E+02,&
     & .5187700E+02,.5237300E+02,.5300400E+02,.5377000E+02,.5466900E+02,&
     & .5195900E+02,.5248100E+02,.5313600E+02,.5392700E+02,.5485100E+02,&
     & .5204900E+02,.5259600E+02,.5327800E+02,.5409400E+02,.5504400E+02,&
     & .5214800E+02,.5272300E+02,.5343100E+02,.5427400E+02,.5525200E+02,&
     & .5225900E+02,.5286200E+02,.5359900E+02,.5447100E+02,.5547700E+02,&
     & .5238700E+02,.5302200E+02,.5379000E+02,.5469300E+02,.5573000E+02,&
     & .5252500E+02,.5319100E+02,.5399200E+02,.5492600E+02,.5599500E+02,&
     & .5267200E+02,.5337100E+02,.5420300E+02,.5517000E+02,.5627100E+02,&
     & .5283300E+02,.5356500E+02,.5443000E+02,.5543000E+02,.5656500E+02,&
     & .5300300E+02,.5376800E+02,.5466800E+02,.5570200E+02,.5687000E+02,&
     & .5318600E+02,.5398600E+02,.5492000E+02,.5598800E+02,.5719000E+02,&
     & .5338100E+02,.5421600E+02,.5518400E+02,.5628700E+02,.5752500E+02,&
     & .5358700E+02,.5445700E+02,.5546000E+02,.5659900E+02,.5787100E+02,&
     & .5377300E+02,.5467300E+02,.5570700E+02,.5687600E+02,.5817900E+02,&
     & .5388300E+02,.5480000E+02,.5585200E+02,.5703900E+02,.5835900E+02,&
     & .5390100E+02,.5482200E+02,.5587600E+02,.5706600E+02,.5838900E+02/

      data   absb(121:235, 4)   /                                       &
     & .5379100E+02,.5469400E+02,.5573100E+02,.5690300E+02,.5820800E+02,&
     & .5367600E+02,.5456000E+02,.5557800E+02,.5673100E+02,.5801800E+02,&
     & .5356300E+02,.5442900E+02,.5542900E+02,.5656300E+02,.5783200E+02,&
     & .5337500E+02,.5420800E+02,.5517500E+02,.5627700E+02,.5751300E+02,&
     & .5318400E+02,.5398300E+02,.5491600E+02,.5598400E+02,.5718600E+02,&
     & .5300100E+02,.5376600E+02,.5466500E+02,.5569900E+02,.5686600E+02,&
     & .5279000E+02,.5351300E+02,.5437000E+02,.5536200E+02,.5648800E+02,&
     & .5257700E+02,.5325400E+02,.5406600E+02,.5501200E+02,.5609300E+02,&
     & .5237900E+02,.5301100E+02,.5377700E+02,.5467800E+02,.5571300E+02,&
     & .5218400E+02,.5276800E+02,.5348600E+02,.5433900E+02,.5532600E+02,&
     & .5198700E+02,.5251700E+02,.5318200E+02,.5398000E+02,.5491300E+02,&
     & .5181200E+02,.5228800E+02,.5289800E+02,.5364300E+02,.5452200E+02,&
     & .5165900E+02,.5208100E+02,.5263700E+02,.5332700E+02,.5415200E+02,&
     & .5153500E+02,.5190600E+02,.5241200E+02,.5305200E+02,.5382600E+02,&
     & .5143300E+02,.5175500E+02,.5221100E+02,.5280200E+02,.5352700E+02,&
     & .5134900E+02,.5162200E+02,.5202900E+02,.5257100E+02,.5324700E+02,&
     & .5128500E+02,.5150700E+02,.5186500E+02,.5235800E+02,.5298500E+02,&
     & .5124000E+02,.5141700E+02,.5173000E+02,.5217800E+02,.5276000E+02,&
     & .5121100E+02,.5134600E+02,.5161500E+02,.5202000E+02,.5255900E+02,&
     & .5119400E+02,.5128800E+02,.5151400E+02,.5187500E+02,.5237100E+02,&
     & .5119000E+02,.5124500E+02,.5142600E+02,.5174600E+02,.5219800E+02,&
     & .5120000E+02,.5121600E+02,.5135700E+02,.5163500E+02,.5204600E+02,&
     & .5120700E+02,.5120700E+02,.5133300E+02,.5159400E+02,.5198900E+02/

      data   absb(  1:120, 5)   /                                       &
     & .1705100E+03,.1716800E+03,.1729700E+03,.1743700E+03,.1759000E+03,&
     & .1706100E+03,.1717900E+03,.1730900E+03,.1745100E+03,.1760400E+03,&
     & .1707300E+03,.1719200E+03,.1732300E+03,.1746600E+03,.1762100E+03,&
     & .1708500E+03,.1720500E+03,.1733800E+03,.1748200E+03,.1763800E+03,&
     & .1709700E+03,.1721800E+03,.1735200E+03,.1749700E+03,.1765500E+03,&
     & .1710900E+03,.1723200E+03,.1736600E+03,.1751300E+03,.1767100E+03,&
     & .1712100E+03,.1724500E+03,.1738100E+03,.1752800E+03,.1768800E+03,&
     & .1713400E+03,.1726000E+03,.1739700E+03,.1754700E+03,.1770800E+03,&
     & .1714900E+03,.1727600E+03,.1741500E+03,.1756500E+03,.1772800E+03,&
     & .1717200E+03,.1730100E+03,.1744200E+03,.1759400E+03,.1775900E+03,&
     & .1719500E+03,.1732700E+03,.1747000E+03,.1762500E+03,.1779200E+03,&
     & .1722100E+03,.1735500E+03,.1750000E+03,.1765800E+03,.1782700E+03,&
     & .1724800E+03,.1738500E+03,.1753300E+03,.1769300E+03,.1786500E+03,&
     & .1727900E+03,.1741800E+03,.1756900E+03,.1773200E+03,.1790600E+03,&
     & .1731100E+03,.1745300E+03,.1760600E+03,.1777200E+03,.1794900E+03,&
     & .1734400E+03,.1748800E+03,.1764500E+03,.1781300E+03,.1799400E+03,&
     & .1737900E+03,.1752600E+03,.1768600E+03,.1785700E+03,.1804000E+03,&
     & .1741400E+03,.1756500E+03,.1772700E+03,.1790200E+03,.1808800E+03,&
     & .1745200E+03,.1760500E+03,.1777100E+03,.1794800E+03,.1813700E+03,&
     & .1749100E+03,.1764700E+03,.1781600E+03,.1799600E+03,.1818800E+03,&
     & .1753100E+03,.1769000E+03,.1786200E+03,.1804500E+03,.1824100E+03,&
     & .1756600E+03,.1772800E+03,.1790300E+03,.1808900E+03,.1828700E+03,&
     & .1758600E+03,.1775000E+03,.1792600E+03,.1811400E+03,.1831300E+03,&
     & .1759000E+03,.1775400E+03,.1793000E+03,.1811800E+03,.1831800E+03/

      data   absb(121:235, 5)   /                                       &
     & .1756900E+03,.1773200E+03,.1790600E+03,.1809300E+03,.1829100E+03,&
     & .1754700E+03,.1770800E+03,.1788100E+03,.1806600E+03,.1826300E+03,&
     & .1752600E+03,.1768500E+03,.1785700E+03,.1804000E+03,.1823500E+03,&
     & .1748900E+03,.1764600E+03,.1781400E+03,.1799500E+03,.1818700E+03,&
     & .1745100E+03,.1760500E+03,.1777000E+03,.1794800E+03,.1813700E+03,&
     & .1741400E+03,.1756500E+03,.1772700E+03,.1790100E+03,.1808700E+03,&
     & .1736900E+03,.1751600E+03,.1767500E+03,.1784600E+03,.1802800E+03,&
     & .1732300E+03,.1746500E+03,.1762000E+03,.1778700E+03,.1796500E+03,&
     & .1727700E+03,.1741600E+03,.1756700E+03,.1772900E+03,.1790400E+03,&
     & .1723000E+03,.1736500E+03,.1751100E+03,.1766900E+03,.1784000E+03,&
     & .1717900E+03,.1730900E+03,.1745100E+03,.1760400E+03,.1777000E+03,&
     & .1713000E+03,.1725500E+03,.1739200E+03,.1754100E+03,.1770200E+03,&
     & .1708300E+03,.1720400E+03,.1733600E+03,.1748000E+03,.1763600E+03,&
     & .1704100E+03,.1715700E+03,.1728500E+03,.1742400E+03,.1757600E+03,&
     & .1700200E+03,.1711300E+03,.1723700E+03,.1737200E+03,.1751900E+03,&
     & .1696400E+03,.1707100E+03,.1719000E+03,.1732100E+03,.1746400E+03,&
     & .1692700E+03,.1703100E+03,.1714500E+03,.1727200E+03,.1741100E+03,&
     & .1689600E+03,.1699500E+03,.1710600E+03,.1722800E+03,.1736300E+03,&
     & .1686600E+03,.1696200E+03,.1706900E+03,.1718800E+03,.1731800E+03,&
     & .1683900E+03,.1693000E+03,.1703300E+03,.1714800E+03,.1727500E+03,&
     & .1681200E+03,.1689900E+03,.1699900E+03,.1711000E+03,.1723300E+03,&
     & .1678800E+03,.1687100E+03,.1696800E+03,.1707500E+03,.1719500E+03,&
     & .1677900E+03,.1686100E+03,.1695500E+03,.1706200E+03,.1718000E+03/

      data   absb(  1:120, 6)   /                                       &
     & .3812200E+03,.3840600E+03,.3866800E+03,.3890800E+03,.3912500E+03,&
     & .3814800E+03,.3843100E+03,.3869100E+03,.3892900E+03,.3914400E+03,&
     & .3817800E+03,.3845800E+03,.3871600E+03,.3895100E+03,.3916400E+03,&
     & .3820900E+03,.3848700E+03,.3874200E+03,.3897500E+03,.3918600E+03,&
     & .3823900E+03,.3851400E+03,.3876700E+03,.3899800E+03,.3920600E+03,&
     & .3826800E+03,.3854100E+03,.3879200E+03,.3902000E+03,.3922600E+03,&
     & .3829700E+03,.3856800E+03,.3881600E+03,.3904200E+03,.3924600E+03,&
     & .3833000E+03,.3859800E+03,.3884400E+03,.3906700E+03,.3926800E+03,&
     & .3836300E+03,.3862900E+03,.3887200E+03,.3909200E+03,.3929100E+03,&
     & .3841400E+03,.3867500E+03,.3891400E+03,.3913100E+03,.3932500E+03,&
     & .3846600E+03,.3872300E+03,.3895800E+03,.3917000E+03,.3936000E+03,&
     & .3852000E+03,.3877200E+03,.3900200E+03,.3921000E+03,.3939500E+03,&
     & .3857500E+03,.3882300E+03,.3904800E+03,.3925100E+03,.3943200E+03,&
     & .3863500E+03,.3887700E+03,.3909800E+03,.3929500E+03,.3947000E+03,&
     & .3869400E+03,.3893200E+03,.3914600E+03,.3933900E+03,.3950900E+03,&
     & .3875300E+03,.3898500E+03,.3919400E+03,.3938100E+03,.3954600E+03,&
     & .3881300E+03,.3903900E+03,.3924300E+03,.3942400E+03,.3958300E+03,&
     & .3887100E+03,.3909200E+03,.3929000E+03,.3946600E+03,.3962000E+03,&
     & .3893000E+03,.3914500E+03,.3933800E+03,.3950800E+03,.3965500E+03,&
     & .3898800E+03,.3919700E+03,.3938400E+03,.3954800E+03,.3969000E+03,&
     & .3904500E+03,.3924800E+03,.3942900E+03,.3958700E+03,.3972300E+03,&
     & .3909300E+03,.3929100E+03,.3946700E+03,.3962000E+03,.3975100E+03,&
     & .3912000E+03,.3931600E+03,.3948800E+03,.3963900E+03,.3976600E+03,&
     & .3912500E+03,.3932000E+03,.3949200E+03,.3964200E+03,.3976900E+03/

      data   absb(121:235, 6)   /                                       &
     & .3909800E+03,.3929500E+03,.3947100E+03,.3962300E+03,.3975400E+03,&
     & .3906800E+03,.3926900E+03,.3944700E+03,.3960300E+03,.3973700E+03,&
     & .3903900E+03,.3924300E+03,.3942400E+03,.3958300E+03,.3972000E+03,&
     & .3898600E+03,.3919600E+03,.3938200E+03,.3954700E+03,.3968900E+03,&
     & .3892900E+03,.3914400E+03,.3933700E+03,.3950700E+03,.3965500E+03,&
     & .3887100E+03,.3909200E+03,.3929000E+03,.3946600E+03,.3961900E+03,&
     & .3879700E+03,.3902500E+03,.3923000E+03,.3941300E+03,.3957400E+03,&
     & .3871500E+03,.3895100E+03,.3916400E+03,.3935400E+03,.3952200E+03,&
     & .3863100E+03,.3887400E+03,.3909400E+03,.3929200E+03,.3946800E+03,&
     & .3853800E+03,.3878900E+03,.3901800E+03,.3922400E+03,.3940700E+03,&
     & .3843100E+03,.3869100E+03,.3892900E+03,.3914400E+03,.3933600E+03,&
     & .3832000E+03,.3858900E+03,.3883600E+03,.3906000E+03,.3926200E+03,&
     & .3820600E+03,.3848400E+03,.3873900E+03,.3897200E+03,.3918300E+03,&
     & .3809500E+03,.3838200E+03,.3864600E+03,.3888700E+03,.3910600E+03,&
     & .3798500E+03,.3827900E+03,.3855200E+03,.3880200E+03,.3902900E+03,&
     & .3787300E+03,.3817500E+03,.3845500E+03,.3871300E+03,.3894900E+03,&
     & .3776200E+03,.3806700E+03,.3835500E+03,.3862100E+03,.3886500E+03,&
     & .3766100E+03,.3796500E+03,.3826100E+03,.3853500E+03,.3878600E+03,&
     & .3755900E+03,.3786700E+03,.3816900E+03,.3845000E+03,.3870800E+03,&
     & .3745900E+03,.3776800E+03,.3807400E+03,.3836200E+03,.3862800E+03,&
     & .3735300E+03,.3767300E+03,.3797700E+03,.3827200E+03,.3854500E+03,&
     & .3725100E+03,.3757700E+03,.3788500E+03,.3818500E+03,.3846500E+03,&
     & .3721200E+03,.3753900E+03,.3784700E+03,.3815000E+03,.3843200E+03/

      data   absb(  1:120, 7)   /                                       &
     & .7662137E+03,.7673567E+03,.7684935E+03,.7696200E+03,.7707387E+03,&
     & .7663163E+03,.7674596E+03,.7685948E+03,.7697242E+03,.7708374E+03,&
     & .7664298E+03,.7675754E+03,.7687154E+03,.7698361E+03,.7709498E+03,&
     & .7665525E+03,.7676972E+03,.7688307E+03,.7699581E+03,.7710677E+03,&
     & .7666756E+03,.7678182E+03,.7689507E+03,.7700728E+03,.7711794E+03,&
     & .7667902E+03,.7679332E+03,.7690672E+03,.7701907E+03,.7712951E+03,&
     & .7669059E+03,.7680487E+03,.7691780E+03,.7703003E+03,.7714030E+03,&
     & .7670396E+03,.7681833E+03,.7693099E+03,.7704320E+03,.7715291E+03,&
     & .7671791E+03,.7683177E+03,.7694439E+03,.7705613E+03,.7716607E+03,&
     & .7673923E+03,.7685297E+03,.7696521E+03,.7707717E+03,.7718639E+03,&
     & .7676078E+03,.7687458E+03,.7698715E+03,.7709807E+03,.7720702E+03,&
     & .7678360E+03,.7689728E+03,.7700933E+03,.7712034E+03,.7722895E+03,&
     & .7680791E+03,.7692104E+03,.7703330E+03,.7714362E+03,.7725191E+03,&
     & .7683474E+03,.7694736E+03,.7705880E+03,.7716873E+03,.7727677E+03,&
     & .7686113E+03,.7697379E+03,.7708539E+03,.7719439E+03,.7730324E+03,&
     & .7688808E+03,.7700069E+03,.7711126E+03,.7722067E+03,.7732788E+03,&
     & .7691627E+03,.7702826E+03,.7713890E+03,.7724701E+03,.7735411E+03,&
     & .7694444E+03,.7705613E+03,.7716614E+03,.7727369E+03,.7738092E+03,&
     & .7697324E+03,.7708439E+03,.7719374E+03,.7730147E+03,.7740850E+03,&
     & .7700239E+03,.7711302E+03,.7722186E+03,.7732904E+03,.7743608E+03,&
     & .7703148E+03,.7714174E+03,.7725031E+03,.7735736E+03,.7746415E+03,&
     & .7705710E+03,.7716646E+03,.7727437E+03,.7738133E+03,.7748784E+03,&
     & .7707135E+03,.7718066E+03,.7728840E+03,.7739535E+03,.7750133E+03,&
     & .7707385E+03,.7718282E+03,.7729073E+03,.7739754E+03,.7750350E+03/

      data   absb(121:235, 7)   /                                       &
     & .7705883E+03,.7716906E+03,.7727677E+03,.7738341E+03,.7749005E+03,&
     & .7704377E+03,.7715367E+03,.7726181E+03,.7736893E+03,.7747501E+03,&
     & .7702833E+03,.7713847E+03,.7724701E+03,.7735404E+03,.7746103E+03,&
     & .7700132E+03,.7711226E+03,.7722110E+03,.7732832E+03,.7743549E+03,&
     & .7697309E+03,.7708439E+03,.7719342E+03,.7730115E+03,.7740748E+03,&
     & .7694415E+03,.7705563E+03,.7716549E+03,.7727369E+03,.7738016E+03,&
     & .7690928E+03,.7702101E+03,.7713174E+03,.7724017E+03,.7734724E+03,&
     & .7687110E+03,.7698351E+03,.7709476E+03,.7720367E+03,.7731196E+03,&
     & .7683270E+03,.7694544E+03,.7705715E+03,.7716689E+03,.7727485E+03,&
     & .7679197E+03,.7690513E+03,.7701707E+03,.7712793E+03,.7723644E+03,&
     & .7674632E+03,.7685951E+03,.7697242E+03,.7708374E+03,.7719299E+03,&
     & .7669980E+03,.7681419E+03,.7692751E+03,.7703917E+03,.7714912E+03,&
     & .7665390E+03,.7676847E+03,.7688180E+03,.7699424E+03,.7710534E+03,&
     & .7661024E+03,.7672540E+03,.7683924E+03,.7695234E+03,.7706427E+03,&
     & .7656917E+03,.7668356E+03,.7679808E+03,.7691095E+03,.7702308E+03,&
     & .7652690E+03,.7664166E+03,.7675626E+03,.7687003E+03,.7698229E+03,&
     & .7648234E+03,.7659942E+03,.7671451E+03,.7682868E+03,.7694142E+03,&
     & .7643942E+03,.7656164E+03,.7667615E+03,.7679051E+03,.7690364E+03,&
     & .7640026E+03,.7652413E+03,.7663923E+03,.7675375E+03,.7686735E+03,&
     & .7636170E+03,.7648611E+03,.7660227E+03,.7671711E+03,.7683104E+03,&
     & .7632407E+03,.7644461E+03,.7656618E+03,.7668033E+03,.7679489E+03,&
     & .7628680E+03,.7640652E+03,.7653090E+03,.7664582E+03,.7676042E+03,&
     & .7627137E+03,.7639260E+03,.7651678E+03,.7663198E+03,.7674646E+03/

      data   absb(  1:120, 8)   /                                       &
     & .9936589E+03,.9978124E+03,.1000713E+04,.1002438E+04,.1003006E+04,&
     & .9941671E+03,.9981368E+03,.1000893E+04,.1002525E+04,.1002981E+04,&
     & .9946170E+03,.9984729E+03,.1001113E+04,.1002623E+04,.1003004E+04,&
     & .9950818E+03,.9987774E+03,.1001345E+04,.1002694E+04,.1002984E+04,&
     & .9955234E+03,.9990646E+03,.1001517E+04,.1002779E+04,.1002969E+04,&
     & .9959436E+03,.9994208E+03,.1001709E+04,.1002856E+04,.1002912E+04,&
     & .9963433E+03,.9996951E+03,.1001890E+04,.1002878E+04,.1002898E+04,&
     & .9968098E+03,.1000016E+04,.1002055E+04,.1002942E+04,.1002824E+04,&
     & .9972440E+03,.1000327E+04,.1002227E+04,.1002982E+04,.1002721E+04,&
     & .9979060E+03,.1000761E+04,.1002461E+04,.1002998E+04,.1002590E+04,&
     & .9985543E+03,.1001122E+04,.1002633E+04,.1002989E+04,.1002393E+04,&
     & .9991623E+03,.1001557E+04,.1002797E+04,.1002959E+04,.1002134E+04,&
     & .9997568E+03,.1001925E+04,.1002931E+04,.1002871E+04,.1001821E+04,&
     & .1000377E+04,.1002254E+04,.1002986E+04,.1002709E+04,.1001396E+04,&
     & .1000957E+04,.1002526E+04,.1002996E+04,.1002513E+04,.1000729E+04,&
     & .1001400E+04,.1002743E+04,.1002971E+04,.1002233E+04,.1000328E+04,&
     & .1001865E+04,.1002878E+04,.1002898E+04,.1001880E+04,.9997574E+03,&
     & .1002181E+04,.1002979E+04,.1002724E+04,.1001454E+04,.9990322E+03,&
     & .1002525E+04,.1003008E+04,.1002510E+04,.1000958E+04,.9981303E+03,&
     & .1002747E+04,.1002980E+04,.1002223E+04,.1000349E+04,.9972023E+03,&
     & .1002906E+04,.1002878E+04,.1001840E+04,.9996791E+03,.9962163E+03,&
     & .1002974E+04,.1002724E+04,.1001436E+04,.9990028E+03,.9953155E+03,&
     & .1003027E+04,.1002635E+04,.1001189E+04,.9985939E+03,.9947732E+03,&
     & .1003027E+04,.1002599E+04,.1001143E+04,.9985204E+03,.9946814E+03/

      data   absb(121:235, 8)   /                                       &
     & .1002986E+04,.1002724E+04,.1001390E+04,.9989482E+03,.9952234E+03,&
     & .1002962E+04,.1002812E+04,.1001649E+04,.9993614E+03,.9957896E+03,&
     & .1002875E+04,.1002898E+04,.1001877E+04,.9997601E+03,.9963253E+03,&
     & .1002746E+04,.1002968E+04,.1002233E+04,.1000370E+04,.9972315E+03,&
     & .1002537E+04,.1002981E+04,.1002514E+04,.1000975E+04,.9981846E+03,&
     & .1002227E+04,.1002980E+04,.1002742E+04,.1001469E+04,.9990389E+03,&
     & .1001759E+04,.1002861E+04,.1002915E+04,.1001995E+04,.9999268E+03,&
     & .1001110E+04,.1002626E+04,.1002989E+04,.1002426E+04,.1000671E+04,&
     & .1000338E+04,.1002249E+04,.1002983E+04,.1002727E+04,.1001434E+04,&
     & .9993803E+03,.1001695E+04,.1002852E+04,.1002935E+04,.1002009E+04,&
     & .9981347E+03,.1000918E+04,.1002522E+04,.1002979E+04,.1002520E+04,&
     & .9966771E+03,.9999354E+03,.1002008E+04,.1002956E+04,.1002842E+04,&
     & .9950417E+03,.9987633E+03,.1001326E+04,.1002702E+04,.1002988E+04,&
     & .9933422E+03,.9974970E+03,.1000473E+04,.1002331E+04,.1002942E+04,&
     & .9914154E+03,.9961148E+03,.9995247E+03,.1001775E+04,.1002873E+04,&
     & .9892683E+03,.9945791E+03,.9984181E+03,.1001097E+04,.1002599E+04,&
     & .9870330E+03,.9928966E+03,.9971671E+03,.1000265E+04,.1002220E+04,&
     & .9848582E+03,.9910375E+03,.9958675E+03,.9993760E+03,.1001692E+04,&
     & .9825219E+03,.9891516E+03,.9944982E+03,.9983573E+03,.1001066E+04,&
     & .9797028E+03,.9871901E+03,.9930195E+03,.9972541E+03,.1000326E+04,&
     & .9768957E+03,.9851199E+03,.9912663E+03,.9960243E+03,.9994761E+03,&
     & .9741514E+03,.9830479E+03,.9894876E+03,.9947451E+03,.9985504E+03,&
     & .9730254E+03,.9819457E+03,.9887533E+03,.9941968E+03,.9981408E+03/

!........................................!
      end module module_radsw_kgb27      !
!========================================!


!========================================!
      module module_radsw_kgb28          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 28: 38000-50000 cm-1 (low - o3,o2; high - o3,o2)           !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG28

!
      implicit none
!
      private
!
      integer,  public :: MSA28, MSB28
      parameter (MSA28=585, MSB28=1175)


      real (kind=kind_phys), public ::                                  &
     &       absa(MSA28,NG28), absb(MSB28,NG28)

!  ---  rayleigh extinction coefficient at v =      cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 2.02e-05

!     the array absa(585,NG28) (ka((9,5,13,NG28)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 6, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1:180, 1)   /                                       &
     & .4644700E+02,.1800100E+03,.3133800E+03,.4465200E+03,.5463600E+03,&
     & .5417300E+03,.4570700E+03,.2736300E+03,.3500000E+02,.4527000E+02,&
     & .1788600E+03,.3122500E+03,.4454100E+03,.5417700E+03,.5343600E+03,&
     & .4493200E+03,.2674800E+03,.3500000E+02,.4421400E+02,.1777600E+03,&
     & .3111500E+03,.4442700E+03,.5373300E+03,.5275600E+03,.4422400E+03,&
     & .2619500E+03,.3500000E+02,.4325900E+02,.1767300E+03,.3100500E+03,&
     & .4430500E+03,.5329800E+03,.5212900E+03,.4357400E+03,.2569600E+03,&
     & .3500000E+02,.4239300E+02,.1757500E+03,.3089600E+03,.4417300E+03,&
     & .5288300E+03,.5154800E+03,.4297600E+03,.2524200E+03,.3500000E+02,&
     & .4290800E+02,.1769500E+03,.3107700E+03,.4443400E+03,.5316100E+03,&
     & .5189500E+03,.4333200E+03,.2551200E+03,.3500000E+02,.4189900E+02,&
     & .1759600E+03,.3098000E+03,.4433600E+03,.5268600E+03,.5121300E+03,&
     & .4263200E+03,.2498400E+03,.3500000E+02,.4099500E+02,.1750200E+03,&
     & .3088400E+03,.4423200E+03,.5223000E+03,.5059200E+03,.4199500E+03,&
     & .2451100E+03,.3500000E+02,.4018100E+02,.1741200E+03,.3078800E+03,&
     & .4411900E+03,.5178500E+03,.5002500E+03,.4141400E+03,.2408500E+03,&
     & .3500000E+02,.3944300E+02,.1732600E+03,.3069100E+03,.4399200E+03,&
     & .5136400E+03,.4950300E+03,.4088200E+03,.2369900E+03,.3500000E+02,&
     & .4012600E+02,.1745700E+03,.3087600E+03,.4424800E+03,.5175500E+03,&
     & .4998600E+03,.4137500E+03,.2405600E+03,.3500000E+02,.3924000E+02,&
     & .1737000E+03,.3079200E+03,.4414600E+03,.5125400E+03,.4935800E+03,&
     & .4073500E+03,.2359300E+03,.3500000E+02,.3845000E+02,.1728800E+03,&
     & .3070800E+03,.4403100E+03,.5079300E+03,.4878600E+03,.4015500E+03,&
     & .2317900E+03,.3500000E+02,.3774000E+02,.1721000E+03,.3062400E+03,&
     & .4390800E+03,.5036800E+03,.4826400E+03,.3962900E+03,.2280800E+03,&
     & .3500000E+02,.3709900E+02,.1713400E+03,.3053900E+03,.4378100E+03,&
     & .4997500E+03,.4778700E+03,.3914900E+03,.2247300E+03,.3500000E+02,&
     & .3781800E+02,.1725800E+03,.3070800E+03,.4396700E+03,.5041200E+03,&
     & .4832100E+03,.3968700E+03,.2284800E+03,.3500000E+02,.3703400E+02,&
     & .1718300E+03,.3063600E+03,.4383300E+03,.4993500E+03,.4773800E+03,&
     & .3910000E+03,.2243900E+03,.3500000E+02,.3633800E+02,.1711100E+03,&
     & .3056400E+03,.4370000E+03,.4949600E+03,.4721200E+03,.3857200E+03,&
     & .2207500E+03,.3500000E+02,.3571600E+02,.1704300E+03,.3049100E+03,&
     & .4356900E+03,.4909400E+03,.4673200E+03,.3809400E+03,.2174900E+03,&
     & .3500000E+02,.3515600E+02,.1697700E+03,.3041600E+03,.4343700E+03,&
     & .4872600E+03,.4629400E+03,.3766000E+03,.2145600E+03,.3500000E+02/

      data   absa(181:315, 1)   /                                       &
     & .3585700E+02,.1708900E+03,.3056300E+03,.4361900E+03,.4918600E+03,&
     & .4684100E+03,.3820200E+03,.2182200E+03,.3500000E+02,.3516300E+02,&
     & .1702300E+03,.3050200E+03,.4347800E+03,.4873100E+03,.4630000E+03,&
     & .3766600E+03,.2146000E+03,.3500000E+02,.3455000E+02,.1696100E+03,&
     & .3044100E+03,.4333200E+03,.4832100E+03,.4581300E+03,.3718600E+03,&
     & .2113900E+03,.3500000E+02,.3400400E+02,.1690200E+03,.3037800E+03,&
     & .4319400E+03,.4795000E+03,.4537500E+03,.3675500E+03,.2085300E+03,&
     & .3500000E+02,.3351400E+02,.1684400E+03,.3031200E+03,.4305800E+03,&
     & .4761100E+03,.4497700E+03,.3636500E+03,.2059700E+03,.3500000E+02,&
     & .3421200E+02,.1694600E+03,.3044000E+03,.4324700E+03,.4809200E+03,&
     & .4554200E+03,.3692000E+03,.2096200E+03,.3500000E+02,.3359500E+02,&
     & .1688900E+03,.3038900E+03,.4309300E+03,.4766700E+03,.4504300E+03,&
     & .3643000E+03,.2063900E+03,.3500000E+02,.3305200E+02,.1683500E+03,&
     & .3033700E+03,.4294600E+03,.4728700E+03,.4459900E+03,.3599300E+03,&
     & .2035500E+03,.3500000E+02,.3257100E+02,.1678400E+03,.3028300E+03,&
     & .4280700E+03,.4694700E+03,.4420100E+03,.3560200E+03,.2010300E+03,&
     & .3500000E+02,.3214100E+02,.1673300E+03,.3022700E+03,.4267400E+03,&
     & .4664000E+03,.4384200E+03,.3525000E+03,.1987900E+03,.3500000E+02,&
     & .3281200E+02,.1682300E+03,.3033400E+03,.4285900E+03,.4711800E+03,&
     & .4440100E+03,.3579900E+03,.2023000E+03,.3500000E+02,.3226300E+02,&
     & .1677400E+03,.3029100E+03,.4270800E+03,.4672700E+03,.4394400E+03,&
     & .3535000E+03,.1994200E+03,.3500000E+02,.3178200E+02,.1672800E+03,&
     & .3024800E+03,.4255800E+03,.4638100E+03,.4353900E+03,.3495400E+03,&
     & .1969100E+03,.3500000E+02,.3135700E+02,.1668300E+03,.3020300E+03,&
     & .4241300E+03,.4606900E+03,.4317700E+03,.3459900E+03,.1946800E+03,&
     & .3500000E+02,.3098000E+02,.1664000E+03,.3015400E+03,.4227600E+03,&
     & .4578700E+03,.4285200E+03,.3428000E+03,.1927100E+03,.3500000E+02/

      data   absa(316:450, 1)   /                                       &
     & .3161400E+02,.1671600E+03,.3023900E+03,.4247300E+03,.4625900E+03,&
     & .4339700E+03,.3481400E+03,.1960300E+03,.3500000E+02,.3112500E+02,&
     & .1667400E+03,.3020600E+03,.4231100E+03,.4589600E+03,.4297800E+03,&
     & .3440300E+03,.1934700E+03,.3500000E+02,.3069900E+02,.1663500E+03,&
     & .3017000E+03,.4216300E+03,.4557300E+03,.4260800E+03,.3403900E+03,&
     & .1912400E+03,.3500000E+02,.3032500E+02,.1659700E+03,.3013300E+03,&
     & .4202700E+03,.4528600E+03,.4228100E+03,.3371600E+03,.1892800E+03,&
     & .3500000E+02,.2999300E+02,.1655900E+03,.3009200E+03,.4190100E+03,&
     & .4503000E+03,.4198800E+03,.3342700E+03,.1875500E+03,.3500000E+02,&
     & .3059400E+02,.1662400E+03,.3015600E+03,.4208700E+03,.4549300E+03,&
     & .4251700E+03,.3394900E+03,.1906900E+03,.3500000E+02,.3015800E+02,&
     & .1658900E+03,.3013100E+03,.4193700E+03,.4515700E+03,.4213400E+03,&
     & .3357000E+03,.1884100E+03,.3500000E+02,.2978000E+02,.1655600E+03,&
     & .3010300E+03,.4180000E+03,.4486400E+03,.4179900E+03,.3323900E+03,&
     & .1864300E+03,.3500000E+02,.2945000E+02,.1652300E+03,.3007200E+03,&
     & .4167400E+03,.4460600E+03,.4150400E+03,.3294600E+03,.1847100E+03,&
     & .3500000E+02,.2915900E+02,.1649100E+03,.3003900E+03,.4155700E+03,&
     & .4437700E+03,.4124200E+03,.3268600E+03,.1831800E+03,.3500000E+02,&
     & .2969100E+02,.1654100E+03,.3008100E+03,.4172200E+03,.4479500E+03,&
     & .4172000E+03,.3316000E+03,.1859700E+03,.3500000E+02,.2930700E+02,&
     & .1651200E+03,.3006300E+03,.4158100E+03,.4449300E+03,.4137500E+03,&
     & .3281800E+03,.1839600E+03,.3500000E+02,.2897500E+02,.1648500E+03,&
     & .3004200E+03,.4145400E+03,.4423100E+03,.4107600E+03,.3252000E+03,&
     & .1822200E+03,.3500000E+02,.2868700E+02,.1645800E+03,.3001800E+03,&
     & .4134000E+03,.4400200E+03,.4081400E+03,.3225900E+03,.1807100E+03,&
     & .3500000E+02,.2843400E+02,.1643100E+03,.2998900E+03,.4123500E+03,&
     & .4379900E+03,.4058300E+03,.3202800E+03,.1793900E+03,.3500000E+02/

      data   absa(451:585, 1)   /                                       &
     & .2876000E+02,.1645900E+03,.3001000E+03,.4132100E+03,.4406000E+03,&
     & .4088000E+03,.3232500E+03,.1810900E+03,.3500000E+02,.2844400E+02,&
     & .1643600E+03,.2999700E+03,.4120000E+03,.4380700E+03,.4059200E+03,&
     & .3203800E+03,.1794400E+03,.3500000E+02,.2817300E+02,.1641400E+03,&
     & .2998100E+03,.4109200E+03,.4358700E+03,.4034200E+03,.3178800E+03,&
     & .1780200E+03,.3500000E+02,.2793700E+02,.1639200E+03,.2996000E+03,&
     & .4099400E+03,.4339500E+03,.4012400E+03,.3157000E+03,.1767900E+03,&
     & .3500000E+02,.2773000E+02,.1636900E+03,.2993600E+03,.4090500E+03,&
     & .4322400E+03,.3993100E+03,.3137700E+03,.1757100E+03,.3500000E+02,&
     & .2799500E+02,.1639100E+03,.2995100E+03,.4097000E+03,.4344300E+03,&
     & .4017800E+03,.3162400E+03,.1770900E+03,.3500000E+02,.2773700E+02,&
     & .1637400E+03,.2994300E+03,.4086900E+03,.4323000E+03,.3993800E+03,&
     & .3138400E+03,.1757400E+03,.3500000E+02,.2751500E+02,.1635600E+03,&
     & .2993000E+03,.4077900E+03,.4304600E+03,.3973000E+03,.3117600E+03,&
     & .1745800E+03,.3500000E+02,.2732200E+02,.1633800E+03,.2991300E+03,&
     & .4069700E+03,.4288300E+03,.3954800E+03,.3099400E+03,.1735700E+03,&
     & .3500000E+02,.2715200E+02,.1631800E+03,.2989200E+03,.4062200E+03,&
     & .4274000E+03,.3938700E+03,.3083400E+03,.1726800E+03,.3500000E+02,&
     & .2736900E+02,.1633600E+03,.2990300E+03,.4067100E+03,.4292400E+03,&
     & .3959300E+03,.3103900E+03,.1738200E+03,.3500000E+02,.2715800E+02,&
     & .1632300E+03,.2989800E+03,.4058700E+03,.4274500E+03,.3939300E+03,&
     & .3084000E+03,.1727100E+03,.3500000E+02,.2697600E+02,.1630900E+03,&
     & .2988900E+03,.4051300E+03,.4259000E+03,.3922100E+03,.3066700E+03,&
     & .1717600E+03,.3500000E+02,.2681800E+02,.1629300E+03,.2987500E+03,&
     & .4044500E+03,.4245500E+03,.3907000E+03,.3051600E+03,.1709300E+03,&
     & .3500000E+02,.2667900E+02,.1627600E+03,.2985500E+03,.4038100E+03,&
     & .4233500E+03,.3893700E+03,.3038400E+03,.1702100E+03,.3500000E+02/

      data   absa(  1:180, 2)   /                                       &
     & .1727100E+03,.2764600E+03,.3803900E+03,.4842000E+03,.5693200E+03,&
     & .5753400E+03,.4787300E+03,.2814700E+03,.7184000E+02,.1683400E+03,&
     & .2724600E+03,.3767800E+03,.4809100E+03,.5647100E+03,.5673400E+03,&
     & .4700500E+03,.2759300E+03,.7184000E+02,.1644100E+03,.2688300E+03,&
     & .3734100E+03,.4777700E+03,.5603700E+03,.5600300E+03,.4621000E+03,&
     & .2709600E+03,.7184000E+02,.1608600E+03,.2654800E+03,.3702500E+03,&
     & .4747900E+03,.5562900E+03,.5532700E+03,.4548100E+03,.2664700E+03,&
     & .7184000E+02,.1576400E+03,.2623700E+03,.3672500E+03,.4719500E+03,&
     & .5523500E+03,.5470100E+03,.4481000E+03,.2623900E+03,.7184000E+02,&
     & .1595500E+03,.2650000E+03,.3706700E+03,.4759500E+03,.5548000E+03,&
     & .5507400E+03,.4521000E+03,.2648200E+03,.7184000E+02,.1558000E+03,&
     & .2615700E+03,.3675600E+03,.4730300E+03,.5502800E+03,.5433900E+03,&
     & .4442300E+03,.2600700E+03,.7184000E+02,.1524400E+03,.2584400E+03,&
     & .3646500E+03,.4702300E+03,.5461900E+03,.5366400E+03,.4370800E+03,&
     & .2558100E+03,.7184000E+02,.1494100E+03,.2555600E+03,.3619100E+03,&
     & .4675300E+03,.5425300E+03,.5304100E+03,.4305200E+03,.2519800E+03,&
     & .7184000E+02,.1466700E+03,.2528900E+03,.3593000E+03,.4649400E+03,&
     & .5390800E+03,.5247000E+03,.4244800E+03,.2485100E+03,.7184000E+02,&
     & .1492100E+03,.2560100E+03,.3630800E+03,.4687200E+03,.5423400E+03,&
     & .5299900E+03,.4300700E+03,.2517200E+03,.7184000E+02,.1459100E+03,&
     & .2530100E+03,.3603600E+03,.4659900E+03,.5383400E+03,.5231300E+03,&
     & .4228000E+03,.2475500E+03,.7184000E+02,.1429700E+03,.2502800E+03,&
     & .3578200E+03,.4635000E+03,.5345900E+03,.5169600E+03,.4161900E+03,&
     & .2438300E+03,.7184000E+02,.1403300E+03,.2477700E+03,.3554200E+03,&
     & .4611600E+03,.5310000E+03,.5112200E+03,.4101600E+03,.2404900E+03,&
     & .7184000E+02,.1379500E+03,.2454400E+03,.3531300E+03,.4588200E+03,&
     & .5275100E+03,.5059100E+03,.4046300E+03,.2374700E+03,.7184000E+02,&
     & .1406200E+03,.2485600E+03,.3567800E+03,.4626000E+03,.5313700E+03,&
     & .5118600E+03,.4108200E+03,.2408600E+03,.7184000E+02,.1377100E+03,&
     & .2459200E+03,.3544000E+03,.4602000E+03,.5273200E+03,.5053700E+03,&
     & .4040700E+03,.2371700E+03,.7184000E+02,.1351200E+03,.2435300E+03,&
     & .3521900E+03,.4579600E+03,.5234800E+03,.4995400E+03,.3979800E+03,&
     & .2338900E+03,.7184000E+02,.1328100E+03,.2413300E+03,.3500800E+03,&
     & .4558300E+03,.5198500E+03,.4943100E+03,.3924600E+03,.2309600E+03,&
     & .7184000E+02,.1307300E+03,.2392800E+03,.3480700E+03,.4537800E+03,&
     & .5164300E+03,.4895700E+03,.3874500E+03,.2283300E+03,.7184000E+02/

      data   absa(181:315, 2)   /                                       &
     & .1333300E+03,.2422200E+03,.3514100E+03,.4570200E+03,.5205700E+03,&
     & .4955000E+03,.3937200E+03,.2316300E+03,.7184000E+02,.1307500E+03,&
     & .2399000E+03,.3493400E+03,.4547600E+03,.5165100E+03,.4896300E+03,&
     & .3875200E+03,.2283600E+03,.7184000E+02,.1284700E+03,.2378000E+03,&
     & .3474100E+03,.4526600E+03,.5127100E+03,.4843800E+03,.3819600E+03,&
     & .2254800E+03,.7184000E+02,.1264400E+03,.2358800E+03,.3455700E+03,&
     & .4506600E+03,.5091100E+03,.4796200E+03,.3769600E+03,.2229100E+03,&
     & .7184000E+02,.1246200E+03,.2340900E+03,.3438100E+03,.4487400E+03,&
     & .5057000E+03,.4752600E+03,.3724500E+03,.2206000E+03,.7184000E+02,&
     & .1272200E+03,.2369000E+03,.3468800E+03,.4517700E+03,.5103200E+03,&
     & .4814400E+03,.3788700E+03,.2238900E+03,.7184000E+02,.1249200E+03,&
     & .2348500E+03,.3450800E+03,.4496700E+03,.5062000E+03,.4759800E+03,&
     & .3731900E+03,.2209800E+03,.7184000E+02,.1229000E+03,.2330100E+03,&
     & .3434000E+03,.4476900E+03,.5024200E+03,.4710800E+03,.3681500E+03,&
     & .2184300E+03,.7184000E+02,.1211100E+03,.2313200E+03,.3418000E+03,&
     & .4458200E+03,.4989400E+03,.4666800E+03,.3636300E+03,.2161600E+03,&
     & .7184000E+02,.1195200E+03,.2297600E+03,.3402600E+03,.4440700E+03,&
     & .4957400E+03,.4627000E+03,.3595600E+03,.2141400E+03,.7184000E+02,&
     & .1220100E+03,.2323500E+03,.3430000E+03,.4470000E+03,.5005400E+03,&
     & .4688900E+03,.3659000E+03,.2173000E+03,.7184000E+02,.1199700E+03,&
     & .2305500E+03,.3414500E+03,.4449200E+03,.4965700E+03,.4638300E+03,&
     & .3607100E+03,.2147100E+03,.7184000E+02,.1181800E+03,.2289400E+03,&
     & .3399900E+03,.4430900E+03,.4929200E+03,.4593400E+03,.3561300E+03,&
     & .2124500E+03,.7184000E+02,.1166000E+03,.2274600E+03,.3385900E+03,&
     & .4414600E+03,.4895900E+03,.4553300E+03,.3520800E+03,.2104500E+03,&
     & .7184000E+02,.1152000E+03,.2260900E+03,.3372400E+03,.4399600E+03,&
     & .4865700E+03,.4517500E+03,.3484700E+03,.2086700E+03,.7184000E+02/

      data   absa(316:450, 2)   /                                       &
     & .1175600E+03,.2284400E+03,.3396500E+03,.4426100E+03,.4914700E+03,&
     & .4577600E+03,.3545400E+03,.2116600E+03,.7184000E+02,.1157400E+03,&
     & .2268700E+03,.3383000E+03,.4408200E+03,.4876700E+03,.4531300E+03,&
     & .3498600E+03,.2093600E+03,.7184000E+02,.1141500E+03,.2254600E+03,&
     & .3370400E+03,.4391500E+03,.4842900E+03,.4490800E+03,.3457900E+03,&
     & .2073500E+03,.7184000E+02,.1127600E+03,.2241700E+03,.3358400E+03,&
     & .4375800E+03,.4812600E+03,.4454900E+03,.3422000E+03,.2055900E+03,&
     & .7184000E+02,.1115300E+03,.2229800E+03,.3346600E+03,.4361700E+03,&
     & .4785500E+03,.4422800E+03,.3390200E+03,.2040300E+03,.7184000E+02,&
     & .1137600E+03,.2251000E+03,.3367400E+03,.4388200E+03,.4833600E+03,&
     & .4480700E+03,.3447900E+03,.2068600E+03,.7184000E+02,.1121400E+03,&
     & .2237200E+03,.3355900E+03,.4370600E+03,.4798900E+03,.4438800E+03,&
     & .3406000E+03,.2048000E+03,.7184000E+02,.1107400E+03,.2225000E+03,&
     & .3345100E+03,.4354700E+03,.4767900E+03,.4402000E+03,.3369700E+03,&
     & .2030300E+03,.7184000E+02,.1095100E+03,.2213800E+03,.3334800E+03,&
     & .4340400E+03,.4740400E+03,.4369400E+03,.3337900E+03,.2014700E+03,&
     & .7184000E+02,.1084300E+03,.2203400E+03,.3324600E+03,.4327700E+03,&
     & .4715900E+03,.4340500E+03,.3309600E+03,.2001000E+03,.7184000E+02,&
     & .1104100E+03,.2221400E+03,.3341400E+03,.4351700E+03,.4760400E+03,&
     & .4393200E+03,.3361100E+03,.2026100E+03,.7184000E+02,.1089800E+03,&
     & .2209500E+03,.3331700E+03,.4335800E+03,.4728300E+03,.4355200E+03,&
     & .3323900E+03,.2008000E+03,.7184000E+02,.1077400E+03,.2198900E+03,&
     & .3322600E+03,.4321300E+03,.4700200E+03,.4322200E+03,.3291700E+03,&
     & .1992400E+03,.7184000E+02,.1066700E+03,.2189300E+03,.3313900E+03,&
     & .4308300E+03,.4675300E+03,.4293200E+03,.3263400E+03,.1978800E+03,&
     & .7184000E+02,.1057300E+03,.2180300E+03,.3305300E+03,.4296500E+03,&
     & .4653200E+03,.4267700E+03,.3238600E+03,.1966900E+03,.7184000E+02/

      data   absa(451:585, 2)   /                                       &
     & .1069400E+03,.2191100E+03,.3315200E+03,.4312700E+03,.4681600E+03,&
     & .4300500E+03,.3270600E+03,.1982200E+03,.7184000E+02,.1057700E+03,&
     & .2181500E+03,.3307400E+03,.4298400E+03,.4654100E+03,.4268700E+03,&
     & .3239600E+03,.1967400E+03,.7184000E+02,.1047600E+03,.2172800E+03,&
     & .3300000E+03,.4285500E+03,.4630100E+03,.4241100E+03,.3213000E+03,&
     & .1954600E+03,.7184000E+02,.1038800E+03,.2164900E+03,.3292800E+03,&
     & .4274100E+03,.4608900E+03,.4216900E+03,.3189700E+03,.1943500E+03,&
     & .7184000E+02,.1031100E+03,.2157500E+03,.3285500E+03,.4263900E+03,&
     & .4590200E+03,.4195400E+03,.3169200E+03,.1933800E+03,.7184000E+02,&
     & .1041000E+03,.2166300E+03,.3293700E+03,.4277800E+03,.4614100E+03,&
     & .4222900E+03,.3195500E+03,.1946300E+03,.7184000E+02,.1031400E+03,&
     & .2158500E+03,.3287500E+03,.4265400E+03,.4590900E+03,.4196100E+03,&
     & .3170000E+03,.1934100E+03,.7184000E+02,.1023100E+03,.2151500E+03,&
     & .3281500E+03,.4254400E+03,.4570700E+03,.4172900E+03,.3148000E+03,&
     & .1923600E+03,.7184000E+02,.1015900E+03,.2144900E+03,.3275400E+03,&
     & .4244500E+03,.4553200E+03,.4152600E+03,.3128800E+03,.1914500E+03,&
     & .7184000E+02,.1009600E+03,.2138700E+03,.3269200E+03,.4235600E+03,&
     & .4537700E+03,.4134700E+03,.3111900E+03,.1906600E+03,.7184000E+02,&
     & .1017700E+03,.2146000E+03,.3276100E+03,.4247500E+03,.4557500E+03,&
     & .4157600E+03,.3133500E+03,.1916800E+03,.7184000E+02,.1009900E+03,&
     & .2139700E+03,.3271200E+03,.4236800E+03,.4538300E+03,.4135300E+03,&
     & .3112500E+03,.1906800E+03,.7184000E+02,.1003100E+03,.2134000E+03,&
     & .3266300E+03,.4227200E+03,.4521600E+03,.4116000E+03,.3094300E+03,&
     & .1898300E+03,.7184000E+02,.9972000E+02,.2128600E+03,.3261200E+03,&
     & .4218700E+03,.4507100E+03,.4099100E+03,.3078500E+03,.1890800E+03,&
     & .7184000E+02,.9920400E+02,.2123300E+03,.3255800E+03,.4211200E+03,&
     & .4494200E+03,.4084200E+03,.3064600E+03,.1884300E+03,.7184000E+02/

      data   absa(  1:180, 3)   /                                       &
     & .6511264E+03,.6329084E+03,.6146856E+03,.5966652E+03,.6074063E+03,&
     & .6528610E+03,.5099603E+03,.3749181E+03,.2774131E+03,.6346287E+03,&
     & .6184704E+03,.6023125E+03,.5863930E+03,.6020757E+03,.6446762E+03,&
     & .4994119E+03,.3715684E+03,.2774131E+03,.6198152E+03,.6055045E+03,&
     & .5912038E+03,.5771678E+03,.5973361E+03,.6369385E+03,.4900862E+03,&
     & .3685614E+03,.2774131E+03,.6064342E+03,.5938030E+03,.5811717E+03,&
     & .5688415E+03,.5930984E+03,.6294842E+03,.4817900E+03,.3658423E+03,&
     & .2774131E+03,.5942932E+03,.5831782E+03,.5720635E+03,.5612806E+03,&
     & .5892330E+03,.6221730E+03,.4743525E+03,.3633754E+03,.2774131E+03,&
     & .6015146E+03,.5894958E+03,.5774770E+03,.5658111E+03,.5943788E+03,&
     & .6267589E+03,.4787630E+03,.3648436E+03,.2774131E+03,.5873680E+03,&
     & .5771175E+03,.5668670E+03,.5570576E+03,.5905502E+03,.6181492E+03,&
     & .4701504E+03,.3619729E+03,.2774131E+03,.5746935E+03,.5660284E+03,&
     & .5573637E+03,.5492284E+03,.5870483E+03,.6098467E+03,.4625325E+03,&
     & .3594001E+03,.2774131E+03,.5632785E+03,.5560360E+03,.5487986E+03,&
     & .5422048E+03,.5837067E+03,.6020593E+03,.4557839E+03,.3570844E+03,&
     & .2774131E+03,.5529351E+03,.5469870E+03,.5410436E+03,.5358636E+03,&
     & .5804343E+03,.5946671E+03,.4497552E+03,.3549858E+03,.2774131E+03,&
     & .5625080E+03,.5553643E+03,.5482206E+03,.5421149E+03,.5857773E+03,&
     & .6015260E+03,.4553353E+03,.3569280E+03,.2774131E+03,.5500901E+03,&
     & .5445026E+03,.5389099E+03,.5346354E+03,.5824284E+03,.5925618E+03,&
     & .4481108E+03,.3544056E+03,.2774131E+03,.5390110E+03,.5348064E+03,&
     & .5306017E+03,.5279541E+03,.5791807E+03,.5842402E+03,.4417818E+03,&
     & .3521607E+03,.2774131E+03,.5290630E+03,.5260980E+03,.5231382E+03,&
     & .5219585E+03,.5760337E+03,.5766944E+03,.4361804E+03,.3501376E+03,&
     & .2774131E+03,.5200781E+03,.5182394E+03,.5163958E+03,.5165553E+03,&
     & .5729753E+03,.5696064E+03,.4311945E+03,.3483165E+03,.2774131E+03,&
     & .5301497E+03,.5270514E+03,.5239531E+03,.5232218E+03,.5779936E+03,&
     & .5775254E+03,.4367865E+03,.3503547E+03,.2774131E+03,.5191743E+03,&
     & .5174492E+03,.5157190E+03,.5168800E+03,.5747472E+03,.5688802E+03,&
     & .4306977E+03,.3481297E+03,.2774131E+03,.5094140E+03,.5089090E+03,&
     & .5084039E+03,.5112313E+03,.5716791E+03,.5610450E+03,.4253723E+03,&
     & .3461523E+03,.2774131E+03,.5006908E+03,.5012725E+03,.5018541E+03,&
     & .5061415E+03,.5687148E+03,.5539480E+03,.4206699E+03,.3443815E+03,&
     & .2774131E+03,.4928322E+03,.4944017E+03,.4959664E+03,.5015429E+03,&
     & .5658453E+03,.5474233E+03,.4164958E+03,.3427822E+03,.2774131E+03/

      data   absa(181:315, 3)   /                                       &
     & .5026617E+03,.5029964E+03,.5033359E+03,.5079576E+03,.5703975E+03,&
     & .5555608E+03,.4217244E+03,.3447801E+03,.2774131E+03,.4929459E+03,&
     & .4944957E+03,.4960504E+03,.5025983E+03,.5672753E+03,.5475129E+03,&
     & .4165470E+03,.3428074E+03,.2774131E+03,.4843464E+03,.4869729E+03,&
     & .4895995E+03,.4978824E+03,.5642908E+03,.5402030E+03,.4120341E+03,&
     & .3410570E+03,.2774131E+03,.4766902E+03,.4802750E+03,.4838598E+03,&
     & .4936340E+03,.5614813E+03,.5336150E+03,.4080600E+03,.3395081E+03,&
     & .2774131E+03,.4698243E+03,.4742636E+03,.4787081E+03,.4897906E+03,&
     & .5587886E+03,.5276339E+03,.4045435E+03,.3381108E+03,.2774131E+03,&
     & .4796093E+03,.4828287E+03,.4860480E+03,.4958831E+03,.5631234E+03,&
     & .5361276E+03,.4095724E+03,.3400983E+03,.2774131E+03,.4709554E+03,&
     & .4752614E+03,.4795626E+03,.4913793E+03,.5601219E+03,.5286289E+03,&
     & .4051219E+03,.3383427E+03,.2774131E+03,.4633537E+03,.4686031E+03,&
     & .4738526E+03,.4874207E+03,.5572835E+03,.5219272E+03,.4012664E+03,&
     & .3367990E+03,.2774131E+03,.4566014E+03,.4626954E+03,.4687945E+03,&
     & .4838547E+03,.5545520E+03,.5159087E+03,.3978937E+03,.3354268E+03,&
     & .2774131E+03,.4505703E+03,.4574252E+03,.4642700E+03,.4806076E+03,&
     & .5519154E+03,.5104788E+03,.3949126E+03,.3342010E+03,.2774131E+03,&
     & .4599800E+03,.4656593E+03,.4713286E+03,.4861245E+03,.5561434E+03,&
     & .5189328E+03,.3995824E+03,.3361129E+03,.2774131E+03,.4522794E+03,&
     & .4589169E+03,.4655493E+03,.4823660E+03,.5531814E+03,.5120152E+03,&
     & .3957540E+03,.3345540E+03,.2774131E+03,.4455371E+03,.4530143E+03,&
     & .4604964E+03,.4790627E+03,.5504387E+03,.5058908E+03,.3924488E+03,&
     & .3331819E+03,.2774131E+03,.4395801E+03,.4478082E+03,.4560363E+03,&
     & .4760716E+03,.5478683E+03,.5004584E+03,.3895600E+03,.3319712E+03,&
     & .2774131E+03,.4342899E+03,.4431749E+03,.4520702E+03,.4733295E+03,&
     & .5454191E+03,.4955934E+03,.3870166E+03,.3309017E+03,.2774131E+03/

      data   absa(316:450, 3)   /                                       &
     & .4431908E+03,.4509594E+03,.4587381E+03,.4781967E+03,.5493562E+03,&
     & .5037547E+03,.3913036E+03,.3327077E+03,.2774131E+03,.4363300E+03,&
     & .4449580E+03,.4535964E+03,.4750441E+03,.5465445E+03,.4974754E+03,&
     & .3879943E+03,.3313103E+03,.2774131E+03,.4303582E+03,.4397370E+03,&
     & .4491215E+03,.4722875E+03,.5439163E+03,.4918909E+03,.3851474E+03,&
     & .3301045E+03,.2774131E+03,.4251124E+03,.4351433E+03,.4451950E+03,&
     & .4698098E+03,.5414411E+03,.4869096E+03,.3826759E+03,.3290350E+03,&
     & .2774131E+03,.4204694E+03,.4310783E+03,.4417179E+03,.4675130E+03,&
     & .5390622E+03,.4824898E+03,.3805037E+03,.3280966E+03,.2774131E+03,&
     & .4288912E+03,.4384478E+03,.4480252E+03,.4717437E+03,.5428850E+03,&
     & .4904977E+03,.3844535E+03,.3298019E+03,.2774131E+03,.4227713E+03,&
     & .4330984E+03,.4434414E+03,.4691286E+03,.5401449E+03,.4846799E+03,&
     & .3815770E+03,.3285608E+03,.2774131E+03,.4174811E+03,.4284602E+03,&
     & .4394805E+03,.4668405E+03,.5375978E+03,.4796375E+03,.3791166E+03,&
     & .3274861E+03,.2774131E+03,.4128529E+03,.4244148E+03,.4360182E+03,&
     & .4647682E+03,.5352111E+03,.4752154E+03,.3769855E+03,.3265430E+03,&
     & .2774131E+03,.4087727E+03,.4208485E+03,.4329611E+03,.4628378E+03,&
     & .5329651E+03,.4713043E+03,.3751287E+03,.3257206E+03,.2774131E+03,&
     & .4162315E+03,.4273735E+03,.4385419E+03,.4662891E+03,.5364495E+03,&
     & .4784476E+03,.3785445E+03,.3272290E+03,.2774131E+03,.4108377E+03,&
     & .4226565E+03,.4345069E+03,.4641370E+03,.5338298E+03,.4732874E+03,&
     & .3760645E+03,.3261392E+03,.2774131E+03,.4061995E+03,.4185915E+03,&
     & .4310298E+03,.4622539E+03,.5314356E+03,.4688122E+03,.3739650E+03,&
     & .3251960E+03,.2774131E+03,.4021541E+03,.4150596E+03,.4280071E+03,&
     & .4605480E+03,.5292326E+03,.4648804E+03,.3721593E+03,.3243736E+03,&
     & .2774131E+03,.3986074E+03,.4119528E+03,.4253550E+03,.4589564E+03,&
     & .5271755E+03,.4613869E+03,.3705866E+03,.3236571E+03,.2774131E+03/

      data   absa(451:585, 3)   /                                       &
     & .4031715E+03,.4159438E+03,.4287676E+03,.4609884E+03,.5292837E+03,&
     & .4658721E+03,.3726094E+03,.3245855E+03,.2774131E+03,.3987507E+03,&
     & .4120761E+03,.4254586E+03,.4593570E+03,.5269580E+03,.4615362E+03,&
     & .3706474E+03,.3236875E+03,.2774131E+03,.3949475E+03,.4087520E+03,&
     & .4226140E+03,.4578984E+03,.5248194E+03,.4578690E+03,.3689687E+03,&
     & .3229107E+03,.2774131E+03,.3916430E+03,.4058525E+03,.4201396E+03,&
     & .4565441E+03,.5228408E+03,.4544830E+03,.3675280E+03,.3222398E+03,&
     & .2774131E+03,.3887335E+03,.4033137E+03,.4179614E+03,.4552601E+03,&
     & .5210222E+03,.4515774E+03,.3662647E+03,.3216544E+03,.2774131E+03,&
     & .3924531E+03,.4065686E+03,.4207468E+03,.4569108E+03,.5228838E+03,&
     & .4553196E+03,.3678766E+03,.3224061E+03,.2774131E+03,.3888324E+03,&
     & .4034026E+03,.4180454E+03,.4556544E+03,.5208037E+03,.4516870E+03,&
     & .3663098E+03,.3216696E+03,.2774131E+03,.3857204E+03,.4006760E+03,&
     & .4157143E+03,.4545126E+03,.5188851E+03,.4485672E+03,.3649658E+03,&
     & .3210391E+03,.2774131E+03,.3830138E+03,.3983049E+03,.4136846E+03,&
     & .4534319E+03,.5171572E+03,.4458556E+03,.3638040E+03,.3204890E+03,&
     & .2774131E+03,.3806279E+03,.3962256E+03,.4118963E+03,.4523829E+03,&
     & .5156086E+03,.4434924E+03,.3627937E+03,.3200048E+03,.2774131E+03,&
     & .3836807E+03,.3988929E+03,.4141833E+03,.4537401E+03,.5172110E+03,&
     & .4465237E+03,.3640918E+03,.3206253E+03,.2774131E+03,.3807168E+03,&
     & .3962996E+03,.4119655E+03,.4527776E+03,.5153732E+03,.4435721E+03,&
     & .3628289E+03,.3200248E+03,.2774131E+03,.3781631E+03,.3940670E+03,&
     & .4100595E+03,.4518853E+03,.5137515E+03,.4410437E+03,.3617483E+03,&
     & .3195054E+03,.2774131E+03,.3759453E+03,.3921257E+03,.4084000E+03,&
     & .4510200E+03,.5123154E+03,.4388535E+03,.3608140E+03,.3190564E+03,&
     & .2774131E+03,.3739993E+03,.3904218E+03,.4069430E+03,.4501424E+03,&
     & .5110249E+03,.4369317E+03,.3599960E+03,.3186578E+03,.2774131E+03/

      data   absa(  1:180, 4)   /                                       &
     & .1490491E+04,.1320345E+04,.1150192E+04,.9800544E+03,.8098902E+03,&
     & .7197150E+03,.6792147E+03,.7402425E+03,.8002384E+03,.1452731E+04,&
     & .1287287E+04,.1121843E+04,.9564311E+03,.7910079E+03,.7120191E+03,&
     & .6771483E+03,.7390036E+03,.7998958E+03,.1418780E+04,.1257599E+04,&
     & .1096434E+04,.9352534E+03,.7740540E+03,.7025255E+03,.6751582E+03,&
     & .7376910E+03,.7993864E+03,.1388200E+04,.1230836E+04,.1073482E+04,&
     & .9161180E+03,.7587353E+03,.6923034E+03,.6731990E+03,.7363185E+03,&
     & .7987103E+03,.1360397E+04,.1206504E+04,.1052610E+04,.8987339E+03,&
     & .7448387E+03,.6822317E+03,.6712136E+03,.7348370E+03,.7978722E+03,&
     & .1376903E+04,.1220978E+04,.1065010E+04,.9090577E+03,.7531056E+03,&
     & .6894651E+03,.6733436E+03,.7372884E+03,.8003310E+03,.1344538E+04,&
     & .1192629E+04,.1040735E+04,.8888218E+03,.7369143E+03,.6768296E+03,&
     & .6715747E+03,.7362303E+03,.8000578E+03,.1315550E+04,.1167251E+04,&
     & .1018964E+04,.8706883E+03,.7224512E+03,.6656647E+03,.6698697E+03,&
     & .7351070E+03,.7996179E+03,.1289379E+04,.1144358E+04,.9993773E+03,&
     & .8543538E+03,.7095866E+03,.6560533E+03,.6681935E+03,.7339123E+03,&
     & .7990113E+03,.1265740E+04,.1123642E+04,.9815969E+03,.8395554E+03,&
     & .6981281E+03,.6477726E+03,.6664850E+03,.7326277E+03,.7982380E+03,&
     & .1287647E+04,.1142819E+04,.9980492E+03,.8532534E+03,.7089617E+03,&
     & .6555751E+03,.6687079E+03,.7349497E+03,.8004236E+03,.1259199E+04,&
     & .1117994E+04,.9767252E+03,.8354916E+03,.6953763E+03,.6460584E+03,&
     & .6671911E+03,.7340872E+03,.8002431E+03,.1233874E+04,.1095802E+04,&
     & .9576897E+03,.8196373E+03,.6835316E+03,.6380393E+03,.6657230E+03,&
     & .7331401E+03,.7999004E+03,.1211081E+04,.1075833E+04,.9406536E+03,&
     & .8054044E+03,.6731147E+03,.6310221E+03,.6642968E+03,.7321310E+03,&
     & .7993957E+03,.1190512E+04,.1057848E+04,.9251886E+03,.7925536E+03,&
     & .6639048E+03,.6250737E+03,.6628407E+03,.7310159E+03,.7987242E+03,&
     & .1213605E+04,.1078027E+04,.9425174E+03,.8069604E+03,.6747932E+03,&
     & .6317831E+03,.6648333E+03,.7329850E+03,.8004607E+03,.1188427E+04,&
     & .1056052E+04,.9236605E+03,.7912500E+03,.6637826E+03,.6247883E+03,&
     & .6635334E+03,.7322842E+03,.8003773E+03,.1166126E+04,.1036497E+04,&
     & .9069190E+03,.7772941E+03,.6542493E+03,.6188027E+03,.6622900E+03,&
     & .7315166E+03,.8001412E+03,.1146104E+04,.1019036E+04,.8919134E+03,&
     & .7648103E+03,.6459064E+03,.6135679E+03,.6610584E+03,.7306623E+03,&
     & .7997430E+03,.1128166E+04,.1003292E+04,.8784385E+03,.7535801E+03,&
     & .6385155E+03,.6090063E+03,.6598247E+03,.7297259E+03,.7991780E+03/

      data   absa(181:315, 4)   /                                       &
     & .1150667E+04,.1022958E+04,.8952985E+03,.7676352E+03,.6485262E+03,&
     & .6146217E+03,.6614995E+03,.7312576E+03,.8004375E+03,.1128412E+04,&
     & .1003487E+04,.8786170E+03,.7537286E+03,.6397248E+03,.6092128E+03,&
     & .6603950E+03,.7307176E+03,.8004468E+03,.1108741E+04,.9862884E+03,&
     & .8638632E+03,.7414333E+03,.6321640E+03,.6047173E+03,.6593415E+03,&
     & .7301049E+03,.8003079E+03,.1091213E+04,.9709590E+03,.8507153E+03,&
     & .7304755E+03,.6255493E+03,.6008338E+03,.6583092E+03,.7294008E+03,&
     & .8000115E+03,.1075469E+04,.9572202E+03,.8389248E+03,.7206527E+03,&
     & .6197031E+03,.5974555E+03,.6572486E+03,.7286139E+03,.7995531E+03,&
     & .1097870E+04,.9768019E+03,.8557256E+03,.7346486E+03,.6288307E+03,&
     & .6020713E+03,.6586595E+03,.7297565E+03,.8003542E+03,.1078052E+04,&
     & .9594624E+03,.8408779E+03,.7222787E+03,.6218550E+03,.5979982E+03,&
     & .6577350E+03,.7293614E+03,.8004561E+03,.1060670E+04,.9442276E+03,&
     & .8278139E+03,.7113901E+03,.6158860E+03,.5946078E+03,.6568508E+03,&
     & .7288896E+03,.8004144E+03,.1045194E+04,.9306942E+03,.8162219E+03,&
     & .7017312E+03,.6107127E+03,.5916997E+03,.6559770E+03,.7283342E+03,&
     & .8002199E+03,.1031388E+04,.9186406E+03,.8058789E+03,.6931126E+03,&
     & .6061547E+03,.5891687E+03,.6550888E+03,.7276821E+03,.7998680E+03,&
     & .1052963E+04,.9374921E+03,.8220302E+03,.7065729E+03,.6139864E+03,&
     & .5928231E+03,.6561758E+03,.7284058E+03,.8001736E+03,.1035335E+04,&
     & .9220734E+03,.8088077E+03,.6955512E+03,.6085341E+03,.5897484E+03,&
     & .6554522E+03,.7281509E+03,.8004051E+03,.1019870E+04,.9085454E+03,&
     & .7972311E+03,.6859069E+03,.6038862E+03,.5871960E+03,.6547087E+03,&
     & .7278186E+03,.8004607E+03,.1006236E+04,.8966542E+03,.7870120E+03,&
     & .6773876E+03,.5998494E+03,.5849802E+03,.6539834E+03,.7273980E+03,&
     & .8003588E+03,.9941610E+03,.8860349E+03,.7779226E+03,.6698202E+03,&
     & .5962793E+03,.5830334E+03,.6532431E+03,.7268747E+03,.8001088E+03/

      data   absa(316:450, 4)   /                                       &
     & .1014525E+04,.9038521E+03,.7932066E+03,.6825518E+03,.6028710E+03,&
     & .5859051E+03,.6539946E+03,.7271787E+03,.7999513E+03,.9987919E+03,&
     & .8901187E+03,.7814262E+03,.6727344E+03,.5986427E+03,.5835388E+03,&
     & .6534210E+03,.7270632E+03,.8002847E+03,.9851423E+03,.8781550E+03,&
     & .7711724E+03,.6641905E+03,.5950831E+03,.5816419E+03,.6528439E+03,&
     & .7268557E+03,.8004468E+03,.9731156E+03,.8676435E+03,.7621676E+03,&
     & .6566870E+03,.5920005E+03,.5800708E+03,.6522571E+03,.7265599E+03,&
     & .8004375E+03,.9624848E+03,.8583463E+03,.7541885E+03,.6500361E+03,&
     & .5892896E+03,.5786674E+03,.6516491E+03,.7261614E+03,.8002755E+03,&
     & .9817773E+03,.8752109E+03,.7686499E+03,.6620890E+03,.5946884E+03,&
     & .5807066E+03,.6520760E+03,.7260596E+03,.7996781E+03,.9677814E+03,&
     & .8629603E+03,.7581484E+03,.6533365E+03,.5914754E+03,.5790226E+03,&
     & .6516633E+03,.7260782E+03,.8001134E+03,.9556546E+03,.8523548E+03,&
     & .7490543E+03,.6457592E+03,.5888086E+03,.5776101E+03,.6512193E+03,&
     & .7259955E+03,.8003773E+03,.9450453E+03,.8430869E+03,.7411152E+03,&
     & .6391429E+03,.5865198E+03,.5764053E+03,.6507649E+03,.7258146E+03,&
     & .8004607E+03,.9357266E+03,.8349146E+03,.7341173E+03,.6333100E+03,&
     & .5843768E+03,.5753367E+03,.6502809E+03,.7255256E+03,.8003866E+03,&
     & .9528090E+03,.8498570E+03,.7469182E+03,.6439800E+03,.5885144E+03,&
     & .5767160E+03,.6503433E+03,.7250381E+03,.7994049E+03,.9404359E+03,&
     & .8390577E+03,.7376609E+03,.6362634E+03,.5861145E+03,.5754558E+03,&
     & .6500784E+03,.7251769E+03,.7999235E+03,.9298105E+03,.8297558E+03,&
     & .7296865E+03,.6296179E+03,.5839417E+03,.5744210E+03,.6497768E+03,&
     & .7252044E+03,.8002708E+03,.9205811E+03,.8216675E+03,.7227532E+03,&
     & .6238388E+03,.5819578E+03,.5735632E+03,.6494410E+03,.7251291E+03,&
     & .8004422E+03,.9124496E+03,.8145610E+03,.7166625E+03,.6187693E+03,&
     & .5800439E+03,.5728294E+03,.6490647E+03,.7249410E+03,.8004422E+03/

      data   absa(451:585, 4)   /                                       &
     & .9228965E+03,.8236997E+03,.7244976E+03,.6252963E+03,.5826587E+03,&
     & .5732062E+03,.6487682E+03,.7242367E+03,.7994003E+03,.9127905E+03,&
     & .8148481E+03,.7169103E+03,.6189724E+03,.5806411E+03,.5724380E+03,&
     & .6486216E+03,.7244324E+03,.7999189E+03,.9040966E+03,.8072260E+03,&
     & .7103786E+03,.6135258E+03,.5787682E+03,.5716879E+03,.6484230E+03,&
     & .7245206E+03,.8002662E+03,.8965007E+03,.8005998E+03,.7046989E+03,&
     & .6087980E+03,.5770291E+03,.5712760E+03,.6481694E+03,.7244914E+03,&
     & .8004422E+03,.8898544E+03,.7947815E+03,.6997086E+03,.6046349E+03,&
     & .5752976E+03,.5708062E+03,.6478707E+03,.7243347E+03,.8004422E+03,&
     & .8983644E+03,.8022304E+03,.7060963E+03,.6099576E+03,.5774937E+03,&
     & .5708077E+03,.6474759E+03,.7235789E+03,.7994003E+03,.8901061E+03,&
     & .7949847E+03,.6998817E+03,.6047834E+03,.5757149E+03,.5704095E+03,&
     & .6474223E+03,.7238307E+03,.7999235E+03,.8829565E+03,.7887454E+03,&
     & .6945344E+03,.6003279E+03,.5739676E+03,.5700508E+03,.6473059E+03,&
     & .7239504E+03,.8002662E+03,.8767566E+03,.7833188E+03,.6898858E+03,&
     & .5964527E+03,.5721368E+03,.5697050E+03,.6471298E+03,.7239527E+03,&
     & .8004422E+03,.8713100E+03,.7785510E+03,.6858020E+03,.5930430E+03,&
     & .5702838E+03,.5693558E+03,.6468926E+03,.7238360E+03,.8004422E+03,&
     & .8782779E+03,.7846517E+03,.6910308E+03,.5974092E+03,.5724839E+03,&
     & .5692167E+03,.6464156E+03,.7230395E+03,.7994003E+03,.8714985E+03,&
     & .7787195E+03,.6859459E+03,.5931669E+03,.5705842E+03,.5689690E+03,&
     & .6464442E+03,.7233274E+03,.7999235E+03,.8656602E+03,.7736099E+03,&
     & .6815696E+03,.5895194E+03,.5686553E+03,.5687141E+03,.6463947E+03,&
     & .7234879E+03,.8002708E+03,.8605853E+03,.7691745E+03,.6777583E+03,&
     & .5863474E+03,.5667574E+03,.5684512E+03,.6462754E+03,.7235209E+03,&
     & .8004422E+03,.8561252E+03,.7652692E+03,.6744132E+03,.5835572E+03,&
     & .5649152E+03,.5681796E+03,.6460944E+03,.7234350E+03,.8004422E+03/

      data   absa(  1:180, 5)   /                                       &
     & .1921397E+04,.1686175E+04,.1450918E+04,.1215651E+04,.9804589E+03,&
     & .7485868E+03,.8483659E+03,.9768410E+03,.1106468E+04,.1872699E+04,&
     & .1643569E+04,.1414370E+04,.1185266E+04,.9561203E+03,.7344024E+03,&
     & .8467260E+03,.9754207E+03,.1105202E+04,.1828966E+04,.1605287E+04,&
     & .1381610E+04,.1157925E+04,.9342532E+03,.7247494E+03,.8449271E+03,&
     & .9738067E+03,.1103587E+04,.1789486E+04,.1570750E+04,.1351993E+04,&
     & .1133258E+04,.9145250E+03,.7183685E+03,.8429653E+03,.9719387E+03,&
     & .1101677E+04,.1753694E+04,.1539382E+04,.1325155E+04,.1110855E+04,&
     & .8966094E+03,.7139851E+03,.8408707E+03,.9698995E+03,.1099550E+04,&
     & .1775002E+04,.1558008E+04,.1341156E+04,.1124178E+04,.9072693E+03,&
     & .7199288E+03,.8471982E+03,.9765647E+03,.1106903E+04,.1733216E+04,&
     & .1521534E+04,.1309834E+04,.1098091E+04,.8863907E+03,.7174123E+03,&
     & .8457590E+03,.9752894E+03,.1105710E+04,.1695794E+04,.1488814E+04,&
     & .1281745E+04,.1074723E+04,.8676958E+03,.7156385E+03,.8441368E+03,&
     & .9737966E+03,.1104271E+04,.1662143E+04,.1459341E+04,.1256502E+04,&
     & .1053659E+04,.8508412E+03,.7137237E+03,.8423258E+03,.9720960E+03,&
     & .1102513E+04,.1631619E+04,.1432618E+04,.1233592E+04,.1034579E+04,&
     & .8355734E+03,.7116486E+03,.8403867E+03,.9701499E+03,.1100500E+04,&
     & .1659891E+04,.1457315E+04,.1254812E+04,.1052240E+04,.8497202E+03,&
     & .7175527E+03,.8464715E+03,.9765415E+03,.1107518E+04,.1623228E+04,&
     & .1425304E+04,.1227301E+04,.1029375E+04,.8314009E+03,.7161499E+03,&
     & .8452402E+03,.9754612E+03,.1106487E+04,.1590539E+04,.1396639E+04,&
     & .1202788E+04,.1008908E+04,.8150480E+03,.7145901E+03,.8438213E+03,&
     & .9741374E+03,.1105224E+04,.1561180E+04,.1371014E+04,.1180792E+04,&
     & .9905638E+03,.8003605E+03,.7128878E+03,.8422042E+03,.9726066E+03,&
     & .1103635E+04,.1534665E+04,.1347786E+04,.1160902E+04,.9740042E+03,&
     & .7871058E+03,.7110306E+03,.8404136E+03,.9708259E+03,.1101753E+04,&
     & .1564387E+04,.1373801E+04,.1183199E+04,.9925688E+03,.8019746E+03,&
     & .7163274E+03,.8458907E+03,.9765642E+03,.1107990E+04,.1532032E+04,&
     & .1345436E+04,.1158880E+04,.9723223E+03,.7857601E+03,.7151483E+03,&
     & .8448704E+03,.9756467E+03,.1107164E+04,.1503216E+04,.1320225E+04,&
     & .1137324E+04,.9543379E+03,.7713655E+03,.7138061E+03,.8436484E+03,&
     & .9745256E+03,.1106074E+04,.1477480E+04,.1297700E+04,.1117987E+04,&
     & .9382370E+03,.7585035E+03,.7123059E+03,.8422273E+03,.9731665E+03,&
     & .1104675E+04,.1454275E+04,.1277417E+04,.1100599E+04,.9237584E+03,&
     & .7469097E+03,.7106555E+03,.8406082E+03,.9715630E+03,.1103011E+04/

      data   absa(181:315, 5)   /                                       &
     & .1483270E+04,.1302824E+04,.1122346E+04,.9418665E+03,.7613975E+03,&
     & .7152456E+03,.8453518E+03,.9765193E+03,.1108347E+04,.1454554E+04,&
     & .1277730E+04,.1100817E+04,.9239508E+03,.7470711E+03,.7142698E+03,&
     & .8445296E+03,.9757838E+03,.1107704E+04,.1429277E+04,.1255537E+04,&
     & .1081803E+04,.9080984E+03,.7343933E+03,.7131346E+03,.8435047E+03,&
     & .9748486E+03,.1106819E+04,.1406637E+04,.1235745E+04,.1064862E+04,&
     & .8939670E+03,.7230809E+03,.7118341E+03,.8422567E+03,.9736530E+03,&
     & .1105584E+04,.1386352E+04,.1217991E+04,.1049654E+04,.8813175E+03,&
     & .7129541E+03,.7103644E+03,.8408216E+03,.9722114E+03,.1104099E+04,&
     & .1415280E+04,.1243273E+04,.1071368E+04,.8993573E+03,.7273941E+03,&
     & .7142932E+03,.8448580E+03,.9764281E+03,.1108541E+04,.1389733E+04,&
     & .1220958E+04,.1052188E+04,.8834056E+03,.7146427E+03,.7135269E+03,&
     & .8442338E+03,.9758859E+03,.1108141E+04,.1367294E+04,.1201275E+04,&
     & .1035372E+04,.8693794E+03,.7034205E+03,.7125886E+03,.8433949E+03,&
     & .9751144E+03,.1107408E+04,.1347369E+04,.1183915E+04,.1020401E+04,&
     & .8569345E+03,.6934430E+03,.7114752E+03,.8423448E+03,.9741143E+03,&
     & .1106423E+04,.1329578E+04,.1168334E+04,.1007064E+04,.8458218E+03,&
     & .6845663E+03,.7101924E+03,.8410834E+03,.9728599E+03,.1105097E+04,&
     & .1357347E+04,.1192598E+04,.1027874E+04,.8631699E+03,.6984448E+03,&
     & .7134207E+03,.8443831E+03,.9763011E+03,.1108758E+04,.1334644E+04,&
     & .1172715E+04,.1010833E+04,.8489539E+03,.6870762E+03,.7128438E+03,&
     & .8439045E+03,.9759286E+03,.1108430E+04,.1314672E+04,.1155279E+04,&
     & .9959069E+03,.8365335E+03,.6771252E+03,.7120902E+03,.8432747E+03,&
     & .9753288E+03,.1107888E+04,.1297153E+04,.1139973E+04,.9827356E+03,&
     & .8255379E+03,.6683354E+03,.7111590E+03,.8424009E+03,.9744943E+03,&
     & .1107103E+04,.1281549E+04,.1126289E+04,.9710294E+03,.8157849E+03,&
     & .6605299E+03,.7100490E+03,.8413065E+03,.9734035E+03,.1105937E+04/

      data   absa(316:450, 5)   /                                       &
     & .1307799E+04,.1149258E+04,.9907270E+03,.8322002E+03,.6736660E+03,&
     & .7126027E+03,.8439097E+03,.9761135E+03,.1108805E+04,.1287540E+04,&
     & .1131537E+04,.9755603E+03,.8195432E+03,.6635490E+03,.7122058E+03,&
     & .8436121E+03,.9758993E+03,.1108673E+04,.1269927E+04,.1116126E+04,&
     & .9623208E+03,.8085370E+03,.6547258E+03,.7116273E+03,.8431237E+03,&
     & .9754680E+03,.1108244E+04,.1254438E+04,.1102594E+04,.9507274E+03,&
     & .7988474E+03,.6469982E+03,.7108636E+03,.8424204E+03,.9747864E+03,&
     & .1107597E+04,.1240737E+04,.1090596E+04,.9404282E+03,.7903006E+03,&
     & .6401369E+03,.7099113E+03,.8414889E+03,.9738621E+03,.1106649E+04,&
     & .1265622E+04,.1112367E+04,.9590789E+03,.8058196E+03,.6525657E+03,&
     & .7118328E+03,.8434406E+03,.9758605E+03,.1108716E+04,.1247560E+04,&
     & .1096553E+04,.9455416E+03,.7945397E+03,.6435460E+03,.7116013E+03,&
     & .8432871E+03,.9758224E+03,.1108744E+04,.1231894E+04,.1082859E+04,&
     & .9338167E+03,.7847896E+03,.6357324E+03,.7111927E+03,.8429522E+03,&
     & .9755286E+03,.1108560E+04,.1218302E+04,.1070950E+04,.9235833E+03,&
     & .7762412E+03,.6289015E+03,.7105830E+03,.8424056E+03,.9749952E+03,&
     & .1108010E+04,.1206262E+04,.1060398E+04,.9145570E+03,.7687192E+03,&
     & .6229813E+03,.7097942E+03,.8416349E+03,.9742335E+03,.1107264E+04,&
     & .1228234E+04,.1079659E+04,.9310619E+03,.7824865E+03,.6338926E+03,&
     & .7111057E+03,.8429845E+03,.9755941E+03,.1108579E+04,.1212319E+04,&
     & .1065758E+04,.9191269E+03,.7725251E+03,.6260392E+03,.7110095E+03,&
     & .8429498E+03,.9756589E+03,.1108771E+04,.1198612E+04,.1053738E+04,&
     & .9088499E+03,.7639808E+03,.6194625E+03,.7107383E+03,.8427382E+03,&
     & .9755143E+03,.1108675E+04,.1186692E+04,.1043289E+04,.8999106E+03,&
     & .7565209E+03,.6138184E+03,.7102789E+03,.8423199E+03,.9751033E+03,&
     & .1108322E+04,.1176220E+04,.1034168E+04,.8920570E+03,.7499699E+03,&
     & .6090103E+03,.7096231E+03,.8416809E+03,.9744656E+03,.1107679E+04/

      data   absa(451:585, 5)   /                                       &
     & .1189698E+04,.1045913E+04,.9021705E+03,.7583950E+03,.6153540E+03,&
     & .7104698E+03,.8425981E+03,.9754239E+03,.1108576E+04,.1176635E+04,&
     & .1034523E+04,.8923869E+03,.7502341E+03,.6094352E+03,.7104222E+03,&
     & .8425962E+03,.9755058E+03,.1108734E+04,.1165429E+04,.1024685E+04,&
     & .8839611E+03,.7432232E+03,.6045181E+03,.7101914E+03,.8424044E+03,&
     & .9753409E+03,.1108675E+04,.1155697E+04,.1016166E+04,.8766431E+03,&
     & .7371151E+03,.6003622E+03,.7097679E+03,.8420139E+03,.9749621E+03,&
     & .1108322E+04,.1147121E+04,.1008658E+04,.8702118E+03,.7317633E+03,&
     & .5967883E+03,.7091490E+03,.8413991E+03,.9743362E+03,.1107679E+04,&
     & .1158071E+04,.1018256E+04,.8784521E+03,.7386226E+03,.6015990E+03,&
     & .7099541E+03,.8422884E+03,.9752818E+03,.1108579E+04,.1147403E+04,&
     & .1008908E+04,.8704370E+03,.7319491E+03,.5971897E+03,.7099414E+03,&
     & .8423049E+03,.9753726E+03,.1108766E+04,.1138174E+04,.1000855E+04,&
     & .8635459E+03,.7261979E+03,.5935081E+03,.7097458E+03,.8421398E+03,&
     & .9752320E+03,.1108695E+04,.1130203E+04,.9938656E+03,.8575435E+03,&
     & .7212024E+03,.5905613E+03,.7093511E+03,.8417643E+03,.9748491E+03,&
     & .1108322E+04,.1123201E+04,.9877521E+03,.8522707E+03,.7168232E+03,&
     & .5880771E+03,.7087599E+03,.8411666E+03,.9742254E+03,.1107679E+04,&
     & .1132182E+04,.9955937E+03,.8590183E+03,.7224580E+03,.5914804E+03,&
     & .7095304E+03,.8420290E+03,.9751646E+03,.1108576E+04,.1123422E+04,&
     & .9879637E+03,.8524632E+03,.7169730E+03,.5885566E+03,.7095493E+03,&
     & .8420734E+03,.9752650E+03,.1108766E+04,.1115932E+04,.9813727E+03,&
     & .8468160E+03,.7122910E+03,.5862722E+03,.7093820E+03,.8419175E+03,&
     & .9751333E+03,.1108697E+04,.1109330E+04,.9756452E+03,.8419083E+03,&
     & .7081783E+03,.5843612E+03,.7090122E+03,.8415604E+03,.9747502E+03,&
     & .1108322E+04,.1103600E+04,.9706010E+03,.8376039E+03,.7045930E+03,&
     & .5827075E+03,.7084398E+03,.8409654E+03,.9741292E+03,.1107699E+04/

      data   absa(  1:180, 6)   /                                       &
     & .1959531E+04,.1718854E+04,.1478161E+04,.1237527E+04,.9965716E+03,&
     & .7556952E+03,.8878598E+03,.1025441E+04,.1163518E+04,.1910117E+04,&
     & .1675238E+04,.1441129E+04,.1206451E+04,.9715933E+03,.7486894E+03,&
     & .8856157E+03,.1023242E+04,.1161590E+04,.1865741E+04,.1636653E+04,&
     & .1407543E+04,.1178524E+04,.9494810E+03,.7463519E+03,.8833101E+03,&
     & .1021156E+04,.1159586E+04,.1825329E+04,.1601414E+04,.1377281E+04,&
     & .1153436E+04,.9292048E+03,.7441661E+03,.8810616E+03,.1019015E+04,&
     & .1157664E+04,.1788734E+04,.1569552E+04,.1349728E+04,.1130211E+04,&
     & .9109703E+03,.7423144E+03,.8791860E+03,.1017194E+04,.1156120E+04,&
     & .1810667E+04,.1588518E+04,.1365889E+04,.1144195E+04,.9217339E+03,&
     & .7496752E+03,.8873525E+03,.1025601E+04,.1164351E+04,.1767935E+04,&
     & .1551019E+04,.1334009E+04,.1117420E+04,.9004784E+03,.7475656E+03,&
     & .8852456E+03,.1023476E+04,.1162461E+04,.1730070E+04,.1517401E+04,&
     & .1305332E+04,.1093709E+04,.8814078E+03,.7453610E+03,.8830458E+03,&
     & .1021470E+04,.1160440E+04,.1695360E+04,.1487399E+04,.1279829E+04,&
     & .1072014E+04,.8642946E+03,.7431708E+03,.8808442E+03,.1019364E+04,&
     & .1158466E+04,.1664351E+04,.1460274E+04,.1256503E+04,.1052675E+04,&
     & .8488905E+03,.7412704E+03,.8787936E+03,.1017361E+04,.1156598E+04,&
     & .1692847E+04,.1485753E+04,.1277843E+04,.1070752E+04,.8629425E+03,&
     & .7489667E+03,.8873589E+03,.1026231E+04,.1165463E+04,.1655805E+04,&
     & .1452438E+04,.1250044E+04,.1047170E+04,.8442632E+03,.7470728E+03,&
     & .8854219E+03,.1024283E+04,.1163647E+04,.1622518E+04,.1423594E+04,&
     & .1225186E+04,.1026305E+04,.8276242E+03,.7450344E+03,.8833410E+03,&
     & .1022268E+04,.1161651E+04,.1592305E+04,.1397067E+04,.1202289E+04,&
     & .1007702E+04,.8128099E+03,.7429351E+03,.8811642E+03,.1020198E+04,&
     & .1159643E+04,.1565538E+04,.1374036E+04,.1182269E+04,.9906999E+03,&
     & .7993235E+03,.7408812E+03,.8791490E+03,.1018131E+04,.1157722E+04,&
     & .1595853E+04,.1400292E+04,.1205058E+04,.1009761E+04,.8142929E+03,&
     & .7485108E+03,.8875129E+03,.1026919E+04,.1166552E+04,.1562424E+04,&
     & .1371454E+04,.1180385E+04,.9891526E+03,.7980135E+03,.7468116E+03,&
     & .8857444E+03,.1025101E+04,.1164785E+04,.1533329E+04,.1345902E+04,&
     & .1157909E+04,.9706094E+03,.7833915E+03,.7449296E+03,.8838178E+03,&
     & .1023113E+04,.1162949E+04,.1506815E+04,.1322783E+04,.1138509E+04,&
     & .9542707E+03,.7700244E+03,.7429706E+03,.8817581E+03,.1021134E+04,&
     & .1160909E+04,.1483525E+04,.1302151E+04,.1120847E+04,.9395369E+03,&
     & .7583538E+03,.7409557E+03,.8796852E+03,.1019111E+04,.1158929E+04/

      data   absa(181:315, 6)   /                                       &
     & .1513129E+04,.1327990E+04,.1143143E+04,.9581972E+03,.7731973E+03,&
     & .7481396E+03,.8876647E+03,.1027513E+04,.1167635E+04,.1483937E+04,&
     & .1302350E+04,.1121247E+04,.9397374E+03,.7584554E+03,.7466326E+03,&
     & .8860686E+03,.1025860E+04,.1165939E+04,.1457483E+04,.1279624E+04,&
     & .1101680E+04,.9236394E+03,.7454131E+03,.7449496E+03,.8842965E+03,&
     & .1024016E+04,.1164125E+04,.1434740E+04,.1259742E+04,.1084277E+04,&
     & .9093323E+03,.7341125E+03,.7431059E+03,.8823611E+03,.1022090E+04,&
     & .1162195E+04,.1414268E+04,.1241399E+04,.1069062E+04,.8962430E+03,&
     & .7237532E+03,.7411783E+03,.8803301E+03,.1020088E+04,.1160046E+04,&
     & .1443413E+04,.1267263E+04,.1090769E+04,.9146514E+03,.7383327E+03,&
     & .7478733E+03,.8878478E+03,.1028105E+04,.1168673E+04,.1417362E+04,&
     & .1244305E+04,.1071260E+04,.8985200E+03,.7252711E+03,.7465473E+03,&
     & .8864251E+03,.1026618E+04,.1167048E+04,.1394536E+04,.1224485E+04,&
     & .1054035E+04,.8840393E+03,.7137145E+03,.7450404E+03,.8848148E+03,&
     & .1024911E+04,.1165266E+04,.1374225E+04,.1206403E+04,.1039030E+04,&
     & .8713103E+03,.7039093E+03,.7433577E+03,.8830245E+03,.1023032E+04,&
     & .1163474E+04,.1355994E+04,.1190773E+04,.1025482E+04,.8599459E+03,&
     & .6945499E+03,.7415048E+03,.8810884E+03,.1021122E+04,.1161496E+04,&
     & .1384460E+04,.1215767E+04,.1046795E+04,.8776721E+03,.7087541E+03,&
     & .7476361E+03,.8879985E+03,.1028620E+04,.1169529E+04,.1361124E+04,&
     & .1195393E+04,.1029345E+04,.8633440E+03,.6972452E+03,.7464861E+03,&
     & .8867535E+03,.1027287E+04,.1168092E+04,.1341125E+04,.1177698E+04,&
     & .1014028E+04,.8503722E+03,.6871520E+03,.7451498E+03,.8852979E+03,&
     & .1025745E+04,.1166403E+04,.1323047E+04,.1161999E+04,.1000760E+04,&
     & .8395248E+03,.6782983E+03,.7436254E+03,.8836587E+03,.1024005E+04,&
     & .1164622E+04,.1306967E+04,.1147930E+04,.9887562E+03,.8294106E+03,&
     & .6702789E+03,.7419326E+03,.8818404E+03,.1022008E+04,.1162734E+04/

      data   absa(316:450, 6)   /                                       &
     & .1334109E+04,.1171345E+04,.1008923E+04,.8461200E+03,.6835456E+03,&
     & .7474174E+03,.8881153E+03,.1029011E+04,.1170451E+04,.1313327E+04,&
     & .1153250E+04,.9930497E+03,.8333106E+03,.6731748E+03,.7464228E+03,&
     & .8870239E+03,.1027877E+04,.1168986E+04,.1295219E+04,.1137547E+04,&
     & .9799434E+03,.8219514E+03,.6643371E+03,.7452458E+03,.8857278E+03,&
     & .1026491E+04,.1167438E+04,.1279553E+04,.1123751E+04,.9678431E+03,&
     & .8123012E+03,.6562950E+03,.7438793E+03,.8842286E+03,.1024816E+04,&
     & .1165703E+04,.1265724E+04,.1111449E+04,.9576137E+03,.8031933E+03,&
     & .6494532E+03,.7423259E+03,.8825429E+03,.1023034E+04,.1163868E+04,&
     & .1290515E+04,.1133583E+04,.9764835E+03,.8192474E+03,.6620508E+03,&
     & .7472103E+03,.8882292E+03,.1029567E+04,.1171352E+04,.1272606E+04,&
     & .1117592E+04,.9626389E+03,.8077755E+03,.6527492E+03,.7463629E+03,&
     & .8872547E+03,.1028405E+04,.1169825E+04,.1256652E+04,.1103698E+04,&
     & .9508267E+03,.7976036E+03,.6448586E+03,.7453404E+03,.8861112E+03,&
     & .1027107E+04,.1168295E+04,.1242559E+04,.1091491E+04,.9403080E+03,&
     & .7891129E+03,.6378974E+03,.7441189E+03,.8847532E+03,.1025673E+04,&
     & .1166733E+04,.1230258E+04,.1080509E+04,.9310347E+03,.7814702E+03,&
     & .6317827E+03,.7427070E+03,.8832010E+03,.1023957E+04,.1164903E+04,&
     & .1252792E+04,.1100066E+04,.9479412E+03,.7952615E+03,.6429338E+03,&
     & .7470122E+03,.8882931E+03,.1029973E+04,.1172174E+04,.1236775E+04,&
     & .1086029E+04,.9357359E+03,.7854390E+03,.6347013E+03,.7462389E+03,&
     & .8873859E+03,.1028765E+04,.1170458E+04,.1222725E+04,.1073962E+04,&
     & .9252515E+03,.7763294E+03,.6276457E+03,.7453420E+03,.8863540E+03,&
     & .1027586E+04,.1169067E+04,.1210413E+04,.1063428E+04,.9161707E+03,&
     & .7689610E+03,.6219689E+03,.7442455E+03,.8851226E+03,.1026224E+04,&
     & .1167546E+04,.1199891E+04,.1053872E+04,.9082626E+03,.7623785E+03,&
     & .6166061E+03,.7429574E+03,.8836539E+03,.1024662E+04,.1165837E+04/

      data   absa(451:585, 6)   /                                       &
     & .1213435E+04,.1066122E+04,.9182536E+03,.7707858E+03,.6233804E+03,&
     & .7465268E+03,.8879869E+03,.1029819E+04,.1172194E+04,.1200202E+04,&
     & .1054292E+04,.9083498E+03,.7627042E+03,.6165912E+03,.7457906E+03,&
     & .8870951E+03,.1028665E+04,.1170524E+04,.1188659E+04,.1044342E+04,&
     & .8998639E+03,.7554328E+03,.6110074E+03,.7449213E+03,.8860867E+03,&
     & .1027451E+04,.1169072E+04,.1178858E+04,.1035658E+04,.8922892E+03,&
     & .7492634E+03,.6059606E+03,.7438596E+03,.8848729E+03,.1026124E+04,&
     & .1167566E+04,.1169796E+04,.1027749E+04,.8857341E+03,.7436862E+03,&
     & .6022412E+03,.7425912E+03,.8834588E+03,.1024571E+04,.1165837E+04,&
     & .1181101E+04,.1037633E+04,.8940681E+03,.7506851E+03,.6074386E+03,&
     & .7461265E+03,.8877353E+03,.1029711E+04,.1172179E+04,.1170091E+04,&
     & .1028027E+04,.8859634E+03,.7439636E+03,.6039202E+03,.7454203E+03,&
     & .8868566E+03,.1028531E+04,.1170527E+04,.1161160E+04,.1020142E+04,&
     & .8789356E+03,.7381596E+03,.6035305E+03,.7445721E+03,.8858628E+03,&
     & .1027386E+04,.1169067E+04,.1152818E+04,.1012820E+04,.8728840E+03,&
     & .7330614E+03,.6026851E+03,.7435297E+03,.8846310E+03,.1025991E+04,&
     & .1167566E+04,.1145364E+04,.1006499E+04,.8676452E+03,.7284487E+03,&
     & .6016648E+03,.7422863E+03,.8832630E+03,.1024470E+04,.1165837E+04,&
     & .1154769E+04,.1014599E+04,.8744908E+03,.7339035E+03,.6044416E+03,&
     & .7457969E+03,.8875272E+03,.1029611E+04,.1172169E+04,.1146093E+04,&
     & .1006648E+04,.8677377E+03,.7287407E+03,.6038007E+03,.7451183E+03,&
     & .8866614E+03,.1028431E+04,.1170524E+04,.1138431E+04,.1000012E+04,&
     & .8620420E+03,.7235398E+03,.6031357E+03,.7442908E+03,.8856774E+03,&
     & .1027286E+04,.1169074E+04,.1131504E+04,.9941179E+03,.8570033E+03,&
     & .7197199E+03,.6023129E+03,.7432633E+03,.8844627E+03,.1025923E+04,&
     & .1167566E+04,.1125720E+04,.9892480E+03,.8524798E+03,.7159207E+03,&
     & .6012892E+03,.7420375E+03,.8831033E+03,.1024380E+04,.1165837E+04/

!     the array absb(1175,6) (kb(5,5,13:59,6)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 6,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:175, 1)   /                                       &
     & .2736900E+02,.2990300E+03,.4292400E+03,.3103900E+03,.3500000E+02,&
     & .2715800E+02,.2989800E+03,.4274500E+03,.3084000E+03,.3500000E+02,&
     & .2697600E+02,.2988900E+03,.4259000E+03,.3066800E+03,.3500000E+02,&
     & .2681800E+02,.2987500E+03,.4245500E+03,.3051700E+03,.3500000E+02,&
     & .2667900E+02,.2985500E+03,.4233500E+03,.3038400E+03,.3500000E+02,&
     & .2684000E+02,.2986300E+03,.4247400E+03,.3053800E+03,.3500000E+02,&
     & .2666900E+02,.2986200E+03,.4232700E+03,.3037500E+03,.3500000E+02,&
     & .2652200E+02,.2985400E+03,.4219900E+03,.3023400E+03,.3500000E+02,&
     & .2639400E+02,.2984200E+03,.4208800E+03,.3011100E+03,.3500000E+02,&
     & .2628200E+02,.2982400E+03,.4199000E+03,.3000200E+03,.3500000E+02,&
     & .2640800E+02,.2983100E+03,.4210000E+03,.3012400E+03,.3500000E+02,&
     & .2627100E+02,.2983200E+03,.4198000E+03,.2999100E+03,.3500000E+02,&
     & .2615200E+02,.2982600E+03,.4187600E+03,.2987700E+03,.3500000E+02,&
     & .2604900E+02,.2981400E+03,.4178500E+03,.2977600E+03,.3500000E+02,&
     & .2595800E+02,.2979700E+03,.4170500E+03,.2968800E+03,.3500000E+02,&
     & .2605700E+02,.2980600E+03,.4179300E+03,.2978400E+03,.3500000E+02,&
     & .2594600E+02,.2980700E+03,.4169500E+03,.2967600E+03,.3500000E+02,&
     & .2585100E+02,.2980300E+03,.4161000E+03,.2958300E+03,.3500000E+02,&
     & .2576700E+02,.2979200E+03,.4153600E+03,.2950200E+03,.3500000E+02,&
     & .2569400E+02,.2977600E+03,.4147000E+03,.2943000E+03,.3500000E+02,&
     & .2577200E+02,.2978500E+03,.4154000E+03,.2950600E+03,.3500000E+02,&
     & .2568300E+02,.2978700E+03,.4146000E+03,.2941900E+03,.3500000E+02,&
     & .2560600E+02,.2978400E+03,.4139100E+03,.2934300E+03,.3500000E+02,&
     & .2553800E+02,.2977400E+03,.4133100E+03,.2927700E+03,.3500000E+02,&
     & .2547900E+02,.2975800E+03,.4127800E+03,.2921900E+03,.3500000E+02,&
     & .2554100E+02,.2976800E+03,.4133400E+03,.2928000E+03,.3500000E+02,&
     & .2546900E+02,.2977200E+03,.4126900E+03,.2920900E+03,.3500000E+02,&
     & .2540700E+02,.2976800E+03,.4121300E+03,.2914700E+03,.3500000E+02,&
     & .2535300E+02,.2975800E+03,.4116300E+03,.2909400E+03,.3500000E+02,&
     & .2530500E+02,.2974300E+03,.4112000E+03,.2904600E+03,.3500000E+02,&
     & .2535400E+02,.2975500E+03,.4116500E+03,.2909500E+03,.3500000E+02,&
     & .2529600E+02,.2975800E+03,.4111200E+03,.2903700E+03,.3500000E+02,&
     & .2524500E+02,.2975500E+03,.4106700E+03,.2898800E+03,.3500000E+02,&
     & .2520100E+02,.2974600E+03,.4102700E+03,.2894400E+03,.3500000E+02,&
     & .2516300E+02,.2973000E+03,.4099100E+03,.2890500E+03,.3500000E+02/

      data   absb(176:350, 1)   /                                       &
     & .2520100E+02,.2974400E+03,.4102600E+03,.2894300E+03,.3500000E+02,&
     & .2515400E+02,.2974800E+03,.4098400E+03,.2889700E+03,.3500000E+02,&
     & .2511300E+02,.2974500E+03,.4094700E+03,.2885600E+03,.3500000E+02,&
     & .2507800E+02,.2973500E+03,.4091400E+03,.2882100E+03,.3500000E+02,&
     & .2504700E+02,.2971900E+03,.4088600E+03,.2879000E+03,.3500000E+02,&
     & .2507600E+02,.2973600E+03,.4091300E+03,.2881900E+03,.3500000E+02,&
     & .2503900E+02,.2973900E+03,.4087900E+03,.2878200E+03,.3500000E+02,&
     & .2500600E+02,.2973600E+03,.4084900E+03,.2874900E+03,.3500000E+02,&
     & .2497800E+02,.2972600E+03,.4082300E+03,.2872200E+03,.3500000E+02,&
     & .2495200E+02,.2971000E+03,.4080000E+03,.2869500E+03,.3500000E+02,&
     & .2497300E+02,.2972900E+03,.4081900E+03,.2871700E+03,.3500000E+02,&
     & .2494400E+02,.2973200E+03,.4079200E+03,.2868700E+03,.3500000E+02,&
     & .2491700E+02,.2972800E+03,.4076800E+03,.2866100E+03,.3500000E+02,&
     & .2489500E+02,.2971700E+03,.4074700E+03,.2863800E+03,.3500000E+02,&
     & .2487400E+02,.2970000E+03,.4072800E+03,.2861800E+03,.3500000E+02,&
     & .2489000E+02,.2972400E+03,.4074300E+03,.2863400E+03,.3500000E+02,&
     & .2486600E+02,.2972600E+03,.4072100E+03,.2861000E+03,.3500000E+02,&
     & .2484600E+02,.2972100E+03,.4070200E+03,.2858900E+03,.3500000E+02,&
     & .2482700E+02,.2971000E+03,.4068500E+03,.2857100E+03,.3500000E+02,&
     & .2481100E+02,.2969200E+03,.4067100E+03,.2855400E+03,.3500000E+02,&
     & .2482300E+02,.2972100E+03,.4068100E+03,.2856600E+03,.3500000E+02,&
     & .2480400E+02,.2972100E+03,.4066400E+03,.2854700E+03,.3500000E+02,&
     & .2478700E+02,.2971500E+03,.4064900E+03,.2853000E+03,.3500000E+02,&
     & .2477300E+02,.2970300E+03,.4063500E+03,.2851600E+03,.3500000E+02,&
     & .2476000E+02,.2968400E+03,.4062300E+03,.2850300E+03,.3500000E+02,&
     & .2476900E+02,.2971700E+03,.4063100E+03,.2851100E+03,.3500000E+02,&
     & .2475300E+02,.2971700E+03,.4061800E+03,.2849600E+03,.3500000E+02,&
     & .2474000E+02,.2971000E+03,.4060600E+03,.2848300E+03,.3500000E+02,&
     & .2472900E+02,.2969600E+03,.4059500E+03,.2847200E+03,.3500000E+02,&
     & .2471800E+02,.2967600E+03,.4058500E+03,.2846200E+03,.3500000E+02,&
     & .2472400E+02,.2971500E+03,.4059100E+03,.2846700E+03,.3500000E+02,&
     & .2471200E+02,.2971300E+03,.4058000E+03,.2845600E+03,.3500000E+02,&
     & .2470200E+02,.2970400E+03,.4057000E+03,.2844500E+03,.3500000E+02,&
     & .2469300E+02,.2968900E+03,.4056200E+03,.2843500E+03,.3500000E+02,&
     & .2468400E+02,.2966800E+03,.4055400E+03,.2842700E+03,.3500000E+02/

      data   absb(351:525, 1)   /                                       &
     & .2468900E+02,.2971300E+03,.4055800E+03,.2843100E+03,.3500000E+02,&
     & .2467900E+02,.2970900E+03,.4055000E+03,.2842200E+03,.3500000E+02,&
     & .2467100E+02,.2969900E+03,.4054200E+03,.2842300E+03,.3500000E+02,&
     & .2466400E+02,.2968300E+03,.4053500E+03,.2840500E+03,.3500000E+02,&
     & .2465700E+02,.2966000E+03,.4052900E+03,.2839900E+03,.3500000E+02,&
     & .2466000E+02,.2971000E+03,.4053200E+03,.2840400E+03,.3500000E+02,&
     & .2465300E+02,.2970600E+03,.4052500E+03,.2839500E+03,.3500000E+02,&
     & .2464600E+02,.2969400E+03,.4051900E+03,.2838900E+03,.3500000E+02,&
     & .2464000E+02,.2967600E+03,.4051400E+03,.2838300E+03,.3500000E+02,&
     & .2463500E+02,.2965100E+03,.4050900E+03,.2837600E+03,.3500000E+02,&
     & .2463700E+02,.2970800E+03,.4051100E+03,.2838000E+03,.3500000E+02,&
     & .2463100E+02,.2970200E+03,.4050500E+03,.2837400E+03,.3500000E+02,&
     & .2462600E+02,.2968900E+03,.4050000E+03,.2836800E+03,.3500000E+02,&
     & .2462100E+02,.2966900E+03,.4049600E+03,.2836400E+03,.3500000E+02,&
     & .2461700E+02,.2964300E+03,.4049200E+03,.2835900E+03,.3500000E+02,&
     & .2461900E+02,.2970600E+03,.4049400E+03,.2836000E+03,.3500000E+02,&
     & .2461400E+02,.2969800E+03,.4048900E+03,.2835500E+03,.3500000E+02,&
     & .2461000E+02,.2968300E+03,.4048500E+03,.2835100E+03,.3500000E+02,&
     & .2460600E+02,.2966200E+03,.4048200E+03,.2834700E+03,.3500000E+02,&
     & .2460300E+02,.2963400E+03,.4047900E+03,.2834400E+03,.3500000E+02,&
     & .2460400E+02,.2970400E+03,.4048000E+03,.2834500E+03,.3500000E+02,&
     & .2460000E+02,.2969400E+03,.4047700E+03,.2834100E+03,.3500000E+02,&
     & .2459700E+02,.2967800E+03,.4047300E+03,.2833800E+03,.3500000E+02,&
     & .2459400E+02,.2965500E+03,.4047100E+03,.2833600E+03,.3500000E+02,&
     & .2459100E+02,.2962500E+03,.4046800E+03,.2833200E+03,.3500000E+02,&
     & .2459200E+02,.2970100E+03,.4046900E+03,.2833300E+03,.3500000E+02,&
     & .2458900E+02,.2968900E+03,.4046600E+03,.2833000E+03,.3500000E+02,&
     & .2458600E+02,.2967200E+03,.4046400E+03,.2832800E+03,.3500000E+02,&
     & .2458400E+02,.2964700E+03,.4046100E+03,.2832400E+03,.3500000E+02,&
     & .2458200E+02,.2961500E+03,.4045900E+03,.2832500E+03,.3500000E+02,&
     & .2458200E+02,.2969800E+03,.4046000E+03,.2832300E+03,.3500000E+02,&
     & .2458000E+02,.2968500E+03,.4045800E+03,.2832100E+03,.3500000E+02,&
     & .2457800E+02,.2966500E+03,.4045600E+03,.2831800E+03,.3500000E+02,&
     & .2457600E+02,.2963900E+03,.4045400E+03,.2831700E+03,.3500000E+02,&
     & .2457400E+02,.2960500E+03,.4045200E+03,.2831500E+03,.3500000E+02/

      data   absb(526:700, 1)   /                                       &
     & .2457400E+02,.2969500E+03,.4045300E+03,.2831500E+03,.3500000E+02,&
     & .2457200E+02,.2968000E+03,.4045100E+03,.2831300E+03,.3500000E+02,&
     & .2457100E+02,.2965900E+03,.4044900E+03,.2831200E+03,.3500000E+02,&
     & .2456900E+02,.2963100E+03,.4044800E+03,.2831000E+03,.3500000E+02,&
     & .2456800E+02,.2959600E+03,.4044700E+03,.2830800E+03,.3500000E+02,&
     & .2456800E+02,.2969300E+03,.4044700E+03,.2830900E+03,.3500000E+02,&
     & .2456700E+02,.2967800E+03,.4044600E+03,.2830800E+03,.3500000E+02,&
     & .2456500E+02,.2965600E+03,.4044400E+03,.2830600E+03,.3500000E+02,&
     & .2456400E+02,.2962700E+03,.4044300E+03,.2830500E+03,.3500000E+02,&
     & .2456300E+02,.2959000E+03,.4044200E+03,.2830400E+03,.3500000E+02,&
     & .2456300E+02,.2969200E+03,.4044300E+03,.2830500E+03,.3500000E+02,&
     & .2456200E+02,.2967700E+03,.4044200E+03,.2830400E+03,.3500000E+02,&
     & .2456100E+02,.2965500E+03,.4044000E+03,.2830100E+03,.3500000E+02,&
     & .2456000E+02,.2962600E+03,.4044000E+03,.2830200E+03,.3500000E+02,&
     & .2455900E+02,.2958900E+03,.4043900E+03,.2830100E+03,.3500000E+02,&
     & .2456000E+02,.2969300E+03,.4043900E+03,.2830100E+03,.3500000E+02,&
     & .2455900E+02,.2967900E+03,.4043800E+03,.2829900E+03,.3500000E+02,&
     & .2455800E+02,.2965800E+03,.4043700E+03,.2829900E+03,.3500000E+02,&
     & .2455700E+02,.2963000E+03,.4043700E+03,.2829700E+03,.3500000E+02,&
     & .2455600E+02,.2959400E+03,.4043600E+03,.2829700E+03,.3500000E+02,&
     & .2455700E+02,.2969500E+03,.4043600E+03,.2829700E+03,.3500000E+02,&
     & .2455600E+02,.2968100E+03,.4043600E+03,.2829600E+03,.3500000E+02,&
     & .2455500E+02,.2966100E+03,.4043500E+03,.2829700E+03,.3500000E+02,&
     & .2455400E+02,.2963400E+03,.4043400E+03,.2829600E+03,.3500000E+02,&
     & .2455400E+02,.2959900E+03,.4043400E+03,.2829400E+03,.3500000E+02,&
     & .2455400E+02,.2969600E+03,.4043400E+03,.2829500E+03,.3500000E+02,&
     & .2455300E+02,.2968300E+03,.4043300E+03,.2829400E+03,.3500000E+02,&
     & .2455300E+02,.2966400E+03,.4043300E+03,.2829300E+03,.3500000E+02,&
     & .2455200E+02,.2963800E+03,.4043200E+03,.2829200E+03,.3500000E+02,&
     & .2455200E+02,.2960400E+03,.4043200E+03,.2829200E+03,.3500000E+02,&
     & .2455200E+02,.2969800E+03,.4043200E+03,.2829200E+03,.3500000E+02,&
     & .2455100E+02,.2968700E+03,.4043200E+03,.2829300E+03,.3500000E+02,&
     & .2455100E+02,.2966900E+03,.4043100E+03,.2829200E+03,.3500000E+02,&
     & .2455000E+02,.2964500E+03,.4043100E+03,.2829100E+03,.3500000E+02,&
     & .2455000E+02,.2961300E+03,.4043000E+03,.2829100E+03,.3500000E+02/

      data   absb(701:875, 1)   /                                       &
     & .2455000E+02,.2969900E+03,.4043100E+03,.2829000E+03,.3500000E+02,&
     & .2455000E+02,.2969000E+03,.4043000E+03,.2829100E+03,.3500000E+02,&
     & .2454900E+02,.2967400E+03,.4043000E+03,.2828900E+03,.3500000E+02,&
     & .2454900E+02,.2965100E+03,.4042900E+03,.2829000E+03,.3500000E+02,&
     & .2454900E+02,.2962200E+03,.4042900E+03,.2828900E+03,.3500000E+02,&
     & .2454900E+02,.2970000E+03,.4042900E+03,.2828900E+03,.3500000E+02,&
     & .2454900E+02,.2969300E+03,.4042900E+03,.2828900E+03,.3500000E+02,&
     & .2454800E+02,.2967900E+03,.4042900E+03,.2828900E+03,.3500000E+02,&
     & .2454800E+02,.2965800E+03,.4042800E+03,.2828900E+03,.3500000E+02,&
     & .2454700E+02,.2963000E+03,.4042800E+03,.2828800E+03,.3500000E+02,&
     & .2454800E+02,.2970100E+03,.4042800E+03,.2828800E+03,.3500000E+02,&
     & .2454700E+02,.2969600E+03,.4042800E+03,.2828800E+03,.3500000E+02,&
     & .2454700E+02,.2968400E+03,.4042800E+03,.2828800E+03,.3500000E+02,&
     & .2454700E+02,.2966500E+03,.4042700E+03,.2828800E+03,.3500000E+02,&
     & .2454700E+02,.2963900E+03,.4042700E+03,.2828800E+03,.3500000E+02,&
     & .2454700E+02,.2970200E+03,.4042700E+03,.2828700E+03,.3500000E+02,&
     & .2454700E+02,.2969800E+03,.4042700E+03,.2828700E+03,.3500000E+02,&
     & .2454600E+02,.2968800E+03,.4042700E+03,.2828700E+03,.3500000E+02,&
     & .2454600E+02,.2967200E+03,.4042700E+03,.2828700E+03,.3500000E+02,&
     & .2454600E+02,.2964900E+03,.4042600E+03,.2828600E+03,.3500000E+02,&
     & .2454600E+02,.2970100E+03,.4042700E+03,.2828700E+03,.3500000E+02,&
     & .2454600E+02,.2970000E+03,.4042600E+03,.2828700E+03,.3500000E+02,&
     & .2454600E+02,.2969200E+03,.4042600E+03,.2828600E+03,.3500000E+02,&
     & .2454500E+02,.2967800E+03,.4042600E+03,.2828600E+03,.3500000E+02,&
     & .2454500E+02,.2965700E+03,.4042600E+03,.2828600E+03,.3500000E+02,&
     & .2454500E+02,.2969900E+03,.4042600E+03,.2828700E+03,.3500000E+02,&
     & .2454500E+02,.2970100E+03,.4042600E+03,.2828500E+03,.3500000E+02,&
     & .2454500E+02,.2969600E+03,.4042600E+03,.2828600E+03,.3500000E+02,&
     & .2454500E+02,.2968400E+03,.4042500E+03,.2828500E+03,.3500000E+02,&
     & .2454500E+02,.2966500E+03,.4042500E+03,.2828500E+03,.3500000E+02,&
     & .2454500E+02,.2969600E+03,.4042500E+03,.2828500E+03,.3500000E+02,&
     & .2454500E+02,.2970100E+03,.4042500E+03,.2828600E+03,.3500000E+02,&
     & .2454400E+02,.2969900E+03,.4042500E+03,.2828500E+03,.3500000E+02,&
     & .2454400E+02,.2969000E+03,.4042500E+03,.2828500E+03,.3500000E+02,&
     & .2454400E+02,.2967400E+03,.4042500E+03,.2828600E+03,.3500000E+02/

      data   absb(876:1050, 1)   /                                      &
     & .2454400E+02,.2969100E+03,.4042500E+03,.2828600E+03,.3500000E+02,&
     & .2454400E+02,.2970000E+03,.4042500E+03,.2828500E+03,.3500000E+02,&
     & .2454400E+02,.2970100E+03,.4042500E+03,.2828400E+03,.3500000E+02,&
     & .2454400E+02,.2969400E+03,.4042500E+03,.2828500E+03,.3500000E+02,&
     & .2454400E+02,.2968100E+03,.4042500E+03,.2828400E+03,.3500000E+02,&
     & .2454400E+02,.2968500E+03,.4042500E+03,.2828400E+03,.3500000E+02,&
     & .2454400E+02,.2969800E+03,.4042500E+03,.2828400E+03,.3500000E+02,&
     & .2454400E+02,.2970100E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454400E+02,.2969700E+03,.4042400E+03,.2828500E+03,.3500000E+02,&
     & .2454400E+02,.2968700E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454400E+02,.2967800E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454400E+02,.2969400E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2970100E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2970000E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2969200E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2967000E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2968900E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2969900E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2970100E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2969500E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2966000E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2968400E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2969700E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2970100E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2969800E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2964800E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2967600E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2969300E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2970100E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2970000E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2963500E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2966800E+03,.4042400E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2968800E+03,.4042400E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2969900E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2970100E+03,.4042300E+03,.2828500E+03,.3500000E+02/

      data   absb(1051:1175, 1)   /                                     &
     & .2454300E+02,.2962200E+03,.4042400E+03,.2828500E+03,.3500000E+02,&
     & .2454300E+02,.2965900E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2968300E+03,.4042300E+03,.2828400E+03,.3500000E+02,&
     & .2454300E+02,.2969600E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2970100E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2960700E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2964900E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2967600E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2969300E+03,.4042300E+03,.2828200E+03,.3500000E+02,&
     & .2454200E+02,.2970000E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454300E+02,.2959100E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2963700E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2966900E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2968900E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2969900E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2957400E+03,.4042300E+03,.2828200E+03,.3500000E+02,&
     & .2454200E+02,.2962400E+03,.4042300E+03,.2828200E+03,.3500000E+02,&
     & .2454200E+02,.2966000E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2968400E+03,.4042300E+03,.2828400E+03,.3500000E+02,&
     & .2454200E+02,.2969700E+03,.4042300E+03,.2828200E+03,.3500000E+02,&
     & .2454200E+02,.2956600E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2961900E+03,.4042300E+03,.2828200E+03,.3500000E+02,&
     & .2454200E+02,.2965700E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2968200E+03,.4042300E+03,.2828300E+03,.3500000E+02,&
     & .2454200E+02,.2969600E+03,.4042300E+03,.2828300E+03,.3500000E+02/

      data   absb(  1:175, 2)   /                                       &
     & .1017700E+03,.3276100E+03,.4557500E+03,.3133500E+03,.7183900E+02,&
     & .1009900E+03,.3271300E+03,.4538300E+03,.3112500E+03,.7183900E+02,&
     & .1003100E+03,.3266300E+03,.4521600E+03,.3094300E+03,.7183900E+02,&
     & .9972100E+02,.3261200E+03,.4507100E+03,.3078500E+03,.7183900E+02,&
     & .9920400E+02,.3255800E+03,.4494200E+03,.3064600E+03,.7183900E+02,&
     & .9980300E+02,.3261300E+03,.4509100E+03,.3080700E+03,.7183900E+02,&
     & .9916900E+02,.3257400E+03,.4493400E+03,.3063600E+03,.7183900E+02,&
     & .9862200E+02,.3253400E+03,.4479700E+03,.3048900E+03,.7183900E+02,&
     & .9814600E+02,.3249100E+03,.4467700E+03,.3036200E+03,.7183900E+02,&
     & .9772800E+02,.3244400E+03,.4457100E+03,.3024800E+03,.7183900E+02,&
     & .9819800E+02,.3249300E+03,.4469000E+03,.3037500E+03,.7183900E+02,&
     & .9768700E+02,.3246200E+03,.4456000E+03,.3023700E+03,.7183900E+02,&
     & .9724500E+02,.3242900E+03,.4444800E+03,.3011900E+03,.7183900E+02,&
     & .9686100E+02,.3239200E+03,.4435000E+03,.3001400E+03,.7183900E+02,&
     & .9652300E+02,.3235000E+03,.4426300E+03,.2992300E+03,.7183900E+02,&
     & .9689200E+02,.3239500E+03,.4435800E+03,.3002300E+03,.7183900E+02,&
     & .9648000E+02,.3237100E+03,.4425200E+03,.2991200E+03,.7183900E+02,&
     & .9612500E+02,.3234300E+03,.4416100E+03,.2981700E+03,.7183900E+02,&
     & .9581400E+02,.3231100E+03,.4408100E+03,.2973300E+03,.7183900E+02,&
     & .9554100E+02,.3227300E+03,.4401000E+03,.2966100E+03,.7183900E+02,&
     & .9583300E+02,.3231600E+03,.4408600E+03,.2973700E+03,.7183900E+02,&
     & .9550100E+02,.3229700E+03,.4400000E+03,.2964800E+03,.7183900E+02,&
     & .9521400E+02,.3227400E+03,.4392500E+03,.2956900E+03,.7183900E+02,&
     & .9496400E+02,.3224500E+03,.4386000E+03,.2950200E+03,.7183900E+02,&
     & .9474300E+02,.3221000E+03,.4380300E+03,.2944200E+03,.7183900E+02,&
     & .9497400E+02,.3225100E+03,.4386300E+03,.2950400E+03,.7183900E+02,&
     & .9470700E+02,.3223600E+03,.4379300E+03,.2943200E+03,.7183900E+02,&
     & .9447500E+02,.3221700E+03,.4373300E+03,.2936900E+03,.7183900E+02,&
     & .9427200E+02,.3219100E+03,.4368000E+03,.2931500E+03,.7183900E+02,&
     & .9409400E+02,.3215800E+03,.4363300E+03,.2926700E+03,.7183900E+02,&
     & .9427700E+02,.3219900E+03,.4368100E+03,.2931600E+03,.7183900E+02,&
     & .9406100E+02,.3218700E+03,.4362400E+03,.2925800E+03,.7183900E+02,&
     & .9387400E+02,.3217000E+03,.4357500E+03,.2920700E+03,.7183900E+02,&
     & .9371100E+02,.3214600E+03,.4353200E+03,.2916400E+03,.7183900E+02,&
     & .9356600E+02,.3211500E+03,.4349400E+03,.2912500E+03,.7183900E+02/

      data   absb(176:350, 2)   /                                       &
     & .9370800E+02,.3215700E+03,.4353100E+03,.2916300E+03,.7183900E+02,&
     & .9353400E+02,.3214700E+03,.4348500E+03,.2911700E+03,.7183900E+02,&
     & .9338300E+02,.3213200E+03,.4344500E+03,.2907600E+03,.7183900E+02,&
     & .9325100E+02,.3210900E+03,.4341100E+03,.2904100E+03,.7183900E+02,&
     & .9313500E+02,.3207900E+03,.4338000E+03,.2901000E+03,.7183900E+02,&
     & .9324600E+02,.3212200E+03,.4340900E+03,.2904000E+03,.7183900E+02,&
     & .9310600E+02,.3211400E+03,.4337200E+03,.2900300E+03,.7183900E+02,&
     & .9298500E+02,.3210000E+03,.4334000E+03,.2897000E+03,.7183900E+02,&
     & .9287800E+02,.3207900E+03,.4331100E+03,.2894100E+03,.7183900E+02,&
     & .9278400E+02,.3204900E+03,.4328600E+03,.2891700E+03,.7183900E+02,&
     & .9286300E+02,.3209400E+03,.4330700E+03,.2893800E+03,.7183900E+02,&
     & .9275100E+02,.3208700E+03,.4327800E+03,.2890800E+03,.7183900E+02,&
     & .9265500E+02,.3207300E+03,.4325200E+03,.2888200E+03,.7183900E+02,&
     & .9256900E+02,.3205100E+03,.4322900E+03,.2886000E+03,.7183900E+02,&
     & .9249400E+02,.3202200E+03,.4320900E+03,.2884000E+03,.7183900E+02,&
     & .9255300E+02,.3207200E+03,.4322500E+03,.2885500E+03,.7183900E+02,&
     & .9246500E+02,.3206400E+03,.4320100E+03,.2883200E+03,.7183900E+02,&
     & .9238700E+02,.3205000E+03,.4318000E+03,.2881100E+03,.7183900E+02,&
     & .9231900E+02,.3202900E+03,.4316200E+03,.2879300E+03,.7183900E+02,&
     & .9225900E+02,.3199800E+03,.4314600E+03,.2877800E+03,.7183900E+02,&
     & .9230300E+02,.3205300E+03,.4315800E+03,.2878900E+03,.7183900E+02,&
     & .9223300E+02,.3204600E+03,.4313900E+03,.2877000E+03,.7183900E+02,&
     & .9217100E+02,.3203100E+03,.4312200E+03,.2875400E+03,.7183900E+02,&
     & .9211700E+02,.3200900E+03,.4310800E+03,.2874000E+03,.7183900E+02,&
     & .9206800E+02,.3197800E+03,.4309500E+03,.2872700E+03,.7183900E+02,&
     & .9210100E+02,.3203800E+03,.4310300E+03,.2873600E+03,.7183900E+02,&
     & .9204500E+02,.3203000E+03,.4308800E+03,.2872100E+03,.7183900E+02,&
     & .9199600E+02,.3201500E+03,.4307500E+03,.2870800E+03,.7183900E+02,&
     & .9195200E+02,.3199100E+03,.4306400E+03,.2869600E+03,.7183900E+02,&
     & .9191400E+02,.3195900E+03,.4305300E+03,.2868400E+03,.7183900E+02,&
     & .9193700E+02,.3202500E+03,.4305900E+03,.2869300E+03,.7183900E+02,&
     & .9189200E+02,.3201600E+03,.4304700E+03,.2868000E+03,.7183900E+02,&
     & .9185300E+02,.3200000E+03,.4303700E+03,.2867000E+03,.7183900E+02,&
     & .9181900E+02,.3197500E+03,.4302800E+03,.2866100E+03,.7183900E+02,&
     & .9178800E+02,.3194200E+03,.4301900E+03,.2865300E+03,.7183900E+02/

      data   absb(351:525, 2)   /                                       &
     & .9180500E+02,.3201500E+03,.4302400E+03,.2865800E+03,.7183900E+02,&
     & .9177000E+02,.3200500E+03,.4301400E+03,.2864800E+03,.7183900E+02,&
     & .9173800E+02,.3198600E+03,.4300600E+03,.2863000E+03,.7183900E+02,&
     & .9171100E+02,.3196000E+03,.4299800E+03,.2863300E+03,.7183900E+02,&
     & .9168700E+02,.3192500E+03,.4299200E+03,.2862600E+03,.7183900E+02,&
     & .9169800E+02,.3200500E+03,.4299500E+03,.2862800E+03,.7183900E+02,&
     & .9167000E+02,.3199400E+03,.4298700E+03,.2862200E+03,.7183900E+02,&
     & .9164600E+02,.3197400E+03,.4298100E+03,.2861500E+03,.7183900E+02,&
     & .9162400E+02,.3194600E+03,.4297500E+03,.2860900E+03,.7183900E+02,&
     & .9160400E+02,.3191000E+03,.4297000E+03,.2860500E+03,.7183900E+02,&
     & .9161300E+02,.3199700E+03,.4297200E+03,.2860500E+03,.7183900E+02,&
     & .9159100E+02,.3198400E+03,.4296600E+03,.2860000E+03,.7183900E+02,&
     & .9157100E+02,.3196300E+03,.4296000E+03,.2859500E+03,.7183900E+02,&
     & .9155400E+02,.3193300E+03,.4295600E+03,.2859000E+03,.7183900E+02,&
     & .9153800E+02,.3189500E+03,.4295200E+03,.2858600E+03,.7183900E+02,&
     & .9154400E+02,.3199000E+03,.4295300E+03,.2858900E+03,.7183900E+02,&
     & .9152600E+02,.3197500E+03,.4294800E+03,.2858400E+03,.7183900E+02,&
     & .9151100E+02,.3195200E+03,.4294400E+03,.2858000E+03,.7183900E+02,&
     & .9149700E+02,.3192000E+03,.4294000E+03,.2857700E+03,.7183900E+02,&
     & .9148400E+02,.3188000E+03,.4293700E+03,.2857300E+03,.7183900E+02,&
     & .9148800E+02,.3198300E+03,.4293800E+03,.2857400E+03,.7183900E+02,&
     & .9147400E+02,.3196600E+03,.4293400E+03,.2857100E+03,.7183900E+02,&
     & .9146200E+02,.3194100E+03,.4293100E+03,.2856700E+03,.7183900E+02,&
     & .9145100E+02,.3190800E+03,.4292800E+03,.2856400E+03,.7183900E+02,&
     & .9144100E+02,.3186600E+03,.4292500E+03,.2856200E+03,.7183900E+02,&
     & .9144300E+02,.3197600E+03,.4292600E+03,.2856200E+03,.7183900E+02,&
     & .9143200E+02,.3195800E+03,.4292300E+03,.2855900E+03,.7183900E+02,&
     & .9142300E+02,.3193000E+03,.4292000E+03,.2855700E+03,.7183900E+02,&
     & .9141400E+02,.3189500E+03,.4291800E+03,.2855500E+03,.7183900E+02,&
     & .9140600E+02,.3185100E+03,.4291600E+03,.2855100E+03,.7183900E+02,&
     & .9140700E+02,.3197000E+03,.4291600E+03,.2855300E+03,.7183900E+02,&
     & .9139900E+02,.3194900E+03,.4291400E+03,.2855100E+03,.7183900E+02,&
     & .9139100E+02,.3192000E+03,.4291200E+03,.2854900E+03,.7183900E+02,&
     & .9138400E+02,.3188200E+03,.4291000E+03,.2854700E+03,.7183900E+02,&
     & .9137800E+02,.3183700E+03,.4290800E+03,.2854600E+03,.7183900E+02/

      data   absb(526:700, 2)   /                                       &
     & .9137900E+02,.3196400E+03,.4290800E+03,.2854600E+03,.7183900E+02,&
     & .9137200E+02,.3194100E+03,.4290600E+03,.2854400E+03,.7183900E+02,&
     & .9136500E+02,.3191000E+03,.4290500E+03,.2854200E+03,.7183900E+02,&
     & .9136000E+02,.3187100E+03,.4290300E+03,.2854100E+03,.7183900E+02,&
     & .9135500E+02,.3182400E+03,.4290200E+03,.2854000E+03,.7183900E+02,&
     & .9135600E+02,.3196000E+03,.4290200E+03,.2853900E+03,.7183900E+02,&
     & .9135000E+02,.3193600E+03,.4290100E+03,.2853800E+03,.7183900E+02,&
     & .9134500E+02,.3190400E+03,.4289900E+03,.2853700E+03,.7183900E+02,&
     & .9134100E+02,.3186300E+03,.4289800E+03,.2853500E+03,.7183900E+02,&
     & .9133700E+02,.3181600E+03,.4289700E+03,.2853500E+03,.7183900E+02,&
     & .9133800E+02,.3195800E+03,.4289700E+03,.2853500E+03,.7183900E+02,&
     & .9133400E+02,.3193400E+03,.4289600E+03,.2853300E+03,.7183900E+02,&
     & .9133000E+02,.3190200E+03,.4289500E+03,.2853300E+03,.7183900E+02,&
     & .9132600E+02,.3186200E+03,.4289400E+03,.2853000E+03,.7183900E+02,&
     & .9132300E+02,.3181400E+03,.4289300E+03,.2853000E+03,.7183900E+02,&
     & .9132400E+02,.3195900E+03,.4289400E+03,.2853100E+03,.7183900E+02,&
     & .9132000E+02,.3193700E+03,.4289300E+03,.2853000E+03,.7183900E+02,&
     & .9131700E+02,.3190600E+03,.4289200E+03,.2852900E+03,.7183900E+02,&
     & .9131400E+02,.3186600E+03,.4289100E+03,.2852900E+03,.7183900E+02,&
     & .9131100E+02,.3181900E+03,.4289000E+03,.2852700E+03,.7183900E+02,&
     & .9131300E+02,.3196100E+03,.4289000E+03,.2852800E+03,.7183900E+02,&
     & .9131000E+02,.3193900E+03,.4289000E+03,.2852800E+03,.7183900E+02,&
     & .9130700E+02,.3191000E+03,.4288900E+03,.2852600E+03,.7183900E+02,&
     & .9130400E+02,.3187100E+03,.4288800E+03,.2852500E+03,.7183900E+02,&
     & .9130200E+02,.3182600E+03,.4288800E+03,.2852500E+03,.7183900E+02,&
     & .9130300E+02,.3196200E+03,.4288800E+03,.2852500E+03,.7183900E+02,&
     & .9130100E+02,.3194200E+03,.4288700E+03,.2852500E+03,.7183900E+02,&
     & .9129800E+02,.3191400E+03,.4288600E+03,.2852500E+03,.7183900E+02,&
     & .9129600E+02,.3187700E+03,.4288600E+03,.2852400E+03,.7183900E+02,&
     & .9129400E+02,.3183200E+03,.4288500E+03,.2852300E+03,.7183900E+02,&
     & .9129600E+02,.3196500E+03,.4288600E+03,.2852400E+03,.7183900E+02,&
     & .9129300E+02,.3194700E+03,.4288500E+03,.2852200E+03,.7183900E+02,&
     & .9129200E+02,.3192100E+03,.4288500E+03,.2852200E+03,.7183900E+02,&
     & .9129000E+02,.3188600E+03,.4288400E+03,.2852200E+03,.7183900E+02,&
     & .9128800E+02,.3184300E+03,.4288400E+03,.2852100E+03,.7183900E+02/

      data   absb(701:875, 2)   /                                       &
     & .9128900E+02,.3196800E+03,.4288400E+03,.2852300E+03,.7183900E+02,&
     & .9128800E+02,.3195200E+03,.4288400E+03,.2852100E+03,.7183900E+02,&
     & .9128600E+02,.3192800E+03,.4288300E+03,.2852200E+03,.7183900E+02,&
     & .9128400E+02,.3189500E+03,.4288300E+03,.2852100E+03,.7183900E+02,&
     & .9128300E+02,.3185400E+03,.4288200E+03,.2852100E+03,.7183900E+02,&
     & .9128400E+02,.3197000E+03,.4288300E+03,.2852100E+03,.7183900E+02,&
     & .9128300E+02,.3195700E+03,.4288200E+03,.2852100E+03,.7183900E+02,&
     & .9128100E+02,.3193400E+03,.4288200E+03,.2852000E+03,.7183900E+02,&
     & .9128000E+02,.3190400E+03,.4288200E+03,.2851900E+03,.7183900E+02,&
     & .9127900E+02,.3186500E+03,.4288100E+03,.2852000E+03,.7183900E+02,&
     & .9128000E+02,.3197200E+03,.4288200E+03,.2852000E+03,.7183900E+02,&
     & .9127900E+02,.3196100E+03,.4288100E+03,.2852000E+03,.7183900E+02,&
     & .9127700E+02,.3194200E+03,.4288100E+03,.2851900E+03,.7183900E+02,&
     & .9127600E+02,.3191400E+03,.4288100E+03,.2851800E+03,.7183900E+02,&
     & .9127500E+02,.3187800E+03,.4288000E+03,.2851800E+03,.7183900E+02,&
     & .9127600E+02,.3197400E+03,.4288100E+03,.2851900E+03,.7183900E+02,&
     & .9127500E+02,.3196600E+03,.4288000E+03,.2851900E+03,.7183900E+02,&
     & .9127400E+02,.3194900E+03,.4288000E+03,.2851800E+03,.7183900E+02,&
     & .9127300E+02,.3192400E+03,.4288000E+03,.2851800E+03,.7183900E+02,&
     & .9127300E+02,.3189000E+03,.4288000E+03,.2851800E+03,.7183900E+02,&
     & .9127400E+02,.3197400E+03,.4288000E+03,.2851800E+03,.7183900E+02,&
     & .9127300E+02,.3196900E+03,.4288000E+03,.2851700E+03,.7183900E+02,&
     & .9127200E+02,.3195500E+03,.4287900E+03,.2851800E+03,.7183900E+02,&
     & .9127100E+02,.3193300E+03,.4287900E+03,.2851700E+03,.7183900E+02,&
     & .9127000E+02,.3190300E+03,.4287900E+03,.2851700E+03,.7183900E+02,&
     & .9127100E+02,.3197400E+03,.4287900E+03,.2851600E+03,.7183900E+02,&
     & .9127000E+02,.3197200E+03,.4287900E+03,.2851700E+03,.7183900E+02,&
     & .9127000E+02,.3196100E+03,.4287900E+03,.2851700E+03,.7183900E+02,&
     & .9126900E+02,.3194200E+03,.4287900E+03,.2851700E+03,.7183900E+02,&
     & .9126800E+02,.3191400E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126900E+02,.3197300E+03,.4287900E+03,.2851700E+03,.7183900E+02,&
     & .9126800E+02,.3197400E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126800E+02,.3196600E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126700E+02,.3195100E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126700E+02,.3192600E+03,.4287800E+03,.2851500E+03,.7183900E+02/

      data   absb(876:1050, 2)   /                                      &
     & .9126700E+02,.3197000E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126700E+02,.3197400E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126600E+02,.3197000E+03,.4287800E+03,.2851700E+03,.7183900E+02,&
     & .9126600E+02,.3195800E+03,.4287800E+03,.2851500E+03,.7183900E+02,&
     & .9126500E+02,.3193700E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126600E+02,.3196600E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126500E+02,.3197300E+03,.4287800E+03,.2851600E+03,.7183900E+02,&
     & .9126500E+02,.3197300E+03,.4287700E+03,.2851600E+03,.7183900E+02,&
     & .9126500E+02,.3196400E+03,.4287800E+03,.2851500E+03,.7183900E+02,&
     & .9126400E+02,.3194600E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126500E+02,.3196200E+03,.4287700E+03,.2851600E+03,.7183900E+02,&
     & .9126400E+02,.3197200E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126400E+02,.3197400E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126400E+02,.3196800E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126300E+02,.3195400E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126400E+02,.3195800E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126300E+02,.3196900E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126300E+02,.3197400E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126300E+02,.3197100E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126200E+02,.3196000E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126300E+02,.3195300E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126300E+02,.3196500E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126200E+02,.3197300E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126200E+02,.3197300E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126200E+02,.3196500E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126200E+02,.3194800E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126200E+02,.3196100E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126200E+02,.3197100E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126100E+02,.3197400E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126100E+02,.3196900E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126200E+02,.3194200E+03,.4287700E+03,.2851500E+03,.7183900E+02,&
     & .9126100E+02,.3195700E+03,.4287700E+03,.2851400E+03,.7183900E+02,&
     & .9126100E+02,.3196800E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126100E+02,.3197300E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126100E+02,.3197100E+03,.4287600E+03,.2851300E+03,.7183900E+02/

      data   absb(1051:1175, 2)   /                                     &
     & .9126100E+02,.3193700E+03,.4287600E+03,.2851300E+03,.7183900E+02,&
     & .9126100E+02,.3195300E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126100E+02,.3196500E+03,.4287600E+03,.2851300E+03,.7183900E+02,&
     & .9126100E+02,.3197300E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3197300E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126100E+02,.3193200E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3194700E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3196100E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3197100E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3197400E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3192700E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3194300E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3195700E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3196800E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3197400E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3192200E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3193800E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3195300E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3196500E+03,.4287600E+03,.2851300E+03,.7183900E+02,&
     & .9126000E+02,.3197300E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9126000E+02,.3192000E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9126000E+02,.3193600E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9125900E+02,.3195200E+03,.4287600E+03,.2851500E+03,.7183900E+02,&
     & .9125900E+02,.3196400E+03,.4287600E+03,.2851400E+03,.7183900E+02,&
     & .9125900E+02,.3197200E+03,.4287600E+03,.2851400E+03,.7183900E+02/

      data   absb(  1:175, 3)   /                                       &
     & .3836807E+03,.4141833E+03,.5172110E+03,.3640918E+03,.2774131E+03,&
     & .3807168E+03,.4119655E+03,.5153720E+03,.3628289E+03,.2774131E+03,&
     & .3781683E+03,.4100543E+03,.5137563E+03,.3617431E+03,.2774131E+03,&
     & .3759505E+03,.4084000E+03,.5123154E+03,.3608140E+03,.2774131E+03,&
     & .3739993E+03,.4069430E+03,.5110249E+03,.3599960E+03,.2774131E+03,&
     & .3762615E+03,.4086321E+03,.5121752E+03,.3609451E+03,.2774131E+03,&
     & .3738708E+03,.4068442E+03,.5106639E+03,.3599404E+03,.2774131E+03,&
     & .3718062E+03,.4053084E+03,.5093378E+03,.3590820E+03,.2774131E+03,&
     & .3700131E+03,.4039599E+03,.5081625E+03,.3583348E+03,.2774131E+03,&
     & .3684325E+03,.4027847E+03,.5070887E+03,.3576839E+03,.2774131E+03,&
     & .3702108E+03,.4041080E+03,.5079686E+03,.3584156E+03,.2774131E+03,&
     & .3682844E+03,.4026711E+03,.5067285E+03,.3576132E+03,.2774131E+03,&
     & .3666197E+03,.4014215E+03,.5056295E+03,.3569267E+03,.2774131E+03,&
     & .3651676E+03,.4003400E+03,.5046264E+03,.3563362E+03,.2774131E+03,&
     & .3638932E+03,.3993817E+03,.5037145E+03,.3558113E+03,.2774131E+03,&
     & .3652861E+03,.4004288E+03,.5043985E+03,.3563818E+03,.2774131E+03,&
     & .3637351E+03,.3992685E+03,.5033654E+03,.3557405E+03,.2774131E+03,&
     & .3623914E+03,.3982658E+03,.5024332E+03,.3551856E+03,.2774131E+03,&
     & .3612211E+03,.3973916E+03,.5015961E+03,.3547114E+03,.2774131E+03,&
     & .3601936E+03,.3966211E+03,.5008297E+03,.3542768E+03,.2774131E+03,&
     & .3612951E+03,.3974409E+03,.5013689E+03,.3547466E+03,.2774131E+03,&
     & .3600403E+03,.3965075E+03,.5005214E+03,.3542320E+03,.2774131E+03,&
     & .3589636E+03,.3957025E+03,.4997495E+03,.3537982E+03,.2774131E+03,&
     & .3580154E+03,.3949965E+03,.4990576E+03,.3534148E+03,.2774131E+03,&
     & .3571805E+03,.3943741E+03,.4984268E+03,.3530769E+03,.2774131E+03,&
     & .3580546E+03,.3950209E+03,.4988311E+03,.3534300E+03,.2774131E+03,&
     & .3570472E+03,.3942704E+03,.4981389E+03,.3530214E+03,.2774131E+03,&
     & .3561678E+03,.3936184E+03,.4975118E+03,.3526631E+03,.2774131E+03,&
     & .3554121E+03,.3930456E+03,.4969402E+03,.3523553E+03,.2774131E+03,&
     & .3547353E+03,.3925469E+03,.4964247E+03,.3520830E+03,.2774131E+03,&
     & .3554270E+03,.3930604E+03,.4967238E+03,.3523605E+03,.2774131E+03,&
     & .3546120E+03,.3924480E+03,.4961563E+03,.3520326E+03,.2774131E+03,&
     & .3539056E+03,.3919245E+03,.4956492E+03,.3517500E+03,.2774131E+03,&
     & .3532932E+03,.3914654E+03,.4951929E+03,.3514977E+03,.2774131E+03,&
     & .3527448E+03,.3910603E+03,.4947773E+03,.3512758E+03,.2774131E+03/

      data   absb(176:350, 3)   /                                       &
     & .3532784E+03,.3914554E+03,.4949665E+03,.3514929E+03,.2774131E+03,&
     & .3526264E+03,.3909667E+03,.4945186E+03,.3512254E+03,.2774131E+03,&
     & .3520584E+03,.3905468E+03,.4941167E+03,.3509983E+03,.2774131E+03,&
     & .3515593E+03,.3901714E+03,.4937504E+03,.3507916E+03,.2774131E+03,&
     & .3511198E+03,.3898455E+03,.4934100E+03,.3506149E+03,.2774131E+03,&
     & .3515397E+03,.3901565E+03,.4935235E+03,.3507864E+03,.2774131E+03,&
     & .3510113E+03,.3897615E+03,.4931757E+03,.3505745E+03,.2774131E+03,&
     & .3505518E+03,.3894209E+03,.4928538E+03,.3503830E+03,.2774131E+03,&
     & .3501516E+03,.3891195E+03,.4925527E+03,.3502215E+03,.2774131E+03,&
     & .3498009E+03,.3888529E+03,.4922746E+03,.3500751E+03,.2774131E+03,&
     & .3500923E+03,.3890750E+03,.4923299E+03,.3501963E+03,.2774131E+03,&
     & .3496725E+03,.3887640E+03,.4920480E+03,.3500247E+03,.2774131E+03,&
     & .3493070E+03,.3884875E+03,.4917957E+03,.3498784E+03,.2774131E+03,&
     & .3489908E+03,.3882505E+03,.4915549E+03,.3497473E+03,.2774131E+03,&
     & .3487043E+03,.3880380E+03,.4913246E+03,.3496361E+03,.2774131E+03,&
     & .3489268E+03,.3882061E+03,.4913510E+03,.3497221E+03,.2774131E+03,&
     & .3485958E+03,.3879591E+03,.4911335E+03,.3495858E+03,.2774131E+03,&
     & .3483044E+03,.3877418E+03,.4909320E+03,.3494698E+03,.2774131E+03,&
     & .3480474E+03,.3875440E+03,.4907313E+03,.3493639E+03,.2774131E+03,&
     & .3478153E+03,.3873763E+03,.4905513E+03,.3492731E+03,.2774131E+03,&
     & .3479882E+03,.3874996E+03,.4905518E+03,.3493387E+03,.2774131E+03,&
     & .3477164E+03,.3873023E+03,.4903795E+03,.3492327E+03,.2774131E+03,&
     & .3474843E+03,.3871294E+03,.4902184E+03,.3491368E+03,.2774131E+03,&
     & .3472817E+03,.3869713E+03,.4900628E+03,.3490560E+03,.2774131E+03,&
     & .3470992E+03,.3868380E+03,.4899176E+03,.3489804E+03,.2774131E+03,&
     & .3472225E+03,.3869268E+03,.4898977E+03,.3490308E+03,.2774131E+03,&
     & .3470103E+03,.3867739E+03,.4897703E+03,.3489501E+03,.2774131E+03,&
     & .3468274E+03,.3866354E+03,.4896391E+03,.3488745E+03,.2774131E+03,&
     & .3466645E+03,.3865122E+03,.4895135E+03,.3488037E+03,.2774131E+03,&
     & .3465212E+03,.3864033E+03,.4893984E+03,.3487433E+03,.2774131E+03,&
     & .3466053E+03,.3864725E+03,.4893673E+03,.3487785E+03,.2774131E+03,&
     & .3464372E+03,.3863441E+03,.4892706E+03,.3487130E+03,.2774131E+03,&
     & .3462891E+03,.3862356E+03,.4891699E+03,.3486526E+03,.2774131E+03,&
     & .3461606E+03,.3861367E+03,.4890743E+03,.3485970E+03,.2774131E+03,&
     & .3460421E+03,.3860479E+03,.4889791E+03,.3485566E+03,.2774131E+03/

      data   absb(351:525, 3)   /                                       &
     & .3461062E+03,.3860971E+03,.4889425E+03,.3485818E+03,.2774131E+03,&
     & .3459729E+03,.3859934E+03,.4888662E+03,.3485263E+03,.2774131E+03,&
     & .3458544E+03,.3859146E+03,.4887903E+03,.3484759E+03,.2774131E+03,&
     & .3457556E+03,.3858305E+03,.4887099E+03,.3484355E+03,.2774131E+03,&
     & .3456567E+03,.3857613E+03,.4886347E+03,.3484003E+03,.2774131E+03,&
     & .3457063E+03,.3858009E+03,.4885977E+03,.3484155E+03,.2774131E+03,&
     & .3455975E+03,.3857169E+03,.4885414E+03,.3483751E+03,.2774131E+03,&
     & .3455086E+03,.3856528E+03,.4884806E+03,.3483347E+03,.2774131E+03,&
     & .3454245E+03,.3855836E+03,.4884203E+03,.3483044E+03,.2774131E+03,&
     & .3453505E+03,.3855291E+03,.4883603E+03,.3482740E+03,.2774131E+03,&
     & .3453801E+03,.3855539E+03,.4883177E+03,.3482844E+03,.2774131E+03,&
     & .3452961E+03,.3854947E+03,.4882766E+03,.3482540E+03,.2774131E+03,&
     & .3452220E+03,.3854355E+03,.4882358E+03,.3482236E+03,.2774131E+03,&
     & .3451580E+03,.3853910E+03,.4881858E+03,.3481984E+03,.2774131E+03,&
     & .3450987E+03,.3853414E+03,.4881306E+03,.3481732E+03,.2774131E+03,&
     & .3451183E+03,.3853614E+03,.4880977E+03,.3481784E+03,.2774131E+03,&
     & .3450543E+03,.3853170E+03,.4880718E+03,.3481480E+03,.2774131E+03,&
     & .3449999E+03,.3852674E+03,.4880362E+03,.3481328E+03,.2774131E+03,&
     & .3449454E+03,.3852281E+03,.4879958E+03,.3481076E+03,.2774131E+03,&
     & .3448962E+03,.3851933E+03,.4879510E+03,.3480925E+03,.2774131E+03,&
     & .3449110E+03,.3852033E+03,.4879177E+03,.3480925E+03,.2774131E+03,&
     & .3448566E+03,.3851637E+03,.4878970E+03,.3480725E+03,.2774131E+03,&
     & .3448121E+03,.3851293E+03,.4878714E+03,.3480521E+03,.2774131E+03,&
     & .3447725E+03,.3850996E+03,.4878362E+03,.3480369E+03,.2774131E+03,&
     & .3447381E+03,.3850700E+03,.4878014E+03,.3480269E+03,.2774131E+03,&
     & .3447429E+03,.3850800E+03,.4877729E+03,.3480269E+03,.2774131E+03,&
     & .3446985E+03,.3850504E+03,.4877618E+03,.3480117E+03,.2774131E+03,&
     & .3446640E+03,.3850156E+03,.4877414E+03,.3479917E+03,.2774131E+03,&
     & .3446344E+03,.3849960E+03,.4877114E+03,.3479765E+03,.2774131E+03,&
     & .3445996E+03,.3849712E+03,.4876818E+03,.3479665E+03,.2774131E+03,&
     & .3446096E+03,.3849812E+03,.4876577E+03,.3479713E+03,.2774131E+03,&
     & .3445752E+03,.3849515E+03,.4876518E+03,.3479613E+03,.2774131E+03,&
     & .3445456E+03,.3849267E+03,.4876366E+03,.3479461E+03,.2774131E+03,&
     & .3445208E+03,.3849119E+03,.4876118E+03,.3479361E+03,.2774131E+03,&
     & .3444960E+03,.3848871E+03,.4875822E+03,.3479261E+03,.2774131E+03/

      data   absb(526:700, 3)   /                                       &
     & .3444960E+03,.3848971E+03,.4875677E+03,.3479261E+03,.2774131E+03,&
     & .3444763E+03,.3848775E+03,.4875670E+03,.3479209E+03,.2774131E+03,&
     & .3444515E+03,.3848575E+03,.4875518E+03,.3479057E+03,.2774131E+03,&
     & .3444319E+03,.3848427E+03,.4875270E+03,.3478957E+03,.2774131E+03,&
     & .3444071E+03,.3848279E+03,.4874973E+03,.3478906E+03,.2774131E+03,&
     & .3444171E+03,.3848331E+03,.4874977E+03,.3478957E+03,.2774131E+03,&
     & .3443923E+03,.3848183E+03,.4874922E+03,.3478857E+03,.2774131E+03,&
     & .3443775E+03,.3847983E+03,.4874818E+03,.3478757E+03,.2774131E+03,&
     & .3443579E+03,.3847883E+03,.4874622E+03,.3478706E+03,.2774131E+03,&
     & .3443430E+03,.3847734E+03,.4874377E+03,.3478606E+03,.2774131E+03,&
     & .3443479E+03,.3847834E+03,.4874377E+03,.3478654E+03,.2774131E+03,&
     & .3443330E+03,.3847686E+03,.4874373E+03,.3478554E+03,.2774131E+03,&
     & .3443182E+03,.3847538E+03,.4874270E+03,.3478554E+03,.2774131E+03,&
     & .3443034E+03,.3847438E+03,.4874122E+03,.3478454E+03,.2774131E+03,&
     & .3442886E+03,.3847390E+03,.4873877E+03,.3478402E+03,.2774131E+03,&
     & .3442934E+03,.3847442E+03,.4873881E+03,.3478454E+03,.2774131E+03,&
     & .3442786E+03,.3847342E+03,.4873873E+03,.3478402E+03,.2774131E+03,&
     & .3442638E+03,.3847242E+03,.4873870E+03,.3478302E+03,.2774131E+03,&
     & .3442590E+03,.3847094E+03,.4873722E+03,.3478302E+03,.2774131E+03,&
     & .3442442E+03,.3846994E+03,.4873477E+03,.3478250E+03,.2774131E+03,&
     & .3442490E+03,.3847146E+03,.4873433E+03,.3478250E+03,.2774131E+03,&
     & .3442390E+03,.3847046E+03,.4873525E+03,.3478202E+03,.2774131E+03,&
     & .3442294E+03,.3846946E+03,.4873522E+03,.3478150E+03,.2774131E+03,&
     & .3442194E+03,.3846846E+03,.4873422E+03,.3478150E+03,.2774131E+03,&
     & .3442094E+03,.3846798E+03,.4873177E+03,.3478050E+03,.2774131E+03,&
     & .3442146E+03,.3846850E+03,.4873085E+03,.3478150E+03,.2774131E+03,&
     & .3442046E+03,.3846750E+03,.4873225E+03,.3478050E+03,.2774131E+03,&
     & .3441946E+03,.3846698E+03,.4873222E+03,.3478050E+03,.2774131E+03,&
     & .3441898E+03,.3846650E+03,.4873173E+03,.3477998E+03,.2774131E+03,&
     & .3441798E+03,.3846550E+03,.4872977E+03,.3477998E+03,.2774131E+03,&
     & .3441898E+03,.3846650E+03,.4872737E+03,.3477998E+03,.2774131E+03,&
     & .3441749E+03,.3846602E+03,.4872929E+03,.3477950E+03,.2774131E+03,&
     & .3441701E+03,.3846502E+03,.4872973E+03,.3477898E+03,.2774131E+03,&
     & .3441649E+03,.3846453E+03,.4872922E+03,.3477898E+03,.2774131E+03,&
     & .3441601E+03,.3846402E+03,.4872825E+03,.3477898E+03,.2774131E+03/

      data   absb(701:875, 3)   /                                       &
     & .3441601E+03,.3846453E+03,.4872441E+03,.3477898E+03,.2774131E+03,&
     & .3441553E+03,.3846453E+03,.4872681E+03,.3477898E+03,.2774131E+03,&
     & .3441501E+03,.3846353E+03,.4872773E+03,.3477846E+03,.2774131E+03,&
     & .3441453E+03,.3846305E+03,.4872773E+03,.3477798E+03,.2774131E+03,&
     & .3441405E+03,.3846253E+03,.4872673E+03,.3477798E+03,.2774131E+03,&
     & .3441453E+03,.3846305E+03,.4872244E+03,.3477798E+03,.2774131E+03,&
     & .3441353E+03,.3846305E+03,.4872433E+03,.3477798E+03,.2774131E+03,&
     & .3441305E+03,.3846205E+03,.4872577E+03,.3477798E+03,.2774131E+03,&
     & .3441305E+03,.3846205E+03,.4872622E+03,.3477746E+03,.2774131E+03,&
     & .3441205E+03,.3846105E+03,.4872573E+03,.3477746E+03,.2774131E+03,&
     & .3441305E+03,.3846205E+03,.4871948E+03,.3477746E+03,.2774131E+03,&
     & .3441205E+03,.3846157E+03,.4872237E+03,.3477746E+03,.2774131E+03,&
     & .3441157E+03,.3846157E+03,.4872377E+03,.3477746E+03,.2774131E+03,&
     & .3441157E+03,.3846057E+03,.4872473E+03,.3477698E+03,.2774131E+03,&
     & .3441109E+03,.3846057E+03,.4872473E+03,.3477646E+03,.2774131E+03,&
     & .3441157E+03,.3846109E+03,.4871704E+03,.3477698E+03,.2774131E+03,&
     & .3441109E+03,.3846057E+03,.4872041E+03,.3477646E+03,.2774131E+03,&
     & .3441057E+03,.3846057E+03,.4872229E+03,.3477646E+03,.2774131E+03,&
     & .3441009E+03,.3846009E+03,.4872325E+03,.3477646E+03,.2774131E+03,&
     & .3441009E+03,.3845957E+03,.4872373E+03,.3477646E+03,.2774131E+03,&
     & .3441009E+03,.3846009E+03,.4871412E+03,.3477646E+03,.2774131E+03,&
     & .3441009E+03,.3846009E+03,.4871844E+03,.3477646E+03,.2774131E+03,&
     & .3440961E+03,.3846009E+03,.4872085E+03,.3477646E+03,.2774131E+03,&
     & .3440909E+03,.3845909E+03,.4872225E+03,.3477646E+03,.2774131E+03,&
     & .3440909E+03,.3845909E+03,.4872273E+03,.3477646E+03,.2774131E+03,&
     & .3440909E+03,.3845961E+03,.4871167E+03,.3477646E+03,.2774131E+03,&
     & .3440909E+03,.3845909E+03,.4871600E+03,.3477646E+03,.2774131E+03,&
     & .3440861E+03,.3845909E+03,.4871937E+03,.3477594E+03,.2774131E+03,&
     & .3440861E+03,.3845861E+03,.4872129E+03,.3477594E+03,.2774131E+03,&
     & .3440861E+03,.3845861E+03,.4872225E+03,.3477594E+03,.2774131E+03,&
     & .3440861E+03,.3845961E+03,.4870879E+03,.3477594E+03,.2774131E+03,&
     & .3440861E+03,.3845861E+03,.4871408E+03,.3477594E+03,.2774131E+03,&
     & .3440813E+03,.3845861E+03,.4871741E+03,.3477546E+03,.2774131E+03,&
     & .3440761E+03,.3845861E+03,.4871981E+03,.3477546E+03,.2774131E+03,&
     & .3440761E+03,.3845809E+03,.4872125E+03,.3477546E+03,.2774131E+03/

      data   absb(876:1050, 3)   /                                      &
     & .3440761E+03,.3845913E+03,.4870490E+03,.3477546E+03,.2774131E+03,&
     & .3440761E+03,.3845861E+03,.4871115E+03,.3477546E+03,.2774131E+03,&
     & .3440761E+03,.3845813E+03,.4871548E+03,.3477546E+03,.2774131E+03,&
     & .3440713E+03,.3845813E+03,.4871837E+03,.3477546E+03,.2774131E+03,&
     & .3440713E+03,.3845761E+03,.4872029E+03,.3477546E+03,.2774131E+03,&
     & .3440761E+03,.3845865E+03,.4870153E+03,.3477546E+03,.2774131E+03,&
     & .3440713E+03,.3845813E+03,.4870875E+03,.3477546E+03,.2774131E+03,&
     & .3440713E+03,.3845761E+03,.4871352E+03,.3477546E+03,.2774131E+03,&
     & .3440713E+03,.3845761E+03,.4871689E+03,.3477494E+03,.2774131E+03,&
     & .3440713E+03,.3845761E+03,.4871929E+03,.3477494E+03,.2774131E+03,&
     & .3440713E+03,.3845917E+03,.4869717E+03,.3477494E+03,.2774131E+03,&
     & .3440713E+03,.3845813E+03,.4870582E+03,.3477494E+03,.2774131E+03,&
     & .3440713E+03,.3845761E+03,.4871160E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845713E+03,.4871544E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845713E+03,.4871833E+03,.3477494E+03,.2774131E+03,&
     & .3440665E+03,.3845917E+03,.4869380E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845765E+03,.4870246E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845713E+03,.4870919E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845713E+03,.4871400E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845713E+03,.4871737E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845869E+03,.4868891E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845817E+03,.4869957E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845713E+03,.4870727E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845661E+03,.4871256E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845713E+03,.4871541E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845921E+03,.4868455E+03,.3477494E+03,.2774131E+03,&
     & .3440613E+03,.3845817E+03,.4869569E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845765E+03,.4870386E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845713E+03,.4871012E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845713E+03,.4871396E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845921E+03,.4867966E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845869E+03,.4869180E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845765E+03,.4870146E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845665E+03,.4870819E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845613E+03,.4871300E+03,.3477494E+03,.2774131E+03/

      data   absb(1051:1175, 3)   /                                     &
     & .3440565E+03,.3845972E+03,.4867477E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845821E+03,.4868843E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845717E+03,.4869857E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845665E+03,.4870627E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845613E+03,.4871156E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845976E+03,.4866985E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845872E+03,.4868403E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845769E+03,.4869520E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845665E+03,.4870386E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845665E+03,.4871012E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3846028E+03,.4866393E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845872E+03,.4868014E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845769E+03,.4869180E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3845717E+03,.4870146E+03,.3477494E+03,.2774131E+03,&
     & .3440465E+03,.3845613E+03,.4870819E+03,.3477494E+03,.2774131E+03,&
     & .3440565E+03,.3846028E+03,.4865852E+03,.3477494E+03,.2774131E+03,&
     & .3440513E+03,.3845924E+03,.4867526E+03,.3477494E+03,.2774131E+03,&
     & .3440465E+03,.3845821E+03,.4868839E+03,.3477494E+03,.2774131E+03,&
     & .3440465E+03,.3845717E+03,.4869857E+03,.3477494E+03,.2774131E+03,&
     & .3440465E+03,.3845665E+03,.4870627E+03,.3477442E+03,.2774131E+03,&
     & .3440465E+03,.3846080E+03,.4865652E+03,.3477494E+03,.2774131E+03,&
     & .3440465E+03,.3845924E+03,.4867329E+03,.3477494E+03,.2774131E+03,&
     & .3440465E+03,.3845821E+03,.4868695E+03,.3477442E+03,.2774131E+03,&
     & .3440465E+03,.3845717E+03,.4869761E+03,.3477442E+03,.2774131E+03,&
     & .3440465E+03,.3845665E+03,.4870531E+03,.3477442E+03,.2774131E+03/

      data   absb(  1:175, 4)   /                                       &
     & .8782833E+03,.6910354E+03,.5724886E+03,.6464156E+03,.7993949E+03,&
     & .8715031E+03,.6859459E+03,.5705842E+03,.6464442E+03,.7999135E+03,&
     & .8656602E+03,.6815696E+03,.5686553E+03,.6463947E+03,.8002608E+03,&
     & .8605853E+03,.6777583E+03,.5667574E+03,.6462801E+03,.8004368E+03,&
     & .8561252E+03,.6744132E+03,.5649105E+03,.6460944E+03,.8004368E+03,&
     & .8612940E+03,.6782938E+03,.5669216E+03,.6455568E+03,.7994505E+03,&
     & .8558228E+03,.6741854E+03,.5649339E+03,.6456484E+03,.7999506E+03,&
     & .8511049E+03,.6706518E+03,.5630723E+03,.6456464E+03,.8002840E+03,&
     & .8469964E+03,.6675691E+03,.5613310E+03,.6455686E+03,.8004461E+03,&
     & .8433882E+03,.6648582E+03,.5596704E+03,.6454205E+03,.8004322E+03,&
     & .8474420E+03,.6679062E+03,.5613362E+03,.6448724E+03,.7995061E+03,&
     & .8430265E+03,.6645957E+03,.5595545E+03,.6450016E+03,.7999923E+03,&
     & .8392252E+03,.6617362E+03,.5578650E+03,.6450418E+03,.8003071E+03,&
     & .8359047E+03,.6592484E+03,.5563042E+03,.6449917E+03,.8004461E+03,&
     & .8329859E+03,.6570576E+03,.5548590E+03,.6448650E+03,.8004183E+03,&
     & .8361725E+03,.6594516E+03,.5562448E+03,.6443217E+03,.7995709E+03,&
     & .8326189E+03,.6567852E+03,.5546746E+03,.6444831E+03,.8000339E+03,&
     & .8295516E+03,.6544805E+03,.5532717E+03,.6445409E+03,.8003349E+03,&
     & .8268752E+03,.6524736E+03,.5519977E+03,.6445269E+03,.8004507E+03,&
     & .8245113E+03,.6507091E+03,.5508204E+03,.6444124E+03,.8004090E+03,&
     & .8270338E+03,.6525975E+03,.5518584E+03,.6438793E+03,.7996218E+03,&
     & .8241743E+03,.6504514E+03,.5505821E+03,.6440683E+03,.8000756E+03,&
     & .8216964E+03,.6485930E+03,.5494126E+03,.6441476E+03,.8003534E+03,&
     & .8195356E+03,.6469724E+03,.5483183E+03,.6441412E+03,.8004600E+03,&
     & .8176273E+03,.6455403E+03,.5473063E+03,.6440475E+03,.8003951E+03,&
     & .8196249E+03,.6470370E+03,.5481983E+03,.6435253E+03,.7996774E+03,&
     & .8173102E+03,.6453072E+03,.5470865E+03,.6437357E+03,.8001127E+03,&
     & .8153180E+03,.6438105E+03,.5460498E+03,.6438219E+03,.8003720E+03,&
     & .8135635E+03,.6424969E+03,.5451238E+03,.6438316E+03,.8004553E+03,&
     & .8120275E+03,.6413426E+03,.5443103E+03,.6437493E+03,.8003812E+03,&
     & .8136081E+03,.6425269E+03,.5450518E+03,.6432481E+03,.7997283E+03,&
     & .8117397E+03,.6411295E+03,.5440951E+03,.6434654E+03,.8001451E+03,&
     & .8101291E+03,.6399152E+03,.5434568E+03,.6435677E+03,.8003905E+03,&
     & .8087217E+03,.6388594E+03,.5425558E+03,.6435788E+03,.8004553E+03,&
     & .8074728E+03,.6379229E+03,.5418598E+03,.6434981E+03,.8003720E+03/

      data   absb(176:350, 4)   /                                       &
     & .8086917E+03,.6388394E+03,.5425140E+03,.6430324E+03,.7997885E+03,&
     & .8071950E+03,.6377198E+03,.5416354E+03,.6432465E+03,.8001867E+03,&
     & .8058968E+03,.6367433E+03,.5409750E+03,.6433603E+03,.8004044E+03,&
     & .8047572E+03,.6358860E+03,.5404031E+03,.6433729E+03,.8004507E+03,&
     & .8037507E+03,.6351327E+03,.5398890E+03,.6432929E+03,.8003442E+03,&
     & .8047072E+03,.6358514E+03,.5402679E+03,.6428628E+03,.7998441E+03,&
     & .8034982E+03,.6349442E+03,.5396729E+03,.6430884E+03,.8002192E+03,&
     & .8024525E+03,.6341615E+03,.5391603E+03,.6431890E+03,.8004183E+03,&
     & .8015306E+03,.6334675E+03,.5387162E+03,.6432031E+03,.8004461E+03,&
     & .8007226E+03,.6328626E+03,.5383107E+03,.6431246E+03,.8003210E+03,&
     & .8014021E+03,.6333736E+03,.5385286E+03,.6427532E+03,.7999274E+03,&
     & .8004402E+03,.6326495E+03,.5380753E+03,.6429610E+03,.8002701E+03,&
     & .7996030E+03,.6320254E+03,.5376858E+03,.6430585E+03,.8004368E+03,&
     & .7988743E+03,.6314752E+03,.5373442E+03,.6430640E+03,.8004368E+03,&
     & .7982202E+03,.6309896E+03,.5370303E+03,.6429770E+03,.8002794E+03,&
     & .7987304E+03,.6313713E+03,.5371412E+03,.6426697E+03,.8000062E+03,&
     & .7979624E+03,.6307965E+03,.5368049E+03,.6428597E+03,.8003164E+03,&
     & .7972983E+03,.6302955E+03,.5365078E+03,.6429533E+03,.8004507E+03,&
     & .7967135E+03,.6298546E+03,.5362486E+03,.6429503E+03,.8004183E+03,&
     & .7961933E+03,.6294629E+03,.5359978E+03,.6428455E+03,.8002331E+03,&
     & .7965696E+03,.6297507E+03,.5360417E+03,.6426162E+03,.8000849E+03,&
     & .7959601E+03,.6292944E+03,.5357832E+03,.6427892E+03,.8003581E+03,&
     & .7954299E+03,.6288935E+03,.5355640E+03,.6428696E+03,.8004600E+03,&
     & .7949590E+03,.6285410E+03,.5353686E+03,.6428489E+03,.8003905E+03,&
     & .7945427E+03,.6282340E+03,.5351678E+03,.6427355E+03,.8001775E+03,&
     & .7948251E+03,.6284372E+03,.5351625E+03,.6425749E+03,.8001590E+03,&
     & .7943395E+03,.6280755E+03,.5349778E+03,.6427340E+03,.8003951E+03,&
     & .7939186E+03,.6277584E+03,.5348178E+03,.6427966E+03,.8004553E+03,&
     & .7935415E+03,.6274807E+03,.5346625E+03,.6427574E+03,.8003581E+03,&
     & .7932145E+03,.6272282E+03,.5344964E+03,.6426309E+03,.8001127E+03,&
     & .7934130E+03,.6273768E+03,.5344664E+03,.6425483E+03,.8002238E+03,&
     & .7930260E+03,.6270890E+03,.5343317E+03,.6426996E+03,.8004229E+03,&
     & .7926943E+03,.6268366E+03,.5342117E+03,.6427344E+03,.8004461E+03,&
     & .7923919E+03,.6266134E+03,.5340864E+03,.6426820E+03,.8003164E+03,&
     & .7921295E+03,.6264149E+03,.5339549E+03,.6425323E+03,.8000339E+03/

      data   absb(351:525, 4)   /                                       &
     & .7922680E+03,.6265242E+03,.5339049E+03,.6425377E+03,.8002886E+03,&
     & .7919656E+03,.6262964E+03,.5338156E+03,.6426613E+03,.8004414E+03,&
     & .7916978E+03,.6260932E+03,.5337303E+03,.6426876E+03,.8004275E+03,&
     & .7914600E+03,.6259147E+03,.5336295E+03,.6426074E+03,.8002608E+03,&
     & .7912522E+03,.6257562E+03,.5335134E+03,.6424353E+03,.7999413E+03,&
     & .7913561E+03,.6258354E+03,.5334642E+03,.6425425E+03,.8003396E+03,&
     & .7911083E+03,.6256569E+03,.5334042E+03,.6426337E+03,.8004553E+03,&
     & .7908952E+03,.6254984E+03,.5333388E+03,.6426322E+03,.8004044E+03,&
     & .7907120E+03,.6253545E+03,.5332580E+03,.6425389E+03,.8002006E+03,&
     & .7905435E+03,.6252260E+03,.5331566E+03,.6423436E+03,.7998395E+03,&
     & .7906128E+03,.6252806E+03,.5331127E+03,.6425289E+03,.8003859E+03,&
     & .7904242E+03,.6251367E+03,.5330727E+03,.6426115E+03,.8004553E+03,&
     & .7902511E+03,.6250128E+03,.5330219E+03,.6425922E+03,.8003673E+03,&
     & .7901026E+03,.6248989E+03,.5329519E+03,.6424757E+03,.8001266E+03,&
     & .7899687E+03,.6247997E+03,.5328605E+03,.6422481E+03,.7997283E+03,&
     & .7900179E+03,.6248343E+03,.5328366E+03,.6425305E+03,.8004183E+03,&
     & .7898694E+03,.6247204E+03,.5328112E+03,.6425854E+03,.8004461E+03,&
     & .7897355E+03,.6246211E+03,.5327658E+03,.6425429E+03,.8003210E+03,&
     & .7896116E+03,.6245319E+03,.5327051E+03,.6423987E+03,.8000432E+03,&
     & .7895077E+03,.6244526E+03,.5326190E+03,.6421432E+03,.7995987E+03,&
     & .7895370E+03,.6244726E+03,.5326105E+03,.6425329E+03,.8004414E+03,&
     & .7894185E+03,.6243834E+03,.5325958E+03,.6425646E+03,.8004275E+03,&
     & .7893092E+03,.6243041E+03,.5325597E+03,.6424944E+03,.8002655E+03,&
     & .7892153E+03,.6242295E+03,.5325043E+03,.6423270E+03,.7999460E+03,&
     & .7891307E+03,.6241702E+03,.5324182E+03,.6420445E+03,.7994598E+03,&
     & .7891507E+03,.6241848E+03,.5324351E+03,.6425307E+03,.8004553E+03,&
     & .7890568E+03,.6241156E+03,.5324243E+03,.6425392E+03,.8003998E+03,&
     & .7889722E+03,.6240463E+03,.5323943E+03,.6424466E+03,.8001960E+03,&
     & .7888929E+03,.6239917E+03,.5323429E+03,.6422514E+03,.7998348E+03,&
     & .7888283E+03,.6239424E+03,.5322521E+03,.6419319E+03,.7993069E+03,&
     & .7888437E+03,.6239471E+03,.5322936E+03,.6425292E+03,.8004553E+03,&
     & .7887644E+03,.6238924E+03,.5322836E+03,.6425100E+03,.8003627E+03,&
     & .7886998E+03,.6238432E+03,.5322582E+03,.6423942E+03,.8001173E+03,&
     & .7886405E+03,.6237985E+03,.5322021E+03,.6421666E+03,.7997144E+03,&
     & .7885859E+03,.6237585E+03,.5321068E+03,.6418054E+03,.7991402E+03/

      data   absb(526:700, 4)   /                                       &
     & .7885959E+03,.6237639E+03,.5321829E+03,.6425185E+03,.8004461E+03,&
     & .7885312E+03,.6237193E+03,.5321775E+03,.6424761E+03,.8003210E+03,&
     & .7884766E+03,.6236793E+03,.5321475E+03,.6423418E+03,.8000386E+03,&
     & .7884320E+03,.6236446E+03,.5320868E+03,.6420918E+03,.7995987E+03,&
     & .7883873E+03,.6236100E+03,.5319860E+03,.6416882E+03,.7989828E+03,&
     & .7883973E+03,.6236200E+03,.5320921E+03,.6425078E+03,.8004414E+03,&
     & .7883527E+03,.6235854E+03,.5320868E+03,.6424568E+03,.8002933E+03,&
     & .7883081E+03,.6235507E+03,.5320621E+03,.6423040E+03,.7999923E+03,&
     & .7882688E+03,.6235207E+03,.5320014E+03,.6420401E+03,.7995246E+03,&
     & .7882288E+03,.6234961E+03,.5318953E+03,.6416133E+03,.7988948E+03,&
     & .7882488E+03,.6235061E+03,.5320114E+03,.6424978E+03,.8004368E+03,&
     & .7882042E+03,.6234715E+03,.5320168E+03,.6424468E+03,.8002886E+03,&
     & .7881696E+03,.6234469E+03,.5319914E+03,.6422940E+03,.7999830E+03,&
     & .7881396E+03,.6234269E+03,.5319360E+03,.6420255E+03,.7995153E+03,&
     & .7881103E+03,.6234022E+03,.5318299E+03,.6415941E+03,.7988763E+03,&
     & .7881249E+03,.6234169E+03,.5319460E+03,.6424924E+03,.8004507E+03,&
     & .7880903E+03,.6233922E+03,.5319614E+03,.6424554E+03,.8003164E+03,&
     & .7880603E+03,.6233676E+03,.5319414E+03,.6423118E+03,.8000339E+03,&
     & .7880357E+03,.6233476E+03,.5318907E+03,.6420618E+03,.7995848E+03,&
     & .7880110E+03,.6233276E+03,.5317953E+03,.6416589E+03,.7989689E+03,&
     & .7880257E+03,.6233376E+03,.5318907E+03,.6424878E+03,.8004507E+03,&
     & .7880010E+03,.6233176E+03,.5319107E+03,.6424646E+03,.8003442E+03,&
     & .7879710E+03,.6233030E+03,.5319014E+03,.6423342E+03,.8000849E+03,&
     & .7879510E+03,.6232830E+03,.5318553E+03,.6420981E+03,.7996589E+03,&
     & .7879318E+03,.6232683E+03,.5317753E+03,.6417184E+03,.7990662E+03,&
     & .7879464E+03,.6232783E+03,.5318507E+03,.6424778E+03,.8004553E+03,&
     & .7879218E+03,.6232583E+03,.5318707E+03,.6424639E+03,.8003673E+03,&
     & .7879018E+03,.6232483E+03,.5318607E+03,.6423574E+03,.8001266E+03,&
     & .7878818E+03,.6232337E+03,.5318307E+03,.6421351E+03,.7997283E+03,&
     & .7878672E+03,.6232183E+03,.5317553E+03,.6417832E+03,.7991588E+03,&
     & .7878772E+03,.6232283E+03,.5318053E+03,.6424639E+03,.8004553E+03,&
     & .7878572E+03,.6232137E+03,.5318353E+03,.6424778E+03,.8004044E+03,&
     & .7878425E+03,.6232037E+03,.5318407E+03,.6423844E+03,.8002006E+03,&
     & .7878225E+03,.6231891E+03,.5318153E+03,.6421946E+03,.7998395E+03,&
     & .7878125E+03,.6231791E+03,.5317499E+03,.6418751E+03,.7993116E+03/

      data   absb(701:875, 4)   /                                       &
     & .7878225E+03,.6231891E+03,.5317645E+03,.6424400E+03,.8004461E+03,&
     & .7878079E+03,.6231791E+03,.5317999E+03,.6424770E+03,.8004275E+03,&
     & .7877925E+03,.6231691E+03,.5318153E+03,.6424168E+03,.8002655E+03,&
     & .7877779E+03,.6231544E+03,.5317999E+03,.6422548E+03,.7999460E+03,&
     & .7877679E+03,.6231444E+03,.5317445E+03,.6419769E+03,.7994598E+03,&
     & .7877779E+03,.6231544E+03,.5317245E+03,.6424122E+03,.8004183E+03,&
     & .7877633E+03,.6231444E+03,.5317699E+03,.6424770E+03,.8004461E+03,&
     & .7877533E+03,.6231344E+03,.5317899E+03,.6424400E+03,.8003210E+03,&
     & .7877433E+03,.6231244E+03,.5317853E+03,.6423057E+03,.8000432E+03,&
     & .7877333E+03,.6231198E+03,.5317445E+03,.6420556E+03,.7996033E+03,&
     & .7877433E+03,.6231244E+03,.5316899E+03,.6423705E+03,.8003766E+03,&
     & .7877333E+03,.6231144E+03,.5317399E+03,.6424678E+03,.8004553E+03,&
     & .7877186E+03,.6231098E+03,.5317699E+03,.6424585E+03,.8003766E+03,&
     & .7877086E+03,.6231044E+03,.5317699E+03,.6423566E+03,.8001451E+03,&
     & .7876986E+03,.6230998E+03,.5317445E+03,.6421383E+03,.7997561E+03,&
     & .7877086E+03,.6231044E+03,.5316492E+03,.6423150E+03,.8003071E+03,&
     & .7876986E+03,.6230998E+03,.5317045E+03,.6424392E+03,.8004461E+03,&
     & .7876940E+03,.6230898E+03,.5317445E+03,.6424670E+03,.8004229E+03,&
     & .7876840E+03,.6230798E+03,.5317599E+03,.6424022E+03,.8002423E+03,&
     & .7876786E+03,.6230798E+03,.5317499E+03,.6422216E+03,.7999089E+03,&
     & .7876886E+03,.6230898E+03,.5316092E+03,.6422494E+03,.8002238E+03,&
     & .7876786E+03,.6230798E+03,.5316792E+03,.6424068E+03,.8004229E+03,&
     & .7876740E+03,.6230698E+03,.5317245E+03,.6424717E+03,.8004461E+03,&
     & .7876640E+03,.6230698E+03,.5317445E+03,.6424300E+03,.8003210E+03,&
     & .7876540E+03,.6230652E+03,.5317445E+03,.6422957E+03,.8000386E+03,&
     & .7876640E+03,.6230698E+03,.5315738E+03,.6421614E+03,.8001080E+03,&
     & .7876540E+03,.6230652E+03,.5316438E+03,.6423605E+03,.8003720E+03,&
     & .7876540E+03,.6230598E+03,.5316992E+03,.6424578E+03,.8004553E+03,&
     & .7876440E+03,.6230552E+03,.5317345E+03,.6424578E+03,.8003812E+03,&
     & .7876394E+03,.6230552E+03,.5317399E+03,.6423559E+03,.8001590E+03,&
     & .7876494E+03,.6230552E+03,.5315245E+03,.6420410E+03,.7999552E+03,&
     & .7876440E+03,.6230552E+03,.5316045E+03,.6422957E+03,.8002840E+03,&
     & .7876340E+03,.6230452E+03,.5316692E+03,.6424300E+03,.8004414E+03,&
     & .7876294E+03,.6230452E+03,.5317145E+03,.6424670E+03,.8004322E+03,&
     & .7876294E+03,.6230406E+03,.5317299E+03,.6424068E+03,.8002655E+03/

      data   absb(876:1050, 4)   /                                      &
     & .7876340E+03,.6230452E+03,.5314838E+03,.6419067E+03,.7997700E+03,&
     & .7876294E+03,.6230452E+03,.5315692E+03,.6422077E+03,.8001729E+03,&
     & .7876194E+03,.6230352E+03,.5316392E+03,.6423883E+03,.8003998E+03,&
     & .7876194E+03,.6230352E+03,.5316892E+03,.6424670E+03,.8004553E+03,&
     & .7876194E+03,.6230352E+03,.5317192E+03,.6424439E+03,.8003488E+03,&
     & .7876194E+03,.6230352E+03,.5314438E+03,.6417493E+03,.7995616E+03,&
     & .7876194E+03,.6230352E+03,.5315284E+03,.6421012E+03,.8000293E+03,&
     & .7876094E+03,.6230306E+03,.5316038E+03,.6423281E+03,.8003581E+03,&
     & .7876094E+03,.6230252E+03,.5316692E+03,.6424485E+03,.8004507E+03,&
     & .7876047E+03,.6230252E+03,.5317045E+03,.6424624E+03,.8004090E+03,&
     & .7876094E+03,.6230306E+03,.5314031E+03,.6415826E+03,.7993394E+03,&
     & .7876094E+03,.6230252E+03,.5314884E+03,.6419808E+03,.7998719E+03,&
     & .7876047E+03,.6230206E+03,.5315738E+03,.6422540E+03,.8002377E+03,&
     & .7875994E+03,.6230206E+03,.5316438E+03,.6424115E+03,.8004275E+03,&
     & .7875994E+03,.6230206E+03,.5316892E+03,.6424670E+03,.8004414E+03,&
     & .7875994E+03,.6230206E+03,.5313623E+03,.6414066E+03,.7991032E+03,&
     & .7875994E+03,.6230206E+03,.5314538E+03,.6418512E+03,.7996959E+03,&
     & .7875947E+03,.6230152E+03,.5315392E+03,.6421707E+03,.8001266E+03,&
     & .7875947E+03,.6230152E+03,.5316138E+03,.6423652E+03,.8003766E+03,&
     & .7875894E+03,.6230106E+03,.5316745E+03,.6424578E+03,.8004553E+03,&
     & .7875947E+03,.6230152E+03,.5313123E+03,.6412029E+03,.7988393E+03,&
     & .7875947E+03,.6230152E+03,.5314184E+03,.6417030E+03,.7995014E+03,&
     & .7875894E+03,.6230106E+03,.5315084E+03,.6420688E+03,.7999876E+03,&
     & .7875847E+03,.6230106E+03,.5315838E+03,.6423096E+03,.8003071E+03,&
     & .7875847E+03,.6230106E+03,.5316538E+03,.6424346E+03,.8004507E+03,&
     & .7875894E+03,.6230106E+03,.5312631E+03,.6409852E+03,.7985568E+03,&
     & .7875847E+03,.6230106E+03,.5313777E+03,.6415409E+03,.7992838E+03,&
     & .7875847E+03,.6230106E+03,.5314738E+03,.6419484E+03,.7998302E+03,&
     & .7875847E+03,.6230106E+03,.5315538E+03,.6422355E+03,.8002099E+03,&
     & .7875794E+03,.6230052E+03,.5316292E+03,.6424022E+03,.8004183E+03,&
     & .7875847E+03,.6230106E+03,.5312184E+03,.6407722E+03,.7982743E+03,&
     & .7875847E+03,.6230106E+03,.5313370E+03,.6413696E+03,.7990569E+03,&
     & .7875794E+03,.6230006E+03,.5314384E+03,.6418280E+03,.7996635E+03,&
     & .7875747E+03,.6230006E+03,.5315284E+03,.6421522E+03,.8001034E+03,&
     & .7875747E+03,.6230006E+03,.5315992E+03,.6423559E+03,.8003673E+03/

      data   absb(1051:1175, 4)   /                                     &
     & .7875794E+03,.6230006E+03,.5311645E+03,.6405500E+03,.7979826E+03,&
     & .7875747E+03,.6230006E+03,.5313070E+03,.6411936E+03,.7988254E+03,&
     & .7875747E+03,.6230006E+03,.5314031E+03,.6416937E+03,.7994875E+03,&
     & .7875747E+03,.6230006E+03,.5314984E+03,.6420642E+03,.7999784E+03,&
     & .7875747E+03,.6230006E+03,.5315738E+03,.6423050E+03,.8003025E+03,&
     & .7875747E+03,.6230006E+03,.5311153E+03,.6403138E+03,.7976770E+03,&
     & .7875747E+03,.6230006E+03,.5312577E+03,.6409991E+03,.7985753E+03,&
     & .7875747E+03,.6230006E+03,.5313723E+03,.6415502E+03,.7992977E+03,&
     & .7875747E+03,.6230006E+03,.5314684E+03,.6419577E+03,.7998395E+03,&
     & .7875701E+03,.6230006E+03,.5315484E+03,.6422401E+03,.8002238E+03,&
     & .7875747E+03,.6230006E+03,.5310514E+03,.6400638E+03,.7973482E+03,&
     & .7875747E+03,.6230006E+03,.5312184E+03,.6408000E+03,.7983067E+03,&
     & .7875701E+03,.6230006E+03,.5313423E+03,.6413927E+03,.7990847E+03,&
     & .7875647E+03,.6230006E+03,.5314331E+03,.6418419E+03,.7996866E+03,&
     & .7875647E+03,.6229959E+03,.5315238E+03,.6421660E+03,.8001173E+03,&
     & .7875701E+03,.6230006E+03,.5309975E+03,.6398091E+03,.7970241E+03,&
     & .7875647E+03,.6230006E+03,.5311692E+03,.6405870E+03,.7980382E+03,&
     & .7875647E+03,.6229959E+03,.5313023E+03,.6412260E+03,.7988717E+03,&
     & .7875647E+03,.6229959E+03,.5314077E+03,.6417115E+03,.7995246E+03,&
     & .7875647E+03,.6229906E+03,.5314984E+03,.6420727E+03,.8000015E+03,&
     & .7875647E+03,.6229959E+03,.5309682E+03,.6397072E+03,.7968852E+03,&
     & .7875647E+03,.6229959E+03,.5311492E+03,.6404983E+03,.7979224E+03,&
     & .7875647E+03,.6229906E+03,.5312931E+03,.6411512E+03,.7987791E+03,&
     & .7875647E+03,.6229906E+03,.5313977E+03,.6416652E+03,.7994505E+03,&
     & .7875647E+03,.6229906E+03,.5314884E+03,.6420356E+03,.7999552E+03/

      data   absb(  1:175, 5)   /                                       &
     & .1132160E+04,.8590321E+03,.5914771E+03,.8420290E+03,.1108579E+04,&
     & .1123473E+04,.8524625E+03,.5885652E+03,.8420681E+03,.1108734E+04,&
     & .1115900E+04,.8468347E+03,.5862702E+03,.8419164E+03,.1108697E+04,&
     & .1109376E+04,.8419137E+03,.5843636E+03,.8415502E+03,.1108322E+04,&
     & .1103614E+04,.8376140E+03,.5827038E+03,.8409654E+03,.1107682E+04,&
     & .1110333E+04,.8425995E+03,.5848362E+03,.8418344E+03,.1108619E+04,&
     & .1103219E+04,.8373133E+03,.5831283E+03,.8418712E+03,.1108805E+04,&
     & .1097195E+04,.8327409E+03,.5816627E+03,.8417168E+03,.1108673E+04,&
     & .1091865E+04,.8287858E+03,.5803753E+03,.8413520E+03,.1108225E+04,&
     & .1087188E+04,.8252839E+03,.5792120E+03,.8407467E+03,.1107577E+04,&
     & .1092412E+04,.8291999E+03,.5808570E+03,.8416771E+03,.1108633E+04,&
     & .1086772E+04,.8249577E+03,.5797842E+03,.8417051E+03,.1108778E+04,&
     & .1081872E+04,.8212500E+03,.5788998E+03,.8415393E+03,.1108609E+04,&
     & .1077591E+04,.8180528E+03,.5780798E+03,.8411541E+03,.1108196E+04,&
     & .1073799E+04,.8152246E+03,.5772425E+03,.8405502E+03,.1107530E+04,&
     & .1077932E+04,.8183131E+03,.5785343E+03,.8415558E+03,.1108628E+04,&
     & .1073370E+04,.8148772E+03,.5779321E+03,.8415699E+03,.1108773E+04,&
     & .1069367E+04,.8118993E+03,.5773345E+03,.8413946E+03,.1108586E+04,&
     & .1065923E+04,.8093364E+03,.5766911E+03,.8409868E+03,.1108139E+04,&
     & .1062906E+04,.8070496E+03,.5759923E+03,.8403687E+03,.1107413E+04,&
     & .1066103E+04,.8094792E+03,.5771890E+03,.8414562E+03,.1108645E+04,&
     & .1062427E+04,.8067082E+03,.5767835E+03,.8414606E+03,.1108749E+04,&
     & .1059242E+04,.8043159E+03,.5763730E+03,.8412660E+03,.1108562E+04,&
     & .1056471E+04,.8022149E+03,.5759120E+03,.8408523E+03,.1108069E+04,&
     & .1053956E+04,.8003803E+03,.5753382E+03,.8402087E+03,.1107311E+04,&
     & .1056573E+04,.8023128E+03,.5763612E+03,.8413744E+03,.1108681E+04,&
     & .1053614E+04,.8000761E+03,.5761904E+03,.8413677E+03,.1108744E+04,&
     & .1050995E+04,.7981568E+03,.5759572E+03,.8411623E+03,.1108557E+04,&
     & .1048738E+04,.7964524E+03,.5755889E+03,.8407256E+03,.1108012E+04,&
     & .1046781E+04,.7949640E+03,.5750302E+03,.8400566E+03,.1107234E+04,&
     & .1048831E+04,.7964881E+03,.5759871E+03,.8413116E+03,.1108751E+04,&
     & .1046421E+04,.7947095E+03,.5759550E+03,.8412858E+03,.1108763E+04,&
     & .1044326E+04,.7931394E+03,.5755277E+03,.8410568E+03,.1108455E+04,&
     & .1042519E+04,.7917703E+03,.5753386E+03,.8406038E+03,.1107955E+04,&
     & .1040912E+04,.7905575E+03,.5748294E+03,.8399213E+03,.1107129E+04/

      data   absb(176:350, 5)   /                                       &
     & .1042504E+04,.7917365E+03,.5757083E+03,.8412594E+03,.1108725E+04,&
     & .1040562E+04,.7902912E+03,.5758192E+03,.8412189E+03,.1108733E+04,&
     & .1038874E+04,.7890443E+03,.5756096E+03,.8409680E+03,.1108430E+04,&
     & .1037398E+04,.7879351E+03,.5752218E+03,.8404909E+03,.1107888E+04,&
     & .1036091E+04,.7869710E+03,.5746527E+03,.8397830E+03,.1107005E+04,&
     & .1037338E+04,.7878931E+03,.5757405E+03,.8412192E+03,.1108773E+04,&
     & .1035785E+04,.7867242E+03,.5757045E+03,.8411493E+03,.1108704E+04,&
     & .1034434E+04,.7857049E+03,.5754770E+03,.8408953E+03,.1108371E+04,&
     & .1033255E+04,.7848307E+03,.5750727E+03,.8403820E+03,.1107804E+04,&
     & .1032221E+04,.7840296E+03,.5744847E+03,.8396477E+03,.1106878E+04,&
     & .1033083E+04,.7846984E+03,.5756617E+03,.8411849E+03,.1108731E+04,&
     & .1031809E+04,.7837832E+03,.5755890E+03,.8410839E+03,.1108675E+04,&
     & .1030787E+04,.7829597E+03,.5753383E+03,.8407754E+03,.1108320E+04,&
     & .1029807E+04,.7822396E+03,.5749009E+03,.8402301E+03,.1107645E+04,&
     & .1028979E+04,.7816263E+03,.5742825E+03,.8394539E+03,.1106694E+04,&
     & .1029657E+04,.7821089E+03,.5755832E+03,.8411501E+03,.1108780E+04,&
     & .1028643E+04,.7813695E+03,.5754905E+03,.8410195E+03,.1108611E+04,&
     & .1027767E+04,.7807233E+03,.5752049E+03,.8406589E+03,.1108196E+04,&
     & .1027001E+04,.7801530E+03,.5747429E+03,.8400708E+03,.1107518E+04,&
     & .1026393E+04,.7796516E+03,.5740912E+03,.8392499E+03,.1106487E+04,&
     & .1026855E+04,.7800266E+03,.5755207E+03,.8411092E+03,.1108746E+04,&
     & .1026108E+04,.7794325E+03,.5753911E+03,.8409396E+03,.1108560E+04,&
     & .1025405E+04,.7789140E+03,.5750771E+03,.8405361E+03,.1108069E+04,&
     & .1024798E+04,.7784615E+03,.5745776E+03,.8399056E+03,.1107291E+04,&
     & .1024258E+04,.7780677E+03,.5738970E+03,.8390391E+03,.1106226E+04,&
     & .1024599E+04,.7783363E+03,.5754652E+03,.8410717E+03,.1108755E+04,&
     & .1023976E+04,.7778616E+03,.5752948E+03,.8408626E+03,.1108455E+04,&
     & .1023441E+04,.7774535E+03,.5749424E+03,.8404102E+03,.1107910E+04,&
     & .1022936E+04,.7771014E+03,.5744109E+03,.8397256E+03,.1107107E+04,&
     & .1022522E+04,.7767688E+03,.5736917E+03,.8388102E+03,.1105992E+04,&
     & .1022769E+04,.7769692E+03,.5754043E+03,.8410269E+03,.1108704E+04,&
     & .1022268E+04,.7765895E+03,.5751966E+03,.8407518E+03,.1108371E+04,&
     & .1021836E+04,.7762638E+03,.5748030E+03,.8402567E+03,.1107761E+04,&
     & .1021436E+04,.7759755E+03,.5742299E+03,.8395192E+03,.1106876E+04,&
     & .1021119E+04,.7757229E+03,.5734692E+03,.8385490E+03,.1105715E+04/

      data   absb(351:525, 5)   /                                       &
     & .1021292E+04,.7758615E+03,.5753413E+03,.8409718E+03,.1108668E+04,&
     & .1020929E+04,.7755643E+03,.5750916E+03,.8406449E+03,.1108220E+04,&
     & .1020592E+04,.7753101E+03,.5746540E+03,.8400874E+03,.1107585E+04,&
     & .1020272E+04,.7750812E+03,.5740383E+03,.8393001E+03,.1106614E+04,&
     & .1019972E+04,.7748751E+03,.5732385E+03,.8382776E+03,.1105351E+04,&
     & .1020107E+04,.7749807E+03,.5752706E+03,.8408913E+03,.1108584E+04,&
     & .1019789E+04,.7747389E+03,.5749801E+03,.8405246E+03,.1108134E+04,&
     & .1019546E+04,.7745422E+03,.5745060E+03,.8399165E+03,.1107403E+04,&
     & .1019286E+04,.7743596E+03,.5738419E+03,.8390704E+03,.1106328E+04,&
     & .1019084E+04,.7741942E+03,.5730024E+03,.8379946E+03,.1105002E+04,&
     & .1019170E+04,.7742594E+03,.5751978E+03,.8408308E+03,.1108460E+04,&
     & .1018904E+04,.7740885E+03,.5748611E+03,.8403860E+03,.1107988E+04,&
     & .1018723E+04,.7739185E+03,.5743390E+03,.8397187E+03,.1107159E+04,&
     & .1018501E+04,.7737687E+03,.5736328E+03,.8388110E+03,.1106037E+04,&
     & .1018361E+04,.7736360E+03,.5727529E+03,.8376904E+03,.1104671E+04,&
     & .1018405E+04,.7736814E+03,.5751164E+03,.8407303E+03,.1108371E+04,&
     & .1018184E+04,.7735401E+03,.5747312E+03,.8402361E+03,.1107809E+04,&
     & .1018016E+04,.7734164E+03,.5741681E+03,.8395069E+03,.1106893E+04,&
     & .1017890E+04,.7732973E+03,.5734155E+03,.8385510E+03,.1105713E+04,&
     & .1017756E+04,.7731912E+03,.5724903E+03,.8373611E+03,.1104249E+04,&
     & .1017792E+04,.7732316E+03,.5750249E+03,.8406141E+03,.1108220E+04,&
     & .1017606E+04,.7731185E+03,.5745953E+03,.8400642E+03,.1107577E+04,&
     & .1017494E+04,.7730061E+03,.5739811E+03,.8392794E+03,.1106592E+04,&
     & .1017366E+04,.7729075E+03,.5731866E+03,.8382606E+03,.1105362E+04,&
     & .1017266E+04,.7728198E+03,.5722190E+03,.8370243E+03,.1103792E+04,&
     & .1017292E+04,.7728676E+03,.5749177E+03,.8404941E+03,.1108136E+04,&
     & .1017190E+04,.7727693E+03,.5744446E+03,.8398803E+03,.1107391E+04,&
     & .1017083E+04,.7726878E+03,.5737816E+03,.8390343E+03,.1106328E+04,&
     & .1016964E+04,.7726050E+03,.5729440E+03,.8379568E+03,.1105000E+04,&
     & .1016882E+04,.7725450E+03,.5719281E+03,.8366665E+03,.1103367E+04,&
     & .1016898E+04,.7725446E+03,.5748005E+03,.8403534E+03,.1107950E+04,&
     & .1016789E+04,.7724810E+03,.5742785E+03,.8396774E+03,.1107107E+04,&
     & .1016697E+04,.7724226E+03,.5735704E+03,.8387721E+03,.1105974E+04,&
     & .1016611E+04,.7723504E+03,.5726872E+03,.8376384E+03,.1104578E+04,&
     & .1016571E+04,.7723041E+03,.5716302E+03,.8363014E+03,.1102913E+04/

      data   absb(526:700, 5)   /                                       &
     & .1016584E+04,.7723202E+03,.5746950E+03,.8402176E+03,.1107804E+04,&
     & .1016488E+04,.7722577E+03,.5741300E+03,.8394890E+03,.1106878E+04,&
     & .1016426E+04,.7722026E+03,.5733811E+03,.8385253E+03,.1105713E+04,&
     & .1016380E+04,.7721456E+03,.5724560E+03,.8373415E+03,.1104249E+04,&
     & .1016306E+04,.7721127E+03,.5713670E+03,.8359703E+03,.1102470E+04,&
     & .1016292E+04,.7721168E+03,.5746264E+03,.8401340E+03,.1107701E+04,&
     & .1016228E+04,.7720667E+03,.5740386E+03,.8393714E+03,.1106751E+04,&
     & .1016205E+04,.7720362E+03,.5732662E+03,.8383847E+03,.1105553E+04,&
     & .1016128E+04,.7719854E+03,.5723210E+03,.8371730E+03,.1104011E+04,&
     & .1016090E+04,.7719561E+03,.5712032E+03,.8357753E+03,.1102230E+04,&
     & .1016110E+04,.7719675E+03,.5746079E+03,.8401177E+03,.1107679E+04,&
     & .1016065E+04,.7719395E+03,.5740164E+03,.8393596E+03,.1106716E+04,&
     & .1016018E+04,.7718925E+03,.5732410E+03,.8383584E+03,.1105484E+04,&
     & .1015979E+04,.7718623E+03,.5722888E+03,.8371394E+03,.1103992E+04,&
     & .1015924E+04,.7718361E+03,.5711776E+03,.8357521E+03,.1102176E+04,&
     & .1015947E+04,.7718575E+03,.5746707E+03,.8401949E+03,.1107761E+04,&
     & .1015896E+04,.7718201E+03,.5741037E+03,.8394653E+03,.1106876E+04,&
     & .1015883E+04,.7717903E+03,.5733511E+03,.8384984E+03,.1105710E+04,&
     & .1015843E+04,.7717658E+03,.5724260E+03,.8373113E+03,.1104216E+04,&
     & .1015798E+04,.7717449E+03,.5713285E+03,.8359335E+03,.1102415E+04,&
     & .1015834E+04,.7717612E+03,.5747316E+03,.8402799E+03,.1107885E+04,&
     & .1015782E+04,.7717396E+03,.5741902E+03,.8395813E+03,.1107005E+04,&
     & .1015801E+04,.7717242E+03,.5734642E+03,.8386440E+03,.1105870E+04,&
     & .1015718E+04,.7716962E+03,.5725620E+03,.8374954E+03,.1104414E+04,&
     & .1015732E+04,.7716614E+03,.5714866E+03,.8361383E+03,.1102708E+04,&
     & .1015729E+04,.7716742E+03,.5747862E+03,.8403517E+03,.1107983E+04,&
     & .1015723E+04,.7716656E+03,.5742723E+03,.8396911E+03,.1107164E+04,&
     & .1015702E+04,.7716450E+03,.5735710E+03,.8387908E+03,.1106042E+04,&
     & .1015640E+04,.7716167E+03,.5726959E+03,.8376665E+03,.1104643E+04,&
     & .1015624E+04,.7716061E+03,.5716434E+03,.8363327E+03,.1102967E+04,&
     & .1015631E+04,.7716188E+03,.5748861E+03,.8404775E+03,.1108129E+04,&
     & .1015600E+04,.7716030E+03,.5744127E+03,.8398706E+03,.1107389E+04,&
     & .1015613E+04,.7715764E+03,.5737556E+03,.8390288E+03,.1106328E+04,&
     & .1015597E+04,.7715697E+03,.5729199E+03,.8379553E+03,.1105002E+04,&
     & .1015567E+04,.7715535E+03,.5719103E+03,.8366668E+03,.1103378E+04/

      data   absb(701:875, 5)   /                                       &
     & .1015593E+04,.7715621E+03,.5749733E+03,.8405981E+03,.1108255E+04,&
     & .1015568E+04,.7715451E+03,.5745491E+03,.8400486E+03,.1107577E+04,&
     & .1015524E+04,.7715394E+03,.5739392E+03,.8392612E+03,.1106614E+04,&
     & .1015503E+04,.7715146E+03,.5731507E+03,.8382433E+03,.1105384E+04,&
     & .1015505E+04,.7715078E+03,.5721820E+03,.8370057E+03,.1103824E+04,&
     & .1015532E+04,.7715161E+03,.5750518E+03,.8407005E+03,.1108396E+04,&
     & .1015508E+04,.7715165E+03,.5746709E+03,.8402103E+03,.1107804E+04,&
     & .1015497E+04,.7714992E+03,.5741118E+03,.8394869E+03,.1106898E+04,&
     & .1015491E+04,.7714822E+03,.5733668E+03,.8385286E+03,.1105713E+04,&
     & .1015451E+04,.7714746E+03,.5724432E+03,.8373437E+03,.1104253E+04,&
     & .1015486E+04,.7714820E+03,.5751302E+03,.8408091E+03,.1108535E+04,&
     & .1015439E+04,.7714848E+03,.5748127E+03,.8403847E+03,.1108012E+04,&
     & .1015433E+04,.7714633E+03,.5743039E+03,.8397376E+03,.1107232E+04,&
     & .1015424E+04,.7714749E+03,.5736175E+03,.8388518E+03,.1106104E+04,&
     & .1015410E+04,.7714655E+03,.5727504E+03,.8377374E+03,.1104743E+04,&
     & .1015436E+04,.7714612E+03,.5751941E+03,.8408969E+03,.1108616E+04,&
     & .1015415E+04,.7714466E+03,.5749379E+03,.8405563E+03,.1108196E+04,&
     & .1015431E+04,.7714422E+03,.5744930E+03,.8399786E+03,.1107523E+04,&
     & .1015404E+04,.7714254E+03,.5738675E+03,.8391725E+03,.1106533E+04,&
     & .1015390E+04,.7714259E+03,.5730571E+03,.8381379E+03,.1105227E+04,&
     & .1015396E+04,.7714361E+03,.5752396E+03,.8409521E+03,.1108709E+04,&
     & .1015390E+04,.7714279E+03,.5750483E+03,.8406969E+03,.1108371E+04,&
     & .1015395E+04,.7714259E+03,.5746631E+03,.8402006E+03,.1107804E+04,&
     & .1015371E+04,.7714032E+03,.5740972E+03,.8394745E+03,.1106873E+04,&
     & .1015337E+04,.7714081E+03,.5733553E+03,.8385150E+03,.1105708E+04,&
     & .1015386E+04,.7714066E+03,.5752600E+03,.8409913E+03,.1108744E+04,&
     & .1015370E+04,.7713985E+03,.5751351E+03,.8408196E+03,.1108557E+04,&
     & .1015337E+04,.7713964E+03,.5748223E+03,.8404045E+03,.1108012E+04,&
     & .1015367E+04,.7713981E+03,.5743244E+03,.8397592E+03,.1107234E+04,&
     & .1015333E+04,.7713847E+03,.5736421E+03,.8388851E+03,.1106199E+04,&
     & .1015347E+04,.7714005E+03,.5752587E+03,.8410118E+03,.1108805E+04,&
     & .1015311E+04,.7713902E+03,.5752086E+03,.8409157E+03,.1108668E+04,&
     & .1015318E+04,.7713893E+03,.5749690E+03,.8405927E+03,.1108222E+04,&
     & .1015319E+04,.7713841E+03,.5745459E+03,.8400467E+03,.1107577E+04,&
     & .1015312E+04,.7713874E+03,.5739392E+03,.8392646E+03,.1106614E+04/

      data   absb(876:1050, 5)   /                                      &
     & .1015326E+04,.7713768E+03,.5752286E+03,.8409923E+03,.1108751E+04,&
     & .1015341E+04,.7713759E+03,.5752489E+03,.8409777E+03,.1108755E+04,&
     & .1015327E+04,.7713686E+03,.5750877E+03,.8407611E+03,.1108430E+04,&
     & .1015298E+04,.7713645E+03,.5747398E+03,.8402950E+03,.1107885E+04,&
     & .1015298E+04,.7713757E+03,.5742018E+03,.8396055E+03,.1107040E+04,&
     & .1015315E+04,.7713740E+03,.5751637E+03,.8409394E+03,.1108628E+04,&
     & .1015298E+04,.7713705E+03,.5752637E+03,.8410091E+03,.1108775E+04,&
     & .1015291E+04,.7713551E+03,.5751720E+03,.8408697E+03,.1108543E+04,&
     & .1015296E+04,.7713612E+03,.5748985E+03,.8405081E+03,.1108141E+04,&
     & .1015287E+04,.7713533E+03,.5744395E+03,.8399107E+03,.1107418E+04,&
     & .1015330E+04,.7713626E+03,.5750740E+03,.8408557E+03,.1108508E+04,&
     & .1015299E+04,.7713625E+03,.5752467E+03,.8410077E+03,.1108736E+04,&
     & .1015318E+04,.7713499E+03,.5752320E+03,.8409497E+03,.1108707E+04,&
     & .1015318E+04,.7713544E+03,.5750237E+03,.8406755E+03,.1108347E+04,&
     & .1015282E+04,.7713600E+03,.5746353E+03,.8401644E+03,.1107726E+04,&
     & .1015292E+04,.7713486E+03,.5749639E+03,.8407482E+03,.1108372E+04,&
     & .1015292E+04,.7713504E+03,.5752056E+03,.8409790E+03,.1108713E+04,&
     & .1015299E+04,.7713406E+03,.5752580E+03,.8409900E+03,.1108719E+04,&
     & .1015282E+04,.7713457E+03,.5751224E+03,.8408012E+03,.1108533E+04,&
     & .1015288E+04,.7713329E+03,.5748024E+03,.8403755E+03,.1108010E+04,&
     & .1015278E+04,.7713411E+03,.5748265E+03,.8406142E+03,.1108188E+04,&
     & .1015285E+04,.7713391E+03,.5751400E+03,.8409197E+03,.1108633E+04,&
     & .1015300E+04,.7713296E+03,.5752623E+03,.8410092E+03,.1108780E+04,&
     & .1015260E+04,.7713359E+03,.5751941E+03,.8408949E+03,.1108613E+04,&
     & .1015249E+04,.7713381E+03,.5749404E+03,.8405598E+03,.1108215E+04,&
     & .1015290E+04,.7713457E+03,.5746781E+03,.8404406E+03,.1107881E+04,&
     & .1015286E+04,.7713522E+03,.5750445E+03,.8408353E+03,.1108489E+04,&
     & .1015263E+04,.7713448E+03,.5752353E+03,.8410044E+03,.1108763E+04,&
     & .1015276E+04,.7713348E+03,.5752379E+03,.8409611E+03,.1108729E+04,&
     & .1015251E+04,.7713276E+03,.5750531E+03,.8407073E+03,.1108401E+04,&
     & .1015261E+04,.7713452E+03,.5745201E+03,.8402523E+03,.1107669E+04,&
     & .1015285E+04,.7713402E+03,.5749379E+03,.8407260E+03,.1108316E+04,&
     & .1015244E+04,.7713261E+03,.5751964E+03,.8409694E+03,.1108716E+04,&
     & .1015225E+04,.7713244E+03,.5752625E+03,.8409972E+03,.1108744E+04,&
     & .1015265E+04,.7713303E+03,.5751375E+03,.8408146E+03,.1108557E+04/

      data   absb(1051:1175, 5)   /                                     &
     & .1015244E+04,.7713264E+03,.5743548E+03,.8400510E+03,.1107362E+04,&
     & .1015239E+04,.7713259E+03,.5748138E+03,.8406007E+03,.1108145E+04,&
     & .1015247E+04,.7713243E+03,.5751348E+03,.8409159E+03,.1108592E+04,&
     & .1015239E+04,.7713274E+03,.5752584E+03,.8410086E+03,.1108765E+04,&
     & .1015225E+04,.7713165E+03,.5751971E+03,.8409024E+03,.1108626E+04,&
     & .1015239E+04,.7713283E+03,.5741622E+03,.8398247E+03,.1107071E+04,&
     & .1015242E+04,.7713198E+03,.5746895E+03,.8404515E+03,.1107922E+04,&
     & .1015259E+04,.7713226E+03,.5750496E+03,.8408398E+03,.1108489E+04,&
     & .1015245E+04,.7713144E+03,.5752404E+03,.8410002E+03,.1108765E+04,&
     & .1015213E+04,.7713098E+03,.5752355E+03,.8409584E+03,.1108726E+04,&
     & .1015251E+04,.7713192E+03,.5739542E+03,.8395683E+03,.1106711E+04,&
     & .1015233E+04,.7713215E+03,.5745434E+03,.8402746E+03,.1107712E+04,&
     & .1015233E+04,.7713191E+03,.5749555E+03,.8407368E+03,.1108359E+04,&
     & .1015225E+04,.7713188E+03,.5752003E+03,.8409744E+03,.1108718E+04,&
     & .1015230E+04,.7713144E+03,.5752584E+03,.8409937E+03,.1108785E+04,&
     & .1015229E+04,.7713118E+03,.5737443E+03,.8392982E+03,.1106304E+04,&
     & .1015223E+04,.7713138E+03,.5743800E+03,.8400877E+03,.1107435E+04,&
     & .1015225E+04,.7713224E+03,.5748430E+03,.8406293E+03,.1108163E+04,&
     & .1015242E+04,.7713219E+03,.5751451E+03,.8409290E+03,.1108631E+04,&
     & .1015245E+04,.7713273E+03,.5752596E+03,.8410108E+03,.1108780E+04,&
     & .1015256E+04,.7713168E+03,.5736529E+03,.8391840E+03,.1106202E+04,&
     & .1015245E+04,.7713129E+03,.5743168E+03,.8400016E+03,.1107303E+04,&
     & .1015228E+04,.7713181E+03,.5747952E+03,.8405779E+03,.1108104E+04,&
     & .1015223E+04,.7713094E+03,.5751176E+03,.8408989E+03,.1108617E+04,&
     & .1015221E+04,.7713103E+03,.5752541E+03,.8410111E+03,.1108805E+04/

      data   absb(  1:175, 6)   /                                       &
     & .1154807E+04,.8742824E+03,.6044416E+03,.8875279E+03,.1172179E+04,&
     & .1145735E+04,.8678557E+03,.6038007E+03,.8866634E+03,.1170524E+04,&
     & .1138341E+04,.8618214E+03,.6031354E+03,.8856771E+03,.1169072E+04,&
     & .1131676E+04,.8569035E+03,.6023105E+03,.8844938E+03,.1167566E+04,&
     & .1125765E+04,.8523731E+03,.6012823E+03,.8831031E+03,.1165768E+04,&
     & .1132423E+04,.8576942E+03,.6040145E+03,.8872823E+03,.1171938E+04,&
     & .1125222E+04,.8521701E+03,.6034057E+03,.8864202E+03,.1170389E+04,&
     & .1118840E+04,.8476932E+03,.6027501E+03,.8854305E+03,.1168930E+04,&
     & .1113657E+04,.8434208E+03,.6018925E+03,.8841002E+03,.1167350E+04,&
     & .1108930E+04,.8400398E+03,.6009245E+03,.8828407E+03,.1165664E+04,&
     & .1114323E+04,.8441225E+03,.6036531E+03,.8870573E+03,.1171838E+04,&
     & .1108136E+04,.8393878E+03,.6030685E+03,.8861976E+03,.1170255E+04,&
     & .1103477E+04,.8360310E+03,.6024155E+03,.8851986E+03,.1168810E+04,&
     & .1098878E+04,.8326282E+03,.6015839E+03,.8839936E+03,.1167258E+04,&
     & .1095319E+04,.8298688E+03,.6005810E+03,.8825764E+03,.1165436E+04,&
     & .1099193E+04,.8328782E+03,.6033501E+03,.8868553E+03,.1171666E+04,&
     & .1094160E+04,.8293321E+03,.6027849E+03,.8859925E+03,.1170121E+04,&
     & .1090881E+04,.8264769E+03,.6021240E+03,.8849814E+03,.1168673E+04,&
     & .1087108E+04,.8233209E+03,.6012865E+03,.8837601E+03,.1167082E+04,&
     & .1083860E+04,.8212156E+03,.6002431E+03,.8823316E+03,.1165266E+04,&
     & .1087573E+04,.8238271E+03,.6030927E+03,.8866823E+03,.1171492E+04,&
     & .1083841E+04,.8210481E+03,.6025442E+03,.8858138E+03,.1169959E+04,&
     & .1080483E+04,.8185717E+03,.6018720E+03,.8847842E+03,.1168471E+04,&
     & .1077525E+04,.8165982E+03,.6010297E+03,.8835474E+03,.1166895E+04,&
     & .1075183E+04,.8146439E+03,.6000078E+03,.8821013E+03,.1165101E+04,&
     & .1077583E+04,.8165170E+03,.6028710E+03,.8865245E+03,.1171303E+04,&
     & .1074539E+04,.8142927E+03,.6023309E+03,.8856430E+03,.1169820E+04,&
     & .1072137E+04,.8120580E+03,.6016533E+03,.8845555E+03,.1168326E+04,&
     & .1069843E+04,.8106399E+03,.6007990E+03,.8833472E+03,.1166722E+04,&
     & .1067730E+04,.8091125E+03,.5997668E+03,.8818852E+03,.1164922E+04,&
     & .1069797E+04,.8107198E+03,.6026903E+03,.8863807E+03,.1171163E+04,&
     & .1067434E+04,.8085376E+03,.6021520E+03,.8854952E+03,.1169596E+04,&
     & .1065302E+04,.8070848E+03,.6014625E+03,.8844335E+03,.1168167E+04,&
     & .1063248E+04,.8057185E+03,.6005661E+03,.8831570E+03,.1166538E+04,&
     & .1061700E+04,.8046590E+03,.5995523E+03,.8816793E+03,.1164754E+04/

      data   absb(176:350, 6)   /                                       &
     & .1063387E+04,.8058664E+03,.6025225E+03,.8862332E+03,.1170967E+04,&
     & .1061291E+04,.8043546E+03,.6019839E+03,.8853313E+03,.1169477E+04,&
     & .1059375E+04,.8029392E+03,.6012771E+03,.8842066E+03,.1167966E+04,&
     & .1058118E+04,.8019047E+03,.6003971E+03,.8829591E+03,.1166269E+04,&
     & .1057001E+04,.8008290E+03,.5993321E+03,.8814537E+03,.1164472E+04,&
     & .1058126E+04,.8017380E+03,.6023731E+03,.8860941E+03,.1170779E+04,&
     & .1056377E+04,.8006266E+03,.6018305E+03,.8851768E+03,.1169335E+04,&
     & .1055112E+04,.7997092E+03,.6011088E+03,.8838887E+03,.1167803E+04,&
     & .1053591E+04,.7985932E+03,.6002092E+03,.8827556E+03,.1166104E+04,&
     & .1052710E+04,.7980450E+03,.5991296E+03,.8811895E+03,.1164303E+04,&
     & .1053535E+04,.7985078E+03,.6022154E+03,.8859028E+03,.1170510E+04,&
     & .1052649E+04,.7973207E+03,.6016135E+03,.8849596E+03,.1169037E+04,&
     & .1051227E+04,.7967413E+03,.6008956E+03,.8838144E+03,.1167519E+04,&
     & .1050475E+04,.7962211E+03,.5999630E+03,.8824642E+03,.1165766E+04,&
     & .1049370E+04,.7952948E+03,.5988541E+03,.8809085E+03,.1163969E+04,&
     & .1050325E+04,.7960009E+03,.6020625E+03,.8857073E+03,.1170186E+04,&
     & .1049398E+04,.7952026E+03,.6014647E+03,.8847301E+03,.1168748E+04,&
     & .1048230E+04,.7945935E+03,.6006799E+03,.8835488E+03,.1167182E+04,&
     & .1047667E+04,.7940912E+03,.5997159E+03,.8821562E+03,.1165463E+04,&
     & .1047042E+04,.7935326E+03,.5985744E+03,.8805687E+03,.1163647E+04,&
     & .1047281E+04,.7937988E+03,.6019135E+03,.8855050E+03,.1169920E+04,&
     & .1046440E+04,.7933166E+03,.6012784E+03,.8844912E+03,.1168471E+04,&
     & .1045887E+04,.7928309E+03,.6004624E+03,.8832652E+03,.1166849E+04,&
     & .1045297E+04,.7923255E+03,.5994646E+03,.8818357E+03,.1165063E+04,&
     & .1044505E+04,.7918388E+03,.5982572E+03,.8802121E+03,.1163214E+04,&
     & .1045195E+04,.7920747E+03,.6017576E+03,.8852861E+03,.1169584E+04,&
     & .1044449E+04,.7917074E+03,.6010868E+03,.8840962E+03,.1168127E+04,&
     & .1043910E+04,.7912987E+03,.6002298E+03,.8829669E+03,.1166499E+04,&
     & .1043360E+04,.7907238E+03,.5992000E+03,.8814921E+03,.1164668E+04,&
     & .1042953E+04,.7905956E+03,.5979889E+03,.8798344E+03,.1162852E+04,&
     & .1043336E+04,.7906786E+03,.6015878E+03,.8850466E+03,.1169289E+04,&
     & .1042560E+04,.7904597E+03,.6008783E+03,.8839470E+03,.1167776E+04,&
     & .1042396E+04,.7901232E+03,.5999833E+03,.8826272E+03,.1166071E+04,&
     & .1041865E+04,.7898279E+03,.5989107E+03,.8811075E+03,.1164265E+04,&
     & .1041536E+04,.7894512E+03,.5976609E+03,.8794176E+03,.1162232E+04/

      data   absb(351:525, 6)   /                                       &
     & .1041735E+04,.7896371E+03,.6014126E+03,.8847933E+03,.1168848E+04,&
     & .1041235E+04,.7894095E+03,.6006558E+03,.8836405E+03,.1167384E+04,&
     & .1040839E+04,.7890744E+03,.5997245E+03,.8822749E+03,.1165633E+04,&
     & .1040637E+04,.7888720E+03,.5986096E+03,.8807131E+03,.1163845E+04,&
     & .1040454E+04,.7886684E+03,.5973211E+03,.8789887E+03,.1161882E+04,&
     & .1040668E+04,.7886417E+03,.6012280E+03,.8845275E+03,.1168593E+04,&
     & .1040157E+04,.7885219E+03,.6004316E+03,.8833250E+03,.1166996E+04,&
     & .1039836E+04,.7881981E+03,.5994554E+03,.8819140E+03,.1165226E+04,&
     & .1039810E+04,.7880131E+03,.5982992E+03,.8802613E+03,.1163390E+04,&
     & .1039396E+04,.7878410E+03,.5969813E+03,.8785538E+03,.1161371E+04,&
     & .1039719E+04,.7879584E+03,.6010368E+03,.8841057E+03,.1168138E+04,&
     & .1039383E+04,.7875535E+03,.6001922E+03,.8829897E+03,.1166558E+04,&
     & .1038687E+04,.7875554E+03,.5991728E+03,.8815245E+03,.1164748E+04,&
     & .1038748E+04,.7874938E+03,.5979743E+03,.8798787E+03,.1162892E+04,&
     & .1038605E+04,.7873559E+03,.5966233E+03,.8780670E+03,.1160883E+04,&
     & .1038508E+04,.7874554E+03,.6008293E+03,.8839425E+03,.1167803E+04,&
     & .1038655E+04,.7872552E+03,.5999122E+03,.8825924E+03,.1166035E+04,&
     & .1038537E+04,.7869660E+03,.5988787E+03,.8811226E+03,.1164305E+04,&
     & .1038302E+04,.7868881E+03,.5976363E+03,.8794383E+03,.1162412E+04,&
     & .1038080E+04,.7868277E+03,.5962613E+03,.8776832E+03,.1160344E+04,&
     & .1037957E+04,.7867389E+03,.6006076E+03,.8836238E+03,.1167419E+04,&
     & .1037919E+04,.7865797E+03,.5996797E+03,.8822650E+03,.1165664E+04,&
     & .1037579E+04,.7865886E+03,.5985653E+03,.8807029E+03,.1163863E+04,&
     & .1037883E+04,.7866633E+03,.5972822E+03,.8789799E+03,.1161882E+04,&
     & .1037601E+04,.7866140E+03,.5958926E+03,.8772343E+03,.1159884E+04,&
     & .1037461E+04,.7861854E+03,.6003749E+03,.8832869E+03,.1166982E+04,&
     & .1037326E+04,.7862094E+03,.5994026E+03,.8818705E+03,.1165163E+04,&
     & .1037061E+04,.7861750E+03,.5982418E+03,.8802656E+03,.1163356E+04,&
     & .1037419E+04,.7862176E+03,.5969241E+03,.8785113E+03,.1161372E+04,&
     & .1037145E+04,.7860371E+03,.5955189E+03,.8767759E+03,.1159373E+04,&
     & .1037013E+04,.7863353E+03,.6001314E+03,.8829351E+03,.1166533E+04,&
     & .1036952E+04,.7860923E+03,.5991081E+03,.8814656E+03,.1164747E+04,&
     & .1036909E+04,.7858192E+03,.5979043E+03,.8797720E+03,.1162848E+04,&
     & .1037101E+04,.7860878E+03,.5965532E+03,.8780077E+03,.1160772E+04,&
     & .1037052E+04,.7858782E+03,.5951462E+03,.8762643E+03,.1158850E+04/

      data   absb(526:700, 6)   /                                       &
     & .1036873E+04,.7857554E+03,.5999088E+03,.8825714E+03,.1166104E+04,&
     & .1036902E+04,.7858253E+03,.5988157E+03,.8811019E+03,.1164303E+04,&
     & .1036596E+04,.7858513E+03,.5976035E+03,.8794171E+03,.1162395E+04,&
     & .1036614E+04,.7859297E+03,.5962301E+03,.8776601E+03,.1160341E+04,&
     & .1036517E+04,.7857179E+03,.5947950E+03,.8759039E+03,.1158401E+04,&
     & .1036789E+04,.7858044E+03,.5997786E+03,.8824249E+03,.1165869E+04,&
     & .1036568E+04,.7858527E+03,.5986878E+03,.8808902E+03,.1164075E+04,&
     & .1036290E+04,.7856385E+03,.5973952E+03,.8791810E+03,.1162152E+04,&
     & .1036375E+04,.7857728E+03,.5960430E+03,.8773906E+03,.1160111E+04,&
     & .1036584E+04,.7856477E+03,.5946509E+03,.8756676E+03,.1158142E+04,&
     & .1036449E+04,.7856991E+03,.5997524E+03,.8823952E+03,.1165837E+04,&
     & .1036546E+04,.7855390E+03,.5986607E+03,.8808071E+03,.1164007E+04,&
     & .1036324E+04,.7856328E+03,.5973933E+03,.8791438E+03,.1162099E+04,&
     & .1036443E+04,.7855912E+03,.5960116E+03,.8773979E+03,.1160087E+04,&
     & .1036138E+04,.7855274E+03,.5946182E+03,.8754914E+03,.1158117E+04,&
     & .1036317E+04,.7854628E+03,.5998802E+03,.8825810E+03,.1166071E+04,&
     & .1036407E+04,.7855544E+03,.5988135E+03,.8810642E+03,.1164250E+04,&
     & .1036350E+04,.7855655E+03,.5975664E+03,.8793754E+03,.1162334E+04,&
     & .1036135E+04,.7854510E+03,.5961891E+03,.8776234E+03,.1160316E+04,&
     & .1036293E+04,.7854258E+03,.5947873E+03,.8758601E+03,.1158372E+04,&
     & .1036190E+04,.7854073E+03,.6000093E+03,.8827730E+03,.1166300E+04,&
     & .1036230E+04,.7852517E+03,.5989682E+03,.8812856E+03,.1164529E+04,&
     & .1035870E+04,.7851557E+03,.5977157E+03,.8796174E+03,.1162655E+04,&
     & .1036108E+04,.7852461E+03,.5963833E+03,.8777687E+03,.1160603E+04,&
     & .1035856E+04,.7854329E+03,.5949784E+03,.8761109E+03,.1158641E+04,&
     & .1036008E+04,.7854391E+03,.6001409E+03,.8829635E+03,.1166558E+04,&
     & .1035672E+04,.7852254E+03,.5991240E+03,.8815002E+03,.1164752E+04,&
     & .1035771E+04,.7851997E+03,.5979225E+03,.8798541E+03,.1162890E+04,&
     & .1035959E+04,.7853362E+03,.5965775E+03,.8780456E+03,.1160878E+04,&
     & .1036077E+04,.7852519E+03,.5951718E+03,.8763106E+03,.1158912E+04,&
     & .1036065E+04,.7853034E+03,.6003541E+03,.8832840E+03,.1166995E+04,&
     & .1036122E+04,.7852033E+03,.5993846E+03,.8818706E+03,.1165191E+04,&
     & .1035790E+04,.7853073E+03,.5982296E+03,.8802674E+03,.1163356E+04,&
     & .1035706E+04,.7852523E+03,.5969107E+03,.8785167E+03,.1161368E+04,&
     & .1035830E+04,.7852339E+03,.5955085E+03,.8767385E+03,.1159384E+04/

      data   absb(701:875, 6)   /                                       &
     & .1035691E+04,.7852423E+03,.6005711E+03,.8836138E+03,.1167419E+04,&
     & .1035908E+04,.7853193E+03,.5996475E+03,.8822488E+03,.1165664E+04,&
     & .1035762E+04,.7850877E+03,.5985371E+03,.8806907E+03,.1163865E+04,&
     & .1035976E+04,.7852798E+03,.5972262E+03,.8789734E+03,.1161918E+04,&
     & .1035858E+04,.7852361E+03,.5958649E+03,.8772263E+03,.1159899E+04,&
     & .1035722E+04,.7852842E+03,.6007813E+03,.8839238E+03,.1167804E+04,&
     & .1035719E+04,.7850227E+03,.5999002E+03,.8826192E+03,.1166124E+04,&
     & .1035600E+04,.7850937E+03,.5988393E+03,.8811085E+03,.1164303E+04,&
     & .1035738E+04,.7852072E+03,.5975958E+03,.8794250E+03,.1162408E+04,&
     & .1035762E+04,.7851633E+03,.5962227E+03,.8776696E+03,.1160304E+04,&
     & .1035752E+04,.7852392E+03,.6010174E+03,.8841948E+03,.1168330E+04,&
     & .1035945E+04,.7849973E+03,.6001604E+03,.8830466E+03,.1166677E+04,&
     & .1035925E+04,.7851444E+03,.5991890E+03,.8815961E+03,.1164885E+04,&
     & .1035817E+04,.7848046E+03,.5979682E+03,.8799616E+03,.1162996E+04,&
     & .1035850E+04,.7847913E+03,.5966615E+03,.8782015E+03,.1161017E+04,&
     & .1035638E+04,.7850513E+03,.6012543E+03,.8846111E+03,.1168741E+04,&
     & .1035730E+04,.7851337E+03,.6004922E+03,.8834894E+03,.1167254E+04,&
     & .1035445E+04,.7850312E+03,.5995458E+03,.8821086E+03,.1165365E+04,&
     & .1035700E+04,.7851973E+03,.5984185E+03,.8805353E+03,.1163672E+04,&
     & .1035789E+04,.7850241E+03,.5971269E+03,.8788018E+03,.1161708E+04,&
     & .1035632E+04,.7849920E+03,.6014692E+03,.8850013E+03,.1169266E+04,&
     & .1035755E+04,.7850265E+03,.6007663E+03,.8838589E+03,.1167803E+04,&
     & .1035534E+04,.7848868E+03,.5998828E+03,.8825962E+03,.1166100E+04,&
     & .1035605E+04,.7851301E+03,.5988206E+03,.8810847E+03,.1164299E+04,&
     & .1035576E+04,.7849624E+03,.5975789E+03,.8793971E+03,.1162393E+04,&
     & .1035448E+04,.7851267E+03,.6016773E+03,.8853457E+03,.1169825E+04,&
     & .1035833E+04,.7851690E+03,.6010423E+03,.8842300E+03,.1168330E+04,&
     & .1035814E+04,.7850287E+03,.6002225E+03,.8830915E+03,.1166729E+04,&
     & .1035463E+04,.7849992E+03,.5992224E+03,.8816502E+03,.1164925E+04,&
     & .1035596E+04,.7851550E+03,.5980443E+03,.8800202E+03,.1163074E+04,&
     & .1035621E+04,.7849593E+03,.6018789E+03,.8857046E+03,.1170389E+04,&
     & .1035804E+04,.7849848E+03,.6013182E+03,.8847579E+03,.1168918E+04,&
     & .1035665E+04,.7849816E+03,.6005690E+03,.8836138E+03,.1167420E+04,&
     & .1035718E+04,.7848933E+03,.5996475E+03,.8822566E+03,.1165667E+04,&
     & .1035684E+04,.7848079E+03,.5985373E+03,.8806947E+03,.1163864E+04/

      data   absb(876:1050, 6)   /                                      &
     & .1035396E+04,.7850994E+03,.6020651E+03,.8860430E+03,.1170959E+04,&
     & .1035443E+04,.7850165E+03,.6015627E+03,.8851600E+03,.1169522E+04,&
     & .1035416E+04,.7850543E+03,.6008940E+03,.8838713E+03,.1168065E+04,&
     & .1035678E+04,.7850493E+03,.6000376E+03,.8828269E+03,.1166399E+04,&
     & .1035656E+04,.7848387E+03,.5990054E+03,.8813440E+03,.1164574E+04,&
     & .1035799E+04,.7849568E+03,.6022304E+03,.8863649E+03,.1171665E+04,&
     & .1035662E+04,.7849150E+03,.6017841E+03,.8855314E+03,.1170121E+04,&
     & .1035790E+04,.7851167E+03,.6011844E+03,.8845494E+03,.1168673E+04,&
     & .1035751E+04,.7849497E+03,.6004028E+03,.8833168E+03,.1167013E+04,&
     & .1035765E+04,.7850336E+03,.5994082E+03,.8819645E+03,.1165343E+04,&
     & .1035223E+04,.7849097E+03,.6024415E+03,.8866859E+03,.1172339E+04,&
     & .1035610E+04,.7848771E+03,.6019601E+03,.8858592E+03,.1170680E+04,&
     & .1035292E+04,.7850288E+03,.6014302E+03,.8849412E+03,.1169154E+04,&
     & .1035255E+04,.7849051E+03,.6007179E+03,.8837423E+03,.1167730E+04,&
     & .1035724E+04,.7847308E+03,.5998232E+03,.8825148E+03,.1166003E+04,&
     & .1035548E+04,.7850328E+03,.6026138E+03,.8870664E+03,.1173037E+04,&
     & .1035703E+04,.7849473E+03,.6021374E+03,.8861617E+03,.1171252E+04,&
     & .1035663E+04,.7850676E+03,.6016439E+03,.8852969E+03,.1169745E+04,&
     & .1035770E+04,.7849775E+03,.6010002E+03,.8842177E+03,.1168267E+04,&
     & .1035731E+04,.7851009E+03,.6001688E+03,.8830217E+03,.1166634E+04,&
     & .1035751E+04,.7850608E+03,.6027803E+03,.8874485E+03,.1173984E+04,&
     & .1035764E+04,.7849860E+03,.6023128E+03,.8864542E+03,.1171860E+04,&
     & .1035482E+04,.7850930E+03,.6018352E+03,.8856260E+03,.1170255E+04,&
     & .1035535E+04,.7849801E+03,.6012570E+03,.8846634E+03,.1168815E+04,&
     & .1035673E+04,.7849343E+03,.6004942E+03,.8834988E+03,.1167257E+04,&
     & .1035524E+04,.7848552E+03,.6029106E+03,.8880716E+03,.1174957E+04,&
     & .1035658E+04,.7846826E+03,.6024834E+03,.8867342E+03,.1172519E+04,&
     & .1035489E+04,.7848135E+03,.6020058E+03,.8859421E+03,.1170818E+04,&
     & .1035424E+04,.7848886E+03,.6014905E+03,.8850368E+03,.1169362E+04,&
     & .1035663E+04,.7850003E+03,.6007966E+03,.8839492E+03,.1167867E+04,&
     & .1035504E+04,.7848111E+03,.6031050E+03,.8887172E+03,.1175944E+04,&
     & .1035279E+04,.7847971E+03,.6026411E+03,.8871385E+03,.1173254E+04,&
     & .1035415E+04,.7850072E+03,.6021673E+03,.8862095E+03,.1171363E+04,&
     & .1035656E+04,.7850409E+03,.6016799E+03,.8853554E+03,.1169825E+04,&
     & .1035406E+04,.7848736E+03,.6010491E+03,.8843357E+03,.1168391E+04/

      data   absb(1051:1175, 6)   /                                     &
     & .1035713E+04,.7849729E+03,.6032946E+03,.8893995E+03,.1176941E+04,&
     & .1035444E+04,.7849707E+03,.6027864E+03,.8875237E+03,.1174025E+04,&
     & .1035565E+04,.7850045E+03,.6023190E+03,.8864702E+03,.1171867E+04,&
     & .1035712E+04,.7849606E+03,.6018460E+03,.8856449E+03,.1170317E+04,&
     & .1035707E+04,.7850798E+03,.6012695E+03,.8846865E+03,.1168843E+04,&
     & .1035705E+04,.7849167E+03,.6035391E+03,.8900967E+03,.1177953E+04,&
     & .1035730E+04,.7849972E+03,.6028979E+03,.8880295E+03,.1174870E+04,&
     & .1035405E+04,.7849160E+03,.6024731E+03,.8867542E+03,.1172461E+04,&
     & .1035544E+04,.7850359E+03,.6019902E+03,.8859187E+03,.1170739E+04,&
     & .1035719E+04,.7850739E+03,.6014757E+03,.8850119E+03,.1169315E+04,&
     & .1035505E+04,.7850447E+03,.6039271E+03,.8909936E+03,.1179187E+04,&
     & .1035517E+04,.7849136E+03,.6030826E+03,.8886380E+03,.1175823E+04,&
     & .1035643E+04,.7849538E+03,.6026205E+03,.8870923E+03,.1173152E+04,&
     & .1035536E+04,.7849523E+03,.6021490E+03,.8861784E+03,.1171261E+04,&
     & .1035534E+04,.7850334E+03,.6016601E+03,.8853177E+03,.1169773E+04,&
     & .1035546E+04,.7850318E+03,.6043242E+03,.8919185E+03,.1180418E+04,&
     & .1035602E+04,.7849911E+03,.6032555E+03,.8892747E+03,.1176756E+04,&
     & .1035555E+04,.7849176E+03,.6027602E+03,.8874056E+03,.1173878E+04,&
     & .1035566E+04,.7849074E+03,.6022939E+03,.8864253E+03,.1171787E+04,&
     & .1035501E+04,.7847965E+03,.6018145E+03,.8855936E+03,.1170187E+04,&
     & .1035450E+04,.7849937E+03,.6045469E+03,.8923132E+03,.1180947E+04,&
     & .1035470E+04,.7849930E+03,.6033415E+03,.8895406E+03,.1177165E+04,&
     & .1035648E+04,.7849151E+03,.6028185E+03,.8876106E+03,.1174216E+04,&
     & .1035655E+04,.7850350E+03,.6023184E+03,.8865228E+03,.1172007E+04,&
     & .1035589E+04,.7850346E+03,.6018757E+03,.8856974E+03,.1170389E+04/

!........................................!
      end module module_radsw_kgb28      !
!========================================!


!========================================!
      module module_radsw_kgb29          !
!........................................!
!
! *********        the original program descriptions        ********* !
!                                                                     !
!     originally by j.delamere, atmospheric & environmental research. !
!     revision: 2.4                                                   !
!     band 29:   820-2600 cm-1 (low - h2o; high - co2)                !
!     reformatted for f90 by jjmorcrette, ecmwf                       !
!                                                                     !
!     this table has been re-generated for reduced number of g-point  !
!     by y.t.hou, ncep                                                !
!                                                                     !
! *********    *********    end description    *********    ********* !
!
      use physparam,               only : kind_phys
      use module_radsw_parameters, only : NG29

!
      implicit none
!
      private
!
      integer,  public :: MSA29, MSB29, MSF29, MFR29
      parameter (MSA29=65, MSB29=235, MSF29=10, MFR29=4)

      real (kind=kind_phys), public ::           forref(MFR29,NG29),    &
     &       absa(MSA29,NG29), absb(MSB29,NG29), selfref(MSF29,NG29),   &
     &       absh2o(NG29), absco2(NG29)

!  ---  rayleigh extinction coefficient at v = 2200 cm-1.
      real (kind=kind_phys), parameter, public :: rayl = 9.30e-11

!  --- h2o
      data   absh2o (1:12)       /            .2995080E-03,.3950120E-02,&
     & .1493160E-01,.3243840E-01,.9440181E-01,.1006542E+01,.9383158E+01,&
     & .2134138E+00,.2155620E+00,.2180870E+00,.2209180E+00,.2185460E+00/

!  --- co2
      data   absco2 (1:12)       /            .2900730E-05,.2123820E-04,&
     & .1030320E-03,.1864810E-03,.5136065E-03,.2118687E-01,.4146680E+01,&
     & .4301567E+02,.1641290E+03,.8322820E+03,.4995020E+04,.1267810E+05/

!     the array absa(65,NG29) (ka(5,13,NG29)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels> ~100mb,
!     temperatures, and binary species parameters (see taumol.f for definition).
!     the first index in the array, js, runs from 1 to 9, and corresponds to
!     different values of the binary species parameter.  for instance,
!     js=1 refers to dry air, js = 2 corresponds to the paramter value 1/8,
!     js = 3 corresponds to the parameter value 2/8, etc.  the second index
!     in the array, jt, which runs from 1 to 5, corresponds to different
!     temperatures.  more specifically, jt = 3 means that the data are for
!     the reference temperature tref for this  pressure level, jt = 2 refers
!     to tref-15, jt = 1 is for tref-30, jt = 4 is for tref+15, and jt = 5
!     is for tref+30.  the third index, jp, runs from 1 to 13 and refers
!     to the jpth reference pressure level (see taumol.f for these levels
!     in mb).  the fourth index, ig, goes from 1 to 12, and indicates
!     which g-interval the absorption coefficients are for.

      data   absa(  1: 65, 1)   /                                       &
     & .1156500E-03,.1012300E-03,.9080400E-04,.8228200E-04,.7108300E-04,&
     & .9643400E-04,.8283000E-04,.7236600E-04,.6180300E-04,.5249700E-04,&
     & .6453900E-04,.5665900E-04,.4660500E-04,.3981500E-04,.3711800E-04,&
     & .3441700E-04,.2711300E-04,.2536200E-04,.3034500E-04,.3627500E-04,&
     & .1226000E-04,.1585600E-04,.2083400E-04,.2736300E-04,.3611400E-04,&
     & .1022100E-04,.1459800E-04,.1969500E-04,.2597600E-04,.3370200E-04,&
     & .9756300E-05,.1380900E-04,.2023100E-04,.2723800E-04,.3611000E-04,&
     & .1406200E-04,.1958700E-04,.2706900E-04,.3693700E-04,.4941500E-04,&
     & .3637100E-04,.4812200E-04,.6158600E-04,.7764700E-04,.9989700E-04,&
     & .9920300E-04,.1284200E-03,.1658800E-03,.2083400E-03,.2600000E-03,&
     & .1323300E-03,.1731800E-03,.2205900E-03,.2890400E-03,.3606200E-03,&
     & .1337900E-03,.1748400E-03,.2368700E-03,.3028600E-03,.3750400E-03,&
     & .1174000E-03,.1566700E-03,.2096200E-03,.2676800E-03,.3348500E-03/

      data   absa(  1: 65, 2)   /                                       &
     & .1024600E-03,.1045000E-03,.9738300E-04,.9639800E-04,.1054900E-03,&
     & .1058900E-03,.1024000E-03,.9580100E-04,.9850900E-04,.1199300E-03,&
     & .9405400E-04,.8700900E-04,.1094100E-03,.1348600E-03,.1591800E-03,&
     & .1188300E-03,.1423600E-03,.1663600E-03,.1823500E-03,.1978500E-03,&
     & .1780000E-03,.1934700E-03,.2097700E-03,.2273000E-03,.2511100E-03,&
     & .2224300E-03,.2415700E-03,.2656700E-03,.2854900E-03,.3072300E-03,&
     & .2924800E-03,.3224200E-03,.3463500E-03,.3791500E-03,.4080300E-03,&
     & .4338600E-03,.4861100E-03,.5268100E-03,.5681200E-03,.6064200E-03,&
     & .8910900E-03,.1034500E-02,.1179400E-02,.1304500E-02,.1430300E-02,&
     & .2153800E-02,.2445900E-02,.2732900E-02,.3093200E-02,.3525300E-02,&
     & .2927200E-02,.3267600E-02,.3635300E-02,.4006200E-02,.4544100E-02,&
     & .3076200E-02,.3436500E-02,.3814600E-02,.4155600E-02,.4716400E-02,&
     & .2780800E-02,.3111400E-02,.3438300E-02,.3795400E-02,.4257600E-02/

      data   absa(  1: 65, 3)   /                                       &
     & .2404200E-03,.3271900E-03,.4437000E-03,.5683600E-03,.7035700E-03,&
     & .2330300E-03,.3190100E-03,.4192600E-03,.5240000E-03,.6236500E-03,&
     & .3505000E-03,.4215800E-03,.4749000E-03,.5339000E-03,.6041100E-03,&
     & .4995400E-03,.5306700E-03,.5633800E-03,.6085900E-03,.6656900E-03,&
     & .6690800E-03,.6972700E-03,.7289800E-03,.7675600E-03,.8035800E-03,&
     & .8863400E-03,.9285300E-03,.9685600E-03,.1010100E-02,.1055600E-02,&
     & .1165900E-02,.1235500E-02,.1323800E-02,.1388900E-02,.1449100E-02,&
     & .1748500E-02,.1822300E-02,.1941100E-02,.2070500E-02,.2202500E-02,&
     & .4244200E-02,.4356100E-02,.4506100E-02,.4750500E-02,.5035800E-02,&
     & .1094000E-01,.1149900E-01,.1190600E-01,.1257000E-01,.1280300E-01,&
     & .1428700E-01,.1501000E-01,.1558100E-01,.1605400E-01,.1660900E-01,&
     & .1485600E-01,.1554600E-01,.1607400E-01,.1647800E-01,.1705700E-01,&
     & .1325700E-01,.1383400E-01,.1415500E-01,.1472300E-01,.1509500E-01/

      data   absa(  1: 65, 4)   /                                       &
     & .2439100E-02,.2872000E-02,.3349700E-02,.3892600E-02,.4505400E-02,&
     & .2250600E-02,.2600400E-02,.3000200E-02,.3449000E-02,.3957900E-02,&
     & .2215300E-02,.2571100E-02,.2932000E-02,.3333000E-02,.3772800E-02,&
     & .2248300E-02,.2544500E-02,.2874500E-02,.3248400E-02,.3655400E-02,&
     & .2338800E-02,.2598500E-02,.2891400E-02,.3237700E-02,.3611300E-02,&
     & .2466900E-02,.2657900E-02,.2883400E-02,.3153800E-02,.3459300E-02,&
     & .3253600E-02,.3341300E-02,.3437500E-02,.3590500E-02,.3801000E-02,&
     & .5122800E-02,.5196700E-02,.5282000E-02,.5344600E-02,.5379700E-02,&
     & .1302900E-01,.1306500E-01,.1289100E-01,.1284800E-01,.1277700E-01,&
     & .2991100E-01,.3011700E-01,.2834000E-01,.2732100E-01,.2748500E-01,&
     & .3666300E-01,.3687700E-01,.3468800E-01,.3300400E-01,.3243700E-01,&
     & .3728200E-01,.3777500E-01,.3491000E-01,.3346300E-01,.3307400E-01,&
     & .3341200E-01,.3296100E-01,.3032300E-01,.2943700E-01,.2957200E-01/

      data   absa(  1: 65, 5)   /                                       &
     & .5737669E-01,.6072679E-01,.6409123E-01,.6738949E-01,.7052420E-01,&
     & .5012852E-01,.5294581E-01,.5563579E-01,.5843403E-01,.6127801E-01,&
     & .4354787E-01,.4609744E-01,.4874155E-01,.5138444E-01,.5397080E-01,&
     & .3827336E-01,.4064492E-01,.4287087E-01,.4511010E-01,.4735934E-01,&
     & .3418365E-01,.3615136E-01,.3817782E-01,.4009673E-01,.4202725E-01,&
     & .3135060E-01,.3301048E-01,.3465038E-01,.3632036E-01,.3815047E-01,&
     & .2990607E-01,.3131211E-01,.3276228E-01,.3421907E-01,.3588465E-01,&
     & .3047960E-01,.3187197E-01,.3323626E-01,.3465826E-01,.3619150E-01,&
     & .4306469E-01,.4376339E-01,.4465622E-01,.4556160E-01,.4646970E-01,&
     & .8678394E-01,.8535671E-01,.8559355E-01,.8558578E-01,.8553480E-01,&
     & .1070789E+00,.1044012E+00,.1040583E+00,.1040024E+00,.1036748E+00,&
     & .1081364E+00,.1052681E+00,.1050326E+00,.1048529E+00,.1046392E+00,&
     & .9356527E-01,.9192786E-01,.9227488E-01,.9212577E-01,.9173403E-01/

      data   absa(  1: 65, 6)   /                                       &
     & .5500343E+00,.5549109E+00,.5588616E+00,.5629779E+00,.5667564E+00,&
     & .4994114E+00,.5037790E+00,.5100008E+00,.5169978E+00,.5240736E+00,&
     & .5046611E+00,.5105279E+00,.5158277E+00,.5226637E+00,.5303378E+00,&
     & .5302528E+00,.5393967E+00,.5458119E+00,.5537515E+00,.5618237E+00,&
     & .5411203E+00,.5556023E+00,.5677558E+00,.5802746E+00,.5898268E+00,&
     & .5252486E+00,.5447502E+00,.5651635E+00,.5824674E+00,.6014351E+00,&
     & .5100890E+00,.5352169E+00,.5583076E+00,.5833005E+00,.6058393E+00,&
     & .5180699E+00,.5497814E+00,.5808233E+00,.6108346E+00,.6398736E+00,&
     & .6522488E+00,.6891866E+00,.7325459E+00,.7797722E+00,.8260960E+00,&
     & .9640811E+00,.1024659E+01,.1093085E+01,.1161786E+01,.1235652E+01,&
     & .1047820E+01,.1114446E+01,.1186733E+01,.1258652E+01,.1334983E+01,&
     & .1011033E+01,.1074570E+01,.1140350E+01,.1211241E+01,.1280754E+01,&
     & .9007538E+00,.9506568E+00,.1005449E+01,.1058464E+01,.1116021E+01/

      data   absa(  1: 65, 7)   /                                       &
     & .4649358E+01,.4659442E+01,.4675452E+01,.4685713E+01,.4692899E+01,&
     & .4599705E+01,.4611058E+01,.4611629E+01,.4612755E+01,.4607683E+01,&
     & .4619900E+01,.4611704E+01,.4611644E+01,.4599592E+01,.4592568E+01,&
     & .4687320E+01,.4681368E+01,.4668829E+01,.4653209E+01,.4637546E+01,&
     & .4851120E+01,.4815554E+01,.4796404E+01,.4771634E+01,.4773910E+01,&
     & .5130952E+01,.5103158E+01,.5076979E+01,.5135909E+01,.5190723E+01,&
     & .5614324E+01,.5655787E+01,.5731968E+01,.5785423E+01,.5820571E+01,&
     & .6802466E+01,.6894140E+01,.6919677E+01,.6955254E+01,.6983671E+01,&
     & .9510364E+01,.9545777E+01,.9589682E+01,.9667897E+01,.9623013E+01,&
     & .9447407E+01,.9253876E+01,.9037997E+01,.8820753E+01,.8587497E+01,&
     & .9261615E+01,.9040240E+01,.8802081E+01,.8566366E+01,.8317559E+01,&
     & .9408423E+01,.9193385E+01,.8974202E+01,.8739541E+01,.8508727E+01,&
     & .9731443E+01,.9563647E+01,.9382645E+01,.9207221E+01,.9018631E+01/

      data   absa(  1: 65, 8)   /                                       &
     & .1544733E+02,.1554976E+02,.1552859E+02,.1550737E+02,.1552125E+02,&
     & .1738703E+02,.1741986E+02,.1747727E+02,.1744027E+02,.1739669E+02,&
     & .1987281E+02,.1989151E+02,.1998931E+02,.1999712E+02,.1998411E+02,&
     & .2258389E+02,.2260003E+02,.2268696E+02,.2264651E+02,.2270608E+02,&
     & .2504125E+02,.2510226E+02,.2517712E+02,.2519913E+02,.2495283E+02,&
     & .2747847E+02,.2717648E+02,.2695792E+02,.2626421E+02,.2537969E+02,&
     & .2961485E+02,.2896370E+02,.2795835E+02,.2715755E+02,.2643866E+02,&
     & .2732825E+02,.2612667E+02,.2538192E+02,.2458336E+02,.2384624E+02,&
     & .5753079E+01,.4812161E+01,.3663267E+01,.2188469E+01,.1578210E+01,&
     & .1936683E+00,.1796201E+00,.1673070E+00,.1564801E+00,.1466219E+00,&
     & .2745909E+00,.2544928E+00,.2369058E+00,.2210443E+00,.2070125E+00,&
     & .2913703E+00,.2698520E+00,.2503822E+00,.2335086E+00,.2184884E+00,&
     & .2490616E+00,.2300611E+00,.2134111E+00,.1990053E+00,.1862957E+00/

      data   absa(  1: 65, 9)   /                                       &
     & .2967200E+02,.2929100E+02,.2919100E+02,.2917000E+02,.2911600E+02,&
     & .2471300E+02,.2496500E+02,.2503900E+02,.2535500E+02,.2565000E+02,&
     & .3051000E+02,.3116600E+02,.3066300E+02,.3102100E+02,.3090100E+02,&
     & .3869500E+02,.3892200E+02,.3874100E+02,.3920400E+02,.3882600E+02,&
     & .4511800E+02,.4433700E+02,.4366400E+02,.4326700E+02,.4352400E+02,&
     & .3465200E+02,.3518200E+02,.3533900E+02,.3534000E+02,.3589900E+02,&
     & .1880100E+02,.1859800E+02,.1885500E+02,.1881900E+02,.1905200E+02,&
     & .1229700E-01,.1142500E-01,.1067000E-01,.9999400E-02,.9413900E-02,&
     & .4682100E-01,.4336100E-01,.4036800E-01,.3775200E-01,.3549300E-01,&
     & .1981700E+00,.1830300E+00,.1700200E+00,.1588300E+00,.1493300E+00,&
     & .2814500E+00,.2599500E+00,.2418100E+00,.2266000E+00,.2132600E+00,&
     & .2951000E+00,.2729400E+00,.2551900E+00,.2394700E+00,.2255100E+00,&
     & .2483000E+00,.2307400E+00,.2155600E+00,.2022600E+00,.1902900E+00/

      data   absa(  1: 65,10)   /                                       &
     & .4716800E+02,.4689000E+02,.4661200E+02,.4635300E+02,.4608800E+02,&
     & .4777100E+02,.4698000E+02,.4644500E+02,.4582800E+02,.4543700E+02,&
     & .4294600E+02,.4265200E+02,.4379100E+02,.4323700E+02,.4410400E+02,&
     & .3095700E+02,.3009800E+02,.3005500E+02,.2955500E+02,.2990200E+02,&
     & .2839700E+01,.3857200E+01,.4290600E+01,.4709100E+01,.4813500E+01,&
     & .3697800E-02,.3459100E-02,.3252400E-02,.3064900E-02,.2901800E-02,&
     & .6173300E-02,.5758900E-02,.5392500E-02,.5074200E-02,.4783500E-02,&
     & .1239000E-01,.1152600E-01,.1076600E-01,.1009600E-01,.9501000E-02,&
     & .4710500E-01,.4364800E-01,.4066500E-01,.3805400E-01,.3572200E-01,&
     & .1991500E+00,.1841200E+00,.1709900E+00,.1595700E+00,.1494800E+00,&
     & .2828000E+00,.2612400E+00,.2426600E+00,.2264500E+00,.2121400E+00,&
     & .2989100E+00,.2761300E+00,.2564800E+00,.2392700E+00,.2242300E+00,&
     & .2542100E+00,.2348400E+00,.2180900E+00,.2034700E+00,.1905700E+00/

      data   absa(  1: 65,11)   /                                       &
     & .6499400E+02,.6428300E+02,.6375500E+02,.6340700E+02,.6328700E+02,&
     & .7826600E+02,.7736400E+02,.7672200E+02,.7628500E+02,.7599200E+02,&
     & .4171000E+02,.3837900E+02,.3507600E+02,.3541800E+02,.3330600E+02,&
     & .1417000E-02,.1340100E-02,.1271600E-02,.1207300E-02,.1153500E-02,&
     & .2365900E-02,.2226100E-02,.2104200E-02,.1995000E-02,.1897000E-02,&
     & .3743200E-02,.3510500E-02,.3300600E-02,.3122200E-02,.2952100E-02,&
     & .6240800E-02,.5825800E-02,.5478200E-02,.5159400E-02,.4876700E-02,&
     & .1252200E-01,.1165200E-01,.1092200E-01,.1026500E-01,.9667200E-02,&
     & .4754500E-01,.4415200E-01,.4125100E-01,.3866600E-01,.3635800E-01,&
     & .2009700E+00,.1859200E+00,.1734100E+00,.1620300E+00,.1523500E+00,&
     & .2849900E+00,.2642800E+00,.2458500E+00,.2298000E+00,.2157500E+00,&
     & .3017200E+00,.2792900E+00,.2598000E+00,.2430600E+00,.2278900E+00,&
     & .2565500E+00,.2373100E+00,.2209200E+00,.2063900E+00,.1936700E+00/

      data   absa(  1: 65,12)   /                                       &
     & .8081000E+02,.8109900E+02,.8119000E+02,.8110700E+02,.8098900E+02,&
     & .9931900E+02,.9970800E+02,.9982200E+02,.9987100E+02,.9999300E+02,&
     & .4692700E+02,.5431600E+02,.5735500E+02,.5371500E+02,.5280200E+02,&
     & .1280200E-02,.1202700E-02,.1138600E-02,.1086600E-02,.1052500E-02,&
     & .2167500E-02,.2071500E-02,.1942200E-02,.1840200E-02,.1768100E-02,&
     & .3470700E-02,.3279600E-02,.3098600E-02,.2924600E-02,.2803400E-02,&
     & .5865900E-02,.5531000E-02,.5182000E-02,.4882900E-02,.4678400E-02,&
     & .1187500E-01,.1118300E-01,.1043800E-01,.9803800E-02,.9366500E-02,&
     & .4551900E-01,.4285000E-01,.3971500E-01,.3744700E-01,.3536900E-01,&
     & .1939200E+00,.1810800E+00,.1681700E+00,.1588300E+00,.1486400E+00,&
     & .2783000E+00,.2571500E+00,.2410000E+00,.2259500E+00,.2132100E+00,&
     & .2953300E+00,.2742200E+00,.2563600E+00,.2393600E+00,.2265800E+00,&
     & .2531000E+00,.2353500E+00,.2185500E+00,.2055200E+00,.1929300E+00/

!     the array absb(235,12) (kb(5,13:59,12)) contains absorption coefs at
!     the 16 chosen g-values for a range of pressure levels < ~100mb and
!     temperatures. the first index in the array, jt, which runs from 1 to 5,
!     corresponds to different temperatures.  more specifically, jt = 3 means
!     that the data are for the reference temperature tref for this pressure
!     level, jt = 2 refers to the temperature tref-15, jt = 1 is for
!     tref-30, jt = 4 is for tref+15, and jt = 5 is for tref+30.
!     the second index, jp, runs from 13 to 59 and refers to the jpth
!     reference pressure level (see taumol.f for the value of these
!     pressure levels in mb).  the third index, ig, goes from 1 to 12,
!     and tells us which g-interval the absorption coefficients are for.

      data   absb(  1:120, 1)   /                                       &
     & .1837900E-05,.2329600E-05,.2900700E-05,.3590200E-05,.4343700E-05,&
     & .1591900E-05,.1983200E-05,.2472000E-05,.3068300E-05,.3725300E-05,&
     & .1385000E-05,.1711500E-05,.2122500E-05,.2629200E-05,.3194500E-05,&
     & .1189600E-05,.1468000E-05,.1814200E-05,.2253700E-05,.2728400E-05,&
     & .1022800E-05,.1259700E-05,.1553200E-05,.1927200E-05,.2316300E-05,&
     & .8824300E-06,.1083500E-05,.1331600E-05,.1660300E-05,.1980800E-05,&
     & .7567700E-06,.9292500E-06,.1140100E-05,.1410700E-05,.1690100E-05,&
     & .6461400E-06,.7936100E-06,.9735300E-06,.1196400E-05,.1441400E-05,&
     & .5532300E-06,.6758000E-06,.8264700E-06,.1013700E-05,.1231800E-05,&
     & .4771900E-06,.5814900E-06,.7106000E-06,.8700500E-06,.1060500E-05,&
     & .4112800E-06,.5008000E-06,.6110400E-06,.7534700E-06,.9120000E-06,&
     & .3530600E-06,.4286000E-06,.5232000E-06,.6515600E-06,.7812200E-06,&
     & .3036300E-06,.3673100E-06,.4467300E-06,.5609700E-06,.6679100E-06,&
     & .2607300E-06,.3153700E-06,.3873800E-06,.4821000E-06,.5738200E-06,&
     & .2230300E-06,.2698600E-06,.3362800E-06,.4116100E-06,.4872200E-06,&
     & .1895400E-06,.2295200E-06,.2879700E-06,.3516600E-06,.4144000E-06,&
     & .1610600E-06,.1943000E-06,.2466100E-06,.2981500E-06,.3515100E-06,&
     & .1363000E-06,.1652800E-06,.2092200E-06,.2519300E-06,.2991300E-06,&
     & .1150400E-06,.1427600E-06,.1780700E-06,.2148200E-06,.2545100E-06,&
     & .9698000E-07,.1221500E-06,.1515900E-06,.1817900E-06,.2151500E-06,&
     & .8159800E-07,.1031500E-06,.1273000E-06,.1519000E-06,.1809300E-06,&
     & .6770000E-07,.8597000E-07,.1055900E-06,.1260800E-06,.1500600E-06,&
     & .5517700E-07,.6951900E-07,.8596700E-07,.1022300E-06,.1214700E-06,&
     & .4392400E-07,.5439500E-07,.6799800E-07,.8114200E-07,.9571700E-07/

      data   absb(121:235, 1)   /                                       &
     & .3446900E-07,.4103600E-07,.5257500E-07,.6311700E-07,.7430100E-07,&
     & .2677700E-07,.3173400E-07,.4055500E-07,.4903200E-07,.5765300E-07,&
     & .2067000E-07,.2472100E-07,.3068500E-07,.3787300E-07,.4455500E-07,&
     & .1586000E-07,.1903200E-07,.2261200E-07,.2884500E-07,.3424600E-07,&
     & .1212900E-07,.1460300E-07,.1735700E-07,.2185100E-07,.2634600E-07,&
     & .9231900E-08,.1119900E-07,.1336500E-07,.1617400E-07,.2001900E-07,&
     & .6928900E-08,.8453700E-08,.1015900E-07,.1199200E-07,.1494000E-07,&
     & .5154600E-08,.6338000E-08,.7659300E-08,.9056200E-08,.1071600E-07,&
     & .3833500E-08,.4708500E-08,.5721900E-08,.6814800E-08,.7951800E-08,&
     & .2858300E-08,.3472400E-08,.4244700E-08,.5088800E-08,.5949400E-08,&
     & .2094500E-08,.2541800E-08,.3105800E-08,.3749400E-08,.4427400E-08,&
     & .1516400E-08,.1856000E-08,.2262800E-08,.2737100E-08,.3277300E-08,&
     & .1091600E-08,.1346500E-08,.1638700E-08,.1974000E-08,.2386500E-08,&
     & .7832400E-09,.9836100E-09,.1197700E-08,.1451700E-08,.1745700E-08,&
     & .5726300E-09,.7178600E-09,.8768900E-09,.1070500E-08,.1288100E-08,&
     & .4157400E-09,.5126400E-09,.6415200E-09,.7831500E-09,.9470500E-09,&
     & .2994000E-09,.3683600E-09,.4588600E-09,.5640500E-09,.6916600E-09,&
     & .2261700E-09,.2739500E-09,.3375400E-09,.4209500E-09,.5154400E-09,&
     & .1748300E-09,.2084500E-09,.2554600E-09,.3181700E-09,.3911900E-09,&
     & .1349600E-09,.1593000E-09,.1938000E-09,.2386100E-09,.2977000E-09,&
     & .1040600E-09,.1224700E-09,.1474900E-09,.1807800E-09,.2259700E-09,&
     & .8134500E-10,.9481600E-10,.1132900E-09,.1383200E-09,.1716100E-09,&
     & .6715900E-10,.7831200E-10,.9474500E-10,.1154400E-09,.1444200E-09/

      data   absb(  1:120, 2)   /                                       &
     & .1665400E-04,.1872800E-04,.2123800E-04,.2449100E-04,.2903800E-04,&
     & .1563600E-04,.1757400E-04,.1991800E-04,.2367100E-04,.2723700E-04,&
     & .1485300E-04,.1662900E-04,.1970000E-04,.2260400E-04,.2558600E-04,&
     & .1351900E-04,.1592300E-04,.1828900E-04,.2053000E-04,.2279900E-04,&
     & .1241100E-04,.1443900E-04,.1643100E-04,.1823500E-04,.2010900E-04,&
     & .1126700E-04,.1288000E-04,.1442900E-04,.1600500E-04,.1753800E-04,&
     & .1002500E-04,.1133200E-04,.1255000E-04,.1385300E-04,.1510300E-04,&
     & .8634200E-05,.9745500E-05,.1077800E-04,.1189800E-04,.1278900E-04,&
     & .7400400E-05,.8310700E-05,.9163300E-05,.1007100E-04,.1067400E-04,&
     & .6370000E-05,.7054900E-05,.7827100E-05,.8517000E-05,.9002200E-05,&
     & .5440400E-05,.6015600E-05,.6639300E-05,.7132000E-05,.7500700E-05,&
     & .4599700E-05,.5071800E-05,.5571300E-05,.5945500E-05,.6211800E-05,&
     & .3874500E-05,.4261700E-05,.4647900E-05,.4929700E-05,.5163800E-05,&
     & .3248800E-05,.3559200E-05,.3868500E-05,.4075900E-05,.4309200E-05,&
     & .2699500E-05,.2960100E-05,.3206500E-05,.3366600E-05,.3560900E-05,&
     & .2235900E-05,.2442900E-05,.2640800E-05,.2775200E-05,.2929300E-05,&
     & .1848700E-05,.2008900E-05,.2166800E-05,.2279300E-05,.2402900E-05,&
     & .1529100E-05,.1658900E-05,.1773300E-05,.1870600E-05,.1958400E-05,&
     & .1255400E-05,.1358900E-05,.1443600E-05,.1519500E-05,.1592800E-05,&
     & .1024800E-05,.1104600E-05,.1171600E-05,.1228800E-05,.1291600E-05,&
     & .8327200E-06,.8912000E-06,.9503600E-06,.9904000E-06,.1040900E-05,&
     & .6751800E-06,.7209700E-06,.7685400E-06,.8024600E-06,.8391200E-06,&
     & .5357700E-06,.5787000E-06,.6171500E-06,.6498700E-06,.6722000E-06,&
     & .4240200E-06,.4582500E-06,.4930700E-06,.5200500E-06,.5402900E-06/

      data   absb(121:235, 2)   /                                       &
     & .3341900E-06,.3621300E-06,.3891100E-06,.4135900E-06,.4317700E-06,&
     & .2624000E-06,.2835500E-06,.3053000E-06,.3288100E-06,.3461700E-06,&
     & .2046800E-06,.2224500E-06,.2398000E-06,.2577600E-06,.2742400E-06,&
     & .1607000E-06,.1747800E-06,.1885600E-06,.2023000E-06,.2180100E-06,&
     & .1258300E-06,.1368000E-06,.1485100E-06,.1591700E-06,.1718400E-06,&
     & .9849400E-07,.1063900E-06,.1163400E-06,.1250000E-06,.1349000E-06,&
     & .7519600E-07,.8280900E-07,.9009500E-07,.9795200E-07,.1050900E-06,&
     & .5730500E-07,.6378700E-07,.6922700E-07,.7563500E-07,.8168700E-07,&
     & .4296500E-07,.4824500E-07,.5334600E-07,.5806100E-07,.6312000E-07,&
     & .3104800E-07,.3645700E-07,.4051400E-07,.4455700E-07,.4855000E-07,&
     & .2309400E-07,.2635600E-07,.3044300E-07,.3395400E-07,.3678900E-07,&
     & .1685800E-07,.1940200E-07,.2244900E-07,.2540100E-07,.2825400E-07,&
     & .1141300E-07,.1405500E-07,.1604600E-07,.1892700E-07,.2108000E-07,&
     & .7865000E-08,.9987900E-08,.1184600E-07,.1365400E-07,.1587100E-07,&
     & .5541700E-08,.6897700E-08,.8700600E-08,.1010800E-07,.1182900E-07,&
     & .3914300E-08,.4802500E-08,.6018000E-08,.7376200E-08,.8585100E-08,&
     & .2763700E-08,.3339700E-08,.4199600E-08,.5278700E-08,.6292300E-08,&
     & .1969100E-08,.2452900E-08,.3007100E-08,.3804100E-08,.4698100E-08,&
     & .1303400E-08,.1825700E-08,.2235900E-08,.2821000E-08,.3538600E-08,&
     & .1002600E-08,.1342200E-08,.1668700E-08,.2079300E-08,.2605400E-08,&
     & .7773000E-09,.9119000E-09,.1246800E-08,.1530200E-08,.1941800E-08,&
     & .6035900E-09,.6784800E-09,.9241400E-09,.1151100E-08,.1440100E-08,&
     & .5044100E-09,.5679700E-09,.7736900E-09,.9561200E-09,.1195700E-08/

      data   absb(  1:120, 3)   /                                       &
     & .9812400E-04,.1014900E-03,.1030300E-03,.1066900E-03,.1087000E-03,&
     & .8811000E-04,.8973000E-04,.9067900E-04,.9189200E-04,.9141300E-04,&
     & .7855500E-04,.7863300E-04,.7914500E-04,.7768900E-04,.7779100E-04,&
     & .6724500E-04,.6774300E-04,.6669000E-04,.6576500E-04,.6595000E-04,&
     & .5755300E-04,.5749000E-04,.5601200E-04,.5563400E-04,.5601600E-04,&
     & .4972900E-04,.4830200E-04,.4741800E-04,.4726200E-04,.4773200E-04,&
     & .4210100E-04,.4090600E-04,.4054100E-04,.4022900E-04,.4067100E-04,&
     & .3504600E-04,.3419300E-04,.3403200E-04,.3388100E-04,.3469100E-04,&
     & .2919200E-04,.2881300E-04,.2852500E-04,.2848500E-04,.2976400E-04,&
     & .2421200E-04,.2397000E-04,.2380400E-04,.2394700E-04,.2431000E-04,&
     & .2016700E-04,.1996100E-04,.1983700E-04,.2052800E-04,.1988700E-04,&
     & .1681700E-04,.1662400E-04,.1652500E-04,.1726300E-04,.1648100E-04,&
     & .1389600E-04,.1380600E-04,.1381000E-04,.1434800E-04,.1363400E-04,&
     & .1151500E-04,.1147400E-04,.1155300E-04,.1170500E-04,.1116700E-04,&
     & .9500400E-05,.9454300E-05,.9565000E-05,.9531600E-05,.9165100E-05,&
     & .7813500E-05,.7786200E-05,.7871500E-05,.7782500E-05,.7487700E-05,&
     & .6361500E-05,.6360100E-05,.6419800E-05,.6350700E-05,.6120000E-05,&
     & .5173800E-05,.5178100E-05,.5225000E-05,.5170900E-05,.5024300E-05,&
     & .4182500E-05,.4224000E-05,.4251000E-05,.4224100E-05,.4105800E-05,&
     & .3402400E-05,.3430200E-05,.3451400E-05,.3443600E-05,.3347400E-05,&
     & .2751000E-05,.2782800E-05,.2805900E-05,.2812500E-05,.2739400E-05,&
     & .2219500E-05,.2251700E-05,.2274300E-05,.2272800E-05,.2225400E-05,&
     & .1802900E-05,.1823800E-05,.1843200E-05,.1846100E-05,.1824100E-05,&
     & .1447700E-05,.1473400E-05,.1493600E-05,.1511600E-05,.1487400E-05/

      data   absb(121:235, 3)   /                                       &
     & .1163700E-05,.1192900E-05,.1209800E-05,.1226500E-05,.1207600E-05,&
     & .9329200E-06,.9624600E-06,.9816900E-06,.9943500E-06,.9874900E-06,&
     & .7492700E-06,.7674200E-06,.7902000E-06,.8060400E-06,.8141500E-06,&
     & .6047900E-06,.6174300E-06,.6397300E-06,.6529500E-06,.6626100E-06,&
     & .4845000E-06,.4974200E-06,.5136600E-06,.5287600E-06,.5381300E-06,&
     & .3829800E-06,.4008800E-06,.4119200E-06,.4265100E-06,.4359500E-06,&
     & .3031000E-06,.3190500E-06,.3290700E-06,.3415400E-06,.3500700E-06,&
     & .2374500E-06,.2503500E-06,.2622400E-06,.2714500E-06,.2814600E-06,&
     & .1865300E-06,.1951000E-06,.2064400E-06,.2151000E-06,.2235400E-06,&
     & .1475900E-06,.1532200E-06,.1614500E-06,.1703400E-06,.1774400E-06,&
     & .1141000E-06,.1200700E-06,.1250700E-06,.1323700E-06,.1399500E-06,&
     & .8832700E-07,.9396700E-07,.9768300E-07,.1025700E-06,.1092100E-06,&
     & .6671500E-07,.7217200E-07,.7644200E-07,.7921400E-07,.8400600E-07,&
     & .5145800E-07,.5576100E-07,.5923900E-07,.6229200E-07,.6447000E-07,&
     & .3995700E-07,.4240000E-07,.4587000E-07,.4859200E-07,.5028700E-07,&
     & .3115000E-07,.3272500E-07,.3559800E-07,.3763300E-07,.3960700E-07,&
     & .2378300E-07,.2541500E-07,.2663400E-07,.2888500E-07,.3062000E-07,&
     & .1853100E-07,.1993800E-07,.2098800E-07,.2267800E-07,.2386700E-07,&
     & .1481000E-07,.1577200E-07,.1665100E-07,.1772700E-07,.1886100E-07,&
     & .1165200E-07,.1255700E-07,.1331900E-07,.1393000E-07,.1507700E-07,&
     & .9152300E-08,.9983500E-08,.1060300E-07,.1108100E-07,.1199900E-07,&
     & .7238300E-08,.7906000E-08,.8448900E-08,.8861500E-08,.9445600E-08,&
     & .6031500E-08,.6593100E-08,.7100100E-08,.7401200E-08,.7876100E-08/

      data   absb(  1:120, 4)   /                                       &
     & .2188200E-03,.2096800E-03,.1864800E-03,.1750500E-03,.1732000E-03,&
     & .1914000E-03,.1702600E-03,.1590300E-03,.1555200E-03,.1600900E-03,&
     & .1600300E-03,.1453100E-03,.1400700E-03,.1451100E-03,.1461900E-03,&
     & .1338700E-03,.1242300E-03,.1244400E-03,.1274300E-03,.1274900E-03,&
     & .1126600E-03,.1078000E-03,.1111000E-03,.1105100E-03,.1107500E-03,&
     & .9615600E-04,.9561700E-04,.9677500E-04,.9622000E-04,.9607800E-04,&
     & .8271200E-04,.8294600E-04,.8326500E-04,.8282700E-04,.8204400E-04,&
     & .7108000E-04,.7116700E-04,.7104000E-04,.7044300E-04,.6906900E-04,&
     & .6011000E-04,.6083600E-04,.5999400E-04,.5899200E-04,.5723700E-04,&
     & .5068800E-04,.5129600E-04,.5027100E-04,.4888900E-04,.4847100E-04,&
     & .4270600E-04,.4262800E-04,.4176400E-04,.3999900E-04,.4089300E-04,&
     & .3580300E-04,.3531600E-04,.3453500E-04,.3322900E-04,.3412700E-04,&
     & .2990600E-04,.2920100E-04,.2833600E-04,.2745500E-04,.2804400E-04,&
     & .2465900E-04,.2392600E-04,.2315900E-04,.2282700E-04,.2326600E-04,&
     & .2022100E-04,.1957800E-04,.1883800E-04,.1885300E-04,.1906700E-04,&
     & .1634500E-04,.1591300E-04,.1532700E-04,.1541200E-04,.1556000E-04,&
     & .1319500E-04,.1287000E-04,.1243000E-04,.1251800E-04,.1263900E-04,&
     & .1062200E-04,.1038400E-04,.1006800E-04,.1015100E-04,.1025600E-04,&
     & .8578500E-05,.8352300E-05,.8122700E-05,.8199800E-05,.8331300E-05,&
     & .6911400E-05,.6709300E-05,.6575800E-05,.6622200E-05,.6767900E-05,&
     & .5545600E-05,.5434200E-05,.5322900E-05,.5361000E-05,.5481900E-05,&
     & .4446100E-05,.4389100E-05,.4311200E-05,.4363200E-05,.4456000E-05,&
     & .3572500E-05,.3519200E-05,.3470300E-05,.3507500E-05,.3591900E-05,&
     & .2884200E-05,.2836200E-05,.2814300E-05,.2809700E-05,.2905700E-05/

      data   absb(121:235, 4)   /                                       &
     & .2337400E-05,.2298700E-05,.2293400E-05,.2281200E-05,.2345100E-05,&
     & .1896600E-05,.1867900E-05,.1861300E-05,.1843700E-05,.1889400E-05,&
     & .1530000E-05,.1518000E-05,.1509600E-05,.1491500E-05,.1510700E-05,&
     & .1242600E-05,.1235700E-05,.1217100E-05,.1217000E-05,.1222700E-05,&
     & .1010200E-05,.1007700E-05,.9859300E-06,.9847800E-06,.9921300E-06,&
     & .8220100E-06,.8105000E-06,.8031300E-06,.8018100E-06,.8036100E-06,&
     & .6731800E-06,.6534700E-06,.6547800E-06,.6436000E-06,.6472200E-06,&
     & .5559800E-06,.5335500E-06,.5277800E-06,.5209700E-06,.5199700E-06,&
     & .4615700E-06,.4345700E-06,.4241200E-06,.4213700E-06,.4175000E-06,&
     & .3915600E-06,.3569000E-06,.3449800E-06,.3399500E-06,.3361400E-06,&
     & .3339200E-06,.2974600E-06,.2807400E-06,.2734400E-06,.2707900E-06,&
     & .2706600E-06,.2536900E-06,.2311500E-06,.2221300E-06,.2176600E-06,&
     & .2219800E-06,.2139500E-06,.1927400E-06,.1804000E-06,.1754200E-06,&
     & .1807000E-06,.1736700E-06,.1647300E-06,.1491600E-06,.1430600E-06,&
     & .1459500E-06,.1428700E-06,.1384700E-06,.1241800E-06,.1169700E-06,&
     & .1181900E-06,.1158800E-06,.1122500E-06,.1060400E-06,.9626900E-07,&
     & .9502800E-07,.9309400E-07,.9130300E-07,.8890000E-07,.8026200E-07,&
     & .7700500E-07,.7623800E-07,.7422800E-07,.7252600E-07,.6796500E-07,&
     & .6297900E-07,.6269200E-07,.6093000E-07,.5957900E-07,.5770000E-07,&
     & .5157100E-07,.5119600E-07,.5003100E-07,.4891600E-07,.4812700E-07,&
     & .4260200E-07,.4167800E-07,.4100600E-07,.3970900E-07,.3928300E-07,&
     & .3515500E-07,.3405200E-07,.3356100E-07,.3239500E-07,.3205700E-07,&
     & .2931800E-07,.2851600E-07,.2809700E-07,.2688800E-07,.2684000E-07/

      data   absb(  1:120, 5)   /                                       &
     & .5646872E-03,.5312795E-03,.5136090E-03,.4988131E-03,.4861599E-03,&
     & .4760303E-03,.4583276E-03,.4410876E-03,.4269357E-03,.4186013E-03,&
     & .4126853E-03,.3974268E-03,.3837132E-03,.3732654E-03,.3704958E-03,&
     & .3556932E-03,.3439374E-03,.3332957E-03,.3278663E-03,.3305577E-03,&
     & .3078248E-03,.2969033E-03,.2891762E-03,.2901475E-03,.2947469E-03,&
     & .2641839E-03,.2566180E-03,.2537464E-03,.2556275E-03,.2621378E-03,&
     & .2264940E-03,.2242650E-03,.2242408E-03,.2276949E-03,.2362173E-03,&
     & .1928877E-03,.1922497E-03,.1939580E-03,.2003837E-03,.2081227E-03,&
     & .1635486E-03,.1630146E-03,.1664181E-03,.1738976E-03,.1825902E-03,&
     & .1382623E-03,.1384386E-03,.1433107E-03,.1505458E-03,.1586465E-03,&
     & .1166097E-03,.1183846E-03,.1231655E-03,.1300138E-03,.1374265E-03,&
     & .9834271E-04,.1010835E-03,.1058259E-03,.1119955E-03,.1194249E-03,&
     & .8297870E-04,.8621261E-04,.9065743E-04,.9659067E-04,.1036206E-03,&
     & .7010856E-04,.7362109E-04,.7751831E-04,.8322905E-04,.8970605E-04,&
     & .5878661E-04,.6208067E-04,.6580331E-04,.7089987E-04,.7670386E-04,&
     & .4892928E-04,.5181644E-04,.5527233E-04,.5975857E-04,.6469853E-04,&
     & .4032031E-04,.4266267E-04,.4576398E-04,.4929704E-04,.5363977E-04,&
     & .3303540E-04,.3480435E-04,.3759482E-04,.4053873E-04,.4384522E-04,&
     & .2673301E-04,.2829877E-04,.3049062E-04,.3280744E-04,.3503681E-04,&
     & .2153689E-04,.2293629E-04,.2454493E-04,.2633007E-04,.2814543E-04,&
     & .1733556E-04,.1848199E-04,.1972836E-04,.2105193E-04,.2271445E-04,&
     & .1408268E-04,.1485471E-04,.1586389E-04,.1695541E-04,.1841529E-04,&
     & .1134570E-04,.1192884E-04,.1268134E-04,.1362048E-04,.1483775E-04,&
     & .9049567E-05,.9502726E-05,.1010072E-04,.1086491E-04,.1182301E-04/

      data   absb(121:235, 5)   /                                       &
     & .7259175E-05,.7610477E-05,.8079510E-05,.8694654E-05,.9481533E-05,&
     & .5822175E-05,.6083328E-05,.6462080E-05,.6945629E-05,.7579310E-05,&
     & .4661372E-05,.4850691E-05,.5152008E-05,.5531782E-05,.6026600E-05,&
     & .3768035E-05,.3907858E-05,.4148848E-05,.4439852E-05,.4849767E-05,&
     & .3050174E-05,.3152689E-05,.3345838E-05,.3584204E-05,.3905864E-05,&
     & .2466186E-05,.2558199E-05,.2693344E-05,.2890245E-05,.3143101E-05,&
     & .1984827E-05,.2063835E-05,.2154679E-05,.2317622E-05,.2513311E-05,&
     & .1591044E-05,.1657599E-05,.1723355E-05,.1849164E-05,.2007180E-05,&
     & .1271611E-05,.1327766E-05,.1384275E-05,.1468719E-05,.1594678E-05,&
     & .1012700E-05,.1059571E-05,.1106049E-05,.1165592E-05,.1261722E-05,&
     & .8134862E-06,.8456735E-06,.8847293E-06,.9280001E-06,.9964830E-06,&
     & .6601437E-06,.6702279E-06,.7033487E-06,.7382687E-06,.7869122E-06,&
     & .5354273E-06,.5343277E-06,.5561338E-06,.5862383E-06,.6207320E-06,&
     & .4385697E-06,.4345687E-06,.4441542E-06,.4677126E-06,.4945272E-06,&
     & .3616858E-06,.3547589E-06,.3578695E-06,.3746540E-06,.3960932E-06,&
     & .2980067E-06,.2907198E-06,.2906884E-06,.2989166E-06,.3168227E-06,&
     & .2459704E-06,.2382262E-06,.2364831E-06,.2390246E-06,.2515295E-06,&
     & .2050509E-06,.1972206E-06,.1946661E-06,.1956373E-06,.2034775E-06,&
     & .1708630E-06,.1642309E-06,.1612950E-06,.1615287E-06,.1659752E-06,&
     & .1426277E-06,.1371033E-06,.1336511E-06,.1333485E-06,.1355595E-06,&
     & .1182556E-06,.1146132E-06,.1111012E-06,.1106095E-06,.1111974E-06,&
     & .9823259E-07,.9553915E-07,.9253575E-07,.9185070E-07,.9208712E-07,&
     & .8406156E-07,.8113764E-07,.7830100E-07,.7796398E-07,.7863117E-07/

      data   absb(  1:120, 6)   /                                       &
     & .1876746E-01,.1987652E-01,.2118689E-01,.2284887E-01,.2484965E-01,&
     & .1797735E-01,.1911766E-01,.2049609E-01,.2208055E-01,.2390495E-01,&
     & .1742326E-01,.1850993E-01,.1995764E-01,.2157831E-01,.2349113E-01,&
     & .1632945E-01,.1753946E-01,.1884180E-01,.2053655E-01,.2251614E-01,&
     & .1528056E-01,.1648257E-01,.1780611E-01,.1941673E-01,.2129325E-01,&
     & .1439040E-01,.1554780E-01,.1695518E-01,.1850787E-01,.2029757E-01,&
     & .1364127E-01,.1485684E-01,.1620134E-01,.1772745E-01,.1951994E-01,&
     & .1258716E-01,.1381209E-01,.1512804E-01,.1667637E-01,.1840520E-01,&
     & .1143111E-01,.1260773E-01,.1397144E-01,.1543741E-01,.1714336E-01,&
     & .1034838E-01,.1151140E-01,.1288246E-01,.1434110E-01,.1596120E-01,&
     & .9434287E-02,.1058629E-01,.1186231E-01,.1323644E-01,.1478338E-01,&
     & .8644308E-02,.9731339E-02,.1097791E-01,.1231507E-01,.1383818E-01,&
     & .7945748E-02,.8966157E-02,.1013410E-01,.1144951E-01,.1288499E-01,&
     & .7251090E-02,.8253964E-02,.9351761E-02,.1060751E-01,.1200048E-01,&
     & .6579748E-02,.7509664E-02,.8524470E-02,.9703299E-02,.1107385E-01,&
     & .5951880E-02,.6756479E-02,.7704468E-02,.8845964E-02,.1010461E-01,&
     & .5237878E-02,.5963536E-02,.6868194E-02,.7873626E-02,.9058934E-02,&
     & .4564340E-02,.5246869E-02,.6058901E-02,.6986798E-02,.8100707E-02,&
     & .3927773E-02,.4563237E-02,.5276212E-02,.6126524E-02,.7132796E-02,&
     & .3396746E-02,.3962408E-02,.4607092E-02,.5396614E-02,.6293512E-02,&
     & .2938365E-02,.3441438E-02,.4024490E-02,.4739722E-02,.5537536E-02,&
     & .2568850E-02,.3016246E-02,.3558015E-02,.4198155E-02,.4896116E-02,&
     & .2216648E-02,.2607125E-02,.3108998E-02,.3673395E-02,.4285795E-02,&
     & .1888038E-02,.2238310E-02,.2672219E-02,.3167173E-02,.3704008E-02/

      data   absb(121:235, 6)   /                                       &
     & .1589604E-02,.1898430E-02,.2275337E-02,.2698109E-02,.3168934E-02,&
     & .1330328E-02,.1601088E-02,.1927308E-02,.2289886E-02,.2702347E-02,&
     & .1106058E-02,.1339532E-02,.1622095E-02,.1933708E-02,.2294633E-02,&
     & .9232322E-03,.1129388E-02,.1374137E-02,.1641005E-02,.1954041E-02,&
     & .7695851E-03,.9499734E-03,.1161581E-02,.1391249E-02,.1659990E-02,&
     & .6391411E-03,.7954699E-03,.9764327E-03,.1175728E-02,.1405980E-02,&
     & .5221445E-03,.6543645E-03,.8097738E-03,.9801297E-03,.1177108E-02,&
     & .4212175E-03,.5316807E-03,.6646766E-03,.8090125E-03,.9773287E-03,&
     & .3365734E-03,.4274844E-03,.5406354E-03,.6629665E-03,.8061362E-03,&
     & .2659082E-03,.3406968E-03,.4346710E-03,.5386141E-03,.6603153E-03,&
     & .2084988E-03,.2688505E-03,.3471450E-03,.4345779E-03,.5367697E-03,&
     & .1617409E-03,.2100645E-03,.2734876E-03,.3468514E-03,.4310783E-03,&
     & .1230350E-03,.1615169E-03,.2121423E-03,.2726679E-03,.3415988E-03,&
     & .9571658E-04,.1268700E-03,.1676066E-03,.2182467E-03,.2753833E-03,&
     & .7483539E-04,.1000772E-03,.1334398E-03,.1757280E-03,.2239868E-03,&
     & .5772027E-04,.7803984E-04,.1048734E-03,.1398792E-03,.1799578E-03,&
     & .4394505E-04,.6019842E-04,.8138845E-04,.1097945E-03,.1427362E-03,&
     & .3482804E-04,.4819989E-04,.6606100E-04,.8999622E-04,.1184452E-03,&
     & .2824956E-04,.3942832E-04,.5468547E-04,.7556789E-04,.1004720E-03,&
     & .2272475E-04,.3212105E-04,.4507284E-04,.6295422E-04,.8486593E-04,&
     & .1816142E-04,.2604650E-04,.3689143E-04,.5210509E-04,.7114512E-04,&
     & .1460417E-04,.2125717E-04,.3044873E-04,.4347568E-04,.6004634E-04,&
     & .1318588E-04,.1949094E-04,.2845284E-04,.4096246E-04,.5730208E-04/

      data   absb(  1:120, 7)   /                                       &
     & .3908183E+01,.4021612E+01,.4146726E+01,.4284083E+01,.4438438E+01,&
     & .3284599E+01,.3380615E+01,.3488659E+01,.3611242E+01,.3752108E+01,&
     & .2750113E+01,.2833708E+01,.2928922E+01,.3039836E+01,.3169125E+01,&
     & .2307053E+01,.2379505E+01,.2464791E+01,.2564854E+01,.2681085E+01,&
     & .1936077E+01,.2001044E+01,.2079025E+01,.2171276E+01,.2276173E+01,&
     & .1622572E+01,.1681244E+01,.1752186E+01,.1836321E+01,.1928176E+01,&
     & .1369492E+01,.1421024E+01,.1484095E+01,.1556113E+01,.1634126E+01,&
     & .1180311E+01,.1227272E+01,.1282719E+01,.1341372E+01,.1403599E+01,&
     & .1020024E+01,.1063783E+01,.1111019E+01,.1162039E+01,.1215624E+01,&
     & .8833680E+00,.9215968E+00,.9637706E+00,.1008479E+01,.1057170E+01,&
     & .7624334E+00,.7970585E+00,.8346574E+00,.8755899E+00,.9193781E+00,&
     & .6579273E+00,.6887119E+00,.7224049E+00,.7595641E+00,.7997812E+00,&
     & .5668077E+00,.5946957E+00,.6255016E+00,.6592894E+00,.6963418E+00,&
     & .4881446E+00,.5135679E+00,.5418786E+00,.5730494E+00,.6077831E+00,&
     & .4200990E+00,.4431037E+00,.4688805E+00,.4974060E+00,.5298295E+00,&
     & .3617384E+00,.3821010E+00,.4055724E+00,.4323227E+00,.4633921E+00,&
     & .3109227E+00,.3293125E+00,.3507763E+00,.3762455E+00,.4052117E+00,&
     & .2674240E+00,.2843698E+00,.3044221E+00,.3282655E+00,.3551022E+00,&
     & .2297019E+00,.2456549E+00,.2646151E+00,.2871844E+00,.3119468E+00,&
     & .1979105E+00,.2130118E+00,.2313507E+00,.2522008E+00,.2753351E+00,&
     & .1710687E+00,.1855135E+00,.2029507E+00,.2226155E+00,.2446203E+00,&
     & .1485780E+00,.1623096E+00,.1788155E+00,.1972862E+00,.2180352E+00,&
     & .1285687E+00,.1414891E+00,.1566691E+00,.1740024E+00,.1933684E+00,&
     & .1107491E+00,.1226288E+00,.1365049E+00,.1524192E+00,.1703572E+00/

      data   absb(121:235, 7)   /                                       &
     & .9456369E-01,.1051244E+00,.1175196E+00,.1318208E+00,.1481765E+00,&
     & .8066956E-01,.9002222E-01,.1011931E+00,.1140601E+00,.1288872E+00,&
     & .6867032E-01,.7709429E-01,.8709495E-01,.9877474E-01,.1121550E+00,&
     & .5832603E-01,.6556338E-01,.7444062E-01,.8488976E-01,.9692322E-01,&
     & .4942472E-01,.5585419E-01,.6358733E-01,.7286335E-01,.8372396E-01,&
     & .4180841E-01,.4754454E-01,.5435437E-01,.6256355E-01,.7229908E-01,&
     & .3508489E-01,.4013132E-01,.4608275E-01,.5327848E-01,.6197794E-01,&
     & .2922066E-01,.3367674E-01,.3885124E-01,.4519646E-01,.5282863E-01,&
     & .2427508E-01,.2810853E-01,.3265710E-01,.3827469E-01,.4498559E-01,&
     & .2006569E-01,.2335359E-01,.2727995E-01,.3220620E-01,.3815593E-01,&
     & .1645503E-01,.1925848E-01,.2264332E-01,.2682076E-01,.3208388E-01,&
     & .1342964E-01,.1577348E-01,.1865261E-01,.2224444E-01,.2683651E-01,&
     & .1090974E-01,.1285284E-01,.1527245E-01,.1835280E-01,.2231368E-01,&
     & .8925769E-02,.1055870E-01,.1261660E-01,.1523191E-01,.1867393E-01,&
     & .7332235E-02,.8686707E-02,.1044642E-01,.1267886E-01,.1562635E-01,&
     & .6005950E-02,.7116692E-02,.8597108E-02,.1050258E-01,.1303347E-01,&
     & .4886737E-02,.5803324E-02,.7029411E-02,.8649638E-02,.1082164E-01,&
     & .4045146E-02,.4827713E-02,.5866804E-02,.7255958E-02,.9118484E-02,&
     & .3386333E-02,.4055976E-02,.4960277E-02,.6153907E-02,.7764407E-02,&
     & .2833529E-02,.3404009E-02,.4187592E-02,.5222472E-02,.6607700E-02,&
     & .2365720E-02,.2858268E-02,.3529390E-02,.4419998E-02,.5636682E-02,&
     & .1989667E-02,.2413827E-02,.2992976E-02,.3780796E-02,.4842075E-02,&
     & .1799658E-02,.2201226E-02,.2751996E-02,.3518772E-02,.4547327E-02/

      data   absb(  1:120, 8)   /                                       &
     & .4033644E+02,.4167953E+02,.4301570E+02,.4435920E+02,.4567833E+02,&
     & .3531377E+02,.3641380E+02,.3752525E+02,.3863489E+02,.3972127E+02,&
     & .3071410E+02,.3160600E+02,.3251995E+02,.3343214E+02,.3433462E+02,&
     & .2650555E+02,.2724233E+02,.2800390E+02,.2877429E+02,.2956326E+02,&
     & .2275030E+02,.2336977E+02,.2400486E+02,.2465588E+02,.2535498E+02,&
     & .1940546E+02,.1993156E+02,.2047118E+02,.2104774E+02,.2171625E+02,&
     & .1638172E+02,.1683509E+02,.1732028E+02,.1787385E+02,.1854784E+02,&
     & .1365200E+02,.1402511E+02,.1446383E+02,.1502684E+02,.1574369E+02,&
     & .1137022E+02,.1169679E+02,.1212312E+02,.1267823E+02,.1337177E+02,&
     & .9506496E+01,.9827971E+01,.1025128E+02,.1081497E+02,.1148308E+02,&
     & .8052710E+01,.8344535E+01,.8752518E+01,.9286286E+01,.9908091E+01,&
     & .7005129E+01,.7297737E+01,.7690454E+01,.8150077E+01,.8636301E+01,&
     & .6181432E+01,.6493698E+01,.6873626E+01,.7267363E+01,.7701472E+01,&
     & .5486352E+01,.5805127E+01,.6164438E+01,.6541233E+01,.6910418E+01,&
     & .4874205E+01,.5201845E+01,.5533903E+01,.5877592E+01,.6211504E+01,&
     & .4354064E+01,.4662856E+01,.4960467E+01,.5268278E+01,.5590873E+01,&
     & .3870676E+01,.4151579E+01,.4424561E+01,.4713940E+01,.5019965E+01,&
     & .3432801E+01,.3695370E+01,.3951313E+01,.4223258E+01,.4521613E+01,&
     & .3046684E+01,.3280864E+01,.3528685E+01,.3785450E+01,.4072281E+01,&
     & .2707644E+01,.2931362E+01,.3163706E+01,.3412966E+01,.3691298E+01,&
     & .2413017E+01,.2622420E+01,.2842869E+01,.3091981E+01,.3368488E+01,&
     & .2161287E+01,.2357984E+01,.2569356E+01,.2810989E+01,.3097338E+01,&
     & .1925443E+01,.2108054E+01,.2317393E+01,.2555114E+01,.2840446E+01,&
     & .1703781E+01,.1879720E+01,.2080458E+01,.2316042E+01,.2597137E+01/

      data   absb(121:235, 8)   /                                       &
     & .1493092E+01,.1657976E+01,.1851046E+01,.2073961E+01,.2343340E+01,&
     & .1311161E+01,.1466732E+01,.1645509E+01,.1859825E+01,.2118946E+01,&
     & .1150929E+01,.1297548E+01,.1468104E+01,.1672256E+01,.1922471E+01,&
     & .1004409E+01,.1141010E+01,.1301607E+01,.1492747E+01,.1726160E+01,&
     & .8769985E+00,.1002882E+01,.1151293E+01,.1329471E+01,.1549835E+01,&
     & .7649098E+00,.8805424E+00,.1016103E+01,.1181723E+01,.1389132E+01,&
     & .6598121E+00,.7637212E+00,.8887136E+00,.1040010E+01,.1230413E+01,&
     & .5657958E+00,.6586754E+00,.7718739E+00,.9084100E+00,.1082238E+01,&
     & .4835182E+00,.5668068E+00,.6687172E+00,.7926556E+00,.9499754E+00,&
     & .4118145E+00,.4852766E+00,.5756963E+00,.6868494E+00,.8286511E+00,&
     & .3488019E+00,.4139755E+00,.4938550E+00,.5910229E+00,.7163238E+00,&
     & .2935726E+00,.3516563E+00,.4223583E+00,.5090453E+00,.6171835E+00,&
     & .2445646E+00,.2963715E+00,.3583947E+00,.4363324E+00,.5322580E+00,&
     & .2063336E+00,.2514543E+00,.3069415E+00,.3769572E+00,.4647802E+00,&
     & .1745065E+00,.2143204E+00,.2634212E+00,.3259536E+00,.4065898E+00,&
     & .1469747E+00,.1815841E+00,.2245475E+00,.2809591E+00,.3526955E+00,&
     & .1226579E+00,.1531164E+00,.1907781E+00,.2400747E+00,.3043525E+00,&
     & .1036864E+00,.1314203E+00,.1647845E+00,.2083891E+00,.2666072E+00,&
     & .8816513E-01,.1136512E+00,.1440561E+00,.1835360E+00,.2351002E+00,&
     & .7437710E-01,.9768257E-01,.1255978E+00,.1607081E+00,.2071018E+00,&
     & .6227610E-01,.8346943E-01,.1093711E+00,.1408269E+00,.1825617E+00,&
     & .5263552E-01,.7135997E-01,.9530909E-01,.1242291E+00,.1612246E+00,&
     & .4825418E-01,.6650163E-01,.8992737E-01,.1186811E+00,.1555922E+00/

      data   absb(  1:120, 9)   /                                       &
     & .1646500E+03,.1629800E+03,.1641300E+03,.1679600E+03,.1742500E+03,&
     & .1413400E+03,.1422300E+03,.1458400E+03,.1518300E+03,.1582300E+03,&
     & .1233400E+03,.1264200E+03,.1317000E+03,.1371900E+03,.1424800E+03,&
     & .1090300E+03,.1135000E+03,.1182500E+03,.1228400E+03,.1272000E+03,&
     & .9674400E+02,.1007700E+03,.1047300E+03,.1085600E+03,.1122500E+03,&
     & .8536800E+02,.8866900E+02,.9193400E+02,.9512700E+02,.9821600E+02,&
     & .7486500E+02,.7756000E+02,.8026300E+02,.8291200E+02,.8549900E+02,&
     & .6530400E+02,.6758800E+02,.6984900E+02,.7209900E+02,.7431600E+02,&
     & .5670400E+02,.5861600E+02,.6054100E+02,.6245700E+02,.6444400E+02,&
     & .4909900E+02,.5078800E+02,.5245100E+02,.5411900E+02,.5597000E+02,&
     & .4207600E+02,.4364600E+02,.4523000E+02,.4688800E+02,.4879800E+02,&
     & .3545700E+02,.3690400E+02,.3844700E+02,.4026200E+02,.4256500E+02,&
     & .2983200E+02,.3103600E+02,.3250800E+02,.3450200E+02,.3697900E+02,&
     & .2520500E+02,.2634800E+02,.2779200E+02,.2978800E+02,.3245900E+02,&
     & .2148600E+02,.2258600E+02,.2416200E+02,.2626000E+02,.2894400E+02,&
     & .1871200E+02,.1969300E+02,.2137000E+02,.2359100E+02,.2615800E+02,&
     & .1663000E+02,.1771600E+02,.1936900E+02,.2156500E+02,.2410200E+02,&
     & .1492300E+02,.1611900E+02,.1778300E+02,.1998900E+02,.2237900E+02,&
     & .1353200E+02,.1485300E+02,.1657600E+02,.1874500E+02,.2108400E+02,&
     & .1245300E+02,.1378400E+02,.1558700E+02,.1771800E+02,.2002200E+02,&
     & .1158000E+02,.1303800E+02,.1485100E+02,.1689200E+02,.1919200E+02,&
     & .1082000E+02,.1237600E+02,.1413400E+02,.1615200E+02,.1840200E+02,&
     & .1011600E+02,.1168600E+02,.1340200E+02,.1534600E+02,.1755600E+02,&
     & .9393300E+01,.1091000E+02,.1257200E+02,.1443200E+02,.1653600E+02/

      data   absb(121:235, 9)   /                                       &
     & .8516000E+01,.9975500E+01,.1155400E+02,.1335800E+02,.1535000E+02,&
     & .7750500E+01,.9122500E+01,.1067000E+02,.1238000E+02,.1427300E+02,&
     & .7089700E+01,.8388500E+01,.9869400E+01,.1150200E+02,.1334300E+02,&
     & .6329500E+01,.7550600E+01,.8946600E+01,.1049500E+02,.1228700E+02,&
     & .5630200E+01,.6798800E+01,.8095700E+01,.9601300E+01,.1128000E+02,&
     & .5023400E+01,.6107100E+01,.7338900E+01,.8781600E+01,.1038000E+02,&
     & .4422500E+01,.5428200E+01,.6596300E+01,.7943300E+01,.9466600E+01,&
     & .3855800E+01,.4791200E+01,.5890500E+01,.7147800E+01,.8614500E+01,&
     & .3357500E+01,.4235100E+01,.5247500E+01,.6429200E+01,.7829600E+01,&
     & .2905300E+01,.3707400E+01,.4643200E+01,.5748400E+01,.7067400E+01,&
     & .2470600E+01,.3185100E+01,.4052400E+01,.5083700E+01,.6299000E+01,&
     & .2090700E+01,.2732000E+01,.3523200E+01,.4468900E+01,.5599800E+01,&
     & .1765200E+01,.2339700E+01,.3052500E+01,.3913000E+01,.4964100E+01,&
     & .1498000E+01,.1999500E+01,.2638600E+01,.3430900E+01,.4393000E+01,&
     & .1288300E+01,.1722900E+01,.2280800E+01,.3005800E+01,.3892100E+01,&
     & .1103200E+01,.1487800E+01,.1994500E+01,.2622500E+01,.3432700E+01,&
     & .9425600E+00,.1280300E+01,.1743600E+01,.2309500E+01,.3025800E+01,&
     & .8084700E+00,.1104500E+01,.1525000E+01,.2057300E+01,.2704800E+01,&
     & .6997200E+00,.9500800E+00,.1321500E+01,.1817800E+01,.2432600E+01,&
     & .6065900E+00,.8129800E+00,.1143000E+01,.1592500E+01,.2178300E+01,&
     & .5266400E+00,.6967000E+00,.9799800E+00,.1389100E+01,.1933300E+01,&
     & .4598500E+00,.6068200E+00,.8433700E+00,.1211300E+01,.1718800E+01,&
     & .4385300E+00,.5787900E+00,.7901800E+00,.1113000E+01,.1603900E+01/

      data   absb(  1:120,10)   /                                       &
     & .8601600E+03,.8462600E+03,.8322800E+03,.8181700E+03,.8045000E+03,&
     & .7598100E+03,.7466300E+03,.7334500E+03,.7210400E+03,.7122400E+03,&
     & .6620100E+03,.6499300E+03,.6389100E+03,.6321000E+03,.6297800E+03,&
     & .5701400E+03,.5602700E+03,.5546600E+03,.5537800E+03,.5572100E+03,&
     & .4886100E+03,.4835400E+03,.4832400E+03,.4876000E+03,.4960200E+03,&
     & .4198700E+03,.4196400E+03,.4241100E+03,.4329100E+03,.4457500E+03,&
     & .3630300E+03,.3670200E+03,.3755600E+03,.3883400E+03,.4047100E+03,&
     & .3165100E+03,.3242100E+03,.3362700E+03,.3521900E+03,.3707200E+03,&
     & .2786700E+03,.2894700E+03,.3043400E+03,.3213100E+03,.3382300E+03,&
     & .2491600E+03,.2625100E+03,.2776800E+03,.2929600E+03,.3084400E+03,&
     & .2257100E+03,.2393200E+03,.2529100E+03,.2667000E+03,.2806900E+03,&
     & .2050300E+03,.2176500E+03,.2302900E+03,.2429100E+03,.2557500E+03,&
     & .1857400E+03,.1976300E+03,.2096000E+03,.2215100E+03,.2335600E+03,&
     & .1682500E+03,.1794500E+03,.1908500E+03,.2024200E+03,.2140600E+03,&
     & .1525400E+03,.1631100E+03,.1740200E+03,.1852100E+03,.1969600E+03,&
     & .1381500E+03,.1485700E+03,.1591300E+03,.1702200E+03,.1823800E+03,&
     & .1256000E+03,.1357100E+03,.1463000E+03,.1576300E+03,.1701900E+03,&
     & .1149700E+03,.1247600E+03,.1355000E+03,.1471900E+03,.1604400E+03,&
     & .1060200E+03,.1157700E+03,.1267100E+03,.1388900E+03,.1528900E+03,&
     & .9851900E+02,.1085500E+03,.1197800E+03,.1326500E+03,.1474400E+03,&
     & .9251700E+02,.1027400E+03,.1145200E+03,.1282200E+03,.1438900E+03,&
     & .8744100E+02,.9793600E+02,.1104400E+03,.1249500E+03,.1414800E+03,&
     & .8249900E+02,.9333300E+02,.1063500E+03,.1215100E+03,.1387400E+03,&
     & .7744200E+02,.8849300E+02,.1018400E+03,.1174000E+03,.1351300E+03/

      data   absb(121:235,10)   /                                       &
     & .7177100E+02,.8273100E+02,.9603200E+02,.1115800E+03,.1294100E+03,&
     & .6672700E+02,.7761000E+02,.9079500E+02,.1063200E+03,.1241500E+03,&
     & .6233900E+02,.7311300E+02,.8620000E+02,.1016700E+03,.1194200E+03,&
     & .5760100E+02,.6799900E+02,.8072400E+02,.9585800E+02,.1132300E+03,&
     & .5328500E+02,.6320300E+02,.7557700E+02,.9024200E+02,.1072600E+03,&
     & .4939200E+02,.5890800E+02,.7083400E+02,.8504500E+02,.1016300E+03,&
     & .4544600E+02,.5445200E+02,.6575100E+02,.7940200E+02,.9539700E+02,&
     & .4162000E+02,.5018400E+02,.6077700E+02,.7378600E+02,.8907600E+02,&
     & .3805900E+02,.4628200E+02,.5622600E+02,.6853900E+02,.8313600E+02,&
     & .3455400E+02,.4250100E+02,.5182900E+02,.6338300E+02,.7722000E+02,&
     & .3100900E+02,.3861000E+02,.4738500E+02,.5805800E+02,.7108000E+02,&
     & .2771500E+02,.3493400E+02,.4331900E+02,.5319000E+02,.6534700E+02,&
     & .2465800E+02,.3147800E+02,.3950000E+02,.4874300E+02,.5999700E+02,&
     & .2203400E+02,.2843700E+02,.3609300E+02,.4489400E+02,.5534600E+02,&
     & .1968400E+02,.2562600E+02,.3293700E+02,.4138500E+02,.5115400E+02,&
     & .1750400E+02,.2299500E+02,.2992500E+02,.3805700E+02,.4730700E+02,&
     & .1555200E+02,.2054400E+02,.2707900E+02,.3483200E+02,.4372200E+02,&
     & .1388600E+02,.1847600E+02,.2460700E+02,.3196600E+02,.4054000E+02,&
     & .1239600E+02,.1668100E+02,.2238500E+02,.2938400E+02,.3759200E+02,&
     & .1102800E+02,.1500600E+02,.2029700E+02,.2695500E+02,.3478000E+02,&
     & .9749300E+01,.1345200E+02,.1835400E+02,.2464400E+02,.3209900E+02,&
     & .8641100E+01,.1206800E+02,.1662700E+02,.2256900E+02,.2968400E+02,&
     & .8089400E+01,.1144900E+02,.1590500E+02,.2175300E+02,.2872800E+02/

      data   absb(  1:120,11)   /                                       &
     & .5004600E+04,.5003400E+04,.4995000E+04,.4978100E+04,.4954200E+04,&
     & .5181200E+04,.5173800E+04,.5158200E+04,.5135700E+04,.5105400E+04,&
     & .5279300E+04,.5264500E+04,.5245000E+04,.5217200E+04,.5183600E+04,&
     & .5300100E+04,.5281800E+04,.5256800E+04,.5227400E+04,.5192400E+04,&
     & .5244800E+04,.5224200E+04,.5200400E+04,.5171500E+04,.5137700E+04,&
     & .5120000E+04,.5103100E+04,.5081700E+04,.5057100E+04,.5030800E+04,&
     & .4938900E+04,.4928400E+04,.4915300E+04,.4900700E+04,.4881900E+04,&
     & .4718100E+04,.4716800E+04,.4716300E+04,.4712900E+04,.4711100E+04,&
     & .4471800E+04,.4486500E+04,.4499000E+04,.4515300E+04,.4540300E+04,&
     & .4219200E+04,.4249700E+04,.4286100E+04,.4331600E+04,.4380900E+04,&
     & .3972700E+04,.4027500E+04,.4093600E+04,.4164600E+04,.4240400E+04,&
     & .3750700E+04,.3833600E+04,.3923900E+04,.4020600E+04,.4120900E+04,&
     & .3560600E+04,.3669400E+04,.3782600E+04,.3902600E+04,.4024400E+04,&
     & .3404300E+04,.3536400E+04,.3673100E+04,.3813000E+04,.3951900E+04,&
     & .3280300E+04,.3434600E+04,.3591200E+04,.3747600E+04,.3901400E+04,&
     & .3187600E+04,.3361000E+04,.3534400E+04,.3704200E+04,.3869900E+04,&
     & .3123900E+04,.3313700E+04,.3499500E+04,.3681300E+04,.3857300E+04,&
     & .3085000E+04,.3287200E+04,.3483700E+04,.3674500E+04,.3859100E+04,&
     & .3067400E+04,.3279500E+04,.3484800E+04,.3682600E+04,.3872400E+04,&
     & .3068300E+04,.3287900E+04,.3499000E+04,.3702200E+04,.3895300E+04,&
     & .3083100E+04,.3308700E+04,.3524300E+04,.3730300E+04,.3925000E+04,&
     & .3101700E+04,.3330800E+04,.3549600E+04,.3758000E+04,.3952700E+04,&
     & .3105700E+04,.3338700E+04,.3560100E+04,.3770000E+04,.3965400E+04,&
     & .3089500E+04,.3326000E+04,.3550400E+04,.3763000E+04,.3960600E+04/

      data   absb(121:235,11)   /                                       &
     & .3040000E+04,.3281100E+04,.3509600E+04,.3726300E+04,.3927700E+04,&
     & .2991300E+04,.3236100E+04,.3469000E+04,.3689000E+04,.3894200E+04,&
     & .2944100E+04,.3192600E+04,.3429400E+04,.3652600E+04,.3861200E+04,&
     & .2872200E+04,.3125400E+04,.3366700E+04,.3594300E+04,.3807300E+04,&
     & .2797100E+04,.3054800E+04,.3300400E+04,.3532600E+04,.3750500E+04,&
     & .2722600E+04,.2983700E+04,.3233600E+04,.3470100E+04,.3692500E+04,&
     & .2633200E+04,.2898100E+04,.3152500E+04,.3394400E+04,.3622300E+04,&
     & .2537200E+04,.2805000E+04,.3064300E+04,.3311400E+04,.3544100E+04,&
     & .2441700E+04,.2711300E+04,.2974800E+04,.3226800E+04,.3465100E+04,&
     & .2339500E+04,.2611500E+04,.2878300E+04,.3135100E+04,.3379200E+04,&
     & .2224300E+04,.2499800E+04,.2769400E+04,.3031300E+04,.3281000E+04,&
     & .2108300E+04,.2387700E+04,.2659200E+04,.2925700E+04,.3180300E+04,&
     & .1992500E+04,.2274100E+04,.2548800E+04,.2818000E+04,.3078300E+04,&
     & .1884100E+04,.2166600E+04,.2444700E+04,.2715900E+04,.2980600E+04,&
     & .1779800E+04,.2061800E+04,.2342800E+04,.2616000E+04,.2883900E+04,&
     & .1676200E+04,.1957400E+04,.2239800E+04,.2515600E+04,.2786200E+04,&
     & .1573100E+04,.1853000E+04,.2135800E+04,.2415200E+04,.2687200E+04,&
     & .1480200E+04,.1758100E+04,.2040100E+04,.2321900E+04,.2595600E+04,&
     & .1391500E+04,.1667500E+04,.1948800E+04,.2231800E+04,.2508000E+04,&
     & .1303900E+04,.1577800E+04,.1857700E+04,.2140600E+04,.2420000E+04,&
     & .1216700E+04,.1489000E+04,.1767200E+04,.2049300E+04,.2331300E+04,&
     & .1134900E+04,.1405700E+04,.1681800E+04,.1963100E+04,.2246400E+04,&
     & .1102500E+04,.1372100E+04,.1647600E+04,.1928300E+04,.2211700E+04/

      data   absb(  1:120,12)   /                                       &
     & .1249200E+05,.1260200E+05,.1267800E+05,.1273000E+05,.1274900E+05,&
     & .1469600E+05,.1479200E+05,.1486200E+05,.1489000E+05,.1488900E+05,&
     & .1715300E+05,.1724200E+05,.1727500E+05,.1727400E+05,.1723100E+05,&
     & .1984300E+05,.1989400E+05,.1989800E+05,.1984500E+05,.1974700E+05,&
     & .2272700E+05,.2273100E+05,.2266700E+05,.2255100E+05,.2238600E+05,&
     & .2576100E+05,.2568400E+05,.2554400E+05,.2534200E+05,.2507000E+05,&
     & .2887800E+05,.2870100E+05,.2845400E+05,.2813100E+05,.2775900E+05,&
     & .3198100E+05,.3169500E+05,.3131400E+05,.3087100E+05,.3036000E+05,&
     & .3501700E+05,.3457500E+05,.3406400E+05,.3348000E+05,.3282100E+05,&
     & .3785400E+05,.3726400E+05,.3658900E+05,.3584000E+05,.3504600E+05,&
     & .4047300E+05,.3971000E+05,.3886200E+05,.3796700E+05,.3702300E+05,&
     & .4279600E+05,.4186400E+05,.4086900E+05,.3981700E+05,.3871800E+05,&
     & .4481300E+05,.4371900E+05,.4257800E+05,.4138000E+05,.4014300E+05,&
     & .4650400E+05,.4526300E+05,.4397300E+05,.4264100E+05,.4129400E+05,&
     & .4789300E+05,.4650700E+05,.4508700E+05,.4364700E+05,.4219700E+05,&
     & .4899300E+05,.4748000E+05,.4595200E+05,.4441700E+05,.4287800E+05,&
     & .4982300E+05,.4820600E+05,.4658800E+05,.4496900E+05,.4335300E+05,&
     & .5041500E+05,.4871600E+05,.4702500E+05,.4533900E+05,.4365900E+05,&
     & .5081100E+05,.4904200E+05,.4728800E+05,.4554300E+05,.4381300E+05,&
     & .5103200E+05,.4920300E+05,.4740000E+05,.4561000E+05,.4384100E+05,&
     & .5110700E+05,.4923400E+05,.4739300E+05,.4556500E+05,.4377500E+05,&
     & .5113000E+05,.4922800E+05,.4735000E+05,.4549700E+05,.4368800E+05,&
     & .5125600E+05,.4931600E+05,.4741300E+05,.4554000E+05,.4371100E+05,&
     & .5151900E+05,.4955500E+05,.4762600E+05,.4573100E+05,.4388500E+05/

      data   absb(121:235,12)   /                                       &
     & .5203400E+05,.5004100E+05,.4809000E+05,.4616800E+05,.4430200E+05,&
     & .5252900E+05,.5051100E+05,.4853500E+05,.4659500E+05,.4470600E+05,&
     & .5298500E+05,.5095200E+05,.4895400E+05,.4699600E+05,.4508500E+05,&
     & .5363300E+05,.5157500E+05,.4955600E+05,.4757800E+05,.4564800E+05,&
     & .5428900E+05,.5221100E+05,.5017100E+05,.4817100E+05,.4622000E+05,&
     & .5493200E+05,.5283600E+05,.5077800E+05,.4875900E+05,.4678500E+05,&
     & .5567300E+05,.5356900E+05,.5148800E+05,.4944900E+05,.4745600E+05,&
     & .5646100E+05,.5434600E+05,.5224700E+05,.5019000E+05,.4817800E+05,&
     & .5724100E+05,.5512100E+05,.5300700E+05,.5092900E+05,.4889700E+05,&
     & .5806300E+05,.5593300E+05,.5380600E+05,.5171000E+05,.4965600E+05,&
     & .5897800E+05,.5682800E+05,.5469700E+05,.5258200E+05,.5050500E+05,&
     & .5989100E+05,.5772000E+05,.5558600E+05,.5345900E+05,.5136700E+05,&
     & .6079500E+05,.5862100E+05,.5647200E+05,.5433600E+05,.5222400E+05,&
     & .6163900E+05,.5946600E+05,.5729800E+05,.5516200E+05,.5303400E+05,&
     & .6244700E+05,.6028400E+05,.5811000E+05,.5596100E+05,.5382400E+05,&
     & .6324000E+05,.6109800E+05,.5891600E+05,.5676000E+05,.5461200E+05,&
     & .6403000E+05,.6190300E+05,.5972700E+05,.5755800E+05,.5541100E+05,&
     & .6473500E+05,.6263700E+05,.6047100E+05,.5828700E+05,.5613800E+05,&
     & .6540700E+05,.6333200E+05,.6118100E+05,.5900300E+05,.5683400E+05,&
     & .6606900E+05,.6401500E+05,.6188100E+05,.5970200E+05,.5752900E+05,&
     & .6672500E+05,.6468300E+05,.6257600E+05,.6041100E+05,.5822500E+05,&
     & .6732700E+05,.6531500E+05,.6323200E+05,.6107800E+05,.5889100E+05,&
     & .6757700E+05,.6557200E+05,.6349600E+05,.6134900E+05,.5916100E+05/

!  ---
      data   forref(1:4,1:12)  / .2998180E-05,.2092820E-05,.9883530E-04,&
     & .6321780E-03,.6336480E-05,.5092140E-04,.6505350E-03,.2640190E-02,&
     & .6367820E-04,.1365770E-03,.1665000E-02,.7508210E-02,.4723140E-03,&
     & .9882960E-03,.5857510E-02,.1873520E-01,.1306617E-01,.1521008E-01,&
     & .1910962E-01,.1640369E-01,.2664425E-01,.2836468E-01,.1997971E-01,&
     & .6508449E-02,.3071268E-01,.1794967E-01,.9099105E-02,.1435908E-02,&
     & .3256760E-01,.2153142E-01,.9594611E-03,.2499136E-02,.3451570E-01,&
     & .1686790E-01,.5053610E-06,.2766470E-02,.4487650E-01,.1237910E-02,&
     & .4883670E-06,.1222450E-02,.4869250E-01,.4643710E-06,.4642410E-06,&
     & .7538460E-06,.5305110E-01,.3762340E-06,.4098240E-06,.4706500E-06/

!     the array selfref contains the coefficient of the water vapor
!     self-continuum (including the energy term).  the first index
!     refers to temperature in 7.2 degree increments.  For instance,
!     jt = 1 refers to a temperature of 245.6, jt = 2 refers to 252.8,
!     etc.  the second index runs over the g-channel (1 to 12).

      data   selfref(1:10,1:12)  /                                      &
     & .1180690E+00,.7135230E-01,.4311990E-01,.2605840E-01,.1574770E-01,&
     & .9516750E-02,.5751210E-02,.3475600E-02,.2100390E-02,.1269320E-02,&
     & .1370810E-01,.1390460E-01,.1410400E-01,.1430610E-01,.1451120E-01,&
     & .1471930E-01,.1493030E-01,.1514430E-01,.1536140E-01,.1558160E-01,&
     & .1665750E-01,.1649160E-01,.1632730E-01,.1616470E-01,.1600370E-01,&
     & .1584430E-01,.1568640E-01,.1553020E-01,.1537550E-01,.1522240E-01,&
     & .5973790E-01,.5095170E-01,.4345790E-01,.3706620E-01,.3161450E-01,&
     & .2696470E-01,.2299880E-01,.1961620E-01,.1673110E-01,.1427030E-01,&
     & .3320371E+00,.2986281E+00,.2687252E+00,.2419448E+00,.2179475E+00,&
     & .1964309E+00,.1771281E+00,.1598017E+00,.1442398E+00,.1302553E+00,&
     & .5353614E+00,.4870549E+00,.4431120E+00,.4031396E+00,.3667769E+00,&
     & .3336985E+00,.3036077E+00,.2762334E+00,.2513302E+00,.2286753E+00,&
     & .2654145E+00,.2716462E+00,.2780447E+00,.2846162E+00,.2913638E+00,&
     & .2982924E+00,.3054070E+00,.3127136E+00,.3202159E+00,.3279193E+00,&
     & .3297702E+00,.3292642E+00,.3288167E+00,.3284279E+00,.3280982E+00,&
     & .3278270E+00,.3276143E+00,.3274609E+00,.3273666E+00,.3273316E+00,&
     & .2274450E+00,.2415450E+00,.2565190E+00,.2724220E+00,.2893110E+00,&
     & .3072470E+00,.3262940E+00,.3465230E+00,.3680050E+00,.3908200E+00,&
     & .6162030E-02,.1135230E-01,.2091440E-01,.3853070E-01,.7098520E-01,&
     & .1307760E+00,.2409290E+00,.4438650E+00,.8177330E+00,.1506510E+01,&
     & .2795520E-03,.8084720E-03,.2338120E-02,.6761920E-02,.1955570E-01,&
     & .5655550E-01,.1635600E+00,.4730200E+00,.1367990E+01,.3956260E+01,&
     & .2610060E-03,.7710430E-03,.2277760E-02,.6728790E-02,.1987770E-01,&
     & .5872120E-01,.1734700E+00,.5124520E+00,.1513850E+01,.4472090E+01/

!........................................!
      end module module_radsw_kgb29      !
!========================================!

